/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "groups.h"
#include "translation.h"
#include "olc.h"
#include "mysql5_interface.h"

/* just a test */
bool    check_swear_string  args( ( char *argument) );
bool check_social args(( CHAR_DATA *ch, char *command, char *argument ));


bool brew_cure_fatigue args((OBJ_DATA *obj));
bool brew_cure_fever args((OBJ_DATA *obj));
bool brew_cure_stomachache args((OBJ_DATA *obj));
bool brew_cure_headache args((OBJ_DATA *obj));
bool brew_poisonous args((OBJ_DATA *obj));
bool brew_painkiller args((OBJ_DATA *obj));
bool brew_sleep_better args((OBJ_DATA *obj));
bool brew_cure_health args((OBJ_DATA *obj));

char *check_dye args((OBJ_DATA *obj, char *string, int dye_vnum));
bool check_vowell args((const char *str));

// from prool:
bool is_alt_of(CHAR_DATA *ch, CHAR_DATA *ach);
bool is_same_group( CHAR_DATA *ach, CHAR_DATA *bch );

int str_cmp(char *str1, char*str2)
{
return strcmp(str1,str2);
}
// end of prool

char *channel_time (time_t time)
{   
	static char buf[20];
	char *date;
   
	if (time < 0)
	{
		//bugf( "Invalid integer time: (%d)", time );
		time = current_time;
	}
   
	date = ctime (&time);
   
	buf[0] = date[11];
	buf[1] = date[12];
	buf[2] = date[13];
	buf[3] = date[14];
	buf[4] = date[15];
	buf[5] = date[16];  
	buf[6] = date[17];
	buf[7] = date[18];
	buf[8] = '\0';
    
	return buf;
}


/* MYSQL Channel Logging by Rokann */

void add_channel_log (char *channel, char *from, char *to, char *msg)
{
	char mysql_buf[MSL];

	if (mud_port == build_port)	return;

	if ( !str_cmp( msg, "repop" ) ) return;
	if ( !str_cmp( msg, "weavie" ) ) return;

	sprintf(mysql_buf, "insert into chan_history values (\"\", \"%s\", \"%s\", \"%s\", \"%s\", now());", channel, from, to, sql_escape_string(msg));
	mysql_real_query(result_db, mysql_buf, strlen(mysql_buf));

	if( mysql_affected_rows(result_db) < 1)
           log_string("MYSQL: Error saving channel log.");

	return;
}



int check_buddy2(CHAR_DATA * ch, char *name)
{
	BUDDY_DATA *buddy;

	if (!ch)	return -1;
	if (IS_NPC(ch))	return -1;

	for (buddy = ch->pcdata->buddies; buddy != NULL; buddy = buddy->next)
	{
		if (buddy == NULL)	break;
		if (is_name(name, buddy->name))
			return 1;
	}
	return -1;
}

/* Channel types: tell, global, immortal, guild, staff, group */

void show_channel_logs (CHAR_DATA *ch, char *channel)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char mysql_buf[MSL];
	int total = 0, max = 15;
	char buf[MSL];
	int hr, mn;

	if (mud_port == build_port)	return;

	/* TODO Needs buddy talk support */

	if (!str_cmp(channel, "guild")) /* Covers darktalk, towertalk, handtalk, etc */
		sprintf(buf, "where chan_id = 'guild' and to_ch = '%s'", clan_table[ch->clan].gcom);
	else if (!str_cmp(channel, "clan")) /* Covers darkfriend, aessedai, clanchief, etc */
		sprintf(buf, "where chan_id = 'clan' and to_ch = '%s'", clan_table[ch->clan].name);
	else if (!str_cmp(channel, "group"))
		sprintf(buf, "where chan_id = 'group' and to_ch = '%s'", ch->leader->name);
	else	sprintf(buf, "where chan_id = '%s' and (to_ch = '%s' or from_ch = '%s' or to_ch = 'all')", channel, ch->name, ch->name);

	sprintf(mysql_buf, "select count(id) from chan_history %s", buf);
	result = query_db(mysql_buf, result_db, FALSE);

        if ( result == NULL )
        {
		stc("A MySQL error was encountered.\n\r", ch);
		bug("MySQL Error, result NULL in query_db from show_channel_logs",0);
		return;
	}

        row = mysql_fetch_row(result);
	total = UMAX(max, atoi(row[0]));
	mysql_free_result(result);

	if (total <= 0)	return;
	if (max > total) max = total;

	sprintf(mysql_buf, "select month(timestamp), day(timestamp), hour(timestamp), minute(timestamp), message from chan_history %s order by id asc limit %d, %d", 
		buf, total - max, max);
	result = query_db(mysql_buf, result_db, FALSE);

	if (result == NULL)	return;

	while ((row = mysql_fetch_row(result)) != NULL)
	{
		hr = atoi(row[2]);
		mn = atoi(row[3]);

		if (hr > 12) hr -= 12;
		if (hr == 0) hr = 12;

		ptc(ch, "%s[{c%s/%-2s %2d:%s%s%s] %s\n\r", (atoi(row[2]) >= 12) ? "{D" : "{W", row[0], row[1],
		hr, mn > 9 ? "" : "0", row[3], (atoi(row[2]) >= 12) ? "{D" : "{W", row[4]);
	}

	if (result != NULL)
        mysql_free_result(result);

	return;
}

void do_delet( CHAR_DATA *ch, char *argument)
{
	stc("You must type the full command to delete yourself.\n\r",ch);
}

void do_delete( CHAR_DATA *ch, char *argument)
{
	char strsave[MIL];
	DESCRIPTOR_DATA *d_mentor;

	if (IS_NPC(ch))
		return;

	if (ch->pcdata->confirm_delete)
	{
		if (argument[0] != '\0')
		{
			stc("Delete status removed.\n\r",ch);
			ch->pcdata->confirm_delete = FALSE;
			return;
		}
		else
		{
			sprintf( strsave, "%s%s", PLAYER_DIR, capit_colorsafe( ch->name ) );
			wiznet("$N turns $Mself into line noise.",ch,NULL,0,0,0);
			stop_all_fighting( ch );
			do_function(ch, &do_quit, "");
			unlink(strsave);
			return;
		}
	}

	if (argument[0] != '\0')
	{
		stc("Just type delete. You will then be asked to confirm your decision.\n\r",ch);
		return;
	}

	stc("{cType delete again to confirm this command.{c\n\r",ch);
	stc("{cWARNING: this command is irreversible.{c\n\r",ch);
	stc("{cTyping delete with an argument will undo delete status.{c\n\r",ch);
	ch->pcdata->confirm_delete = TRUE;
	wiznet("$N is contemplating deletion.",ch,NULL,0,0,get_trust(ch));

	for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
	{
		if ( d_mentor->connected == CON_PLAYING
		&& IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
		{
			act("{w({RMentor Alert{w){W::{x $N is contemplating {Rdeletion!{x",
				d_mentor->character,NULL,ch,TO_CHAR);
		}
	}
}

void do_deaf( CHAR_DATA *ch, char *argument)
{

	if (IS_SET(ch->comm,COMM_DEAF))
	{
		stc("You can now hear tells again.\n\r",ch);
		REMOVE_BIT(ch->comm,COMM_DEAF);
	}
	else
	{
		stc("From now on, you won't hear tells.\n\r",ch);
		SET_BIT(ch->comm,COMM_DEAF);
	}
}

void do_afk ( CHAR_DATA *ch, char * argument)
{
	if ( IS_NPC( ch ) )
		return;

	if ( argument[0] == '\0' )
	{
		affect_strip( ch, gsn_invis );
		affect_strip( ch, gsn_hide );
		affect_strip( ch, gsn_sneak );

	    if (IS_SET(ch->comm,COMM_AFK))
	    {
	      stc("You return to the keyboard. Type 'replay' to see tells.\n\r",ch);
	      act("$n returns to the keyboard.",ch,0,0,TO_ROOM);
	      REMOVE_BIT(ch->comm,COMM_AFK);
	    }
	    else
	    {
	      stc("You step away from the keyboard. AFK mode implemented.\n\r",ch);
	      act("$n steps away from the keyboard.",ch,0,0,TO_ROOM);
	      SET_BIT(ch->comm,COMM_AFK);
	    }
	    return;
	}
	else if (!str_cmp(argument,"clear"))
	{
		free_string( ch->pcdata->afkmsg );
		ch->pcdata->afkmsg = str_dup("" );
		stc( "Your AFK Message is now cleared.\n\r", ch );
	}
	else
	{
		smash_tilde( argument );
		if ( strlen_color( argument ) > 80 )
		{
			stc( "String cannot be greater than 80 character, not counting color codes.\n\r", ch );
			return;
		}
		stc( "Setting your AFK Message:\n\r", ch );
		free_string( ch->pcdata->afkmsg );
		ch->pcdata->afkmsg = str_dup( argument );
		stc( ch->pcdata->afkmsg, ch );
		stc( "\n\r", ch );
	}
	return;
}

void do_replay (CHAR_DATA *ch, char *argument)
{
	if ( IS_NPC(ch) || !ch->desc || !ch->pcdata )
	{
		stc("You can't replay.\n\r",ch);
		return;
	}

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}

	if (buf_string(ch->pcdata->buffer)[0] == '\0')
	{
		stc("You have no tells to replay.\n\r",ch);
		return;
	}

	page_to_char(buf_string(ch->pcdata->buffer),ch);
	clear_buf(ch->pcdata->buffer);
}

void do_coder( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;
	if (argument[0] == '\0' )
	{
		stc("Say what over the Coder Channel?\n\r",ch);
		return;
	}
	else
	{
		if (!IS_SET(ch->war_flags,WAR_CODER))
			return;
		else
		{
			ptc( ch, "{W[(Coder) {w%s{W]{W: {w%s{c\n\r", ch->name, argument );
  		        for ( d = descriptor_list; d != NULL; d = d->next )
			{
				CHAR_DATA *victim;
				victim = d->original ? d->original : d->character;
				if (d->connected == CON_PLAYING && IS_SET(victim->war_flags, WAR_CODER) )
				{
					act_new("{W[(Coder) {w$n{w] {W: {w$t{c", ch, argument, d->character, TO_VICT, POS_SLEEPING);
				}
			}
		}
	}
}

void do_inchar( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;

	if ( IS_SET( ch->penalty_flags, PENAL_NOCHATS ) )
	{
		stc("Your channel privileges have been revoked.\n\r", ch);
		return;
	}

	if (argument[0] == '\0' )
	{
		stc("Say what over InChar Channel?\n\r",ch);
		return;
	}
	else
	{
		if (IS_SET(ch->comm,COMM_QUIET))
		{
			stc("You must turn off quiet mode first.\n\r",ch);
			return;
		}

		if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
		{
			ptc( ch, "The %s has revoked your channel priviliges.\n\r", is_shadow(ch) ? "RGreat Lord{x" : "{wCreator{x" );
			return;
		}

		affect_strip( ch, gsn_invis );
		affect_strip( ch, gsn_hide );
		affect_strip( ch, gsn_sneak );

		REMOVE_BIT(ch->comm,COMM_NOIC);

		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		ptc( ch, "{m[{MYour IC{m]{M: {m'{M%s{m'{c\n\r", argument );

		for ( d = descriptor_list; d != NULL; d = d->next )
		{
			CHAR_DATA *victim;
			victim = d->original ? d->original : d->character;

			if (d->connected == CON_PLAYING
			&&  d->character->in_room != NULL && ch->in_room != NULL
			&&  d->character->in_room->area == ch->in_room->area &&
			!IS_SET(victim->comm,COMM_NOIC) &&
			!IS_SET(victim->comm,COMM_QUIET))
			{
				act_new( "{m[{M({m$n{M) {mInChar]: '{M$t{m'{c",ch,argument, d->character, TO_VICT,POS_SLEEPING );
			}
			if ( d->connected == CON_PLAYING && d->character->in_room != NULL
			&& ch->in_room != NULL
			&& IS_SET(d->character->war_flags,WAR_MODERATOR)
			&& d->character->in_room->area != ch->in_room->area)
			{
				act_new( "{m[{M({m$n{M) {mInChar]: '{M$t{m'{c",ch,argument, d->character, TO_VICT,POS_SLEEPING );
			}

		}
	}
}

bool has_commdev( CHAR_DATA *ch )
{
	bool has_dev = FALSE;
	OBJ_DATA *commdev, *commdev_next;
	for ( commdev = ch->carrying; commdev != NULL; commdev = commdev_next )
	{
		commdev_next = commdev->next_content;
		if ( commdev->item_type == ITEM_COMM_DEV && commdev->wear_loc != WEAR_NONE )
		{
			has_dev = TRUE;
			break;
		}
	}
	if ( has_dev ) return TRUE;
	return FALSE;
}
void do_gsocial(CHAR_DATA *ch, char *argument)
{
	char command[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	int cmd;
	bool found;
	char arg[MAX_INPUT_LENGTH];
	DESCRIPTOR_DATA *d;
	char buf[MAX_STRING_LENGTH];
	char cFound[MSL], vFound[MSL], oFound[MSL];
	int counter;
	int count;
	char buf2[MAX_STRING_LENGTH];
	argument = one_argument(argument,command);

	/* iinit the bufs */
	buf[0] = '\0';
	buf2[0] = '\0';
	cFound[0] = '\0';
	vFound[0] = '\0';
	oFound[0] = '\0';

	if ( IS_SET( ch->penalty_flags, PENAL_NOCHATS ) )
	{
		stc("Your channel privileges have been revoked.\n\r", ch);
		return;
	}

	if (command[0] == '\0')
	{
		stc("What do you wish to gocial?\n\r",ch);
		return;
	}
	found = FALSE;
	for (cmd = 0; social_table[cmd].name[0] != '\0'; cmd++)
	{
		if (command[0] == social_table[cmd].name[0]
		&& !str_prefix( command,social_table[cmd].name ) )
		{
			found = TRUE;
			break;
		}
	}
	if (!found)
	{
		stc("What kind of social is that?!?!\n\r",ch);
		return;
	}
	if (!IS_NPC(ch) && IS_SET(ch->comm,   COMM_QUIET))
	{
		stc("You must turn off quiet mode first.\n\r",ch);
		return;
	}
	if ( !IS_NPC(ch) && IS_SET(ch->comm, COMM_NOOOC))
	{
		stc("But you have the OOC channel turned off!\n\r",ch);
		return;
	}
	if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOCHANNELS))
	{
		stc("The gods have revoked your channel priviliges.\n\r",ch);
		return;
	}
	if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE))
	{
		stc("You are anti-social!\n\r",ch);
		return;
	}
	switch (ch->position)
	{
		case POS_DEAD:
			stc("Lie still; you are DEAD!\n\r",ch);
			return;
		case POS_INCAP:
		case POS_MORTAL:
			stc("You are hurt far too bad for that.\n\r",ch);
			return;
		case POS_STUNNED:
			stc("You are too stunned for that.\n\r",ch);
			return;
	}

	one_argument(argument,arg);
	victim = NULL;


	if (arg[0] == '\0')
	{
		sprintf(buf, "{C[Global]{w %s{x", social_table[cmd].char_no_arg );
		act_new(buf,ch,NULL,NULL,TO_CHAR,POS_DEAD);
		buf[0] = '\0';

		if ( social_table[cmd].others_no_arg )
		{
			sprintf(buf, "{C[Global]{w %s{x", social_table[cmd].others_no_arg );
			for (d = descriptor_list; d != NULL; d = d->next)
			{
				CHAR_DATA *vch;
				vch = d->original ? d->original : d->character;

				if (d->connected == CON_PLAYING &&
				d->character != ch &&
				!IS_SET(vch->comm,COMM_NOOOC) &&
				!IS_SET(vch->comm,COMM_QUIET))
					act_new(buf,ch,NULL,vch,TO_VICT,POS_DEAD);
			}
			buf[0] = '\0';
		}
	}
	else if ((victim = get_pc_world(ch,arg)) == NULL)
	{
		stc("They aren't here.\n\r",ch);
		return;
	}
	else if ((victim->invis_level || victim->incog_level) && (victim != ch))
	{
		stc("They aren't here.\n\r",ch);
		return;
	}
	else if (victim == ch)
	{
		sprintf(buf,"{C[Global]{w %s{x", social_table[cmd].char_auto);
		act_new(buf,ch,NULL,NULL,TO_CHAR,POS_DEAD);
		buf[0] = '\0';

		sprintf(buf,"{C[Global]{w %s{x", social_table[cmd].others_auto);

		for (d = descriptor_list; d != NULL; d = d->next)
		{
			CHAR_DATA *vch;
			vch = d->original ? d->original : d->character;

			if (d->connected == CON_PLAYING &&
			d->character != ch &&
			!IS_SET(vch->comm,COMM_NOOOC) &&
			!IS_SET(vch->comm,COMM_QUIET))
					act_new(buf,ch,NULL,vch,TO_VICT,POS_DEAD);
		}
		buf[0] = '\0';
		return;
	}
	else
	{
		cFound[0] = '\0';
		sprintf(cFound,"{C[Global]{w %s{x", social_table[cmd].char_found);
		act_new(cFound,ch,NULL,victim,TO_CHAR,POS_DEAD);

		buf[0] = '\0';

		if ( social_table[cmd].vict_found )
		{
			vFound[0] = '\0';
			sprintf(vFound,"{C[Global]{w %s{x", social_table[cmd].vict_found);
			act_new(vFound,ch,NULL,victim,TO_VICT,POS_DEAD);
		}

		buf[0] = '\0';

		if ( social_table[cmd].others_found )
		{
			sprintf(buf,"{C[Global]{w %s{x", social_table[cmd].others_found);
			for (counter = 0; buf[counter+1] != '\0'; counter++)
			{
			    if (buf[counter] == '$' && buf[counter + 1] == 'N')
			    {
					strcpy(buf2,buf);
					buf2[counter] = '\0';
					strcat(buf2,victim->name);
					for (count = 0; buf[count] != '\0'; count++)
					{
					    buf[count] = buf[count+counter+2];
					}
					strcat(buf2,buf);
					strcpy(buf,buf2);
			    }
			    else if (buf[counter] == '$' && buf[counter + 1] == 'E')
			    {
					switch (victim->sex)
					{
					default:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"it");
					    for (count = 0; buf[count] != '\0'; count ++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
				    break;
					case 1:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"it");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					case 2:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"it");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					}
			    }
			    else if (buf[counter] == '$' && buf[counter + 1] == 'M')
			    {
					buf[counter] = '%';
					buf[counter + 1] = 's';
					switch (victim->sex)
					{
						default:
			  		    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"it");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					case 1:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"him");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					case 2:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"her");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					}
			    }
			    else if (buf[counter] == '$' && buf[counter + 1] == 'S')
			    {
					switch (victim->sex)
					{
						default:
						strcpy(buf2,buf);
						buf2[counter] = '\0';
						strcat(buf2,"its");
						for (count = 0;buf[count] != '\0'; count++)
						{
						    buf[count] = buf[count+counter+2];
						}
						strcat(buf2,buf);
						strcpy(buf,buf2);
						break;
					case 1:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"his");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					case 2:
					    strcpy(buf2,buf);
					    buf2[counter] = '\0';
					    strcat(buf2,"hers");
					    for (count = 0; buf[count] != '\0'; count++)
					    {
							buf[count] = buf[count+counter+2];
					    }
					    strcat(buf2,buf);
					    strcpy(buf,buf2);
					    break;
					}
			    }
			}
			for (d=descriptor_list; d != NULL; d = d->next)
			{
				CHAR_DATA *vch;
				vch = d->original ? d->original : d->character;

				if (d->connected == CON_PLAYING &&
				d->character != ch &&
				d->character != victim &&
				!IS_SET(vch->comm, COMM_NOOOC) &&
				!IS_SET(vch->comm,COMM_QUIET))
					act_new(buf,ch,NULL,vch,TO_VICT,POS_DEAD);
			}
		}
	}
	buf[0] = '\0';
	buf2[0] = '\0';
	return;
}

void do_global( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;
	char buf[MSL], buf2[MSL];

	if (argument[0] == '\0' )
	{
		stc("Say what over the global channel?\n\r",ch);
		return;
	}
	else
	{
		if ( !str_cmp( argument, "last" ))
		{
			show_channel_logs(ch, "global");
			return;
		}

		if (IS_SET(ch->comm,COMM_QUIET))
		{
			stc("{cYou must turn off quiet mode first.{c\n\r",ch);
			return;
		}

		if ( IS_SET(ch->penalty_flags,PENAL_NOCHANNELS) || IS_SET(ch->penalty_flags,PENAL_NOCHATS) )
		{
			ptc( ch, "The %s has revoked your channel priviliges.\n\r", is_shadow(ch) ? "{RGreat Lord{x" : "{wCreator{x" );
			return;
		}

		sprintf( buf2, "{C[Global{c({C%s%s{c){C]{c{x {C'{y%s{C'{x",
			IS_SET( ch->war_flags, WAR_IMMORTAL ) ? "Immortal " :
			IS_SET( ch->war_flags, WAR_MODERATOR ) ? "Moderator " :
			IS_SET( ch->war_flags, WAR_MENTOR ) ? "Mentor " :
			IS_SET( ch->war_flags, WAR_CODER ) ? "Coder " :
			ch->pcdata->newbie == 1 ? "Newbie " :
			"", ch->colorname, argument );

		act_new( buf2,ch,argument,NULL,TO_CHAR,POS_DEAD );


		sprintf( buf, "{C[Global{c({C%s%s{c){C]{c{x {C'{y%s{C'{x",
			IS_SET( ch->war_flags, WAR_IMMORTAL ) ? "Immortal " :
			IS_SET( ch->war_flags, WAR_MODERATOR ) ? "Moderator " :
			IS_SET( ch->war_flags, WAR_MENTOR ) ? "Mentor " :
			IS_SET( ch->war_flags, WAR_CODER ) ? "Coder " :
			ch->pcdata->newbie == 1 ? "Newbie " :
			"", ch->colorname, argument );


		add_channel_log("global", ch->name, "all", buf);

		for ( d = descriptor_list; d != NULL; d = d->next )
		{
			CHAR_DATA *victim;
			if ( d->connected != CON_PLAYING )
				continue;
			victim = d->original ? d->original : d->character;
			if ( !victim || victim == NULL )
				continue;
			if ( !victim->desc )
				continue;
			if ( victim->desc->connected != CON_PLAYING )
				continue;
			if ( IS_SET (victim->comm, COMM_QUIET))
				continue;
			if ( IS_SET (victim->comm, COMM_NOOOC))
				continue;

			act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );

		}
	}
}

/* RT question channel */
void do_question( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;

	if ( IS_SET( ch->penalty_flags, PENAL_NOCHATS ) )
	{
		stc("Your channel privileges have been revoked.\n\r", ch);
		return;
	}

	if (argument[0] == '\0' )
	{
		stc("Ask what over the Q/A channel?\n\r",ch);
		return;
	}
	else
	{
		if (IS_SET(ch->comm,COMM_QUIET))
		{
			stc("You must turn off quiet mode first.\n\r",ch);
			return;
		}

		if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
		{
			ptc( ch, "The %s has revoked your channel priviliges.{c\n\r", is_shadow(ch) ? "{RGreat Lord{x" : "{wCreator{x" );
			return;
		}

		add_channel_log("question", ch->name, "all", argument);

		REMOVE_BIT(ch->comm,COMM_NOQUESTION);
		affect_strip( ch, gsn_invis );
		affect_strip( ch, gsn_hide );
		affect_strip( ch, gsn_sneak );

		ptc( ch, "{m[{MYour Q/A{m]{M: '{m%s{M'{c\n\r", argument );

		for ( d = descriptor_list; d != NULL; d = d->next )
		{
			CHAR_DATA *victim;

			victim = d->original ? d->original : d->character;
			if ( d->connected == CON_PLAYING
			&& IS_SET(victim->spec_flags,SPEC_MENTOR)
			&& d->character != ch 
			&& !IS_SET(victim->comm,COMM_NOQUESTION)
			&& !IS_SET(victim->comm,COMM_QUIET))
			{
				act_new("{c{m[{MQ/A {m({M$n{m)]{M: '{m$t{M'{c",ch,argument,d->character,TO_VICT,POS_SLEEPING);
			}
		}
	}
}

bool is_towermilitary( CHAR_DATA *ch )
{
	if ( !str_cmp( clan_table[ch->clan].name, "towerguard" )
	|| !str_cmp( clan_table[ch->clan].name, "warder" )
	|| !str_cmp( clan_table[ch->clan].name, "youngling" ) )
		return TRUE;

	return FALSE;
}

void do_immtalk( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;
	char buf[MSL];

	if ( argument[0] == '\0' )
	{
		stc("Immortal Talk what?\n\r",ch);
		return;
	}

	if (IS_SET(ch->comm,COMM_NOWIZ))
		REMOVE_BIT(ch->comm,COMM_NOWIZ);

	if ( !str_cmp( argument, "last" ))
	{
		show_channel_logs(ch, "immortal");
		return;
	}

	sprintf( buf, "{W[{w%s{W]{w: {W%s{x", ch->name, argument );
	add_channel_log("immortal", ch->name, "all", buf);
	act_new("{c{W[{w$n{W]{w: {W$t{c",ch,argument,NULL,TO_CHAR,POS_DEAD);

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( d->connected == CON_PLAYING && is_granted_name( d->character, "immtalk" ) && !IS_SET(d->character->comm,COMM_NOWIZ) )
		{
			act_new("{c{W[{w$n{W]{w: {W$t{c",ch,argument,d->character,TO_VICT,POS_DEAD);
		}
	}
	return;
}

void do_stafftalk( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;
	char buf[MSL];

	if (!IS_SET(ch->hist_flags,HIST_GUILDLEADER) && !IS_IMMORTAL(ch) && !IS_SET( ch->spec_flags,SPEC_MENTOR ) && !IS_SET(ch->war_flags,WAR_MODERATOR))
	{
		stc("You are not a Guildleader, Mentor, Moderator, or Immortal.\n\r",ch);
		return;
	}

	if ( argument[0] == '\0' )
	{
		stc("Staff Talk what?\n\r",ch);
		return;
	}

	if ( !str_cmp( argument, "last" ))
	{
		show_channel_logs(ch, "staff");
		return;
	}

	sprintf( buf, "{c{W[{RS{rt{Ra{rf{Rf {w%s{W]{w: {Y%s{x", ch->name, argument );
	add_channel_log("staff", ch->name, "all", buf);

	act_new("{c{W[{RS{rt{Ra{rf{Rf {w$n{W]{w: {Y$t{c",ch,argument,NULL,TO_CHAR,POS_DEAD);

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( ! d->character ) continue;
		if ( !d->character->pcdata ) continue;
		if ( d->connected != CON_PLAYING ) continue;

		if ( ( IS_SET( d->character->hist_flags, HIST_GUILDLEADER )
		&& !IS_SET( d->character->comm, COMM_NOSTAFF ) )

		|| ( IS_SET( d->character->war_flags, WAR_MODERATOR )
		&& !IS_SET( d->character->comm, COMM_NOSTAFF ) )

		|| ( IS_SET( d->character->war_flags, WAR_MENTOR )
		&& !IS_SET( d->character->comm, COMM_NOSTAFF ) )

		|| ( IS_IMMORTAL( d->character )
		&& !IS_SET( d->character->comm, COMM_NOSTAFF ) ) )
		{
		    act_new("{c{W[{RS{rt{Ra{rf{Rf {w$n{W]{w: {Y$t{c",ch,argument,d->character,TO_VICT,POS_DEAD);
		}
	}
	return;
}

void do_gltalk( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;

	if (!IS_SET(ch->hist_flags,HIST_GUILDLEADER) && !IS_IMMORTAL(ch))
	{
		stc("You are not a GuildLeader or an Immortal.\n\r",ch);
		return;
	}

	if ( argument[0] == '\0' )
	{
		stc("GuildLeader Talk what?\n\r",ch);
		return;
	}

	act_new("{c{R[{GGL {w$n{R]{w: {W$t{c",ch,argument,NULL,TO_CHAR,POS_DEAD);

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( ! d->character ) continue;
		if ( !d->character->pcdata ) continue;
		if ( d->connected != CON_PLAYING ) continue;
		/* GLs */
		if ( ( IS_SET( d->character->hist_flags, HIST_GUILDLEADER )
		&& !IS_SET( d->character->comm, COMM_NOGL ) )
		|| ( IS_IMMORTAL( d->character ) && !IS_SET( d->character->comm, COMM_NOGL ) ) )
		{
			act_new("{c{R[{GGL {w$n{R]{w: {W$t{c",ch,argument,d->character,TO_VICT,POS_DEAD);
		}
	}
	return;
}

void do_howl( CHAR_DATA *ch, char *argument )
{
	char arg[MIL];

	argument = one_argument( argument, arg );

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("{cYou cannot howl because you abused your channel priviledges.{c\n\r",ch);
		return;
	}

	if (argument[0] == '\0')
	{
		if (IS_SET(ch->spec_flags,SPEC_WOLFBROTHER))
		{
			if (IS_AFFECTED(ch, AFF_HIDE)
			|| IS_AFFECTED(ch, AFF_SNEAK))
			{
				act("{cThe sound of howling wolves seems to come from all different directions.",ch,0,0,TO_ROOM);
				stc("{cYou throw back your head and *Howl* to be answered by your wolf brethren from all about.\n\r", ch);
			}
			else
			{
				act("{c$n throws back $s and *Howls*",ch,0,0,TO_ROOM);
				stc("{cYou throw back your head and *Howl*\n\r", ch);
			}
			return;
		}
		else
		{
			stc("{c*Howl*\n\r", ch);
			act("{c$n *Howls*",ch,0,0,TO_ROOM);
			return;
		}
	}
	return;
}


void do_say( CHAR_DATA *ch, char *argument )
{
	char endbit[2];
	int length;

	if ( argument[0] == '\0' )
	{
		stc( "{cSay what?{c{c\n\r", ch );
		return;
	}

	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_hide );
	affect_strip( ch, gsn_sneak );

	if (!IS_NPC(ch) && IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("{cYou cannot use channels.{c\n\r",ch);
		return;
	}
	if ( !IS_NPC( ch ) && ch->pcdata->newbie == 1 )
	{
		int toroom = 0;
		if( !str_cmp( argument, "ready" ) && ch->in_room->vnum ==31002 )
		{
			stc("Great! Now let's move on to the next section.\n\r", ch );
			toroom = 31003;
			char_from_room( ch );
			char_to_room( ch, get_room_index( toroom ) );
			do_function( ch, &do_look, "auto" );
			return;
		}
	}

	endbit[0] = argument[strlen(argument)-1];
	endbit[1] = '\0';
	length = strlen ( argument );

	if ( !strcmp ( endbit, "!" ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act("{g$n exclaims, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act("{gYou exclaim, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}
	else if ( !strcmp ( endbit, "?" ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act("{g$n asks, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act("{gYou ask, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '.'
	&& argument[length - 2] == '.'
	&& argument[length - 3] == '.' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n mutters, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou mutter, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '('
	&& argument[length - 2] == '-'
	&& ( argument[length - 3] == ':' || argument[length - 3] == '=' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n say sadly, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou says sadly, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '|'
	&& argument[length - 2] == '-'
	&& ( argument[length - 3] == ':' || argument[length - 3] == '=' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n looks confused and says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou look confused and say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == ')'
	&& argument[length - 2] == 'o'
	&& ( argument[length - 3] == ':' || argument[length - 3] == '=' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n clownishy says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou clownishly say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == 'O'
	&& argument[length - 2] == '.'
	&& argument[length - 3] == 'o' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n says in disbelief, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou say in disbelief, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == 'o'
	&& argument[length - 2] == '.'
	&& argument[length - 3] == 'O' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gDisbelievingly, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gDisbelievingly, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == 'O'
	&& argument[length - 2] == '.'
	&& argument[length - 3] == 'O' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gWide-eyed, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gWide-eyed, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '?' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		if ( argument[length - 2] == '!' )
		{
			act ( "{g$n boggles and says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
			act ( "{gYou boggle and say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
		}
		else if ( argument[length - 2] != '?' )
		{
			act ( "{g$n asks, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
			act ( "{gYou ask, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
		}
		else
		{
			act ( "{g$n demands, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
			act ( "{gYou demand, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
		}
	}

	else if ( argument[length - 1] == ')' && ( argument[length - 2] == '=' || argument[length - 2] == ':' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n smiles and says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou smile and say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '^' && ( argument[length - 2] == '-' || argument[length - 2] == '^' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gBlushing, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gBlushing, You say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '-' && ( argument[length - 2] == '.' || argument[length - 2] == '-' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gSleepily, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gSleepily, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '(' && ( argument[length - 2] == '-' || argument[length - 2] == '8' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gSurprised, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gSurprised, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == ')' && ( argument[length - 2] == '-' || argument[length - 2] == '8' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gPleasantly surprised, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gPleasantly surprised, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '(' && argument[length - 2] == '"' && argument[length - 3] == ':' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gCrying, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gCrying, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '(' && argument[length - 2] == '"' && argument[length - 3] == '=' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gWeeping, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gWeeping, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '~' && argument[length - 2] == ')' && argument[length - 2] == ':' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{gDrooling, $n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gDrooling, you say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '@' && ( argument[length - 2] == '=' || argument[length - 2] == ':' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n angrily screams, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou angrily scream, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '(' && ( argument[length - 2] == '=' || argument[length - 2] == ':' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n sulks and says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou sulk and say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( ( argument[length - 1] == 'P' || argument[length - 1] == 'p' ) && ( argument[length - 2] == '=' || argument[length - 2] == ':' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n says with a smirk, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou say with a smirk, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == ')' && argument[length - 2] == ';' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n says with a wink, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou say with a wink, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == '>' && argument[length - 2] == '-' && argument[length - 3] == ':' )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n says mischeviously, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou say mischeviously, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == 'O' && ( argument[length - 2] == '=' || argument[length - 2] == ':' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n cheerfully says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou cheerfully say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else if ( argument[length - 1] == 'D' && ( argument[length - 2] == '=' || argument[length - 2] == ':' ) )
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act ( "{g$n says with a grin, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act ( "{gYou say with a grin, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	else
	{
		if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
			argument = makedrunk(argument,ch);

		act("{g$n says, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_ROOM );
		act("{gYou say, {G'{g$T{G'{x", ch, NULL, capit2(argument), TO_CHAR );
	}

	if ( !IS_NPC(ch) )
	{
		CHAR_DATA *mob, *mob_next;
		OBJ_DATA *obj, *obj_next;

		for ( mob = ch->in_room->people; mob != NULL; mob = mob_next )
		{
			mob_next = mob->next_in_room;

			if ( IS_NPC(mob) && HAS_TRIGGER_MOB( mob, TRIG_SPEECH )
			&&   mob->position == mob->pIndexData->default_pos )
				p_act_trigger( argument, mob, NULL,NULL,ch, NULL, NULL, TRIG_SPEECH );

			for ( obj = mob->carrying; obj; obj = obj_next )
			{
				obj_next = obj->next_content;

				if ( HAS_TRIGGER_OBJ( obj, TRIG_SPEECH ) )
					p_act_trigger( argument, NULL, obj, NULL, ch, NULL, NULL, TRIG_SPEECH );
			}
		}

		for ( obj = ch->in_room->contents; obj; obj = obj_next )
		{
			obj_next = obj->next_content;

			if ( HAS_TRIGGER_OBJ( obj, TRIG_SPEECH ) )
				p_act_trigger( argument, NULL, obj, NULL, ch, NULL, NULL, TRIG_SPEECH );
		}

		if ( HAS_TRIGGER_ROOM( ch->in_room, TRIG_SPEECH ) )
			p_act_trigger( argument, NULL, NULL, ch->in_room, ch, NULL, NULL, TRIG_SPEECH );
	}
	return;
}

void do_esay( CHAR_DATA *ch, char *argument )
{
	char arg1[MIL];
	char arg2[MIL];
	char buf1[MSL];
	char buf2[MSL];
	char buf3[MIL];
	char buf4[MSL];
	char endbit[2];
	char name_buf[MSL];
	char word_buf[MSL];

	buf1[0] = '\0';
	buf2[0] = '\0';
	buf3[0] = '\0';
	buf4[0] = '\0';

	argument = one_argument ( argument, arg1 );

	if ( arg1[0] == '\0' )
	{
		stc( "{cESay how?{c\n\r", ch );
		stc("angry           disbelief\n\r",ch);
		stc("sad             threaten\n\r",ch);
		stc("happy           loving\n\r",ch);
		stc("evil            careful\n\r",ch);
		stc("confusion       nonchalant\n\r",ch);
		stc("stubborn        tired\n\r",ch);
		stc("stupid          impatient\n\r",ch);
		stc("bitter          cordial\n\r",ch);
		stc("cold            smug\n\r",ch);
		stc("sarcastic\n\r",ch);
		return;
	}
	if ( argument[0] == '\0' )
	{
		stc( "{cYou mutter something incomprehensible...{c\n\r", ch );
		act( "$n mutters something incomprehensible...",ch,0,0,TO_ROOM);
		return;
	}

	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_hide );
	affect_strip( ch, gsn_sneak );

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("{cYou cannot use channels.{c\n\r",ch);
		return;
	}

	if (!str_cmp (arg1, "disbelief" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';

		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s in disbelief {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s in disbelief {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s in disbelief {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s in disbelief {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says in disbelief {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say in disbelief {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "threaten" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';

		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s threateningly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s threateningly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s threateningly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s threateningly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says threateningly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say threateningly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "loving" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s lovingly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s lovingly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s lovingly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s lovingly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says lovingly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say lovingly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "careful" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s carefully {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s carefully {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s carefully {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s carefully {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says carefully {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say carefully {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "nonchalant" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s nonchalantly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s nonchalantly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s nonchalantly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s nonchalantly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says nonchalantly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say nonchalantly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "tired" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s tiredly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s tiredly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s tiredly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s tiredly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says tiredly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say tiredly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "impatient" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s impatiently {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s impatiently {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s impatiently {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s impatiently {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says impatiently {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say impatiently {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "cordial" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s cordially {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s cordially {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s cordially {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s cordially {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says cordially {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say cordially {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "evil" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s evilly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s evilly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s evilly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s evilly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says evilly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say evilly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "confusion" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s in confusion {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s in confusion {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s in confusion {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s in confusion {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
	   	 	sprintf(buf1, "{g%s says in confusion {G'{g%s{G'{x",ch->name, arg2);
	   	 	sprintf(buf2, "{gYou say in confusion {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "stubborn" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s stubbornly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s stubbornly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s stubbornly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s stubbornly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says stubbornly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say stubbornly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "stupid" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s stupidly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s stupidly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s stupidly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s stupidly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says stupidly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say stupidly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "bitter" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s bitterly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s bitterly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s bitterly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s bitterly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says bitterly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say bitterly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "cold" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s coldly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s coldly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s coldly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s coldly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says coldly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say coldly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "sarcastic" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s sarcastically {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s sarcastically {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s sarcastically {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s sarcastically {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says sarcastically {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say sarcastically {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "smug" ) )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s smugly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s smugly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s smugly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s smugly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says smugly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say smugly {G'{g%s{G'{x", arg2);
		}
	}

	else if (!str_cmp (arg1, "angry") )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s angrily {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s angrily {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s angrily {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s angrily {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says angrily {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say angrily {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "sad") )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s sadly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s sadly {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s sadly {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s sadly {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says sadly {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say sadly {G'{g%s{G'{x", arg2);
		}
	}
	else if (!str_cmp (arg1, "happy") )
	{
		strcpy ( arg2, argument );
		endbit[0] = argument[strlen(argument)-1];
		endbit[1] = '\0';
		if ( !strcmp ( endbit, "!" ) )
		{
			sprintf ( buf3, "exclaims" );
			sprintf ( buf4, "exclaim" );
			sprintf(buf1, "{g%s %s happily {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s happily {G'{g%s{G'{x", buf4, arg2);
		}
		else if ( !strcmp ( endbit, "?" ) )
		{
			sprintf ( buf3, "asks" );
			sprintf ( buf4, "ask" );
			sprintf(buf1, "{g%s %s happily {G'{g%s{G'{x",ch->name, buf3, arg2);
			sprintf(buf2, "{gYou %s happily {G'{g%s{G'{x", buf4, arg2);
		}
		else
		{
			sprintf(buf1, "{g%s says happily {G'{g%s{G'{x",ch->name, arg2);
			sprintf(buf2, "{gYou say happily {G'{g%s{G'{x", arg2);
		}
	}
	else
	{
		stc( "{cESay how?{c\n\r", ch );
		stc("angry           disbelief\n\r",ch);
		stc("sad             threaten\n\r",ch);
		stc("happy           loving\n\r",ch);
		stc("evil            careful\n\r",ch);
		stc("confusion       nonchalant\n\r",ch);
		stc("stubborn        tired\n\r",ch);
		stc("stupid          impatient\n\r",ch);
		stc("bitter          cordial\n\r",ch);
		stc("cold            smug\n\r",ch);
		stc("sarcastic\n\r",ch);
		return;
	}

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
		argument = makedrunk(arg2,ch);

	if ( strlen(buf1) > MAX_INPUT_LENGTH || strlen(buf2) > MAX_INPUT_LENGTH )
	{
		stc( "Line too long.\n\r", ch );
		return;
	}

	strcpy(name_buf, ch->name);
	strcpy(word_buf, buf1);

	act("$T",ch,0,buf1,TO_ROOM);
	act("$T",ch,0,buf2,TO_CHAR);

	if ( !IS_NPC(ch) )
	{
		CHAR_DATA *mob, *mob_next;
		for ( mob = ch->in_room->people; mob != NULL; mob = mob_next )
		{
			mob_next = mob->next_in_room;
			if ( IS_NPC(mob) && HAS_TRIGGER_MOB( mob, TRIG_SPEECH )
			&& mob->position == mob->pIndexData->default_pos )
				p_act_trigger( argument, mob, NULL, NULL, ch, NULL, NULL, TRIG_SPEECH );
		}
	}
	return;
}

bool in_rprecall( CHAR_DATA *ch )
{
	int room;

	if ( ch->in_room == NULL )
		return FALSE;

	room = ch->in_room->vnum;

	if ( room < 905 || room > 913 )
		return FALSE;

	return TRUE;
}
void do_osay( CHAR_DATA *ch, char *argument )
{
	if ( argument[0] == '\0' )
	{
		stc( "Osay what?\n\r", ch );
		return;
	}

	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_hide );
	affect_strip( ch, gsn_sneak );

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
		argument = makedrunk(argument,ch);

	act("$n says oocly, {C'{c$T{C'{x", ch, NULL, capit2(argument), TO_ROOM );
	act("You say oocly, {C'{c$T{C'{x", ch, NULL, capit2(argument), TO_CHAR );

	if ( !IS_NPC(ch) )
	{
		CHAR_DATA *mob, *mob_next;
		for ( mob = ch->in_room->people; mob != NULL; mob = mob_next )
		{
			mob_next = mob->next_in_room;
			if ( IS_NPC(mob) && HAS_TRIGGER_MOB( mob, TRIG_SPEECH )
			&& mob->position == mob->pIndexData->default_pos )
				p_act_trigger( argument, mob, NULL, NULL, ch, NULL, NULL, TRIG_SPEECH );
		}
	}
	return;
}

void do_sayto( CHAR_DATA *ch, char *argument )
{
	char arg[MIL];
	CHAR_DATA *victim;

	argument = one_argument( argument, arg );

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("{cYou cannot use channels.{c\n\r",ch);
		return;
	}

	if ( arg[0] == '\0' || argument[0] == '\0' )
	{
		stc( "{cSayto whom what?{c\n\r", ch );
		return;
	}

	if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
	{
		stc( "{cThey aren't here.{c\n\r", ch );
		return;
	}

	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_hide );
	affect_strip( ch, gsn_sneak );

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
		argument = makedrunk(argument,ch);

	if (victim == ch)
	{
		act("{cYou mumble to yourself.",  ch, argument, victim, TO_CHAR );
		act("{c$n mumbles to $mself.",    ch, argument, victim, TO_ROOM );
		return;
	}

	act("{gYou say to $N {G'{g$t{G'{x",  ch, argument, victim, TO_CHAR );
	act("{g$n says to you {G'{g$t{G'{x", ch, argument, victim, TO_VICT );
	act("{g$n says to $N, {G'{g$t{G'{x",  ch, argument, victim, TO_NOTVICT);
	return;
}


void do_tell( CHAR_DATA *ch, char *argument )
{
	char arg[MIL],buf[MSL];
	char *comma_arg;
	CHAR_DATA *victim;

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}

	if ( IS_SET(ch->penalty_flags, PENAL_NOTELL) || IS_SET(ch->comm,COMM_DEAF))
	{
		stc( "{cYour message didn't get through.{c\n\r", ch );
		return;
	}

	if ( IS_SET(ch->comm, COMM_QUIET) )
	{
		stc( "{cYou must turn off quiet mode first.{c\n\r", ch);
		return;
	}

	if (IS_SET(ch->comm,COMM_DEAF))
	{
		stc("{cYou must turn off deaf mode first.{c{c\n\r",ch);
		return;
	}

	argument = one_argument( argument, arg );

	if (arg[0] != '\0' && !str_cmp(arg, "last"))
	{
		stc("Your most recent tells:\n\r", ch);
		show_channel_logs(ch, "tell");
		return;
	}

	if ( arg[0] == '\0' || argument[0] == '\0' )
	{
		stc( "{cTell whom what?{c\n\r", ch );
		return;
	}


	/* this next line looks kinda wierd, but it works. */
	for( comma_arg = one_comma_argument(arg, arg); arg[0] != '\0'; comma_arg = one_comma_argument( comma_arg, arg))
	{
		if ( ( victim = get_exactchar_world( ch, arg ) ) == NULL
		|| ( IS_NPC(victim) && victim->in_room != ch->in_room ) )
		{
			stc( "They aren't here.\n\r", ch );
			continue;
		}
		if ( !can_see( ch, victim ) )
		{
			stc( "They aren't here.\n\r", ch );
			continue;
		}

		if ( victim->desc == NULL && !IS_NPC(victim))
		{
			act("{c$N seems to have misplaced $S link...try again later.{c", ch,NULL,victim,TO_CHAR);
			sprintf(buf,"{G%s {Gtells you {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
			buf[0] = UPPER(buf[0]);
			add_buf(victim->pcdata->buffer,buf);
			continue;
		}

		if ((IS_SET(victim->comm,COMM_QUIET) || IS_SET(victim->comm,COMM_DEAF)) && !IS_IMMORTAL(ch))
		{
			act("{c$E is not receiving tells.{c", ch, 0, victim, TO_CHAR );
			continue;
		}

		if (IS_SET(victim->comm,COMM_AFK))
		{
			if (IS_NPC(victim))
			{
				act("{c$E is AFK, and not receiving tells.{c",ch,NULL,victim,TO_CHAR);
				continue;
			}

			if ( victim->pcdata->afkmsg[0] != '\0' )
			{
				act( "Message from $N", ch, NULL, victim, TO_CHAR );
				stc( victim->pcdata->afkmsg, ch );
				stc( "\n\r", ch );
				continue;
			}
			act("{c$E is AFK, but your tell will go through when $E returns.{c", ch,NULL,victim,TO_CHAR);
			sprintf(buf,"{c[{wAFK{c] {G%s tells you {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
			buf[0] = UPPER(buf[0]);
			add_buf(victim->pcdata->buffer,buf);
			continue;
		}

		if ( is_fighting(victim) && IS_SET(victim->auto_flags,AUTO_AUTOLOG) )
		{
			if (IS_NPC(victim))
			{
				act("$E is fighting, and cannot receive tells.",ch,NULL,victim,TO_CHAR);
				continue;
			}

			act("$N is fighting, but your tell will go through when $E returns.",ch,NULL,victim,TO_CHAR);
			sprintf(buf,"{c[{wFIGHT{c] {G%s {Gtells you {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
			buf[0] = UPPER(buf[0]);
			add_buf(victim->pcdata->buffer,buf);
			continue;
		}


		sprintf(buf, "{G%s {Gtells %s {g'{G%s{g'{x", ch->name, victim->name, argument);
		add_channel_log("tell", ch->name, victim->name, buf);
		act("{GYou tell $N {g'{G$t{g'{x", ch, argument, victim, TO_CHAR );
		act_new("{G$n {Gtells you {g'{G$t{g'{x",ch,argument,victim,TO_VICT,POS_DEAD);

		if ( !IS_NPC( ch ) )
		{
			ch->pcdata->last_tell_to = str_dup( arg );
		}
		victim->reply = ch;

		if ( !IS_NPC( victim ) && victim->timer > 5 )
			ptc( ch, "%s has been idle for a while...\n\r", victim->name );

		if ( !IS_NPC(ch) && IS_NPC(victim) && HAS_TRIGGER_MOB(victim,TRIG_SPEECH) )
			p_act_trigger( argument, victim, NULL, NULL, ch, NULL, NULL, TRIG_SPEECH );

	}
	return;
}

void do_retell( CHAR_DATA *ch, char *argument )
{
	char last_tell_to[MSL], buf[MSL];
	CHAR_DATA *victim;

	if ( IS_NPC( ch ) )
		return;

	if ( !ch->pcdata
	|| !ch->pcdata->last_tell_to
	|| ch->pcdata->last_tell_to == NULL
	|| ch->pcdata->last_tell_to[0] == '\0' )
	{
		stc("Last tell to data is null.\n\r",ch);
		return;
	}

	strcpy( last_tell_to, ch->pcdata->last_tell_to );

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}

	if ( IS_SET(ch->penalty_flags, PENAL_NOTELL) || IS_SET(ch->comm,COMM_DEAF))
	{
		stc( "Your message didn't get through.\n\r", ch );
		return;
	}

	if ( IS_SET(ch->comm, COMM_QUIET) )
	{
		stc( "You must turn off quiet mode first.\n\r", ch);
		return;
	}

	if (IS_SET(ch->comm,COMM_DEAF))
	{
		stc("You must turn off deaf mode first.\n\r",ch);
		return;
	}

	if ( ( victim = get_exactchar_world( ch, last_tell_to ) ) == NULL )
	{
		stc( "The person we last recorded for you is gone it seems.\n\r", ch );
		return;
	}

	if ( argument[0] == '\0' || !str_cmp( argument, "?" ) || !str_cmp( argument, "help" ) )
	{
		stc( "ReTell what?\n\r", ch );
		stc( "ReTell will send the argument to the last person you sent a tell to provided they are still online.\n\r", ch);
		stc( "You do not need to type their name.\n\r", ch );
		return;
	}


	if ( victim->desc == NULL && !IS_NPC(victim))
	{
		act("{c$N seems to have misplaced $S link...try again later.{c", ch,NULL,victim,TO_CHAR);
		sprintf(buf,"{G%s {Gtells you {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
		buf[0] = UPPER(buf[0]);
		add_buf(victim->pcdata->buffer,buf);
		return;
	}

	if ( !(IS_IMMORTAL(ch) && ch->level > LEVEL_IMMORTAL) && !IS_AWAKE(victim) )
	{
		act("{c$E can't hear you.{c", ch, 0, victim, TO_CHAR );
		return;
	}

	if ((IS_SET(victim->comm,COMM_QUIET) || IS_SET(victim->comm,COMM_DEAF))
	&& !IS_IMMORTAL(ch))
	{
		act("{c$E is not receiving tells.{c", ch, 0, victim, TO_CHAR );
		return;
	}

	if (IS_SET(victim->comm,COMM_AFK))
	{
		if (IS_NPC(victim))
		{
			act("{c$E is AFK, and not receiving tells.{c",ch,NULL,victim,TO_CHAR);
			return;
		}

		if ( victim->pcdata->afkmsg[0] != '\0' )
		{
			act( "Message from $N", ch, NULL, victim, TO_CHAR );
			stc( victim->pcdata->afkmsg, ch );
			stc( "\n\r", ch );
			return;
		}

		act("{c$E is AFK, but your tell will go through when $E returns.{c", ch,NULL,victim,TO_CHAR);
		sprintf(buf,"{c[{wAFK{c] {G%s {Gtells you {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
		buf[0] = UPPER(buf[0]);
		add_buf(victim->pcdata->buffer,buf);
		return;
	}

	if ( is_fighting(victim) && IS_SET(victim->auto_flags,AUTO_AUTOLOG) )
	{
		if (IS_NPC(victim))
		{
			act("$E is fighting, and cannot receive tells.",ch,NULL,victim,TO_CHAR);
			return;
		}

		act("$N is fighting, but your tell will go through when $E returns.",ch,NULL,victim,TO_CHAR);
		sprintf(buf,"{c[{wFIGHT{c] {G%s {Gtells you {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
		buf[0] = UPPER(buf[0]);
		add_buf(victim->pcdata->buffer,buf);
		return;
	}

	sprintf(buf, "{G%s {Gtells %s {g'{G%s{g'{x", ch->name, victim->name, argument);
	add_channel_log("tell", ch->name, victim->name, buf);
	act("{GYou tell $N {g'{G$t{g'{x", ch, argument, victim, TO_CHAR );
	act_new("{G$n {Gtells you {g'{G$t{g'{x",ch,argument,victim,TO_VICT,POS_DEAD);
	victim->reply = ch;

	if ( !IS_NPC( victim ) && victim->timer > 5 )
		ptc( ch, "%s has been idle for a while...\n\r", victim->name );

	if ( !IS_NPC(ch) && IS_NPC(victim) && HAS_TRIGGER_MOB(victim,TRIG_SPEECH) )
		p_act_trigger( argument, victim, NULL, NULL, ch, NULL, NULL, TRIG_SPEECH );

	return;
}


void do_reply( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char buf[MSL];

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("{cYou cannot use channels.{c\n\r",ch);
		return;
	}

	if ( IS_SET(ch->penalty_flags, PENAL_NOTELL) )
	{
		stc( "{cYour message didn't get through.\n\r{c", ch );
		return;
	}

	if ( ( victim = ch->reply ) == NULL )
	{
		stc( "{cThey aren't here.{c{c\n\r", ch );
		return;
	}

	if ((IS_SET(victim->comm,COMM_QUIET) || IS_SET(victim->comm,COMM_DEAF)) && !IS_IMMORTAL(ch))
	{
		act("{c$E is not receiving tells.{c", ch, 0, victim, TO_CHAR );
		return;
	}

	if ( !str_cmp( argument, "who" ) )
	{
		if ( !can_see( ch, victim ) )
		{
			if ( IS_IMMORTAL( victim ) )
				stc("An Immortal.\n\r", ch);
			else if ( IS_SET( victim->war_flags, WAR_MODERATOR ) )
				stc("A Moderator.\n\r", ch);
			else
				ptc( ch, "%s.\n\r", victim->name );
		}
		else
			ptc( ch, "%s.\n\r", victim->name );
		return;
	}

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
		argument = makedrunk(argument,ch);

	if ( victim->desc == NULL && !IS_NPC(victim))
	{
		act("{c$N seems to have misplaced $S link...try again later.{c", ch,NULL,victim,TO_CHAR);
		return;
	}

	if ( !IS_IMMORTAL(ch) && !IS_AWAKE(victim) )
	{
		act("{c$E can't hear you.{c", ch, 0, victim, TO_CHAR );
		return;
	}

	if ((IS_SET(victim->comm,COMM_QUIET) || IS_SET(victim->comm,COMM_DEAF)) && !IS_IMMORTAL(ch) && !IS_IMMORTAL(victim))
	{
		act_new( "{c$E is not receiving tells.{c", ch, 0, victim, TO_CHAR,POS_DEAD);
		return;
	}


	if (IS_SET(victim->comm,COMM_AFK))
	{
		if (IS_NPC(victim))
		{
			act_new("{c$E is AFK, and not receiving tells.{c", ch,NULL,victim,TO_CHAR,POS_DEAD);
			return;
		}

		act_new("{c$E is AFK, but your tell will go through when $E returns.{c", ch,NULL,victim,TO_CHAR,POS_DEAD);
		sprintf(buf,"{G%s {Greplies, {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
		buf[0] = UPPER(buf[0]);
		add_buf(victim->pcdata->buffer,buf);
		return;
	}

	if ( is_fighting(victim) )
	{
		if (IS_NPC(victim))
		{
			act("$E is fighting, and cannot receive tells.",ch,NULL,victim,TO_CHAR);
			return;
		}

		act("$E is fighting, but your tell will go through when $E returns.",ch,NULL,victim,TO_CHAR);
		sprintf(buf,"{c[{wFIGHT{c] {G%s {Greplies, {g'{G%s{g'{x\n\r",PERS(ch,victim),argument);
		buf[0] = UPPER(buf[0]);
		add_buf(victim->pcdata->buffer,buf);
		return;
	}

	sprintf(buf, "{G%s {Gtells %s {g'{G%s{g'{x", ch->name, victim->name, argument);
	add_channel_log("tell", ch->name, victim->name, buf);
	act_new("{GYou reply to $N {g'{G$t{g'{x",ch,argument,victim,TO_CHAR,POS_DEAD);
	act_new("{G$n {Greplies, {g'{G$t{g'{x",ch,argument,victim,TO_VICT,POS_DEAD);
	victim->reply = ch;
	return;
}



void do_yell( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;

	if ( IS_SET( ch->penalty_flags, PENAL_NOCHATS ) )
	{
		stc("Your channel privileges have been revoked.\n\r", ch);
		return;
	}

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}

	if ( IS_SET(ch->penalty_flags, PENAL_NOSHOUT) )
	{
		stc( "You can't yell.\n\r", ch );
		return;
	}

	if ( argument[0] == '\0' )
	{
		stc( "Yell what?\n\r", ch );
		return;
	}

	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_hide );
	affect_strip( ch, gsn_sneak );

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
		argument = makedrunk(argument,ch);

	act("{cYou yell '$t'{c",ch,argument,NULL,TO_CHAR);

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( d->connected == CON_PLAYING
		&&   d->character != ch
		&&   d->character->in_room != NULL
		&&   d->character->in_room->area == ch->in_room->area
		&&   !IS_SET(d->character->comm,COMM_QUIET) )
		{
			act("$n yells '$t'",ch,argument,d->character,TO_VICT);
		}
	}
	return;
}


void do_emote( CHAR_DATA *ch, char *argument )
{
	if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE) )
	{
		stc( "You can't show your emotions.\n\r", ch );
		return;
	}

	if ( argument[0] == '\0' )
	{
		stc( "Emote what?\n\r", ch );
		return;
	}

	affect_strip( ch, gsn_invis );
	affect_strip( ch, gsn_hide );
	affect_strip( ch, gsn_sneak );

	MOBtrigger = FALSE;

	if(argument[0] != '\'')
	{
		act("$n $T", ch, NULL, argument, TO_ROOM );
		act("$n $T", ch, NULL, argument, TO_CHAR );
	}
	else
	{
		act("$n$T", ch, NULL, argument, TO_ROOM );
		act("$n$T", ch, NULL, argument, TO_CHAR );
	}
	if ( !IS_NPC( ch ) )
	{
		ch->pcdata->summary[SUM_MOTES] += 1;
		ch->pcdata->summary[SUM_MOTES_EVER] += 1;
	}
	MOBtrigger = TRUE;
	return;
}


void do_pmote( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *vch;
	char *letter,*name;
	char last[MAX_INPUT_LENGTH], temp[MAX_STRING_LENGTH];
	int matches = 0;

	if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE) )
	{
		stc( "You can't show your emotions.\n\r", ch );
		return;
	}

	if ( argument[0] == '\0' )
	{
		stc( "Emote what?\n\r", ch );
		return;
	}

	act("$n $t", ch, argument, NULL, TO_CHAR );

	for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
	{
		if (vch->desc == NULL || vch == ch)
			continue;

		if ((letter = strstr(argument,vch->name)) == NULL)
		{
			MOBtrigger = FALSE;
			act("$N $t",vch,argument,ch,TO_CHAR);
			MOBtrigger = TRUE;
			continue;
		}

		strcpy(temp,argument);
		temp[strlen(argument) - strlen(letter)] = '\0';
		last[0] = '\0';
		name = vch->name;

		for (; *letter != '\0'; letter++)
		{
			if (*letter == '\'' && matches == strlen(vch->name))
			{
				strcat(temp,"r");
				continue;
			}

			if (*letter == 's' && matches == strlen(vch->name))
			{
				matches = 0;
				continue;
			}

			if (matches == strlen(vch->name))
				matches = 0;

			if (*letter == *name)
			{
				matches++;
				name++;
				if (matches == strlen(vch->name))
				{
					strcat(temp,"you");
					last[0] = '\0';
					name = vch->name;
					continue;
				}
				strncat(last,letter,1);
				continue;
			}

			matches = 0;
			strcat(temp,last);
			strncat(temp,letter,1);
			last[0] = '\0';
			name = vch->name;
		}

		MOBtrigger = FALSE;
		act("$N $t",vch,temp,ch,TO_CHAR);

		if ( !IS_NPC( ch ) )
		{
				ch->pcdata->summary[SUM_MOTES] += 1;
				ch->pcdata->summary[SUM_MOTES_EVER] += 1;
		}
		MOBtrigger = TRUE;
	}
	return;
}


/*
 * All the posing stuff.
 */
struct	pose_table_type
{
	char * message[2*MAX_CLASS];
};

const	struct	pose_table_type	pose_table	[]	=
{
	{
		{
			"{cYou sizzle with the True Source.{c",
			"{c$n sizzles with the True Source.{c",
			"{cYou feel very holy.{c",
			"{c$n looks very holy.{c",
			"{cYou perform a small card trick.{c",
			"{c$n performs a small card trick.{c",
			"{cYou show your bulging muscles.{c",
			"{c$n shows $s bulging muscles.{c"
		}
	},

	{
		{
			"{cYou turn into a butterfly, then return to your normal shape.{c",
			"{c$n turns into a butterfly, then returns to $s normal shape.{c",
			"{cYou nonchalantly turn wine into water.{c",
			"{c$n nonchalantly turns wine into water.{c",
			"{cYou wiggle your ears alternately.{c",
			"{c$n wiggles $s ears alternately.{c",
			"{cYou crack nuts between your fingers.{c",
			"{c$n cracks nuts between $s fingers.{c"
		}
	},

	{
		{
			"{cBlue sparks fly from your fingers.{c",
			"{cBlue sparks fly from $n's fingers.{c",
			"{cA halo appears over your head.{c",
			"{cA halo appears over $n's head.{c",
			"{cYou nimbly tie yourself into a knot.{c",
			"{c$n nimbly ties $mself into a knot.{c",
			"{cYou grizzle your teeth and look mean.{c",
			"{c$n grizzles $s teeth and looks mean.{c"
		}
	},

	{
		{
			"{cBlack spots of saa dance in your eyes.{c",
			"{cBlack spots of saa dance in $n's eyes.{c",
			"{cYou recite words of wisdom.{c",
			"{c$n recites words of wisdom.{c",
			"{cYou juggle with daggers, apples, and eyeballs.{c",
			"{c$n juggles with daggers, apples, and eyeballs.{c",
			"{cYou hit your head, and your eyes roll.{c",
			"{c$n hits $s head, and $s eyes roll.{c"
		}
	},

	{
		{
			"{cA slimy green monster appears before you and bows.{c",
			"{cA slimy green monster appears before $n and bows.{c",
			"{cDeep in prayer, you levitate.{c",
			"{cDeep in prayer, $n levitates.{c",
			"{cYou steal the underwear off every person in the room.{c",
			"{cYour underwear is gone!  $n stole it!",
			"{cCrunch, crunch -- you munch a bottle.{c",
			"{cCrunch, crunch -- $n munches a bottle.{c"
		}
	},

	{
		{
			"{cYou turn everybody into a little pink elephant.{c",
			"{cYou are turned into a little pink elephant by $n.{c",
			"{cThe Creator consults you.{c",
			"{cThe Creator consults $n.{c",
			"{cThe dice roll ... and you win again.{c",
			"{cThe dice roll ... and $n wins again.{c",
			"{c... 98, 99, 100 ... you do pushups.{c",
			"{c... 98, 99, 100 ... $n does pushups.{c"
		}
	},

	{
		{
			"{cA small ball of light dances on your fingertips.{c",
			"{cA small ball of light dances on $n's fingertips.{c",
			"{cYour body glows with an unearthly light.{c",
			"{c$n's body glows with an unearthly light.{c",
			"{cYou count the money in everyone's pockets.{c",
			"{cCheck your money, $n is counting it.{c",
			"{cPerrin Aybara admires your physique.{c",
			"{cPerrin Aybara admires $n's physique.{c"
		}
	},

	{
		{
			"{cSmoke and fumes leak from your nostrils.{c",
			"{cSmoke and fumes leak from $n's nostrils.{c",
			"{cA spot light hits you.{c",
			"{cA spot light hits $n.{c",
			"{cYou balance a pocket knife on your tongue.{c",
			"{c$n balances a pocket knife on your tongue.{c",
			"{cWatch your feet, you are juggling granite boulders.{c",
			"{cWatch your feet, $n is juggling granite boulders.{c"
		}
	},

	{
		{
			"{cThe light flickers as you rap in magical languages.{c",
			"{cThe light flickers as $n raps in magical languages.{c",
			"{cEveryone levitates as you pray.{c",
			"{cYou levitate as $n prays.{c",
			"{cYou produce a coin from everyone's ear.{c",
			"{c$n produces a coin from your ear.{c",
			"{cOomph!  You squeeze water out of a granite boulder.{c",
			"{cOomph!  $n squeezes water out of a granite boulder.{c"
		}
	},

	{
		{
			"{cYour head disappears.{c",
			"{c$n's head disappears.{c",
			"{cA cool breeze refreshes you.{c",
			"{cA cool breeze refreshes $n.{c",
			"{cYou step behind your shadow.{c",
			"{c$n steps behind $s shadow.{c",
			"{cYou pick your teeth with a spear.{c",
			"{c$n picks $s teeth with a spear.{c"
		}
	},

	{
		{
			"{cA darkhound singes your hair.{c",
			"{cA darkhound singes $n's hair.{c",
			"{cThe sun pierces through the clouds to illuminate you.{c",
			"{cThe sun pierces through the clouds to illuminate $n.{c",
			"{cYour eyes dance with greed.{c",
			"{c$n's eyes dance with greed.{c",
			"{cEveryone is swept off their feet by your hug.{c",
			"{cYou are swept off your feet by $n's hug.{c"
		}
	},

	{
		{
			"{cThe sky changes color to match your eyes.{c",
			"{cThe sky changes color to match $n's eyes.{c",
			"{cThe ocean parts before you.{c",
			"{cThe ocean parts before $n.{c",
			"{cYou deftly steal everyone's weapon.{c",
			"{c$n deftly steals your weapon.{c",
			"{cYour karate chop splits a tree.{c",
			"{c$n's karate chop splits a tree.{c"
		}
	},

	{
		{
			"{cThe stones dance to your command.{c",
			"{cThe stones dance to $n's command.{c",
			"{cA thunder cloud kneels to you.{c",
			"{cA thunder cloud kneels to $n.{c",
			"{cMatrim Cauthon buys you a beer.{c",
			"{cMatrim Cauthon buys $n a beer.{c",
			"{cA strap of your armor breaks over your mighty thews.{c",
			"{cA strap of $n's armor breaks over $s mighty thews.{c"
		}
	},

	{
		{
			"{cThe heavens and grass change colour as you smile.{c",
			"{cThe heavens and grass change colour as $n smiles.{c",
			"{cThe Green Man speaks to you.{c",
			"{cThe Green Man speaks to $n.{c",
			"{cEveryone's pocket explodes with your fireworks.{c",
			"{cYour pocket explodes with $n's fireworks.{c",
			"{cA boulder cracks at your frown.{c",
			"{cA boulder cracks at $n's frown.{c"
		}
	},

	{
		{
			"{cEveryone's clothes are transparent, and you are laughing.{c",
			"{cYour clothes are transparent, and $n is laughing.{c",
			"{cAn eye in a pyramid winks at you.{c",
			"{cAn eye in a pyramid winks at $n.{c",
			"{cEveryone discovers your dagger a centimeter from their eye.{c",
			"{cYou discover $n's dagger a centimeter from your eye.{c",
			"{cAn army of trollocs arrive to do your bidding.{c",
			"{cAn army of trollocs arrive to do $n's bidding.{c"
		}
	},

	{
		{
			"{cTel'aran'rhiod swallows you.{c",
			"{cTel'aran'rhiod swallows $n.{c",
			"{cBa'alzamon offers you a glass of water.{c",
			"{cBa'alzamon offers $n a glass of water.{c",
			"{cWhere did you go?",
			"{cWhere did $n go?",
			"{cFour darkhounds bring in your chariot.{c",
			"{cFour darkhounds bring in $n's chariot.{c"
		}
	},

	{
		{
			"{cThe world shimmers in time with your whistling.{c",
			"{cThe world shimmers in time with $n's whistling.{c",
			"{cThe Creator gives you a staff.{c",
			"{cThe Creator gives $n a staff.{c",
			"{cClick.{c",
			"{cClick.{c",
			"{cRand al'Thor asks you to relieve him.{c",
			"{cRand al'Thor asks $n to relieve him.{c"
		}
	}
};



void do_pose( CHAR_DATA *ch, char *argument )
{
	int level;
	int pose;

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}

	if ( IS_NPC(ch) )
		return;

	level = UMIN( ch->level, sizeof(pose_table) / sizeof(pose_table[0]) - 1 );
	pose  = number_range(0, level);

	act( pose_table[pose].message[2*ch->class+0], ch, NULL, NULL, TO_CHAR );
	act( pose_table[pose].message[2*ch->class+1], ch, NULL, NULL, TO_ROOM );

	return;
}

void do_typo( CHAR_DATA *ch, char *argument )
{
	append_file( ch, TYPO_FILE, argument );
	stc( "Typo logged.\n\r", ch );
	return;
}

void do_rent( CHAR_DATA *ch, char *argument )
{
	stc( "There is no rent here.  Just save and quit.\n\r", ch );
	return;
}


void do_qui( CHAR_DATA *ch, char *argument )
{
	stc( "If you want to QUIT, you have to spell it out.\n\r", ch );
	return;
}

void world_alert ( char *msg, ... )
{
	DESCRIPTOR_DATA *d;
	char buf[MSL], buf2[MSL];
	va_list args;

	va_start( args, msg );
	vsprintf(buf, msg, args );
	va_end( args );

	sprintf( buf2, "%s{x\n\r", buf );
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( d->connected == CON_PLAYING )
			stc( buf2, d->character );
	}
}

void do_quit( CHAR_DATA *ch, char *argument )
{
	char mysql_buf[MSL];
	DESCRIPTOR_DATA *d,*d_next;
	int id;

	if ( IS_NPC(ch) )
		return;

	if ( is_fighting( ch ))
	{
		stc( "No way! {wYou are fighting.{x\n\r", ch );
		return;
	}
	if( IS_SET( ch->war_flags, WAR_BATTLE ) )
	{
		stc( "But you're in a battle!", ch );
		return;
	}
	if( IS_SET( ch->war_flags, WAR_TOURNEY ) )
	{
		stc( "But you're in a tourney!", ch );
		return;
	}
	if( ch->pcdata->dfhunt_player_info[0] > 0 && ch->pcdata->dfhunt_player_info[0] != 5 ) /* not in hunt or link dead */
	{
		do_function(ch, &do_dfhunt, "leave quit_force");
	}
	if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
	{
		stc("You'll have to leave the tag game first.\n\r", ch);
		return;
	}
	if ( ch->position  < POS_STUNNED  )
	{
		stc( "You're not DEAD yet.\n\r", ch );
		if ( ch->hit < 0 )
		{
			ch->hit = 1;
			stc( "You appear to have caught a bug. If you are not fixed now, please alert a staff member as soon as you can.\n\r", ch );
		}
		return;
	}
	if ( auction_list != NULL && (auction_list->high_bidder == ch || auction_list->owner == ch) )
	{
		stc("You still have a stake in the auction!\n\r",ch);
		return;
	}
	if (IS_SET( ch->op_flags, OP_GRASPED))
	{
		do_function(ch, &do_release, "");
		stc("You release your hold on the True Source.\n\r",ch);
	}
	check_leaderboard( ch );
	if ( ch->pcdata->in_vault )
	{
		do_function( ch, &do_locker, "close" );
	}

	stc("You return home after a hard day of adventuring.\n\r",ch);
	world_alert( "%s{x returns home after a hard day of adventuring.", ch->colorname );
	sprintf( log_buf, "%s has quit.", ch->name );
	log_string( log_buf );

	/*
	 * After extract_char the ch is no longer valid!
	 */
	clear_irc( ch );
	//save_char_obj( ch );
	save_char_obj(ch, FALSE, FALSE);

	sprintf(mysql_buf, "update characters set online = 0 where name = '%s'", ch->name);
	//mysql_real_query(result_db, mysql_buf, strlen(mysql_buf)); // prool fool


	/* Free note that might be there somehow */
	if (ch->pcdata->in_progress)
		free_note (ch->pcdata->in_progress);

	id = ch->id;
	d = ch->desc;
	extract_char( ch, TRUE );

	if ( d != NULL )
		close_socket( d );

	/* toast evil cheating bastards */
	for (d = descriptor_list; d != NULL; d = d_next)
	{
		CHAR_DATA *tch;

		d_next = d->next;
		tch = d->original ? d->original : d->character;
		if (tch && tch->id == id)
		{
			extract_char(tch,TRUE);
			close_socket(d);
		}
	}
	return;
}

void save_db_character(CHAR_DATA *ch)
{
	char sql_update[MSL*4], sql_insert[MSL*4];
	int remorts;

	if(IS_NPC(ch))
		return;

	remorts = ch->pcdata->rebirth;
	if (IS_IMMORTAL(ch)) remorts = -1;

	sprintf(sql_update, "update characters set last_name = \"%s\", title = \"%s\", online = 1, remorts = %d, level = %d, blademaster = %d, race = '%s' where name = '%s'",
		sql_escape_string(ch->pcdata->last_name),
		ch->pcdata->title, 
		remorts,
		ch->level,
		blademaster(ch) ? 1 : 0,
		pc_race_table[ch->race].who_name,
		ch->name);

	sprintf(sql_insert, "insert into characters values('', \"%s\", \"%s\", \"%s\", '1', '%d', '%d', '%d', '%s', '%s')",
		ch->name,
		sql_escape_string(ch->pcdata->last_name),
		ch->pcdata->title, 
		remorts,
		ch->level,
		blademaster(ch) ? 1 : 0,
		ch->desc ? ch->desc->host : "0.0.0.0",
		pc_race_table[ch->race].who_name);

#if 0 // prool fool
	if( mysql_real_query(result_db, sql_update, strlen(sql_update)) == 0)
		if( mysql_affected_rows(result_db) < 1)
			if( mysql_real_query(result_db, sql_insert, strlen(sql_insert)) == 0)
				if( mysql_affected_rows(result_db) < 1)
					log_string("SQL-CHAR: Save Error on insert");
#endif


	// Update host.. but only if they are connected.

	if (ch->desc)
	{
		sprintf(sql_update, "update characters set host = '%s' where name = '%s'", ch->desc->host, ch->name);
		//mysql_real_query(result_db, sql_update, strlen(sql_update)); // prool fool
	}

	return;
}


void do_save( CHAR_DATA *ch, char *argument )
{
	if ( IS_NPC(ch) )
		return;

	compose_title(ch);

	stc("{cUpdating the player record, please wait...{c\n\r", ch);
	save_char_obj(ch, FALSE, FALSE);
	save_db_character(ch);
	stc("{cDone!{c\n\r", ch);
	return;
}


void do_follow( CHAR_DATA *ch, char *argument )
{
	char arg[MIL];
	CHAR_DATA *victim;

	one_argument( argument, arg );

	if ( arg[0] == '\0' )
	{
		stc( "Who do you want to follow?\n\r", ch );
		return;
	}

	if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
	{
		stc( "They aren't here.\n\r", ch );
		return;
	}

	if ( !victim->desc )
	{
		stc("That person is battling the Net Beast at the moment.\n\r",ch);
		return;
	}

	if ( IS_NPC( victim ) )
	{
		stc("You cannot follow mobiles.\n\r",ch);
		return;
	}

	if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master != NULL )
	{
		act("But you'd rather follow $N!", ch, NULL, ch->master, TO_CHAR );
		return;
	}

	if ( victim == ch )
	{
		if(ch->leader != NULL && ch->leader != ch )
		{
			ptc(ch->leader,"%s no longer follows you.\n\r",ch->name);
		}
		ch->leader = ch;
		ch->master = ch;
		return;
	}

	if (!IS_NPC(victim) && IS_SET(victim->act,PLR_NOFOLLOW) && !IS_IMMORTAL(ch))
	{
		act("$N doesn't seem to want any followers.\n\r",ch,NULL,victim, TO_CHAR);
		return;
	}

	if ( victim->pcdata->accept_follower_name != NULL )
	{
		act("$N already has a pending query to follow $M.\n\r",ch,NULL,victim,TO_CHAR);
		return;
	}

	if (!IS_NPC(ch))
	{
		ptc(ch,"You ask %s if you may join their group.\n\r", victim->name);
		ptc(victim,"%s wishes to join your group. Will you allow them?\n\r If so, type Group <char name> ",ch->name);
		victim->pcdata->accept_follower_name = ch;
		WAIT_STATE( ch, PULSE_VIOLENCE * 2 );
	}
	else
	{
		act("$n now follows you.", ch, NULL, victim, TO_VICT);
		act("$n starts following $N.", ch, NULL, victim, TO_NOTVICT);
		ch->leader = victim;
		ch->master = victim;
	}

	return;
}


void add_follower( CHAR_DATA *ch, CHAR_DATA *master )
{
	if ( ch->master != NULL )
	{
		bug( "Add_follower: non-null master.", 0 );
		return;
	}

	ch->master        = master;
	ch->leader        = master;

	if ( can_see( master, ch ) )
		act("$n follows along behind you.", ch, NULL, master, TO_VICT );

	act("You follow along behind $N.",  ch, NULL, master, TO_CHAR );

	return;
}



void stop_follower( CHAR_DATA *ch )
{
	if ( ch->master == NULL && IS_NPC(ch) )
	{
		bug( "Stop_follower: null master.", 0 );
		return;
	}

	if ( ch->leader == NULL && !IS_NPC(ch) )
	{
		bug( "Stop_follower: null master.", 0 );
		return;
	}

	if ( IS_AFFECTED(ch, AFF_CHARM) )
	{
		REMOVE_BIT( ch->affected_by, AFF_CHARM );
		affect_strip( ch, gsn_charm_person );
	}

	if ( IS_NPC( ch ) && ch->master != NULL )
	{
		if ( can_see( ch->master, ch ) && ch->in_room != NULL)
		{
			act("$n stops following you.",ch, NULL, ch->master, TO_VICT    );
			act("You stop following $N.",ch, NULL, ch->master, TO_CHAR    );
		}
	}
	else if ( !IS_NPC( ch ) && ch->leader != ch )
	{
		if ( can_see( ch->leader, ch ) && ch->in_room != NULL)
		{
			act("$n stops following you.",ch, NULL, ch->leader, TO_VICT    );
			act("You stop following $N.",ch, NULL, ch->leader, TO_CHAR    );
		}
	}
	if ( ch->master != NULL )
	{
		if ( ch->master->pet != NULL )
		{
			if (ch->master->pet == ch)
				ch->master->pet = NULL;
		}
	}

	ch->master = ch;
	ch->leader = ch;
	return;
}

/* nukes charmed monsters and pets */
void nuke_pets( CHAR_DATA *ch )
{
	CHAR_DATA *pet;

	if (IS_NPC(ch))
		return;

	if ((pet = ch->pet) != NULL)
	{
		stop_follower(pet);

		if (pet->in_room != NULL)
			act("{c$N slowly fades away.",ch,NULL,pet,TO_NOTVICT);

		extract_char(pet,TRUE);
	}

	ch->pet = NULL;

	/* Mounts */
	if ( ch->mount && (ch->mount->in_room == ch->in_room || ch->mount->in_room==NULL) )
	{
		pet = ch->mount;
		do_dismount(ch, "");

		if (pet->in_room != NULL)
			act("$N slowly fades away.",ch,NULL,pet,TO_NOTVICT);
		else
			log_string("void nuke_pets: Extracting null pet");

		ch->mount = NULL;
		ch->riding = FALSE;
		extract_char(pet, TRUE);
	}
	else if (ch->mount)
	{
		ch->mount->mount = NULL;
		ch->mount->riding = FALSE;
	}
	ch->mount = NULL;

	return;
}



void die_follower( CHAR_DATA *ch )
{
	CHAR_DATA *fch;

	if ( ch->master != NULL )
	{
		if (ch->master->pet == ch)
			ch->master->pet = NULL;
		stop_follower( ch );
	}

	ch->leader = ch;

	for ( fch = char_list; fch != NULL; fch = fch->next )
	{
		if ( fch->master == ch )
			stop_follower( fch );
		if ( fch->leader == ch )
			fch->leader = fch;
	}

	return;
}



void do_order( CHAR_DATA *ch, char *argument )
{
	char buf[MSL];
	char arg[MIL],arg2[MIL];
	CHAR_DATA *victim;
	CHAR_DATA *och;
	CHAR_DATA *och_next;
	bool found, cmdOk = FALSE;
	bool fAll, fUnto = FALSE;
	int MaxOrder = 4, i;

	/* acceptable order commands */
	char *order_list[] = {
		"sit", "rest", "stand", "recall", "leave", "kill"
	};

	argument = one_argument( argument, arg );
	one_argument(argument,arg2);

	if ( arg[0] == '\0' || argument[0] == '\0' )
	{
		stc( "Order whom to do what?\n\r", ch );
		return;
	}

	if ( IS_AFFECTED( ch, AFF_CHARM ) )
	{
		stc( "You feel like taking, not giving, orders.\n\r", ch );
		return;
	}

	if ( !str_cmp( arg, "all" ) )
	{
		fAll   = TRUE;
		victim = NULL;
	}
	else
	{
		fAll   = FALSE;

		/* Mounts */
		if (!strcmp(arg, "mount"))
		{
			if (!ch->mount)
			{
				stc("Your don't have a mount.\n\r", ch);
				return;
			}
			if (ch->mount->in_room != ch->in_room)
			{
				stc("Your mount isn't here!\n\r", ch);
				return;
			}
			else
			{
				victim = ch->mount;
			}
		}
		else if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
		{
			stc( "They aren't here.\n\r", ch );
			return;
		}

		if ( victim == ch )
		{
			stc( "Aye aye, right away!\n\r", ch );
			return;
		}

		if ( check_social( victim, argument, argument ) )
			cmdOk = TRUE;
		else
		{
			for ( i = 0; i < MaxOrder; i++ )
			{
				if ( !str_cmp( argument, order_list[i] ) )
					cmdOk = TRUE;
			}
		}
		if ( !cmdOk )
		{
			stc("\n\rThe command you issued is either not a valid command,\n\r",ch);
			stc("or is not an acceptable command.\n\r",ch);
			stc("\n\rValid commands are:\n\r\n\r",ch);
			stc("socials (without a target)\n\r",ch);
			for ( i = 0; i < MaxOrder; i++ )
			{
				ptc( ch, order_list[i] );
				stc("\n\r",ch);
			}
			fUnto = TRUE;
		}
		if ( victim->mount == ch && !fUnto )
		{
			if ( !mount_success(ch, victim, FALSE) )
			{
				act("$N ignores your orders.", ch, NULL, victim, TO_CHAR);
				return;
			}
			else
			{
				ptc(ch,"%s orders you to '%s'.", ch->name, argument);
				interpret( victim, argument );
				return;
			}
		}
		if ( !IS_AFFECTED(victim,AFF_CHARM) )
		{
			stc( "Do it yourself!\n\r", ch );
			fUnto = TRUE;
		}
		if ( victim->master != ch )
		{
			stc( "You feel yourself being ignored.\n\r", ch );
			fUnto = TRUE;
		}
	}

	found = FALSE;
	for ( och = ch->in_room->people; och != NULL; och = och_next )
	{
		if ( fUnto )
			break;
		och_next = och->next_in_room;

		if ( IS_AFFECTED(och, AFF_CHARM) && och->master == ch && ( fAll || och == victim ) )
		{
			found = TRUE;
			sprintf( buf, "$n orders you to '%s'.", argument );
			act( buf, ch, NULL, och, TO_VICT );
			interpret( och, argument );
		}
	}

	/* get back at those who try to hack or bug abuse */
	if ( fUnto )
	{
		stc("There is a such thing as going too far.\n\r",ch );
		interpret( ch, argument );
	}

	if ( found )
	{
		WAIT_STATE(ch,PULSE_VIOLENCE);
		stc( "Ok.\n\r", ch );
	}
	else
		stc( "You have no followers here.\n\r", ch );
	return;
}



void do_group( CHAR_DATA *ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	char arg2[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	bool lynchmob = FALSE;

	argument = one_argument( argument, arg );
	argument = one_argument( argument, arg2 );

	if ( gevent->running && gevent->num == 15 ) /* Lynch Mob */
		lynchmob = TRUE;

    if ( arg[0] == '\0' )
    {
		CHAR_DATA *gch;
		CHAR_DATA *leader;

		leader = (ch->leader != NULL) ? ch->leader : ch;
		ptc( ch, "{c%s's group:{c\n\r", PERS(leader, ch) );

		for ( gch = char_list; gch != NULL; gch = gch->next )
		{
		    if ( is_same_group( gch, ch ) )
		    {

			int hp_perc = 0, op_perc = 0;
			if (get_maxhit(gch) > 0)
				hp_perc = gch->hit * 100 / get_maxhit(gch);
			if (get_maxstam(gch) > 0)
				op_perc = gch->stam * 100 / get_maxstam(gch);

			ptc( ch,"{b[{w%3d{b]%s {w%-16s {wHealth: {W%s%3d%{w Stamina: {W%s%3d%{x\n\r",
				gch->level, gch->level == 1000 ? "" : " ",
			    capit_colorsafe( PERS(gch, ch) ),
			    hp_perc >= 65 ? "{g" : hp_perc >= 25 ? "{y" : "{r",
			    hp_perc,
			    op_perc >= 65 ? "{g" : op_perc >= 25 ? "{y" : "{r",
			    op_perc);
		    }
		}
		return;
    }

    if ( !str_cmp( arg, "disband" ) )
    {
		CHAR_DATA *grp;
		for ( grp = ch->in_room->people; grp != NULL; grp = grp->next_in_room )
		{
			if ( !IS_NPC( grp ) && grp->leader == ch )
			{
				ptc( grp, "%s disbands the group, leaving you leaderless.\n\r", ch->name );
				grp->leader = grp;
				grp->master = grp;
			}
		}
		ch->leader = ch;
		ch->master = ch;
		return;
	}

	if ( ( victim = get_char_world( ch, arg ) ) == NULL )
	{
		stc("Who?", ch);
		return;
	}
	if ( IS_NPC( victim ) )
	{
		stc("Not on mobs.\n\r", ch);
		return;
	}

    if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
    {
		stc( "{cThey aren't here.{c\n\r", ch );
		return;
    }

    if ( ch->master != ch || ch->leader != ch )
    {
		stc( "{cBut you are following someone else!{c\n\r", ch );
		return;
    }

    if (IS_AFFECTED(victim,AFF_CHARM))
    {
        stc("{cYou can't remove charmed mobs from your group.{c\n\r",ch);
        return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM))
    {
    	act_new("{cYou like your master too much to leave $m!",
	    ch,NULL,victim,TO_VICT,POS_SLEEPING);
    	return;
    }

    if ( !str_cmp( arg2, "remove" ) )
    {
		if ( !is_same_group( victim, ch ) )
		{
			stc( "You're not in the same group.\n\r", ch );
			return;
		}
		if ( victim->leader != ch )
		{
			stc( "You're not their leader.\n\r", ch );
			return;
		}
		ptc( victim, "%s removes you from the group, leaving you leaderless.\n\r", ch->name );
		ptc( ch, "You remove %s from the group, leaving them leaderless.\n\r", victim->name );
		victim->leader = victim;
		victim->master = victim;
		return;
	}

    if ( !str_cmp( arg2, "leader" ) )
    {
		CHAR_DATA *grp;
		if ( !is_same_group( victim, ch ) )
		{
			stc( "You're not in the same group.\n\r", ch );
			return;
		}
		if ( victim->leader != ch )
		{
			stc( "You're not their leader.\n\r", ch );
			return;
		}
		for ( grp = ch->in_room->people; grp != NULL; grp = grp->next_in_room )
		{
			if ( !IS_NPC( grp ) && grp->leader == ch )
			{
				ptc( grp, "%s passes leadership of the group to %s, and you follow.\n\r", ch->name, victim->name );
				grp->leader = victim;
				grp->master = victim;
			}
		}
		ch->leader = victim;
		ch->master = victim;
		return;
	}

	if ( is_same_group( victim, ch ) && ch != victim )
	{
		if ( victim->master != ch && ch != victim )
		{
			act_new("{c$N isn't following you.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
			return;
		}
		victim->leader = victim;
		victim->master = victim;
		act_new("{c$n removes $N from $s group.",ch,NULL,victim,TO_NOTVICT,POS_RESTING);
		act_new("{c$n removes you from $s group.",ch,NULL,victim,TO_VICT,POS_SLEEPING);
		act_new("{cYou remove $N from your group.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
		return;
	}
	if ( ch->pcdata->accept_follower_name == victim )
	{
		CHAR_DATA *gch;
		if ( victim->master != victim || ( victim->leader != victim && victim->leader != ch ) )
		{
			stc( "{cBut they are following someone else!{c\n\r", ch );
			return;
		}
		/* while Lynch Mob is running, grouping anyone is allowed - Dalsor 04.23.2013 */
		if ( !lynchmob )
		{
			if(is_alt_of(ch,victim))
			{
				stc( "{cYou can not group with your alts.\n\r", ch );
				return;
			}
			for ( gch = char_list; gch != NULL; gch = gch->next )
			{
				if ( is_same_group( gch, ch) && (is_alt_of(gch,victim)))
				{
					stc( "{cThey can not group with their alts.\n\r", ch );
					return;
				}
			}
		}
		/*add_follower( victim, ch );*/
		victim->leader = ch;
		victim->master = ch;
		act_new("{c$N joins $n's group.",ch,NULL,victim,TO_NOTVICT,POS_RESTING);
		act_new("{cYou join $n's group.",ch,NULL,victim,TO_VICT,POS_SLEEPING);
		act_new("{c$N joins your group.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
		ch->pcdata->accept_follower_name = NULL;
		ch->pcdata->accept_follower = FALSE;
		return;
	}
	else
	{
		stc("They aren't following you.\n\r",ch);
		return;
	}
}

int MAX_IP_ALLOW = 20;

bool ip_group_allow( CHAR_DATA *ch, CHAR_DATA *victim )
{
	int i = 0;
	bool found = FALSE;

	for ( i = 0; i < MAX_IP_ALLOW; i++ )
	{
		if ( ip_groups_table[i].name == NULL )
		{
			found = FALSE;
			break;
		}
		if ( !str_cmp( ch->name, ip_groups_table[i].name ) )
		{
			if ( !str_cmp( victim->name, ip_groups_table[i].withName ) )
			{
				found = TRUE;
				break;
			}
		}
		else if ( !str_cmp( victim->name, ip_groups_table[i].name ) )
		{
			if ( !str_cmp( ch->name, ip_groups_table[i].withName ) )
			{
				found = TRUE;
				break;
			}
		}
	}
	if ( found ) return TRUE;
	return FALSE;
}

bool is_alt_of(CHAR_DATA *ch, CHAR_DATA *ach)
{
    if(!((ch->desc) && (ach->desc)))
	return FALSE;  /* weak but what can we do? */
    if(ch==ach)
    	return FALSE;
    if(!str_cmp(ch->desc->host,ach->desc->host))
    {
	if(!ip_group_allow(ch,ach))
	    return TRUE;
    }
    return FALSE;
}
void group_char( CHAR_DATA *ch )
{
    CHAR_DATA *victim;

    victim = ch->pcdata->accept_follower_name;

    if ( victim == NULL )
    {
    	stc("No one is asking to follow you.\n\r",ch);
    	return;
    }

    if ( victim != ch->pcdata->accept_follower_name )
    {
    	stc("This person has not asked to follow you.\n\r",ch);
    	return;
    }

	if ( !victim->desc || !ch->desc )
    {
    	stc("That is impossible.\n\r",ch);
    	ch->pcdata->accept_follower_name = NULL;
    	return;
    }

    if ( !str_cmp( victim->desc->host, ch->desc->host )
    || !str_cmp( ch->desc->host, victim->desc->host ) )
    {
	stc("Impossible!\n\r",ch);
	return;
	/*
		if ( !ip_group_allow( ch, victim ) )
		{
	    	stc("Impossible.\n\r",ch);
    		ch->pcdata->accept_follower_name = NULL;
    		return;
		}
	*/
    }

    victim->leader = ch;
   	add_follower( victim, ch );
    act_new("{c$N joins $n's group.",ch,NULL,victim,TO_NOTVICT,POS_RESTING);
    act_new("{cYou join $n's group.",ch,NULL,victim,TO_VICT,POS_SLEEPING);
    act_new("{c$N joins your group.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
    ch->pcdata->accept_follower_name = NULL;
    ch->pcdata->accept_follower = FALSE;
    return;
}

/* new split code to handle copper and silver pennies */

void do_split( CHAR_DATA *ch, char *argument )
{
    char buf1[MSL];
    char buf2[MSL];
    char buf3[MSL];
    char buf4[MSL];
    char arg1[MIL],arg2[MIL],arg3[MIL],arg4[MIL];
    CHAR_DATA *gch;
    int members;
    int amount_cpenny = 0, amount_spenny = 0, amount_gmark = 0, amount_smark = 0;
    int share_cpenny, share_spenny, share_gmark, share_smark;
    int extra_cpenny, extra_spenny, extra_gmark, extra_smark;

	buf1[0] = '\0';
	buf2[0] = '\0';
	buf3[0] = '\0';
	buf4[0] = '\0';

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    argument = one_argument( argument, arg3 );
    argument = one_argument( argument, arg4 );

    if ( arg1[0] == '\0' )
    {
	stc( "Split how many coins?\n\r", ch );
	stc( "Syntax: split '#copper pennies' '#silver pennies' '#silver marks' '#gold marks'\n\r", ch );
	return;
    }

    if (arg1[0] != '\0')
		amount_cpenny = atoi(arg1);

    if (arg2[0] != '\0')
		amount_spenny = atoi(arg2);

    if (arg3[0] != '\0')
		amount_smark = atoi(arg3);

    if (arg4[0] != '\0')
		amount_gmark = atoi(arg4);

    if ( amount_cpenny < 0
    || amount_spenny < 0
    || amount_smark < 0
    || amount_gmark < 0 )
    {
	stc( "Your group wouldn't like that.\n\r", ch );
	return;
    }

    if ( ch->money[0][0] < amount_cpenny )
    {
	stc( "You don't have that many copper pennies to split.\n\r", ch );
	return;
    }
    if ( ch->money[1][0] < amount_spenny )
    {
	stc( "You don't have that many silver pennies to split.\n\r", ch );
	return;
    }
    if ( ch->money[2][0] < amount_smark )
    {
	stc( "You don't have that many silver marks to split.\n\r", ch );
	return;
    }
    if ( ch->money[3][0] < amount_gmark )
    {
	stc( "You don't have that many gold marks to split.\n\r", ch );
	return;
    }


    members = 0;
    for ( gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    {
	if ( is_same_group( gch, ch ) && !IS_AFFECTED(gch,AFF_CHARM) && !IS_NPC(gch))
	    members++;
    }

    if ( members < 2 )
    {
	stc( "Just keep it all.\n\r", ch );
	return;
    }

    share_cpenny = amount_cpenny / members;
    extra_cpenny = amount_cpenny % members;

    share_spenny = amount_spenny / members;
    extra_spenny = amount_spenny % members;

    share_smark = amount_smark / members;
    extra_smark = amount_smark % members;

    share_gmark   = amount_gmark / members;
    extra_gmark   = amount_gmark % members;

    if ( share_gmark == 0
    && share_cpenny == 0
    && share_spenny == 0
    && share_smark == 0 )
    {
	stc( "Don't even bother.\n\r", ch );
	return;
    }

    ch->money[0][0]	-= amount_cpenny;
    ch->money[0][0]	+= share_cpenny + extra_cpenny;

    ch->money[1][0] 	-= amount_spenny;
    ch->money[1][0] 	+= share_spenny + extra_spenny;

    ch->money[2][0]	-= amount_smark;
    ch->money[2][0]	+= share_smark + extra_smark;

    ch->money[3][0] 	-= amount_gmark;
    ch->money[3][0] 	+= share_gmark + extra_gmark;

    if (share_cpenny > 0)
    {
	ptc(ch,
	    "You split %d copper pennies. Your share is %d copper pennies.\n\r",
 	    amount_cpenny,share_cpenny + extra_cpenny);
	sprintf(buf1,
	    "You split %d copper pennies. Your share is %d copper pennies.\n\r",
 	    amount_cpenny,share_cpenny + extra_cpenny);
    }

    if (share_spenny > 0)
    {
	ptc(ch,
	    "You split %d silver pennies. Your share is %d silver pennies.\n\r",
 	    amount_spenny,share_spenny + extra_spenny);
	sprintf(buf2,
	    "You split %d silver pennies. Your share is %d silver pennies.\n\r",
 	    amount_spenny,share_spenny + extra_spenny);
    }

    if (share_smark > 0)
    {
	ptc(ch,
	    "You split %d silver marks. Your share is %d silver marks.\n\r",
 	    amount_smark,share_smark + extra_smark);
	sprintf(buf3,
	    "You split %d silver marks. Your share is %d silver marks.\n\r",
 	    amount_smark,share_smark + extra_smark);
    }

    if (share_gmark > 0)
    {
	ptc(ch,"You split %d gold marks. Your share is %d gold marks.\n\r",
	     amount_gmark,share_gmark + extra_gmark);
	sprintf(buf4,"You split %d gold marks. Your share is %d gold marks.\n\r",
	     amount_gmark,share_gmark + extra_gmark);
    }

    for ( gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    {
	if ( gch != ch && is_same_group(gch,ch) && !IS_AFFECTED(gch,AFF_CHARM) && !IS_NPC(gch) )
	{
		if ( buf1[0] != '\0' )
		    act( buf1, ch, NULL, gch, TO_VICT );
		if ( buf2[0] != '\0' )
		    act( buf2, ch, NULL, gch, TO_VICT );
		if ( buf3[0] != '\0' )
		    act( buf3, ch, NULL, gch, TO_VICT );
		if ( buf4[0] != '\0' )
		    act( buf4, ch, NULL, gch, TO_VICT );
		gch->money[0][0] += share_cpenny;
		gch->money[1][0] += share_spenny;
		gch->money[2][0] += share_smark;
		gch->money[3][0] += share_gmark;
	}
    }

    return;
}

void do_gtell( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *gch;
	char buf[MSL];

	if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	{
		stc("You cannot use channels.\n\r",ch);
		return;
	}
	if ( argument[0] == '\0' )
	{
		stc( "Tell your group what?\n\r", ch );
		return;
	}

	if (!str_cmp(argument, "last"))
	{
		show_channel_logs(ch, "group");
		return;
	}

	if ( IS_SET( ch->penalty_flags, PENAL_NOTELL ) )
	{
		stc( "Your message didn't get through!\n\r", ch );
		return;
	}

	act_new("{g[{GGTELL{g]{W:: {g({wYou{g) {G'{w$t{G'{x",ch,argument,0,TO_CHAR,POS_SLEEPING);

	sprintf(buf, "{g[{GGTELL{g]{W:: {g({w%s{g) {G'{w%s{G'{x", ch->name, argument);
	add_channel_log("group", ch->name, ch->leader ? ch->leader->name: ch->name, buf);

	for ( gch = char_list; gch != NULL; gch = gch->next )
	{
		if ( is_same_group( gch, ch ) )
			act_new("{g[{GGTELL{g]{W:: {g({w$n{g) {G'{w$t{G'{x",ch,argument,gch,TO_VICT,POS_SLEEPING);
	}
	return;
}



/*
 * It is very important that this be an equivalence relation:
 * (1) A ~ A
 * (2) if A ~ B then B ~ A
 * (3) if A ~ B  and B ~ C, then A ~ C
 */
bool is_same_group( CHAR_DATA *ach, CHAR_DATA *bch )
{
    if ( ach == NULL || bch == NULL)
	return FALSE;

    if ( ach->leader != NULL ) ach = ach->leader;
    if ( bch->leader != NULL ) bch = bch->leader;
    return ach == bch;
}

/*
 * ColoUr setting and unsetting, way cool, Ant Oct 94
 *        revised to include config colour, Ant Feb 95
 */
void do_colour( CHAR_DATA *ch, char *argument )
{
    char 	arg[ MAX_STRING_LENGTH ];

    if( IS_NPC( ch ) )
    {
	send_to_char_bw( "{cColoUr is not ON, Way Moron!{c\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg );

    if( !*arg )
    {
	if( !IS_SET( ch->act, PLR_COLOUR ) )
	{
	    SET_BIT( ch->act, PLR_COLOUR );
	    stc( "{cColoUr is now ON, Way Cool!\n\r"
		"{cFurther syntax:\n\r   colour {c<{cfield{c> <{ccolour{c>{c\n\r"
		"{c   colour {c<{cfield{c>{c {cbeep{c|{cnobeep{c\n\r"
		"{cType help {ccolour{c and {ccolour2{c for details.\n\r"
		"{cColoUr is brought to you by Lope, ant@solace.mh.se.{c\n\r", ch );
	}
	else
	{
	    send_to_char_bw( "{cColoUr is now OFF, <sigh>{c\n\r", ch );
	    REMOVE_BIT( ch->act, PLR_COLOUR );
	}
	return;
    }

    if( !str_cmp( arg, "default" ) )
    {
	default_colour( ch );
	send_to_char_bw( "{cColoUr setting set to default values.{c\n\r", ch );
	return;
    }

    if( !str_cmp( arg, "all" ) )
    {
	all_colour( ch, argument );
	return;
    }

    /*
     * Yes, I know this is ugly and unnessessary repetition, but its old
     * and I can't justify the time to make it pretty. -Lope
     */
    if( !str_cmp( arg, "text" ) )
    {
	ALTER_COLOUR( text )
    }
    else if( !str_cmp( arg, "auction" ) )
    {
	ALTER_COLOUR( auction )
    }
    else if( !str_cmp( arg, "auction_text" ) )
    {
	ALTER_COLOUR( auction_text )
    }
    else if( !str_cmp( arg, "gossip" ) )
    {
	ALTER_COLOUR( gossip )
    }
    else if( !str_cmp( arg, "gossip_text" ) )
    {
	ALTER_COLOUR( gossip_text )
    }
    else if( !str_cmp( arg, "music" ) )
    {
	ALTER_COLOUR( music )
    }
    else if( !str_cmp( arg, "music_text" ) )
    {
	ALTER_COLOUR( music_text )
    }
    else if( !str_cmp( arg, "question" ) )
    {
	ALTER_COLOUR( question )
    }
    else if( !str_cmp( arg, "question_text" ) )
    {
	ALTER_COLOUR( question_text )
    }
    else if( !str_cmp( arg, "answer" ) )
    {
	ALTER_COLOUR( answer )
    }
    else if( !str_cmp( arg, "answer_text" ) )
    {
	ALTER_COLOUR( answer_text )
    }
    else if( !str_cmp( arg, "quote" ) )
    {
	ALTER_COLOUR( quote )
    }
    else if( !str_cmp( arg, "quote_text" ) )
    {
	ALTER_COLOUR( quote_text )
    }
    else if( !str_cmp( arg, "immtalk_text" ) )
    {
	ALTER_COLOUR( immtalk_text )
    }
    else if( !str_cmp( arg, "immtalk_type" ) )
    {
	ALTER_COLOUR( immtalk_type )
    }
    else if( !str_cmp( arg, "info" ) )
    {
	ALTER_COLOUR( info )
    }
    else if( !str_cmp( arg, "say" ) )
    {
	ALTER_COLOUR( say )
    }
    else if( !str_cmp( arg, "say_text" ) )
    {
	ALTER_COLOUR( say_text )
    }
    else if( !str_cmp( arg, "tell" ) )
    {
	ALTER_COLOUR( tell )
    }
    else if( !str_cmp( arg, "tell_text" ) )
    {
	ALTER_COLOUR( tell_text )
    }
    else if( !str_cmp( arg, "reply" ) )
    {
	ALTER_COLOUR( reply )
    }
    else if( !str_cmp( arg, "reply_text" ) )
    {
	ALTER_COLOUR( reply_text )
    }
    else if( !str_cmp( arg, "gtell_text" ) )
    {
	ALTER_COLOUR( gtell_text )
    }
    else if( !str_cmp( arg, "gtell_type" ) )
    {
	ALTER_COLOUR( gtell_type )
    }
    else if( !str_cmp( arg, "wiznet" ) )
    {
	ALTER_COLOUR( wiznet )
    }
    else if( !str_cmp( arg, "room_title" ) )
    {
	ALTER_COLOUR( room_title )
    }
    else if( !str_cmp( arg, "room_text" ) )
    {
	ALTER_COLOUR( room_text )
    }
    else if( !str_cmp( arg, "room_exits" ) )
    {
	ALTER_COLOUR( room_exits )
    }
    else if( !str_cmp( arg, "room_things" ) )
    {
	ALTER_COLOUR( room_things )
    }
    else if( !str_cmp( arg, "prompt" ) )
    {
	ALTER_COLOUR( prompt )
    }
    else if( !str_cmp( arg, "fight_death" ) )
    {
	ALTER_COLOUR( fight_death )
    }
    else if( !str_cmp( arg, "fight_yhit" ) )
    {
	ALTER_COLOUR( fight_yhit )
    }
    else if( !str_cmp( arg, "fight_ohit" ) )
    {
	ALTER_COLOUR( fight_ohit )
    }
    else if( !str_cmp( arg, "fight_thit" ) )
    {
	ALTER_COLOUR( fight_thit )
    }
    else if( !str_cmp( arg, "fight_skill" ) )
    {
	ALTER_COLOUR( fight_skill )
    }
    else
    {
	send_to_char_bw( "Unrecognised Colour Parameter Not Set.{c\n\r", ch );
	return;
    }

    send_to_char_bw( "New Colour Parameter Set.{c\n\r", ch );
    return;
}

int calc_weapon_damage( OBJ_DATA *obj )
{
	int wdam;

	if ( !obj )
		return 0;

	/* just return the average for now -- Dalsor 06/10/2013 */
	return ( (1 + obj->value[2]) * obj->value[1] / 2);

	if ( ( wdam = obj->value[1] * obj->value[2] ) <= 0 )
		return 0;
	else
		return ( obj->value[1] * obj->value[2] );
}

void appraise_object (CHAR_DATA *ch, OBJ_DATA *obj, bool plrmarket)
{
    char buf[MAX_STRING_LENGTH];
    AFFECT_DATA *paf;
    OBJ_INDEX_DATA *oID;
    int i = 0, wdam = 0;

    sprintf( buf, "{cThis is %s (%s){c\n\r", check_dye(obj, NULL, obj->dye_vnum), obj->name);
    stc( buf, ch );

    sprintf( buf, "{cIt might cost %d silver marks and weigh around %d stones.{c\n\r",
	obj->cost, obj->weight );
    stc( buf, ch );

    sprintf( buf, "{cIt belongs to the %s Guild.{c\n\r",
	obj->guild );
    stc( buf, ch );

    if (IS_SET(obj->extra_flags,ITEM_REMORT))
    {
		stc("It is a remort-only item.\n\r",ch);
	}
    if (IS_SET(obj->extra_flags,ITEM_STAYDEATH))
    {
		stc("It will not drop to the ground if you die.\n\r",ch);
	}
    if (IS_SET(obj->extra_flags,ITEM_NOSELL))
    {
	    if(obj->owner)
		ptc( ch, "It has been flagged keep by %s.\n\r", capit2(obj->owner));
	    else
		ptc( ch, "It has been flagged keep.\n\r");
	}
	else if(obj->owner)
		ptc( ch, "It is owned by %s.\n\r", capit2(obj->owner));

	if (obj->item_type == ITEM_WEAPON)
	{
	    if (IS_WEAPON_STAT(obj, WEAPON_FLAMING))
		stc("It is extremely warm to the touch.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_FROST))
		stc("It is extremely cold to the touch.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_VAMPIRIC))
		stc("It will drain the life force from its enemies.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_SHARP))
		stc("It is razor sharp.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_VORPAL))
		stc("It appears as if it could do some real damage.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_TWO_HANDS))
		stc("It would take both hands to wield it effectively.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_SHOCKING))
		stc("It is crackling with energy.\n\r",ch);

	    if (IS_WEAPON_STAT(obj, WEAPON_POISON))
		stc("It is dripping with a deadly poison.\n\r",ch);
	}

	ptc( ch, "Wear flags: %s\n\r", wear_bit_name(obj->wear_flags) );


	if (obj->pIndexData->vnum == 24) /* treasure chest key */
	{
		ptc(ch, "This key is labeled '%s'.\n\r",
		get_room_index(obj->value[0]) ? get_room_index(obj->value[0])->name : "Invalid");
		return;
	}

    switch ( obj->item_type )
    {
	case ITEM_HERB:
		if (number_percent() <= get_skill(ch, gsn_herbalknowledge) / 2)
		{
			if (brew_poisonous(obj))
				stc("This herb is known to be poisonous.\n\r", ch);

			if (brew_cure_health(obj))
				stc("This herb is known to work well as a painkiller.\n\r", ch);

			if (brew_cure_fatigue(obj))
				stc("This herb is known to help restore fatigue.\n\r", ch);

			if (brew_cure_fever(obj))
				stc("This herb is known to help relieve fevers.\n\r", ch);

			if (brew_sleep_better(obj))
				stc("This herb is known to help get deeper and more relaxed sleep.\n\r", ch);

			if (brew_cure_stomachache(obj))
				stc("This herb is known to help relieve stomach ache.\n\r", ch);

			if (brew_cure_headache(obj))
				stc("This herb is known to help ease a headache.\n\r", ch);

			check_improve(ch, gsn_herbalknowledge, TRUE, 5);
		}
		else	check_improve(ch, gsn_herbalknowledge, FALSE, 5);
		break;

    	case ITEM_SCROLL:
    	case ITEM_POTION:
    	case ITEM_PILL:

		if (obj->pIndexData->vnum == 3279)
		{
			stc("It is filled with an herbal mixture.\n\r", ch);
			if (number_percent() <= get_skill(ch, gsn_herbalknowledge) / 2)
			{
				if (brew_poisonous(obj))
					stc("It appears to be toxic.\n\r", ch);

				if (brew_cure_health(obj))
					stc("It appears to be some sort of painkiller.\n\r", ch);

				if (brew_cure_fatigue(obj))
					stc("It looks to be some sort of energy boost.\n\r", ch);

				if (brew_cure_fever(obj))
					stc("This appears to be a cure for a fever.\n\r", ch);

				if (brew_sleep_better(obj))
					stc("This looks to be some kind of sleep aid.\n\r", ch);

				if (brew_cure_stomachache(obj))
					stc("This would help cure a stomach ache.\n\r", ch);

				if (brew_cure_headache(obj))
					stc("This would help ease a headache.\n\r", ch);

				if (is_name("failure", obj->material) || brew_poisonous(obj))
					stc("Something about it seems dangerous to drink.\n\r", ch);

				check_improve(ch, gsn_herbalknowledge, TRUE, 5);
			}
			else	check_improve(ch, gsn_herbalknowledge, FALSE, 5);
		}
		else
		{
		    stc( "{cIt is a valuable treasure!\n\r", ch );
			for ( i = 1; i <= 4; i++ )
			{
			    if ( obj->value[i] >= 0
			    && obj->value[i] < MAX_SKILL )
			    {
			    	ptc( ch, "[%d] %s\n\r", i, skill_table[obj->value[i]].name );
			    }
			}
		}
	break;

	case ITEM_KEYRING:
		stc( "This is a keyring, designed to hold room keys.\n\rHolds:\n\r", ch );
		for ( i = 0; i < 5; i++ )
		{
			if ( obj->value[i] == 0 )
			{
				ptc( ch, "Slot [%d] None{x\n\r", i );
				continue;
			}
			else if ( ( oID = get_obj_index( obj->value[i] ) ) == NULL )
			{
				ptc( ch, "An error occured processing slot %d.{x\n\r", i );
				continue;
			}
			else if ( obj->value[i] > 0 )
				ptc( ch, "Slot [%d] %s{x\n\r", i, oID->short_descr  );
		}
		break;
	case ITEM_ANGREAL_MALE:
		if (( check_angrealstudy(ch) && ch->sex == SEX_MALE ) || plrmarket)
		ptc( ch, "It affects weaves by %d-fold when used by a male channeler.\n\r",
			obj->value[1] );
		break;
	case ITEM_ANGREAL_FEMALE:
		if (( check_angrealstudy(ch) && ch->sex == SEX_FEMALE) || plrmarket)
		ptc( ch, "It affects weaves by %d-fold when used by a female channeler.\n\r",
			obj->value[1] );
		break;
    case ITEM_QUIVER:
		ptc(ch,"{WIt holds {R%d %d{Wd{R%d {Warrows.{c{c\n\r",
		obj->value[0], obj->value[1], obj->value[2]);
		break;
	case ITEM_ARROW:
		ptc(ch,"{WThis arrow will do {R%d{Wd{R%d {Wdamage for an average of {R%d{W.{c{c\n\r",
		obj->value[1], obj->value[2], ( obj->value[1] + obj->value[2] ) / 2 );
		break;

    	case ITEM_WAND:
    	case ITEM_STAFF:
/*	    ptc( ch, "{cThis is an item of the One Power.\n\rHas %d(%d) charges of level %d",
	    	obj->value[1], obj->value[2], obj->value[0] );  */
	    ptc( ch, "{cThis is an item of the One Power.\n\rHas %d(%d) charges",
	    	obj->value[1], obj->value[2]);

	    if ( obj->value[3] >= 0 && obj->value[3] < MAX_SKILL )
	    {
	    	stc( "{c '", ch );
	    	stc( skill_table[obj->value[3]].name, ch );
	    	stc( "{c'", ch );
	    }

	    stc( "{c.{c\n\r", ch );
	break;

	case ITEM_DRINK_CON:
	    ptc(ch,"{cIt holds %s-colored %s.{c\n\r",
		liq_table[obj->value[2]].liq_color,
		liq_table[obj->value[2]].liq_name);
	    break;


    	case ITEM_WEAPON:
 	    ptc(ch,"{cThis weapon is a %s.\n\r",weapon_table[obj->value[0]].name);

	wdam = calc_weapon_damage( obj );

        ptc( ch, "It looks as if it will do %s %s damage (average of %d).\n\r",
        	wdam <= 3 ? "almost non-existant" :
        	wdam <= 5 ? "extremely poor" :
        	wdam <= 8 ? "very poor" :
        	wdam <= 12 ? "poor" :
        	wdam <= 15 ? "extremely low" :
        	wdam <= 18 ? "very low" :
        	wdam <= 21 ? "low" :
        	wdam <= 24 ? "fairly low" :
        	wdam <= 38 ? "moderately low" :
        	wdam <= 41 ? "very light" :
        	wdam <= 44 ? "light" :
        	wdam <= 47 ? "fairly light" :
        	wdam <= 50 ? "fair" :
        	wdam <= 53 ? "decent" :
        	wdam <= 56 ? "moderate" :
        	wdam <= 59 ? "medium" :
        	wdam <= 62 ? "good" :
        	wdam <= 65 ? "very good" :
        	wdam <= 68 ? "extremely good" :
        	wdam <= 71 ? "strong" :
        	wdam <= 74 ? "very strong" :
        	wdam <= 77 ? "extremely strong" :
        	wdam <= 81 ? "excellent" :
        	wdam <= 84 ? "high" :
        	wdam <= 87 ? "fairly high" :
        	wdam <= 90 ? "very high" :
        	wdam <= 93 ? "extremely high" :
        	wdam <= 96 ? "heavy" :
        	wdam <= 99 ? "fairly heavy" :
        	wdam <= 102 ? "very heavy" :
        	wdam <= 105 ? "extremely heavy" :
        	wdam <= 110 ? "great" :
        	wdam <= 115 ? "very great" :
        	wdam <= 120 ? "extreme" :
        	wdam <= 125 ? "massive" :
        	wdam <= 130 ? "extremely massive" :
        	wdam <= 135 ? "critical" :
        	wdam <= 150 ? "awesome" :
        	wdam <= 177 ? "god-like" :
        	"{RILLEGAL",
			(obj->value[3] > 0 && obj->value[3] < MAX_DAMAGE_MESSAGE) ?
		    attack_table[obj->value[3]].noun : "{cdo an unknown something{c", wdam);

		if ( wdam > 177 )
		{
			DESCRIPTOR_DATA *d;

			for (d = descriptor_list; d; d = d->next)
			{
				if (d->connected == CON_PLAYING && IS_IMMORTAL(d->character))
				{
					stc ("{W<< \t[B010] \t[F050]ILLEGAL WEAPON {x {W>> {G", d->character);
					stc (obj->short_descr, d->character);
					stc (" {xwas appriased by {x", d->character);
					stc (ch->name, d->character);
					stc (" {xjust now.{x\n\r", d->character);
				}
			}
		}

	break;

    	case ITEM_ARMOR:
	    ptc( ch,"{cArmor class is {w%d {cpierce, {w%d {cbash, {w%d {cslash, and {w%d {cvs. magic{c\n\r",
	        obj->value[0], obj->value[1], obj->value[2], obj->value[3] );
	break;
	case ITEM_LIGHT:
	    ptc( ch, "{cIt will burn for %d hours.\n\r", obj->value[2] );
	break;

        case ITEM_CONTAINER:
            ptc(ch,"{cCapacity{W: {w%d#  {cMaximum weight{W: {w%d#  {cflags{W: {w%s{c\n\r",
                obj->value[0], obj->value[3], cont_bit_name(obj->value[1]));
            if (obj->value[4] != 100)
            {
                ptc(ch,"{cWeight multiplier{W: {w%d%%{c\n\r",
			    obj->value[4]);
            }
        break;

	case ITEM_ADORNMENT:
		ptc(ch,
		"{cIt is %s %s adornment. Can be used on: '%s'.\n\r", check_vowell(craftadorn_table[obj->value[0]].name) ? "an" : "a",
		craftadorn_table[obj->value[0]].name, craftadorn_table[obj->value[0]].allowed);
	break;

	case ITEM_DYE:
		stc("{cIt is a dye. [", ch);

		if (str_cmp(craftcolors_table[obj->value[0]].name, "None"))
			ptc(ch, "{%s#", craftcolors_table[obj->value[0]].name);
		if (str_cmp(craftcolors_table[obj->value[1]].name, "None"))
			ptc(ch, "{%s#", craftcolors_table[obj->value[1]].name);
		if (str_cmp(craftcolors_table[obj->value[2]].name, "None"))
			ptc(ch, "{%s#", craftcolors_table[obj->value[2]].name);
		if (str_cmp(craftcolors_table[obj->value[3]].name, "None"))
			ptc(ch, "{%s#", craftcolors_table[obj->value[3]].name);

		stc("{c]{x\n\r", ch);
	break;

	case ITEM_CRAFTING_MATERIAL:
	    ptc( ch,
	        "This %s material can be used for %s other items.\n\r",
	        obj->value[3] == 5 ? "super rare" : obj->value[3] == 4 ? "rare" : obj->value[3] == 3 ? "uncommon" : obj->value[3] == 2 ? "common" : "super common", flag_string( craftskill_flags, obj->value[0]));
	break;

	case ITEM_CRAFTING_MANUAL:
	    ptc( ch,
	        "This %s manual is simply titled '%s'.\n\r",
	        flag_string( craftskill_flags, obj->value[0]),
	        get_obj_index(obj->value[1]) == NULL ? "Fail" : get_obj_index(obj->value[1])->name);
	    break;






    }


    if ( obj->extra_descr != NULL || obj->pIndexData->extra_descr != NULL )
    {
	EXTRA_DESCR_DATA *ed;

	stc( "{cExtra description keywords: '", ch );

	for ( ed = obj->extra_descr; ed != NULL; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
	    	stc( "{c ", ch );
	}

	for ( ed = obj->pIndexData->extra_descr; ed != NULL; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
		stc( "{c ", ch );
	}

	stc( "{c'{c\n\r", ch );
    }



	if ( !strcmp(obj->material, "unknown")
	||   !strcmp(obj->material, "(Null)"))
	;
	else if (obj->pIndexData->vnum != 3279)
		ptc(ch,"It is made from %s\n\r", obj->material);

	if (obj->pIndexData->vnum == 3279)
	{
		if (obj->item_type == ITEM_TREASURE)
		stc("It's completely empty.\n\r", ch);
		else if (obj->condition <= 50)
		stc("It's about a third of the way full.\n\r", ch);
		else if (obj->condition <= 75)
		stc("It's about two thirds of the way full.\n\r", ch);
		else if (obj->condition <= 100)
		stc("It's filled to the brim.\n\r", ch);

	}
	else if ( obj->condition < 1 )
		ptc( ch, "%s is on the verge of breakage.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 10 )
		ptc( ch, "%s looks ready to break any time.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 20 )
		ptc( ch, "%s is in very poor condition.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 30 )
		ptc( ch, "%s is in pretty bad shape.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 40 )
		ptc( ch, "%s has seen a lot of wear and tear.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 50 )
		ptc( ch, "%s is in poor condition.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 60 )
		ptc( ch, "%s has seen better days.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 70 )
		ptc( ch, "%s is nicked and worn.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 80 )
		ptc( ch, "%s is in fair condition.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 90 )
		ptc( ch, "%s is in good condition.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition <= 100 )
		ptc( ch, "%s is in excellent condition.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );
	else if ( obj->condition > 100 )
		ptc( ch, "%s is in mint condition.\n\r", capit_colorsafe(check_dye(obj, NULL, obj->dye_vnum)) );

    for ( paf = obj->affected; paf != NULL; paf = paf->next )
    {
	if (paf->location == APPLY_MANA || paf->location == APPLY_MOVE)
	ptc( ch, "{cAffects {wstamina {cby {w%d{c", paf->modifier );
	else
	ptc( ch, "{cAffects {w%s {cby {w%d{c", affect_loc_name( paf->location ), paf->modifier );

	if ( paf->duration > -1)
	    ptc(ch,"{c, {w%d {chours.{c\n\r",paf->duration);
	else
	    ptc(ch,"{c.\n\r");
	if (paf->bitvector)
	{
	    switch(paf->where)
	    {
		case TO_AFFECTS:
		    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
			affect_bit_name(paf->bitvector));
		    break;
		case TO_AFFECTS2:
		    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
			affect_bit_name(paf->bitvector));
		    break;
        case TO_WEAPON:
            ptc(ch,"{cAdds {w%s {cweapon flags.{c{c\n",
            weapon_bit_name(paf->bitvector));
		    break;
		case TO_OBJECT:
		    ptc(ch,"{cAdds {w%s {cobject flag.{c{c\n",
			extra_bit_name(paf->bitvector));
		    break;
		case TO_IMMUNE:
		    ptc(ch,"{cAdds immunity to {w%s{a.{c{c\n",
			imm_bit_name(paf->bitvector));
		    break;
		case TO_RESIST:
		    ptc(ch,"{cAdds resistance to {w%s{c.{c\n\r",
			imm_bit_name(paf->bitvector));
		    break;
		case TO_VULN:
		    ptc(ch,"{cAdds vulnerability to {w%s{c.{c\n\r",
			imm_bit_name(paf->bitvector));
		    break;
		default:
		    ptc(ch,"{cUnknown bit {w%d{c: {w%d{c\n\r",
			paf->where,paf->bitvector);
		    break;
	    }
	}
    }


    if (!obj->enchanted)
    for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
    {
	ptc(ch, "{cAffects {w%s {cby {w%d{c\n\r",
	    affect_loc_name( paf->location ), paf->modifier );
        if (paf->bitvector)
        {
            switch(paf->where)
            {
                case TO_AFFECTS:
                    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
                        affect_bit_name(paf->bitvector));
                    break;
                case TO_AFFECTS2:
                    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
                        affect2_bit_name(paf->bitvector));
                    break;
                case TO_OBJECT:
                    ptc(ch,"{cAdds {w%s{c object flag.{c\n",
                        extra_bit_name(paf->bitvector));
                    break;
                case TO_IMMUNE:
                    ptc(ch,"{cAdds immunity to {w%s{c.{c\n",
                        imm_bit_name(paf->bitvector));
                    break;
                case TO_RESIST:
                    ptc(ch,"{cAdds resistance to {w%s{c.{c\n\r",
                        imm_bit_name(paf->bitvector));
                    break;
                case TO_VULN:
                    ptc(ch,"{cAdds vulnerability to {w%s{c.{c\n\r",
                        imm_bit_name(paf->bitvector));
                    break;
                default:
                    ptc(ch,"{cUnknown bit {w%d{c: {w%d{c\n\r",
                        paf->where,paf->bitvector);
                    break;
            }
        }
    }

	return;
}



void do_appraise( CHAR_DATA *ch, char *argument )
{
	char arg[MIL], arg2[MIL];
	OBJ_DATA *obj;

	argument = one_argument( argument, arg );
	argument = one_argument( argument, arg2 );

	if ( arg[0] == '\0' )
	{
		stc( "{cAppraise what?{c\n\r", ch );
		stc( "{cYou can appraise an item in your inventory using {wappraise <item name>{x\n\r", ch );
		stc( "{cTo appraise an item you are wearing use {wappraise my <item name>{x\n\r", ch );
		return;
	}

	if ( !str_cmp( arg, "my" ) )
	{
		if ( ( obj = get_obj_wear( ch, arg2, TRUE ) ) == NULL )
		{
			stc( "{cYou're not wearing that item on your person.{c\n\r", ch );
			return;
		}
	}
	else
	{
		if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
		{
			stc( "{cYou do not have that item in your inventory.{c\n\r", ch );
			return;
		}
	}

	act("$n begins to carefully examine $p.",ch,obj,NULL,TO_ROOM);
	appraise_object(ch, obj, FALSE);

	return;
}


/* writing, reading, and sealing letters */

void do_write(CHAR_DATA *ch, char *argument)
{
  OBJ_DATA *letter;
  char buffer[MSL];

  if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
  {
  stc("{cYou cannot use channels.{c\n\r",ch);
  return;
  }
  if (IS_NPC(ch))
  {
      stc("You can't write.\n\r",ch);
      return;
  }

  for ( letter = ch->carrying; letter; letter = letter->next_content )
  {
      if (letter->item_type == ITEM_NOTEPAPER)
      break;
  }


  if (letter == NULL)
  {
      stc("What do you want to write on?\n\r",ch);
      return;
  }

  if (letter->value[1] != 0)
  {
      stc("It's already been written on.\n\r",ch);
      return;
  }

  smash_tilde(argument);

  letter->extra_descr = new_extra_descr();
  letter->extra_descr->description = str_dup(argument);
  letter->extra_descr->keyword = str_dup("");
  letter->extra_descr->next = NULL;
  letter->value[1] = 1;

  letter->value[2] = 0;

  sprintf(buffer,"%s from %s.",letter->short_descr,ch->name);
  letter->short_descr = str_dup(buffer);

  sprintf(buffer,"%s.",letter->short_descr);
  letter->description = str_dup(buffer);

  letter->cost = 0;
  stc("You pen your note.\n\r",ch);
  act("$n writes a note.",ch,NULL,NULL,TO_ROOM);

  return;
}

void do_read(CHAR_DATA *ch, char *argument)
{
  OBJ_DATA *letter;

  letter = get_obj_list(ch,argument,ch->carrying);

  if (letter == NULL)
  {
      stc("What do you want to read?\n\r",ch);
      return;
  }

  if ( letter->item_type == ITEM_SCROLL )
  {
	do_function( ch, &do_recite, letter->name );
	return;
  }
  if (letter->item_type != ITEM_NOTEPAPER)
  {
      stc("You can't read that.\n\r",ch);
      return;
  }
  /*if(letter->pIndexData->vnum != 3278 || letter->pIndexData->vnum != 15092)
  {
      ptc(ch,"Inform the Immortals obj %d is not valid for do_read::parchment.\n\r",letter->pIndexData->vnum);
      return;
  }*/

  if ((letter->value[2] != 0) && !IS_IMMORTAL(ch))
  {
      stc("It is sealed.\n\r",ch);
      return;
  }

  if (letter->value[1] == 0 || letter->value[1] > 1)
  {
      stc("It is blank.\n\r",ch);
      return;
  }
  if (!letter->extra_descr)
  {
      stc("The note is blank.\n\r",ch);
      return;
  }

  stc("The note reads:\n\r",ch);
  stc(letter->extra_descr->description,ch);
  stc("\n\r",ch);

  /*act("$n reads $p.",ch,NULL,letter,TO_ROOM);*/
}

void do_seal(CHAR_DATA *ch, char *argument)
{
  OBJ_DATA *letter;
  char buffer[MAX_STRING_LENGTH];

  letter = get_obj_list(ch,argument,ch->carrying);

  if (letter == NULL)
  {
      stc("What do you want to seal?\n\r",ch);
      return;
  }

  if (letter->item_type != ITEM_NOTEPAPER)
  {
      stc("You can't seal that.\n\r",ch);
      return;
  }

  if (letter->value[2] != 0)
  {
      stc("It is already sealed.\n\r",ch);
      return;
  }

  letter->value[2]=1;
  letter->description = '\0';
  sprintf(buffer,"A %s is here, sealed with the crest of %s.",letter->short_descr,ch->name);
  letter->description = str_dup(buffer);

  stc("You seal the note.\n\r",ch);
  act("$n seals a note.",ch,NULL,NULL,TO_ROOM);
}

void do_unseal(CHAR_DATA *ch, char *argument)
{
  OBJ_DATA *letter;
  char buffer[MAX_STRING_LENGTH];

  letter = get_obj_list(ch,argument,ch->carrying);

  if (letter == NULL)
  {
      stc("What do you want to unseal?\n\r",ch);
      return;
  }

  if (letter->item_type != ITEM_NOTEPAPER)
  {
      stc("You can't unseal that.\n\r",ch);
      return;
  }

  if (letter->value[2] == 0)
  {
      stc("It is already unsealed.\n\r",ch);
      return;
  }

  letter->value[2]=0;
  free_string(letter->description);
  sprintf(buffer,"A hand-written note is here.");
  letter->description = str_dup(buffer);

  stc("You unseal the note.\n\r",ch);
  act("$n unseals a note.",ch,NULL,NULL,TO_ROOM);
}

void do_sendto (CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *letter;
    CHAR_DATA *victim;

    argument = one_argument( argument, arg );

    if ( arg[0] == '\0' || argument[0] == '\0' )
    {
	stc( "{cSendto whom what?{c\n\r", ch );
	stc( "{cExample:\n\r",ch);
	stc( "{c          Sendto ThatPerson MyNote\n\r",ch);
	return;
    }

    if ( ( victim = get_pc_world( ch, arg ) ) == NULL )
    {
	stc( "{cYour pigeon can't find that person.{c{c\n\r", ch );
	return;
    }

    if (victim == ch)
    {
    act("{cHehe. Sure, ok, uh huh. Right away.",  ch, argument, victim, TO_CHAR );
    return;
	}

	letter = get_obj_list(ch,argument,ch->carrying);

	if (letter == NULL)
    {
    stc("{cPigeons will only carry letters written on notepaper.\n\r",ch);
    return;
    }

	if (letter->item_type != ITEM_NOTEPAPER)
    {
    stc("{cYou can't send that.\n\r",ch);
    return;
    }

    obj_from_char(letter);
    act("$n ties a sealed letter to a pigeon's foot and releases it into the air.",
    	ch,0,0,TO_ROOM);
    act("You tie a sealed letter to a pigeon's foot and release it into the air.",
    	ch,0,0,TO_CHAR);

    obj_to_char(letter, victim);
	act("A pigeon flies in a delivers a sealed letter to $N.",
		ch,0,victim,TO_ROOM);
	act("A pigeon flies in a delivers a sealed letter to you.",
		ch,0,victim,TO_VICT);

	/* add appriate code to keep letter */

    return;
}

void do_rp ( CHAR_DATA *ch, char * argument)
{
    if (IS_SET(ch->comm,COMM_RP))
    {
      stc("{cYou are no longer {RRoleplaying{c.{c\n\r",ch);
      act("{c$n is no longer {RRoleplaying{c.",ch,0,0,TO_ROOM);
      REMOVE_BIT(ch->comm,COMM_RP);
    }
    else
    {
      stc("{cYou feel yourself turn into your character, {BRoleplaying{c mode Active.{c\n\r",ch);
      act("{c$n is now {BRoleplaying.",ch,0,0,TO_ROOM);
      SET_BIT(ch->comm,COMM_RP);
    }
}
void do_translate (CHAR_DATA *ch, char *argument)
{
	int i;
	char buf[MSL];
	char arg[MSL];
	char *word;
	bool found=FALSE;

	if (IS_NPC(ch))
	{
		stc("Don't think so.\n\r", ch);
		return;
	}

	if (argument[0] == '\0')
	{
		stc("What do you want to translate?\n\r", ch);
		stc("Syntax 'translate OT <word in Old Tongue>'\n\r", ch);
		stc("or 'translate NT <word in New Tongue>'\n\r", ch);
		stc("You can use the first few letters as well...\n\r", ch);
		stc("Translating Old Tongue words is default.\n\r", ch);
		return;
	}
	word = one_argument( argument, arg );

	if (str_prefix(arg, "OT") && str_prefix(arg, "NT"))
	{
		sprintf(arg,"OT");
		word=argument;
	}
	if (word[0] == '\0')
	{
		stc("What do you want to translate?", ch);
		stc("Syntax 'translate OT <word in Old Tongue>'\n\r", ch);
		stc("or 'translate NT <word in New Tongue>'\n\r", ch);
		stc("You can use the first few letters as well...\n\r", ch);
		stc("Translating Old Tongue words is default.\n\r", ch);
		return;
	}

	if (!str_prefix(arg, "OT"))
	{
		if (strlen(word)>2)
		for (i = 0; (translation_table[i].otword != NULL) && !found; i++)
		{
			if (!str_cmp(translation_table[i].otword, word))
			{
				stc("{xI found a match!\n\r", ch);
				sprintf (buf, "{w%s{x means {g%s{x.\n\r", word, translation_table[i].engword);
				stc(buf, ch);
				found=TRUE;
			}
		}
		if (!found)
		{
			for (i = 0; translation_table[i].otword != NULL; i++)
			{
				if (!str_prefix(word, translation_table[i].otword ))
				{
					if (!found)
						stc("{xI found a near-match!\n\r", ch);
					sprintf (buf, "{w%s{x looks a bit like {r%s{x which means {g%s{x.\n\r", word, translation_table[i].otword, translation_table[i].engword);
					stc(buf, ch);
					found=TRUE;
				}
			}
		}
	}
	else
	{
		if (strlen(word)>2)
		for (i = 0; (translation_table[i].engword != NULL) && !found; i++)
		{
			if (!str_cmp(translation_table[i].engword, word))
			{
				stc("{xI found a match!\n\r", ch);
				sprintf (buf, "{w%s{x means {g%s{x.\n\r", word, translation_table[i].otword);
				stc(buf, ch);
				found=TRUE;
			}
		}
		if (!found)
		{
			for (i = 0; translation_table[i].engword != NULL; i++)
			{
				if (!str_prefix(word, translation_table[i].engword ))
				{
					if (!found)
						stc("{xI found a near-match!\n\r", ch);
					sprintf (buf, "{w%s{x looks a bit like {r%s{x which means {g%s{x.\n\r", word, translation_table[i].engword, translation_table[i].otword);
					stc(buf, ch);
					found=TRUE;
				}
			}
		}
	}

	if (!found)
	{
		stc("I'm sorry, that word is not in our database.\n\r", ch);
		sprintf (buf, "%s tried to find a translation for %s in %s.\n", ch->name, word, arg);
		append_file (ch, TRANSLATION_FILE, buf);
		return;
	}
	return;
}
