/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "aww.h"
#include "interp.h"
#include "tables.h"
#include "mysql5_interface.h"

bool is_shadow args((CHAR_DATA *ch));

char * const dir_name[] =
{
	"north", "east", "south", "west", "up", "down", "northeast",
	"southeast", "southwest", "northwest"
};

const sh_int rev_dir[] =
{
	2, 3, 0, 1, 5, 4, 8, 9, 6, 7, 0
      /*2, 3, 0, 1, 5, 4, 6, 7, 8, 9*/
};

/*
SECT_INSIDE 0, SECT_CITY 1, SECT_FIELD 2, SECT_FOREST 3, SECT_HILLS 4,
SECT_MOUNTAIN 5, SECT_WATER_SWIM 6, SECT_WATER_NOSWIM 7, SECT_UNUSED 8,
SECT_AIR 9, SECT_DESERT 10, SECT_TUNDRA 11, SECT_BLIGHT 12, SECT_COASTAL 13,
SECT_ROAD 14, SECT_MARSH 15, SECT_WASTE 16, SECT_LAKE_FRESH 17, SECT_LAKE_SALT 18,
SECT_RIVER_FRESH 19, SECT_RIVER_SALT 20, SECT_VOLCANIC 21, SECT_OCEAN 22,
SECT_UNDERGROUND 23, SECT_UNDERWATER 24, SECT_FOOTHILLS 25, SECT_JUNGLE 26,
SECT_FARMLAND 27, SECT_STEPPE 28, SECT_PLAIN 29, SECT_MEADOW 30, SECT_MAX 30
*/
const sh_int movement_loss[SECT_MAX] =
{
 /* 0  1  2  3  4  5  6  7  8   9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30*/
    1, 2, 2, 3, 3, 6, 4, 6, 10, 6, 3, 4, 3, 1, 6, 5, 5, 5, 4, 4, 4, 5, 3, 8, 8, 5, 3, 2, 2, 2
};



/*
 * Local functions.
 */
int	find_door	args( ( CHAR_DATA *ch, char *arg, bool trigger ) );
bool	has_key		args( ( CHAR_DATA *ch, int key ) );
bool	check_blind		args( ( CHAR_DATA *ch ) );
void	forage		args ((CHAR_DATA *ch));

void move_char( CHAR_DATA *ch, int door, bool follow)
{
    CHAR_DATA *fch;
    CHAR_DATA *fch_next;
    ROOM_INDEX_DATA *in_room;
    ROOM_INDEX_DATA *to_room;
    EXIT_DATA *pexit;
	char buf[MSL];
	bool silent = FALSE;
	int shop = -1;

	if (ch->run_buf || (ch->leader && ch->leader->run_buf))
		silent = TRUE;

    if ( door < 0 || door >= MAX_DIR )
    {
		bug( "move_char: bad door %d.{c", door );
		return;
    }

	if ( !IS_NPC( ch ) )
	{
	    if ( MOUNTED(ch) )
	    {
			if (MOUNTED(ch)->position < POS_FIGHTING)
			{
			    stc("Your mount must be standing.\n\r", ch);
		        return;
		    }
			if (!mount_success(ch, MOUNTED(ch), FALSE))
			{
		        stc("Your mount stubbornly refuses to go that way.\n\r", ch);
	            return;
		    }
		}
	}
    /*
     * Exit trigger, if activated, bail out. Only PCs are triggered.
     */
    if ( !IS_NPC(ch)
    && (p_exit_trigger( ch, door, PRG_MPROG )
    ||  p_exit_trigger( ch, door, PRG_OPROG )
    ||  p_exit_trigger( ch, door, PRG_RPROG )) )
		return;

    in_room = ch->in_room;
    if ( ( pexit   = in_room->exit[door] ) == NULL
    ||   ( to_room = pexit->u1.to_room   ) == NULL
    ||	 !can_see_room(ch,pexit->u1.to_room))
    {
		stc( "Alas, you cannot go that way.\n\r", ch );
		return;
    }

	if (pexit->u1.to_room->clan > 0 && !is_clan_name(ch, clan_table[pexit->u1.to_room->clan].name))
	{
		ptc( ch, "Only %s are allowed in there.\n\r", clan_table[pexit->u1.to_room->clan].name );
		return;
	}
/*
	if (ch->in_room->vnum == 3001 && pexit->u1.to_room->vnum != 3700)
	{
	    if ( ch->pcdata && ch->pcdata->allowed == 0 )
	    {
			stc( "You are not allowed to leave the area until a Staff member\n\r",ch);
			stc( "has granted you access to the game. You should be approached\n\r",ch);
			stc( "soon by a Mentor. If you are not approached within an hour or\n\r",ch);
			stc( "two, you may ask over Global, or by sending a Note to Staff.\n\r",ch);

   			WAIT_STATE( ch, 20 );
   			mentor_alert( "%s needs verification!", ch->name );
			return;
		}
	}
*/
	if ( !IS_NPC( ch ) )
	{
	    if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
	    {
			if ( pexit->u1.to_room->vnum == 3001 )
			{
				send_to_char ("Wait till the tag game is over.\n\r", ch);
				return;
			}
		}
    }

    if (IS_SET(pexit->exit_info, EX_CLOSED)
    &&  (!IS_AFFECTED(ch, AFF_PASS_DOOR) || IS_SET(pexit->exit_info,EX_NOPASS))
    &&   !IS_IMMORTAL(ch))
    {
		char dBuf[24];
		if ( pexit->keyword == NULL
		|| pexit->keyword[0] == '\0'
		|| pexit->keyword[0] == ' ' )
			sprintf( dBuf, "door" );
		else
			sprintf( dBuf, pexit->keyword );
		ptc( ch, "The %s %s is closed.\n\r", dBuf, dir_name[door] );
		return;
    }
	if (IS_SET(pexit->u1.to_room->room_flags, ROOM_CLIMB))
    {
        ptc( ch, "You have to climb to go %s from here.\n\r", dir_name[door] );
        return;
    }

	if (!IS_NPC (ch) && IS_SET (ch->pcdata->tag_flags, TAG_FROZEN)
	&& IS_SET (ch->pcdata->tag_flags, TAG_PLAYING))
	{
		send_to_char ("You've been frozen, you can't move!\n\r", ch);
		return;
	}

    if ( IS_AFFECTED(ch, AFF_CHARM)
    &&   ch->master != NULL
    &&   in_room == ch->master->in_room )
    {
		stc( "{cWhat?  And leave your beloved master?\n\r{c", ch );
		return;
    }

    if ( !is_room_owner(ch,to_room) && room_is_private( to_room ) )
    {
	stc( "{cThat room is private right now.\n\r{c", ch );
	return;
    }

    if ( !IS_NPC(ch) )
    {
	int move;


	if ( in_room->sector_type == SECT_AIR
	||   to_room->sector_type == SECT_AIR )
	{
	    if ( MOUNTED(ch) )
 	    {
			if( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
			{
		    send_to_char( "Your mount can't fly.\n\r", ch );
                return;
	        }
	    }
	    else
	    {
			if ( !IS_AFFECTED(ch, AFF_FLYING) && !IS_IMMORTAL(ch))
			{
		    send_to_char( "You can't fly.\n\r", ch );
		    return;
			}
	    }
	}

	if ( (in_room->sector_type == SECT_WATER_NOSWIM
    ||    to_room->sector_type == SECT_WATER_NOSWIM )
    &&    (MOUNTED(ch) && IS_AFFECTED(MOUNTED(ch), AFF_FLYING) ))
    {
        sprintf( buf,"You can't take your mount there.\n\r");
        send_to_char(buf, ch);
        return;
    }

	if (( in_room->sector_type == SECT_WATER_SWIM
	||    to_room->sector_type == SECT_WATER_SWIM )
  	&&    !IS_AFFECTED(ch,AFF_FLYING))
	{
	    if ( get_skill(ch,gsn_swim) < 1 )
	    {
		stc( "{cYou dont know how to swim!\n\r{c", ch );
		return;
	    }
	    if ( number_percent() >= get_skill(ch,gsn_swim) )
	    {
		stc( "{cYou flounder about in the water, but dont seem to make much progress.\n\r{c", ch );
		act("$n flounders about in the water, but doesn't seem to make much progress.",ch,NULL,NULL,TO_ROOM);
		check_improve(ch,gsn_swim,FALSE,3);
		return;
		}
		else
		{
			check_improve(ch,gsn_swim,TRUE,8);
			stc( "{cYou swim through the water.\n\r{c", ch );
			act("$n swims through the water.",ch,NULL,NULL,TO_ROOM);
		}
	}

	if (( in_room->sector_type == SECT_WATER_NOSWIM
	||    to_room->sector_type == SECT_WATER_NOSWIM )
  	&&    !IS_AFFECTED(ch,AFF_FLYING))
	{
	    OBJ_DATA *obj;
	    bool found;

	    /*
	     * Look for a boat.
	     */
	    found = FALSE;

	    if (IS_IMMORTAL(ch))
		found = TRUE;

	    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	    {
		if ( obj->item_type == ITEM_BOAT )
		{
		    found = TRUE;
		    break;
		}
	    }
	    if ( !found )
	    {
		stc( "{cYou need a boat to go there.\n\r{c", ch );
		return;
	    }
	}

	move = movement_loss[UMIN(SECT_MAX-1, in_room->sector_type)]
	     + movement_loss[UMIN(SECT_MAX-1, to_room->sector_type)]
	     ;

        move /= 2;  /* i.e. the average */

	if ( !MOUNTED(ch) )
	{
	    /* conditional effects */
	    if (IS_AFFECTED(ch,AFF_FLYING) || IS_AFFECTED(ch,AFF_HASTE))
		move /= 2;

	    if (IS_AFFECTED(ch,AFF_SLOW))
			move *= 2;

	    if ( ch->stam < move )
	    {
			send_to_char( "You are so exhausted you have to force your legs to move.\n\r", ch );
			WAIT_STATE( ch, PULSE_VIOLENCE );
	    }
	}
	else
	{
	    if (IS_AFFECTED( MOUNTED(ch), AFF_FLYING)
	    || IS_AFFECTED( MOUNTED(ch), AFF_HASTE))
		    move /= 2;

	    if (IS_AFFECTED( MOUNTED(ch), AFF_SLOW))
          move *= 2;

	    if ( MOUNTED(ch)->stam < move )
	    {
			send_to_char( "Your mount is too exhausted.\n\r", ch );
			return;
	    }
    }

	if (!silent && !IS_NPC( ch ) && IS_SET( ch->op_flags, OP_CHANNELER ))
		WAIT_STATE( ch, 1 );

	if (!IS_NPC(ch) && IS_SET(ch->act, PLR_FORAGE))
	{
		WAIT_STATE(ch, ch->wait + number_range(3, 5));
		move *= 2;
	}
        if (!IS_NPC(ch) && IS_SET(ch->auto_flags, AUTO_AUTOSEARCH))
        {
                move *= 2;
        }

	if (!MOUNTED(ch))
        ch->stam -= move;
    else
        MOUNTED(ch)->stam -= move;
    }

    if (RIDDEN(ch))
    {
		CHAR_DATA *rch;
		rch = RIDDEN(ch);

		if (!mount_success(rch, ch, FALSE))
		{
		    act( "Your mount escapes your control, and leaves $T.", rch, NULL, dir_name[door], TO_CHAR );
		    if (RIDDEN(ch))
				ch = RIDDEN(ch);
		}
		else
	 	{
		    send_to_char("You steady your mount.\n\r", rch);
		    return;
		}
    }
    if (is_fighting(ch))
    {
/*	    bugf("Char moving while fighting");
 *	    	apparently this is allowed since chars advancing are considered fighting */
	    stop_all_fighting( ch);
    }

    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
        if (ch->pcdata != NULL && ch->pcdata->wout[0] != '\0')
	{
		/* FIXME needs to be rewritten
		if(strstr(ch->pcdata->wout,"$T"))
	            act( "{c$t{c",ch,walkdir(ch->pcdata->wout,dir_name[door]),NULL,TO_ROOM);
		else
		*/
		 act( "{c$t to the $T.{c",ch,ch->pcdata->wout,dir_name[door],TO_ROOM);

	}
        else
            /*act( "{c$n leaves $T.{c", ch, NULL, dir_name[door], TO_ROOM );*/
    {
	if (MOUNTED(ch))
	{
	    if( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		sprintf(buf, "$n leaves $T, riding on %s.", MOUNTED(ch)->short_descr);
        else
           sprintf(buf, "$n soars $T, on %s.", MOUNTED(ch)->short_descr);
           act( buf, ch, NULL, dir_name[door], TO_ROOM );
    }
	else
	{

	    if ( IS_NPC( ch ) && IS_SET(ch->act,ACT_UNDEAD) )
		    ;
	    else
		    act( "$n leaves $T.", ch, NULL, dir_name[door], TO_ROOM );
    }
    }
    }

    char_from_room( ch );
    char_to_room( ch, to_room );

    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
    if (ch->pcdata != NULL && ch->pcdata->wkin[0] != '\0')
    {
	/* FIXME needs to be rewritten
	if(strstr(ch->pcdata->wout,"$T"))
       		act( "{c$t from the $T.{c",ch,walkdir(ch->pcdata->wkin,dir_name[rev_dir[door]]), NULL,TO_ROOM);
	else */
     act( "{c$t from the $T.{c",ch,ch->pcdata->wkin,dir_name[rev_dir[door]],TO_ROOM);
    }
    else
           /*act( "{c$n has arrived.{c", ch, NULL, NULL, TO_ROOM );*/
    {
	if(!MOUNTED(ch))
	{
	    if ( IS_NPC( ch ) && IS_SET(ch->act,ACT_UNDEAD) )
		    ;
		else
		    act( "$n has arrived.", ch, NULL, NULL, TO_ROOM );
	}
	else
	{
	    if ( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		act( "$n has arrived, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
	    else
		act( "$n soars in, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
        }
    }

    if (MOUNTED(ch))
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), to_room );
    }
    }

	if (silent)
		do_function(ch, &do_look, "silent" );
	else
		do_function(ch, &do_look, "auto" );

    if (in_room == to_room) /* no circular follows */
	return;

    for ( fch = in_room->people; fch != NULL; fch = fch_next )
    {
	fch_next = fch->next_in_room;

	if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM)
	&&   fch->position < POS_STANDING)
	    do_function(fch, &do_stand, "");

	if ( fch->master == ch && fch->position == POS_STANDING
	&&   can_see_room(fch,to_room))
	{

	    if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
	    &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
	    {
		act( "{cYou can't bring $N into the city.{c",
		    ch,NULL,fch,TO_CHAR);
		act( "{cYou aren't allowed in the city.{c",
		    fch,NULL,NULL,TO_CHAR);
		continue;
	    }

		if (!silent)
	    act( "{cYou follow $N.{c", fch, NULL, ch, TO_CHAR );
	    move_char( fch, door, TRUE);
	}
    }

	if (!IS_NPC(ch) && IS_SET(ch->act, PLR_FORAGE) && !follow)
		forage(ch);
        if (!IS_NPC(ch) && IS_SET(ch->auto_flags, AUTO_AUTOSEARCH) && !follow)
                do_function( ch, &do_search, "");

	if ( IS_SET( ch->in_room->room_flags,ROOM_STEDDING ) )
	{
	    if (IS_SET(ch->op_flags,OP_GRASPED))
	    {
	    	REMOVE_BIT(ch->op_flags,OP_GRASPED);
			stc("{wYou cannot seem to sense the True Source for some reason.\n\r", ch);
		}
	}

    /*
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC( ch ) && HAS_TRIGGER_MOB( ch, TRIG_ENTRY ) )
	p_percent_trigger( ch, NULL, NULL, NULL, NULL, NULL, TRIG_ENTRY );

    if ( !IS_NPC( ch ) )
    {
	if ((shop = get_slot_shop(ch->in_room->vnum)) > 0)
	{
		ptc(ch, "{g%s says, ", show_shop(shop, "keeper_name"));
		sprintf(buf, "{G'{g%s{G'{x", show_shop(shop, "greeting"));
		act(buf, ch, show_shop(shop, "name"), ch, TO_CHAR);
	}

   	p_greet_trigger( ch, PRG_MPROG );
	p_greet_trigger( ch, PRG_OPROG );
	p_greet_trigger( ch, PRG_RPROG );
    }

    return;
}



/*The Sliding Code was created by Hortin (Chris), with help from Igneo and Dalsor*/
/*The below code goes in the general movement commands*/
void slide(CHAR_DATA *ch)
{
	int doornumber = -1, i=0, numberofdoors= 0;
	return;
	/* this is really annoying for speedwalkers --Sil */

	if ((IS_SET( ch->in_room->sector_type, SECT_TUNDRA ) &&  !(SEASON == 2) && (number_range(1,100)< 10))
			|| ((SEASON == 4) && (number_range(1, 100) < 2)))
	{	/*the chances are pretty slim, but people tend to move a lot, so I think they'd get annoyed
			when slipping in 20% of the moves during 'winter' :)*/ /* 1 is spring, 4 is winter */
		for (; i < MAX_DIR; i++)
		{
			if (ch->in_room->exit[i] == NULL)
				continue;
			if (i == DIR_UP) /*Can't slide Uphill*/
				continue;
			/*add some other checks or extract them from move_char and put a call to the extracted block
				so it doesn't give 'fail' messages...?*/
			numberofdoors++;
			if (number_range(1,numberofdoors) == 1)
				doornumber = i;
		}
		/*Did we get an exit?*/
		if (doornumber == -1)
			return;
		/*If The Random Exit Is Down, Then FALL!*/
		if ( doornumber == DIR_DOWN )
		{
			stc ( "{WYou lose your balance and fall!{x\n\r", ch );
			act ( "$n loses $s balance and falls!", ch, NULL, NULL, TO_ROOM );
			move_char (ch, doornumber, FALSE);
			return;
		}
		/*If The Random Exit Is Anything Else, Then SLIDE!*/
		else
		{
			act ( "{WYou lose your balance and slide $T!{x", ch, NULL, dir_name[doornumber], TO_CHAR );
			act ( "$n falls down and slides $T.", ch, NULL, dir_name[doornumber], TO_ROOM );
			move_char (ch, doornumber, FALSE);
			return;
		}
	}
}

void do_north( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run north.\n\r", ch);

	move_char( ch, DIR_NORTH, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_northeast( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run northeast.\n\r", ch);

	move_char( ch, DIR_NORTHEAST, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_northwest( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run northwest.\n\r", ch);

	move_char( ch, DIR_NORTHWEST, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_east( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run east.\n\r", ch);

	move_char( ch, DIR_EAST, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
/*
void do_out( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *was_room;


    was_room = ch->in_room;
    move_char( ch, DIR_OUT, FALSE);
        if (!IS_NPC(ch))
        {
                if(ch->run_buf)
                {
                        if (was_room == ch->in_room)
                        {
                                ptc(ch, "Something went wrong and you stop running.\n\rYou still had to do %s\n\r",
				ch->run_buf);
                                free_runbuf(ch);
                        }
                        else
                        {
                                stc("You run out.\n\r", ch);
                        }
                }
                else
                        slide(ch);
        }
    return;
}
*/
void do_west( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run west.\n\r", ch);

	move_char( ch, DIR_WEST, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_southeast( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run southeast.\n\r", ch);

	move_char( ch, DIR_SOUTHEAST, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_southwest( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run southwest.\n\r", ch);

	move_char( ch, DIR_SOUTHWEST, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_south( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run south.\n\r", ch);

	move_char( ch, DIR_SOUTH, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}

void do_down( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run down.\n\r", ch);

	move_char( ch, DIR_DOWN, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}
void do_up( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *was_room;

	was_room = ch->in_room;

	if (!IS_NPC(ch) && ch->run_buf)
		stc("You run up.\n\r", ch);

	move_char( ch, DIR_UP, FALSE);

	if (!IS_NPC(ch) && ch->run_buf)
	{
		if (was_room == ch->in_room)
		{
			ptc(ch, "Something went wrong and you stop running.\n\r", ch->run_buf);
			free_runbuf(ch);
		}
	}
	else slide(ch);

	return;
}

int find_door( CHAR_DATA *ch, char *arg, bool trigger )
{
    EXIT_DATA *pexit;
    int door;

	 if ( !str_cmp( arg, "n" ) || !str_cmp( arg, "north" ) ) door = 0;
    else if ( !str_cmp( arg, "e" ) || !str_cmp( arg, "east"  ) ) door = 1;
    else if ( !str_cmp( arg, "s" ) || !str_cmp( arg, "south" ) ) door = 2;
    else if ( !str_cmp( arg, "w" ) || !str_cmp( arg, "west"  ) ) door = 3;
    else if ( !str_cmp( arg, "u" ) || !str_cmp( arg, "up"    ) ) door = 4;
    else if ( !str_cmp( arg, "d" ) || !str_cmp( arg, "down"  ) ) door = 5;
	else if ( !str_cmp( arg, "ne" ) || !str_cmp( arg, "northeast" ) ) door = 6;
	else if ( !str_cmp( arg, "se" ) || !str_cmp( arg, "southeast" ) ) door = 7;
	else if ( !str_cmp( arg, "sw" ) || !str_cmp( arg, "southwest" ) ) door = 8;
	else if ( !str_cmp( arg, "nw" ) || !str_cmp( arg, "northwest" ) ) door = 9;
    else
    {
		for ( door = 0; door < MAX_DIR; door++ )
		{
		    if ( ( pexit = ch->in_room->exit[door] ) != NULL
		    &&   IS_SET(pexit->exit_info, EX_ISDOOR)
		    &&   pexit->keyword != NULL
		    &&   is_name( arg, pexit->keyword ) )
			return door;
		}
		if ( !trigger )
			act( "{cI see no $T here.{c", ch, NULL, arg, TO_CHAR );
		return -1;
    }

    if ( ( pexit = ch->in_room->exit[door] ) == NULL )
    {
		if ( !trigger )
			act( "{cI see no door $T here.{c", ch, NULL, arg, TO_CHAR );
		return -1;
    }

    if ( !IS_SET(pexit->exit_info, EX_ISDOOR) )
    {
		if ( !trigger )
			stc( "{cYou can't do that.\n\r{c", ch );
		return -1;
    }

    return door;
}



void do_open( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cOpen what?\n\r{c", ch );
	return;
    }

    if ( ( obj = get_obj_here( ch, NULL, arg ) ) != NULL )
    {
 	/* open portal */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (!IS_SET(obj->value[1], EX_ISDOOR))
	    {
		stc( "{cYou can't do that.\n\r{c",ch);
		return;
	    }

	    if (!IS_SET(obj->value[1], EX_CLOSED))
	    {
		stc( "{cIt's already open.\n\r{c",ch);
		return;
	    }

	    if (IS_SET(obj->value[1], EX_LOCKED))
	    {
		stc( "{cIt's locked.\n\r{c",ch);
		return;
	    }

	    REMOVE_BIT(obj->value[1], EX_CLOSED);
	    act( "{cYou open $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n opens $p.{c",ch,obj,NULL,TO_ROOM);
	    return;
 	}

	/* 'open object' */
	if ( obj->item_type != ITEM_CONTAINER )
	{
		if ( obj->item_type != ITEM_TREASURECHEST )
	    {
			stc( "{cThat's not a container.\n\r{c", ch );
			return;
		}
	}
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { stc( "{cIt's already open.\n\r{c",      ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSEABLE) )
	    { stc( "{cYou can't fiure out how to close that.\n\r{c",      ch ); return; }
	if ( IS_SET(obj->value[1], CONT_LOCKED) )
	    { stc( "{cIt's locked.\n\r{c",            ch ); return; }


	if ( obj->item_type == ITEM_TREASURECHEST )
	{
		REMOVE_BIT(obj->value[1], CONT_CLOSED);
		act( "{cYou open $p.{c",ch,obj,NULL,TO_CHAR);
		act( "{c$n opens $p.{c", ch, obj, NULL, TO_ROOM );
		return;
	}

	REMOVE_BIT(obj->value[1], CONT_CLOSED);
	act( "{cYou open $p.{c",ch,obj,NULL,TO_CHAR);
	act( "{c$n opens $p.{c", ch, obj, NULL, TO_ROOM );
	return;
    }

    if ( ( door = find_door( ch, arg, FALSE ) ) >= 0 )
    {
	/* 'open door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit = ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) )
	    { stc( "{cIt's already open.\n\r{c",      ch ); return; }
	if (  IS_SET(pexit->exit_info, EX_LOCKED) )
	    { stc( "{cIt's locked.\n\r{c",            ch ); return; }

	REMOVE_BIT(pexit->exit_info, EX_CLOSED);
	act( "{c$n opens the $d.{c", ch, NULL, pexit->keyword, TO_ROOM );
	act( "{cYou open the $d.{c", ch, NULL, pexit->keyword, TO_CHAR );
/*	stc( "{cOk.\n\r{c", ch ); */

	/* open the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    CHAR_DATA *rch;

	    REMOVE_BIT( pexit_rev->exit_info, EX_CLOSED );
	    for ( rch = to_room->people; rch != NULL; rch = rch->next_in_room )
		act( "{cThe $d opens.{c", rch, NULL, pexit_rev->keyword, TO_CHAR );
	}
    }

    return;
}



void do_close( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cClose what?\n\r{c", ch );
	return;
    }

    if ( ( obj = get_obj_here( ch, NULL, arg ) ) != NULL )
    {
	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{

	    if (!IS_SET(obj->value[1],EX_ISDOOR)
	    ||   IS_SET(obj->value[1],EX_NOCLOSE))
	    {
		stc( "{cYou can't seem do that for some reason.\n\r{c",ch);
		return;
	    }

	    if (IS_SET(obj->value[1],EX_CLOSED))
	    {
		stc( "{cIt's already closed.\n\r{c",ch);
		return;
	    }

	    SET_BIT(obj->value[1],EX_CLOSED);
	    act( "{cYou close $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n closes $p.{c",ch,obj,NULL,TO_ROOM);
	    return;
	}

	/* 'close object' */
	if ( obj->item_type != ITEM_CONTAINER )
	{
		if ( obj->item_type != ITEM_TREASURECHEST )
	    {
			stc( "{cThat's not a container.\n\r{c", ch );
			return;
		}
	}
	if ( IS_SET(obj->value[1], CONT_CLOSED) )
	    { stc( "{cIt's already closed.\n\r{c",    ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSEABLE) )
	    { stc( "{cYou don't seem to be able to do that.\n\r{c",      ch ); return; }

	SET_BIT(obj->value[1], CONT_CLOSED);
	act( "{cYou close $p.{c",ch,obj,NULL,TO_CHAR);
	act( "{c$n closes $p.{c", ch, obj, NULL, TO_ROOM );
	return;
    }

    if ( ( door = find_door( ch, arg, FALSE ) ) >= 0 )
    {
	/* 'close door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit	= ch->in_room->exit[door];
	if ( IS_SET(pexit->exit_info, EX_CLOSED) )
	    { stc( "{cIt's already closed.\n\r{c",    ch ); return; }

	SET_BIT(pexit->exit_info, EX_CLOSED);
	act( "{c$n closes the $d.{c", ch, NULL, pexit->keyword, TO_ROOM );
	act( "{cYou close the $d.{c", ch, NULL, pexit->keyword, TO_CHAR );

/*	stc( "{cOk.\n\r{c", ch ); */

	/* close the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != 0
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    CHAR_DATA *rch;

	    SET_BIT( pexit_rev->exit_info, EX_CLOSED );
	    for ( rch = to_room->people; rch != NULL; rch = rch->next_in_room )
		act( "{cThe $d closes.{c", rch, NULL, pexit_rev->keyword, TO_CHAR );
	}
    }

    return;
}



bool has_key( CHAR_DATA *ch, int key )
{
    OBJ_DATA *obj;
    int i = 0;

    if ( key == 0 )
	return FALSE;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
		if (obj->item_type != ITEM_KEYRING &&  obj->pIndexData->vnum == key )
		    return TRUE;
		if ( obj->item_type == ITEM_KEYRING )
		{
			for ( i = 0; i < 5; i++ )
			{
				if ( obj->value[i] == key )
				{
					ptc( ch, "You flip through %s for the key.\n\r", obj->short_descr );
					return TRUE;
				}
			}
		}
    }

    return FALSE;
}


OBJ_DATA *find_treasure_key( CHAR_DATA *ch, OBJ_DATA *chest )
{
    OBJ_DATA *key;

    for ( key = ch->carrying; key != NULL; key = key->next_content )
    {
		if (key->pIndexData->vnum != chest->value[2])		continue;
		if (key->value[0] == chest->in_room->vnum)		return key;
    }

    return NULL;
}



void do_lock( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cLock what?\n\r{c", ch );
	return;
    }

    if (MOUNTED(ch))
    {
        send_to_char("You can't reach the lock from your mount.\n\r", ch);
        return;
    }

    if ( ( obj = get_obj_here( ch, NULL, arg ) ) != NULL )
    {
	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (MOUNTED(ch))
	    {
        send_to_char("You can't reach the lock from your mount.\n\r", ch);
        return;
	    }
	    if (!IS_SET(obj->value[1],EX_ISDOOR)
	    ||  IS_SET(obj->value[1],EX_NOCLOSE))
	    {
		stc( "{cYou can't do that for some odd reason.\n\r{c",ch);
		return;
	    }
	    if (!IS_SET(obj->value[1],EX_CLOSED))
	    {
		stc( "{cIt's not closed.\n\r{c",ch);
	 	return;
	    }

	    if (obj->pIndexData->vnum == 19 || obj->value[4] < 0 || IS_SET(obj->value[1],EX_NOLOCK))
	    {
		stc( "{cIt can't be locked.\n\r{c",ch);
		return;
	    }

	    if (!has_key(ch,obj->value[4]))
	    {
		stc( "{cYou lack the key.\n\r{c",ch);
		return;
	    }

	    if (IS_SET(obj->value[1],EX_LOCKED))
	    {
		stc( "{cIt's already locked.\n\r{c",ch);
		return;
	    }

	    SET_BIT(obj->value[1],EX_LOCKED);
	    act( "{cYou lock $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n locks $p.{c",ch,obj,NULL,TO_ROOM);
	    return;
	}

	/* 'lock object' */
	if ( obj->item_type != ITEM_CONTAINER )
	{
		if ( obj->item_type != ITEM_TREASURECHEST )
	    {
			stc( "{cThat's not a container.\n\r{c", ch );
			return;
		}
	}
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { stc( "{cIt's not closed.\n\r{c",        ch ); return; }
	if ( obj->pIndexData->vnum == 19 || obj->value[2] < 0 )
	    { stc( "{cIt can't be locked.\n\r{c",     ch ); return; }
	if ( !has_key( ch, obj->value[2] ) )
	    { stc( "{cYou lack the key.\n\r{c",       ch ); return; }
	if ( IS_SET(obj->value[1], CONT_LOCKED) )
	    { stc( "{cIt's already locked.\n\r{c",    ch ); return; }

	SET_BIT(obj->value[1], CONT_LOCKED);
	act( "{cYou lock $p.{c",ch,obj,NULL,TO_CHAR);
	act( "{c$n locks $p.{c", ch, obj, NULL, TO_ROOM );
	return;
    }

    if ( ( door = find_door( ch, arg, FALSE ) ) >= 0 )
    {
	/* 'lock door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit	= ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) )
	    { stc( "{cIt's not closed.\n\r{c",        ch ); return; }
	if ( pexit->key < 0 )
	    { stc( "{cIt can't be locked.\n\r{c",     ch ); return; }
	if ( !has_key( ch, pexit->key) )
	    { stc( "{cYou lack the key.\n\r{c",       ch ); return; }
	if ( IS_SET(pexit->exit_info, EX_LOCKED) )
	    { stc( "{cIt's already locked.\n\r{c",    ch ); return; }

	SET_BIT(pexit->exit_info, EX_LOCKED);
	stc( "{c*Click*\n\r{c", ch );
	act( "{c$n locks the $d.{c", ch, NULL, pexit->keyword, TO_ROOM );

	/* lock the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != 0
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    SET_BIT( pexit_rev->exit_info, EX_LOCKED );
	}
    }

    return;
}



void do_unlock( CHAR_DATA *ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	OBJ_DATA *obj;
	int door;

	one_argument( argument, arg );

	if ( arg[0] == '\0' )
	{
		stc( "Unlock what?\n\r", ch );
		return;
	}

	if ( ( obj = get_obj_here( ch, NULL, arg ) ) != NULL )
	{
		if ( obj->item_type == ITEM_TREASURECHEST )
		{
			stc( "You'll need a lockpick to pick the lock.\n\r", ch );
			return;
		}
		/* portal stuff */
		if (obj->item_type == ITEM_PORTAL)
		{
			if (!IS_SET(obj->value[1],EX_ISDOOR))
			{
				act( "You can't find a way to open $p.",ch,obj,NULL,TO_CHAR);
				return;
			}
			if (!IS_SET(obj->value[1],EX_CLOSED))
			{
				act( "$p is not closed.",ch,obj,NULL,TO_CHAR);
				return;
			}
			if (obj->value[4] < 0)
			{
				act( "$p can't be unlocked.",ch,obj,NULL,TO_CHAR);
				return;
			}
			if (!has_key(ch,obj->value[4]))
			{
				act( "You lack the key to open $p.",ch,obj,NULL,TO_CHAR);
				return;
			}
			if (!IS_SET(obj->value[1],EX_LOCKED))
			{
				act( "$p is already unlocked.",ch,obj,NULL,TO_CHAR);
				return;
			}
			REMOVE_BIT(obj->value[1],EX_LOCKED);
			act( "You unlock $p.",ch,obj,NULL,TO_CHAR);
			act( "$n unlocks $p.",ch,obj,NULL,TO_ROOM);
			return;
		}
		/* 'unlock object' */
		if ( obj->item_type != ITEM_CONTAINER )
		{
			act( "$p is not a container.",ch,obj,NULL,TO_CHAR );
			return;
		}

		if ( !IS_SET(obj->value[1], CONT_LOCKED) )
		{
			stc( "It's already unlocked.\n\r", ch ); 
			return;
		}
		if ( !IS_SET(obj->value[1], CONT_CLOSED) )
		{ 
			stc( "It's not closed.\n\r", ch ); 
			return; 
		}
		if ( obj->value[2] < 0 )
		{ 
			stc( "It can't be unlocked.\n\r", ch ); 
			return; 
		}

		if (obj->pIndexData->vnum == 19)
		{
			OBJ_DATA *key;

			if ((key = find_treasure_key(ch, obj)) == NULL)
			{
				stc( "You lack the key.\n\r", ch);
				return;
			}

			extract_obj(key);
			obj->timer = 5;
		}
		else if ( !has_key( ch, obj->value[2] ) )
		{ 
			stc( "You lack the key.\n\r", ch ); 
			return; 
		}
		REMOVE_BIT(obj->value[1], CONT_LOCKED);
		act( "You unlock $p.",ch,obj,NULL,TO_CHAR);
		act( "$n unlocks $p.", ch, obj, NULL, TO_ROOM );
		return;
	}

	if ( ( door = find_door( ch, arg, FALSE ) ) >= 0 )
	{
		/* 'unlock door' */
		ROOM_INDEX_DATA *to_room;
		EXIT_DATA *pexit;
		EXIT_DATA *pexit_rev;

		pexit = ch->in_room->exit[door];
		if ( !IS_SET(pexit->exit_info, EX_CLOSED) )
		{ 
			stc( "It's not closed.\n\r", ch ); 
			return; 
		}
		if ( pexit->key < 0 )
		{ 
			stc( "It can't be unlocked.\n\r",   ch ); 
			return; 
		}
		if ( !has_key( ch, pexit->key) )
		{ 
			stc( "You lack the key.\n\r",       ch ); 
			return; 
		}
		if ( !IS_SET(pexit->exit_info, EX_LOCKED) )
		{ 
			stc( "It's already unlocked.\n\r",  ch ); 
			return; 
		}

		REMOVE_BIT(pexit->exit_info, EX_LOCKED);
		stc( "{c*{CClick{c*{x\n\r", ch );
		act( "$n unlocks the $d.", ch, NULL, pexit->keyword, TO_ROOM );

		/* unlock the other side */
		if ( ( to_room   = pexit->u1.to_room            ) != NULL
		&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
		&&   pexit_rev->u1.to_room == ch->in_room )
		{
			REMOVE_BIT( pexit_rev->exit_info, EX_LOCKED );
		}
	}
	return;
}



void do_pick( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *gch;
    OBJ_DATA *obj;
    OBJ_DATA *pick;
    OBJ_DATA *pick_next;
    int door, lock_diff, chance, result;
    bool found;
    bool held;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cPick what?\n\r{c", ch );
	return;
    }
    if (MOUNTED(ch))
    {
        send_to_char("You can't pick locks while mounted.\n\r", ch);
        return;
    }

	if ( get_skill(ch,gsn_pick_lock) == 0 )
	{
		stc("Pick locks? You can barely pick your nose!\n\r",ch);
		return;
	}

	chance = get_skill(ch,gsn_pick_lock);
	chance += ref_app[get_curr_stat(ch,STAT_REF)].lockpicking;

	found = FALSE;
	held = FALSE;
	for ( pick = ch->carrying; pick != NULL; pick = pick_next )
	{
	    pick_next = pick->next_content;
		if ( pick->item_type == ITEM_LOCKPICK )
		{
			found = TRUE;
			if (! (pick->wear_loc == WEAR_NONE) )
			{
				held = TRUE;
				break;
			}

		}
		else continue;
    }
	if ( !found )
	{
        stc( "How do you propose to pick the lock with your fingers?\n\r", ch );
        stc( "You will need to hold a lockpick.\n\r", ch );
        return;
	}
	if ( found )
	{
/*		if ( pick->wear_loc == WEAR_NONE )*/
		if (!held)
		{
        stc( "You will need to hold your lockpick.\n\r", ch );
        return;
		}
	}


    WAIT_STATE( ch, skill_table[gsn_pick_lock].beats );

    /* look for guards */
    for ( gch = ch->in_room->people; gch; gch = gch->next_in_room )
    {
	if ( IS_NPC(gch) && IS_AWAKE(gch) && ch->level + 5 < gch->level )
	{
	    act( "{c$N is standing too close to the lock.{c",
		ch, NULL, gch, TO_CHAR );
	    return;
	}
    }

    if ( !IS_NPC(ch) && number_percent( ) > get_skill(ch,gsn_pick_lock))
    {
	stc( "{cYou failed.\n\r{c", ch);
	check_improve(ch,gsn_pick_lock,FALSE,9);
	return;
    }

    if ( ( obj = get_obj_here( ch, NULL, arg ) ) != NULL )
    {
	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (!IS_SET(obj->value[1],EX_ISDOOR))
	    {
		stc( "{cYou aren't able to figure out how to do that.\n\r{c",ch);
		return;
	    }

	    if (!IS_SET(obj->value[1],EX_CLOSED))
	    {
		stc( "{cIt's not closed.\n\r{c",ch);
		return;
	    }

	    if (obj->value[4] < 0)
	    {
		stc( "{cIt can't be unlocked.\n\r{c",ch);
		return;
	    }

	    if (IS_SET(obj->value[1],EX_PICKPROOF))
	    {
		stc( "{cYou failed.\n\r{c",ch);
		return;
	    }

	    REMOVE_BIT(obj->value[1],EX_LOCKED);
	    act( "{cYou pick the lock on $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n picks the lock on $p.{c",ch,obj,NULL,TO_ROOM);
	    check_improve(ch,gsn_pick_lock,TRUE,2);
	    return;
	}

	/* 'pick object' */
	if ( obj->item_type != ITEM_CONTAINER )
	{
		if ( obj->item_type != ITEM_TREASURECHEST )
	    {
			stc( "{cThat's not a container.\n\r{c", ch );
			return;
		}
	}
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { stc( "{cIt's not closed.\n\r{c",        ch ); return; }
	if ( obj->value[2] < 0 )
	    { stc( "{cIt can't be unlocked.\n\r{c",   ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_LOCKED) )
	    { stc( "{cIt's already unlocked.\n\r{c",  ch ); return; }
	if ( IS_SET(obj->value[1], CONT_PICKPROOF) )
	    { stc( "{cYou failed.\n\r{c",             ch ); return; }

	if ( obj->item_type == ITEM_TREASURECHEST )
	{
		if ( ch->in_room != NULL )
		{
			if ( ch->in_room->vnum == 3017 || ch->in_room->vnum == 3001 )
			{
				stc("Go somewhere else to create spam.\n\r", ch);
				return;
			}
		}
		lock_diff = obj->value[2];

		/* traps! these only kick in if not unarmed first */
		if ( obj->value[3] == TRAP_BOOMER )
		{
			trap_damage( ch, obj );

			/* if the box explodes or erupts in flame, remove it */
/*			if ( obj->value[3] == TRAP_BOOMER || obj->value[3] == TRAP_FIRE )
			{*/
				obj_from_char( obj );
				extract_obj( obj );
				return;
/*			} */
		}
		/* blade needs to sever limbs in the future */
		if ( obj->value[3] == TRAP_BLADE || obj->value[3] == TRAP_MANA )
		{
			trap_damage( ch, obj );
		}
		/* acid - nasty stuff */
		if ( obj->value[3] == TRAP_ACID )
		{
			trap_damage( ch, obj );
		}
		/* poison */
		if ( obj->value[3] == TRAP_NEEDLE || obj->value[3] == TRAP_GASSER )
		{
			/* get the char */
			if ( obj->value[3] == TRAP_NEEDLE )
			{
				trap_damage( ch, obj );
			}
			/* get the room */
			if ( obj->value[3] == TRAP_GASSER )
			{
				trap_damage( ch, obj );
			}
		}
		if ( obj->value[3] == TRAP_FIRE )
		{
			trap_damage( ch, obj );

			/* if the box explodes or erupts in flame, remove it */
			obj_from_char( obj );
			extract_obj( obj );
			return;
		}
		/* end traps */

		result = number_range(chance*8/12,chance)+ref_app[get_curr_stat(ch,STAT_REF)].lockpicking;
		result += pick->condition / 10;
		if ( result < lock_diff )
		{
			ptc(ch,"You attempt to pick the lock on %s, but fail.\n\r",obj->short_descr);
			/*ptc(ch,"Result was %d chance, and you needed to go over %d lock_diff.\n\r", result, lock_diff);*/
			obj->value[2] -= ref_app[get_curr_stat(ch,STAT_REF)].lockpicking / 3;
			if ( number_percent() > number_percent() + ref_app[get_curr_stat(ch,STAT_REF)].lockpicking )
			{
				act("You wince as you feel your lockpick bend inside the lock.",ch,NULL,NULL,TO_CHAR);
				act("You hear a wince from $n as $e tests $s skill at lockpicking.",ch,NULL,NULL,TO_ROOM);
				pick->condition -= 1;
				if ( pick->condition < 1 )
				{
					act("With a *snap* your lockpick breaks off inside the lock.",ch,NULL,NULL,TO_CHAR);
					act("You hear a *snap* and a muttered curse from $n as $s lockpick breaks in two.",ch,NULL,NULL,TO_ROOM);
					extract_obj ( pick );
				}
			}
			check_improve(ch,gsn_pick_lock,FALSE,9);
			return;
		}
	}

	REMOVE_BIT(obj->value[1], CONT_LOCKED);
	if ( obj->item_type == ITEM_TREASURECHEST )
		open_treasurechest( obj );
        act( "{cYou pick the lock on $p.{c",ch,obj,NULL,TO_CHAR);
        act( "{c$n picks the lock on $p.{c",ch,obj,NULL,TO_ROOM);
	check_improve(ch,gsn_pick_lock,TRUE,7);
	return;
    }

    if ( ( door = find_door( ch, arg, FALSE ) ) >= 0 )
    {
	/* 'pick door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit = ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) && !IS_IMMORTAL(ch))
	    { stc( "{cIt's not closed.\n\r{c",        ch ); return; }
	if ( pexit->key < 0 && !IS_IMMORTAL(ch))
	    { stc( "{cIt can't be picked.\n\r{c",     ch ); return; }
	if ( !IS_SET(pexit->exit_info, EX_LOCKED) )
	    { stc( "{cIt's already unlocked.\n\r{c",  ch ); return; }
	if ( IS_SET(pexit->exit_info, EX_PICKPROOF) && !IS_IMMORTAL(ch))
	    { stc( "{cYou failed.\n\r{c",             ch ); return; }

	REMOVE_BIT(pexit->exit_info, EX_LOCKED);
	stc( "{c*Click*\n\r{c", ch );
	act( "{c$n picks the $d.{c", ch, NULL, pexit->keyword, TO_ROOM );
	check_improve(ch,gsn_pick_lock,TRUE,9);

	/* pick the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    REMOVE_BIT( pexit_rev->exit_info, EX_LOCKED );
	}
    }

    return;
}




void do_stand( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;

    if (argument[0] != '\0')
    {
	if (ch->position == POS_FIGHTING)
	{
	    stc( "{cMaybe you should finish fighting first?\n\r{c",ch);
	    return;
	}

	obj = get_obj_list(ch,argument,ch->in_room->contents);
	if (obj == NULL)
	{
	    stc( "{cYou don't see that here.\n\r{c",ch);
	    return;
	}
	if (obj->item_type != ITEM_FURNITURE
	||  (!IS_SET(obj->value[2],STAND_AT)
	&&   !IS_SET(obj->value[2],STAND_ON)
	&&   !IS_SET(obj->value[2],STAND_IN)))
	{
	    stc( "{cYou can't seem to find a place to stand.\n\r{c",ch);
	    return;
	}
	if (ch->on != obj && count_users(obj) >= obj->value[0])
	{
	    act_new("{c{cThere's no room to stand on $p.{c",
		ch,obj,NULL,TO_CHAR,POS_DEAD);
	    return;
	}

 	ch->on = obj;

	if ( HAS_TRIGGER_OBJ( obj, TRIG_SIT ) )
		p_percent_trigger( NULL, obj, NULL, ch, NULL, NULL, TRIG_SIT );
    }

    switch ( ch->position )
    {
    case POS_SLEEPING:

    if ( IS_SET(ch->spec_flags,SPEC_DREAMING ) )
    {
    stc("You must leave Tel'aran'rhiod first.\n\r",ch);
    return;
    }

	if ( IS_AFFECTED(ch, AFF_SLEEP) )
	    { stc( "{cYou can't wake up!\n\r{c", ch ); return; }

	if (obj == NULL)
	{
	    stc( "{cYou wake and stand up.\n\r{c", ch );
	    act( "{c$n wakes and stands up.{c", ch, NULL, NULL, TO_ROOM );
	    ch->on = NULL;
	}
	else if (IS_SET(obj->value[2],STAND_AT))
	{
	   act_new("{cYou wake and stand at $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
	   act( "{c$n wakes and stands at $p.{c",ch,obj,NULL,TO_ROOM);
	}
	else if (IS_SET(obj->value[2],STAND_ON))
	{
	    act_new("{cYou wake and stand on $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
	    act( "{c$n wakes and stands on $p.{c",ch,obj,NULL,TO_ROOM);
	}
	else
	{
	    act_new("{cYou wake and stand in $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
	    act( "{c$n wakes and stands in $p.{c",ch,obj,NULL,TO_ROOM);
	}
	ch->position = POS_STANDING;
	do_function(ch, &do_look, "auto");
	break;

    case POS_RESTING: case POS_SITTING:
	if (obj == NULL)
	{
	    stc( "{cYou stand up.\n\r{c", ch );
	    act( "{c$n stands up.{c", ch, NULL, NULL, TO_ROOM );
	    ch->on = NULL;
	}
	else if (IS_SET(obj->value[2],STAND_AT))
	{
	    act( "{cYou stand at $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n stands at $p.{c",ch,obj,NULL,TO_ROOM);
	}
	else if (IS_SET(obj->value[2],STAND_ON))
	{
	    act( "{cYou stand on $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n stands on $p.{c",ch,obj,NULL,TO_ROOM);
	}
	else
	{
	    act( "{cYou stand in $p.{c",ch,obj,NULL,TO_CHAR);
	    act( "{c$n stands on $p.{c",ch,obj,NULL,TO_ROOM);
	}
	ch->position = POS_STANDING;
	break;

    case POS_STANDING:
	stc( "{cYou are already standing.\n\r{c", ch );
	break;

    case POS_FIGHTING:
	stc( "{cYou are already fighting!\n\r{c", ch );
	break;
    }

    return;
}



void do_rest( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;

    if (ch->position == POS_FIGHTING)
    {
	stc( "{cYou are already fighting!\n\r{c",ch);
	return;
    }
    if (MOUNTED(ch))
    {
        send_to_char("You can't rest while mounted.\n\r", ch);
        return;
    }
    if (RIDDEN(ch))
    {
        send_to_char("You can't rest while being ridden.\n\r", ch);
        return;
    }

	/* fix for letting chars sleep on items in inv - Dalsor */
#if 0
    if (argument[0] == '\0')
		obj = ch->on;
    else if ( ( obj = get_obj_list( ch, argument, ch->carrying ) ) != NULL )
	{
		/*ptc( ch, "found a %s.\n\r", obj->short_descr );*/
	}
	else
	{
		stc( "Could not find that.\n\r", ch );
	}


    if (obj == NULL)
    {
    	obj = get_obj_list( ch, argument,  ch->in_room->contents );
		if (obj == NULL)
		{
			stc("You can't find it.\n\r",ch);
			return;
		}
    }
	/* this was causing rest obj to fail */
    /*else obj = ch->on;*/
#endif
    if (argument[0] == '\0')
		obj = ch->on;
    else if ( ( obj = get_obj_list( ch, argument, ch->carrying ) ) == NULL )
	{
    	    	obj = get_obj_list( ch, argument,  ch->in_room->contents );
		if (obj == NULL)
		{
			stc("You can't find it.\n\r",ch);
			return;
		}
	}



    if (obj != NULL)
    {
        if (obj->item_type != ITEM_FURNITURE
    	||  (!IS_SET(obj->value[2],REST_ON)
    	&&   !IS_SET(obj->value[2],REST_IN)
    	&&   !IS_SET(obj->value[2],REST_AT)))
    	{
	    stc( "{cYou can't rest on that.\n\r{c",ch);
	    return;
    	}

        if (obj != NULL && ch->on != obj && count_users(obj) >= obj->value[0])
        {
	    act_new("{cThere's no more room on $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
	    return;
    	}

		ch->on = obj;

		if ( HAS_TRIGGER_OBJ( obj, TRIG_SIT ) )
		p_percent_trigger( NULL, obj, NULL, ch, NULL, NULL, TRIG_SIT );

    }

    switch ( ch->position )
    {
	    case POS_SLEEPING:
	    if ( IS_SET(ch->spec_flags,SPEC_DREAMING ) )
	    {
		    stc("You must leave Tel'aran'rhiod first.\n\r",ch);
		    return;
	    }
		if (IS_AFFECTED(ch,AFF_SLEEP))
		{
		    stc( "{cYou can't wake up!\n\r{c",ch);
		    return;
		}
		if (obj == NULL)
		{
		    stc( "{cYou wake up and start resting.\n\r{c", ch );
		    act( "{c$n wakes up and starts resting.{c",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_SET(obj->value[2],REST_AT))
		{
		    act_new("{cYou wake up and rest at $p.{c",
			    ch,obj,NULL,TO_CHAR,POS_SLEEPING);
		    act( "{c$n wakes up and rests at $p.{c",ch,obj,NULL,TO_ROOM);
		}
        else if (IS_SET(obj->value[2],REST_ON))
        {
            act_new("{cYou wake up and rest on $p.{c",
                    ch,obj,NULL,TO_CHAR,POS_SLEEPING);
            act( "{c$n wakes up and rests on $p.{c",ch,obj,NULL,TO_ROOM);
        }
        else
        {
            act_new("{cYou wake up and rest in $p.{c",
                    ch,obj,NULL,TO_CHAR,POS_SLEEPING);
            act( "{c$n wakes up and rests in $p.{c",ch,obj,NULL,TO_ROOM);
        }
		ch->position = POS_RESTING;
		break;

    case POS_RESTING:
		stc( "{cYou are already resting.\n\r{c", ch );
	break;

    case POS_STANDING:
		if (obj == NULL)
		{
		    stc( "{cYou rest.\n\r{c", ch );
		    act( "{c$n sits down and rests.{c", ch, NULL, NULL, TO_ROOM );
		}
        else if (IS_SET(obj->value[2],REST_AT))
        {
		    act( "{cYou sit down at $p and rest.{c",ch,obj,NULL,TO_CHAR);
		    act( "{c$n sits down at $p and rests.{c",ch,obj,NULL,TO_ROOM);
        }
        else if (IS_SET(obj->value[2],REST_ON))
        {
		    act( "{cYou sit on $p and rest.{c",ch,obj,NULL,TO_CHAR);
		    act( "{c$n sits on $p and rests.{c",ch,obj,NULL,TO_ROOM);
        }
        else
        {
		    act( "{cYou rest in $p.{c",ch,obj,NULL,TO_CHAR);
		    act( "{c$n rests in $p.{c",ch,obj,NULL,TO_ROOM);
        }
		ch->position = POS_RESTING;
	break;

    case POS_SITTING:
		if (obj == NULL)
		{
		    stc( "{cYou rest.\n\r{c",ch);
		    act( "{c$n rests.{c",ch,NULL,NULL,TO_ROOM);
		}
        else if (IS_SET(obj->value[2],REST_AT))
        {
		    act( "{cYou rest at $p.{c",ch,obj,NULL,TO_CHAR);
		    act( "{c$n rests at $p.{c",ch,obj,NULL,TO_ROOM);
        }
        else if (IS_SET(obj->value[2],REST_ON))
        {
		    act( "{cYou rest on $p.{c",ch,obj,NULL,TO_CHAR);
		    act( "{c$n rests on $p.{c",ch,obj,NULL,TO_ROOM);
        }
        else
        {
		    act( "{cYou rest in $p.{c",ch,obj,NULL,TO_CHAR);
		    act( "{c$n rests in $p.{c",ch,obj,NULL,TO_ROOM);
		}
		ch->position = POS_RESTING;
		break;
    }
    return;
}


void do_sit (CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;

    if (MOUNTED(ch))
    {
        send_to_char("You can't sit while mounted.\n\r", ch);
        return;
    }

    if (RIDDEN(ch))
    {
        send_to_char("You can't sit while being ridden.\n\r", ch);
        return;
    }

    if (ch->position == POS_FIGHTING)
    {
		stc( "{cMaybe you should finish this fight first?\n\r{c",ch);
		return;
    }

	/* fix for letting chars sleep on items in inv - Dalsor */
    if (argument[0] == '\0')
		obj = ch->on;
    else
		obj = get_obj_list( ch, argument,  ch->carrying );

    if (obj == NULL)
    {
    	obj = get_obj_list( ch, argument,  ch->in_room->contents );
		if (obj == NULL)
		;
    }
    /* okay, now that we know we can sit, find an object to sit on
    if (argument[0] != '\0')
    {
	obj = get_obj_list(ch,argument,ch->in_room->contents);
	if (obj == NULL)
	{
	    stc( "{cYou don't see that here.\n\r{c",ch);
	    return;
	}
    }
    else obj = ch->on;*/

    if (obj != NULL)
    {
	if (obj->item_type != ITEM_FURNITURE
	||  (!IS_SET(obj->value[2],SIT_ON)
	&&   !IS_SET(obj->value[2],SIT_IN)
	&&   !IS_SET(obj->value[2],SIT_AT)))
	{
	    stc( "{cYou can't sit on that.\n\r{c",ch);
	    return;
	}

	if (obj != NULL && ch->on != obj && count_users(obj) >= obj->value[0])
	{
	    act_new("{cThere's no more room on $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
	    return;
	}

	ch->on = obj;

	if ( HAS_TRIGGER_OBJ( obj, TRIG_SIT ) )
		p_percent_trigger( NULL, obj, NULL, ch, NULL, NULL, TRIG_SIT );

    }
    switch (ch->position)
    {
		case POS_SLEEPING:
		    if ( IS_SET(ch->spec_flags,SPEC_DREAMING ) )
		    {
			    stc("You must leave Tel'aran'rhiod first.\n\r",ch);
			    return;
		    }
		    if (IS_AFFECTED(ch,AFF_SLEEP))
		    {
				stc( "{cYou can't wake up!\n\r{c",ch);
				return;
		    }

            if (obj == NULL)
            {
            	stc( "{cYou wake and sit up.\n\r{c", ch );
            	act( "{c$n wakes and sits up.{c", ch, NULL, NULL, TO_ROOM );
            }
            else if (IS_SET(obj->value[2],SIT_AT))
            {
            	act_new("{cYou wake and sit at $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
            	act( "{c$n wakes and sits at $p.{c",ch,obj,NULL,TO_ROOM);
            }
            else if (IS_SET(obj->value[2],SIT_ON))
            {
            	act_new("{cYou wake and sit on $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
            	act( "{c$n wakes and sits at $p.{c",ch,obj,NULL,TO_ROOM);
            }
            else
            {
            	act_new("{cYou wake and sit in $p.{c",ch,obj,NULL,TO_CHAR,POS_DEAD);
            	act( "{c$n wakes and sits in $p.{c",ch,obj,NULL,TO_ROOM);
            }

		    ch->position = POS_SITTING;
		    break;
		case POS_RESTING:
		    if (obj == NULL)
				stc( "{cYou stop resting.\n\r{c",ch);
		    else if (IS_SET(obj->value[2],SIT_AT))
		    {
				act( "{cYou sit at $p.{c",ch,obj,NULL,TO_CHAR);
				act( "{c$n sits at $p.{c",ch,obj,NULL,TO_ROOM);
		    }

		    else if (IS_SET(obj->value[2],SIT_ON))
		    {
				act( "{cYou sit on $p.{c",ch,obj,NULL,TO_CHAR);
				act( "{c$n sits on $p.{c",ch,obj,NULL,TO_ROOM);
		    }
		    ch->position = POS_SITTING;
		    break;

		case POS_SITTING:
		    stc( "{cYou are already sitting down.\n\r{c",ch);
	    break;

		case POS_STANDING:
		    if (obj == NULL)
    	    {
				stc( "{cYou sit down.\n\r{c",ch);
    	        act( "{c$n sits down on the ground.{c",ch,NULL,NULL,TO_ROOM);
		    }
		    else if (IS_SET(obj->value[2],SIT_AT))
		    {
				act( "{cYou sit down at $p.{c",ch,obj,NULL,TO_CHAR);
				act( "{c$n sits down at $p.{c",ch,obj,NULL,TO_ROOM);
		    }
		    else if (IS_SET(obj->value[2],SIT_ON))
		    {
				act( "{cYou sit on $p.{c",ch,obj,NULL,TO_CHAR);
				act( "{c$n sits on $p.{c",ch,obj,NULL,TO_ROOM);
		    }
		    else
		    {
				act( "{cYou sit down in $p.{c",ch,obj,NULL,TO_CHAR);
				act( "{c$n sits down in $p.{c",ch,obj,NULL,TO_ROOM);
		    }
    	    ch->position = POS_SITTING;
    	    break;
    }
    return;
}


void do_sleep( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;

    if (MOUNTED(ch))
    {
        send_to_char("You can't sleep while mounted.\n\r", ch);
        return;
    }

    if (RIDDEN(ch))
    {
        send_to_char("You can't sleep while being ridden.\n\r", ch);
        return;
    }

    switch ( ch->position )
    {
    case POS_SLEEPING:
	stc( "{cYou are already sleeping.\n\r{c", ch );
	break;

	/* no sleeping while hold the source */
	do_function(ch, &do_release, "");

    case POS_RESTING:
    case POS_SITTING:
    case POS_STANDING:
	while (ch->weaves && ch->weaves->action[0] == ACTION_CASTING)
	    stop_spell( ch );
	if (argument[0] == '\0' && ch->on == NULL)
	{
	    if (IS_SET( ch->op_flags, OP_GRASPED))
	    {
	    	do_function(ch, &do_release, "");
	    	stc("You release your hold on the True Source.\n\r",ch);
		}
	    stc( "{cYou go to sleep.\n\r{c", ch );
	    act( "{c$n goes to sleep.{c", ch, NULL, NULL, TO_ROOM );
	    ch->position = POS_SLEEPING;
	}
	else  /* find an object and sleep on it */
	{
	    if (argument[0] == '\0')
			obj = ch->on;
	    else
			obj = get_obj_list( ch, argument,  ch->carrying );

	    if (obj == NULL)
	    {
	    	obj = get_obj_list( ch, argument,  ch->in_room->contents );
			if (obj == NULL)
			{
			/*stc( "{cYou don't see that here.\n\r{c",ch);*/
			stc("You can't find it.\n\r",ch);
			return;
			}
	    }

	    if (obj->item_type != ITEM_FURNITURE
	    ||  (!IS_SET(obj->value[2],SLEEP_ON)
	    &&   !IS_SET(obj->value[2],SLEEP_IN)
	    &&	 !IS_SET(obj->value[2],SLEEP_AT)))
	    {
			stc( "{cYou can't sleep on that!\n\r{c",ch);
			return;
	    }

	    if (ch->on != obj && count_users(obj) >= obj->value[0])
	    {
			act_new("{cThere is no room on $p for you.{c",
			    ch,obj,NULL,TO_CHAR,POS_DEAD);
		return;
	    }

	    ch->on = obj;

		if ( HAS_TRIGGER_OBJ( obj, TRIG_SIT ) )
			p_percent_trigger( NULL, obj, NULL, ch, NULL, NULL, TRIG_SIT );

	    if (IS_SET(obj->value[2],SLEEP_AT))
	    {
	    if (IS_SET( ch->op_flags, OP_GRASPED))
	    {
	    	do_function(ch, &do_release, "");
	    	stc("You release your hold on the True Source.\n\r",ch);
		}
		act( "{cYou go to sleep at $p.{c",ch,obj,NULL,TO_CHAR);
		act( "{c$n goes to sleep at $p.{c",ch,obj,NULL,TO_ROOM);
	    }
	    else if (IS_SET(obj->value[2],SLEEP_ON))
	    {
	    if (IS_SET( ch->op_flags, OP_GRASPED))
	    {
	    	do_function(ch, &do_release, "");
	    	stc("You release your hold on the True Source.\n\r",ch);
		}
	        act( "{cYou go to sleep on $p.{c",ch,obj,NULL,TO_CHAR);
	        act( "{c$n goes to sleep on $p.{c",ch,obj,NULL,TO_ROOM);
	    }
	    else
	    {
	    if (IS_SET( ch->op_flags, OP_GRASPED))
	    {
	    	do_function(ch, &do_release, "");
	    	stc("You release your hold on the True Source.\n\r",ch);
		}
		act( "{cYou go to sleep in $p.{c",ch,obj,NULL,TO_CHAR);
		act( "{c$n goes to sleep in $p.{c",ch,obj,NULL,TO_ROOM);
	    }
	    ch->position = POS_SLEEPING;
	}
	break;

    case POS_FIGHTING:
	stc( "{cYou are already fighting!\n\r{c", ch );
	break;
    }

    return;
}



void do_wake( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );
    if ( IS_SET(ch->spec_flags,SPEC_DREAMING ) )
    {
    stc("You must leave Tel'aran'rhiod first.\n\r",ch);
    return;
    }
    if ( arg[0] == '\0' )
	{ do_function(ch, &do_stand, ""); return; }

    if ( !IS_AWAKE(ch) )
	{ stc( "{cYou are asleep yourself!\n\r{c",       ch ); return; }

    if ( ( victim = get_char_room( ch,NULL,arg ) ) == NULL )
	{ stc( "{cThey aren't here.\n\r{c",              ch ); return; }

    if ( IS_AWAKE(victim) )
	{ act( "{c$N is already awake.{c", ch, NULL, victim, TO_CHAR ); return; }

    if ( IS_AFFECTED(victim, AFF_SLEEP) )
	{ act( "{cYou can't wake $M!{c",   ch, NULL, victim, TO_CHAR );  return; }

    act_new("{c$n is trying to wake you.{c", ch, NULL, victim, TO_VICT,POS_SLEEPING );
    /*do_function(ch, &do_stand, "");*/
    return;
}



void do_sneak( CHAR_DATA *ch, char *argument )
{
	AFFECT_DATA af;

	if (MOUNTED(ch))
	{
		stc("You can't sneak while riding.\n\r", ch);
		return;
	}

	if ( !IS_NPC(ch) && get_skill( ch, gsn_sneak ) <= 0 )
	{
		stc("You are too clumsy to sneak.\n\r", ch );
		return;
	}

	if (IS_AFFECTED(ch,AFF_SNEAK))
	{
		stc( "You already move silently.\n\r", ch );
		return;
	}
	if ( number_percent( ) < get_skill(ch,gsn_sneak))
	{
		check_improve(ch,gsn_sneak,TRUE,5);
		af.where     = TO_AFFECTS;
		af.type      = gsn_sneak;
		af.level     = ch->level;
		af.duration  = ch->level;
		af.location  = APPLY_NONE;
		af.modifier  = 0;
		af.bitvector = AFF_SNEAK;
		affect_to_char( ch, &af );
		stc( "You begin to move silently.\n\r", ch );
	}
	else
	{
		check_improve(ch,gsn_sneak,FALSE,3);
		stc( "You fail to move silently.\n\r", ch );
	}
	return;
}
void do_hide( CHAR_DATA *ch, char *argument )
{
	if (MOUNTED(ch))
	{
		stc("You can't hide while riding.\n\r", ch);
		return;
	}

	stc( "{cYou attempt to hide.\n\r{c", ch );

	if ( IS_AFFECTED(ch, AFF_HIDE) )
		REMOVE_BIT(ch->affected_by, AFF_HIDE);

	if ( number_percent( ) < get_skill(ch,gsn_hide))
	{
		stc("You slip into the shadows.\n\r",ch);
		SET_BIT(ch->affected_by, AFF_HIDE);
		check_improve(ch,gsn_hide,TRUE,3);
	}
	else
	{
		stc("You attempt to hide, but fail to find a suitable spot.\n\r",ch);
		check_improve(ch,gsn_hide,FALSE,3);
	}
	return;
}



/*
 * Contributed by Alander.
 */
void do_visible( CHAR_DATA *ch, char *argument )
{
    affect_strip ( ch, gsn_invis			);
    affect_strip ( ch, gsn_mass_invis			);
    affect_strip ( ch, gsn_sneak			);
    REMOVE_BIT   ( ch->affected_by, AFF_HIDE		);
    REMOVE_BIT   ( ch->affected_by, AFF_INVISIBLE	);
    REMOVE_BIT   ( ch->affected_by, AFF_SNEAK		);
    stc( "{cOk.\n\r{c", ch );
    return;
}



void do_recall( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    DESCRIPTOR_DATA *d_mentor;
    ROOM_INDEX_DATA *location;
	int lose;

    /*if (IS_NPC(ch) && !IS_SET(ch->act,ACT_PET))*/
    if (IS_NPC(ch) && !IS_SET(ch->act,ACT_PET) && !ch->mount )
    {
		stc( "{cOnly players can recall.\n\r{c",ch);
		return;
    }
    if ( ch->pcdata->in_vault )
    {
    do_function( ch, &do_locker, "close" );
    }
    if ( ch->pcdata->in_vault )
    {
	    stc( "You can't recall here. Type vault close to leave.\n\r",ch);
	    return;
    }

    if ( ch->pcdata && ch->pcdata->allowed == 0 )
    {
		stc( "You are not allowed to leave the school until a Staff member\n\r",ch);
		stc( "has granted you access to the game. You should be approached\n\r",ch);
		stc( "soon by a Mentor. If you are not approached within an hour or\n\r",ch);
		stc( "two, you may ask over OOC, or by sending a Note to Staff.\n\r",ch);
        for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
        {
	        if ( d_mentor->connected == CON_PLAYING
            && IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
	        {
            act("{w({GMentor Alert{w){W::{x {G$N needs verification!{x",
            d_mentor->character,NULL,ch,TO_CHAR);
	        }
	    }
		return;
	}

	if ( !IS_NPC( ch ) )
	{
	    if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
	    {
	      send_to_char ("Wait till the tag game is over.\n\r", ch);
	      return;
	    }
	}

	if ( IS_SET( ch->war_flags, WAR_BATTLE ) )
	{
		stc("Not from a war.\n\r",ch);
		return;
	}
	if ( IS_SET( ch->war_flags, WAR_TOURNEY ) )
	{
		stc("Not from a tourney.\n\r",ch);
		return;
	}
	if (is_shadow(ch))
    {
	    act( "{c$n beseeches transportation and receives it.{c",ch, 0,0,TO_ROOM);
		stc("You beseech the Great Lord for aid!\n\r",ch);
	}
    else
    {
	    act( "{c$n beseeches transportation and receives it.{c", ch, 0, 0, TO_ROOM );
		stc("You beseech the Creator for aid!\n\r",ch);
	}

	if ( !IS_NPC( ch ) )
	{
	    if ( ( location = get_room_index( pc_race_table[ch->race].hometown ) ) == NULL )
	    {
			stc( "{cYou are completely lost.\n\r{c", ch );
			return;
	    }
	}
	else
		location = get_room_index( ROOM_VNUM_PLAZA );

    if ( ch->in_room == location )
    {
	stc("{cBut you're already there!\n\r",ch);
	return;
	}

    if ( IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_AFFECTED(ch, AFF_CURSE))
    {
        if (is_shadow(ch))
		{
			stc( "{cThe Great Lord has forsaken you.\n\r", ch );
			return;
		}
	else
		{
			stc( "{cThe Creator has forsaken you.\n\r", ch );
			return;
	    }
	}

    if (is_fighting(ch) && ( victim = ch->advancing_on ) != NULL )
    {
	if ( number_percent() > 60 )
	{
	    WAIT_STATE( ch, 4 );
	    sprintf( buf, "{cYou failed!.\n\r");
	    stc( buf, ch );
	    return;
	}

	lose = (ch->desc != NULL) ? 25 : 50;
	gain_exp( ch, 0 - lose );
	sprintf( buf, "{cYou recall from combat!  You lose %d exps.\n\r{c", lose );
	stc( buf, ch );
	stop_all_fighting( ch);

    }

    act( "$n disappears.", ch, NULL, NULL, TO_ROOM );
    char_from_room( ch );
    char_to_room( ch, location );
    act( "$n stumbles into the room, trembling from some horrible experience.", ch, NULL, NULL, TO_ROOM );
    do_function(ch, &do_look, "auto" );
    stc("You stumble into the room, exhausted from your trip through {wTel'aran'rhiod{c.\n\r",ch);

    if (ch->pet != NULL)
    {
		char_from_room( ch->pet );
		char_to_room( ch->pet, ch->in_room );
	}

    if (ch->mount != NULL)
    {
		char_from_room( ch->mount );
		char_to_room( ch->mount, ch->in_room );
	}

    return;
}

void do_rprecall( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    ROOM_INDEX_DATA *location;
	int lose;
    if (IS_NPC(ch) && !IS_SET(ch->act,ACT_PET) && !ch->mount )
    {
		stc( "{cOnly players can rprecall.\n\r{c",ch);
		return;
    }
    if ( ch->pcdata->in_vault )
    {
    do_function( ch, &do_locker, "close" );
    }

	if ( !IS_NPC( ch ) )
	{
	    if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
	    {
	      send_to_char ("Wait till the tag game is over.\n\r", ch);
	      return;
	    }
	}
	if ( IS_SET( ch->war_flags, WAR_BATTLE ) )
	{
		stc("Not from a war.\n\r",ch);
		return;
	}
	if ( IS_SET( ch->war_flags, WAR_TOURNEY ) )
	{
		stc("Not from a tourney.\n\r",ch);
		return;
	}
	act( "{c$n wanders away looking for some roleplay.{c",ch, 0,0,TO_ROOM);
	stc("You wander off to the Eye of the World Inn searching for roleplay.\n\r",ch);

	location = get_room_index( 905 );

    if ( ch->in_room == location )
    {
		stc("{cBut you're already there!\n\r",ch);
		return;
	}

    if ( IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_AFFECTED(ch, AFF_CURSE))
    {
        if (is_shadow(ch))
		{
			stc( "{cThe Great Lord has forsaken you.\n\r", ch );
			return;
		}
		else
		{
			stc( "{cThe Creator has forsaken you.\n\r", ch );
			return;
	    }
	}
    if (is_fighting(ch) && ( victim = ch->advancing_on ) != NULL )
    {
	if ( number_percent() > 60 )
	{
	    WAIT_STATE( ch, 4 );
	    sprintf( buf, "{cYou failed!.\n\r");
	    stc( buf, ch );
	    return;
	}

	lose = (ch->desc != NULL) ? 25 : 50;
	gain_exp( ch, 0 - lose );
	check_improve(ch,gsn_recall,TRUE,4);
	sprintf( buf, "{cYou recall from combat!  You lose %d exps.\n\r{c", lose );
	stc( buf, ch );
	stop_all_fighting( ch);

    }

    act( "{c$n disappears.{c", ch, NULL, NULL, TO_ROOM );
    char_from_room( ch );
    char_to_room( ch, location );
    act( "{c$n arrives at the Inn from a pass through the mountains.{c", ch, NULL, NULL, TO_ROOM );
    do_function(ch, &do_look, "auto" );
    stc("{cYou arrive at the Inn through a pass between the mountains.\n\r",ch);

    if (ch->pet != NULL)
    {
		char_from_room( ch->pet );
		char_to_room( ch->pet, ch->in_room );
	}

    if (ch->mount != NULL)
    {
		char_from_room( ch->mount );
		char_to_room( ch->mount, ch->in_room );
	}

    return;
}

/*substitutes all %s in '*dir's*/
char *walkdir( char *argument, char *dir )
{
	char buf[MIL] = "";
	char buf1[MIL];
	char *tmp_string;

	while ((tmp_string = strstr(argument, "%s")) != NULL)
	{
		buf1[0] = '\0';
		strcpy(buf1, str_dup(argument));
		buf1[strlen(argument)-strlen(tmp_string)] = '\0';
		strcat(buf, buf1);
		strcat(buf, dir);
		++tmp_string;
		argument = ++tmp_string;
	}
	strcat(buf, str_dup(argument));

	return str_dup(buf);
}


/* added for Walkcode */
void do_walkin( CHAR_DATA *ch, char *argument )
{
    char buf[MSL];
    char *color_free_arg;
    int played;

	if (IS_SET(ch->penalty_flags,PENAL_NOWALK))
	{
		stc("You cannot set your Walkin.\n\r",ch);
		return;
	}
    if ( !IS_NPC(ch) )
    {
		smash_tilde( argument );

		played = ( ch->played + (int) (current_time - ch->logon) ) / 3600;
		if ( played < 4 && ch->pcdata->rebirth <= 0 )
		{
			stc("You must have 4 hours of playing time to set your Walkin.\n\r",ch);
			return;
		}
		if (argument[0] == '\0')
		{
	/*	    sprintf(buf,"{cYour Walk In is %s\n\r{c",walkdir(ch->pcdata->wkin, "<dir>")); */
			/* FIXME rewrite to use $T of acts etc */
		    sprintf(buf,"{cYour Walk In is %s\n\r{c",ch->pcdata->wkin);
		    stc(buf,ch);
		    return;
		}

		if (!strcmp(argument, "clear"))
		{
			/*(ch->pcdata->wkin = "");*/
			free_string( ch->pcdata->wkin );
			/*ch->pcdata->wkin = NULL;*/
			ch->pcdata->wkin = str_dup("");
			stc( "WalkIn cleared.\n\r", ch);
			return;
		}

		if ( strlen( argument ) < 20 )
		{
			stc( "The WalkIn message must be at least twenty characters in length.\n\r", ch);
			return;
		}

		color_free_arg = color_strip(argument);
		if ( strstr( color_free_arg,ch->name ) == NULL )
		{
		    stc( "{cYou must include your name.\n\r",ch);
		    free_string(color_free_arg);
		    return;  /* this return makes the free_string tricky */
		}
		else
		    free_string(color_free_arg);

        free_string( ch->pcdata->wkin );
        ch->pcdata->wkin = str_dup( argument );

        ptc( ch, "Your Walk In is %s\n\r",ch->pcdata->wkin );
        stc( "Check out RPScore to see other messages.\n\r", ch );
    }
    return;
}

void do_walkout( CHAR_DATA *ch, char *argument )
{
    char buf[MSL];
    char *color_free_arg;
    int played;

	if (IS_SET(ch->penalty_flags,PENAL_NOWALK))
	{
		stc("You cannot set your Walkout.\n\r",ch);
		return;
	}
    if ( !IS_NPC(ch) )
    {
        smash_tilde( argument );
		played = ( ch->played + (int) (current_time - ch->logon) ) / 3600;
		if ( played < 4 && ch->pcdata->rebirth <= 0 )
		{
			stc("You must have 4 hours of playing time to set your Walkout.\n\r",ch);
			return;
		}

        if (argument[0] == '\0')
        {
			sprintf(buf,"{cYour Walk Out is %s\n\r{c",ch->pcdata->wout);
			stc(buf,ch);
            return;
        }

		if (!strcmp(argument, "clear"))
		{
			free_string( ch->pcdata->wout );
			ch->pcdata->wout = str_dup("");;
			stc( "{cWalk Out cleared.\n\r{c", ch);
			return;
		}
		if ( strlen( argument ) < 20 )
		{
			stc( " The WalkIn message must be at least twenty characters in length.\n\r", ch);
			return;
		}
		color_free_arg = color_strip(argument);
		if ( strstr(color_free_arg,ch->name) == NULL)
		{
		    stc( "{cYou must include your name.\n\r",ch);
		    free_string(color_free_arg);
		    return;  /* this return makes the free_string tricky */
		}
		else
		    free_string(color_free_arg);

        free_string( ch->pcdata->wout );
        ch->pcdata->wout = str_dup( argument );

/*        sprintf(buf,"{cYour Walk Out is %s\n\r{c",walkdir(ch->pcdata->wout, "<dir>"));*/
        ptc( ch,"Your Walk Out is %s\n\r",ch->pcdata->wout);
        stc( "Check out RPScore to see other messages.\n\r", ch );
    }
    return;
}

void do_subclanrecall (CHAR_DATA *ch, char *argument)
{
	do_clanrecall(ch, "sub");
	return;
}

void do_clanrecall( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    DESCRIPTOR_DATA *d_mentor;
    ROOM_INDEX_DATA *location;
    bool srecall = FALSE;

	if (argument[0] != '\0' && !str_cmp(argument, "sub"))
		srecall = TRUE;

    if (IS_NPC(ch) && !IS_SET(ch->act,ACT_PET) && !ch->mount )
    {
 	send_to_char("Only players can recall.\n\r",ch);
 	return;
	}
    if ( ch->pcdata->in_vault )
    {
    do_function( ch, &do_locker, "close" );
    }
    if ( ch->pcdata->in_vault )
    {
	    stc( "You can't recall here. Type vault close to leave.\n\r",ch);
	    return;
    }

    if ( ch->pcdata && ch->pcdata->allowed == 0 )
    {
		stc( "{GYou are not allowed to leave the school until a Staff member\n\r",ch);
		stc( "has granted you access to the game. You should be approached\n\r",ch);
		stc( "soon by a Mentor. If you are not approached within an hour or\n\r",ch);
		stc( "two, you may ask over OOC, or by sending a Note to Staff.{x\n\r",ch);
        for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
        {
	        if ( d_mentor->connected == CON_PLAYING
            && IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
	        {
            act("{w({GMentor Alert{w){W::{x {G$N needs verification!{x",
            d_mentor->character,NULL,ch,TO_CHAR);
	        }
	    }
		return;
	}
	if ( IS_SET( ch->war_flags, WAR_BATTLE ) )
	{
		stc("Not from a war.\n\r",ch);
		return;
	}
    if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
    {
      send_to_char ("Wait till the tag game is over.\n\r", ch);
      return;
    }

    if (is_shadow(ch))
    {
	    act( "{c$n beseeches transportation and receives it.{c",ch, 0,0,TO_ROOM);
		stc("{DYou beseech the {RGreat Lord {Dfor aid!{x\n\r",ch);
	}
    else
    {
	    act( "{c$n beseeches transportation and receives it.{c", ch, 0, 0, TO_ROOM );
		stc("{rYou beseech the {WCreator{r for aid!{c\n\r",ch);
	}

	/*location = get_room_index( clan_table[ch->clan].rec_room);*/

    /*if ( ( location = get_room_index( clan_table[ch->clan].rec_room) ) == NULL )*/
	if ( ( location = get_room_index( srecall ? clan_table[ch->sub_clan].room[0] : clan_table[ch->clan].room[0] ) ) == NULL )
    {
	stc( "{cYou are completely lost.\n\r", ch );
	return;
    }

    if ( ch->in_room == location )
    {
	stc("{cBut you're already there!\n\r",ch);
	return;
	}

    if ( IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_AFFECTED(ch, AFF_CURSE))
    {
       if (is_shadow(ch))
	{
		stc( "{cThe Great Lord has forsaken you.\n\r", ch );
		return;
	}
	else
	{
		stc( "{cThe Creator has forsaken you.\n\r", ch );
		return;
    }
	}
    if (is_fighting(ch) && ( victim = ch->advancing_on ) != NULL )
    {
	int lose;

	if ( number_percent() > 60 )
	{
	    WAIT_STATE( ch, 4 );
	    sprintf( buf, "{cYou failed!.\n\r");
	    stc( buf, ch );
	    return;
	}

	lose = (ch->desc != NULL) ? 25 : 50;
	gain_exp( ch, 0 - lose );
	sprintf( buf, "{cYou recall from combat!  You lose %d exps.\n\r{c", lose );
	stc( buf, ch );
	stop_all_fighting( ch);
    }

    act( "{c$n disappears.", ch, NULL, NULL, TO_ROOM );
    char_from_room( ch );
    char_to_room( ch, location );
    if( MOUNTED(ch) )
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), location );
    }
    act( "$n stumbles into the room, trembling from some horrible experience.", ch, NULL, NULL, TO_ROOM );
    do_look( ch, "auto" );
    stc("You stumble into the room, exhausted from your trip through {wTel'aran'rhiod{c.\n\r",ch);

    if (ch->pet != NULL)
    {
		char_from_room( ch->pet );
		char_to_room( ch->pet, location );
	}

    return;
}


/**************************************************************************/

/*
 *  This function allows a character to knock on a door.
 */

void do_knock(CHAR_DATA *ch, char *argument)
{
  /* Constructs taken from do_open().  */
  int door;
  char arg[MAX_INPUT_LENGTH];

  one_argument(argument,arg);

  if (arg[0] == '\0')
    {
      send_to_char("Knock on what?\n\r",ch);
      return;
    }

  if ( ( door = find_door( ch, arg, FALSE ) ) >= 0 )
    {
      ROOM_INDEX_DATA *to_room;
      EXIT_DATA *pexit;
      EXIT_DATA *pexit_rev;

      pexit = ch->in_room->exit[door];

      act( "$n knocks on the $d.", ch, NULL, pexit->keyword, TO_ROOM );
      act( "You knock on the $d.", ch, NULL, pexit->keyword, TO_CHAR );

      /* Notify the other side.  */
      if (   ( to_room   = pexit->u1.to_room            ) != NULL
          && ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
          && pexit_rev->u1.to_room == ch->in_room )
        {
          CHAR_DATA *rch;
          for ( rch = to_room->people; rch != NULL; rch = rch->next_in_room )
            act( "A knocking sound comes from the other side of the $d.", rch, NULL, pexit_rev->keyword, TO_CHAR );
        }
    }

  return;
}

/* Climbing exits. Great for tree and mountain rooms - Talon of TOTG MUD
  * tgods.mudhaven.com 8000
  * Email: tgods@mudhaven.com. All code copyrighted (c) 1999 NML
  */
void do_climb( CHAR_DATA *ch, char *argument )
{
    EXIT_DATA       *pexit;
    int             door;
    int skill, grade;
    ROOM_INDEX_DATA *pRoom;
    OBJ_DATA *eq, *piton = NULL;
    bool use_piton = FALSE;
    char arg1[MIL], arg2[MIL], arg3[ML];

    if ( IS_NPC( ch ) )
    	skill = 85;
    else
    	skill = ch->pcdata->learned[gsn_climb];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

    if ( arg1[0] == '\0' )
    {
        stc( "What direction do you want to climb?\n\r", ch );
        return;
    }

    if ( !str_cmp( arg1, "?" ) || !str_cmp( arg1, "help" ) )
    {
        stc( "Syntax:\n\r", ch );
        stc( "climb <direction>\n\r", ch );
        stc( "climb <direction> with <piton name>\n\r", ch );
        stc( "Note: If you are holding a rope it will automaticly be used.\n\r", ch );
        stc( "If you climb using a piton, the piton will be lost upon use if the climb succeeds.\n\r", ch );
        return;
    }
    if ( arg2 != NULL && arg2[0] != '\0' && strcmp( arg2, "with" ) )
    {
        stc( "If you are trying to use gear to help you climb, you must specify what it is with.\n\r", ch );
        do_function( ch, &do_climb, "?" );
        return;
    }
    if ( arg2 != NULL && arg2[0] != '\0' && ( arg3 == NULL || arg3[0] == '\0' ) )
    {
        stc( "What are you trying to climb with?\n\r", ch );
        do_function( ch, &do_climb, "?" );
        return;
    }
	if ( get_eq_char( ch, WEAR_HOLD ) != NULL )
	{
		eq = get_eq_char( ch, WEAR_HOLD );
		if ( eq->item_type == ITEM_ROPE )
		{
			ptc( ch, "You use %s to help you climb.\n\r", eq->short_descr );
			skill += 15;
		}
	}
    if ( arg2 != NULL && arg2[0] != '\0' && arg3 != NULL && arg3[0] != '\0' )
    {
		if ( ( piton = get_obj_carry( ch, arg3, ch ) ) != NULL )
		{
			if ( piton->item_type == ITEM_PITON )
			{
				ptc( ch, "You use %s to help you climb.\n\r", piton->short_descr );
				skill += 30;
				use_piton = TRUE;
			}
			else
			{
				ptc( ch, "%s will not help you climb.\n\r", capit2( piton->short_descr ) );
				return;
			}
		}
		else
		{
			stc("Whatever that is you're trying to use will not help you.\n\r", ch);
			return;
		}
    }

         if ( !str_prefix( arg1, "north" ) ) door = DIR_NORTH;
    else if ( !str_prefix( arg1, "east"  ) ) door = DIR_EAST;
    else if ( !str_prefix( arg1, "south" ) ) door = DIR_SOUTH;
    else if ( !str_prefix( arg1, "west"  ) ) door = DIR_WEST;
    else if ( !str_prefix( arg1, "up"    ) ) door = DIR_UP;
    else if ( !str_prefix( arg1, "down"  ) ) door = DIR_DOWN;
    else
    {
        for ( door = 0; door < MAX_DIR; door++ )
        {
            if ( ( pexit = ch->in_room->exit[door] )
                && pexit->keyword
                && is_name( arg1, pexit->keyword ) )
                return;
        }
        act( "You can't go $T here.", ch, NULL, arg1, TO_CHAR );
        return;
    }

    /* This is a bit repetitive but just in case */
    if ( !( pexit = ch->in_room->exit[door] ) )
    {
        act( "There is no direction $T here.", ch, NULL, arg1, TO_CHAR );
        return;
    }

    /*if ( !IS_SET( pexit->exit_info, EX_CLIMB ) )*/
	if (!IS_SET(pexit->u1.to_room->room_flags, ROOM_CLIMB))
    {
        act( "You don't have to climb in that direction.",
	     ch, NULL, argument, TO_CHAR );
        return;
    }

    if ( IS_SET( pexit->exit_info, EX_CLOSED ) )
    {
        act( "The exit $T is closed.", ch, NULL, arg1, TO_CHAR );
        return;
    }

    pRoom = pexit->u1.to_room;
    grade = pRoom->grade;

    if ( grade > 0 )
    {
		int chance = number_range( 10, 100 );
		int diff = ( grade * 10 ) + ( ( ( chance * 9 ) / 10 ) );
/*
		if ( ch->pcdata->true_sex == SEX_MALE && ch->pcdata->beard == BEARD_MAX-1 )
		{
			stc( "Your long beard hampers your climb as it tries to tangle in your hands.\n\r", ch );
			diff += 7;
		}
		if ( ch->pcdata->true_sex == SEX_FEMALE && ch->pcdata->hair == 6 )
		{
			stc( "Your mess of hair hampers your climb as it tries to tangle in your hands.\n\r", ch );
			diff += 7;
		}
*/
		/*ptc( ch, "Skill: %d, Grade: %d, Chance: %d, Diff: %d.\n\r", skill, grade, chance, diff );*/
		if ( skill >= diff )
		{
		    act( "You climb $T.", ch, NULL, dir_name[door], TO_CHAR );
		    act( "$n climbs $T.", ch, NULL, dir_name[door], TO_ROOM );
		    char_from_room( ch );
		    char_to_room( ch, pexit->u1.to_room );
		    do_look( ch, "auto" );

		    act( "$n climbs in.", ch, NULL, NULL, TO_ROOM );
		    if ( use_piton == TRUE && piton != NULL )
		    {
				obj_from_char( piton );
				extract_obj( piton );
			}
		    check_improve(ch,gsn_climb,TRUE,6);
		}
		else
		{
		    check_improve(ch,gsn_climb,FALSE,6);
		    act( "You try to climb $T, but only manage to slip and slide.", ch, NULL, dir_name[door], TO_CHAR );
		    act( "$n tries to climb $T, but only manages to slip and slide.", ch, NULL, dir_name[door], TO_ROOM );
		}
	}
	else
	{
		/* this needs to go... all climb rooms need a grade */
	    act( "You climb $T.", ch, NULL, dir_name[door], TO_CHAR );
	    act( "$n climbs $T.", ch, NULL, dir_name[door], TO_ROOM );
	    char_from_room( ch );
	    char_to_room( ch, pexit->u1.to_room );
	    do_look( ch, "auto" );

	    act( "$n climbs in.", ch, NULL, NULL, TO_ROOM );
	}


    /*act( "$n climbs in from $T.", ch, NULL, rev_dir[door], TO_ROOM );*/
    return;
}

TAG_DATA tag_game;

void do_red(CHAR_DATA *ch, char *argument)
{
  DESCRIPTOR_DATA *d;
  char buf[MAX_STRING_LENGTH];

  if (IS_NPC (ch))
    return;

  if (tag_game.status == TAG_OFF)
  {
    send_to_char ("There is no tag game playing.\n\r", ch);
    return;
  }

  if (argument[0] == '\0')
  {
    send_to_char ("Syntax:  red <message>\n\r", ch);
    return;
  }

  if (!IS_IMMORTAL (ch)
      && !IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
  {
    send_to_char ("You must be a freeze tag player to use this channel.\n\r", ch);
    return;
  }

  if (IS_SET (ch->pcdata->tag_flags, TAG_BLUE))
  {
    sprintf (buf, "{R{{RED}{B %s{x: %s\n\r", ch->name, argument);
    send_to_char (buf, ch);
  }
  else
  {
    sprintf (buf, "{R{{RED}{R %s{x: %s\n\r", ch->name, argument);
  }

  for (d = descriptor_list; d != NULL; d = d->next)
  {
    if (d->connected == CON_PLAYING && d->character && !IS_NPC(d->character)
	&& IS_SET (d->character->pcdata->tag_flags, TAG_RED))
    {
      send_to_char (buf, d->character);
    }
  }
}

void do_blue(CHAR_DATA *ch, char *argument)
{
  DESCRIPTOR_DATA *d;
  char buf[MAX_STRING_LENGTH];

  if (IS_NPC (ch))
    return;

  if (tag_game.status == TAG_OFF)
  {
    send_to_char ("There is no tag game playing.\n\r", ch);
    return;
  }

  if (argument[0] == '\0')
  {
    send_to_char ("Syntax:  blue <message>\n\r", ch);
    return;
  }

  if (!IS_IMMORTAL (ch)
      && !IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
  {
    send_to_char ("You must be a freeze tag player to use this channel.\n\r", ch);
    return;
  }

  if (IS_SET (ch->pcdata->tag_flags, TAG_RED))
  {
    sprintf (buf, "{B{{BLUE}{R %s{x: %s\n\r", ch->name, argument);
    send_to_char (buf, ch);
  }
  else
  {
    sprintf (buf, "{B{{BLUE}{B %s{x: %s\n\r", ch->name, argument);
  }

  for (d = descriptor_list; d != NULL; d = d->next)
  {
    if (d->connected == CON_PLAYING && d->character && !IS_NPC(d->character)
	&& IS_SET (d->character->pcdata->tag_flags, TAG_BLUE))
    {
      send_to_char (buf, d->character);
    }
  }
}

void end_tag (void)
{
  DESCRIPTOR_DATA *d;

  tag_game.status = TAG_OFF;
  tag_game.timer = -1;
  tag_game.next = number_range (30, 50);
  tag_game.playing = 0;
  tag_game.rteam = 0;
  tag_game.bteam = 0;

  for (d = descriptor_list; d != NULL; d = d->next)
  {
    if (d->connected != CON_PLAYING || !d->character || IS_NPC(d->character))
      continue;

    if (IS_SET (d->character->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
    {
      char_from_room (d->character);
      char_to_room (d->character, get_room_index (ROOM_VNUM_PLAZA));
      do_look (d->character, "auto");
      send_to_char ("Freeze tag has been stopped!\n\r", d->character);
    }
    d->character->pcdata->tag_flags = 0;
  }
}

void start_tag (void)
{
  DESCRIPTOR_DATA *d;
  ROOM_INDEX_DATA *loc;
  int count = 0;
  char buf[MIL];

  tag_game.status = TAG_ISPLAY;
  tag_game.timer = 10;
  /*
  tag_game.rteam = 0;
  tag_game.bteam = 0;
  */
  for (d = descriptor_list; d != NULL; d = d->next)
  {
    if (d->connected != CON_PLAYING || !d->character || IS_NPC(d->character))
      continue;

    if (IS_SET (d->character->pcdata->tag_flags, TAG_WAITING))
    {
      count++;
      loc = get_room_index (number_range (FTAG_MIN_VNUM, FTAG_MAX_VNUM));
      REMOVE_BIT (d->character->pcdata->tag_flags, TAG_FROZEN);
      REMOVE_BIT (d->character->pcdata->tag_flags, TAG_WAITING);
      SET_BIT (d->character->pcdata->tag_flags, TAG_PLAYING);
      char_from_room (d->character);
      char_to_room (d->character, loc);
      do_look (d->character, "auto");
    }
  }
  sprintf(buf, "Freeze Tag has started! %d people playing.", count);
  tag_channel (NULL, buf);
}

bool fRed = FALSE;

void check_tag_victory(void)
{
	DESCRIPTOR_DATA *d;
	int bcount = 0, rcount = 0;
	bool bwin = FALSE, rwin = FALSE;
	char buf[MSL];
	buf[0] = '\0';

    for (d = descriptor_list; d != NULL; d = d->next)
	{
		if (d->connected == CON_PLAYING
		&& d->character && !IS_NPC(d->character)
		&& IS_SET (d->character->pcdata->tag_flags, TAG_PLAYING)
		&& IS_SET (d->character->pcdata->tag_flags, TAG_RED)
		&& IS_SET (d->character->pcdata->tag_flags, TAG_FROZEN))
			rcount += 1;
		if ( rcount == tag_game.rteam )
		{
			bwin = TRUE;
		}
	}
/*
	sprintf( buf, "RED: Count is now %d of %d", rcount, tag_game.rteam );
	tag_channel(NULL, buf );
*/
    for (d = descriptor_list; d != NULL; d = d->next)
	{
		if (d->connected == CON_PLAYING
		&& d->character && !IS_NPC(d->character)
		&& IS_SET (d->character->pcdata->tag_flags, TAG_PLAYING)
		&& IS_SET (d->character->pcdata->tag_flags, TAG_BLUE)
		&& IS_SET (d->character->pcdata->tag_flags, TAG_FROZEN))
			bcount += 1;
		if ( bcount == tag_game.bteam )
		{
			rwin = TRUE;
		}
	}
/*
	sprintf( buf, "BLUE: Count is now %d of %d", bcount, tag_game.bteam );
	tag_channel(NULL, buf );
*/
	if ( rcount == tag_game.rteam )
	{
		tag_channel (NULL, "The {BBLUE{x team has won FREEZE TAG!");
		end_tag ();
	}
	else if ( bcount == tag_game.bteam )
	{
		tag_channel (NULL, "The {RRED{x team has won FREEZE TAG!");
		end_tag ();
	}
	return;
}

void check_team_frozen (CHAR_DATA * ch)
{
  DESCRIPTOR_DATA *d;

  if (IS_NPC (ch) || !ch->pcdata)
    return;

  if (IS_SET (ch->pcdata->tag_flags, TAG_BLUE))
  {
    for (d = descriptor_list; d != NULL; d = d->next)
    {
      if (d->connected == CON_PLAYING
      && d->character && !IS_NPC(d->character)
	  && IS_SET (ch->pcdata->tag_flags, TAG_PLAYING)
	  && IS_SET (ch->pcdata->tag_flags, TAG_BLUE)
	  && !IS_SET (ch->pcdata->tag_flags, TAG_FROZEN))
      {
		return;
      }
    }

	if ( tag_game.timer <= 0 )
	{
    tag_channel (NULL, "The {RRED{x team has won FREEZE TAG!!!");
    end_tag ();
	}
    return;
  }

  if (IS_SET (ch->pcdata->tag_flags, TAG_RED))
  {
    for (d = descriptor_list; d != NULL; d = d->next)
    {
      if (d->connected == CON_PLAYING
      && d->character && !IS_NPC(d->character)
	  && IS_SET (d->character->pcdata->tag_flags, TAG_PLAYING)
	  && IS_SET (d->character->pcdata->tag_flags, TAG_RED)
	  && !IS_SET (d->character->pcdata->tag_flags, TAG_FROZEN))
      {
		return;
      }
    }
	if ( tag_game.timer <= 0 )
	{
    tag_channel (NULL, "The {BBLUE{x team has won FREEZE TAG!!!");
    end_tag ();
	}
    return;
  }
}

void do_ftag(CHAR_DATA *ch, char *argument)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];
  char buf[MSL];
  DESCRIPTOR_DATA *d;
  CHAR_DATA *victim;

  if (IS_NPC (ch))
    return;

  argument = one_argument (argument, arg1);

  if (arg1[0] == '\0')
  {
    send_to_char ("Syntax:  ftag join\n\r", ch);
    send_to_char ("         ftag info\n\r", ch);
    send_to_char ("         ftag start\n\r", ch);
    send_to_char ("         ftag fstart\n\r", ch);
    send_to_char ("To communicate use the 'blue' and 'red' channels.\n\r", ch);
    send_to_char("To tag someone once the game has started use the 'tag' comand.\n\r", ch);
    if (IS_IMMORTAL (ch))
    {
      send_to_char ("\n\rSyntax:  ftag reset\n\r", ch);
      send_to_char ("         ftag next\n\r", ch);
      send_to_char ("         ftag red <player>\n\r", ch);
      send_to_char ("         ftag blue <player>\n\r", ch);
    }
    return;
  }

  if (!str_cmp (arg1, "join"))
  {
    ROOM_INDEX_DATA *loc;

    if (tag_game.status != TAG_ISWAIT)
    {
      send_to_char ("There is no tag game to join.\n\r", ch);
      return;
    }
	if ( is_fighting(ch) )
	{
		stc("Finish the fight first, or flee if you're not fighting.\n\r", ch);
		return;
	}

/* Add some checks here
    if (ON_GQUEST (ch) || IS_QUESTOR (ch) || IS_SET (ch->act, PLR_WAR))
    {
      send_to_char ("You can't join right now.\n\r", ch);
      return;
    }
*/
	if ( IS_SET(ch->in_room->room_flags, ROOM_NOGATE )
	|| IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL) )
	{
		stc("Not from this room.\n\r",ch);
		return;
	}
    if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
    {
      send_to_char ("You're already playing.\n\r", ch);
      return;
    }

    if ((loc = get_room_index (FTAG_WAIT_ROOM)) == NULL)
    {
      send_to_char ("The freeze tag arena hasn't been finished yet.\n\r", ch);
      return;
    }
    send_to_char ("You join freeze tag.\n\r", ch);
    char_from_room (ch);
    char_to_room (ch, loc);
    tag_game.playing += 1;
    do_look (ch, "auto");
    if ((fRed = !fRed))
    {
      SET_BIT (ch->pcdata->tag_flags, TAG_WAITING);
      REMOVE_BIT (ch->pcdata->tag_flags, TAG_FROZEN);
      SET_BIT (ch->pcdata->tag_flags, TAG_RED);
      REMOVE_BIT (ch->pcdata->tag_flags, TAG_BLUE);
      send_to_char ("You are on the {RRED{x team!\n\r", ch);
      tag_channel (ch, "$n is on the {RRED{x team!");
      tag_game.rteam += 1;
    }
    else
    {
      SET_BIT (ch->pcdata->tag_flags, TAG_WAITING);
      REMOVE_BIT (ch->pcdata->tag_flags, TAG_FROZEN);
      SET_BIT (ch->pcdata->tag_flags, TAG_BLUE);
      REMOVE_BIT (ch->pcdata->tag_flags, TAG_RED);
      send_to_char ("You are on the {BBLUE{x team!\n\r", ch);
      tag_channel (ch, "$n is on the {BBLUE{x team!");
      tag_game.bteam += 1;
    }
    SET_BIT (ch->pcdata->tag_flags, TAG_WAITING);
    return;
  }

  if (!str_cmp (arg1, "info"))
  {
      if(tag_game.status == TAG_OFF)
      {
		sprintf (buf, "The next game of freeze tag will start in %d minute%s.\n\r",
			tag_game.next, tag_game.next == 1 ? "" : "s");
		send_to_char(buf, ch);
      }
      else
      {
	    for (d = descriptor_list; d != NULL; d = d->next)
		{
			if (d->connected == CON_PLAYING
			&& d->character && !IS_NPC(d->character)
			&& IS_SET (d->character->pcdata->tag_flags, TAG_PLAYING) )
			{
				ptc( ch, "%s %s %s\n\r",
				d->character->name,
				IS_SET (d->character->pcdata->tag_flags, TAG_RED) ? "{RRED{x" : "{BBLUE{x",
				IS_SET (d->character->pcdata->tag_flags, TAG_FROZEN) ? "Tagged" : ""
				);
			}
		}
		ptc( ch, "Reds: %d Blues: %d\n\r", tag_game.rteam, tag_game.bteam );

        ptc(ch,"A tag game is currently playing. %d minutes left.\n\r", tag_game.timer);
	  }
    return;
  }

  if (!str_cmp (arg1, "fstart"))
  {
    if (tag_game.status != TAG_OFF)
    {
      send_to_char ("A game has already started.\n\r", ch);
      return;
    }

    /* add cost here */
    if (!IS_IMMORTAL (ch) /*&& ch->pcdata->trivia < 2*/)
    {
//      send_to_char ("It costs 2 trivia points to start a game.\n\r", ch);
        send_to_char("Only Immortals can start a tag game.\n\r", ch);
      return;
    }

    tag_channel (NULL, "A Freeze Tag Game has started! Type 'ftag join' to play.");
    tag_game.status = TAG_ISWAIT;
    tag_game.timer = 1;
    tag_game.playing = 0;
    tag_game.next = -1;
    tag_game.rteam = 0;
    tag_game.bteam = 0;
/*    if (!IS_IMMORTAL (ch))
      ch->pcdata->trivia -= 2; */
    return;
  }

  if (!str_cmp (arg1, "start"))
  {
    if (tag_game.status != TAG_OFF)
    {
      send_to_char ("A game has already started.\n\r", ch);
      return;
    }

    /* add cost here */
    if (!IS_IMMORTAL (ch) /*&& ch->pcdata->trivia < 2*/)
    {
//      send_to_char ("It costs 2 trivia points to start a game.\n\r", ch);
        send_to_char("Only Immortals can start a tag game.\n\r", ch);
      return;
    }

    tag_channel (NULL, "A Freeze Tag Game has started! Type 'ftag join' to play.");
    tag_game.status = TAG_ISWAIT;
    tag_game.timer = 3;
    tag_game.playing = 0;
    tag_game.next = -1;
    tag_game.rteam = 0;
    tag_game.bteam = 0;
/*    if (!IS_IMMORTAL (ch))
      ch->pcdata->trivia -= 2; */
    return;
  }

  if (!IS_IMMORTAL (ch))
  {
    do_ftag (ch, "");
    return;
  }

  if (!str_cmp (arg1, "next"))
  {
    tag_game.next = atoi (argument);
    ptc (ch, "Next freeze tag game will start in %d ticks.\n\r",
	      tag_game.next);
    return;
  }
  if (!str_cmp (arg1, "reset"))
  {
    end_tag ();
    send_to_char ("All players reset.\n\r", ch);
    return;
  }

  argument = one_argument (argument, arg2);
  if (arg2[0] == '\0' || (str_cmp (arg1, "red") && str_cmp (arg1, "blue")))
  {
    send_to_char ("Syntax:  ftag red <player>\n\r", ch);
    send_to_char ("         ftag blue <player>\n\r", ch);
    return;
  }

  if (tag_game.status == TAG_ISPLAY)
  {
    send_to_char ("The tag game has already started.\n\r", ch);
    return;
  }
  if ((victim = get_char_world (ch, arg2)) == NULL)
  {
    send_to_char ("They aren't here.\n\r", ch);
    return;
  }

  if (IS_NPC (victim))
  {
    send_to_char ("They can't play.\n\r", ch);
    return;
  }

  if (!str_cmp (arg1, "red"))
  {
    SET_BIT (victim->pcdata->tag_flags, TAG_WAITING);
    REMOVE_BIT (victim->pcdata->tag_flags, TAG_FROZEN);
    SET_BIT (victim->pcdata->tag_flags, TAG_RED);
    REMOVE_BIT (victim->pcdata->tag_flags, TAG_BLUE);
    act ("You are on the {RRED{x team!", ch, NULL, victim, TO_VICT);
    act ("$N is on the {RRED{x team!", ch, NULL, victim, TO_NOTVICT);
    act ("$N is on the {RRED{x team!", ch, NULL, victim, TO_CHAR);
  }
  else if (!str_cmp (arg1, "blue"))
  {
    SET_BIT (victim->pcdata->tag_flags, TAG_WAITING);
    REMOVE_BIT (victim->pcdata->tag_flags, TAG_FROZEN);
    SET_BIT (victim->pcdata->tag_flags, TAG_BLUE);
    REMOVE_BIT (victim->pcdata->tag_flags, TAG_RED);
    act ("You are on the {BBLUE{x team!", ch, NULL, victim, TO_VICT);
    act ("$N is on the {BBLUE{x team!", ch, NULL, victim, TO_NOTVICT);
    act ("$N is on the {BBLUE{x team!", ch, NULL, victim, TO_CHAR);
  }

  return;
}

void do_tag(CHAR_DATA *ch, char *argument)
{
  char arg[MAX_INPUT_LENGTH];
  char buf[MSL];
  CHAR_DATA *victim;

  buf[0] = '\0';

  if (IS_NPC (ch))
    return;

  if (tag_game.status == TAG_OFF)
  {
    send_to_char ("There is no tag game playing.\n\r", ch);
    return;
  }

  argument = one_argument (argument, arg);

  if (!IS_SET (ch->pcdata->tag_flags, TAG_PLAYING))
  {
    send_to_char ("You're not playing freeze tag.\n\r", ch);
    return;
  }

  if (arg[0] == '\0')
  {
    send_to_char ("Tag whom?\n\r", ch);
    return;
  }

  if ((victim = get_char_room (ch, NULL, arg)) == NULL)
  {
    send_to_char ("They aren't here.\n\r", ch);
    return;
  }

  if (victim == ch)
  {
    send_to_char ("You tag yourself.  How amusing.\n\r", ch);
    return;
  }

  if (IS_NPC (victim))
  {
    send_to_char ("You can't tag them.\n\r", ch);
    return;
  }

  if (!IS_SET (victim->pcdata->tag_flags, TAG_PLAYING))
  {
    send_to_char ("They're not playing freeze tag.\n\r", ch);
    return;
  }

  if (IS_SET (ch->pcdata->tag_flags, TAG_FROZEN))
  {
    send_to_char ("You can't tag, you're frozen!\n\r", ch);
    return;
  }

  act ("$n tags you.", ch, NULL, victim, TO_VICT);
  act ("$n tags $N.", ch, NULL, victim, TO_NOTVICT);
  act ("You tag $N.", ch, NULL, victim, TO_CHAR);

  if ((IS_SET (ch->pcdata->tag_flags, TAG_RED) &&
       IS_SET (victim->pcdata->tag_flags, TAG_RED))
      || (IS_SET (ch->pcdata->tag_flags, TAG_BLUE) &&
	  IS_SET (victim->pcdata->tag_flags, TAG_BLUE)))
  {
    if (IS_SET (victim->pcdata->tag_flags, TAG_FROZEN))
    {
      REMOVE_BIT (victim->pcdata->tag_flags, TAG_FROZEN);
      act ("You are no longer frozen!", ch, NULL, victim, TO_VICT);
      act ("$N is no longer frozen!", ch, NULL, victim, TO_NOTVICT);
      act ("$N is no longer frozen!", ch, NULL, victim, TO_CHAR);
      sprintf( buf, "%s is no longer frozen!", victim->name );
    tag_channel (NULL, buf);
    }
    else
    {
      act ("$N is not frozen!", ch, NULL, victim, TO_CHAR);
    }
  }
  else
  {
    if (IS_SET (victim->pcdata->tag_flags, TAG_FROZEN))
    {
      act ("$N is already frozen!", ch, NULL, victim, TO_CHAR);
    }
    else
    {
      SET_BIT (victim->pcdata->tag_flags, TAG_FROZEN);
      act ("You are frozen!", ch, NULL, victim, TO_VICT);
      act ("$N is frozen!", ch, NULL, victim, TO_NOTVICT);
      act ("$N is frozen!", ch, NULL, victim, TO_CHAR);
      sprintf( buf, "%s is frozen!", victim->name );
    tag_channel (NULL, buf);
      /*check_team_frozen (victim);*/
    }
  }
check_tag_victory();
  return;
}

void auto_tag (void)
{
  DESCRIPTOR_DATA *d;
  int count = 0;

  for (d = descriptor_list; d != NULL; d = d->next)
    if (d->connected == CON_PLAYING)
      count++;

  if (count <= 2)
  {
    end_tag ();
    return;
  }

  tag_channel (NULL, "A Freeze Tag Game has started! Type 'ftag join' to play.");
  tag_game.status = TAG_ISWAIT;
  tag_game.timer = 3;
  tag_game.playing = 0;
  tag_game.next = -1;
}

void tag_update ( void )
{
    char buf[MSL];

    if ( tag_game.next > 0 && tag_game.status == TAG_OFF )
    {
        if ( --tag_game.next == 0 )
            auto_tag (  );
    }

    else if ( tag_game.status == TAG_ISWAIT )
    {

        tag_game.timer--;

        if ( tag_game.timer > 0 )
        {
            sprintf ( buf, "%d minute%s left to join freeze tag.", tag_game.timer,
                       tag_game.timer == 1 ? "" : "s" );
            tag_channel(NULL, buf);
        }
        else
        {
            if ( tag_game.playing < 3 )
            {
                end_tag (  );
                sprintf ( buf, "Not enough people for freeze tag, next game in %d minutes.",
                           tag_game.next );
                tag_channel(NULL, buf);
                return;
            }
            else
            {
                start_tag (  );
            }
        }
    }
    else if ( tag_game.status == TAG_ISPLAY )
    {
        if ( tag_game.playing == 0 )
        {
            end_tag (  );
            sprintf ( buf, "No one left in freeze tag, next game in %d minutes.",
                       tag_game.next );
            tag_channel(NULL, buf);
            return;
        }
        /*check_tag_victory();*/
        switch ( tag_game.timer )
        {
            case 0:
                end_tag (  );
                sprintf ( buf, "Time has run out for freeze tag, next game will start in %d minutes.",
                           tag_game.next );
                tag_channel(NULL, buf);
                return;
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 10:
            case 15:
                sprintf ( buf, "%d minute%s remaining in freeze tag.",
                           tag_game.timer, tag_game.timer > 1 ? "s" : "" );
                tag_channel(NULL, buf);
            default:
                tag_game.timer--;
                break;
        }
    }
}


bool is_tagging (CHAR_DATA * ch)
{
  if (!ch || IS_NPC (ch))
    return FALSE;

  if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING)
      && tag_game.status != TAG_OFF)
    return TRUE;

  return FALSE;
}

void tag_channel(CHAR_DATA *ch, char *message)
{
    DESCRIPTOR_DATA *d;
    CHAR_DATA *dch;
    char buf[MAX_INPUT_LENGTH];

    for(d = descriptor_list; d != NULL; d = d->next)
    {
        if(d->connected != CON_PLAYING)
            continue;

        if((dch = d->character) == NULL || dch == ch)
            continue;

        if ( !IS_SET( dch->auto_flags, AUTO_FTAG ) )
        	continue;

        sprintf(buf, "FREEZE TAG: %s\n\r", message);

        if(ch) /* don't use $N only $n in message */
            act(buf, ch, NULL, dch, TO_VICT);
        else
            send_to_char(buf, dch);
    }
}

char *pathfind2 (CHAR_DATA *ch, ROOM_INDEX_DATA *from, ROOM_INDEX_DATA *to)
{
	const char *exit_name [] = { "n", "e", "s", "w", "u", "d", "j", "l", "k", "h" };

	ROOM_INDEX_DATA *current = from;
	int dir;
	static char path[MSL];
	int cnt = 0;
	int max = 5;

	path[0] = '\0';
	ch->jog_full = TRUE;

	if (MOUNTED(ch))
	{
		max += 15;
	}
	else
	{
		max += UMIN(100, get_curr_stat(ch, STAT_CON)) / 10;	// up to 5	(16)

		if (ch->race == race_lookup("aiel"))
			max += 4;					// non-aiel: 16, aiel: 20
	}


	max = 10;

	while (current != to)
	{
		cnt++;
		dir = find_path(current->vnum, to->vnum, NULL, -40000, FALSE );
		if (dir < 0 || dir > MAX_DIR)	return NULL;

		strcat(path, exit_name[dir]);

		if (cnt > max)
		{
			ch->jog_full = FALSE;
			break;
		}

		current = current->exit[dir]->u1.to_room;
	}

	path[cnt] = '\0';
	return path;
}

void do_jog( CHAR_DATA *ch, char *argument )
{
	char buf[MAX_INPUT_LENGTH],arg[MAX_INPUT_LENGTH];
	char *p;
	bool dFound = FALSE;

	if (!ch->desc || *argument == '\0'|| !str_cmp(argument,"help"))
	{
	  send_to_char("You run in place!\n\r",ch);
	  stc("jogging help:\n\r",ch);
	  stc("n north\ts south\te east\tw west\n\r",ch);
	  stc("j ne\tl se\tk sw\th nw\n\r",ch);
	  stc("N ne\tE se\tS sw\tW nw\n\r",ch);
	  stc("o open\tc climb\tP pick\tL lock\tU unlock C close\n\r",ch);
	  stc("example: jog 3eonnCs8N\n\r",ch);
	  stc("will go 3 east, then open north, go north, close the door behind you\n\r",ch);
	  stc("then go 8 northeast.\n\r",ch);
	  stc("\n\rYou can also use 'jog to #' to jog to memorized rooms via the 'remember' command.\n\r", ch);

	  return;
	}

	buf[0] = '\0';

	if (!str_prefix("to ", argument))
	{
		int num, slots = 5;
		ROOM_INDEX_DATA *pRoomIndex;
		char *path;

		if ( IS_SET(ch->op_flags,OP_CHANNELER) || IS_SET(ch->spec_flags,SPEC_DREAMWALKER) )
			slots += 5;

		argument = one_argument(argument, arg);	// to get rid of the 'to'
		argument = one_argument(argument, arg);	// now we should have our remember #

		if (!is_number(arg))
		{
			stc("You must specify a memorized location.\n\r",ch);
			return;
		}

		num = atoi(arg);

		if( num < 0 || num > slots )
		{
			stc("You must specify a memorized location.\n\r", ch );
			return;
		}

		pRoomIndex = get_room_index( ch->pcdata->beacons[num] );

		if (pRoomIndex == NULL || (path = pathfind2(ch, ch->in_room, pRoomIndex)) == NULL)
		{
			stc("You can't seem to find a way to get there from here.\n\r", ch);
			return;
		}

		if (!str_cmp(path, "far"))
		{
			stc("That's way too far to run at once.\n\r", ch);
			return;
		}

		do_jog(ch, path);
		ch->jog_to = num;
		return;
	}

	while (*argument != '\0')
	{
	  argument = one_argument_with_caps(argument,arg);
	  strcat(buf,arg);
	}

	for( p = buf + strlen(buf)-1; p >= buf; p--)
	{
	  if (!isdigit(*p))
	  {
		switch( *p )
		{
		  case 'n': case 's': case 'e': case 'w': case 'u': case 'd':
		  case 'j': case 'k': case 'l': case 'h':		/*ZMud diagonals*/
		  case 'N': case 'E': case 'S': case 'W':		/*RoA diagonals*/
				dFound = TRUE;
					break;

		  case 'o': case 'c': case 'P' : case 'L' : case 'U' : case 'C' : break;

		  default: stc("Invalid direction!\n\r",ch);
				   return;
		}
	  }
	  else if (!dFound) *p = '\0';
	}

	if (!dFound)
	{
	  send_to_char("No directions specified!\n\r",ch);
	  return;
	}

	ch->run_buf = str_dup( buf );
	ch->run_head = ch->run_buf;
	if (ch->jog_to == -1)
	stc("You start running...\n\r",ch);
	ch->jog_to = -1;
	return;
}

