/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "tables.h"
#include "mysql5_interface.h"
#include "recycle.h"

/*
 * Local functions.
 */
#define CD CHAR_DATA
#define OD OBJ_DATA
bool	remove_obj	args( (CHAR_DATA *ch, int iWear, bool fReplace ) );
void	wear_obj	args( (CHAR_DATA *ch, OBJ_DATA *obj, bool fReplace ) );
CD *	find_keeper	args( (CHAR_DATA *ch ) );
int	get_cost	args( (CHAR_DATA *keeper, OBJ_DATA *obj, bool fBuy ) );
void 	obj_to_keeper	args( (OBJ_DATA *obj, CHAR_DATA *ch ) );
OD *	get_obj_keeper	args( (CHAR_DATA *ch,CHAR_DATA *keeper,char *argument));
int	clan_lookup	args( (const char *name) );
bool    is_obj_char_clan args( ( CHAR_DATA *ch, OBJ_DATA *obj) );
void	delete_home_item	args((OBJ_DATA *obj));
void	save_home_item		args((OBJ_DATA *obj));
#undef OD
#undef	CD

char *check_dye args((OBJ_DATA *obj, char *string, int dye_vnum));

void do_feed( CHAR_DATA * ch, char *argument )
{
	char arg1[MAX_INPUT_LENGTH];
	char arg2[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	OBJ_DATA *obj;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	if (arg1[0] == '\0' || arg2[0] == '\0')
	{
		send_to_char ("Feed what to who?\n\r",ch);
		return;
	}
	if ((obj = get_obj_carry (ch, arg1, ch)) == NULL)
	{
		send_to_char ("You do not have that item.\n\r", ch);
		return;
	}
	if ((victim = get_char_room (ch, NULL, arg2)) == NULL)
	{
		send_to_char ("They aren't here.\n\r", ch);
		return;
	}
	if (!IS_NPC(victim))
	{
		send_to_char ("Um, no force feedings here.\n\r", ch);
		return;
	}
	if (victim->master != ch)
	{
		send_to_char ("Feeding strange mobiles, bad bad bad!\n\r", ch);
		return;
	}
	if ((obj->item_type != ITEM_FOOD)
	&& (obj->item_type != ITEM_PILL)
	&& (obj->item_type != ITEM_POTION))
	{
		send_to_char ("You can't feed them that!\n\r", ch);
		return;
	}
	if (!can_drop_obj (ch, obj))
	{
		send_to_char ("You can't let go of it.\n\r", ch);
		return;
	}
	if (!can_see_obj (victim, obj))
	{
		act ("$N can't see it.", ch, NULL, victim, TO_CHAR);
		return;
	}
	obj_from_char (obj);
	obj_to_char (obj, victim);
	act ("$n feeds $p to $N.", ch, obj, victim, TO_NOTVICT);
	act ("$n feeds you $p.", ch, obj, victim, TO_VICT);
	act ("You feed $p to $N.", ch, obj, victim, TO_CHAR);
	if ((obj->item_type == ITEM_FOOD)
	|| (obj->item_type == ITEM_PILL))
	{
		do_function (victim, &do_eat, arg1);
		return;
	}
	else
	{
		do_function (victim, &do_quaff, arg1);
		return;
	}
	return;
}

void keeper_say( CHAR_DATA *ch, char *argument )
{
    char arg[MIL];
    CHAR_DATA *victim;

    argument = one_argument( argument, arg );

    if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
    {
		stc( "They aren't here.{c\n\r", ch );
		return;
    }

    act("{gYou say to $N {G'{g$t{G'{x", ch, argument, victim, TO_CHAR );
    act("{g$n says to you {G'{g$t{G'{x", ch, argument, victim, TO_VICT );
    act("{g$n says to $N, {G'{g$t{G'{x", ch, argument, victim, TO_NOTVICT);
    return;
}

OBJ_DATA *get_donation_pit(void)
{
	OBJ_DATA *pit;

	for (pit = object_list; pit != NULL; pit = pit->next)
		if (pit->pIndexData->vnum == OBJ_VNUM_PIT)
			return pit;

	return NULL;
}

void donate_object(OBJ_DATA * obj)
{
	OBJ_DATA *container;

	if (obj == NULL)
	{
		bug("donate_object: NULL obj", 0);
		return;
	}

	if (obj->carried_by)
		obj_from_char(obj);
	else if (obj->in_room)
		obj_from_room(obj);
	else if (obj->in_obj)
		obj_from_obj(obj);

	obj->cost = 0;

	if (!(container = get_donation_pit()))
	{
		bug("donate_object: no donation pit!", 0);
		extract_obj(obj);
	}
	else
	{
		int count = 0;
		OBJ_DATA *obj;

		for (obj = container->contains; obj; obj = obj->next_content)
			count++;

		if (count < 100)
			obj_to_obj(obj, container);
	}
}

void do_donate( CHAR_DATA *ch, char *argument )
{
	char arg[MIL];
	OBJ_DATA *obj;

	argument = one_argument(argument, arg);

	if (IS_NULLSTR(arg))
	{
		stc( "Donate what?", ch );
		return;
	}
	if (!str_cmp("all", arg) || !str_prefix("all.", arg))
	{
		OBJ_DATA *obj_next;
		bool found = FALSE;
		int count = 0;

		for (obj = ch->carrying; obj; obj = obj_next)
		{
			obj_next = obj->next_content;

			if (arg[3] != '\0' && !is_name(&arg[4], obj->name))
				continue;

			if (obj->item_type == ITEM_CORPSE_PC
				|| obj->item_type == ITEM_CORPSE_NPC
				|| IS_OBJ_STAT(obj, ITEM_ROT_DEATH)
        || obj->item_type == ITEM_TREASURECHEST
        || IS_SET( obj->extra_flags,ITEM_NOSELL )
        || !can_drop_obj(ch, obj)
        || IS_OBJ_STAT( obj, ITEM_NODROP )
				|| obj->wear_loc != WEAR_NONE)
				continue;

			if (count++ > 90)
			{
				stc( "Too many objects, aborting.", ch );
				return;
			}

			found = TRUE;
			act("$n donates $p to the needy.", ch, obj, NULL, TO_ROOM);
			act("You donate $p to the needy.", ch, obj, NULL, TO_CHAR);

			donate_object(obj);
		}
		if (!found)
			stc( "You have nothing to donate.", ch );
		return;
	}

	if ((obj = get_obj_carry(ch, arg, ch)) == NULL)
	{
		stc( "You do not have that item.", ch );
		return;
	}

	if (!can_drop_obj(ch, obj))
	{
		stc( "You can't let go of it.", ch );
		return;
	}
	if (obj->item_type == ITEM_CORPSE_PC || obj->item_type == ITEM_CORPSE_NPC)
	{
		stc( "That would stink up the place.", ch );
		return;
	}
	if (IS_OBJ_STAT(obj, ITEM_ROT_DEATH) || obj->timer > 1 )
	{
		stc( "Only permanent items may be donated.", ch );
		return;
	}
  if ( IS_SET( obj->extra_flags,ITEM_NOSELL ) )
  {
		stc("That item has been flagged nosell. Type unkeep <item> to toggle this off.\n\r",ch);
		return;
	}
  if ( IS_OBJ_STAT( obj, ITEM_NODROP ) )
  {
		stc("You can't let go of that item.\n\r",ch);
    return;
  }
  if ( obj->item_type == ITEM_TREASURECHEST )
  {
    stc("Try picking it and getting the items from it, then donate those.\n\r",ch);
    return;
  }

	act("$n donates $p to the needy.", ch, obj, NULL, TO_ROOM);
	act("You donate $p to the needy.", ch, obj, NULL, TO_CHAR);
	donate_object(obj);

	return;
}

void save_donation_pit(void)
{
	OBJ_DATA *pit;
	FILE *fp;
	char *TEMPFILE = PIT_FILE ".tmp";

	for (pit = object_list; pit != NULL; pit = pit->next)
	{
		if (pit->pIndexData->vnum == OBJ_VNUM_PIT)
			break;
	}

	if (!pit)
		return;

	if ((fp = fopen(TEMPFILE, "w")) == NULL)
		return;

	if (pit->contains)
		fwrite_obj(NULL, pit->contains, fp, 0);

	fprintf(fp, "#END\n");
	fflush(fp);
	fclose(fp);
	rename(TEMPFILE, PIT_FILE);
}

void load_donation_pit(void)
{
	FILE *fp;
	int count;

	if ((fp = fopen(PIT_FILE, "r")) == NULL)
	{
		bug("load_donation_pit: " PIT_FILE " not found", 0);
		return;
	}

		count = 0;

		for (;;)
		{
			char letter;
			char *word;

			letter = fread_letter(fp);
			if (letter == '*')
			{
				fread_to_eol(fp);
				continue;
			}

			if (letter != '#')
			{
				bug("load_donation_pit: # not found.", 0);
				break;
			}

			word = fread_word(fp);
			if ((!str_cmp(word, "OBJECT")))
			{
				fread_obj(NULL, fp);
				count++;
				if (count >= 100)
					break;
			}
			else if (!str_cmp(word, "END"))
				break;
			else
			{
				bug("load_donation_pit: bad section.", 0);
				break;
			}
		}

	fclose(fp);
	return;
}

void do_unhook( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *key, *keyring;
	char arg1[MIL];
	char arg2[MIL];
	char arg3[MIL];
	int slot = 0;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( arg1[0] == '\0' )
	{
		stc( "Syntax: Unhook <key slot> from <keyring>.\n\r", ch );
		stc( "Arg1 should be the number of the keyring slot.\n\r", ch );
		return;
	}
	if ( !is_number( arg1 ) )
	{
		stc( "Syntax: Unhook <key slot> from <keyring>.\n\r", ch );
		stc( "Arg1 doesn't appear to be a valid keyring slot.\n\r", ch );
		return;
	}

	slot = atoi( arg1 );

	if ( slot < 0 || slot > 4 )
	{
		stc( "The slot should be a number from 0 to 4.\n\r", ch );
		stc( "Arg1 doesn't appear to be a valid slot number.\n\r", ch );
		return;
	}
	if ( strcmp(arg2, "from" ) )
	{
		stc( "Syntax: Unhook <key slot> from <keyring>.\n\r", ch );
		stc( "Arg2 should be 'from'.\n\r", ch );
		return;
	}
	if ( ( keyring = get_obj_carry( ch, arg3, ch ) ) == NULL )
	{
		stc( "Syntax: Unhook <key slot> from <keyring>.\n\r", ch );
		stc( "You don't appear to be carrying that keyring.\n\r", ch );
		return;
	}
	if ( keyring->item_type != ITEM_KEYRING  )
	{
		stc( "That doesn't appear to be a keyring.\n\r", ch );
		return;
	}
	if ( keyring->value[slot] > 0 )
	{
		key = create_object( get_obj_index( keyring->value[slot] ), 1 );
		obj_to_char( key, ch );
		keyring->value[slot] = 0;
		ptc( ch, "You remove %s from %s.\n\r",
			key->short_descr, keyring->short_descr );
	}

	return;
}

void do_hook( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *key, *keyring, *newObj;
	char arg1[MIL];
	char arg2[MIL];
	char arg3[MIL];
	char arg4[MIL];
	int slot;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	argument = one_argument( argument, arg4 );

	if ( arg1[0] == '\0' )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "Arg1 should be the name of the key.\n\r", ch );
		return;
	}
	if ( strcmp(arg2, "on" ) )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "Arg2 should be 'on'.\n\r", ch );
		return;
	}
	if ( arg3[0] == '\0' )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "Arg3 should be the name of the keyring.\n\r", ch );
		return;
	}
	if ( arg4[0] == '\0' )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "Arg4 should be the number of the keyring slot.\n\r", ch );
		return;
	}
	if ( ( key = get_obj_carry( ch, arg1, ch ) ) == NULL )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "You don't appear to be carrying that key.\n\r", ch );
		return;
	}
	if ( key->item_type != ITEM_KEY  )
	{
		if ( key->item_type != ITEM_ROOM_KEY )
		{
			stc( "That doesn't appear to be a key.\n\r", ch );
			return;
		}
	}
	if ( ( keyring = get_obj_carry( ch, arg3, ch ) ) == NULL )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "You don't appear to be carrying that keyring.\n\r", ch );
		return;
	}
	if ( keyring->item_type != ITEM_KEYRING )
	{
		stc( "That doesn't appear to be a keyring.\n\r", ch );
		return;
	}
	if ( !is_number( arg4 ) )
	{
		stc( "Syntax: Hook <key> on <keyring> <slot>.\n\r", ch );
		stc( "Arg4 doesn't appear to be a valid number.\n\r", ch );
		return;
	}

	slot = atoi( arg4 );

	if ( slot < 0 || slot > 4 )
	{
		stc( "The slot should be a number from 0 to 4.\n\r", ch );
		stc( "Arg4 doesn't appear to be a valid slot number.\n\r", ch );
		return;
	}
	if ( keyring->value[slot] == 0 )
	{
		keyring->value[slot] = key->pIndexData->vnum;

		ptc( ch, "You hook %s onto %s.\n\r",
			key->short_descr, keyring->short_descr );
		obj_from_char( key );
		extract_obj( key );
	}
	else if ( keyring->value[slot] > 0 )
	{
		newObj = create_object( get_obj_index( keyring->value[slot] ), 1 );
		obj_to_char( newObj, ch );
		keyring->value[slot] = key->pIndexData->vnum;
		ptc( ch, "You remove %s from %s and replace it with %s.\n\r",
			newObj->short_descr, check_dye(keyring, "", keyring->dye_vnum), key->short_descr );
		obj_from_char( key );
		extract_obj( key );
	}
	return;
}

bool is_vowel( char *txt )
{
	int i;
	bool found = FALSE;
	char * vowels[5] = { "a, e, i, o, u" };
	if ( txt == NULL )
		return FALSE;
	if ( is_number( txt ) )
		return FALSE;
	if ( txt[0] == '\0' )
		return FALSE;
	for ( i = 0; i < 5; i++ )
	{
		if ( !str_cmp( txt, vowels[i] ) )
		found = TRUE;
		break;
	}
	if ( found )
		return TRUE;
	else
		return FALSE;
}


bool can_loot(CHAR_DATA *ch, OBJ_DATA *obj)
{
    CHAR_DATA *owner, *wch;

    if (IS_IMMORTAL(ch))
	return TRUE;

    if (!obj->owner || obj->owner == NULL)
	return TRUE;

    owner = NULL;
    for ( wch = char_list; wch != NULL ; wch = wch->next )
        if (!str_cmp(wch->name,obj->owner))
            owner = wch;

    if (owner == NULL)
	return TRUE;

    if (!str_cmp(ch->name,owner->name))
	return TRUE;

    if (!IS_NPC(owner) && IS_SET(owner->act,PLR_CANLOOT))
	return TRUE;

    if (is_same_group(ch,owner))
	return TRUE;

    return FALSE;
}


bool isQobj args( ( OBJ_DATA *obj ) );

void get_obj( CHAR_DATA *ch, OBJ_DATA *obj, OBJ_DATA *container )
{
    /* variables for AUTOSPLIT */
    CHAR_DATA *gch;
    int members,guild;
    char buffer[100];
    char cbuf[MSL];

	guild = clan_lookup(obj->guild);

    if ( !CAN_WEAR(obj, ITEM_TAKE) )
    {
		stc( "{cYou can't take that.\n\r", ch );
		return;
    }

    if ( obj->item_type == ITEM_FIREOIL && obj->value[1] == 1 )
    {
    	stc("It's burning, and will explode!\n\r",ch);
	   	return;
    }

	if ( isQobj( obj ) )
	{
		if ( strcmp( obj->owner, ch->name ) )
		{
			stc( "You don't own that item.\n\r", ch );
			return;
		}
	}
	if ( obj->owner != NULL && obj->owner[0] != '\0' && strcmp( obj->owner, ch->name ) && !IS_IMMORTAL(ch))
	{
		ptc( ch, "That item belongs to %s. You can't get it.\n\r", obj->owner );
		return;
	}

    if (!is_obj_char_clan(ch,obj) && is_obj_clan(obj))
    {
		ptc(ch,"Only a %s can use that item.\n\r", obj->guild );
		return;
	}

    if (IS_SET(obj->extra_flags,ITEM_REMORT)
    && (IS_NPC(ch) || ch->pcdata->rebirth < 1))
    {
		act("You are not powerful enough to get that.",ch,0,0,TO_CHAR);
		return;
	}

    if ( ch->carry_number + get_obj_number( obj ) > can_carry_n( ch ) )
    {
		act( "{c$d: you can't carry that many items.",ch, NULL, obj->name, TO_CHAR );
		return;
    }

    if ( ( !obj->in_obj || obj->in_obj->carried_by != ch )
    &&  ( get_carry_weight( ch ) + get_obj_weight( obj ) > can_carry_w( ch ) ) )
    {
		act( "{c$d: you can't carry that much weight.",ch, NULL, obj->name, TO_CHAR );
		return;
    }

    if (!can_loot(ch,obj))
    {
		act( "{cCorpse looting is not permitted.",ch,NULL,NULL,TO_CHAR );
		return;
    }

    if (obj->in_room != NULL)
    {
		for (gch = obj->in_room->people; gch != NULL; gch = gch->next_in_room)
		{
		    if (gch->on == obj)
		    {
			act( "{c$N appears to be using $p.", ch,obj,gch,TO_CHAR);
			act( "{c$n starts to get $p, but realizes $N is using it.", ch,obj,gch,TO_ROOM);
			return;
			}
	    }
    }


    if ( container != NULL )
    {
    	if (container->pIndexData->vnum == OBJ_VNUM_PIT
		&&  get_trust(ch) < obj->level)
		{
		    stc( "You are not powerful enough to use it.\n\r",ch);
		    return;
		}

	   	if (container->pIndexData->vnum == OBJ_VNUM_PIT
		&&  !CAN_WEAR(container, ITEM_TAKE)
		&&  !IS_OBJ_STAT(obj,ITEM_HAD_TIMER))
		    obj->timer = 0;

		act( "You get $p from $P.", ch, obj, container, TO_CHAR );
		act( "$n gets $p from $P.", ch, obj, container, TO_ROOM );
		REMOVE_BIT(obj->extra_flags,ITEM_HAD_TIMER);


	        if (container->in_room != NULL && IS_SET(container->in_room->room_flags, ROOM_HOME))
        	        delete_home_item(obj);

		obj_from_obj( obj );
    }
    else
    {
		act( "{cYou get $p.", ch, obj, container, TO_CHAR );
		act( "{c$n gets $p.", ch, obj, container, TO_ROOM );

	        if (ch->in_room != NULL && IS_SET(ch->in_room->room_flags, ROOM_HOME))
        	        delete_home_item(obj);

		obj_from_room( obj );
    }

    if ( obj->item_type == ITEM_MONEY)
    {
	if ( ch->in_room->vnum == 3721 )
	{
		sprintf( cbuf, "CEMETARY:: %s picked up %s.", ch->name,obj->short_descr );
		log_string( cbuf );
	}
		ch->money[0][0] += obj->value[0];
		ch->money[1][0] += obj->value[1];
		ch->money[2][0] += obj->value[2];
		ch->money[3][0] += obj->value[3];
        if (IS_SET(ch->act,PLR_AUTOSPLIT))
        { /* AUTOSPLIT code */
    	  members = 0;
    	  for (gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    	  {
            if (!IS_AFFECTED(gch,AFF_CHARM) && is_same_group( gch, ch ) && !IS_NPC(gch) )
              members++;
    	  }

	  /*if ( members > 1 && (obj->value[0] > 1 || obj->value[1]))*/
	  if ( members > 1 )
	  {
	    sprintf(buffer,"%d %d %d %d",obj->value[0],obj->value[1],obj->value[2],obj->value[3]);
	    do_function(ch, &do_split, buffer);
	  }
        }

	extract_obj( obj );
    }
    else
    {
	if ( ch->in_room->vnum == 3721 )
	{
		sprintf( cbuf, "CEMETARY:: %s picked up %s.", ch->name,obj->short_descr );
		log_string( cbuf );
	}

	obj_to_char( obj, ch );

	if ( HAS_TRIGGER_OBJ( obj, TRIG_GET ) )
	{
	    p_give_trigger( NULL, obj, NULL, ch, obj, TRIG_GET );
	}
	if ( HAS_TRIGGER_ROOM( ch->in_room, TRIG_GET ) )
	{
	    p_give_trigger( NULL, NULL, ch->in_room, ch, obj, TRIG_GET );
	}
    }

    return;
}



void do_get( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    OBJ_DATA *container;
    bool found;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if (!str_cmp(arg2,"from"))
	argument = one_argument(argument,arg2);

    /* Get type. */
    if ( arg1[0] == '\0' )
    {
		stc( "{cGet what?\n\r", ch );
		return;
    }

    if ( arg2[0] == '\0' )
    {
		if ( str_cmp( arg1, "all" ) && str_prefix( "all.", arg1 ) )
		{
		    /* 'get obj' */
		    obj = get_obj_list( ch, arg1, ch->in_room->contents );
		    if ( obj == NULL )
		    {
				act( "{cI see no $T here.", ch, NULL, arg1, TO_CHAR );
				return;
		    }

		    get_obj( ch, obj, NULL );
		}
		else
		{
			int iCnt = 0;
		    /* 'get all' or 'get all.obj' */
		    found = FALSE;
		    for ( obj = ch->in_room->contents; obj != NULL; obj = obj_next )
		    {
				obj_next = obj->next_content;
				if ( ( arg1[3] == '\0' || is_name( &arg1[4], obj->name ) )
				&&   can_see_obj( ch, obj ) )
				{
					iCnt++;
					if ( iCnt > 20 )
					{
						stc( "You've reached the spam limit. Wait a moment to spam more.\n\r", ch );
						return;
					}
				    found = TRUE;
				    get_obj( ch, obj, NULL );
				}
		    }

		    if ( !found )
		    {
				if ( arg1[3] == '\0' )
				    stc( "{cI see nothing here.\n\r", ch );
				else
				    act( "{cI see no $T here.", ch, NULL, &arg1[4], TO_CHAR );
			    }
			}
	    }
	    else
	    {
		/* 'get ... container' */
		if ( !str_cmp( arg2, "all" ) || !str_prefix( "all.", arg2 ) )
		{
		    stc( "{cYou can't do that.\n\r", ch );
		    return;
		}

		if ( ( container = get_obj_here( ch, NULL, arg2 ) ) == NULL )
		{
		    act( "{cI see no $T here.", ch, NULL, arg2, TO_CHAR );
		    return;
		}

		switch ( container->item_type )
		{
			default:
		    stc( "{cThat's not a container.\n\r", ch );
		    return;

			case ITEM_TREASURECHEST:
			if ( IS_SET(container->value[1], CONT_CLOSED) )
			{
			    act( "{cThe $d is closed.", ch, NULL, container->name, TO_CHAR );
			    return;
			}

			if ( str_cmp( arg1, "all" ) && str_prefix( "all.", arg1 ) )
			{
		    /* 'get obj container' */
			    obj = get_obj_list( ch, arg1, container->contains );
			    if ( obj == NULL )
			    {
					act( "{cI see nothing like that in the $T.",ch, NULL, arg2, TO_CHAR );
					return;
			    }
			    get_obj( ch, obj, container );
			}
			else
			{
			    /* 'get all container' or 'get all.obj container' */
				int iCnt = 0;
			    found = FALSE;
			    for ( obj = container->contains; obj != NULL; obj = obj_next )
			    {
					obj_next = obj->next_content;
					if ( ( arg1[3] == '\0' || is_name( &arg1[4], obj->name ) )
					&&   can_see_obj( ch, obj ) )
					{
					    found = TRUE;
						iCnt++;
						if ( iCnt > 20 )
						{
							stc( "You've reached the spam limit. Wait a moment to spam more.\n\r", ch );
							return;
						}
					    if (container->pIndexData->vnum == OBJ_VNUM_PIT
					    &&  !IS_IMMORTAL(ch))
					    {
							stc( "{cDon't be so greedy!\n\r",ch);
							return;
					    }
					    get_obj( ch, obj, container );
					}
			    }

			    if ( !found )
			    {
					if ( arg1[3] == '\0' )
					    act( "{cI see nothing in the $T.",ch, NULL, arg2, TO_CHAR );
					else
					    act( "{cI see nothing like that in the $T.",ch, NULL, arg2, TO_CHAR );
			    }

			    return;
			}

			case ITEM_CONTAINER:
			case ITEM_CORPSE_NPC:
		    break;

			case ITEM_CORPSE_PC:
		    {
				if (!can_loot(ch,container))
				{
				    stc( "{cYou can't do that.\n\r", ch );
				    return;
				}
		    }
		}

		if ( IS_SET(container->value[1], CONT_CLOSED) )
		{
		    act( "{cThe $d is closed.", ch, NULL, container->name, TO_CHAR );
		    return;
		}

		if ( str_cmp( arg1, "all" ) && str_prefix( "all.", arg1 ) )
		{
		    /* 'get obj container' */
		    obj = get_obj_list( ch, arg1, container->contains );
		    if ( obj == NULL )
		    {
				act( "{cI see nothing like that in the $T.", ch, NULL, arg2, TO_CHAR );
				return;
		    }
		    get_obj( ch, obj, container );
		}
		else
		{
			int iCnt = 0;
		    /* 'get all container' or 'get all.obj container' */
		    found = FALSE;
		    for ( obj = container->contains; obj != NULL; obj = obj_next )
		    {
				obj_next = obj->next_content;
				if ( ( arg1[3] == '\0' || is_name( &arg1[4], obj->name ) )
				&&   can_see_obj( ch, obj ) )
				{
					iCnt++;
					if ( iCnt > 20 )
					{
						stc( "You've reached the spam limit. Wait a moment and try again.\n\r", ch );
						return;
					}
				    found = TRUE;
				    if (container->pIndexData->vnum == OBJ_VNUM_PIT
				    &&  !IS_IMMORTAL(ch))
				    {
						stc( "{cDon't be so greedy!\n\r",ch);
						return;
				    }
				    get_obj( ch, obj, container );
				}
		    }

		    if ( !found )
		    {
				if ( arg1[3] == '\0' )
				    act( "{cI see nothing in the $T.",ch, NULL, arg2, TO_CHAR );
				else
				    act( "{cI see nothing like that in the $T.",ch, NULL, arg2, TO_CHAR );
		    }
		}
    }

    return;
}



void do_put( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    OBJ_DATA *container;
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if (!str_cmp(arg2,"in") || !str_cmp(arg2,"on"))
	argument = one_argument(argument,arg2);

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
		stc( "{cPut what in what?\n\r", ch );
		return;
    }

    if ( !str_cmp( arg2, "all" ) || !str_prefix( "all.", arg2 ) )
    {
		stc( "{cYou can't do that.\n\r", ch );
		return;
    }

    if ( ( container = get_obj_here( ch, NULL, arg2 ) ) == NULL )
    {
		act( "{cI see no $T here.", ch, NULL, arg2, TO_CHAR );
		return;
    }

    if ( container->item_type != ITEM_CONTAINER )
    {
	    if ( container->item_type != ITEM_TREASURECHEST )
	    {
			stc( "{cThat's not an appropriate container.\n\r", ch );
			return;
		}
    }

    if ( IS_SET(container->value[1], CONT_CLOSED) )
    {
		act( "{cThe $d is closed.", ch, NULL, container->name, TO_CHAR );
		return;
    }

    if ( str_cmp( arg1, "all" ) && str_prefix( "all.", arg1 ) )
    {
		/* 'put obj container' */
		if ( ( obj = get_obj_carry( ch, arg1, ch ) ) == NULL )
		{
	    stc( "{cYou do not have that item.\n\r", ch );
	    return;
		}

		if ( obj == container )
		{
	    stc( "{cYou can't fold it into itself.\n\r", ch );
	    return;
		}

		if ( !can_drop_obj( ch, obj ) )
		{
	    stc( "{cYou can't let go of it.\n\r", ch );
	    return;
		}

	    if (WEIGHT_MULT(obj) != 100)
	    {
  		stc( "{cYou have a feeling that would be a bad idea.\n\r",ch);
    	return;
	    }

	    if ( container->item_type == ITEM_TREASURECHEST )
	    {
  		stc( "{cCannot place items in a treasure chest.\n\r",ch);
    	return;
	    }
	    if ( obj->item_type == ITEM_TREASURECHEST )
	    {
  		stc( "{cCannot put a container in a treasure chest.\n\r",ch);
    	return;
	    }
	    if ( obj->item_type == ITEM_TREASURECHEST && container->item_type == ITEM_TREASURECHEST )
	    {
  		stc( "{cCannot put a treasure chest in a treasure chest.\n\r",ch);
    	return;
	    }
        if ( container->item_type == ITEM_CONTAINER && obj->item_type == ITEM_CONTAINER )
        {
      		stc( "{cCannot put a container in another container.\n\r",ch);
        	return;
        }

	    if ( container->item_type != ITEM_CONTAINER )
	    {
			if (get_obj_weight( obj ) + get_true_weight( container ) > (container->value[0] * 10)
			||  get_obj_weight(obj) > (container->value[3] * 10))
			{
		    stc( "{cIt won't fit.\n\r", ch );
		    return;
			}
		}
	    if ( container->item_type != ITEM_CONTAINER )
	    {
		obj_to_obj( obj, container );
		}
		if (container->pIndexData->vnum == OBJ_VNUM_PIT
		&&  !CAN_WEAR(container,ITEM_TAKE))
		{
	    if (obj->timer)
		SET_BIT(obj->extra_flags,ITEM_HAD_TIMER);
	    else
	        obj->timer = number_range(100,200);
		}

		obj_from_char( obj );
		obj_to_obj( obj, container );

		if (IS_SET(container->value[1],CONT_PUT_ON))
		{
	    act( "{c$n puts $p on $P.",ch,obj,container, TO_ROOM);
	    act( "{cYou put $p on $P.",ch,obj,container, TO_CHAR);
		}
		else
		{
	    act( "{c$n puts $p in $P.", ch, obj, container, TO_ROOM );
	    act( "{cYou put $p in $P.", ch, obj, container, TO_CHAR );
		}

	        if (container->in_room != NULL && IS_SET(container->in_room->room_flags, ROOM_HOME))
        	        save_home_item(obj);


	}
    else
    {
		int iCnt = 0;
		/* 'put all container' or 'put all.obj container' */
		for ( obj = ch->carrying; obj != NULL; obj = obj_next )
		{
		    obj_next = obj->next_content;

		    if ( ( arg1[3] == '\0' || is_name( &arg1[4], obj->name ) )
		    &&   can_see_obj( ch, obj )
		    &&   WEIGHT_MULT(obj) == 100
		    &&   obj->wear_loc == WEAR_NONE
		    &&   obj != container
		    &&   can_drop_obj( ch, obj )
		    &&   get_obj_weight( obj ) + get_true_weight( container )
			 <= (container->value[0] * 10)
		    &&   get_obj_weight(obj) < (container->value[3] * 10))
		    {
				iCnt++;
				if ( iCnt > 20 )
				{
					stc( "You've reached the spam limit. Wait a moment and try again.\n\r", ch );
					return;
				}
                if ( container->item_type == ITEM_CONTAINER && obj->item_type == ITEM_CONTAINER )
                {
              		stc( "{cCannot put a container in another container.\n\r",ch);
                	continue;
                }
		    	if (container->pIndexData->vnum == OBJ_VNUM_PIT
		    	&&  !CAN_WEAR(obj, ITEM_TAKE) )
		    	{
		    	    if (obj->timer)
					SET_BIT(obj->extra_flags,ITEM_HAD_TIMER);
		    	    else
	    	    	obj->timer = number_range(100,200);
				}
				/*stc( "Second ofc\n\r", ch );*/
				obj_from_char( obj );
				obj_to_obj( obj, container );

	        	if (IS_SET(container->value[1],CONT_PUT_ON))
	        	{
           	    act( "{c$n puts $p on $P.",ch,obj,container, TO_ROOM);
           	    act( "{cYou put $p on $P.",ch,obj,container, TO_CHAR);
	        	}
				else
				{
			    act( "{c$n puts $p in $P.", ch, obj, container, TO_ROOM );
			    act( "{cYou put $p in $P.", ch, obj, container, TO_CHAR );
				}

			        if (container->in_room != NULL && IS_SET(container->in_room->room_flags, ROOM_HOME))
	        	        save_home_item(obj);

		    }
		}
    }

    return;
}



void do_drop( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    bool found;

    argument = one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cDrop what?\n\r", ch );
	return;
    }

    if ( is_number( arg ) )
    {
	/* 'drop NNNN coins' */
	int amount, copper = 0, silver = 0, mark = 0, gold = 0;

	amount   = atoi(arg);
	argument = one_argument( argument, arg );

	if ( amount <= 0
	|| ( str_cmp( arg, "coins" ) && str_cmp( arg, "coin" )
	&& str_cmp( arg, "marks" ) && str_cmp( arg, "gold" )
	&& str_cmp( arg, "copper"  ) && str_cmp( arg, "silver") ) )
	{
	    stc( "{cSorry, you can't do that.\n\r", ch );
	    return;
	}

        if ( !str_prefix( arg, "coin" ) || !str_prefix( arg, "coins" ) )
        {
            stc("Specify the coin type.\n\r",ch);
            return;
        }

	if ( !str_prefix( arg, "silver") )
	{
	    if (ch->money[1][0] < amount)
	    {
		stc( "You don't have that many silver pennies.\n\r",ch);
		return;
	    }

	    ch->money[1][0] -= amount;
	    silver = amount;
	}
	else if( !str_prefix( arg, "copper") )
	{
	    if (ch->money[0][0] < amount)
	    {
		stc( "You don't have that many copper pennies.\n\r",ch);
		return;
	    }

	    ch->money[0][0] -= amount;
  	    copper = amount;
	}
	else if( !str_prefix( arg, "marks") )
	{
	    if (ch->money[2][0] < amount)
	    {
		stc( "You don't have that many silver marks.\n\r",ch);
		return;
	    }

	    ch->money[2][0] -= amount;
  	    mark = amount;
	}
	else if( !str_prefix( arg, "gold") )
	{
	    if (ch->money[3][0] < amount)
	    {
		stc( "You don't have that many gold marks.\n\r",ch);
		return;
	    }

	    ch->money[3][0] -= amount;
  	    gold = amount;
	}
/*
	for ( obj = ch->in_room->contents; obj != NULL; obj = obj_next )
	{
	    obj_next = obj->next_content;

	    switch ( obj->pIndexData->vnum )
	    {
	    case OBJ_VNUM_SILVER_ONE:
		silver += 1;
		extract_obj(obj);
		break;

	    case OBJ_VNUM_GOLD_ONE:
		copper += 1;
		extract_obj( obj );
		break;

	    case OBJ_VNUM_SILVER_SOME:
		silver += obj->value[0];
		extract_obj(obj);
		break;

	    case OBJ_VNUM_GOLD_SOME:
		copper += obj->value[1];
		extract_obj( obj );
		break;

	    case OBJ_VNUM_COINS:
		silver += obj->value[0];
		copper += obj->value[1];
		extract_obj(obj);
		break;
	    }
	}
*/
	obj_to_room( create_money( copper, silver, mark, gold ), ch->in_room );
	act( "$n drops some coins.", ch, NULL, NULL, TO_ROOM );
	stc( "OK.\n\r", ch );
	return;
    }

    if ( str_cmp( arg, "all" ) && str_prefix( "all.", arg ) )
    {
	/* 'drop obj' */
	if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
	{
	    stc( "{cYou do not have that item.\n\r", ch );
	    return;
	}

	if ( ( obj->item_type == ITEM_CONTAINER || obj->item_type == ITEM_TREASURECHEST )
	&& ch->pcdata != NULL && ch->pcdata->in_vault  )
	{
		stc("You can't drop containers in here.\n\r", ch);
		return;
	}
	if ( !can_drop_obj( ch, obj ) )
	{
	    stc( "{cYou can't let go of it.\n\r", ch );
	    return;
	}

	obj_from_char( obj );
	obj_to_room( obj, ch->in_room );
	act( "{c$n releases and drops $p.", ch, obj, NULL, TO_ROOM );
	act( "{cYou release and drop $p.", ch, obj, NULL, TO_CHAR );

        if (IS_SET(ch->in_room->room_flags, ROOM_HOME))
                save_home_item(obj);

	if ( HAS_TRIGGER_OBJ( obj, TRIG_DROP ) )
	    p_give_trigger( NULL, obj, NULL, ch, obj, TRIG_DROP );
	if ( HAS_TRIGGER_ROOM( ch->in_room, TRIG_DROP ) )
	    p_give_trigger( NULL, NULL, ch->in_room, ch, obj, TRIG_DROP );

	if ( obj && IS_OBJ_STAT(obj,ITEM_MELT_DROP))
	{
	    act("$p dissolves into smoke.",ch,obj,NULL,TO_ROOM);
	    act("$p dissolves into smoke.",ch,obj,NULL,TO_CHAR);
	    extract_obj(obj);
	}
    }
    else
    {
		int iCnt = 0;
		/* 'drop all' or 'drop all.obj' */
		found = FALSE;
		for ( obj = ch->carrying; obj != NULL; obj = obj_next )
		{
		    obj_next = obj->next_content;

		    if ( ( arg[3] == '\0' || is_name( &arg[4], obj->name ) )
		    &&   can_see_obj( ch, obj )
		    &&   obj->wear_loc == WEAR_NONE
		    &&   can_drop_obj( ch, obj ) )
		    {
				iCnt++;
				if ( iCnt > 20 )
				{
					stc( "You've reached the spam limit. Wait a moment and try again.\n\r", ch );
					return;
				}
			        if ( ( obj->item_type == ITEM_CONTAINER || obj->item_type == ITEM_TREASURECHEST )
			        && ch->pcdata != NULL && ch->pcdata->in_vault  )
			        {
			                stc("You can't drop containers in here.\n\r", ch);
			                continue;
			        }
				found = TRUE;
				obj_from_char( obj );
				obj_to_room( obj, ch->in_room );
				act( "{c$n drops $p.", ch, obj, NULL, TO_ROOM );
				act( "{cYou drop $p.", ch, obj, NULL, TO_CHAR );

			        if (IS_SET(ch->in_room->room_flags, ROOM_HOME))
			                save_home_item(obj);

				if ( HAS_TRIGGER_OBJ( obj, TRIG_DROP ) )
				    p_give_trigger( NULL, obj, NULL, ch, obj, TRIG_DROP );
				if ( HAS_TRIGGER_ROOM( ch->in_room, TRIG_DROP ) )
				    p_give_trigger( NULL, NULL, ch->in_room, ch, obj, TRIG_DROP );

				if ( obj && IS_OBJ_STAT(obj,ITEM_MELT_DROP))
				{
				    act("$p dissolves into smoke.",ch,obj,NULL,TO_ROOM);
				    act("$p dissolves into smoke.",ch,obj,NULL,TO_CHAR);
				    extract_obj(obj);
				}
		    }
		}

		if ( !found )
		{
		    if ( arg[3] == '\0' )
			act( "{cYou are not carrying anything.",
			    ch, NULL, arg, TO_CHAR );
		    else
			act( "{cYou are not carrying any $T.",
			    ch, NULL, &arg[4], TO_CHAR );
		}
    }

    return;
}

bool check_give_trigger (CHAR_DATA *mob, OBJ_DATA *obj)
{
	PROG_LIST *prg;
	char objvnum[MSL];

	sprintf(objvnum, "%ld", obj->pIndexData->vnum);

	for (prg = mob->pIndexData->mprogs; prg; prg = prg->next)
	{
		if ( prg->trig_type != TRIG_GIVE )	continue;
		if (str_cmp(objvnum, prg->trig_phrase))	continue;
		return TRUE;
	}

	return FALSE;
}


void do_give( CHAR_DATA *ch, char *argument )
{
    char arg1[MIL];
    char arg2[MIL];
    char arg3[MIL];
    char buf[MSL];
    CHAR_DATA *victim;
    OBJ_DATA  *obj;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
	stc( "{cGive what to whom?\n\r", ch );
	return;
    }

    if (is_number(arg1))/*Adapted by Anthony to make sure bribing works correctly*/
    {
		/* begin give coins */
		int amount,i=0;
		int coin=-1;
		char moneytypes[4][6]		=	{"copper", "silver", "marks", "gold"};
		char MT_long[4][15]	=	{"copper pennies", "silver pennies", "silver marks", "gold marks"};
		int moneyvalues[4] = {1,10,100,1000};

		amount = atoi(arg1);

		if (amount <= 0)
		{
		    stc("You can't do that.\n\r", ch);
		    return;
		}

		for(i=0;i<4;i++)
		{
			if ( !str_prefix( arg2, moneytypes[i]) )
			{
				coin = i;
			}
		}

		if (coin < 0)
		{
			stc("Give what to whom?\n\r", ch);
			return;
		}

		argument = one_argument(argument, arg2);
		if (arg2[0] == '\0')
		{
		    stc("Give what to whom?\n\r", ch);
		    return;
		}

		if ((victim = get_char_room(ch, NULL, arg2)) == NULL &&
	    (victim = get_char_room(ch, NULL, arg3)) == NULL)
		{
		    stc("They aren't here.\n\r", ch);
		    return;
		}

		if ( ch->money[coin][0] < amount )
		{
                       ptc(ch,"You have only %d %s.\n\r",ch->money[coin][0],MT_long[coin]);
                       return;

		}

		ch->money[coin][0] -= amount;
		victim->money[coin][0] += amount;

		sprintf(buf, "$n gives you %d %s.", amount, MT_long[coin]);
		act(buf, ch, NULL, victim, TO_VICT);
		act("$n gives $N some coins.", ch, NULL, victim, TO_NOTVICT);
		sprintf(buf, "You give $N %d %s.", amount, MT_long[coin]);
		act(buf, ch, NULL, victim, TO_CHAR);

		/*
		 * Bribe trigger
		 */
		if ( IS_NPC(victim) && HAS_TRIGGER_MOB( victim, TRIG_BRIBE ) )
			p_bribe_trigger( victim, ch, amount * moneyvalues[coin]);

		return;

    }
    /* end give coins */

    if ( ( obj = get_obj_carry( ch, arg1, ch ) ) == NULL )
    {
	stc( "{cYou do not have that item.\n\r", ch );
	return;
    }

    if ( obj->wear_loc != WEAR_NONE )
    {
	stc( "{cYou must remove it first.\n\r", ch );
	return;
    }

    if ( IS_SET( obj->extra_flags, ITEM_NOSELL ) )
    {
	stc("You'll have to unkeep it first.\n\r", ch);
	return;
    }

    if ( ( victim = get_char_room( ch, NULL, arg2 ) ) == NULL )
    {
	stc( "{cThey aren't here.\n\r", ch );
	return;
    }

    if (IS_NPC(victim) && victim->pIndexData->pShop != NULL)
    {
	if (!check_give_trigger(victim, obj))
	{
		act( "{c$N tells you 'Sorry, you'll have to sell that.'", ch, NULL, victim, TO_CHAR);
		return;
	}
    }

	if (!IS_NPC(victim) && IS_SET(obj->extra_flags, ITEM_MELT_DROP) )
	{
	stc( "{cYou can't give this item away to players.\n\r", ch );
        return;
	}

	if ( obj->owner != NULL && obj->owner[0] != '\0' && str_cmp( obj->owner, victim->name ) && str_cmp( obj->owner, ch->name)  )
	{
		ptc( ch, "This item belongs to %s. They must unkeep it first.\n\r", obj->owner );
		return;
	}
    if ( !can_drop_obj( ch, obj ) )
    {
	stc( "{cYou can't let go of it.\n\r", ch );
	return;
    }

    if ( victim->carry_number + get_obj_number( obj ) > can_carry_n( victim ) )
    {
	act( "{c$N has $S hands full.", ch, NULL, victim, TO_CHAR );
	return;
    }

    if (get_carry_weight(victim) + get_obj_weight(obj) > can_carry_w( victim ) )
    {
	act( "{c$N can't carry that much weight.", ch, NULL, victim, TO_CHAR );
	return;
    }

    if ( !can_see_obj( victim, obj ) )
    {
	act( "{c$N can't see it.", ch, NULL, victim, TO_CHAR );
	return;
    }

    obj_from_char( obj );
    obj_to_char( obj, victim );
    MOBtrigger = FALSE;
    act( "{c$n gives $p to $N.", ch, obj, victim, TO_NOTVICT );
    act( "{c$n gives you $p.",   ch, obj, victim, TO_VICT    );
    act( "{cYou give $p to $N.", ch, obj, victim, TO_CHAR    );
    MOBtrigger = TRUE;
    if ( HAS_TRIGGER_OBJ( obj, TRIG_GIVE ) )
	p_give_trigger( NULL, obj, NULL, ch, obj, TRIG_GIVE );
    if ( HAS_TRIGGER_ROOM( ch->in_room, TRIG_GIVE ) )
	p_give_trigger( NULL, NULL, ch->in_room, ch, obj, TRIG_GIVE );

    /*
     * Give trigger
     */
    if ( IS_NPC(victim) && HAS_TRIGGER_MOB( victim, TRIG_GIVE ) )
	p_give_trigger( victim, NULL, NULL, ch, obj, TRIG_GIVE );

    return;
}


void do_fill( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *obj;
    OBJ_DATA *fountain;
    bool found;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cFill what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
	stc( "{cYou do not have that item.\n\r", ch );
	return;
    }

    found = FALSE;
    for ( fountain = ch->in_room->contents; fountain != NULL;
	fountain = fountain->next_content )
    {
	if ( fountain->item_type == ITEM_FOUNTAIN )
	{
	    found = TRUE;
	    break;
	}
    }

    if ( !found )
    {
	stc( "{cThere is no fountain here!\n\r", ch );
	return;
    }

    if ( obj->item_type != ITEM_DRINK_CON )
    {
	stc( "{cYou can't fill that.\n\r", ch );
	return;
    }

    if ( obj->value[1] != 0 && obj->value[2] != fountain->value[2] )
    {
	stc( "{cThere is already another liquid in it.\n\r", ch );
	return;
    }

    if ( obj->value[1] >= obj->value[0] )
    {
	stc( "{cYour container is full.\n\r", ch );
	return;
    }

    sprintf(buf,"{cYou fill $p with %s from $P.",
	liq_table[fountain->value[2]].liq_name);
    act( buf, ch, obj,fountain, TO_CHAR );
    sprintf(buf,"{c$n fills $p with %s from $P.",
	liq_table[fountain->value[2]].liq_name);
    act(buf,ch,obj,fountain,TO_ROOM);
    obj->value[2] = fountain->value[2];
    obj->value[1] = obj->value[0];
    return;
}

void do_pour (CHAR_DATA *ch, char *argument)
{
    char arg[MAX_STRING_LENGTH],buf[MAX_STRING_LENGTH];
    OBJ_DATA *out, *in;
    CHAR_DATA *vch = NULL;
    int amount;

    argument = one_argument(argument,arg);

    if (arg[0] == '\0' || argument[0] == '\0')
    {
	stc( "{cPour what into what?\n\r",ch);
	return;
    }


    if ((out = get_obj_carry(ch,arg, ch)) == NULL)
    {
	stc( "{cYou don't have that item.\n\r",ch);
	return;
    }

    if (out->item_type != ITEM_DRINK_CON)
    {
	stc( "{cThat's not a drink container.\n\r",ch);
	return;
    }

    if (!str_cmp(argument,"out"))
    {
	if (out->value[1] == 0)
	{
	    stc( "{cIt's already empty.\n\r",ch);
	    return;
	}

	out->value[1] = 0;
	out->value[3] = 0;
	sprintf(buf,"{cYou invert $p, spilling %s all over the ground.",
		liq_table[out->value[2]].liq_name);
	act(buf,ch,out,NULL,TO_CHAR);

	sprintf(buf,"{c$n inverts $p, spilling %s all over the ground.",
		liq_table[out->value[2]].liq_name);
	act(buf,ch,out,NULL,TO_ROOM);
	return;
    }

    if ((in = get_obj_here(ch, NULL, argument)) == NULL)
    {
	vch = get_char_room(ch, NULL, argument);

	if (vch == NULL)
	{
	    stc( "{cPour into what?\n\r",ch);
	    return;
	}

	in = get_eq_char(vch,WEAR_HOLD);

	if (in == NULL)
	{
	    stc( "{cThey aren't holding anything.",ch);
 	    return;
	}
    }

    if (in->item_type != ITEM_DRINK_CON)
    {
	stc( "{cYou can only pour into other drink containers.\n\r",ch);
	return;
    }

    if (in == out)
    {
	stc( "{cYou cannot change the laws of physics!\n\r",ch);
	return;
    }

    if (in->value[1] != 0 && in->value[2] != out->value[2])
    {
	stc( "{cThey don't hold the same liquid.\n\r",ch);
	return;
    }

    if (out->value[1] == 0)
    {
	act( "{cThere's nothing in $p to pour.",ch,out,NULL,TO_CHAR);
	return;
    }

    if (in->value[1] >= in->value[0])
    {
	act( "{c$p is already filled to the top.",ch,in,NULL,TO_CHAR);
	return;
    }

    amount = UMIN(out->value[1],in->value[0] - in->value[1]);

    in->value[1] += amount;
    out->value[1] -= amount;
    in->value[2] = out->value[2];

    if (vch == NULL)
    {
    	sprintf(buf,"{cYou pour %s from $p into $P.",
	    liq_table[out->value[2]].liq_name);
    	act(buf,ch,out,in,TO_CHAR);
    	sprintf(buf,"{c$n pours %s from $p into $P.",
	    liq_table[out->value[2]].liq_name);
    	act(buf,ch,out,in,TO_ROOM);
    }
    else
    {
        sprintf(buf,"{cYou pour some %s for $N.",
            liq_table[out->value[2]].liq_name);
        act(buf,ch,NULL,vch,TO_CHAR);
	sprintf(buf,"{c$n pours you some %s.",
	    liq_table[out->value[2]].liq_name);
	act(buf,ch,NULL,vch,TO_VICT);
        sprintf(buf,"{c$n pours some %s for $N.",
            liq_table[out->value[2]].liq_name);
        act(buf,ch,NULL,vch,TO_NOTVICT);

    }
}

void do_sip( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int amount;
    int liquid;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	    stc( "Sip from what?\n\r", ch );
	    return;
    }
    else
    {
		if ( ( obj = get_obj_here( ch, NULL, arg ) ) == NULL )
		{
		    stc( "You don't see that here.\n\r", ch );
		    return;
		}
    }

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10 )
    {
		stc( "{cYou fail to reach your mouth.  *Hic*\n\r", ch );
		return;
    }

    switch ( obj->item_type )
    {
    default:
	stc( "{cYou can't sip from that.\n\r", ch );
	return;

    case ITEM_FOUNTAIN:
    case ITEM_DRINK_CON:
	break;

    }

	if ( obj->value[1] <= 0 )
	{
	    stc( "It is already empty.\n\r", ch );
	    return;
	}

	if ( ( liquid = obj->value[2] )  < 0 )
	{
	    bug( "Do_drink: bad liquid number %d.", liquid );
	    liquid = obj->value[2] = 0;
	}

    amount = liq_table[liquid].liq_affect[4];
    amount = 1;

    act( "$n sips $T from $p.", ch, obj, liq_table[liquid].liq_name, TO_ROOM );
    act( "You sip $T from $p.", ch, obj, liq_table[liquid].liq_name, TO_CHAR );

	if ( obj->value[3] != 0 )
	{
	    /* The drink was poisoned! */
	    AFFECT_DATA *af = new_affect();

	    act( "{c$n chokes and gags.", ch, 0, 0, TO_ROOM );
	    stc( "{cYou choke and gag.\n\r", ch );

	    af->where	 = TO_AFFECTS;
	    af->type      = gsn_poison;
	    af->level 	 = number_fuzzy(1);
	    af->duration  = 5;
	    af->location  = APPLY_NONE;
	    af->modifier  = 0;
	    af->bitvector = AFF_POISON;
	    affect_join( ch, af );
	}

    if (obj->value[0] > 0)
        obj->value[1] -= amount;

    return;
}

void do_drink( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int amount;
    int liquid;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
		for ( obj = ch->in_room->contents; obj; obj = obj->next_content )
		{
		    if ( obj->item_type == ITEM_FOUNTAIN )
			break;
		}

		if ( obj == NULL )
		{
		    stc( "{cDrink what?\n\r", ch );
		    return;
		}
    }
    else
    {
		if ( ( obj = get_obj_here( ch, NULL, arg ) ) == NULL )
		{
		    stc( "{cYou can't find it.\n\r", ch );
		    return;
		}
    }

/*
    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10 )
    {
		stc( "{cYou fail to reach your mouth.  *Hic*\n\r", ch );
		return;
    }
*/
    if (obj==0) // prool
	{
	printf("prool: obj==0 \r\n");
	return;
	}
    switch ( obj->item_type ) // prool: crash wuz here
    {
	    default:
			stc( "{cYou can't drink from that.\n\r", ch );
		return;

	    case ITEM_POTION:
			do_function( ch, &do_quaff, obj->name );
		return;

	    case ITEM_FOUNTAIN:
	        if ( ( liquid = obj->value[2] )  < 0 )
	        {
	            bug( "Do_drink: bad liquid number %d.", liquid );
	            liquid = obj->value[2] = 0;
	        }
			amount = liq_table[liquid].liq_affect[4] * 3;
		break;

	    case ITEM_DRINK_CON:
			if ( obj->value[1] <= 0 )
			{
			    stc( "{cIt is already empty.\n\r", ch );
			    return;
			}
			if ( ( liquid = obj->value[2] )  < 0 )
			{
			    bug( "Do_drink: bad liquid number %d.", liquid );
			    liquid = obj->value[2] = 0;
			}

	        amount = liq_table[liquid].liq_affect[4];
	        amount = UMIN(amount, obj->value[1]);
		break;
     }

    act( "{c$n drinks $T from $p.",	ch, obj, liq_table[liquid].liq_name, TO_ROOM );
    act( "{cYou drink $T from $p.",	ch, obj, liq_table[liquid].liq_name, TO_CHAR );

	WAIT_STATE(ch,PULSE_VIOLENCE);

	if ( liq_table[liquid].liq_affect[COND_DRUNK] < 0
	&& !IS_NPC( ch )
	&& ch->pcdata->condition[COND_DRUNK] >= 2 )
	    ch->pcdata->condition[COND_DRUNK] -= liq_table[liquid].liq_affect[COND_DRUNK];
	else
	    gain_condition( ch, COND_DRUNK, amount * liq_table[liquid].liq_affect[COND_DRUNK] / 36 );

    gain_condition( ch, COND_FULL, amount * liq_table[liquid].liq_affect[COND_FULL] / 4 );
    gain_condition( ch, COND_THIRST, amount * liq_table[liquid].liq_affect[COND_THIRST] / 10 );
    gain_condition(ch, COND_HUNGER,	amount * liq_table[liquid].liq_affect[COND_HUNGER] / 2 );

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK]  > 10 )
		stc( "{cYou feel drunk.\n\r", ch );
    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_FULL]   > 40 )
		stc( "{cYou are full.\n\r", ch );
    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_THIRST] > 40 )
		stc( "{cYour thirst is quenched.\n\r", ch );

    if ( obj->value[3] != 0 )
    {
	    /* The food was poisoned! */
	    AFFECT_DATA *af = new_affect();

	    act( "{c$n chokes and gags.", ch, 0, 0, TO_ROOM );
	    stc( "{cYou choke and gag.\n\r", ch );

	    af->where	 = TO_AFFECTS;
	    af->type      = gsn_poison;
	    af->level 	 = number_fuzzy(amount);
	    af->duration  = UMIN(50, amount / 2);
	    af->location  = APPLY_NONE;
	    af->modifier  = 0;
	    af->bitvector = AFF_POISON;
	    affect_join( ch, af );
	}

    if (obj->value[0] > 0)
        obj->value[1] -= amount;

    return;
}


void do_nibble( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
		stc( "{cNibble what?\n\r", ch );
		return;
    }

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
		stc( "{cYou do not have that item.\n\r", ch );
		return;
    }

    if ( !IS_IMMORTAL(ch) )
    {
		if ( obj->item_type != ITEM_FOOD )
		{
		    stc( "You can only nibble food.\n\r", ch );
		    return;
		}
    }

	if ( obj->value[0] <= 0 )
	{
		stc( "There's not enough to nibble on.\n\r", ch );
		return;
	}

    act( "{c$n nibbles on $p.",  ch, obj, NULL, TO_ROOM );
    act( "{cYou nibble $p.", ch, obj, NULL, TO_CHAR );
	obj->value[0]--;

	if ( obj->value[3] != 0 )
	{
	    /* The food was poisoned! */
	    AFFECT_DATA *af = new_affect();

	    act( "{c$n chokes and gags.", ch, 0, 0, TO_ROOM );
	    stc( "{cYou choke and gag.\n\r", ch );

	    af->where	 = TO_AFFECTS;
	    af->type      = gsn_poison;
	    af->level 	 = number_fuzzy(1);
	    af->duration  = 5;
	    af->location  = APPLY_NONE;
	    af->modifier  = 0;
	    af->bitvector = AFF_POISON;
	    affect_join( ch, af );
	}

	if ( obj->value[0] <= 0 )
	{
		stc( "That was the last of it.\n\r", ch );
	    extract_obj( obj );
	}

    return;
}

void do_eat( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc( "{cEat what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
	stc( "{cYou do not have that item.\n\r", ch );
	return;
    }

    if ( !IS_IMMORTAL(ch) )
    {
	if ( obj->item_type != ITEM_FOOD && obj->item_type != ITEM_PILL )
	{
	    stc( "{cThat's not edible.\n\r", ch );
	    return;
	}

	/*if ( !IS_NPC(ch) && ch->pcdata->condition[COND_FULL] > 40 )
	{
	    stc( "{cYou are too full to eat more.\n\r", ch );
	    return;
	}*/
    }

    act( "{c$n eats $p.",  ch, obj, NULL, TO_ROOM );
    act( "{cYou eat $p.", ch, obj, NULL, TO_CHAR );

	WAIT_STATE(ch,PULSE_VIOLENCE);

    switch ( obj->item_type )
    {

    case ITEM_FOOD:
	if ( !IS_NPC(ch) )
	{
	    int condition;

	    condition = ch->pcdata->condition[COND_HUNGER];
	    gain_condition( ch, COND_FULL, obj->value[0] );
	    gain_condition( ch, COND_HUNGER, obj->value[1]);
	    if ( condition == 0 && ch->pcdata->condition[COND_HUNGER] > 0 )
		stc( "{cYou are no longer hungry.\n\r", ch );
	    else if ( ch->pcdata->condition[COND_FULL] > 40 )
		stc( "{cYou are full.\n\r", ch );
	}

	if ( obj->value[3] != 0 )
	{
	    /* The food was poisoned! */
	    AFFECT_DATA *af = new_affect();

	    act( "{c$n chokes and gags.", ch, 0, 0, TO_ROOM );
	    stc( "{cYou choke and gag.\n\r", ch );

	    af->where	 = TO_AFFECTS;
	    af->type      = gsn_poison;
	    af->level 	 = number_fuzzy(obj->value[0]);
	    af->duration  = UMIN(50, obj->value[0] / 2);
	    af->location  = APPLY_NONE;
	    af->modifier  = 0;
	    af->bitvector = AFF_POISON;
	    affect_join( ch, af );
	}
	break;

    case ITEM_PILL:
	obj_cast_spell( obj->value[1], obj->value[0], ch, ch, NULL );
	obj_cast_spell( obj->value[2], obj->value[0], ch, ch, NULL );
	obj_cast_spell( obj->value[3], obj->value[0], ch, ch, NULL );
	obj_cast_spell( obj->value[4], obj->value[0], ch, ch, NULL );
	break;
    }

    extract_obj( obj );
    return;
}



/*
 * Remove an object.
 */
bool remove_obj( CHAR_DATA *ch, int iWear, bool fReplace )
{
    OBJ_DATA *obj;

    if ( ( obj = get_eq_char( ch, iWear ) ) == NULL )
	return TRUE;

    if ( !fReplace )
	return FALSE;

    if ( IS_SET(obj->extra_flags, ITEM_NOREMOVE) )
    {
	act( "{cYou can't remove $p.", ch, obj, NULL, TO_CHAR );
	return FALSE;
    }

    unequip_char( ch, obj );
    act( "{c$n stops using $p.", ch, obj, NULL, TO_ROOM );
    act( "{cYou stop using $p.", ch, obj, NULL, TO_CHAR );
    return TRUE;
}



/*
 * Wear one object.
 * Optional replacement of existing objects.
 * Big repetitive code, ick.
 */
void wear_obj( CHAR_DATA *ch, OBJ_DATA *obj, bool fReplace )
{
    int guild;

	guild = clan_lookup(obj->guild);

    /*if ( ch->level < obj->level )
    {
	sprintf( buf, "{cYou must be level %d to use this object.\n\r",
	    obj->level );
	stc( buf, ch );
	act( "{c$n tries to use $p, but is too inexperienced.",
	    ch, obj, NULL, TO_ROOM );
	return;
    }*/

    if (IS_SET(obj->extra_flags,ITEM_REMORT)
    && (IS_NPC(ch) || ch->pcdata->rebirth < 1) )
    {
		act("You are not powerful enough to wear that.",ch,0,0,TO_CHAR);
		return;
	}

    if (!is_obj_char_clan(ch,obj) && is_obj_clan( obj ) )
    {
		ptc( ch, "You can't wear items that belong to the %s Guild.\n\r", capit2(obj->guild) );
		return;
	}
	if ( obj->owner != NULL && obj->owner[0] != '\0' && str_cmp( obj->owner, ch->name)  )
	{
		ptc( ch, "This item belongs to %s. You can not wear it.\n\r", obj->owner );
		return;
	}
    if ( obj->item_type == ITEM_LIGHT )
    {
	if ( !remove_obj( ch, WEAR_LIGHT, fReplace ) )
	    return;
	act( "{c$n lights $p and holds it.", ch, obj, NULL, TO_ROOM );
	act( "{cYou light $p and hold it.",  ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_LIGHT );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_FINGER ) )
    {
	if ( get_eq_char( ch, WEAR_FINGER_L ) != NULL
	&&   get_eq_char( ch, WEAR_FINGER_R ) != NULL
	&&   !remove_obj( ch, WEAR_FINGER_L, fReplace )
	&&   !remove_obj( ch, WEAR_FINGER_R, fReplace ) )
	    return;

	if ( get_eq_char( ch, WEAR_FINGER_L ) == NULL )
	{
	    act( "{c$n wears $p on $s left finger.",    ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p on your left finger.",  ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_FINGER_L );
	    return;
	}

	if ( get_eq_char( ch, WEAR_FINGER_R ) == NULL )
	{
	    act( "{c$n wears $p on $s right finger.",   ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p on your right finger.", ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_FINGER_R );
	    return;
	}

	bug( "Wear_obj: no free finger.", 0 );
	stc( "{cYou already wear two rings.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_NECK ) )
    {
	if ( get_eq_char( ch, WEAR_NECK_1 ) != NULL
	&&   get_eq_char( ch, WEAR_NECK_2 ) != NULL
	&&   !remove_obj( ch, WEAR_NECK_1, fReplace )
	&&   !remove_obj( ch, WEAR_NECK_2, fReplace ) )
	    return;

	if ( get_eq_char( ch, WEAR_NECK_1 ) == NULL )
	{
	    act( "{c$n wears $p around $s neck.",   ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p around your neck.", ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_NECK_1 );
	    return;
	}

	if ( get_eq_char( ch, WEAR_NECK_2 ) == NULL )
	{
	    act( "{c$n wears $p around $s neck.",   ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p around your neck.", ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_NECK_2 );
	    return;
	}

	bug( "Wear_obj: no free neck.", 0 );
	stc( "{cYou already wear two neck items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_BODY ) )
    {
	if ( !remove_obj( ch, WEAR_BODY, fReplace ) )
	    return;
	act( "{c$n wears $p on $s body.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p on your body.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_BODY );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_HEAD ) )
    {
	if ( !remove_obj( ch, WEAR_HEAD, fReplace ) )
	    return;
	act( "{c$n wears $p on $s head.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p on your head.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_HEAD );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_LEGS ) )
    {
	if ( !remove_obj( ch, WEAR_LEGS, fReplace ) )
	    return;
	act( "{c$n wears $p on $s legs.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p on your legs.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_LEGS );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_FEET ) )
    {
	if ( !remove_obj( ch, WEAR_FEET, fReplace ) )
	    return;
	act( "{c$n wears $p on $s feet.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p on your feet.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_FEET );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_HANDS ) )
    {
	if ( !remove_obj( ch, WEAR_HANDS, fReplace ) )
	    return;
	act( "{c$n wears $p on $s hands.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p on your hands.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_HANDS );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_ARMS ) )
    {
	if ( !remove_obj( ch, WEAR_ARMS, fReplace ) )
	    return;
	act( "{c$n wears $p on $s arms.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p on your arms.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_ARMS );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_ABOUT ) )
    {
	if ( !remove_obj( ch, WEAR_ABOUT, fReplace ) )
	    return;
	act( "{c$n wears $p about $s body.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p about your body.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_ABOUT );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_WAIST ) )
    {
	if ( !remove_obj( ch, WEAR_WAIST, fReplace ) )
	    return;
	act( "{c$n wears $p about $s waist.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p about your waist.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_WAIST );
	return;
    }

/* new wear code */
    if ( CAN_WEAR( obj, ITEM_WEAR_CALF ) )
    {
        if ( get_eq_char( ch, WEAR_CALF_L ) != NULL
        &&   get_eq_char( ch, WEAR_CALF_R ) != NULL
        &&   !remove_obj( ch, WEAR_CALF_L, fReplace )
        &&   !remove_obj( ch, WEAR_CALF_R, fReplace ) )
	    return;

        if ( get_eq_char( ch, WEAR_CALF_L ) == NULL )
	{
            act( "{c$n wears $p around $s left calf.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p around your left calf.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_CALF_L );
	    return;
	}

        if ( get_eq_char( ch, WEAR_CALF_R ) == NULL )
	{
            act( "{c$n wears $p around $s right calf.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p around your right calf.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_CALF_R );
	    return;
	}

        bug( "Wear_obj: no free calf.", 0 );
        stc( "{cYou already wear two calf items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_THIGH ) )
    {
        if ( get_eq_char( ch, WEAR_THIGH_L ) != NULL
        &&   get_eq_char( ch, WEAR_THIGH_R ) != NULL
        &&   !remove_obj( ch, WEAR_THIGH_L, fReplace )
        &&   !remove_obj( ch, WEAR_THIGH_R, fReplace ) )
	    return;

        if ( get_eq_char( ch, WEAR_THIGH_L ) == NULL )
	{
            act( "{c$n wears $p around $s left thigh.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p around your left thigh.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_THIGH_L );
	    return;
	}

        if ( get_eq_char( ch, WEAR_THIGH_R ) == NULL )
	{
            act( "{c$n wears $p around $s right thigh.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p around your right thigh.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_THIGH_R );
	    return;
	}

        bug( "Wear_obj: no free thigh.", 0 );
        stc( "{cYou already wear two thigh items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_FOREARM ) )
    {
        if ( get_eq_char( ch, WEAR_FOREARM_L ) != NULL
        &&   get_eq_char( ch, WEAR_FOREARM_R ) != NULL
        &&   !remove_obj( ch, WEAR_FOREARM_L, fReplace )
        &&   !remove_obj( ch, WEAR_FOREARM_R, fReplace ) )
	    return;

        if ( get_eq_char( ch, WEAR_FOREARM_L ) == NULL )
	{
            act( "{c$n wears $p around $s left forearm.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p around your left forearm.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_FOREARM_L );
	    return;
	}

        if ( get_eq_char( ch, WEAR_FOREARM_R ) == NULL )
	{
            act( "{c$n wears $p around $s right forearm.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p around your right forearm.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_FOREARM_R );
	    return;
	}

        bug( "Wear_obj: no free forearm.", 0 );
        stc( "{cYou already wear two forearm items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_NOSE ) )
    {
        if ( !remove_obj( ch, WEAR_NOSE, fReplace ) )
	    return;
        act( "{c$n wears $p in $s nose.",   ch, obj, NULL, TO_ROOM );
        act( "{cYou wear $p in your nose.", ch, obj, NULL, TO_CHAR );
        equip_char( ch, obj, WEAR_NOSE );
	return;
    }
    if ( CAN_WEAR( obj, ITEM_WEAR_EAR ) )
    {
        if ( get_eq_char( ch, WEAR_EAR_1 ) != NULL
        &&   get_eq_char( ch, WEAR_EAR_2 ) != NULL
        &&   !remove_obj( ch, WEAR_EAR_1, fReplace )
        &&   !remove_obj( ch, WEAR_EAR_2, fReplace ) )
	    return;

        if ( get_eq_char( ch, WEAR_EAR_1 ) == NULL )
	{
            act( "{c$n wears $p in $s left ear.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p in your left ear.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_EAR_1 );
	    return;
	}

        if ( get_eq_char( ch, WEAR_EAR_2 ) == NULL )
	{
            act( "{c$n wears $p in $s right ear.",
		ch, obj, NULL, TO_ROOM );
            act( "{cYou wear $p in your right ear.",
		ch, obj, NULL, TO_CHAR );
            equip_char( ch, obj, WEAR_EAR_2 );
	    return;
	}

        bug( "Wear_obj: no free ear.", 0 );
        stc( "{cYou already wear two ear items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_SHOULDERS ) )
    {
        if ( !remove_obj( ch, WEAR_SHOULDERS, fReplace ) )
	    return;
        act( "{c$n wears $p on $s shoulders.",   ch, obj, NULL, TO_ROOM );
        act( "{cYou wear $p on your shoulders.", ch, obj, NULL, TO_CHAR );
        equip_char( ch, obj, WEAR_SHOULDERS );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_BACK ) )
    {
        if ( !remove_obj( ch, WEAR_BACK, fReplace ) )
	    return;
        act( "{c$n wears $p on $s back.",   ch, obj, NULL, TO_ROOM );
        act( "{cYou wear $p on your back.", ch, obj, NULL, TO_CHAR );
        equip_char( ch, obj, WEAR_BACK );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_EYES ) )
    {
        if ( !remove_obj( ch, WEAR_EYES, fReplace ) )
	    return;
        act( "{c$n wears $p over $s eyes.",   ch, obj, NULL, TO_ROOM );
        act( "{cYou wear $p over your eyes.", ch, obj, NULL, TO_CHAR );
        equip_char( ch, obj, WEAR_EYES );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_ANKLE ) )
    {
	if ( get_eq_char( ch, WEAR_ANKLE_L ) != NULL
	&&   get_eq_char( ch, WEAR_ANKLE_R ) != NULL
	&&   !remove_obj( ch, WEAR_ANKLE_L, fReplace )
	&&   !remove_obj( ch, WEAR_ANKLE_R, fReplace ) )
	    return;

	if ( get_eq_char( ch, WEAR_ANKLE_L ) == NULL )
	{
	    act( "{c$n wears $p around $s left ankle.",
		ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p around your left ankle.",
		ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_ANKLE_L );
	    return;
	}

	if ( get_eq_char( ch, WEAR_ANKLE_R ) == NULL )
	{
	    act( "{c$n wears $p around $s right ankle.",
		ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p around your right ankle.",
		ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_ANKLE_R );
	    return;
	}

	bug( "Wear_obj: no free ankle.", 0 );
	stc( "{cYou already wear two ankle items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_WRIST ) )
    {
	if ( get_eq_char( ch, WEAR_WRIST_L ) != NULL
	&&   get_eq_char( ch, WEAR_WRIST_R ) != NULL
	&&   !remove_obj( ch, WEAR_WRIST_L, fReplace )
	&&   !remove_obj( ch, WEAR_WRIST_R, fReplace ) )
	    return;

	if ( get_eq_char( ch, WEAR_WRIST_L ) == NULL )
	{
	    act( "{c$n wears $p around $s left wrist.",
		ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p around your left wrist.",
		ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_WRIST_L );
	    return;
	}

	if ( get_eq_char( ch, WEAR_WRIST_R ) == NULL )
	{
	    act( "{c$n wears $p around $s right wrist.",
		ch, obj, NULL, TO_ROOM );
	    act( "{cYou wear $p around your right wrist.",
		ch, obj, NULL, TO_CHAR );
	    equip_char( ch, obj, WEAR_WRIST_R );
	    return;
	}

	bug( "Wear_obj: no free wrist.", 0 );
	stc( "{cYou already wear two wrist items.\n\r", ch );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WEAR_SHIELD ) )
    {
	OBJ_DATA *weapon;

	if ( !remove_obj( ch, WEAR_SHIELD, fReplace ) )
	    return;

	weapon = get_eq_char(ch,WEAR_WIELD);
	if (weapon != NULL && ch->size < SIZE_LARGE
	&&  IS_WEAPON_STAT(weapon,WEAPON_TWO_HANDS))
	{
	    stc( "{cYour hands are tied up with your weapon!\n\r",ch);
	    return;
	}

    /* secondary weapon code */
    if (get_eq_char (ch, WEAR_SECONDARY) != NULL)
    {
        stc( "{cYou cannot use a shield while wielding two weapons.\n\r",ch);
        return;
    }

	act( "{c$n wears $p as a shield.", ch, obj, NULL, TO_ROOM );
	act( "{cYou wear $p as a shield.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_SHIELD );
	return;
    }

    if ( CAN_WEAR( obj, ITEM_WIELD ) )
    {
	int sn,skill;

	if ( !remove_obj( ch, WEAR_WIELD, fReplace ) )
	    return;

	if ( !IS_NPC(ch)
	&& get_obj_weight(obj) > ( get_curr_stat(ch,STAT_STR) + 25 ) )
	{
	    stc( "{cIt is too heavy for you to wield.\n\r", ch );
	    return;
	}

	if (!IS_NPC(ch) && ch->size < SIZE_LARGE
	&&  IS_WEAPON_STAT(obj,WEAPON_TWO_HANDS)
 	&&  get_eq_char(ch,WEAR_SHIELD) != NULL)
	{
	    stc( "{cYou need two hands free for that weapon.\n\r",ch);
	    return;
	}

	act( "{c$n wields $p.", ch, obj, NULL, TO_ROOM );
	act( "{cYou wield $p.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_WIELD );

        sn = get_weapon_sn(ch);

	if (sn == gsn_hand_to_hand)
	   return;

        skill = get_weapon_skill(ch,sn);

        if (skill >= 100)
            act( "{c$p feels like a part of you!",ch,obj,NULL,TO_CHAR);
        else if (skill > 85)
            act( "{cYou feel quite confident with $p.",ch,obj,NULL,TO_CHAR);
        else if (skill > 70)
            act( "{cYou are skilled with $p.",ch,obj,NULL,TO_CHAR);
        else if (skill > 50)
            act( "{cYour skill with $p is adequate.",ch,obj,NULL,TO_CHAR);
        else if (skill > 25)
            act( "{c$p feels a little clumsy in your hands.",ch,obj,NULL,TO_CHAR);
        else if (skill > 1)
            act( "{cYou fumble and almost drop $p.",ch,obj,NULL,TO_CHAR);
        else
            act( "{cYou don't even know which end is up on $p.",
                ch,obj,NULL,TO_CHAR);

	return;
    }

    if ( CAN_WEAR( obj, ITEM_HOLD ) )
    {
	if ( !remove_obj( ch, WEAR_HOLD, fReplace ) )
	    return;
        if (get_eq_char (ch, WEAR_SECONDARY) != NULL)
        {
            stc( "{cYou cannot hold on item while wielding two weapons.\n\r",ch);
            return;
        }

	act( "{c$n holds $p in $s hand.",   ch, obj, NULL, TO_ROOM );
	act( "{cYou hold $p in your hand.", ch, obj, NULL, TO_CHAR );
	equip_char( ch, obj, WEAR_HOLD );
	return;
    }

    if ( CAN_WEAR(obj,ITEM_WEAR_NEAR_MIND) )
    {
	if (!remove_obj(ch,WEAR_NEAR_MIND, fReplace) )
	    return;
	act( "{c$n wears $p near $s mind.",ch,obj,NULL,TO_ROOM);
	act( "{cYou wear $p near your mind.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_NEAR_MIND);
	return;
    }
/*
    if ( CAN_WEAR(obj,ITEM_WEAR_SHEATH) )
    {
	if (!remove_obj(ch,WEAR_SHEATH, fReplace) )
	    return;
	act( "{c$n wears $p as a sheath.",ch,obj,NULL,TO_ROOM);
	act( "{cYou wear $p as a sheath.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_SHEATH);
	return;
    }

    if ( CAN_WEAR(obj,ITEM_WEAR_POUCH) )
    {
	if (!remove_obj(ch,WEAR_POUCH, fReplace) )
	    return;
	act( "{c$n wears $p as a pouch.",ch,obj,NULL,TO_ROOM);
	act( "{cYou wear $p as a pouch.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_POUCH);
	return;
    }

    if ( CAN_WEAR(obj,ITEM_WEAR_UNDER) )
    {
	if (!remove_obj(ch,WEAR_UNDER, fReplace) )
	    return;
	act( "{c$n wears $p as underclothes.",ch,obj,NULL,TO_ROOM);
	act( "{cYou wear $p as underclothes.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_UNDER);
	return;
    }
*/
    if ( CAN_WEAR(obj,ITEM_WEAR_FACE) )
    {
	if (!remove_obj(ch,WEAR_FACE, fReplace) )
	    return;
	act( "{c$n wears $p over $s face.",ch,obj,NULL,TO_ROOM);
	act( "{cYou wear $p over your face.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_FACE);
	return;
    }

    if ( CAN_WEAR(obj,ITEM_WEAR_ANGREAL) )
    {
	if (!remove_obj(ch,WEAR_ANGREAL, fReplace) )
	    return;
	act( "{c$n wears $p as an angreal.",ch,obj,NULL,TO_ROOM);
	act( "{cYou wear $p as an angreal.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_ANGREAL);
	return;
    }

    if ( CAN_WEAR(obj,ITEM_WEAR_FLOAT) )
    {
	if (!remove_obj(ch,WEAR_FLOAT, fReplace) )
	    return;
	act( "{c$n releases $p to float next to $m.",ch,obj,NULL,TO_ROOM);
	act( "{cYou release $p and it floats next to you.",ch,obj,NULL,TO_CHAR);
	equip_char(ch,obj,WEAR_FLOAT);
	return;
    }

    if ( fReplace )
	stc( "{cYou can't wear, wield, or hold that.\n\r", ch );

    return;
}



void do_wear( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cWear, wield, or hold what?\n\r", ch );
	return;
    }

    if ( !str_cmp( arg, "all" ) )
    {
	OBJ_DATA *obj_next;

	for ( obj = ch->carrying; obj != NULL; obj = obj_next )
	{
	    obj_next = obj->next_content;
	    if ( obj->wear_loc == WEAR_NONE && can_see_obj( ch, obj ) )
		wear_obj( ch, obj, FALSE );
	}
	return;
    }
    else
    {
		if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
		{
		    stc( "{cYou do not have that item.\n\r", ch );
		    return;
		}
	wear_obj( ch, obj, TRUE );
    }

    return;
}



void do_remove( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cRemove what?\n\r", ch );
	return;
    }

    if ( !str_cmp( arg, "all" ) )
    for (obj = ch->carrying; obj; obj = obj_next)
    {
    obj_next = obj->next_content;

    if (obj->wear_loc != WEAR_NONE)
    remove_obj (ch, obj->wear_loc, TRUE);

    }


    if ( ( obj = get_obj_wear( ch, arg, TRUE ) ) == NULL )
    {
	stc( "{cYou do not have that item.\n\r", ch );
	return;
    }

    remove_obj( ch, obj->wear_loc, TRUE );
    return;
}



void do_sacrifice( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *obj;
    int silver;

    /* variables for AUTOSPLIT */
    CHAR_DATA *gch;
    int members;
    char buffer[100];


    one_argument( argument, arg );

    if ( arg[0] == '\0' || !str_cmp( arg, ch->name ) )
    {
	act( "{c$n offers $mself to the Creator, who graciously declines.",
	    ch, NULL, NULL, TO_ROOM );
	stc("{cThe Creator appreciates your offer and may accept it later.\n\r", ch );
	return;
    }

    obj = get_obj_list( ch, arg, ch->in_room->contents );
    if ( obj == NULL )
    {
		stc( "{cYou can't find it.\n\r", ch );
		return;
    }

    if ( obj->item_type == ITEM_CORPSE_PC )
    {
	if (obj->contains)
        {
	   stc("{cThe Creator wouldn't like that.\n\r",ch);
	   return;
        }
    }


    if ( !CAN_WEAR(obj, ITEM_TAKE) || CAN_WEAR(obj, ITEM_NO_SAC))
    {
	act( "{c$p is not an acceptable sacrifice.", ch, obj, 0, TO_CHAR );
	return;
    }

    if (obj->in_room != NULL)
    {
	for (gch = obj->in_room->people; gch != NULL; gch = gch->next_in_room)
	    if (gch->on == obj)
	    {
		act( "{c$N appears to be using $p.",
		    ch,obj,gch,TO_CHAR);
		return;
	    }
    }

    if ( IS_SET( obj->extra_flags,ITEM_NOSELL ) )
    {
		stc("That item has been flagged nosell. Type unkeep <item> to toggle this off.\n\r",ch);
		return;
	}

    silver = 1;

    if (obj->item_type != ITEM_CORPSE_NPC && obj->item_type != ITEM_CORPSE_PC)
    	silver = UMIN(silver,obj->cost);

    if (silver == 1)
	ptc( ch, "{cThe %s gives you one copper penny for your sacrifice.\n\r",
	           is_shadow( ch ) ? "{RG{rreat {RL{rord{x" : "{WCreator{x" );
    else
    {
	sprintf(buf,"{cThe %s gives you %d copper pennies for your sacrifice.\n\r",
	           is_shadow( ch ) ? "{RG{rreat {RL{rord{x" : "{WCreator{x", silver);
	stc(buf,ch);
    }

    ch->money[0][0] += silver;

    if (IS_SET(ch->act,PLR_AUTOSPLIT) )
    { /* AUTOSPLIT code */
    	members = 0;
	for (gch = ch->in_room->people; gch != NULL; gch = gch->next_in_room )
    	{
            if (!IS_AFFECTED(gch,AFF_CHARM) && is_same_group( gch, ch ) && !IS_NPC(gch) )
            members++;
    	}

	if ( members > 1 && silver > 1)
	{
	    sprintf(buffer,"%d",silver);
	    do_function(ch, &do_split, buffer);
	}
    }

    act( "{c$n sacrifices $p to the Creator.", ch, obj, NULL, TO_ROOM );
    wiznet("$N sends up $p as an offering to the Creator.",
	   ch,obj,WIZ_SACCING,0,0);
    extract_obj( obj );
    return;
}

bool brew_cure_fatigue (OBJ_DATA *obj)
{
	if (is_name("honey", obj->material))	return TRUE;
	if (is_name("andilay", obj->material))	return TRUE;
	if (is_name("flatwort", obj->material))	return TRUE;

	return FALSE;
}

bool brew_cure_headache (OBJ_DATA *obj)
{
	if (is_name("timsin root", obj->material))	return TRUE;
	if (is_name("silverleaf", obj->material))	return TRUE;
	if (is_name("sleepwell", obj->material))	return TRUE;
	if (is_name("healall", obj->material))		return TRUE;

	return FALSE;
}

bool brew_cure_stomachache (OBJ_DATA *obj)
{
	if (is_name("red fennel", obj->material))	return TRUE;
	if (is_name("marshwhite", obj->material))	return TRUE;
	if (is_name("greenwort", obj->material))	return TRUE;
	if (is_name("healall", obj->material))		return TRUE;
	if (is_name("goosemint", obj->material))		return TRUE;
	if (is_name("goatstongue", obj->material))		return TRUE;
	if (is_name("chainleaf", obj->material))		return TRUE;
	if (is_name("bluewort", obj->material))		return TRUE;

	return FALSE;
}


bool brew_sleep_better (OBJ_DATA *obj)
{
	if (is_name("foxtail", obj->material))	return TRUE;
	if (is_name("marisin", obj->material))	return TRUE;
	if (is_name("sleepwell", obj->material))	return TRUE;
	if (is_name("greenwort", obj->material))		return TRUE;
	if (is_name("goatstongue", obj->material))		return TRUE;

	return FALSE;
}


bool brew_cure_fever (OBJ_DATA *obj)
{
	if (is_name("willow bark", obj->material))	return TRUE;
	if (is_name("worrynot", obj->material))	return TRUE;
	if (is_name("healall", obj->material))		return TRUE;
	if (is_name("feverbane", obj->material))		return TRUE;

	return FALSE;
}

bool brew_cure_health (OBJ_DATA *obj)
{
	if (is_name("willow bark", obj->material))	return TRUE;
	if (is_name("sunburst root", obj->material))	return TRUE;
	if (is_name("sleepwell", obj->material))		return TRUE;
	if (is_name("dogwort", obj->material))		return TRUE;
	if (is_name("healall", obj->material))		return TRUE;
	if (is_name("acem", obj->material))		return TRUE;
	if (is_name("blue goatflowers", obj->material))		return TRUE;
	if (is_name("five-finger", obj->material))		return TRUE;
	if (is_name("ground ivy", obj->material))		return TRUE;
	if (is_name("lionheart", obj->material))		return TRUE;
	if (is_name("mardroot", obj->material))		return TRUE;

	return FALSE;
}

bool brew_poisonous (OBJ_DATA *obj)
{
	if (is_name("tarchrot leaf", obj->material))	return TRUE;
	if (is_name("peach pit", obj->material))	return TRUE;
	if (is_name("crimsonthorn", obj->material))		return TRUE;
	if (is_name("grey fennel", obj->material))		return TRUE;
	if (is_name("asping rot", obj->material))		return TRUE;

	return FALSE;
}

void drink_brew(CHAR_DATA *ch, OBJ_DATA *obj)
{
	int level = obj->value[0];
	AFFECT_DATA *af;

	if (brew_poisonous(obj))
	{
		af = new_affect();
		af->type = gsn_poison;
		af->where	 = TO_AFFECTS;
		af->level 	 = number_fuzzy(level * ch->level / 37);
		af->duration  = UMAX(5, level / 50);
		af->location  = APPLY_CON;
		af->modifier  = number_fuzzy(level) / -44;
		af->bitvector = AFF_POISON;
		affect_join( ch, af );

		act( "{c$n looks sickly..", ch, 0, 0, TO_ROOM );
		stc( "{cYou feel sick..\n\r", ch );
	}

	if (brew_cure_health(obj))
	{
		ch->hit += number_fuzzy(level * ch->level / 23);

		if (ch->hit > get_maxhit(ch))
			ch->hit = get_maxhit(ch);

		stc("Your pain dulls.\n\r", ch);
	}

	if (brew_cure_fatigue(obj))
	{
		ch->stam += number_fuzzy(level * ch->level / 37);

		if (ch->stam > get_maxstam(ch))
			ch->stam = get_maxstam(ch);

		stc("You feel refreshed!\n\r", ch);
	}

	if (brew_cure_fever(obj))
	{
		if ((af = affect_find(ch->affected, skill_lookup("fever"))) != NULL)
		{
			stc("You overcome your fever!\n\r", ch);
		}
	}

	if (brew_sleep_better(obj))
	{
		af = new_affect();
		af->type = skill_lookup("restful");
		af->where	 = TO_AFFECTS;
		af->level 	 = number_fuzzy(level * ch->level / 37);
		af->duration  = UMAX(5, level / 50);
		af->location  = APPLY_NONE;
		af->modifier  = 0;
		af->bitvector = 0;
		affect_join( ch, af );

		stc("You feel very comfortable and restful.\n\r", ch);
	}

	if (brew_cure_stomachache(obj))
	{
		if ((af = affect_find(ch->affected, skill_lookup("stomach ache"))) != NULL)
		{
			af->duration -= UMIN(af->duration - 1, level / 65);
			af->level /= 2;
			stc("Your stomach begins to feel a little settled.\n\r", ch);
		}
	}

	if (brew_cure_headache(obj))
	{
		if ((af = affect_find(ch->affected, skill_lookup("headache"))) != NULL)
		{
			af->duration -= UMIN(af->duration - 1, level / 65);
			af->level /= 2;
			stc("Your headache begins to fade away a little bit.\n\r", ch);
		}
	}

	if (is_name("failure", obj->material))
	{
		int roll = number_percent();

		if (roll <= 10)
		{
			af = new_affect();
			af->type = skill_lookup("fever");
			af->where	 = TO_AFFECTS;
			af->level 	 = number_fuzzy(level * ch->level / 34);
			af->duration  = UMAX(5, level / 50);
			af->location  = APPLY_STR;
			af->modifier  = number_fuzzy(level) / -44;
			af->bitvector = 0;
			affect_join( ch, af );

			af = new_affect();
			af->type = skill_lookup("fever");
			af->where	 = TO_AFFECTS;
			af->level 	 = number_fuzzy(level * ch->level / 34);
			af->duration  = UMAX(5, level / 50);
			af->location  = APPLY_REF;
			af->modifier  = number_fuzzy(level) / -44;
			af->bitvector = 0;

			af = new_affect();
			af->type = skill_lookup("fever");
			af->where	 = TO_AFFECTS;
			af->level 	 = number_fuzzy(level * ch->level / 34);
			af->duration  = UMAX(5, level / 50);
			af->location  = APPLY_CON;
			af->modifier  = number_fuzzy(level) / -44;
			af->bitvector = 0;


			stc("You break out in a feverish sweat.\n\r", ch);
		}
		else if (roll <= 28)
		{
			af = new_affect();
			af->type = skill_lookup("stomach ache");
			af->where	 = TO_AFFECTS;
			af->level 	 = number_fuzzy(level * ch->level / 34);
			af->duration  = UMAX(5, level / 50);
			af->location  = APPLY_DEX;
			af->modifier  = number_fuzzy(level) / -44;
			af->bitvector = 0;
			affect_join( ch, af );

			stc("You feel a bad stomach ache coming on.\n\r", ch);
		}
		else if (roll <= 46)
		{
			af = new_affect();
			af->type = skill_lookup("headache");
			af->where	 = TO_AFFECTS;
			af->level 	 = number_fuzzy(level * ch->level / 34);
			af->duration  = UMAX(5, level / 50);
			af->location  = APPLY_INS;
			af->modifier  = number_fuzzy(level) / -44;
			af->bitvector = 0;
			affect_join( ch, af );

			stc("Your head starts throbbing with pain.\n\r", ch);
		}
		else if (roll <= 64)
		{
			ch->hit -= number_fuzzy(level * ch->level / 41);

			if (ch->hit < 1)
				ch->hit = 1;

			stc("Whew! That did NOT feel healthy!\n\r", ch);
		}
		else if (roll <= 82)
		{
			ch->stam -= number_fuzzy(level * ch->level / 153);

			if (ch->stam < 0)
				ch->stam = 0;

			stc("You begin to feel physically drained..\n\r", ch);
		}
		else
		{
			af = new_affect();
			af->type = skill_lookup("sicken");
			af->where	 = TO_AFFECTS;
			af->level 	 = 100;
			af->duration  = UMAX(5, level / 50);
			af->location  = number_range(APPLY_STR, APPLY_CON);
			af->modifier  = number_fuzzy(level) / -44;
			af->bitvector = 0;
			affect_join( ch, af );

			stc("You start feeling ill.\n\r", ch);
		}

			
	}
	
	return;
}

void do_quaff( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cQuaff what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
	stc( "{cYou do not have that potion.\n\r", ch );
	return;
    }

    if ( obj->item_type != ITEM_POTION )
    {
	stc( "{cYou can quaff only potions.\n\r", ch );
	return;
    }

    if (ch->level < obj->level)
    {
	stc( "{cThis liquid is too powerful for you to drink.\n\r",ch);
	return;
    }

    act( "{c$n quaffs $p.", ch, obj, NULL, TO_ROOM );
    act( "{cYou quaff $p.", ch, obj, NULL ,TO_CHAR );

	WAIT_STATE(ch,PULSE_VIOLENCE);

	if (obj->pIndexData->vnum == 3279)
		drink_brew(ch, obj);
	else
	{
    obj_cast_spell( obj->value[1], obj->value[0], ch, ch, NULL );
    obj_cast_spell( obj->value[2], obj->value[0], ch, ch, NULL );
    obj_cast_spell( obj->value[3], obj->value[0], ch, ch, NULL );
    obj_cast_spell( obj->value[4], obj->value[0], ch, ch, NULL );
	}

    extract_obj( obj );
    return;
}



void do_recite( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *scroll;
    OBJ_DATA *obj;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( ( scroll = get_obj_carry( ch, arg1, ch ) ) == NULL )
    {
	stc( "{cYou do not have that scroll.\n\r", ch );
	return;
    }

    if ( scroll->item_type != ITEM_SCROLL )
    {
	stc( "{cYou can recite only scrolls.\n\r", ch );
	return;
    }

    if ( ch->level < scroll->level)
    {
	stc("{cThis scroll is too complex for you to comprehend.\n\r",ch);
	return;
    }

    obj = NULL;
    if ( arg2[0] == '\0' )
    {
	victim = ch;
    }
    else
    {
	if ( ( victim = get_char_room ( ch, NULL, arg2 ) ) == NULL
	&&   ( obj    = get_obj_here  ( ch, NULL, arg2 ) ) == NULL )
	{
	    stc( "{cYou can't find it.\n\r", ch );
	    return;
	}
    }

    act( "{c$n recites $p.", ch, scroll, NULL, TO_ROOM );
    act( "{cYou recite $p.", ch, scroll, NULL, TO_CHAR );

    if (number_percent() >= 20 + get_skill(ch,gsn_scrolls) * 4/5)
    {
	stc( "{cYou mispronounce a syllable.\n\r",ch);
	check_improve(ch,gsn_scrolls,FALSE,2);
    }

    else
    {
    	obj_cast_spell( scroll->value[1], scroll->value[0], ch, victim, obj );
    	obj_cast_spell( scroll->value[2], scroll->value[0], ch, victim, obj );
    	obj_cast_spell( scroll->value[3], scroll->value[0], ch, victim, obj );
    	obj_cast_spell( scroll->value[4], scroll->value[0], ch, victim, obj );
	check_improve(ch,gsn_scrolls,TRUE,2);
    }

    extract_obj( scroll );
    return;
}



void do_brandish( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *vch;
    CHAR_DATA *vch_next;
    OBJ_DATA *staff;
    int sn;

    if ( ( staff = get_eq_char( ch, WEAR_HOLD ) ) == NULL )
    {
	stc( "{cYou hold nothing in your hand.\n\r", ch );
	return;
    }

    if ( staff->item_type != ITEM_STAFF )
    {
	stc( "{cYou can brandish only with a staff.\n\r", ch );
	return;
    }

    if ( ( sn = staff->value[3] ) < 0
    ||   sn >= MAX_SKILL
    ||   skill_table[sn].spell_fun == 0 )
    {
	bug( "Do_brandish: bad sn %d.", sn );
	return;
    }

    WAIT_STATE( ch, 2 * PULSE_VIOLENCE );

    if ( staff->value[2] > 0 )
    {
	act( "{c$n brandishes $p.", ch, staff, NULL, TO_ROOM );
	act( "{cYou brandish $p.",  ch, staff, NULL, TO_CHAR );
	if ( ch->level < staff->level
	||   number_percent() >= 20 + get_skill(ch,gsn_staves) * 4/5)
 	{
	    act( "{cYou fail to invoke $p.",ch,staff,NULL,TO_CHAR);
	    act( "{c...and nothing happens.",ch,NULL,NULL,TO_ROOM);
	    check_improve(ch,gsn_staves,FALSE,2);
	}

	else for ( vch = ch->in_room->people; vch; vch = vch_next )
	{
	    vch_next	= vch->next_in_room;

	    switch ( skill_table[sn].target )
	    {
	    default:
		bug( "Do_brandish: bad target for sn %d.", sn );
		return;

	    case TAR_IGNORE:
		if ( vch != ch )
		    continue;
		break;

	    case TAR_CHAR_OFFENSIVE:
		if ( IS_NPC(ch) ? IS_NPC(vch) : !IS_NPC(vch) )
		    continue;
		break;

	    case TAR_CHAR_DEFENSIVE:
		if ( IS_NPC(ch) ? !IS_NPC(vch) : IS_NPC(vch) )
		    continue;
		break;

	    case TAR_CHAR_SELF:
		if ( vch != ch )
		    continue;
		break;
	    }

		if (vch->pIndexData->pShop != NULL)
		{
		    stc("{cThe shopkeeper wouldn't like that.\n\r",ch);
		    return;
		}
		if (IS_SET(vch->in_room->room_flags,ROOM_SAFE))
		{
		    stc("{cNot in this room.\n\r",ch);
		    return;
		}
		if (IS_SET(ch->in_room->room_flags,ROOM_SAFE))
		{
		    stc("{cNot in this room.\n\r",ch);
		    return;
		}
		if ( IS_NPC(vch)
		&& IS_SET( vch->act, ACT_UNDEAD )
		&& ch->questmob != vch->pIndexData->vnum )
		{
			stc("Thats not your quest mob.\n\r",ch);
			return;
		}
		/* no killing healers, trainers, etc */
		if (IS_SET(vch->act,ACT_TRAIN)
		||  IS_SET(vch->act,ACT_PRACTICE)
        ||  (vch->pIndexData->pShop != NULL)
		||  IS_SET(vch->act,ACT_IS_HEALER)
		||  IS_SET(vch->act,ACT_IS_CHANGER))
		{
		    stc("{cI don't think the Creator would approve.\n\r",ch);
		    return;
		}

	    obj_cast_spell( staff->value[3], staff->value[0], ch, vch, NULL );
	    check_improve(ch,gsn_staves,TRUE,2);
	}
    }

    if ( --staff->value[2] <= 0 )
    {
	act( "{c$n's $p blazes bright and is gone.", ch, staff, NULL, TO_ROOM );
	act( "{cYour $p blazes bright and is gone.", ch, staff, NULL, TO_CHAR );
	extract_obj( staff );
    }

    return;
}



void do_zap( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *wand;
    OBJ_DATA *obj;

    one_argument( argument, arg );
    if ( arg[0] == '\0' && !is_fighting(ch) )
    {
	stc( "{cZap whom or what?\n\r", ch );
	return;
    }

    if ( ( wand = get_eq_char( ch, WEAR_HOLD ) ) == NULL )
    {
	stc( "{cYou hold nothing in your hand.\n\r", ch );
	return;
    }

    if ( wand->item_type != ITEM_WAND )
    {
	stc( "{cYou can zap only with a wand.\n\r", ch );
	return;
    }

    obj = NULL;
    if ( arg[0] == '\0' )
    {
	if ( is_fighting(ch) )
	{
	    victim = ch->advancing_on;
	}
	else
	{
	    stc( "{cZap whom or what?\n\r", ch );
	    return;
	}
    }
    else
    {
	if ( ( victim = get_char_room ( ch, NULL, arg ) ) == NULL
	&&   ( obj    = get_obj_here  ( ch, NULL, arg ) ) == NULL )
	{
	    stc( "{cYou can't find it.\n\r", ch );
	    return;
	}
    }

    WAIT_STATE( ch, 2 * PULSE_VIOLENCE );

    if ( wand->value[2] > 0 )
    {
	if ( victim != NULL )
	{
		if (victim->pIndexData->pShop != NULL)
		{
		    stc("{cThe shopkeeper wouldn't like that.\n\r",ch);
		    return;
		}
		if (IS_SET(victim->in_room->room_flags,ROOM_SAFE))
		{
		    stc("{cNot in this room.\n\r",ch);
		    return;
		}
		if (IS_SET(ch->in_room->room_flags,ROOM_SAFE))
		{
		    stc("{cNot in this room.\n\r",ch);
		    return;
		}
		if ( IS_NPC(victim)
		&& IS_SET( victim->act, ACT_UNDEAD )
		&& ch->questmob != victim->pIndexData->vnum )
		{
			stc("Thats not your quest mob.\n\r",ch);
			return;
		}

		/* no killing healers, trainers, etc */
		if (IS_SET(victim->act,ACT_TRAIN)
		||  IS_SET(victim->act,ACT_PRACTICE)
        ||  (victim->pIndexData->pShop != NULL)
		||  IS_SET(victim->act,ACT_IS_HEALER)
		||  IS_SET(victim->act,ACT_IS_CHANGER))
		{
		    stc("{cI don't think the Creator would approve.\n\r",ch);
		    return;
		}
	    act( "{c$n zaps $N with $p.", ch, wand, victim, TO_NOTVICT );
	    act( "{cYou zap $N with $p.", ch, wand, victim, TO_CHAR );
	    act( "{c$n zaps you with $p.",ch, wand, victim, TO_VICT );
	}
	else
	{
	    act( "{c$n zaps $P with $p.", ch, wand, obj, TO_ROOM );
	    act( "{cYou zap $P with $p.", ch, wand, obj, TO_CHAR );
	}

 	if (ch->level < wand->level
	||  number_percent() >= 20 + get_skill(ch,gsn_wands) * 4/5)
	{
	    act( "{cYour efforts with $p produce only smoke and sparks.",
		 ch,wand,NULL,TO_CHAR);
	    act( "{c$n's efforts with $p produce only smoke and sparks.",
		 ch,wand,NULL,TO_ROOM);
	    check_improve(ch,gsn_wands,FALSE,2);
	}
	else
	{
	    obj_cast_spell( wand->value[3], wand->value[0], ch, victim, obj );
	    check_improve(ch,gsn_wands,TRUE,2);
	}
    }

    if ( --wand->value[2] <= 0 )
    {
	act( "{c$n's $p explodes into fragments.", ch, wand, NULL, TO_ROOM );
	act( "{cYour $p explodes into fragments.", ch, wand, NULL, TO_CHAR );
	extract_obj( wand );
    }

    return;
}



void do_steal( CHAR_DATA *ch, char *argument )
{
    char buf  [MAX_STRING_LENGTH];
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    int percent;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
	stc( "{cSteal what from whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch, NULL, arg2 ) ) == NULL )
    {
	stc( "{cThey aren't here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	stc( "{cThat's pointless.\n\r", ch );
	return;
    }

    if (is_safe(ch,victim))
	return;

    if ( IS_NPC(victim)
	  && victim->position == POS_FIGHTING)
    {
	stc(  "{cKill stealing is not permitted.\n\r"
		       "You'd better not -- you might get hit.\n\r",ch);
	return;
    }

    WAIT_STATE( ch, skill_table[gsn_steal].beats );
    percent  = number_percent();

    if (!IS_AWAKE(victim))
    	percent -= 10;
    else if (!can_see(victim,ch))
    	percent += 25;
    else
	percent += 50;

    if ( ((ch->level + 7 < victim->level || ch->level -7 > victim->level)
    && !IS_NPC(victim) && !IS_NPC(ch) )
    || ( !IS_NPC(ch) && percent > get_skill(ch,gsn_steal))
    || ( !IS_NPC(ch) && !is_clan(ch)) )
    {
	/*
	 * Failure.
	 */
	stc( "{cOops.\n\r", ch );
	affect_strip(ch,gsn_sneak);
	REMOVE_BIT(ch->affected_by,AFF_SNEAK);

	act( "{c$n tried to steal from you.\n\r", ch, NULL, victim, TO_VICT    );
	act( "{c$n tried to steal from $N.\n\r",  ch, NULL, victim, TO_NOTVICT );
	/*switch(number_range(0,3))
	{
	case 0 :
	   sprintf( buf, "{c%s is a lousy thief!", ch->name );
	   break;
        case 1 :
	   sprintf( buf, "{c%s couldn't rob %s way out of a paper bag!",
		    ch->name,(ch->sex == 2) ? "her" : "his");
	   break;
	case 2 :
	    sprintf( buf,"{c%s tried to rob me!",ch->name );
	    break;
	case 3 :
	    sprintf(buf,"{cKeep your hands out of there, %s!",ch->name);
	    break;
        }*/
        if (!IS_AWAKE(victim))
            do_function(victim, &do_wake, "");
	/*if (IS_AWAKE(victim))*/
	    /*do_function(victim, &do_yell, buf );*/
	if ( !IS_NPC(ch) )
	{
	    if ( IS_NPC(victim) )
	    {
	        check_improve(ch,gsn_steal,FALSE,2);
		multi_hit( victim, ch, TYPE_UNDEFINED );
	    }
	    else
	    {
		sprintf(buf,"{c$N tried to steal from %s.",victim->name);
		wiznet(buf,ch,NULL,WIZ_FLAGS,0,0);
		if ( !IS_SET(ch->act, PLR_THIEF) )
		{
		    SET_BIT(ch->act, PLR_THIEF);
		    stc( "{c*** {CYou are now a THIEF!!{c ***\n\r", ch );
		    //save_char_obj( ch );
		    save_char_obj(ch, FALSE, FALSE);
		}
	    }
	}

	return;
    }

    if ( !str_cmp( arg1, "coin"  )
    ||   !str_cmp( arg1, "coins" )
    ||   !str_cmp( arg1, "gold"  )
    ||	 !str_cmp( arg1, "silver"))
    {
	int gold, silver;

	gold = victim->money[3][0] * number_range(1, ch->level) / MAX_LEVEL;
	silver = victim->money[2][0] * number_range(1,ch->level) / MAX_LEVEL;
	if ( gold <= 0 && silver <= 0 )
	{
	    stc( "{cYou couldn't get any coins.\n\r", ch );
	    return;
	}

	ch->money[3][0]     	+= gold;
	ch->money[2][0]   	+= silver;
	victim->money[3][0] 	-= silver;
	victim->money[2][0] 	-= gold;
	if (silver <= 0)
	    sprintf( buf, "{cBingo!  You got %d gold coins.\n\r", gold );
	else if (gold <= 0)
	    sprintf( buf, "{cBingo!  You got %d silver coins.\n\r",silver);
	else
	    sprintf(buf, "{cBingo!  You got %d silver and %d gold coins.\n\r",
		    silver,gold);

	stc( buf, ch );
	check_improve(ch,gsn_steal,TRUE,2);
	return;
    }

    if ( ( obj = get_obj_carry( victim, arg1, ch ) ) == NULL )
    {
	stc( "{cYou can't find it.\n\r", ch );
	return;
    }

    if ( !can_drop_obj( ch, obj )
    ||   IS_SET(obj->extra_flags, ITEM_INVENTORY)
    ||   obj->level > ch->level )
    {
	stc( "{cYou can't pry it away.\n\r", ch );
	return;
    }

    if ( ch->carry_number + get_obj_number( obj ) > can_carry_n( ch ) )
    {
	stc( "{cYou have your hands full.\n\r", ch );
	return;
    }

    if ( ch->carry_weight + get_obj_weight( obj ) > can_carry_w( ch ) )
    {
	stc( "{cYou can't carry that much weight.\n\r", ch );
	return;
    }

    obj_from_char( obj );
    obj_to_char( obj, ch );
    act( "{cYou pocket $p.",ch,obj,NULL,TO_CHAR);
    check_improve(ch,gsn_steal,TRUE,2);
    stc( "{cGot it!\n\r", ch );
    return;
}



/*
 * Shopping commands.
 */
CHAR_DATA *find_keeper( CHAR_DATA *ch )
{
    /*char buf[MAX_STRING_LENGTH];*/
    CHAR_DATA *keeper;
    SHOP_DATA *pShop;

    pShop = NULL;
    for ( keeper = ch->in_room->people; keeper; keeper = keeper->next_in_room )
    {
	if ( IS_NPC(keeper) && (pShop = keeper->pIndexData->pShop) != NULL )
	    break;
    }

    if ( pShop == NULL )
    {
	stc( "{cYou can't do that here.\n\r", ch );
	return NULL;
    }

    /*
     * Undesirables.
     *
    if ( !IS_NPC(ch) && IS_SET(ch->act, PLR_KILLER) )
    {
	do_function(keeper, &do_say, "Killers are not welcome!");
	sprintf(buf, "%s the KILLER is over here!\n\r", ch->name);
	do_function(keeper, &do_yell, buf );
	return NULL;
    }

    if ( !IS_NPC(ch) && IS_SET(ch->act, PLR_THIEF) )
    {
	do_function(keeper, &do_say, "Thieves are not welcome!");
	sprintf(buf, "%s the THIEF is over here!\n\r", ch->name);
	do_function(keeper, &do_yell, buf );
	return NULL;
    }
	*/
    /*
     * Shop hours.
     */
    if ( time_info.hour < pShop->open_hour )
    {
	do_function(keeper, &do_say, "{cSorry, I am closed. Come back later.");
	return NULL;
    }

    if ( time_info.hour > pShop->close_hour )
    {
	do_function(keeper, &do_say, "{cSorry, I am closed. Come back tomorrow.");
	return NULL;
    }

    /*
     * Invisible or hidden people.
     */
    if ( !can_see( keeper, ch ) )
    {
	do_function(keeper, &do_say, "{cI don't trade with folks I can't see.");
	return NULL;
    }

    return keeper;
}

/* insert an object at the right spot for the keeper */
void obj_to_keeper( OBJ_DATA *obj, CHAR_DATA *ch )
{
    OBJ_DATA *t_obj, *t_obj_next;

    /* see if any duplicates are found */
    for (t_obj = ch->carrying; t_obj != NULL; t_obj = t_obj_next)
    {
	t_obj_next = t_obj->next_content;

	if (obj->pIndexData == t_obj->pIndexData
	&&  !str_cmp(obj->short_descr,t_obj->short_descr))
	{
	    /* if this is an unlimited item, destroy the new one */
	    if (IS_OBJ_STAT(t_obj,ITEM_INVENTORY))
	    {
		extract_obj(obj);
		return;
	    }
	    obj->cost = t_obj->cost; /* keep it standard */
	    break;
	}
    }

    if (t_obj == NULL)
    {
	obj->next_content = ch->carrying;
	ch->carrying = obj;
    }
    else
    {
	obj->next_content = t_obj->next_content;
	t_obj->next_content = obj;
    }

    obj->carried_by      = ch;
    obj->in_room         = NULL;
    obj->in_obj          = NULL;
    ch->carry_number    += get_obj_number( obj );
    ch->carry_weight    += get_obj_weight( obj );
}

/* get an object from a shopkeeper's list */
OBJ_DATA *get_obj_keeper( CHAR_DATA *ch, CHAR_DATA *keeper, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;

    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = keeper->carrying; obj != NULL; obj = obj->next_content )
    {
        if ( obj->wear_loc == WEAR_NONE
        &&  is_name( arg, obj->name ) )
        {
            if ( ++count == number )
                return obj;

		    /* skip other objects of the same name */
		    while ( obj->next_content != NULL
		    && obj->pIndexData == obj->next_content->pIndexData
		    && !str_cmp(obj->short_descr,obj->next_content->short_descr ) )
				obj = obj->next_content;
        }
    }

    return NULL;
}

int get_cost( CHAR_DATA *keeper, OBJ_DATA *obj, bool fBuy )
{
    SHOP_DATA *pShop;
    int cost;

    if ( obj == NULL )
		return -1;

	if ( ( pShop = keeper->pIndexData->pShop ) == NULL )
		return -2;

	if ( obj->item_type == ITEM_CRAFTING_MANUAL )
		return 2500;

	if ( obj->pIndexData->vnum == 9 )
		return 5000;

	if ( obj->pIndexData->vnum == 18 )
		return 500;

	if ( obj->item_type == ITEM_CRAFTING_MATERIAL )
		return 500;


    if ( fBuy )
    {
		cost = obj->cost;
		return cost;
    }
    else
    {
		OBJ_DATA *obj2;
		int itype;

		cost = obj->cost;
		return cost;

		cost = 0;
		for ( itype = 0; itype < MAX_TRADE; itype++ )
		{
		    if ( obj->item_type == pShop->buy_type[itype] )
		    {
			/*cost = obj->cost * pShop->profit_sell / 100;*/
                        cost = obj->cost;
			break;
		    }
		}

		if (!IS_OBJ_STAT(obj,ITEM_SELL_EXTRACT))
		{
		    for ( obj2 = keeper->carrying; obj2; obj2 = obj2->next_content )
		    {
		    	if ( obj->pIndexData == obj2->pIndexData
				&&   !str_cmp(obj->short_descr,obj2->short_descr) )
				{
		 	    if (IS_OBJ_STAT(obj2,ITEM_INVENTORY))
					cost /= 2;
			    else
                   	cost = cost * 3 / 4;
				}
		    }
		}
    }

    if ( obj->item_type == ITEM_STAFF || obj->item_type == ITEM_WAND )
    {
	if (obj->value[1] == 0)
	    cost /= 4;
	else
	    cost = cost * obj->value[2] / obj->value[1];
    }

    return cost;
}



void do_buy( CHAR_DATA *ch, char *argument )
{
	char buf[MAX_STRING_LENGTH];
	int cost,roll;
	bool matrim;

	if ( gevent->running && gevent->num == 10 )
		matrim = TRUE;

	if ( argument[0] == '\0' )
	{
		stc( "{cBuy what?\n\r", ch );
		return;
	}

	if ( IS_SET(ch->in_room->room_flags, ROOM_MOUNT_SHOP) )
	{
		do_buy_mount( ch, argument );
		return;
	}

	if ( IS_SET(ch->in_room->room_flags, ROOM_PET_SHOP) )
	{
		char arg[MAX_INPUT_LENGTH];
		CHAR_DATA *pet;
		ROOM_INDEX_DATA *pRoomIndexNext;
		ROOM_INDEX_DATA *in_room;

		smash_tilde(argument);

		if ( IS_NPC(ch) )
			return;

		argument = one_argument(argument,arg);

		pRoomIndexNext = get_room_index( ch->in_room->vnum + 1 );
		if ( pRoomIndexNext == NULL )
		{
			bug( "Do_buy: bad pet shop at vnum %d.", ch->in_room->vnum );
			stc( "Sorry, you can't buy a pet here.\n\r", ch );
			return;
		}

		in_room     = ch->in_room;
		ch->in_room = pRoomIndexNext;
		pet         = get_char_room( ch, NULL, arg );
		ch->in_room = in_room;

		if ( pet == NULL || !IS_SET(pet->act, ACT_PET) )
		{
			stc( "Sorry, you can't buy that here.\n\r", ch );
			return;
		}

		if ( ch->pet != NULL )
		{
			stc( "{cYou already own a pet.\n\r",ch);
			return;
		}

	 	cost = 10 * pet->level * pet->level;

		if ( !check_coins( ch, cost*100,0,0 ) )
		{
			stc("You can't afford it.", ch);
			return;
		}

		if ( ch->level < pet->level )
		{
			stc("{cYou're not powerful enough to master this pet.\n\r", ch );
			return;
		}

		/* haggle */
		roll = number_percent();
		if ( matrim )
			roll /= 2;
		if ( matrim || (!IS_NPC(ch) && roll < (ch->pcdata->learned[gsn_haggle]+get_curr_stat(ch,STAT_CHA)/2) ))
		{
			cost -= (cost / 2) * (roll / 100);

			if ( matrim )
				cost -= ( cost / 2 );

			sprintf(buf,"{cYou haggle the price down to %d coins%s.\n\r",cost, matrim ? " with the help of Matrim's Luck" : "");
			stc(buf,ch);
			check_improve(ch,gsn_haggle,TRUE,4);
		}

		deduct_cost(ch,cost*100,0);
		pet			= create_mobile( pet->pIndexData );
		SET_BIT(pet->act, ACT_PET);
		SET_BIT(pet->affected_by, AFF_CHARM);
		pet->penalty_flags = PENAL_NOTELL|PENAL_NOSHOUT|PENAL_NOCHANNELS;

		argument = one_argument( argument, arg );

		if ( arg[0] != '\0' )
		{
		    sprintf( buf, "%s %s", pet->name, arg );
		    free_string( pet->name );
		    pet->name = str_dup( buf );
		}

		sprintf( buf, "{c%sA neck tag says 'I belong to %s'.\n\r", pet->description, ch->name );
		free_string( pet->description );
		pet->description = str_dup( buf );

		char_to_room( pet, ch->in_room );
		add_follower( pet, ch );
		pet->leader = ch;
		ch->pet = pet;
		stc( "{cEnjoy your pet.\n\r", ch );
		act( "{c$n bought $N as a pet.", ch, NULL, pet, TO_ROOM );
		return;
	}
	else
	{
		CHAR_DATA *keeper;
		OBJ_DATA *obj,*t_obj;
		char arg[MAX_INPUT_LENGTH];
		char buf[MIL];
		int number, count = 1;

		if ( ( keeper = find_keeper( ch ) ) == NULL )
		    return;

		number = mult_argument(argument,arg);
		obj  = get_obj_keeper( ch,keeper, arg );
		cost = get_cost( keeper, obj, TRUE );

		if (number < 1 || number > 99)
		{
			sprintf( buf, "%s Get real!", ch->name );
			keeper_say( keeper, buf );
			return;
		}
		if ( cost == -1 )
		{
			sprintf( buf, "%s I can't seem to find that in my shop.", ch->name );
			keeper_say( keeper, buf );
			return;
		}
		if ( cost == -2 )
		{
			sprintf( buf, "%s My shop seems to be broken right now...", ch->name );
			keeper_say( keeper, buf );
			return;
		}
		if ( cost <= 0 )
		{
			sprintf( buf, "%s I haven't decided on a cost for that yet...", ch->name );
			keeper_say( keeper, buf );
			return;
		}

		if (IS_SET(obj->extra_flags,ITEM_REMORT)
		&& ch->pcdata->rebirth < 1)
		{
			sprintf( buf, "%s This item is only for those who have been reborn.", ch->name );
			keeper_say( keeper, buf );
			return;
		}

	        if ( !is_obj_char_clan( ch, obj ) && is_obj_clan(obj))
		{
			sprintf( buf, "%s This item is only for %s.", ch->name, obj->guild );
			keeper_say( keeper, buf );
			return;
		}

		if (!IS_OBJ_STAT(obj,ITEM_INVENTORY))
		{
		    for (t_obj = obj->next_content; count < number && t_obj != NULL; t_obj = t_obj->next_content)
		    {
		    	if (t_obj->pIndexData == obj->pIndexData
		    	&&  !str_cmp(t_obj->short_descr,obj->short_descr))
			    count++;
		    	else
			    break;
		    }

		    if (count < number)
		    {
				sprintf( buf, "%s I don't have that many in stock.", ch->name );
				keeper_say( keeper, buf );
		    	return;
		    }
		}

		if ( !check_coins( ch, cost*10*number, 0, 0 ) )
		{
		    if (number > 1)
		    {
				sprintf( buf, "%s You can't afford to buy that many.", ch->name );
				keeper_say( keeper, buf );
			}
		    else
		    {
			sprintf( buf, "%s You can't afford to buy %s.", ch->name, check_dye(obj, "", obj->dye_vnum));
			keeper_say( keeper, buf );
		    }
		    return;
		}

		if (ch->carry_number +  number * get_obj_number(obj) > can_carry_n(ch))
		{
			sprintf( buf, "%s You can't carry that many items.", ch->name );
			keeper_say( keeper, buf );
		    return;
		}

		if ( ch->carry_weight + number * get_obj_weight(obj) > can_carry_w(ch))
		{
			sprintf( buf, "%s You can't carry that much weight.", ch->name );
			keeper_say( keeper, buf );
		    return;
		}

		/* haggle */
		roll = number_percent();
		if (!IS_OBJ_STAT(obj,ITEM_SELL_EXTRACT) && roll < (ch->pcdata->learned[gsn_haggle]+get_curr_stat(ch,STAT_CHA)/2) )
		{
		    cost -= obj->cost / 2 * roll / 95;
		    act( "You haggle with $N.",ch,NULL,keeper,TO_CHAR);
		    check_improve(ch,gsn_haggle,TRUE,4);
		}

		if (number > 1)
		{
		    sprintf(buf,"$n buys $p[%d].",number);
		    act(buf,ch,obj,NULL,TO_ROOM);
		    sprintf(buf,"You buy $p[%d] for %d silver pennies.",number,cost * number);
		    act(buf,ch,obj,NULL,TO_CHAR);
		}
		else
		{
		    act( "$n buys $p.", ch, obj, NULL, TO_ROOM );
		    sprintf(buf,"You buy $p for %d silver pennies.",cost);
		    act( buf, ch, obj, NULL, TO_CHAR );
		}
		deduct_cost(ch,cost * number*10,0);

		for (count = 0; count < number; count++)
		{
		    if ( IS_SET( obj->extra_flags, ITEM_INVENTORY ) )
		    	t_obj = create_object( obj->pIndexData, obj->level );
		    else
		    {
				t_obj = obj;
				obj = obj->next_content;
		    	obj_from_char( t_obj );
		    }

		    if (t_obj->timer > 0 && !IS_OBJ_STAT(t_obj,ITEM_HAD_TIMER))
		    	t_obj->timer = 0;

			if (t_obj->pIndexData->vnum == 9)
			{
				free_string(t_obj->owner);
				t_obj->owner = str_dup(ch->name);
			}

		    REMOVE_BIT(t_obj->extra_flags,ITEM_HAD_TIMER);
		    obj_to_char( t_obj, ch );
		    if (cost < t_obj->cost)
		    	t_obj->cost = cost;
		}




    }
}



void do_list( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

	if (get_slot_shop(ch->in_room->vnum) > 0)
	{
		shop_list(ch, get_slot_shop(ch->in_room->vnum));
		return;
	}

	if (ch->in_room->vnum == 34713 && get_shop_id(ch) > 0)
	{
		shop_list(ch, get_shop_id(ch));
		return;
	}


    if ( IS_SET(ch->in_room->room_flags, ROOM_PET_SHOP)
    || IS_SET(ch->in_room->room_flags, ROOM_MOUNT_SHOP) )
    {
		ROOM_INDEX_DATA *pRoomIndexNext;
		CHAR_DATA *pet;
		bool found;

		pRoomIndexNext = get_room_index( ch->in_room->vnum + 1 );

		if ( pRoomIndexNext == NULL )
		{
	    bug( "Do_list: bad pet shop at vnum %d.", ch->in_room->vnum );
	    stc( "{cYou can't do that here.\n\r", ch );
	    return;
		}

		found = FALSE;
		for ( pet = pRoomIndexNext->people; pet; pet = pet->next_in_room )
		{
		    if ( IS_SET(pet->act, ACT_PET)
			|| IS_SET(pet->act, ACT_MOUNT) )
		    {
				if ( !found )
				{
				    found = TRUE;
				    if (IS_SET(pet->act, ACT_PET))
				        send_to_char( "Pets for sale:\n\r", ch );
				    else if (IS_SET(pet->act, ACT_MOUNT))
				    {
					stc("When a Mount is purchased, it will be created with your experience,\n\r",ch);
					stc("The costs to purchase a Mount is in silver marks.\n\r",ch);
			        send_to_char( "Mounts for sale:\n\r", ch );
					}
				}
				if (IS_SET(pet->act,ACT_PET))
				{
				sprintf( buf, "{C[{w%10d{C]{c %8d - %s\n\r",
			    pet->level * 1000,
			    10 * pet->level * pet->level,
			    pet->short_descr );
				stc( buf, ch );
				}
				else if (IS_SET(pet->act,ACT_MOUNT))
				{
				sprintf( buf, "{C[{w%10d{C]{c %8d - %s\n\r",
			    pet->level * 1000,
			    100 * ch->level,
			    pet->short_descr );
				stc( buf, ch );
				}
		    }
		}
		if ( !found )
		{
	        if (IS_SET(ch->in_room->room_flags, ROOM_PET_SHOP) )
	        {
            stc( "Sorry, we're out of pets right now.\n\r", ch );
			return;
			}
	        else
	        {
            stc( "Sorry, we're out of mounts right now.\n\r", ch );
			return;
			}
	    }
	}
    else
    {
		CHAR_DATA *keeper;
		OBJ_DATA *obj;
		int cost,count;
		bool found;
		char arg[MAX_INPUT_LENGTH];

		if ( ( keeper = find_keeper( ch ) ) == NULL )
		{
		    return;
		}

	    one_argument(argument,arg);

		found = FALSE;
		for ( obj = keeper->carrying; obj; obj = obj->next_content )
		{
		    if ( obj->wear_loc == WEAR_NONE
		    &&   can_see_obj( ch, obj )
		    &&   ( cost = get_cost( keeper, obj, TRUE ) ) > 0
		    &&   ( arg[0] == '\0'
		    ||  is_name(arg,obj->name) ))
		    {
				if ( !found )
				{
				    found = TRUE;
					stc( "\n\r", ch );
				    stc( "{C[{w   Silver Where Worn{C ]{c Item{x\n\r", ch );
				}

				if (IS_OBJ_STAT(obj,ITEM_INVENTORY))
			    sprintf(buf,"{C[{w%10d %12s {C]{c %s{x\n\r",
				cost,
				wear_bit_name(obj->wear_flags),
				check_dye(obj, "", obj->dye_vnum));
				else
				{
				    count = 1;

				    while (obj->next_content != NULL
				    && obj->pIndexData == obj->next_content->pIndexData
				    && !str_cmp(obj->short_descr,obj->next_content->short_descr))
				    {
						obj = obj->next_content;
						count++;
				    }
				    sprintf(buf,"{C[{c%10d %2d {C]{c %s{x\n\r",
					cost,count,check_dye(obj, "", obj->dye_vnum));
				}
				stc( buf, ch );
		    }
		}

		if ( !found )
		    stc( "{cYou can't buy anything here.\n\r", ch );
		return;
    }
}

void do_sell( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *keeper;
    OBJ_DATA *obj;
    int cost,roll;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cSell what?\n\r", ch );
	return;
    }

    if ( ( keeper = find_keeper( ch ) ) == NULL )
	return;

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
	act( "{c$n tells you 'You don't have that item'.",
	    keeper, NULL, ch, TO_VICT );
	ch->reply = keeper;
	return;
    }
    if ( IS_SET( obj->extra_flags,ITEM_NOSELL ) )
    {
		stc("That item has been flagged nosell. Type unkeep <item> to toggle this off.\n\r",ch);
		return;
	}

    if ( !can_drop_obj( ch, obj ) )
    {
	stc( "{cYou can't let go of it.\n\r", ch );
	return;
    }

    if (!can_see_obj(keeper,obj))
    {
	act( "{c$n doesn't see what you are offering.",keeper,NULL,ch,TO_VICT);
	return;
    }

    if (( ( cost = get_cost( keeper, obj, FALSE ) ) <= 0 )
    && (!OBJ_VNUM_SKIN))
    {
	act( "{c$n looks uninterested in $p.", keeper, obj, ch, TO_VICT );
	return;
    }
	/*HELP NEEDED the 10* was 100* it seems there are rests of old code lingering about...*/
    if (( cost > (keeper->money[2][0] + 10 * keeper->money[3][0]) )
    && (!OBJ_VNUM_SKIN))
    {
	act( "{c$n tells you 'I'm afraid I don't have enough wealth to buy $p.",
	    keeper,obj,ch,TO_VICT);
	return;
    }
	cost /= 2;
	cost *= 11 / 9;
    act( "{c$n sells $p.", ch, obj, NULL, TO_ROOM );
    /* haggle */
    roll = number_percent();
    if (!IS_OBJ_STAT(obj,ITEM_SELL_EXTRACT) && roll <
	(ch->pcdata->learned[gsn_haggle]+get_curr_stat(ch,STAT_CHA)/2) )
    {
        stc( "{cYou haggle with the shopkeeper.\n\r",ch);
        cost += obj->cost / 2 * roll / 100;
        cost = UMIN(cost,95 * get_cost(keeper,obj,TRUE) / 100);
	cost = UMIN(cost,(keeper->money[2][0] + 10 * keeper->money[3][0]));
        check_improve(ch,gsn_haggle,TRUE,4);
    }
    sprintf( buf, "{cYou sell $p for %d silver marks and %d gold marks piece%s.",
	cost % 10, cost/10, cost == 1 ? "" : "s" );
    act( buf, ch, obj, NULL, TO_CHAR );
    ch->money[3][0]     += cost/10;
    ch->money[2][0] 	 += cost % 10;
    deduct_cost(keeper,cost*100,0);
    if ( keeper->money[3][0] < 0 )
	keeper->money[3][0] = 0;
    if ( keeper->money[2][0] < 0)
	keeper->money[2][0] = 0;

    if ( obj->item_type == ITEM_TRASH || IS_OBJ_STAT(obj,ITEM_SELL_EXTRACT))
    {
	extract_obj( obj );
    }
    else
    {
	obj_from_char( obj );
	if (obj->timer)
	    SET_BIT(obj->extra_flags,ITEM_HAD_TIMER);
	else
	    obj->timer = number_range(50,100);
	obj_to_keeper( obj, keeper );
    }

    return;
}



void do_value( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int cost;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cValue what?\n\r", ch );
	return;
    }

    /*if ( ( keeper = find_keeper( ch ) ) == NULL )
	return;*/

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
		act( "{cYou don't have that item.", ch, NULL, NULL, TO_CHAR );
		return;
    }

	if (!IS_SET(ch->in_room->room_flags, ROOM_PAWNSHOP))
	{
		stc("You are not in a pawnshop.\n\r",ch);
		return;
	}

    /*if (!can_see_obj(keeper,obj))
    {
        act( "{c$n doesn't see what you are offering.",keeper,NULL,ch,TO_VICT);
        return;
    }*/

    if ( !can_drop_obj( ch, obj ) )
    {
		stc( "{cYou can't let go of it.\n\r", ch );
		return;
    }

    /*if ( ( cost = get_cost( keeper, obj, FALSE ) ) <= 0 )*/
    if ( obj->cost <= 0 )
    {
		act( "{c$t has no value.", ch, obj, NULL, TO_CHAR );
		return;
    }

    cost = obj->cost / 10;
    if ( cost < 1 ) cost = 1;
	cost = number_range(cost*9/11,cost*11/9);
	/*cost - (cost/100) * 100, cost/100 );
    act( buf, keeper, obj, ch, TO_VICT );
    ch->reply = keeper;*/

    return;
}

void do_draw( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *quiver;
	OBJ_DATA *arrow;
	OBJ_DATA *bow;
	OBJ_DATA *bow2;
    bool hasbow = FALSE;

    if ( ( quiver = get_eq_char( ch, WEAR_SHOULDERS ) ) == NULL )
    {
	stc( "You aren't wearing a quiver where you can get to it.\n\r", ch );
	return;
    }

    if ( quiver->item_type != ITEM_QUIVER )
    {
	stc( "You can only draw arrows from a quiver.\n\r", ch );
	return;
    }
    bow = get_eq_char(ch, WEAR_WIELD);
    bow2 = get_eq_char(ch, WEAR_SECONDARY);
   	if (bow != NULL || bow2 != NULL )
   	{
   		if ( bow != NULL )
   		{
    	if ( bow->value[0] == WEAPON_SHORTBOW
   		|| bow->value[0] == WEAPON_LONGBOW
    	|| bow->value[0] == WEAPON_RECURVEDBOW
    	|| bow->value[0] == WEAPON_COMPOSITEBOW
	    || bow->value[0] == WEAPON_CROSSBOW )
		    hasbow = TRUE;
		}
		else if ( bow2 != NULL )
		{
    	if ( bow2->value[0] == WEAPON_SHORTBOW
   		|| bow2->value[0] == WEAPON_LONGBOW
    	|| bow2->value[0] == WEAPON_RECURVEDBOW
    	|| bow2->value[0] == WEAPON_COMPOSITEBOW
	    || bow2->value[0] == WEAPON_CROSSBOW )
		    hasbow = TRUE;
		}
		else
			hasbow = FALSE;
	}
	else
	{
		hasbow = FALSE;
	}
	if ( !hasbow )
	{
		stc("You need a bow, bub.\n\r",ch);
		return;
	}

    if ( quiver->value[0] > 0 )
    {
	if (IS_SET( ch->op_flags, OP_CHANNELER))
		WAIT_STATE( ch, PULSE_VIOLENCE );
	else
		WAIT_STATE( ch, PULSE_VIOLENCE / 2 );
	if ( bow != NULL )
	{
	act( "{c$n draws an arrow from $p and loads $T.", ch, quiver, bow->short_descr, TO_ROOM );
	act( "{cYou draw an arrow from $p and load $T.", ch, quiver, bow->short_descr, TO_CHAR );
	}
	else if ( bow2 != NULL )
	{
	act( "{c$n draws an arrow from $p and loads $T.", ch, quiver, bow2->short_descr, TO_ROOM );
	act( "{cYou draw an arrow from $p and load $T.", ch, quiver, bow2->short_descr, TO_CHAR );
	}
	else
	{
	stc("How did we get this far?\n\r",ch);
	return;
	}

	arrow = create_object(get_obj_index(OBJ_VNUM_ARROW), 0);
	arrow->value[1] = quiver->value[1];
	arrow->value[2] = quiver->value[2];
	arrow->level    = quiver->level;
	obj_to_char(arrow,ch);
	quiver->value[0] -= quiver->value[1];


    if ( quiver->value[0] <= 0 )
    {
	act( "{cYour $p is now out of arrows, you need to find another one.", ch, quiver, NULL, TO_CHAR );
	extract_obj(quiver);
    }

    return;
	}
}

void do_fire( CHAR_DATA *ch, char *argument )
{
    char arg1[MIL];
    char arg2[MIL];
    char arg3[MIL];
    char arg4[MIL];
    CHAR_DATA *victim = NULL;
    OBJ_DATA *arrow;
	OBJ_DATA *bow;
    bool hasbow = FALSE, twoFire = FALSE;
    int dam ,diceroll, arr_dmg = 0, bow_bonus = 0;
    int sec_att = get_skill(ch,gsn_second_attack)/2;
	int third_att = get_skill(ch,gsn_third_attack)/3;
	int archer = get_skill(ch,gsn_archery);

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	argument = one_argument( argument, arg4 );

	if ( arg1[0] == '\0' || arg2[0] == '\0' || arg2[0] == '\0' )
	{
		stc("Syntax:\n\r",ch);
		stc("fire <arrow> at <victim>\n\r",ch);
		stc("Example:\n\r",ch);
		stc("fire 'barbed arrow' at trolloc\n\r",ch);
		stc("If you have archery, you can fire two arrows at a time.\n\r",ch);
		stc("Example:\n\r",ch);
		stc("fire 'barbed arrow' at trolloc 2\n\r",ch);
		stc("Note: This doesn't actually use two arrows.\n\r",ch);
		return;
	}
	if ( str_cmp( arg2, "at" ) )
	{
		stc("You must provide the correct, exact syntax.\n\r\n\r", ch );
		stc("Syntax:\n\r",ch);
		stc("fire <arrow> at <victim>\n\r",ch);
		stc("Example:\n\r",ch);
		stc("fire 'barbed arrow' at trolloc\n\r",ch);
		stc("If you have archery, you can fire two arrows at a time.\n\r",ch);
		stc("Example:\n\r",ch);
		stc("fire 'barbed arrow' at trolloc 2\n\r",ch);
		stc("Note: This doesn't actually use two arrows.\n\r",ch);
		return;
	}
	if ( arg4[0] != '\0' && is_number( arg4 ) )
	{
		if ( atoi( arg4 ) == 2 )
			twoFire = TRUE;
	}

    bow = get_eq_char(ch, WEAR_WIELD);
    if (bow == NULL)
	{
	    bow = get_eq_char(ch, WEAR_SECONDARY);
    	if (bow == NULL)
    	{
	    	hasbow = FALSE;
			stc("What are you going to do, throw the arrow at them?\n\r", ch);
			return;
		}
    	else if ( bow->value[0] == WEAPON_SHORTBOW
    	|| bow->value[0] == WEAPON_LONGBOW
	    || bow->value[0] == WEAPON_RECURVEDBOW
    	|| bow->value[0] == WEAPON_COMPOSITEBOW
	    || bow->value[0] == WEAPON_CROSSBOW
		|| bow->value[0] == WEAPON_LIGHTCROSSBOW )
		    hasbow = TRUE;
	}
    if ( bow != NULL )
    {
    	if ( bow->value[0] == WEAPON_SHORTBOW
    	|| bow->value[0] == WEAPON_LONGBOW
	    || bow->value[0] == WEAPON_RECURVEDBOW
    	|| bow->value[0] == WEAPON_COMPOSITEBOW
	    || bow->value[0] == WEAPON_CROSSBOW
		|| bow->value[0] == WEAPON_LIGHTCROSSBOW )
		    hasbow = TRUE;
	}
    if ( !hasbow )
	{
		stc("{cYou might want to use a bow to fire that arrow with.\n\r", ch);
		return;
	}
    if ( arg3[0] == '\0' && !is_fighting(ch) )
    {
        stc( "Fire an arrow at who?\n\r", ch );
        return;
    }
	if (!str_cmp(arg3, "none") || !str_cmp(arg3, "self") || victim == ch)
	{
		stc("How exactly did you plan on firing an arrow at yourself?\n\r", ch );
		return;
	}
	if ( ( arrow = get_obj_carry( ch, arg1, ch ) ) == NULL )
    {
        stc( "That's not going to work.\n\r", ch );
        return;
    }
    if ( arrow->item_type != ITEM_ARROW )
    {
        stc( "You can only a fire arrows or quarrels.\n\r", ch );
        return;
    }
    if ( arg3[0] == '\0' )
    {
        if ( is_fighting(ch) )
        {
            victim = ch->advancing_on;
        }
        else
        {
            stc( "Fire at whom or what?\n\r", ch );
            return;
        }
    }
	/* check for a target */
	if ( ( victim = get_char_room ( ch, NULL, arg3 ) ) != NULL)
	{
		if ( IS_NPC( victim ) )
		{
			if (victim->pIndexData->pShop != NULL)
			{
			    stc("{cThe shopkeeper wouldn't like that.\n\r",ch);
		    	return;
			}
			/* no killing healers, trainers, etc */
			if (IS_SET(victim->act,ACT_TRAIN)
			||  IS_SET(victim->act,ACT_PRACTICE)
	        ||  (victim->pIndexData->pShop != NULL)
			||  IS_SET(victim->act,ACT_IS_HEALER)
			||  IS_SET(victim->act,ACT_IS_CHANGER))
			{
			    stc("{cI don't think the Creator would approve.\n\r",ch);
			    return;
			}
		}
	    if ( victim->advancing == TRUE && victim->advancing_on != NULL && !is_same_group(victim->advancing_on, ch) )
	    {
			stc( "They're busy with another fight and you can't kill steal.\n\r", ch );
			return;
	    }
	    if ( is_fighting(victim) && !check_fighting_groupie(victim, ch) )
	    {
			stc( "They're busy with another fight and you can't kill steal.\n\r", ch );
			return;
	    }
		if (!IS_NPC(ch))
		{
			if (!IS_NPC(victim)
			&& !IS_SET(ch->in_room->room_flags,ROOM_PK)
			&& !IS_SET(victim->in_room->room_flags,ROOM_PK)
			&& !IS_IMMORTAL(ch))
			{
			    stc("Go to the Arena if you want to PK.\n\r",ch);
			    return;
			}
		}

	    if (IS_NPC(victim) &&
		 is_fighting(victim) &&
		!check_fighting_groupie(victim, ch))
	    {
	        stc("{cKill stealing is not permitted.\n\r",ch);
	        return;
	    }

	    if ( is_safe( ch, victim ) )
	      return;

		if (!IS_NPC(ch))
		{
		    /* no pets */
		    if (IS_SET(victim->act,ACT_PET))
		    {
			act("{cBut $N looks so cute and cuddly...",ch,NULL,victim,TO_CHAR);
			return;
		    }
		    /* no charmed creatures unless owner */
		    if (IS_AFFECTED(victim,AFF_CHARM) && ch != victim->master)
		    {
			stc("{cYou don't own that monster.\n\r",ch);
			return;
		    }
		}
		if((ch->in_room) == (victim->in_room))
			act("{w$n targets $N with a $t.", ch,bow->short_descr,victim,TO_ROOM);
		/* Lag the bowman... */
		start_first_fight(ch, victim, RANGE_LONG);
	    if ( bow->value[0] == WEAPON_SHORTBOW )
	    {
	    	bow_bonus = get_curr_stat( ch, STAT_STR ) / 4;
			WAIT_STATE( ch, PULSE_VIOLENCE / 5 );
		}
    	if ( bow->value[0] == WEAPON_LONGBOW )
	    {
	    	bow_bonus = get_curr_stat( ch, STAT_STR ) / 3;
			WAIT_STATE( ch, PULSE_VIOLENCE / 4 );
		}
	    if ( bow->value[0] == WEAPON_RECURVEDBOW )
	    {
	    	bow_bonus = get_curr_stat( ch, STAT_STR ) / 2;
			WAIT_STATE( ch, PULSE_VIOLENCE / 3 );
		}
    	if ( bow->value[0] == WEAPON_COMPOSITEBOW )
	    {
	    	bow_bonus = get_curr_stat( ch, STAT_STR );
			WAIT_STATE( ch, PULSE_VIOLENCE / 2 );
		}
	    if ( bow->value[0] == WEAPON_CROSSBOW )
	    {
	    	bow_bonus = get_skill( ch, gsn_bow ) * 2;
			WAIT_STATE( ch, PULSE_VIOLENCE );
		}
		/* Fire the damn thing finally! */
		if(arrow->item_type== ITEM_ARROW )
		{
	        if ( victim != NULL )
	        {
	            act( "{c$n fires $t at $N{W.", ch,arrow->short_descr, victim, TO_NOTVICT );
	            act( "{cYou fire $t at $N{W.", ch,arrow->short_descr, victim, TO_CHAR );
	            act( "{c$n fires $t at you.",ch,arrow->short_descr, victim, TO_VICT );
	        }
	        if ( number_percent() > get_skill(ch,gsn_bow) )
	        {
	              act( "{cYou fire $t missing, and it lands harmlessly on the ground.",
	                 ch,arrow->short_descr,NULL,TO_CHAR);
	              act( "{c$r fires $t missing, and it lands harmlessly on the ground.",
	                 ch,arrow->short_descr,NULL,TO_ROOM);
	              obj_from_char(arrow);
				  obj_to_room(arrow, victim->in_room);
		          check_improve(ch,gsn_bow,FALSE,2);
	        }
	        else
	        {
            if ( get_skill(ch,gsn_enhanced_damage) > 0 )
			{
			    diceroll = number_percent();
			    if (diceroll <= get_skill(ch,gsn_enhanced_damage))
			    {
			        check_improve(ch,gsn_enhanced_damage,TRUE,6);
			        dam += get_curr_stat(ch,STAT_STR) / 2;
			    }
			}
			if ( !twoFire )
			{
	            dam =  dice(arrow->value[1],arrow->value[2]);
	            dam += get_skill(ch,gsn_bow);
	            dam += bow_bonus;
				damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
			    if ( number_percent( ) < sec_att )
			    {
					damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
					check_improve(ch,gsn_second_attack,TRUE,6);
			    }
			    if ( number_percent( ) < third_att )
			    {
					damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
					check_improve(ch,gsn_third_attack,TRUE,6);
			    }
			}
		    else
		    {
	            dam =  dice(arrow->value[1],arrow->value[2]);
	            dam += get_skill(ch,gsn_bow);
	            dam += bow_bonus;
				if ( number_percent() < archer )
		            dam += archer;
				damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
				if ( number_percent() < archer )
				{
		            dam =  dice(arrow->value[1],arrow->value[2]);
		            dam += get_skill(ch,gsn_bow);
		            dam += bow_bonus;
					if ( number_percent() < archer )
			            dam += archer;
					damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
				}
			    if ( number_percent( ) < sec_att )
			    {
		            dam =  dice(arrow->value[1],arrow->value[2]);
		            dam += get_skill(ch,gsn_bow);
		            dam += bow_bonus;
					if ( number_percent() < archer )
			            dam += archer;
					damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
					check_improve(ch,gsn_second_attack,TRUE,6);
			    }
			    if ( number_percent( ) < third_att )
			    {
		            dam =  dice(arrow->value[1],arrow->value[2]);
		            dam += get_skill(ch,gsn_bow);
		            dam += bow_bonus;
					if ( number_percent() < archer )
			            dam += archer;
					damage( ch, victim, dam, gsn_bow, DAM_PIERCE, TRUE );
					check_improve(ch,gsn_third_attack,TRUE,6);
			    }
			}
	        obj_from_char(arrow);
			obj_to_room(arrow, ch->in_room);

			check_improve(ch,gsn_bow,TRUE,2);
			if ( archer > 0 )
				check_improve(ch,gsn_archery,TRUE,2);

			arr_dmg = number_range(1,5);
			arrow->condition -= arr_dmg;
			if ( number_percent() > arrow->condition )
			{
	        act( "{c$t shatters into splinters from the force!",ch,arrow->short_descr,NULL,TO_ROOM);
	        act( "{c$t shatters into splinters from the force!",ch,arrow->short_descr,NULL,TO_CHAR);
	        extract_obj( arrow );
			}
	   }
	   }
   }
   else
	   stc("They're not here.\n\r",ch);
   return;
}

/* Bowfire code -- Used to dislodge an arrow already lodged */
void do_dislodge( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA * arrow = NULL;
	int dam = 0;

    if (argument[0] == '\0') /* empty */
    {
        stc( "{cDislodge what?\n\r",ch);
        return;
    }

	if ( get_eq_char(ch, WEAR_LODGE_RIB) != NULL)
	{
	arrow = get_eq_char( ch, WEAR_LODGE_RIB );
	act( "{cWith a wrenching pull, you dislodge $p from your chest.", ch, arrow, NULL, TO_CHAR );
	unequip_char( ch, arrow );
	arrow->extra_flags = arrow->extra_flags - 134217728;
	dam      =  dice((3 * arrow->value[1]), (3 * arrow->value[2]));
	damage( ch, ch, dam, gsn_bow, DAM_SLASH, TRUE );
	return;
	}

	else
	if (get_eq_char(ch,WEAR_LODGE_ARM) != NULL)
	{
	arrow = get_eq_char( ch, WEAR_LODGE_ARM );
	act( "{cWith a tug you dislodge $p from your arm.", ch, arrow, NULL, TO_CHAR );
	unequip_char( ch, arrow );
	arrow->extra_flags = arrow->extra_flags - 134217728;
	dam      =  dice((3 * arrow->value[1]), (2 * arrow->value[2]));
	damage( ch, ch, dam, gsn_bow, DAM_SLASH, TRUE );
	return;
	}

	else
	if (get_eq_char(ch,WEAR_LODGE_LEG) != NULL)
	{
	arrow = get_eq_char( ch, WEAR_LODGE_LEG );
	act( "{cWith a tug you dislodge $p from your leg.", ch, arrow, NULL, TO_CHAR );
	unequip_char( ch, arrow );
	arrow->extra_flags = arrow->extra_flags - 134217728;
	dam      =  dice((2 * arrow->value[1]), (2 * arrow->value[2]));
	damage( ch, ch, dam, gsn_bow, DAM_SLASH, TRUE );
	return;
	}
	else
	{
	stc( "{cYou have nothing lodged in your body.\n\r", ch);
	return;
	}
}

void	set_fighting	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );

void do_bubble( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *bubble;
    CHAR_DATA *parent;
    char lbuf[MSL];
    int i = 0;

	if ( ( parent = get_char_world( ch, argument ) ) == NULL )
	{
		stc("Your victim could not be found.\n\r",ch);
		return;
	}

    strcat( lbuf, parent->name );
    strcat( lbuf, "\n\r" );

	bubble = create_mobile(get_mob_index(MOB_VNUM_BUBBLE));
	free_string(bubble->name);
	    bubble->name 	= str_dup(parent->name);
	free_string(bubble->colorname);
	bubble->colorname	= str_dup(parent->colorname);
	free_string(bubble->short_descr);
	    bubble->short_descr	= str_dup(parent->short_descr);
	free_string(bubble->long_descr);
	    bubble->long_descr	= str_dup(parent->long_descr);
	free_string(bubble->description);
	    bubble->description	= str_dup("");
    bubble->group	= parent->group;
    bubble->sex		= parent->sex;
    bubble->race		= parent->race;
    bubble->level	= parent->level;
    bubble->trust	= 0;
    bubble->hit		= parent->max_hit;
    bubble->max_hit	= parent->max_hit;
    bubble->stam	= parent->max_stam;
    bubble->max_stam	= parent->max_stam;
    bubble->money[3][0]		= 0;
    bubble->money[2][0]	= 0;
    bubble->imm_flags	= parent->imm_flags;
    bubble->res_flags	= parent->res_flags;
    bubble->vuln_flags	= parent->vuln_flags;
    bubble->position	= POS_STANDING;
    bubble->saving_throw	= parent->saving_throw;
    bubble->alignment	= -1000;
    bubble->hitroll	= parent->hitroll*10;
    bubble->damroll	= parent->damroll*10;
    bubble->form		= parent->form;
    bubble->parts	= parent->parts;
    bubble->size		= parent->size;
    bubble->dam_type	= parent->dam_type;
    bubble->start_pos	= parent->start_pos;

    for (i = 0; i < 4; i++)
    	bubble->armor[i]	= parent->armor[i];

    for (i = 0; i < MAX_STATS; i++)
    {
	bubble->perm_stat[i]	= parent->perm_stat[i];
	bubble->mod_stat[i]	= parent->mod_stat[i];
    }

    for (i = 0; i < 3; i++)
	bubble->damage[i]	= parent->damage[i];
	char_to_room(bubble,parent->in_room);
	act("The likeness of $n appears from out of a sudden mist.",parent,NULL,bubble,TO_ROOM);
	act("Your likeness appears from out of a sudden mist.",parent,NULL,bubble,TO_VICT);
    start_hating(bubble, parent);
        start_extra_fight(parent, bubble, RANGE_MELEE);
	return;
}

void do_light( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *obj;
	CHAR_DATA *keeper;
    SHOP_DATA *pShop;
	char arg[MIL];

    pShop = NULL;
	argument = one_argument( argument, arg );

	if ( arg[0] == '\0' )
	{
	stc( "What are you trying to light?\n\r",ch);
	return;
	}

    if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
    {
	stc( "You do not have that item.\n\r", ch );
	return;
    }

    if ( obj->item_type != ITEM_FIREOIL )
    {
    stc( "You can't light that item.\n\r", ch );
    return;
    }

    if ( obj->value[1] == 1 )
    {
    stc( "It's already lit!\n\r", ch );
    return;
    }

    for ( keeper = ch->in_room->people; keeper; keeper = keeper->next_in_room )
    {
		if ( IS_NPC(keeper) && (pShop = keeper->pIndexData->pShop) != NULL )
		{
		    stc("I don't think the shop keeper would approve.\n\r",ch);
		    return;
		}
    }

    if ( pShop != NULL )
    {
		stc( "You can't do that here.\n\r", ch );
		return;
    }

	if (IS_SET( ch->in_room->room_flags, ROOM_SAFE) )
	{
	    stc("Not in this room.\n\r",ch);
	    return;
	}

    obj->value[1] = 1;
    act("You light a $t and drop it on the ground.",ch,check_dye(obj, "", obj->dye_vnum),NULL,TO_CHAR);
    act("$n lights a $t and drops it on the ground.",ch,check_dye(obj, "", obj->dye_vnum),NULL,TO_ROOM);
	obj_from_char( obj );
	obj_to_room( obj, ch->in_room );

    tail_chain();
}
