/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "mysql5_interface.h"

void rplog_command( CHAR_DATA *ch, char * command );

/*
 * Local functions.
 */
#define TOGGLE_BIT(var, bit)    ((var) ^= (bit))

ROOM_INDEX_DATA * find_location	args( ( CHAR_DATA *ch, char *arg ) );
bool	write_to_descriptor	args( ( int desc, char *txt, int length ) );
int flag_value args(( const struct flag_type *flag_table, char *argument));

int master_perc args((CHAR_DATA *ch));

const   struct  pair_type        pair_table       [] =
{
  {"switch", "return",FALSE},
  {"reboo", "reboot",FALSE},
  {"shutdow", "shutdown",FALSE},
  {"grant","gran",FALSE},
  {"revoke","revok",FALSE},
  {"sla", "slay",FALSE},
  {"", "",FALSE}
};

bool is_granted( CHAR_DATA *ch, DO_FUN *do_fun)
{
GRANT_DATA *gran;

if (ch->desc == NULL) return FALSE;

if (ch->desc->original != NULL) ch = ch->desc->original;

for (gran=ch->pcdata->granted; gran != NULL; gran=gran->next)
  if (do_fun == gran->do_fun)
    return TRUE;

return FALSE;
}

bool is_granted_name( CHAR_DATA *ch, char *name)
{
GRANT_DATA *gran;

if (ch->desc == NULL) return FALSE;

if (ch->desc->original != NULL) ch = ch->desc->original;

for (gran=ch->pcdata->granted; gran != NULL; gran=gran->next)
  if (is_exact_name(gran->name,name))
    return TRUE;

return FALSE;
}

int grant_duration(CHAR_DATA *ch, DO_FUN *do_fun)
{
GRANT_DATA *gran;

if (ch->desc->original != NULL) ch=ch->desc->original;

/*  Replace the x's in the line below with the name of
    a character that is allowed to grant commands to
    anyone, even if they don't have the command
    themselves.  This is useful when you add new
    imm commands, and need to give them to yourself.
    Additional names can be added as needed and
    should be seperated by spaces.  */

if (is_exact_name(ch->name,"Tjor")
|| is_exact_name(ch->name, "Avryel") || is_exact_name(ch->name, "Rokann"))
  return -1;

for (gran=ch->pcdata->granted; gran != NULL; gran=gran->next)
  if (gran->do_fun == do_fun)
    return gran->duration;

return 0;
}

void grant_add(CHAR_DATA *ch, char *name, DO_FUN *do_fun, int duration,
               int level)
{
GRANT_DATA *gran;

if (ch->desc->original != NULL) ch=ch->desc->original;

gran = alloc_mem(sizeof(*gran));
gran->name = str_dup(name);
gran->do_fun = do_fun;
gran->duration = duration;
gran->level = level;

gran->next = ch->pcdata->granted;
ch->pcdata->granted = gran;

return;
}

void grant_remove(CHAR_DATA *ch, DO_FUN *do_fun, bool mshow)
{
GRANT_DATA *p,*gran;
char buf[MAX_STRING_LENGTH];
CHAR_DATA *rch;

rch = ch->desc->original?ch->desc->original:ch;

p=NULL;
gran=rch->pcdata->granted;
if (gran->do_fun == do_fun)
  rch->pcdata->granted=gran->next;
else
  for (gran=rch->pcdata->granted; gran != NULL; gran=gran->next)
  {
    if (gran->do_fun == do_fun) break;
    p=gran;
  }

if (p != NULL) p->next=gran->next;
sprintf(buf,"You have lost access to the %s command.\n\r",gran->name);
if (mshow) send_to_char(buf,ch);
free_string(gran->name);
free_mem(gran,sizeof(*gran));
return;
}

void grant_level( CHAR_DATA *ch, CHAR_DATA *victim, int level, int
duration )
{
int cmd;

for (cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
  if ( cmd_table[cmd].level == level
  && !is_granted(victim,cmd_table[cmd].do_fun)
  && grant_duration(ch,cmd_table[cmd].do_fun) == -1)
    grant_add(victim,cmd_table[cmd].name,cmd_table[cmd].do_fun,
              duration, cmd_table[cmd].level);
return;
}

void revoke_level( CHAR_DATA *ch, CHAR_DATA *victim, int level )
{
int cmd;

for (cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
  if ( cmd_table[cmd].level == level
  && is_granted(victim,cmd_table[cmd].do_fun)
  && grant_duration(ch,cmd_table[cmd].do_fun) == -1)
    grant_remove(victim,cmd_table[cmd].do_fun,FALSE);
return;
}

void do_grant( CHAR_DATA *ch, char *argument )
{
char buf[MAX_STRING_LENGTH];
char arg1[MAX_INPUT_LENGTH];
char arg2[MAX_INPUT_LENGTH];
char arg3[MAX_INPUT_LENGTH];
CHAR_DATA *victim=NULL,*rch,*rvictim=NULL;
int  dur,cmd,x;
bool found=FALSE, isok=FALSE;
DESCRIPTOR_DATA *d;

argument = one_argument(argument,arg1);
argument = one_argument(argument,arg2);
one_argument(argument,arg3);

if ( is_exact_name( ch->name, "Tjor" ) ) {
	isok = TRUE;
}
else if ( is_exact_name( ch->name, "Avryel" ) ) {
	isok = TRUE;
}
else if ( is_exact_name( ch->name, "Rokann" ) ) {
	isok = TRUE;
}

if ( !isok )
	return;
rch = ch->desc->original?ch->desc->original:ch;

if (arg1[0] == '\0')
  {
    send_to_char("Grant who, what?\n\r",ch);
    return;
  }

for (d = descriptor_list; d != NULL; d = d->next)
{
  rvictim = d->original?d->original:d->character;

  if (rvictim == NULL) continue;

  if (!str_cmp(rvictim->name,arg1))
  {
     victim = d->character;
     break;
  }
}

if (victim == NULL && !str_cmp("self",arg1))
{
   rvictim = rch;
   victim = ch;
}

if (victim==NULL)
  {
    send_to_char("Victim not found.\n\r",ch);
    return;
  }

if (arg2[0] == '\0')
  {
    int col=0;
    int lvl;

    sprintf(buf,"%s has not been granted the following commands:\n\r",
            rvictim->name);
    send_to_char(buf,ch);

    for ( lvl = IM; lvl <= ( L1 + 1 ) ; lvl++ )
    for (cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
      if (cmd_table[cmd].level >= LEVEL_IMMORTAL
      && !is_granted(victim,cmd_table[cmd].do_fun)
      && cmd_table[cmd].level == lvl)
      {
        sprintf( buf,"[L%3d] %-12s", cmd_table[cmd].level,
cmd_table[cmd].name );
        send_to_char(buf,ch);
        if ( ++col % 4 == 0 )
          send_to_char( "\n\r", ch );
      }
    if ( col % 4 != 0 )
      send_to_char( "\n\r", ch);
    return;
  }

dur = arg3[0]=='\0' ? -1 : is_number(arg3) ? atoi(arg3) : 0;

if (dur<1 && dur != -1)
  {
    send_to_char("Invalid duration!\n\r",ch);
    return;
  }

if (is_number(arg2))
  {
    if (atoi(arg2)<LEVEL_IMMORTAL || atoi(arg2)>MAX_LEVEL)
    {
      send_to_char("Invalid grant level.\n\r",ch);
      return;
    }
    grant_level(ch, victim, atoi(arg2), dur );
    sprintf(buf, "You have been granted level %d commands.\n\r",
atoi(arg2));
    send_to_char("Ok.\n\r",ch);
    send_to_char(buf,victim);
    return;
  }

for (cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
  if ( arg2[0] == cmd_table[cmd].name[0]
  &&   is_exact_name( arg2, cmd_table[cmd].name ) )
  {
      found = TRUE;
      break;
  }

if (found)
  {
    if (cmd_table[cmd].level < LEVEL_IMMORTAL)
    {
      send_to_char("You can only grant immortal commands.\n\r",ch);
      return;
    }

    if (grant_duration(ch,cmd_table[cmd].do_fun) != -1)
    {
      send_to_char("You can't grant that!\n\r",ch);
      return;
    }

    if (is_granted(victim,cmd_table[cmd].do_fun))
    {
      send_to_char("They already have that command!\n\r",ch);
      return;
    }

    grant_add(victim,cmd_table[cmd].name,cmd_table[cmd].do_fun,
              dur, cmd_table[cmd].level);

    sprintf(buf,"%s has been granted the %s command.\n\r",rvictim->name,
            cmd_table[cmd].name);
    send_to_char(buf,ch);
    sprintf(buf,"%s has granted you the %s command.\n\r",rch->name,
            cmd_table[cmd].name);
    send_to_char(buf,victim);

    for (x=0; pair_table[x].first[0] != '\0'; x++)
      if (!str_cmp(arg2,pair_table[x].first)
      && !is_granted_name(victim,pair_table[x].second))
      {
        sprintf(buf,"%s %s %s",rvictim->name, pair_table[x].second, arg3);
        do_grant(ch,buf);
      }
      else if (!str_cmp(arg2,pair_table[x].second)
           && pair_table[x].one_way != TRUE
           && !is_granted_name(victim,pair_table[x].first))
      {
        sprintf(buf,"%s %s %s",rvictim->name,pair_table[x].first, arg3);
        do_grant(ch,buf);
      }

    return;
  }
  send_to_char("Command not found!\n\r",ch);
  return;
}

void do_revoke( CHAR_DATA *ch, char *argument )
{
char arg1[MAX_INPUT_LENGTH];
char arg2[MAX_INPUT_LENGTH];
CHAR_DATA *victim=NULL,*rvictim=NULL;
DESCRIPTOR_DATA *d;
int cmd,x;
bool had_return, found=FALSE, isok=FALSE;

argument = one_argument(argument,arg1);
one_argument(argument,arg2);

if ( is_exact_name( ch->name, "Tjor" ) ) {
	isok = TRUE;
}
else if ( is_exact_name( ch->name, "Avryel" ) ) {
	isok = TRUE;
}
else if ( is_exact_name( ch->name, "Rokann") ) {
	isok = TRUE;
}
if ( !isok )
	return;
if (arg1[0] == '\0' )
  {
    send_to_char("Revoke who, what?\n\r",ch);
    return;
  }

for (d = descriptor_list; d != NULL; d = d->next)
{
  rvictim = d->original?d->original:d->character;

  if (rvictim == NULL) continue;

  if (!str_cmp(rvictim->name,arg1))
  {
     victim = d->character;
     break;
  }
}

if (victim == NULL && !str_cmp("self",arg1))
{
  rvictim = ch->desc->original ? ch->desc->original : ch;
  victim = ch;
}

if (victim==NULL)
  {
    send_to_char("Victim not found.\n\r",ch);
    return;
  }

had_return = is_granted_name(victim,"return");

if (arg2[0] == '\0')
  {
    int col=0,lvl;
    char buf[MAX_STRING_LENGTH];

    sprintf(buf,"%s has been granted the following commands:\n\r",
            rvictim->name);
    send_to_char(buf,ch);

    for ( lvl = IM; lvl <= ( L1 + 1 ) ; lvl++ )
    for (cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
      if (cmd_table[cmd].level >= LEVEL_IMMORTAL
      && is_granted(victim,cmd_table[cmd].do_fun)
      && cmd_table[cmd].level == lvl )
      {
        sprintf( buf,"[L%3d] %-12s", cmd_table[cmd].level,
cmd_table[cmd].name );
        send_to_char(buf,ch);
        if ( ++col % 4 == 0 )
          send_to_char( "\n\r", ch );
      }
    if ( col % 4 != 0 )
      send_to_char( "\n\r", ch);
    return;
  }

if (is_number(arg2))
  {
    char buf[MAX_STRING_LENGTH];

    if (atoi(arg2)<LEVEL_IMMORTAL || atoi(arg2)>MAX_LEVEL)
    {
      send_to_char("Invalid revoke level.\n\r",ch);
      return;
    }
    revoke_level(ch, victim, atoi(arg2));
    sprintf(buf, "You have lost acess to level %d commands.\n\r",
atoi(arg2));
    send_to_char("Ok.\n\r",ch);
    send_to_char(buf,victim);

    if (had_return && !is_granted_name(victim,"return") &&
        rvictim != victim) do_return(victim,"");

    return;
  }

for (cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
  if ( arg2[0] == cmd_table[cmd].name[0]
  &&   is_exact_name( arg2, cmd_table[cmd].name ) )
  {
      found = TRUE;
      break;
  }

if (found)
  {
    char buf[MAX_STRING_LENGTH];

    if (grant_duration(ch,cmd_table[cmd].do_fun) != -1)
    {
      send_to_char("You can't revoke that!\n\r",ch);
      return;
    }

    if (!is_granted(victim,cmd_table[cmd].do_fun))
    {
      send_to_char("They don't have that command!\n\r",ch);
      return;
    }

    grant_remove(victim,cmd_table[cmd].do_fun,TRUE);

    sprintf(buf,"%s has lost access to the %s command.\n\r",
             rvictim->name,cmd_table[cmd].name);
    send_to_char(buf,ch);

    for (x=0; pair_table[x].first[0] != '\0'; x++)
      if (!str_cmp(arg2,pair_table[x].first)
      && is_granted_name(victim,pair_table[x].second))
      {
        sprintf(buf,"%s %s",rvictim->name,pair_table[x].second);
        do_revoke(ch,buf);
      }
      else if (!str_cmp(arg2,pair_table[x].second)
           && pair_table[x].one_way != TRUE
           && is_granted_name(victim,pair_table[x].first))
      {
        sprintf(buf,"%s %s",rvictim->name,pair_table[x].first);
        do_revoke(ch,buf);
      }

    if (had_return && !is_granted_name(victim,"return") &&
        rvictim != victim) do_return(victim,"");

    return;
  }
  send_to_char("Command not found!\n\r",ch);
  return;
}


void do_gstat( CHAR_DATA *ch, char *argument )
{
char arg[MAX_INPUT_LENGTH];
char buf[MAX_STRING_LENGTH];
BUFFER *buffer;
GRANT_DATA *grant;
CHAR_DATA *victim;
int col=0;

one_argument(argument,arg);

if (arg[0] == '\0')
  {
    send_to_char("Gstat who?\n\r",ch);
    return;
  }

if ( ( victim = get_char_world( ch, arg ) ) == NULL )
  {
    send_to_char( "They aren't here.\n\r", ch );
    return;
  }

if (IS_NPC(victim))
  {
    send_to_char("Not on mobs.\n\r",ch);
    return;
  }

if (get_trust(ch)<get_trust(victim))
  {
    send_to_char("You can't do that.\n\r",ch);
    return;
  }

buffer=new_buf();

sprintf(buf,"Grant status for %s:\n\r\n\r",victim->name );

add_buf(buffer,buf);

for (grant=victim->pcdata->granted; grant!=NULL; grant=grant->next)
  {
    char ds[50],ss[25],s2[25];
    int x,sl;

    sprintf(ds,"%d",grant->duration);
    ss[0]='\0';
    sl = (int)((6-strlen(ds)) / 2);

    for (x=0; x<sl ;x++)
      strcat(ss," ");

    strcpy(s2,ss);

    if ((strlen(ss)+strlen(ds)) % 2 == 1) strcat(s2," ");

    if (grant->duration==-1)
      sprintf(buf,"[ perm ] %-11s",grant->name);
    else
      sprintf(buf,"[%s%d%s] %-11s",ss, grant->duration, s2, grant->name);

    add_buf(buffer,buf);

    col++;
    col %= 4;

    if (col==0) add_buf(buffer,"\n\r");
  }

if (col != 0) add_buf(buffer,"\n\r");

page_to_char(buf_string(buffer),ch);

free_buf(buffer);
return;
}

void do_imortflag( CHAR_DATA *ch, char *argument )
{
send_to_char( "You have been flaged as an Immortal.\n\r", ch );
return;
}


void do_wiznet( CHAR_DATA *ch, char *argument )
{
    int flag;
    char buf[MAX_STRING_LENGTH];

    if ( argument[0] == '\0' )
    {
      	if (IS_SET(ch->wiznet,WIZ_ON))
      	{
            stc("{cSigning off of Wiznet.\n\r",ch);
            REMOVE_BIT(ch->wiznet,WIZ_ON);
      	}
      	else
      	{
            stc("{cWelcome to Wiznet!\n\r",ch);
            SET_BIT(ch->wiznet,WIZ_ON);
      	}
      	return;
    }

    if (!str_prefix(argument,"on"))
    {
	stc("{cWelcome to Wiznet!\n\r",ch);
	SET_BIT(ch->wiznet,WIZ_ON);
	return;
    }

    if (!str_prefix(argument,"off"))
    {
	stc("{cSigning off of Wiznet.\n\r",ch);
	REMOVE_BIT(ch->wiznet,WIZ_ON);
	return;
    }

    /* show wiznet status */
    if (!str_prefix(argument,"status"))
    {
	buf[0] = '\0';

	if (!IS_SET(ch->wiznet,WIZ_ON))
	    strcat(buf,"off ");

	for (flag = 0; wiznet_table[flag].name != NULL; flag++)
	    if (IS_SET(ch->wiznet,wiznet_table[flag].flag))
	    {
		strcat(buf,wiznet_table[flag].name);
		strcat(buf," ");
	    }

	strcat(buf,"\n\r");

	stc("{cWiznet status:\n\r",ch);
	stc(buf,ch);
	return;
    }

    if (!str_prefix(argument,"show"))
    /* list of all wiznet options */
    {
	buf[0] = '\0';

	for (flag = 0; wiznet_table[flag].name != NULL; flag++)
	{
	    if (wiznet_table[flag].level <= get_trust(ch))
	    {
	    	strcat(buf,wiznet_table[flag].name);
	    	strcat(buf,"\n\r");
	    }
	}

	strcat(buf,"\n\r");

	stc("{cWiznet options available to you are:\n\r",ch);
	stc(buf,ch);
	return;
    }

    flag = wiznet_lookup(argument);

    if (flag == -1 || get_trust(ch) < wiznet_table[flag].level)
    {
	stc("{cNo such option.\n\r",ch);
	return;
    }

    if (IS_SET(ch->wiznet,wiznet_table[flag].flag))
    {
	sprintf(buf,"{cYou will no longer see %s on wiznet.\n\r",
	        wiznet_table[flag].name);
	stc(buf,ch);
	REMOVE_BIT(ch->wiznet,wiznet_table[flag].flag);
    	return;
    }
    else
    {
    	sprintf(buf,"{cYou will now see %s on wiznet.\n\r",
		wiznet_table[flag].name);
	stc(buf,ch);
    	SET_BIT(ch->wiznet,wiznet_table[flag].flag);
	return;
    }

}

void wiznet(char *string, CHAR_DATA *ch, OBJ_DATA *obj,
	    long flag, long flag_skip, int min_level)
{
    DESCRIPTOR_DATA *d;

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
        if (d->connected == CON_PLAYING
        &&  d->character != NULL
		&&  IS_IMMORTAL(d->character)
		&&  IS_SET(d->character->wiznet,WIZ_ON)
		&&  (!flag || IS_SET(d->character->wiznet,flag))
		&&  (!flag_skip || !IS_SET(d->character->wiznet,flag_skip))
		&&  get_trust(d->character) >= min_level
		&&  d->character != ch)
        {
	    if (IS_SET(d->character->wiznet,WIZ_PREFIX))
	  	stc("{c{Z--> ",d->character);
	    else
	  	stc("{c{Z", d->character );
        act_new(string,d->character,obj,ch,TO_CHAR,POS_DEAD);
	    stc("{c{x", d->character );
        }
    }

    return;
}
/*
void do_guild( CHAR_DATA *ch, char *argument )
{
    char arg1[MIL], arg2[MIL];
    CHAR_DATA *victim;
    int clan, sn;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
        stc("{cSyntax: guild <char> <cln name>\n\r",ch);
        return;
    }
    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
        stc("{cThey aren't playing.\n\r", ch );
        return;
    }

    if ((!str_prefix(arg2,"implementor"))
    && (victim->level < 1000))
	{
		stc("{cThey are not powerful enough for that Guild.",ch);
		return;
	}
    if ((clan = clan_lookup(arg2)) == 0)
    {
		stc("{cNo such Guild exists.\n\r",ch);
		return;
    }
	if ( is_clan(victim) )
	{
		ptc( victim,"You are no longer a member of the %s Guild.\n\r",
			capitalize(clan_table[victim->clan].name) );
		if (IS_SET(victim->hist_flags,HIST_RECRUITER))
			REMOVE_BIT(victim->hist_flags,HIST_RECRUITER);
		ptc( victim,"You are no longer a recruiter of the %s Guild.\n\r",
			capitalize(clan_table[victim->clan].name) );
	    for (sn = 0; sn < MAX_SKILL; sn++)
	    {
	        if (skill_table[sn].name == NULL )
			    break;
			if ( skill_table[sn].guild_num != GSKILL_NONE
			&& ch->pcdata->learned[sn] > 0 )
			{
			ptc( victim,"You are no longer able to use the %s skill.\n\r",
				capitalize(skill_table[sn].name) );
			ch->pcdata->learned[sn] = 0;
			}
		}
	}
    if (clan_table[clan].independent)
    {
		ptc(ch,"{cThey are now a %s.\n\r",clan_table[clan].name);
		ptc(victim,"{cYou are now a %s.\n\r",clan_table[clan].name);
		if ( victim->pcdata )
			victim->pcdata->deguilded = 1;
    }
    else
    {
		if ( !IS_IMMORTAL( ch ) && victim->pcdata && victim->pcdata->deguilded == 1 )
		{
			stc("They have been deguilded without remorting yet.\n\r",ch);
			return;
		}
		else if ( IS_IMMORTAL( ch ) && victim->pcdata && victim->pcdata->deguilded == 1 )
		{
			stc("{RThey have been deguilded without remorting yet.{x\n\r",ch);
			stc("{RYou are bypassing the deguild so be sure this is what you want.{x\n\r",ch);
			stc("{RThe deguild has been removed. Now guild them again.{x\n\r",ch);
			return;
		}
		ptc(ch,"{cThey are now a member of the %s Guild.\n\r",
		    capitalize(clan_table[clan].name));
		ptc(victim,"{cYou are now a member of the %s Guild.\n\r",
		    capitalize(clan_table[clan].name));
    }

    victim->clan = clan;
}
*/
/* equips a character */
void do_outfit ( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj;
    int i,sn,vnum;

    if (ch->level > 5 || IS_NPC(ch))
    {
    	stc("{cFind it yourself!\n\r",ch);
    	return;
    }
/*
    if ( ( obj = get_eq_char( ch, WEAR_LIGHT ) ) == NULL )
    {
        obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_BANNER), 0 );
    	obj->cost = 0;
        obj_to_char( obj, ch );
        equip_char( ch, obj, WEAR_LIGHT );
    }
*/
    if ( ( obj = get_eq_char( ch, WEAR_BODY ) ) == NULL )
    {

        if (ch->race == race_lookup("aiel"))
    	{
        	obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_CADINSOR), 0 );
        	obj->cost = 0;
            obj_to_char( obj, ch );
            equip_char( ch, obj, WEAR_BODY );
        	obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_SHOUFA), 0 );
        	obj->cost = 0;
            obj_to_char( obj, ch );
            equip_char( ch, obj, WEAR_HEAD );
        	obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_BEDROLL), 0 );
        	obj->cost = 0;
            obj_to_char( obj, ch );
    	}
        else if (ch->sex == SEX_FEMALE)
        {
        	obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_DRESS), 0 );
        	obj->cost = 0;
            obj_to_char( obj, ch );
            equip_char( ch, obj, WEAR_BODY );
        	obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_BEDROLL), 0 );
        	obj->cost = 0;
            obj_to_char( obj, ch );
        }
        else if (ch->sex == SEX_MALE)
        {
    	    obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_VEST), 0 );
    	    obj->cost = 0;
            obj_to_char( obj, ch );
            equip_char( ch, obj, WEAR_BODY );
        	obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_BEDROLL), 0 );
        	obj->cost = 0;
            obj_to_char( obj, ch );
	    }
    }

    /* do the weapon thing */
    if ((obj = get_eq_char(ch,WEAR_WIELD)) == NULL)
    {
      	sn = 0;
       	vnum = OBJ_VNUM_SCHOOL_SWORD; /* just in case! */

	   	for (i = 0; weapon_table[i].name != NULL; i++)
	   	{
	        if (ch->pcdata->learned[sn] < ch->pcdata->learned[*weapon_table[i].gsn])
	        {
	       	sn = *weapon_table[i].gsn;
	       	vnum = weapon_table[i].vnum;
	        }
	   	}
       	obj = create_object(get_obj_index(vnum),0);
      	obj_to_char(obj,ch);
       	equip_char(ch,obj,WEAR_WIELD);
    }

    if (((obj = get_eq_char(ch,WEAR_WIELD)) == NULL
    ||   !IS_WEAPON_STAT(obj,WEAPON_TWO_HANDS))
    &&  (obj = get_eq_char( ch, WEAR_SHIELD ) ) == NULL )
    {
        obj = create_object( get_obj_index(OBJ_VNUM_SCHOOL_SHIELD), 0 );
    	obj->cost = 0;
        obj_to_char( obj, ch );
        equip_char( ch, obj, WEAR_SHIELD );
    }

    stc("{cYou have been equipped by the Creator.\n\r",ch);
}

void do_smote(CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *vch;
    char *letter,*name,*double_temp;
    char last[MAX_INPUT_LENGTH], temp[MAX_STRING_LENGTH];
    int matches = 0;
    bool in_doubles;

    if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE) )
    {
        stc("{cYou can't show your emotions.\n\r", ch );
        return;
    }

    if ( argument[0] == '\0' )
    {
        stc("{cEmote what?\n\r", ch );
        return;
    }

    if (strstr(argument,ch->name) == NULL)
    {
	stc("{cYou must include your name in an smote.\n\r",ch);
	return;
    }

    stc(argument,ch);
    stc("{c\n\r",ch);

    for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
    {
        if (vch->desc == NULL || vch == ch)
            continue;
        if (!IS_AWAKE(vch))
            continue;

        if ((letter = strstr(argument,vch->name)) == NULL)
        {
	    stc(argument,vch);
	    stc("{c\n\r",vch);
            continue;
        }

	in_doubles = FALSE;
        strcpy(temp,argument);
        temp[strlen(argument) - strlen(letter)] = '\0';
        last[0] = '\0';
        name = vch->name;
	for(double_temp = argument; double_temp < letter; double_temp++)
	{
	    if (*double_temp== '\"')
	    {
		in_doubles = !in_doubles;
	    }
	}


        for (; *letter != '\0'; letter++)
        {

            if (*letter == '\'' && matches == strlen(vch->name))
            {
                strcat(temp,"r");
                continue;
            }

            if (*letter == 's' && matches == strlen(vch->name))
            {
                matches = 0;
                continue;
            }

            if (matches == strlen(vch->name))
            {
                matches = 0;
            }

            if (*letter == *name && !in_doubles)
            {
                matches++;
                name++;
                if (matches == strlen(vch->name))
                {
                    strcat(temp,"you");
                    last[0] = '\0';
                    name = vch->name;
                    continue;
                }
                strncat(last,letter,1);
                continue;
            }

	    if (*letter == '\"')
	    {
		in_doubles = !in_doubles;
	    }

            matches = 0;
            strcat(temp,last);
            strncat(temp,letter,1);
            last[0] = '\0';
            name = vch->name;
        }

	stc(temp,vch);
	stc("{c\n\r",vch);

    }

    if ( !IS_NPC( ch ) )
    {
	    ch->pcdata->summary[SUM_MOTES] += 1;
	    ch->pcdata->summary[SUM_MOTES_EVER] += 1;
	}
	if ( IS_SET( ch->comm, COMM_RP ) && !IS_NPC (ch) )
	{
		rplog_command(ch, argument);
	}
    return;
}

void do_nmote(CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *vch;
    /*
    char *letter,*name;
    char last[MAX_INPUT_LENGTH], temp[MAX_STRING_LENGTH];
    int matches = 0;
    */

    if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE) )
    {
        stc("{cYou can't show your emotions.\n\r", ch );
        return;
    }

    if ( argument[0] == '\0' )
    {
        stc("{cEmote what?\n\r", ch );
        return;
    }

    if (strstr(argument,ch->name) == NULL)
    {
	stc("{cYou must include your name in an smote.\n\r",ch);
	return;
    }

    stc(argument,ch);
    stc("{c\n\r",ch);

    for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
    {
        if (vch->desc == NULL || vch == ch)
            continue;
        if (!IS_AWAKE(vch))
            continue;
/*
        if ((letter = strstr(argument,vch->name)) == NULL)
        {
	    stc(argument,vch);
	    stc("{c\n\r",vch);
            continue;
        }

        strcpy(temp,argument);
        temp[strlen(argument) - strlen(letter)] = '\0';
        last[0] = '\0';
        name = vch->name;

        for (; *letter != '\0'; letter++)
        {
            if (*letter == '\'' && matches == strlen(vch->name))
            {
                strcat(temp,"r");
                continue;
            }

            if (*letter == 's' && matches == strlen(vch->name))
            {
                matches = 0;
                continue;
            }

            if (matches == strlen(vch->name))
            {
                matches = 0;
            }

            if (*letter == *name)
            {
                matches++;
                name++;
                if (matches == strlen(vch->name))
                {
                    strcat(temp,"you");
                    last[0] = '\0';
                    name = vch->name;
                    continue;
                }
                strncat(last,letter,1);
                continue;
            }

            matches = 0;
            strcat(temp,last);
            strncat(temp,letter,1);
            last[0] = '\0';
            name = vch->name;
        }
	stc(temp,vch);
	stc("{c\n\r",vch);
*/
	    stc(argument,vch);
	    stc("{c\n\r",vch);
    }

    if ( !IS_NPC( ch ) )
    {
	    ch->pcdata->summary[SUM_MOTES] += 1;
	    ch->pcdata->summary[SUM_MOTES_EVER] += 1;
	}

    return;
}

void do_bamfin( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( !IS_NPC(ch) )
    {
	smash_tilde( argument );

	if (argument[0] == '\0')
	{
	    sprintf(buf,"{cYour poofin is %s\n\r",ch->pcdata->bamfin);
	    stc(buf,ch);
	    return;
	}

	if ( strstr(argument,ch->name) == NULL)
	{
	    stc("{cYou must include your name.\n\r",ch);
	    return;
	}

	free_string( ch->pcdata->bamfin );
	ch->pcdata->bamfin = str_dup( argument );

        sprintf(buf,"{cYour poofin is now %s\n\r",ch->pcdata->bamfin);
        stc(buf,ch);
    }
    return;
}

void do_bamfout( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( !IS_NPC(ch) )
    {
        smash_tilde( argument );

        if (argument[0] == '\0')
        {
            sprintf(buf,"{cYour poofout is %s\n\r",ch->pcdata->bamfout);
            stc(buf,ch);
            return;
        }

        if ( strstr(argument,ch->name) == NULL)
        {
            stc("{cYou must include your name.\n\r",ch);
            return;
        }

        free_string( ch->pcdata->bamfout );
        ch->pcdata->bamfout = str_dup( argument );

        sprintf(buf,"{cYour poofout is now %s\n\r",ch->pcdata->bamfout);
        stc(buf,ch);
    }
    return;
}



void do_deny( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH],buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc("{cDeny whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
	stc("{cThey aren't here.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	stc("{cNot on NPC's.\n\r", ch );
	return;
    }

    if ( get_trust( victim ) >= get_trust( ch ) )
    {
        stc("{cYou failed.\n\r", ch );
        act("{w$n just tried to deny you!{c",ch,NULL,victim,TO_VICT);
        return;
    }

    SET_BIT(victim->act, PLR_DENY);
    stc("{cYou are denied access!\n\r", victim );
    sprintf(buf,"{c$N denies access to %s",victim->name);
    wiznet(buf,ch,NULL,WIZ_PENALTIES,WIZ_SECURE,0);
    stc("{cOK.\n\r", ch );
    //save_char_obj(victim);
    save_char_obj(victim, FALSE, FALSE);
    stop_all_fighting(victim);
    do_function(victim, &do_quit, "" );

    return;
}



void do_disconnect( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    DESCRIPTOR_DATA *d;
    CHAR_DATA *victim;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc("{cDisconnect whom?\n\r", ch );
	return;
    }

    if (is_number(arg))
    {
	int desc;

	desc = atoi(arg);
    	for ( d = descriptor_list; d != NULL; d = d->next )
    	{
            if ( d->descriptor == desc )
            {
            	close_socket( d );
            	stc("{cOk.\n\r", ch );
            	ch->pcdata->mod_disconnects++;
		return;
            }
	}
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
	stc("{cThey aren't here.\n\r", ch );
	return;
    }

    if ( IS_IMMORTAL( victim ) && victim->level >= get_trust( ch ) )
    {
        stc("{cYou failed.\n\r", ch );
        act("{w$n just tried to disconnect you!{c",ch,NULL,victim,TO_VICT);
        return;
    }

    if ( victim->desc == NULL )
    {
	act( "{c$N doesn't have a descriptor.", ch, NULL, victim, TO_CHAR );
	return;
    }

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
	if ( d == victim->desc )
	{
	    close_socket( d );
	    stc("{cCharacter Disconnected.\n\r", ch );
	    //read_mysql_staff( ch );
	    ch->pcdata->mod_disconnects++;
	   // update_staff( ch, "commands" );
	    return;
	}
    }

    bug( "Do_disconnect: desc not found.", 0 );
    stc("{cDescriptor not found!\n\r", ch );
    return;
}



void do_pardon( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
	stc("{cSyntax: pardon <character> <killer|thief>.\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	stc("{cThey aren't here.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	stc("{cNot on NPC's.\n\r", ch );
	return;
    }

    if ( !str_cmp( arg2, "killer" ) )
    {
	if ( IS_SET(victim->act, PLR_KILLER) )
	{
	    REMOVE_BIT( victim->act, PLR_KILLER );
	    stc("{cKiller flag removed.\n\r", ch );
	    stc("{cYou are no longer a KILLER.\n\r", victim );
	}
	return;
    }

    if ( !str_cmp( arg2, "thief" ) )
    {
	if ( IS_SET(victim->act, PLR_THIEF) )
	{
	    REMOVE_BIT( victim->act, PLR_THIEF );
	    stc("{cThief flag removed.\n\r", ch );
	    stc("{cYou are no longer a THIEF.\n\r", victim );
	}
	return;
    }

    stc("{cSyntax: pardon <character> <killer|thief>.\n\r", ch );
    return;
}



void do_echo( CHAR_DATA *ch, char *argument )
{
    DESCRIPTOR_DATA *d;

    if ( argument[0] == '\0' )
    {
	stc("{cGlobal echo what?\n\r", ch );
	return;
    }

    for ( d = descriptor_list; d; d = d->next )
    {
	if ( d->connected == CON_PLAYING )
	{
	    /*if (get_trust(d->character) >= get_trust(ch))
		stc("{cglobal{C>{c ",d->character);*/
	    stc( argument, d->character );
	    stc("{c\n\r",   d->character );
	}
    }

    return;
}



void do_recho( CHAR_DATA *ch, char *argument )
{
    DESCRIPTOR_DATA *d;

    if ( argument[0] == '\0' )
    {
	stc("{cLocal echo what?\n\r", ch );

	return;
    }

    for ( d = descriptor_list; d; d = d->next )
    {
	if ( d->connected == CON_PLAYING
	&&   d->character->in_room == ch->in_room )
	{
            /*if (get_trust(d->character) >= get_trust(ch))
                stc("{clocal{C>{c ",d->character);*/
	    stc( argument, d->character );
	    stc("{c\n\r",   d->character );
	}
    }

    return;
}

void do_zecho(CHAR_DATA *ch, char *argument)
{
    DESCRIPTOR_DATA *d;

    if (argument[0] == '\0')
    {
	stc("{cZone echo what?\n\r",ch);
	return;
    }

    for (d = descriptor_list; d; d = d->next)
    {
	if (d->connected == CON_PLAYING
	&&  d->character->in_room != NULL && ch->in_room != NULL
	&&  d->character->in_room->area == ch->in_room->area)
	{
	    /*if (get_trust(d->character) >= get_trust(ch))
		stc("{czone{C>{c ",d->character);*/
	    stc(argument,d->character);
	    stc("{c\n\r",d->character);
	}
    }
}

void do_pecho( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    argument = one_argument(argument, arg);

    if ( argument[0] == '\0' || arg[0] == '\0' )
    {
	stc("{cPersonal echo what?\n\r", ch);
	return;
    }

    if  ( (victim = get_char_world(ch, arg) ) == NULL )
    {
	stc("{cTarget not found.\n\r",ch);
	return;
    }

    /*if (get_trust(victim) >= get_trust(ch) && get_trust(ch) != MAX_LEVEL)
        stc("{cpersonal{C>{c ",victim);*/

    stc(argument,victim);
    stc("{c\n\r",victim);
    stc("{cpersonal{C>{c ",ch);
    stc(argument,ch);
    stc("{c\n\r",ch);
}


ROOM_INDEX_DATA *find_location( CHAR_DATA *ch, char *arg )
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;

    if ( is_number(arg) )
	return get_room_index( atoi( arg ) );

    if ( ( victim = get_char_world( ch, arg ) ) != NULL )
	return victim->in_room;

    if ( ( obj = get_obj_world( ch, arg ) ) != NULL )
	return obj->in_room;

    return NULL;
}



void do_transfer( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *location;
    DESCRIPTOR_DATA *d;
    CHAR_DATA *victim;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' )
    {
	stc("{cSyntax:\n\r", ch );
	stc("{c       <victim>            -Tranfers to you\n\r", ch );
	stc("{c       <victim> <location> -Transfers to location\n\r", ch );
	return;
    }

    if ( !str_cmp( arg1, "all" ) )
    {
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
	    if ( d->connected == CON_PLAYING
	    &&   d->character != ch
	    &&   d->character->in_room != NULL
	    &&   can_see( ch, d->character ) )
	    {
		char buf[MAX_STRING_LENGTH];
		sprintf( buf, "%s %s", d->character->name, arg2 );
		do_function(ch, &do_transfer, buf );
	    }
	}
	return;
    }

    /*
     * Thanks to Grodyn for the optional location parameter.
     */
    if ( arg2[0] == '\0' )
    {
	location = ch->in_room;
    }
    else
    {
	if ( ( location = find_location( ch, arg2 ) ) == NULL )
	{
	    stc("{cNo such location.\n\r", ch );
	    return;
	}

	if ( !is_room_owner(ch,location) && room_is_private( location )
	&&  get_trust(ch) < MAX_LEVEL)
	{
	    stc("{cThat room is private right now.\n\r", ch );
	    return;
	}
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
		stc("{cThey aren't here.\n\r", ch );
		return;
    }

    if ( IS_IMMORTAL( victim ) && victim->level >= get_trust( ch ) )
    {
        if ( !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
		{
	        stc("{cYou failed.\n\r", ch );
	        act("{w$n just tried to transfer you!{c",ch,NULL,victim,TO_VICT);
	        return;
		}
    }

    if ( victim->in_room == NULL )
    {
		stc("{cThey are in limbo.\n\r", ch );
		return;
    }

    if ( is_fighting(victim))
	    if ( victim->advancing_on )
		    stop_all_fighting(victim);
    act( "{cYou are drawn into a vortex and find yourself elsewhere.", NULL, NULL, NULL, TO_VICT );
    act( "{c$n is drawn into a yawning vortex.", victim, NULL, NULL, TO_ROOM );
    char_from_room( victim );
    char_to_room( victim, location );
    act( "{c$n arrives from the maw of a yawning vortex.", victim, NULL, NULL, TO_ROOM );
    if( MOUNTED(victim) )
    {
        char_from_room( MOUNTED(victim) );
        char_to_room( MOUNTED(victim), location );
        send_to_char("Your rider is being transferred, and so are you.\n\r", MOUNTED(victim));
    }

    if (ch->pet != NULL)
    {
   	char_from_room (ch->pet);
	char_to_room (ch->pet, location);
    }
    if ( ch != victim )
	act( "{c$n has transferred you.", ch, NULL, victim, TO_VICT );
    do_function(victim, &do_look, "auto" );
    stc("{cOk.\n\r", ch );
}



void do_at( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *location;
    ROOM_INDEX_DATA *original;
    OBJ_DATA *on;
    CHAR_DATA *wch;

    argument = one_argument( argument, arg );

    if ( arg[0] == '\0' || argument[0] == '\0' )
    {
	stc("{cAt where what?\n\r", ch );
	stc("{cAt <vnum|person> <command>\n\r", ch );
	return;
    }

    if ( ( location = find_location( ch, arg ) ) == NULL )
    {
	stc("{cNo such location.\n\r", ch );
	return;
    }

    if (!is_room_owner(ch,location) && room_is_private( location )
    &&  get_trust(ch) < MAX_LEVEL)
    {
	stc("{cThat room is private right now.\n\r", ch );
	return;
    }

    original = ch->in_room;
    on = ch->on;
    char_from_room( ch );
    char_to_room( ch, location );
    interpret( ch, argument );

    /*
     * See if 'ch' still exists before continuing!
     * Handles 'at XXXX quit' case.
     */
    for ( wch = char_list; wch != NULL; wch = wch->next )
    {
	if ( wch == ch )
	{
	    char_from_room( ch );
	    char_to_room( ch, original );
	    ch->on = on;
	    break;
	}
    }

    return;
}

void perform_goto( CHAR_DATA *ch, ROOM_INDEX_DATA *location )
{
    CHAR_DATA *rch;
    int count = 0;
    count = 0;

	if ( location->vnum <= 0 )
	{
		stc("That location is not valid. Perhaps the room doesn't exist?\n\r", ch);
		return;
	}

    if  (room_is_private(location))
    {
		stc("That room is private, nice try.\n\r", ch);
		return;
	}

    for ( rch = location->people; rch != NULL; rch = rch->next_in_room )
        count++;

    if ( is_fighting(ch) )
		stop_all_fighting( ch);

    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
    {
		if (get_trust(rch) >= ch->invis_level)
		{
		    if (ch->pcdata != NULL && ch->pcdata->bamfout[0] != '\0')
				act("{c$t",ch,ch->pcdata->bamfout,rch,TO_VICT);
		    else
				act("{c$n leaves in a swirling mist.",ch,NULL,rch,TO_VICT);
		}
    }

    if ( ch->position != POS_STANDING )
	do_function( ch, &do_stand, "" );

    char_from_room( ch );
    char_to_room( ch, location );

    if (ch->pet != NULL)
    {
    	char_from_room (ch->pet);
		char_to_room (ch->pet, location);
    }

    if( MOUNTED(ch) )
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), location );
        send_to_char("Your rider is a god, and did a goto. You went along for the ride.\n\r", MOUNTED(ch));
    }

    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
    {
        if (get_trust(rch) >= ch->invis_level)
        {
            if (ch->pcdata != NULL && ch->pcdata->bamfin[0] != '\0')
                act("{c$t",ch,ch->pcdata->bamfin,rch,TO_VICT);
            else
                act("{c$n appears in a swirling mist.",ch,NULL,rch,TO_VICT);
        }
    }

    do_function(ch, &do_look, "auto" );
    return;
}
void do_new_goto( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *location = NULL;
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || !str_cmp( arg1, "?" ) || !str_cmp( arg1, "help" ) )
    {
		stc("Goto where?\n\r", ch );
		stc("Goto Char  <PC Name>\n\r", ch );
		stc("Goto Mob   <Mobile Name>\n\r", ch );
		stc("Goto Mobv  <Mobile Vnum>\n\r", ch );
		stc("Goto Room  <Room Vnum>\n\r", ch );
		stc("Goto Obj   <Obj Name>\n\r", ch );
		stc("Goto Objv  <Obj Vnum>\n\r", ch );
		return;
    }

	/* working 05.01.2004 - Dalsor */
	if ( !str_cmp( arg1, "char" ) )
	{
		CHAR_DATA *victim;
		if ( ( victim = get_pc_world( ch, arg2 ) ) != NULL)
		{
			location = victim->in_room;
			if ( location != NULL )
				perform_goto( ch, location );
			else
			{
				stc("The location appears to be null.\n\r", ch);
				return;
			}
		}
		else
		{
			stc("No such PC in the world.\n\r", ch);
			return;
		}
	}
	/* working 05.01.2004 - Dalsor */
	if ( !str_cmp( arg1, "mob" ) )
	{
		CHAR_DATA *victim;
		if ( ( victim = get_mob_world( ch, arg2 ) ) != NULL )
		{
			if ( !IS_NPC( victim ) )
			{
				stc("For some reason, get_mob_world returned a PC\n\r", ch);
				return;
			}
			if ( ( location = victim->in_room ) != NULL )
				perform_goto( ch, location );
			else
			{
				stc("The location appears to be null.\n\r", ch);
				return;
			}
		}
		else
		{
			stc("Could not find that mobile.\n\r", ch);
			return;
		}
	}
	/* working 05.01.2004 - Dalsor */
	if ( !str_cmp( arg1, "mobv" ) )
	{
		CHAR_DATA *victim;
		MOB_INDEX_DATA *pMob;
		long vnum;
		if ( !is_number( arg2 ) )
		{
			stc("Location must be a valid mobile vnum.\n\r", ch);
			return;
		}
		vnum = atol( arg2 );
		if ( !( pMob = get_mob_index( vnum ) ) )
		{
			stc("No such mobile index.\n\r", ch);
			return;
		}
		if ( ( victim = get_mob_by_vnum( ch, arg2 ) ) != NULL )
		{
			if ( ( location = victim->in_room ) != NULL )
			{
				if ( location->vnum <= 0 )
				{
					stc("The object does not appear to be reset in the world.\n\r", ch);
					return;
				}
				else
					perform_goto( ch, location );
			}
			else
			{
				stc("The location appears to be null.\n\r", ch);
				return;
			}
		}
		else
		{
			stc("Could not find that mobile.\n\r", ch);
			return;
		}
	}
	/* working 05.01.2004 - Dalsor */
	if ( !str_cmp( arg1, "room" ) )
	{
		long vnum;
		if ( !is_number( arg2 ) )
		{
			stc("Location must be a valid room vnum.\n\r", ch);
			return;
		}
		vnum = atol( arg2 );
		if ( ( location = get_room_index( vnum ) ) != NULL )
			perform_goto( ch, location );
		else
		{
			stc("The location appears to be null.\n\r", ch);
			return;
		}
	}
	/* working 05.01.2004 - Dalsor */
	if ( !str_cmp( arg1, "obj" ) )
	{
		OBJ_DATA *obj;
		if ( ( obj = get_obj_world( ch, arg2 ) ) != NULL )
		{
			/*
			if ( obj->carried_by != NULL )
			{
				ptc( ch, "%s is carried by %s in room %ld.",
					obj->short_descr, obj->carried_by->name, obj->carried_by->in_room->vnum );
				location = obj->carried_by->in_room;
			}
			else
				location = obj->in_room;
			*/
			while(obj->carried_by ==NULL && obj->in_room == NULL)
			{
				ptc( ch, "%s is in %s.\n\r",
					obj->short_descr, obj->in_obj->short_descr);
				if(!obj->in_obj)
				{
					ptc(ch,"%s is not in_room, carried_by or in_obj.\n\r",obj->short_descr);
					return;
				}
				obj = obj->in_obj;
			}
			if ( obj->carried_by != NULL )
			{
				ptc( ch, "%s is carried by %s in room %ld.\n\r",
					obj->short_descr, obj->carried_by->name, obj->carried_by->in_room->vnum );
				location = obj->carried_by->in_room;
			}
			else
			{
				ptc( ch, "%s is in room %ld.\n\r",
						obj->short_descr, obj->in_room->vnum);
				location = obj->in_room;
			}
		}
		if ( location != NULL )
			perform_goto( ch, location );
		else
		{
			stc("The location appears to be null.\n\r", ch);
			return;
		}
	}
	/* working 05.01.2004 - Dalsor */
	if ( !str_cmp( arg1, "objv" ) )
	{
		OBJ_DATA *obj;
		OBJ_INDEX_DATA *pObj;
		long vnum;
		if ( !is_number( arg2 ) )
		{
			stc("Location must be a valid object vnum.\n\r", ch);
			return;
		}
		vnum = atol( arg2 );
		if ( !( pObj = get_obj_index( vnum ) ) )
		{
			stc("No such object index.\n\r", ch);
			return;
		}
		if ( ( obj = get_obj_by_vnum( ch, arg2 ) ) != NULL )
		{
			/*
			if ( obj->carried_by != NULL )
			{
				ptc( ch, "%s is carried by %s in room %ld.",
					obj->short_descr, obj->carried_by->name, obj->carried_by->in_room->vnum );
				location = obj->carried_by->in_room;
			}
			else
				location = obj->in_room;
			*/
			while(obj->carried_by ==NULL && obj->in_room == NULL)
			{
				ptc( ch, "%s is in %s.\n\r",
					obj->short_descr, obj->in_obj->short_descr);
				if(!obj->in_obj)
				{
					ptc(ch,"%s is not in_room, carried_by or in_obj.\n\r",obj->short_descr);
					return;
				}
				obj = obj->in_obj;
			}
			if ( obj->carried_by != NULL )
			{
				ptc( ch, "%s is carried by %s in room %ld.\n\r",
					obj->short_descr, obj->carried_by->name, obj->carried_by->in_room->vnum );
				location = obj->carried_by->in_room;
			}
			else
			{
				ptc( ch, "%s is in room %ld.\n\r",
						obj->short_descr, obj->in_room->vnum);
				location = obj->in_room;
			}
		}
		/* redundant and crash prone *
		if ( location->vnum <= 0 )
		{
			stc("The object does not appear to be reset in the world.\n\r", ch);
			return;
		} */
		if ( location != NULL )
		{
			if ( location->vnum <= 0 )
			{
				stc("The object does not appear to be reset in the world.\n\r", ch);
				return;
			}
			else
				perform_goto( ch, location );
		}
		else
		{
			stc("The location appears to be null.\n\r", ch);
			return;
		}
	}
    return;
}

void do_goto( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *location;
    CHAR_DATA *rch;
    int count = 0;

    if ( argument[0] == '\0' )
    {
	stc("{cGoto where?\n\r", ch );
	return;
    }

    if ( ( location = find_location( ch, argument ) ) == NULL )
    {
	stc("{cNo such location.\n\r", ch );
	return;
    }

    count = 0;
    for ( rch = location->people; rch != NULL; rch = rch->next_in_room )
        count++;

    if (!is_room_owner(ch,location) && room_is_private(location)
    &&  (count > 1 || get_trust(ch) < MAX_LEVEL))
    {
	stc("{cThat room is private right now.\n\r", ch );
	return;
    }

    if ( is_fighting(ch) )
	stop_all_fighting( ch);

    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
    {
	if (get_trust(rch) >= ch->invis_level)
	{
	    if (ch->pcdata != NULL && ch->pcdata->bamfout[0] != '\0')
		act("{c$t",ch,ch->pcdata->bamfout,rch,TO_VICT);
	    else
		act("{c$n leaves in a swirling mist.",ch,NULL,rch,TO_VICT);
	}
    }

    char_from_room( ch );
    char_to_room( ch, location );

    if (ch->pet != NULL)
    {
    	char_from_room (ch->pet);
	char_to_room (ch->pet, location);
    }

    if( MOUNTED(ch) )
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), location );
        send_to_char("Your rider is a god, and did a goto. You went along for the ride.\n\r", MOUNTED(ch));
    }

    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
    {
        if (get_trust(rch) >= ch->invis_level)
        {
            if (ch->pcdata != NULL && ch->pcdata->bamfin[0] != '\0')
                act("{c$t",ch,ch->pcdata->bamfin,rch,TO_VICT);
            else
                act("{c$n appears in a swirling mist.",ch,NULL,rch,TO_VICT);
        }
    }

    do_function(ch, &do_look, "auto" );
    return;
}

void do_violate( CHAR_DATA *ch, char *argument )
{
    ROOM_INDEX_DATA *location;
    CHAR_DATA *rch;

    if ( argument[0] == '\0' )
    {
        stc("{cGoto where?\n\r", ch );
        return;
    }

    if ( ( location = find_location( ch, argument ) ) == NULL )
    {
        stc("{cNo such location.\n\r", ch );
        return;
    }

    if (!room_is_private( location ))
    {
        stc("{cThat room isn't private, use goto.\n\r", ch );
        return;
    }

    if ( is_fighting(ch))
        stop_all_fighting( ch);

    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
    {
        if (get_trust(rch) >= ch->invis_level)
        {
            if (ch->pcdata != NULL && ch->pcdata->bamfout[0] != '\0')
                act("{c$t",ch,ch->pcdata->bamfout,rch,TO_VICT);
            else
                act("{c$n leaves in a swirling mist.",ch,NULL,rch,TO_VICT);
        }
    }

    char_from_room( ch );
    char_to_room( ch, location );

    if (ch->pet != NULL)
    {
    	char_from_room (ch->pet);
	char_to_room (ch->pet, location);
    }

    if( MOUNTED(ch) )
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), location );
        send_to_char("Your rider is a god, and did a violate. You went along for the ride.\n\r", MOUNTED(ch));
    }

    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room)
    {
        if (get_trust(rch) >= ch->invis_level)
        {
            if (ch->pcdata != NULL && ch->pcdata->bamfin[0] != '\0')
                act("{c$t",ch,ch->pcdata->bamfin,rch,TO_VICT);
            else
                act("{c$n appears in a swirling mist.",ch,NULL,rch,TO_VICT);
        }
    }

    do_function(ch, &do_look, "auto" );
    return;
}

/* RT to replace the 3 stat commands */

void do_stat ( CHAR_DATA *ch, char *argument )
{
   char arg[MAX_INPUT_LENGTH];
   char *string;
   OBJ_DATA *obj;
   ROOM_INDEX_DATA *location;
   CHAR_DATA *victim;

   string = one_argument(argument, arg);
   if ( arg[0] == '\0')
   {
	stc("{cSyntax:\n\r",ch);
	stc("{c  {cstat <name>\n\r",ch);
	stc("{c  {cstat obj <name>\n\r",ch);
	stc("{c  {cstat mob <name>\n\r",ch);
 	stc("{c  {cstat room <number>\n\r",ch);
	return;
   }

   if (!str_cmp(arg,"room"))
   {
	do_function(ch, &do_rstat, string);
	return;
   }

   if (!str_cmp(arg,"obj"))
   {
	do_function(ch, &do_ostat, string);
	return;
   }

   if(!str_cmp(arg,"char")  || !str_cmp(arg,"mob"))
   {
	do_function(ch, &do_mstat, string);
	return;
   }

   /* do it the old way */

   obj = get_obj_world(ch,argument);
   if (obj != NULL)
   {
     do_function(ch, &do_ostat, argument);
     return;
   }

  victim = get_char_world(ch,argument);
  if (victim != NULL)
  {
    do_function(ch, &do_mstat, argument);
    return;
  }

  location = find_location(ch,argument);
  if (location != NULL)
  {
    do_function(ch, &do_rstat, argument);
    return;
  }

  stc("{cNothing by that name found anywhere.\n\r",ch);
}

void do_rstat( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *location;
    OBJ_DATA *obj;
    CHAR_DATA *rch;
    int door;

    one_argument( argument, arg );
    location = ( arg[0] == '\0' ) ? ch->in_room : find_location( ch, arg );
    if ( location == NULL )
    {
	stc("{cNo such location.\n\r", ch );
	return;
    }

    if (!is_room_owner(ch,location) && ch->in_room != location
    &&  room_is_private( location ) && !IS_TRUSTED(ch,IMPLEMENTOR))
    {
	stc("{cThat room is private right now.\n\r", ch );
	return;
    }

    sprintf( buf, "{cName: '%s'\n\rArea: '%s'\n\r",
	location->name,
	location->area->name );
    stc( buf, ch );

    sprintf( buf,
	"{cVnum{w:{c %ld  Sector{w:{c %d  Light{w:{c %d  Healing{w:{c %d  Mana{w:{c %d\n\r",
	location->vnum,
	location->sector_type,
	location->light,
	location->heal_rate,
	location->mana_rate );
    stc( buf, ch );

    sprintf( buf,
	"{cRoom flags{w:{c %d.\n\rDescription{w:{c\n\r%s",
	location->room_flags,
	location->description );
    stc( buf, ch );

    if ( location->extra_descr != NULL )
    {
	EXTRA_DESCR_DATA *ed;

	stc("{cExtra description keywords: '", ch );
	for ( ed = location->extra_descr; ed; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
		stc("{c ", ch );
	}
	stc("{c'.\n\r", ch );
    }

    stc("{cCharacters:", ch );
    for ( rch = location->people; rch; rch = rch->next_in_room )
    {
	if (can_see(ch,rch))
        {
	    stc("{c ", ch );
	    one_argument( rch->name, buf );
	    stc( buf, ch );
	}
    }

    stc("{c.\n\rObjects:   ", ch );
    for ( obj = location->contents; obj; obj = obj->next_content )
    {
	stc("{c ", ch );
	one_argument( obj->name, buf );
	stc( buf, ch );
    }
    stc("{c.\n\r", ch );

    for ( door = 0; door < MAX_DIR; door++ )
    {
	EXIT_DATA *pexit;

	if ( ( pexit = location->exit[door] ) != NULL )
	{
	    sprintf( buf,
		"{cDoor{w:{c %d.  To{w:{c %ld.  Key{w:{c %ld.  Exit flags{w:{c %d.\n\rKeyword{w:{c '%s'.  Description{w:{c %s",
		door,
		(pexit->u1.to_room == NULL ? -1 : pexit->u1.to_room->vnum),
	    	pexit->key,
	    	pexit->exit_info,
	    	pexit->keyword,
	    	pexit->description[0] != '\0'
		    ? pexit->description : "{c(none).\n\r" );
	    stc( buf, ch );
	}
    }

    return;
}



void do_ostat( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    AFFECT_DATA *paf;
    OBJ_DATA *obj;
    bool showContContains = FALSE;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc("{cStat what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_world( ch, argument ) ) == NULL )
    {
	stc("{cNothing like that under Light or Shadow.\n\r", ch );
	return;
    }

    ptc( ch, "{cName{C({cs{C){w:{c %s Of the %s Guild.\n\r",
		obj->name, obj->guild );

    ptc( ch, "{cVnum{w:{c %d  Format{w:{c %s  Type{w:{c %s  Resets{w:{c %d\n\r",
		obj->pIndexData->vnum, obj->pIndexData->new_format ? "new" : "old",
		item_name(obj->item_type), obj->pIndexData->reset_num );

    ptc( ch, "{cShort description{w:{c %s\n\rLong description{w:{c %s\n\r",
		obj->short_descr, obj->description );

	if ( obj->owner != NULL && obj->owner[0] != '\0' )
		ptc( ch, "Owned by: %s.    ", obj->owner );
	else
		stc( "Owned by: None.   ", ch );

	ptc(ch, "Dye Vnum: %d\n\r", obj->dye_vnum);

    ptc( ch, "{cWear bits{w:{c %s\n\rExtra bits{w:{c %s\n\r",
		wear_bit_name(obj->wear_flags), extra_bit_name( obj->extra_flags ) );

    ptc( ch, "{cNumber{w:{c %d/%d  Weight{w:{c %d/%d/%d {C({c10th pounds{C){c\n\r",
		1,           get_obj_number( obj ),
		obj->weight, get_obj_weight( obj ),get_true_weight(obj) );

    ptc( ch, "{cLevel{w:{c %d  Cost{w:{c %d  Condition{w:{c %d  Timer{w:{c %d\n\r",
		obj->level, obj->cost, obj->condition, obj->timer );

    ptc( ch, "{cIn room{w:{c %d  In object{w:{c %s  Carried by{w:{c %s  Wear_loc{w:{c %d\n\r",
		obj->in_room    == NULL    ?        0 : obj->in_room->vnum,
		obj->in_obj     == NULL    ? "{C({cnone{C){c" : obj->in_obj->short_descr,
		obj->carried_by == NULL    ? "{C({cnone{C){c" :
	    can_see(ch,obj->carried_by) ? obj->carried_by->name	: "someone", obj->wear_loc );

    ptc( ch, "{cValues{w:{c %d %d %d %d %d\n\r",
		obj->value[0], obj->value[1], obj->value[2], obj->value[3],
		obj->value[4] );

    if ( obj->item_type == ITEM_TREASURECHEST )
    {
		if ( obj->trapped )
		{
			stc( "It is trapped.\n\r", ch);
		}
		if ( obj->value[3] > TRAP_NONE )
		{
		ptc( ch, "You see a %s.\n\r", obj->value[3] == TRAP_BOOMER ? "small black cube inside the lock" :
			obj->value[3] == TRAP_GASSER ? "small green cannister hidden within the lock" :
			obj->value[3] == TRAP_NEEDLE ? "small needle dripping with a green liquid inside the lock" :
			obj->value[3] == TRAP_BLADE ? "razor sharp blade hidden within the lock" :
			obj->value[3] == TRAP_MANA ? "small shimmering sphere tucked inside the lock" :
			obj->value[3] == TRAP_FIRE ? "reddish-gold casement placed behind the lock" :
			obj->value[3] == TRAP_ACID ? "bluish-green vial rigged to burst inside the lock" :
			"odd shape within the lock");
		}
        if ( obj->contains != NULL )
        {
			stc("This box holds some items. See below output.\n\r", ch);
           	showContContains = TRUE;
		}
        else if ( obj->contains == NULL )
        {
			stc("This box holds no items.\n\r", ch);
		}
	}

    /* now give out vital statistics as per identify */

    switch ( obj->item_type )
    {
    	case ITEM_SCROLL:
    	case ITEM_POTION:
    	case ITEM_PILL:
	    sprintf( buf, "{cLevel %d spells of{w:{c", obj->value[0] );
	    stc( buf, ch );

	    if ( obj->value[1] >= 0 && obj->value[1] < MAX_SKILL )
	    {
	    	stc("{c '", ch );
	    	stc( skill_table[obj->value[1]].name, ch );
	    	stc("{c'", ch );
	    }

	    if ( obj->value[2] >= 0 && obj->value[2] < MAX_SKILL )
	    {
	    	stc("{c '", ch );
	    	stc( skill_table[obj->value[2]].name, ch );
	    	stc("{c'", ch );
	    }

	    if ( obj->value[3] >= 0 && obj->value[3] < MAX_SKILL )
	    {
	    	stc("{c '", ch );
	    	stc( skill_table[obj->value[3]].name, ch );
	    	stc("{c'", ch );
	    }

	    if (obj->value[4] >= 0 && obj->value[4] < MAX_SKILL)
	    {
		stc("{c '",ch);
		stc(skill_table[obj->value[4]].name,ch);
		stc("{c'",ch);
	    }

	    stc("{c.\n\r", ch );
	break;

    case ITEM_QUIVER:
		sprintf(buf,"{WIt holds {R%d %d{Wd{R%d {Warrows.{x\n\r",
		obj->value[0], obj->value[1], obj->value[2]);
		stc(buf,ch);
		break;
	case ITEM_ARROW:
		sprintf(buf,"{WThis arrow will do {R%d{Wd{R%d {Wdamage for an average of {R%d{W.{x\n\r",
		obj->value[1], obj->value[2], ( obj->value[1] + obj->value[2] ) / 2 );
		stc(buf,ch);
		break;

    	case ITEM_WAND:
    	case ITEM_STAFF:
	    sprintf( buf, "{cHas %d{C({c%d{C){c charges of level %d",
	    	obj->value[1], obj->value[2], obj->value[0] );
	    stc( buf, ch );

	    if ( obj->value[3] >= 0 && obj->value[3] < MAX_SKILL )
	    {
	    	stc("{c '", ch );
	    	stc( skill_table[obj->value[3]].name, ch );
	    	stc("{c'", ch );
	    }

	    stc("{c.\n\r", ch );
	break;

	case ITEM_DRINK_CON:
	    sprintf(buf,"{cIt holds %s-colored %s.\n\r",
		liq_table[obj->value[2]].liq_color,
		liq_table[obj->value[2]].liq_name);
	    stc(buf,ch);
	    break;


    	case ITEM_WEAPON:
 	    stc("{cWeapon type is ",ch);
	    switch (obj->value[0])
	    {
	    	case(WEAPON_EXOTIC):
		    stc("{cexotic\n\r",ch);
		    break;
	    	case(WEAPON_SWORD):
		    stc("{csword\n\r",ch);
		    break;
	    	case(WEAPON_DAGGER):
		    stc("{cdagger\n\r",ch);
		    break;
	    	case(WEAPON_SPEAR):
		    stc("{cspear\n\r",ch);
		    break;
	    	case(WEAPON_STAFF):
		    stc("{cstaff\n\r",ch);
		    break;
	    	case(WEAPON_MACE):
		    stc("{cmace/club\n\r",ch);
		    break;
		   	case(WEAPON_AXE):
		    stc("{caxe\n\r",ch);
		    break;
	    	case(WEAPON_FLAIL):
		    stc("{cflail\n\r",ch);
		    break;
	    	case(WEAPON_WHIP):
		    stc("{cwhip\n\r",ch);
		    break;
	    	case(WEAPON_POLEARM):
		    stc("{cpolearm\n\r",ch);
		    break;
	    	case(WEAPON_SHORTBOW):
		    stc("{cshortbow\n\r",ch);
		    break;
	    	case(WEAPON_LONGBOW):
		    stc("{clongbow\n\r",ch);
		    break;
	    	case(WEAPON_RECURVEDBOW):
		    stc("{crecurved bow\n\r",ch);
		    break;
	    	case(WEAPON_COMPOSITEBOW):
		    stc("{ccomposite bow\n\r",ch);
		    break;
	    	case(WEAPON_CROSSBOW):
		    stc("{ccrossbow\n\r",ch);
		    break;
	    	default:
		    stc("{cunknown\n\r",ch);
		    break;
 	    }
	    if (obj->pIndexData->new_format)
	    	sprintf(buf,"{cDamage is %dd%d {C({caverage %d{C){c\n\r",
		    obj->value[1],obj->value[2],
		    (1 + obj->value[2]) * obj->value[1] / 2);
	    else
	    	sprintf( buf, "{cDamage is %d to %d {C({caverage %d{C){c\n\r",
	    	    obj->value[1], obj->value[2],
	    	    ( obj->value[1] + obj->value[2] ) / 2 );
	    stc( buf, ch );

	    sprintf(buf,"{cDamage noun is %s.\n\r",
		(obj->value[3] > 0 && obj->value[3] < MAX_DAMAGE_MESSAGE) ?
		    attack_table[obj->value[3]].noun : "undefined");
	    stc(buf,ch);

	    if (obj->value[4])  /* weapon flags */
	    {
	        sprintf(buf,"{cWeapons flags: %s\n\r",
		    weapon_bit_name(obj->value[4]));
	        stc(buf,ch);
            }
	break;

    	case ITEM_ARMOR:
	    sprintf( buf,
	    "{cArmor class is %d pierce, %d bash, %d slash, and %d vs. magic\n\r",
	        obj->value[0], obj->value[1], obj->value[2], obj->value[3] );
	    stc( buf, ch );
	break;

        case ITEM_CONTAINER:
            sprintf(buf,"{cCapacity{w:{c %d#  Maximum weight{w:{c %d#  flags{w:{c %s\n\r",
                obj->value[0], obj->value[3], cont_bit_name(obj->value[1]));
            stc(buf,ch);
            if (obj->value[4] != 100)
            {
                sprintf(buf,"{cWeight multiplier{w:{c %d%%\n\r",
		    obj->value[4]);
                stc(buf,ch);
            }
            if ( obj->contains != NULL )
            {
				stc("This container holds some items. See below output.\n\r", ch);
            	showContContains = TRUE;
			}
            else if ( obj->contains == NULL )
            {
				stc("This container holds no items.\n\r", ch);
			}
        break;
    }


    if ( obj->extra_descr != NULL || obj->pIndexData->extra_descr != NULL )
    {
	EXTRA_DESCR_DATA *ed;

	stc("{cExtra description keywords{w:{c '", ch );

	for ( ed = obj->extra_descr; ed != NULL; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
	    	stc("{c ", ch );
	}

	for ( ed = obj->pIndexData->extra_descr; ed != NULL; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
		stc("{c ", ch );
	}

	stc("{c'\n\r", ch );
    }

    for ( paf = obj->affected; paf != NULL; paf = paf->next )
    {
	sprintf( buf, "{cAffects %s by %d, level %d",
	    affect_loc_name( paf->location ), paf->modifier,paf->level );
	stc(buf,ch);
	if ( paf->duration > -1)
	    sprintf(buf,"{c, %d hours.\n\r",paf->duration);
	else
	    sprintf(buf,".\n\r");
	stc( buf, ch );
	if (paf->bitvector)
	{
	    switch(paf->where)
	    {
		case TO_AFFECTS:
		    sprintf(buf,"{cAdds %s affect.\n",
			affect_bit_name(paf->bitvector));
		    break;
		case TO_AFFECTS2:
		    sprintf(buf,"{cAdds %s affect.\n",
			affect_bit_name(paf->bitvector));
		    break;
                case TO_WEAPON:
                    sprintf(buf,"{cAdds %s weapon flags.\n",
                        weapon_bit_name(paf->bitvector));
		    break;
		case TO_OBJECT:
		    sprintf(buf,"{cAdds %s object flag.\n",
			extra_bit_name(paf->bitvector));
		    break;
		case TO_IMMUNE:
		    sprintf(buf,"{cAdds immunity to %s.\n",
			imm_bit_name(paf->bitvector));
		    break;
		case TO_RESIST:
		    sprintf(buf,"{cAdds resistance to %s.\n\r",
			imm_bit_name(paf->bitvector));
		    break;
		case TO_VULN:
		    sprintf(buf,"{cAdds vulnerability to %s.\n\r",
			imm_bit_name(paf->bitvector));
		    break;
		default:
		    sprintf(buf,"{cUnknown bit %d: %d\n\r",
			paf->where,paf->bitvector);
		    break;
	    }
	    stc(buf,ch);
	}
    }

    if (!obj->enchanted)
    for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
    {
	sprintf( buf, "{cAffects %s by %d, level %d.\n\r",
	    affect_loc_name( paf->location ), paf->modifier,paf->level );
	stc( buf, ch );
        if (paf->bitvector)
        {
            switch(paf->where)
            {
                case TO_AFFECTS:
                    sprintf(buf,"{cAdds %s affect.\n",
                        affect_bit_name(paf->bitvector));
                    break;
                case TO_OBJECT:
                    sprintf(buf,"{cAdds %s object flag.\n",
                        extra_bit_name(paf->bitvector));
                    break;
                case TO_IMMUNE:
                    sprintf(buf,"{cAdds immunity to %s.\n",
                        imm_bit_name(paf->bitvector));
                    break;
                case TO_RESIST:
                    sprintf(buf,"{cAdds resistance to %s.\n\r",
                        imm_bit_name(paf->bitvector));
                    break;
                case TO_VULN:
                    sprintf(buf,"{cAdds vulnerability to %s.\n\r",
                        imm_bit_name(paf->bitvector));
                    break;
                default:
                    sprintf(buf,"{cUnknown bit %d: %d\n\r",
                        paf->where,paf->bitvector);
                    break;
            }
            stc(buf,ch);
        }
    }
    if ( showContContains )
		show_list_to_char( obj->contains, ch, TRUE, TRUE, TRUE );

    return;
}

void do_mstat( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    AFFECT_DATA *paf, *paf_last = NULL;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc("{cStat whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, argument ) ) == NULL )
    {
	stc("{cThey aren't here.\n\r", ch );
	return;
    }

    /* Begin mstat revision
     * This just shows a clone of score to the imm
     */
    /* can also use show_score(victim,ch );  for up to date color version */
    if ( !IS_NPC( victim ) && victim->desc != NULL )
    {
	show_new_score( victim, ch );
    }


	stc(" ~--------------------------------------------------------------~\n\r",ch);

	ptc(ch, "| {cName: {C%s ", victim->name);

	if (!IS_NPC(victim))
		ptc(ch, "%s {cPSize: {C%d  {cFormat: {CPC  ", 
	    		victim->pcdata->last_name,
		    	victim->pcdata->pfile_size);
	else
		ptc(ch, "{cShort: {x%s  {cFormat: {C%s  ",
			victim->short_descr,
			victim->pIndexData->new_format ? "NEW" : "OLD");

	ptc (ch, "{C%s{c in room {C%d{x\n\r",
			position_table[victim->position].name,
			victim->in_room == NULL ? 0 : victim->in_room->vnum );

	if (IS_NPC(victim))
    ptc( ch, "| {cLong: {x%s{c",
		victim->long_descr[0] != '\0' ? victim->long_descr : "(none)\n\r" );

    ptc( ch, "| {cVnum: {C%d  {cRace: {C%s  {cGroup: {C%d  {cSex: {C%s  {cHR: {C%d  {cDR: {C%d{c {x\n\r",
		IS_NPC(victim) ? victim->pIndexData->vnum : 0,
		race_table[victim->race].name,
		IS_NPC(victim) ? victim->group : 0, sex_table[victim->sex].name,
 		GET_HITROLL(victim), GET_DAMROLL(victim));

    if (IS_NPC(victim))
    {
	sprintf(buf,"| {cCount{w:{c {C%d{c  Killed{w:{c {C%d{c\n\r",
	    victim->pIndexData->count,victim->pIndexData->killed);
	stc(buf,ch);
    }

    ptc( ch,
   	"| {cStr: {C%d({c%d{C)  {cInt: {C%d({c%d{C)  {cWis: {C%d({c%d{C)  {cDex: {C%d({c%d{C){c\n\r" \
   	"| {cCon: {C%d({c%d{C)  {cRef: {C%d({c%d{C)  {cIns: {C%d({c%d{C)  {cCha: {C%d({c%d{C){c\n\r",
		victim->perm_stat[STAT_STR], get_curr_stat(victim,STAT_STR),
		victim->perm_stat[STAT_INT], get_curr_stat(victim,STAT_INT),
		victim->perm_stat[STAT_WIS], get_curr_stat(victim,STAT_WIS),
		victim->perm_stat[STAT_DEX], get_curr_stat(victim,STAT_DEX),
		victim->perm_stat[STAT_CON], get_curr_stat(victim,STAT_CON),
		victim->perm_stat[STAT_REF], get_curr_stat(victim,STAT_REF),
		victim->perm_stat[STAT_INS], get_curr_stat(victim,STAT_INS),
		victim->perm_stat[STAT_CHA], get_curr_stat(victim,STAT_CHA) );

    ptc( ch, "| {cHp: {C%d{c/{C%d (%d)   {cSt: {C%d{c/{C%d{c (%d)   Prac: {C%d{c  Trn: {C%d{c\n\r",
		victim->hit, get_maxhit(victim), victim->max_hit,
		victim->stam, get_maxstam(victim), victim->max_stam,
		IS_NPC(ch) ? 0 : victim->practice,
		IS_NPC(ch) ? 0 : victim->train);

    ptc( ch, "| {cLv: {C%d(%d)  {cCP: {C%ld SP: {C%ld  SM: {C%ld{c  GM: {C%ld{c\n\r",
	victim->level, victim->exp,
	victim->money[0][0], victim->money[1][0], victim->money[2][0], victim->money[3][0] );

    ptc( ch, "| {cAC: {cP: {C%d {cB: {C%d {cS: {C%d {cM: {C%d{c  SV: {C%d  {cSZ: {C%s  {cWimp: {C%ld{c\n\r",
	    GET_AC(victim,AC_PIERCE), GET_AC(victim,AC_BASH),
	    GET_AC(victim,AC_SLASH),  GET_AC(victim,AC_EXOTIC),
		victim->saving_throw,
		size_table[victim->size].name,
		victim->wimpy );

    if (IS_NPC(victim) && victim->pIndexData->new_format)
    {
		ptc( ch, "| {cDmg:{C %d{cd{C%d {cMsg{w:{C  %s{c\n\r",
		    victim->damage[DICE_NUMBER],victim->damage[DICE_TYPE],
		    attack_table[victim->dam_type].noun);
    }

    ptc( ch, "| {cFighting{w:{C %s{c Range{w:{C %s{c  {cCarrying {C%d {citems ({C%d {clbs)\n\r",
		is_fighting(victim) ? victim->advancing_on->name : "{C({cnone{C){c",
		victim->range == RANGE_NONE ? "None" :
		victim->range == RANGE_MISSLE ? "Missle" :
		victim->range == RANGE_LONG ? "Long" : "Melee",
		victim->carry_number, get_carry_weight(victim) / 10 );

    if ( !IS_NPC(victim) )
    {
		ptc( ch,"| {cThirst:{C %d{c  Hunger:{C %d{c  Full:{C %d{c  Drunk:{C %d{c\n\r",
		    victim->pcdata->condition[COND_THIRST],
		    victim->pcdata->condition[COND_HUNGER],
		    victim->pcdata->condition[COND_FULL],
		    victim->pcdata->condition[COND_DRUNK] );
    }



    if (!IS_NPC(victim))
    {
    	ptc( ch,"| {cAge:{C %d{c  Played:{C %d{c  LastLvl:{C %d{c  Timer:{C %d{c  Security:{C %d\n\r",
		    get_age(victim),
		    (int) (victim->played + current_time - victim->logon) / 3600,
		    victim->pcdata->last_level,
		    victim->timer,
		    victim->pcdata->security );	/* OLC */
    }

    ptc( ch, "| {cAct:{C %s{c\n\r",act_bit_name(victim->act));

	if (!IS_NPC(victim))
	{
	    if (victim->comm)
	    {
	    	ptc(ch,"| {cComm:{C %s{c\n\r",comm_bit_name(victim->comm));
	    }
	}

    if (IS_NPC(victim) && victim->off_flags)
    {
    	ptc( ch, "| {cOff:{C %s{c\n\r",off_bit_name(victim->off_flags));
    }

    if (victim->imm_flags)
    {
		ptc( ch, "| {cImm: {C%s{c\n\r",imm_bit_name(victim->imm_flags));
    }

    if (victim->res_flags)
    {
		ptc( ch, "| {cResist: {C%s{c\n\r", imm_bit_name(victim->res_flags));
    }

    if (victim->vuln_flags)
    {
		ptc( ch, "| {cVuln: {C%s{c\n\r", imm_bit_name(victim->vuln_flags));
    }

    if (victim->war_flags || victim->op_flags)
    {
		ptc( ch, "| {cWarf: {C%s{c", victim->war_flags ? war_bit_name(victim->war_flags) : "none");
		ptc( ch, "  {cOneP: {C%s{c\n\r", victim->op_flags ? op_bit_name(victim->op_flags) : "none");
    }

    if (victim->hist_flags || victim->spec_flags)
    {
		ptc( ch, "| {cHist: {C%s{c", victim->hist_flags ? hist_bit_name(victim->hist_flags) : "none");
		ptc( ch, "  {cSpec: {C%s{c\n\r", victim->spec_flags ? spec_bit_name(victim->spec_flags) : "none");
    }

    if (victim->auto_flags)
    {
		ptc( ch, "| {cAuto: {C%s{c\n\r", auto_bit_name(victim->auto_flags));
    }

    ptc( ch, "| {cMaster: {C%s  {cLeader: {C%s  {cPet: {C%s  {cHorse: {C%s{c\n\r",
		victim->master      ? victim->master->name   : "{C({cnone{C){c",
		victim->leader      ? victim->leader->name   : "{C({cnone{C){c",
		victim->pet 	    ? victim->pet->name	     : "(none)",
		victim->mount 	    ? victim->mount->name    : "(none)");

    if ( IS_NPC( victim ) )
    {
		if ( victim->questor != NULL )
			ptc( ch, "| Being quested by %s. %d time remaining till poof.\n\r",
				victim->questor->name, victim->timer );
	}

    if ( IS_NPC(victim) && victim->spec_fun != 0 )
    {
		ptc( ch,"| {cMob has spec_proc {C%s{c.\n\r",
			spec_name(victim->spec_fun));
    }

    if (!IS_NPC(victim))
    {
		if ( victim->pcdata && victim->pcdata->allowed == 0 )
			ptc( ch,"| %s has NOT been verified.\n\r", victim->name);
		if ( victim->pcdata->warning > 0 )
			ptc( ch,"| %s has been warned %d times.\n\r", victim->name, victim->pcdata->warning );
	    if (IS_SET(victim->hist_flags,HIST_BONDED))
	    {
			ptc( ch,"| {c%s was bonded by %s.{c\n\r",victim->name,victim->pcdata->bonded_by);
	    }
	    if (IS_SET(victim->hist_flags,HIST_BONDER))
	    {
			ptc( ch,"| {c%s has bonded %s.{c\n\r",victim->name,victim->pcdata->bonded_to);
	    }
	}

    if (!IS_NPC(victim))
    {

	if (victim->pcdata->wkin[0] != '\0')
		ptc(ch, "| Walkin is %s\n\r", victim->pcdata->wkin);
	if (victim->pcdata->wout[0] != '\0')
        ptc(ch, "| Walkout is %s\n\r", victim->pcdata->wout);
	if (victim->pcdata->rdesc[0] != '\0')
    	ptc(ch, "| Rdesc is %s\n\r", victim->pcdata->rdesc);
    }

	stc(" ~--------------------------------------------------------------~\n\r",ch);

        if ( ch->pcdata->char_barrier > 0 )
        {
        	if ( IS_SET(ch->op_flags, OP_BARRIER) && ch->pcdata->char_barrier > 0 )
                        ptc( ch, "Affect: {wbarrier{c       : modifies damage by %d\n\r", ch->pcdata->char_barrier );
        }

        for ( paf = ch->affected; paf != NULL; paf = paf->next )
        {
            if (paf_last != NULL && paf->type == paf_last->type)
                if (ch->level >= 1)
                    sprintf( buf, "{w                      {c");
                else
                    continue;
            else
                sprintf( buf, "{cAffect:{w %-15s{c", skill_table[paf->type].name );

            stc( buf, ch );

            if ( ch->level >= 1 )
            {
                sprintf( buf,
                    "{c: modifies %s by %d{c ",
                    affect_loc_name( paf->location ),
                    paf->modifier);
                stc( buf, ch );
                if ( paf->duration == -1 )
                    sprintf( buf, "{cpermanently{c" );
                else
                    sprintf( buf, "{cfor %d hours{c", paf->duration );
                stc( buf, ch );
            }

            stc( "{c\n\r", ch );
		paf_last = paf;
	}

    return;
}

/* ofind and mfind replaced with vnum, vnum skill also added */

void do_vnum(CHAR_DATA *ch, char *argument)
{
    char arg[MAX_INPUT_LENGTH];
    char *string;

    string = one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  vnum obj <name>\n\r",ch);
	stc("{c  vnum mob <name>\n\r",ch);
	stc("{c  vnum skill <skill or spell>\n\r",ch);
	return;
    }

    if (!str_cmp(arg,"obj"))
    {
	do_function(ch, &do_ofind, string);
 	return;
    }

    if (!str_cmp(arg,"mob") || !str_cmp(arg,"char"))
    {
	do_function(ch, &do_mfind, string);
	return;
    }

    if (!str_cmp(arg,"skill") || !str_cmp(arg,"spell"))
    {
	do_function (ch, &do_slookup, string);
	return;
    }
    /* do both */
    do_function(ch, &do_mfind, argument);
    do_function(ch, &do_ofind, argument);
}


void do_mfind( CHAR_DATA *ch, char *argument )
{
    extern int top_mob_index;
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    MOB_INDEX_DATA *pMobIndex;
    int vnum;
    int nMatch;
    bool fAll;
    bool found;
    BUFFER *output;
    output = new_buf();

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc("{cFind whom?\n\r", ch );
	return;
    }

    fAll	= FALSE; /* !str_cmp( arg, "all" ); */
    found	= FALSE;
    nMatch	= 0;

    /*
     * Yeah, so iterating over all vnum's takes 10,000 loops.
     * Get_mob_index is fast, and I don't feel like threading another link.
     * Do you?
     * -- Furey
     */
    for ( vnum = 0; nMatch < top_mob_index; vnum++ )
    {
	if ( ( pMobIndex = get_mob_index( vnum ) ) != NULL )
	{
	    nMatch++;
	    if ( fAll || is_name( argument, pMobIndex->player_name ) )
	    {
		found = TRUE;
		sprintf( buf, "{C[{c%5ld{C]{c %s\n\r",
		    pMobIndex->vnum, pMobIndex->short_descr );
	    	add_buf(output,buf);
	/*	stc( buf, ch );  */
	    }
	}
    }

    if ( !found )
	stc("{cNo mobiles by that name.\n\r", ch );
    else
    	page_to_char( buf_string(output), ch );
    free_buf(output);
    return;
}



void do_ofind( CHAR_DATA *ch, char *argument )
{
    extern int top_obj_index;
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    OBJ_INDEX_DATA *pObjIndex;
    int vnum;
    int nMatch;
    bool fAll;
    bool found;
    BUFFER *output;
    output = new_buf();

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc("{cFind what?\n\r", ch );
	return;
    }

    fAll	= FALSE; /* !str_cmp( arg, "all" ); */
    found	= FALSE;
    nMatch	= 0;

    /*
     * Yeah, so iterating over all vnum's takes 10,000 loops.
     * Get_obj_index is fast, and I don't feel like threading another link.
     * Do you?
     * -- Furey
     */
    for ( vnum = 0; nMatch < top_obj_index; vnum++ )
    {
	if ( ( pObjIndex = get_obj_index( vnum ) ) != NULL )
	{
	    nMatch++;
	    if ( fAll || is_name( argument, pObjIndex->name ) )
	    {
		found = TRUE;
		sprintf( buf, "{C[{c%5ld{C]{c %s\n\r",
		    pObjIndex->vnum, pObjIndex->short_descr );
	    	add_buf(output,buf);
		/*stc( buf, ch ); */
	    }
	}
    }

    if ( !found )
	stc("{cNo objects by that name.\n\r", ch );
    else
    	page_to_char( buf_string(output), ch );
    free_buf(output);

    return;
}

void do_reboo( CHAR_DATA *ch, char *argument )
{
    stc("{cIf you want to REBOOT, spell it out.\n\r", ch );
    return;
}

void do_reboot( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    extern bool merc_down;
    DESCRIPTOR_DATA *d,*d_next;
    CHAR_DATA *vch;

    if (ch->invis_level < LEVEL_HERO)
    {
    	sprintf( buf, "{cReboot by %s.", ch->name );
    	do_function(ch, &do_echo, buf );
    }

    save_gquest_data();
    save_donation_pit();

    merc_down = TRUE;
    for ( d = descriptor_list; d != NULL; d = d_next )
    {
    	d_next = d->next;
    	vch = d->original ? d->original : d->character;
	    if (vch != NULL)
  	    //save_char_obj(vch);
	    save_char_obj(vch, FALSE, FALSE);
     	close_socket(d);
    }

	mysql_close(result_db);

    return;
}

void do_shutdow( CHAR_DATA *ch, char *argument )
{
    stc("{cIf you want to SHUTDOWN, spell it out.\n\r", ch );
    return;
}

void do_shutdown( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    extern bool merc_down;
    DESCRIPTOR_DATA *d,*d_next;
    CHAR_DATA *vch;

    save_gquest_data();
    save_donation_pit();

    if (ch->invis_level < LEVEL_HERO)
      sprintf( buf, "{cShutdown by %s.", ch->name );
    append_file( ch, SHUTDOWN_FILE, buf );
    strcat( buf, "\n\r" );
    if (ch->invis_level < LEVEL_HERO)
    {
    	do_function(ch, &do_echo, buf );
    }
    merc_down = TRUE;
    for ( d = descriptor_list; d != NULL; d = d_next)
    {
    	d_next = d->next;
    	vch = d->original ? d->original : d->character;
    	if (vch != NULL)
  	    //save_char_obj(vch);
	    save_char_obj(vch, FALSE, FALSE);
	    close_socket(d);
    }
    return;
}

void do_protect( CHAR_DATA *ch, char *argument)
{
    CHAR_DATA *victim;

    if (argument[0] == '\0')
    {
	stc("{cProtect whom from snooping?\n\r",ch);
	return;
    }

    if ((victim = get_char_world(ch,argument)) == NULL)
    {
	stc("{cYou can't find them.\n\r",ch);
	return;
    }

    if (IS_SET(victim->comm,COMM_SNOOP_PROOF))
    {
	act_new("{c$N is no longer snoop-proof.",ch,NULL,victim,TO_CHAR,POS_DEAD);
	stc("{cYour snoop-proofing was just removed.\n\r",victim);
	REMOVE_BIT(victim->comm,COMM_SNOOP_PROOF);
    }
    else
    {
	act_new("{c$N is now snoop-proof.",ch,NULL,victim,TO_CHAR,POS_DEAD);
	stc("{cYou are now immune to snooping.\n\r",victim);
	SET_BIT(victim->comm,COMM_SNOOP_PROOF);
    }
}



void do_snoop( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    DESCRIPTOR_DATA *d;
    CHAR_DATA *victim;
    char buf[MAX_STRING_LENGTH];

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc("{cSnoop whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }

    if ( victim->desc == NULL )
    {
	stc("{cNo descriptor to snoop.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	stc("{cCancelling all snoops.\n\r", ch );
	wiznet("{c$N stops being such a snoop.",
		ch,NULL,WIZ_SNOOPS,WIZ_SECURE,get_trust(ch));
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
	    if ( d->snoop_by == ch->desc )
		d->snoop_by = NULL;
	}
	return;
    }

    if ( victim->desc->snoop_by != NULL )
    {
	stc("{cBusy already.\n\r", ch );
	return;
    }

    if (!is_room_owner(ch,victim->in_room) && ch->in_room != victim->in_room
    &&  room_is_private(victim->in_room) && !IS_TRUSTED(ch,IMPLEMENTOR))
    {
        stc("{cThat character is in a private room.\n\r",ch);
        return;
    }

    if ( IS_IMMORTAL( victim ) && !IS_IMMORTAL( ch ) )
    {
	stc("{cYou failed.\n\r", ch );
	return;
    }

    if ( ch->desc != NULL )
    {
	for ( d = ch->desc->snoop_by; d != NULL; d = d->snoop_by )
	{
	    if ( d->character == victim || d->original == victim )
	    {
		stc("{cNo snoop loops.\n\r", ch );
		return;
	    }
	}
    }

    victim->desc->snoop_by = ch->desc;
    sprintf(buf,"{c$N starts snooping on %s",
	(IS_NPC(ch) ? victim->short_descr : victim->name));
    wiznet(buf,ch,NULL,WIZ_SNOOPS,WIZ_SECURE,get_trust(ch));
    stc("{cOk.\n\r", ch );
    return;
}



void do_switch( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH], buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );
	return;
    if ( arg[0] == '\0' )
    {
	stc("{cSwitch into whom?\n\r", ch );
	return;
    }

    if ( ch->desc == NULL )
	return;

    if ( ch->desc->original != NULL )
    {
	stc("{cYou are already switched.\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	stc("{cOk.\n\r", ch );
	return;
    }

    if (!IS_NPC(victim))
    {
	stc("{cYou can only switch into mobiles.\n\r",ch);
	return;
    }

    if (!is_room_owner(ch,victim->in_room) && ch->in_room != victim->in_room
    &&  room_is_private(victim->in_room) && !IS_TRUSTED(ch,IMPLEMENTOR))
    {
	stc("{cThat character is in a private room.\n\r",ch);
	return;
    }

    if ( victim->desc != NULL )
    {
	stc("{cCharacter in use.\n\r", ch );
	return;
    }

    sprintf(buf,"{c$N switches into %s",victim->short_descr);
    wiznet(buf,ch,NULL,WIZ_SWITCHES,WIZ_SECURE,get_trust(ch));

    ch->desc->character = victim;
    ch->desc->original  = ch;
    victim->desc        = ch->desc;
    ch->desc            = NULL;
    /* change communications to match */
    if (ch->prompt != NULL)
        victim->prompt = str_dup(ch->prompt);
    victim->comm = ch->comm;
    victim->lines = ch->lines;
    stc("{cOk.\n\r", victim );
    return;
}



void do_return( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( ch->desc == NULL )
		return;

    if ( ch->desc->original == NULL )
    {
		stc("{cYou arent switched.\n\r", ch );
		return;
    }

    stc("{cYou return to your original body. Type replay to see any missed tells.\n\r",	ch );
    if (ch->prompt != NULL)
    {
		free_string(ch->prompt);
		ch->prompt = NULL;
    }

    sprintf(buf,"{c$N returns from %s.",ch->short_descr);
    wiznet(buf,ch->desc->original,0,WIZ_SWITCHES,WIZ_SECURE,get_trust(ch));
    ch->desc->character       = ch->desc->original;
    ch->desc->original        = NULL;
    ch->desc->character->desc = ch->desc;
    ch->desc                  = NULL;
    return;
}

/* trust levels for load and clone */
bool obj_check (CHAR_DATA *ch, OBJ_DATA *obj)
{
    if (IS_TRUSTED(ch,GOD)
	|| (IS_TRUSTED(ch,IMMORTAL) && obj->level <= 20 && obj->cost <= 1000)
	|| (IS_TRUSTED(ch,DEMI)	    && obj->level <= 10 && obj->cost <= 500)
	|| (IS_TRUSTED(ch,ANGEL)    && obj->level <=  5 && obj->cost <= 250)
	|| (IS_TRUSTED(ch,AVATAR)   && obj->level ==  0 && obj->cost <= 100))
	return TRUE;
    else
	return FALSE;
}

/* for clone, to insure that cloning goes many levels deep */
void recursive_clone(CHAR_DATA *ch, OBJ_DATA *obj, OBJ_DATA *clone)
{
    OBJ_DATA *c_obj, *t_obj;


    for (c_obj = obj->contains; c_obj != NULL; c_obj = c_obj->next_content)
    {
	if (obj_check(ch,c_obj))
	{
	    t_obj = create_object(c_obj->pIndexData,0);
	    clone_object(c_obj,t_obj);
	    obj_to_obj(t_obj,clone);
	    recursive_clone(ch,c_obj,t_obj);
	}
    }
}

/* command that is similar to load */
void do_clone(CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    char *rest;
    CHAR_DATA *mob;
    OBJ_DATA  *obj;

    rest = one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	stc("{cClone what?\n\r",ch);
	return;
    }

    if (!str_prefix(arg,"object"))
    {
	mob = NULL;
	obj = get_obj_here(ch,NULL,rest);
	if (obj == NULL)
	{
	    stc("{cYou dont see that here.\n\r",ch);
	    return;
	}
    }
    else if (!str_prefix(arg,"mobile") || !str_prefix(arg,"character"))
    {
	obj = NULL;
	mob = get_char_room(ch,NULL,rest);
	if (mob == NULL)
	{
	    stc("{cYou dont see that here.\n\r",ch);
	    return;
	}
    }
    else /* find both */
    {
	mob = get_char_room(ch,NULL,argument);
	obj = get_obj_here(ch,NULL,argument);
	if (mob == NULL && obj == NULL)
	{
	    stc("{cYou dont see that here.\n\r",ch);
	    return;
	}
    }

    /* clone an object */
    if (obj != NULL)
    {
	OBJ_DATA *clone;

	clone = create_object(obj->pIndexData,0);
	clone_object(obj,clone);
	if (obj->carried_by != NULL)
	    obj_to_char(clone,ch);
	else
	    obj_to_room(clone,ch->in_room);
 	recursive_clone(ch,obj,clone);

	act("{c$n pulls $p from the Pattern.",ch,clone,NULL,TO_ROOM);
	act("{cYou pull $p from the Pattern.",ch,clone,NULL,TO_CHAR);
	wiznet("{c$N clones $p.",ch,clone,WIZ_LOAD,WIZ_SECURE,get_trust(ch));
	return;
    }
    else if (mob != NULL)
    {
	CHAR_DATA *clone;
	OBJ_DATA *new_obj;
	char buf[MAX_STRING_LENGTH];

	if (!IS_NPC(mob))
	{
	    stc("{cYou can only clone mobiles.\n\r",ch);
	    return;
	}

	if ((mob->level > 20 && !IS_TRUSTED(ch,GOD))
	||  (mob->level > 10 && !IS_TRUSTED(ch,IMMORTAL))
	||  (mob->level >  5 && !IS_TRUSTED(ch,DEMI))
	||  (mob->level >  0 && !IS_TRUSTED(ch,ANGEL))
	||  !IS_TRUSTED(ch,AVATAR))
	{
	    stc(
		"{cYour powers are not great enough for such a task.\n\r",ch);
	    return;
	}

	clone = create_mobile(mob->pIndexData);
	clone_mobile(mob,clone);

	for (obj = mob->carrying; obj != NULL; obj = obj->next_content)
	{
	    if (obj_check(ch,obj))
	    {
		new_obj = create_object(obj->pIndexData,0);
		clone_object(obj,new_obj);
		recursive_clone(ch,obj,new_obj);
		obj_to_char(new_obj,clone);
		new_obj->wear_loc = obj->wear_loc;
	    }
	}
	char_to_room(clone,ch->in_room);
        act("{c$n pulls $N from the Pattern.",ch,NULL,clone,TO_ROOM);
        act("{cYou pull $N from the Pattern.",ch,NULL,clone,TO_CHAR);
	sprintf(buf,"{c$N clones %s.",clone->short_descr);
	wiznet(buf,ch,NULL,WIZ_LOAD,WIZ_SECURE,get_trust(ch));
        return;
    }
}

/* RT to replace the two load commands */

void do_load(CHAR_DATA *ch, char *argument )
{
   char arg[MAX_INPUT_LENGTH];

    argument = one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  load mob <vnum>\n\r",ch);
	stc("{c  load obj <vnum> <level>\n\r",ch);
	return;
    }

    if (!str_cmp(arg,"mob") || !str_cmp(arg,"char"))
    {
	do_function(ch, &do_mload, argument);
	return;
    }

    if (!str_cmp(arg,"obj"))
    {
	do_function(ch, &do_oload, argument);
	return;
    }
    /* echo syntax */
    do_function(ch, &do_load, "");
}


void do_mload( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    MOB_INDEX_DATA *pMobIndex;
    AREA_DATA *pArea;
    CHAR_DATA *victim;
    char buf[MAX_STRING_LENGTH];

    one_argument( argument, arg );

    if ( arg[0] == '\0' || !is_number(arg) )
    {
	stc("{cSyntax: load mob <vnum>.\n\r", ch );
	return;
    }

    if ( ( pMobIndex = get_mob_index( atoi( arg ) ) ) == NULL )
    {
	stc("{cNo mob has that vnum.\n\r", ch );
	return;
    }

    pArea = pMobIndex->area;

    if ( !IS_BUILDER( ch, pArea ) )
    {
	stc( "{cInsufficient security to load mobile from that area.\n\r", ch );
	return;
    }

    victim = create_mobile( pMobIndex );
    char_to_room( victim, ch->in_room );
    act( "{c$n pulls $N from the Pattern!", ch, NULL, victim, TO_ROOM );
    sprintf(buf,"{c$N pulls %s from the Pattern.",victim->short_descr);
    wiznet(buf,ch,NULL,WIZ_LOAD,WIZ_SECURE,get_trust(ch));
    stc("{cOk.\n\r", ch );
    return;
}



void do_oload( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH] ,arg2[MAX_INPUT_LENGTH];
    OBJ_INDEX_DATA *pObjIndex;
    AREA_DATA *pArea;
    OBJ_DATA *obj;
    int level, guild;

    argument = one_argument( argument, arg1 );
    one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || !is_number(arg1))
    {
	stc("{cSyntax: load obj <vnum> <level>.\n\r", ch );
	return;
    }

    level = get_trust(ch); /* default */

    /*if ( arg2[0] != '\0')  load with a level - removed due to abuse - Dalsor
    {
		if (!is_number(arg2))
        {
		  stc("{cSyntax: oload <vnum> <level>.\n\r", ch );
		  return;
		}
        level = atoi(arg2);
        if (level < 0 || level > get_trust(ch))
		{
		  stc("{cLevel must be be between 0 and your level.\n\r",ch);
	  	  return;
		}
    }*/

    if ( ( pObjIndex = get_obj_index( atoi( arg1 ) ) ) == NULL )
    {
	stc("{cNo object has that vnum.\n\r", ch );
	return;
    }

    pArea = pObjIndex->area;

    if ( !IS_BUILDER( ch, pArea ) )
    {
	stc( "{cInsufficient security to load object from that area.\n\r", ch );
	return;
    }

    obj = create_object( pObjIndex, level );
	if ( IS_SET(obj->extra_flags, ITEM_NOLOAD) && ch->level < MAX_LEVEL)
	{
		stc("You are not powerful enough to create this object.\n\r",ch);
		extract_obj(obj);
		return;
	}

	guild = clan_lookup(obj->guild);

    if (clan_lookup(obj->guild) != ch->clan
    && guild > 2
    && ch->level < MAX_LEVEL )
    {
		stc("You can't load items that dont belong to your Guild.\n\r",ch);
		return;
	}

    if ( CAN_WEAR(obj, ITEM_TAKE) )
	obj_to_char( obj, ch );
    else
	obj_to_room( obj, ch->in_room );
    act( "{c$n pulls $p from the Pattern!", ch, obj, NULL, TO_ROOM );
    wiznet("{c$N pulls $p from the Pattern.",ch,obj,WIZ_LOAD,WIZ_SECURE,get_trust(ch));
    stc("{cOk.\n\r", ch );
    return;
}



void do_purge( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    DESCRIPTOR_DATA *d;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
		/* 'purge' */
		CHAR_DATA *vnext;
		OBJ_DATA  *obj_next;

		for ( victim = ch->in_room->people; victim != NULL; victim = vnext )
		{
		    vnext = victim->next_in_room;
		    if ( IS_NPC(victim) && !IS_SET(victim->act,ACT_NOPURGE)
		    &&   victim != ch /* safety precaution */ )
			extract_char( victim, TRUE );
		    	ch->pcdata->mod_disconnects++;
		}

		for ( obj = ch->in_room->contents; obj != NULL; obj = obj_next )
		{
		    obj_next = obj->next_content;
		    if (!IS_OBJ_STAT(obj,ITEM_NOPURGE))
				extract_obj( obj );
		}

		act( "{c$n unleashes balefire!", ch, NULL, NULL, TO_ROOM);
		stc("{cYou unleash balefire!\n\r", ch );
		return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
		stc("{cThey arent here.\n\r", ch );
		return;
    }

    if ( !IS_NPC(victim) )
    {

		if (ch == victim)
		{
			stc("{cHo ho ho.\n\r",ch);
			return;
		}

		act("$n lashes out at $N with balefire!",ch,0,victim,TO_NOTVICT);

		clear_irc( victim );

    	if (victim->level > 1)
		    //save_char_obj( victim );
		   save_char_obj(victim, FALSE, FALSE);
    	d = victim->desc;
    	//extract_char( victim, TRUE );
    	if ( d != NULL )
			close_socket( d );

		//do_disconnect(victim, NULL);
		return;
    }

    act( "{c$n unleashes balefire on $N.", ch, NULL, victim, TO_NOTVICT );
	stc("{cYou unleash balefire!\n\r", ch );
    extract_char( victim, TRUE );
     return;
}



void do_advance( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int level;
    int iLevel;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || !is_number( arg2 ) )
    {
	stc("{cSyntax: advance <char> <level>.\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	stc("{cThat player is not here.\n\r", ch);
	return;
    }

    if ( IS_NPC(victim) )
    {
	stc("{cNot on NPCs.\n\r", ch );
	return;
    }

    if ( ( level = atoi( arg2 ) ) < 1 || level > ch->level )
    {
	sprintf(buf,"{cLevel must be 1 to %d.\n\r", ch->level);
	stc(buf, ch);
	return;
    }

/*
    if ( ch->level < victim->level )
    {
        sprintf(buf, "{cYeah right... Gain some levels, first.\n\r");
        stc(buf, ch);
    	sprintf(buf,"{c%s tried to advance you!\n\r",ch->name);
    	stc(buf,victim);
        return;
    }

    if ( level > get_trust( ch ) )
    {
	stc("{cLimited to your trust level.\n\r", ch );
	return;
    }
*/
    /*
     * Lower level:
     *   Reset to level 1.
     *   Then raise again.
     *   Currently, an imp can lower another imp.
     *   -- Swiftest
     *   Ch must now be higher level than victim to lower - Dal
     */
    if ( level <= victim->level )
    {
        int temp_prac;

	stc("{cLowering a players level!\n\r", ch );
	stc("{C**** {cOOOOHHHHHHHHHH  NNNNOOOO {C****{c\n\r", victim );
	temp_prac = victim->practice;
	victim->level    = 1;
	victim->exp      = exp_per_level(victim,victim->pcdata->points);
	victim->max_hit  = 50;
	victim->max_stam = 50;
	victim->practice = 0;
	victim->hit      = victim->max_hit;
	victim->stam     = victim->max_stam;
	advance_level( victim, TRUE );
	victim->practice = temp_prac;
    }
    else
    {
	stc("{cRaising a players level!\n\r", ch );
	stc("{C**** {cOOOOHHHHHHHHHH  YYYYEEEESSS {C****{c\n\r", victim );
    }

    for ( iLevel = victim->level ; iLevel < level; iLevel++ )
    {
	victim->level += 1;
	advance_level( victim,TRUE);
    }
    sprintf(buf,"{cYou are now level %d.\n\r",victim->level);
    stc(buf,victim);
    victim->exp   = exp_per_level(victim,victim->pcdata->points)
		  * UMAX( 1, victim->level );
    victim->trust = 0;
    //save_char_obj(victim);
    save_char_obj(victim, FALSE, FALSE);
    return;
}



void do_trust( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    int level;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || !is_number( arg2 ) )
    {
	stc("{cSyntax: trust <char> <level>.\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	stc("{cThat player is not here.\n\r", ch);
	return;
    }

    if ( ch->level < victim->level )
    {
        sprintf(buf, "{cYeah right... Gain some levels, first.\n\r");
        stc(buf, ch);
    	sprintf(buf,"{c%s tried to mess with your trust!\n\r",ch->name);
    	stc(buf,victim);
        return;
    }

    if ( ( level = atoi( arg2 ) ) < 0 || level > MAX_LEVEL )
    {
	sprintf(buf, "{cLevel must be 0 {C({creset{C){c or 1 to %d.\n\r",MAX_LEVEL);
	stc(buf, ch);
	return;
    }

    if ( level > get_trust( ch ) )
    {
	stc("{cLimited to your trust.\n\r", ch );
	return;
    }

    victim->trust = level;
    return;
}



void do_restore( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH], buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    CHAR_DATA *vch;
    DESCRIPTOR_DATA *d;

    one_argument( argument, arg );

    if (!str_cmp(arg,"self") || arg[0] == '\0')
    {
    /* cure self */
        ch->hit 	= get_maxhit(ch);
        ch->stam	= get_maxstam(ch);
        update_pos( ch);
        stc("{cYou restore yourself.\n\r",ch);
        return;
    }

    if (!str_cmp(arg,"room"))
    {
    /* cure room */

        for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
        {
            affect_strip(vch,gsn_plague);
            affect_strip(vch,gsn_poison);
            affect_strip(vch,gsn_blindness);
            affect_strip(vch,gsn_sleep);
            affect_strip(vch,gsn_curse);

            vch->hit 	= get_maxhit(vch);
            vch->stam	= get_maxstam(vch);
            update_pos( vch);
            act("{cYou feel the divine power of the Creator flow through your body.",ch,NULL,vch,TO_VICT);
        }

        sprintf(buf,"{cYou feel the divine power of the Creator flow through your body.");
        wiznet(buf,ch,NULL,WIZ_RESTORE,WIZ_SECURE,get_trust(ch));

        stc("{cRoom restored.\n\r",ch);
        return;

    }

    if ( !str_cmp(arg,"all"))
    {
    /* cure all */

        for (d = descriptor_list; d != NULL; d = d->next)
        {
	    victim = d->character;

	    if (victim == NULL || IS_NPC(victim))
		continue;

            affect_strip(victim,gsn_plague);
            affect_strip(victim,gsn_poison);
            affect_strip(victim,gsn_blindness);
            affect_strip(victim,gsn_sleep);
            affect_strip(victim,gsn_curse);

            victim->hit 	= get_maxhit(victim);
            victim->stam	= get_maxstam(victim);
            update_pos( victim);
	    if (victim->in_room != NULL)
            act("{cYou feel the divine power of the Creator flow through your body.",ch,NULL,victim,TO_VICT);
        }
	stc("{cAll active players restored.\n\r",ch);
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL && victim != ch)
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }
    if( ch->level < MAX_LEVEL -1 )
    {
		stc("You arent powerful enough.\n\r",ch);
		return;
	}

    affect_strip(victim,gsn_plague);
    affect_strip(victim,gsn_poison);
    affect_strip(victim,gsn_blindness);
    affect_strip(victim,gsn_sleep);
    affect_strip(victim,gsn_curse);
    victim->hit = get_maxhit(victim);
    victim->stam = get_maxstam(victim);
    update_pos( victim );
    act( "{cYou feel the divine power of the Creator flowing through your body.", ch, NULL, victim, TO_VICT );
    /*sprintf(buf,"{c",
	IS_NPC(victim) ? victim->short_descr : victim->name);*/
    wiznet(buf,ch,NULL,WIZ_RESTORE,WIZ_SECURE,get_trust(ch));
    stc("{cPlayer Restored.\n\r", ch );
    return;
}


void do_freeze( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH],buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc("{cFreeze whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	stc("{cNot on NPCs.\n\r", ch );
	return;
    }

    if ( get_trust( victim ) > get_trust( ch ) )
    {
    	stc("{cYou failed.\n\r", ch );
    	sprintf(buf,"{c%s tried to freeze you!\n\r",ch->name);
    	stc(buf,victim);
    	return;
    }

    if ( IS_SET(victim->act, PLR_FREEZE) )
    {
	REMOVE_BIT(victim->act, PLR_FREEZE);
	stc("{cYou can play again.\n\r", victim );
	stc("{cFREEZE removed.\n\r", ch );
	sprintf(buf,"{c$N thaws %s.",victim->name);
	wiznet(buf,ch,NULL,WIZ_PENALTIES,WIZ_SECURE,0);
    }
    else
    {
	SET_BIT(victim->act, PLR_FREEZE);
	stc("{cYou can't do ANYthing!\n\r", victim );
	stc("{cFREEZE set.\n\r", ch );
	sprintf(buf,"{c$N puts %s in the deep freeze.",victim->name);
	wiznet(buf,ch,NULL,WIZ_PENALTIES,WIZ_SECURE,0);
    	//read_mysql_staff( ch );
	ch->pcdata->mod_freezings++;
	//update_staff( ch, "commands" );
    }

    //save_char_obj( victim );
    save_char_obj(victim, FALSE, FALSE);

    return;
}



void do_log( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH],buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc("{cLog whom?\n\r", ch );
	return;
    }

    if ( !str_cmp( arg, "all" ) )
    {
	if ( fLogAll )
	{
	    fLogAll = FALSE;
	    stc("{cLog ALL off.\n\r", ch );
	}
	else
	{
	    fLogAll = TRUE;
	    stc("{cLog ALL on.\n\r", ch );
	}
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL )
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	stc("{cNot on NPCs.\n\r", ch );
	return;
    }

	if ( is_exact_name(victim->name, MUD_OWNER))
	{
		stc("{cNo logging the Owner.\n\r", ch);
		sprintf(buf, "%s log", ch->name);
		do_revoke(ch,buf);
		return;
	}

	if ( is_exact_name(victim->name, MUD_COIMP))
	{
		stc("{cKerrick doesn't like nosey people...\n\r", ch);
		sprintf(buf, "%s log", ch->name);
		do_revoke(ch,buf);
		return;
	}
    /*
     * No level check, gods can log anyone.
     */
    if ( IS_SET(victim->act, PLR_LOG) )
    {
	REMOVE_BIT(victim->act, PLR_LOG);
	stc("{cLOG removed.\n\r", ch );
    }
    else
    {
		SET_BIT(victim->act, PLR_LOG);
		stc("{cLOG set.\n\r", ch );
	}

    return;
}

void do_peace( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *rch;

    for ( rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room )
    {
	if ( is_fighting(rch) )
	    stop_all_fighting( rch );
	if (IS_NPC(rch) && IS_SET(rch->act,ACT_AGGRESSIVE))
	    REMOVE_BIT(rch->act,ACT_AGGRESSIVE);
    if (rch->hate)
        stop_hating(rch);
    /*rch->range = RANGE_NONE;
    rch->advancing = FALSE;
    rch->advancing_on = NULL;*/
    }

    stc("{cOk.\n\r", ch );
    return;
}

void do_wizlock( CHAR_DATA *ch, char *argument )
{
    extern bool wizlock;
    wizlock = !wizlock;

    wizlock_timer = -1;

    if ( wizlock )
    {
	wiznet("{c$N has wizlocked the game.",ch,NULL,0,0,0);
	stc("{cGame wizlocked.\n\r", ch );
    }
    else
    {
	wiznet("{c$N removes wizlock.",ch,NULL,0,0,0);
	stc("{cGame un-wizlocked.\n\r", ch );
    }

    return;
}

/* RT anti-newbie code */

void do_newlock( CHAR_DATA *ch, char *argument )
{
    extern bool newlock;
    newlock = !newlock;

    if ( newlock )
    {
	wiznet("{c$N locks out new characters.",ch,NULL,0,0,0);
        stc("{cNew characters have been locked out.\n\r", ch );
    }
    else
    {
	wiznet("{c$N allows new characters back in.",ch,NULL,0,0,0);
        stc("{cNewlock removed.\n\r", ch );
    }

    return;
}


void do_slookup( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    int sn;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc("{cLookup which skill or spell?\n\r", ch );
	return;
    }

    if ( !str_cmp( arg, "all" ) )
    {
	for ( sn = 0; sn < MAX_SKILL; sn++ )
	{
	    if ( skill_table[sn].name == NULL )
		break;
	    sprintf( buf, "{cSn: %3d  Slot: %3d  Skill/spell: '%s'\n\r",
		sn, skill_table[sn].slot, skill_table[sn].name );
	    stc( buf, ch );
	}
    }
    else
    {
	if ( ( sn = skill_lookup( arg ) ) < 0 )
	{
	    stc("{cNo such skill or spell.\n\r", ch );
	    return;
	}

	sprintf( buf, "{cSn: %3d  Slot: %3d  Skill/spell: '%s'\n\r",
	    sn, skill_table[sn].slot, skill_table[sn].name );
	stc( buf, ch );
    }

    return;
}

/* RT set replaces sset, mset, oset, and rset */

void do_set( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];

    argument = one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  set mob   <name> <field> <value>\n\r",ch);
	stc("{c  set obj   <name> <field> <value>\n\r",ch);
	stc("{c  set room  <room> <field> <value>\n\r",ch);
        stc("{c  set skill <name> <spell or skill> <value>\n\r",ch);
	return;
    }

    if (!str_prefix(arg,"mobile") || !str_prefix(arg,"character"))
    {
	do_function(ch, &do_mset, argument);
	return;
    }

    if (!str_prefix(arg,"skill") || !str_prefix(arg,"spell"))
    {
	do_function(ch, &do_sset, argument);
	return;
    }

    if (!str_prefix(arg,"object"))
    {
	do_function(ch, &do_oset, argument);
	return;
    }

    if (!str_prefix(arg,"room"))
    {
	do_function(ch, &do_rset, argument);
	return;
    }
    /* echo syntax */
    do_function(ch, &do_set, "");
}


void do_sset( CHAR_DATA *ch, char *argument )
{
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    char arg3 [MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int value;
    int sn;
    bool fAll;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    argument = one_argument( argument, arg3 );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  set skill <name> <spell or skill> <value>\n\r", ch);
	stc("{c  set skill <name> all <value>\n\r",ch);
	stc("{c   (use the name of the skill, not the number)\n\r",ch);
	return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	stc("{cNot on NPCs.\n\r", ch );
	return;
    }

    fAll = !str_cmp( arg2, "all" );
    sn   = 0;
    if ( !fAll && ( sn = skill_lookup( arg2 ) ) < 0 )
    {
	stc("{cNo such skill or spell.\n\r", ch );
	return;
    }

    /*
     * Snarf the value.
     */
    if ( !is_number( arg3 ) )
    {
	stc("{cValue must be numeric.\n\r", ch );
	return;
    }

    value = atoi( arg3 );
    if ( value < 0 || value > 109 )
    {
	stc("{cValue range is 0 to 109.\n\r", ch );
	return;
    }

    if ( fAll )
    {
	for ( sn = 0; sn < MAX_SKILL; sn++ )
	{
	    if ( skill_table[sn].name != NULL )
		victim->pcdata->learned[sn]	= value;
	}
    }
    else
    {
	victim->pcdata->learned[sn] = value;
    }

    return;
}


void do_mset( CHAR_DATA *ch, char *argument )
{
	char arg1[MIL], arg2[MIL], arg3[MIL];
	char buf[100];
	CHAR_DATA *victim;
	int value;

	smash_tilde( argument );
	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	strcpy( arg3, argument );

	if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
	{
		stc("{cSyntax:\n\r",ch);
		stc("{c  set char <name> <field> <value>\n\r",ch);
		stc("{c  Field being one of:\n\r",			ch );
		stc("{c    str int wis dex con ref ins cha sex class level\n\r",	ch );
		stc("{c    race group gold smark spenny cpenny hp stamina prac\n\r",ch);
		stc("{c    align train thirst hunger drunk full\n\r",	ch );
		stc("{c    security channeler moderator newbie\n\r", ch );
		stc("{c    qtime qps rbi clanchief\n\r", ch);
		stc("{c    earth air fire water spirit\n\r",ch);
		return;
	}

	if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
	{
		stc("They arent here.\n\r", ch );
		return;
	}

	/* clear zones for mobs */
	victim->zone = NULL;

    /*
     * Snarf the value (which need not be numeric).
     */
    value = is_number( arg3 ) ? atoi( arg3 ) : -1;

    /*
     * Set something.
     */

        if ( !str_cmp( arg2, "earth" ) )
        {
                if ( value < 0 || value > 999 )
                {
                        stc("QPs range is 0 to 999.\n\r", ch);
                        return;
                }
                victim->pcdata->op_earth = value;
                return;
        }
        if ( !str_cmp( arg2, "air" ) )
        {
                if ( value < 0 || value > 999 )
                {
                        stc("QPs range is 0 to 999.\n\r", ch);
                        return;
                }
                victim->pcdata->op_air = value;
                return;
        }
        if ( !str_cmp( arg2, "fire" ) )
        {
                if ( value < 0 || value > 999 )
                {
                        stc("QPs range is 0 to 999.\n\r", ch);
                        return;
                }
                victim->pcdata->op_fire = value;
                return;
        }
        if ( !str_cmp( arg2, "water" ) )
        {
                if ( value < 0 || value > 999 )
                {
                        stc("QPs range is 0 to 999.\n\r", ch);
                        return;
                }
                victim->pcdata->op_water = value;
                return;
        }
        if ( !str_cmp( arg2, "spirit" ) )
        {
                if ( value < 0 || value > 999 )
                {
                        stc("QPs range is 0 to 999.\n\r", ch);
                        return;
                }
                victim->pcdata->op_spirit = value;
                return;
        }

	if ( !str_cmp( arg2, "newbie" ) )
	{
		if ( IS_NPC( victim ) )
		{
			stc("Only on PCs.\n\r",ch);
			return;
		}
		if ( victim->pcdata->newbie == 0 )
		{
			stc("Newbified.\n\r",ch);
			init_newbie( victim );
			victim->pcdata->newbie = 1;
			return;
		}
		else
		{
			stc("Un-Newbified.\n\r",ch);
			victim->pcdata->newbie = 0;
			return;
		}
	}
	if ( !str_cmp( arg2, "clanchief" ) )
	{
		if ( IS_NPC( victim ) )
		{
			stc("Only on PCs.\n\r",ch);
			return;
		}
		if ( victim->race != race_lookup("aiel") )
		{
			stc("Only on Aiel.\n\r",ch);
			return;
		}
		if ( victim->sex == SEX_FEMALE )
		{
			stc("They will be made Wise One.\n\r",ch);
			victim->clan = 23;
			return;
		}
		if ( victim->sex == SEX_MALE )
		{
			stc("They will be made Clan Chief.\n\r",ch);
			victim->clan = 63;
			return;
		}
	}

	if ( !str_cmp( arg2, "qps" ) )
	{
		if ( value < 0 || value > 9999 )
		{
			stc("QPs range is 0 to 9999.\n\r", ch);
			return;
		}
		victim->questpoints = value;
		return;
	}

	if ( !str_cmp( arg2, "qtime" ) )
	{
		if ( IS_NPC( victim ) )
		{
			stc("Only on PCs.\n\r",ch);
			return;
		}

		if (ch != victim)
		stc("They may now quest again.\n\r",ch);
		stc("You may now quest again.\n\r",victim);
		victim->nextquest = 0;
		return;
	}
	if (!str_cmp(arg2, "moderator" ) )
	{
		if ( IS_NPC( victim ) )
		{
			stc("Only on PCs.\n\r",ch);
			return;
		}
		if (!IS_SET(victim->war_flags,WAR_MODERATOR))
		{
		stc("As you wish.\n\r",ch);
		stc("You are now a game moderator, and have the power to penalize bad people.\n\r",victim);
		SET_BIT(victim->war_flags,WAR_MODERATOR);
		}
		else
		{
		stc("As you wish.\n\r",ch);
		stc("You are no long a game moderator.\n\r",victim);
		REMOVE_BIT(victim->war_flags,WAR_MODERATOR);
		}
		return;
	}
    if ( !str_cmp( arg2, "channeler" ) )
    {
		if ( IS_NPC( victim ) )
		{
			stc("Only on PCs.\n\r",ch);
			return;
		}
	    if (!IS_SET( victim->op_flags, OP_CHANNELER))
	    {
	        SET_BIT(victim->op_flags, OP_CHANNELER);
			stc("Toggled ON. Set again to remove channeling ability.\n\r",ch);
			stc("You can now sense the True Source.\n\r",victim);
	    	return;
		}
	    else if (IS_SET( victim->op_flags, OP_CHANNELER))
	    {
	        REMOVE_BIT(victim->op_flags, OP_CHANNELER);
			stc("Toggled OFF. Set again to remove channeling ability.\n\r",ch);
			stc("You can no longer sense the True Source.\n\r",victim);
	    	return;
		}
		else
		{
			stc("Failed for some reason.\n\r",ch);
			return;
		}
	}

    if ( !str_cmp( arg2, "rbi" ) )
    {
		if ( value < 0 || value > 250 )
		{
		    stc("RBI range is 0 to 250.\n\r", ch);
		    return;
		}
		victim->pcdata->rebirth = value;
		return;
    }
    if ( !str_cmp( arg2, "str" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_STR) )
		{
	    sprintf(buf,
		"{cStrength range is 3 to %d.\n\r",
		get_max_train(victim,STAT_STR));
	    stc(buf,ch);
	    return;
		}
		victim->perm_stat[STAT_STR] = value;
		return;
    }

    if ( !str_cmp( arg2, "int" ) )
    {
        if ( value < 3 || value > get_max_train(victim,STAT_INT) )
        {
        sprintf(buf,"{cIntelligence range is 3 to %d.\n\r",get_max_train(victim,STAT_INT));
        stc(buf,ch);
        return;
        }
        victim->perm_stat[STAT_INT] = value;
        return;
    }

    if ( !str_cmp( arg2, "wis" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_WIS) )
		{
	    sprintf(buf,"{cWisdom range is 3 to %d.\n\r",get_max_train(victim,STAT_WIS));
	    stc( buf, ch );
	    return;
		}
		victim->perm_stat[STAT_WIS] = value;
		return;
    }

    if ( !str_cmp( arg2, "dex" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_DEX) )
		{
	    sprintf(buf,"{cDexterity range is 3 to %d.\n\r",
		get_max_train(victim,STAT_DEX));
	    stc( buf, ch );
	    return;
		}
		victim->perm_stat[STAT_DEX] = value;
		return;
    }

    if ( !str_cmp( arg2, "con" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_CON) )
		{
	    sprintf(buf,"{cConstitution range is 3 to %d.\n\r",get_max_train(victim,STAT_CON));
	    stc( buf, ch );
	    return;
		}
		victim->perm_stat[STAT_CON] = value;
		return;
    }

    if ( !str_cmp( arg2, "ref" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_REF) )
		{
	    sprintf(buf,
		"{cReflex range is 3 to %d.\n\r",
		get_max_train(victim,STAT_REF));
	    stc(buf,ch);
	    return;
		}
		victim->perm_stat[STAT_REF] = value;
		return;
    }

    if ( !str_cmp( arg2, "ins" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_INS) )
		{
	    sprintf(buf,
		"{cInstinct range is 3 to %d.\n\r",
		get_max_train(victim,STAT_INS));
	    stc(buf,ch);
	    return;
		}
		victim->perm_stat[STAT_INS] = value;
		return;
    }

    if ( !str_cmp( arg2, "cha" ) )
    {
		if ( value < 3 || value > get_max_train(victim,STAT_CHA) )
		{
	    sprintf(buf,
		"{cCharisma range is 3 to %d.\n\r",
		get_max_train(victim,STAT_CHA));
	    stc(buf,ch);
	    return;
		}
		victim->perm_stat[STAT_CHA] = value;
		return;
    }

    if ( !str_cmp( arg2, "security" ) )	/* OLC */
    {
		if ( IS_NPC(ch) )
		{
			stc("{cMobiles cannot set security.\n\r", ch );
			return;
		}
        if ( IS_NPC( victim ) )
        {
            stc("{cNot on NPCs.\n\r", ch );
            return;
        }
		if ( value > ch->pcdata->security || value < 0 )
		{
		    if ( ch->pcdata->security != 0 )
		    {
				sprintf( buf, "{cValid security is 0-%d.\n\r",
			    ch->pcdata->security );
				stc( buf, ch );
				return;
		    }
		    else
		    {
				stc("{cValid security is 0 only.\n\r", ch );
			    return;
		    }
		}
		victim->pcdata->security = value;
		return;
    }

    if ( !str_prefix( arg2, "sex" ) )
    {
	if ( value < 0 || value > 2 )
	{
	    stc("{cSex range is 0 to 2.\n\r", ch );
	    return;
	}
	victim->sex = value;
	if (!IS_NPC(victim))
	    victim->pcdata->true_sex = value;
	return;
    }

    if ( !str_prefix( arg2, "class" ) )
    {
	int class;

	if (IS_NPC(victim))
	{
	    stc("{cMobiles have no class.\n\r",ch);
	    return;
	}

	class = class_lookup(arg3);
	if ( class == -1 )
	{
	    char buf[MAX_STRING_LENGTH];

        	strcpy( buf, "{cPossible classes are: " );
        	for ( class = 0; class < MAX_CLASS; class++ )
        	{
            	    if ( class > 0 )
                    	strcat( buf, " " );
            	    strcat( buf, class_table[class].name );
        	}
            strcat( buf, ".\n\r" );

	    stc(buf,ch);
	    return;
	}

	victim->class = class;
	return;
    }

    if ( !str_prefix( arg2, "level" ) )
    {
	if ( !IS_NPC(victim) )
	{
	    stc("{cNot on PCs.\n\r", ch );
	    return;
	}

	if ( value < 0 || value > MAX_LEVEL )
	{
	    sprintf(buf, "{cLevel range is 0 to %d.\n\r", MAX_LEVEL);
	    stc(buf, ch);
	    return;
	}
	victim->level = value;
	return;
    }

    if ( !str_prefix( arg2, "gold" ) )
    {
	victim->money[3][0] = value;
	return;
    }

    if ( !str_prefix(arg2, "smark" ) )
    {
	victim->money[2][0] = value;
	return;
    }

    if ( !str_prefix(arg2, "spenny" ) )
    {
	victim->money[1][0] = value;
	return;
    }

    if ( !str_prefix(arg2, "cpenny" ) )
    {
	victim->money[0][0] = value;
	return;
    }

    if ( !str_prefix( arg2, "hp" ) )
    {
	if ( value < -10 || value > 1000000 )
	{
	    stc("{cHp range is -10 to 1,000,000 hit points.\n\r", ch );
	    return;
	}
	victim->max_hit = value;
        if (!IS_NPC(victim))
            victim->pcdata->perm_hit = value;
	return;
    }

    if ( !str_prefix( arg2, "stamina" ) )
    {
	if ( value < 0 || value > 1000000 )
	{
	    stc("{cStamina range is 0 to 1,000,000 stamina points.\n\r", ch );
	    return;
	}
	victim->max_stam = value;
        if (!IS_NPC(victim))
            victim->pcdata->perm_mana = value;
	return;
    }

    if ( !str_prefix( arg2, "practice" ) )
    {
	if ( value < 0 || value > 2000 )
	{
	    stc("{cPractice range is 0 to 2000 sessions.\n\r", ch );
	    return;
	}
	victim->practice = value;
	return;
    }

    if ( !str_prefix( arg2, "train" ))
    {
	if (value < 0 || value > 2000)
	{
	    stc("{cTraining session range is 0 to 2000 sessions.\n\r",ch);
	    return;
	}
	victim->train = value;
	return;
    }

    if ( !str_prefix( arg2, "align" ) )
    {
	if ( value < -1000 || value > 1000 )
	{
	    stc("{cAlignment range is -1000 to 1000.\n\r", ch );
	    return;
	}
	victim->alignment = value;
	return;
    }

    if ( !str_prefix( arg2, "thirst" ) )
    {
	if ( IS_NPC(victim) )
	{
	    stc("{cNot on NPCs.\n\r", ch );
	    return;
	}

	if ( value < -1 || value > 100 )
	{
	    stc("{cThirst range is -1 to 100.\n\r", ch );
	    return;
	}

	victim->pcdata->condition[COND_THIRST] = value;
	return;
    }

    if ( !str_prefix( arg2, "drunk" ) )
    {
		if ( IS_NPC(victim) )
		{
		    stc("{cNot on NPCs.\n\r", ch );
		    return;
		}

		if ( value < -1 || value > 100 )
		{
		    stc("{cDrunk range is -1 to 100.\n\r", ch );
		    return;
		}

		victim->pcdata->condition[COND_DRUNK] = value;
		return;
    }

    if ( !str_prefix( arg2, "full" ) )
    {
		if ( IS_NPC(victim) )
		{
		    stc("{cNot on NPCs.\n\r", ch );
		    return;
		}

		if ( value < -1 || value > 100 )
		{
		    stc("{cFull range is -1 to 100.\n\r", ch );
		    return;
		}

		victim->pcdata->condition[COND_FULL] = value;
		return;
    }

    if ( !str_prefix( arg2, "hunger" ) )
    {
        if ( IS_NPC(victim) )
        {
            stc("{cNot on NPCs.\n\r", ch );
            return;
        }

        if ( value < -1 || value > 100 )
        {
            stc("{cFull range is -1 to 100.\n\r", ch );
            return;
        }

        victim->pcdata->condition[COND_HUNGER] = value;
        return;
    }

    if (!str_prefix( arg2, "race" ) )
    {
	int race;

	race = race_lookup(arg3);

	if ( race == 0)
	{
	    stc("{cThat is not a valid race.\n\r",ch);
	    return;
	}

	if (!IS_NPC(victim) && !race_table[race].pc_race)
	{
	    stc("{cThat is not a valid player race.\n\r",ch);
	    return;
	}

	victim->race = race;
	return;
    }

    if (!str_prefix(arg2,"group"))
    {
	if (!IS_NPC(victim))
	{
	    stc("{cOnly on NPCs.\n\r",ch);
	    return;
	}
	victim->group = value;
	return;
    }


    /*
     * Generate usage message.
     */
    do_function(ch, &do_mset, "" );
    return;
}

void do_string( CHAR_DATA *ch, char *argument )
{
    char type [MIL];
    char arg1 [MIL];
    char arg2 [MIL];
    char arg3 [MIL];
    CHAR_DATA *victim;
    OBJ_DATA *obj;

    smash_tilde( argument );
    argument = one_argument( argument, type );
    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    strcpy( arg3, argument );

    if ( type[0] == '\0' || arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  string char <name> <field> <string>\n\r",ch);
	stc("{c    fields: name short long desc title spec\n\r",ch);
	stc("      fields: walkin walkout rdesc lastname\n\r",ch);
	stc("{c  string obj  <name> <field> <string>\n\r",ch);
	stc("{c    fields: name short long extended material\n\r",ch);
	return;
    }

    if (!str_prefix(type,"character") || !str_prefix(type,"mobile"))
    {
    	if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    	{
	    stc("{cThey arent here.\n\r", ch );
	    return;
    	}
/*	rem-bellas 4-2-02 disallows moderators from doing thier job
	    if ( get_trust( victim ) >= get_trust( ch ) && ch != victim )
	    {
	    	sprintf(buf, "{c%s will be notified of the attempt.\n\r", victim->name);
	    	stc(buf, ch);
	    	sprintf(buf, "{c%s tried to string you.\n\r",ch->name);
	    	stc(buf,victim);
	    	return;
	    }
*/
	/* clear zone for mobs */
	victim->zone = NULL;

	/* string something */

     	if ( !str_prefix( arg2, "name" ) )
    	{
	    if ( !IS_NPC(victim) )
	    {
	    	stc("{cNot on PCs.\n\r", ch );
	    	return;
	    }
	    free_string( victim->name );
	    victim->name = str_dup( arg3 );
	    return;
    	}
	if ( !str_prefix( arg2, "host" ) )
	{
		if ( IS_NPC( victim ) )
		{
			stc("Not on mobiles.\n\r",ch);
			return;
		}
		if ( victim->desc == NULL )
		{
			stc("No descriptior.\n\r",ch);
			return;
		}
		free_string(victim->desc->host);
		victim->desc->host = str_dup(arg3);
		return;
	}
        if ( !str_prefix( arg2, "lastname" ) )
        {
	    if ( IS_NPC(victim) )
	    {
	    	stc("{cNot on NPCs.\n\r", ch );
	    	return;
	    }
	    free_string( victim->pcdata->last_name );
	    victim->pcdata->last_name = str_dup( arg3 );
	    compose_title( victim );
	    return;
        }

     	if ( !str_prefix( arg2, "walkin" ) )
    	{
	    if ( IS_NPC(victim) )
	    {
	    	stc("{cNot on NPCs.\n\r", ch );
	    	return;
	    }
	    free_string( victim->pcdata->wkin );
	    victim->pcdata->wkin = str_dup( arg3 );
	    return;
    	}

     	if ( !str_prefix( arg2, "walkout" ) )
    	{
	    if ( IS_NPC(victim) )
	    {
	    	stc("{cNot on NPCs.\n\r", ch );
	    	return;
	    }
	    free_string( victim->pcdata->wout );
	    victim->pcdata->wout = str_dup( arg3 );
	    return;
    	}

     	if ( !str_prefix( arg2, "rdesc" ) )
    	{
	    if ( IS_NPC(victim) )
	    {
	    	stc("{cNot on NPCs.\n\r", ch );
	    	return;
	    }
	    free_string( victim->pcdata->rdesc );
	    victim->pcdata->rdesc = str_dup( arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "description" ) )
    	{
    	    free_string(victim->description);
    	    victim->description = str_dup(arg3);
    	    return;
    	}

    	if ( !str_prefix( arg2, "short" ) )
    	{
	    free_string( victim->short_descr );
	    victim->short_descr = str_dup( arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "long" ) )
    	{
	    free_string( victim->long_descr );
	    strcat(arg3,"\n\r");
	    victim->long_descr = str_dup( arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "title" ) )
    	{
	    if ( IS_NPC(victim) )
	    {
	    	stc("{cNot on NPCs.\n\r", ch );
	    	return;
	    }

	    set_title( victim, arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "spec" ) )
    	{
	    if ( !IS_NPC(victim) )
	    {
	    	stc("{cNot on PCs.\n\r", ch );
	    	return;
	    }

	    if ( ( victim->spec_fun = spec_lookup( arg3 ) ) == 0 )
	    {
	    	stc("{cNo such spec fun.\n\r", ch );
	    	return;
	    }

	    return;
    	}
    }

    if (!str_prefix(type,"object"))
    {
    	/* string an obj */

   	if ( ( obj = get_obj_world( ch, arg1 ) ) == NULL )
    	{
	    stc("{cNothing like that in heaven or earth.\n\r", ch );
	    return;
    	}

        if ( !str_prefix( arg2, "name" ) )
    	{
	    free_string( obj->name );
	    obj->name = str_dup( arg3 );
	    return;
    	}

        if ( !str_prefix( arg2, "material" ) )
    	{
	    free_string( obj->material );
	    obj->material = str_dup( arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "short" ) )
    	{
	    free_string( obj->short_descr );
	    obj->short_descr = str_dup( arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "long" ) )
    	{
	    free_string( obj->description );
	    obj->description = str_dup( arg3 );
	    return;
    	}

    	if ( !str_prefix( arg2, "ed" ) || !str_prefix( arg2, "extended"))
    	{
	    EXTRA_DESCR_DATA *ed;

	    argument = one_argument( argument, arg3 );
	    if ( argument == NULL )
	    {
	    	stc("{cSyntax: oset <object> ed <keyword> <string>\n\r",
		    ch );
	    	return;
	    }

 	    strcat(argument,"\n\r");

	    ed = new_extra_descr();

	    ed->keyword		= str_dup( arg3     );
	    ed->description	= str_dup( argument );
	    ed->next		= obj->extra_descr;
	    obj->extra_descr	= ed;
	    return;
    	}
    }


    /* echo bad use message */
    do_function(ch, &do_string, "");
}



void do_oset( CHAR_DATA *ch, char *argument )
{
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    char arg3 [MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int value;

    smash_tilde( argument );
    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    strcpy( arg3, argument );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  set obj <object> <field> <value>\n\r",ch);
	stc("{c  Field being one of:\n\r",				ch );
	stc("{c    value0 value1 value2 value3 value4 (v0 - v4)\n\r",	ch );
	stc("{c    extra wear level weight cost timer condition dye\n\r",		ch );
	return;
    }

    if ( ( obj = get_obj_world( ch, arg1 ) ) == NULL )
    {
	stc("{cNothing like that in heaven or earth.\n\r", ch );
	return;
    }

    /*
     * Snarf the value (which need not be numeric).
     */
    value = atoi( arg3 );

    /*
     * Set something.
     */
    if ( !str_cmp( arg2, "value0" ) || !str_cmp( arg2, "v0" ) )
    {
	obj->value[0] = value;
	return;
    }

    if ( !str_cmp( arg2, "value1" ) || !str_cmp( arg2, "v1" ) )
    {
	obj->value[1] = value;
	return;
    }

    if ( !str_cmp( arg2, "value2" ) || !str_cmp( arg2, "v2" ) )
    {
	obj->value[2] = value;
	return;
    }

    if ( !str_cmp( arg2, "value3" ) || !str_cmp( arg2, "v3" ) )
    {
	obj->value[3] = value;
	return;
    }

    if ( !str_cmp( arg2, "value4" ) || !str_cmp( arg2, "v4" ) )
    {
	obj->value[4] = value;
	return;
    }

    if ( !str_cmp( arg2, "dye" ))
    {
	obj->dye_vnum = value;
	return;
    }

    if ( !str_prefix( arg2, "extra" ) )
    {
	obj->extra_flags = value;
	return;
    }

    if ( !str_prefix( arg2, "wear" ) )
    {
		if ( ( value = flag_value( wear_flags, argument ) ) != NO_FLAG )
		{
		    TOGGLE_BIT(obj->wear_flags, value);
			return;
		}
    }

    if ( !str_prefix( arg2, "level" ) )
    {
	obj->level = value;
	return;
    }

    if ( !str_prefix( arg2, "weight" ) )
    {
	obj->weight = value;
	return;
    }

    if ( !str_prefix( arg2, "condition" ) )
    {
	obj->condition = value;
	return;
    }

    if ( !str_prefix( arg2, "cost" ) )
    {
	obj->cost = value;
	return;
    }

    if ( !str_prefix( arg2, "timer" ) )
    {
	obj->timer = value;
	return;
    }

    /*
     * Generate usage message.
     */
    do_function(ch, &do_oset, "" );
    return;
}



void do_rset( CHAR_DATA *ch, char *argument )
{
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    char arg3 [MAX_INPUT_LENGTH];
    ROOM_INDEX_DATA *location;
    int value;

    smash_tilde( argument );
    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    strcpy( arg3, argument );

    if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
    {
	stc("{cSyntax:\n\r",ch);
	stc("{c  set room <location> <field> <value>\n\r",ch);
	stc("{c  Field being one of:\n\r",			ch );
	stc("{c    flags sector\n\r",				ch );
	return;
    }

    if ( ( location = find_location( ch, arg1 ) ) == NULL )
    {
	stc("{cNo such location.\n\r", ch );
	return;
    }

    if (!is_room_owner(ch,location) && ch->in_room != location
    &&  room_is_private(location) && !IS_TRUSTED(ch,IMPLEMENTOR))
    {
        stc("{cThat room is private right now.\n\r",ch);
        return;
    }

    /*
     * Snarf the value.
     */
    if ( !is_number( arg3 ) )
    {
	stc("{cValue must be numeric.\n\r", ch );
	return;
    }
    value = atoi( arg3 );

    /*
     * Set something.
     */
    if ( !str_prefix( arg2, "flags" ) )
    {
	location->room_flags	= value;
	return;
    }

    if ( !str_prefix( arg2, "sector" ) )
    {
	location->sector_type	= value;
	return;
    }

    /*
     * Generate usage message.
     */
    do_function(ch, &do_rset, "");
    return;
}


void do_sockets( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA       *vch;
    DESCRIPTOR_DATA *d;
    char            buf  [ MAX_STRING_LENGTH ];
    char            buf2 [ MAX_STRING_LENGTH ];
    int             count;
    char *          st;
    char            s[100];
    char            idle[10];


    count       = 0;
    buf[0]      = '\0';
    buf2[0]     = '\0';

    strcat( buf2, "\n\r[Num Connected_State Login@ Idl] Player Name Host\n\r" );
    strcat( buf2,
"--------------------------------------------------------------------------\n\r");
    for ( d = descriptor_list; d; d = d->next )
    {
        if ( d->character && can_see( ch, d->character ) )
        {
           /* NB: You may need to edit the CON_ values */
           switch( d->connected )
           {
              case CON_PLAYING:              st = "    PLAYING    ";    break;
              case CON_GET_NAME:             st = "   Get Name    ";    break;
              case CON_GET_OLD_PASSWORD:     st = "Get Old Passwd ";    break;
              case CON_CONFIRM_NEW_NAME:     st = " Confirm Name  ";    break;
              case CON_GET_NEW_PASSWORD:     st = "Get New Passwd ";    break;
              case CON_CONFIRM_NEW_PASSWORD: st = "Confirm Passwd ";    break;
              case CON_GET_NEW_RACE:         st = "  Get New Race ";    break;
              case CON_GET_NEW_SEX:          st = "  Get New Sex  ";    break;
              case CON_GET_NEW_CLASS:        st = " Get New Class ";    break;
              case CON_BREAK_CONNECT: 	     st = "   LINKDEAD    ";    break;
              case CON_COPYOVER_RECOVER:     st = "   COPYOVER    ";    break;
              case CON_CHANNELER:            st = "   CHANNELER   ";    break;
              case CON_CHAR_BACKG:           st = "  BACKGROUND   ";    break;
              case CON_CHAR_EYE:             st = "     EYE       ";    break;
              case CON_GET_HAIR1:            st = "    HAIR1      ";    break;
              case CON_GET_HAIR2:            st = "    HAIR2      ";    break;
              case CON_GET_HAIR3:            st = "    HAIR3      ";    break;
              case CON_GET_HAIR4:            st = "    HAIR4      ";    break;
              case CON_CONNECT_CHAR:         st = " CONNECT CHAR  ";    break;
              case CON_CHAR_RANDOM:          st = "   RANDOM      ";    break;
              case CON_GET_AIEL_SOCIETY:     st = "  AIEL SOC     ";    break;
              case CON_GET_TROLLOC_CLAN:     st = " TROLLOC CLAN  ";    break;
              case CON_CLEAN_UP:             st = "   CLEAN UP    ";    break;
              case CON_KEEP_STATS:           st = "  KEEP STATS   ";    break;
              case CON_GET_AIEL_CLAN:        st = "  AIEL CLAN    ";    break;
              case CON_ACCEPT_FOLLOWER:      st = "   GROUPING    ";    break;
              case CON_SHOW_POLICY:          st = "    POLICY     ";    break;
              case CON_ACCEPT_RULES:         st = " ACCEPT RULES  ";    break;
              case CON_AIEL_SOCIETY_NEW:     st = " AIEL SOC NEW  ";    break;
              case CON_SEANCHAN:             st = "   SEANCHAN    ";    break;
              case CON_BUGREPORTING_MSG:     st = "     BUG       ";    break;
              case CON_BUGREPORTING_IMMINFO: st = "     BUG       ";    break;
              case CON_BUGREPORTING_MORTINFO:st = "     BUG       ";    break;
              case CON_GET_LAST_NAME:        st = "   LAST NAME   ";    break;
              case CON_NOTE_TO:              st = "    NOTE       ";    break;
              case CON_NOTE_SUBJECT:         st = "    NOTE       ";    break;
              case CON_NOTE_EXPIRE:          st = "    NOTE       ";    break;
              case CON_NOTE_TEXT:            st = "    NOTE       ";    break;
              case CON_NOTE_FINISH:          st = "    NOTE       ";    break;
              case CON_CHAR_DESCRIPTION:     st = "  DESCRIPTION  ";    break;
              case CON_GET_FAMILY_NAME:      st = "  FAMILY NAME  ";    break;
		case CON_SELECT_GENDER: st = " GET GENDER "; break;
		case CON_SELECT_DETAIL: st = " GET DETAIL "; break;
              default:                       st = "   !UNKNOWN!   ";    break;
           }
           count++;

           /* Format "login" value... */
           vch = d->original ? d->original : d->character;
           strftime( s, 100, "%I:%M%p", localtime( &vch->logon ) );


           if ( vch->timer > 0 )
              sprintf( idle, "%-2d", vch->timer );
           else
              sprintf( idle, "  " );

           sprintf( buf, "[%3d %s %7s %2s] %-12s %-32.32s\n\r",
              d->descriptor,
              st,
              s,
              idle,
              ( d->original ) ? d->original->name : ( d->character ) ? d->character->name : "(None!)", d->host );


           strcat( buf2, buf );

        }
    }

    sprintf( buf, "\n\r%d user%s\n\r", count, count == 1 ? "" : "s" );
    strcat( buf2, buf );
    send_to_char( buf2, ch );
    return;
}

/*
 * Thanks to Grodyn for pointing out bugs in this function.
 */
void do_force( CHAR_DATA *ch, char *argument )
{
//    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];

    argument = one_argument( argument, arg );

    if ( arg[0] == '\0' || argument[0] == '\0' )
    {
	stc("{cForce whom to do what?\n\r", ch );
	return;
    }

    one_argument(argument,arg2);

    if (!str_cmp(arg2,"delete") || !str_prefix(arg2,"mob"))
    {
	stc("{cThat will NOT be done.\n\r",ch);
	return;
    }

    if ( !str_cmp( arg, "all" ) )
    {
	CHAR_DATA *vch;
	CHAR_DATA *vch_next;

	if (get_trust(ch) < MAX_LEVEL - 3)
	{
	    stc("{cNot at your level!\n\r",ch);
	    return;
	}


	for ( vch = char_list; vch != NULL; vch = vch_next )
	{
	    vch_next = vch->next;

	    if ( !IS_NPC(vch) && get_trust( vch ) < get_trust( ch ) )
	    {
//		act( buf, ch, NULL, vch, TO_VICT );
		interpret( vch, argument );
	    }
	}
    }
    else if (!str_cmp(arg,"players"))
    {
        CHAR_DATA *vch;
        CHAR_DATA *vch_next;

        if (get_trust(ch) < MAX_LEVEL - 2)
        {
            stc("{cNot at your level!\n\r",ch);
            return;
        }

        for ( vch = char_list; vch != NULL; vch = vch_next )
        {
            vch_next = vch->next;

            if ( !IS_NPC(vch) && get_trust( vch ) < get_trust( ch )
	    &&	 vch->level < LEVEL_HERO)
            {
//                act( buf, ch, NULL, vch, TO_VICT );
                interpret( vch, argument );
            }
        }
    }
    else if (!str_cmp(arg,"gods"))
    {
        CHAR_DATA *vch;
        CHAR_DATA *vch_next;

        if (get_trust(ch) < MAX_LEVEL - 2)
        {
            stc("{cNot at your level!\n\r",ch);
            return;
        }

        for ( vch = char_list; vch != NULL; vch = vch_next )
        {
            vch_next = vch->next;

            if ( !IS_NPC(vch) && get_trust( vch ) < get_trust( ch )
            &&   vch->level >= LEVEL_HERO)
            {
//                act( buf, ch, NULL, vch, TO_VICT );
                interpret( vch, argument );
            }
        }
    }
    else
    {
	CHAR_DATA *victim;

	if ( ( victim = get_char_world( ch, arg ) ) == NULL )
	{
	    stc("{cThey arent here.\n\r", ch );
	    return;
	}

	if ( victim == ch )
	{
	    stc("{cAye aye, right away!\n\r", ch );
	    return;
	}

    	if (!is_room_owner(ch,victim->in_room)
	&&  ch->in_room != victim->in_room
        &&  room_is_private(victim->in_room) && !IS_TRUSTED(ch,IMPLEMENTOR))
    	{
            stc("{cThat character is in a private room.\n\r",ch);
            return;
        }

	if ( get_trust( victim ) >= get_trust( ch ) && (!is_exact_name(ch->name, MUD_OWNER)) && (!is_exact_name(ch->name, MUD_COIMP)))
	{
	    stc("{cDo it yourself!\n\r", ch );
	    return;
	}

	if ( !IS_NPC(victim) && get_trust(ch) < MAX_LEVEL -3)
	{
	    stc("{cNot at your level!\n\r",ch);
	    return;
	}

//	act( buf, ch, NULL, victim, TO_VICT );
	interpret( victim, argument );
    }

    stc("{cOk.\n\r", ch );
    return;
}



/*
 * New routines by Dionysos.
 */
void do_invis( CHAR_DATA *ch, char *argument )
{
    int level;
    char arg[MAX_STRING_LENGTH];

    /* RT code for taking a level argument */
    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    /* take the default path */

      if ( ch->invis_level)
      {
	  ch->invis_level = 0;
	  act( "{c$n slowly fades into existence.", ch, NULL, NULL, TO_ROOM );
	  stc("{cYou slowly fade back into existence.\n\r", ch );
      }
      else
      {
	  ch->invis_level = get_trust(ch);
	  act( "{c$n slowly fades into thin air.", ch, NULL, NULL, TO_ROOM );
	  stc("{cYou slowly vanish into thin air.\n\r", ch );
      }
    else
    /* do the level thing */
    {
      level = atoi(arg);
      if (level < 2 || level > get_trust(ch))
      {
	stc("{cInvis level must be between 2 and your level.\n\r",ch);
        return;
      }
      else
      {
	  ch->reply = NULL;
          ch->invis_level = level;
          act( "{c$n slowly fades into thin air.", ch, NULL, NULL, TO_ROOM );
          stc("{cYou slowly vanish into thin air.\n\r", ch );
      }
    }

    return;
}


void do_incognito( CHAR_DATA *ch, char *argument )
{
    int level;
    char arg[MAX_STRING_LENGTH];

    /* RT code for taking a level argument */
    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    /* take the default path */

      if ( ch->incog_level)
      {
          ch->incog_level = 0;
          act( "{c$n is no longer cloaked.", ch, NULL, NULL, TO_ROOM );
          stc("{cYou are no longer cloaked.\n\r", ch );
      }
      else
      {
          ch->incog_level = get_trust(ch);
          act( "{c$n cloaks $s presence.", ch, NULL, NULL, TO_ROOM );
          stc("{cYou cloak your presence.\n\r", ch );
      }
    else
    /* do the level thing */
    {
      level = atoi(arg);
      if (level < 2 || level > get_trust(ch))
      {
        stc("{cIncog level must be between 2 and your level.\n\r",ch);
        return;
      }
      else
      {
          ch->reply = NULL;
          ch->incog_level = level;
          act( "{c$n cloaks $s presence.", ch, NULL, NULL, TO_ROOM );
          stc("{cYou cloak your presence.\n\r", ch );
      }
    }

    return;
}



void do_holylight( CHAR_DATA *ch, char *argument )
{
    if ( IS_NPC(ch) )
	return;

    if ( IS_SET(ch->act, PLR_HOLYLIGHT) )
    {
	REMOVE_BIT(ch->act, PLR_HOLYLIGHT);
	stc("{cHoly light mode off.\n\r", ch );
    }
    else
    {
	SET_BIT(ch->act, PLR_HOLYLIGHT);
	stc("{cHoly light mode on.\n\r", ch );
    }

    return;
}

/* prefix command: it will put the string typed on each line typed */

void do_prefi (CHAR_DATA *ch, char *argument)
{
    stc("{cYou cannot abbreviate the prefix command.\r\n",ch);
    return;
}

void do_prefix (CHAR_DATA *ch, char *argument)
{
    char buf[MAX_INPUT_LENGTH];

    if (argument[0] == '\0')
    {
	if (ch->prefix[0] == '\0')
	{
	    stc("{cYou have no prefix to clear.\r\n",ch);
	    return;
	}

	stc("{cPrefix removed.\r\n",ch);
	free_string(ch->prefix);
	ch->prefix = str_dup("");
	return;
    }

    if (ch->prefix[0] != '\0')
    {
	sprintf(buf,"{cPrefix changed to %s.\r\n",argument);
	free_string(ch->prefix);
    }
    else
    {
	sprintf(buf,"{cPrefix set to %s.\r\n",argument);
    }

    ch->prefix = str_dup(argument);
}

/*#define CH(descriptor)  ((descriptor)->original ? \
(descriptor)->original : (descriptor)->character)
in aww.h
*/
/* This file holds the copyover data */
#define COPYOVER_FILE "copyover.data"

/* This is the executable file */
#define EXE_FILE      "../src/tse"

void sysmsg (char *argument)
{
   DESCRIPTOR_DATA *d;

   for (d = descriptor_list; d; d = d->next)
   {
      if (d->connected == CON_PLAYING)
      {
	 stc ("{W<< \t[B010] \t[F050]System Message {x {W>> {G", d->character);
	 stc (argument, d->character);
         stc ("{x\n\r", d->character);
      }
   }

   return;
}

/*  Copyover - Original idea: Fusion of MUD++
 *  Adapted to Diku by Erwin S. Andreasen, <erwin@pip.dknet.dk>
 *  http://pip.dknet.dk/~pip1773
 *  Changed into a ROM patch after seeing the 100th request for it :)
 *  Fixes for saving areas and characters prior to copyover - Dalsor
 */
void copyover ()
{
	FILE *fp;
	DESCRIPTOR_DATA *d, *d_next;
	char buf [100], buf2[100];
	extern int port,control; /* db.c */

	/*Save the MUDs current time. - Dalsor 09-18-2002 */
	if (!( fp = fopen( TIME_FILE,"w+" ) ) )
	{
		log_string ("creation of new time_file failed");
	}
	else
	{
		int total;
		total = fprintf (fp, "%d %d %d %d",
			time_info.hour,
			time_info.day,
			time_info.month,
			time_info.year);
			if (total<4)
				log_string("failed fprintf to time_file");
			fclose(fp);
	}

	fp = fopen (COPYOVER_FILE, "w");

	if (!fp)
	{
		sysmsg ( "The copyover file was not writeable! Copyover FAILED :(");
		logf ("Could not write to copyover file: %s", COPYOVER_FILE);
		perror ("do_copyover:fopen");
		return;
	}

	sprintf(buf, "\n\r *** The Light battles the Shadow, and in the end, the Land is changed. ***\n\r");

    save_gquest_data();
    save_donation_pit();
	if(dfhunt_game_info[0] != 0)
	{
		int i;
		CHAR_DATA *wch;

		sysmsg ("Darkfriend Hunt has been shut down for the copyover");

		for(d=descriptor_list; d != NULL; d=d->next)
		{
			if( d->connected != CON_PLAYING )
				continue;
			wch = (d->original != NULL) ? d->original : d->character;
			if( wch == NULL )
				continue;

			for(i=0; i<7; i++)
			{
				if(i == 1 || i == 2)
					continue;

				wch->pcdata->dfhunt_player_info[i] = 0;
			}
		}

		for(i=0; i<7; i++)
			dfhunt_game_info[i] = 0;
	}
	while(auction_list != NULL)
	{
		/* doesn't show anyway */
		/*
		sprintf( buf2, "%s -- Auction is being halted.\n\r",
			capit_colorsafe( auction_list->item->short_descr ) );
		auction_channel( buf2 );
		*/
		if(auction_list->high_bidder != NULL)
			auction_list->high_bidder->money[0][1] += auction_list->gold_held;
		if ( auction_list->owner != NULL && !IS_NPC( auction_list->owner ) )
			obj_to_char( auction_list->item, auction_list->owner );
		reset_auc( auction_list );
	}



	/* For each playing descriptor, save its state */
	for (d = descriptor_list; d ; d = d_next)
	{
		CHAR_DATA * och = CH (d);
		d_next = d->next; /* We delete from the list , so need to save this */

		if (!d->character || d->connected > CON_PLAYING) /* drop those logging on */
		{
			write_to_descriptor (d->descriptor, "\n\rSorry, we are rebooting. Come back in a few minutes.\n\r", 0);
			close_socket (d); /* throw'em out */
		}
		else
		{
		 #ifdef THREADED_DNS
		   fprintf (fp, "%d %s %s %d.%d.%d.%d\n",
			d->descriptor, och->name, CopyoverGet(d),
			d->Host[0],d->Host[1],d->Host[2],d->Host[3]);
		#else
		    fprintf (fp, "%d %s %s %s\n", d->descriptor, och->name, CopyoverGet(d), d->host);
		#endif

#if 0			 /* This is not necessary for ROM */
			if (och->level == 1)
			{
				write_to_descriptor (d->descriptor, "Since you are level one, and level one characters do not save, you gain a free level!\n\r", 0);
				advance_level (och);
				och->level++; /* Advance_level doesn't do that */
			}
#endif
		//	save_char_obj (och);
			save_char_obj(och, FALSE, FALSE);

			write_to_descriptor (d->descriptor, buf, 0);
		}
	}

	mysql_close(result_db);

	fprintf (fp, "-1\n");
	fclose (fp);

	/* exec - descriptors are inherited */

	sprintf (buf, "%d", port);
	sprintf (buf2, "%d", control);
	execl (EXE_FILE, "tse", buf, "copyover", buf2, (char *) NULL);

	/* Failed - sucessful exec will not return */

	perror ("do_copyover: execl");
	sysmsg ("Copyover FAILED!");
}

void do_copyover (CHAR_DATA * ch, char *argument)
{
   char buf[MAX_STRING_LENGTH];
   int boottime;

   if (argument[0] == '\0' && ch)
   {
      stc ("Syntax: Copyover <STOP/number of seconds/NOW>\n\r", ch);
      return;
   }

   if (!str_cmp (argument, "now"))
   {
	char buf[MAX_STRING_LENGTH];

	sprintf (buf, "Copyover by %s.", ch->name);
	sysmsg (buf);
	copyover();
      return;
   }
   if (!str_cmp (argument, "stop"))
   {
	sprintf (buf, "The copyover has been aborted by %s.", ch->name);
	sysmsg (buf);

      bootcount = tmp_bootcount;
      tmp_bootcount = -1;
      return;
   }

   if (!is_number (argument))
   {
      stc ("Syntax: Copyover <STOP/number of seconds/NOW>\n\r", ch);
      return;
   }

   if ((boottime = atoi (argument)) < 1 || boottime > 600)
   {
      stc ("Please enter between 1 and 600 seconds.\n\r", ch);
      return;
   }

   tmp_bootcount = bootcount;
   bootcount = ++boottime;
   return;
}




/* Recover from a copyover - load players */
void copyover_recover ()
{
	DESCRIPTOR_DATA *d;
	FILE *fp;
	char name [100];
	#ifdef THREADED_DNS
		int h1,h2,h3,h4;
	#else
		char host[MSL];
	#endif
	int desc;
	bool fOld;
	char protocol[MSL];

	logf ("Copyover recovery initiated");

	fp = fopen (COPYOVER_FILE, "r");

	if (!fp) /* there are some descriptors open which will hang forever then ? */
	{
		perror ("copyover_recover:fopen");
		logf ("Copyover file not found. Exitting.\n\r");
		exit (1);
	}

	unlink (COPYOVER_FILE); /* In case something crashes - doesn't prevent reading	*/

	for (;;)
	{
	   #ifdef THREADED_DNS
	      fscanf (fp, "%d %s %s %d.%d.%d.%d\n", &desc, name, protocol, &h1,&h2,&h3,&h4);
	   #else
		fscanf (fp, "%d %s %s %s\n", &desc, name, protocol, host);
	   #endif
		if (desc == -1)
			break;

		/* Write something, and check if it goes error-free */
		if (!write_to_descriptor (desc, "\n\rRestoring the world...\n\r",0))
		{
			close (desc); /* nope */
			continue;
		}

		d = new_descriptor();
		d->descriptor = desc;

	   #ifdef THREADED_DNS
		d->Host[0]=h1;
		d->Host[1]=h2;
		d->Host[2]=h3;
		d->Host[3]=h4;
	   #else
		free_string(d->host);
		d->host=str_dup(host);
	   #endif
		d->next = descriptor_list;
		descriptor_list = d;
		CopyoverSet(d, protocol);
		d->connected = CON_COPYOVER_RECOVER; /* -15, so close_socket frees the char */


		/* Now, find the pfile */

		fOld = load_char_obj (d, name);

		if (!fOld) /* Player file not found?! */
		{
			write_to_descriptor (desc, "\n\rSomehow, your character was lost in the copyover. Sorry.\n\r", 0);
			close_socket (d);
		}
		else /* ok! */
		{
			write_to_descriptor (desc, "\n\rYou may now resume your adventure.\n\r",0);

			/* Just In Case */
			if (!d->character->in_room)
				d->character->in_room = get_room_index (ROOM_VNUM_PLAZA);

			/* Insert in the char_list */
			d->character->next = char_list;
			char_list = d->character;

			reset_char(d->character);

			if ( d->character->pcdata->newbie == 1 )
			{
				char_to_room (d->character, get_room_index( 31000 ) );
				do_look (d->character, "auto");
				act( "{c$n materializes!", d->character, NULL, NULL, TO_ROOM);
				d->connected = CON_PLAYING;
				stc( "{wWe have just recovered from a copyover where changes to the MUD occured.{x\n\r", d->character );
			}
			else
			{
				char_to_room (d->character, d->character->in_room);
				if ( IS_IMMORTAL( d->character ) && mud_port == 6500
					&& d->character->invis_level <= 0
					&& is_granted_name( d->character, "invis" ) )
						do_function( d->character, &do_invis, "" );
				do_look (d->character, "auto");
				act ("{c$n materializes!", d->character, NULL, NULL, TO_ROOM);
				d->connected = CON_PLAYING;
				if (d->character->pet != NULL)
				{
				    char_to_room(d->character->pet,d->character->in_room);
				    act("{c$n materializes!.",d->character->pet,NULL,NULL,TO_ROOM);
				}
			    if (d->character->mount != NULL)
			    {
					char_to_room(d->character->mount,d->character->in_room);
					act("$n materializes.",d->character->mount,NULL,NULL,TO_ROOM);
					add_follower(d->character->mount, d->character);
					do_mount(d->character, d->character->mount->name);
			    }
			}
		}

	}
   fclose (fp);
/*
do_invade( d->character, "waste");
*/
}
/*
 * Immortal invasion call.
 * Can accept a "random" argument to generate a random invasion of a
 * random town (that is located in the tables) by a random race of random
 * size.  Or, it can accept any (or all) of town name, race name, or
 * size of the invading force, in that order.  The town, race, and force
 * variables are declared and initialized as they are to shut up the
 * compiler. *grin*
 *
 * Coded by Thale
 */
void do_invade(CHAR_DATA *ch, char *arg)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];
  char arg3[MAX_INPUT_LENGTH];
  char buf[MAX_INPUT_LENGTH];
  int town = -1, race = -1;
  int min, max, temp, chance;
  int force = 0, amount;
  int d, x;
  bool found_enemy = FALSE;


  arg = one_argument(arg,arg1);
  arg = one_argument(arg,arg2);
  arg = one_argument(arg,arg3);

  if (arg1[0] == '\0')
  {
    send_to_char("Syntax: invade <town> <race> <force>\n\r",ch);
    send_to_char("Alternate Syntax: invade random\n\r",ch);
    return;
  }

  if (!str_cmp(arg1,"random"))
  {
    check_invade();
    send_to_char("Invasion generated randomly.\n\r",ch);
    return;
  }
  else
  {
    for (x = 0; town_invade_table[x].name != NULL; x++)
    {
      if (UPPER(town_invade_table[x].name[0]) == UPPER(arg1[0])
	&& !str_prefix(town_invade_table[x].name, arg1))
      {
	town = x;
	break;
      }
      town = -1;
    }
    if (town == -1)
    {
      send_to_char("There is no town with that name.\n\r",ch);
      return;
    }
  }

  if (arg2[0] == '\0')
  {
    for (d = 0; d <= 1000; d++)
    {
      chance = number_range(1,5);
      switch(chance)
      {
	case 1:
          if (town_invade_table[town].enemy1 != NULL)
          {
            race = racei_lookup(town_invade_table[town].enemy1);
	    found_enemy = TRUE;
          }
          else
            break;

	case 2:
          if (town_invade_table[town].enemy2 != NULL)
          {
            race = racei_lookup(town_invade_table[town].enemy2);
	    found_enemy = TRUE;
          }
          else
            break;

	case 3:
          if (town_invade_table[town].enemy3 != NULL)
          {
            race = racei_lookup(town_invade_table[town].enemy3);
	    found_enemy = TRUE;
          }
          else
            break;

	case 4:
          if (town_invade_table[town].enemy4 != NULL)
          {
            race = racei_lookup(town_invade_table[town].enemy4);
	    found_enemy = TRUE;
          }
	  else
	    break;

	case 5:
          if (town_invade_table[town].enemy5 != NULL)
          {
            race = racei_lookup(town_invade_table[town].enemy5);
	    found_enemy = TRUE;
          }
	if (found_enemy)
	  break;
      }
    }
  }
  else
    race = racei_lookup(arg2);

  if (race == -1)
  {
    send_to_char("There is no war-like race named that.\n\r",ch);
    return;
  }

  min = race_invade_table[race].min;
  max = race_invade_table[race].max;
  if (min > max)
  {
    temp = min;
    min = max;
    max = temp;
  }
  force = number_range(min,max);

  if (arg3[0] != '\0')
    force = atoi(arg3);

  if (force < 3 && ch->level < MAX_LEVEL - 1)
  {
    send_to_char("You must have at least 3 people in an invading force!\n\r",ch);
    return;
  }
  else if ((force > race_invade_table[race].max)
	&& (ch->level < MAX_LEVEL - 1))
  {
    sprintf(buf,"The maximum force for that race is %d\n\r",
	race_invade_table[race].max);
    send_to_char(buf,ch);
    return;
  }
  else if (force > race_invade_table[race].max + 75)
  {
    sprintf(buf,"The maximum force for that race is %d\n\r",
	race_invade_table[race].max + 75);
    send_to_char(buf,ch);
    return;
  }

  amount = racei_amount(race);

	if (invade(town,race,force,amount))
	{
		invasion->running = TRUE;
		free_string( invasion->location );
		invasion->location = str_dup( town_invade_table[town].name );
		free_string( invasion->invader );
		invasion->invader = str_dup( race_invade_table[race].name );
		stc("Done.  The screams should begin soon.\n\r",ch);
	}
  else
    stc("Hmmm, something went wrong.  Have the log files checked.\n\r",ch);

  return;
}

/** 	    Loads a player object into the mud, bringing them (and their
  *         pet) to you for easy modification.  Player must not be connected.
  *         Note: be sure to send them back when your done with them.
  */
void do_pload( CHAR_DATA *ch, char *argument )
{
  DESCRIPTOR_DATA d;
  bool isChar = FALSE;
  char name[MAX_INPUT_LENGTH];

  if (argument[0] == '\0')
  {
    send_to_char("Load who?\n\r", ch);
    return;
  }

  argument[0] = UPPER(argument[0]);
  argument = one_argument(argument, name);

  /* Dont want to load a second copy of a player who's allready online! */
  if ( get_char_world( ch, name ) != NULL )
  {
    send_to_char( "That person is allready connected!\n\r", ch );
    return;
  }

  isChar = load_char_obj(&d, name); /* char pfile exists? */

  if (!isChar)
  {
    send_to_char("Load Who? Are you sure? I can't seem to find them.\n\r", ch);
    return;
  }

  d.character->desc     = NULL;
  d.character->next     = char_list;
  char_list             = d.character;
  d.connected           = CON_PLAYING;
  reset_char(d.character);

  /* bring player to imm */
  if ( d.character->in_room != NULL )
  {
    /*
     * so ch returns to quit room when unloaded
     */
    d.character->was_in_room = ch->in_room;
    char_to_room( d.character, ch->in_room); /* put in room imm is in */
  }

  act( "$n has pulled $N from the pattern!",
        ch, NULL, d.character, TO_ROOM );

  if (d.character->pet != NULL)
   {
     char_to_room(d.character->pet,d.character->in_room);
     act("$n has entered the game.",d.character->pet,NULL,NULL,TO_ROOM);
   }

  return;
}

/**           Returns a player, previously 'ploaded' back to the void from
  *           whence they came.  This does not work if the player is actually
  *           connected.
  */
void do_punload( CHAR_DATA *ch, char *argument )
{
  CHAR_DATA *victim;
  char who[MAX_INPUT_LENGTH];

  argument = one_argument(argument, who);

  if ( ( victim = get_char_world( ch, who ) ) == NULL )
  {
    send_to_char( "They arent here.\n\r", ch );
    return;
  }

  /** Person is legitametly logged on... was not ploaded.
   */
  if (victim->desc != NULL)
  {
    send_to_char("I dont think that would be a good idea...\n\r", ch);
    return;
  }

  /* return player and pet to orig room */
  if (victim->was_in_room != NULL && victim->in_room != victim->was_in_room)
  {
    char_to_room(victim, victim->was_in_room);
    if (victim->pet != NULL)
      char_to_room(victim->pet, victim->in_room);
  }

  //save_char_obj(victim);
  save_char_obj(victim, FALSE, FALSE);
  do_quit(victim,"");

  act("$n has released $N back to the Pattern.",
       ch, NULL, victim, TO_ROOM);

  return;
}

void do_taint( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH], buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    CHAR_DATA *vch;
    DESCRIPTOR_DATA *d;

    one_argument( argument, arg );

    if (!str_cmp(arg,"room"))
    {
    /* evil room */

        for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
        {

            vch->hit 	= vch->hit/2;
            vch->stam	= vch->stam/2;
            update_pos( vch);
            act("{cYou feel sick.",ch,NULL,vch,TO_VICT);
        }

        sprintf(buf,"{cYou feel sick.");
        wiznet(buf,ch,NULL,WIZ_RESTORE,WIZ_SECURE,get_trust(ch));

        stc("{cRoom tainted.\n\r",ch);
        return;

    }

    if ( !str_cmp(arg,"all"))
    {
    /* taint all */

        for (d = descriptor_list; d != NULL; d = d->next)
        {
	    victim = d->character;

	    if (victim == NULL || IS_NPC(victim))
		continue;


            victim->hit 	= victim->hit/2;
            victim->stam	= victim->stam/2;
            update_pos( victim);
	    if (victim->in_room != NULL)
            act("{cYou feel the taint of the Shadow nearby and you begin to feel ill.",ch,NULL,victim,TO_VICT);
        }
	stc("{cAll active players have been tainted\n\r",ch);
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL && victim != ch)
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }
    if( ch->level < MAX_LEVEL -1 )
    {
		stc("You arent powerful enough.\n\r",ch);
		return;
	}
	/* taint player */
    victim->hit  = victim->hit/2;
    victim->stam = victim->stam/2;
    update_pos( victim );
    act( "{cYou suddenly feel very sick.", ch, NULL, victim, TO_VICT );
    /*sprintf(buf,"{c",
	IS_NPC(victim) ? victim->short_descr : victim->name);*/
    wiznet(buf,ch,NULL,WIZ_RESTORE,WIZ_SECURE,get_trust(ch));
    stc("{cPlayer Tainted\n\r", ch );
    return;
}

void do_shadowtaint( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH], buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    CHAR_DATA *vch;
    DESCRIPTOR_DATA *d;

    one_argument( argument, arg );

    if (!str_cmp(arg,"room"))
    {
    /* Shadow Taint room */

        for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
        {

            vch->hit 	= vch->hit/10;
            vch->stam	= vch->stam/10;
            update_pos( vch);
            act("{wA wave of nausea overcomes you.{x",ch,NULL,vch,TO_VICT);
	    do_function( ch, &do_weavehalt, "" );
        }

        sprintf(buf,"{cYou feel very sick.");
        wiznet(buf,ch,NULL,WIZ_RESTORE,WIZ_SECURE,get_trust(ch));

        stc("{cRoom Shadow tainted.\n\r",ch);
        return;

    }

    if ( !str_cmp(arg,"all"))
    {
    /* Shadow taint all */

        for (d = descriptor_list; d != NULL; d = d->next)
        {
	    victim = d->character;

	    if (victim == NULL || IS_NPC(victim))
		continue;


            victim->hit 	= victim->hit/10;
            victim->stam	= victim->stam/10;
            update_pos( victim);
	    if (victim->in_room != NULL)
            act("{wAs the Seals weaken you feel the presence of the Shadow overcome you.{x",ch,NULL,victim,TO_VICT);
	    do_function( ch, &do_weavehalt, "" );

        }
	stc("{cAll active players have been Shadow tainted\n\r",ch);
	return;
    }

    if ( ( victim = get_char_world( ch, arg ) ) == NULL && victim != ch)
    {
	stc("{cThey arent here.\n\r", ch );
	return;
    }
    if( ch->level < MAX_LEVEL -1 )
    {
		stc("You arent powerful enough.\n\r",ch);
		return;
	}
	/* Shadow taint player */
    victim->hit  = victim->hit/10;
    victim->stam = victim->stam/10;
    update_pos( victim );
    act( "{wA wave of nausea overcomes you.", ch, NULL, victim, TO_VICT );
    do_function( ch, &do_weavehalt, "" );
    wiznet(buf,ch,NULL,WIZ_RESTORE,WIZ_SECURE,get_trust(ch));
    stc("{cPlayer Shadow Tainted\n\r", ch );
    return;
}
