/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "aww.h"

struct   quick_alias
{
	char *key;
	char *result;
};
const struct quick_alias qa_type[] =
{
	{ "ga", "get all", },
	{ "wa", "wear all", },
	{ "pas", "put all in sack", },
	{ "pab", "put all in bag", },
	{ "da", "drop all", },
	{ "xag", "exchange all to gold", },
	{ "ra", "remove all", },
	{ "lg", "letgo all", },
	{ "hrecall", "home recall", },

	{ "g0", "weave 10 gate 0", },
	{ "g1", "weave 10 gate 1", },
	{ "g2", "weave 10 gate 2", },
	{ "g3", "weave 10 gate 3", },
	{ "g4", "weave 10 gate 4", },
	{ "g5", "weave 10 gate 5", },
	{ "g6", "weave 10 gate 6", },
	{ "g7", "weave 10 gate 7", },
	{ "g8", "weave 10 gate 8", },
	{ "g9", "weave 10 gate 9", },
	{ "p0", "weave 10 portal 0", },
	{ "p1", "weave 10 portal 1", },
	{ "p2", "weave 10 portal 2", },
	{ "p3", "weave 10 portal 3", },
	{ "p4", "weave 10 portal 4", },
	{ "p5", "weave 10 portal 5", },
	{ "p6", "weave 10 portal 6", },
	{ "p7", "weave 10 portal 7", },
	{ "p8", "weave 10 portal 8", },
	{ "p9", "weave 10 portal 9", },
	{ "wk", "weave 1 'knit flesh'", },
	{ "wl", "weave 1 'cure light'", },
	{ "wc", "weave 1 'cure critical'", },
	{ "wh", "weave 1 'heal'", },
	{ "", "" },
};

void do_qa( CHAR_DATA *ch, char *argument )
{
	int i;
	for ( i = 0; qa_type[i].key[0] != '\0'; i++ )
	{
		ptc( ch, "Alias: %s calls %s.\n\r", qa_type[i].key, qa_type[i].result );
	}
	return;
}

/* does aliasing and other fun stuff */
void substitute_alias(DESCRIPTOR_DATA *d, char *argument)
{
    CHAR_DATA *ch;
    char buf[MAX_STRING_LENGTH],prefix[MAX_INPUT_LENGTH],name[MAX_INPUT_LENGTH];
    char *point;
    int alias, i;

    ch = d->original ? d->original : d->character;

	for ( i = 0; qa_type[i].key[0] != '\0'; i++ )
	{
		if ( !str_cmp( argument, qa_type[i].key ) )
		{
			interpret( ch, qa_type[i].result );
			return;
		}
	}
    /* check for prefix */
    if (ch->prefix[0] != '\0' && str_prefix("prefix",argument))
    {
		if (strlen(ch->prefix) + strlen(argument) > MAX_INPUT_LENGTH)
	    stc("{cLine to long, prefix not processed.\r\n",ch);
		else
		{
		    sprintf(prefix,"%s %s",ch->prefix,argument);
		    argument = prefix;
		}
    }

    if (IS_NPC(ch) || ch->pcdata->alias[0] == NULL
    ||	!str_prefix("alias",argument) || !str_prefix("una",argument)
    ||  !str_prefix("prefix",argument))
    {
	interpret(d->character,argument);
	return;
    }

    strcpy(buf,argument);

    for (alias = 0; alias < MAX_ALIAS; alias++)	 /* go through the aliases */
    {
	if (ch->pcdata->alias[alias] == NULL)
	    break;

	if (!str_prefix(ch->pcdata->alias[alias],argument))
	{
	    point = one_argument(argument,name);
	    if (!strcmp(ch->pcdata->alias[alias],name))
	    {
		buf[0] = '\0';
		strcat(buf,ch->pcdata->alias_sub[alias]);
		if (point[0]) {
		    strcat(buf," ");
		    strcat(buf,point); }

	        if (strlen(buf) > MAX_INPUT_LENGTH - 1)
	        {
		    stc("{cAlias substitution too long. Truncated.\r\n",ch);
		    buf[MAX_INPUT_LENGTH -1] = '\0';
	        }
		break;
	    }
	}
    }
    interpret(d->character,buf);
}

void do_alia(CHAR_DATA *ch, char *argument)
{
    stc("{cI'm sorry, alias must be entered in full.\n\r",ch);
    return;
}

void do_alias(CHAR_DATA *ch, char *argument)
{
    CHAR_DATA *rch;
    char arg[MAX_INPUT_LENGTH],buf[MAX_STRING_LENGTH];
    int pos;

    smash_tilde(argument);

    if (ch->desc == NULL)
	rch = ch;
    else
	rch = ch->desc->original ? ch->desc->original : ch;

    if (IS_NPC(rch))
	return;

    argument = one_argument(argument,arg);


    if (arg[0] == '\0')
    {

	if (rch->pcdata->alias[0] == NULL)
	{
	    stc("{cYou have no aliases defined.\n\r",ch);
	    return;
	}
	stc("{cYour current aliases are:\n\r",ch);

	for (pos = 0; pos < MAX_ALIAS; pos++)
	{
	    if (rch->pcdata->alias[pos] == NULL
	    ||	rch->pcdata->alias_sub[pos] == NULL)
		break;

	    sprintf(buf,"{c    %s:  %s\n\r",rch->pcdata->alias[pos],
		    rch->pcdata->alias_sub[pos]);
	    stc(buf,ch);
	}
	return;
    }

    if (!str_prefix("una",arg) || !str_cmp("alias",arg))
    {
	stc("{cSorry, that word is reserved.\n\r",ch);
	return;
    }

 	if (strchr(arg,' ')
 	|| strchr(arg,'"')
 	|| strchr(arg,'\''))
 	{
		stc("The word to be aliased should not contain a space,\n\ra tick, a slash, or a double-quote.\n\r",ch);
		return;
	}

 	if (argument[0] == '\0')
    {
	for (pos = 0; pos < MAX_ALIAS; pos++)
	{
	    if (rch->pcdata->alias[pos] == NULL
	    ||	rch->pcdata->alias_sub[pos] == NULL)
		break;

	    if (!str_cmp(arg,rch->pcdata->alias[pos]))
	    {
		sprintf(buf,"{c%s aliases to '%s'.\n\r",rch->pcdata->alias[pos],
			rch->pcdata->alias_sub[pos]);
		stc(buf,ch);
		return;
	    }
	}

	stc("{cThat alias is not defined.\n\r",ch);
	return;
    }

    if (!str_prefix(argument,"delete") || !str_prefix(argument,"prefix"))
    {
	stc("{cThat shall not be done!\n\r",ch);
	return;
    }

    for (pos = 0; pos < MAX_ALIAS; pos++)
    {
	if (rch->pcdata->alias[pos] == NULL)
	    break;

	if (!str_cmp(arg,rch->pcdata->alias[pos])) /* redefine an alias */
	{
	    free_string(rch->pcdata->alias_sub[pos]);
	    rch->pcdata->alias_sub[pos] = str_dup(argument);
	    sprintf(buf,"{c%s is now realiased to '%s'.\n\r",arg,argument);
	    stc(buf,ch);
	    return;
	}
     }

     if (pos >= MAX_ALIAS)
     {
	stc("{cSorry, you have reached the alias limit.\n\r",ch);
	return;
     }

     /* make a new alias */
     rch->pcdata->alias[pos]		= str_dup(arg);
     rch->pcdata->alias_sub[pos]	= str_dup(argument);
     sprintf(buf,"{c%s is now aliased to '%s'.\n\r",arg,argument);
     stc(buf,ch);
}


void do_unalias(CHAR_DATA *ch, char *argument)
{
    CHAR_DATA *rch;
    char arg[MAX_INPUT_LENGTH];
    int pos;
    bool found = FALSE;

    if (ch->desc == NULL)
	rch = ch;
    else
	rch = ch->desc->original ? ch->desc->original : ch;

    if (IS_NPC(rch))
	return;

    argument = one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	stc("{cUnalias what?\n\r",ch);
	return;
    }

    for (pos = 0; pos < MAX_ALIAS; pos++)
    {
	if (rch->pcdata->alias[pos] == NULL)
	    break;

	if (found)
	{
	    rch->pcdata->alias[pos-1]		= rch->pcdata->alias[pos];
	    rch->pcdata->alias_sub[pos-1]	= rch->pcdata->alias_sub[pos];
	    rch->pcdata->alias[pos]		= NULL;
	    rch->pcdata->alias_sub[pos]		= NULL;
	    continue;
	}

	if(!strcmp(arg,rch->pcdata->alias[pos]))
	{
	    stc("{cAlias removed.\n\r",ch);
	    free_string(rch->pcdata->alias[pos]);
	    free_string(rch->pcdata->alias_sub[pos]);
	    rch->pcdata->alias[pos] = NULL;
	    rch->pcdata->alias_sub[pos] = NULL;
	    found = TRUE;
	}
    }

    if (!found)
	stc("{cNo alias of that name to remove.\n\r",ch);
}















