/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include "aww.h"
#include "recycle.h"
#include "tables.h"
#include "interp.h"

AUCTION_DATA  * auction_list;

void appraise_object args((CHAR_DATA *ch, OBJ_DATA *obj, bool plrmarket));
char *check_dye args((OBJ_DATA *obj, char *string, int dye_vnum));

int get_auction_fee(int bid)
{
	if (bid > 5000)
	{
		return ((bid+99)/100);
	}
	return 0;
}

void do_auction(CHAR_DATA *ch, char *argument)
{
	AUCTION_DATA *auc;
	AUCTION_DATA *p;
	OBJ_DATA *obj = NULL;
	long minbid = 0,MINIMUM_BID = 10;
	int count;
	char arg1[MIL];
	char buf1[MSL], buf2[MSL];

	if (!(auction_list == NULL) && auction_list->gold_held > 0)
	{
		MINIMUM_BID = UMAX(MINIMUM_BID,auction_list->gold_held/20);
		MINIMUM_BID = UMIN(MINIMUM_BID, 5000);
	}
	argument = one_argument( argument, arg1 );

    if ( ch == NULL || IS_NPC(ch) )
	return;

	/* Requesting info on the item */
	if(arg1[0] == '\0' || !str_cmp(arg1,"info"))
	{
		if(auction_list == NULL)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		obj = auction_list->item;

		if(!obj)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		if(ch == auction_list->owner && !IS_IMMORTAL(ch))
		{
		sprintf(buf1,"You're auctioning %s.{x\n\r",obj->short_descr);
		stc(buf1,ch);
		return;
		}
		ptc(ch,"Auctioner: %s{x\n\r", auction_list->owner->name);

		appraise_object(ch,obj, FALSE);
		return;
	} /* Aborting the auction */
	else if( !str_cmp(arg1,"cancel") && (IS_IMMORTAL(ch) || auction_list->owner == ch))
	{
		if(auction_list == NULL)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		sprintf(buf2, "%s has cancelled the auction!\n\r", ch->name);
		auction_channel(buf2);

		if(auction_list->high_bidder != NULL)
		auction_list->high_bidder->money[0][1] += auction_list->gold_held;

		if(auction_list->item != NULL)
		{
		obj_to_char(auction_list->item,ch);
		sprintf(buf1,"%s appears in your hands.{x\n\r",check_dye(auction_list->item, "", auction_list->item->dye_vnum));
		stc(buf1,ch);
		}

		reset_auc(auction_list);
		return;
	}
	if ( !str_cmp( arg1,"halt" ) && IS_IMMORTAL( ch ) )
	{
		if ( auction_list == NULL )
		{
			stc("There's nothing up for auction right now.\n\r,", ch);
			return;
		}
		/* also change in do_copyover */
		sprintf( buf2, "%s -- Auction is being halted.\n\r",
			capit_colorsafe( auction_list->item->short_descr ) );
		auction_channel( buf2 );
		if(auction_list->high_bidder != NULL)
			auction_list->high_bidder->money[0][1] += auction_list->gold_held;
		if ( auction_list->owner != NULL && !IS_NPC( auction_list->owner ) )
		{
			obj_to_char( auction_list->item, auction_list->owner );
			ptc(auction_list->owner,"%s is returned to you.\n\r", check_dye(auction_list->item, "", auction_list->item->dye_vnum));
		}
		reset_auc( auction_list );
		return;
	}
	/* Listing the items for sale */
	else if(!str_cmp(arg1,"list"))
	{
		if(auction_list == NULL)
		{
			stc("There is nothing up for auction right now!\n\r",ch);
			return;
		}

		if(auction_list->next == NULL)
		{
			do_auction(ch,"info");
			return;
		}

		for(count = 1,auc = auction_list;auc;auc = auc->next,count++)
		{
			if(auc->item != NULL && auc->item->short_descr != NULL)
			{
			sprintf(buf1,"(%d. (Min Bid %5ld) %s, by %s{x\n\r",
			count, auc->min_bid, check_dye(auc->item, "", auc->item->dye_vnum), auc->owner->name);
			stc(buf1,ch);
			}
		}
		return;
	}/* Bidding on an item */
	else if(!str_cmp(arg1,"bid"))
	{
	long bid = 0;

		if(auction_list == NULL)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		obj = auction_list->item;

		if(!obj)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		if ( auction_list->owner == ch )
		{
	    stc("You can't bid on your own items.\n\r",ch);
	    return;
		}

		/* FIXME this needs to be modified to take into account known
		 * people that are not alts, but share an IP address */
		/* also put it in the bankbid section too */
		if ( auction_list->owner->desc && ch->desc )
		{
/*			if ( !str_cmp( auction_list->owner->desc->host, ch->desc->host ) ) */
			if(is_alt_of(ch,auction_list->owner))
			{
			stc("You can't bid on your alt's items.\n\r", ch);
			return;
			}
		}

		if ( auction_list->high_bidder == ch )
		{
	    stc("You already hold the highest bid.\n\r",ch);
	    return;
		}

		if(argument[0] == '\0')
		{
		stc("How much did you say you were bidding?\n\r",ch);
		return;
		}

		bid = atol(argument);

		if ( !check_coins(ch,bid,0,0) )
		{
	    stc("{CYou can't cover that bid.{x\n\r",ch);
	    return;
		}

		if(bid < auction_list->min_bid)
		{
		sprintf( buf1, "The minimum bid is %ld copper pennies.\n\r",auction_list->min_bid);
	    stc(buf1,ch);
	    return;
		}

		if ( bid <= (auction_list->current_bid + MINIMUM_BID-1) )
		{
	   	ptc(ch,"You must bid %d above the current bid of %ld copper pennies.\n\r",
		MINIMUM_BID, auction_list->current_bid );
	    return;
		}
		if(bid <= auction_list->current_bid)
		{
			if(bid < (auction_list->current_bid + 10))
			{
			stc("You have to outbid the current amount by at least 10 copper pennies.",ch);
			return;
			}

			sprintf(buf1, "You must bid above the current bid of %ld copper pennies.\n\r",
			auction_list->current_bid );
			return;
		}

		sprintf(buf1, "%ld copper pennies has been offered for %s by %s.\n\r",
	    bid, check_dye(auction_list->item, "", auction_list->item->dye_vnum), ch->name);
		auction_channel( buf1 );
		/* I disagree with giving out the name of bidder --Sil */

		if ( auction_list->high_bidder != NULL )
		auction_list->high_bidder->money[0][1] += auction_list->gold_held;
		stc("{wYour bid has been accepted.",ch);

		deduct_cost(ch, bid,0);

		auction_list->gold_held		= bid;
		auction_list->high_bidder	= ch;
		auction_list->current_bid	= bid;
		auction_list->status	 	= 0;
		return;
	}
	/* use coins in the bank */
	else if(!str_prefix(arg1,"bankbid"))
	{
	long bid = 0;

		if(auction_list == NULL)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		obj = auction_list->item;

		if(!obj)
		{
		stc("There's nothing up for auction right now.\n\r",ch);
		return;
		}

		if ( auction_list->owner == ch )
		{
	    stc("You can't bid on your own items.\n\r",ch);
	    return;
		}

		if ( auction_list->owner->desc && ch->desc )
		{
			if ( !str_cmp( auction_list->owner->desc->host, ch->desc->host ) )
			{
			stc("You can't bid on your alt's items.\n\r", ch);
			return;
			}
		}

		if ( auction_list->high_bidder == ch )
		{
	    stc("You already hold the highest bid.\n\r",ch);
	    return;
		}

		if(argument[0] == '\0')
		{
		stc("How much did you say you were bidding?\n\r",ch);
		return;
		}

		bid = atol(argument);

		if ( !check_coins(ch, bid,1,1))
		{
	    stc("You don't have that much in the bank.\n\r",ch);
	    return;
		}

		if(bid < auction_list->min_bid)
		{
		sprintf( buf1, "The minimum bid is %ld copper pennies.\n\r",auction_list->min_bid);
	    stc(buf1,ch);
	    return;
		}

		if ( bid <= (auction_list->current_bid + MINIMUM_BID-1) )
		{
	   	ptc(ch,"You must bid %d above the current bid of %ld copper pennies.\n\r",
		MINIMUM_BID, auction_list->current_bid );
	    return;
		}
		if(bid <= auction_list->current_bid)
		{
			if(bid < (auction_list->current_bid + 10))
			{
			stc("You have to outbid the current amount by at least 10 copper pennies.",ch);
			return;
			}

			sprintf(buf1, "You must bid above the current bid of %ld copper pennies.\n\r",
			auction_list->current_bid );
			return;
		}

		sprintf(buf1, "%ld copper pennies has been offered for %s by %s.\n\r",
	    bid, check_dye(auction_list->item, "", auction_list->item->dye_vnum), ch->name);
/*		sprintf(buf1, "%ld copper pennies has been offered for %s.{x\n\r",
	    bid, check_dye(auction_list->item, "", auction_list->item->dye_vnum)); */
		auction_channel( buf1 );
		stc("{wYour bid has been accepted.",ch);

		if ( auction_list->high_bidder != NULL )
		auction_list->high_bidder->money[0][1] += auction_list->gold_held;

		deduct_cost(ch, bid,1);

		auction_list->gold_held		= bid;
		auction_list->high_bidder	= ch;
		auction_list->current_bid	= bid;
		auction_list->status	 	= 0;
		return;
	}
	/* Putting up an item for sale */

	if ( (obj = get_obj_carry( ch, arg1, ch )) == NULL )
    {
		stc("You aren't carrying that item.\n\r",ch);
		return;
    }

    if ( IS_SET( obj->extra_flags,ITEM_NOSELL ) )
    {
		stc("That item has been flagged nosell. Type unkeep <item> to toggle this off.\n\r",ch);
		return;
	}
    if ( IS_OBJ_STAT( obj, ITEM_NODROP ) )
    {
		stc("You can't let go of that item.\n\r",ch);
		return;
    }
    if ( IS_OBJ_STAT( obj, ITEM_ROT_DEATH ) )
    {
    stc("You can't auction rotting items.\n\r",ch);
    return;
    }
    if ( obj->timer > 1 )
    {
    stc("This item might rot, and can't be auctioned.\n\r",ch);
    return;
    }
    if ( obj->pIndexData->vnum == 9 )
    {
    stc("You can't auction enhancement tokens.\n\r",ch);
    return;
    }
    if ( obj->item_type == ITEM_TREASURECHEST )
    {
    stc("Try picking it and getting the items from it, then auction those.\n\r",ch);
    return;
    }
	if (obj->cost < 1)
	{
		stc("This object has no value.\n\r",ch);
		return;
	}

    if ( is_obj_clan( obj ))
	{
		stc("You can't auction guild items.\n\r",ch);
		return;
	}
/*
	if ( !clan_table[clan_lookup( obj->guild )].independent )
	{
		stc("You can't auction guild items.\n\r",ch);
		return;
	}
*/

	if(count_auc(ch) >= 3)
	{
		stc("You are only allowed to auction 3 items a time!\n\r",ch);
		return;
	}

	if (argument[0] != '\0')
	minbid = atol(argument);


	if (minbid>5000)
	{
		if(minbid > 100000)
		{
			minbid = 100000;
		}
		if (!check_coins(ch,get_auction_fee(minbid),1,1))
		{
			stc("You don't have enough coin in the bank to afford the auction fee!\n\r",ch);
			stc("You have to pay a 1% starting fee, if you have an opening bid higher than 5000 copper.\n\r",ch);
			return;
		}
		deduct_cost(ch,get_auction_fee(minbid),1);
		stc("As your opening bid is higher than 5000 copper, you are charged a 1% fee, it will be returned in case of a succesful auction.\n\r",ch);
	}

	if(auction_list == NULL)
	{
		auc = new_auction();
		auction_list = auc;
		auction_list->next = NULL;
	}
	else
	{
		auc = new_auction();

		for(p = auction_list;p;p = p->next)
		{
		if(p->next == NULL)
		p->next = auc;
		auc->next = NULL;
		}
	}

    auc->owner					= ch;
    auc->item					= obj;
    auc->current_bid			= 0;
    auc->status					= -1;
	if(minbid > 0)
	auc->min_bid				= minbid;
	else
	auc->min_bid				= 0;

    obj_from_char( obj );
    return;
}

void auction_update(void)
{
	char buf1[MAX_STRING_LENGTH];
	char temp[MAX_STRING_LENGTH];

	if ( auction_list != NULL && auction_list->item != NULL )
	{
    auction_list->status++;

	if (auction_list->current_bid == 0)
	{
		if (auction_list->status == 0)
		{
		sprintf(buf1,"Now taking bids on %s%s{w", check_dye(auction_list->item, "", auction_list->item->dye_vnum),
		auction_list->min_bid > 0 ? "":".\n\r");
		if(auction_list->min_bid > 0)
		{
		sprintf(temp," (Min Bid %ld){x\n\r",auction_list->min_bid);
		strcat(buf1,temp);
		}

		auction_channel( buf1 );
		}

		if ( auction_list->status == AUCTION_LENGTH )
		{
	    sprintf(buf1, "No bids on %s - {witem removed.{x\n\r",
		check_dye(auction_list->item, "", auction_list->item->dye_vnum));
	    auction_channel( buf1 );

		if ( auction_list->owner != NULL )
		{
		    obj_to_char( auction_list->item, auction_list->owner );

			sprintf(buf1, "%s is returned to you.{x\n\r", capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)) );
			stc( buf1, auction_list->owner );
		}
		else
			bugf("Auction:: Auction_list->owner NULL attempting to end auction!{x");
		reset_auc(auction_list);
	    return;
	}

		if (auction_list->status == AUCTION_LENGTH - 1)
		{
			sprintf(buf1, "%s {w- going twice (No Bids Received).{x\n\r",
		capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)) );
		auction_channel( buf1 );
		return;
		}

		if (auction_list->status == AUCTION_LENGTH - 2)
		{
			sprintf(buf1, "%s {w- going once (No Bids Received).{x\n\r",
		capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)) );
		auction_channel( buf1 );
		return;
		}
	}
	else
	{
    if ( auction_list->status == AUCTION_LENGTH )
    {
	sprintf(buf1,"%s {wsold to %s for %ld copper pennies.{x\n\r",
	    capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)),
	    auction_list->high_bidder->name,
	    auction_list->current_bid );
	auction_channel( buf1);

	if ( !IS_NPC( auction_list->high_bidder ) && auction_list->high_bidder->pcdata != NULL )
	{
		auction_list->high_bidder->pcdata->summary[SUM_AUCTIONS_WON] += 1;
		auction_list->high_bidder->pcdata->summary[SUM_AUCTIONS_WON_EVER] += 1;
	}
	auction_list->owner->money[0][1] += (auction_list->gold_held * 9) / 10+get_auction_fee(auction_list->min_bid);

	sprintf(temp, "%ld copper pennies {x", ((auction_list->gold_held * 9) / 10) + get_auction_fee(auction_list->min_bid));
	sprintf(buf1, "You receive %s, which is the sale price MINUS the Auctioneers' fee.{x\n\r",
		auction_list->gold_held > 0 ? temp : "0 copper pennies{x");
	stc( buf1, auction_list->owner );

	obj_to_char( auction_list->item, auction_list->high_bidder );

	sprintf(buf1, "%s appears in your hands.{x\n\r",
		capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)) );
	stc( buf1, auction_list->high_bidder );

	reset_auc(auction_list);
	return;
    }

    if ( auction_list->status == AUCTION_LENGTH - 1 )
    {
	sprintf(buf1, "%s {w- going twice at %ld copper pennies.{x\n\r",
		capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)),
		auction_list->current_bid );
	auction_channel( buf1 );
	return;
    }

		if ( auction_list->status == AUCTION_LENGTH - 2 )
		{
        sprintf(buf1, "%s {w- going once at %ld copper pennies.{x\n\r",
        	capit_colorsafe(check_dye(auction_list->item, "", auction_list->item->dye_vnum)),
        	auction_list->current_bid );
        auction_channel( buf1 );
        return;
		}
	}
	}

    return;
}

void auction_channel( char * msg )
{
    char buf1[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;

    sprintf(buf1, "\n\r{g[{GAuction{g]{w %s{x", msg ); /* Add color if you wish */

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		CHAR_DATA *victim;

		victim = d->original ? d->original : d->character;

		if ( d->character != NULL && d->connected == CON_PLAYING &&
		!IS_SET(victim->comm,COMM_NOAUCTION) &&
		!IS_SET(victim->comm,COMM_QUIET) )
		{
			stc( buf1, victim );
		}
	}

    return;
}

void reset_auc(AUCTION_DATA *auc)
{
	if(!IS_VALID(auc))
	return;

	auc->current_bid = 0;
	auc->gold_held	 = 0;
	auc->high_bidder = NULL;
	auc->item		 = NULL;
	auc->min_bid	 = 0;
	auc->owner		 = NULL;
	auc->status		 = 0;

	if(auc == auction_list)
	{
		if(auc->next != NULL)
		auction_list = auc->next;
		else
		auction_list = NULL;

		free_auction(auc);
		return;
	}

	free_auction(auc);
	return;
}

int count_auc(CHAR_DATA *ch)
{
	AUCTION_DATA *q;
	int count;

	for(count = 0,q = auction_list;q;q = q->next)
	{
		if(q->owner == ch)
		count++;
	}

	return count;
}

