/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/


#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "magic.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"

char map_chars[10] = "|-|-ud/\\/\\";
char lcolor = 'w';

typedef struct map_type MAP_DATA;

#define    ANSI_KEY '{'
#define    ANSI_CUSTOM '\x11'
#define    ANSI_END '\x12'
#define    MAPX     12
#define    MAPY     12
#define    BOUNDARY(x, y) (((x) < 0) || ((y) < 0) || \
((x) > MAPX) || ((y) > MAPY))

bool count_room args((ROOM_INDEX_DATA *room));

struct map_type
{
	char color;
	char symbol;
	int vnum;
	int depth;
	long info;
	bool up;
	bool dn;
};

MAP_DATA map[MAPX + 1][MAPY + 1];

int get_max_depth(CHAR_DATA *ch)
{
	int depth = 3;

	int sight = 0;

	sight = get_skill( ch, gsn_far_sight );
	sight /= 5;

	if ( sight > 4 )
		sight = 4;

	depth += sight;

	if (get_eq_char(ch, WEAR_LIGHT))	depth += 2;
	if (ch->race == race_lookup("aiel"))	depth += 1;
	if (ch->race == race_lookup("grayman"))	depth += 1;
	if (ch->race == race_lookup("fade"))	depth += 2;
	if (ch->race == race_lookup("warder"))	depth += 2;

	if (IS_IMMORTAL(ch))			depth = 10;

	//ptc( ch, "Map Depth: %d\n\r", depth );
	return depth;
}

bool has_explored_room (CHAR_DATA *ch, ROOM_INDEX_DATA *room)
{
	int offset, bit;

	if (IS_NPC(ch))		return TRUE;
	if (!count_room(room))	return TRUE;

	offset = room->vnum / 8;
	bit    = room->vnum % 8;

	if (ch->pcdata->explored[offset] & 1 << bit)
		return TRUE;

	return FALSE;
}

/* number of rooms that have been explored in the current area */
int areacount(CHAR_DATA * ch)
{
	long pIndex = 0;
	int count = 0;
	int bit, offset;
	ROOM_INDEX_DATA *pRoom;

	if (IS_NPC(ch))
		return 0;

	if ((ch->in_room == NULL) || (ch->in_room->area == NULL))
		return 0;

	for (pIndex = ch->in_room->area->min_vnum; pIndex <= ch->in_room->area->max_vnum; pIndex++)
	{
		if ( (pRoom = get_room_index(pIndex)) == NULL )
                	continue;

		if (!count_room(pRoom))	continue;

		offset = pIndex / 8;
		bit    = pIndex % 8;

		if (ch->pcdata->explored[offset] & 1 << bit)
			count++;
	}

	return count;
}

/* number of rooms that can be explored in the current area */
int arearooms(CHAR_DATA * ch)
{
	int count = 0;
	long pIndex = 0;
	ROOM_INDEX_DATA *pRoom;

	if (IS_NPC(ch))
		return 0;

	if ((ch->in_room == NULL) || (ch->in_room->area == NULL))
		return 0;

	for (pIndex = ch->in_room->area->min_vnum; pIndex <= ch->in_room->area->max_vnum; pIndex++)
	{
		if ((pRoom = get_room_index(pIndex)) == NULL)
			continue;

		if (!count_room(pRoom))	continue;

		count++;
	}

	return count;
}

void get_exit_dir(int dir, int *x, int *y, int xorig, int yorig)
{
	switch (dir)
	{

	case DIR_NORTH:
		*x = xorig;
		*y = yorig - 1;
		break;
	case DIR_EAST:
		*x = xorig + 1;
		*y = yorig;
		break;
	case DIR_SOUTH:
		*x = xorig;
		*y = yorig + 1;
		break;
	case DIR_WEST:
		*x = xorig - 1;
		*y = yorig;
		break;
	case DIR_NORTHEAST:
		*x = xorig + 1;
		*y = yorig - 1;
		break;
	case DIR_NORTHWEST:
		*x = xorig - 1;
		*y = yorig - 1;
		break;
	case DIR_SOUTHEAST:
		*x = xorig + 1;
		*y = yorig + 1;
		break;
	case DIR_SOUTHWEST:
		*x = xorig - 1;
		*y = yorig + 1;
		break;
	
	default:
		*x = -1;
		*y = -1;
		break;
	}
}

void clear_coord(int x, int y)
{
	map[x][y].symbol = ' ';
	map[x][y].color = 'x';
	map[x][y].vnum = 0;
	map[x][y].depth = 0;
	map[x][y].up = FALSE;
	map[x][y].dn = FALSE;
	map[x][y].info = 0;
}

void clear_room(int x, int y)
{
	int dir, exitx, exity;

	for (dir = DIR_NORTH; dir <= DIR_NORTHWEST; dir++)
	{
		if (dir == DIR_UP || dir == DIR_DOWN)	continue;

		get_exit_dir(dir, &exitx, &exity, x, y);
		if (!BOUNDARY(exitx, exity))
			clear_coord(exitx, exity);
	}
}

struct sector_color_type
{
	int bit;
	char display_color;
	char display_symbol;
};

const struct sector_color_type sector_color_table[] = {
	{SECT_INSIDE, 'w', 'o'},
	{SECT_CITY, 'w', 'o'},
	{SECT_FIELD, 'G', '*'},
	{SECT_FOREST, 'g', '%'},
	{SECT_HILLS, 'y', '*'},
	{SECT_MOUNTAIN, 'W', '@'},
	{SECT_WATER_SWIM, 'B', '='},
	{SECT_WATER_NOSWIM, 'b', '='},
	{SECT_UNUSED, 'w', 'o'},
	{SECT_AIR, 'C', '~'},
	{SECT_DESERT, 'y', '+'},
	{SECT_BLIGHT, 'y', '+'},
	{SECT_COASTAL, 'C', '+'},
	{SECT_ROAD, 'y', '+'},
	{SECT_MARSH, 'g', '+'},
	{SECT_TUNDRA, 'W', '+'},
	{SECT_WASTE, 'y', '+'},
	{SECT_LAKE_FRESH, 'c', '+'},
	{SECT_LAKE_SALT, 'c', '+'},
	{SECT_RIVER_FRESH, 'c', '+'},
	{SECT_RIVER_SALT, 'c', '+'},
	{SECT_VOLCANIC, 'r', '+'},
	{SECT_OCEAN, 'B', '+'},
	{SECT_UNDERGROUND, 'y', '+'},
	{SECT_UNDERWATER, 'b', '+'},
	{SECT_FOOTHILLS, 'g', '^'},
	{SECT_JUNGLE, 'G', '&'},
	{SECT_FARMLAND, 'G', '#'},
	{SECT_STEPPE, 'G', '='},
	{SECT_PLAIN, 'g', '='},
	{SECT_MEADOW, 'y', '='},
	{SECT_MAX, 'w', 'o'}
};

char get_sector_color(int sector)
{
	int looper;

	for (looper = 0;; looper++)
		if ((sector_color_table[looper].bit == sector)
			|| (sector_color_table[looper].bit == SECT_MAX))
			break;
	return (sector_color_table[looper].display_color);
}

char get_sector_symbol(int sector)
{
	int looper;

	for (looper = 0;; looper++)
		if ((sector_color_table[looper].bit == sector)
			|| (sector_color_table[looper].bit == SECT_MAX))
			break;
	return (sector_color_table[looper].display_symbol);
}

CHAR_DATA *find_keeper_room( ROOM_INDEX_DATA *room )
{
    CHAR_DATA *keeper;
    SHOP_DATA *pShop;

    pShop = NULL;
    for ( keeper = room->people; keeper; keeper = keeper->next_in_room )
    {
	if ( IS_NPC(keeper) && (pShop = keeper->pIndexData->pShop) != NULL )
	    break;
    }

    if ( pShop == NULL )
	return NULL;

    return keeper;
}

CHAR_DATA *find_pc_room( ROOM_INDEX_DATA *room )
{
    CHAR_DATA *pc;

    for ( pc = room->people; pc; pc = pc->next_in_room )
    {
	if (!IS_NPC(pc))	return pc;
    }

    return NULL;
}




void map_exits(CHAR_DATA * ch, ROOM_INDEX_DATA * pRoom, int x, int y, int depth)
{
	int door;
	int exitx = 0, exity = 0;
	int roomx = 0, roomy = 0;
	EXIT_DATA *pExit;

	if (!can_see_room(ch, pRoom))
		return;

	map[x][y].symbol = get_sector_symbol(pRoom->sector_type);
	map[x][y].color = get_sector_color(pRoom->sector_type);

	if (IS_SET(pRoom->room_flags, ROOM_BUILDHOME))
	{
		map[x][y].symbol = 'H';
		map[x][y].color = 'Y';
	}

	if (find_keeper_room(pRoom) != NULL)
	{
		map[x][y].symbol = '$';
		map[x][y].color = 'G';
	}

	if (find_pc_room(pRoom) != NULL)
	{
		map[x][y].symbol = 'X';
		map[x][y].color = 'Y';
	}

	if (!has_explored_room(ch, pRoom))
		map[x][y].color = 'D';

	map[x][y].vnum = pRoom->vnum;
	map[x][y].depth = depth;
	map[x][y].dn = FALSE;
	map[x][y].up = FALSE;

	/*if (!IS_NPC(ch) && getbit(ch->pcdata->explored, pRoom->vnum))*/
	if ( !IS_NPC(ch) )
	{
		map[x][y].info = pRoom->room_flags;

		if (pRoom->exit[DIR_DOWN] != NULL)
			map[x][y].dn = TRUE;
		if (pRoom->exit[DIR_UP] != NULL)
			map[x][y].up = TRUE;
	}

	//ptc( ch, "Map Depth: %d\n\r", depth );

	if (depth >= get_max_depth(ch))
		return;

	for (door = DIR_NORTH; door <= DIR_NORTHWEST; door++)
	{
		if ((pExit = pRoom->exit[door]) == NULL)
			continue;

		if (pExit->u1.to_room == NULL)
			continue;

		if (!can_see_room(ch, pExit->u1.to_room))
			continue;

		if (is_room_owner(ch, pRoom) && !is_room_owner(ch, pExit->u1.to_room))
			continue;

		get_exit_dir(door, &exitx, &exity, x, y);
		get_exit_dir(door, &roomx, &roomy, exitx, exity);

		if (BOUNDARY(exitx, exity) || BOUNDARY(roomx, roomy))
			continue;

		if ((map[roomx][roomy].vnum != 0)
			&& (map[roomx][roomy].vnum != pExit->u1.to_room->vnum)
			/* only clear exits and rooms of higher depth */
			&& map[roomx][roomy].depth > depth && depth < get_max_depth(ch))
		{
			clear_room(roomx, roomy);
		}

		if (depth == get_max_depth(ch))
			continue;

		map[exitx][exity].depth = depth;
		map[exitx][exity].vnum = pExit->u1.to_room->vnum;
		map[exitx][exity].symbol = map_chars[door];
		map[exitx][exity].color = 'D';
//		map[exitx][exity].color = IS_SET(pExit->exit_info, EX_CLOSED) ? 'W' : 'D';
		map[exitx][exity].info = pExit->exit_info;

		if (((depth < get_max_depth(ch))
			&& ((map[roomx][roomy].vnum == pExit->u1.to_room->vnum)
				|| (map[roomx][roomy].vnum == 0))) && !IS_SET(pExit->exit_info, EX_CLOSED))
		{
			depth++;
			map_exits(ch, pExit->u1.to_room, roomx, roomy, depth + 1);
			depth--;
		}
	}
}

void reformat_desc(char *desc)
{
	char *p;
	unsigned int l, m;
	char buf[MSL * 2];

	l = 0;
	m = 0;
	buf[0] = '\0';

	if (desc[0] == '\0')
		return;

	/* remove all \n & \r */
	for (m = 0; m <= strlen(desc); m++)
		if (desc[m] == '\n' || desc[m] == '\r')
			desc[m] = ' ';

	/* remove multiple spaces */
	for (p = desc; *p != '\0'; p++)
	{
		if (*p == ' ' && *(p + 1) == ' ')
		{
			buf[l] = *p;
			l++;
			do
			{
				p++;
			}
			while (*p == ' ');
		}
		buf[l] = *p;
		l++;
	}
	buf[l] = '\0';
	sprintf(desc, buf);

	return;
}

unsigned int get_line(char *desc, unsigned int max_len)
{
	unsigned int m;
	unsigned int l;
	char buf[MSL];

	if (strlen(desc) <= max_len)
		return 0;

	buf[0] = '\0';
	l = 0;

	for (m = 0; m <= strlen(desc); m++)
	{
		if (desc[m] == ANSI_KEY)
			m += 2;
		else if (desc[m] == ANSI_CUSTOM)
		{
			while (desc[m] != ANSI_END)
				m++;
			m++;
		}
		l++;

		if (l > max_len)
			break;
	}

	for (l = m; l > 0; l--)
	{
		if (desc[l] == ANSI_KEY)
		{
			lcolor = desc[l + 1];
			break;
		}
	}
	for (l = m; l > 0; l--)
		if (desc[l] == ' ')
			break;

	return l + 1;
}

void show_map(CHAR_DATA * ch, char *text)
{
	char buf[MSL * 2], buf2[MSL];
	int x, y, pos;
	char *p;
	bool alldesc = FALSE;
	int rcnt = areacount(ch);
	double rooms = (double) (arearooms(ch));
	double percent = (double) 100 * rcnt / rooms;
	int maxlen = 78;

    /*
	int maxlen = (ch->desc
				  && ch->desc->scr_width > 0) ? ch->desc->scr_width - 2 : 78;
	*/
	int maplen = maxlen - 15;

	if (IS_NULLSTR(text))
		alldesc = TRUE;

	pos = 0;
	p = text;
	buf[0] = '\0';
	lcolor = 'x';

	if (!alldesc && text[0] == '{')
	sprintf(buf2, "{%c", text[1]);
	else sprintf(buf2, "%s", "{w");

	/*if (IS_NPC(ch) || IS_SET(ch->in_room->room_flags, ROOM_NOEXPLORE))
		sprintf(buf, "{R+------------+{%c ", lcolor);
	else*/

	if (!str_cmp(ch->in_room->area->file_name, "plrhomes.are"))
		sprintf(buf, "{R+-------------+%s ", buf2);
	else	sprintf(buf, "{R+-------[{x%3.0f%%{R]+%s ", percent, buf2);

	if (!alldesc)
	{
		pos = get_line(p, maplen);
		if (pos > 0)
		{
			strncat(buf, p, pos);
			p += pos;
		}
		else
		{
			strcat(buf, p);
			alldesc = TRUE;
		}
	}
	strcat(buf, "\n\r");

	for (y = 0; y <= MAPY; y++)
	{
		strcat(buf, "{R|{w");

		for (x = 0; x <= MAPX; x++)
		{
			if (map[x][y].symbol != 'X')
			{
				if (map[x][y].up && map[x][y].dn)
					map[x][y].symbol = 'O';
				if (!map[x][y].up && map[x][y].dn)
					map[x][y].symbol = 'D';
				if (map[x][y].up && !map[x][y].dn)
					map[x][y].symbol = 'U';
			}
			sprintf(buf + strlen(buf), "{%c%c{w",
			map[x][y].color,
/*			get_room_index(map[x][y].vnum) ? get_sector_color(get_room_index(map[x][y].vnum)->sector) : "{D",	*/
			map[x][y].symbol);
		}

		strcat(buf, "{R| ");
		strcat(buf, buf2);
		if (!alldesc)
		{
			pos = get_line(p, maplen);
			if (pos > 0)
			{
				strncat(buf, p, pos);
				p += pos;
			}
			else
			{
				strcat(buf, p);
				alldesc = TRUE;
			}
		}
		strcat(buf, "\n\r");
	}
	strcat(buf, "{R+-------------+");
	strcat(buf, buf2);
	strcat(buf, " ");

	if (!alldesc)
	{
		pos = get_line(p, maplen);
		if (pos > 0)
		{
			strncat(buf, p, pos);
			p += pos;
		}
		else
		{
			strcat(buf, p);
			alldesc = TRUE;
		}
	}

	if (!alldesc)
	{
		do
		{
			pos = get_line(p, maxlen);
			if (pos > 0)
			{
				strncat(buf, p, pos);
				p += pos;
			}
			else
			{
				strcat(buf, p);
				alldesc = TRUE;
			}
		}
		while (!alldesc);
	}
	strcat( buf, "{w" );
	stc( buf, ch );
}

void draw_map(CHAR_DATA * ch, const char *desc)
{
	int x, y;
	static char buf[MSL];

	sprintf(buf, desc);

	reformat_desc(buf);

	for (y = 0; y <= MAPY; y++)
	{
		for (x = 0; x <= MAPX; x++)
		{
			clear_coord(x, y);
		}
	}

	x = MAPX / 2;
	y = MAPY / 2;

	map[x][y].vnum = ch->in_room->vnum;
	map[x][y].depth = 0;

	map_exits(ch, ch->in_room, x, y, 0);

	map[x][y].symbol = 'X';
	map[x][y].color = 'R';
	strcat( buf, "{w" );
	show_map(ch, buf);
}
