/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "mysql5_interface.h"

// prool:
bool is_note_to (CHAR_DATA *ch, NOTE_DATA *note);

char *numlineas args(( char *string ));
char *string_linedel args(( char *, int ));
char *string_lineadd args(( char *, char *, int ));
char *short_date args((time_t time));
char *shorter_date args((time_t time));

#define L_SUP (MAX_LEVEL - 1) /* if not already defined */


BOARD_DATA boards[MAX_BOARD] =
{

	{ 
	"General",
	"General Discussion",
	0, 2, "all", DEF_EXCLUDE, 21, NULL, FALSE 
	},
	{ 
	"Announce",
	"Announcements from Immortals",  
	0, 1000, "all", DEF_NORMAL, 180, NULL, FALSE 
	},
	{ 
	"Lost",	
	"Lost and Found",
	0, 1, "all", DEF_INCLUDE, 60, NULL, FALSE
	},
	{ 
	"Sales",
	"For Sale, Wanted, and Free", 
	0, 1, "all", DEF_INCLUDE, 60, NULL, FALSE
	},
	{ 
	"Ideas",
	"Ideas to Staff",
	0, 1, "all", DEF_EXCLUDE, 60, NULL, FALSE 
	},
	{ 
	"Events",
	"Global Events",
	0, 1000,  "imm",   DEF_NORMAL, 60, NULL, FALSE 
	},
	{ 
	"Roleplay",
	"Roleplaying",
	0, 1, "all", DEF_INCLUDE, 120, NULL, FALSE 
	},
	{ 
	"RPChat",
	"Roleplay Chatter",
	0, 1, "all", DEF_INCLUDE, 60, NULL, FALSE 
	},
	{ 
	"Guilds",
	"Guild Matters",
	0, 1, "all", DEF_NORMAL, 60, NULL, FALSE 
	},
	{ 
	"Personal",
	"Personal Messages",
	0, 1, "all", DEF_EXCLUDE, 28, NULL, FALSE 
	}

};

/* The prompt that the character is given after finishing a note with ~ or END */
const char * szFinishPrompt = "\n\r(C)ontinue, (V)iew, (P)ost or (F)orget it?\n\r";
const char * board_txt_header = "\n\rEnter text. Type ~ or END on an empty line to end note.\n\r[.h help, .f format, .s show, .r replace]\n\r[.ld line delete, .li line insert, .lr line replace]\n\r=======================================================\n\r";
long last_note_stamp = 0; /* To generate unique timestamps on notes */

#define BOARD_NOACCESS -1
#define BOARD_NOTFOUND -1

static bool next_board (CHAR_DATA *ch);

int count_words(const char sentence[ ])
{
    int counted = 0; // result

    // state:
    const char* it = sentence;
    int inword = 0;

    do switch(*it) {
        case '\0': 
        case ' ': case '\t': case '\n': case '\r': // TODO others?
            if (inword) { inword = 0; counted++; }
            break;
        default: inword = 1;
    } while(*it++);

    return counted;
}
/*
int count_words(const char *str)
{
	bool inWord = false;
	int wordCount = 0;
	while (*str)
	{
		if (!inWord && isalpha(*str)) 
		{
			inWord = true;
			wordCount++;
		}
		else if (inWord && *str == ' ') 
		{
			inWord = false;
		}
		str++;
	}
	return wordCount;
}
*/
/* recycle a note */
void free_note (NOTE_DATA *note)
{
	if (note->sender)
		free_string (note->sender);
	if (note->to_list)
		free_string (note->to_list);
	if (note->subject)
		free_string (note->subject);
	if (note->date) /* was note->datestamp for some reason */
		free_string (note->date);
	if (note->text)
		free_string (note->text);
	if (note->poster_host)
		free_string( note->poster_host );

	note->next = note_free;
	note_free = note;
}

/* allocate memory for a new note or recycle */
NOTE_DATA *new_note ()
{
	NOTE_DATA *note;

	if (note_free)
	{
		note = note_free;
		note_free = note_free->next;
	}
	else
		note = alloc_mem (sizeof(NOTE_DATA));

	/* Zero all the field - Envy does not gurantee zeroed memory */
	note->next = NULL;
	note->sender = NULL;
	note->expire = 0;
	note->likes = 0;
	note->to_list = NULL;
	note->subject = NULL;
	note->date = NULL;
	note->date_stamp = 0;
	note->text = NULL;
	note->poster_host = NULL;

	return note;
}

/* append this note to the given file */
static void append_note (FILE *fp, NOTE_DATA *note)
{
	fprintf (fp, "Sender  %s~\n", note->sender);
	fprintf (fp, "PHost   %s~\n", note->poster_host);
	fprintf (fp, "Date    %s~\n", note->date);
	fprintf (fp, "Stamp   %ld\n", note->date_stamp);
	fprintf (fp, "Expire  %ld\n", note->expire);
	fprintf (fp, "Likes   %d\n",  note->likes);
	fprintf (fp, "To      %s~\n", note->to_list);
	fprintf (fp, "Subject %s~\n", note->subject);
	fprintf (fp, "Text\n%s~\n\n", note->text);
}

/* Save a note in a given board */
void finish_note (BOARD_DATA *board, NOTE_DATA *note)
{
	FILE *fp;
	NOTE_DATA *p;
	char filename[200];

	/* The following is done in order to generate unique date_stamps */

	if (last_note_stamp >= current_time)
		note->date_stamp = ++last_note_stamp;
	else
	{
	    note->date_stamp = current_time;
	    last_note_stamp = current_time;
	}

	if (board->note_first) /* are there any notes in there now? */
	{
		for (p = board->note_first; p->next; p = p->next )
			; /* empty */

		p->next = note;
	}
	else /* nope. empty list. */
		board->note_first = note;

	/* append note to note file */

	sprintf (filename, "%s%s", NOTE_DIR, board->short_name);

	fp = fopen (filename, "a");
	if (!fp)
	{
		bug ("Could not open one of the note files in append mode",0);
		board->changed = TRUE; /* set it to TRUE hope it will be OK later? */
		return;
	}

	append_note (fp, note);
	fclose (fp);
}

/* Find the number of a board */
int board_number (const BOARD_DATA *board)
{
	int i;

	for (i = 0; i < MAX_BOARD; i++)
		if (board == &boards[i])
			return i;

	return -1;
}

/* Find a board number based on  a string */
int board_lookup (const char *name)
{
	int i;

	for (i = 0; i < MAX_BOARD; i++)
		if (!str_cmp (boards[i].short_name, name))
			return i;

	return -1;
}

bool is_rp_board( CHAR_DATA *ch, char *board )
{
	if ( !str_cmp( board, "Roleplay" ) )
		return TRUE;
/*
	if ( !str_cmp( board, "Series" ) )
		return TRUE;
*/
	return FALSE;
}

/* Remove list from the list. Do not free note */
static void unlink_note (BOARD_DATA *board, NOTE_DATA *note)
{
	NOTE_DATA *p;

	if (board->note_first == note)
		board->note_first = note->next;
	else
	{
		for (p = board->note_first; p && p->next != note; p = p->next);
		if (!p)
			bug ("unlink_note: could not find note.",0);
		else
			p->next = note->next;
	}
}

/* Find the nth note on a board. Return NULL if ch has no access to that note */
static NOTE_DATA* find_note (CHAR_DATA *ch, BOARD_DATA *board, int num)
{
	int count = 0;
	NOTE_DATA *p;

	for (p = board->note_first; p ; p = p->next)
			if (++count == num)
				break;

	if ( (count == num) && is_note_to (ch, p))
		return p;
	else
		return NULL;

}

/* save a single board */
static void save_board (BOARD_DATA *board)
{
	FILE *fp;
	char filename[200];
	char buf[200];
	NOTE_DATA *note;

	sprintf (filename, "%s%s", NOTE_DIR, board->short_name);

	fp = fopen (filename, "w");
	if (!fp)
	{
		sprintf (buf, "Error writing to: %s", filename);
		bug (buf, 0);
	}
	else
	{
		for (note = board->note_first; note ; note = note->next)
			append_note (fp, note);

		fclose (fp);
	}
}

/* Show one note to a character */
static void show_note_to_char (CHAR_DATA *ch, NOTE_DATA *note, int num)
{
	char buf[4*MAX_STRING_LENGTH];

	/* Ugly colors ? */
	sprintf(buf,"[%4d] %s: %s\n\rDate:  %s\n\rLikes:  %d\n\rTo:    %s\n\r===========================================================================\n\r%s{x\n\r",
	         num, note->sender, note->subject,
	         note->date,
		 note->likes,
	         note->to_list,
	         note->text);

	send_to_char (buf,ch);
}

/* Save changed boards */
void save_notes ()
{
	int i;

	for (i = 0; i < MAX_BOARD; i++)
		if (boards[i].changed) /* only save changed boards */
			save_board (&boards[i]);
}

/* Load a single board */
static void load_board (BOARD_DATA *board)
{
	FILE *fp, *fp_archive;
	NOTE_DATA *last_note;
	char filename[200];

	sprintf (filename, "%s%s", NOTE_DIR, board->short_name);

	fp = fopen (filename, "r");

	/* Silently return */
	if (!fp)
		return;

	/* Start note fetching. copy of db.c:load_notes() */

    last_note = NULL;

    for ( ; ; )
    {
        NOTE_DATA *pnote;
        char letter;

        do
        {
            letter = getc( fp );
            if ( feof(fp) )
            {
                fclose( fp );
                return;
            }
        }
        while ( isspace(letter) );
        ungetc( letter, fp );

        pnote             = alloc_perm( sizeof(*pnote) );

        if ( str_cmp( fread_word( fp ), "sender" ) )
		break;
        pnote->sender     = fread_string( fp );

	if (str_cmp( fread_word( fp ), "phost" ) )
		break;
	pnote->poster_host = fread_string( fp );

        if ( str_cmp( fread_word( fp ), "date" ) )
		break;
        pnote->date       = fread_string( fp );

        if ( str_cmp( fread_word( fp ), "stamp" ) )
		break;
        pnote->date_stamp = fread_number( fp );

        if ( str_cmp( fread_word( fp ), "expire" ) )
		break;
        pnote->expire = fread_number( fp );

	if ( str_cmp( fread_word( fp ), "likes" ) )
		break;
	pnote->likes = fread_number( fp );

        if ( str_cmp( fread_word( fp ), "to" ) )
		break;
        pnote->to_list    = fread_string( fp );

        if ( str_cmp( fread_word( fp ), "subject" ) )
		break;
        pnote->subject    = fread_string( fp );

        if ( str_cmp( fread_word( fp ), "text" ) )
		break;
        pnote->text       = fread_string( fp );

        pnote->next = NULL; /* jic */

        /* Should this note be archived right now ? */

        if (pnote->expire < current_time)
        {
			char archive_name[200];

			sprintf (archive_name, "%s%s.old", NOTE_DIR, board->short_name);
			fp_archive = fopen (archive_name, "a");
			if (!fp_archive)
				bug ("Could not open archive boards for writing",0);
			else
			{
				append_note (fp_archive, pnote);
				fclose (fp_archive); /* it might be more efficient to close this later */
			}

			free_note (pnote);
			board->changed = TRUE;
			continue;

        }


        if ( board->note_first == NULL )
            board->note_first = pnote;
        else
            last_note->next     = pnote;

        last_note         = pnote;
    }

	fclose(fp);
    bug( "Load_notes: bad key word.", 0 );
    return; /* just return */
}

/* Initialize structures. Load all boards. */
void load_boards ()
{
	int i;

	for (i = 0; i < MAX_BOARD; i++)
		load_board (&boards[i]);
}

/* Returns TRUE if the specified note is address to ch */
bool is_note_to (CHAR_DATA *ch, NOTE_DATA *note)
{
	if (!str_cmp (ch->name, note->sender))
		return TRUE;

	if (is_exact_name ("all", note->to_list))
		return TRUE;

	if (is_exact_name("everyone", note->to_list))
		return TRUE;

	if (is_exact_name("everybody", note->to_list))
		return TRUE;

	if (IS_IMMORTAL(ch) && (
		is_exact_name ("imm", note->to_list) ||
		is_exact_name ("imms", note->to_list) ||
		is_exact_name ("immortal", note->to_list) ||
		is_exact_name ("god", note->to_list) ||
		is_exact_name ("gods", note->to_list) ||
		is_exact_name ("immortals", note->to_list)))
		return TRUE;

	if (is_exact_name (ch->name, note->to_list))
		return TRUE;

	/*Coder*/
	if ( (IS_SET(ch->war_flags,WAR_CODER)
	&& is_exact_name( "coder", note->to_list)))
		return TRUE;
	
	/* Staff */
	if ( (IS_SET(ch->spec_flags,SPEC_MENTOR)
	&& is_exact_name( "staff", note->to_list) )
	|| (IS_SET(ch->war_flags,WAR_MENTOR)
	&& is_exact_name( "staff", note->to_list)) )
		return TRUE;

	if ( IS_SET(ch->war_flags,WAR_MODERATOR)
	&& is_exact_name( "staff", note->to_list ) )
		return TRUE;

	if ( IS_IMMORTAL(ch)
	&& is_exact_name( "staff", note->to_list ) )
		return TRUE;

	if ( IS_SET(ch->hist_flags,HIST_GUILDLEADER)
	&& is_exact_name( "staff", note->to_list ) )
		return TRUE;

	if ( (IS_SET(ch->spec_flags,SPEC_MENTOR)
	&& is_exact_name( "mentor", note->to_list) )
	|| (IS_SET(ch->war_flags,WAR_MENTOR)
	&& is_exact_name( "mentor", note->to_list)) )
		return TRUE;

	if ( IS_SET(ch->war_flags,WAR_MODERATOR)
	&& is_exact_name( "moderator", note->to_list ) )
		return TRUE;

	if ( IS_SET(ch->war_flags,WAR_IMMORTAL)
	&& is_exact_name( "moderator", note->to_list ) )
		return TRUE;

	if ( IS_SET(ch->hist_flags,HIST_GUILDLEADER)
	&& is_exact_name( "guildleader", note->to_list ) )
		return TRUE;

	if ( IS_IMMORTAL(ch)
	&& is_exact_name( "guildleader", note->to_list ) )
		return TRUE;

	if ( ( ch->clan == clan_lookup("bandredhand") || ch->clan == clan_lookup("bandofficer") )
	&& is_exact_name( "bandredhand", note->to_list ) )
		return TRUE;


	if (is_shadow(ch) && is_exact_name( "shadow", note->to_list ))
		return TRUE;

 	/* whitetower */
	if ( is_white_tower(ch)	&&  is_exact_name( "whitetower", note->to_list ))
		return TRUE;


	if (ch->clan && is_exact_name(clan_table[ch->clan].name, note->to_list))
		return TRUE;

	if (ch->sub_clan && is_exact_name(clan_table[ch->sub_clan].name, note->to_list))
		return TRUE;

	if ( ch->race && is_exact_name(race_table[ch->race].name, note->to_list))
		return TRUE;

	if (is_exact_name( ch->name, note->to_list ) )
		return TRUE;

	return FALSE;
}

/* Return the number of unread notes 'ch' has in 'board' */
/* Returns BOARD_NOACCESS if ch has no access to board */
int unread_notes (CHAR_DATA *ch, BOARD_DATA *board)
{
	NOTE_DATA *note;
	time_t last_read;
	int count = 0;

	if (board->read_level > get_trust(ch))
		return BOARD_NOACCESS;

	last_read = ch->pcdata->last_note[board_number(board)];

	for (note = board->note_first; note; note = note->next)
		if (is_note_to(ch, note) && ((long)last_read < (long)note->date_stamp))
			count++;

	return count;
}

/*
 * COMMANDS
 */

/* Start writing a note */
static void do_nwrite (CHAR_DATA *ch, char *argument)
{
	char *strtime;
	char buf[200];

	if (IS_NPC(ch)) /* NPC cannot post notes */
		return;

	if ( ch->in_room == NULL )
	{
		stc("Something strange happened. Try to quit and log back in.\n\r", ch);
		return;
	}
	if (ch->pcdata->dfhunt_player_info[0] > 0)
	{
		stc("You might want to finish the DFHunt game first.\n\r", ch);
		return;
	}
        if ( IS_SET( ch->penalty_flags, PENAL_NOCHATS ) )
        {
	        stc("Your channel privileges have been revoked.\n\r", ch);
        	return;
        }

	if (get_trust(ch) < ch->pcdata->board->write_level)
	{
		send_to_char ("You cannot post notes on this board.\n\r",ch);
		return;
	}

	ch->pcdata->in_progress = NULL;

	/* continue previous note, if any text was written*/
	if ( ch->pcdata->in_progress && (ch->pcdata->in_progress->text != NULL) )
	{
		send_to_char ("Note in progress cancelled because you did not manage to write any text\n\rbefore losing link.\n\r\n\r",ch);
		free_note (ch->pcdata->in_progress);
		ch->pcdata->in_progress = NULL;
	}


	if (!ch->pcdata->in_progress)
	{
		ch->pcdata->in_progress = new_note();
		ch->pcdata->in_progress->sender = str_dup (ch->name);
		ch->pcdata->in_progress->poster_host = str_dup(ch->desc->host);

		/* convert to ascii. ctime returns a string which last character is \n, so remove that */
		strtime = ctime (&current_time);
		strtime[strlen(strtime)-1] = '\0';

		ch->pcdata->in_progress->date = str_dup (strtime);
	}

	act("$n starts writing a note.", ch, NULL, NULL, TO_ROOM);
	do_function( ch, &do_afk, "" );
	ch->was_in_room = ch->in_room;
	char_from_room( ch );
	char_to_room( ch, get_room_index( ROOM_VNUM_LIMBO ) );

	/* Begin writing the note ! */
	sprintf (buf, "You are now %s a new note on the %s board.\n\r\n\r",
	               ch->pcdata->in_progress->text ? "continuing" : "posting",
	               ch->pcdata->board->short_name);
	stc( buf,ch );

	ptc( ch, "{yFrom{Y: {y%s{x\n\r", ch->name );

	if (!ch->pcdata->in_progress->text) /* Are we continuing an old note or not? */
	{
		switch (ch->pcdata->board->force_type)
		{
		case DEF_NORMAL:
			ptc( ch, "If you press Return, default recipient %s will be chosen.\n\r", ch->pcdata->board->names );
			break;
		case DEF_INCLUDE:
			ptc( ch, "The recipient list MUST include %s. If not, it will be added automatically.\n\r", ch->pcdata->board->names );
			break;

		case DEF_EXCLUDE:
			ptc( ch, "The recipient of this note must NOT include: %s.", ch->pcdata->board->names );
			break;
		}

		stc( "\n\r{yTo{Y:{y ",ch );

		ch->desc->connected = CON_NOTE_TO;
		/* nanny takes over from here */
	}
	else /* we are continuing, print out all the fields and the note so far*/
	{
		ptc(ch, "To: %s\n\rExpires: %s\n\rSubject: %s\n\r",
			ch->pcdata->in_progress->to_list,
			ctime(&ch->pcdata->in_progress->expire),
			ch->pcdata->in_progress->subject);
		stc( "Your note so far:\n\r", ch );
		ptc( ch, "%s", ch->pcdata->in_progress->text );
		ptc( ch, "%s", board_txt_header );

		ch->desc->connected = CON_NOTE_TEXT;

	}

}

/* Start replying to a note */
static void do_nreply (CHAR_DATA *ch, char *argument)
{
	char *strtime;
	char buf[200];
	char arg[MIL];

	argument = one_argument( argument, arg );

	if (IS_NPC(ch)) /* NPC cannot post notes */
		return;

	if ( ch->in_room == NULL )
	{
		stc( "Something strange happened. Try to quit and log back in.\n\r", ch);
		return;
	}

	if (get_trust(ch) < ch->pcdata->board->write_level)
	{
		stc("You cannot post notes on this board.\n\r",ch);
		return;
	}

	if ( arg[0] == '\0' )
	{
		stc( "Invalid syntax.\n\r", ch );
		stc( "Note reply #\n\r", ch );
		stc( "Note list to find a valid number.\n\r", ch );
		return;
	}
	stc( "This doesn't work yet.\n\r", ch );
	return;

	ch->pcdata->in_progress = NULL;

	/* continue previous note, if any text was written*/
	if ( ch->pcdata->in_progress && (ch->pcdata->in_progress->text != NULL) )
	{
		stc("Note in progress cancelled because you did not manage to write any text\n\rbefore losing link.\n\r\n\r",ch);
		free_note (ch->pcdata->in_progress);
		ch->pcdata->in_progress = NULL;
	}


	if (!ch->pcdata->in_progress)
	{
		ch->pcdata->in_progress = new_note();
		ch->pcdata->in_progress->sender = str_dup (ch->name);

		/* convert to ascii. ctime returns a string which last character is \n, so remove that */
		strtime = ctime (&current_time);
		strtime[strlen(strtime)-1] = '\0';

		ch->pcdata->in_progress->date = str_dup (strtime);
	}

	act("$n starts replying to a note.", ch, NULL, NULL, TO_ROOM);
	do_function( ch, &do_afk, "" );
	ch->was_in_room = ch->in_room;
	char_from_room( ch );
	char_to_room( ch, get_room_index( ROOM_VNUM_LIMBO ) );

	/* Begin writing the note ! */
	sprintf (buf, "You are now %s a new note on the %s board.\n\rIf you are using tintin, type #verbose to turn off alias expansion!\n\r\n\r",
	               ch->pcdata->in_progress->text ? "continuing" : "posting",
	               ch->pcdata->board->short_name);
	send_to_char (buf,ch);

	sprintf (buf, "From:    %s\n\r\n\r", ch->name);
	send_to_char (buf,ch);

	if (!ch->pcdata->in_progress->text) /* Are we continuing an old note or not? */
	{
		switch (ch->pcdata->board->force_type)
		{
		case DEF_NORMAL:
			sprintf (buf, "If you press Return, default recipient %s will be chosen.\n\r",
					  ch->pcdata->board->names);
			break;
		case DEF_INCLUDE:
			sprintf (buf, "The recipient list MUST include %s. If not, it will be added automatically.\n\r",
						   ch->pcdata->board->names);
			break;

		case DEF_EXCLUDE:
			sprintf (buf, "The recipient of this note must NOT include: %s.",
						   ch->pcdata->board->names);

			break;
		}

		send_to_char (buf,ch);
		send_to_char ("\n\rTo:      ",ch);

		ch->desc->connected = CON_NOTE_TO;
		/* nanny takes over from here */

	}
	else /* we are continuing, print out all the fields and the note so far*/
	{
		sprintf(buf, "To:      %s\n\rExpires: %s\n\rSubject: %s\n\r",
		               ch->pcdata->in_progress->to_list,
		               ctime(&ch->pcdata->in_progress->expire),
		               ch->pcdata->in_progress->subject);
		send_to_char (buf,ch);
		send_to_char ("Your note so far:\n\r",ch);
		send_to_char (ch->pcdata->in_progress->text,ch);
		send_to_char( board_txt_header, ch );
		/*send_to_char ("\n\rEnter text. Type ~ or END on an empty line to end note.\n\r=======================================================\n\r",ch);*/

		ch->desc->connected = CON_NOTE_TEXT;

	}

}

/* Read next note in current group. If no more notes, go to next board */
static void do_nread (CHAR_DATA *ch, char *argument)
{
	NOTE_DATA *p;
	int count = 0, number;
	time_t *last_note = &ch->pcdata->last_note[board_number(ch->pcdata->board)];

	if (!str_cmp(argument, "again"))
	{ /* read last note again */

	}
	else if (is_number (argument))
	{
		number = atoi(argument);

		for (p = ch->pcdata->board->note_first; p; p = p->next)
			if (++count == number)
				break;

		if (!p || !is_note_to(ch, p))
			send_to_char ("No such note.\n\r",ch);
		else
		{
			show_note_to_char (ch,p,count);
			*last_note =  UMAX (*last_note, p->date_stamp);
		}
	}
	else /* just next one */
	{
		char buf[200];

		count = 1;
		for (p = ch->pcdata->board->note_first; p ; p = p->next, count++)
			if ((p->date_stamp > *last_note) && is_note_to(ch,p))
			{
				show_note_to_char (ch,p,count);
				/* Advance if new note is newer than the currently newest for that char */
				*last_note =  UMAX (*last_note, p->date_stamp);
				return;
			}

		send_to_char ("No new notes in this board.\n\r",ch);

		if (next_board (ch))
			sprintf (buf, "Changed to next board, %s.\n\r", ch->pcdata->board->short_name);
		else
		{
			ch->pcdata->board = &boards[0];
			sprintf (buf, "There are no more boards. Changing to General.\n\r");
		}

		send_to_char (buf,ch);
	}
}

/* Remove a note */
static void do_nremove (CHAR_DATA *ch, char *argument)
{
	NOTE_DATA *p;

	if (!is_number(argument))
	{
		send_to_char ("Remove which note?\n\r",ch);
		return;
	}

	p = find_note (ch, ch->pcdata->board, atoi(argument));
	if (!p)
	{
		send_to_char ("No such note.\n\r",ch);
		return;
	}

	if (str_cmp(ch->name,p->sender) && (get_trust(ch) < MAX_LEVEL))
	{
		send_to_char ("You are not authorized to remove this note.\n\r",ch);
		return;
	}

	unlink_note (ch->pcdata->board,p);
	free_note (p);
	send_to_char ("Note removed!\n\r",ch);

	save_board(ch->pcdata->board); /* save the board */
}
void nlcr( CHAR_DATA *ch )
{
	stc("\n\r",ch);
	return;
}
/* Like a note, uses mysql to store the like data */
static void do_nlike (CHAR_DATA *ch, char *argument)
{
	NOTE_DATA *p;
	MYSQL_RES *result;
	MYSQL_ROW row;
	char mysql_buf[MSL];
	char * liker = "Error";
	char * liker_host = "0.0.0.0";
	time_t like_date;

	if (!is_number(argument))
	{
		send_to_char ("Like which note?\n\r",ch);
		return;
	}

	p = find_note (ch, ch->pcdata->board, atoi(argument));

	if (!p)
	{
		stc("No such note.\n\r",ch);
		return;
	}


	if ( p->poster_host != NULL && !str_cmp( p->poster_host, ch->desc->host) )
	{
		stc("Yes, we know that you like your own notes, but let someone else upvote it.\n\r",ch);
		return;
	}


	/* Check to see if the character has liked this note already by searching for the character's name in the DB */
	mysql_buf[0] = '\0';
	sprintf( mysql_buf, "select liker, like_date from note_likes where liker = '%s' and post_date = '%ld'", ch->name, p->date_stamp );
	result = query_db(mysql_buf, result_db, FALSE);

	if ( result != NULL )
	{
		while ( (row = mysql_fetch_row(result)) != NULL )
		{
			free_string(liker);
			liker = str_dup(row[0]);
			like_date = atoi(row[1]);
			if ( !str_cmp( liker, ch->name ) )
			{
				ptc(ch,"It appears that you liked this note on %s. You can only like it once.\n\r",shorter_date(like_date));
				return;
			}
		}
	}
	mysql_free_result(result);


        mysql_buf[0] = '\0';
        sprintf( mysql_buf, "select liker, liker_host, like_date from note_likes where liker_host = '%s' and post_date = '%ld'", ch->desc->host, p->date_stamp );
        result = query_db(mysql_buf, result_db, FALSE);

        if ( result != NULL )
        {
                while ( (row = mysql_fetch_row(result)) != NULL )
                {
                        free_string(liker);
                        liker = str_dup(row[0]);
			free_string(liker_host);
			liker_host = str_dup(row[1]);
                        like_date = atoi(row[2]);
                        if ( !str_cmp( liker_host, ch->desc->host ) )
                        {
                                ptc(ch,"%s already liked this note on %s.\n\r",liker,shorter_date(like_date),liker_host,ch->desc->host);
                                return;
                        }
                }
        }
        mysql_free_result(result);

	p->likes += 1;
        ptc(ch,"Note liked! It now has %d likes.\n\r", p->likes);

        save_board(ch->pcdata->board);

	mysql_buf[0] = '\0';
	sprintf(mysql_buf, "insert into note_likes values (\"\", \"%s\", \"%s\", \"%s\", \"%ld\",  \"%ld\", \"%s\", \"%s\")",
		ch->pcdata->board->short_name, p->sender, p->subject, p->date_stamp, current_time, ch->name, ch->desc->host);

	mysql_real_query(result_db, mysql_buf, strlen(mysql_buf));

	if( mysql_affected_rows(result_db) < 1)
	{
		stc("Error writing like to database.", ch);
		log_string("MYSQL: Error saving channel log.");
	}
}

/* List all notes or if argument given, list N of the last notes */
/* Shows REAL note numbers! */
static void do_nlist (CHAR_DATA *ch, char *argument)
{
	int count= 0, show = 0, num = 0, has_shown = 0;
	time_t last_note;
	NOTE_DATA *p;
	char buf[MAX_STRING_LENGTH];
    	BUFFER *output;
    	output = new_buf();


	if (is_number(argument))	 /* first, count the number of notes */
	{
		show = atoi(argument);

		for (p = ch->pcdata->board->note_first; p; p = p->next)
			if (is_note_to(ch,p))
				count++;
	}

	if ( IS_IMMORTAL( ch ) )
		stc("Notes on this board:\n\r[Num]   Author        Date     Likes   IP               Subject\n\r",ch);
	else
		stc("Notes on this board:\n\r[Num]   Author        Date     Likes    Subject\n\r",ch);

	last_note = ch->pcdata->last_note[board_number (ch->pcdata->board)];

	for (p = ch->pcdata->board->note_first; p; p = p->next)
	{
		num++;
		if (is_note_to(ch,p))
		{
			has_shown++; /* note that we want to see X VISIBLE note, not just last X */
			if (!show || ((count-show) < has_shown))
			{
				if ( IS_IMMORTAL( ch ) )
					sprintf (buf, "[%3d] %c %-13s %-8s %-3d     %-16s %s{x\n\r", 
						num, last_note < p->date_stamp ? '*' : ' ', 
						p->sender, shorter_date(p->date_stamp), p->likes, 
						p->poster_host != NULL ? p->poster_host : "NULL", p->subject);
				else
					sprintf (buf, "[%3d] %c %-13s %-8s %-3d      %s{x\n\r", 
						num, last_note < p->date_stamp ? '*' : ' ', p->sender, shorter_date(p->date_stamp), p->likes, p->subject);
				add_buf(output,buf);
			}
		}

	}
	page_to_char( buf_string(output), ch );
	free_buf(output);
}

/* catch up with some notes */
static void do_ncatchup (CHAR_DATA *ch, char *argument)
{
	NOTE_DATA *p;

	/* Find last note */
	for (p = ch->pcdata->board->note_first; p && p->next; p = p->next);

	if (!p)
		send_to_char ("Alas, there are no notes in that board.\n\r",ch);
	else
	{
		ch->pcdata->last_note[board_number(ch->pcdata->board)] = p->date_stamp;
		send_to_char ("All messages skipped.\n\r",ch);
	}
}

/* Dispatch function for backwards compatibility */
void do_note (CHAR_DATA *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];

	if (IS_NPC(ch))
		return;

	argument = one_argument (argument, arg);

	if ((!arg[0]) || (!str_cmp(arg, "read"))) /* 'note' or 'note read X' */
		do_nread (ch, argument);

	else if (!str_cmp (arg, "list"))
		do_nlist (ch, argument);

	else if (!str_cmp (arg, "like"))
		do_nlike (ch, argument);

	else if (!str_cmp (arg, "write"))
		do_nwrite (ch, argument);

	else if (!str_cmp (arg, "remove"))
		do_nremove (ch, argument);

	else if (!str_cmp (arg, "reply"))
		do_nreply (ch, argument);

	else if (!str_cmp (arg, "purge"))
		send_to_char ("Obsolete.\n\r",ch);

	else if (!str_cmp (arg, "archive"))
		send_to_char ("Obsolete.\n\r",ch);

	else if (!str_cmp (arg, "catchup"))
		do_ncatchup (ch, argument);
	else
		do_help (ch, "note");
}

/* Show all accessible boards with their numbers of unread messages OR
   change board. New board name can be given as a number or as a name (e.g.
    board personal or board 4 */
void do_board (CHAR_DATA *ch, char *argument)
{
	int i, count, number;
	char buf[200];

	if (IS_NPC(ch))
		return;

	if (!argument[0]) /* show boards */
	{
		int unread;

		count = 1;
		send_to_char ("{RNum          Name Unread Description\n\r{r==== ============ ====== ==========={x\n\r",ch);
		for (i = 0; i < MAX_BOARD; i++)
		{
			unread = unread_notes (ch,&boards[i]); /* how many unread notes? */
			if (unread != BOARD_NOACCESS)
			{
				sprintf (buf, "{W[{w%2d{W] {y%12s {W[{w%s%4d{W]{y %s{x\n\r", count, boards[i].short_name, unread ? "{R" : "{w", unread, boards[i].long_name);
				stc( buf,ch );
				count++;
			} /* if has access */

		} /* for each board */

		sprintf (buf, "\n\rYour current board is %s.\n\r", ch->pcdata->board->short_name);
		send_to_char (buf,ch);

		/* Inform of rights */
		if (ch->pcdata->board->read_level > get_trust(ch))
			send_to_char ("You cannot read nor write notes on this board.\n\r",ch);
		else if (ch->pcdata->board->write_level > get_trust(ch))
			send_to_char ("You can only read notes from this board.\n\r",ch);
		else
			send_to_char ("You can both read and write on this board.\n\r",ch);

		return;
	} /* if empty argument */

    if (ch->pcdata->in_progress)
    {
    	send_to_char ("Please finish your interrupted note first.\n\r",ch);
        return;
    }

	/* Change board based on its number */
	if (is_number(argument))
	{
		count = 0;
		number = atoi(argument);
		for (i = 0; i < MAX_BOARD; i++)
			if (unread_notes(ch,&boards[i]) != BOARD_NOACCESS)
				if (++count == number)
					break;

		if (count == number) /* found the board.. change to it */
		{
			ch->pcdata->board = &boards[i];
			sprintf (buf, "Current board changed to %s. %s.\n\r",boards[i].short_name,
			              (get_trust(ch) < boards[i].write_level)
			              ? "You can only read here"
			              : "You can both read and write here");
			send_to_char (buf,ch);
		}
		else /* so such board */
			send_to_char ("No such board.\n\r",ch);

		return;
	}

	/* Non-number given, find board with that name */

	for (i = 0; i < MAX_BOARD; i++)
		if (!str_cmp(boards[i].short_name, argument))
			break;

	if (i == MAX_BOARD)
	{
		send_to_char ("No such board.\n\r",ch);
		return;
	}

	/* Does ch have access to this board? */
	if (unread_notes(ch,&boards[i]) == BOARD_NOACCESS)
	{
		send_to_char ("No such board.\n\r",ch);
		return;
	}

	ch->pcdata->board = &boards[i];
	sprintf (buf, "Current board changed to %s. %s.\n\r",boards[i].short_name,
	              (get_trust(ch) < boards[i].write_level)
	              ? "You can only read here"
	              : "You can both read and write here");
	send_to_char (buf,ch);
}

/* Send a note to someone on the personal board */
void personal_message (const char *sender, const char *to, const char *subject, const int expire_days, const char *text)
{
	make_note ("Personal", sender, to, subject, expire_days, text);
}

void make_note (const char* board_name, const char *sender, const char *to, const char *subject, const int expire_days, const char *text)
{
	int board_index = board_lookup (board_name);
	BOARD_DATA *board;
	NOTE_DATA *note;
	char *strtime;

	if (board_index == BOARD_NOTFOUND)
	{
		bug ("make_note: board not found",0);
		return;
	}

	if (strlen(text) > MAX_NOTE_TEXT)
	{
		bug ("make_note: text too long (%d bytes)", strlen(text));
		return;
	}


	board = &boards [board_index];

	note = new_note(); /* allocate new note */

	note->sender = str_dup (sender);
	note->to_list = str_dup(to);
	note->subject = str_dup (subject);
	note->expire = current_time + expire_days * 60 * 60 * 24;
	note->text = str_dup (text);

	/* convert to ascii. ctime returns a string which last character is \n, so remove that */
	strtime = ctime (&current_time);
	strtime[strlen(strtime)-1] = '\0';

	note->date = str_dup (strtime);

	finish_note (board, note);

}

/* tries to change to the next accessible board */
static bool next_board (CHAR_DATA *ch)
{
	int i = board_number (ch->pcdata->board) + 1;

	while ((i < MAX_BOARD) && (unread_notes(ch,&boards[i]) == BOARD_NOACCESS))
		i++;

	if (i == MAX_BOARD)
		return FALSE;
	else
	{
		ch->pcdata->board = &boards[i];
		return TRUE;
	}
}

void handle_con_note_to (DESCRIPTOR_DATA *d, char * argument)
{
	char buf [MAX_INPUT_LENGTH];
	CHAR_DATA *ch = d->character;

	if (!ch->pcdata->in_progress)
	{
		d->connected = CON_PLAYING;
		bug ("nanny: In CON_NOTE_TO, but no note in progress",0);
		return;
	}

	strcpy (buf, argument);
	smash_tilde (buf); /* change ~ to - as we save this field as a string later */

	if ( strstr( buf, "," ) != NULL )
	{
		write_to_buffer( d, "There's no need to put commas in the To field.\n\r", 0 );
		write_to_buffer( d, "Use spaces to seperate names or else the people\n\r", 0 );
		write_to_buffer( d, "you are trying to send the note to will never\n\r", 0 );
		write_to_buffer( d, "receive it!\n\r", 0 );
		write_to_buffer (d, "Specify a proper recipient.\n\rTo:      ", 0);
		d->connected = CON_NOTE_TO;
		return;
	}

	switch (ch->pcdata->board->force_type)
	{
		case DEF_NORMAL: /* default field */
			if (!buf[0]) /* empty string? */
			{
				ch->pcdata->in_progress->to_list = str_dup (ch->pcdata->board->names);
				sprintf (buf, "Assumed default recipient: %s\n\r", ch->pcdata->board->names);
				write_to_buffer (d, buf, 0);
			}
			else
				ch->pcdata->in_progress->to_list = str_dup (buf);

			break;

		case DEF_INCLUDE: /* forced default */
			if (!is_exact_name (ch->pcdata->board->names, buf))
			{
				strcat (buf, " ");
				strcat (buf, ch->pcdata->board->names);
				ch->pcdata->in_progress->to_list = str_dup(buf);

				sprintf (buf, "\n\rYou did not specify %s as recipient, so it was automatically added.\n\rNew To:  %s\n\r",
						 ch->pcdata->board->names, ch->pcdata->in_progress->to_list);
				write_to_buffer (d, buf, 0);
			}
			else
				ch->pcdata->in_progress->to_list = str_dup (buf);
			break;

		case DEF_EXCLUDE: /* forced exclude */
			if (!buf[0])
			{
				write_to_buffer (d, "You must specify a recipient.\n\rTo:      ", 0);
				return;
			}

			if (is_exact_name (ch->pcdata->board->names, buf))
			{
				sprintf (buf, "You are not allowed to send notes to %s on this board. Try again.\n\rTo:      ",
				ch->pcdata->board->names);
				write_to_buffer (d, buf, 0);
				return; /* return from nanny, not changing to the next state! */
			}
			else
				ch->pcdata->in_progress->to_list = str_dup (buf);
			break;

	}

	write_to_buffer (d, "\n\rSubject: ", 0);
	d->connected = CON_NOTE_SUBJECT;
}

void handle_con_note_subject (DESCRIPTOR_DATA *d, char * argument)
{
	char buf [MAX_INPUT_LENGTH];
	CHAR_DATA *ch = d->character;

	if (!ch->pcdata->in_progress)
	{
		d->connected = CON_PLAYING;
		bug ("nanny: In CON_NOTE_SUBJECT, but no note in progress",0);
		return;
	}

	strcpy (buf, argument);
	smash_tilde (buf); /* change ~ to - as we save this field as a string later */

	/* Do not allow empty subjects */

	if (!buf[0])
	{
		write_to_buffer (d, "Please find a meaningful subject!\n\r",0);
		write_to_buffer (d, "Subject: ", 0);
	}
	else  if (strlen(buf)>60)
	{
		write_to_buffer (d, "No, no. This is just the Subject. You're not writing the note yet.\n\r",0);
	}
	else
	/* advance to next stage */
	{
		ch->pcdata->in_progress->subject = str_dup(buf);
		if (IS_IMMORTAL(ch)) /* immortals get to choose number of expire days */
		{
			sprintf (buf,"\n\rHow many days do you want this note to expire in?\n\rPress Enter for default value for this board, %d days.\n\rExpire:  ",
		                 ch->pcdata->board->purge_days);
			write_to_buffer (d, buf, 0);
			d->connected = CON_NOTE_EXPIRE;
		}
		else
		{
			ch->pcdata->in_progress->expire =
				current_time + ch->pcdata->board->purge_days * 24L * 3600L;
			sprintf (buf, "This note will expire %s\r",ctime(&ch->pcdata->in_progress->expire));
			write_to_buffer (d,buf,0);
			write_to_buffer( d, board_txt_header, 0 );
			/*write_to_buffer (d, "\n\rEnter text. Type ~ or END on an empty line to end note.\n\r=======================================================\n\r",0);*/
			d->connected = CON_NOTE_TEXT;
		}
	}
}

void handle_con_note_expire(DESCRIPTOR_DATA *d, char * argument)
{
	CHAR_DATA *ch = d->character;
	char buf[MAX_STRING_LENGTH];
	time_t expire;
	int days;

	if (!ch->pcdata->in_progress)
	{
		d->connected = CON_PLAYING;
		bug ("nanny: In CON_NOTE_EXPIRE, but no note in progress",0);
		return;
	}

	/* Numeric argument. no tilde smashing */
	strcpy (buf, argument);
	if (!buf[0]) /* assume default expire */
		days = 	ch->pcdata->board->purge_days;
	else /* use this expire */
		if (!is_number(buf))
		{
			write_to_buffer (d,"Write the number of days!\n\r",0);
			write_to_buffer (d,"Expire:  ",0);
			return;
		}
		else
		{
			days = atoi (buf);
			if (days <= 0)
			{
				write_to_buffer (d, "This is a positive MUD. Use positive numbers only! :)\n\r",0);
				write_to_buffer (d, "Expire:  ",0);
				return;
			}
		}

	expire = current_time + (days*24L*3600L); /* 24 hours, 3600 seconds */

	ch->pcdata->in_progress->expire = expire;

	/* note that ctime returns XXX\n so we only need to add an \r */

	write_to_buffer( d, board_txt_header, 0 );
	/*write_to_buffer (d, "\n\rEnter text. Type ~ or END on an empty line to end note.\n\r=======================================================\n\r",0);*/

	d->connected = CON_NOTE_TEXT;
}


/*
void handle_con_note_text (DESCRIPTOR_DATA *d, char * argument)
{
	CHAR_DATA *ch = d->character;
	char buf[MAX_STRING_LENGTH];
	char letter[4*MAX_STRING_LENGTH];

	if (!ch->pcdata->in_progress)
	{
		d->connected = CON_PLAYING;
		bug ("nanny: In CON_NOTE_TEXT, but no note in progress",0);
		return;
	}


	strcpy (buf, argument);
	if ((!str_cmp(buf, "~")) || (!str_cmp(buf, "END")))
	{
		write_to_buffer (d, "\n\r\n\r",0);
		write_to_buffer (d, szFinishPrompt, 0);
		write_to_buffer (d, "\n\r", 0);
		d->connected = CON_NOTE_FINISH;
		return;
	}

	smash_tilde (buf);


	if (strlen (buf) > MAX_LINE_LENGTH)
	{
		write_to_buffer (d, "Too long line rejected. Do NOT go over 80 characters!\n\r",0);
		return;
	}

	if (ch->pcdata->in_progress->text)
	{
		strcpy (letter, ch->pcdata->in_progress->text);
		free_string (ch->pcdata->in_progress->text);
		ch->pcdata->in_progress->text = NULL;
	}
	else
		strcpy (letter, "");


	if ((strlen(letter) + strlen (buf)) > MAX_NOTE_TEXT)
	{
		write_to_buffer (d, "Note too long!\n\r", 0);
		free_note (ch->pcdata->in_progress);
		ch->pcdata->in_progress = NULL;
		d->connected = CON_PLAYING;
		return;
	}


	strcat (letter, buf);
	strcat (letter, "\r\n");

	ch->pcdata->in_progress->text = str_dup (letter);
}
*/
void handle_con_note_text(DESCRIPTOR_DATA * d, char *argument)
{
    CHAR_DATA *ch = d->character;
    char buf[MAX_STRING_LENGTH];
    char letter[4 * MAX_STRING_LENGTH];
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char arg3[MAX_INPUT_LENGTH];
    char tmparg3[MAX_INPUT_LENGTH];

    if (!ch->pcdata->in_progress)
    {
	d->connected = CON_PLAYING;
	bugf("nanny: In CON_NOTE_TEXT, but no note in progress");
	return;
    }

    strcpy(buf, argument);

    smash_tilde(argument);

    argument = one_argument(argument, arg1);
    argument = first_arg(argument, arg2, FALSE);
    strcpy(tmparg3, argument);
    argument = first_arg(argument, arg3, FALSE);


    if ( !str_cmp(buf, "~")
      || !str_cmp(buf, "@")
      || !str_cmp(buf, "end")
      || !str_cmp(buf, "done")
       )
    {
	printf_to_char(ch, "\n\r%s ", szFinishPrompt);
	d->connected = CON_NOTE_FINISH;
	return;
    }
    smash_tilde(buf);

    if (buf[0] == '.')
    {
	switch (buf[1])
	{
	case '\0':
	case '\n':
	case '\r':
	    break;

	case 'h':
	    send_to_char(
	        "\n\r\n\r"
	        "{r-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-{x\n\r"
	        "{R Editor help{r:{x\n\r"
	        "{r-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-{x\n\r"
	        "{y   .h{x                   this help.\n\r"
		"{y   .s{x                   show text written so far.\n\r"
		"{y   .f{x                   format text.\n\r"
		"{y   .c{x                   clear all written text.\n\r"
		"{y   .r  {D<{Wold{D>  <{Wnew{D> {x    replace {D<{Wold{D>{x text with {D<{Wnew{X>{x text.\n\r"
		"{y   .li {D<{Wline{D> <{Wtext{D>{x    insert {D<{Wtext{D>{x in front of line {D<{Wline{D>{x.\n\r"
		"{y   .lr {D<{Wline{D> <{Wtext{D>{x    replace text on line {D<{Wline{D>{x with text {D<{Wtext{D>{x.\n\r"
		"{y   .ld {D<{Wline{D>{x           delete line {D<{Wline{D>{x.\n\r"
		"{y    ~{x  or {YEND{x           exit editor.\n\r"
		"{r-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-{x\n\r"
		"\n\r\n\r", ch);
	    return;
	    break;

	case 's':
	    if (ch->pcdata->in_progress->text)
	    {
		printf_to_char(ch, "\n\r\n\r{yYou've written so far{D:{x\n\r\n\r");
		page_to_char(numlineas(ch->pcdata->in_progress->text), ch);
		printf_to_char(ch, "\n\r{yFree space left{D:{W %d{y characters.{x\n\r",
			       MAX_NOTE_TEXT - ((ch->pcdata->in_progress->text) ? strlen(ch->pcdata->in_progress->text) : 0));
	    } else
	    {
		send_to_char("\n\r{yYou haven't written a thing!{x\n\r\n\r", ch);
	    }
	    return;
	    break;

	case 'f':
	    if (ch->pcdata->in_progress->text)
	    {
	    send_to_char("\n\r{yText formatted{D.{x\n\r{yYou've written so far{D:{x\n\r\n\r", ch);
	    ch->pcdata->in_progress->text = format_string(ch->pcdata->in_progress->text);
	    send_to_char(ch->pcdata->in_progress->text, ch);
	    } else
	    {
		send_to_char("\n\r{yYou haven't written a thing!{x\n\r\n\r", ch);
	    }
	    return;
	    break;

	case 'c':
	    send_to_char("\n\r{yText cleared{D.{x\n\r", ch);
	    free_string(ch->pcdata->in_progress->text);
	    ch->pcdata->in_progress->text = &str_empty[0];
	    return;
	    break;

	case 'r':
	    if (arg2[0] == '\0')
	    {
		send_to_char("{yUsage{D:  .{yr {D<{Wold text{D> <{Wnew text{D>{x\n\r", ch);
		return;
	    }
	    if (ch->pcdata->in_progress->text)
	    {
	    ch->pcdata->in_progress->text = string_replace(ch->pcdata->in_progress->text, arg2, arg3);
	    printf_to_char(ch, "\n\r{D'{W%s{D'{x replaced with {D'{W%s{D'{D.{x\n\r", arg2, arg3);
	    } else
	    {
		send_to_char("\n\r{yYou haven't written a thing!{x\n\r\n\r", ch);
	    }
	    return;
	    break;

	case 'l':
	    switch (buf[2])
	    {
	    case '\0':
	    case '\n':
	    case '\r':
		break;

	    case 'r':
	    if (ch->pcdata->in_progress->text)
	    {
		ch->pcdata->in_progress->text = string_linedel(ch->pcdata->in_progress->text, atoi(arg2));
		ch->pcdata->in_progress->text = string_lineadd(ch->pcdata->in_progress->text, tmparg3, atoi(arg2));
		printf_to_char(ch, "{yLine {W%d{y replaced{D.{x\n\r", atoi(arg2));
	    } else
	    {
		send_to_char("\n\r{yYou haven't written a thing!{x\n\r\n\r", ch);
	    }
		return;
		break;

	    case 'i':
	    if (ch->pcdata->in_progress->text)
	    {
		ch->pcdata->in_progress->text = string_lineadd(ch->pcdata->in_progress->text, tmparg3, atoi(arg2));
		printf_to_char(ch, "{yLine {W%d{y added{D.{x\n\r", atoi(arg2));
	    } else
	    {
		send_to_char("\n\r{yYou haven't written a thing!{x\n\r\n\r", ch);
	    }
		return;
		break;

	    case 'd':
	    if (ch->pcdata->in_progress->text)
	    {
		ch->pcdata->in_progress->text = string_linedel(ch->pcdata->in_progress->text, atoi(arg2));
		printf_to_char(ch, "{yLine {W%d{y deleted{D.{x\n\r", atoi(arg2));
	    } else
	    {
		send_to_char("\n\r{yYou haven't written a thing!{x\n\r\n\r", ch);
	    }
		return;
		break;
	    }
	    break;
	}
    }
    smash_tilde(buf);		/* smash it now */

    /* Check for too long lines. Do not allow lines longer than MAX_INPUT_SIZE */

    if (strlen(buf) > MIL)
    {
	printf_to_char(ch, "Too long line rejected. Do NOT go over %d characters!\n\r", MIL);
	return;
    }
    /* Not end of note. Copy current text into temp buffer, add new line, and copy back */

    /* How would the system react to strcpy( , NULL) ? */
    if (ch->pcdata->in_progress->text)
    {
	strcpy(letter, ch->pcdata->in_progress->text);
	free_string(ch->pcdata->in_progress->text);
	ch->pcdata->in_progress->text = NULL;	/* be sure we don't free it twice */
    } else
	strcpy(letter, "");

    /* Check for overflow */

    if ((strlen(letter) + strlen(buf)) > MAX_NOTE_TEXT)
    {				/* Note too long, take appropriate steps */
	write_to_buffer(d, "Note too long!\n\r", 0);
	free_note(ch->pcdata->in_progress);
	ch->pcdata->in_progress = NULL;		/* important */
	d->connected = CON_PLAYING;
	return;
    }
    /* Add new line to the buffer */

    printf_to_char(ch, "%s\n\r", buf);

    strcat(letter, buf);
    strcat(letter, "\r\n");	/* new line. \r first to make note files better readable */

    /* allocate dynamically */
    ch->pcdata->in_progress->text = str_dup(letter);
}


void handle_con_note_finish (DESCRIPTOR_DATA *d, char * argument)
{
	int lines = 0, count = 0, count2 = 0, words = 0, reward = 0;
	CHAR_DATA *ch = d->character;
	char buf[MSL];

	if (!ch->pcdata->in_progress)
	{
		d->connected = CON_PLAYING;
		bug ("nanny: In CON_NOTE_FINISH, but no note in progress",0);
		return;
	}

	if ( ch->pcdata->in_progress->text != NULL && ch->pcdata->in_progress->text[0] != '\0' )
	{
		count2 = strlen_color( ch->pcdata->in_progress->text );
		count = strlen( ch->pcdata->in_progress->text );
		words = count_words( ch->pcdata->in_progress->text );
		lines = count / 80;
	}

	switch (tolower(argument[0]))
	{
		case 'c': /* keep writing */
			write_to_buffer (d,"Continuing note...\n\r",0);
			d->connected = CON_NOTE_TEXT;
			break;

		case 'v': /* view note so far */
			if (ch->pcdata->in_progress->text)
			{
				write_to_buffer (d,"Here is the text of your note so far:\n\r",0);
				write_to_buffer (d, ch->pcdata->in_progress->text, 0);
			}
			else
				write_to_buffer (d,"You haven't written a thing!\n\r\n\r",0);

			write_to_buffer (d, szFinishPrompt, 0);
			write_to_buffer (d, "\n\r",0);
			break;

		case 'p': /* post note */
			if ( ch->pcdata->in_progress->text == NULL
			|| ch->pcdata->in_progress->text[0] == '\0' )
			{
				stc("Find something meaningful to say.\n\r",ch);
				return;
			}
			finish_note (ch->pcdata->board, ch->pcdata->in_progress);
			write_to_buffer (d, "Note posted.\n\r",0);
			d->connected = CON_PLAYING;
			/* remove AFK status */
			for ( d = descriptor_list; d != NULL; d = d->next )
			{
				if ( d->connected == CON_PLAYING
				&&   d->character != ch
				&&   is_note_to( d->character, ch->pcdata->in_progress ) )
				{
				sprintf( buf, "{w***{R %s has posted a new note on %s.{w ***{x\n\r", ch->name, ch->pcdata->board->long_name);
				stc( buf, d->character );
				}
			}
			if ( is_rp_board( ch, ch->pcdata->board->short_name ) )
			{
				//ptc( ch, "Count = %d, NoColorCount = %d, Lines = %d, Words = %d\n\r", count, count2, lines, words );
				stc( "You gain 25 quest points for your submission.\n\r", ch );
				/* 1000 per level base */
				reward = count2 * 10;
				ptc( ch, "You gain %d experience points!\n\r", reward );
				gain_exp( ch, reward );
				ch->questpoints += 25;
			}
			ch->pcdata->in_progress = NULL;
			char_from_room( ch );
			char_to_room( ch,  ch->was_in_room );
			do_function( ch, &do_afk, "" );
			act ("$n finishes $s note." , ch, NULL, NULL, TO_ROOM);
			break;

		case 'f':
			write_to_buffer (d, "Note cancelled!\n\r",0);
			free_note (ch->pcdata->in_progress);
			ch->pcdata->in_progress = NULL;
			d->connected = CON_PLAYING;
			char_from_room( ch );
			char_to_room( ch, ch->was_in_room );
			do_function( ch, &do_afk, "" );
			/* remove afk status */
			break;

		default: /* invalid response */
			write_to_buffer (d, "Huh? Valid answers are:\n\r\n\r",0);
			write_to_buffer (d, szFinishPrompt, 0);
			write_to_buffer (d, "\n\r",0);
	}
}

