/*
   This header can be removed once you're done.  It documents the changes
   necessary to add this to stock ROM code.

1. put bugs.h wherever your .h-files are kept (in the /src directory with
   stock rom)

2. in merc.h, struct CHAR_DATA, add the following:
      int bugsreported;

   where the other connected states are defined, add the following:

      #define CON_BUGREPORTING_MSG	20 //or the next higher number
      #define CON_BUGREPORTING_IMMINFO  21
      #define CON_BUGREPORTING_MORTINFO 22
      #define MAX_ALLOWED_BUGS		200 //feel free to change this

3. update your Makefile to include bug.c

4. In act_comm.c, REMOVE the entire "void do_bug" function

5. Do a 'clean' make ("remove *.o" and then "make")

6. In update.c, add the following:
      under 'command procedures needed':
        void update_bugs args((void));

      somewhere else (mines in 'update_time') that is checked fairly often:
	update_bugs();

7. In string.c, make the following modifications:
      under 'force character out of editing mode.  Save their work':
	if (ch->desc->connected == CON_BUGREPORTING_MSG || ch->desc->connected == CON_BUGREPORTING_IMMINFO || ch->desc->connected == CON_BUGREPORTING_MORTINFO) {
	    save_bugs();
	    ch->desc->connected = CON_PLAYING;
	    ch->desc->pString = NULL;
	}

8. Change the 'is_immortal_address' names further down to the names of your
   own Immortals.

                      -------------------------------

                 What it gives you:

1. BUG command - adds a new bug to the BugTracker, including the persons
   name and the room they are in.  Also 'stamps' the bug with the time it
   was added.

2. Subcommands:
     BUG OPEN: Lists all open bugs in the BugTracker
     BUG CLOSED: Lists all closed bugs in the BugTracker
     BUG OPEN #: Lists the # most recent bugs (ie, 'bug open 3')
     BUG CLOSED #: Same as above, but with closed bugs.
     BUG SEARCH [phrase]: Lets Mortals search thru the bugs they have listed,
         or which were submitted by an Immortal, for a specific phrase.  When
         executed by an Immortal, searches all Bugs for the phrase.  The search
         looks for matches in the 'message' area, as well as 'mortinfo' field
         for mortals, and checks those plus 'imminfo' field for Immortals.
     BUG CONCISE [other subcommand]: Gives a sparse-output of the information,
         for more condensed reading.  Output is very similar to the way the
         old bug command listed information in its file.
     BUG REMOVE #: Removes the specified numer.  Mortals can only remove bugs
         they submitted, Immortals can remove any Bug.
     BUG UPDATE # [additional info]: Lets the submitter add information to the
	 initial bugreport.
     BUG EDIT #: Lets the submitter use the editor in 'string.c' to add info
         to the report.  Useful if they're doing a long report.

   Plus the following Immortal only commands:
     BUG CLOSE #: Closes the bug out.  This flags it as unchangeable, and
         tags it for removal after one week.
     BUG AWARD [bugnumber] [amt]: Sets the game to automatically award the
         specified number of QP's to the submitter of the bug.
     BUG UPDATE # [field] [additional info]: Allows the Immortal to add a
         response to any of the following fields: MESSAGE (the original report),
         IMMINFO (readable only by other Immortals), and MORTINFO (readable by
         both Imm's and morts)
     BUG EDIT # [field]: Lets the Immortal use the string.c editor to edit any
         of the fields listed directly above.

*/


/*********************************************************
 *  The contents of this file are copyright(c) 1999      *
 *  by Ryan Boileau ("Robbert") of The Inquisition MUD   *
 *  unless otherwise specified in a comment prior to the *
 *  function.                                            *
 *                                                       *
 *  Use of this code implies consent to the following:   *
 *    a.  Adherence to any license agreements governing  *
 *        your chosen codebase.                          *
 *    b.  Inclusion of this header, in its entirety,     *
 *        regardless of changes made to this code.  This *
 *        includes 'total rewrites'.                     *
 *                                                       *
 *  Although not required, I do encourage that all coders*
 *  make a 'help snippet' file in their game, and credit *
 *  all the dedicated programmers who have released code *
 *  over the years.  Theirs is all better than mine, but *
 *  I try!  :)                                           *
 *                                                       *
 * I can be reached at robbert@theinquisition.net if you *
 * have any questions or comments.  Enjoy.               *
 *********************************************************/

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "aww.h"
#include "recycle.h"
#include "bug.h"
#include "interp.h"

BUG_DATA *bug_free;
BUG_DATA *bug_list;
BUG_DATA *bug_last;

//locals
char *fix_string args((const char *str));


/*
 * Created: Robbert_2001015
 * Descrip: Holds a list of Immortal names, for verification with the
 *	    bug system that it was reported by an Immortal, so morts can
 *	    read it.
 * ------------------------
 */
bool is_immortal_address(char *arg)
{
    arg = capitalize(arg);

    if (!str_cmp(arg, "Tjor"))
		return TRUE;

    if (!str_cmp(arg, "Rokann"))
		return TRUE;

    if (!str_cmp(arg, "Avryel"))
		return TRUE;

    return FALSE;
}


/*
 * Created: Robbert_20011009
 * Descrip: Shows bugs which have not been resolved.  Can take a numeric
 *	    argument, such as 'show open 5' and show the most recent numbers.
 * -------------------------
 */
void show_open_bugs(CHAR_DATA * ch, char *argument, bool concise)
{
    int number = 0, count = 0;
    BUG_DATA *bug = NULL;
    BUFFER *output = new_buf();
    bool found = FALSE;

    number = atoi(argument);

    if (!bug_list || bug_list == NULL) {
	send_to_char("No current bugs.\n\r", ch);
	free_buf(output);
	return;
    }
    if (number == 0) {
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (bug->closed)
		continue;
	    if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		if (!is_immortal_address(bug->name))
		    continue;
	    }
	    add_buf(output, format_bug(ch, bug, concise));
	    found = TRUE;
	}
    } else {
	int total;
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (bug->closed)
		continue;
	    if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		if (!is_immortal_address(bug->name))
		    continue;
	    }
	    count++;
	}
	total = 0;
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (bug->closed)
		continue;
	    if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		if (!is_immortal_address(bug->name))
		    continue;
	    }
	    total++;
	    if (total <= (count - number))
		continue;
	    else {
		add_buf(output, format_bug(ch, bug, concise));
		found = TRUE;
		total--;
	    }
	}
    }
    if (!found)
	send_to_char("No open bugs found.\n\r", ch);
    else
	page_to_char(buf_string(output), ch);
    free_buf(output);
}

/*
 * Created: Robbert_20011009
 * Descrip: Same as above, bug shows closed bugs.
 * -------------------------
 */
void show_closed_bugs(CHAR_DATA * ch, char *argument, bool concise)
{
    int number = 0, count = 0;
    BUG_DATA *bug = NULL;
    BUFFER *output = new_buf();
    bool found = FALSE;

    number = atoi(argument);

    if (!bug_list || bug_list == NULL) {
	send_to_char("No current bugs.\n\r", ch);
	free_buf(output);
	return;
    }
    if (number == 0) {
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (!bug->closed)
		continue;
	    if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		if (!is_immortal_address(bug->name))
		    continue;
	    }
	    add_buf(output, format_bug(ch, bug, concise));
	    found = TRUE;
	}
    } else {
	int total;
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (!bug->closed)
		continue;
	    if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		if (!is_immortal_address(bug->name))
		    continue;
	    }
	    count++;
	}
	total = 0;
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (!bug->closed)
		continue;
	    if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		if (!is_immortal_address(bug->name))
		    continue;
	    }
	    total++;
	    if (total <= (count - number))
		continue;
	    else {
		add_buf(output, format_bug(ch, bug, concise));
		found = TRUE;
		total--;
	    }
	}
    }
    if (!found)
	send_to_char("No closed bugs found.\n\r", ch);
    else
	page_to_char(buf_string(output), ch);
    free_buf(output);
}

/*
 * Created: Robbert_20011002
 * Descrip: Saves bugs to an ascii text file.
 * -------------------------
 */
void save_bugs()
{
    FILE *fp;
    BUG_DATA *pbug;
    bool found = FALSE;
    int count = 0;

    if ((fp = fopen(NEW_BUG_FILE, "w")) == NULL) {
	perror(NEW_BUG_FILE);
	return;
    }

    for (pbug = bug_list; pbug != NULL; pbug = pbug->next)
	count++;
    while (count != 0) {
	for (pbug = bug_list; pbug != NULL; pbug = pbug->next) {
	    if (count != pbug->number)
		continue;
	    fprintf(fp, "Number %d\n", pbug->number);
	    if (pbug->closed == TRUE)
		fprintf(fp, "Closed %d\n", pbug->closed);
	    if (pbug->remove == TRUE)
		fprintf(fp, "Remove %d\n", pbug->remove);
	    if (pbug->valid == TRUE)
		fprintf(fp, "Valid %d\n", pbug->valid);
	    if (pbug->name && pbug->name[0] != '\0')
		fprintf(fp, "Name %s~\n", pbug->name);
	    if (pbug->msg && pbug->msg[0] != '\0')
		fprintf(fp, "Msg %s~\n", fix_string(pbug->msg));
	    if (pbug->imminfo && pbug->imminfo[0] != '\0')
		fprintf(fp, "Imminfo %s~\n", fix_string(pbug->imminfo));
	    if (pbug->lastname && pbug->lastname[0] != '\0')
		fprintf(fp, "Lastname %s~\n", fix_string(pbug->lastname));
	    if (pbug->mortinfo && pbug->mortinfo[0] != '\0')
		fprintf(fp, "Mortinfo %s~\n", fix_string(pbug->mortinfo));
	    if (pbug->date_listed && pbug->date_listed[0] != '\0')
		fprintf(fp, "Date_listed %s~\n", fix_string(pbug->date_listed));
	    if (pbug->lastmod && pbug->lastmod[0] != '\0')
		fprintf(fp, "Lastmod %s~\n", fix_string(pbug->lastmod));
	    fprintf(fp, "Truenumber %ld\nLastmodtime %ld\n", pbug->true_number, pbug->lastmodtime);
	    fprintf(fp, "Removeafter %ld\nAward %d\nRoom %d\nEnd\n\n", pbug->removeafter, pbug->award, pbug->room);
	    count--;
	    found = TRUE;
	}
    }
    fprintf(fp, "ZEND\n\n");
    fclose(fp);
    if (!found)
	unlink(NEW_BUG_FILE);
}

/*
 * Created: Robbert_20011002
 * Descrip: Makes a bug with no data fields filled.  Called by load_bug()
 * -------------------------
 */
BUG_DATA *make_new_bug()
{
    BUG_DATA *bug;

    if (bug_free == NULL)
	bug = alloc_perm(sizeof(*bug));
    else {
	bug = bug_free;
	bug_free = bug_free->next;
    }

    bug->valid = TRUE;
    bug->closed = FALSE;
    bug->remove = FALSE;
    bug->name = &str_empty[0];
    bug->msg = &str_empty[0];
    bug->imminfo = &str_empty[0];
    bug->mortinfo = &str_empty[0];
    bug->next = NULL;
    bug->date_listed = &str_empty[0];
    bug->lastmod = &str_empty[0];
    bug->lastname = &str_empty[0];
    bug->removeafter = 0;
    bug->number = 0;
    return bug;
}

/*
 * Created: Robbert_20011002
 * Descrip: Loads the bugs from their ascii file
 * -------------------------
 */
void load_bugs()
{
    FILE *fp;
    BUG_DATA *pbug;
    bool fMatch = FALSE;
    char *word = NULL;

    if ((fp = fopen(NEW_BUG_FILE, "r")) == NULL)
	return;

    pbug = make_new_bug();
    for (;;) {
	word = fread_word(fp);
	word[0] = UPPER(word[0]);
	switch (word[0]) {
	case 'A':
	    KEY("Award", pbug->award, fread_number(fp));
	    break;
	case 'C':
	    KEY("Closed", pbug->closed, fread_number(fp));
	    break;
	case 'D':
	    KEYS("Date_listed", pbug->date_listed, fread_string(fp));
	    break;
	case 'E':
	    if (!str_cmp(word, "End")) {
		if (bug_list == NULL)
		    bug_list = pbug;
		else
		    pbug->next = bug_list;
		bug_list = pbug;
		pbug = make_new_bug();
	    }
	    break;
	case 'I':
	    KEYS("Imminfo", pbug->imminfo, fread_string(fp));
	    break;
	case 'L':
	    KEYS("Lastmod", pbug->lastmod, fread_string(fp));
	    KEYS("Lastname", pbug->lastname, fread_string(fp));
	    KEY("Lastmodtime", pbug->lastmodtime, fread_number(fp));
	    break;
	case 'M':
	    KEYS("Msg", pbug->msg, fread_string(fp));
	    KEYS("Mortinfo", pbug->mortinfo, fread_string(fp));
	    break;
	case 'N':
	    KEYS("Name", pbug->name, fread_string(fp));
	    KEY("Number", pbug->number, fread_number(fp));
	    break;
	case 'R':
	    KEY("Remove", pbug->remove, fread_number(fp));
	    KEY("Removeafter", pbug->removeafter, fread_number(fp));
	    KEY("Room", pbug->room, fread_number(fp));
	    break;
	case 'T':
	    KEY("Truenumber", pbug->true_number, fread_number(fp));
	    break;
	case 'V':
	    KEY("Valid", pbug->valid, fread_number(fp));
	    break;
	default:
	    break;
	}
	if (is_name(word, "ZEND"))
	    break;
    }
    fclose(fp);
}

/*
 * Created: Robbert_20010929
 * Descrip: Makes a new bug structure
 * -------------------------
 */
BUG_DATA *new_bug(CHAR_DATA * ch)
{
    static BUG_DATA bug_zero;
    BUG_DATA *bug, *count;
    char buf[MIL];
    int counter = 1;

    if (bug_free == NULL)
	bug = alloc_perm(sizeof(*bug));
    else {
	bug = bug_free;
	bug_free = bug_free->next;
    }

    for (count = bug_list; count != NULL; count = count->next)
	counter++;
    if (counter > MAX_ALLOWED_BUGS)
	return NULL;
    *bug = bug_zero;
    bug->valid = TRUE;
    bug->closed = FALSE;
    bug->remove = FALSE;
    bug->name = str_dup(ch->name);
    bug->msg = &str_empty[0];
    bug->imminfo = &str_empty[0];
    bug->mortinfo = &str_empty[0];
    strcpy(buf, (char *) ctime(&current_time));
    buf[strlen(buf) - 1] = '\0';
    bug->date_listed = str_dup(buf);
    bug->lastmod = &str_empty[0];
    bug->lastname = &str_empty[0];
    bug->removeafter = 0;
    bug->number = counter;
    bug->true_number = current_time;
    bug->award = 0;
    bug->room = 0;
    return bug;
}

/*
 * Created: Robbert_20010929
 * Descrip: Frees removed bugs from list, saves structure for reuse
 * -------------------------
 */
void free_bug(BUG_DATA * bug)
{
    int counter = 1;
    BUG_DATA *count;
    if (!IS_VALID(bug))
	return;

    free_string(bug->name);
    free_string(bug->msg);
    free_string(bug->imminfo);
    free_string(bug->mortinfo);
    free_string(bug->date_listed);
    free_string(bug->lastmod);
    free_string(bug->lastname);
    bug->removeafter = 0;
    bug->valid = FALSE;
    bug->next = bug_free;
    bug_free = bug;
    for (count = bug_list; count != NULL; count = count->next) {
	count->number = counter;
	counter++;
    }
    save_bugs();
}


/*
 * Created: Robbert_20010923
 * Descrip: Does like format_help() in act_info.c - sets the output format
 * 	    for bug information depending upon the character requesting the
 * 	    information.  Imm's get more information than players.
 * -------------------------
 */
char *format_bug(CHAR_DATA * ch, BUG_DATA * bug, bool concise)
{
    char buf[MSL * 10], *final, buf2[MSL];

    buf[0] = '\0';

    if (concise == FALSE) {
	if (bug->closed == TRUE)
	    sprintf(buf, "{D");
	sprintf(buf2, "\n\rReported: %-45.45s    By: %-20.20s\n\r", bug->date_listed, bug->name);
	strcat(buf, buf2);

	if (!IS_IMMORTAL(ch))
	    sprintf(buf2, "%sBUG Report #%d:%s\n\r  %s{w\n\r", bug->closed ? "{D" : "{Y", bug->number, bug->closed ? "{D" : "{w",
		    bug->msg);
	else
	    sprintf(buf2, "%sBUG Report #%d: [Room %d]%s\n\r  %s{w\n\r", bug->closed ? "{D" : "{Y", bug->number, bug->room,
		    bug->closed ? "{D" : "{w", bug->msg);
	strcat(buf, buf2);

	if (bug->mortinfo && bug->mortinfo[0] != '\0') {
	    sprintf(buf2, "\n\r                    %sResponses%s:\n\r  %s{w\n\r", bug->closed ? "{D" : "{Y",
		    bug->closed ? "{D" : "{w", bug->mortinfo);
	    strcat(buf, buf2);
	}

	if (bug->imminfo && bug->imminfo[0] != '\0' && IS_IMMORTAL(ch)) {
	    sprintf(buf2, "\n\r                    %sIMMINFO%s:\n\r  %s{w\n\r", bug->closed ? "{D" : "{Y",
		    bug->closed ? "{D" : "{w", bug->imminfo);
	    strcat(buf, buf2);
	}

	if (bug->lastmod && bug->lastmod[0] != '\0') {
	    sprintf(buf2, "\n\rLast Updated: %-45.45s By: %-20.20s\n\r", bug->lastmod, bug->lastname);
	    strcat(buf, buf2);
	}
	if (bug->closed == TRUE) {
	    char time[MIL];
	    strcpy(time, (char *) ctime(&bug->removeafter));
	    time[strlen(time) - 1] = '\0';
	    strcat(buf, "\n\r(Closed, Will be removed ");
	    strcat(buf, time);
	    strcat(buf, ")\n\r");
	}
	if (bug->award > 0 && !str_cmp(bug->name, ch->name)) {
	    char award[MIL];
	    ch->questpoints += bug->award;
	    sprintf(award, "{GYou have been awarded %d Quest Point%s for this bug report!{w\n\r", bug->award,
		    bug->award > 1 ? "s" : "");
	    strcat(buf, award);
	    sprintf(award, "%s given %d Quest Point%s from BugTracker.", bug->name, bug->award, bug->award > 1 ? "s" : "");
	    append_file(ch, IMM_FILE, award);
	    bug->award = 0;
	    save_bugs();
	}
	if (IS_IMMORTAL(ch) && bug->award > 0) {
	    char award[MIL];
	    sprintf(award, "    {G%d Quest Point%s awarded for this report.{w\n\r", bug->award, bug->award > 1 ? "s" : "");
	    strcat(buf, award);
	}
	strcat(buf, "                    ----------------------------\n\r");
	final = buf;
    } else {
	if (IS_IMMORTAL(ch))
	    sprintf(buf, "%s%3d%s [%d] %s: %s%s%s%s\n\r", bug->closed ? "{D" : "{Y", bug->number, bug->closed ? "{D" : "{w",
		    bug->room, bug->name, bug->msg, bug->mortinfo[0] == '\0' ? "" : "({RMortinfo{w exists{w)",
		    bug->imminfo[0] == '\0' ? "" : "({RImminfo{w exists)", bug->closed == TRUE ? "{Y[Closed]{w" : "");
	else
	    sprintf(buf, "%s%3d%s %s: %s%s%s%s\n\r", bug->closed ? "{D" : "{Y", bug->number, bug->closed ? "{D" : "{w", bug->name,
		    bug->msg, bug->mortinfo[0] == '\0' ? "" : "({RImmortal Responses{w exist{w)",
		    bug->imminfo[0] == '\0' ? "" : "({RImminfo{w exists)", bug->closed == TRUE ? "{Y[Closed]{w" : "");

	if (bug->award > 0) {
	    char msg[MSL];
	    if (!str_cmp(ch->name, bug->name))
		sprintf(msg, "%s {G%d QP%s awarded for this bug.  See {Cbug list %d{G to receive them.\n\r", buf, bug->award,
			bug->award == 1 ? "" : "'s", bug->number);
	    else
		sprintf(msg, "%s {G%d QP%s awarded for this bug.\n\r", buf, bug->award, bug->award == 1 ? "" : "'s");
	    strcpy(buf, msg);
	}
	final = buf;
    }
    if (ch->pcdata->lastbug < bug->true_number)
	ch->pcdata->lastbug = bug->true_number;
    if (ch->pcdata->lastbug < bug->lastmodtime)
	ch->pcdata->lastbug = bug->lastmodtime;
    return final;
}

/*
 * Created: Robbert_20010923
 * Descrip: Puts timestamp info on bugreport.  A seperate function to avoid
 * 	    duplicative code.
 * -------------------------
 */
void timestamp(BUG_DATA * bug, CHAR_DATA * ch, char *time)
{
    free_string(bug->lastmod);
    bug->lastmod = str_dup(time);
    bug->lastmodtime = current_time;
    free_string(bug->lastname);
    bug->lastname = str_dup(ch->name);
    ch->pcdata->lastbug = bug->lastmodtime;
}

/*
 * Created: Robbert_20011002
 * Descrip: Takes bug out of linked list and frees it
 * -------------------------
 */
bool remove_bug(CHAR_DATA * ch, int number)
{
    BUG_DATA *prev = NULL, *bug;
    bool found = FALSE;

    for (bug = bug_list; bug != NULL; prev = bug, bug = bug->next) {
	if (!bug)
	    continue;
	if (str_cmp(bug->name, ch->name) && !IS_IMMORTAL(ch))
	    continue;
	if (number != bug->number)
	    continue;
	if (prev == NULL)
	    bug_list = bug->next;
	else
	    prev->next = bug->next;
	if (bug->closed != TRUE) {
	    bug->closed = TRUE;
	    if (!str_cmp(ch->name, bug->name)) {
		ch->pcdata->bugsreported--;
		if (ch->pcdata->bugsreported < 0)
		    ch->pcdata->bugsreported = 0;
	    }
	}
	free_bug(bug);
	found = TRUE;
    }
    if (found) {
	int count = 1;
	for (prev = bug_list; prev != NULL; prev = prev->next) {
	    prev->number = count;
	    count++;
	}
    }
    return found;
}

/*
 * Created: Robbert_20011007
 * Descrip: Allows char to see recent bugs.
 * -------------------------
 */
void do_recent_bug(CHAR_DATA * ch, bool concise)
{
    BUFFER *output = new_buf();
    BUG_DATA *bug;
    bool found = FALSE;
    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (ch->pcdata->lastbug >= bug->true_number && ch->pcdata->lastbug >= bug->lastmodtime)
	    continue;
	if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
	    if (!is_immortal_address(bug->name))
		continue;
	}
	found = TRUE;
	add_buf(output, format_bug(ch, bug, concise));
    }
    if (!found)
	send_to_char("No new bugs to list.\n\r", ch);
    else
	page_to_char(buf_string(output), ch);
    free_buf(output);
    return;
}

/*
 * Created: Robbert_20011007
 * Descrip: Allows chars to edit bug in editor
 * -------------------------
 */
void do_edit_bug(CHAR_DATA * ch, char *argument, bool concise)
{
    BUG_DATA *bug;
    char time[MIL], added[MSL], arg[MIL];
    argument = one_argument(argument, arg);
    argument = one_argument(argument, arg);

    strcpy(time, (char *) ctime(&current_time));
    time[strlen(time) - 1] = '\0';

    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (bug->number == atoi(arg))
	    break;
    }
    if (!bug || bug == NULL) {
	send_to_char("No bug found to edit.\n\r", ch);
	return;
    }
    if (IS_IMMORTAL(ch)) {
	if (is_name(argument, "msg") || is_name(argument, "message")) {
	    ch->desc->connected = CON_BUGREPORTING_MSG;
	    sprintf(added, "%s\n\r   ({RAddendum:{w %s)\n\r", bug->msg, time);
	    free_string(bug->msg);
	    bug->msg = str_dup(added);
	    string_append(ch, &bug->msg);
	    timestamp(bug, ch, time);
	    save_bugs();
	    return;
	} else if (is_name(argument, "imminfo")) {
	    ch->desc->connected = CON_BUGREPORTING_IMMINFO;
	    sprintf(added, "%s\n\r [{R%s{w] (%s)\n\r", bug->imminfo, ch->name, time);
	    free_string(bug->imminfo);
	    bug->imminfo = str_dup(added);
	    string_append(ch, &bug->imminfo);
	    timestamp(bug, ch, time);
	    save_bugs();
	    return;
	} else if (is_name(argument, "mortinfo")) {
	    ch->desc->connected = CON_BUGREPORTING_MORTINFO;
	    sprintf(added, "%s\n\r [{R%s{w] (%s)\n\r", bug->mortinfo, ch->name, time);
	    free_string(bug->mortinfo);
	    bug->mortinfo = str_dup(added);
	    string_append(ch, &bug->mortinfo);
	    timestamp(bug, ch, time);
	    save_bugs();
	    return;
	} else
	    send_to_char
		("Not a valid field.  Proper fields are:\n\r   Imminfo - Viewable by Immortals only.\n\r   Mortinfo - Send a comment readable by the reporter of the Bug.\n\r   Message - Add more information to the original bugreport body.\n\r",
		 ch);
	return;
    } else {
	ch->desc->connected = CON_BUGREPORTING_MSG;
	sprintf(added, "%s\n\r   ({RAddendum:{w %s)\n\r", bug->msg, time);
	free_string(bug->msg);
	bug->msg = str_dup(added);
	string_append(ch, &bug->msg);
	timestamp(bug, ch, time);
	save_bugs();
	return;
    }
}

/*
 * Created: Robbert_20011009
 * Descrip: Allows Imm to flag qp's on a bug to be given to char
 * 	    when they read it next.
 * -------------------------
 */
void do_award_bug(CHAR_DATA * ch, char *argument, bool concise)
{
    BUG_DATA *bug;
    char arg[MIL], time[MIL];
    bool found = FALSE;
    int amt, number;
    argument = one_argument(argument, arg);
    number = atoi(arg);
    amt = atoi(argument);

    strcpy(time, (char *) ctime(&current_time));
    time[strlen(time) - 1] = '\0';

    if (amt < 0) {
	send_to_char("Must specify a value greater than or equal to zero.\n\r", ch);
	return;
    }
    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (bug->number != number)
	    continue;
	bug->award = amt;
	found = TRUE;
	break;
    }
    if (!found || !bug)
	send_to_char("No bug found with that number.\n\r", ch);
    else {
	printf_to_char(ch, "Bug %d set to give %d Quest Point%s to %s.\n\r",
		       bug->number, bug->award, bug->award > 1 ? "s" : "", bug->name);
	timestamp(bug, ch, time);
	save_bugs();
    }
    return;
}

/*
 * Created: Robbert_20011009
 * Descrip: Allows immortals to close a bug out.  Closed Bugs cannot be
 *	    further modified.
 * -------------------------
 */
void do_close_bug(CHAR_DATA * ch, char *argument, bool concise)
{
    BUG_DATA *bug;
    char arg[MIL], time[MIL];
    int count = 0;
    argument = one_argument(argument, arg);
    count = atoi(arg);

    strcpy(time, (char *) ctime(&current_time));
    time[strlen(time) - 1] = '\0';

    if (count == 0) {
	send_to_char("You must specify which number to close.\n\r", ch);
	return;
    }
    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (bug->number == count)
	    break;
    }
    if (bug == NULL) {
	send_to_char("No bug with that number.\n\r", ch);
	return;
    }
    if (bug->closed == FALSE) {
	bug->closed = TRUE;
	bug->removeafter = current_time + 604800;
	send_to_char("Bug has been closed.  It will remain in the list for one week, then clear.\n\r", ch);
    } else {
	bug->closed = TRUE;
	bug->removeafter = 0;
	send_to_char("Bug has been reopened.\n\r", ch);
    }
    timestamp(bug, ch, time);
    save_bugs();
}

/*
 * Created: Robbert_20011009
 * Descrip: Allows char to update their bug information.  If they are Immortal,
 * 	    allows them to add to the mortinfo or imminfo fields as well.
 * -------------------------
 */
void do_update_bug(CHAR_DATA * ch, char *argument, bool concise)
{
    BUG_DATA *bug;
    char arg[MIL], buf[MIL];
    int count = 0;
    argument = one_argument(argument, arg);
    argument = one_argument(argument, arg);
    count = atoi(arg);
    if (IS_IMMORTAL(ch))
	argument = one_argument(argument, arg);
    if (count == 0) {
	if (IS_IMMORTAL(ch))
	    send_to_char("Syntax: bug update # [field] [msg]\n\r", ch);
	else
	    send_to_char("Syntax: bug update # [msg]\n\r", ch);
    }
    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (bug->number == count)
	    break;
    }
    if (bug == NULL) {
	send_to_char("No bug with that number.\n\r", ch);
	return;
    }
    if (str_cmp(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
	send_to_char("You did not add that bug.  Cannot add to it.\n\r", ch);
	return;
    }
    if (bug->closed == TRUE) {
	send_to_char("This bug report has been closed.  No changes may be made to it.\n\r", ch);
	return;
    }
    if (!IS_IMMORTAL(ch) || is_name(arg, "message")) {
	char time[MIL];
	char added[MSL];
	if (strlen(bug->msg) + strlen(argument) > (MSL - 25)) {
	    send_to_char("Addendum too long to add.\n\r", ch);
	    return;
	}
	strcpy(time, (char *) ctime(&current_time));
	time[strlen(time) - 1] = '\0';
	sprintf(added, "%s\n\r   ({RAddendum:{w %s)\n\r  %s", bug->msg, time, argument);
	free_string(bug->msg);
	argument = added;
	smash_tilde(argument);
	strcpy(added, argument);
	bug->msg = str_dup(added);
	sprintf(buf, "list %d", count);
	timestamp(bug, ch, time);
	do_new_bug(ch, buf);
	save_bugs();
	sprintf(buf, "Bug %d has been updated in the BugTracker by %s.", bug->number, ch->name);
	wiznet(buf, 0, 0, WIZ_FLAGS, 0, 0);
	return;
    }
    if (is_name(arg, "mortinfo")) {
	char time[MIL];
	char added[MSL];
	if (strlen(bug->mortinfo) + strlen(argument) > (MSL - 25)) {
	    send_to_char("Addendum too long to add.\n\r", ch);
	    return;
	}
	strcpy(time, (char *) ctime(&current_time));
	time[strlen(time) - 1] = '\0';
	if (bug->mortinfo[0] == '\0')
	    sprintf(added, " [{R%s{w] (%s)\n\r %s", ch->name, time, argument);
	else
	    sprintf(added, "%s\n\r   [{R%s{w] (%s)\n\r %s", bug->mortinfo, ch->name, time, argument);
	free_string(bug->mortinfo);
	argument = added;
	smash_tilde(argument);
	strcpy(added, argument);
	bug->mortinfo = str_dup(added);
	sprintf(buf, "list %d", count);
	timestamp(bug, ch, time);
	do_new_bug(ch, buf);
	save_bugs();
	sprintf(buf, "Bug %d has been updated in the BugTracker by %s.", bug->number, ch->name);
	wiznet(buf, ch, NULL, WIZ_FLAGS, 0, 0);
	return;
    }
    if (is_name(arg, "imminfo")) {
	char time[MIL];
	char added[MSL];
	if (strlen(bug->imminfo) + strlen(argument) > (MSL - 25)) {
	    send_to_char("Addendum too long to add.\n\r", ch);
	    return;
	}
	strcpy(time, (char *) ctime(&current_time));
	time[strlen(time) - 1] = '\0';
	if (bug->imminfo[0] == '\0')
	    sprintf(added, " [{R%s{w] (%s)\n\r %s", ch->name, time, argument);
	else
	    sprintf(added, "%s\n\r   [{R%s{w] (%s)\n\r %s", bug->imminfo, ch->name, time, argument);
	free_string(bug->imminfo);
	argument = added;
	smash_tilde(argument);
	strcpy(added, argument);
	bug->imminfo = str_dup(added);
	sprintf(buf, "list %d", count);
	timestamp(bug, ch, time);
	do_new_bug(ch, buf);
	save_bugs();
	sprintf(buf, "Bug %d has been updated in the BugTracker by %s.", bug->number, ch->name);
	wiznet(buf, ch, NULL, WIZ_FLAGS, 0, 0);
	return;
    }
    if (IS_IMMORTAL(ch)) {
	send_to_char("Not a valid field.  Proper fields are:\n\r"
		     "   Imminfo - Viewable by Immortals only.\n\r"
		     "   Mortinfo - Send a comment readable by mortals.\n\r"
		     "   Message - Add more information to the message body.\n\r", ch);
    }
    return;
}

/*
 * Created: Robbert_20011009
 * Descrip: Allows char to remove bug from the list, if they wrote it
 *	    or any bug if they are an Immortal
 * -------------------------
 */
void do_remove_bug(CHAR_DATA * ch, char *argument, bool concise)
{
    int number;
    char arg[MIL];
    argument = one_argument(argument, arg);
    number = atoi(argument);
    if (number == 0) {
	send_to_char("Must specify which number to remove.\n\r", ch);
	return;
    }
    if (!remove_bug(ch, number))
	send_to_char("Unable to remove that bug.\n\r", ch);
    else
	send_to_char("Bug removed successfully.\n\r", ch);
    return;
}

/*
 * Created: Robbert_20011009
 * Descrip: Shows char all updated bugs.
 * -------------------------
 */
void do_show_updated(CHAR_DATA * ch, bool concise)
{
    BUG_DATA *bug;
    bool found = FALSE;
    BUFFER *output = new_buf();
    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (ch->pcdata->lastbug >= bug->lastmodtime)
	    continue;
	if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
	    if (!is_immortal_address(bug->name))
		continue;
	}
	found = TRUE;
	add_buf(output, format_bug(ch, bug, concise));
    }
    if (!found)
	send_to_char("No new bugs to list.\n\r", ch);
    else
	page_to_char(buf_string(output), ch);
    free_buf(output);
    return;
}

/*
 * Created: Robbert_20011009
 * Descrip: Lists all (or specified number) bugs to player which they can read.
 * -------------------------
 */
void do_show_list(CHAR_DATA * ch, char *argument, bool concise)
{
    BUG_DATA *bug;
    bool found = FALSE;
    char arg[MIL];
    BUFFER *output = new_buf();
    int count = 0;

    if (bug_list == NULL) {
	send_to_char("No bugs at the moment.\n\r", ch);
	free_buf(output);
	return;
    } else {
	argument = one_argument(argument, arg);
	count = atoi(argument);

	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (count == 0) {
		if (!is_name(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		    if (!is_immortal_address(bug->name))
			continue;
		} else if (IS_IMMORTAL(ch) && !is_name(argument, bug->name) && argument[0] != '\0')
		    continue;
		found = TRUE;
		add_buf(output, format_bug(ch, bug, concise));
		if (ch->pcdata->lastbug < bug->true_number)
		    ch->pcdata->lastbug = bug->true_number;
	    } else {
		if (count > 0 && bug->number != count)
		    continue;
		if (count > bug->number)
		    break;
		if (str_cmp(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
		    if (!is_immortal_address(bug->name))
			continue;
		}
		found = TRUE;
		add_buf(output, format_bug(ch, bug, concise));
		if (ch->pcdata->lastbug < bug->true_number)
		    ch->pcdata->lastbug = bug->true_number;
	    }
	}
    }
    if (!found) {
	if (IS_IMMORTAL(ch))
	    send_to_char("No bugs found matching your list specifications.\n\r", ch);
	else
	    send_to_char("No bugs found reported by you.\n\r", ch);
    } else {
	if (IS_IMMORTAL(ch) && count == 0)
	    send_to_char("The following bugs have been reported:\n\r", ch);
	else if (!IS_IMMORTAL(ch))
	    send_to_char("The following bugs have been reported by you:\n\r", ch);
	page_to_char(buf_string(output), ch);
    }
    free_buf(output);
    return;
}

/*
 * Created: Robbert_20011009
 * Descrip: Allows char to search thru bugs they can read for a phrase
 *	    or name.
 *--------------------------
 */
void do_search_bugs(CHAR_DATA * ch, char *argument, bool concise)
{
    BUG_DATA *bug;
    bool found = FALSE;
    char arg[MIL];
    BUFFER *output = new_buf();
    argument = one_argument(argument, arg);
    if (bug_list == NULL) {
	send_to_char("No bugs at the moment.\n\r", ch);
	free_buf(output);
	return;
    } else {
	for (bug = bug_list; bug != NULL; bug = bug->next) {
	    if (str_cmp(bug->name, ch->name) && !IS_IMMORTAL(ch))
		continue;
	    if (!str_infix(argument, bug->msg) || (bug->mortinfo && !str_infix(argument, bug->mortinfo))) {
		found = TRUE;
		add_buf(output, format_bug(ch, bug, concise));
	    } else if (IS_IMMORTAL(ch) && (bug->imminfo && !str_infix(argument, bug->imminfo))) {
		found = TRUE;
		add_buf(output, format_bug(ch, bug, concise));
	    } else if (is_immortal_address(bug->name) && !str_infix(argument, bug->name)) {
		found = TRUE;
		add_buf(output, format_bug(ch, bug, concise));
	    } else if (IS_IMMORTAL(ch) && !str_infix(argument, bug->name)) {
		found = TRUE;
		add_buf(output, format_bug(ch, bug, concise));
	    }
	}
    }
    if (!found)
	send_to_char("No bugs found to match that phrase.\n\r", ch);
    else {
	send_to_char("The following bugs have been reported by you:\n\r", ch);
	page_to_char(buf_string(output), ch);
    }
    free_buf(output);
    return;
}

/*
 * Created: Robbert_20010923
 * Descrip: Adds a bug to the linked list of them.  If 'list' argument is
 * 	    given, will show all bugs Player has listed, plus any mortal-status
 *	    reports on them.   Imm's can search thru all, add comments for
 *	    imms or morts (or both) to them, or remove them.
 * -------------------------
 * Suggested by Bazarov for a better way to handle bug reports.
 */
void do_new_bug(CHAR_DATA * ch, char *argument)
{
    char buf[MSL], arg[MIL];
    BUG_DATA *bug = NULL;
    bool concise = FALSE;

    if (IS_NPC(ch))
	return;
    one_argument(argument, arg);

    if (is_name(arg, "concise")) {
	argument = one_argument(argument, arg);
	one_argument(argument, arg);
	concise = TRUE;
    }

    if (arg[0] == '\0' || argument[0] == '\0') {
	strcpy(arg, "list");
	strcpy(argument, "list");
    }

    if (is_name(arg, "edit")) {
	do_edit_bug(ch, argument, concise);
	return;
    }
    if (IS_IMMORTAL(ch)) {
	if (is_name(arg, "award")) {
	    argument = one_argument(argument, arg);
	    do_award_bug(ch, argument, concise);
	    return;
	}
	if (is_name(arg, "close")) {
	    argument = one_argument(argument, arg);
	    do_close_bug(ch, argument, concise);
	    return;
	}
    }
    if (is_name(arg, "recent")) {
	do_recent_bug(ch, concise);
	return;
    }
    if (is_name(arg, "remove")) {
	do_remove_bug(ch, argument, concise);
	return;
    }
    if (is_name(arg, "update")) {
	do_update_bug(ch, argument, concise);
	return;
    }
    if (is_name(arg, "list")) {
	if (str_prefix(argument, "open")) {
	    do_show_list(ch, argument, concise);
	    return;
	} else
	    argument = one_argument(argument, arg);
    }
    if (is_name(arg, "search")) {
	do_search_bugs(ch, argument, concise);
	return;
    }
    if (is_name(arg, "updated")) {
	do_show_updated(ch, concise);
	return;
    }
    if (is_name(arg, "closed")) {
	argument = one_argument(argument, arg);
	show_closed_bugs(ch, argument, concise);
	return;
    } else if (is_name(arg, "open")) {
	argument = one_argument(argument, arg);
	show_open_bugs(ch, argument, concise);
	return;
    }
    //otherwise, add a new bug!
    bug = new_bug(ch);
    if (bug == NULL) {
	send_to_char("Too many bugs reported, try again later.\n\r", ch);
	return;
    }
    if (strlen(argument) < 30 )
    {
	stc("Bug report is too short. Did you mean buy?\n\r", ch);
	return;
    }
    smash_tilde(argument);
    bug->msg = str_dup(argument);
    bug->room = ch->in_room->vnum;
    send_to_char
	("\n\r{wBug reported added.  Use {Cbug list{w to see all bugs you have reported, and any \n\rstatus changes on them.  Alternately, you can use {Cbug search [phrase]{w to find\n\rany submitted by you with the specified phrase contained.\n\r",
	 ch);
    printf_to_char(ch, "%s", format_bug(ch, bug, concise));
    sprintf(buf, "Bug %d has been added to the BugTracker by %s.", bug->number, ch->name);
    wiznet(buf, ch, NULL, WIZ_FLAGS, 0, 0);
    ch->pcdata->bugsreported++;
    if (bug_list == NULL)
	bug_list = bug;
    else {
	BUG_DATA *prev = NULL, *pbug;
	for (pbug = bug_list; pbug != NULL; prev = pbug, pbug = pbug->next);
	prev->next = bug;
    }
    save_bugs();
}

/*
 * Created: Robbert_20011002
 * Descrip: Checks for bugs on login, informs char if there are new ones
 * -------------------------
 */
char *show_bugs(CHAR_DATA * ch)
{
    long time = ch->pcdata->lastbug;
    BUG_DATA *bug;
    int count = 0, mod = 0;
    static char buf[MIL];

    if (time == 0 || !time) {
	ch->pcdata->lastbug = current_time;
	return " ";
    }
    for (bug = bug_list; bug != NULL; bug = bug->next) {
	if (str_cmp(bug->name, ch->name) && !IS_IMMORTAL(ch)) {
	    if (!is_immortal_address(bug->name))
		continue;
	}
	if (bug->lastmodtime > ch->pcdata->lastbug && bug->true_number != bug->lastmodtime)
	    mod++;
	if (bug->true_number <= ch->pcdata->lastbug)
	    continue;
	count++;
    }
    if (count > 0 && mod > 0)
	sprintf(buf, "There %s %d new bug%s and %d updated bug%s in the BugTracker.\n\r", count == 1 ? "is" : "are", count,
		count == 1 ? "" : "s", mod, mod == 1 ? "" : "s");
    else if (mod > 0)
	sprintf(buf, "There %s %d updated bug%s in the BugTracker.\n\r", mod == 1 ? "is" : "are", mod, mod == 1 ? "" : "s");
    else if (count > 0)
	sprintf(buf, "There %s %d new bug%s in the BugTracker.\n\r", count == 1 ? "is" : "are", count, count == 1 ? "" : "s");
    else
	sprintf(buf, " ");
    return buf;
}

/*
 * Created: Robbert_20011008
 * Descrip: Removes outdated bugs from the list.  Called in update()
 * -------------------------
 */
void update_bugs()
{
    BUG_DATA *prev = NULL, *bug;
    bool found = FALSE;

    for (bug = bug_list; bug != NULL; prev = bug, bug = bug->next) {
	if (!bug)
	    continue;
	if (!bug->closed)
	    continue;
	if (bug->removeafter == 0 && bug->closed == TRUE)
	    continue;
	if (bug->removeafter > current_time)
	    continue;
	if (prev == NULL)
	    bug_list = bug->next;
	else
	    prev->next = bug->next;
	free_bug(bug);
	if (prev == NULL)
	    bug = bug_list;
	else
	    bug = prev;
	found = TRUE;
    }
    if (found) {
	int count = 1;
	for (prev = bug_list; prev != NULL; prev = prev->next) {
	    prev->number = count;
	    count++;
	}
    }
    /*update_imms();*/
}


/*
 * Created: Robbert_20011015
 * Descrip: Calls the bug function with the command name, so ch can do
 *	    "command bug [message]"
 * -------------------------
 */
void link_bugs(CHAR_DATA * ch, char *argument, int cmd)
{
    BUG_DATA *bug = NULL;
    char arg[MIL];

    sprintf(arg, "  (Reference: %s)\n\r%s", cmd_table[cmd].name, argument);
    bug = new_bug(ch);
    if (bug == NULL) {
	send_to_char("Too many bugs reported, try again later.\n\r", ch);
	return;
    }
    smash_tilde(argument);
    bug->msg = str_dup(arg);
    bug->room = ch->in_room->vnum;
    send_to_char
	("\n\r{wBug reported added.  Use {Cbug list{w to see all bugs you have reported, and any \n\rstatus changes on them.  Alternately, you can use {Cbug search [phrase]{w to find\n\rany submitted by you with the specified phrase contained.\n\r",
	 ch);
    printf_to_char(ch, "%s", format_bug(ch, bug, FALSE));
    sprintf(arg, "Bug %d has been added to the BugTracker by %s.", bug->number, ch->name);
    wiznet(arg, ch, NULL, WIZ_FLAGS, 0, 0);
    if (bug_list == NULL)
	bug_list = bug;
    else {
	BUG_DATA *prev = NULL, *pbug;
	for (pbug = bug_list; pbug != NULL; prev = pbug, pbug = pbug->next);
	prev->next = bug;
    }
}


