/***************************************************************************

                      ___  _      _  _      _
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \__|_  /\_/\  /    \_/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
        intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

        ROM 2.4 is copyright 1993-1998 Russ Taylor
        ROM has been brought to you by the ROM consortium
            Russ Taylor (rtaylor@hypercube.org)
            Gabrielle Taylor (gtaylor@hypercube.org)
            Brian Moore (zump@rom.org)
        By using this code, you have agreed to follow the terms of the
        ROM license, in the file Rom24/doc/rom.license


***************************************************************************/
#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "caravan.h"
#include "tables.h"

sh_int on_which_caravan(CHAR_DATA *ch)
{
	sh_int caravan;
	for (caravan = 1;caravan <=MAX_CARAVANS; caravan++)
	{
		if ((ch->in_room->vnum >=  CaravanTable[caravan].lowVnum) && (ch->in_room->vnum <= CaravanTable[caravan].highVnum))
			return caravan;
	}
	return 0;
}
int check_pointed_at(sh_int caravan)
{
	int i;
	for(i=0;i<CaravanTable[caravan].num_check_points;i++)
		if (CaravanTable[caravan].check_points[i].loc == CaravanActive[caravan].location)
		{
			CaravanActive[caravan].next_check_point=i;
			return CaravanTable[caravan].check_points[i].vnum;
		}
	return 0;
}

bool caravan_schedule(CHAR_DATA *ch, char *arg1)
{
	int caravan,i,j;
	bool found = FALSE;
	if(on_which_caravan(ch) != 0)
	{
		stc("But you are already on a caravan, isn't that enough?\n\r",ch);
		return TRUE;
	}
        for(caravan = 1; caravan<= MAX_CARAVANS; caravan++)
        {
	    if (!((arg1==NULL)||(!str_prefix(arg1,CaravanTable[caravan].name))||!str_prefix(arg1,"list")))
		    continue;
		    i = CaravanActive[caravan].next_check_point;
                if (ch->in_room->vnum == check_pointed_at(caravan))
		{
			ptc(ch,"You may join the {y%s {xnow.\n\r",CaravanTable[caravan].name);
			ptc(ch,"The {y%s {xwill then head for {g%s.{x\n\r\n\r",CaravanTable[caravan].name,CaravanTable[caravan].check_points[i].next_check_points);
			j = 1;
		        i = (i + 1) % CaravanTable[caravan].num_check_points;
		}else
		{
		    j = 0;
		}

	    for(; j<CaravanTable[caravan].num_check_points; j++)
	    {
		    
                if (ch->in_room->vnum == CaravanTable[caravan].check_points[i].vnum)
			{
				ptc(ch,"You can join the {y%s {xhere in about {g%d {xminutes or so.\n\r",CaravanTable[caravan].name,((CaravanTable[caravan].hops - CaravanActive[caravan].location + CaravanTable[caravan].check_points[i].loc ) % CaravanTable[caravan].hops)  + j);
				ptc(ch,"The {y%s {xwill then head for {g%s.{x\n\r",CaravanTable[caravan].name,CaravanTable[caravan].check_points[i].next_check_points);
				found = TRUE;
			}
		i = (i + 1/* + CaravanTable[caravan].num_check_points*/ ) % CaravanTable[caravan].num_check_points;
	    }
	}
        return found;
}



/* The the caravan is at a valid check_point, can leave the caravan for the check_point, else
they jump
 * overboard and learn to swim.
 */

void do_unjoin(CHAR_DATA *ch, char *argument )
{
        int check_point;
	sh_int caravan;
	caravan = on_which_caravan(ch);
        check_point = check_pointed_at(caravan);

        if (caravan == 0)   /* caravan 0 is the null caravan, means not on a
caravan, fix this variable */
                stc("{wYou aren't on a caravan.{x\n\r",ch);
        else if (check_point == 0)
                stc("{wYou are not at a check point.{x\n\r",ch);
        else if (ch->in_room->vnum != CaravanTable[caravan].lowVnum)
                stc("{wYou are not at the exit.{x\n\r",ch);   /*
modify so that they know you have to be in a certain room to get off */
        else
        {
    		CHAR_DATA *fch;
    		CHAR_DATA *fch_next;
    		ROOM_INDEX_DATA *in_room;
    		ROOM_INDEX_DATA *to_room;
    		char buf[MSL];
		to_room = get_room_index(check_point);
		in_room = ch-> in_room;

    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
        if (ch->pcdata != NULL && ch->pcdata->wout[0] != '\0')
            act( "{c$t heading $T.{c",ch,ch->pcdata->wout,"towards the check point",TO_ROOM);
        else
            /*act( "{c$n leaves $T.{c", ch, NULL, dir_name[door], TO_ROOM );*/
    {
	if (MOUNTED(ch))
	{
	    if( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		sprintf(buf, "$n leaves $T, riding on %s.", MOUNTED(ch)->short_descr);
        else
           sprintf(buf, "$n soars $T, on %s.", MOUNTED(ch)->short_descr);
           act( buf, ch, NULL, "towards the check_point", TO_ROOM );
    }
	else
	{
	    act( "$n leaves $T.", ch, NULL, "towards the check_point", TO_ROOM );
    }
    }
    }
                char_from_room(ch);
/*                ch->caravan = 0;  */
                char_to_room(ch,to_room);
    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
    if (ch->pcdata != NULL && ch->pcdata->wkin[0] != '\0')
           act( "{c$t from the $T.{c",ch,ch->pcdata->wkin,CaravanTable[caravan].name,TO_ROOM);
    else
           /*act( "{c$n has arrived.{c", ch, NULL, NULL, TO_ROOM );*/
    {
	if(!MOUNTED(ch))
	{
	    act( "$n has arrived.", ch, NULL, NULL, TO_ROOM );
	}
	else
	{
	    if ( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		act( "$n has arrived, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
	    else
		act( "$n soars in, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
        }
    }

    if (MOUNTED(ch))
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), to_room );
    }
    }

    do_function(ch, &do_look, "auto" );

    if (in_room == to_room) /* no circular follows */
	return;

    for ( fch = in_room->people; fch != NULL; fch = fch_next )
    {
	fch_next = fch->next_in_room;

	if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM)
	&&   fch->position < POS_STANDING)
	    do_function(fch, &do_stand, "");

	if ( fch->master == ch && fch->position == POS_STANDING
	&&   can_see_room(fch,to_room))
	{

	    if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
	    &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
	    {
		act( "{cYou can't bring $N into the city.{c",
		    ch,NULL,fch,TO_CHAR);
		act( "{cYou aren't allowed in the city.{c",
		    fch,NULL,NULL,TO_CHAR);
		continue;
	    }

	    act( "{cYou follow $N.{c", fch, NULL, ch, TO_CHAR );
	    do_function(fch,&do_unjoin,"");
	}
    }


    /*
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC( ch ) && HAS_TRIGGER_MOB( ch, TRIG_ENTRY ) )
	p_percent_trigger( ch, NULL, NULL, NULL, NULL, NULL, TRIG_ENTRY );

    if ( !IS_NPC( ch ) )
    {
   	p_greet_trigger( ch, PRG_MPROG );
	p_greet_trigger( ch, PRG_OPROG );
	p_greet_trigger( ch, PRG_RPROG );
    }

   /*             if (ch->pet != NULL)
                {
                        char_from_room(ch->pet);
                        char_to_room(ch->pet, check_point_index);
                }
                if (ch->mount != NULL)
                {
                        char_from_room(ch->mount);
                        char_to_room(ch->mount, check_point_index);
                }
    		do_function(ch, &do_look, "auto" );
	*/
        }
}
sh_int pick_caravan(CHAR_DATA *ch,char *argument)
{
	sh_int i;
	char arg1[MIL];
	argument = one_argument( argument, arg1 );

        for(i = 1; i<= MAX_CARAVANS; i++)
        {
                if (ch->in_room->vnum == check_pointed_at(i))
			if ((arg1==NULL)||(!str_prefix(arg1,CaravanTable[i].name)))
                        	return i;
        }
                return 0;
}


/* The the char is at a valid check_point, can embark on the caravan by expending a
number of
 * coins equal to the cost for the boat ride, found in the header.
 */
void do_join( CHAR_DATA *ch,char *argument)
{
        sh_int caravan;
        caravan = pick_caravan(ch,argument);
        if (caravan == 0)
	{
		if (argument[0]=='\0')
			stc("{wThere is no caravan waiting here.{x\n\r",ch);
		else
                	stc("{wThat caravan is not stopped here.{x\n\r",ch);
	}
	else if (!check_coins(ch,CARAVAN_COST,0,0))
		stc("{xYou don't have the required amount of money.{x\n\r",ch);
        else
        {
    		CHAR_DATA *fch;
    		CHAR_DATA *fch_next;
    		ROOM_INDEX_DATA *in_room;
    		ROOM_INDEX_DATA *to_room;
    		char buf[MSL];
		in_room = ch-> in_room;
		to_room = get_room_index(CaravanTable[caravan].lowVnum);
		deduct_cost(ch,CARAVAN_COST,0);
		ptc(ch,"{wYou pay %d copper.{x\n\r",CARAVAN_COST);

    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
        if (ch->pcdata != NULL && ch->pcdata->wout[0] != '\0')
            act( "{c$t heading towards the $T.{c",ch,ch->pcdata->wout,CaravanTable[caravan].name,TO_ROOM);
        else
            /*act( "{c$n leaves $T.{c", ch, NULL, dir_name[door], TO_ROOM );*/
    {
	if (MOUNTED(ch))
	{
	    if( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		sprintf(buf, "$n leaves towards the $T, riding on %s.", MOUNTED(ch)->short_descr);
        else
           sprintf(buf, "$n soars towards the $T, on %s.", MOUNTED(ch)->short_descr);
           act( buf, ch, NULL, CaravanTable[caravan].name, TO_ROOM );
    }
	else
	{
	    act( "$n leaves towards the $T.", ch, NULL, CaravanTable[caravan].name, TO_ROOM );
    }
    }
    }
                char_from_room(ch);
                char_to_room(ch,to_room);
    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
    if (ch->pcdata != NULL && ch->pcdata->wkin[0] != '\0')
           act( "{c$t from the $T.{c",ch,ch->pcdata->wkin,"check point",TO_ROOM);
    else
           /*act( "{c$n has arrived.{c", ch, NULL, NULL, TO_ROOM );*/
    {
	if(!MOUNTED(ch))
	{
	    act( "$n has arrived.", ch, NULL, NULL, TO_ROOM );
	}
	else
	{
	    if ( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		act( "$n has arrived, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
	    else
		act( "$n soars in, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
        }
    }

    if (MOUNTED(ch))
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), to_room );
    }
    }

    do_function(ch, &do_look, "auto" );

    if (in_room == to_room) /* no circular follows */
	return;

    for ( fch = in_room->people; fch != NULL; fch = fch_next )
    {
	fch_next = fch->next_in_room;

	if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM)
	&&   fch->position < POS_STANDING)
	    do_function(fch, &do_stand, "");

	if ( fch->master == ch && fch->position == POS_STANDING
	&&   can_see_room(fch,to_room))
	{

	    if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
	    &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
	    {
		act( "{cYou can't bring $N into the city.{c",
		    ch,NULL,fch,TO_CHAR);
		act( "{cYou aren't allowed in the city.{c",
		    fch,NULL,NULL,TO_CHAR);
		continue;
	    }

	    act( "{cYou follow $N.{c", fch, NULL, ch, TO_CHAR );
	    do_function(fch,&do_join,argument);
	}
    }


    /*
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC( ch ) && HAS_TRIGGER_MOB( ch, TRIG_ENTRY ) )
	p_percent_trigger( ch, NULL, NULL, NULL, NULL, NULL, TRIG_ENTRY );

    if ( !IS_NPC( ch ) )
    {
   	p_greet_trigger( ch, PRG_MPROG );
	p_greet_trigger( ch, PRG_OPROG );
	p_greet_trigger( ch, PRG_RPROG );
    }
        }
}
/*void do_schedule(CHAR_DATA *ch, char *argument)
{
	;
}
*/

/* Caravan advance moves the caravan and char from one virtual room to another.
It also
 * sends a nifty message to any chars aboard the caravan concerning the
voyage. If the
 * caravan enter or leaves a check_point, it will inform any chars waiting for or on
the caravan.
 */
void caravan_advance( void )
{

int caravan;
int check_point;
DESCRIPTOR_DATA *d;
char buf[MSL];
#ifdef CHECK_POINTOBJ
OBJ_DATA *obj;
#endif

for (caravan = 1; caravan <=MAX_CARAVANS; caravan++)
{
	check_point = check_pointed_at(caravan);
        if (check_point != 0)
        {
	   if (CaravanActive[caravan].wait_at_check_point<=1)
	   {
                for ( d = descriptor_list; d != NULL; d = d->next )
                {
                        if ( d->connected == CON_PLAYING
                        && d->character->in_room != NULL
                        && !is_fighting(d->character))
                        {

                                if  (d->character->in_room->vnum == check_point)
                                {
                                        ptc(d->character,"{wThe %s leaves the stop.\n\r{x", CaravanTable[caravan].name);
                                }
                        }
                }
		CaravanActive[caravan].next_check_point++;
		if (CaravanActive[caravan].next_check_point == CaravanTable[caravan].num_check_points)
			CaravanActive[caravan].next_check_point = 0;
#ifdef CHECK_POINTOBJ

		obj_from_room(CaravanActive[caravan].check_point_obj);

#endif
	   }
	   else
	   {
		CaravanActive[caravan].location--;
	   }
	   CaravanActive[caravan].wait_at_check_point--;
	}

        if (CaravanActive[caravan].location >= CaravanTable[caravan].hops)
                CaravanActive[caravan].location = 0;
        else
                CaravanActive[caravan].location++;





/* ptc is printf to char   */
        /* Debugable  */

        if ((CaravanActive[caravan].location < 0) || (CaravanActive[caravan].location > CaravanTable[caravan].hops) )
        {
                bugf("Caravan Error: %d", caravan ); /* writes error message to
logs */
                CaravanActive[caravan].location = 1;
        }
        /* end Debugable */
}
for (caravan = 1; caravan <=MAX_CARAVANS; caravan++)
{
	check_point = check_pointed_at(caravan);
        if ((check_point != 0) && (CaravanActive[caravan].wait_at_check_point<=0))
        {
		CaravanActive[caravan].wait_at_check_point = CARAVAN_WAIT;
                for ( d = descriptor_list; d != NULL; d = d->next )
                {
                        if ( d->connected == CON_PLAYING
                        && d->character->in_room != NULL
                        && !is_fighting(d->character) )
                        {
                                if  (d->character->in_room->vnum == check_point)
                                {
                                        ptc(d->character,"{wThe %s stops here. It will be headed for %s soon.\n\r{x", CaravanTable[caravan].name,CaravanTable[caravan].check_points[CaravanActive[caravan].next_check_point].next_check_points);
                                }
                        }
		}
#ifdef CHECK_POINTOBJ

	obj = CaravanActive[caravan].check_point_obj;
	if (obj->short_descr)
		free_string( obj->short_descr );
       	sprintf(buf,"The %s is stopped here.",CaravanTable[caravan].name);
        obj->short_descr = str_dup(buf);
	if (obj->description)
		free_string( obj->description);
       	sprintf(buf,"The %s is stopped here. It will be headed for %s soon.{x",CaravanTable[caravan].name,CaravanTable[caravan].check_points[CaravanActive[caravan].next_check_point].next_check_points);
        obj->description = str_dup(buf);

		obj_to_room(CaravanActive[caravan].check_point_obj,get_room_index(check_point));

#endif
        }
}
/* for all char on caravan */
/* note that dir is parsed before location */

        for ( d = descriptor_list; d != NULL; d = d->next )
        {
                if ( d->connected == CON_PLAYING
                && d->character->in_room != NULL
                && !is_fighting(d->character) )
                {
			sh_int caravan;
			caravan = on_which_caravan(d->character);
                        if  ((caravan != 0) && ((CaravanActive[caravan].wait_at_check_point==CARAVAN_WAIT)||(check_pointed_at(caravan)==0)))
                        {
                                stc("{w",d->character);
           			sprintf(buf,CaravanMotionTable[CaravanTable[caravan].motion_type][CaravanActive[caravan].location],CaravanTable[caravan].name);
                                ptc(d->character,buf);
                                stc("\n\r{x",d->character);
                        }
                }
        }
}
void caravan_init(void)
{
#ifdef CHECK_POINTOBJ
	int caravan;
	for (caravan=1;caravan<=MAX_CARAVANS;caravan++)
		CaravanActive[caravan].check_point_obj = create_object( get_obj_index( CHECK_POINT_OBJ_VNUM ), 1 );
#endif
	return;
}
