/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aww.h"
#include "recycle.h"
#include "interp.h"
#include "combine.h"

/* combine.c */

/* Combining to allow players to combine items into
 * each other to create other items. Requires the
 * mechanics skill.
 */

bool check_vowell args( (const char *str) );



const   struct  combine_type combine_table [] =
{
	{	"",	36998,	36999,	36975	}, // 
	{	"",	36759,	36805,	36976	}, // an axe:			axe blade + short handle	(great/hand/war/battle)
	{	"",	36759,	36807,	36977	}, // a poleaxe:		axe blade + long handle			(2 handed)
	{	"",	36763,	36767,	36978	}, // a broadsword:		medium blade + hilt
	{	"",	36763,	36807,	36979	}, // an ashandarei:		medium blade + long handle
	{	"",	36763,	36805,	36980	}, // a machete:		medium blade + short handle
	{	"",	36761,	36767,	36981	}, // a shortsword:		short blade + hilt
	{	"",	36761,	36807,	36982	}, // a spear:			short blade + long handle
	{	"",	36761,	36805,	36983	}, // a short spear:		short blade + short handle
	{	"",	36765,	36767,	36984	}, // a claymore:		long blade + hilt			(2 handed)
	{	"",	36765,	36807,	36985	}, // a polearm:		long blade + long handle		(2 handed)
	{	"",	36809,	36807,	36986	}, // a maul:			hammer head + long handle		(2 handed)
	{	"",	36809,	36805,	36987	}, // a warhammer:		hammer head + short handle		(1 handed)
	{	"",	36757,	36805,	36988	}, // a mace:			spiked ball + short handle
	{	"",	36757,	36755,	36989	}, // a ball and chain:		spiked ball + chain			(exotic weapon)
	{	"",	36989,	36805,	36990	}, // a flail:			ball and chain + short handle
	{	"",	36755,	36805,	36991	}, // a chain whip:		chain + short handle			(whip)
	{	"",	36991,	36805,	36992	}, // a set of nunchucks	chain whip + short handle
	{	NULL,	-1,	-1,	-1	},
};

char *replace(const char *s, const char *old, const char *new)
{
	char *ret;
	int i, count = 0;
	size_t newlen = strlen(new);
	size_t oldlen = strlen(old);

	for (i = 0; s[i] != '\0'; i++)
	{
		if (strstr(&s[i], old) == &s[i]) {
		count++;
		i += oldlen - 1;
		}
	}

	ret = malloc(1 + i + count * (newlen - oldlen));
	if (ret == NULL)
	return "";

	i = 0;
	while (*s) {
		if (strstr(s, old) == s) {
			strcpy(&ret[i], new);
			i += newlen;
			s += oldlen;
		} else
		ret[i++] = *s++;
	}
	ret[i] = '\0';

	return ret;
}


int MaxCombine = 4;

void do_combine( CHAR_DATA *ch, char *argument )
{
	AFFECT_DATA *paf;
	OBJ_DATA *preItem1, *preItem2, *obj;
	char arg1[MIL];
	char arg2[MIL];
	char arg3[MIL];
	char buf[MSL];
	int i = 0, new_vnum = 0;
	char *material = "", *material2 = "";
	bool swap = FALSE;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( IS_NPC( ch ) )
	return;

	if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' || str_cmp( arg2, "with" ))
	{
		stc("Syntax: combine <item> with <item>\n\r\n\r",ch);
		stc("Not all items may be combined to create other items.\n\r",ch);
		return;
	}

	if ( ( preItem1 = get_obj_carry( ch, arg1, ch ) ) == NULL )
	{
		printf_to_char(ch, "You're not carrying any '%s'.\n\r", arg1);
		return;
	}

	if ( ( preItem2 = get_obj_carry( ch, arg3, ch ) ) == NULL )
	{
		printf_to_char(ch, "You're not carrying any '%s'.\n\r", arg3);
		return;
	}

	act("You are trying to combine $p with $P.\n\r", ch, preItem1, preItem2, TO_CHAR);
	act("$n is trying to combine $p with $P.\n\r", ch, preItem1, preItem2, TO_ROOM);

	for ( i = 0; combine_table[i].name != NULL; i++ )
	{
		if (preItem1->pIndexData->vnum == combine_table[i].vnum1
		&&  preItem2->pIndexData->vnum == combine_table[i].vnum2)
		{
			new_vnum = combine_table[i].comVnum;
			material = replace(preItem1->short_descr, "a ", "");
			material2 = replace(preItem2->short_descr, "a ", "");
			swap = FALSE;
			break;
		}

		if (preItem1->pIndexData->vnum == combine_table[i].vnum2
		&&  preItem2->pIndexData->vnum == combine_table[i].vnum1)
		{
			new_vnum = combine_table[i].comVnum;
			material = replace(preItem2->short_descr, "a ", "");
			material2 = replace(preItem1->short_descr, "a ", "");
			swap = TRUE;
			break;
		}
	}

	material = replace(material, "an ", "");

	if (get_obj_index(new_vnum) == NULL)
	{
		stc("You can't find a way to combine these two items.\n\r",ch);
		return;
	}

	obj = create_object(get_obj_index(new_vnum), 0);
	obj_to_char(obj, ch);
	
	material = replace(material, get_obj_index(combine_table[i].vnum1)->short_descr, "");
	material2 = replace(material2, get_obj_index(combine_table[i].vnum2)->short_descr, "");

/*
an arrow:		arrow shaft + arrow head
a fletched arrow:	feather + arrow
an axe:			axe blade + short handle		(gets randomized prefixes like great/hand/war/battle)
a broadsword:		medium blade + hilt
a shortsword:		short blade + hilt
a spear:		short blade + long handle
a short spear:		short blade + short handle
a claymore:		long blade + hilt			(2 handed)
a polearm:		long blade + long handle		(2 handed)
a maul:			hammer head + long handle		(2 handed)
a warhammer:		hammer head + short handle		(1 handed)
a mace:			spiked ball + short handle
a ball and chain:	spiked ball + chain			(exotic weapon)
a flail:		ball and chain + short handle
a chain whip:		chain + short handle			(whip)
a set of nunchucks	chain whip + short handle
*/

	switch (obj->pIndexData->vnum)
	{
		default: sprintf(buf, "%s %s", check_vowell(obj->short_descr) ? "an" : "a", obj->short_descr);
		break;

/* Arrow */	case 36975:
			sprintf(buf, "%s %s tipped %s%s", check_vowell(material) ? "an" : "a", material, material2, obj->short_descr);
		break;

/* Axe */	case 36976:
/* Poleaxe */	case 36977:
/* Ashandarei */case 36979:
/* Machete */	case 36980:
/* Spear */	case 36982:
/* Shortspear */case 36983:
/* Polearm */	case 36985:
/* Flail */	case 36990:
/* Chain Whip */case 36991:
/* Mace */	case 36988:
			sprintf(buf, "%s %shandled %s%s", check_vowell(material2) ? "an" : "a", material2, material, obj->short_descr);
		break;

/* Maul */	case 36986:
/* Warhammer */	case 36987:
			sprintf(buf, "%s %s%s", check_vowell(material) ? "an" : "a", material, obj->short_descr);
		break;

/* Broadsword */case 36978:
/* Shortsword */case 36981:
/* Claymore */	case 36984:
			sprintf(buf, "%s %s%s", check_vowell(material2) ? "an" : "a", material2, obj->short_descr);
		break;

/* Ball&Chain */case 36989:
			sprintf(buf, "%s %sball and chain", check_vowell(material) ? "an" : "a", material);
		break;

/* Nunchucks */	case 36992:
			sprintf(buf, "%s %sset of nunchucks", check_vowell(material2) ? "an" : "a", material2);
		break;
	}

	if (obj->item_type == ITEM_WEAPON)
	{
		if (!swap)
		{
			for (paf = preItem2->affected; paf != NULL; paf = paf->next)
				affect_to_obj(obj,paf);

			obj->value[1] = preItem1->value[1];
			obj->value[2] = preItem1->value[2];
		}
		else
		{
			for (paf = preItem1->affected; paf != NULL; paf = paf->next)
				affect_to_obj(obj,paf);

			obj->value[1] = preItem2->value[1];
			obj->value[2] = preItem2->value[2];
		}
	}

	// TODO - Handle gives stats and hilt gives power 

	obj->short_descr = str_dup(buf);

	sprintf(buf, "%s %s is lying here.", check_vowell(obj->name) ? "An" : "A", obj->name);
	obj->description = str_dup(buf);

	sprintf(buf, "%s%s%s", replace(preItem1->name, preItem1->pIndexData->name, ""), replace(preItem2->name, preItem2->pIndexData->name, ""), obj->name);
	obj->name = str_dup(buf);

	act("You combine your items and create $p.", ch, obj, NULL, TO_CHAR);
	act("$n combines $s items and create $p.", ch, obj, NULL, TO_ROOM);
	extract_obj(preItem1);
	extract_obj(preItem2);
	return;
}
