/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/*
 * This file contains all of the OS-dependent stuff:
 *   startup, signals, BSD sockets for tcp/ip, i/o, timing.
 *
 * The data flow for input is:
 *    Game_loop ---> Read_from_descriptor ---> Read
 *    Game_loop ---> Read_from_buffer
 *
 * The data flow for output is:
 *    Game_loop ---> Process_Output ---> Write_to_descriptor -> Write
 *
 * The OS-dependent functions are Read_from_descriptor and Write_to_descriptor.
 * -- Furey  26 Jan 1993
 */

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif

#include <sys/wait.h>

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/resource.h>
#include <ulimit.h>
#include <unistd.h> /* OLC -- for close read write etc */
#include <stdarg.h> /* printf_to_char */
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>

#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "event.h"

int getbit	args( ( char *explored, long index) );

bool is_in args( (char *, char *) );
/*
 * Malloc debugging stuff.
 */
#if defined(sun)
#undef MALLOC_DEBUG
#endif

#if defined(MALLOC_DEBUG)
#include <malloc.h>
extern	int	malloc_debug	args( ( int  ) );
extern	int	malloc_verify	args( ( void ) );
#endif



/*
 * Signal handling.
 * Apollo has a problem with __attribute(atomic) in signal.h,
 *   I dance around it.
 */
#if defined(apollo)
#define __attribute(x)
#endif

#if defined(unix)
#include <signal.h>
#endif

#if defined(apollo)
#undef __attribute
#endif

/*
 * Socket and TCP/IP stuff.
 */
#if    defined(unix)
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

const char echo_off_str[] = { IAC, WILL, TELOPT_ECHO, '\0' };
const char echo_on_str[] = { IAC, WONT, TELOPT_ECHO, '\0' };
const char go_ahead_str[] = { IAC, GA, '\0' };
#endif


/*
 * OS-dependent declarations.
 */
#if	defined(_AIX)
#include <sys/select.h>
int	accept		args( ( int s, struct sockaddr *addr, int *addrlen ) );
int	bind		args( ( int s, struct sockaddr *name, int namelen ) );
void	bzero		args( ( char *b, int length ) );
int	getpeername	args( ( int s, struct sockaddr *name, int *namelen ) );
int	getsockname	args( ( int s, struct sockaddr *name, int *namelen ) );
int	gettimeofday	args( ( struct timeval *tp, struct timezone *tzp ) );
int	listen		args( ( int s, int backlog ) );
int	setsockopt	args( ( int s, int level, int optname, void *optval,
			    int optlen ) );
int	socket		args( ( int domain, int type, int protocol ) );
#endif

#if	defined(apollo)
#include <unistd.h>
void	bzero		args( ( char *b, int length ) );
#endif

#if	defined(__hpux)
int	accept		args( ( int s, void *addr, int *addrlen ) );
int	bind		args( ( int s, const void *addr, int addrlen ) );
void	bzero		args( ( char *b, int length ) );
int	getpeername	args( ( int s, void *addr, int *addrlen ) );
int	getsockname	args( ( int s, void *name, int *addrlen ) );
int	gettimeofday	args( ( struct timeval *tp, struct timezone *tzp ) );
int	listen		args( ( int s, int backlog ) );
int	setsockopt	args( ( int s, int level, int optname,
 				const void *optval, int optlen ) );
int	socket		args( ( int domain, int type, int protocol ) );
#endif

#if	defined(interactive)
#include <net/errno.h>
#include <sys/fnctl.h>
#endif

#if	defined(linux)
/*
    Linux shouldn't need these. If you have a problem compiling, try
    uncommenting these functions.
*/
/*
int	accept		args( ( int s, struct sockaddr *addr, int *addrlen ) );
int	bind		args( ( int s, struct sockaddr *name, int namelen ) );
int	getpeername	args( ( int s, struct sockaddr *name, int *namelen ) );
int	getsockname	args( ( int s, struct sockaddr *name, int *namelen ) );
int	listen		args( ( int s, int backlog ) );
*/

int	close		args( ( int fd ) );
//int	gettimeofday	args( ( struct timeval *tp, struct timezone *tzp ) ); // prool
/* int	read		args( ( int fd, char *buf, int nbyte ) ); */
int	select		args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int	socket		args( ( int domain, int type, int protocol ) );
/* int	write		args( ( int fd, char *buf, int nbyte ) ); */ /* read,write in unistd.h */
#endif

#if	defined(macintosh)
#include <console.h>
#include <fcntl.h>
#include <unix.h>
struct	timeval
{
	time_t	tv_sec;
	time_t	tv_usec;
};
#if	!defined(isascii)
#define	isascii(c)		( (c) < 0200 )
#endif
static	long			theKeys	[4];

int	gettimeofday		args( ( struct timeval *tp, void *tzp ) );
#endif

#if	defined(MIPS_OS)
extern	int		errno;
#endif

#if	defined(MSDOS)
int	gettimeofday	args( ( struct timeval *tp, void *tzp ) );
int	kbhit		args( ( void ) );
#endif

#if	defined(NeXT)
int	close		args( ( int fd ) );
int	fcntl		args( ( int fd, int cmd, int arg ) );
#if	!defined(htons)
u_short	htons		args( ( u_short hostshort ) );
#endif
#if	!defined(ntohl)
u_long	ntohl		args( ( u_long hostlong ) );
#endif
int	read		args( ( int fd, char *buf, int nbyte ) );
int	select		args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int	write		args( ( int fd, char *buf, int nbyte ) );
#endif

#if	defined(sequent)
int	accept		args( ( int s, struct sockaddr *addr, int *addrlen ) );
int	bind		args( ( int s, struct sockaddr *name, int namelen ) );
int	close		args( ( int fd ) );
int	fcntl		args( ( int fd, int cmd, int arg ) );
int	getpeername	args( ( int s, struct sockaddr *name, int *namelen ) );
int	getsockname	args( ( int s, struct sockaddr *name, int *namelen ) );
int	gettimeofday	args( ( struct timeval *tp, struct timezone *tzp ) );
#if	!defined(htons)
u_short	htons		args( ( u_short hostshort ) );
#endif
int	listen		args( ( int s, int backlog ) );
#if	!defined(ntohl)
u_long	ntohl		args( ( u_long hostlong ) );
#endif
int	read		args( ( int fd, char *buf, int nbyte ) );
int	select		args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int	setsockopt	args( ( int s, int level, int optname, caddr_t optval,
			    int optlen ) );
int	socket		args( ( int domain, int type, int protocol ) );
int	write		args( ( int fd, char *buf, int nbyte ) );
#endif

/* This includes Solaris Sys V as well */
#if defined(sun)
int	accept		args( ( int s, struct sockaddr *addr, int *addrlen ) );
int	bind		args( ( int s, struct sockaddr *name, int namelen ) );
void	bzero		args( ( char *b, int length ) );
int	close		args( ( int fd ) );
int	getpeername	args( ( int s, struct sockaddr *name, int *namelen ) );
int	getsockname	args( ( int s, struct sockaddr *name, int *namelen ) );
int	listen		args( ( int s, int backlog ) );
int	read		args( ( int fd, char *buf, int nbyte ) );
int	select		args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );

#if !defined(__SVR4)
int	gettimeofday	args( ( struct timeval *tp, struct timezone *tzp ) );

#if defined(SYSV)
int setsockopt		args( ( int s, int level, int optname,
			    const char *optval, int optlen ) );
#else
int	setsockopt	args( ( int s, int level, int optname, void *optval,
			    int optlen ) );
#endif
#endif
int	socket		args( ( int domain, int type, int protocol ) );
int	write		args( ( int fd, char *buf, int nbyte ) );
#endif

#if defined(ultrix)
int	accept		args( ( int s, struct sockaddr *addr, int *addrlen ) );
int	bind		args( ( int s, struct sockaddr *name, int namelen ) );
void	bzero		args( ( char *b, int length ) );
int	close		args( ( int fd ) );
int	getpeername	args( ( int s, struct sockaddr *name, int *namelen ) );
int	getsockname	args( ( int s, struct sockaddr *name, int *namelen ) );
int	gettimeofday	args( ( struct timeval *tp, struct timezone *tzp ) );
int	listen		args( ( int s, int backlog ) );
int	read		args( ( int fd, char *buf, int nbyte ) );
int	select		args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int	setsockopt	args( ( int s, int level, int optname, void *optval,
			    int optlen ) );
int	socket		args( ( int domain, int type, int protocol ) );
int	write		args( ( int fd, char *buf, int nbyte ) );
#endif



/*
 * Global variables.
 */
DESCRIPTOR_DATA *   descriptor_list;	/* All open descriptors		*/
DESCRIPTOR_DATA *   d_next;		/* Next descriptor in loop	*/
bool		    god;		/* All new chars are gods!	*/
bool		    merc_down;		/* Shutdown			*/
char		    str_boot_time[MAX_INPUT_LENGTH];
time_t		    current_time;	/* time of this pulse */
bool		    MOBtrigger = TRUE;  /* act() switch                 */
bool		    PWipe = FALSE;
volatile sig_atomic_t crashed = 0;  /* Are we currently crashing? */
int bootcount = 0;	/* Used for timed reboots */
int tmp_bootcount = -1;

// by prool:
bool		    newlock;		/* Game is newlocked		*/

/*
 * OS-dependent local functions.
 */
#if defined(macintosh) || defined(MSDOS)
void	game_loop_mac_msdos	args( ( void ) );
bool	read_from_descriptor	args( ( DESCRIPTOR_DATA *d ) );
bool	write_to_descriptor	args( ( int desc, char *txt, int length ) );
#endif

#if defined(unix)
void	game_loop_unix		args( ( int control ) );
int	init_socket		args( ( int port ) );
void	init_descriptor		args( ( int control ) );
bool	read_from_descriptor	args( ( DESCRIPTOR_DATA *d ) );
bool	write_to_descriptor	args( ( int desc, char *txt, int length ) );
#endif


#ifdef THREADED_DNS
#include "rdns_cache.h"
#endif


/*
 * Other local functions (OS-independent).
 */
bool	check_parse_name	args( ( char *name ) );
bool	check_swear_string	args( ( char *argument) );
bool	check_reconnect		args( ( DESCRIPTOR_DATA *d, char *name,
				    bool fConn ) );
bool	check_playing		args( ( DESCRIPTOR_DATA *d, char *name ) );
int	main			args( ( int argc, char **argv ) );
void	nanny			args( ( DESCRIPTOR_DATA *d, char *argument ) );
bool	process_output		args( ( DESCRIPTOR_DATA *d, bool fPrompt ) );
void	read_from_buffer	args( ( DESCRIPTOR_DATA *d ) );
void	stop_idling		args( ( CHAR_DATA *ch ) );
void    bust_a_prompt           args( ( CHAR_DATA *ch ) );
void    halt_mud args ( ( int sig ) );
/* Needs to be global because of do_copyover */
int port, control;

// Stop an endless loop
// Uses alarm() to check for game clock advance every 300 seconds.  If more
// than 240 seconds of realtime has elapsed, assume loop and conduct a copyover.
static void sigalrm ( int sig )
{
    time_t ptm;
    time ( &ptm );
    //fprintf ( stderr, "TOCK!\n\r" );

    if ( ( ptm - current_time ) > 240 )
    {
        fprintf ( stderr, "Looping - Last Command: %s\r", last_command );
        // this requires you to add an "if (ch)" before the send_to_char
        // statements in do_copyover.
	// changed to make a direct call to copyover since the timed copyvers prevent it from working - Dalsor 04.16.2013
	copyover();
        //do_copyover ( NULL, "" );
        exit ( 1 );
    }
    alarm ( 300 );
}

void free_multibuf(DESCRIPTOR_DATA *d)
{
  if (d && d->multi_buf)
  {
    free_string(d->multi_buf);
    d->multi_buf = NULL;
  }
  return;
}
void check_multiple_commands(DESCRIPTOR_DATA *d, char *argument)
{

    char buf[MAX_STRING_LENGTH];
    const char *str;
    char *point;

    if(d->character && d->character->run_buf)
	    return;

    point   = buf;
    str     = argument;


    while ( *str != '\0' && *str != '^')
    {
        if ( *str == '\\' )
        {
             if (*++str == '^')
             {
                    /**str++;*/
                 *point++ = *str++;
             }
             else
             {

                 *str--;
                 *point++ = *str++;
/*		*point++ = *(str-1); */ /* is 1 the size of a char anyway? */
             }
        }
        else
          *point++ = *str++;
    }

    if(*str != '\0')
    {
          *str++;
          d->multi_buf = str_dup(str);
     }
     else
     {
          free_multibuf(d);
     }
        *point   = '\0';
       strcpy(d->incomm,buf);
       return;


	/*
    char *point;
    int i=0, j=0;
    char *temp;



   temp = argument;
   while(*temp == '^')
   {
      temp++;
      j++;
   }
   argument = temp;

   strcpy(d->incomm , temp);

   for (;j != 0;j--)
   {
      temp--;
   }

    if ((point = strchr(temp, '^'))==NULL)
    {
      free_multibuf(d);
      return;
    }

   i = strlen(argument) - strlen(point);

   while(point == '^' || point == ' ')
   {
      point++;
   }

   if(point != '\0')
   {
      d->multi_buf = str_dup(point);
   }
   else
   {
      free_multibuf(d);
   }
   argument[i]='\0';
   return;
   */
}

void pwipe(void)
{
	DESCRIPTOR_DATA *d;
	struct dirent *Dir;
	struct stat Stat;
	DIR *Directory;
	char FName[80];

	d = new_descriptor();
	Directory=opendir(PLAYER_DIR);
	Dir=readdir(Directory);
	while(Dir!=NULL)
	{
		sprintf(FName, PLAYER_DIR"%s", Dir->d_name);
		stat( FName, &Stat);
		if(S_ISREG(Stat.st_mode))
		{
			if (Dir->d_name[0] >= 'A' && Dir->d_name[0] <= 'Z')
			{
				load_char_obj( d, Dir->d_name);
				// wipe files older than 60 days or 2*player level days
				// exempt HERO+ from wipe.
				// d->character->login was set equal to lastlogoff ("LogO" in save.c)
				if ( !IS_SET(d->character->war_flags,WAR_IMMORTAL)
				&& (current_time - d->character->logon) > (86400 * 60))
				{
					log_string(Dir->d_name);
					unlink (FName);
				}
			}
		}
		Dir = readdir(Directory);
	}
	closedir(Directory);
}

char *dns_gethostname(char *s)
{
    #ifdef THREADED_DNS
	return gethostname_cached(s,4,FALSE);
    #else
	return s;
    #endif
}

int main( int argc, char **argv )
{
    struct timeval now_time;
    /*struct rlimit limit;*/
    bool fCopyOver = FALSE;

    struct sigaction halt_action, ignore_action, alarm_action;

    halt_action.sa_handler = halt_mud;
    sigemptyset ( &halt_action.sa_mask );
    halt_action.sa_flags = SA_NOMASK;
    /* apparently NOMASK is outdated, Anthony said so */
    /* unable to verify */
/*    halt_action.sa_flags = SA_NODEFER; */

    ignore_action.sa_handler = SIG_IGN;
    sigemptyset ( &ignore_action.sa_mask );
    ignore_action.sa_flags = 0;

    alarm_action.sa_handler = sigalrm;
    sigemptyset ( &alarm_action.sa_mask );
    alarm_action.sa_flags = SA_NOMASK;
/*    alarm_action.sa_flags = SA_NODEFER;  */

    sigaction ( SIGPIPE, &ignore_action, NULL );    /* who cares about pipes? */
    sigaction ( SIGHUP, &ignore_action, NULL ); /* stay alive if user quits */
    sigaction ( SIGINT, &halt_action, NULL );   /* interrupted at keyboard */
    sigaction ( SIGQUIT, &halt_action, NULL );  /* quit at keyboard */
    sigaction ( SIGILL, &halt_action, NULL );   /* illegal instruction */
    sigaction ( SIGFPE, &halt_action, NULL );   /* floating point error */
    sigaction ( SIGSEGV, &halt_action, NULL );  /* invalid memory reference */
    sigaction ( SIGTERM, &halt_action, NULL );  /* terminate */
    sigaction ( SIGBUS, &halt_action, NULL );   /* out of memory?? */
    sigaction ( SIGALRM, &alarm_action, NULL ); /* endless loop check */

    alarm ( 300 );

    /*
     * Memory debugging if needed.
     */
#if defined(MALLOC_DEBUG)
    malloc_debug( 2 );
#endif


    /*
     * Init time.
     */
    gettimeofday( &now_time, NULL );
    current_time 	= (time_t) now_time.tv_sec;
    strcpy( str_boot_time, ctime( &current_time ) );

    #ifdef THREADED_DNS
	rdns_cache_set_ttl(12*60*60);       // twelve hours is moooore than enough
    #endif

    /*
     * Macintosh console initialization.
     */
#if defined(macintosh)
    console_options.nrows = 31;
    cshow( stdout );
    csetmode( C_RAW, stdin );
    cecho2file( "log file", 1, stderr );
#endif

    /*
     * Get the port number.
     */
    port = 4000;
    if ( argc > 1 )
    {
		if ( !str_cmp( argv[1],"PWIPE" ) )
		{
		    PWipe = TRUE;
		    boot_db( );
		    pwipe();
		    exit( 0 );
		}
		if ( !is_number( argv[1] ) )
		{
		    fprintf( stderr, "Usage: %s [port #]\n", argv[0] );
		    exit( 1 );
		}
		else if ( ( port = atoi( argv[1] ) ) <= 1024 )
		{
		    fprintf( stderr, "Port number must be above 1024.\n" );
		    exit( 1 );
		}

		/* Are we recovering from a copyover? */
	 	if (argv[2] && argv[2][0])
	 	{
	 		fCopyOver = TRUE;
	 		control = atoi(argv[3]);
	 		if ( control > 4 ) control = 4;
	 	}
	 	else
	 		fCopyOver = FALSE;

    }
	/* Port handling. Used for who_html, OLC rights, etc.
	 * mostly for allowing what commands can be used on what port
	 * and whether building is allowed, and the online.html gets
	 * updated.
	 */
    mud_port = port;

	/* I dont think we need this for now - Dalsor 04.11.2013 */
	/*
	if ( mud_port == play_port )
	{
		wizlock = TRUE;
		wizlock_timer = 25;
	}
	*/


    if ( mud_port == build_port && !newlock )
    	newlock = TRUE;


    /*
     * Run the game.
     */
#if defined(macintosh) || defined(MSDOS)
    boot_db();
    log_string( "TSE is ready to rock." );
    game_loop_mac_msdos( );
#endif

#if defined(unix)

	if (!fCopyOver)
	    control = init_socket( port );
	if ( control > 4 ) control = 4;

    boot_db();
    bootcount = 86400; /* run a copyover once per 24 hours from boot to keep things sane */
    sprintf( log_buf, "TSE is ready to rock on port %d.", port );
    log_string( log_buf );

    if (fCopyOver)
    	copyover_recover();

    game_loop_unix( control );
    close (control);
#endif

    /*
     * That's all, folks.
     */
    log_string( "Normal termination of game." );
    exit( 0 );
    return 0;
}



#if defined(unix)
int init_socket( int port )
{
    static struct sockaddr_in sa_zero;
    struct sockaddr_in sa;
    int x = 1;
    int fd;

    if ( ( fd = socket( AF_INET, SOCK_STREAM, 0 ) ) < 0 )
    {
	perror( "Init_socket: socket" );
	exit( 1 );
    }

    if ( setsockopt( fd, SOL_SOCKET, SO_REUSEADDR,
    (char *) &x, sizeof(x) ) < 0 )
    {
	perror( "Init_socket: SO_REUSEADDR" );
	close(fd);
	exit( 1 );
    }

#if defined(SO_DONTLINGER) && !defined(SYSV)
    {
	struct	linger	ld;

	ld.l_onoff  = 1;
	ld.l_linger = 1000;

	if ( setsockopt( fd, SOL_SOCKET, SO_DONTLINGER,
	(char *) &ld, sizeof(ld) ) < 0 )
	{
	    perror( "Init_socket: SO_DONTLINGER" );
	    close(fd);
	    exit( 1 );
	}
    }
#endif

    sa		    = sa_zero;
    sa.sin_family   = AF_INET;
    sa.sin_port	    = htons( port );

    if ( bind( fd, (struct sockaddr *) &sa, sizeof(sa) ) < 0 )
    {
	perror("Init socket: bind" );
	close(fd);
	exit(1);
    }


    if ( listen( fd, 3 ) < 0 )
    {
	perror("Init socket: listen");
	close(fd);
	exit(1);
    }

    return fd;
}
#endif

/* Radyth's input buffer clear */
void clear_inputbuffer (DESCRIPTOR_DATA *d)
{
        CHAR_DATA *ch = d->character;

        if (ch == NULL) return;

        d->inbuf[0] = '\0';
        stc("All stacked commands have been cleared.\n\r", ch);
        return;
}

void check_clear_buffer (DESCRIPTOR_DATA *d)
{
        if (!str_infix("\n`", d->inbuf))
                clear_inputbuffer(d);

        return;
}



#if defined(macintosh) || defined(MSDOS)
void game_loop_mac_msdos( void )
{
    struct timeval last_time;
    struct timeval now_time;
    static DESCRIPTOR_DATA dcon;

    gettimeofday( &last_time, NULL );
    current_time = (time_t) last_time.tv_sec;

    /*
     * New_descriptor analogue.
     */
    dcon.descriptor	= 0;
    dcon.connected	= CON_GET_NAME;
    dcon.host		= str_dup( "localhost" );
    dcon.outsize	= 2000;
    dcon.outbuf		= alloc_mem( dcon.outsize );
    dcon.next		= descriptor_list;
    dcon.showstr_head	= NULL;
    dcon.showstr_point	= NULL;
    dcon.pEdit		= NULL;			/* OLC */
    dcon.pString	= NULL;			/* OLC */
    dcon.editor		= 0;			/* OLC */
    descriptor_list	= &dcon;

    /*
     * Send the greeting.
     */
    {
	extern char * help_greeting;
	if ( help_greeting[0] == '.' )
	    write_to_descriptor( dnew, help_greeting+1, 0 );
		write_to_buffer(dnew, "A voice whispers, 'Who stands before the gates?' ", 0);
	else
	    write_to_descriptor( dnew, help_greeting+1, 0 );
		write_to_buffer(dnew, "A voice whispers, 'Who stands before the gates?' ", 0);
    }

    /* Main loop */
    while ( !merc_down )
    {
	DESCRIPTOR_DATA *d;

	/*
	 * Process input.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next	= d->next;
	    d->fcommand	= FALSE;

#if defined(MSDOS)
	    if ( kbhit( ) )
#endif
	    {
		if ( d->character != NULL )
		    d->character->timer = 0;
		if ( !read_from_descriptor( d ) )
		{
		    if ( d->character != NULL && d->connected == CON_PLAYING)
			//save_char_obj( d->character );
			save_char_obj(d->character, FALSE, FALSE);
		    d->outtop	= 0;
		    close_socket( d );
		    continue;
		}
	    }

		check_clear_buffer(d);

	    if (d->character != NULL && d->character->daze > 0)
		--d->character->daze;

	    if ( d->character != NULL && d->character->wait > 0 )
	    {
		--d->character->wait;
		continue;
	    }

	    read_from_buffer( d );
	    if ( d->incomm[0] != '\0' )
	    {
		d->fcommand	= TRUE;

		if ( d->pProtocol != NULL )
			d->pProtocol->WriteOOB = 0;

		stop_idling( d->character );

	        /* OLC */
	        if ( d->showstr_point )
	            show_string( d, d->incomm );
	        else
	        if ( d->pString )
	            string_add( d->character, d->incomm );
	        else
	            switch ( d->connected )
	            {
	                case CON_PLAYING:
			    if ( !run_olc_editor( d ) )
				check_multiple_commands(d,d->incomm);
				substitute_alias( d, d->incomm );
			    break;
	                default:
			    nanny( d, d->incomm );
			    break;
	            }

		d->incomm[0]	= '\0';
	    }
	}



	/*
	 * Autonomous game motion.
	 */
	update_handler( );



	/*
	 * Output.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next = d->next;

	    if ( ( d->fcommand || d->outtop > 0 ) )
	    {
		if ( !process_output( d, TRUE ) )
		{
		    if ( d->character != NULL && d->connected == CON_PLAYING)
			//save_char_obj( d->character );
			save_char_obj(d->character, FALSE, FALSE);
		    d->outtop	= 0;
		    close_socket( d );
		}
	    }
	}



	/*
	 * Synchronize to a clock.
	 * Busy wait (blargh).
	 */
	now_time = last_time;
	for ( ; ; )
	{
	    int delta;

#if defined(MSDOS)
	    if ( kbhit( ) )
#endif
	    {
		if ( dcon.character != NULL )
		    dcon.character->timer = 0;
		if ( !read_from_descriptor( &dcon ) )
		{
		    if ( dcon.character != NULL && d->connected == CON_PLAYING)
			//save_char_obj( d->character );
			save_char_obj(d->character, FALSE, FALSE);
		    dcon.outtop	= 0;
		    close_socket( &dcon );
		}
#if defined(MSDOS)
		break;
#endif
	    }

	    gettimeofday( &now_time, NULL );
	    delta = ( now_time.tv_sec  - last_time.tv_sec  ) * 1000 * 1000
		  + ( now_time.tv_usec - last_time.tv_usec );
	    if ( delta >= 1000000 / PULSE_PER_SECOND )
		break;
	}
	last_time    = now_time;
	current_time = (time_t) last_time.tv_sec;
    }

    return;
}
#endif



#if defined(unix)
void game_loop_unix( int control )
{
    static struct timeval null_time;
    struct timeval last_time;
    int errno;

    if ( control > 4 ) control = 4;
    signal( SIGPIPE, SIG_IGN );
    gettimeofday( &last_time, NULL );
    current_time = (time_t) last_time.tv_sec;

    /* Main loop */
    while ( !merc_down )
    {
	fd_set in_set;
	fd_set out_set;
	fd_set exc_set;
	DESCRIPTOR_DATA *d;
	int maxdesc;

#if defined(MALLOC_DEBUG)
	if ( malloc_verify( ) != 1 )
	    abort( );
#endif


	/*
	 * Poll all active descriptors.
	 */
	FD_ZERO( &in_set  );
	FD_ZERO( &out_set );
	FD_ZERO( &exc_set );
	FD_SET( control, &in_set );
	if ( control > 4 ) control = 4;
	maxdesc	= control;
	for ( d = descriptor_list; d; d = d->next )
	{
	    maxdesc = UMAX( maxdesc, d->descriptor );
	    FD_SET( d->descriptor, &in_set  );
	    FD_SET( d->descriptor, &out_set );
	    FD_SET( d->descriptor, &exc_set );
	}

	if ( select( maxdesc+1, &in_set, &out_set, &exc_set, &null_time ) < 0 )
	{
	    perror( "Game_loop: select: poll" );
	    exit( 1 );
	}

	/*
	 * New connection?
	 */
	if ( FD_ISSET( control, &in_set ) )
	{
		if ( control > 4 ) control = 4;
	    init_descriptor( control );
	 }

	/*
	 * Kick out the freaky folks.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next = d->next;
	    if ( FD_ISSET( d->descriptor, &exc_set ) )
	    {
		FD_CLR( d->descriptor, &in_set  );
		FD_CLR( d->descriptor, &out_set );
		if ( d->character && d->connected == CON_PLAYING)
		    //save_char_obj( d->character );
		    save_char_obj(d->character, FALSE, FALSE);
		d->outtop	= 0;
		close_socket( d );
	    }
	}

	/*
	 * Process input.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next	= d->next;
	    d->fcommand	= FALSE;

	    if ( FD_ISSET( d->descriptor, &in_set ) )
	    {
			if ( d->character != NULL )
			    d->character->timer = 0;
			if ( !read_from_descriptor( d ) )
			{
			    FD_CLR( d->descriptor, &out_set );
			    if ( d->character != NULL && d->connected == CON_PLAYING)
				//save_char_obj( d->character );
				save_char_obj(d->character, FALSE, FALSE);
			    d->outtop	= 0;
			    close_socket( d );
			    continue;
			}
	    }

	    if (!d || d == NULL )
	    	continue;

		check_clear_buffer(d);

	    if (d->character != NULL && d->character->daze > 0)
	    {
		--d->character->daze;
		continue;
		}

	    if ( d->character != NULL && d->character->wait > 0 )
	    {
		--d->character->wait;
		continue;
	    }

	    read_from_buffer( d );
	    if ( d->incomm[0] != '\0' )
	    {
		d->fcommand	= TRUE;

		if ( d->pProtocol != NULL )
			d->pProtocol->WriteOOB = 0;

		stop_idling( d->character );

	/* OLC */
	if ( d->showstr_point )
	    show_string( d, d->incomm );
	else
	if ( d->pString )
	    string_add( d->character, d->incomm );
	else
	    switch ( d->connected )
	    {
	        case CON_PLAYING:
		    if ( !run_olc_editor( d ) )
		    {
				check_multiple_commands(d,d->incomm);
    		    substitute_alias( d, d->incomm );
			}
		    break;
	        default:
		    nanny( d, d->incomm );
		    break;
	    }

		d->incomm[0]	= '\0';
	    }
	}

	/*
	 * Autonomous game motion.
	 */
	update_handler( );
	/* Check Event Que */
	heartbeat();

	/*
	 * Output.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next = d->next;

	    if ( ( d->fcommand || d->outtop > 0 )
	    &&   FD_ISSET(d->descriptor, &out_set) )
	    {
		if ( !process_output( d, TRUE ) )
		{
		    if ( d->character != NULL && d->connected == CON_PLAYING)
			//save_char_obj( d->character );
			save_char_obj(d->character, FALSE, FALSE);
		    d->outtop	= 0;
		    close_socket( d );
		}
	    }
	}



	/*
	 * Synchronize to a clock.
	 * Sleep( last_time + 1/PULSE_PER_SECOND - now ).
	 * Careful here of signed versus unsigned arithmetic.
	 */
	{
	    struct timeval now_time;
	    long secDelta;
	    long usecDelta;

	    gettimeofday( &now_time, NULL );
	    usecDelta	= ((int) last_time.tv_usec) - ((int) now_time.tv_usec)
			+ 1000000 / PULSE_PER_SECOND;
	    secDelta	= ((int) last_time.tv_sec ) - ((int) now_time.tv_sec );
	    while ( usecDelta < 0 )
	    {
		usecDelta += 1000000;
		secDelta  -= 1;
	    }

	    while ( usecDelta >= 1000000 )
	    {
		usecDelta -= 1000000;
		secDelta  += 1;
	    }

	    if ( secDelta > 0 || ( secDelta == 0 && usecDelta > 0 ) )
	    {
			struct timeval stall_time;

			stall_time.tv_usec = usecDelta;
			stall_time.tv_sec  = secDelta;
			if ( select( 0, NULL, NULL, NULL, &stall_time ) < 0 )
			{
				if ( errno != EINTR )
				{
					bugf( "game_loop(): select() failed: %s", strerror( errno ) );
					exit(1);
				}
			}
	    }
	}

	gettimeofday( &last_time, NULL );
	current_time = (time_t) last_time.tv_sec;
    }

    return;
}
#endif



#if defined(unix)
void init_descriptor( int control )
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *dnew;
    struct sockaddr_in sock;
    #ifndef THREADED_DNS
	struct hostent *from;
    #endif
    int unsigned desc;
    int unsigned size;

    if ( control > 4 ) control = 4;

    size = sizeof(sock);
    getsockname( control, (struct sockaddr *) &sock, &size );
    if ( ( desc = accept( control, (struct sockaddr *) &sock, &size) ) < 0 )
    {
		perror( "New_descriptor: accept" );
		return;
    }

#if !defined(FNDELAY)
#define FNDELAY O_NDELAY
#endif

    if ( fcntl( desc, F_SETFL, FNDELAY ) == -1 )
    {
		perror( "New_descriptor: fcntl: FNDELAY" );
		return;
    }

    /*
     * Cons a new descriptor.
     */
    dnew = new_descriptor();

    dnew->descriptor	= desc;
    dnew->pEdit		= NULL;			/* OLC */
    dnew->pString	= NULL;			/* OLC */
    dnew->editor	= 0;			/* OLC */
    dnew->multi_buf	= NULL;

	if (dnew->pProtocol == NULL)	dnew->pProtocol = ProtocolCreate();

    size = sizeof(sock);
    if ( getpeername( desc, (struct sockaddr *) &sock, &size ) < 0 )
    {
		perror( "New_descriptor: getpeername" );
		#ifdef THREADED_DNS
			bzero(dnew->host,sizeof(dnew->Host));
		#else
			dnew->host = str_dup("(unknown)");
		#endif
    }
    else
    {
		/*
		 * Would be nice to use inet_ntoa here but it takes a struct arg,
		 * which ain't very compatible between gcc and system libraries.
		 */
		int addr;

		addr = ntohl( sock.sin_addr.s_addr );
		sprintf( buf, "%d.%d.%d.%d",
		    ( addr >> 24 ) & 0xFF, ( addr >> 16 ) & 0xFF,
		    ( addr >>  8 ) & 0xFF, ( addr       ) & 0xFF );
		sprintf( log_buf, "Sock.sinaddr:  %s", buf );
		log_string( log_buf );
		#ifdef THREADED_DNS
			memcpy(dnew->Host,&sock.sin_addr,sizeof(addr));
			gethostname_cached(dnew->Host,4,TRUE);
		#else
			from = gethostbyaddr( (char *) &sock.sin_addr,
			    sizeof(sock.sin_addr), AF_INET );
			/*dnew->host = str_dup( buf );*/
		#endif
			dnew->host = str_dup( buf );
    }

	if (check_ban(dnew->host, BAN_ALL))
	{
		write_to_descriptor(desc, "Your site has been banned!", 0);
		logf("Host %s attempted to connect, but the site is banned.", dnew->host);
		close( desc );
		free_descriptor(dnew);
		return;
	}
    if ( check_ban(dnew->host,BAN_ALL))
    {
		write_to_descriptor( desc, "Your site has been banned!\n\r", 0 );
		logf("Host %s attempted to connect, but the site is banned.", dnew->host);
		close( desc );
		free_descriptor(dnew);
		return;
    }

    /*
     * Init descriptor data.
     */

    dnew->next			= descriptor_list;
    descriptor_list		= dnew;

	ProtocolNegotiate(dnew);

    /*
     * Send the greeting.
     */

    {
         extern char * help_greeting;
         CHAR_DATA *tmp;

         tmp = new_char();
         dnew->character = tmp;
         tmp->desc = dnew;
         SET_BIT( tmp->act, PLR_COLOUR );

	//printf("prool debug. help_greeting ='%s'\r\n", help_greeting); // prool

	//help_greeting = "\r\n{rTSE MUD{x\r\n\r\n"; // prool fool
	help_greeting = "\r\nTSE MUD\r\n\r\n"; // prool fool

         if ( help_greeting[0] == '.' )
         {
             stc( help_greeting+1, tmp );
			 stc("{cA voice whispers, '{CWho stands before the gates?{c' {x", tmp);
		 }
         else
         {
             stc( help_greeting, tmp );
			 stc("{cA voice whispers, '{CWho stands before the gates?{c' {x", tmp);
		 }

         REMOVE_BIT( tmp->act, PLR_COLOUR );
         free_char( tmp );
         dnew->character = NULL;
    }

    return;
}
#endif



void close_socket( DESCRIPTOR_DATA *dclose )
{
    CHAR_DATA *ch;

    if ( dclose->outtop > 0 )
	process_output( dclose, FALSE );

    if ( dclose->snoop_by != NULL )
		write_to_buffer( dclose->snoop_by,"Your victim has left the game.\n\r", 0 );

    {
		DESCRIPTOR_DATA *d;

		for ( d = descriptor_list; d != NULL; d = d->next )
		{
		    if ( d->snoop_by == dclose )
			d->snoop_by = NULL;
		}
    }

    if ( ( ch = dclose->character ) != NULL )
    {
		sprintf( log_buf, "Closing link to %s.", ch->name );
		log_string( log_buf );
		if(ch->pcdata->dfhunt_player_info[0] > 0 )
		{
		    ch->pcdata->dfhunt_player_info[0] = 5;  /* P_ROLE = P_LINKDEAD */
		    dfhunt_game_info[1]--;
		}
		/* cut down on wiznet spam when rebooting */
		/* If ch is writing note or playing, just lose link otherwise clear char */
		if ((dclose->connected == CON_PLAYING && !merc_down)
	    ||((dclose->connected >= CON_NOTE_TO)
	    && (dclose->connected <= CON_NOTE_FINISH)))
		{
		    act( "{c$n stiffens, and slowly forms into a statue.", ch, NULL, NULL, TO_ROOM );
		    wiznet("{cThe Net Beast has claimed $N.",ch,NULL,WIZ_LINKS,0,0);
		    ch->desc = NULL;
		}
		else
		{
		    free_char(dclose->original ? dclose->original :
			dclose->character );
		}
    }

	free_multibuf(dclose);
	free_runbuf(dclose->character);


    if ( d_next == dclose )
		d_next = d_next->next;

    if ( dclose == descriptor_list )
    {
		descriptor_list = descriptor_list->next;
    }
    else
    {
		DESCRIPTOR_DATA *d;

		for ( d = descriptor_list; d && d->next != dclose; d = d->next )
		    ;
		if ( d != NULL )
		    d->next = dclose->next;
		else
		    bug( "Close_socket: dclose not found.", 0 );
    }

    close( dclose->descriptor );

	if (dclose->pProtocol)
    ProtocolDestroy( dclose->pProtocol ); /* <--- Add this line */
    free_descriptor(dclose);
	
#if defined(MSDOS) || defined(macintosh)
    exit(1);
#endif
    return;
}



bool read_from_descriptor( DESCRIPTOR_DATA *d )
{
    int iStart;
    static char read_buf[MAX_PROTOCOL_BUFFER]; /* <--- Add this line */
    read_buf[0] = '\0';                        /* <--- Add this line */

    /* Hold horses if pending command already. */
    if ( d->incomm[0] != '\0' )
	return TRUE;

    /* Check for overflow. */

    iStart = 0;
    if ( strlen(d->inbuf) >= sizeof(d->inbuf) - 10 )
    {
		sprintf( log_buf, "%s input overflow!", d->host );
		log_string( log_buf );
		write_to_descriptor( d->descriptor,"\n\r*** PUT A LID ON IT!!! ***\n\r", 0 );
		return FALSE;
    }

    /* Snarf input. */
#if defined(macintosh)
    for ( ; ; )
    {
		int c;
		c = getc( stdin );
		if ( c == '\0' || c == EOF )
		    break;
		putc( c, stdout );
		if ( c == '\r' )
		    putc( '\n', stdout );
		read_buf[iStart++] = c;
		if ( iStart > sizeof(d->inbuf) - 10 )
		    break;
    }
#endif

#if defined(MSDOS) || defined(unix)
    for ( ; ; )
    {
		int nRead;

		nRead = read( d->descriptor, read_buf + iStart,
		    sizeof(read_buf) - 10 - iStart );
		if ( nRead > 0 )
		{
		    iStart += nRead;
		    if ( read_buf[iStart-1] == '\n' || read_buf[iStart-1] == '\r' )
			break;
		}
		else if ( nRead == 0 )
		{
		    log_string( "EOF encountered on read." );
		    return FALSE;
		}
		else if ( errno == EWOULDBLOCK )
		    break;
		else
		{
		    perror( "Read_from_descriptor" );
		    return FALSE;
		}
    }
#endif

    read_buf[iStart] = '\0';
    ProtocolInput (d, read_buf, iStart, d->inbuf);
    return TRUE;
}

/*
 * Transfer one line from input buffer to input line.
 */
void read_from_buffer( DESCRIPTOR_DATA *d )
{
    int i, j, k;
    char buf[MSL];
    buf[0]='\0';

	/*Jogging code - Anthony*/
	if ( d->character && d->character->run_buf )
	{
		/* Insert fighting check here */
		/* then duplicate error code from do_north etc */
		while (isdigit(*d->character->run_head) && *d->character->run_head != '\0')
		{
			char *s,*e;

			s = d->character->run_head;
			while( isdigit( *s ) )
				s++;
			e = s;
			while( *(--s) == '0' && s != d->character->run_head );
				if ( isdigit( *s ) && *s != '0' && (*e != 'o' || *e != 'c'|| *e != 'C' || *e != 'P' || *e != 'U' || *e != 'L'))
				{
					switch(*e)
					{
						case 'n' : move_char(d->character, DIR_NORTH, TRUE);	break;
						case 'e' : move_char(d->character, DIR_EAST, TRUE);		break;
						case 's' : move_char(d->character, DIR_SOUTH, TRUE);	break;
						case 'w' : move_char(d->character, DIR_WEST, TRUE);		break;
						case 'u' : move_char(d->character, DIR_UP, TRUE);		break;
						case 'd' : move_char(d->character, DIR_DOWN, TRUE);		break;
						case 'j': case 'N': move_char(d->character, DIR_NORTHEAST, TRUE);	break;
						case 'h': case 'W': move_char(d->character, DIR_NORTHWEST, TRUE);	break;
						case 'k': case 'S': move_char(d->character, DIR_SOUTHWEST, TRUE);	break;
						case 'l': case 'E': move_char(d->character, DIR_SOUTHEAST, TRUE);	break;
						default: return;	break;
					}

					s[0]--;
					while (isdigit(*(++s)))
						*s = '9';
					return;
				}
			if (*e == 'o' || *e == 'c' || *e == 'C' || *e == 'P' || *e == 'U' || *e == 'L')
				d->character->run_head = e;
			else
				d->character->run_head = ++e;
		}
		if (*d->character->run_head != '\0')
		{
			/*char buf[MAX_INPUT_LENGTH];*/

			if(*d->character->run_head == 'o')
			{
				sprintf(buf, "open ");
				d->character->run_head++;
			}
			else if(*d->character->run_head == 'c')
			{
				sprintf(buf, "climb ");
				d->character->run_head++;
			}
			else if(*d->character->run_head == 'C')
                        {
                                sprintf(buf, "close ");
			        d->character->run_head++;
			}
		        else if(*d->character->run_head == 'P')
		        {
				sprintf(buf, "pick ");
				d->character->run_head++;
			}
			else if(*d->character->run_head == 'U')
			{
				sprintf(buf, "unlock ");
				d->character->run_head++;
			}
			else if(*d->character->run_head == 'L')
			{
				sprintf(buf, "lock ");
				d->character->run_head++;
			}
			else
			{
				sprintf(buf, " ");
			}

			switch(*d->character->run_head )
			{
				case 'n' : sprintf( buf+strlen(buf), "north" ); break;
				case 's' : sprintf( buf+strlen(buf), "south" ); break;
				case 'e' : sprintf( buf+strlen(buf), "east" ); break;
				case 'w' : sprintf( buf+strlen(buf), "west" ); break;
				case 'u' : sprintf( buf+strlen(buf), "up" ); break;
				case 'd' : sprintf( buf+strlen(buf), "down" ); break;
				case 'j': case 'N': sprintf( buf+strlen(buf), "northeast" ); break;
				case 'h': case 'W': sprintf( buf+strlen(buf), "northwest" ); break;
				case 'k': case 'S': sprintf( buf+strlen(buf), "southwest" ); break;
				case 'l': case 'E': sprintf( buf+strlen(buf), "southeast" ); break;
				default: return;
			}

			interpret( d->character, buf );
			d->character->run_head++;
			return;
		}
		free_runbuf(d->character);

		if (d->character->jog_full)
		{
			stc("You arrive to your destination and you stop jogging.\n\r", d->character);
			WAIT_STATE(d->character, 12);
			d->character->jog_to = -1;
		}
		else if (d->character->jog_to >= 0 && get_room_index(d->character->pcdata->beacons[d->character->jog_to]))
		{
			sprintf(buf, "to %d", d->character->jog_to);
			do_jog(d->character, buf);
		}
		else stc("You slow to a stop.\n\r", d->character);

		return;	
	}

    /*
     * Hold horses if pending command already.
     */
    if ( d->incomm[0] != '\0' )
	return;

	if(d->multi_buf)
	{
		strcpy( d->incomm, d->multi_buf );
		free_multibuf(d);
		return;
	}
    /*
     * Look for at least one new line.
     */
    for ( i = 0; d->inbuf[i] != '\n' && d->inbuf[i] != '\r'; i++ )
    {
	if ( d->inbuf[i] == '\0' )
	    return;
    }

    /*
     * Canonical input processing.
     */
    for ( i = 0, k = 0; d->inbuf[i] != '\n' && d->inbuf[i] != '\r'; i++ )
    {
	if ( k >= MAX_INPUT_LENGTH - 2 )
	{
	    write_to_descriptor( d->descriptor, "Line too long.\n\r", 0 );

	    /* skip the rest of the line */
	    for ( ; d->inbuf[i] != '\0'; i++ )
	    {
		if ( d->inbuf[i] == '\n' || d->inbuf[i] == '\r' )
		    break;
	    }
	    d->inbuf[i]   = '\n';
	    d->inbuf[i+1] = '\0';
	    break;
	}

	if ( d->inbuf[i] == '\b' && k > 0 )
	    --k;
	else if ( isascii(d->inbuf[i]) && isprint(d->inbuf[i]) )
	    d->incomm[k++] = d->inbuf[i];
    }

    /*
     * Finish off the line.
     */
    if ( k == 0 )
	d->incomm[k++] = ' ';
    d->incomm[k] = '\0';

    /*
     * Deal with bozos with #repeat 1000 ...
     */

    if ( k > 1 || d->incomm[0] == '!' )
    {
    	if ( d->incomm[0] != '!' && strcmp( d->incomm, d->inlast ) )
	{
	    d->repeat = 0;
	}
	else
	{
	    if (++d->repeat >= 50 && d->character
	    &&  d->connected == CON_PLAYING)
	    {
		sprintf( log_buf, "%s input spamming!", d->host );
		log_string( log_buf );
		wiznet("{cSpam $N Spam!",
		       d->character,NULL,WIZ_SPAM,0,get_trust(d->character));
		if (d->incomm[0] == '!')
		    wiznet(d->inlast,d->character,NULL,WIZ_SPAM,0,
			get_trust(d->character));
		else
		    wiznet(d->incomm,d->character,NULL,WIZ_SPAM,0,
			get_trust(d->character));

		d->repeat = 0;

		write_to_descriptor( d->descriptor,
		    "\n\r*** PUT A LID ON IT!!! ***\n\r", 0 );
		if(d->character->pcdata && d->character->pcdata->dfhunt_player_info[0] > 0 )
		{
	    	    d->character->pcdata->dfhunt_player_info[0] = 5;  /* P_ROLE = P_LINKDEAD */
		    dfhunt_game_info[1]--;
		}
		strcpy( d->incomm, "quit" );

	    }
	}
    }


    /*
     * Do '!' substitution.
     */
    if ( d->incomm[0] == '!' )
	strcpy( d->incomm, d->inlast );
    else
	strcpy( d->inlast, d->incomm );

    /*
     * Shift the input buffer.
     */
    while ( d->inbuf[i] == '\n' || d->inbuf[i] == '\r' )
	i++;
    for ( j = 0; ( d->inbuf[j] = d->inbuf[i+j] ) != '\0'; j++ )
	;
    return;
}



/*
 * Low level output function.
 */
bool process_output( DESCRIPTOR_DATA *d, bool fPrompt )
{
	extern bool merc_down;

	if ( d->pProtocol->WriteOOB ); /* <-- Add this, and the ";" and "else" */

	/* The last sent data was OOB, so do NOT draw the prompt */
	else if ( fPrompt && !merc_down && d->showstr_point )
		write_to_buffer( d, "[Hit Return to continue]\n\r", 0 );
	else if ( fPrompt && d->pString && d->connected == CON_PLAYING )
		write_to_buffer( d, "> ", 2 );

	else if ( fPrompt && d->connected == CON_PLAYING )
	{
		CHAR_DATA *ch;
		CHAR_DATA *victim;

		ch = d->character;

		if ( !d || !ch )
		{
			bugf( "Process_output shit itself with no d, no ch, or no victim in fPrompt", 0 );
			return TRUE;
		}

		/* battle prompt */
		if ( ch && d->character && ( victim = ch->advancing_on ) != NULL && can_see( ch,victim ) )
		{
			int percent;
			char wound[100];
			char buf[MSL];

			if (get_maxhit(victim))
				percent = victim->hit * 100 / get_maxhit(victim);
			else
				percent = -1;

			if (percent >= 100)
				sprintf(wound,"is in perfect health.");
			else if (percent >= 90)
				sprintf(wound,"has a couple of scratches.");
			else if (percent >= 75)
				sprintf(wound,"has some minor wounds.");
			else if (percent >= 50)
				sprintf(wound,"has quite a few wounds.");
			else if (percent >= 30)
				sprintf(wound,"has some big nasty wounds.");
			else if (percent >= 15)
				sprintf(wound,"looks pretty beat up.");
			else if (percent >= 0)
				sprintf(wound,"is knocking on death's door.");
			else
				sprintf(wound,"is bleeding to death.");

			sprintf(buf,"%s %s \n\r",IS_NPC(victim) ? victim->short_descr : victim->name,wound);
			buf[0] = UPPER(buf[0]);
			stc( buf, ch );
		}

		ch = d->original ? d->original : d->character;

		if (!IS_SET(ch->comm, COMM_COMPACT) )
			stc( "\n\r", ch );

		if ( IS_SET(ch->comm, COMM_PROMPT) )
			bust_a_prompt( d->character );

		if (IS_SET(ch->comm,COMM_TELNET_GA))
			write_to_buffer(d,go_ahead_str,0);
	}

	/*
	 * Short-circuit if nothing to write.
	 */
	if ( d->outtop == 0 )
		return TRUE;

	/*
	 * Snoop-o-rama.
	 */
	if ( d->snoop_by != NULL )
	{
		if (d->character != NULL)
		write_to_buffer( d->snoop_by, d->character->name,0);
		write_to_buffer( d->snoop_by, "> ", 2 );
		write_to_buffer( d->snoop_by, d->outbuf, d->outtop );
	}

	/*
	 * OS-dependent output.
	 */
	if ( !write_to_descriptor( d->descriptor, d->outbuf, d->outtop ) )
	{
		d->outtop = 0;
		return FALSE;
	}
	else
	{
		d->outtop = 0;
		return TRUE;
	}
}

void bust_a_prompt( CHAR_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    CHAR_DATA *bond;
    char *bond_data;
    int bond_percent;
    CHAR_DATA *bond2;
    char *bond_data2;
    int bond_percent2;
    const char *str;
    const char *i;
    CHAR_DATA *victim;
    char buf3[MSL];
    int percent;
    int percent2;
    char *point;
    char *pbuff;
    char buffer[ MAX_STRING_LENGTH*2 ];
    char doors[MAX_INPUT_LENGTH];
    EXIT_DATA *pexit;
    bool found;
    const char *dir_name[] = {"N","E","S","W","U","D","NE","SE","SW","NW"};
    int door;

	/* Battle prompt declarations */
	char wound[20];
	int bpercent;
	int spercent;
	spercent = ch->hit * 100 / get_maxhit(ch);

	point = buf;
	str = ch->prompt;

	if (str == NULL || str[0] == '\0')
	{
		sprintf( buf, "{C[{cHP{C:{c%d SP{C:{c%d{C]{c %s",
			ch->hit,ch->stam,ch->prefix);
		stc(buf,ch);
		return;
	}

	if (IS_SET(ch->comm,COMM_AFK))
	{
		stc("{c[{wAFK{c]{x ",ch);
		return;
	}

	while( *str != '\0' )
	{
		if( *str != '%' )
		{
			*point++ = *str++;
			continue;
		}
		++str;
		switch( *str )
		{
			default :
				i = "";
			break;

			case 'a':
				if (ON_GQUEST(ch))
					sprintf(buf2, "%d", gquest_info.timer);
				else
					sprintf(buf2, "%dn", gquest_info.next);
				i = buf2;
			break;

			case 'A':
				sprintf( buf2, "Absent" );
				if ( IS_SET(ch->hist_flags, HIST_BONDED) || IS_SET(ch->hist_flags,HIST_BONDER) )
				{
					bond_data = str_dup(ch->pcdata->bonded_to);
					if ( ( bond = get_pc_world( ch, bond_data ) ) != NULL )
					{
						bond_percent = bond->hit * 100 / get_maxhit(bond);
						sprintf( buf2, "%d", bond_percent );
					}
					bond_data2 = str_dup(ch->pcdata->bonded_by);
					if ( ( bond2 = get_pc_world( ch, bond_data2 ) ) != NULL )
					{
						bond_percent2 = bond2->hit * 100 / get_maxhit(bond2);
						sprintf( buf2, "%d", bond_percent2 );
					}
				}
				i = buf2;
			break;

			case 'b':
				if ((victim = ch->advancing_on) != NULL)
				{
					if (get_maxhit(victim) > 0)
						bpercent = victim->hit * 100 / get_maxhit(victim);
					else
						bpercent = -1;

					if (bpercent >= 96)
						sprintf (wound, "{wEnemy: {W[{C++++++++++{W]{x\n\r");
					else if (bpercent >= 91)
						sprintf (wound, "{wEnemy: {W[{C-+++++++++{W]{x\n\r");
					else if (bpercent >= 86)
						sprintf (wound, "{wEnemy: {W[{c +++++++++{W]{x\n\r");
					else if (bpercent >= 81)
						sprintf (wound, "{wEnemy: {W[{c -++++++++{W]{x\n\r");
					else if (bpercent >= 76)
						sprintf (wound, "{wEnemy: {W[{B  ++++++++{W]{x\n\r");
					else if (bpercent >= 71)
						sprintf (wound, "{wEnemy: {W[{B  -+++++++{W]{x\n\r");
					else if (bpercent >= 66)
						sprintf (wound, "{wEnemy: {W[{b   +++++++{W]{x\n\r");
					else if (bpercent >= 61)
						sprintf (wound, "{wEnemy: {W[{b   -++++++{W]{x\n\r");
					else if (bpercent >= 56)
						sprintf (wound, "{wEnemy: {W[{W    ++++++{W]{x\n\r");
					else if (bpercent >= 51)
						sprintf (wound, "{wEnemy: {W[{W    -+++++{W]{x\n\r");
					else if (bpercent >= 46)
						sprintf (wound, "{wEnemy: {W[{w     +++++{W]{x\n\r");
					else if (bpercent >= 41)
						sprintf (wound, "{wEnemy: {W[{w     -++++{W]{x\n\r");
					else if (bpercent >= 36)
						sprintf (wound, "{wEnemy: {W[{Y      ++++{W]{x\n\r");
					else if (bpercent >= 31)
						sprintf (wound, "{wEnemy: {W[{Y      -+++{W]{x\n\r");
					else if (bpercent >= 26)
						sprintf (wound, "{wEnemy: {W[{y       +++{W]{x\n\r");
					else if (bpercent >= 21)
						sprintf (wound, "{wEnemy: {W[{y       -++{W]{x\n\r");
					else if (bpercent >= 16)
						sprintf (wound, "{wEnemy: {W[{R        ++{W]{x\n\r");
					else if (bpercent >= 11)
						sprintf (wound, "{wEnemy: {W[{R        -+{W]{x\n\r");
					else if (bpercent >= 6)
						sprintf (wound, "{wEnemy: {W[{r         +{W]{x\n\r");
					else if (bpercent >= 1)
						sprintf (wound, "{wEnemy: {W[{r         -{W]{x\n\r");
					else
						sprintf (wound, "{wEnemy: {W[{D          {W]{x\n\r");
					sprintf (buf2, " %s", wound);
					i = buf2;
				}
				else
					i = "";
			break;

			case 'B' :
				if (is_fighting(ch) && (victim = ch->advancing_on) != NULL)
				{
					if ( victim == NULL )
					{
						i = "";
						break;
					}
					if ( !is_fighting(ch) )
					{
						i = "";
						break;
					}
					if ( victim->hit == 0 )
					{
						i = "";
						break;
					}

					percent = victim->hit * 100 / get_maxhit(victim);

					if( percent >= 65 )
						sprintf( buf2,"{w[ {WE: {w%d%%",percent );
					else if( percent >= 25 && percent < 65 )
						sprintf( buf2,"{w[ {WE: {Y%d%%{w",percent );
					else
						sprintf( buf2,"{w[ {WE: {r%d{w%%",percent );

					percent2 = ch->hit * 100 / get_maxhit(ch);
					if( percent2 >= 65 )
						sprintf( buf3,"  {w[ {WU: {w%d%%",percent2 );
					else if( percent2 >= 25 && percent2 < 65 )
						sprintf( buf3,"  {w[ {WU: {Y%d%%{w",percent2 );
					else
						sprintf( buf3,"  {w[ {WU: {r%d{w%%",percent2 );
					strcat( buf2,buf3 );
					i = buf2;
				}
				else
					i = "";
			break;

			case 'c' :
				sprintf(buf2,"%s","\n\r");
				i = buf2;
			break;

			case 'C' :
				if ( ch->pcdata != NULL )
					sprintf(buf2,"%d", ch->pcdata->concentration);
				i = buf2;
			break;

			case 'e':
				found = FALSE;
				doors[0] = '\0';
				for (door = 0; door < 10; door++)
				{
					if ( ( pexit = ch->in_room->exit[door] ) != NULL
					&& pexit ->u1.to_room != NULL
					&& (can_see_room(ch,pexit->u1.to_room)
					|| (IS_AFFECTED(ch,AFF_INFRARED)
					&& !IS_AFFECTED(ch,AFF_BLIND)))
					&& !IS_SET(pexit->exit_info,EX_CLOSED))
					{
						found = TRUE;
						strcat(doors,dir_name[door]);
					}
				}
				if (!found)
					strcat(doors,"none");
				sprintf(buf2,"%s",doors);
				i = buf2;
			break;

			case 'E' :
/*				if ( !getbit( ch->pcdata->explored, ch->in_room->vnum ) ) */
				if (ch->pcdata && ch->pcdata->new_room == TRUE)
					sprintf( buf2, "{R*{x" );
				else
					sprintf( buf2, "{G*{x" );
				i = buf2;
			break;

			case 'g' :
				sprintf( buf2, "%ld", ch->money[3][0]);
				i = buf2;
			break;

			case 'G' :
				sprintf( buf2, "%ld", ch->money[3][1]);
				i = buf2;
			break;

			case 'h' :
				sprintf( buf2, "%d", ch->hit );
				i = buf2;
			break;

			case 'H' :
				sprintf( buf2, "%d", get_maxhit(ch) );
				i = buf2;
			break;

			case 'i':
				if (ch->incog_level >= LEVEL_HERO )
					sprintf (buf2, "(Incog)");
				else
					sprintf(buf2, "{w" );
				i = buf2;
			break;

			case 'k':
				if (IS_AFFECTED(ch, AFF_HIDE))
					sprintf( buf2, "(Hide)");
				else
					sprintf( buf2, "{w" );
				i = buf2;
			break;

			case 'K':
				if (IS_AFFECTED(ch, AFF_SNEAK))
					sprintf( buf2, "(Sneak)");
				else
					sprintf( buf2, "{w" );
				i = buf2;
			break;

			case 'l' :
			{
				int tier;
				int level_req;

				tier = calc_remort_tier(ch);
				level_req = tier * 25;
				level_req += 225;

				sprintf(buf2, "%d",level_req - ch->level);
				i = buf2;
			}
			break;

			case 'L' :
				sprintf(buf2, "%d", ch->level);
				i = buf2;
			break;

			case 'm' :
				sprintf( buf2, "%d", ch->stam );
				i = buf2;
			break;

			case 'M' :
				sprintf( buf2, "%d", get_maxstam(ch) );
				i = buf2;
			break;

			case 'o' :
				sprintf( buf2, "%s", olc_ed_name(ch) );
				i = buf2;
			break;

			case 'O' :
				sprintf( buf2, "%s", olc_ed_vnum(ch) );
				i = buf2;
			break;

			case 'p' :
				sprintf( buf2, "%ld %ld", ch->money[1][0], ch->money[0][0]);
				i = buf2;
			break;
			case 'P' :
				{
				    char buf3[MSL];
				    int sn =  ch->pcdata->prompt_skill;
				    if( sn > 0)
				    {
	    				if (ch->pcdata->learned[sn] < 75)
						sprintf( buf3, "{x");/*x*/
			  	   	else if (ch->pcdata->learned[sn] < 85)
						sprintf( buf3, "{y");
	    				else if (ch->pcdata->learned[sn] < 100)
						sprintf( buf3, "{g");/*g*/
	    				else    sprintf( buf3, "{w");
					sprintf( buf2, "%s%s: %d{x", buf3, skill_table[sn].name, ch->pcdata->learned[sn] );
				    }
				    else
					sprintf( buf2, "{w");
				    i = buf2;
				}
			break;

			case 'q' :
				if ( ch->countdown <= 0 )
					sprintf( buf2, "0" );
				else
					sprintf( buf2, "%d", ch->countdown );
				i = buf2;
			break;

			case 'Q' :
				if( ch->nextquest <= 0 )
					sprintf( buf2, "0" );
				else
					sprintf( buf2, "%d", ch->nextquest );
				i = buf2;
			break;

			case 'r' :
				if( ch->in_room != NULL )
					sprintf( buf2, "%s",
						((!IS_NPC(ch) && IS_SET(ch->act,PLR_HOLYLIGHT))
						|| (!IS_AFFECTED(ch,AFF_BLIND)
						&& !room_is_dark( ch->in_room )))
							? ch->in_room->name : "darkness");
				else
					sprintf( buf2, "{w" );
				i = buf2;
			break;

			case 'R' :
				/*
				if( IS_IMMORTAL( ch )
				&& ch->in_room != NULL )*/
				if ( ch->in_room != NULL )
					sprintf( buf2, "%ld", ch->in_room->vnum );
				else
					sprintf( buf2, "{w" );
				i = buf2;
			break;

			case 's' :
				sprintf( buf2, "%ld", ch->money[2][0]);
				i = buf2;
			break;

			case 'S' :
				sprintf( buf2, "%ld", ch->money[2][1]);
				i = buf2;
			break;

			case 'T':
				if ( ch->leader != NULL )
					sprintf( buf2, "Tank: %3d", ch->leader->hit * 100 / get_maxhit(ch->leader));
				else
					sprintf( buf2, "{w" );
				i = buf2;
			break;

			case 'v' :
				sprintf( buf2, "%d", ch->stam );
				i = buf2;
			break;

			case 'V' :
				sprintf( buf2, "%d", get_maxstam(ch) );
				i = buf2;
			break;

			case 'w':
				if ( ch->invis_level >= LEVEL_HERO )
					sprintf (buf2, "(Wizinvis)");
				else
					sprintf(buf2, "{w" );
				i = buf2;
			break;

			case 'W' :
				/* sprintf( buf2, "%d", ch->pcdata->grasped );*/ /* reverse if we decide to grasp amount of OP */
				if (IS_SET(ch->op_flags,OP_GRASPED))
				    sprintf(buf2, "(Grasped)");
				else
				    sprintf(buf2, "{w" );
				i = buf2;
			break;

			case 'x' :
				sprintf( buf2, "%d", ch->exp );
				i = buf2;
			break;

			case 'X' :
				sprintf(buf2, "%d", IS_NPC(ch) ? 0 :
					(ch->level + 1) * exp_per_level(ch,ch->pcdata->points) - ch->exp);
				i = buf2;
			break;

			case 'z' :
				if( IS_IMMORTAL( ch ) && ch->in_room != NULL )
					sprintf( buf2, "%s", ch->in_room->area->name );
				else
					sprintf( buf2, "{w" );
				i = buf2;
			break;

			case '%' :
				sprintf( buf2, "%%" );
				i = buf2;
			break;
        }
        ++str;
        while( (*point = *i) != '\0' )
           ++point, ++i;
   }
   *point	= '\0';
   pbuff	= buffer;
   colourconv( pbuff, buf, ch );
   stc( "{x", ch );
   write_to_buffer( ch->desc, buffer, 0 );
   stc( "{x", ch );

   if (ch->prefix[0] != '\0')
        write_to_buffer(ch->desc,ch->prefix,0);
   return;
}



/*
 * Append onto an output buffer.
 */
void write_to_buffer( DESCRIPTOR_DATA *d, const char *txt, int length )
{
    txt = ProtocolOutput( d, txt, &length );  /* <--- Add this line */
    if ( d->pProtocol->WriteOOB > 0 )         /* <--- Add this line */
        --d->pProtocol->WriteOOB;             /* <--- Add this line */

    /*
     * Find length in case caller didn't.
     */
    if ( length <= 0 )
	length = strlen(txt);

    /*
     * Initial \n\r if needed.
     */
    if ( d->outtop == 0 && !d->fcommand && !d->pProtocol->WriteOOB)
    {
	d->outbuf[0]	= '\n';
	d->outbuf[1]	= '\r';
	d->outtop	= 2;
    }

    /*
     * Expand the buffer as needed.
     */
    while ( d->outtop + length >= d->outsize )
    {
	char *outbuf;

        if (d->outsize >= 32000)
	{
	    bug("Buffer overflow. Closing.\n\r",0);
	    close_socket(d);
	    return;
 	}
	outbuf      = alloc_mem( 2 * d->outsize );
	strncpy( outbuf, d->outbuf, d->outtop );
	free_mem( d->outbuf, d->outsize );
	d->outbuf   = outbuf;
	d->outsize *= 2;
    }

    /*
     * Copy.
     */
    strncpy( d->outbuf + d->outtop, txt, length );
    d->outtop += length;
    return;
}



/*
 * Lowest level output function.
 * Write a block of text to the file descriptor.
 * If this gives errors on very long blocks (like 'ofind all'),
 *   try lowering the max block size.
 */
bool write_to_descriptor( int desc, char *txt, int length )
{
    int iStart;
    int nWrite;
    int nBlock;

#if defined(macintosh) || defined(MSDOS)
    if ( desc == 0 )
	desc = 1;
#endif

    if ( length <= 0 )
	length = strlen(txt);

    for ( iStart = 0; iStart < length; iStart += nWrite )
    {
	nBlock = UMIN( length - iStart, 4096 );
	if ( ( nWrite = write( desc, txt + iStart, nBlock ) ) < 0 )
	    { perror( "Write_to_descriptor" ); return FALSE; }
    }

    return TRUE;
}



 void logprintf (char * fmt, ...)
 {
       char buf [2*MSL];
       va_list args;
       va_start (args, fmt);
       vsprintf (buf, fmt, args);
       va_end (args);

       log_string (buf);
 }


/*
 * Deal with sockets that haven't logged in yet.
 */


char *strlower(char *ip)
{
	static char buffer[MAX_INPUT_LENGTH];
	int pos;

	for (pos = 0; pos < (MAX_INPUT_LENGTH - 1) && ip[pos] != '\0'; pos++) {
		buffer[pos] = tolower(ip[pos]);
	}
	buffer[pos] = '\0';
	return buffer;
}
bool is_in(char *arg, char *ip)
{
	char *lo_arg;
	char cmp[MAX_INPUT_LENGTH];
	int fitted;

	if (ip[0] != '|')
		return FALSE;
	cmp[0] = '\0';
	lo_arg = strlower(arg);
	do {
		ip += strlen(cmp) + 1;
		fitted = sscanf(ip, "%[^*]", cmp);
		if (strstr(lo_arg, cmp) != NULL) {
			return TRUE;
		}
	} while (fitted > 0);
	return FALSE;
}

void stop_idling( CHAR_DATA *ch )
{
    if ( ch == NULL
    ||   ch->desc == NULL
    ||   ch->desc->connected != CON_PLAYING
    ||   ch->was_in_room == NULL
    ||   ch->in_room != get_room_index(ROOM_VNUM_LIMBO))
	return;

    ch->timer = 0;
    char_from_room( ch );
    char_to_room( ch, ch->was_in_room );
    ch->was_in_room	= NULL;
    act( "{c$n struggles free from the maw of the Net Beast.", ch, NULL, NULL, TO_ROOM );
    return;
}


//#define	CORE_EXAMINE_SCRIPT	"../corefiles/gdbscript"
/*#define CH(descriptor)  ((descriptor)->original ? (descriptor)->original : (descriptor)->character)
 * Already defined in aww.h -Anthony*/


void halt_mud ( int sig )
{
	DESCRIPTOR_DATA *d;
	CHAR_DATA *ch;
	extern char last_command2[MAX_INPUT_LENGTH];
	extern int last_descriptor;
	struct sigaction default_action;
	int i;
	pid_t forkpid;

	wait ( NULL );

	if ( !crashed )
	{
		crashed++;
		fprintf ( stderr, "GAME CRASHED (SIGNAL %d).\rLast command: %s\r", sig, last_command );
		// Inform last command typer that he caused the crash
		if ( strlen ( last_command2 ) )
		{
			write_to_descriptor ( last_descriptor, "\n\rThe last command you typed, '", 0 );
			write_to_descriptor ( last_descriptor, last_command2, 0 );
			write_to_descriptor ( last_descriptor, "', might have caused this crash.\n\r"
                                  "Please note any unusual circumstances to Immortal and avoid using that command.\n\r", 0 );
		}
        for ( d = descriptor_list; d != NULL; d = d_next )
        {
            d_next = d->next;
            ch = CH ( d );
            if ( !ch )
            {
                close_socket ( d );
                continue;
            }
            if ( IS_NPC ( ch ) )
                continue;
            write_to_descriptor ( d->descriptor,
                                  "\n\rThe mud has CRASHED.\007\n\r", 0 );
        }

        // try to save all characters - save_char_obj has sanity checking
        for ( d = descriptor_list; d != NULL; d = d_next )
        {
            d_next = d->next;
            ch = CH ( d );
            if ( !ch )
            {
                close_socket ( d );
                continue;
            }
            //save_char_obj ( ch );
	    save_char_obj(ch, FALSE, FALSE);
        }

        // success - proceed with fork/copyover plan.  Otherwise will go to
        // next section and crash with a full reboot to recover
        if ( ( forkpid = fork (  ) ) > 0 )
        {
            // Parent process copyover and exit
            waitpid ( forkpid, NULL, WNOHANG | WUNTRACED );
            // this requires you to add an "if (ch)" before the send_to_char
            // statements in do_copyover.
            //do_copyover ( NULL, "" );
		copyover();
            exit ( 0 );
        }
        else if ( forkpid < 0 )
        {
            exit ( 1 );
        }
        // Child process proceed to dump
        // Close all files!
        for ( i = 255; i >= 0; i-- )
            close ( i );

        // Dup /dev/null to STD{IN,OUT,ERR}
        open ( "/dev/null", O_RDWR );
        dup ( 0 );
        dup ( 0 );

        default_action.sa_handler = SIG_DFL;
        sigaction ( sig, &default_action, NULL );

        // I run different scripts depending on my port
	// Is this why we aren't getting core dumps? -- Dalsor 12.20.2012
	/*
        if ( !fork (  ) )
        {
            execl ( CORE_EXAMINE_SCRIPT, CORE_EXAMINE_SCRIPT,
                    ( char * ) NULL );
            exit ( 0 );
        }
        else
            return;
	*/
        raise ( sig );
    }

    if ( crashed == 1 )
    {
        crashed++;

        for ( d = descriptor_list; d != NULL; d = d_next )
        {
            d_next = d->next;
            ch = d->original ? d->original : d->character;
            if ( ch == NULL )
            {
                close_socket ( d );
                continue;
            }
            if ( IS_NPC ( ch ) )
                continue;
            write_to_descriptor ( d->descriptor,
                                  "** Error saving character files; conducting full reboot. **\007\n\r",
                                  0 );
            close_socket ( d );
            continue;
        }
        fprintf ( stderr, "CHARACTERS NOT SAVED.\r" );
        default_action.sa_handler = SIG_DFL;
        sigaction ( sig, &default_action, NULL );

        if ( !fork (  ) )
        {
            kill ( getppid (  ), sig );
            exit ( 1 );
        }
        else
            return;
        raise ( sig );
    }

    if ( crashed == 2 )
    {
        crashed++;
        fprintf ( stderr, "TOTAL GAME CRASH." );
        default_action.sa_handler = SIG_DFL;
        sigaction ( sig, &default_action, NULL );

        if ( !fork (  ) )
        {
            kill ( getppid (  ), sig );
            exit ( 1 );
        }
        else
            return;
        raise ( sig );
    }

    if ( crashed == 3 )
    {
        default_action.sa_handler = SIG_DFL;
        sigaction ( sig, &default_action, NULL );

        if ( !fork (  ) )
        {
            kill ( getppid (  ), sig );
            exit ( 1 );
        }
        else
            return;
        raise ( sig );
    }
}
