/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <time.h>
#include "aww.h"
#include "magic.h"
#include "interp.h"


/* item type list */
const struct item_type		item_table	[]	=
{
    {	ITEM_LIGHT,	          "light"		 },
    {	ITEM_SCROLL,	      "scroll"	     },
    {	ITEM_WAND,	          "wand"		 },
    {   ITEM_STAFF,	          "staff"		 },
    {   ITEM_WEAPON,	      "weapon"	     },
    {   ITEM_TREASURE,    	  "treasure"	 },
    {   ITEM_ARMOR,	          "armor"		 },
    {	ITEM_POTION,	      "potion"	     },
    {	ITEM_CLOTHING,	      "clothing"	 },
    {   ITEM_FURNITURE,	      "furniture"	 },
    {	ITEM_TRASH,	          "trash"		 },
    {	ITEM_CONTAINER,	      "container"	 },
    {	ITEM_DRINK_CON,       "drink"		 },
    {	ITEM_KEY,	          "key"		     },
    {	ITEM_FOOD,	          "food"		 },
    {	ITEM_MONEY,	          "money"		 },
    {	ITEM_BOAT,	          "boat"		 },
    {	ITEM_CORPSE_NPC,      "npc_corpse"	 },
    {	ITEM_CORPSE_PC,	      "pc_corpse"	 },
    {   ITEM_FOUNTAIN,	      "fountain"	 },
    {	ITEM_PILL,	          "pill"		 },
    {	ITEM_PROTECT,	      "protect"	     },
    {	ITEM_MAP,	          "map"		     },
    {	ITEM_PORTAL,	      "portal"	     },
    {	ITEM_WARP_STONE,      "warp_stone"	 },
    {	ITEM_ROOM_KEY,	      "room_key"	 },
    {	ITEM_GEM,	          "gem"		     },
    {	ITEM_JEWELRY,	      "jewelry"	     },
    {   ITEM_JUKEBOX,	      "jukebox"	     },
	{	ITEM_QUIVER,	      "quiver"	     },
	{	ITEM_ARROW,		      "arrow"	     },
	{   ITEM_PIPE_TABAC,      "pipe"         },
	{   ITEM_CHEW_TABAC,      "chew"         },
	{   ITEM_ANGREAL_MALE,    "maleangreal"  },
	{   ITEM_ANGREAL_FEMALE,  "femaleangreal"},
	{   ITEM_CLOTH_BOLT,	  "cloth_bolt"	 },
	{   ITEM_TREE_PC,         "treepc"       },
	{   ITEM_NOTEPAPER,       "notepaper"    },
	{   ITEM_TRIGGER,         "trigger"      },
	{   ITEM_TREASURECHEST,   "treasurechest"},
	{   ITEM_LOCKPICK,        "lockpick"     },
	{   ITEM_MIRROR,          "mirror"       },
	{   ITEM_RAZOR,           "razor"        },
	{   ITEM_BRUSH,           "brush"        },
	{   ITEM_FIREOIL,         "fireoil"      },
	{   ITEM_INK,			  "ink"			 },	/* Felix - Journal Code */
    {   ITEM_QUILL,			  "quill"		 },	/* Felix - Journal Code */
	{   ITEM_JOURNAL,		  "journal"		 },	/* Felix - Journal Code */
	{   ITEM_WIND_PIPES,	  "pipes"		 },
	{   ITEM_WIND_FLUTE,	  "flute"		 },
	{   ITEM_STRINGED_BITTERN,"bittern"		 },
	{   ITEM_PERCUSSION_DRUMS,"drums"		 },
	{   ITEM_SHAPE_KNIFE     ,"carver_knife" },
	{   ITEM_SHAPE_SHAPER    ,"wood_shaper"  },
	{   ITEM_SHAPE_STICK     ,"wood_stick"   },
	{   ITEM_SHAPE_LIMB      ,"wood_limb"    },
	{   ITEM_SHAPE_BRANCH    ,"wood_branch"  },
	{   ITEM_SHAPE_BLOCK     ,"wood_block"   },
	{   ITEM_SHAPE_KEYBLANK  ,"keyblank"     },
	{   ITEM_SMITH_ORE       ,"ore"          },
	{   ITEM_SMITH_HAMMER    ,"hammer"},
	{   ITEM_SMITH_ANVIL     ,"anvil"},
	{   ITEM_SMITH_BARREL    ,"barrel"},
	{   ITEM_SMITH_FURNACE   ,"furnace"},
	{   ITEM_SMITH_GRINDER   ,"grinder"},
	{   ITEM_COMM_DEV,        "commdev"},
	{   ITEM_WINDOW,          "window"},
	{   ITEM_KEYRING,         "keyring"},
	{   ITEM_SHAPE_MUSHROOM,  "mushroom"},
	{   ITEM_SHAPE_CLEMATIS,  "clematis"},
	{   ITEM_SHAPE_HAWTHORN,  "hawthorn"},
	{   ITEM_SHAPE_BONESET ,  "boneset"},
	{   ITEM_SHAPE_OBSIDION , "obsidion"},
	{   ITEM_SHAPE_SNAKESKIN ,"snakeskin"},
	{   ITEM_TOME,            "tome"},
	{   ITEM_CLOCK,           "clock"},
	{   ITEM_LAMPPOST,        "lamppost"},
	{   ITEM_ROPE,            "rope"},
	{   ITEM_PITON,           "piton"},
	{   ITEM_HERB,		  "herb"},
	{   ITEM_CRAFTING_MATERIAL,	"craft_material"},
	{   ITEM_CRAFTING_MANUAL,	"craft_manual"},
	{   ITEM_DYE,		  "dye"},
	{   ITEM_ADORNMENT,	  "adornment"},
    {   0,		NULL		}
};


/* weapon selection table */
const	struct	weapon_type	weapon_table	[]	=
{
   { "exotic",  OBJ_VNUM_SCHOOL_SWORD,   WEAPON_EXOTIC,  &gsn_sword, DAM_BASH },
   { "sword",	OBJ_VNUM_SCHOOL_SWORD,	WEAPON_SWORD,	&gsn_sword, DAM_SLASH	 },
   { "dagger",	OBJ_VNUM_SCHOOL_DAGGER,	WEAPON_DAGGER,	&gsn_dagger,DAM_PIERCE	 },
   { "spear",	OBJ_VNUM_SCHOOL_SPEAR,	WEAPON_SPEAR,	&gsn_spear, DAM_PIERCE	 },
   { "mace",	OBJ_VNUM_SCHOOL_MACE,	WEAPON_MACE,	&gsn_mace,  DAM_BASH 	 },
   { "axe",		OBJ_VNUM_SCHOOL_AXE,	WEAPON_AXE,		&gsn_axe,   DAM_SLASH	 },
   { "flail",	OBJ_VNUM_SCHOOL_FLAIL,	WEAPON_FLAIL,	&gsn_flail, DAM_BASH	 },
   { "whip",	OBJ_VNUM_SCHOOL_WHIP,	WEAPON_WHIP,	&gsn_whip,  DAM_SLASH	 },
   { "polearm",  	OBJ_VNUM_SCHOOL_POLEARM,WEAPON_POLEARM,	&gsn_polearm, DAM_PIERCE },
   { "shortbow",    OBJ_VNUM_SCHOOL_BOW,    WEAPON_SHORTBOW,&gsn_bow, DAM_PIERCE     },
   { "staff",	OBJ_VNUM_SCHOOL_STAFF,	WEAPON_STAFF,	&gsn_staff, DAM_BASH	 },
   { "longbow",     OBJ_VNUM_SCHOOL_BOW,     WEAPON_LONGBOW, &gsn_bow, DAM_PIERCE     },
   { "recurvedbow", OBJ_VNUM_SCHOOL_BOW, WEAPON_RECURVEDBOW,&gsn_bow, DAM_PIERCE     },
   { "compositebow",OBJ_VNUM_SCHOOL_BOW,WEAPON_COMPOSITEBOW,&gsn_bow, DAM_PIERCE     },
   { "crossbow",    OBJ_VNUM_SCHOOL_BOW,    WEAPON_CROSSBOW,&gsn_bow, DAM_PIERCE     },
   { "net",         OBJ_VNUM_SCHOOL_NET,        WEAPON_NET,        &gsn_net, DAM_PIERCE     },
   { "scythesword",  OBJ_VNUM_SCHOOL_SCYTHESWORD,WEAPON_SCYTHESWORD, &gsn_sword, DAM_SLASH },
   { "trident",      OBJ_VNUM_SCHOOL_TRIDENT,   WEAPON_TRIDENT,    &gsn_spear, DAM_PIERCE   },
   { "hammer",       OBJ_VNUM_SCHOOL_HAMMER,    WEAPON_HAMMER,     &gsn_hammer, DAM_BASH },
   { "sling",        OBJ_VNUM_SCHOOL_SLING,     WEAPON_SLING,      &gsn_sling, DAM_BASH  },
   { "swordbreaker", OBJ_VNUM_SCHOOL_SWORDBREAKER,WEAPON_SWORDBREAKER, &gsn_swordbreaker, DAM_PIERCE },
   { "sap",          OBJ_VNUM_SCHOOL_SAP,       WEAPON_SAP,        &gsn_sap, DAM_BASH },
   { "shortsword",   OBJ_VNUM_SCHOOL_SHORTSWORD, WEAPON_SHORTSWORD, &gsn_shortsword, DAM_PIERCE },
   { "lance",        OBJ_VNUM_SCHOOL_LANCE,     WEAPON_LANCE,      &gsn_lance, DAM_PIERCE },
   { "ashandarei",   OBJ_VNUM_SCHOOL_ASHANDAREI, WEAPON_ASHANDAREI, &gsn_ashandarei, DAM_SLASH },
   { "lightcrossbow",OBJ_VNUM_SCHOOL_BOW,WEAPON_LIGHTCROSSBOW, &gsn_bow, DAM_PIERCE },
   { NULL,	    0,				        0,	            NULL		 }
};



/* wiznet table and prototype for future flag setting */
const   struct wiznet_type      wiznet_table    []              =
{
   {    "on",       WIZ_ON,         IM },
   {    "prefix",	WIZ_PREFIX,	    IM },
   {    "ticks",    WIZ_TICKS,      IM },
   {    "logins",   WIZ_LOGINS,     IM },
   {    "sites",    WIZ_SITES,      L4 },
   {    "links",    WIZ_LINKS,      L7 },
   {	"newbies",	WIZ_NEWBIE,	    IM },
   {	"spam",		WIZ_SPAM,	    L5 },
   {    "deaths",   WIZ_DEATHS,     IM },
   {    "resets",   WIZ_RESETS,     L4 },
   {    "mobdeaths",WIZ_MOBDEATHS,  L4 },
   {    "flags",	WIZ_FLAGS,	    L5 },
   {	"penalties",WIZ_PENALTIES,	L5 },
   {	"saccing",	WIZ_SACCING,	L5 },
   {	"levels",	WIZ_LEVELS,	    IM },
   {	"load",		WIZ_LOAD,	    L2 },
   {	"restore",	WIZ_RESTORE,	L2 },
   {	"snoops",	WIZ_SNOOPS,	    L2 },
   {	"switches",	WIZ_SWITCHES,	L2 },
   {	"secure",	WIZ_SECURE,	    L1 },
   {    "memcheck",     WIZ_MEMCHECK,       L1 },
   {    "strings",  WIZ_STRING,    L1 },
   {    "invasions", WIZ_INVADE,  IM},
   {	"quests",	WIZ_QUESTS,	IM },
   {	"bigpawns",	WIZ_BPAWNS,	IM },
   {	"logstring",	WIZ_LOG_STRING,	IM },
   {	NULL,		0,		        0  }
};

/* attack table  -- not very organized :( */
const 	struct attack_type	attack_table	[MAX_DAMAGE_MESSAGE]	=
{
    { 	"none",		"hit",		-1		    },  /*  0 */
    {	"slice",	"slice", 	DAM_SLASH	},
    {   "stab",		"stab",		DAM_PIERCE	},
    {	"slash",	"slash",	DAM_SLASH	},
    {	"whip",		"whip",		DAM_SLASH	},
    {   "claw",		"claw",		DAM_SLASH	},  /*  5 */
    {	"blast",	"blast",	DAM_BASH	},
    {   "pound",	"pound",	DAM_BASH	},
    {	"crush",	"crush",	DAM_BASH	},
    {   "jab",      "jab",      DAM_PIERCE  },
    {	"bite",		"bite",		DAM_PIERCE	},  /* 10 */
    {   "pierce",	"pierce",	DAM_PIERCE	},
    {   "suction",	"suction",	DAM_BASH	},
    {	"beating",	"beating",	DAM_BASH	},
    {   "digestion","digestion",DAM_ACID	},
    {	"charge",	"charge",	DAM_BASH	},  /* 15 */
    { 	"slap",		"slap",		DAM_BASH	},
    {	"punch",	"punch",	DAM_BASH	},
    {	"wrath",	"wrath",	DAM_ENERGY	},
    {   "kick",     "kick",     DAM_BASH    },
    {   "weave",    "weave",    DAM_HOLY    },  /* 20 */
    {	"cleave",	"cleave",	DAM_SLASH	},
    {	"scratch",	"scratch",	DAM_PIERCE	},
    {   "peck",		"peck",		DAM_PIERCE	},
    {   "ram",      "ram",      DAM_BASH    },
    {   "chop",		"chop",		DAM_SLASH	},  /* 25 */
    {   "sting",	"sting",	DAM_PIERCE	},
    {   "smash",    "smash",    DAM_BASH    },
    {   "crunch",   "crunch",   DAM_BASH    },
    {   "sunder",   "sundering",DAM_SLASH   },
    {   "lance",    "lance",    DAM_PIERCE  },  /* 30 */
    {   "flog",     "flogging", DAM_SLASH   },
    {	"chomp",	"chomp",	DAM_PIERCE	},
    {   "pummel",   "pummel",   DAM_BASH    },
    {   "thrust",	"thrust",	DAM_PIERCE	},
    {   "lunge",    "lunge",    DAM_PIERCE  },
    {   "whap",     "whap",     DAM_BASH    },
    {   "thwack",   "thwack",   DAM_BASH    },
    {   "whack",    "whack",    DAM_BASH    },
    {   "headlock", "head lock",DAM_BASH    },
    {   "sleeper",  "sleeper",  DAM_BASH    }, /* 40 */
    {   "pressure", "pressure", DAM_BASH    },
    {   "uppercut", "upper cut",DAM_BASH    },
    {   "lefthook", "left hook",DAM_BASH    },
    {   "pinch",    "pinch",    DAM_BASH    },
    {   "righthook","right hook",DAM_BASH    },
    {   "chakram",  "chakram",  DAM_BASH    },
    {   "bat",      "bat",      DAM_BASH    },
    {   "tackle",   "tackle",   DAM_BASH    },
    {   "headbutt", "head butt",DAM_BASH    },
    {   "disease",  "disease",  DAM_BASH    }, /* 50 */
    {   "fury",     "fury",     DAM_ENERGY  },
    {	"flay",	    "flay",	DAM_SLASH   },
    {	"gash",	    "gash",	DAM_SLASH   },
    {	"thwock",   "thwock",	DAM_BASH    },
    {	"blow",     "blow",	DAM_BASH    }, /* 55 */
    {	"maul",     "maul",	DAM_BASH    },
    {   "lash",	    "lash",	DAM_SLASH   },
    {   NULL,		NULL,		0		}
};

/* race table */
const 	struct	race_type	race_table	[]		=
{
/*
    {
	name,		pc_race?,
	act bits,	aff_by bits,	off bits,
	imm,		res,		vuln,
	form,		parts
    },
*/
    { "unique",		FALSE, 0, 0, 0, 0, 0, 0, 0, 0 },

/*1*/    { "andor", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Dark-haired, dark eyed, of average height and weight from the midlands."
    },
/*2*/    { "amador", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Cautious, sober people, dark complexions, from the southern westlands."
    },
/*3*/    { "illian", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Southerners, above average height and build, moderate complexions."
    },
/*4*/    { "shienar", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Tall northern folk with dark hair and dark eyes."
    },
/*5*/    { "domani", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height, fair to coppery skin, from the north-westlands."
    },
/*6*/    { "aiel", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Tall, broad-shouldered, fair to red hair and tanned skin."
    },
/*7*/    { "ogier", TRUE, 0, AFF_DARK_VISION, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Tall non-humans, wide flat noses, drooping eyebrows and long tufted ears."
    },
/*8*/    { "tairen", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "South-eastern people, average height and build, fairly pale skin."
    },
/*9*/    { "cairhien", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|H|C|D|E|F|H|I|J|K,
      0,
      "Fair-skinned, shorter than average, dark hair and dark eyes."
    },
/*1*/    { "tuathaan", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|H|C|D|E|F|H|I|J|K,
      0,
      "People of many differet races, never fight even in defense."
    },
/*10*/    { "seanchan", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "From across the sea, average height and build with a lisping speech."
    },
/*11*/    { "altara", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Short to average height southerners, olive skin, brown or black hair"
    },
/*12*/    { "trolloc", TRUE, 0, AFF_DETECT_HIDDEN|AFF_INFRARED|AFF_REGENERATION, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K|R|S|T,
      0,
      "Tall, evil non-humans with animal faces and humanoid bodies."
    },
    { "arafel", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height northerners, pale skin, fair hair and large eyes."
    },
    { "tarabon", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height westerners, bulky builds and dark complexions."
    },
    { "kandor", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Tall northerners with light colored hair and eyes."
    },
    { "mayene", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height and build, fair skin, from the low southeast."
    },
    { "tworiver", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      101,
      "Fairly tall, dark hair and eyes of the western midland villages."
    },
    { "malkier", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      150,
      "Near extinct northern people, tall with dark hair and light eyes."
    },
    { "nym", TRUE, 0, AFF_REGENERATION|AFF_DARK_VISION, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      150,
      "Plant-like non-humans, tall and extremely reclusive."
    },
    { "grayman", TRUE, 0, AFF_SNEAK|AFF_HIDE, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      1000,
      "Very average humans who've given their souls to the Shadow."
    },
    { "ghealdan", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height and build people of the southwestern midlands."
    },
    { "murandy", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height and build, dark eyes and hair of the midlands."
    },
    { "saldaea", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Northern people, light colored titled eyes, dark or red hair."
    },
    { "tarvalon", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Northern midlanders, average height and build, fair complexions."
    },
    { "seafolk", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Sea-farers, dark hair and charcoal to chocolate skin color."
    },
	{"fade", TRUE,	0,
    AFF_DARK_VISION|AFF_SNEAK,
	0 , 0,	0,	0,
	A|H|M|V,		A|B|C|D|E|F|G|H||I|J,
    150,
      "Moderately tall non-humans with eyeless faces and dark hair."
    },
    { "falman", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height with blocky builds, from the coastal westlands."
    },
    { "maddinger", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Dark hair, dark eyes, average builds, of the southern midlands."
    },
    { "borderlander", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Tall northern folk with dark hair and dark eyes."
    },
    { "midlander", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Dark hair, dark eyes, average builds, of the midlands."
    },
    { "southlander", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Southerners, above average height and build, moderate complexions."
    },
    { "westlander", TRUE, 0, 0, 0, 0, 0, 0,
      A|H|M|V, A|B|C|D|E|F|G|H|I|J|K,
      0,
      "Average height, fair to coppery skin, from the westlands."
    },

	/* NPC Races */
    {
    "insect",		FALSE,
	0,		AFF_DARK_VISION,	0,
	0,		RES_PIERCE,	VULN_BASH,
 	A|B|G|O,		A|C|K,
    FALSE,
    },
    {
        "fish",                FALSE,
        0,              0,              OFF_FAST,
        0,              RES_POISON,     VULN_COLD,
        A|G|X|Y|cc,     A|D|E|F|K|L|Q|V|X,
    FALSE,
    },    
{
	"school monster",	FALSE,
	ACT_NOALIGN,		0,		0,
	IMM_CHARM|IMM_SUMMON,	0,		VULN_MAGIC,
	A|M|V,		A|B|C|D|E|F|H|J|K|Q|U,
    FALSE,
    },
    {
	"snake",		FALSE,
	0,		0,		OFF_FAST,
	0,		RES_POISON,	VULN_COLD,
	A|G|X|Y|cc,	A|D|E|F|K|L|Q|V|X,
    FALSE,
    },
    {
	"lizard",		FALSE,
	0,		0,		0,
	0,		RES_POISON,	0,
	A|G|X|Y|cc,	A|D|E|F|K|L|Q|V|X,
    FALSE,
    },
    {
	"bird",		FALSE,
	0,		AFF_FLYING,		OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|W,		A|C|D|E|F|H|K|P,
    FALSE,
    },
    {
	"trolloc_npc",		FALSE,
	0,		AFF_INFRARED|AFF_DETECT_HIDDEN,
	OFF_BERSERK|OFF_DODGE,
 	0,	RES_CHARM,	0,
	A|B|H|M|V,		A|B|C|D|E|F|G|H|I|J|K|R|S|T,
    FALSE,
    },
    {
	"wolf",			FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"dog",			FALSE,
	0,		AFF_DARK_VISION,	OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"horse",			FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_KICK,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"cat",			FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"greatcat",		FALSE,
	0,		AFF_DARK_VISION|AFF_DETECT_HIDDEN,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"fade",		FALSE,
	0,
    AFF_DETECT_HIDDEN|AFF_DETECT_MAGIC|AFF_SNEAK,
	OFF_FAST|OFF_DODGE|OFF_PARRY,
	IMM_POISON|IMM_CHARM|IMM_MENTAL|IMM_DISEASE,	0,	0,
	H|M|cc,		A|C|D|E|F|H|J,
    FALSE,
    },
    {
	"spider",	FALSE,
	0,		AFF_POISON,		OFF_FAST,
	0,		RES_POISON,	VULN_COLD,
	A|B|G|P,	A|C|D|E|F|K|V,
    FALSE,
    },
    {
	"bear",		FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE|OFF_CHARGE|OFF_BERSERK,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    /* seanchan critters */
    {
	"grolm",	FALSE,
	0,
	AFF_DARK_VISION|AFF_DETECT_HIDDEN|AFF_DETECT_INVIS|AFF_REGENERATION,
	OFF_FAST|OFF_DODGE|OFF_CHARGE|OFF_BERSERK,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"torm",	FALSE,
	0,
	AFF_DARK_VISION|AFF_HASTE,
	OFF_FAST|OFF_DODGE|OFF_CHARGE|OFF_BERSERK,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"corlm",	FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE|OFF_CHARGE|OFF_BERSERK,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"lopar",	FALSE,
	0,
	AFF_DARK_VISION|AFF_SWIM,
	OFF_FAST|OFF_DODGE|OFF_CHARGE|OFF_BERSERK,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V,
    FALSE,
    },
    {
	"raken",	FALSE,
	0,
	AFF_DARK_VISION|AFF_FLYING,
	OFF_FAST|OFF_DODGE|OFF_TAIL,
	0,		0,		0,
	A|G|V|W,		A|C|D|E|F|J|K|L|P|Q|U|V,
    FALSE,
    },
    {
	"to_raken",	FALSE,
	0,
	AFF_DARK_VISION|AFF_FLYING,
	OFF_FAST|OFF_DODGE|OFF_TAIL,
	0,		0,		0,
	A|G|V|W,		A|C|D|E|F|J|K|L|P|Q|U|V,
    FALSE,
    },
    {
	"construct",	FALSE,
	0,
	0,
	0,
	0,		0,		0,
	0, 0,
    FALSE,
    },
    {
	"unique",		FALSE,
	0,		0,		0,
	0,		0,		0,
	0,		0,
    FALSE,
    },


    {
	NULL, 0, 0, 0, 0, 0, 0
    }
};

const	struct	pc_race_type	pc_race_table	[]	=
{
    { "null race", "", 0, { 100, 100, 100, 100 },
      { "" }, { 13, 13, 13, 13, 13, 13, 13, 13 }, { 18, 18, 18, 18, 18, 18, 18, 18 }, 0 },

/*
    {
	"race name", 	short name, 	points,	{ class multipliers },
	{ bonus skills },
	{ base stats },		{ max stats },		size
    },
*/

    { "Andor", "Andoran", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 3001, FALSE
    },
    { "Amador", "Amadician", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 28151, FALSE
    },
    { "Illian", "Illianer", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 27929, FALSE
    },
    { "Shienar", "Shienaran", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 29006, FALSE
    },
    { "Domani", "Domani", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 19024, FALSE
    },
    { "Aiel", "Aiel", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 8797, TRUE
    },
    { "Ogier", "Ogier", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_LARGE, 1924, TRUE
    },
    { "Tairen", "Tairen", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 27850, FALSE
    },
    { "Cairhien", "Cairhienin", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 5306, FALSE
    },
    { "Tuathaan", "Tuatha'an", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 3001, TRUE
    },
    { "Seanchan", "Seanchan", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 31700, TRUE
    },
    { "Altara", "Altaran", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 8036, FALSE
    },
    { "Trolloc", "Trolloc", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 60, 40, 40, 40, 60, 50, 50, 50 },
      SIZE_LARGE, 3001, TRUE
    },
    { "Arafel", "Arafellin", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 60, 50, 50, 50, 50 },
      SIZE_MEDIUM, 26700, FALSE
    },
    { "Tarabon", "Taraboner", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 60, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 20132, FALSE
    },
    { "Kandor", "Kandori", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 55, 55, 55, 55, 50, 50, 50 },
      SIZE_MEDIUM, 6800, FALSE
    },
    { "Mayene", "Mayener", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 65, 55, 55, 55, 50, 50, 50, 50 },
      SIZE_MEDIUM, 28051, FALSE
    },
    { "Tworiver", "Two Rivers", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 55, 55, 55, 55, 55, 50, 50, 50 },
      SIZE_MEDIUM, 27700, FALSE
    },
    { "Malkier", "Malkieri", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 60, 50, 55, 50, 60, 50, 50, 50 },
      SIZE_MEDIUM, 29006, FALSE
    },
    { "Nym", "Nym", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 75, 50, 50, 50 },
      SIZE_HUGE, 3001, TRUE
    },
    { "Grayman", "Gray Man", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 75, 50, 50, 50, 50 },
      SIZE_MEDIUM, 3001, TRUE
    },
    { "Ghealdan", "Ghealdan", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 27951, FALSE
    },
    { "Murandy", "Murandian", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 1511, FALSE
    },
    { "Saldaea", "Saldaean", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 26900, FALSE
    },
    { "Tarvalon", "Tar Valon", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 7255, FALSE
    },
    { "Seafolk", "Sea Folk", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 13, 10, 13, 10, 10 }, { 50, 50, 50, 60, 50, 60, 50, 50 },
      SIZE_MEDIUM, 21000, TRUE
    },
    { "Fade", "Fade", 0, { 100, 100, 100, 100 }, { "" },
	      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 75, 50, 50, 50, 50 },
	      SIZE_MEDIUM, 3001, TRUE
    },
    { "Falman", "Falman", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 21251, FALSE
    },
    { "Maddinger", "Maddinger", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 13943, FALSE
    },
    { "Borderlander", "Borderlander", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 29006, FALSE
    },
    { "Midlander", "Midlander", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 3001, FALSE
    },
    { "Southlander", "Southlander", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 27929, FALSE
    },
    { "Westlander", "Westlander", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 3001, FALSE
    },
    { "", "", 0, { 100, 100, 100, 100 }, { "" },
      { 10, 10, 10, 10, 10, 10, 10, 10 }, { 50, 50, 50, 50, 50, 50, 50, 50 },
      SIZE_MEDIUM, 3001, FALSE
    }

};

/*
 * Class table.
 */
const	struct	class_type	class_table	[MAX_CLASS]	=
{
    {
	"merchant", "Mer",  STAT_INT,  OBJ_VNUM_SCHOOL_DAGGER,
	{ 3018, 9618 },  75,  20,  -5,  9, 15, TRUE,
	"mage basics", "mage default"
    },

    {
	"commoner", "Com",  STAT_WIS,  OBJ_VNUM_SCHOOL_DAGGER,
	{ 3003, 9619 },  75,  20,  -5,  9, 15, TRUE,
	"cleric basics", "cleric default"
    },

    {
	"rogue", "Rog",  STAT_DEX,  OBJ_VNUM_SCHOOL_DAGGER,
	{ 3028, 9639 },  75,  20,  -5,  10, 15, TRUE,
	"thief basics", "thief default"
    },

    {
	"warrior", "War",  STAT_STR,  OBJ_VNUM_SCHOOL_SWORD,
	{ 3022, 9633 },  75,  20,  -5,  11, 15, TRUE,
	"warrior basics", "warrior default"
    }
};



/*
 * Titles.
 */
char *	const			title_table	[MAX_CLASS][MAX_LEVEL+1][2] =
{
    {
	{ "Man",			"Woman"				},

        { "Young.",                     "Young."         },
    },

    {
	{ "Man",			"Woman"				},

        { "Young.",                     "Young."                    },
    },

    {
	{ "Man",			"Woman"				},

        { "Young.",                     "Young."                    },
    },

    {
	{ "Man",			"Woman"				},

        { "Young.",                     "Young."                    },
    }
};

/*
 * Liquid properties.
 * Used in world.obj.
 */
const	struct	liq_type	liq_table	[]	=
{
/*    name              color      proof,full thirst,food, ssize */
    { "water",			"clear",	{   0, 1, 10, 0, 16 }	},
    { "beer",			"amber",	{  12, 1,  8, 1, 12 }	},
    { "red wine", 		"dark",	    {  16, 1,  8, 1,  5 }	},
    { "ale",			"brown",	{  15, 1,  8, 1, 12 }	},
    { "dark ale",       "dark",     {  18, 1,  8, 1, 12 }   },
    { "whisky",			"golden",	{ 120, 1,  5, 0,  2 }	},
    { "pale ale",       "pale",     {  14, 1,  9, 2, 12 }   },
    { "lager",          "brown",    {  17, 0,  4, 0,  2 }   },
    { "XXX",            "clear",    { 140, 1,  3, 0,  2 }   },
    { "grog",           "green",    {  25, 1,  8, 1,  2 }   },
    { "warm cider",     "pale",     {   0, 1,  8, 1,  2 }   },
    { "milk",			"white",	{   0, 2,  9, 3, 12 }	},
    { "tea",			"tan",		{  -1, 1,  8, 0,  6 }	},
    { "kaf",            "black",    {  -2, 1,  8, 0,  6 }   },
    { "blood",			"red",		{   0, 2, -1, 2,  6 }	},
    { "salt water",		"clear",	{   0, 1, -2, 0,  1 }	},
    { "Illian wine",    "clear",    {  30, 2,  9, 2, 12 }   },
    { "Tairen wine",    "golden",   {  30, 2,  9, 2, 12 }   },
    { "Caemlyn wine",   "sparkling",{  30, 2,  8, 1,  5 }   },
    { "white wine",     "golden",   {  30, 1,  8, 1,  5 }   },
    { "champagne",		"golden",	{  32, 1,  8, 1,  5 }   },
    { "mead",           "honey",    {  40, 2,  8, 2, 12 }   },
    { "rose wine",		"pink",		{  26, 1,  8, 1,  5 }	},
    { "Tar Valon wine", "burgundy", {  40, 1,  8, 1,  5 }   },
    { "spirits",        "clear",    { 130, 1,  5, 0,  2 }   },
    { "berry juice",    "berry",    {   0, 1,  9, 2, 12 }   },
    { "orange juice",	"orange",	{   0, 2,  9, 3, 12 }   },
    { "oosquai",        "brown",    { 200, 1,  4, 0,  2 }   },
    { "brandy",			"golden",	{  80, 1,  5, 0,  4 }	},
    { "homebrew",       "brown",    {  30, 1,  5, 0,  2 }   },
    { "boilermaker",    "clear",    { 160, 1,  5, 0,  2 }   },
    { "icewine",		"purple",	{  50, 2,  6, 1,  5 }	},
    { "watered wine",   "burgundy", {  25, 2,  8, 1,  5 }   },
    { "sherry",			"red",		{  38, 2,  7, 1,  5 }   },
    { "honey mead",     "honey",    {  60, 1,  7, 1,  5 }   },
    { "rum",			"amber",	{ 151, 1,  4, 0,  2 }	},
    { "cordial",		"clear",	{ 100, 1,  5, 0,  2 }   },
    { NULL,			NULL,			{   0, 0,  0, 0,  0 }	}
};




const   struct  group_type      group_table     [MAX_GROUP]     =
{

    {
	"rom basics",		{ 0, 0, 0, 0 },
        { "dagger" }
    },

    {
	"mage basics",		{ 0, -1, -1, -1 },
        { "dagger" }
    },

    {
	"cleric basics",	{ -1, 0, -1, -1 },
        { "dagger" }
    },

    {
	"thief basics",		{ -1, -1, 0, -1 },
        { "dagger", "hide" }
    },

    {
	"warrior basics",	{ -1, -1, -1, 0 },
	{ "sword", "second attack" }
    },

    {
	"mage default",		{ 40, -1, -1, -1 },
        { "dagger" }
    },

    {
	"cleric default",	{ -1, 40, -1, -1 },
        { "dagger" }
    },

    {
	"thief default",	{ -1, -1, 40, -1 },
        { "dagger" }
    },

    {
	"warrior default",	{ -1, -1, -1, 40 },
        { "weaponsmaster" }
    },

    {
	"weaponsmaster",	{ 40, 40, 40, 20 },
        { "axe", "dagger", "flail", "mace", "polearm", "spear", "sword", "whip", "staff" }
    },




};
