/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "olc.h"

char *check_dye args((OBJ_DATA *obj, char *string, int dye_vnum));

bool check_vowell (const char *str)
{
	int pos = 0;
//	char buf[MSL];

	while (pos <= strlen(str))
	{
		if (str[pos] == '{')
		{
			pos += 2;
			continue;
		}

		if ( str[pos] == 'a' || str[pos] == 'e' || str[pos] == 'i' || str[pos] == 'o' || str[pos] == 'u' )
		{
//			sprintf(buf, "Checking vowell on %s. TRUE", str);
//			bug(buf, 0);
			return TRUE;
		}
		
		return FALSE;
	}

//	sprintf(buf, "Checking vowell on %s. FALSE", str);
//	bug(buf, 0);
	return FALSE;
}

void show_craft_steps (CHAR_DATA *ch, OBJ_DATA *material, OBJ_DATA *manual, OBJ_DATA *obj)
{
	char buf[MSL];

	switch (manual->value[0])
	{
		default:
		WAIT_STATE(ch, PULSE_EVENT * 24);
		printf_to_char(ch, "You craft %s out of %s and hold it up proudly.\n\r",
		check_dye(obj, "", obj->dye_vnum), material->short_descr);
		act("$n crafts $p out of $P and holds it up proudly.",ch,obj,material,TO_ROOM);
		break;

		case CRAFT_SEW:
		act("$n begins rolling out a length of $t.",
		ch,check_dye(NULL, material->pIndexData->short_descr, material->dye_vnum),0,TO_ROOM);
		act("You begin rolling out a length of $t.",
		ch,check_dye(NULL, material->pIndexData->short_descr, material->dye_vnum),0,TO_CHAR);
		WAIT_STATE(ch, PULSE_EVENT * 24);

		wait_printf(ch,7,"You carefully sketch out the outline of %s %s%s on the %s.\n\r",
		manual->value[3] != 0 ? "a" : check_vowell(obj->pIndexData->short_descr) ? "an" : "a",
		manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
		obj->pIndexData->short_descr, check_dye(NULL, material->pIndexData->short_descr, material->dye_vnum));
		sprintf(buf, "$n carefully sketches out the outline of %s %s%s on the %s.",
		manual->value[3] != 0 ? "a" : check_vowell(obj->pIndexData->short_descr) ? "an" : "a",
		manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
		obj->pIndexData->short_descr, check_dye(NULL, material->pIndexData->short_descr, material->dye_vnum));
		wait_act(7, buf,ch,NULL,0,TO_ROOM);

		wait_act(14, "You carefully cut out your pattern and lay it out to be stitched.",ch,NULL,0,TO_CHAR);
		wait_act(14, "$n carefully cuts out $s pattern and lays it out to be stitched.",ch,NULL,0,TO_ROOM);

		wait_printf(ch, 21,"You carefully stitch %s together and hold it up proudly.\n\r",
		check_dye(obj, "", obj->dye_vnum));
		sprintf(buf, "$n carefully stitches %s together and holds it up proudly.",
		check_dye(obj, "", obj->dye_vnum));
		wait_act(21, buf,ch,NULL,0,TO_ROOM);
		break;

		case CRAFT_FORGE:
		act("$n starts a fire and works the bellows until the flames burn extremely hot.",ch,NULL,0,TO_ROOM);
		act("You start a fire and work the bellows until the flames burn extremely hot.",ch,NULL,0,TO_CHAR);
		WAIT_STATE(ch, PULSE_EVENT * 24);

		wait_act(7, "You heat up your $t until it glows red hot and begin hammering it on your anvil.",
		ch,material->short_descr,0,TO_CHAR);
		wait_act(7, "$n heats up $s $t until it glows red hot and begins hammering it on $s anvil.",
		ch,material->short_descr,0,TO_ROOM);

		wait_printf(ch,14,"You work expertly forming the %s into a %s%s.\n\r",
		material->short_descr, manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
		obj->pIndexData->short_descr);

		sprintf(buf, "$n works expertly forming the %s into a %s%s.",
		material->short_descr, manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
		obj->pIndexData->short_descr);
		wait_act(14,buf,ch,NULL,0,TO_ROOM);

		wait_act(21,"You quickly cool $t down and hold it up proudly.",ch,obj->short_descr,0,TO_CHAR);
		wait_act(21,"$n quickly cools $t down and holds it up proudly.",ch,obj->short_descr,0,TO_ROOM);
		break;

		case CRAFT_JEWELCRAFT:
		act("You examine $t, searching carefully for where to cut it.",
		ch,check_dye(material, "", material->dye_vnum),0,TO_CHAR);
		act("$n examines $t, searching carefully for where to cut it.",
		ch,check_dye(material, "", material->dye_vnum),0,TO_ROOM);
		WAIT_STATE(ch, PULSE_EVENT * 24);

		wait_act(8, "You cut $p with precision to form the desired shape.", ch, material, 0,TO_CHAR);
		wait_act(8, "$n cuts $p with precision to form the desired shape.", ch, material, 0,TO_ROOM);

/*
		wait_printf(ch, 14,"You inset %s, forming %s %s%s.\n\r",
		check_dye(material, "", material->dye_vnum),
		manual->value[3] != 0 ? "a" : check_vowell(obj->pIndexData->short_descr) ? "an" : "a",
		manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
		obj->pIndexData->short_descr);
		sprintf(buf, "$n insets $t, forming %s %s%s.",
		manual->value[3] != 0 ? "a" : check_vowell(obj->pIndexData->short_descr) ? "an" : "a",
		manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
		obj->pIndexData->short_descr);
		wait_act(14, buf,
		ch,check_dye(material, "", material->dye_vnum),0,TO_ROOM);
*/
		wait_act(16,"You polish $p and admire your handy work.", ch, obj, 0,TO_CHAR);
		wait_act(16, "$n polishes $p and admires $s handy work.", ch, obj, 0,TO_ROOM);


		break;

		case CRAFT_WOODCRAFT:
		WAIT_STATE(ch, PULSE_EVENT * 24);
		act("You carve $t out of your wood and hold it up proudly.",ch,obj->short_descr,0,TO_CHAR);
		act("$n carves $t out of wood and holds it up proudly.",ch,obj->short_descr,0,TO_ROOM);
		break;
	}
	return;
}

int craft_skill (int craft)
{
	if (craft == CRAFT_SEW)			return gsn_sew;
	if (craft == CRAFT_FORGE)		return gsn_forge;
	if (craft == CRAFT_WOODCRAFT)		return gsn_woodcraft;
	if (craft == CRAFT_JEWELCRAFT)		return gsn_jewelcraft;

	return -1;
}

int get_craft_skill (CHAR_DATA *ch, int craft)
{
	int skill;

	if (craft_skill(craft) > 0)
		skill = get_skill(ch, craft_skill(craft));
	else	skill = URANGE(0, ch->level / 3, 250);

	return skill;
}

void do_craft (CHAR_DATA *ch, char *argument)
{
	char arg1[MIL], arg2[MIL], arg3[MIL];
	char buf[MSL], buf2[MSL];
	OBJ_DATA *material, *manual, *obj;
	AFFECT_DATA *af, *af2 = NULL;
	int mod;
	char design[MSL];

	design[0] = '\0';

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
	argument = one_argument(argument, arg3);

	if (arg1[0] == '\0' || arg2[0] == '\0' || str_cmp(arg2, "into") || arg3[0] == '\0')
	{
		stc("Syntax: craft <material> into <item>\n\r", ch);
		return;
	}

	if ((material = get_obj_carry(ch, arg1, ch)) == NULL
	|| material->item_type != ITEM_CRAFTING_MATERIAL)
	{
		stc("You can't use that item for crafting anything.\n\r", ch);
		return;
	}

	if ((manual = get_obj_carry(ch, arg3, ch)) == NULL
	|| manual->item_type != ITEM_CRAFTING_MANUAL)
	{
		stc("That's not an item you can craft.\n\r", ch);
		return;
	}

	if (material->value[0] != manual->value[0])
	{
		stc("You can't craft that item with that material.\n\r", ch);
		return;
	}

	if (get_obj_index(manual->value[1]) == NULL)
	{
		stc("This manual doesn't seem to make anything.\n\r", ch);
		return;
	}

	if (UMIN(get_skill(ch, craft_skill(manual->value[0])), 100) < number_percent())
	{
		check_improve(ch, craft_skill(manual->value[0]), FALSE, 3);
		stc("You can't seem to figure out the manual.\n\r", ch);
		WAIT_STATE(ch, 18);
		return;
	}

	if (get_obj_index(manual->value[1])->item_type == ITEM_ADORNMENT)
	{
		if (argument[0] == '\0')
		{
			stc(" You can make the following:\n\r", ch);
			stc(" gem, star, moon, flower, monkey, wolf, dragon, tiger, fox, bear\n\r", ch);
			stc("What kind of design do you want to make?  [craft <material> into <adornment> <design>]\n\r", ch);
			return;
		}

		if (IS_IMMORTAL(ch)
		|| !str_cmp("gem", argument)
		|| !str_cmp("star", argument)
		|| !str_cmp("moon", argument)
		|| !str_cmp("flower", argument)
		|| !str_cmp("monkey", argument)
		|| !str_cmp("wolf", argument)
		|| !str_cmp("dragon", argument)
		|| !str_cmp("tiger", argument)
		|| !str_cmp("fox", argument)
		|| !str_cmp("bear", argument))
			sprintf(design, "%s", argument);
		else
		{
			stc(" You can make the following:\n\r", ch);
			stc(" gem, star, moon, flower, monkey, wolf, dragon, tiger, fox, bear\n\r", ch);
			stc("What kind of design do you want to make?  [craft <material> into <adornment> <design>]\n\r", ch);
			return;
		}
	}

	obj = create_object(get_obj_index(manual->value[1]), 0);
	obj->material = material->material;

	if (get_obj_index(material->dye_vnum) == NULL)
	obj->dye_vnum = material->value[2];
	else obj->dye_vnum = material->dye_vnum;

	sprintf(buf2, "%s%s %s{x",
	manual->value[3] != 0 ? flag_string(craftpadj_flags, manual->value[3]) : "",
	check_dye(NULL, material->short_descr, material->dye_vnum), obj->short_descr);

	if (obj->item_type == ITEM_ADORNMENT)
	{
		sprintf(buf, "%s%s %s{x", manual->value[3] != 0 ? flag_string(craftpadj_flags,
		manual->value[3]) : "", check_dye(NULL, material->short_descr, material->dye_vnum), design);
	}
	else if (check_vowell(buf2))	sprintf(buf, "an %s", buf2);
	else				sprintf(buf, "a %s", buf2);

	free_string(obj->short_descr);
	obj->short_descr = str_dup(buf);

	sprintf(buf, "%s %s %s", material->name, obj->name, obj->item_type == ITEM_ADORNMENT ? design : "");
	free_string(obj->name);
	obj->name = str_dup(buf);
	
	if (check_vowell(buf))
		sprintf(buf, "An %s %s lying here on the ground.", buf2,
		manual->value[3] == 0 ? "is" : "are");
	else
		sprintf(buf, "A %s %s lying here on the ground.", buf2,
		manual->value[3] == 0 ? "is" : "are");

	free_string(obj->description);
	obj->description = str_dup(buf);
	SET_BIT(obj->extra_flags, ITEM_CRAFTED);

	show_craft_steps(ch, material, manual, obj);
	check_improve(ch, craft_skill(manual->value[0]), TRUE, 6);

	if (obj->item_type == ITEM_WEAPON)
	{
		obj->value[2] += material->value[3] * 5 + number_range(1, UMIN(get_craft_skill(ch, manual->value[0]) / 10, 10));
		if (obj->value[2] > 75) obj->value[2] = 75;
	}

	for (af2 = manual->affected; af2 != NULL; af2 = af2->next)
	{
		mod = 5 + number_range(1, UMAX(get_craft_skill(ch, manual->value[0]) / 10, 10));
		mod += UMIN(af2->modifier, 5) * 2;

		mod *= 1000;
		mod /= 5;
		mod *= material->value[3];
		mod /= 1000;

		mod += ch->level / 200;
		mod = URANGE(1, mod, 25);

		mod += get_craft_skill(ch, manual->value[0]) / 20;

		if (mod > 0)
		{
			af = new_affect();
			af->modifier		= URANGE(5, mod, 30);
			af->location		= af2->location;
			af->where		= TO_OBJECT;
			af->type		= -1;
			af->duration		= -1;
			af->bitvector		= 0;
			af->level		= 1;
			af->next		= obj->affected;
			obj->affected	= af;
		}
	}

	for (af2 = manual->pIndexData->affected; af2 != NULL; af2 = af2->next)
	{
		mod = 5 + number_range(1, UMAX(get_craft_skill(ch, manual->value[0]) / 10, 10));
		mod += UMIN(af2->modifier, 5) * 2;

		mod *= 1000;
		mod /= 5;
		mod *= material->value[3];
		mod /= 1000;

		mod += ch->level / 200;
		mod = URANGE(1, mod, 25);

		mod += get_craft_skill(ch, manual->value[0]) / 20;

		if (mod > 0)
		{
			af = new_affect();
			af->modifier		= URANGE(5, mod, 30);
			af->location		= af2->location;
			af->where		= TO_OBJECT;
			af->type		= -1;
			af->duration		= -1;
			af->bitvector		= 0;
			af->level		= 1;
			af->next		= obj->affected;
			obj->affected	= af;
		}
	}


	obj_to_char(obj, ch);
	extract_obj(material);
	return;
}

/* determines whether or not an item is dyeable */
bool check_add_dye(OBJ_DATA *material)
{
	if (!str_infix("{1", material->short_descr))
		return TRUE;

	if (!str_infix("{2", material->short_descr))
		return TRUE;

	if (!str_infix("{3", material->short_descr))
		return TRUE;

	if (!str_infix("{4", material->short_descr))
		return TRUE;

	return FALSE;
}

void do_dye (CHAR_DATA *ch, char *argument)
{
	char arg1[MIL], arg2[MIL], arg3[MIL];
	OBJ_DATA *material, *dye;
	char buf[MSL];

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
	argument = one_argument(argument, arg3);

	if (arg1[0] == '\0' || arg2[0] == '\0' || str_cmp(arg2, "with") || arg3[0] == '\0')
	{
		stc("Syntax: dye <material> with <dye>\n\r", ch);
		return;
	}

	if ((material = get_obj_carry(ch, arg1, ch)) == NULL)
	{
		stc("What are you trying to dye?\n\r", ch);
		return;
	}

	if ((dye = get_obj_carry(ch, arg3, ch)) == NULL
	|| dye->item_type != ITEM_DYE)
	{
		stc("That's not a dye.\n\r", ch);
		return;
	}

	if (!check_add_dye(material))
	{
		stc("You can't dye that item.\n\r", ch);
		return;
	}

	material->dye_vnum = dye->pIndexData->vnum;
	ptc(ch, "You dye %s using a vial of %s {xdye, creating %s.\n\r",
	material->short_descr, dye->short_descr, check_dye(material, "", material->dye_vnum));

	sprintf(buf, "$n dyes %s using a vial of %s {xdye, creating %s.\n\r",
	material->short_descr, dye->short_descr, check_dye(material, "", material->dye_vnum));
	act(buf, ch, NULL, NULL, TO_ROOM);

	extract_obj(dye);
	return;
}

bool is_wood2( OBJ_DATA *pObj )
{
	if ( !pObj )
	return FALSE;
	if ( !str_cmp( pObj->material, "pine" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "leatherleaf" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "rosewood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "satinwood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "hemlock" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "poplar" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "fir" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "beech" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "spruce" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "cypress" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "juniper" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "walnut" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "cedar" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "maple" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "mahogany" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "teak" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "ebony" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "redwood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "willow" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "oak" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "yew" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "soft wood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "hard wood" ) )
	return TRUE;

	return FALSE;
}

bool is_metal( OBJ_DATA *pObj )
{
	if ( !pObj )
	return FALSE;
	if ( !str_cmp( pObj->material, "precious metal" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "soft metal" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "bronze" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "copper" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "lead" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "silver" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "nickel" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "gold" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "iron" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "steel" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "platinum" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "hard metal" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "black steel" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "powerwrought steel" ) )	return TRUE;

	return FALSE;
}

bool is_leather( OBJ_DATA *pObj )
{
	if ( !pObj )
	return FALSE;
	if ( !str_cmp( pObj->material, "fur" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "rabbit skin" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "soft leather" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "boiled leather" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "cured leather" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "hard leather" ) )	return TRUE;
	return FALSE;
}

bool is_cloth( OBJ_DATA *pObj )
{
	if ( !pObj )
	return FALSE;
	if (is_leather(pObj))	return TRUE;
	if ( !str_cmp( pObj->material, "silk" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "cloth" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "algode" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "burlap" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "wool" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "linen" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "velvet" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "lace" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "hemp" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "fancloth" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "silver cloth" ) )	return TRUE;
	if ( !str_cmp( pObj->material, "gold cloth" ) )	return TRUE;

	return FALSE;
}
























void do_adorn (CHAR_DATA *ch, char *argument)
{
	char arg1[MIL], arg2[MIL], arg3[MIL];
	char buf[MSL], buf2[MSL];
	OBJ_DATA *obj, *adorn;
	bool can_adorn = FALSE;

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
	argument = one_argument(argument, arg3);

	if (arg1[0] == '\0' || arg2[0] == '\0' || str_cmp(arg2, "with") || arg3[0] == '\0')
	{
		stc("Syntax: adorn <item> with <adornment>\n\r", ch);
		return;
	}

	if ((obj = get_obj_carry(ch, arg1, ch)) == NULL)
	{
		stc("What are you trying to adorn?\n\r", ch);
		return;
	}

	if (IS_SET(obj->extra_flags, ITEM_ADORNED))
	{
		stc("That object already has an adornment on it!\n\r", ch);
		return;
	}

	if ((adorn = get_obj_carry(ch, arg3, ch)) == NULL
	|| adorn->item_type != ITEM_ADORNMENT)
	{
		stc("That's not an adornment.\n\r", ch);
		return;
	}

	if (!str_infix(craftadorn_table[adorn->value[0]].allowed, "wood") && is_wood2(obj))
		can_adorn = TRUE;

	if (!str_infix(craftadorn_table[adorn->value[0]].allowed, "metal") && is_metal(obj))
		can_adorn = TRUE;

	if (!str_infix(craftadorn_table[adorn->value[0]].allowed, "leather") && is_leather(obj))
		can_adorn = TRUE;

	if (!str_infix(craftadorn_table[adorn->value[0]].allowed, "cloth") && is_cloth(obj))
		can_adorn = TRUE;

	if (!str_infix(craftadorn_table[adorn->value[0]].allowed, "belts")
	&& obj->item_type == ITEM_ARMOR && CAN_WEAR(obj, ITEM_WEAR_WAIST))
		can_adorn = TRUE;

	if (!str_infix(craftadorn_table[adorn->value[0]].allowed, "any"))
		can_adorn = TRUE;

	if (!IS_SET(obj->extra_flags, ITEM_CRAFTED))
		can_adorn = FALSE;

	if (!can_adorn)
	{
		stc("You can't adorn this object with that adornment.\n\r", ch);
		stc("Appraise the adornment to see what you can use it on.\n\r", ch);
		stc("Only crafted items may be adorned.\n\r", ch);
		return;
	}

	if (UMIN(get_skill(ch, gsn_adorn), 100) < number_percent())
	{
		check_improve(ch, gsn_adorn, FALSE, 3);
		stc("You can't seem to figure out how to attach it.\n\r", ch);
		WAIT_STATE(ch, 12);
		return;
	}

	act ("You adorn $p with $P.", ch, obj, adorn, TO_CHAR);
	act ("$n adorns $p with $P.", ch, obj, adorn, TO_ROOM);

	sprintf(buf, "%s %s", obj->name, adorn->name);
	free_string(obj->name);
	obj->name = str_dup(buf);

	sprintf(buf, "{1%s %s", check_vowell(adorn->short_descr) ? "an" : "a",
	check_dye(NULL, adorn->short_descr, adorn->dye_vnum));

	sprintf(buf2, craftadorn_table[adorn->value[0]].suffix, buf);
	sprintf(buf, "{1%s %s{x", check_dye(obj, "", obj->dye_vnum), buf2);

	free_string(obj->short_descr);
	obj->short_descr = str_dup(buf);

	if (!str_infix(obj->short_descr, "pair of") || !str_infix(obj->short_descr, "set of"))
		sprintf(buf2, "%s are lying here.", buf);
	else	sprintf(buf2, "%s is lying here.", buf);
	free_string(obj->description);
	obj->description = str_dup(buf2);

	SET_BIT(obj->extra_flags, ITEM_ADORNED);
	obj->dye_vnum = 0;

	extract_obj(adorn);
	check_improve(ch, gsn_adorn, TRUE, 6);
	WAIT_STATE(ch, 12);
	return;
}


/*
#define ADORNRAND_NONE		0
#define ADORNRAND_ANY		1
#define ADORNRAND_FLOWER	2
#define ADORNRAND_TREASURE	3
#define ADORNRAND_GEM		4
#define ADORNRAND_BEAST		5
#define ADORNRAND_BIRD		6
*/


const struct randadorn_type adorn_prefix_table [] =
{
	{	"none",			"none",			ADORNRAND_NONE,		0	},
	{	"silver",		"{1s{2i{3lv{2e{1r",	ADORNRAND_ANY,		36712	},
	{	"golden",		"{1g{2olde{1n",		ADORNRAND_ANY,		36713	},
	{	"red",			"{1r{2e{1d",		ADORNRAND_ANY,		36700	},
	{	"blue",			"{1b{2lu{1e",		ADORNRAND_ANY,		36701	},
	{	"green",		"{1g{2ree{1n",		ADORNRAND_ANY,		36703	},
	{	"white",		"{1w{2hit{1e",		ADORNRAND_ANY,		36702	},
	{	"yellow",		"{1y{2ello{1w",		ADORNRAND_ANY,		36704	},
	{	"black",		"{1b{2lac{1k",		ADORNRAND_ANY,		36707	},
	{	"blooming",		"{1b{2lo{3om{2in{1g",	ADORNRAND_FLOWER,	0	},
	{	"beautiful",		"{1b{2ea{3uti{2fu{1l",	ADORNRAND_FLOWER,	0	},
	{	"wilting",		"{1w{2il{3t{2in{1g",	ADORNRAND_FLOWER,	0	},
	{	"growling",		"{1g{2ro{3wl{2in{1g",	ADORNRAND_BEAST,	0	},
	{	"howling",		"{1h{2ow{3l{2in{1g",	ADORNRAND_BEAST,	0	},
	{	"sleeping",		"{1s{2le{3ep{2in{1g",	ADORNRAND_BEAST,	0	},
	{	"snarling",		"{1s{2na{3rl{2in{1g",	ADORNRAND_BEAST,	0	},
	{	"flying",		"{1f{2l{3yi{2n{1g",	ADORNRAND_BIRD,		0	},
	{	"soaring",		"{1s{2oa{3r{2in{1g",	ADORNRAND_BIRD,		0	},
	{	"diving",		"{1d{2i{3vi{2n{1g",	ADORNRAND_BIRD,		0	},
};

const struct randadorn_type adorn_suffix_table [] =
{
	{	"none",			"none",				ADORNRAND_NONE,		0	},
	{	"flower",		"{1f{2l{3ow{2e{1r",		ADORNRAND_FLOWER,	0	},
	{	"rose",			"{1r{2os{1e",			ADORNRAND_FLOWER,	36700	},
	{	"daisy",		"{1d{2ais{1y",			ADORNRAND_FLOWER,	36704	},
	{	"fox",			"{1f{2o{1x",			ADORNRAND_BEAST,	0	},
	{	"lion",			"{1l{2io{1n",			ADORNRAND_BEAST,	0	},
	{	"wolf",			"{1w{2ol{1f",			ADORNRAND_BEAST,	0	},
	{	"tiger",		"{1t{2i{3g{2e{1r",		ADORNRAND_BEAST,	0	},
	{	"bear",			"{1b{2ea{1r",			ADORNRAND_BEAST,	0	},
	{	"boar",			"{1b{2oa{1r",			ADORNRAND_BEAST,	0	},
	{	"grolm",		"{1g{2r{3o{2l{1m",		ADORNRAND_BEAST,	0	},
	{	"dragon",		"{1d{2r{3ag{2o{1n",		ADORNRAND_BEAST,	0	},
	{	"heron",		"{1h{2e{3r{2o{1n",		ADORNRAND_BIRD,		0	},
	{	"eagle",		"{1e{2a{3g{2l{1e",		ADORNRAND_BIRD,		0	},
	{	"hawk",			"{1h{2aw{1k",			ADORNRAND_BIRD,		0	},
	{	"raven",		"{1r{2a{3v{2e{1n",		ADORNRAND_BIRD,		0	},
	{	"raken",		"{1r{2a{3k{2e{1n",		ADORNRAND_BIRD,		0	},
	{	"to'raken",		"{1t{2o'{3ra{2ke{1n",		ADORNRAND_BIRD,		0	},
	{	"dragon",		"{1d{2r{3ag{2o{1n",		ADORNRAND_BIRD,		0	},
	{	"crow",			"{1c{2ro{1w",			ADORNRAND_BIRD,		0	},
};

OBJ_DATA *random_adornment ()
{
	int type = number_range(1, 23);
	OBJ_DATA *obj = create_object(get_obj_index(36950), 1);
	int pre, suf, dye = 0;
	char buf[MSL];
	int pre_size = sizeof(adorn_prefix_table) / sizeof(struct randadorn_type) - 1;
	int suf_size = sizeof(adorn_suffix_table) / sizeof(struct randadorn_type) - 1;

	suf = number_range(1, suf_size);
	pre = 0;

	while  (!str_cmp(craftadorn_table[type].name, "carving")
	||	!str_cmp(craftadorn_table[type].name, "brand")
	||	!str_cmp(craftadorn_table[type].name, "inscription")
	||	!str_cmp(craftadorn_table[type].name, "engraving")
	||	!str_cmp(craftadorn_table[type].name, "inset")
	||	!str_cmp(craftadorn_table[type].name, "etching")
	||	!str_cmp(craftadorn_table[type].name, "fashioning")
	||	!str_cmp(craftadorn_table[type].name, "wrap")
	||	!str_cmp(craftadorn_table[type].name, "affixment")
	||	!str_cmp(craftadorn_table[type].name, "tooling")
	||	!str_cmp(craftadorn_table[type].name, "binding")
	||	!str_cmp(craftadorn_table[type].name, "bearing"))
		type = number_range(1, 23);

	while (adorn_prefix_table[pre].type != adorn_suffix_table[suf].type && adorn_prefix_table[pre].type != ADORNRAND_ANY)
		pre = number_range(1, pre_size);

	if (adorn_suffix_table[suf].dye > 0)	dye = adorn_suffix_table[suf].dye;
	if (adorn_prefix_table[pre].dye > 0)	dye = adorn_prefix_table[pre].dye;

	if (dye == 0)	dye = 36702;

	obj->dye_vnum = dye;
	obj->value[0] = type;

	sprintf(buf, "%s %s %s adornment{x", adorn_prefix_table[pre].name, adorn_suffix_table[suf].name, craftadorn_table[type].name);
	free_string(obj->name);
	obj->name = str_dup(buf);	
	
	sprintf(buf, "%s %s{x", adorn_prefix_table[pre].color, adorn_suffix_table[suf].color);
	free_string(obj->short_descr);
	obj->short_descr = str_dup(buf);

	return obj;
}

bool can_enhance(OBJ_DATA *obj)
{
	AFFECT_DATA *af;

	for (af = obj->affected; af != NULL; af = af->next)
	{
		if (af->location == APPLY_DAMROLL)	return FALSE;
		if (af->location == APPLY_HITROLL)	return FALSE;
	}

	for (af = obj->pIndexData->affected; af != NULL; af = af->next)
	{
		if (af->location == APPLY_DAMROLL)	return FALSE;
		if (af->location == APPLY_HITROLL)	return FALSE;
	}

	return TRUE;
}

OBJ_DATA *get_enhance_token (CHAR_DATA *ch)
{
	OBJ_DATA *token;

	for (token = ch->carrying; token != NULL; token = token->next_content)
	{
		if (token->pIndexData->vnum == 9)	break;
		if (token->next_content == NULL)	return NULL;
	}

	return token;
}

void do_enhance (CHAR_DATA *ch, char *argument)
{
	OBJ_DATA *upgrade, *token;
	char arg[MSL];
	char buf[MSL];
	AFFECT_DATA *af;
	int bit;

	argument = one_argument(argument, arg);

	if (arg[0] == '\0')
	{
		stc("Syntax: enhance <item>\n\r", ch);
		return;
	}

        if ( ( upgrade = get_obj_carry( ch, arg, ch ) ) == NULL )
        {
        	stc( "You must supply a valid item to enhance.\n\r", ch );
                return;
        }

	if ((token = get_enhance_token(ch)) == NULL)
	{
        	stc( "You must have an enhancement token to use this skill.\n\r", ch );
                return;
	}

	bit = upgrade->wear_flags;
	REMOVE_BIT(bit, ITEM_TAKE);
	REMOVE_BIT(bit, ITEM_HOLD);

	if (bit <= 0)
        {
                stc( "This item cannot be enhanced.\n\r", ch );
        	return;
        }

        if (!can_enhance(upgrade))
        {
                stc( "This item cannot be enhanced any further.\n\r", ch );
        	return;
        }
/*
	if (token->owner != NULL && strcmp(token->owner, ch->name))
	{
        	stc( "You must own the enhancement token that you're trying to use.\n\r", ch );
		return;
	}
*/
	sprintf(buf, "You use $p to strengthen the power of %s.", upgrade->short_descr);
	act(buf, ch, token, NULL, TO_CHAR);
	sprintf(buf, "$n uses $p to strengthen the power of %s.", upgrade->short_descr);
	act(buf, ch, token, NULL, TO_ROOM);

        upgrade->condition = 100;

	af = new_affect();
	af->modifier		= 20;
	af->location		= APPLY_HITROLL;
	af->where		= TO_OBJECT;
	af->type		= -1;
	af->duration		= -1;
	af->bitvector		= 0;
	af->level		= 1;
	af->next		= upgrade->affected;
	upgrade->affected	= af;

	af = new_affect();
	af->modifier		= 20;
	af->location		= APPLY_DAMROLL;
	af->where		= TO_OBJECT;
	af->type		= -1;
	af->duration		= -1;
	af->bitvector		= 0;
	af->level		= 1;
	af->next		= upgrade->affected;
	upgrade->affected	= af;

	extract_obj(token);
	return;
}

void do_reinforce (CHAR_DATA *ch, char *argument)
{
	OBJ_DATA *upgrade, *metal;
	AFFECT_DATA *af;
	char arg1[MIL], arg2[MIL], arg3[MIL];
	int mod;

	if (IS_NPC(ch))	return;

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
	argument = one_argument(argument, arg3);

	if (arg1[0] == '\0' || arg2[0] == '\0' || str_cmp(arg2, "with") || arg3[0] == '\0')
	{
		stc("Syntax: reinforce <armor> with <metal>\n\r", ch);
		return;
	}

        if ((upgrade = get_obj_carry(ch, arg1, ch)) == NULL)
        {
        	stc( "You must supply valid armor to reinforce.\n\r", ch );
                return;
        }

	if (upgrade->item_type != ITEM_ARMOR)
        {
        	stc( "You must supply valid armor to reinforce.\n\r", ch );
                return;
        }

	if ((metal = get_obj_carry(ch, arg3, ch)) == NULL
	|| metal->item_type != ITEM_CRAFTING_MATERIAL
	|| metal->value[0] != CRAFT_FORGE)
        {
        	stc( "You must use a metal ore to reinforce your armor.\n\r", ch );
                return;
        }

	mod = metal->value[3] * 100;

	if (!IS_NPC(ch))
	{
		mod *= ch->pcdata->learned[gsn_reinforce];
		mod /= 100;
	}

	if (mod > 500)	mod = 500;

	if ((af = affect_find(upgrade->affected, gsn_reinforce)) == NULL)
	{
		af = new_affect();
		af->modifier		= mod * -1;
		af->location		= APPLY_AC;
		af->where		= TO_OBJECT;
		af->type		= gsn_reinforce;
		af->duration		= -1;
		af->bitvector		= 0;
		af->level		= 1;
		af->next		= upgrade->affected;
		upgrade->affected	= af;
	}
	else if (af->modifier <= mod * -1)
	{
		stc("You can't reinforce this item any further.\n\r", ch);
		return;
	}
	else af->modifier = mod * -1;

	act ("You reinforce $p with $P.", ch, upgrade, metal, TO_CHAR);
	act ("$n reinforces $p with $P.", ch, upgrade, metal, TO_ROOM);

	return;
}

void do_repair (CHAR_DATA *ch, char *argument)
{
	OBJ_DATA *upgrade;
	char arg[MSL];
	char buf[MSL];

	argument = one_argument(argument, arg);

                if ( ( upgrade = get_obj_carry( ch, arg, ch ) ) == NULL )
                {
                        stc( "You must supply a valid item to repair.\n\r", ch );
                        return;
                }
                if ( upgrade->condition >= 100 )
                {
                        stc( "This item isn't in need of repair!\n\r", ch );
                        return;
                }

	// REPLACE ME PLZ
		sprintf(buf, "You begin inspecting %s for imperfections and wear.", upgrade->short_descr);
		act(buf, ch, NULL, 0, TO_CHAR);
                WAIT_STATE(ch, PULSE_EVENT * 12);
		sprintf(buf, "$n begins inspecting %s for imperfections and wear.", upgrade->short_descr);
		act(buf, ch, NULL, 0, TO_ROOM);

		switch(number_range(1, 3))
		{
			case 1: default:
			sprintf(buf, "You skillfully hone out the nicks in %s.", upgrade->short_descr);
			wait_act(4, buf, ch, NULL, 0, TO_CHAR);
			sprintf(buf, "$n skillfully hones out the nicks in %s.", upgrade->short_descr);
			wait_act(4, buf, ch, NULL, 0, TO_ROOM);
			break;

			case 2:
			sprintf(buf, "You skillfully polish out the scratches in %s.", upgrade->short_descr);
			wait_act(4, buf, ch, NULL, 0, TO_CHAR);
			sprintf(buf, "$n skillfully polish out the scratches in %s.", upgrade->short_descr);
			wait_act(4, buf, ch, NULL, 0, TO_ROOM);
			break;

			case 3:
			sprintf(buf, "You skillfully make some much needed repairs to %s.", upgrade->short_descr);
			wait_act(4, buf, ch, NULL, 0, TO_CHAR);
			sprintf(buf, "$n skillfully makes some much needed repairs to %s.", upgrade->short_descr);
			wait_act(4, buf, ch, NULL, 0, TO_ROOM);
			break;
		}

		if (UMIN(get_skill(ch, gsn_repair), 100) < number_percent())
		{
			sprintf(buf, "You inspect %s, but don't really feel that much improvement has been made.", upgrade->short_descr);
			wait_act(8, buf, ch, NULL, 0, TO_CHAR);
			sprintf(buf, "$n inspects %s, and looks disgusted with $s work.", upgrade->short_descr);
			wait_act(8, buf, ch, NULL, 0, TO_ROOM);
			
			check_improve(ch, gsn_repair, FALSE, 6);
			return;
		}
		

	sprintf(buf, "You turn %s around in your hands, admiring your repairs.", upgrade->short_descr);
	wait_act(8, buf, ch, NULL, 0, TO_CHAR);
	sprintf(buf, "$n turns %s around in $s hands, admiring $s repairs.", upgrade->short_descr);
	wait_act(8, buf, ch, NULL, 0, TO_ROOM);

        upgrade->condition += number_range(1, 5);
	check_improve(ch, gsn_repair, TRUE, 6);
	return;
}
