#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <sys/wait.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/resource.h>
#include <ulimit.h>
#include <unistd.h> /* OLC -- for close read write etc */
#include <stdarg.h> /* printf_to_char */
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>

#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "mysql5_interface.h"

#if    defined(unix)
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
extern const char echo_off_str[];
extern const char echo_on_str[];
extern const char go_ahead_str[];
#endif

void create_newbie( DESCRIPTOR_DATA *d, char * argument )
{
	switch( d->connected )
	{
		default:
			bug( "Nanny: bad d->connected %d.", d->connected );
			close_socket( d );
		return;

		/* states for progress of sending email */
		case CON_EMAIL_TO:
			handle_email_to ( d, argument );
		break;

		case CON_EMAIL_SUBJECT:
			handle_email_subject ( d, argument );
		break;

		case CON_EMAIL_BODY:
			handle_email_body ( d, argument );
		break;

		case CON_EMAIL_FINISH:
			handle_email_finish ( d, argument );
		break;

		case CON_NOTE_TO:
			handle_con_note_to (d, argument);
		break;

		case CON_NOTE_SUBJECT:
			handle_con_note_subject (d, argument);
		break;

		case CON_NOTE_EXPIRE:
			handle_con_note_expire (d, argument);
		break;

		case CON_NOTE_TEXT:
			handle_con_note_text (d, argument);
		break;

		case CON_NOTE_FINISH:
			handle_con_note_finish (d, argument);
		break;

		case CON_GET_NAME:
			if ( argument[0] == '\0' )
			{
				close_socket( d );
				return;
			}
			argument[0] = UPPER(argument[0]);
			if ( wizlock && !is_wizlock_allowed( argument ) )
			{
				display_game_wizlocked_descriptor(d);
				close_socket( d );
				return;
			}
			if ( !check_parse_name( argument ) )
			{
				write_to_buffer(d,"\n\r", 0 );
				write_to_buffer(d,"  +------------------------------------------------------------------+ \n\r", 0);
				write_to_buffer(d,"  | WARNING! Illegal name!                                           | \n\r", 0);
				write_to_buffer(d,"  | This is likely a name that the Creator has denied usage          | \n\r", 0);
				write_to_buffer(d,"  | to, because it does not fit into a Wheel of Time theme, or       | \n\r", 0);
				write_to_buffer(d,"  | is not being used appropriately here.                            | \n\r", 0);
				write_to_buffer(d,"  | Try another name.                                                | \n\r", 0);
				write_to_buffer(d,"  +------------------------------------------------------------------+ \n\r", 0);
				write_to_buffer(d,"\n\r", 0 );
				return;
			}

			fOld = load_char_obj( d, argument );
			ch   = d->character;
			SET_BIT(ch->act,PLR_COLOUR);

			if (IS_SET(ch->act, PLR_DENY))
			{
				sprintf( log_buf, "Denying access to %s@%s.", argument, d->host );
				log_string( log_buf );
				display_denied_char(d->character);
				close_socket( d );
				return;
			}
			if (check_ban(d->host,BAN_PERMIT) && !IS_SET(ch->act,PLR_PERMIT))
			{
				display_banned_site(d->character);
				close_socket(d);
				return;
			}
			if (check_ban(d->host, BAN_ALL))
			{
				display_banned_site(d->character);
				close_socket(d);
				return;
			}
			if ( wizlock && !IS_IMMORTAL(ch))
			{
				display_game_wizlocked(d->character);
				close_socket( d );
				return;
			}
			if ( fOld )
			{
				stc(  "The voice intones, 'Whisper to me your password.' ", ch );
				stc(  echo_off_str, ch );
				d->connected = CON_GET_OLD_PASSWORD;
				return;
			}
			else
			{
				if (newlock)
				{
					display_game_newlocked(d->character);
				        close_socket( d );
				        return;
			        }
				if (check_ban(d->host,BAN_NEWBIES))
				{
					logf( "Checking Ban from bottom of Get Name..." );
					write_to_buffer(d,"The voice says, 'New players are not allowed from your site.'\n\r",0);
					close_socket(d);
					return;
				}
				argument[0] = UPPER(argument[0]);
				display_new_name(d->character);
				confirm_new_name(d->character);
				ptc( ch, "The voice whispers, 'Did I get that right, %s' (Y/N)?\n\r-> ", argument );
				d->connected = CON_CONFIRM_NEW_NAME;
				return;
			}
		break;

		case CON_GET_OLD_PASSWORD:
			if ( strcmp( crypt( argument, ch->pcdata->pwd ), ch->pcdata->pwd ))
			{
				display_invalid_password(d->character);
				close_socket( d );
				return;
			}
			stc(  echo_on_str, ch );

			if (check_playing(d,ch->name))
				return;
			if ( check_reconnect( d, ch->name, TRUE ) )
				return;

			sprintf( log_buf, "%s@%s has connected.", ch->name, d->host );
			log_string( log_buf );
			wiznet(log_buf,NULL,NULL,WIZ_SITES,0,get_trust(ch));

			d->connected = CON_CONNECT_CHAR;
		break;

		case CON_BREAK_CONNECT:
			switch( *argument )
			{
				case 'y' : case 'Y':
					for ( d_old = descriptor_list; d_old != NULL; d_old = d_next )
					{
						d_next = d_old->next;

						if (d_old == d || d_old->character == NULL)
							continue;

						if (str_cmp(ch->name,d_old->original ? d_old->original->name : d_old->character->name))
							continue;

						close_socket(d_old);
					}

					if (check_reconnect(d,ch->name,TRUE))
						return;

					write_to_buffer(d,"Reconnect attempt failed.\n\rEnter Your Name: ",0);

					if ( d->character != NULL )
					{
						free_char( d->character );
						d->character = NULL;
					}
					d->connected = CON_GET_NAME;
					break;

				case 'n' : case 'N':
					write_to_buffer(d,"Enter Your Name: ",0);
					if ( d->character != NULL )
					{
						free_char( d->character );
						d->character = NULL;
					}
					d->connected = CON_GET_NAME;
				break;

				default:
					write_to_buffer(d,"The voice urges, 'Please type Y or N' ",0);
				break;
			}
		break;

		case CON_CONFIRM_NEW_NAME:
			if ( !IS_SET( ch->act, PLR_COLOUR ) )
				SET_BIT( ch->act, PLR_COLOUR );

			switch ( *argument )
			{
				case 'y': case 'Y':
					sprintf( buf, "The voice says, 'You are a new character.'\n\rIt continues, 'Give me a password for %s: %s'",ch->name, echo_off_str );
					stc(  buf, ch );
					ch->pcdata->newbie = 1;
					init_newbie( ch );
					d->connected = CON_GET_NEW_PASSWORD;
				break;

				case 'n': case 'N':
					stc(  "The voice urges, 'What is it, then?' ", ch );
					free_char( d->character );
					d->character = NULL;
					d->connected = CON_GET_NAME;
				break;

				default:
					stc(  "The voice urges, 'Please type Yes or No.' ", ch );
				break;
			}
		break;

		case CON_GET_NEW_PASSWORD:
			if ( strlen(argument) < 5 )
			{
				stc("The voice intones, 'The password must be at least five characters long.'\n\rIt continues, 'What is the password?' ",ch );
				return;
			}
			pwdnew = crypt( argument, ch->name );
			for ( p = pwdnew; *p != '\0'; p++ )
			{
				if ( *p == '~' )
				{
					stc( "The voice intones, 'The new password is not acceptable. Try again.'\n\rIt asks, 'What is the password?' ", ch );
					return;
				}
			}

			free_string( ch->pcdata->pwd );
			ch->pcdata->pwd	= str_dup( pwdnew );
			stc("The voice says, 'Please retype password:' ", ch );
			d->connected = CON_CONFIRM_NEW_PASSWORD;
		break;

		case CON_CONFIRM_NEW_PASSWORD:
			if ( strcmp( crypt( argument, ch->pcdata->pwd ), ch->pcdata->pwd ) )
			{
				stc("The voice intones, 'The passwords do not match.'\n\rIt continues, 'Retype the password:' ", ch );
				d->connected = CON_GET_NEW_PASSWORD;
				return;
			}
			stc(  echo_on_str, ch );
			sprintf( log_buf, "%s@%s new player.", ch->name, d->host );
			log_string( log_buf );
			wiznet("{RNewbie alert{g!  $N sighted.",ch,NULL,WIZ_NEWBIE,0,0);
			wiznet(log_buf,NULL,NULL,WIZ_SITES,0,get_trust(ch));

			for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
			{
				if ( d_mentor->connected == CON_PLAYING && IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
				{
					act_new("{w({WMentor Alert{w){W::{x $N. A new player sighted!{x", d_mentor->character,NULL,ch,TO_CHAR,POS_DEAD);
				}
			}

			stc("{C+{c-----------------------------------------------------------------------------------{C+{c\n\r", ch );
			stc("{c|{w TSE uses last names for characters. SOme races, such as Seanchan, Ogier, and Aiel {c|{w\n\r", ch );
			stc("{c|{w will use the last name as a family name (such as 'Son of' for ogier) or not use   {c|{w\n\r", ch );
			stc("{c|{w them at all. Due to the way last names are used for other purposes, though, you   {c|{w\n\r", ch );
			stc("{c|{w must specify one here.                                                            {c|{w\n\r", ch );
			stc("{C+{c-----------------------------------------------------------------------------------{C+{c\n\r", ch );
			d->connected = CON_GET_LAST_NAME;
		break;

		case CON_GET_LAST_NAME:
			if ( strlen(argument) < 2 )
			{
				stc( "The voice intones, 'That is not a suitable last name.'\n\rIt continues, 'What is your last name?'\n\r-> ", ch );
				d->connected = CON_GET_LAST_NAME;
				break;
			}
			if ( strstr(argument, "'") != NULL )
			{
				stc( "The voice intones, 'Your last name may only consist of letters.'\n\rIt continues, 'What is your last name?'\n\r-> ", ch );
				d->connected = CON_GET_LAST_NAME;
				break;
			}
			if ( !check_parse_name( argument ) )
			{
				stc(" +------------------------------------------------------------------+ \n\r", ch );
				stc(" | WARNING! Unsuitable last name!                                   | \n\r", ch );
				stc(" | This is likely a name that the Creator has denied usage          | \n\r", ch );
				stc(" | to, because it does not fit into a Wheel of Time theme, or       | \n\r", ch );
				stc(" | is not being used appropriately here.                            | \n\r", ch );
				stc(" | Try another name.                                                | \n\r", ch );
				stc(" +------------------------------------------------------------------+ \n\r\n\r", ch );
				ptc( ch, "The voice asks, 'What is %s's last name?'\n\r-> ",ch->name );
				d->connected = CON_GET_LAST_NAME;
				break;
			}
			else
			{
				ch->pcdata->last_name = str_dup( capit2(argument) );
				stc("{C+{c---------------------------------------------------------------------{C+{c\n\r", ch );
				stc("{c|{w TSE uses email addresses to keep in touch with the players and for  {c|{w\n\r", ch );
				stc("{c|{w other purposes, such as linking your forum account to your in-game  {c|{w\n\r", ch );
				stc("{c|{w characters. We will never give out your email address and it will   {c|{w\n\r", ch );
				stc("{c|{w not be used without your persmission.                               {c|{w\n\r", ch );
				stc("{C+{c---------------------------------------------------------------------{C+{c\n\r\n\r", ch );
				stc("The voice asks, 'What is your email address?'\n\r-> ", ch);
				d->connected = CON_GET_EMAIL;
				break;
			}
		break;

		case CON_GET_EMAIL:
			fail = FALSE;

			if (strlen(argument) < 5)
				fail = TRUE;
			if (str_infix("@", argument))
				fail = TRUE;
			if (str_infix(".", argument))
				fail = TRUE;
			if (!isalpha(argument[0]))
				fail = TRUE;
			if (!isalpha(argument[strlen(argument) - 1]))
				fail = TRUE;

			if (!fail)
			{
				if (ch->pcdata && ch->pcdata->email)
					free_string(ch->pcdata->email);

				ch->pcdata->email = str_dup(argument);
				write_to_buffer(d,"The voice says, 'Welcome, young adventurer.'\n\r",0);
				write_to_buffer(d,"\n\r[Press Enter to Continue]\n\r",0);
				d->connected = CON_CLEAN_UP;
				break;
			}
			else
			{
				stc( "The voice says, 'Sorry, your email address does not appear to be valid.'\n\r", ch );
				stc( "The voice whispers, 'What is your email address?'\n\r-> ", ch );
				d->connected = CON_GET_EMAIL;
				break;
			}
		break;

		case CON_CLEAN_UP:
			if ( d->character->pcdata->rebirthing )
				stc( "The voice says, 'Fare you well in your new life...'\n\r", ch );
			else
				stc( "The voice says, 'Since you are new we will give you a tour of the game.'\n\r", ch );

			stc( "\n\r[Press Enter to Continue]\n\r", ch );
			d->connected = CON_CONNECT_CHAR;
		break;

		case CON_CONNECT_CHAR:
			if ( ch->pcdata != NULL && ch->pcdata->pwd[0] == '\0')
			{
				stc( "{RWarning! Null password!{x\n\r",ch );
				stc( "{RType {r'{Rpassword null {r<{Rnew password{r>' {Rto fix this problem.{x\n\r",ch );
			}
			if (d->character->pcdata->rebirthing == FALSE)
			{
				ch->next = char_list;
				char_list = ch;
			}
			reset_char(ch);
			d->connected = CON_PLAYING;

			if (d->character->pcdata->rebirthing == TRUE)
			{
				for ( sk = 0; sk < MAX_SKILL; sk++ )
				{
					if (ch->pcdata->learned[sk] <= 0)
						continue;
					if (!str_cmp(skill_table[sk].name, "reserved"))
						continue;
					if (!str_cmp(skill_table[sk].name, "gifted"))
						continue;
					if (calc_remort_tier(ch) == 1)
					{
						ch->pcdata->learned[sk] -= 75;
						if (ch->pcdata->learned[sk] < 0)
							ch->pcdata->learned[sk] = 0;
					}
					else
					{
						ch->pcdata->learned[sk] -= 100;
						if (ch->pcdata->learned[sk] < 0)
							ch->pcdata->learned[sk] = 0;
					}
				}
				ch->pcdata->rebirth += 1;
				ch->level = 1;
				ch->exp = exp_per_level(ch,ch->pcdata->points);
				ch->max_hit = 50;
				ch->max_stam = 50;
				ch->pcdata->perm_hit = 50;
				ch->pcdata->perm_mana = 50;
				ch->hit = get_maxhit(ch);
				ch->stam = get_maxstam(ch);
				ch->train = ch->train * .75;
			    	compose_title(ch);
				char_from_room (ch);
		    	        char_to_room( ch, get_room_index( ROOM_VNUM_SCHOOL ) );
	   			ch->pcdata->rebirthing = FALSE;
				init_newbie( ch );
			}
			else if ( ch->level == 0 || ch->pcdata->perm_hit == 0 || ch->pcdata->perm_mana == 0)
			{
				ch->perm_stat[class_table[ch->class].attr_prime] += 3;
				ch->level = 1;
				ch->exp = exp_per_level(ch,ch->pcdata->points);
				ch->max_hit	= 50;
				ch->max_stam	= 50;
				ch->pcdata->perm_hit	= 50;
				ch->pcdata->perm_mana	= 50;
				ch->hit	   	= get_maxhit(ch);
				ch->stam	= get_maxstam(ch);
				ch->train	 = 50;
				ch->version = 6;
				compose_title(ch);
				char_to_room( ch, get_room_index( ROOM_VNUM_SCHOOL ) );
			}
			else if ( ch->was_in_room != NULL )
			{
				char_to_room( ch, ch->was_in_room );
			}
			else if ( ch->in_room != NULL )
			{
				if ( ch->pcdata && ch->pcdata->allowed == 0 )
				{
					char_to_room(ch,get_room_index(3700));
				}
				else
					char_to_room( ch, ch->in_room );
			}
			else if ( IS_IMMORTAL(ch) )
				char_to_room( ch, get_room_index( ROOM_VNUM_BETWEEN ) );
			else if ( ch->version <= 11 )
				char_to_room(ch,get_room_index(3700));
			else
				char_to_room( ch, get_room_index( ROOM_VNUM_PLAZA ) );

			if ( ch->pcdata->newbie != 1 )
				do_function(ch, &do_look, "auto" );

			wiznet("{c$N busts in on the party.",ch,NULL,WIZ_LOGINS,WIZ_SITES,get_trust(ch));
			act( "{c$n joins in, looking for some adventure.",ch,NULL,NULL,TO_ROOM );

			if (ch->pet != NULL)
			{
				char_to_room(ch->pet,ch->in_room);
				act("{c$n joins in, looking for some adventure.",ch->pet,NULL,NULL,TO_ROOM);
			}
			if (ch->mount != NULL)
			{
				char_to_room(ch->mount,ch->in_room);
				act("$n has entered the lands.",ch->mount,NULL,NULL,TO_ROOM);
				add_follower(ch->mount, ch);
				do_mount(ch, ch->mount->name);
			}

			if (ch->clan < 1 || clan_table[ch->clan].name == NULL)	ch->clan = 1;

			if ( d && d->character != NULL && d->connected == CON_PLAYING )
			{
				DESCRIPTOR_DATA *vch;
				for ( vch = descriptor_list; vch != NULL; vch = vch->next )
				{
					if ( !vch || vch == NULL )
						continue;
					if ( !vch->character || vch->connected != CON_PLAYING )
						continue;
					if ( IS_IMMORTAL( d->character ) )
						continue;
					else if ( IS_SET( vch->character->comm, COMM_PENTER) && (!IS_SET(vch->character->comm,COMM_QUIET)) )
					{
						act_new("\n\r{w*** {c$n joins in, looking for some adventure.\n\r",
						d->character,NULL,vch->character,TO_VICT,POS_DEAD);
					}
				}
				if ( d->character->pcdata && d->character->pcdata->allowed != 1 && d->character->pcdata->newbie != 1)
				{
					for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
					{
						if ( d_mentor->connected == CON_PLAYING && IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
						{
							act_new("{w({WMentor Alert{w){W::{x $N. An unverified player sighted!{x", d_mentor->character,NULL,ch,TO_CHAR,POS_DEAD);
						}
					}
				}
			}
			sprintf( log_buf, "%s busts in on the party.", ch->name );
			log_string( log_buf );
			wiznet(log_buf,NULL,NULL,WIZ_SITES,0,get_trust(ch));
			stc("\n\r",ch);
			MXPSendTag( d, "<VERSION>" );  /* <--- Add this line */
			save_db_character(ch);

			if ( ch->pcdata->newbie != 1 )
			{
				/* display the time and date */
				do_function(ch, &do_time, "");
				stc("\n\r",ch);

				/* show the status of game events */
				do_function( ch, &do_events, "" );
				stc( "{x\n\r",ch);

				vote_message( ch );

				/* check for unread notes */
				do_function(ch, &do_board, "");

				if ( ch->pcdata->pfile_size > 150000 )
				{
					stc("{RWARNING{r!!! {WYour pfile size is VERY LARGE! Clean out your inventory!{x\n\r", ch);
				}
			}
			else
			{
	    			compose_title(ch);
				SET_BIT(ch->comm, COMM_NODFH);
				do_function( ch, &do_tour, "" );
			}
			ch->pcdata->last_in = current_time;
			break;
		}
	return;
}

