/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/********************************************************************************
 * Email code copyright 1999-2001                                               *
 * Markanth : dlmud@dlmud.com                                                   *
 * Devil's Lament : dlmud.com port 3778                                      *
 * Web Page : http://www.dlmud.com                                              *
 *                                                                              *
 * Provides ability to send an email with your character from within a mud.     *
 * String parsing code based off roguemud.                                      *
 *                                                                              *
 * All I ask in return is that you give me credit on your mud somewhere         *
 * or email me if you use it, or if you make any changes.                       *
 ********************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "recycle.h"

const char *szEmailPrompt = "\n\r(C)ontinue, (V)iew, (S)end or (F)orget it?";

bool valid_email ( char *argument )
{
    int i;
    bool pAnd = FALSE, pDot = FALSE;

    for ( i = 0; argument[i] != '\0'; i++ )
    {
        if ( argument[i] == '@' )
            pAnd = TRUE;
        else if ( argument[i] == '.' )
            pDot = TRUE;
    }
    if ( !pAnd || !pDot )
        return FALSE;
    else
        return TRUE;
}

void handle_email_to ( DESCRIPTOR_DATA * d, char *argument )
{
    CHAR_DATA *ch = CH ( d );

    if ( !ch || !ch->pcdata )
    {
        bug ( "handle_email_to: NULL pcdata", 0 );
        return;
    }
    if ( argument[0] == '\0' )
    {
        write_to_buffer ( d,
                          "Please enter a valid e-mail address.\n\rType 'cancel' to cancel this letter.\n\r",
                          0 );
        return;
    }
    else if ( !str_cmp ( argument, "cancel" ) )
    {
        write_to_buffer ( d, "E-Mail cancelled.\n\r", 0 );
        free_email_data ( ch->pcdata->email_data );
        ch->pcdata->email_data = NULL;
        d->connected = CON_PLAYING;
        return;
    }
    else if ( !valid_email ( argument ) || strlen ( argument ) > 50 )
    {
        write_to_buffer ( d,
                          "Please enter a valid e-mail address.\n\rType 'cancel' to cancel this letter.\n\r",
                          0 );
        return;
    }
    free_string( ch->pcdata->email_data->to );
	ch->pcdata->email_data->to = str_dup( argument );
    write_to_buffer ( d,
                      "E-Mail address set.\n\rPlease enter a subject for your letter:\n\r",
                      0 );
    d->connected = CON_EMAIL_SUBJECT;
    return;
}

void handle_email_subject ( DESCRIPTOR_DATA * d, char *argument )
{
    CHAR_DATA *ch = CH ( d );

    if ( !ch || !ch->pcdata )
    {
        bug ( "handle_email_subject: NULL pcdata", 0 );
        return;
    }
    if ( argument[0] == '\0' )
    {
        write_to_buffer ( d,
                          "Please enter a subject for your E-Mail.\n\rType 'cancel' to cancel this letter.\n\r",
                          0 );
        return;
    }
    else if ( !str_cmp ( argument, "cancel" ) )
    {
        write_to_buffer ( d, "E-Mail cancelled.\n\r", 0 );
        free_email_data ( ch->pcdata->email_data );
        ch->pcdata->email_data = NULL;
        d->connected = CON_PLAYING;
        return;
    }
    else if ( strlen ( argument ) > 50 )
    {
        write_to_buffer ( d,
                          "No, no. This is just the Subject. You're note writing the letter yet.\n\r",
                          0 );
        return;
    }
    free_string( ch->pcdata->email_data->subject );
	ch->pcdata->email_data->subject = str_dup( argument );
    write_to_buffer ( d,
                      "Subject set.\n\rPlease enter the body of your e-mail.\n\r",
                      0 );

    write_to_buffer ( d,
                      "\n\r"
                      "Enter text. Type .h for help or .q to end email.\n\r"
                      "=====================================================\n\r",
                      0 );
    d->connected = CON_EMAIL_BODY;
    return;
}

void parse_email_text ( int command, char *string, CHAR_DATA * ch )
{
    switch ( command )
    {
        case PARSE_HELP:
            send_to_char
                ( "=======================================================\n\r",
                  ch );
            send_to_char ( "Edit help (commands on blank line):\n\r", ch );
            send_to_char ( ".h               - get help (this info)\n\r", ch );
            send_to_char ( ".s               - show string so far\n\r", ch );
            send_to_char ( ".c               - clear string so far\n\r", ch );
            send_to_char ( ".q               - end string\n\r", ch );
            send_to_char
                ( "=======================================================\n\r",
                  ch );
            break;
        case PARSE_SHOW:
            send_to_char ( "String so far:\n\r", ch );
            page_to_char ( ch->pcdata->email_data->body, ch );
            break;
        default:
            send_to_char ( "Invalid command.\n\r", ch );
            bug ( "invalid command passed to parse_email_text", 0 );
            break;
    }
}

void handle_email_body ( DESCRIPTOR_DATA * d, char *argument )
{
    bool terminate = FALSE;
    int i = 2, j = 0, action = 0;
    CHAR_DATA *ch = d->character;
    char buf[MAX_STRING_LENGTH];
    char actions[MAX_INPUT_LENGTH];

    if ( !ch->pcdata->email_data )
    {
        d->connected = CON_PLAYING;
        bug ( "nanny: In CON_EMAIL_TEXT, buf no email in progress.", 0 );
        return;
    }

    if ( ( action = ( *argument == '.' ) ) )
    {
        while ( argument[i] != '\0' )
        {
            actions[j] = argument[i];
            i++;
            j++;
        }
        actions[j] = '\0';
        *argument = '\0';
        switch ( argument[1] )
        {
            case 'q':
                terminate = TRUE;
                *argument = '\0';
                break;
            case 'c':
                send_to_char ( "String cleared.\n\r", ch );
                free_string( ch->pcdata->email_data->body );
                ch->pcdata->email_data->body = str_dup( "" );
                return;
            case 's':
                parse_email_text ( PARSE_SHOW, actions, ch );
                return;
            case 'h':
                parse_email_text ( PARSE_HELP, actions, ch );
                return;
            default:
                send_to_char ( "Invalid command.\n\r", ch );
                return;
        }
    }

    if ( terminate )
    {
        sprintf(buf, "\n\r%s", szEmailPrompt );
        send_to_char(buf, ch);
        d->connected = CON_EMAIL_FINISH;
        return;
    }

    if ( ch->pcdata->email_data->body )
    {
        strcpy ( buf, ch->pcdata->email_data->body );
        free_string( ch->pcdata->email_data->body );
        ch->pcdata->email_data->body = str_dup( "" );
    }
    else
        strcpy ( buf, "" );

    if ( ( strlen ( argument ) + strlen ( buf ) ) > MAX_EMAIL_TEXT )
    {
        send_to_char ( "E-mail too long, bailing out!\n\r", ch );
        free_email_data ( ch->pcdata->email_data );
        ch->pcdata->email_data = NULL;
        d->connected = CON_PLAYING;
        return;
    }

    strcat ( buf, argument );
    strcat ( buf, "\n" );
    free_string( ch->pcdata->email_data->body );
	ch->pcdata->email_data->body = str_dup( buf );
}

void handle_email_finish ( DESCRIPTOR_DATA * d, char *argument )
{
    CHAR_DATA *ch = d->character;
    char buf[MAX_STRING_LENGTH];

    if ( !ch->pcdata->email_data )
    {
        d->connected = CON_PLAYING;
        bug ( "nanny: In CON_EMAIL_FINISH, but no email in progress.", 0 );
        return;
    }

    switch ( argument[0] )
    {
        case 'C':
        case 'c':              /* keep writing */
            send_to_char ( "Continuing e-mail...\n\r\n\r", ch );
            send_to_char ( ch->pcdata->email_data->body, ch );
            d->connected = CON_EMAIL_BODY;
            break;
        case 'V':
        case 'v':              /* view note so far */
            if ( ch->pcdata->email_data->body )
            {
                send_to_char ( "Your note so far:\n\r", ch );
                sprintf(buf, "To: %s\n\r", ch->pcdata->email_data->to );
                send_to_char(buf, ch);
                sprintf(buf, "Subject: %s\n\r",
                           ch->pcdata->email_data->subject );
                send_to_char(buf, ch);
                page_to_char ( ch->pcdata->email_data->body, ch );
                write_to_buffer ( d, "\r\n", 0 );
            }
            else
                write_to_buffer ( d, "You haven't written a thing!\n\r\n\r",
                                  0 );
            send_to_char ( szEmailPrompt, ch );
            break;
        case 'S':
        case 's':              /* post note */
            if ( emailf ( ch ) )
                write_to_buffer ( d, "E-Mail sent.\n\r", 0 );
            else
                write_to_buffer ( d,
                                  "There was a problem sending your e-mail, contact an Imm.\n\r",
                                  0 );
            d->connected = CON_PLAYING;
            ch->pcdata->email_data = NULL;
            act ( "$n finishes sending $s e-mail.", ch, NULL, NULL,
                  TO_ROOM );
            break;
        case 'F':
        case 'f':
            write_to_buffer ( d, "Email cancelled!\n\r", 0 );
            free_email_data ( ch->pcdata->email_data );
            ch->pcdata->email_data = NULL;
            d->connected = CON_PLAYING;
            act ( "$n cancels $s e-mail.", ch, NULL, NULL, TO_ROOM );
            break;
        default:               /* invalid response */
            write_to_buffer ( d, "Huh? Valid answers are:\n\r\n\r", 0 );
            send_to_char ( szEmailPrompt, ch );
    }
}

void do_sendmail (CHAR_DATA *ch, char *argument)
{
    if ( !ch->desc || !ch->pcdata )
        return;

	if ( ch->pcdata->email[0] == '\0' )
	{
		stc( "You must have defined your own email address first.\n\rUse the Email command to do this.\n\r", ch );
		return;
	}
    send_to_char ( "Please enter the E-Mail address you wish to send to:\n\r",
             ch );
    ch->pcdata->email_data = new_email_data (  );
    ch->desc->connected = CON_EMAIL_TO;
    return;
}

#define MUD_NAME    "The Shadow's Embrace"
#define MUD_TELNET  "telnet://theshadowsembrace.com:4000"
#define MUD_WWW     "http://www.theshadowsembrace.com"
#define MUD_REPLYTO "noreply@theshadowsembrace.com"

bool emailf ( CHAR_DATA * ch )
{
    FILE *mail;
    char buf[MAX_INPUT_LENGTH];
    EMAIL_DATA *pMail;

    if ( !ch || !ch->pcdata )
        return FALSE;

    if ( !( pMail = ch->pcdata->email_data ) )
        return FALSE;

    if ( pMail->to[0] == '\0' || pMail->body[0] == '\0' )
        return FALSE;

    /* path to sendmail program here */
    sprintf ( buf, "/usr/sbin/sendmail -t -i -F%s", ch->name );

    if ( ( mail = popen ( buf, "w" ) ) == NULL )
    {
        bug ( "Error opening sendmail", 0 );
        return FALSE;
    }

    fprintf ( mail, "To: %s\n", pMail->to );

    fprintf ( mail, "Reply-to: %s\n",
              ch->pcdata->email[0] == '\0' ? MUD_REPLYTO : ch->pcdata->email );

    fprintf ( mail, "Subject: %s\n\n", pMail->subject[0] != '\0' ?
        pMail->subject : "Mail from " MUD_NAME "" );
    fprintf ( mail, "%s", pMail->body );

/* Print mud signature file */
    fprintf ( mail, "\n----------------------------------\n" );
    fprintf ( mail, "%s\n", MUD_NAME );
    fprintf ( mail, "%s\n", MUD_TELNET );
    fprintf ( mail, "%s\n", MUD_WWW );
    fprintf ( mail, "----------------------------------\n" );
    pclose ( mail );
    return TRUE;
}

void do_email (CHAR_DATA *ch, char *argument)
{
    char buf[MAX_STRING_LENGTH];

    if ( !ch || IS_NPC ( ch ) )
        return;

    if ( argument[0] == '\0' )
    {
        sprintf ( buf, "Your current email address is: %s\n\r",
                  ch->pcdata->email );
        send_to_char ( buf, ch );
        return;
    }
    if ( is_name ( argument, "clear none delete" ) )
    {
        free_string( ch->pcdata->email );
        ch->pcdata->email = str_dup( "" );
        send_to_char ( "E-mail cleared.\n\r", ch );
        return;
    }
    if ( !valid_email ( argument ) )
    {
        send_to_char ( "That is not a valid email address.\n\r", ch );
        return;
    }
    smash_tilde ( argument );
    free_string( ch->pcdata->email );
    ch->pcdata->email = str_dup( argument );
    sprintf ( buf,
              "Email changed to: %s\n\r",
              argument );
    send_to_char ( buf, ch );
    return;
}
