/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "aww.h"

/*
 * bitcount is used in roomcount procedure, everything what it does is that it
 * returns amount of bits set in the 8bit variable
 */
int bitcount(char c)
{
    int count = 0;

    if (c & A) count++;
    if (c & B) count++;
    if (c & C) count++;
    if (c & D) count++;
    if (c & E) count++;
    if (c & F) count++;
    if (c & G) count++;
    if (c & H) count++;

    return count;
}

/*
 * roomcount is called from do_score in act_info.c, it returns number of
 * explored rooms
 */
int roomcount(CHAR_DATA * ch)
{
    long index = 0;
    int count = 0;

    if (IS_NPC(ch)) return top_room;

    for ( index = 0; index < 4096; index++)
	count += bitcount(ch->pcdata->explored[index]);

    return count;
}

/*
 * getbit returns 1 if the specific bit is set, 0 otherwise, it is used by
 * fwrite_rle procedure.
 */
int getbit(char *explored, long index)
{
    int bit = 0;

    switch (index % 8)
    {
	case 0:	 bit = A; break;
	case 1:	 bit = B; break;
        case 2:	 bit = C; break;
        case 3:	 bit = D; break;
        case 4:	 bit = E; break;
        case 5:	 bit = F; break;
        case 6:	 bit = G; break;
        case 7:	 bit = H; break;
        default: bit = 0; break;
    }

    if (IS_SET(explored[index/8], bit)) return 1;
    else return 0;

}

/*
 * setbit sets the specific bit pointed on by the index [vnum] of the room,
 * there is no need for remove bit at this time. setbit is used in char_to_room
 * and also in fread_rle to restore the array from pfile.
 */
void setbit(char *explored, long index)
{
    int bit = 0;

    switch (index % 8)
     {
	case 0:	 bit = A; break;
	case 1:	 bit = B; break;
        case 2:	 bit = C; break;
        case 3:	 bit = D; break;
	case 4:	 bit = E; break;
        case 5:	 bit = F; break;
	case 6:	 bit = G; break;
        case 7:	 bit = H; break;
	default: bit = 0; break;
    }

    SET_BIT(explored[index/8], bit);
    return;
}

/* fwrite_rle saves the status of the array in already open file it doesn't
 * check for NPCs or anything, however it uses simple RLE-like bit compression
 * to save some space on the drive. this procedure is called from fwrite_char
 */
void fwrite_rle(char *explored, FILE *fp)
{
    long index;
    int bit = getbit(explored,0);
    int count = 0;

    fprintf(fp,"RoomRLE  ");

    for (index = 0; index < 32768; index++)
    {
	if (getbit(explored,index) == bit) count++;
	else
	{
	    fprintf(fp," %d %d",count,bit);
	    count = 1;
	    bit = getbit(explored,index);
	}
    }
    fprintf(fp," %d %d -1\n",count,bit);
    return;
}

/* fread_rle reads the data back in the bit array, it automatically
 * decompresses them to their original state, the bit compression was hammered
 * out on #rom channel with Peter Vidler :)
 */
void fread_rle(char *explored, FILE *fp)
{
    long index;
    int bit   = 0;
    int count = 0;
    int pos   = 0;

    index = 0;

    for (;;)
    {
	count = fread_number(fp);
	if (count < 0) break;

	bit = fread_number(fp);

	do
	{
	    if (bit == 1) setbit(explored,index);
	    index++;
	} while (index < pos+count);
	pos = index;
    }
    return;
}

bool has_exits args((ROOM_INDEX_DATA *pRoom));

/* returns whether or not a room should be counted in exploration */
bool count_room (ROOM_INDEX_DATA *room)
{
	if (IS_SET(room->room_flags, ROOM_SOLITARY))	return FALSE;
	if (IS_SET(room->room_flags, ROOM_PRIVATE))	return FALSE;
	if (!has_exits(room))				return FALSE;
	if (room->name[0] == '\0')			return FALSE;
	if (!str_cmp(room->name, ""))			return FALSE;
	if (room->clan > 0)				return FALSE;

	return TRUE;
}


void do_explored( CHAR_DATA *ch, char *argument )
{
    char arg[MIL];
    int offset;
    int bit;
    float count = 0;

    if (IS_NPC(ch)) return;

    one_argument(argument, arg);

    if ( arg[0] == '\0' )
    {
        for ( offset = 0; offset < 8192; offset++ )
            for( bit = 0; bit < 8; bit++ )
                if (ch->pcdata->explored[offset] & 1 << bit)
                    count++;

        printf_to_char(ch, "{wYou have explored {c%.0f {wof {c%d {wrooms "
                "{C({c%.2f%% {wof the mwd{C)\n\r{x",
                count, top_room, 100 * count  / top_room );

        return;
    }

    if (!str_cmp(arg, "area"))
    {
        ROOM_INDEX_DATA *pRoom;
        long start, end;
        int area_count;
        long i;

        if (!ch->in_room ) return;

        start = ch->in_room->area->min_vnum;
        end   = ch->in_room->area->max_vnum;

        area_count = 0;

        for ( i = start; i <= end; i++ )
        {
            if ( ( pRoom = get_room_index(i) ) == NULL )
                continue;

	    if (!count_room(pRoom))	continue;

            area_count++;

            offset = i / 8;
            bit    = i % 8;

            if (ch->pcdata->explored[offset] & 1 << bit )
                count++;
        }

        printf_to_char(ch, "{wYou have explored {c%.0f {wof {c%d {wrooms in "
                "this area. {C({c%.2f%%{C)\n\r{x", count, area_count,
                            (area_count != 0 ) ? 100 * count / area_count : 0 );

        return;
    }
    else
    {
        send_to_char("{wUsage:\n\r"
         "{wexplored      - {cShow total amount of the explored\n\r"
         "{wexplored area - {cShow total amount of the current area explored.\n\r",
         ch);

        return;
    }

    return;
}

void check_explore_reward( CHAR_DATA *ch )
{
	int rooms = top_room;
	int rcnt = roomcount( ch );
    int percent = rcnt * 100 / rooms;

	/*if ( percent > ( ch->pcdata->explored_pcnt * 10 ) )*/
	if ( percent > ( ch->pcdata->explored_pcnt ) )
	{
		stc("{r<<{Y*{r>>{W You gain a bonus for your efforts in exploring the world! {r<<{Y*{r>>{x\n\r",ch);
		/*gain_exp(ch, 1000);*/
		gain_exp(ch, exp_per_level( ch, ch->pcdata->points) );
		ch->pcdata->explored_pcnt++;
	}
	/*if ( ( roomcount( ch ) / rooms ) * 100 ) ) > ( ch->pcdata->explore_pcnt * 10 ) )
	{
		gain_exp(ch, 1000);
		ch->pcdata->explored_pcnt++;
	}*/
	/*if (!IS_NPC(ch))
    {
		int    rcnt    = roomcount(ch);
		int i = 0, x = 1;
		int rooms   = top_room;
		ptc( ch, "Rcnt %d\n\r", rcnt );
		for ( i = 0; i < 10; i++ )
		{
			ptc( ch, "%d\n\r", ( rooms * ( i+1*100 ) ) / rcnt );
			if ( ( rooms * ( i+1*100 ) ) / rcnt == x && ch->pcdata->explore_exp[i] == 0 )
			{
				stc( "*** You would have just gained 1000 exp, 4 trains. ****\n\r", ch );
				ch->pcdata->explore_exp[i] = 1;
			}
			x++;
			gain_exp(ch,1000);
		}
	}*/
}
