/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "fishing.h"

/* implicit declaration of lookup_fish - moved here - Dalsor */
/*looking up requires: minsize, maxsize, location and outputs an ID*/

int lookup_fish(int minsize, int maxsize, int location)
{
	int		number_of_possible_fish=0;
	int 	fish;
	int i = 0;

	/* Check how many fish are ok */
	for(i=0;fish_table[i].name != NULL;i++)
	{
		if ( fish_table[i].location == location )
		{
			if ( fish_table[i].minsize <= maxsize && minsize <= fish_table[i].maxsize )
				number_of_possible_fish += fish_table[i].rarity;
		}
	}

	/*reject if there are no possible fish (shouldn't happen but anyway)*/
	if(number_of_possible_fish == 0)
	{
		return -1;
		/*Bug this, as it shouldn't occur?*/
	}

	/*Pick one of the fish*/
	fish=number_range(1,number_of_possible_fish);
	for ( i = 0; i < fish; i++ )
	{
		if (fish_table[i].location == location)
		{
			if(fish_table[i].minsize <= maxsize && minsize <= fish_table[i].maxsize)
				fish -= fish_table[i].rarity;
		}
	}

	return i;
}

void do_tickle( CHAR_DATA *ch, char *argument)
{
	int check;
	int temp_val;
	int minsize;
	int maxsize;
	int size;
	char *buf;
	int timer;
	ROOM_INDEX_DATA *terrain = ch->in_room;
	OBJ_DATA *fish;

/* This will not work - Dalsor
	if (IS_NPC(ch) || str_cmp(argument, "fish"))
	{
		check_social( ch, command, argument );
		return;
	}
*/
	if ( IS_NPC(ch) )
	{
		return;
	}

	if(	terrain->sector_type != SECT_RIVER_FRESH &&
		terrain->sector_type != SECT_RIVER_SALT)
	{
		stc("You should try to find a river when fishing.\n\r",ch);
		return;
	}
	if(ch->position != POS_RESTING)
	{
		stc("You will have to lie down when trying to tickle fish.\n\r",ch);
		return;
	}

	check = (get_curr_stat( ch, STAT_DEX )- number_percent()-number_percent())/3;/*dex to get your hand underneath the fish*/
	check += (get_curr_stat( ch, STAT_REF ) - number_percent()-30)/10;/*reflex to flop the fish on land*/


	/* A Taveren special, as they are not too common anyway,
		I even think it doesn't do a thing */
	if ( IS_SET( ch->spec_flags, SPEC_TAVEREN ) )
	{
		int temp_val;
		temp_val = check/10;
		if((temp_val % 10) == (check % 10))
		{
			if (check>22)
			{
				check += 33;
			}
			else if (check < 23)
			{
				check *= 2;
			}
		}
	}
	if(check < -25)
	{
		stc("Clumsy as you are, you slide from the riverbank and drop in the water, disturbing all fish.\n\r",ch);
		WAIT_STATE(ch,PULSE_VIOLENCE *5);
		/*Possibly catch a COLD and LOOSE a piece of eq or some coin here
		The chance of this event is quite small anyway*/
		return;
	}
	else if (check < - 10)
	{
		stc("Enthusiastically you try to get some fish, but instead splash a handful of water on yourself.\n\r",ch);
		WAIT_STATE(ch,PULSE_VIOLENCE * 4);
		/*Possibly catch a COLD here*/
		return;
	}
	else if (check < 0)
	{
		stc("Lying at the riverside for ages, you fail to spot any fish.\n\r",ch);
		WAIT_STATE(ch,PULSE_VIOLENCE * 3);
		return;
	}
	else if (check < 10)
	{
		buf = str_dup("Despite your efforts, you only catch a $t.\n\r");
		timer = 3;
		minsize = 1;
		maxsize = 6;
	}
	else if (check < 30)
	{
		buf = str_dup("Not too long after you started looking you can grab $t.\n\r");
		timer= 2;
		minsize = 5;
		maxsize = 9;
	}
	else /*if (check < 50)*/
	{
		buf = str_dup("Clearly it`s your lucky day as you caught $t in no time at all.\n\r");
		timer =  1;
		minsize = 7;
		maxsize = 13;
	}
	/* Possibly add an extra chance for a gifted Taveren...
	'Seeing the fish jump on land, you realize Taverenness is at work
	again.\n\r'*/

	temp_val = lookup_fish(minsize,maxsize,FISH_SECT_RIVER);

	if ( temp_val < 0 )
	{
		bugf( "FISH: Action = tickling, minsize %d, maxsize %d", minsize, maxsize );
		return;
	}

	size = number_range(UMAX(minsize,fish_table[temp_val].minsize),UMIN(maxsize,fish_table[temp_val].maxsize));
	fish = create_object( get_obj_index( 30 ), 1 );
	free_string( fish->name );
	fish->name 	= str_dup(fish_table[temp_val].name);
	free_string( fish->short_descr );
	fish->short_descr 	= str_dup(fish_table[temp_val].short_desc);
	free_string( fish->description );
    fish->description	= str_dup(fish_table[temp_val].long_desc);
    fish->item_type	= ITEM_FOOD;
    fish->weight	= size;
    fish->cost		= (size * 15) + ch->level/2;
    fish->level		= 1;
    free_string(fish->material);
    fish->material	= str_dup( "food" );
    free_string(fish->guild);
    fish->guild 	= str_dup("Loner");
    fish->value[0]	= size;/*Maxsize*/
    fish->value[1]	= size;/*Pieces that can be nibbled*/
    fish->timer = number_range(10,UMAX(11,ch->level));
	if ( !CAN_WEAR(fish, ITEM_TAKE) ) /* set take if not set */
		fish->wear_flags = ITEM_TAKE;


	/* debugging routine */
	/*ptc( ch, "Size1: %d Size2: %d Size: %d\n\r", size1, size2, size );*/
	/*ptc( ch, "temp_val: %d\n\r", temp_val );
	ptc( ch, "fish_table[temp_val].name: %s\n\r", fish_table[temp_val].name );
	ptc( ch, "fish_table[temp_val].short_desc: %s\n\r", fish_table[temp_val].short_desc );
	ptc( ch, "fish_table[temp_val].long_desc: %s\n\r", fish_table[temp_val].long_desc );*/
    if(fish_table[temp_val].extraflags && FISH_EXTRA_POISONOUS)
    {
		fish->value[3] = 1;
	}

    act( buf, ch, fish->short_descr, NULL, TO_CHAR );
    act( "$n grabs a $t out of the river.\n\r", ch, fish->short_descr, NULL, TO_ROOM );/*should give the fishes shortdesc?*/
    obj_to_room(fish,ch->in_room);
	WAIT_STATE(ch,PULSE_VIOLENCE * timer);
}
