/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "tables.h"

int flag_lookup args( ( const char *name, const struct flag_type *flag_table) );
int	clan_lookup	args( (const char *name) );

void do_flag(CHAR_DATA *ch, char *argument)
{
	char arg1[MIL], arg2[MIL], arg3[MIL];
	char word[MAX_INPUT_LENGTH];
	CHAR_DATA *victim;
	long *flag, old = 0, new = 0, marked = 0, pos;
	char type;
	const struct flag_type *flag_table;

	argument = one_argument(argument,arg1);
	argument = one_argument(argument,arg2);
	argument = one_argument(argument,arg3);

	type = argument[0];

	if (type == '=' || type == '-' || type == '+')
		argument = one_argument(argument,word);

	if (arg1[0] == '\0')
	{
		stc("{cSyntax:\n\r",ch);
		stc("{c  flag mob  <name> <field> <flags>\n\r",ch);
		stc("{c  flag char <name> <field> <flags>\n\r",ch);
		stc("{c  mob  flags: act, aff, off, imm, res, vuln, form, part\n\r",ch);
		stc("{c  char flags: plr, comm, aff, imm, res, vuln,\n\r",ch);
		stc("{c              hist, spec, war\n\r",ch);
		stc("{c  +: add flag, -: remove flag, = set equal to\n\r",ch);
		stc("{c  otherwise flag toggles the flags listed.\n\r",ch);
		return;
	}

    if (arg2[0] == '\0')
    {
	stc("{cWhat do you wish to set flags on?\n\r",ch);
	return;
    }

    if (arg3[0] == '\0')
    {
	stc("{cYou need to specify a flag to set.\n\r",ch);
	return;
    }

    if (argument[0] == '\0')
    {
	stc("{cWhich flags do you wish to change?\n\r",ch);
	return;
    }

    if (!str_prefix(arg1,"mob") || !str_prefix(arg1,"char"))
    {
	victim = get_char_world(ch,arg2);
	if (victim == NULL)
	{
	    stc("{cYou can't find them.\n\r",ch);
	    return;
	}

        /* select a flag to set */
	if (!str_prefix(arg3,"act"))
	{
	    if (!IS_NPC(victim))
	    {
		stc("{cUse plr for PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->act;
	    flag_table = act_flags;
	}

	else if (!str_prefix(arg3,"auto"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->auto_flags;
	    flag_table = auto_flags;
	}

	else if (!str_prefix(arg3,"war"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->war_flags;
	    flag_table = war_flags;
	}

	else if (!str_prefix(arg3,"talent"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->talent_flags;
	    flag_table = talent_flags;
	}

	else if (!str_prefix(arg3,"ajah"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }
	    if (victim->clan != clan_lookup("aessedai"))
	    {
		stc("Only on Aes Sedai.\n\r",ch);
		return;
		}
		/*if (victim->ajah_flags != '\0')
		{
		stc("An Ajah flag is already set on this person.\n\r",ch);
		return;
		}*/
	    flag = &victim->ajah_flags;
	    flag_table = ajah_flags;
	}

	else if (!str_prefix(arg3,"spec"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->spec_flags;
	    flag_table = spec_flags;
	}

	else if (!str_prefix(arg3,"onepower"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->op_flags;
	    flag_table = op_flags;
	}

	else if (!str_prefix(arg3,"hist"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cOnly on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->hist_flags;
	    flag_table = hist_flags;
	}

	else if (!str_prefix(arg3,"plr"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cUse act for NPCs.\n\r",ch);
		return;
	    }

	    flag = &victim->act;
	    flag_table = plr_flags;
	}

 	else if (!str_prefix(arg3,"aff"))
	{
	    flag = &victim->affected_by;
	    flag_table = affect_flags;
	}

  	else if (!str_prefix(arg3,"immunity"))
	{
	    flag = &victim->imm_flags;
	    flag_table = imm_flags;
	}

	else if (!str_prefix(arg3,"resist"))
	{
	    flag = &victim->res_flags;
	    flag_table = imm_flags;
	}

	else if (!str_prefix(arg3,"vuln"))
	{
	    flag = &victim->vuln_flags;
	    flag_table = imm_flags;
	}

	else if (!str_prefix(arg3,"form"))
	{
	    if (!IS_NPC(victim))
	    {
	 	stc("{cForm can't be set on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->form;
	    flag_table = form_flags;
	}

	else if (!str_prefix(arg3,"parts"))
	{
	    if (!IS_NPC(victim))
	    {
		stc("{cParts can't be set on PCs.\n\r",ch);
		return;
	    }

	    flag = &victim->parts;
	    flag_table = part_flags;
	}

	else if (!str_prefix(arg3,"comm"))
	{
	    if (IS_NPC(victim))
	    {
		stc("{cComm can't be set on NPCs.\n\r",ch);
		return;
	    }

	    flag = &victim->comm;
	    flag_table = comm_flags;
	}

	else
	{
	    stc("{cThat's not an acceptable flag.\n\r",ch);
	    return;
	}

	old = *flag;
	victim->zone = NULL;

	if (type != '=')
	    new = old;

        /* mark the words */
        for (; ;)
        {
	    argument = one_argument(argument,word);

	    if (word[0] == '\0')
		break;

	    pos = flag_lookup(word,flag_table);

	    if (pos == NO_FLAG)
	    {
		stc("{cThat flag doesn't exist!\n\r",ch);
		return;
	    }
	    else
		SET_BIT(marked,pos);
	}

	for (pos = 0; flag_table[pos].name != NULL; pos++)
	{
	    if (!flag_table[pos].settable && IS_SET(old,flag_table[pos].bit))
	    {
		SET_BIT(new,flag_table[pos].bit);
		continue;
	    }

	    if (IS_SET(marked,flag_table[pos].bit))
	    {
		switch(type)
		{
		    case '=':
		    case '+':
			SET_BIT(new,flag_table[pos].bit);
			break;
		    case '-':
			REMOVE_BIT(new,flag_table[pos].bit);
			break;
		    default:
			if (IS_SET(new,flag_table[pos].bit))
			    REMOVE_BIT(new,flag_table[pos].bit);
			else
			    SET_BIT(new,flag_table[pos].bit);
		}
	    }
	}
	*flag = new;
	return;
    }
}




