/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aww.h"
#include "recycle.h"
#include "interp.h"
#include "tables.h"
#include "olc.h"

#define OBJ_VNUM_SHAPING_PRODUCT 909
#define OBJ_VNUM_SHAPING_MATERIAL 911

void do_forage(CHAR_DATA *ch, char *argument)
{
	if (IS_NPC(ch))	return;

	if (IS_SET(ch->act, PLR_FORAGE))
	{
		stc("You will no longer search for useful items while exploring.\n\r", ch);
		REMOVE_BIT(ch->act, PLR_FORAGE);
	}
	else
	{
		stc("You will now search for useful items while exploring.\n\r", ch);
		SET_BIT(ch->act, PLR_FORAGE);
	}

	return;
}

OBJ_DATA *random_herb ( void )
{
	OBJ_DATA *herb;
	int roll = number_range(1, MAX_HERB - 1);
	char buf[MSL];

	herb = create_object(get_obj_index(8), 0);

	herb->value[0] = roll;

	sprintf(buf, "%s herb", herb_flags[roll].name);
	free_string(herb->name);
	herb->name = str_dup(buf);

	sprintf(buf, "some %s", herb_flags[roll].name);
	free_string(herb->short_descr);
	herb->short_descr = str_dup(buf);

	sprintf(buf, "Some %s is lying here on the ground.", herb_flags[roll].name);
	free_string(herb->description);
	herb->description = str_dup(buf);

	free_string(herb->material);
	herb->material = str_dup(herb_flags[roll].name);
	
	return herb;
	
}

void forage ( CHAR_DATA *ch )
{
	int type = 0;
	bool found = FALSE;
	OBJ_DATA *obj;

    switch(ch->in_room->sector_type)
    {
    	default:                        type = 0; break;
		case(SECT_INSIDE):				type = 0; break;
		case(SECT_CITY):				type = 0; break;
		case(SECT_FIELD):				type = 1; break;
		case(SECT_FOREST):				type = 2; break;
		case(SECT_HILLS):				type = 3; break;
		case(SECT_MOUNTAIN):			type = 4; break;
		case(SECT_WATER_SWIM):			type = 0; break;
		case(SECT_WATER_NOSWIM):		type = 0; break;
		case(SECT_AIR):					type = 0; break;
	    case(SECT_TUNDRA):              type = 5; break;
	    case(SECT_BLIGHT):              type = 6; break;
	    case(SECT_COASTAL):             type = 7; break;
	    case(SECT_ROAD):                type = 1; break;
	    case(SECT_MARSH):               type = 8; break;
		case(SECT_DESERT):				type = 9; break;
		case(SECT_WASTE):               type = 9; break;
		case(SECT_LAKE_FRESH):          type = 10; break;
		case(SECT_RIVER_FRESH):         type = 10; break;
		case(SECT_LAKE_SALT):           type = 11; break; /* not used yet */
		case(SECT_RIVER_SALT):          type = 11; break; /* not used yet */
		case(SECT_VOLCANIC):            type = 12; break; /* not used yet */
    }

    if ( type == 0 )
	return;

	act("You scan the ground, looking for items of use.",ch,NULL,NULL,TO_CHAR);
	act("$n scans the ground, looking for items of use.",ch,NULL,NULL,TO_ROOM);

    if ( get_skill(ch, gsn_forage) / 2 > number_percent() && type > 0 )
    	found = TRUE;

    if ( type == 0 || !found )
    {
        check_improve(ch, gsn_forage, FALSE, 8);
	return;
    }

    if ( type == 1)
    {
		switch( number_range(1,8) )
		{
		case 1:
			act("You manage to find a stick.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a stick.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_STICK;
			obj->name = str_dup("stick");
			obj->short_descr = str_dup("a stick");
			obj->description = str_dup("Here lies a stick.");
			obj->material = str_dup("softwood");
			obj->weight = 5;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 2:
			act("You manage to find a honeycomb.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a honeycomb.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_FOOD;
			obj->name = str_dup("honeycomb");
			obj->short_descr = str_dup("a honeycomb");
			obj->description = str_dup("Here lies a honeycomb.");
			obj->material = str_dup("honey");
			obj->weight = 2;
			obj->cost = 0;
			obj->value[0] = 5;
			obj->value[1] = 5;
			obj->value[3] = 0;
			obj_to_char( obj, ch );
		break;
		case 3:
			act("You manage to find a mushroom.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a mushroom.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_MUSHROOM;
			obj->name = str_dup("mushroom");
			obj->short_descr = str_dup("a mushroom");
			obj->description = str_dup("Here lies a mushroom.");
			obj->material = str_dup("food");
			obj->weight = 2;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 4:
			obj = random_herb();
			obj->weight = 1;
			obj->cost = 0;
 
			act("You manage to find $p.",ch, obj, NULL,TO_CHAR);
			act("$n manages to find $p.",ch, obj, NULL,TO_ROOM);
			obj_to_char( obj, ch );
		break;
		default:
			act("You can only manage to find a few weeds and grasses.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}

	}
    else if ( type == 2 && found )
    {
    	switch( number_range( 1, 6 ) )
    	{
    	case 1:
			act("You manage to find a stick.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a stick.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_STICK;
			obj->name = str_dup("stick");
			obj->short_descr = str_dup("a stick");
			obj->description = str_dup("Here lies a stick.");
			obj->material = str_dup("softwood");
			obj->weight = 5;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
    	case 2:
			act("You manage to find a limb.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a limb.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_LIMB;
			obj->name = str_dup("limb");
			obj->short_descr = str_dup("a limb");
			obj->description = str_dup("Here lies a limb.");
			obj->material = str_dup("softwood");
			obj->weight = 10;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
    	case 3:
			act("You manage to find a branch.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a branch.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_BRANCH;
			obj->name = str_dup("branch");
			obj->short_descr = str_dup("a branch");
			obj->description = str_dup("Here lies a branch.");
			obj->material = str_dup("softwood");
			obj->weight = 20;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 4:
			act("You manage to find a mushroom.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a mushroom.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_MUSHROOM;
			obj->name = str_dup("mushroom");
			obj->short_descr = str_dup("a mushroom");
			obj->description = str_dup("Here lies a mushroom.");
			obj->material = str_dup("softwood");
			obj->weight = 2;
			obj->cost = 0;
			obj_to_char( obj, ch );
			break;
		case 5:
			obj = random_herb();
			obj->weight = 1;
			obj->cost = 0;
 
			act("You manage to find $p.",ch, obj, NULL,TO_CHAR);
			act("$n manages to find $p.",ch, obj, NULL,TO_ROOM);
			obj_to_char( obj, ch );
		break;

    		default:
			act("You can only manage to find a few leaves and twigs.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}
	}
    else if ( type == 3 && found )
    {
		switch( number_range(1,4) )
		{
		case 1:
			act("You manage to find a stick.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a stick.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_STICK;
			obj->name = str_dup("stick");
			obj->short_descr = str_dup("a stick");
			obj->description = str_dup("Here lies a stick.");
			obj->material = str_dup("softwood");
			obj->weight = 5;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 2:
			obj = random_herb();
			obj->weight = 1;
			obj->cost = 0;
 
			act("You manage to find $p.",ch, obj, NULL,TO_CHAR);
			act("$n manages to find $p.",ch, obj, NULL,TO_ROOM);
			obj_to_char( obj, ch );
		break;

		default:
			act("You can only manage to find a few clumps of dirt.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}
	}
    else if ( type == 4 && found )
    {
		switch( number_range(1,4 ) )
		{
		case 1:
			act("You manage to find a stick.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a stick.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_STICK;
			obj->name = str_dup("stick");
			obj->short_descr = str_dup("a stick");
			obj->description = str_dup("Here lies a stick.");
			obj->material = str_dup("softwood");
			obj->weight = 5;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 2:
			act("You manage to find a limb.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a limb.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_LIMB;
			obj->name = str_dup("limb");
			obj->short_descr = str_dup("a limb");
			obj->description = str_dup("Here lies a limb.");
			obj->material = str_dup("softwood");
			obj->weight = 5;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 3:
			obj = random_herb();
			obj->weight = 1;
			obj->cost = 0;
 
			act("You manage to find $p.",ch, obj, NULL,TO_CHAR);
			act("$n manages to find $p.",ch, obj, NULL,TO_ROOM);
			obj_to_char( obj, ch );
		break;

		default:
			act("You can only manage to find a few rocks and pebbles.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}

	}
    else if ( type == 5 && found )
    {
		act("You can only manage to find some snow.",ch,NULL,NULL,TO_CHAR);
		found = FALSE;
	}
    else if ( type == 6 && found )
    {
		act("You can only manage to find an old bone.",ch,NULL,NULL,TO_CHAR);
		found = FALSE;
	}
    else if ( type == 7 && found )
    {
		act("You can only manage to find some sand and seaweed.",ch,NULL,NULL,TO_CHAR);
		found = FALSE;
	}
    else if ( type == 8 && found )
    {
		switch( number_range(1,2 ) )
		{
		case 1:
			obj = random_herb();
			obj->weight = 1;
			obj->cost = 0;
 
			act("You manage to find $p.",ch, obj, NULL,TO_CHAR);
			act("$n manages to find $p.",ch, obj, NULL,TO_ROOM);
			obj_to_char( obj, ch );
		break;

		default:
			act("You can only manage to find some cattails.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}

	}
    else if ( type == 9 && found )
    {
		act("You can only manage to find sand.",ch,NULL,NULL,TO_CHAR);
		found = FALSE;
	}
    else if ( type == 10 && found )
    {
		switch( number_range( 1,7 ) )
		{
		case 1:
			act("You manage to find a mushroom.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a muchroom.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_MUSHROOM;
			obj->name = str_dup("mushroom");
			obj->short_descr = str_dup("a mushroom");
			obj->description = str_dup("Here lies a mushroom.");
			obj->material = str_dup("softwood");
			obj->weight = 2;
			obj->cost = 0;
			obj_to_char( obj, ch );
			break;
		case 2:
			act("You manage to find a clematis flower.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a clematis flower.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_CLEMATIS;
			obj->name = str_dup("clematis flower");
			obj->short_descr = str_dup("a clematis flower");
			obj->description = str_dup("Here lies a clematis flower.");
			obj->material = str_dup("softwood");
			obj->weight = 1;
			obj->cost = 0;
			obj_to_char( obj, ch );
			break;
		case 3:
			act("You manage to find some hawthorn berries.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find some hawthorn berries.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_HAWTHORN;
			obj->name = str_dup("hawthorn berries");
			obj->short_descr = str_dup("some hawthorn berries");
			obj->description = str_dup("Here lies some hawthorn berries.");
			obj->material = str_dup("softwood");
			obj->weight = 1;
			obj->cost = 0;
			obj_to_char( obj, ch );
			break;
		case 4:
			act("You manage to find some boneset flowers.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find some boneset flowers.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_BONESET;
			obj->name = str_dup("boneset flowers");
			obj->short_descr = str_dup("some boneset flowers");
			obj->description = str_dup("Here lies some boneset flowers.");
			obj->material = str_dup("softwood");
			obj->weight = 1;
			obj->cost = 0;
			obj_to_char( obj, ch );
			break;
    	case 5:
			act("You manage to find a stick.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a stick.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_STICK;
			obj->name = str_dup("stick");
			obj->short_descr = str_dup("a stick");
			obj->description = str_dup("Here lies a stick.");
			obj->material = str_dup("softwood");
			obj->weight = 5;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
    	case 6:
			act("You manage to find a limb.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a limb.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_LIMB;
			obj->name = str_dup("limb");
			obj->short_descr = str_dup("a limb");
			obj->description = str_dup("Here lies a limb.");
			obj->material = str_dup("softwood");
			obj->weight = 10;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
    	case 7:
			act("You manage to find a branch.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a branch.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_BRANCH;
			obj->name = str_dup("branch");
			obj->short_descr = str_dup("a branch");
			obj->description = str_dup("Here lies a branch.");
			obj->material = str_dup("softwood");
			obj->weight = 20;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 8:
			act("You manage to find a snake skin.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a snake skin.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_SNAKESKIN;
			obj->name = str_dup("snake skin");
			obj->short_descr = str_dup("a snake skin");
			obj->description = str_dup("Here lies a snake skin.");
			obj->material = str_dup("soft leather");
			obj->weight = 3;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 9:
			act("You manage to find a honeycomb.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a honeycomb.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_FOOD;
			obj->name = str_dup("honeycomb");
			obj->short_descr = str_dup("a honeycomb");
			obj->description = str_dup("Here lies a honeycomb.");
			obj->material = str_dup("honey");
			obj->weight = 2;
			obj->cost = 0;
			obj->value[0] = 5;
			obj->value[1] = 5;
			obj->value[3] = 0;
			obj_to_char( obj, ch );
		break;
		default:
			act("You can only manage to find some soggy leaves.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}
	}
    else if ( type == 12 && found )
    {
		switch( number_range( 1,2 ) )
		{
		case 1:
			act("You manage to find a sharp wedge of obsidian.",ch,NULL,NULL,TO_CHAR);
			act("$n manages to find a sharp wedge of obsidian.",ch,NULL,NULL,TO_ROOM);
			obj = create_object(get_obj_index(OBJ_VNUM_SHAPING_MATERIAL), 0 );
			obj->item_type = ITEM_SHAPE_OBSIDION;
			obj->name = str_dup("obsidian wedge");
			obj->short_descr = str_dup("an obsidian wedge");
			obj->description = str_dup("Here lies a sharp wedge of obsidian.");
			obj->material = str_dup("hard stone");
			obj->weight = 3;
			obj->cost = 0;
			obj_to_char( obj, ch );
		break;
		case 2:
			act("You can only manage to find some ash.",ch,NULL,NULL,TO_CHAR);
			found = FALSE;
		break;
		}
	}
	else
	{
		act("You can only manage to find some soggy leaves.",ch,NULL,NULL,TO_CHAR);
		found = FALSE;
	}

	if (found)	check_improve(ch, gsn_forage, TRUE, 6);
	else		check_improve(ch, gsn_forage, FALSE, 7);
	return;
}
