/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/* -----------------------------------------------------------------------
The following snippet was written by Gary McNickle (dharvest) for
Rom 2.4 specific MUDs and is released into the public domain. My thanks to
the originators of Diku, and Rom, as well as to all those others who have
released code for this mud base.  Goes to show that the freeware idea can
actually work. ;)  In any case, all I ask is that you credit this code
properly, and perhaps drop me a line letting me know it's being used.

from: gary@dharvest.com
website: http://www.dharvest.com
or http://www.dharvest.com/resource.html (rom related)

Send any comments, flames, bug-reports, suggestions, requests, etc...
to the above email address.
----------------------------------------------------------------------- */
/**  file   : guild.c
  *  author : Gary McNickle (gary@dharvest.com)
  *  purpose: Meant to provide a guild system configurable without
  *           recompiling. This is done via an ASCII file called "guild.dat"
  *           that you can edit or create with any text editor
  *           (vi/pico/jed/jaret/etc).  All gulid/clan related commands from
  *           ROM 2.4 have been moved to this file in an effort to keep
  *           related code located all in one spot.

  * relations[int]
  * relations take the integer o the clan associated with it so they can be added
  * or editted on the fly. -- Dalsor
  * same with chats... used for related guilds to talk to each other -- Dalsor
  * same with lists... -- Dalsor
  * also gcom and gcomooc set the channels which can be used -- Dalsor
  */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "tables.h"
#include "recycle.h"

// by prool:
char *player_rank(CHAR_DATA * ch, bool sub);
bool is_same_clan(CHAR_DATA * ch, CHAR_DATA * victim);
bool is_clan_name( CHAR_DATA *ch, char *argument );
bool is_clan(CHAR_DATA * ch);

/** Function: guild_bit_name
  * Descr   : Returns the ascii name of a GUILD_ bit vector.
  * Returns : (char *)
  * Syntax  : (n/a- called by system)
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
char *guild_bit_name( int guild_flags )
{
  static char buf[512];

  buf[0] = '\0';
  if ( guild_flags & GUILD_INDEPENDENT	) strcat( buf, " independent"	);
  if ( guild_flags & GUILD_CHANGED	) strcat( buf, " changed"	);
  if ( guild_flags & GUILD_DELETED	) strcat( buf, " deleted"	);
  if ( guild_flags & GUILD_IMMORTAL	) strcat( buf, " immortal"	);

  return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char * get_guild_command( CHAR_DATA *ch )
{
	return "";
}
void guild_talk( CHAR_DATA *ch, char *argument, int guild)
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    char *guild_channel;
    sh_int target_guild = ch->clan;
  /* sh_int guild: (0-> no guild-channel, 1->primary guild channel, 2->sub-guild channel, 3->primary OOC, 4->sub-OOC) */
    if(guild<1 || guild > 4)
    {
	    ptc(ch,"You aren't part of a guild %d\n\r", guild);
	    return;
    }

    if(guild == 2 || guild == 4)
	    target_guild = ch->sub_clan;

    if (!is_clan(ch))
    {
		stc("{cYou aren't in a clan.{c\n\r",ch);
		return;
    }
    if ( argument[0] == '\0' )
    {
        stc("Say what to your guild friends?\n\r",ch);
	    return;
    }

	if ( !str_cmp( argument, "last" ))
	{
		show_channel_logs(ch, "guild");
		return;
	}

    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
	    stc("{cThe Creator has revoked your channel priviliges.{c\n\r",ch);
	    return;
    }



    REMOVE_BIT(ch->comm,COMM_NOCLAN);

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
	    argument = makedrunk(argument,ch);


		    sprintf( buf, "{x%s%s %s%s%s%s %s{x",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				clan_table[target_guild].gcom,
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				ch->name,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );
		add_channel_log("guild", ch->name, clan_table[target_guild].gcom, buf);


	switch (guild)
	{
		default:
			bugf("Invalid guild number in guild_talk: %s %d %s",ch->name,guild,argument);
			return;
			break;
		case 1: case 2:
		    ptc( ch, "{x%s%s %sYou%s%s %s{x\n\r",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				clan_table[target_guild].gcom,
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

		    sprintf( buf, "{x%s%s %s$n%s%s %s{x",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				clan_table[target_guild].gcom,
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

			guild_channel = clan_table[target_guild].gcom;

		    break;
		case 3: case 4:
		    ptc( ch, "{x%s%s %sYou%s%s %s{x\n\r",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				clan_table[target_guild].gcomooc,
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

		    sprintf( buf, "{x%s%s %s$n%s%s %s{x",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				clan_table[target_guild].gcomooc,
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

		    guild_channel = clan_table[target_guild].gcomooc;
		    break;
	}

/*
	ptc( ch, "[%s] %s{x\n\r", clan_table[ch->clan].gcom, argument );
    sprintf( buf, "{m[{M%s {M({m$n{M){m] {m'{M%s{m'{M{x", player_rank(ch), argument );
*/
    for ( d = descriptor_list; d != NULL; d = d->next )
    {
        if ( d != NULL
        && d->character != NULL
        && d->connected == CON_PLAYING
        && d->character != ch
        && !IS_SET(d->character->comm,COMM_NOCLAN)
        && !IS_SET(d->character->comm,COMM_QUIET)
	&& d->character->pcdata->newbie != 1 )
        {
		if (is_clan_name(ch, "ashaman") && !is_clan_name(d->character, "ashaman"))	continue;
		if (is_white_tower(ch) && !is_white_tower(d->character))			continue;

		if (  d->character->clan == target_guild || ( d->character->pcdata && (d->character->sub_clan == target_guild)) )
                	act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );
			else
			{
				if(
					(clan_table[d->character->clan].gcom
						&& !str_cmp(guild_channel,clan_table[d->character->clan].gcom))
					|| (d->character->pcdata
						&& clan_table[d->character->sub_clan].gcom
						&& !str_cmp(guild_channel,clan_table[d->character->sub_clan].gcom))
					|| (clan_table[d->character->clan].gcomooc
						&& !str_cmp(guild_channel,clan_table[d->character->clan].gcomooc))
					|| (d->character->pcdata
						&& clan_table[d->character->sub_clan].gcomooc
						&& !str_cmp(guild_channel,clan_table[d->character->sub_clan].gcomooc)  )
					)
			                        act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );
			}
        }
    }
	return;
}
void clan_talk( CHAR_DATA *ch, char *argument, int guild)
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    int i;
    sh_int target_guild = ch->clan;
  /* sh_int guild: (0-> no guild-channel, 1->primary guild channel, 2->sub-guild channel, 3->primary OOC, 4->sub-OOC) */
    if(guild<1 || guild > 4)
    {
	    ptc(ch,"You aren't part of a guild %d\n\r", guild);
	    return;
    }
    if(guild == 2 || guild == 4)
	    target_guild = ch->sub_clan;

    if (!is_clan(ch))
    {
		stc("{cYou aren't in a clan.{c\n\r",ch);
		return;
    }
    if ( argument[0] == '\0' )
    {
        stc("Say what to your guild friends?\n\r",ch);
	    return;
    }

	if ( !str_cmp( argument, "last" ))
	{
		show_channel_logs(ch, "clan");
		return;
	}


    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
	    stc("{cThe Creator has revoked your channel priviliges.{c\n\r",ch);
	    return;
    }

    REMOVE_BIT(ch->comm,COMM_NOCLAN);

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
	    argument = makedrunk(argument,ch);

		    sprintf( buf, "{x%s%s %s%s%s%s %s{x",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				player_rank(ch, guild == 4),
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				ch->name,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );
		add_channel_log("clan", ch->name, clan_table[target_guild].name, buf);



	switch (guild)
	{
		default: break;


		case 1: case 2:
		    ptc( ch, "{x%s%s %sYou%s%s %s{x\n\r",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				player_rank(ch, guild == 2),
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

		    sprintf( buf, "{x%s%s %s$n%s%s %s{x",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				player_rank(ch, guild == 2),
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

		    break;
		case 3: case 4:
		    ptc( ch, "{x%s%s OOC %sYou%s%s %s{x\n\r",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				player_rank(ch, guild == 4),
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );

		    sprintf( buf, "{x%s%s OOC %s$n%s%s %s{x",
				clan_table[target_guild].lefty != NULL
				&& clan_table[target_guild].lefty[0] != '\0' ? clan_table[target_guild].lefty : "<" ,
				player_rank(ch, guild == 4),
				clan_table[target_guild].inner != NULL
				&& clan_table[target_guild].inner[0] != '\0' ? clan_table[target_guild].inner : "(" ,
				clan_table[target_guild].outer != NULL
				&& clan_table[target_guild].outer[0] != '\0' ? clan_table[target_guild].outer : ")" ,
				clan_table[target_guild].righty != NULL
				&& clan_table[target_guild].righty[0] != '\0' ? clan_table[target_guild].righty : ">",
				argument );



/*	Ugly? Set it to use inner/outter/etc -  Rokann
		case 1: case 2:
		    ptc( ch, "{m[{M%s{m] {m'{M%s{m'{x\n\r", player_rank(ch), argument );
		    sprintf( buf, "{m[{M%s {M({m$n{M){m] {m'{M%s{m'{M{x", player_rank(ch), argument );
		    break;
		case 3: case 4:
		    ptc( ch, "{m[{M%s{m]{MOOC {m'{M%s{m'{x\n\r", player_rank(ch), argument );
		    sprintf( buf, "{m[{M%s {M({m$n{M){m]{MOOC {m'{M%s{m'{x", player_rank(ch), argument );
		    break;
*/
	}




    for ( d = descriptor_list; d != NULL; d = d->next )
    {
        if ( d != NULL
        && d->character != NULL
        && d->connected == CON_PLAYING
        && d->character != ch
        && !IS_SET(d->character->comm,COMM_NOCLAN)
        && !IS_SET(d->character->comm,COMM_QUIET)
	&& d->character->pcdata->newbie != 1 )
        {

			if (  d->character->clan == target_guild || ( d->character->pcdata && (d->character->sub_clan == target_guild)) )
                		act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );
			else
			{
				for ( i = 0; i < MAX_GUILD_RELATIONS; i++ )
				{
					if ( clan_table[target_guild].chats[i] != NULL
					&& clan_table[target_guild].chats[i][0] != '\0'
					&&(!str_cmp( clan_table[target_guild].chats[i], clan_table[d->character->clan].name )
					|| (d->character->pcdata && !str_cmp( clan_table[target_guild].chats[i], clan_table[d->character->sub_clan].name )) ))
			                        act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );
				}
			}
        }
    }
    return;
}
/*
void clan_talk( CHAR_DATA *ch, char *argument, sh_int guild)
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    int i;

    if (!is_clan(ch))
    {
		stc("{cYou aren't in a clan.{c\n\r",ch);
		return;
    }
    if ( argument[0] == '\0' )
    {
        stc("Say what to your Clan members?\n\r",ch);
	    return;
    }

    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
	    stc("{cThe Creator has revoked your channel priviliges.{c\n\r",ch);
	    return;
    }

    REMOVE_BIT(ch->comm,COMM_NOCLAN);

	if (!IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK] > 10)
    argument = makedrunk(argument,ch);
      if(guild == ch->clan)
	ptc( ch, "{m[{M%s{m] {m'{M%s{m'{M{x\n\r",capit2(clan_table[ch->clan].name), argument );
      else if(ch->pcdata)
	ptc( ch, "{m[{M%s{m] {m'{M%s{m'{M{x\n\r",capit2(clan_table[ch->sub_clan].name), argument );

    sprintf( buf, "{m[{M%s {M({m$n{M){m] {m'{M%s{m'{M{x", player_rank(ch), argument );

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
        if ( d != NULL
        && d->character != NULL
        && d->connected == CON_PLAYING
        && d->character != ch
        && !IS_SET(d->character->comm,COMM_NOCLAN)
        && !IS_SET(d->character->comm,COMM_QUIET))
        {
			if ( !is_same_clan( ch, d->character ) )
			{
				for ( i = 0; i < MAX_GUILD_RELATIONS; i++ )
				{
					if ( clan_table[ch->clan].chats[i] != NULL
					&& clan_table[ch->clan].chats[i][0] != '\0'
					&& !str_cmp( clan_table[ch->clan].chats[i], clan_table[d->character->clan].name ) )
                        act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );
				}
			}
			else if ( is_same_clan( ch, d->character ) )
                act_new( buf, ch, argument, d->character, TO_VICT, POS_DEAD );
			else
			;
        }
    }
}
*/
void do_clantalk( CHAR_DATA *ch, char *argument )
{
    if (!is_clan(ch))
    {
		stc( "You aren't in a guild.\n\r", ch );
		return;
    }
    if ( argument[0] == '\0' )
    {
        stc("Say what to your guild?\n\r", ch );
	    return;
    }

    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
	    stc("The Creator has revoked your channel priviliges.\n\r",ch);
	    return;
    }



	if (is_aiel_clan(ch) || is_shadow(ch) || is_white_tower(ch) || is_seanchan_clan(ch))
	clan_talk( ch, argument, 1 );
	else
	guild_talk( ch, argument, 1 );
	return;
}

/** Function: load_guilds
  * Descr   : Loads the information specific to each guild/clan from
  *           the ../data/guild.dat file.
  * Returns : (n/a)
  * Syntax  : (n/a)
  * Written : v1.0 1/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

void load_guilds(void)
{
    FILE *fp;
    char buf[MAX_STRING_LENGTH];
    char *word;
    int count = 0;
    int i;
    bool fMatch = FALSE;

    for (i=0; i < MAX_CLAN; i++)
    {
		clan_table[i].name = str_dup("");
		clan_table[i].who_name = str_dup("");
		clan_table[i].room[0]= 3001;
		clan_table[i].room[1]= 3721;
		clan_table[i].room[2]= 3001;
		clan_table[i].rank[0].rankname = str_dup("");
		clan_table[i].rank[0].skillname = str_dup("");
		clan_table[i].ml[0] = 0;
		clan_table[i].ml[1] = 0;
		clan_table[i].ml[2] = 0;
		clan_table[i].ml[3] = 0;
		clan_table[i].flags = 0;
    }

    sprintf(buf, "%sguild.dat", DATA_DIR);

    if ((fp = fopen(buf, "r")) == NULL)
    {
		log_string("Error: guild.dat file not found!");
		exit(1);
    }
    for (;;)
    {
		word = feof(fp) ? "End" : fread_word(fp);

		if (!str_cmp(word, "End"))
		    break;

		switch (UPPER(word[0]))
		{
			case 'C':
				if ( !str_cmp( word, "Chat" ) )
				{
					i = fread_number( fp );
					clan_table[count].chats[i-1] = fread_string( fp );
					fMatch = TRUE;
				}
			break;
			case 'F':
				clan_table[count].flags  = fread_flag( fp );
				fMatch = TRUE;
				break;

			case 'G':
				if ( !str_cmp( word, "GCom" ) )
				{
					clan_table[count].gcom = fread_string( fp );
					fMatch = TRUE;
					break;
				}
				if ( !str_cmp( word, "GComOOC" ) )
				{
					clan_table[count].gcomooc = fread_string( fp );
					fMatch = TRUE;
					break;
				}
				else if ( !str_cmp( word, "GList" ) )
				{
					i = fread_number( fp );
					clan_table[count].lists[i-1] = fread_string( fp );
					fMatch = TRUE;
					break;
				}
				count++;
				clan_table[count].name = fread_string(fp);
				fMatch = TRUE;
			break;

			case 'I':
				if ( !str_cmp( word, "Inner") )
				{
					clan_table[count].inner = fread_string(fp);
					fMatch = TRUE;
				}
			break;

			case 'L':
				if ( !str_cmp( word, "Lefty") )
				{
					clan_table[count].lefty = fread_string(fp);
					fMatch = TRUE;
				}
			break;

			case 'O':
				if ( !str_cmp( word, "Outer") )
				{
					clan_table[count].outer = fread_string(fp);
					fMatch = TRUE;
				}
			break;

			case 'R':
				if (!str_cmp( word, "Rooms"))
				{
					clan_table[count].room[0] = fread_number(fp);	/* hall   */
					clan_table[count].room[1] = fread_number(fp);	/* morgue */
					clan_table[count].room[2] = fread_number(fp);	/* temple */
					fMatch = TRUE;
				}
				else if (!str_cmp( word, "Righty"))
				{
					clan_table[count].righty = fread_string(fp);
					fMatch = TRUE;
				}
				else if (!str_cmp( word, "Rank"))
				{
					i = fread_number(fp);
					clan_table[count].rank[i - 1].rankname = fread_string(fp);
					fMatch = TRUE;
				}
				else if ( !str_cmp( word, "Rel" ) )
				{
					i = fread_number( fp );
					clan_table[count].related[i-1] = fread_string( fp );
					fMatch = TRUE;
				}
				break;

			case 'S':
				i = fread_number(fp);
				clan_table[count].rank[i - 1].skillname = fread_string(fp);
				fMatch = TRUE;
				break;

			case 'M':
				clan_table[count].ml[0] = fread_number(fp);
				clan_table[count].ml[1] = fread_number(fp);
				clan_table[count].ml[2] = fread_number(fp);
				clan_table[count].ml[3] = fread_number(fp);
				fMatch = TRUE;
				break;

			case 'N':
				if ( !str_cmp( word, "NGL" ) )
				{
					clan_table[count].gl = fread_string( fp );
					fMatch = TRUE;
				}
				else if ( !str_cmp( word, "NSeer" ) )
				{
					clan_table[count].seer = fread_string( fp );
					fMatch = TRUE;
				}

			case 'T':
				if ( !str_cmp( word, "Tag" ) )
				{
					clan_table[count].tag = fread_string(fp);
					fMatch = TRUE;
					break;
				}

			case 'W':
				if ( !str_cmp( word, "Who" ) )
				{
					clan_table[count].who_name = fread_string(fp);
					fMatch = TRUE;
					break;
				}

		}
	}

	if (!fMatch)
	{
		bug("Fread_guilds: no match.", 0);
		fread_to_eol(fp);
	}
	fclose(fp);
	return;
}



/** Function: is_leader
  * Descr   : Validates guild leadership of (ch).
  * Returns : True (yes, ch is a mortal leader) or False
  * Syntax  : (n/a)
  * Written : v1.1 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

bool is_leader(CHAR_DATA * ch)
{
    return IS_SET(ch->hist_flags, HIST_GUILDLEADER) ? 1 : 0;
}

/** Function: is_recruiter
  * Descr   : Validates guild leadership of (ch).
  * Returns : True (yes, ch is a mortal leader) or False
  * Syntax  : (n/a)
  * Written : v1.1 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

bool is_recruiter(CHAR_DATA * ch)
{
    return IS_SET( ch->hist_flags, HIST_RECRUITER) ? 1 : 0;
}



/** Function: can_guild
  * Descr   : Validates ability of (ch) to guild other players into
  *         : a specific (or generic, if implementor) guild.
  * Returns : TRUE/FALSE
  * Syntax  : (n/a)
  * Written : v1.1 2/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

bool can_guild(CHAR_DATA * ch)
{
    if (IS_IMMORTAL(ch))
		return TRUE;

    if (is_leader(ch))
	    return TRUE;

    if (ch->clan == 0 || !is_recruiter(ch))
		return FALSE;

    return clan_table[ch->clan].ml[0];
}



/** Function: can_deguild
  * Descr   : Validates ability of (ch) to de-guild players from a
  *         : particular guild, or generic (if ch=implementor)
  * Returns : TRUE/FALSE
  * Syntax  : (n/a)
  * Written : v1.1 2/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

bool can_deguild(CHAR_DATA * ch)
{
    if ( IS_IMMORTAL( ch ) )
		return TRUE;
    if (is_leader(ch))
	    return TRUE;

    if (ch->clan == 0 || !is_recruiter(ch))
		return FALSE;

    return clan_table[ch->clan].ml[1];
}



/** Function: can_promote
  * Descr   : Validates ability of (ch) to promote players in their guilds
  * Returns : True/False
  * Syntax  : (n/a)
  * Written : v1.1 2/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

bool can_promote(CHAR_DATA * ch)
{
    if ( IS_IMMORTAL( ch ) )
		return TRUE;

    if ( is_leader(ch) )
	return TRUE;

    if (ch->clan == 0 || !is_recruiter(ch))
		return FALSE;

    return clan_table[ch->clan].ml[2];
}



/** Function: can_demote
  * Descr   : Validates ability of (ch) to demote a player within their guild.
  * Returns : True/False
  * Syntax  : (n/a)
  * Written : v1.1 2/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

bool can_demote(CHAR_DATA * ch)
{
    if ( IS_IMMORTAL( ch ) )
		return TRUE;

    if ( is_leader(ch) )
	return TRUE;

    if (ch->clan == 0 || !is_recruiter(ch))
		return FALSE;

    return clan_table[ch->clan].ml[3];
}




/** Function: is_clan
  * Descr   : Returns wether or not (ch) is a member of any guild/clan
  * Returns : True/False
  * Syntax  : (n/a)
  * Written : v1.0 ?
  * Author  : Rom Consortium.
  */

bool is_clan(CHAR_DATA * ch)
{
	if(ch->clan == clan_lookup("loner") || ch->clan <= 0)
			return FALSE;

	return TRUE;
}


bool is_same_guild(CHAR_DATA *ch, CHAR_DATA *victim)
{
	int i;

	if (is_same_clan(ch, victim))	return TRUE;

        for (i = 0; i < MAX_GUILD_RELATIONS; i++)
        {
		if (clan_table[ch->clan].lists[i] != NULL
		&& (!str_cmp(clan_table[ch->clan].lists[i], clan_table[victim->clan].name)
		|| !str_cmp(clan_table[ch->clan].lists[i], clan_table[victim->sub_clan].name)))
			return TRUE;

		if (clan_table[ch->sub_clan].lists[i] != NULL
		&& (!str_cmp(clan_table[ch->sub_clan].lists[i], clan_table[victim->clan].name)
		|| !str_cmp(clan_table[ch->sub_clan].lists[i], clan_table[victim->sub_clan].name)))
			return TRUE;
        }

	return FALSE;
}

/** Function: is_same_clan
  * Descr   : Returns wether or not (ch) and (victim) are members of
  *         : the same guild/clan
  * Returns : True/False
  * Syntax  : (n/a)
  * Written : v1.2 1/98
  * Author  : Rom Consortium. Updated by Gary McNickle <gary@dharvest.com>
  */

bool is_same_clan(CHAR_DATA * ch, CHAR_DATA * victim)
{
    if (IS_SET(clan_table[ch->clan].flags,GUILD_INDEPENDENT))
		return FALSE;

	if (ch->clan == victim->clan)		return TRUE;
	if (ch->clan == victim->sub_clan)	return TRUE;
	if (ch->sub_clan == victim->clan)	return TRUE;
	if (ch->sub_clan == victim->sub_clan)	return TRUE;

	return FALSE;
}

/** Function: is_this_clan
  * Descr   : Returns whether or not (victim) is a member of
  *         : the guild/clan
  * Returns : True/False
  * Syntax  : (n/a)
  * Written : v1.0 6/03
  * Author  : Silmandil
  */
bool is_this_clan(sh_int t_clan, CHAR_DATA * victim)
{
    if (IS_SET(clan_table[t_clan].flags,GUILD_INDEPENDENT))
		return FALSE;
    else
		return (t_clan == victim->clan||t_clan == victim->sub_clan);
}



/** Function: is_clan_name
  * Descr   : Returns whether the argument is the name of a clan within the
  *         : clan_table to be used in clantalk checks
  * Returns : (True/False)
  * Syntax  : (n/a)
  * Written : v1.0
  * Author  : Dalsor
  */

bool is_clan_name( CHAR_DATA *ch, char *argument )
{
    int clan;

    for (clan = 0; clan < MAX_CLAN; clan++)
    {
		if (str_prefix( argument, clan_table[clan].name ))	continue;
		if (ch->clan != clan && ch->sub_clan != clan)		continue;

		return TRUE;
    }
    return FALSE;
}

// Is this a clan name? Loner doesn't count.
bool is_arg_clan (char *argument)
{
    int clan = clan_lookup(argument);

    if (clan <= 0)
	return FALSE;

    if (IS_SET(clan_table[clan].flags, GUILD_INDEPENDENT))
	return FALSE;

    return TRUE;
}

bool is_obj_clan( OBJ_DATA *obj )
{
    int clan = clan_lookup(obj->guild);

    if (clan <= 0)
	return FALSE;

    if (IS_SET(clan_table[clan].flags, GUILD_INDEPENDENT))
	return FALSE;

    return TRUE;
}

/** Function: is_subguild_name
  * Descr   : Returns whether the argument is the name of a clan within the
  *         : clan_table to be used in clantalk checks
  * Returns : (True/False)
  * Syntax  : (n/a)
  * Written : v1.0
  * Author  : Dalsor
  */

bool is_subguild_name( CHAR_DATA *ch, char *argument )
{
    int clan;
    if(IS_NPC(ch)||!ch->pcdata)
	    return FALSE;

    for (clan = 0; clan < MAX_CLAN; clan++)
    {
		if ( !str_prefix( argument, clan_table[clan].name ) && ch->sub_clan == clan )
		    return TRUE;
    }
    return FALSE;
}

/** Function: is_multi_guild_channel
  * Descr   : Returns whether the argument is the channel of multiple guilds
  * Returns : (0-> no guild-channel, 1->primary guild channel, 2->sub-guild channel, 3->primary OOC, 4->sub-OOC)
  * Syntax  : (n/a)
  * Written : v1.0
  * Author  : Dalsor
  */

int is_multi_guild_channel( CHAR_DATA *ch, char *argument )
{
    int clan;
    if( IS_NPC(ch) || !ch->pcdata)
	    return 0;

    if (IS_SET(clan_table[ch->clan].flags,GUILD_INDEPENDENT))
		return 0;

    for (clan = 0; clan < MAX_CLAN; clan++)
    {
	    if(clan_table[clan].gcom)
	    {
		if (  !str_prefix( argument, clan_table[clan].gcom )
		&&  ch->clan == clan  )
		    return 1;
		if (  !str_prefix( argument, clan_table[clan].gcom )
		&& ch->sub_clan == clan )
		    return 2;
	    }
	    if(clan_table[clan].gcomooc)
	    {
		if ( !str_prefix( argument, clan_table[clan].gcomooc )
		&&  ch->clan == clan  )
		    return 3;
		if ( !str_prefix( argument, clan_table[clan].gcomooc )
		&&  ch->sub_clan == clan  )
		    return 4;
	    }
    }
    return 0;
}
/** Function: is_guild_channel
  * Descr   : Returns whether the argument is the channel of a guild
  * Returns : (0-> no guild-channel, 1->primary guild channel, 2->sub-guild channel, 3->primary OOC, 4->sub-OOC)
  * Syntax  : (n/a)
  * Written : v1.0
  * Author  : Dalsor
  */

int is_guild_channel( CHAR_DATA *ch, char *argument )
{
    int clan;
    char test_clan[MSL];
    if( IS_NPC(ch) || !ch->pcdata)
	    return 0;

    if (IS_SET(clan_table[ch->clan].flags,GUILD_INDEPENDENT))
		return 0;

    for (clan = 0; clan < MAX_CLAN; clan++)
    {
	    sprintf(test_clan, "%sooc", clan_table[clan].name);
		if (  !str_prefix( argument, clan_table[clan].name)
		&&  ch->clan == clan  )
		    return 1;
		if (  !str_prefix( argument, clan_table[clan].name)
		&& ch->sub_clan == clan )
		    return 2;
		if ( !str_prefix( argument, test_clan )
		&&  ch->clan == clan  )
		    return 3;
		if ( !str_prefix( argument, test_clan )
		&&  ch->sub_clan == clan  )
		    return 4;
		/*
		if ( ( !str_cmp( argument, clan_table[clan].gcom )
		|| !str_cmp( argument, clan_table[clan].gcomooc ) )
		&& ( ch->clan == clan || ch->sub_clan == clan ) )
		    return TRUE;
		    */
    }
    return 0;
}

/** Function: clan_lookup
  * Descr   : Returns the numeric position of a clan name within the
  *         : clan_table
  * Returns : (int)
  * Syntax  : (n/a)
  * Written : v1.0
  * Author  : Rom Consortium
  */

int clan_lookup(const char *name)
{
    int clan;

    for (clan = 0; clan < MAX_CLAN; clan++)
    {
		if (!str_prefix(name, clan_table[clan].name))
		    return clan;
    }

    return 0;
}

/** Function: player_rank
  * Descr   : Returns the players (ch) rank as a string.
  * Returns : char*
  * Syntax  : (n/a)
  * Written : v1.0 1/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

char *player_rank(CHAR_DATA * ch, bool sub)
{
    if (ch->clan == 0 || ch->rank == -1)
	return str_dup("");

	if (sub)
		return clan_table[ch->sub_clan].rank[ch->sub_rank].rankname;
	else	return clan_table[ch->clan].rank[ch->rank].rankname;
}


/** Function: player_clan
  * Descr   : Returns the name of the players (ch) clan as a string
  * Returns : char*
  * Syntax  : (n/a)
  * Written : v1.0 1/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

char *player_clan(CHAR_DATA * ch)
{
    if (ch->clan == 0)
	return '\0';
    return clan_table[ch->clan].name;
}

/** Function: do_promote
  * Descr   : Promotes (or demotes) a character within their own guild
  * Returns : List of ranks if given no argument
  * Syntax  : promote <who> <rank #>
  * Written : v1.5 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

void do_promote(CHAR_DATA * ch, char *argument)
{
    char arg1[MAX_INPUT_LENGTH], arg2[MIL];
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    int cnt;
    int sn = 0;
    bool sub = FALSE;
    int vclan, vrank;

    argument = one_argument(argument, arg1);
    argument = one_argument(argument, arg2);

    if (!can_promote(ch))
    {
	send_to_char("Huh?\n\r", ch);
	return;
    }

    if (arg1[0] == '\0')
	victim = ch;
    else if ((victim = get_char_world(ch, arg1)) == NULL)
    {
	send_to_char("They aren't playing.\n\r", ch);
	return;
    }

    if (argument[0] != '\0' && !str_cmp(argument, "sub"))
	sub = TRUE;

    if (!is_clan(victim))
    {
	if (victim == ch)	send_to_char("You aren't in a guild!\n\r", ch);
	else			send_to_char("They are not a member of any guilds!\n\r", ch);
        return;
    }

	vclan = sub ? victim->sub_clan : victim->clan;
	vrank = sub ? victim->sub_rank : victim->rank;

    if (!is_same_guild(ch, victim) && !IS_IMMORTAL( ch ) )
    {
	send_to_char("They are a member of a guild different than yours!\n\r", ch);
	return;
    }

    if (arg2[0] == '\0')
    {
	/*
	 * --------------------------------------------------------------
	 * Keep in mind that we are displaying the ranks as 1 - MAX_RANK,
	 * so, since the structure is actually 0 - MAX_RANK-1, we need to
	 * set "cnt" to cnt-1.
	 * --------------------------------------------------------------
	 */

	send_to_char("Syntax: promote <who> <rank #>\n\r", ch);
	send_to_char("where rank is one of the following:\n\r", ch);
	send_to_char("Recruiter (toggle)\n\r", ch);

	for (cnt = 0; cnt < MAX_RANK; cnt++)
	{
	  sprintf(buf, "%2d] %s\n\r", cnt + 1,
 	    is_clan(victim) ? clan_table[victim->clan].rank[cnt].rankname : "(None)");
	  send_to_char(buf, ch);
	}
	send_to_char("\n\r", ch);
	return;
    }

    if (!str_prefix(arg2, "recruiter") && ch != victim)
    {
	if(!is_recruiter(victim) )
	{
	    SET_BIT(victim->hist_flags, HIST_RECRUITER);
	    send_to_char("They are now a recruiter.\n\r", ch);
	    send_to_char("You have just been promoted to a recruiter of your guild!\n\r", victim);
	    return;
	}
	else
	{
		REMOVE_BIT(victim->hist_flags, HIST_RECRUITER);
		stc("They are no longer a recruiter.\n\r", ch);
		stc("You are no longer a recruiter of your guild.\n\r", victim);
		return;
	}
    }

    cnt = atoi(arg2) - 1;


    if (cnt < 0 ||
	cnt > MAX_RANK -1 ||
	clan_table[vclan].rank[cnt].rankname == NULL)
    {
	send_to_char("That rank does not exist!\n\r", ch);
	return;
    }

    if (cnt > victim->rank && ((ch == victim) & (!IS_IMMORTAL(ch)) &
(!IS_SET(victim->hist_flags, HIST_GUILDLEADER))))
    {
	send_to_char("Heh. I dont think so...\n\r", ch);
	return;
    }

    if (cnt > vrank)
    {
	int i;

	sprintf(buf, "You have been promoted to %s!\n\r",
		clan_table[vclan].rank[cnt].rankname);
	send_to_char(buf, victim);

	sprintf(buf, "%s has been promoted to %s!\n\r",
	      victim->name, clan_table[vclan].rank[cnt].rankname);
	send_to_char(buf, ch);

	for (i = vrank; i < cnt; i++)
	    if (clan_table[vclan].rank[i].skillname != NULL)
	    {
		sn = skill_lookup(clan_table[vclan].rank[i].skillname);
		if (sn < 0)
		{
		    sprintf(buf, "Bug: Add skill [%s] is not a valid skill",
			    clan_table[vclan].rank[cnt].skillname);
		    log_string(buf);
		} else if (!victim->pcdata->learned[sn])
		    victim->pcdata->learned[sn] = 20 + (victim->level / 4);
	    }
    }

    else if (cnt < vrank)
    {

	sprintf(buf, "You have been demoted to %s!\n\r",
		clan_table[vclan].rank[cnt].rankname);

	send_to_char(buf, victim);
	sprintf(buf, "%s has been demoted to %s!\n\r",
	      victim->name, clan_table[vclan].rank[cnt].rankname);

	send_to_char(buf, ch);
	/*
	 * ---------------------------------------------------------------
	 * Note: I dont think it would be fair here to take away any skills
	 * the victim may have earned at a higher rank. It makes no RP sense
	 * to do so and only hurts the player (loss of practices etc). Imms
	 * may want to keep an eye on this, as we dont want players jumping
	 * guilds just to gain new skills.
	 * --------------------------------------------------------------
	 */
    }				/* else no change */

	if (sub)
	victim->sub_rank = cnt;
	else
	victim->rank = cnt;

    compose_title( victim );
    return;
} /* end: do_promote */


/** Function: do_guild
  * Descr   : Adds a character to the membership of a guild, or optionally,
  *         : removes them from a guild.
  * Returns : (n/a)
  * Syntax  : guild <who> <clan name>
  * Written : v1.3 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */

//void check_mysql_guild args(( CHAR_DATA *ch ));

void do_guild(CHAR_DATA * ch, char *argument)
{
    char arg1[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *victim;
    int clan;

    argument = one_argument(argument, arg1);
    argument = one_argument(argument, arg2);

    if ( arg1[0] == '\0' || !can_guild(ch))
    {
	stc("See also: {Rgcom list{x\n\r", ch);
	do_function( ch, &do_gcom, "list" );

	if (can_guild(ch))
	{
		stc("\n\rFor the guild list : guild list <all>\n\r", ch);
		stc("To guild/deguild   : guild <char> <guild name>\n\r", ch);
	}
	return;
    }
    if ( !str_prefix( arg1, "list" ) )
    {
	if ( !str_prefix( arg2, "all" ) || !str_prefix( arg2, "full" ) )
	{
		char buff[MIL];
		BUFFER *buffer;
		int i;

		buffer = new_buf();

		sprintf(buff, "Guild Name  | Tag | Channels (Who Name in Color)\n\r-----------------------------------------------\n\r");
		add_buf(buffer, buff);

		for (i=1; i <= MAX_CLAN; i++)
		{
			if (clan_table[i].name != NULL && clan_table[i].name[0] != '\0')
			{
				sprintf(buff,"%12s %5s %s%s%s %s%s%s %s\n\r",
                  			clan_table[i].name,
                  			clan_table[i].tag,
					clan_table[i].lefty, clan_table[i].gcom, clan_table[i].righty,
					clan_table[i].lefty, clan_table[i].gcomooc, clan_table[i].righty,
					clan_table[i].who_name );
 					add_buf(buffer, buff);
  			}
		}
		page_to_char( buf_string(buffer), ch );
		free_buf(buffer);
		return;
	}
	else
	{
		DESCRIPTOR_DATA *d;
		bool found = FALSE;
		int count = 1;

		stc("See also: gcom list\n\r", ch);
		stc("See also: guild list all\n\r", ch);

		for ( d = descriptor_list; d != NULL; d = d->next )
		{
			if ( d->character
			&& d->connected == CON_PLAYING
			&& can_see( ch, d->character )
			&& is_same_guild(ch,d->character) )
				ptc( ch, "[%s{x]{w %s%s %s\n\r{x",
					clan_table[d->character->clan].rank[d->character->rank].rankname,
					IS_SET(d->character->comm, COMM_AFK) ? "{c[{wAFK{c]{w " : "",
					d->character->name, d->character->pcdata->last_name );
				found = TRUE;
				count ++;
		}
		if ( !found )
			stc( "There's only you at the moment.\n\r", ch );
		return;
	}
    }
    if (arg1[0] == '\0' || arg2[0] == '\0')
    {
		send_to_char("Syntax: guild <char> <guild name>\n\r", ch);
		return;
    }
    if ((victim = get_pc_world(ch, arg1)) == NULL)
    {
		send_to_char("They aren't playing.\n\r", ch);
		return;
    }

    if ( is_clan( victim )
    && !is_same_guild( ch, victim )
    && !IS_IMMORTAL( ch ) )
    {
		send_to_char("They are a member of a guild other than your own, and are not a Loner.\n\r", ch);
		return;
    }
    if( is_same_guild( ch, victim) && !can_deguild(ch))
    {
	    stc("You can't deguild.\n\r", ch);
	    return;
    }

    if (!str_cmp(arg2, "none") /*|| !str_cmp(arg2, "loner")*/) /* no. If you do that, they won't be a loner and then they won't be guildable. */
    {
		send_to_char("They are no longer a member of any guild.\n\r", ch);
		send_to_char("You are no longer a member of any guild!\n\r", victim);
		victim->clan = 0;
		victim->rank = 0;
		if(ch != victim)
		    victim->pcdata->deguilded = 1;

		//check_mysql_guild( victim );

		if (IS_SET(victim->hist_flags, HIST_GUILDLEADER))
		    REMOVE_BIT(victim->hist_flags, HIST_GUILDLEADER);
		if (IS_SET(victim->hist_flags, HIST_RECRUITER))
		    REMOVE_BIT(victim->hist_flags, HIST_RECRUITER);

	    compose_title( victim );

		return;
    }
    if(victim->pcdata->deguilded == 1)
    {
	    stc("They need to remort first.\n\r", ch);
	    return;
    }

    if ((clan = clan_lookup(arg2)) == 0)
    {
		send_to_char("No such guild exists.\n\r", ch);
		return;
    }

	if ( clan !=clan_lookup("loner")  )
	{
	    if(ch != victim)
	        victim->rank = 0;		/* lowest, default */
	    sprintf(buf, "They are now a %s of the %s.\n\r",
		    clan_table[clan].rank[victim->rank].rankname, clan_table[clan].name);
	    send_to_char(buf, ch);

	    sprintf(buf, "You are now a %s of the %s.\n\r",
		    clan_table[clan].rank[victim->rank].rankname, clan_table[clan].name);
	    send_to_char(buf, victim);

	    victim->clan = clan;
	    compose_title( victim );
		//check_mysql_guild( victim );
	}
	else
	{
		ptc( victim, "You are now a %s.\n\r", clan_table[clan].name );
		ptc( ch , "They are now a %s.\n\r", clan_table[clan].name );
	    victim->clan = clan;
	    victim->rank = 0;		/* lowest, default */
	    if(ch != victim)
	        victim->pcdata->deguilded = 1;
	    compose_title( victim );
		//check_mysql_guild( victim );
		if (IS_SET(victim->hist_flags, HIST_GUILDLEADER))
		    REMOVE_BIT(victim->hist_flags, HIST_GUILDLEADER);
		if (IS_SET(victim->hist_flags, HIST_RECRUITER))
		    REMOVE_BIT(victim->hist_flags, HIST_RECRUITER);
	}
}

void do_enlist( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    int clan;

    argument = one_argument(argument, arg1);

	if ( is_clan( ch ) )
    {
		stc("You're already in a guild.\n\r", ch);
		return;
    }

    if( ch->pcdata->deguilded == 1 )
    {
	    stc("You need to remort first.\n\r", ch);
	    return;
    }

    if ( ( clan = clan_lookup(arg1) ) == 0 )
    {
		stc("No such guild exists.\n\r", ch);
		return;
    }

	if ( clan != clan_lookup("loner")  )
	{
	    sprintf(buf, "You are now a %s of the %s.\n\r",
		    clan_table[clan].rank[ch->rank].rankname, clan_table[clan].name);
	    stc(buf, ch);

	    ch->clan = clan;
	    compose_title( ch );
		//check_mysql_guild( ch );
	}
	else
	{
		ptc( ch, "You are now a %s.\n\r", clan_table[clan].name );
	    ch->clan = clan;
	    ch->rank = 0;		/* lowest, default */
        ch->pcdata->deguilded = 1;
	    compose_title( ch );
		//check_mysql_guild( ch );
		if (IS_SET(ch->hist_flags, HIST_GUILDLEADER))
		    REMOVE_BIT(ch->hist_flags, HIST_GUILDLEADER);
		if (IS_SET(ch->hist_flags, HIST_RECRUITER))
		    REMOVE_BIT(ch->hist_flags, HIST_RECRUITER);
	}
}
