/***************************************************************************
                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/
    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.
    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.
    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.
    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.
    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.
	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include "aww.h"
#include "interp.h"
#include "magic.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "mysql5_interface.h"

/*
 * Local functions.
 */
void	affect_modify	args( ( CHAR_DATA *ch, AFFECT_DATA *paf, bool fAdd ) );
void	room_update	args( ( AREA_DATA *pArea ) );
void 	room_aff_update	args( ( ROOM_INDEX_DATA *room ) );
bool    remove_obj      args( ( CHAR_DATA *ch, int iWear, bool fReplace ) );
void 	setbit		args( ( char *explored, int index) );
void    check_explore_reward args(( CHAR_DATA *ch ));

// by prool:
bool can_see_obj( CHAR_DATA *ch, OBJ_DATA *obj );
bool can_see( CHAR_DATA *ch, CHAR_DATA *victim );
bool is_name ( char *str, char *namelist );

bool is_same_sex( CHAR_DATA *ch, CHAR_DATA *victim )
{
	if ( !ch->sex || !victim->sex )
	return FALSE;
	if ( ch->sex == SEX_MALE && victim->sex == SEX_FEMALE )
	return TRUE;
	if ( ch->sex == SEX_MALE && victim->sex == SEX_FEMALE )
	return FALSE;
	if ( ch->sex == SEX_FEMALE && victim->sex == SEX_MALE )
	return FALSE;
	return FALSE;
}
void affect_to_room( ROOM_INDEX_DATA *room, AFFECT_DATA *paf )
{
    AFFECT_DATA *paf_new;
    paf_new = new_affect();
    *paf_new		= *paf;
    paf_new->next	= room->affected;
    room->affected	= paf_new;
    if ( paf->where == TO_AFFECTS )
	SET_BIT(room->affected_by,paf->bitvector);
    if ( paf->where == TO_AFFECTS2 )
	SET_BIT(room->affected_by,paf->bitvector);
    return;
}
void affect_remove_room( ROOM_INDEX_DATA *room, AFFECT_DATA *paf )
{
    int where, vector;
    if ( room->affected == NULL )
    {
        bug( "Affect_remove_room: no affect.", 0 );
        return;
    }
    where = paf->where;
    vector = paf->bitvector;
    if (paf->bitvector)
	switch( paf->where)
        {
        case TO_AFFECTS:
            REMOVE_BIT(room->affected_by,paf->bitvector);
            break;
        case TO_AFFECTS2:
            REMOVE_BIT(room->affected_by,paf->bitvector);
            break;
        }
    if ( paf == room->affected )
    {
        room->affected    = paf->next;
    }
    else
    {
        AFFECT_DATA *prev;
        for ( prev = room->affected; prev != NULL; prev = prev->next )
        {
            if ( prev->next == paf )
            {
                prev->next = paf->next;
                break;
            }
        }
        if ( prev == NULL )
        {
            bug( "Affect_remove_room: cannot find paf.", 0 );
            return;
        }
    }
    free_affect(paf);
    return;
}
CHAR_DATA *get_char_area( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *ach;
    int number,count;
    if ( (ach = get_char_room( ch,NULL, argument )) != NULL )
	return ach;
    number = number_argument( argument, arg );
    count = 0;
    for ( ach = char_list; ach != NULL; ach = ach->next )
    {
	if (ach->in_room->area != ch->in_room->area
	||  !can_see( ch, ach ) || !is_name( arg, ach->name ))
	    continue;
	if (++count == number)
	    return ach;
    }
    return ach;
}
/* friend stuff -- for NPC's mostly */
bool is_friend(CHAR_DATA *ch,CHAR_DATA *victim)
{
    if (is_same_group(ch,victim))
	return TRUE;
    if (!IS_NPC(ch))
	return FALSE;
    if (!IS_NPC(victim))
    {
	if (IS_SET(ch->off_flags,ASSIST_PLAYERS))
	    return TRUE;
	else
	    return FALSE;
    }
    if (IS_AFFECTED(ch,AFF_CHARM))
	return FALSE;
    if (IS_SET(ch->off_flags,ASSIST_ALL))
	return TRUE;
    if (ch->group && ch->group == victim->group)
	return TRUE;
    if (IS_SET(ch->off_flags,ASSIST_VNUM)
    &&  ch->pIndexData == victim->pIndexData)
	return TRUE;
    if (IS_SET(ch->off_flags,ASSIST_RACE) && ch->race == victim->race)
	return TRUE;
    if (IS_SET(ch->off_flags,ASSIST_ALIGN)
    &&  !IS_SET(ch->act,ACT_NOALIGN) && !IS_SET(victim->act,ACT_NOALIGN)
    &&  ((IS_GOOD(ch) && IS_GOOD(victim))
    ||	 (IS_EVIL(ch) && IS_EVIL(victim))
    ||   (IS_NEUTRAL(ch) && IS_NEUTRAL(victim))))
	return TRUE;
    return FALSE;
}
/* returns number of people on an object */
int count_users(OBJ_DATA *obj)
{
    CHAR_DATA *fch;
    int count = 0;
    if (obj->in_room == NULL)
	return 0;
    for (fch = obj->in_room->people; fch != NULL; fch = fch->next_in_room)
	if (fch->on == obj)
	    count++;
    return count;
}
/* returns material number */
int material_lookup (const char *name)
{
	int material;
	for ( material = 0; material_table[material].name != NULL; material++)
	{
		if (LOWER(name[0]) == LOWER(material_table[material].name[0])
		&&  !str_prefix( name,material_table[material].name))
			return material;
	}
	return 0;
}
int weapon_lookup (const char *name)
{
    int type;
    for (type = 0; weapon_table[type].name != NULL; type++)
    {
	if (LOWER(name[0]) == LOWER(weapon_table[type].name[0])
	&&  !str_prefix(name,weapon_table[type].name))
	    return type;
    }
    return -1;
}
int weapon_type (const char *name)
{
    int type;
    for (type = 0; weapon_table[type].name != NULL; type++)
    {
        if (LOWER(name[0]) == LOWER(weapon_table[type].name[0])
        &&  !str_prefix(name,weapon_table[type].name))
            return weapon_table[type].type;
    }
    return WEAPON_EXOTIC;
}
char *item_name(int item_type)
{
    int type;
    for (type = 0; item_table[type].name != NULL; type++)
	if (item_type == item_table[type].type)
	    return item_table[type].name;
    return "none";
}
char *weapon_name( int weapon_type)
{
    int type;
    for (type = 0; weapon_table[type].name != NULL; type++)
        if (weapon_type == weapon_table[type].type)
            return weapon_table[type].name;
    return "exotic";
}
int attack_lookup  (const char *name)
{
    int att;
    for ( att = 0; attack_table[att].name != NULL; att++)
    {
	if (LOWER(name[0]) == LOWER(attack_table[att].name[0])
	&&  !str_prefix(name,attack_table[att].name))
	    return att;
    }
    return 0;
}
/* returns a flag for wiznet */
long wiznet_lookup (const char *name)
{
    int flag;
    for (flag = 0; wiznet_table[flag].name != NULL; flag++)
    {
	if (LOWER(name[0]) == LOWER(wiznet_table[flag].name[0])
	&& !str_prefix(name,wiznet_table[flag].name))
	    return flag;
    }
    return -1;
}
/* returns class number */
int class_lookup (const char *name)
{
   int class;
   for ( class = 0; class < MAX_CLASS; class++)
   {
        if (LOWER(name[0]) == LOWER(class_table[class].name[0])
        &&  !str_prefix( name,class_table[class].name))
            return class;
   }
   return -1;
}
/* for immunity, vulnerabiltiy, and resistant
   the 'globals' (magic and weapons) may be overriden
   three other cases -- wood, silver, and iron -- are checked in fight.c */
int check_immune(CHAR_DATA *ch, int dam_type)
{
    int immune, def;
    int bit;
    immune = -1;
    def = IS_NORMAL;
    if (dam_type == DAM_NONE)
	return immune;
    if (dam_type <= 3)
    {
	if (IS_SET(ch->imm_flags,IMM_WEAPON))
	    def = IS_IMMUNE;
	else if (IS_SET(ch->res_flags,RES_WEAPON))
	    def = IS_RESISTANT;
	else if (IS_SET(ch->vuln_flags,VULN_WEAPON))
	    def = IS_VULNERABLE;
    }
    else /* magical attack */
    {
	if (IS_SET(ch->imm_flags,IMM_MAGIC))
	    def = IS_IMMUNE;
	else if (IS_SET(ch->res_flags,RES_MAGIC))
	    def = IS_RESISTANT;
	else if (IS_SET(ch->vuln_flags,VULN_MAGIC))
	    def = IS_VULNERABLE;
    }
    /* set bits to check -- VULN etc. must ALL be the same or this will fail */
    switch (dam_type)
    {
	case(DAM_BASH):		bit = IMM_BASH;		break;
	case(DAM_PIERCE):	bit = IMM_PIERCE;	break;
	case(DAM_SLASH):	bit = IMM_SLASH;	break;
	case(DAM_FIRE):		bit = IMM_FIRE;		break;
	case(DAM_COLD):		bit = IMM_COLD;		break;
	case(DAM_LIGHTNING):	bit = IMM_LIGHTNING;	break;
	case(DAM_ACID):		bit = IMM_ACID;		break;
	case(DAM_POISON):	bit = IMM_POISON;	break;
	case(DAM_NEGATIVE):	bit = IMM_NEGATIVE;	break;
	case(DAM_HOLY):		bit = IMM_HOLY;		break;
	case(DAM_ENERGY):	bit = IMM_ENERGY;	break;
	case(DAM_MENTAL):	bit = IMM_MENTAL;	break;
	case(DAM_DISEASE):	bit = IMM_DISEASE;	break;
	case(DAM_DROWNING):	bit = IMM_DROWNING;	break;
	case(DAM_LIGHT):	bit = IMM_LIGHT;	break;
	case(DAM_CHARM):	bit = IMM_CHARM;	break;
	case(DAM_SOUND):	bit = IMM_SOUND;	break;
	default:		return def;
    }
    if (IS_SET(ch->imm_flags,bit))
	immune = IS_IMMUNE;
    else if (IS_SET(ch->res_flags,bit) && immune != IS_IMMUNE)
	immune = IS_RESISTANT;
    else if (IS_SET(ch->vuln_flags,bit))
    {
	if (immune == IS_IMMUNE)
	    immune = IS_RESISTANT;
	else if (immune == IS_RESISTANT)
	    immune = IS_NORMAL;
	else
	    immune = IS_VULNERABLE;
    }
    if (immune == -1)
	return def;
    else
      	return immune;
}

/* checks mob format */
bool is_old_mob(CHAR_DATA *ch)
{
    if (ch->pIndexData == NULL)
	return FALSE;
    else if (ch->pIndexData->new_format)
	return FALSE;
    return TRUE;
}


int get_maxhit (CHAR_DATA *ch)
{
	int max;

	if (IS_NPC(ch))	max = ch->max_hit;
	else		max = ch->pcdata->perm_hit;

	max += ch->mod_hit;

	return URANGE(50, max, 1000000);
}

int get_maxstam (CHAR_DATA *ch)
{
	int max;

	if (IS_NPC(ch))	max = ch->max_stam;
	else		max = ch->pcdata->perm_mana;

	max += ch->mod_stam;

	return URANGE(50, max, 1000000);
}






/* for returning skill information */
int get_skill(CHAR_DATA *ch, int sn)
{
    int skill;
    if (sn == -1) /* shorthand for level based skills */
    {
	skill = ch->level * 5 / 2;
    }
    else if (sn < -1 || sn > MAX_SKILL)
    {
	bug("Bad sn %d in get_skill.",sn);
	skill = 0;
    }
    else if (!IS_NPC(ch))
    {
	/*if (ch->level < skill_table[sn].skill_level[ch->class])
	    skill = 0;
	else*/
	    skill = ch->pcdata->learned[sn];
    }
    else /* mobiles */
    {
        if (skill_table[sn].spell_fun != spell_null)
	    skill = 40 + 2 * ch->level;
	else if (sn == gsn_sneak || sn == gsn_hide)
	    skill = ch->level * 2 + 20;
        else if ((sn == gsn_dodge && IS_SET(ch->off_flags,OFF_DODGE))
 	||       (sn == gsn_parry && IS_SET(ch->off_flags,OFF_PARRY)))
	    skill = ch->level * 2;
 	else if (sn == gsn_shield_block)
	    skill = 10 + 2 * ch->level;
	else if (sn == gsn_second_attack
	&& (IS_SET(ch->act,ACT_WARRIOR) || IS_SET(ch->act,ACT_THIEF)))
	    skill = 10 + 3 * ch->level;
	else if (sn == gsn_third_attack && IS_SET(ch->act,ACT_WARRIOR))
	    skill = 4 * ch->level - 40;
	else if (sn == gsn_hand_to_hand)
	    skill = 40 + 2 * ch->level;
 	else if (sn == gsn_trip && IS_SET(ch->off_flags,OFF_TRIP))
	    skill = 10 + 3 * ch->level;
 	else if (sn == gsn_bash && IS_SET(ch->off_flags,OFF_BASH))
	    skill = 10 + 3 * ch->level;
	else if (sn == gsn_disarm
	     &&  (IS_SET(ch->off_flags,OFF_DISARM)
	     ||   IS_SET(ch->act,ACT_WARRIOR)
	     ||	  IS_SET(ch->act,ACT_THIEF)))
	    skill = 20 + 3 * ch->level;
	else if (sn == gsn_berserk && IS_SET(ch->off_flags,OFF_BERSERK))
	    skill = 3 * ch->level;
	else if (sn == gsn_kick)
	    skill = 10 + 3 * ch->level;
	else if (sn == gsn_backstab && IS_SET(ch->act,ACT_THIEF))
	    skill = 20 + 2 * ch->level;
  	else if (sn == gsn_rescue)
	    skill = 40 + ch->level;
	else if (sn == gsn_recall)
	    skill = 40 + ch->level;
	else if (sn == gsn_sword
	||  sn == gsn_dagger
	||  sn == gsn_spear
	||  sn == gsn_staff
	||  sn == gsn_mace
	||  sn == gsn_axe
	||  sn == gsn_flail
	||  sn == gsn_whip
	||  sn == gsn_polearm)
	    skill = 40 + 5 * ch->level / 2;
	else
	   skill = 0;
    }
    if (ch->daze > 0)
    {
	if (skill_table[sn].spell_fun != spell_null)
	    skill /= 2;
	else
	    skill = 2 * skill / 3;
    }
    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK]  > 10 )
	skill = 9 * skill / 10;

	return UMAX (0, skill);
//    return URANGE(0,skill,100);
}

int get_secondary_sn(CHAR_DATA *ch)
{
    OBJ_DATA *wield;
    int sn;
    wield = get_eq_char( ch, WEAR_SECONDARY );
    if (wield == NULL || wield->item_type != ITEM_WEAPON)
        sn = gsn_hand_to_hand;
    else switch (wield->value[0])
    {
        default :               sn = -1;                break;
        case(WEAPON_SWORD):     sn = gsn_sword;         break;
        case(WEAPON_DAGGER):    sn = gsn_dagger;        break;
        case(WEAPON_SPEAR):     sn = gsn_spear;         break;
        case(WEAPON_STAFF):     sn = gsn_staff;         break;
        case(WEAPON_MACE):      sn = gsn_mace;          break;
        case(WEAPON_AXE):       sn = gsn_axe;           break;
        case(WEAPON_FLAIL):     sn = gsn_flail;         break;
        case(WEAPON_WHIP):      sn = gsn_whip;          break;
        case(WEAPON_POLEARM):   sn = gsn_polearm;       break;
        case(WEAPON_SHORTBOW):  sn = gsn_bow;           break;
        case(WEAPON_LONGBOW):   sn = gsn_bow;           break;
        case(WEAPON_RECURVEDBOW):sn = gsn_bow;           break;
        case(WEAPON_COMPOSITEBOW):sn = gsn_bow;           break;
        case(WEAPON_CROSSBOW):  sn = gsn_bow;           break;
	case(WEAPON_NET):	sn = gsn_net;		break;
	case(WEAPON_SCYTHESWORD):sn = gsn_polearm;	break; /* Should this be sword? */
	case(WEAPON_TRIDENT):	sn = gsn_polearm;	break; /* Should this be polearm? */
	case(WEAPON_HAMMER):	sn = gsn_hammer;	break;
	case(WEAPON_SLING):	sn = gsn_sling;		break;
	case(WEAPON_SWORDBREAKER):sn = gsn_swordbreaker;break;
	case(WEAPON_SAP):	sn = gsn_sap;		break;
	case(WEAPON_SHORTSWORD):sn = gsn_shortsword;	break;
	case(WEAPON_LANCE):	sn = gsn_lance;		break;
	case(WEAPON_ASHANDAREI):sn = gsn_ashandarei;	break;
	case(WEAPON_LIGHTCROSSBOW):sn = gsn_bow;	break;
   }
   return sn;
}

/* for returning weapon information */
int get_weapon_sn(CHAR_DATA *ch)
{
    OBJ_DATA *wield;
    int sn;
    wield = get_eq_char( ch, WEAR_WIELD );
    if (wield == NULL || wield->item_type != ITEM_WEAPON)
        sn = gsn_hand_to_hand;
    else switch (wield->value[0])
    {
        default :               sn = -1;                break;
        case(WEAPON_SWORD):     sn = gsn_sword;         break;
        case(WEAPON_DAGGER):    sn = gsn_dagger;        break;
        case(WEAPON_SPEAR):     sn = gsn_spear;         break;
        case(WEAPON_STAFF):     sn = gsn_staff;         break;
        case(WEAPON_MACE):      sn = gsn_mace;          break;
        case(WEAPON_AXE):       sn = gsn_axe;           break;
        case(WEAPON_FLAIL):     sn = gsn_flail;         break;
        case(WEAPON_WHIP):      sn = gsn_whip;          break;
        case(WEAPON_POLEARM):   sn = gsn_polearm;       break;
        case(WEAPON_SHORTBOW):  sn = gsn_bow;           break;
        case(WEAPON_LONGBOW):   sn = gsn_bow;           break;
        case(WEAPON_RECURVEDBOW):sn = gsn_bow;           break;
        case(WEAPON_COMPOSITEBOW):sn = gsn_bow;           break;
        case(WEAPON_CROSSBOW):  sn = gsn_bow;           break;
	case(WEAPON_NET):	sn = gsn_net;		break;
	case(WEAPON_SCYTHESWORD):sn = gsn_polearm;	break; /* Should this be sword? */
	case(WEAPON_TRIDENT):	sn = gsn_polearm;	break; /* Should this be polearm? */
	case(WEAPON_HAMMER):	sn = gsn_hammer;	break;
	case(WEAPON_SLING):	sn = gsn_sling;		break;
	case(WEAPON_SWORDBREAKER):sn = gsn_swordbreaker;break;
	case(WEAPON_SAP):	sn = gsn_sap;		break;
	case(WEAPON_SHORTSWORD):sn = gsn_shortsword;	break;
	case(WEAPON_LANCE):	sn = gsn_lance;		break;
	case(WEAPON_ASHANDAREI):sn = gsn_ashandarei;	break;
	case(WEAPON_LIGHTCROSSBOW):sn = gsn_bow;	break;
   }
   return sn;
}
int get_weapon_skill(CHAR_DATA *ch, int sn)
{
     int skill;
     /* -1 is exotic */
    if (IS_NPC(ch))
    {
	if (sn == -1)
	    skill = 3 * ch->level;
	else if (sn == gsn_hand_to_hand)
	    skill = 40 + 2 * ch->level;
	else
	    skill = 40 + 5 * ch->level / 2;
    }
    else
    {
	if (sn == -1)
	    skill = 3 * ch->level;
	else
	    skill = ch->pcdata->learned[sn];
    }
    return URANGE(0,skill,100);
}
/* used to de-screw characters */
void reset_char(CHAR_DATA *ch)
{
	int loc,mod,stat;
	OBJ_DATA *obj;
	AFFECT_DATA *af;
	int i, sk;

	if (IS_NPC(ch))
		return;

	if ( ch->pcdata->concentration > 9 || ch->pcdata->concentration < 0 )
		ch->pcdata->concentration = 1;

	/* now restore the character to his/her true condition */
	for (stat = 0; stat < MAX_STATS; stat++)
		ch->mod_stat[stat] = 0;

	if (ch->pcdata->true_sex < 0 || ch->pcdata->true_sex > 2)
		ch->pcdata->true_sex = 0;

	ch->sex		= ch->pcdata->true_sex;
	ch->max_hit 	= ch->pcdata->perm_hit;
	ch->max_stam	= ch->pcdata->perm_mana;

	for (i = 0; i < 4; i++)
		ch->armor[i]	= 100;

	ch->hitroll		= 0;
	ch->damroll		= 0;
	ch->saving_throw	= 0;

	/* now start adding back the effects */
	for (loc = 0; loc < MAX_WEAR; loc++)
	{
		obj = get_eq_char(ch,loc);

		if (obj == NULL)
			continue;

		for (i = 0; i < 4; i++)
			ch->armor[i] -= apply_ac( obj, loc, i );

		if (!obj->enchanted)
			for ( af = obj->pIndexData->affected; af != NULL; af = af->next )
			{
				mod = af->modifier;
				switch(af->location)
				{
					case APPLY_STR:		ch->mod_stat[STAT_STR]	+= mod;	break;
					case APPLY_DEX:		ch->mod_stat[STAT_DEX]	+= mod; break;
					case APPLY_INT:		ch->mod_stat[STAT_INT]	+= mod; break;
					case APPLY_WIS:		ch->mod_stat[STAT_WIS]	+= mod; break;
					case APPLY_CON:		ch->mod_stat[STAT_CON]	+= mod; break;
					case APPLY_REF:		ch->mod_stat[STAT_REF]	+= mod; break;
					case APPLY_INS:		ch->mod_stat[STAT_INS]	+= mod; break;
					case APPLY_CHA:		ch->mod_stat[STAT_CHA]	+= mod; break;
					case APPLY_SEX:		ch->sex			+= mod; break;
					case APPLY_MOVE: case APPLY_MANA:	ch->mod_stam		+= mod; break;
					case APPLY_HIT:		ch->mod_hit		+= mod; break;
					case APPLY_AC:
						for (i = 0; i < 4; i ++)
							ch->armor[i] += mod;
					break;
					case APPLY_HITROLL:	ch->hitroll		+= mod; break;
					case APPLY_DAMROLL:	ch->damroll		+= mod; break;
					case APPLY_SAVES:		ch->saving_throw += mod; break;
					case APPLY_SAVING_ROD: 		ch->saving_throw += mod; break;
					case APPLY_SAVING_PETRI:	ch->saving_throw += mod; break;
					case APPLY_SAVING_BREATH: 	ch->saving_throw += mod; break;
					case APPLY_SAVING_SPELL:	ch->saving_throw += mod; break;
				}
			}
			for ( af = obj->affected; af != NULL; af = af->next )
			{
				mod = af->modifier;
				switch(af->location)
				{
					case APPLY_STR:         ch->mod_stat[STAT_STR]  += mod; break;
					case APPLY_DEX:         ch->mod_stat[STAT_DEX]  += mod; break;
					case APPLY_INT:         ch->mod_stat[STAT_INT]  += mod; break;
					case APPLY_WIS:         ch->mod_stat[STAT_WIS]  += mod; break;
					case APPLY_CON:         ch->mod_stat[STAT_CON]  += mod; break;
					case APPLY_REF:		ch->mod_stat[STAT_REF]	+= mod; break;
					case APPLY_INS:		ch->mod_stat[STAT_INS]	+= mod; break;
					case APPLY_CHA:		ch->mod_stat[STAT_CHA]	+= mod; break;
					case APPLY_SEX:         ch->sex                 += mod; break;
					case APPLY_MOVE: case APPLY_MANA:        ch->mod_stam            += mod; break;
					case APPLY_HIT:         ch->mod_hit             += mod; break;
					case APPLY_AC:
					for (i = 0; i < 4; i ++)
						ch->armor[i] += mod;
					break;
					case APPLY_HITROLL:     ch->hitroll             += mod; break;
					case APPLY_DAMROLL:     ch->damroll             += mod; break;
					case APPLY_SAVES:         ch->saving_throw += mod; break;
					case APPLY_SAVING_ROD:          ch->saving_throw += mod; break;
					case APPLY_SAVING_PETRI:        ch->saving_throw += mod; break;
					case APPLY_SAVING_BREATH:       ch->saving_throw += mod; break;
					case APPLY_SAVING_SPELL:        ch->saving_throw += mod; break;
				}
			}
		}

		/* now add back spell effects */
		for (af = ch->affected; af != NULL; af = af->next)
		{
			mod = af->modifier;
			switch(af->location)
			{
				case APPLY_STR:         ch->mod_stat[STAT_STR]  += mod; break;
				case APPLY_DEX:         ch->mod_stat[STAT_DEX]  += mod; break;
				case APPLY_INT:         ch->mod_stat[STAT_INT]  += mod; break;
				case APPLY_WIS:         ch->mod_stat[STAT_WIS]  += mod; break;
				case APPLY_CON:         ch->mod_stat[STAT_CON]  += mod; break;
				case APPLY_REF:			ch->mod_stat[STAT_REF]	+= mod; break;
				case APPLY_INS:			ch->mod_stat[STAT_INS]	+= mod; break;
				case APPLY_CHA:			ch->mod_stat[STAT_CHA]	+= mod; break;
				case APPLY_SEX:         ch->sex                 += mod; break;
				case APPLY_MOVE: case APPLY_MANA:        ch->mod_stam            += mod; break;
				case APPLY_HIT:         ch->mod_hit             += mod; break;
				case APPLY_AC:
				for (i = 0; i < 4; i ++)
					ch->armor[i] += mod;
				break;
				case APPLY_HITROLL:     ch->hitroll             += mod; break;
				case APPLY_DAMROLL:     ch->damroll             += mod; break;
				case APPLY_SAVES:         ch->saving_throw += mod; break;
				case APPLY_SAVING_ROD:          ch->saving_throw += mod; break;
				case APPLY_SAVING_PETRI:        ch->saving_throw += mod; break;
				case APPLY_SAVING_BREATH:       ch->saving_throw += mod; break;
				case APPLY_SAVING_SPELL:        ch->saving_throw += mod; break;
			}
		}

		/* make sure sex is RIGHT!!!! */
		if (ch->sex < 0 || ch->sex > 2)
			ch->sex = ch->pcdata->true_sex;

	if ( ch->position < POS_SLEEPING )
	{
		ch->position = POS_SLEEPING;
	}

	for ( sk = 0; sk < MAX_SKILL; sk++ )
	{
		if (ch->pcdata->learned[sk] <= 0)
			continue;

		if (!str_cmp(skill_table[sk].name, "reserved"))
			continue;

		if ( is_qskill(sk) ) /* All quest skill stick now, but they also cap at QSKILL_CAP -- Dalsor 06/26/2013 */
		{
			if ( ch->pcdata->learned[sk] > QSKILL_CAP )
				ch->pcdata->learned[sk] = QSKILL_CAP;
			continue;
		}
		else
		{
			if ( ch->pcdata->learned[sk] > SKILL_CAP )
				ch->pcdata->learned[sk] = SKILL_CAP;
		}
	}

	return;
}
/*
 * Retrieve a character's trusted level for permission checking.
 */
int get_trust( CHAR_DATA *ch )
{
	if ( IS_NPC( ch ) )
		return 0;
    if ( ch->desc != NULL && ch->desc->original != NULL )
	ch = ch->desc->original;

    if (ch->trust)
	return ch->trust;

	return LEVEL_HERO - 1;

    if ( IS_NPC(ch) && ch->level >= LEVEL_HERO )
	return LEVEL_HERO - 1;
    else
	return ch->level;
}
/*
 * Retrieve a character's age.
 */
int get_age( CHAR_DATA *ch )
{
	if (ch->race == race_lookup("ogier"))
		return 100 + ( ch->played + (int) (current_time - ch->logon) ) / 72000;
	else
    	return 17 + ( ch->played + (int) (current_time - ch->logon) ) / 72000;
}
/* command for retrieving stats */
int get_curr_stat( CHAR_DATA *ch, int stat )
{
    int max;

	if ( IS_NPC( ch ) )
		max = 200;
	else
	{
		max = 100;

		if (get_skill(ch,gsn_gifted) > 0)
		{
			max += 50;
			max += calc_remort_tier(ch) * 5;
		}
        }

	if ((stat == STAT_REF || stat == STAT_DEX || stat == STAT_INT || stat == STAT_WIS) && IS_AFFECTED(ch, AFF_FEAR))
		max -= 10;

    return URANGE(3,ch->perm_stat[stat] + ch->mod_stat[stat] + ch->quest_stat[stat], max);
}

/* Command for returning max training score
 * If a character has the gifted quest skill,
 * they can train any stat to 74, else 49.
 */
int get_max_train( CHAR_DATA *ch, int stat )
{
	int max = 0, gift = 100;

	if (IS_NPC(ch))
		return 100;
	else
	{
		if ( get_skill( ch,gsn_gifted ) == 0 )
			max = 50;
		else
		{
			gift = 120;
			max = 75;
		}
	}
    return UMIN(max,gift);
}
/*
 * Retrieve a character's carry capacity.
 */
int can_carry_n( CHAR_DATA *ch )
{
    if ( !IS_NPC(ch) && IS_SET( ch->war_flags, WAR_IMMORTAL ) )
	return 1000;
    if ( IS_NPC(ch) && IS_SET(ch->act, ACT_PET) )
	return 0;
    /*return MAX_WEAR +  2 * get_curr_stat(ch,STAT_CON) + get_curr_stat(ch,STAT_STR);*/
    if(ch->pcdata)
        return MAX_WEAR + 250 + ch->pcdata->rebirth;
    else
	return MAX_WEAR + 250;
}
/*
 * Retrieve a character's carry capacity.
 */
int can_carry_w( CHAR_DATA *ch )
{
    if ( !IS_NPC(ch) && IS_IMMORTAL(ch) && ch->level >= LEVEL_IMMORTAL )
	return 10000000;
    if ( IS_NPC(ch) && IS_SET(ch->act, ACT_PET) )
	return 0;
    return str_app[get_curr_stat(ch,STAT_STR)].carry * 10 + ch->level * 25;
}
/*
 * See if a string is one of the names of an object.
 */
bool is_name ( char *str, char *namelist )
{
    char name[MAX_INPUT_LENGTH], part[MAX_INPUT_LENGTH];
    char *list, *string;
    /* fix crash on NULL namelist */
    if (namelist == NULL || namelist[0] == '\0')
    	return FALSE;
    /* fixed to prevent is_name on "" returning TRUE */
    if (str[0] == '\0')
	return FALSE;
    string = str;
    /* we need ALL parts of string to match part of namelist */
    for ( ; ; )  /* start parsing string */
    {
	str = one_argument(str,part);
	if (part[0] == '\0' )
	    return TRUE;
	/* check to see if this is part of namelist */
	list = namelist;
	for ( ; ; )  /* start parsing namelist */
	{
	    list = one_argument(list,name);
	    if (name[0] == '\0')  /* this name was not found */
		return FALSE;
	    if (!str_prefix(string,name))
		return TRUE; /* full pattern match */
	    if (!str_prefix(part,name))
		break;
	}
    }
}
bool is_exact_name(char *str, char *namelist )
{
    char name[MAX_INPUT_LENGTH];
    if (namelist == NULL)
	return FALSE;
    for ( ; ; )
    {
	namelist = one_argument( namelist, name );
	if ( name[0] == '\0' )
	    return FALSE;
	if ( !str_cmp( str, name ) )
	    return TRUE;
    }
}
/* enchanted stuff for eq */
void affect_enchant(OBJ_DATA *obj)
{
    /* okay, move all the old flags into new vectors if we have to */
    if (!obj->enchanted)
    {
        AFFECT_DATA *paf, *af_new;
        obj->enchanted = TRUE;
        for (paf = obj->pIndexData->affected; paf != NULL; paf = paf->next)
        {
	    af_new = new_affect();
            af_new->next = obj->affected;
            obj->affected = af_new;
	    af_new->where	= paf->where;
            af_new->type        = paf->type;
            af_new->level       = paf->level;
            af_new->duration    = paf->duration;
            af_new->location    = paf->location;
            af_new->modifier    = paf->modifier;
            af_new->bitvector   = paf->bitvector;
        }
    }
}
/*
 * Apply or remove an affect to a character.
 */
void affect_modify( CHAR_DATA *ch, AFFECT_DATA *paf, bool fAdd )
{
    OBJ_DATA *wield;
    int mod,i;
    mod = paf->modifier;
    if ( fAdd )
    {
	switch (paf->where)
	{
	case TO_AFFECTS:
	    SET_BIT(ch->affected_by, paf->bitvector);
	    break;
	case TO_AFFECTS2:
	    SET_BIT(ch->affected2_by, paf->bitvector);
	    break;
	case TO_IMMUNE:
	    SET_BIT(ch->imm_flags,paf->bitvector);
	    break;
	case TO_RESIST:
	    SET_BIT(ch->res_flags,paf->bitvector);
	    break;
	case TO_VULN:
	    SET_BIT(ch->vuln_flags,paf->bitvector);
	    break;
	}
    }
    else
    {
        switch (paf->where)
        {
        case TO_AFFECTS:
            REMOVE_BIT(ch->affected_by, paf->bitvector);
            break;
        case TO_AFFECTS2:
            REMOVE_BIT(ch->affected2_by, paf->bitvector);
            break;
        case TO_IMMUNE:
            REMOVE_BIT(ch->imm_flags,paf->bitvector);
            break;
        case TO_RESIST:
            REMOVE_BIT(ch->res_flags,paf->bitvector);
            break;
        case TO_VULN:
            REMOVE_BIT(ch->vuln_flags,paf->bitvector);
            break;
        }
	mod = 0 - mod;
    }
    switch ( paf->location )
    {
    default:
	bug( "Affect_modify: unknown location %d.", paf->location );
	return;
    case APPLY_NONE:						break;
    case APPLY_STR:           ch->mod_stat[STAT_STR]	+= mod;	break;
    case APPLY_DEX:           ch->mod_stat[STAT_DEX]	+= mod;	break;
    case APPLY_INT:           ch->mod_stat[STAT_INT]	+= mod;	break;
    case APPLY_WIS:           ch->mod_stat[STAT_WIS]	+= mod;	break;
    case APPLY_CON:           ch->mod_stat[STAT_CON]	+= mod;	break;
	case APPLY_REF:			  ch->mod_stat[STAT_REF]	+= mod; break;
	case APPLY_INS:			  ch->mod_stat[STAT_INS]	+= mod; break;
	case APPLY_CHA:			  ch->mod_stat[STAT_CHA]	+= mod; break;
    case APPLY_SEX:           ch->sex			+= mod;	break;
    case APPLY_CLASS:						break;
    case APPLY_LEVEL:						break;
    case APPLY_AGE:						break;
    case APPLY_HEIGHT:						break;
    case APPLY_WEIGHT:						break;
    case APPLY_MOVE: case APPLY_MANA:          ch->mod_stam		+= mod;	break;
    case APPLY_HIT:           ch->mod_hit		+= mod;	break;
    case APPLY_GOLD:						break;
    case APPLY_EXP:						break;
    case APPLY_AC:
        for (i = 0; i < 4; i ++)
            ch->armor[i] += mod;
        break;
    case APPLY_HITROLL:       ch->hitroll		+= mod;	break;
    case APPLY_DAMROLL:       ch->damroll		+= mod;	break;
    case APPLY_SAVES:   ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_ROD:    ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_PETRI:  ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_BREATH: ch->saving_throw		+= mod;	break;
    case APPLY_SAVING_SPELL:  ch->saving_throw		+= mod;	break;
    case APPLY_SPELL_AFFECT:  					break;
    }
    /*
     * Check for weapon wielding.
     * Guard against recursion (for weapons with affects).
     */
    if ( !IS_NPC(ch) && ( wield = get_eq_char( ch, WEAR_WIELD ) ) != NULL
    &&   get_obj_weight(wield) > ( get_curr_stat(ch,STAT_STR) + 25 ) )
    {
	/*if ( depth == 0 )
	{
	    depth++;
	    act( "{cYou drop $p.", ch, wield, NULL, TO_CHAR );
	    act( "{c$n drops $p.", ch, wield, NULL, TO_ROOM );
	    if (wield->wear_loc != WEAR_NONE)
	    remove_obj (ch, wield->wear_loc, TRUE);
	    obj_from_char( wield );
	    obj_to_room( wield, ch->in_room );
	    depth--;
	}*/
    }
    return;
}
/* find an effect in an affect list */
AFFECT_DATA  *affect_find(AFFECT_DATA *paf, int sn)
{
    AFFECT_DATA *paf_find;
    for ( paf_find = paf; paf_find != NULL; paf_find = paf_find->next )
    {
        if ( paf_find->type == sn )
	return paf_find;
    }
    return NULL;
}
/* fix object affects when removing one */
void affect_check(CHAR_DATA *ch,int where,int vector)
{
    AFFECT_DATA *paf;
    OBJ_DATA *obj;
    if (where == TO_OBJECT || where == TO_WEAPON || vector == 0)
	return;
    for (paf = ch->affected; paf != NULL; paf = paf->next)
	if (paf->where == where && paf->bitvector == vector)
	{
	    switch (where)
	    {
	        case TO_AFFECTS:
		    SET_BIT(ch->affected_by,vector);
		    break;
	        case TO_AFFECTS2:
		    SET_BIT(ch->affected2_by,vector);
		    break;
	        case TO_IMMUNE:
		    SET_BIT(ch->imm_flags,vector);
		    break;
	        case TO_RESIST:
		    SET_BIT(ch->res_flags,vector);
		    break;
	        case TO_VULN:
		    SET_BIT(ch->vuln_flags,vector);
		    break;
	    }
	    return;
	}
    for (obj = ch->carrying; obj != NULL; obj = obj->next_content)
    {
	if (obj->wear_loc == -1)
	    continue;
            for (paf = obj->affected; paf != NULL; paf = paf->next)
            if (paf->where == where && paf->bitvector == vector)
            {
                switch (where)
                {
                    case TO_AFFECTS:
                        SET_BIT(ch->affected_by,vector);
                        break;
                    case TO_AFFECTS2:
                        SET_BIT(ch->affected2_by,vector);
                        break;
                    case TO_IMMUNE:
                        SET_BIT(ch->imm_flags,vector);
                        break;
                    case TO_RESIST:
                        SET_BIT(ch->res_flags,vector);
                        break;
                    case TO_VULN:
                        SET_BIT(ch->vuln_flags,vector);
                }
                return;
            }
        if (obj->enchanted)
	    continue;
        for (paf = obj->pIndexData->affected; paf != NULL; paf = paf->next)
            if (paf->where == where && paf->bitvector == vector)
            {
                switch (where)
                {
                    case TO_AFFECTS:
                        SET_BIT(ch->affected_by,vector);
                        break;
                    case TO_AFFECTS2:
                        SET_BIT(ch->affected2_by,vector);
                        break;
                    case TO_IMMUNE:
                        SET_BIT(ch->imm_flags,vector);
                        break;
                    case TO_RESIST:
                        SET_BIT(ch->res_flags,vector);
                        break;
                    case TO_VULN:
                        SET_BIT(ch->vuln_flags,vector);
                        break;
                }
                return;
            }
    }
}
/*
 * Give an affect to a char.
 */
void affect_to_char( CHAR_DATA *ch, AFFECT_DATA *paf )
{
    AFFECT_DATA *paf_new;
    paf_new = new_affect();
    *paf_new		= *paf;
    VALIDATE(paf_new);	/* in case we missed it when we set up paf */
    paf_new->next	= ch->affected;
    ch->affected	= paf_new;
    affect_modify( ch, paf_new, TRUE );
    return;
}
/* give an affect to an object */
void affect_to_obj(OBJ_DATA *obj, AFFECT_DATA *paf)
{
    AFFECT_DATA *paf_new;
    paf_new = new_affect();
    *paf_new		= *paf;
    VALIDATE(paf_new);	/* in case we missed it when we set up paf */
    paf_new->next	= obj->affected;
    obj->affected	= paf_new;
    /* apply any affect vectors to the object's extra_flags */
    if (paf->bitvector)
        switch (paf->where)
        {
        case TO_OBJECT:
    	    SET_BIT(obj->extra_flags,paf->bitvector);
	    break;
        case TO_WEAPON:
	    if (obj->item_type == ITEM_WEAPON)
	        SET_BIT(obj->value[4],paf->bitvector);
	    break;
        }
    return;
}
/*
 * Remove an affect from a char.
 */
void affect_remove( CHAR_DATA *ch, AFFECT_DATA *paf )
{
    int where;
    int vector;
    if ( ch->affected == NULL )
    {
	bug( "Affect_remove: no affect.", 0 );
	return;
    }
    affect_modify( ch, paf, FALSE );
    where = paf->where;
    vector = paf->bitvector;
    if ( paf == ch->affected )
    {
	ch->affected	= paf->next;
    }
    else
    {
	AFFECT_DATA *prev;
	for ( prev = ch->affected; prev != NULL; prev = prev->next )
	{
	    if ( prev->next == paf )
	    {
		prev->next = paf->next;
		break;
	    }
	}
	if ( prev == NULL )
	{
	    bug( "Affect_remove: cannot find paf.", 0 );
	    return;
	}
    }
    free_affect(paf);
    affect_check(ch,where,vector);
    return;
}
void affect_remove_obj( OBJ_DATA *obj, AFFECT_DATA *paf)
{
    int where, vector;
    if ( obj->affected == NULL )
    {
        bug( "Affect_remove_object: no affect.", 0 );
        return;
    }
    if (obj->carried_by != NULL && obj->wear_loc != -1)
	affect_modify( obj->carried_by, paf, FALSE );
    where = paf->where;
    vector = paf->bitvector;
    /* remove flags from the object if needed */
    if (paf->bitvector)
	switch( paf->where)
        {
        case TO_OBJECT:
            REMOVE_BIT(obj->extra_flags,paf->bitvector);
            break;
        case TO_WEAPON:
            if (obj->item_type == ITEM_WEAPON)
                REMOVE_BIT(obj->value[4],paf->bitvector);
            break;
        }
    if ( paf == obj->affected )
    {
        obj->affected    = paf->next;
    }
    else
    {
        AFFECT_DATA *prev;
        for ( prev = obj->affected; prev != NULL; prev = prev->next )
        {
            if ( prev->next == paf )
            {
                prev->next = paf->next;
                break;
            }
        }
        if ( prev == NULL )
        {
            bug( "Affect_remove_object: cannot find paf.", 0 );
            return;
        }
    }
    free_affect(paf);
    if (obj->carried_by != NULL && obj->wear_loc != -1)
	affect_check(obj->carried_by,where,vector);
    return;
}
/*
 * Strip all affects of a given sn.
 */
void affect_strip( CHAR_DATA *ch, int sn )
{
    AFFECT_DATA *paf;
    AFFECT_DATA *paf_next;
    for ( paf = ch->affected; paf != NULL; paf = paf_next )
    {
	paf_next = paf->next;
	if ( paf->type == sn )
	    affect_remove( ch, paf );
    }
    return;
}
/*
 * Return true if a char is affected by a spell.
 */
bool is_affected( CHAR_DATA *ch, int sn )
{
    AFFECT_DATA *paf;
    for ( paf = ch->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->type == sn )
	    return TRUE;
    }
    return FALSE;
}
/*
 * Add or enhance an affect.
 */
void affect_join( CHAR_DATA *ch, AFFECT_DATA *paf )
{
    AFFECT_DATA *paf_old;
    bool found;
    found = FALSE;
    for ( paf_old = ch->affected; paf_old != NULL; paf_old = paf_old->next )
    {
	if ( paf_old->type == paf->type )
	{
	    paf->level = (paf->level += paf_old->level) / 2;
	    paf->duration += paf_old->duration;
	    paf->modifier += paf_old->modifier;
	    affect_remove( ch, paf_old );
	    break;
	}
    }
    affect_to_char( ch, paf );
    return;
}
/*
 * Move a char out of a room.
 */
void char_from_room( CHAR_DATA *ch )
{
    OBJ_DATA *obj;
    if ( ch->in_room == NULL )
    {
	bug( "Char_from_room: NULL.", 0 );
	return;
    }
    if ( !IS_NPC(ch) )
	--ch->in_room->area->nplayer;
    if ( ( obj = get_eq_char( ch, WEAR_LIGHT ) ) != NULL
    &&   obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   ch->in_room->light > 0 )
	--ch->in_room->light;
    if ( ch == ch->in_room->people )
    {
	ch->in_room->people = ch->next_in_room;
    }
    else
    {
	CHAR_DATA *prev;
	for ( prev = ch->in_room->people; prev; prev = prev->next_in_room )
	{
	    if ( prev->next_in_room == ch )
	    {
		prev->next_in_room = ch->next_in_room;
		break;
	    }
	}
	if ( prev == NULL )
	    bug( "Char_from_room: ch not found.", 0 );
    }
    ch->in_room      = NULL;
    ch->next_in_room = NULL;
    ch->on 	     = NULL;  /* sanity check! */
    return;
}
/*
 * Move a char into a room.
 */
int roomcount args((CHAR_DATA * ch));
void char_to_room( CHAR_DATA *ch, ROOM_INDEX_DATA *pRoomIndex )
{
    OBJ_DATA *obj;

    if ( pRoomIndex == NULL )
    {
		ROOM_INDEX_DATA *room;
		bug( "Char_to_room: NULL.", 0 );
		if ((room = get_room_index(ROOM_VNUM_PLAZA)) != NULL)
		    char_to_room(ch,room);
		return;
    }

    ch->in_room		= pRoomIndex;
    ch->next_in_room	= pRoomIndex->people;
    pRoomIndex->people	= ch;

    if (!IS_NPC(ch))
    {
	if ( ch->pcdata != NULL )
	{
		if(!getbit(ch->pcdata->explored,pRoomIndex->vnum))
		{
		    ch->pcdata->new_room = TRUE;
		    setbit(ch->pcdata->explored,pRoomIndex->vnum);
		    check_explore_reward(ch);
		}
		else
		    ch->pcdata->new_room = FALSE;
	}
	if (ch->in_room->area->empty)
	{
	    ch->in_room->area->empty = FALSE;
	    ch->in_room->area->age = 0;
	}
	++ch->in_room->area->nplayer;
	if ( ( obj = get_eq_char( ch, WEAR_LIGHT ) ) != NULL
	    &&   obj->item_type == ITEM_LIGHT
	    &&   obj->value[2] != 0 )
		++ch->in_room->light;
    }
    if (IS_AFFECTED(ch,AFF_PLAGUE))
    {
        AFFECT_DATA *af, plague;
        CHAR_DATA *vch;
        for ( af = ch->affected; af != NULL; af = af->next )
        {
            if (af->type == gsn_plague)
                break;
        }
        if (af == NULL)
        {
            REMOVE_BIT(ch->affected_by,AFF_PLAGUE);
            return;
        }
        if (af->level == 1)
            return;
	    plague.where		= TO_AFFECTS;
        plague.type 		= gsn_plague;
        plague.level 		= af->level - 1;
        plague.duration 	= number_range(1,2 * plague.level);
        plague.location		= APPLY_STR;
        plague.modifier 	= -5;
        plague.bitvector 	= AFF_PLAGUE;
        for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
        {
            if ( !saves_spell( plague.level - 2, vch, DAM_DISEASE )
	        && !IS_IMMORTAL( vch )
            && !IS_NPC( ch )
            && !IS_AFFECTED( vch,AFF_PLAGUE )
            && number_bits( 6 ) == 0 )
            {
            	stc("{cYou feel hot and feverish.\n\r",vch);
            	act("{c$n shivers and looks very ill.",vch,NULL,NULL,TO_ROOM);
            	affect_join(vch,&plague);
            }
            /* this is where aggression should now occur but it's flaky - Dalsor
            if ( IS_NPC( vch )
            && IS_SET( vch->act, ACT_AGGRESSIVE )
            && !IS_SET( vch->in_room->room_flags, ROOM_SAFE )
            && !IS_AFFECTED( vch, AFF_CALM )

            && vch->fighting == NULL
            && vch->advancing == FALSE
            && vch->advancing_on == NULL
            && ch->fighting == NULL
            && ch->advancing == FALSE
            && ch->advancing_on == NULL

            && IS_AWAKE( vch )
            && can_see( vch, ch ) )
            {
			    vch->advancing = TRUE;
			    vch->advancing_on = ch;
			    vch->range = RANGE_MELEE;

			    ch->advancing = TRUE;
			    ch->advancing_on = vch;
			    ch->range = RANGE_MELEE;
		        set_fighting( vch, ch );
            }
            */
        }
    }
    return;
}
/*
 * Give an obj to a char.
 */
void obj_to_char( OBJ_DATA *obj, CHAR_DATA *ch )
{
    obj->next_content	 = ch->carrying;
    ch->carrying	 = obj;
    obj->carried_by	 = ch;
    obj->in_room	 = NULL;
    obj->in_obj		 = NULL;
    ch->carry_number	+= get_obj_number( obj );
    ch->carry_weight	+= get_obj_weight( obj );
}
/*
 * Take an obj from its character.
 */
void obj_from_char( OBJ_DATA *obj )
{
    CHAR_DATA *ch;
    if ( ( ch = obj->carried_by ) == NULL )
    {
	bug( "Obj_from_char: null ch.", 0 );
	return;
    }
    if ( obj->wear_loc != WEAR_NONE )
	unequip_char( ch, obj );
    if ( ch->carrying == obj )
    {
	ch->carrying = obj->next_content;
    }
    else
    {
	OBJ_DATA *prev;
	for ( prev = ch->carrying; prev != NULL; prev = prev->next_content )
	{
	    if ( prev->next_content == obj )
	    {
		prev->next_content = obj->next_content;
		break;
	    }
	}
	if ( prev == NULL )
	    bug( "Obj_from_char: obj not in list.", 0 );
    }
    obj->carried_by	 = NULL;
    obj->next_content	 = NULL;
    ch->carry_number	-= get_obj_number( obj );
    ch->carry_weight	-= get_obj_weight( obj );
    return;
}
/*
 * Find the ac value of an obj, including position effect.
 */
int apply_ac( OBJ_DATA *obj, int iWear, int type )
{
    if ( obj->item_type != ITEM_ARMOR )
	return 0;
    switch ( iWear )
    {
    case WEAR_BODY:	return     obj->value[type];
    case WEAR_HEAD:	return     obj->value[type];
    case WEAR_BACK:	return     obj->value[type];
    case WEAR_LEGS:	return     obj->value[type];
    case WEAR_FEET:	return     obj->value[type];
    case WEAR_HANDS:	return     obj->value[type];
    case WEAR_ARMS:	return     obj->value[type];
    case WEAR_SHIELD:	return     obj->value[type];
    case WEAR_NECK_1:	return     obj->value[type];
    case WEAR_NECK_2:	return     obj->value[type];
    case WEAR_ABOUT:	return     obj->value[type];
    case WEAR_WAIST:	return     obj->value[type];
    case WEAR_WRIST_L:	return     obj->value[type];
    case WEAR_WRIST_R:	return     obj->value[type];
    case WEAR_HOLD:	return     obj->value[type];
    case WEAR_NOSE:     return     obj->value[type];
    case WEAR_SHOULDERS:return     obj->value[type];
    case WEAR_EYES:     return     obj->value[type];
    case WEAR_CALF_L:   return     obj->value[type];
    case WEAR_CALF_R:   return     obj->value[type];
    case WEAR_THIGH_L:  return     obj->value[type];
    case WEAR_ANKLE_L:  return     obj->value[type];
    case WEAR_ANKLE_R:  return     obj->value[type];
    case WEAR_FOREARM_L:return     obj->value[type];
    case WEAR_FOREARM_R:return     obj->value[type];
    case WEAR_EAR_1:    return     obj->value[type];
    case WEAR_EAR_2:    return     obj->value[type];
    case WEAR_FACE:     return     obj->value[type];
    }
    return 0;
}
/*
 * Find a piece of eq on a character.
 */
OBJ_DATA *get_eq_char( CHAR_DATA *ch, int iWear )
{
    OBJ_DATA *obj;
    if (ch == NULL)
	return NULL;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc == iWear )
	    return obj;
    }
    return NULL;
}
/*
 * Equip a char with an obj.
 */
void equip_char( CHAR_DATA *ch, OBJ_DATA *obj, int iWear )
{
    AFFECT_DATA *paf;
    int i;
    if ( get_eq_char( ch, iWear ) != NULL )
    {
	bugf( "Equip_char: %s already equipped (iwear %d room %d).", ch->name, iWear, ch->in_room->vnum );
	return;
    }
    if ( ( IS_OBJ_STAT(obj, ITEM_ANTI_EVIL)    && IS_EVIL(ch)    )
    ||   ( IS_OBJ_STAT(obj, ITEM_ANTI_GOOD)    && IS_GOOD(ch)    ) )
    {
	/*
	 * Thanks to Morgenes for the bug fix here!
	 */
	act( "{cYou are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
	act( "{c$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
	obj_from_char( obj );
	obj_to_room( obj, ch->in_room );
	return;
    }
	if ( ch->level >= obj->level )
	{
	    for (i = 0; i < 4; i++)
	    	ch->armor[i]      	-= apply_ac( obj, iWear,i );
	    obj->wear_loc	 = iWear;
	    if (!obj->enchanted)
		for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
		    if ( paf->location != APPLY_SPELL_AFFECT )
		        affect_modify( ch, paf, TRUE );
	    for ( paf = obj->affected; paf != NULL; paf = paf->next )
		if ( paf->location == APPLY_SPELL_AFFECT )
	    	    affect_to_char ( ch, paf );
		else
		    affect_modify( ch, paf, TRUE );
	}
	else
	{
	    obj->wear_loc	 = iWear;
	}
    if ( obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   ch->in_room != NULL )
	++ch->in_room->light;
    return;
}
/*
 * Unequip a char with an obj.
 */
void unequip_char( CHAR_DATA *ch, OBJ_DATA *obj )
{
    AFFECT_DATA *paf = NULL;
    AFFECT_DATA *lpaf = NULL;
    AFFECT_DATA *lpaf_next = NULL;
    int i;
    if ( obj->wear_loc == WEAR_NONE )
    {
	bug( "Unequip_char: already unequipped.", 0 );
	return;
    }
	if ( ch->level >= obj->level )
	{
	    for (i = 0; i < 4; i++)
	    	ch->armor[i]	+= apply_ac( obj, obj->wear_loc,i );
	    obj->wear_loc	 = -1;
	    if (!obj->enchanted)
	    {
		for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
		    if ( paf->location == APPLY_SPELL_AFFECT )
		    {
		        for ( lpaf = ch->affected; lpaf != NULL; lpaf = lpaf_next )
		        {
			    lpaf_next = lpaf->next;
			    if ((lpaf->type == paf->type) &&
		        (lpaf->level == paf->level) &&
		        (lpaf->location == APPLY_SPELL_AFFECT))
			    {
		        affect_remove( ch, lpaf );
				lpaf_next = NULL;
			    }
		        }
		    }
		    else
		    {
	        affect_modify( ch, paf, FALSE );
			affect_check(ch,paf->where,paf->bitvector);
		    }
	    }
	    for ( paf = obj->affected; paf != NULL; paf = paf->next )
		if ( paf->location == APPLY_SPELL_AFFECT )
		{
		    bug ( "Norm-Apply: %d", 0 );
		    for ( lpaf = ch->affected; lpaf != NULL; lpaf = lpaf_next )
		    {
			lpaf_next = lpaf->next;
			if ((lpaf->type == paf->type) &&
		    (lpaf->level == paf->level) &&
		    (lpaf->location == APPLY_SPELL_AFFECT))
			{
			    bug ( "location = %d", lpaf->location );
			    bug ( "type = %d", lpaf->type );
			    affect_remove( ch, lpaf );
			    lpaf_next = NULL;
			}
		    }
		}
		else
		{
		    affect_modify( ch, paf, FALSE );
		    affect_check(ch,paf->where,paf->bitvector);
		}
	}
	else
	{
	    obj->wear_loc	 = -1;
	}
    if ( obj->item_type == ITEM_LIGHT
    &&   obj->value[2] != 0
    &&   ch->in_room != NULL
    &&   ch->in_room->light > 0 )
	--ch->in_room->light;
    return;
}
/*
 * Count occurrences of an obj in a list.
 */
int count_obj_list( OBJ_INDEX_DATA *pObjIndex, OBJ_DATA *list )
{
    OBJ_DATA *obj;
    int nMatch;
    nMatch = 0;
    for ( obj = list; obj != NULL; obj = obj->next_content )
    {
	if ( obj->pIndexData == pObjIndex )
	    nMatch++;
    }
    return nMatch;
}

void delete_home_item(OBJ_DATA *obj)
{
	char sql_buf[MSL];
	OBJ_DATA *o2;

//	if (build_port == mud_port)	return;

	if (obj->stash_id < 0)	return;
        sprintf(sql_buf, "delete from home_items where id = %ld", obj->stash_id);
        mysql_real_query(result_db, sql_buf, strlen(sql_buf));

	for (o2 = obj->contains; o2 != NULL; o2 = o2->next_content)
		delete_home_item(o2);	

	return;
}

void save_home_item(OBJ_DATA *obj)
{
        MYSQL_RES *result;
        MYSQL_ROW row;
	char sql_buf[MSL];
	char obj_values[MSL];
	char obj_affect1[MSL], obj_affect2[MSL];
	char obj_affect3[MSL], obj_affect4[MSL];
	AFFECT_DATA *paf;
	long onum, rnum;
	OBJ_DATA *o2;

//	if (build_port == mud_port)	return;

	obj_affect1[0] = '\0';
	obj_affect2[0] = '\0';
	obj_affect3[0] = '\0';
	obj_affect4[0] = '\0';

	sprintf(obj_values, "%d %d %d %d %d", obj->value[0], obj->value[1], obj->value[2], obj->value[3], obj->value[4]);

	for (paf = obj->affected; paf != NULL; paf = paf->next)
	{
		if (obj_affect1[0] == '\0')
		sprintf(obj_affect1, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
		else if (obj_affect2[0] == '\0')
		sprintf(obj_affect2, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
		else if (obj_affect3[0] == '\0')
		sprintf(obj_affect3, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
		else if (obj_affect4[0] == '\0')
		sprintf(obj_affect4, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
	}

	if (obj_affect1[0] == '\0')	sprintf(obj_affect1, "%d", -1);
	if (obj_affect2[0] == '\0')	sprintf(obj_affect2, "%d", -1);
	if (obj_affect3[0] == '\0')	sprintf(obj_affect3, "%d", -1);
	if (obj_affect4[0] == '\0')	sprintf(obj_affect4, "%d", -1);

	onum = -1;
	rnum = 1;

	if (obj->in_obj)
	{
		onum = obj->in_obj->stash_id;
		if (obj->in_obj->in_room)
		rnum = obj->in_obj->in_room->vnum;
	}

	if (obj->in_room)
		rnum = obj->in_room->vnum;

	sprintf(sql_buf, "insert into home_items values ('', %ld, \"%s\", \"%s\", \"%s\", \"%s\", %d, %d, %d, \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", %ld, %ld)",
	obj->pIndexData->vnum, obj->name, obj->short_descr, obj->description, obj->material, obj->wear_flags, obj->item_type, 
	obj->cost, obj_values, obj_affect1, obj_affect2, obj_affect3, obj_affect4, onum, rnum);
	mysql_real_query(result_db, sql_buf, strlen(sql_buf));

	sprintf(sql_buf, "select id from home_items order by id desc limit %d", 1);
	result = query_db(sql_buf, result_db, FALSE);

	if (result && ((row = mysql_fetch_row(result)) != NULL))
		obj->stash_id = atoi(row[0]);

	mysql_free_result(result);

	for (o2 = obj->contains; o2 != NULL; o2 = o2->next_content)
		save_home_item(o2);	

	return;
}






/*
 * Move an obj out of a room.
 */
void _obj_from_room( OBJ_DATA *obj, char *file, int line )
{
    ROOM_INDEX_DATA *in_room;
    CHAR_DATA *ch;
    if ( ( in_room = obj->in_room ) == NULL )
    {
		bugf( "obj_from_room: NULL. (%s %d)", file, line );
		return;
    }

    for (ch = in_room->people; ch != NULL; ch = ch->next_in_room)
		if (ch->on == obj)
		    ch->on = NULL;

    if ( obj == in_room->contents )
    {
		in_room->contents = obj->next_content;
    }
    else
    {
		OBJ_DATA *prev;
		for ( prev = in_room->contents; prev; prev = prev->next_content )
		{
		    if ( prev->next_content == obj )
		    {
				prev->next_content = obj->next_content;
				break;
		    }
		}
		if ( prev == NULL )
		{
		    bug( "Obj_from_room: obj not found.", 0 );
		    return;
		}
    }

    obj->in_room      = NULL;
    obj->next_content = NULL;
    return;
}
/*
void obj_from_room( OBJ_DATA *obj )
{
    ROOM_INDEX_DATA *in_room;
    CHAR_DATA *ch;
    if ( ( in_room = obj->in_room ) == NULL )
    {
		bug( "obj_from_room: NULL.", 0 );
		return;
    }

    for (ch = in_room->people; ch != NULL; ch = ch->next_in_room)
		if (ch->on == obj)
		    ch->on = NULL;

    if ( obj == in_room->contents )
    {
		in_room->contents = obj->next_content;
    }
    else
    {
		OBJ_DATA *prev;
		for ( prev = in_room->contents; prev; prev = prev->next_content )
		{
		    if ( prev->next_content == obj )
		    {
				prev->next_content = obj->next_content;
				break;
		    }
		}
		if ( prev == NULL )
		{
		    bug( "Obj_from_room: obj not found.", 0 );
		    return;
		}
    }
    obj->in_room      = NULL;
    obj->next_content = NULL;
    return;
}
*/
/*
 * Move an obj into a room.
 */
void obj_to_room( OBJ_DATA *obj, ROOM_INDEX_DATA *pRoomIndex )
{
    obj->next_content		= pRoomIndex->contents;
    pRoomIndex->contents	= obj;
    obj->in_room		= pRoomIndex;
    obj->carried_by		= NULL;
    obj->in_obj			= NULL;

    return;
}
/*
 * Move an object into an object.
 */
void obj_to_obj( OBJ_DATA *obj, OBJ_DATA *obj_to )
{
    obj->next_content		= obj_to->contains;
    obj_to->contains		= obj;
    obj->in_obj			= obj_to;
    obj->in_room		= NULL;
    obj->carried_by		= NULL;
    if (obj_to->pIndexData->vnum == OBJ_VNUM_PIT)
        obj->cost = 0;
    for ( ; obj_to != NULL; obj_to = obj_to->in_obj )
    {
	if ( obj_to->carried_by != NULL )
	{
	    obj_to->carried_by->carry_number += get_obj_number( obj );
	    obj_to->carried_by->carry_weight += get_obj_weight( obj )
		* WEIGHT_MULT(obj_to) / 100;
	}
    }

    return;
}
/*
 * Move an object out of an object.
 */
void obj_from_obj( OBJ_DATA *obj )
{
    OBJ_DATA *obj_from;
    if ( ( obj_from = obj->in_obj ) == NULL )
    {
	bug( "Obj_from_obj: null obj_from.", 0 );
	return;
    }
    if ( obj == obj_from->contains )
    {
	obj_from->contains = obj->next_content;
    }
    else
    {
	OBJ_DATA *prev;
	for ( prev = obj_from->contains; prev; prev = prev->next_content )
	{
	    if ( prev->next_content == obj )
	    {
		prev->next_content = obj->next_content;
		break;
	    }
	}
	if ( prev == NULL )
	{
	    bug( "Obj_from_obj: obj not found.", 0 );
	    return;
	}
    }
    obj->next_content = NULL;
    obj->in_obj       = NULL;
    for ( ; obj_from != NULL; obj_from = obj_from->in_obj )
    {
	if ( obj_from->carried_by != NULL )
	{
	    obj_from->carried_by->carry_number -= get_obj_number( obj );
	    obj_from->carried_by->carry_weight -= get_obj_weight( obj )
		* WEIGHT_MULT(obj_from) / 100;
	}
    }

    return;
}




/*
 * Extract an obj from the world.
 */
void extract_obj( OBJ_DATA *obj )
{
    OBJ_DATA *obj_content;
    OBJ_DATA *obj_next;

    if (obj->in_room && IS_SET(obj->in_room->room_flags, ROOM_HOME))
        delete_home_item(obj);

    if (obj->in_obj && obj->in_obj->in_room && IS_SET(obj->in_obj->in_room->room_flags, ROOM_HOME))
        delete_home_item(obj);

    if ( obj->in_room != NULL )
	obj_from_room( obj );
    else if ( obj->carried_by != NULL )
	obj_from_char( obj );
    else if ( obj->in_obj != NULL )
	obj_from_obj( obj );
    for ( obj_content = obj->contains; obj_content; obj_content = obj_next )
    {
	obj_next = obj_content->next_content;
	extract_obj( obj_content );
    }
    if ( object_list == obj )
    {
	object_list = obj->next;
    }
    else
    {
	OBJ_DATA *prev;
	for ( prev = object_list; prev != NULL; prev = prev->next )
	{
	    if ( prev->next == obj )
	    {
		prev->next = obj->next;
		break;
	    }
	}
	if ( prev == NULL )
	{
	    bug( "Extract_obj: obj %d not found.", obj->pIndexData->vnum );
	    return;
	}
    }
    --obj->pIndexData->count;
    free_obj(obj);
    return;
}
/*
 * Extract a char from the world.
 */
void extract_char( CHAR_DATA *ch, bool fPull )
{
    CHAR_DATA *wch;
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    /* doesn't seem to be necessary
    if ( ch->in_room == NULL )
    {
		bug( "Extract_char: NULL.", 0 );
		return;
    }
    */

	if (!IS_NPC(ch))
	{
		reset_teaching(ch);
		reset_learning(ch);
	}

    nuke_pets(ch);
    ch->pet = NULL; /* just in case */

    if ( fPull )
		die_follower( ch );

    stop_all_fighting( ch );

    for ( obj = ch->carrying; obj != NULL; obj = obj_next )
    {
		obj_next = obj->next_content;
		extract_obj( obj );
    }

    if (ch->in_room != NULL)
        char_from_room( ch );

    /* Death room is set in the clan table */
    if ( !fPull )
    {   /* send char to clan morgue (room[1]) */
      if ( is_clan(ch) &&
           (get_room_index(clan_table[ch->clan].room[1]) != NULL) )
        char_to_room(ch,get_room_index(clan_table[ch->clan].room[1]));
      else
        char_to_room(ch,get_room_index(ROOM_VNUM_PLAZA));
      return;
    }
    /*
    if ( !fPull )
    {
		if ( !IS_NPC( ch ) && ch->pcdata->newbie == 1 )
		{
			char_from_room( ch );
			char_to_room( ch, get_room_index( 31020 ) );
			do_look( ch, "auto" );
		}
		else
	        char_to_room(ch,get_room_index(clan_table[ch->clan].hall));

		return;
    }
    */

    if ( IS_NPC(ch) )
		--ch->pIndexData->count;
    else
    {
		nuke_pets(ch);
		ch->pet = NULL;
    }

    if ( ch->mount && ch->mount->mount == ch )
    {
		ch->mount->mount = NULL;
		if (ch->mount->riding)
		{
		    act("You fall off of $N.", ch->mount, NULL, ch, TO_CHAR);
		    act("$n falls off of $N.", ch->mount, NULL, ch, TO_ROOM);
		    ch->mount->riding = FALSE;
		    if (!IS_IMMORTAL(ch->mount))
			ch->mount->position = POS_SITTING;
		}
    }

    if ( ch->desc != NULL && ch->desc->original != NULL )
    {
		do_function(ch, &do_return, "" );
		ch->desc = NULL;
    }

    for ( wch = char_list; wch != NULL; wch = wch->next )
    {
		if ( wch->reply == ch )
		    wch->reply = NULL;
		if ( ch->mprog_target == wch )
		    wch->mprog_target = NULL;
    }

    if ( ch == char_list )
    {
       char_list = ch->next;
    }
    else
    {
		CHAR_DATA *prev;
		for ( prev = char_list; prev != NULL; prev = prev->next )
		{
		    if ( prev->next == ch )
		    {
				prev->next = ch->next;
				break;
		    }
		}
		if ( prev == NULL )
		{
		    bug( "Extract_char: char not found.", 0 );
		    return;
		}
    }

    if ( ch->desc != NULL )
		ch->desc->character = NULL;

    free_char( ch );
    return;
}
/*
 * Find a char in the room.
CHAR_DATA *get_char_room( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *rch;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    if ( !str_cmp( arg, "self" ) )
	return ch;
    for ( rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room )
    {
	if ( !can_see( ch, rch ) || !is_name( arg, rch->name ) )
	    continue;
	if ( ++count == number )
	    return rch;
    }
    return NULL;
}
*/
CHAR_DATA *get_char_room( CHAR_DATA *ch, ROOM_INDEX_DATA *room, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *rch;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    if ( !str_cmp( arg, "self" ) )
	return ch;
    if ( ch && room )
    {
	bug( "get_char_room received multiple types (ch/room)", 0 );
	return NULL;
    }

	if (ch == NULL && room == NULL)
	{
		bug("get_char_room received null ch and null room!", 0);
		return NULL;
	}

    if ( ch && ch->in_room )
    {
	    for (rch = ch->in_room->people; rch != NULL; rch = rch->next_in_room )
	    {

            if ( IS_NPC( rch ) && IS_SET(rch->act,ACT_UNDEAD) && !IS_IMMORTAL(ch))
            {
                    if ( rch->pIndexData->vnum != ch->questmob )
                    {
                        if ( rch->questor != NULL && rch->questor == ch )
                            ;
                            else
                                continue;
                        }
                }


		if ( !can_see( ch, rch ) || !is_name( arg, rch->name ) )
		    continue;
		if ( ++count == number)
		    return rch;
	    }
    }
    else if (room)
    {
	    for (rch = room->people; rch != NULL; rch = rch->next_in_room )
	    {
		if (!is_name( arg, rch->name ) )
		    continue;
		if ( ++count == number)
		    return rch;
	    }
    }

    return NULL;
}
CHAR_DATA *get_mob_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *wch;
    int number;
    int count;

    if ( ch && ( wch = get_char_room( ch, NULL, argument ) ) != NULL )
		return wch;

    number = number_argument( argument, arg );
    count  = 0;

    for ( wch = char_list; wch != NULL ; wch = wch->next )
    {
		if ( wch->in_room == NULL || ( ch && !can_see( ch, wch ) )
		||   !is_name( arg, wch->name ) )
		    continue;
		if ( !IS_NPC( wch ) )
			continue;
		if ( ++count == number )
		    return wch;
    }
    return NULL;
}
CHAR_DATA *get_mob_by_vnum( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *wch;
    int vnum;

	if ( !is_number( argument ) )
	{
		bugf( "get_mob_by_vnum:: non-number lookup", 0 );
		return NULL;
	}
	vnum = atol( argument );
    for ( wch = char_list; wch != NULL ; wch = wch->next )
    {
		if ( wch->in_room == NULL || ( ch && !can_see( ch, wch ) ) )
		    continue;
		if ( !IS_NPC( wch ) )
			continue;
		if ( !wch->pIndexData )
			continue;
		if ( wch->pIndexData->vnum != vnum )
			continue;
		if ( wch->pIndexData->vnum == vnum )
		    return wch;
    }
    return NULL;
}
CHAR_DATA *get_pc_world( CHAR_DATA *ch, char*argument )
{
	DESCRIPTOR_DATA *d;
	if(!str_cmp(argument,"self"))
		return ch;
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( !d || !d->character || d->connected != CON_PLAYING )
		{
			continue;
		}
		if ( IS_NPC( d->character )) /* how the hell would this happen? why can people bondtalk to npcs? */
			continue;
		if ( is_exact_name(argument,  d->character->name ) )
			return (d->character);
	}
	return NULL;
}

CHAR_DATA *get_exactchar_world( CHAR_DATA *ch, char*argument )
{
        DESCRIPTOR_DATA *d;
	if(!str_cmp(argument,"self"))
		return ch;
        for ( d = descriptor_list; d != NULL; d = d->next )
        {
                if ( !d || !d->character || d->connected != CON_PLAYING )
                {
                        continue;
                }
                if ( is_exact_name(argument,  d->character->name ) )
                        return (d->character);
        }
        return NULL;
}

/*
 * Find a char in the world.
CHAR_DATA *get_char_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *wch;
    int number;
    int count;
    if ( ( wch = get_char_room( ch,NULL, argument ) ) != NULL )
	return wch;
    number = number_argument( argument, arg );
    count  = 0;
    for ( wch = char_list; wch != NULL ; wch = wch->next )
    {
	if ( wch->in_room == NULL || !can_see( ch, wch )
	||   !is_name( arg, wch->name ) )
	    continue;
	if ( ++count == number )
	    return wch;
    }
    return NULL;
}
*/
CHAR_DATA *get_char_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *wch;
    int number;
    int count;
    if ( ch && ( wch = get_char_room( ch, NULL, argument ) ) != NULL )
	return wch;
    number = number_argument( argument, arg );
    count  = 0;
    for ( wch = char_list; wch != NULL ; wch = wch->next )
    {
	if ( wch->in_room == NULL || ( ch && !can_see( ch, wch ) )
	||   !is_name( arg, wch->name ) )
	    continue;
	if ( ++count == number )
	    return wch;
    }
    return NULL;
}
/*
 * Find some object with a given index data.
 * Used by area-reset 'P' command.
 */
OBJ_DATA *get_obj_type( OBJ_INDEX_DATA *pObjIndex )
{
    OBJ_DATA *obj;
    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
	if ( obj->pIndexData == pObjIndex )
	    return obj;
    }
    return NULL;
}
/*
 * Find an obj in a list.
 */
OBJ_DATA *get_obj_list( CHAR_DATA *ch, char *argument, OBJ_DATA *list )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = list; obj != NULL; obj = obj->next_content )
    {
	if ( can_see_obj( ch, obj ) && is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }
    return NULL;
}
/*
 * Find an obj in player's inventory.
OBJ_DATA *get_obj_carry( CHAR_DATA *ch, char *argument, CHAR_DATA *viewer )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc == WEAR_NONE
	&&   (can_see_obj( viewer, obj ) )
	&&   is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }
    return NULL;
}
*/
OBJ_DATA *get_obj_carry( CHAR_DATA *ch, char *argument, CHAR_DATA *viewer )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
        if ( obj->wear_loc == WEAR_NONE
        &&   ( viewer ? can_see_obj( viewer, obj ) : TRUE )
        &&   is_name( arg, obj->name ) )
        {
            if ( ++count == number )
                            return obj;
        }
    }
    return NULL;
}
/*
 * Find an obj in player's equipment.
OBJ_DATA *get_obj_wear( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc != WEAR_NONE
	&&   can_see_obj( ch, obj )
	&&   is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }
    return NULL;
}
*/
OBJ_DATA *get_obj_wear( CHAR_DATA *ch, char *argument, bool character )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
        if ( obj->wear_loc != WEAR_NONE
                &&  ( character ? can_see_obj( ch, obj ) : TRUE)
        &&   is_name( arg, obj->name ) )
        {
            if ( ++count == number )
                return obj;
        }
    }
    return NULL;
}
/*
 * Find an obj in the room or in inventory.
OBJ_DATA *get_obj_here( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj;
    obj = get_obj_list( ch, argument, ch->in_room->contents );
    if ( obj != NULL )
	return obj;
    if ( ( obj = get_obj_carry( ch, argument, ch ) ) != NULL )
	return obj;
    if ( ( obj = get_obj_wear( ch, argument, TRUE ) ) != NULL )
	return obj;
    return NULL;
}
*/
OBJ_DATA *get_obj_here( CHAR_DATA *ch, ROOM_INDEX_DATA *room, char *argument )
{
    OBJ_DATA *obj;
    int number, count;
    char arg[MAX_INPUT_LENGTH];
    if ( ch && room )
    {
	bug( "get_obj_here received a ch and a room",0);
	return NULL;
    }
    number = number_argument( argument, arg );
    count = 0;
    if ( ch )
    {
	obj = get_obj_list( ch, argument, ch->in_room->contents );
	if ( obj != NULL )
	    return obj;
	if ( ( obj = get_obj_carry( ch, argument, ch ) ) != NULL )
	    return obj;
	if ( ( obj = get_obj_wear( ch, argument, TRUE ) ) != NULL )
	    return obj;
    }
    else
    {
	for ( obj = room->contents; obj; obj = obj->next_content )
	{
	    if ( !is_name( arg, obj->name ) )
		continue;
	    if ( ++count == number )
		return obj;
	}
    }
    return NULL;
}
/*
 * Find an obj in the world.
OBJ_DATA *get_obj_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    if ( ( obj = get_obj_here( ch, NULL, argument ) ) != NULL )
	return obj;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
	if ( can_see_obj( ch, obj ) && is_name( arg, obj->name ) )
	{
	    if ( ++count == number )
		return obj;
	}
    }
    return NULL;
}
*/
OBJ_DATA *get_obj_world( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int number;
    int count;
    if ( ch && ( obj = get_obj_here( ch, NULL, argument ) ) != NULL )
	return obj;
    number = number_argument( argument, arg );
    count  = 0;
    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
	if ( ( ch && !can_see_obj( ch, obj ) )
	|| !is_name( arg, obj->name ) )
	    continue;
	if ( ++count == number )
	    return obj;
    }
    return NULL;
}
OBJ_DATA *get_obj_by_vnum( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj;
    long vnum;

	if ( !is_number( argument ) )
	{
		bugf( "get_obj_by_vnum:: non number lookup", 0 );
		return NULL;
	}

	vnum = atol( argument );

    if ( ch && ( obj = get_obj_here( ch, NULL, argument ) ) != NULL )
		return obj;

    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
		if ( ( ch && !can_see_obj( ch, obj ) ) )
		    continue;
		if ( obj->pIndexData->vnum == vnum )
		    return obj;
    }
    return NULL;
}

/*int small_change; - Anthony*/
bool check_coins(CHAR_DATA *ch, int cost, int start,int stop)
{
	int i=0,j=0;
	int moneyvalues[4] = {1,10,100,1000};
	long Total=0;
	for (i=start;i<stop+1;i++)
	{
		for (j=0;j<4;j++)
		{
			Total += ch->money[j][i] * moneyvalues[j];
		}
		if (cost <= Total - ch->money[MONEY_BLOCKED][i])  /* what is MONEY_BLOCKED?????????? */
		{
			return TRUE;
		}
	}
	return FALSE;
}
/* deduct cost from a character - Anthony*/
void deduct_cost(CHAR_DATA *ch, int cost, int account)
{
	int tempvar = 0, i=0;
	extern char last_command[MAX_STRING_LENGTH];
	ch->money[i][account] -= cost;
	for (i=0;i<3;i++)
	{
		if (ch->money[i][account] < 0)
		{
			tempvar = ch->money[i][account];
			if ((-tempvar % 10) > 0)
			{
				ch->money[i+1][account] += (tempvar/10-1);
				ch->money[i][account] = 10 + (tempvar % 10);
			}
			else
			{
				ch->money[i+1][account] += tempvar/10;
				ch->money[i][account] = 0;
			}
		}
		else
		{
			return;
		}
	}
	if(ch->money[3][account] < 0)
	    bugf("Negative money: %s",last_command); /* make sure check_coins is done first */
}
/* OLD FUNCTIONS (before putting money in an array)
int small_change;
 I don't know whether cost is send as value or
 as reference, I hope it is a value else it has to be stored in another variable first
bool check_coins(CHAR_DATA *ch, int cost)
{
 First I multiply the sm cost with 100 so it's in copper
 I deduct the number of gold * 1000
 Same for silver marks and so on
 If the rest cost is bigger than nil, you can't cover it, else you can
 hence a false for cost>0 and a true in the other cases
	cost *= 100;
	cost -= ch->gold_mark * 1000;
	cost -= ch->silver_mark * 100;
	cost -= ch->silver_penny * 10;
	cost -= ch->copper_penny;
	if (cost > 0)
	{
		return FALSE;
	}
	return TRUE;
}
 deduct cost from a character
void deduct_cost(CHAR_DATA *ch, int cost)
{
 if (ch->copper_penny >= cost * 100)
 {
  ch->copper_penny -= cost * 100;
 }
 else
 {
  cost -= ch->copper_penny / 100;
  ch->copper_penny = ch->copper_penny % 100;
  if (ch->silver_penny >= cost*10)
  {
   ch->silver_penny -= 10*cost;
  }
  else
  {
   cost -= ch->silver_penny / 10;
   ch->silver_penny = ch->silver_penny % 10;
   small_change = ch->copper_penny + ch->silver_penny * 10;
   if (small_change > 100)
   {
    ch->silver_penny = 0;
    ch->copper_penny = small_change % 100;
    cost -= 1;
   }
   if (ch->silver_mark >= cost)
   {
    ch->silver_mark -= cost;
   }
   else
   {
BAH changing the results for the if -> true and if false is just plainly stupid :(
  Anyway i reversed them
    cost -= ch->silver_mark;
    if (cost % 10 > 0)
    {
     ch->silver_mark = 10-(cost % 10);
     ch->gold_mark -= ((cost / 10) + 1);
    }
    else
    {
     ch->silver_mark = 0;
     ch->gold_mark -= cost/10;
    }
   }
  }
 }
 small_change = 0;
 return;
}
*/
/*
 * Create a 'money' obj.
 */
OBJ_DATA *create_money( int copper, int silver, int mark, int gold )
{
    char buf[MAX_STRING_LENGTH];
    int total;
    OBJ_DATA *obj;
    if ( gold < 0
    || silver < 0
    || copper < 0
    || mark < 0
    || ( gold == 0 && silver == 0 && copper == 0 && mark == 0 ) )
    {
		bug( "Create_money: zero or negative money.", 0 );
		copper = UMAX(1,copper);
		mark = UMAX(1,mark);
		gold = UMAX(1,gold);
		silver = UMAX(1,silver);
    }
    total = copper + silver + mark + gold;
	/*
    if (gold == 0 && silver == 1)
    {
	obj = create_object( get_obj_index( OBJ_VNUM_SILVER_ONE ), 0 );
    }
    else if (gold == 1 && silver == 0)
    {
	obj = create_object( get_obj_index( OBJ_VNUM_GOLD_ONE), 0 );
    }
    else if (silver == 0)
    {
        obj = create_object( get_obj_index( OBJ_VNUM_GOLD_SOME ), 0 );
        sprintf( buf, obj->short_descr, gold );
        free_string( obj->short_descr );
        obj->short_descr        = str_dup( buf );
        obj->value[1]           = gold;
        obj->cost               = gold;
	obj->weight		= gold/5;
    }
    else if (gold == 0)
    {
        obj = create_object( get_obj_index( OBJ_VNUM_SILVER_SOME ), 0 );
        sprintf( buf, obj->short_descr, silver );
        free_string( obj->short_descr );
        obj->short_descr        = str_dup( buf );
        obj->value[0]           = silver;
        obj->cost               = silver;
	obj->weight		= silver/20;
    }
    else
    {
	*/
	obj = create_object( get_obj_index( OBJ_VNUM_COINS ), 0 );
	sprintf( buf, obj->short_descr, copper, silver, mark, gold );
	free_string( obj->short_descr );
	obj->short_descr	= str_dup( buf );
	obj->value[0]		= copper;
	obj->value[1]		= silver;
	obj->value[2]       = mark;
	obj->value[3]       = gold;
	if ( copper > 0 )
		obj->cost		= copper;
	if ( silver > 10 )
		obj->cost		+= silver / 10;
	if ( mark > 100 )
		obj->cost		+= mark / 100;
	if ( gold > 1000 )
		obj->cost		+= gold / 1000;
	obj->weight		= total;
    return obj;
}
/*
 * Return # of objects which an object counts as.
 * Thanks to Tony Chamberlain for the correct recursive code here.
 */
int get_obj_number( OBJ_DATA *obj )
{
    int number;
    if ( obj == NULL )
    {
    	bug("GET_OBJ_NUMBER:: NULL Object",0);
    	return 0;
    }
    if ( obj->pIndexData == NULL )
    {
    	bug("GET_OBJ_NUMBER:: NULL Object pIndexData",0);
    	return 0;
    }
    if (obj->item_type == ITEM_CONTAINER || obj->item_type == ITEM_MONEY
    ||  obj->item_type == ITEM_GEM || obj->item_type == ITEM_JEWELRY || obj->item_type == ITEM_TREASURECHEST
    ||  obj->item_type == ITEM_CRAFTING_MATERIAL || obj->item_type == ITEM_DYE )
        number = 0;
    else
        number = 1;
    for ( obj = obj->contains; obj != NULL; obj = obj->next_content )
        number += get_obj_number( obj );
    return number;
}
/*
 * Return weight of an object, including weight of contents.
 */
int get_obj_weight( OBJ_DATA *obj )
{
    int weight;
    OBJ_DATA *tobj;

    if ( obj->item_type == ITEM_MONEY )
    return 0;

    weight = obj->weight;
    for ( tobj = obj->contains; tobj != NULL; tobj = tobj->next_content )
		weight += get_obj_weight( tobj ) * WEIGHT_MULT(obj) / 100;
    return weight;
}
int get_true_weight(OBJ_DATA *obj)
{
    int weight;
    weight = obj->weight;
    for ( obj = obj->contains; obj != NULL; obj = obj->next_content )
        weight += get_obj_weight( obj );
    return weight;
}
/*
 * True if room is dark.
 */
bool room_is_dark( ROOM_INDEX_DATA *pRoomIndex )
{
    if ( pRoomIndex->light > 0 )
	return FALSE;
    if ( IS_SET(pRoomIndex->room_flags, ROOM_DARK) )
	return TRUE;
    if ( pRoomIndex->sector_type == SECT_INSIDE
    ||   pRoomIndex->sector_type == SECT_CITY )
	return FALSE;
    return FALSE;
}
bool is_room_owner(CHAR_DATA *ch, ROOM_INDEX_DATA *room)
{
    if (room->owner == NULL || room->owner[0] == '\0')
	return FALSE;
	if ( IS_MARRIED ( ch ) &&
	is_name ( ch->pcdata->spouse, room->owner ) )
	return TRUE;
    return is_name(ch->name,room->owner);
}
/*
 * True if room is private.
 */
bool room_is_private( ROOM_INDEX_DATA *pRoomIndex )
{
    CHAR_DATA *rch;
    int count;
    if (pRoomIndex->owner != NULL && pRoomIndex->owner[0] != '\0')
	return TRUE;
    count = 0;
    for ( rch = pRoomIndex->people; rch != NULL; rch = rch->next_in_room )
	count++;
    if ( IS_SET(pRoomIndex->room_flags, ROOM_PRIVATE)  && count >= 2 )
	return TRUE;
    if ( IS_SET(pRoomIndex->room_flags, ROOM_SOLITARY) && count >= 1 )
	return TRUE;
    if ( IS_SET(pRoomIndex->room_flags, ROOM_IMP_ONLY) )
	return TRUE;
    return FALSE;
}
/* visibility on a room -- for entering and exits */
bool can_see_room( CHAR_DATA *ch, ROOM_INDEX_DATA *pRoomIndex )
{
    if (IS_SET(pRoomIndex->room_flags, ROOM_IMP_ONLY)
    &&  !IS_IMMORTAL(ch))
	return FALSE;
    if (IS_SET(pRoomIndex->room_flags, ROOM_GODS_ONLY)
    &&  !IS_IMMORTAL(ch))
	return FALSE;
    if (IS_SET(pRoomIndex->room_flags, ROOM_HEROES_ONLY)
    &&  !IS_IMMORTAL(ch))
	return FALSE;
    if (IS_SET(pRoomIndex->room_flags,ROOM_NEWBIES_ONLY)
    &&  ch->level > 5 && !IS_IMMORTAL(ch))
	return FALSE;
    if (!IS_IMMORTAL(ch) && pRoomIndex->clan && ch->clan != pRoomIndex->clan)
	return FALSE;
    return TRUE;
}
/*
 * True if char can see victim.
 */
bool can_see( CHAR_DATA *ch, CHAR_DATA *victim )
{
	int sight = get_skill( ch, gsn_far_sight );

	if ( !ch || !victim )
		return FALSE;

	if ( ch == victim )
		return TRUE;

	if ( !IS_IMMORTAL(ch) && victim->invis_level > 0)
		return FALSE;

	if (!IS_IMMORTAL(ch) && victim->incog_level > 0 && ch->in_room != victim->in_room)
		return FALSE;

	if (!IS_NPC (ch) && !IS_NPC (victim))
	{
		if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING)
		&& IS_SET (victim->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
		return TRUE;
	}

	if ( (!IS_NPC(ch) && IS_SET(ch->act, PLR_HOLYLIGHT)) || (IS_NPC(ch) && IS_IMMORTAL(ch)))
		return TRUE;

	if ( IS_AFFECTED(ch, AFF_BLIND) )
		return FALSE;

	if ( room_is_dark( ch->in_room ) && !IS_AFFECTED(ch, AFF_INFRARED) && !WOLF(ch))
		return FALSE;

	if ( IS_MARRIED ( ch ) && IS_MARRIED ( victim ) && !str_cmp ( ch->pcdata->spouse, victim->name ) && !str_cmp ( victim->pcdata->spouse, ch->name ) )
		return TRUE;

	if (gquest_info.running == GQUEST_RUNNING
	&& ON_GQUEST(ch)
	&& IS_NPC(victim)
	&& is_gqmob(ch, victim->pIndexData->vnum) != -1)
		return TRUE;

	if ( IS_AFFECTED(victim, AFF_INVISIBLE) && !IS_AFFECTED(ch, AFF_DETECT_INVIS) )
	{
		int chance = 1;
		chance += victim->level;
		chance -= ch->level / 2;
		//ptc( ch, "Invis: Vic level %d, Ch level %d, Chance %d plus sight %d.\n\r", victim->level, ch->level, chance, sight );
		chance += sight;
		//ptc( ch, "Modified chance %d.\n\r", chance );
		if ( number_percent() < chance )
		{
			//stc("Can_See Failure!\n\r",ch);
			return FALSE;
		}
		else 
		{
			//stc("Can_See Success!\n\r",ch);
			return TRUE;
		}
	}

	/* sneaking */
	if ( IS_AFFECTED(victim, AFF_SNEAK)
	&&   !IS_AFFECTED(ch,AFF_DETECT_HIDDEN)
	&&   !is_fighting(victim))
	{
		int chance = 1;
		chance = get_skill(victim,gsn_sneak);
		chance += get_curr_stat(victim,STAT_DEX) * 3/2;
	 	chance -= get_curr_stat(ch,STAT_INT) * 2;
		chance -= sight;
		chance -= ch->level - victim->level * 3/2;

		//ptc( ch, "Sneak: Vic level %d, Ch level %d, Chance %d plus sight %d.\n\r", victim->level, ch->level, chance, sight );
		if ( number_percent() >= chance )
		{
			//stc("Can_See Success!\n\r",ch);
			return FALSE;
		}
		else
		{
			//stc("Can_See Failuire!\n\r",ch);
			return TRUE;
		}

		if (number_percent() < chance)
		    return FALSE;
	}

	if ( IS_AFFECTED(victim, AFF_HIDE) && !IS_AFFECTED(ch, AFF_DETECT_HIDDEN) && !is_fighting(victim))
		return FALSE;

	return TRUE;
}
/*
 * True if char can see obj.
 */
bool can_see_obj( CHAR_DATA *ch, OBJ_DATA *obj )
{
    if ( !IS_NPC(ch) && IS_SET(ch->act, PLR_HOLYLIGHT) )
	return TRUE;
    if ( !IS_NPC(ch) && IS_SET(ch->spec_flags, SPEC_WOLFBROTHER) )
	return TRUE;
    if ( IS_SET(obj->extra_flags,ITEM_VIS_DEATH))
	return FALSE;
    if ( IS_AFFECTED( ch, AFF_BLIND ) && obj->item_type != ITEM_POTION)
	return FALSE;
    if ( obj->item_type == ITEM_LIGHT && obj->value[2] != 0 )
	return TRUE;
    if ( IS_SET(obj->extra_flags, ITEM_INVIS)
    &&   !IS_AFFECTED(ch, AFF_DETECT_INVIS) )
        return FALSE;
    if ( IS_OBJ_STAT(obj,ITEM_GLOW))
	return TRUE;
    if ( room_is_dark( ch->in_room ) && !WOLF(ch) && !IS_AFFECTED(ch, AFF_DARK_VISION) )
	return FALSE;
    return TRUE;
}
/*
 * True if char can drop obj.
 */
bool can_drop_obj( CHAR_DATA *ch, OBJ_DATA *obj )
{
    if ( !IS_SET(obj->extra_flags, ITEM_NODROP) )
	return TRUE;
    if ( !IS_NPC(ch) && ch->level >= LEVEL_IMMORTAL )
	return TRUE;
    return FALSE;
}
/*
 * Return ascii name of an affect location.
 */
char *affect_loc_name( int location )
{
    switch ( location )
    {
    case APPLY_NONE:		return "none";
    case APPLY_STR:		return "strength";
    case APPLY_DEX:		return "dexterity";
    case APPLY_INT:		return "intelligence";
    case APPLY_WIS:		return "wisdom";
    case APPLY_CON:		return "constitution";
    case APPLY_REF:		return "reflex";
    case APPLY_INS:		return "instinct";
    case APPLY_CHA:		return "charisma";
    case APPLY_SEX:		return "sex";
    case APPLY_CLASS:		return "class";
    case APPLY_LEVEL:		return "level";
    case APPLY_AGE:		return "age";
    case APPLY_MANA:		return "stamina";
    case APPLY_HIT:		return "hp";
    case APPLY_MOVE:		return "stamina";
    case APPLY_GOLD:		return "gold";
    case APPLY_EXP:		return "experience";
    case APPLY_AC:		return "armor class";
    case APPLY_HITROLL:		return "hit roll";
    case APPLY_DAMROLL:		return "damage roll";
    case APPLY_SAVES:		return "saves";
    case APPLY_SAVING_ROD:	return "save vs rod";
    case APPLY_SAVING_PETRI:	return "save vs petrification";
    case APPLY_SAVING_BREATH:	return "save vs breath";
    case APPLY_SAVING_SPELL:	return "save vs spell";
    case APPLY_SPELL_AFFECT:	return "none";
    }
    bug( "Affect_location_name: unknown location %d.", location );
    return "(unknown)";
}
/*
 * Return ascii name of an affect bit vector.
 */
char *affect_bit_name( int vector )
{
    static char buf[512];
    buf[0] = '\0';
    if ( vector & AFF_BLIND         ) strcat( buf, " blind"         );
    if ( vector & AFF_FEAR          ) strcat( buf, " fear"         );
    if ( vector & AFF_INVISIBLE     ) strcat( buf, " invisible"     );
    if ( vector & AFF_DETECT_EVIL   ) strcat( buf, " detect_evil"   );
    if ( vector & AFF_DETECT_GOOD   ) strcat( buf, " detect_good"   );
    if ( vector & AFF_DETECT_INVIS  ) strcat( buf, " detect_invis"  );
    if ( vector & AFF_DETECT_MAGIC  ) strcat( buf, " detect_magic"  );
    if ( vector & AFF_DETECT_HIDDEN ) strcat( buf, " detect_hidden" );
    if ( vector & AFF_SANCTUARY     ) strcat( buf, " sanctuary"     );
    if ( vector & AFF_FAERIE_FIRE   ) strcat( buf, " faerie_fire"   );
    if ( vector & AFF_INFRARED      ) strcat( buf, " infrared"      );
    if ( vector & AFF_CURSE         ) strcat( buf, " curse"         );
    if ( vector & AFF_POISON        ) strcat( buf, " poison"        );
    if ( vector & AFF_PROTECT_EVIL  ) strcat( buf, " prot_evil"     );
    if ( vector & AFF_PROTECT_GOOD  ) strcat( buf, " prot_good"     );
    if ( vector & AFF_SLEEP         ) strcat( buf, " sleep"         );
    if ( vector & AFF_SNEAK         ) strcat( buf, " sneak"         );
    if ( vector & AFF_HIDE          ) strcat( buf, " hide"          );
    if ( vector & AFF_CHARM         ) strcat( buf, " charm"         );
    if ( vector & AFF_FLYING        ) strcat( buf, " flying"        );
    if ( vector & AFF_PASS_DOOR     ) strcat( buf, " pass_door"     );
    if ( vector & AFF_BERSERK	    ) strcat( buf, " berserk"	    );
    if ( vector & AFF_CALM	    ) strcat( buf, " calm"	    );
    if ( vector & AFF_HASTE	    ) strcat( buf, " haste"	    );
    if ( vector & AFF_SLOW          ) strcat( buf, " slow"          );
    if ( vector & AFF_PLAGUE	    ) strcat( buf, " plague" 	    );
    if ( vector & AFF_DARK_VISION   ) strcat( buf, " dark_vision"   );
    if ( vector & AFF_VOID          ) strcat( buf, " void"          );
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *affect2_bit_name( int vector )
{
    static char buf[512];
    buf[0] = '\0';
    if ( vector & AFF_BANDAGE ) strcat( buf, " bandage" );
    if ( vector & AFF_VEILED ) strcat( buf, " veil" );
    if ( vector & AFF_BIGGER ) strcat( buf, " bigger" );
    if ( vector & AFF_QUICKEN ) strcat( buf, " quicken" );
    if ( vector & AFF_INNERVATION ) strcat( buf, " innervation" );
    if ( vector & AFF_FORTITUDE ) strcat( buf, " fortitude" );
    if ( vector & AFF_SIXTH_SENSE ) strcat( buf, " sixth sense" );
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
/*
 * Return ascii name of extra flags vector.
 */
char *extra_bit_name( int extra_flags )
{
    static char buf[512];
    buf[0] = '\0';
    if ( extra_flags & ITEM_GLOW         ) strcat( buf, " glow"         );
    if ( extra_flags & ITEM_HUM          ) strcat( buf, " hum"          );
    if ( extra_flags & ITEM_DARK         ) strcat( buf, " dark"         );
    if ( extra_flags & ITEM_LOCK         ) strcat( buf, " lock"         );
    if ( extra_flags & ITEM_EVIL         ) strcat( buf, " evil"         );
    if ( extra_flags & ITEM_INVIS        ) strcat( buf, " invis"        );
    if ( extra_flags & ITEM_MAGIC        ) strcat( buf, " magic"        );
    if ( extra_flags & ITEM_NODROP       ) strcat( buf, " nodrop"       );
    if ( extra_flags & ITEM_CRAFTED      ) strcat( buf, " crafted"      );
    if ( extra_flags & ITEM_ADORNED      ) strcat( buf, " adorned"      );
    if ( extra_flags & ITEM_ANTI_GOOD    ) strcat( buf, " anti-good"    );
    if ( extra_flags & ITEM_ANTI_EVIL    ) strcat( buf, " anti-evil"    );
    if ( extra_flags & ITEM_NOREMOVE     ) strcat( buf, " noremove"     );
    if ( extra_flags & ITEM_INVENTORY    ) strcat( buf, " inventory"    );
    if ( extra_flags & ITEM_NOPURGE	 ) strcat( buf, " nopurge"	);
    if ( extra_flags & ITEM_VIS_DEATH	 ) strcat( buf, " vis_death"	);
    if ( extra_flags & ITEM_SHIMMER		 ) strcat( buf, " shimmer"	);
    if ( extra_flags & ITEM_ROT_DEATH	 ) strcat( buf, " rot_death"	);
    if ( extra_flags & ITEM_NOLOCATE	 ) strcat( buf, " no_locate"	);
    if ( extra_flags & ITEM_SELL_EXTRACT ) strcat( buf, " sell_extract" );
    if ( extra_flags & ITEM_SPIKED		 ) strcat( buf, " spiked" );
    if ( extra_flags & ITEM_BURN_PROOF	 ) strcat( buf, " burn_proof"	);
    if ( extra_flags & ITEM_NOUNCURSE	 ) strcat( buf, " no_uncurse"	);
	if ( extra_flags & ITEM_NOSELL		 ) strcat( buf, " nosell"		);
	if ( extra_flags & ITEM_BALANCED	 ) strcat( buf, " balanced"		);
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
/* return ascii name of an act vector */
char *act_bit_name( int act_flags )
{
    static char buf[512];
    buf[0] = '\0';
    if (IS_SET(act_flags,ACT_IS_NPC))
    {
 	strcat(buf," npc");
    	if (act_flags & ACT_SENTINEL 	) strcat(buf, " sentinel");
    	if (act_flags & ACT_SCAVENGER	) strcat(buf, " scavenger");
	if (act_flags & ACT_AGGRESSIVE	) strcat(buf, " aggressive");
	if (act_flags & ACT_STAY_AREA	) strcat(buf, " stay_area");
	if (act_flags & ACT_WIMPY	) strcat(buf, " wimpy");
	if (act_flags & ACT_PET		) strcat(buf, " pet");
	if (act_flags & ACT_MOUNT	) strcat(buf, " mount");
	if (act_flags & ACT_TRAIN	) strcat(buf, " train");
	if (act_flags & ACT_PRACTICE	) strcat(buf, " practice");
	if (act_flags & ACT_UNDEAD	) strcat(buf, " undead");
	if (act_flags & ACT_CLERIC	) strcat(buf, " cleric");
	if (act_flags & ACT_MAGE	) strcat(buf, " mage");
	if (act_flags & ACT_THIEF	) strcat(buf, " thief");
	if (act_flags & ACT_WARRIOR	) strcat(buf, " warrior");
	if (act_flags & ACT_NOALIGN	) strcat(buf, " no_align");
	if (act_flags & ACT_NOPURGE	) strcat(buf, " no_purge");
	if (act_flags & ACT_IS_HEALER	) strcat(buf, " healer");
	if (act_flags & ACT_IS_CHANGER  ) strcat(buf, " changer");
	if (act_flags & ACT_GAIN	) strcat(buf, " skill_train");
	if (act_flags & ACT_UPDATE_ALWAYS) strcat(buf," update_always");
    }
    else
    {
	strcat(buf," player");
	if (act_flags & PLR_AUTOASSIST	) strcat(buf, " autoass");
	if (act_flags & PLR_AUTOEXIT	) strcat(buf, " autoex");
	if (act_flags & PLR_AUTOLOOT	) strcat(buf, " autoloot");
	if (act_flags & PLR_AUTOSAC	) strcat(buf, " autosac");
	if (act_flags & PLR_AUTOGOLD	) strcat(buf, " autogold");
	if (act_flags & PLR_AUTOSPLIT	) strcat(buf, " autosplit");
	if (act_flags & PLR_HOLYLIGHT	) strcat(buf, " holylight");
	if (act_flags & PLR_CANLOOT	) strcat(buf, " lootcorpse");
	if (act_flags & PLR_NOSUMMON	) strcat(buf, " nosumm");
	if (act_flags & PLR_NOFOLLOW	) strcat(buf, " nofoll");
	if (act_flags & PLR_FREEZE	) strcat(buf, " frozen");
	if (act_flags & PLR_THIEF	) strcat(buf, " thief");
	if (act_flags & PLR_KILLER	) strcat(buf, " killer");
    }
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *comm_bit_name(int comm_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (comm_flags & COMM_QUIET		) strcat(buf, " quiet");
    if (comm_flags & COMM_DEAF		) strcat(buf, " deaf");
    if (comm_flags & COMM_NOWIZ		) strcat(buf, " no_wiz");
    if (comm_flags & COMM_NOAUCTION	) strcat(buf, " no_auction");
    if (comm_flags & COMM_NOIC	    ) strcat(buf, " no_inchar");
    if (comm_flags & COMM_NOOOC 	) strcat(buf, " no_ooc");
    if (comm_flags & COMM_NOQUESTION) strcat(buf, " no_question");
    if (comm_flags & COMM_NOMUSIC	) strcat(buf, " no_music");
    if (comm_flags & COMM_NOQUOTE	) strcat(buf, " no_quote");
    if (comm_flags & COMM_COMPACT	) strcat(buf, " compact");
    if (comm_flags & COMM_BRIEF		) strcat(buf, " brief");
    if (comm_flags & COMM_PROMPT	) strcat(buf, " prompt");
    if (comm_flags & COMM_COMBINE	) strcat(buf, " combine");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *off_forms_bit_name(int off_forms_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (off_forms_flags & StanceHeron)     strcat(buf, " Heron");
    if (off_forms_flags & StanceWind)      strcat(buf, " Wind");
    if (off_forms_flags & StanceWater)     strcat(buf, " Water");
    if (off_forms_flags & StanceSky)       strcat(buf, " Sky");
    if (off_forms_flags & StanceStone)     strcat(buf, " Stone");
    if (off_forms_flags & StanceStorm)     strcat(buf, " Storm");
    if (off_forms_flags & StanceLizard)    strcat(buf, " Lizard");
    if (off_forms_flags & StanceBoar)      strcat(buf, " Boar");
    if (off_forms_flags & StanceBird)      strcat(buf, " Bird");
    if (off_forms_flags & StanceCat)       strcat(buf, " Cat");
    if (off_forms_flags & StanceElemental) strcat(buf, " Elemental");
    if (off_forms_flags & StanceWill)      strcat(buf, " Will");
    if (off_forms_flags & StanceFire)      strcat(buf, " Fire");
    if (off_forms_flags & StanceIce)       strcat(buf, " Ice");
    if (off_forms_flags & StanceSpirit)    strcat(buf, " Spirit");
    if (off_forms_flags & StanceCircle)    strcat(buf, " Circle");
    if (off_forms_flags & StanceCrown)     strcat(buf, " Crown");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *off_sub_forms_bit_name(int off_sub_forms_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (off_sub_forms_flags & StanceHeronHOW)     strcat(buf, " Heron");
    if (off_sub_forms_flags & StanceHeronHRW)     strcat(buf, " Heron");
    if (off_sub_forms_flags & StanceHeronHSW)     strcat(buf, " Heron");
    if (off_sub_forms_flags & StanceHeronHRA)     strcat(buf, " Heron");
    if (off_sub_forms_flags & StanceHeronSTS)     strcat(buf, " Heron");
    if (off_sub_forms_flags & StanceWindABW)      strcat(buf, " Wind");
    if (off_sub_forms_flags & StanceWindCTF)      strcat(buf, " Wind");
    if (off_sub_forms_flags & StanceWindFTF)      strcat(buf, " Wind");
    if (off_sub_forms_flags & StanceWindLWR)      strcat(buf, " Wind");
    if (off_sub_forms_flags & StanceWindWOM)      strcat(buf, " Wind");
    if (off_sub_forms_flags & StanceWaterMOW)     strcat(buf, " Water");
    if (off_sub_forms_flags & StanceWaterMRW)     strcat(buf, " Water");
    if (off_sub_forms_flags & StanceWaterRUB)     strcat(buf, " Water");
    if (off_sub_forms_flags & StanceWaterWFD)     strcat(buf, " Water");
    if (off_sub_forms_flags & StanceWaterTWS)     strcat(buf, " Water");
    if (off_sub_forms_flags & StanceSkyAOM)       strcat(buf, " Sky");
    if (off_sub_forms_flags & StanceSkyLLS)       strcat(buf, " Sky");
    if (off_sub_forms_flags & StanceSkyRIA)       strcat(buf, " Sky");
    if (off_sub_forms_flags & StanceSkyWAR)       strcat(buf, " Sky");
    if (off_sub_forms_flags & StanceSkyLIS)       strcat(buf, " Sky");
    if (off_sub_forms_flags & StanceStoneBTS)     strcat(buf, " Stone");
    if (off_sub_forms_flags & StanceStoneTGT)     strcat(buf, " Stone");
    if (off_sub_forms_flags & StanceStoneSFC)     strcat(buf, " Stone");
    if (off_sub_forms_flags & StanceStoneSFM)     strcat(buf, " Stone");
    if (off_sub_forms_flags & StanceStoneFOS)     strcat(buf, " Stone");
    if (off_sub_forms_flags & StanceStormCAG)     strcat(buf, " Storm");
    if (off_sub_forms_flags & StanceStormTSG)     strcat(buf, " Storm");
    if (off_sub_forms_flags & StanceStormLOH)     strcat(buf, " Storm");
    if (off_sub_forms_flags & StanceStormTSA)     strcat(buf, " Storm");
    if (off_sub_forms_flags & StanceStormLSS)     strcat(buf, " Storm");
    if (off_sub_forms_flags & StanceLizardTLL)    strcat(buf, " Lizard");
    if (off_sub_forms_flags & StanceLizardLIT)    strcat(buf, " Lizard");
    if (off_sub_forms_flags & StanceLizardLTA)    strcat(buf, " Lizard");
    if (off_sub_forms_flags & StanceLizardTLS)    strcat(buf, " Lizard");
    if (off_sub_forms_flags & StanceLizardLFT)    strcat(buf, " Lizard");
    if (off_sub_forms_flags & StanceBoarBIB)      strcat(buf, " Boar");
    if (off_sub_forms_flags & StanceBoarHOB)      strcat(buf, " Boar");
    if (off_sub_forms_flags & StanceBoarBRB)      strcat(buf, " Boar");
    if (off_sub_forms_flags & StanceBoarBOH)      strcat(buf, " Boar");
    if (off_sub_forms_flags & StanceBoarBRM)      strcat(buf, " Boar");
    if (off_sub_forms_flags & StanceBirdHKH)      strcat(buf, " Bird");
    if (off_sub_forms_flags & StanceBirdWGD)      strcat(buf, " Bird");
    if (off_sub_forms_flags & StanceBirdSTF)      strcat(buf, " Bird");
    if (off_sub_forms_flags & StanceBirdSRA)      strcat(buf, " Bird");
    if (off_sub_forms_flags & StanceBirdSDW)      strcat(buf, " Bird");
    if (off_sub_forms_flags & StanceCatCSP)       strcat(buf, " Cat");
    if (off_sub_forms_flags & StanceCatCTP)       strcat(buf, " Cat");
    if (off_sub_forms_flags & StanceCatCTC)       strcat(buf, " Cat");
    if (off_sub_forms_flags & StanceCatCSA)       strcat(buf, " Cat");
    if (off_sub_forms_flags & StanceCatTCP)       strcat(buf, " Cat");
    if (off_sub_forms_flags & StanceElementalTFL) strcat(buf, " Elemental");
    if (off_sub_forms_flags & StanceElementalTFW) strcat(buf, " Elemental");
    if (off_sub_forms_flags & StanceElementalUTF) strcat(buf, " Elemental");
    if (off_sub_forms_flags & StanceElementalGOE) strcat(buf, " Elemental");
    if (off_sub_forms_flags & StanceElementalROR) strcat(buf, " Elemental");
    if (off_sub_forms_flags & StanceWillTOW)      strcat(buf, " Will");
    if (off_sub_forms_flags & StanceWillSOW)      strcat(buf, " Will");
    if (off_sub_forms_flags & StanceWillPOS)      strcat(buf, " Will");
    if (off_sub_forms_flags & StanceWillSIS)      strcat(buf, " Will");
    if (off_sub_forms_flags & StanceWillFOW)      strcat(buf, " Will");
    if (off_sub_forms_flags & StanceFireBRM)      strcat(buf, " Fire");
    if (off_sub_forms_flags & StanceFireFLS)      strcat(buf, " Fire");
    if (off_sub_forms_flags & StanceFirePOF)      strcat(buf, " Fire");
    if (off_sub_forms_flags & StanceFireSTB)      strcat(buf, " Fire");
    if (off_sub_forms_flags & StanceFireSOM)      strcat(buf, " Fire");
    if (off_sub_forms_flags & StanceIceSFO)       strcat(buf, " Ice");
    if (off_sub_forms_flags & StanceIceITR)       strcat(buf, " Ice");
    if (off_sub_forms_flags & StanceIceTCI)       strcat(buf, " Ice");
    if (off_sub_forms_flags & StanceIceIOW)       strcat(buf, " Ice");
    if (off_sub_forms_flags & StanceIceTFW)       strcat(buf, " Ice");
    if (off_sub_forms_flags & StanceSpiritGIW)    strcat(buf, " Spirit");
    if (off_sub_forms_flags & StanceSpiritVOW)    strcat(buf, " Spirit");
    if (off_sub_forms_flags & StanceSpiritSDS)    strcat(buf, " Spirit");
    if (off_sub_forms_flags & StanceSpiritGIM)    strcat(buf, " Spirit");
    if (off_sub_forms_flags & StanceSpiritROS)    strcat(buf, " Spirit");
    if (off_sub_forms_flags & StanceCircleCOT)    strcat(buf, " Circle");
    if (off_sub_forms_flags & StanceCircleCOF)    strcat(buf, " Circle");
    if (off_sub_forms_flags & StanceCircleCOP)    strcat(buf, " Circle");
    if (off_sub_forms_flags & StanceCircleCOI)    strcat(buf, " Circle");
    if (off_sub_forms_flags & StanceCircleCOS)    strcat(buf, " Circle");
    if (off_sub_forms_flags & StanceCrownTJC)     strcat(buf, " Crown");
    if (off_sub_forms_flags & StanceCrownTCS)     strcat(buf, " Crown");
    if (off_sub_forms_flags & StanceCrownTEC)     strcat(buf, " Crown");
    if (off_sub_forms_flags & StanceCrownTCW)     strcat(buf, " Crown");
    if (off_sub_forms_flags & StanceCrownTCU)     strcat(buf, " Crown");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *imm_bit_name(int imm_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (imm_flags & IMM_SUMMON		) strcat(buf, " summon");
    if (imm_flags & IMM_CHARM		) strcat(buf, " charm");
    if (imm_flags & IMM_MAGIC		) strcat(buf, " magic");
    if (imm_flags & IMM_WEAPON		) strcat(buf, " weapon");
    if (imm_flags & IMM_BASH		) strcat(buf, " blunt");
    if (imm_flags & IMM_PIERCE		) strcat(buf, " piercing");
    if (imm_flags & IMM_SLASH		) strcat(buf, " slashing");
    if (imm_flags & IMM_FIRE		) strcat(buf, " fire");
    if (imm_flags & IMM_COLD		) strcat(buf, " cold");
    if (imm_flags & IMM_LIGHTNING	) strcat(buf, " lightning");
    if (imm_flags & IMM_ACID		) strcat(buf, " acid");
    if (imm_flags & IMM_POISON		) strcat(buf, " poison");
    if (imm_flags & IMM_NEGATIVE	) strcat(buf, " negative");
    if (imm_flags & IMM_HOLY		) strcat(buf, " holy");
    if (imm_flags & IMM_ENERGY		) strcat(buf, " energy");
    if (imm_flags & IMM_MENTAL		) strcat(buf, " mental");
    if (imm_flags & IMM_DISEASE	) strcat(buf, " disease");
    if (imm_flags & IMM_DROWNING	) strcat(buf, " drowning");
    if (imm_flags & IMM_LIGHT		) strcat(buf, " light");
    if (imm_flags & VULN_IRON		) strcat(buf, " iron");
    if (imm_flags & VULN_WOOD		) strcat(buf, " wood");
    if (imm_flags & VULN_SILVER	) strcat(buf, " silver");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *wear_bit_name(int wear_flags)
{
    static char buf[512];
    buf [0] = '\0';
    if (wear_flags & ITEM_TAKE		) strcat(buf, " take");
    if (wear_flags & ITEM_WEAR_FINGER	) strcat(buf, " finger");
    if (wear_flags & ITEM_WEAR_NECK	) strcat(buf, " neck");
    if (wear_flags & ITEM_WEAR_BODY	) strcat(buf, " body");
    if (wear_flags & ITEM_WEAR_HEAD	) strcat(buf, " head");
    if (wear_flags & ITEM_WEAR_LEGS	) strcat(buf, " legs");
    if (wear_flags & ITEM_WEAR_FEET	) strcat(buf, " feet");
    if (wear_flags & ITEM_WEAR_HANDS	) strcat(buf, " hands");
    if (wear_flags & ITEM_WEAR_ARMS	) strcat(buf, " arms");
    if (wear_flags & ITEM_WEAR_SHIELD	) strcat(buf, " shield");
    if (wear_flags & ITEM_WEAR_ABOUT	) strcat(buf, " about");
    if (wear_flags & ITEM_WEAR_WAIST	) strcat(buf, " waist");
    if (wear_flags & ITEM_WEAR_WRIST	) strcat(buf, " wrist");
    if (wear_flags & ITEM_WEAR_NOSE ) strcat(buf, " nose");
    if (wear_flags & ITEM_WEAR_SHOULDERS ) strcat(buf, " shoulders");
    if (wear_flags & ITEM_WEAR_BACK ) strcat(buf, " back");
    if (wear_flags & ITEM_WEAR_EYES ) strcat(buf, " eyes");
    if (wear_flags & ITEM_WEAR_THIGH ) strcat(buf, " thigh");
    if (wear_flags & ITEM_WEAR_CALF ) strcat(buf, " calf");
    if (wear_flags & ITEM_WEAR_FOREARM ) strcat(buf, " forearm");
    if (wear_flags & ITEM_WEAR_EAR ) strcat(buf, " ear");
    if (wear_flags & ITEM_WIELD		) strcat(buf, " wield");
    if (wear_flags & ITEM_HOLD		) strcat(buf, " hold");
    if (wear_flags & ITEM_NO_SAC	) strcat(buf, " nosac");
    if (wear_flags & ITEM_WEAR_FLOAT	) strcat(buf, " float");
    if (wear_flags & ITEM_WEAR_ANKLE	) strcat(buf, " ankle");
    if (wear_flags & ITEM_WEAR_NEAR_MIND) strcat(buf, " nearmind");
    if (wear_flags & ITEM_WEAR_FACE) strcat(buf, " face");
    if (wear_flags & ITEM_WEAR_ANGREAL) strcat(buf, " angreal");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *form_bit_name(int form_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (form_flags & FORM_POISON	) strcat(buf, " poison");
    else if (form_flags & FORM_EDIBLE	) strcat(buf, " edible");
    if (form_flags & FORM_MAGICAL	) strcat(buf, " magical");
    if (form_flags & FORM_INSTANT_DECAY	) strcat(buf, " instant_rot");
    if (form_flags & FORM_OTHER		) strcat(buf, " other");
    if (form_flags & FORM_ANIMAL	) strcat(buf, " animal");
    if (form_flags & FORM_SENTIENT	) strcat(buf, " sentient");
    if (form_flags & FORM_UNDEAD	) strcat(buf, " undead");
    if (form_flags & FORM_CONSTRUCT	) strcat(buf, " construct");
    if (form_flags & FORM_MIST		) strcat(buf, " mist");
    if (form_flags & FORM_INTANGIBLE	) strcat(buf, " intangible");
    if (form_flags & FORM_BIPED		) strcat(buf, " biped");
    if (form_flags & FORM_CENTAUR	) strcat(buf, " centaur");
    if (form_flags & FORM_INSECT	) strcat(buf, " insect");
    if (form_flags & FORM_SPIDER	) strcat(buf, " spider");
    if (form_flags & FORM_CRUSTACEAN	) strcat(buf, " crustacean");
    if (form_flags & FORM_WORM		) strcat(buf, " worm");
    if (form_flags & FORM_BLOB		) strcat(buf, " blob");
    if (form_flags & FORM_MAMMAL	) strcat(buf, " mammal");
    if (form_flags & FORM_BIRD		) strcat(buf, " bird");
    if (form_flags & FORM_REPTILE	) strcat(buf, " reptile");
    if (form_flags & FORM_SNAKE		) strcat(buf, " snake");
    if (form_flags & FORM_DRAGON	) strcat(buf, " dragon");
    if (form_flags & FORM_AMPHIBIAN	) strcat(buf, " amphibian");
    if (form_flags & FORM_FISH		) strcat(buf, " fish");
    if (form_flags & FORM_COLD_BLOOD 	) strcat(buf, " cold_blooded");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *part_bit_name(int part_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (part_flags & PART_HEAD		) strcat(buf, " head");
    if (part_flags & PART_ARMS		) strcat(buf, " arms");
    if (part_flags & PART_LEGS		) strcat(buf, " legs");
    if (part_flags & PART_HEART		) strcat(buf, " heart");
    if (part_flags & PART_BRAINS	) strcat(buf, " brains");
    if (part_flags & PART_GUTS		) strcat(buf, " guts");
    if (part_flags & PART_HANDS		) strcat(buf, " hands");
    if (part_flags & PART_FEET		) strcat(buf, " feet");
    if (part_flags & PART_FINGERS	) strcat(buf, " fingers");
    if (part_flags & PART_EAR		) strcat(buf, " ears");
    if (part_flags & PART_EYE		) strcat(buf, " eyes");
    if (part_flags & PART_LONG_TONGUE	) strcat(buf, " long_tongue");
    if (part_flags & PART_EYESTALKS	) strcat(buf, " eyestalks");
    if (part_flags & PART_TENTACLES	) strcat(buf, " tentacles");
    if (part_flags & PART_FINS		) strcat(buf, " fins");
    if (part_flags & PART_WINGS		) strcat(buf, " wings");
    if (part_flags & PART_TAIL		) strcat(buf, " tail");
    if (part_flags & PART_CLAWS		) strcat(buf, " claws");
    if (part_flags & PART_FANGS		) strcat(buf, " fangs");
    if (part_flags & PART_HORNS		) strcat(buf, " horns");
    if (part_flags & PART_SCALES	) strcat(buf, " scales");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *weapon_bit_name(int weapon_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (weapon_flags & WEAPON_FLAMING	) strcat(buf, " flaming");
    if (weapon_flags & WEAPON_FROST	) strcat(buf, " frost");
    if (weapon_flags & WEAPON_VAMPIRIC	) strcat(buf, " vampiric");
    if (weapon_flags & WEAPON_SHARP	) strcat(buf, " sharp");
    if (weapon_flags & WEAPON_VORPAL	) strcat(buf, " vorpal");
    if (weapon_flags & WEAPON_TWO_HANDS ) strcat(buf, " two-handed");
    if (weapon_flags & WEAPON_SHOCKING 	) strcat(buf, " shocking");
    if (weapon_flags & WEAPON_POISON	) strcat(buf, " poison");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *auto_bit_name( int auto_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (auto_flags & AUTO_AUTODAMAGE) strcat(buf, " dmg");
    if (auto_flags & AUTO_AUTOBUTCHER) strcat(buf, " butcher");
    if (auto_flags & AUTO_AUTOSLAY) strcat(buf, " slay");
    if (auto_flags & AUTO_AUTOLOG) strcat(buf, " log");
    if (auto_flags & AUTO_AUTOSKIN) strcat(buf, " skin");
    if (auto_flags & AUTO_HINTS) strcat(buf, " hints");
    if (auto_flags & AUTO_AUTORECALL) strcat(buf, " recall");
    if (auto_flags & AUTO_AUTOMAP) strcat(buf, " map");
    if (auto_flags & AUTO_COUNTER) strcat(buf, " counter");
    if (auto_flags & AUTO_PARRY) strcat(buf, " parry");
    if (auto_flags & AUTO_DODGE) strcat(buf, " dodge");
    if (auto_flags & AUTO_WAR) strcat(buf, " war");
    if (auto_flags & AUTO_PEEK) strcat(buf, " peek");
    if (auto_flags & AUTO_ROA) strcat(buf, " roa");
    if (auto_flags & AUTO_FTAG) strcat(buf, " ftag");
    if (auto_flags & AUTO_TOURNEY) strcat(buf, " tourney");
    if (auto_flags & AUTO_TRIVIA) strcat(buf, " trivia");
    if (auto_flags & AUTO_AUTOBOX) strcat(buf, " box");
    if (auto_flags & AUTO_SAVEMESSAGE) strcat(buf, " savemsg");
    if (auto_flags & AUTO_AUTOH2H) strcat(buf, " h2h");
    if (auto_flags & AUTO_AUTOSEARCH) strcat(buf, " search");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *op_bit_name( int op_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (op_flags & OP_CHANNELER) strcat(buf, " channeler");
    if (op_flags & OP_GRASPED) strcat(buf, " grasped");
    if (op_flags & OP_FSHIELD) strcat(buf, " fshield");
    if (op_flags & OP_LARGE) strcat(buf, " large");
    if (op_flags & OP_BARRIER) strcat(buf, " barrier");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *spec_bit_name( int spec_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (spec_flags & SPEC_WOLFBROTHER) strcat(buf, " wolfkin");
    if (spec_flags & SPEC_TAVEREN) strcat(buf, " taveren");
    if (spec_flags & SPEC_AGELESS) strcat(buf, " ageless");
    if (spec_flags & SPEC_DREAMWALKER) strcat(buf, " dreamer");
    if (spec_flags & SPEC_DREAMING) strcat(buf, " dreaming");
    if (spec_flags & SPEC_MENTOR) strcat(buf, " mentor");
    if (spec_flags & SPEC_SEER) strcat(buf, " seer");
    if (spec_flags & SPEC_MARRIED) strcat(buf, " married");
    if (spec_flags & SPEC_X) strcat(buf, " xsoc");
    if (spec_flags & SPEC_NOTEACH) strcat(buf, " noteach");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *hist_bit_name( int hist_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (hist_flags & HIST_MERCHANT) strcat(buf, " merchant");
    if (hist_flags & HIST_SOLDIER) strcat(buf, " soldier");
    if (hist_flags & HIST_BEGGAR) strcat(buf, " beggar");
    if (hist_flags & HIST_ARTIST) strcat(buf, " artist");
    if (hist_flags & HIST_FARMER) strcat(buf, " farmer");
    if (hist_flags & HIST_CHILD) strcat(buf, " child");
    if (hist_flags & HIST_MUSICIAN) strcat(buf, " musician");
    if (hist_flags & HIST_SAILOR) strcat(buf, " sailor");
    if (hist_flags & HIST_NOBLE) strcat(buf, " noble");
    if (hist_flags & HIST_SHADOW) strcat(buf, " shadow");
    if (hist_flags & HIST_RECRUITER) strcat(buf, " recruiter");
    if (hist_flags & HIST_GUILDLEADER) strcat(buf, " gl");
    if (hist_flags & HIST_SHADOWGIFT) strcat(buf, " shadgift");
    if (hist_flags & HIST_BONDER) strcat(buf, " bonder");
    if (hist_flags & HIST_BONDED) strcat(buf, " bonded");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
char *war_bit_name( int war_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (war_flags & WAR_VEILED) strcat(buf, " veil");
    if (war_flags & WAR_QUESTOR) strcat(buf, " questor");
    if (war_flags & WAR_BOUNTY) strcat(buf, " bounty");
    if (war_flags & WAR_PK) strcat(buf, " pk");
    if (war_flags & WAR_MSP) strcat(buf, " msp");
    if (war_flags & WAR_HELD) strcat(buf, " held");
    if (war_flags & WAR_MODERATOR) strcat(buf, " moderator");
    if (war_flags & WAR_DAISHAN) strcat(buf, " daishan");
    if (war_flags & WAR_MHAEL) strcat(buf, " mhael");
    if (war_flags & WAR_NAEBLIS) strcat(buf, " naeblis");
    if (war_flags & WAR_MASTBARD) strcat(buf, " mastbard");
    if (war_flags & WAR_QGUARD) strcat(buf, " qguard");
    if (war_flags & WAR_WISEST) strcat(buf, " wisest");
    if (war_flags & WAR_MENTOR) strcat(buf, " mentor");
    if (war_flags & WAR_CODER) strcat(buf, " coder");
    if (war_flags & WAR_PROPHET) strcat(buf, " prophet");
    if (war_flags & WAR_AUTOFIGHT) strcat(buf, " autofight");
    if (war_flags & WAR_IMMORTAL) strcat(buf, " immortal");
    if (war_flags & WAR_GQUEST) strcat(buf, " gquest");
    if (war_flags & WAR_BATTLE) strcat(buf, " battle");
    if (war_flags & WAR_TOURNEY) strcat(buf, " yourney");
    return (buf[0] != '\0' ) ? buf+1 : "none";
}
char *cont_bit_name( int cont_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (cont_flags & CONT_CLOSEABLE	) strcat(buf, " closable");
    if (cont_flags & CONT_PICKPROOF	) strcat(buf, " pickproof");
    if (cont_flags & CONT_CLOSED	) strcat(buf, " closed");
    if (cont_flags & CONT_LOCKED	) strcat(buf, " locked");
    return (buf[0] != '\0' ) ? buf+1 : "none";
}
char *off_bit_name(int off_flags)
{
    static char buf[512];
    buf[0] = '\0';
    if (off_flags & OFF_AREA_ATTACK	) strcat(buf, " area attack");
    if (off_flags & OFF_BACKSTAB	) strcat(buf, " backstab");
    if (off_flags & OFF_BASH		) strcat(buf, " bash");
    if (off_flags & OFF_BERSERK		) strcat(buf, " berserk");
    if (off_flags & OFF_DISARM		) strcat(buf, " disarm");
    if (off_flags & OFF_DODGE		) strcat(buf, " dodge");
    if (off_flags & OFF_FADE		) strcat(buf, " fade");
    if (off_flags & OFF_FAST		) strcat(buf, " fast");
    if (off_flags & OFF_KICK		) strcat(buf, " kick");
    if (off_flags & OFF_KICK_DIRT	) strcat(buf, " kick_dirt");
    if (off_flags & OFF_PARRY		) strcat(buf, " parry");
    if (off_flags & OFF_RESCUE		) strcat(buf, " rescue");
    if (off_flags & OFF_TAIL		) strcat(buf, " tail");
    if (off_flags & OFF_TRIP		) strcat(buf, " trip");
    if (off_flags & OFF_CRUSH		) strcat(buf, " crush");
    if (off_flags & ASSIST_ALL		) strcat(buf, " assist_all");
    if (off_flags & ASSIST_ALIGN	) strcat(buf, " assist_align");
    if (off_flags & ASSIST_RACE		) strcat(buf, " assist_race");
    if (off_flags & ASSIST_PLAYERS	) strcat(buf, " assist_players");
    if (off_flags & ASSIST_GUARD	) strcat(buf, " assist_guard");
    if (off_flags & ASSIST_VNUM		) strcat(buf, " assist_vnum");
    return ( buf[0] != '\0' ) ? buf+1 : "none";
}
/*
 * Config Colour stuff
 */
void default_colour( CHAR_DATA *ch )
{
    if( IS_NPC( ch ) )
	return;
    if( !ch->pcdata )
	return;
    ch->pcdata->text[1]		= ( WHITE );
    ch->pcdata->auction[1]	= ( YELLOW );
    ch->pcdata->auction_text[1]	= ( WHITE );
    ch->pcdata->gossip[1]	= ( MAGENTA );
    ch->pcdata->gossip_text[1]	= ( MAGENTA );
    ch->pcdata->music[1]	= ( RED );
    ch->pcdata->music_text[1]	= ( RED );
    ch->pcdata->question[1]	= ( YELLOW );
    ch->pcdata->question_text[1] = ( WHITE );
    ch->pcdata->answer[1]	= ( YELLOW );
    ch->pcdata->answer_text[1]	= ( WHITE );
    ch->pcdata->quote[1]	= ( GREEN );
    ch->pcdata->quote_text[1]	= ( GREEN );
    ch->pcdata->immtalk_text[1]	= ( CYAN );
    ch->pcdata->immtalk_type[1]	= ( YELLOW );
    ch->pcdata->info[1]		= ( YELLOW );
    ch->pcdata->tell[1]		= ( GREEN );
    ch->pcdata->tell_text[1]	= ( GREEN );
    ch->pcdata->say[1]		= ( GREEN );
    ch->pcdata->say_text[1]	= ( GREEN );
    ch->pcdata->reply[1]	= ( GREEN );
    ch->pcdata->reply_text[1]	= ( GREEN );
    ch->pcdata->gtell_text[1]	= ( GREEN );
    ch->pcdata->gtell_type[1]	= ( RED );
    ch->pcdata->wiznet[1]	= ( GREEN );
    ch->pcdata->room_title[1]	= ( CYAN );
    ch->pcdata->room_text[1]	= ( WHITE );
    ch->pcdata->room_exits[1]	= ( GREEN );
    ch->pcdata->room_things[1]	= ( CYAN );
    ch->pcdata->prompt[1]	= ( CYAN );
    ch->pcdata->fight_death[1]	= ( RED );
    ch->pcdata->fight_yhit[1]	= ( GREEN );
    ch->pcdata->fight_ohit[1]	= ( YELLOW );
    ch->pcdata->fight_thit[1]	= ( RED );
    ch->pcdata->fight_skill[1]	= ( WHITE );
    ch->pcdata->text[0]		= ( NORMAL );
    ch->pcdata->auction[0]	= ( BRIGHT );
    ch->pcdata->auction_text[0]	= ( BRIGHT );
    ch->pcdata->gossip[0]	= ( NORMAL );
    ch->pcdata->gossip_text[0]	= ( BRIGHT );
    ch->pcdata->music[0]	= ( NORMAL );
    ch->pcdata->music_text[0]	= ( BRIGHT );
    ch->pcdata->question[0]	= ( BRIGHT );
    ch->pcdata->question_text[0] = ( BRIGHT );
    ch->pcdata->answer[0]	= ( BRIGHT );
    ch->pcdata->answer_text[0]	= ( BRIGHT );
    ch->pcdata->quote[0]	= ( NORMAL );
    ch->pcdata->quote_text[0]	= ( BRIGHT );
    ch->pcdata->immtalk_text[0]	= ( NORMAL );
    ch->pcdata->immtalk_type[0]	= ( NORMAL );
    ch->pcdata->info[0]		= ( NORMAL );
    ch->pcdata->say[0]		= ( NORMAL );
    ch->pcdata->say_text[0]	= ( BRIGHT );
    ch->pcdata->tell[0]		= ( NORMAL );
    ch->pcdata->tell_text[0]	= ( BRIGHT );
    ch->pcdata->reply[0]	= ( NORMAL );
    ch->pcdata->reply_text[0]	= ( BRIGHT );
    ch->pcdata->gtell_text[0]	= ( NORMAL );
    ch->pcdata->gtell_type[0]	= ( NORMAL );
    ch->pcdata->wiznet[0]	= ( NORMAL );
    ch->pcdata->room_title[0]	= ( NORMAL );
    ch->pcdata->room_text[0]	= ( NORMAL );
    ch->pcdata->room_exits[0]	= ( NORMAL );
    ch->pcdata->room_things[0]	= ( NORMAL );
    ch->pcdata->prompt[0]	= ( NORMAL );
    ch->pcdata->fight_death[0]	= ( NORMAL );
    ch->pcdata->fight_yhit[0]	= ( NORMAL );
    ch->pcdata->fight_ohit[0]	= ( NORMAL );
    ch->pcdata->fight_thit[0]	= ( NORMAL );
    ch->pcdata->fight_skill[0]	= ( NORMAL );
    ch->pcdata->text[2]		= 0;
    ch->pcdata->auction[2]	= 0;
    ch->pcdata->auction_text[2]	= 0;
    ch->pcdata->gossip[2]	= 0;
    ch->pcdata->gossip_text[2]	= 0;
    ch->pcdata->music[2]	= 0;
    ch->pcdata->music_text[2]	= 0;
    ch->pcdata->question[2]	= 0;
    ch->pcdata->question_text[2] = 0;
    ch->pcdata->answer[2]	= 0;
    ch->pcdata->answer_text[2]	= 0;
    ch->pcdata->quote[2]	= 0;
    ch->pcdata->quote_text[2]	= 0;
    ch->pcdata->immtalk_text[2]	= 0;
    ch->pcdata->immtalk_type[2]	= 0;
    ch->pcdata->info[2]		= 1;
    ch->pcdata->say[2]		= 0;
    ch->pcdata->say_text[2]	= 0;
    ch->pcdata->tell[2]		= 0;
    ch->pcdata->tell_text[2]	= 0;
    ch->pcdata->reply[2]	= 0;
    ch->pcdata->reply_text[2]	= 0;
    ch->pcdata->gtell_text[2]	= 0;
    ch->pcdata->gtell_type[2]	= 0;
    ch->pcdata->wiznet[2]	= 0;
    ch->pcdata->room_title[2]	= 0;
    ch->pcdata->room_text[2]	= 0;
    ch->pcdata->room_exits[2]	= 0;
    ch->pcdata->room_things[2]	= 0;
    ch->pcdata->prompt[2]	= 0;
    ch->pcdata->fight_death[2]	= 0;
    ch->pcdata->fight_yhit[2]	= 0;
    ch->pcdata->fight_ohit[2]	= 0;
    ch->pcdata->fight_thit[2]	= 0;
    ch->pcdata->fight_skill[2]	= 0;
    return;
}
void all_colour( CHAR_DATA *ch, char *argument )
{
    char	buf[  100 ];
    char	buf2[ 100 ];
    int		colour;
    int		bright;
    if( IS_NPC( ch ) || !ch->pcdata )
	return;
    if( !*argument )
	return;
    if( !str_prefix( argument, "red" ) )
    {
	colour = ( RED );
	bright = NORMAL;
	sprintf( buf2, "Red" );
    }
    if( !str_prefix( argument, "hi-red" ) )
    {
	colour = ( RED );
	bright = BRIGHT;
	sprintf( buf2, "Red" );
    }
    else if( !str_prefix( argument, "green" ) )
    {
	colour = ( GREEN );
	bright = NORMAL;
	sprintf( buf2, "Green" );
    }
    else if( !str_prefix( argument, "hi-green" ) )
    {
	colour = ( GREEN );
	bright = BRIGHT;
	sprintf( buf2, "Green" );
    }
    else if( !str_prefix( argument, "yellow" ) )
    {
	colour = ( YELLOW );
	bright = NORMAL;
	sprintf( buf2, "Yellow" );
    }
    else if( !str_prefix( argument, "hi-yellow" ) )
    {
	colour = ( YELLOW );
	bright = BRIGHT;
	sprintf( buf2, "Yellow" );
    }
    else if( !str_prefix( argument, "blue" ) )
    {
	colour = ( BLUE );
	bright = NORMAL;
	sprintf( buf2, "Blue" );
    }
    else if( !str_prefix( argument, "hi-blue" ) )
    {
	colour = ( BLUE );
	bright = BRIGHT;
	sprintf( buf2, "Blue" );
    }
    else if( !str_prefix( argument, "magenta" ) )
    {
	colour = ( MAGENTA );
	bright = NORMAL;
	sprintf( buf2, "Magenta" );
    }
    else if( !str_prefix( argument, "hi-magenta" ) )
    {
	colour = ( MAGENTA );
	bright = BRIGHT;
	sprintf( buf2, "Magenta" );
    }
    else if( !str_prefix( argument, "cyan" ) )
    {
	colour = ( CYAN );
	bright = NORMAL;
	sprintf( buf2, "Cyan" );
    }
    else if( !str_prefix( argument, "hi-cyan" ) )
    {
	colour = ( CYAN );
	bright = BRIGHT;
	sprintf( buf2, "Cyan" );
    }
    else if( !str_prefix( argument, "white" ) )
    {
	colour = ( WHITE );
	bright = NORMAL;
	sprintf( buf2, "White" );
    }
    else if( !str_prefix( argument, "hi-white" ) )
    {
	colour = ( WHITE );
	bright = BRIGHT;
	sprintf( buf2, "White" );
    }
    else if( !str_prefix( argument, "grey" ) )
    {
	colour = ( BLACK );
	bright = BRIGHT;
	sprintf( buf2, "White" );
    }
    else
    {
	send_to_char_bw( "Unrecognised colour, unchanged.\n\r", ch );
	return;
    }
    ch->pcdata->text[1]		= colour;
    ch->pcdata->auction[1]	= colour;
    ch->pcdata->gossip[1]	= colour;
    ch->pcdata->music[1]	= colour;
    ch->pcdata->question[1]	= colour;
    ch->pcdata->answer[1]	= colour;
    ch->pcdata->quote[1]	= colour;
    ch->pcdata->quote_text[1]	= colour;
    ch->pcdata->immtalk_text[1]	= colour;
    ch->pcdata->immtalk_type[1]	= colour;
    ch->pcdata->info[1]		= colour;
    ch->pcdata->say[1]		= colour;
    ch->pcdata->say_text[1]	= colour;
    ch->pcdata->tell[1]		= colour;
    ch->pcdata->tell_text[1]	= colour;
    ch->pcdata->reply[1]	= colour;
    ch->pcdata->reply_text[1]	= colour;
    ch->pcdata->gtell_text[1]	= colour;
    ch->pcdata->gtell_type[1]	= colour;
    ch->pcdata->wiznet[1]	= colour;
    ch->pcdata->room_title[1]	= colour;
    ch->pcdata->room_text[1]	= colour;
    ch->pcdata->room_exits[1]	= colour;
    ch->pcdata->room_things[1]	= colour;
    ch->pcdata->prompt[1]	= colour;
    ch->pcdata->fight_death[1]	= colour;
    ch->pcdata->fight_yhit[1]	= colour;
    ch->pcdata->fight_ohit[1]	= colour;
    ch->pcdata->fight_thit[1]	= colour;
    ch->pcdata->fight_skill[1]	= colour;
    ch->pcdata->text[0]		= bright;
    ch->pcdata->auction[0]	= bright;
    ch->pcdata->gossip[0]	= bright;
    ch->pcdata->music[0]	= bright;
    ch->pcdata->question[0]	= bright;
    ch->pcdata->answer[0]	= bright;
    ch->pcdata->quote[0]	= bright;
    ch->pcdata->quote_text[0]	= bright;
    ch->pcdata->immtalk_text[0]	= bright;
    ch->pcdata->immtalk_type[0]	= bright;
    ch->pcdata->info[0]		= bright;
    ch->pcdata->say[0]		= bright;
    ch->pcdata->say_text[0]	= bright;
    ch->pcdata->tell[0]		= bright;
    ch->pcdata->tell_text[0]	= bright;
    ch->pcdata->reply[0]	= bright;
    ch->pcdata->reply_text[0]	= bright;
    ch->pcdata->gtell_text[0]	= bright;
    ch->pcdata->gtell_type[0]	= bright;
    ch->pcdata->wiznet[0]	= bright;
    ch->pcdata->room_title[0]	= bright;
    ch->pcdata->room_text[0]	= bright;
    ch->pcdata->room_exits[0]	= bright;
    ch->pcdata->room_things[0]	= bright;
    ch->pcdata->prompt[0]	= bright;
    ch->pcdata->fight_death[0]	= bright;
    ch->pcdata->fight_yhit[0]	= bright;
    ch->pcdata->fight_ohit[0]	= bright;
    ch->pcdata->fight_thit[0]	= bright;
    ch->pcdata->fight_skill[0]	= bright;
    sprintf( buf, "{cAll Colour settings set to %s.\n\r", buf2 );
    send_to_char_bw( buf, ch );
    return;
}
char *item_type_name( OBJ_DATA *obj )
{
    switch ( obj->item_type )
    {
    case ITEM_LIGHT:		return "light";
    case ITEM_SCROLL:		return "scroll";
    case ITEM_WAND:		return "wand";
    case ITEM_STAFF:		return "staff";
    case ITEM_WEAPON:		return "weapon";
    case ITEM_TREASURE:		return "treasure";
    case ITEM_HERB:		return "herb";
    case ITEM_ARMOR:		return "armor";
    case ITEM_POTION:		return "potion";
    case ITEM_FURNITURE:	return "furniture";
    case ITEM_TRIGGER:      return "trigger";
    case ITEM_MIRROR:       return "mirror";
    case ITEM_NOTEPAPER:    return "note paper";
    case ITEM_ANGREAL_MALE:  return "male angreal";
    case ITEM_ANGREAL_FEMALE:  return "female angreal";
    case ITEM_TREASURECHEST: return "treasure chest";
    case ITEM_LOCKPICK:     return "lockpick";
    case ITEM_TRASH:		return "trash";
    case ITEM_CONTAINER:	return "container";
    case ITEM_DRINK_CON:	return "drink container";
    case ITEM_KEY:		return "key";
    case ITEM_FOOD:		return "food";
    case ITEM_MONEY:		return "money";
    case ITEM_BOAT:		return "boat";
    case ITEM_CORPSE_NPC:	return "npc corpse";
    case ITEM_CORPSE_PC:	return "pc corpse";
    case ITEM_FOUNTAIN:		return "fountain";
    case ITEM_PILL:		return "pill";
    case ITEM_MAP:		return "map";
    case ITEM_RAZOR:    return "razor";
    case ITEM_BRUSH:    return "brush";
    }
    bug( "Item_type_name: unknown type %d.", obj->item_type );
    return "(unknown)";
}
bool filter_string (char *argument)
{
	/*char temp[100]="L FUCK HI FUCK FUCK FUCK HELLO ALSDJ HELLO GUCK FUCK YOUA I KKKKKN FUCK";*/
	char fuck[5]="fuck";
	char shit[5]="shit";
	char dick[5]="dick";
	char cunt[5]="cunt";
	char twat[5]="twat";
	char cock[5]="cock";
	char slut[5]="slut";
	char *ptemp,*x;
	int z,j,match,sum=0,y=3;
	bool found = FALSE;
	/*ptemp=temp;*/
	ptemp=argument;
	while(*ptemp!='\0'){
		//*************
		//*If array is max out this will bring x back to a valid position
		//*************
		//if (y>5)
		//	x=ptemp+(10-y);
		//else
			x=ptemp+5;
		//****************
		//*This checks last word to see if it is a four letter
		//*word.  If it is it checks if to see if it is a match
		//****************
		if ((*x=='\0') && (isspace(*ptemp))){
			++ptemp;
			++y;
			for(z=0,j=0;z<4;z++){
				if (*ptemp==fuck[j]
				|| *ptemp==shit[j]
				|| *ptemp==dick[j]
				|| *ptemp==cunt[j]
				|| *ptemp==twat[j]
				|| *ptemp==cock[j]
				|| *ptemp==slut[j]){
					match=1;
					++j;
					++ptemp;
					++y;
					found = TRUE;
				}
				else{
					match=0;
					break;
				}
			}
			sum+=match;
		}
		if (*x=='\0')
			break;
		//*************
		//*This condition advances ptemp to next letter
		//*to avoid ifinite loop
		//*************
		if ((isalpha(*x)) && (isspace(*ptemp))){
			++ptemp;
			++y;
		}
		//*************
		//*If this condition is true we know we have found a
		//*four letter word
		//*************
		if((isspace(*ptemp)) && (isspace(*x))){
			++ptemp;
			++y;
			for(z=0,j=0;z<4;z++){
				if (*ptemp==fuck[j]
				|| *ptemp==shit[j]
				|| *ptemp==dick[j]
				|| *ptemp==cunt[j]
				|| *ptemp==twat[j]
				|| *ptemp==cock[j]
				|| *ptemp==slut[j]){
					match=1;
					++j;
					++ptemp;
					++y;
					found = TRUE;
				}
				else{
					match=0;
					break;
				}
			}
			sum+=match;
		}
		else{
			//************
			//*This advances ptemp to next word
			//************
			while (!(isspace(*ptemp))){
				if (*ptemp=='\0')
					break;
				++ptemp;
				++y;
			}
		}
	}
	if (found) return TRUE;
	else return FALSE;
}
