/***************************************************************************
                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/
    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.
    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.
    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.
    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.
    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.
	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "db.h"
#include "olc.h"
#include "mysql5_interface.h"

void do_helpcheck (CHAR_DATA *ch, char *argument)
{
    HELP_DATA *pHelp;
    char buf[MSL], buf2[MSL];
    BUFFER *buffer;
    int cmd, col = 0;
    bool found;
    buffer = new_buf();

    if ( argument[0] == '\0' )
    {
        stc ( "Syntax: helpcheck <commands/skills/socials>\n\r", ch );
        return;
    }
    if ( !str_prefix ( argument, "socials" ) )
    {
        int iSoc;

        found = FALSE;
        stc ( "Comparing socials and commands....\n\r", ch );
        for ( cmd = 0; cmd_table[cmd].name != NULL && cmd_table[cmd].name[0] != '\0'; cmd++ )
        {
            for ( iSoc = 0; social_table[iSoc].name != NULL; iSoc++ )
            {
                if ( !str_prefix( social_table[iSoc].name, cmd_table[cmd].name ) )
                {
                    sprintf ( buf2, "%-20s", social_table[iSoc].name );
                    add_buf(buffer, buf2);
                    /*stc ( buf, ch );*/
                    found = TRUE;
                    if ( ++col % 4 == 0 )
                        /*stc ( "\n\r", ch );*/
                        add_buf(buffer, "\n\r" );
                    if ( iSoc > MAX_SOCIALS )
                    	break;

                }
            }
        }
        if ( !found )
            stc ( "No socials are the names of commands.\n\r", ch );

        else if ( ++col % 4 != 0 )
            stc ( "\n\r", ch );
        stc ( "\n\r", ch );

        page_to_char(buf_string(buffer), ch);
        free_buf( buffer );

        return;
    }
    else if ( !str_prefix ( argument, "commands" ) )
    {
        stc ( "Checking for missing command help entries....\n\r", ch );
        for ( cmd = 0; cmd_table[cmd].name != NULL; cmd++ )
        {
            if ( cmd_table[cmd].show == 0 )
                continue;

            found = FALSE;
            for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
            {
                if ( pHelp->keyword[0] == '\0' )
                    continue;

                if ( is_name ( cmd_table[cmd].name, pHelp->keyword ) )
                {
                    found = TRUE;
                    break;
                }
            }
            if ( !found )
            {
                sprintf ( buf, "%-20s", cmd_table[cmd].name );
                stc ( buf, ch );
                if ( ++col % 4 == 0 )
                    stc ( "\n\r", ch );
            }
        }
        if ( ++col % 4 != 0 )
            stc ( "\n\r", ch );
        stc ( "\n\r", ch );

        return;
    }
    else if ( !str_prefix ( argument, "skills" ) )
    {
        stc ( "Checking for missing skill help entries....\n\r", ch );
        for ( cmd = 1; skill_table[cmd].name != NULL; cmd++ )
        {
            found = FALSE;
            for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
            {
                if ( pHelp->keyword[0] == '\0' )
                    continue;

                if ( is_name ( skill_table[cmd].name, pHelp->keyword ) )
                {
                    found = TRUE;
                    break;
                }
            }
            if ( !found )
            {
                sprintf ( buf, "%-20s", skill_table[cmd].name );
                stc ( buf, ch );
                if ( ++col % 4 == 0 )
                    stc ( "\n\r", ch );
            }
        }
        if ( ++col % 4 != 0 )
            stc ( "\n\r", ch );
        stc ( "\n\r", ch );
        stc ( "Checking for missing group help entries....\n\r", ch );
        for ( cmd = 0; group_table[cmd].name != NULL; cmd++ )
        {
            found = FALSE;
            for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
            {
                if ( pHelp->keyword[0] == '\0' )
                    continue;

                if ( is_name ( group_table[cmd].name, pHelp->keyword ) )
                {
                    found = TRUE;
                    break;
                }
            }
            if ( !found )
            {
                sprintf ( buf, "%-20s", group_table[cmd].name );
                stc ( buf, ch );
                if ( ++col % 4 == 0 )
                    stc ( "\n\r", ch );
            }
        }
        if ( ++col % 4 != 0 )
            stc( "\n\r", ch );
        stc( "\n\r", ch );

        return;
    }
    else
        do_helpcheck ( ch, "");
}
