/***************************************************************************
                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/
    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.
    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.
    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.
    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.
    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.
	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "db.h"
#include "olc.h"
#include "mysql5_interface.h"

/* This file contains new immortal commands */
void jail_char args( ( CHAR_DATA *ch ) );

/*
 * Local functions.
 */
FILE 		*	popen		args( ( const char *command, const char *type ) );
int 			pclose		args( ( FILE *stream ) );
char		*	fgetf		args( ( char *s, int n, register FILE *iop ) );

void do_shell( CHAR_DATA *ch, char *argument )
{
    char buf[5000];
    FILE *fp;

    fp = popen( argument, "r" );

    fgetf( buf, 5000, fp );

    page_to_char( buf, ch );

    pclose( fp );

    return;
}

char *fgetf( char *s, int n, register FILE *iop )
{
    register int c;
    register char *cs;

    c = '\0';
    cs = s;
    while( --n > 0 && (c = getc(iop)) != EOF)
	if ((*cs++ = c) == '\0')
	    break;
    *cs = '\0';
    return((c == EOF && cs == s) ? NULL : s);
}

void announce(CHAR_DATA *ch, char *message)
{
    DESCRIPTOR_DATA *d;
    CHAR_DATA *dch;
    char buf[MAX_INPUT_LENGTH];
    for(d = descriptor_list; d != NULL; d = d->next)
    {
        if(d->connected != CON_PLAYING)
            continue;
        if((dch = d->character) == NULL || dch == ch)
            continue;
        sprintf(buf, "INFO: %s", message);
        if(ch) /* don't use $N only $n in message */
            act(buf, ch, NULL, dch, TO_VICT);
        else
            send_to_char(buf, dch);
    }
}
bool isInElList( CHAR_DATA *ch )
{
		if ( is_exact_name( ch->name, "Rokann" ) 
		|| is_exact_name( ch->name, "Avryel" )
		|| is_exact_name( ch->name, "Dalsor" )
		|| is_exact_name( ch->name, "Faden" ) )
			return TRUE;

	return FALSE;
}
void do_openvlist( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    BUFFER *buffer;
    int loop = 1;
    long lowvnum=0, x;
    bool found = FALSE;
    bool havehighvnum = FALSE;
    AREA_DATA *pArea;
    buffer = new_buf();
    for (x = 1; x <= MAX_VNUM ; x++)
    {
        int count = 0;
        havehighvnum = FALSE;
        for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
        {
           if (x <= pArea->max_vnum && x >= pArea->min_vnum)
           {
               count++;                 /* count adds 1 if vnum in area    */
               havehighvnum = TRUE;     /* Might be the high vnum, trap it */
           }
        }
        if (x == MAX_VNUM && count == 0)  /* Lame hack to catch last vnum  */
            havehighvnum = TRUE;
        if (count == 0 && lowvnum == 0)   /* no areas found in and no low  */
            lowvnum = x;                  /* vnum yet, assign this one     */
        if (lowvnum > 0 && havehighvnum)  /* Have low and high vnum now    */
        {
            sprintf(buf, "[%d] Unassigned block of vnums from %ld to %ld\n\r"
                    ,loop, lowvnum, x - 1);
            add_buf(buffer,buf);
            lowvnum = 0;                  /* Reset low vnum                */
            loop++;
            found = TRUE;
        }
    }
    if (!found)
        send_to_char("No free range of vnums was found.\n\r",ch);
    else
        page_to_char(buf_string(buffer),ch);
    free_buf(buffer);
  return;
}

bool read_nstring( CHAR_DATA *ch )
{
	if ( IS_SET( ch->war_flags, WAR_IMMORTAL ) )
		return TRUE;
	if ( IS_SET( ch->war_flags, WAR_MODERATOR ) )
		return TRUE;

	return FALSE;
}

int CmdCount args( (const void *v1, const void *v2) );
void html_colourconv args (( char *buffer, const char *txt, CHAR_DATA *ch ));

void make_dismemberment( CHAR_DATA *ch, CHAR_DATA *victim, int vnum )
{
    char buf[MSL];
    char *name;
    OBJ_DATA *obj;
    name		= IS_NPC(victim) ? victim->short_descr : victim->name;
    obj		= create_object( get_obj_index( vnum ), 0 );
    obj->timer	= number_range( 7, 14 );
    sprintf( buf, obj->short_descr, name );
    free_string( obj->short_descr );
    obj->short_descr = str_dup( buf );
    sprintf( buf, obj->description, name );
    free_string( obj->description );
    obj->description = str_dup( buf );
    obj_to_room( obj, victim->in_room );
    return;
}

void draw_world_map args(( CHAR_DATA *ch ));
void create_map_exits args(( CHAR_DATA *ch ));

int sort_remort_race_by_alpha(const void *a, const void *b )
{
	return strcmp( (*(struct remort_race_sort_struct *)a).name , (*(struct remort_race_sort_struct *)b).name);
}
int sort_remort_race_by_remorts(const void *a, const void *b )
{
	return (*(struct remort_race_sort_struct *)a).remorts - (*(struct remort_race_sort_struct *)b).remorts;
}
void do_warn ( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	FILE *fp;
	char arg1[MIL];
    argument = one_argument(argument,arg1);
	if ( arg1[0] == '\0' )
	{
		stc("Who are you going to warn for what?\n\r",ch);
		stc("Syntax: warn <person name> <reason>\n\r",ch);
		return;
	}
	if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
	{
		stc("Could not find that person.\n\r",ch);
		return;
	}
	if ( IS_NPC( victim ) || IS_NPC( ch ) || !victim->desc || !victim->pcdata )
	{
		stc("Not on NPCs, linkdead folk, or those without pcdata.\n\r",ch);
		return;
	}
	if ( victim && victim->pcdata )
	{
		ptc( ch, "{WWarned for:{x\n\r{R%s.{x\n\r", argument );
		victim->pcdata->warning += 1;
		ptc( victim, "{W%s has issued you a warning for:{x\n\r{R%s{x\n\r", ch->name, argument );
		ptc( victim, "{WThis is your %d%s warning. After three, you will be jailed.\n\r",
			victim->pcdata->warning,
			victim->pcdata->warning == 1 ? "st" :
			victim->pcdata->warning == 2 ? "nd" :
			victim->pcdata->warning == 3 ? "rd" : "th" );
		if ( victim->pcdata->warning >= 3 )
		{
			ptc( ch, "%s has been jailed for their actions.\n\r", victim->name );
			stc( "You have been jailed.\n\rYou will be released after a time. Until then, think about what you have done.\n\r", victim );
			victim->pcdata->jail_timer = 60 * (victim->pcdata->warning-2);  /* 1 hour * warnings at 3(if a tick is a minute)*/
/*			victim->pcdata->warning = 0;   lets not reset these */
			jail_char( victim );
		}
		////////read_mysql_staff( ch );
		ch->pcdata->mod_warnings++; /* that would only increment it if they were jailed, not just warned */
		//update_staff( ch, "commands" );
		do_function( victim, &do_save, "" );

	}

    if ( ( fp = fopen( WARN_FILE, "a" ) ) == NULL )
    {
        perror( WARN_FILE );
	return;
    }

    fprintf( fp, "%s by %s (%d): %s\n", victim->name, ch->name, victim->pcdata->warning, argument );
    fclose(fp);

	tail_chain();
}

void do_verify( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char arg[MIL];
    one_argument(argument,arg);
	if ( ( victim = get_char_world( ch, arg ) ) == NULL )
	{
		stc("Could not find that person.\n\r",ch);
		return;
	}
	if ( IS_NPC( victim ) || IS_NPC( ch ) || !victim->desc || !victim->pcdata )
	{
		stc("Not on NPCs, linkdead folk, or those without pcdata.\n\r",ch);
		return;
	}
	if ( victim->pcdata && victim->pcdata->allowed == 1 )
	{
		stc("Already verified.\n\r", ch);
		return;
	}
	if ( victim->pcdata && victim->pcdata->allowed == 0 )
	{
		stc( "Verified.\n\r",ch);
		mentor_alert( "%s has been verified by %s.", victim->name, ch->name );
		victim->pcdata->allowed = 1;
		ptc( victim, "%s has verified you. You are now allowed to leave the school.\n\rBe sure to SAVE!\n\r", ch->name );
		//////read_mysql_staff( ch );
		ch->pcdata->verifications++;
		//update_staff( ch, "commands" );
		return;
	}
}
/* ADDED BY SOKOL */
void do_wrlist( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *room;
	ROOM_INDEX_DATA *in_room;
	MOB_INDEX_DATA *mob;
	OBJ_INDEX_DATA *obj;
	char arg[MAX_STRING_LENGTH];
	char arg1[MAX_STRING_LENGTH];
	char arg2[MAX_STRING_LENGTH];
	int uvnum;
	int lvnum;
	int MR = 60000;
	int type = -1;

	argument = one_argument( argument, arg );
	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	uvnum = ( is_number( arg2 ) ) ? atoi( arg2 ) : 1;
	lvnum = ( is_number( arg1 ) ) ? atoi( arg1 ) : 1;

	if ( !str_cmp( arg, "o" ) )
		type = 2;
	if ( !str_cmp( arg, "m" ) )
		type = 1;
	if ( !str_cmp( arg, "r" ) )
		type = 0;
	if ( !str_cmp( arg, "p" ) )
		type = 3;
	if ( !str_cmp( arg, "c" ) )
		type = 4;

	if ( ( uvnum - lvnum ) > 200 && type != 4 )
	{
		send_to_char( "{WThat range is too large.\n\r{x", ch );
		return;
	}

	if ( ( ( uvnum == 0 ) && ( lvnum == 0 ) ) || ( arg[0] == '\0' )
	|| ( type == -1 ) )
	{
		send_to_char( "{MSyntax: wrlist [type] [lvnum] [uvnum]\n\r{x", ch );
		return;
	}

	if ( uvnum > MR || uvnum < 1 || lvnum > MR || lvnum < 1 || lvnum > uvnum )
	{
		send_to_char( "{WInvalid level(s).\n\r{x", ch );
		return;
	}

	for ( MR = lvnum; MR <= uvnum; MR++ )
	{
		if ( type == 4 )
		{
			if ( ( room = get_room_index( MR ) ) )
			{
				if ( IS_SET( room->room_flags, ROOM_CLIMB ) )
					ptc( ch, "{R%-5ld  {w%d\n\r", room->vnum, room->grade );
			}
		}
		else if ( type == 0 )
		{
			if ( ( room = get_room_index( MR ) ) )
			{
				in_room = ch->in_room;
				char_from_room( ch );
				sprintf( log_buf, "{R%-5ld  {w%-20s\n\r", room->vnum, room->name );
				send_to_char( log_buf, ch );
				char_to_room( ch, room );
				do_resets( ch, "" );
				char_from_room( ch );
				char_to_room( ch, in_room );
			}
		}
		else if ( type == 2 )
		{
			if ( ( obj = get_obj_index( MR ) ) )
			{
				sprintf( log_buf, "{R%-5ld  {w%-20s\n\r",  obj->vnum, obj->name );
				send_to_char( log_buf, ch );
			}
		}
		else if ( type == 1 )
		{
			if ( ( mob = get_mob_index( MR ) ) )
			{
				sprintf( log_buf, "{R%-5ld  {w%-20s\n\r", mob->vnum, mob->player_name );
				send_to_char( log_buf, ch );
			}
		}
		else if ( type == 3 )
		{
			if ( ( mob = get_mob_index( MR ) ) )
			{
				;
			}
		}
	}
	return;
}

void do_gocial(CHAR_DATA *ch, char *argument)
{
    char command[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int cmd;
    bool found;
    char arg[MAX_INPUT_LENGTH];
    DESCRIPTOR_DATA *d;
    char buf[MAX_STRING_LENGTH];
    int counter;
    int count;
    char buf2[MAX_STRING_LENGTH];
    argument = one_argument(argument,command);
    if (command[0] == '\0')
    {
	send_to_char("What do you wish to gocial?\n\r",ch);
	return;
    }
    found = FALSE;
    for (cmd = 0; social_table[cmd].name[0] != '\0'; cmd++)
    {
	if (command[0] == social_table[cmd].name[0]
	&& !str_prefix( command,social_table[cmd].name ) )
	{
	    found = TRUE;
	    break;
	}
    }
    if (!found)
    {
	send_to_char("What kind of social is that?!?!\n\r",ch);
	return;
    }
    if (!IS_NPC(ch) && IS_SET(ch->comm,   COMM_QUIET))
    {
	send_to_char("You must turn off quiet mode first.\n\r",ch);
	return;
    }
    if ( !IS_NPC(ch) && IS_SET(ch->comm, COMM_NOOOC))
    {
	send_to_char("But you have the OOC channel turned off!\n\r",ch);
	return;
    }
    if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOCHANNELS))
    {
	send_to_char("The gods have revoked your channel priviliges.\n\r",ch);
	return;
    }
    if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE))
    { send_to_char("You are anti-social!\n\r",ch);
      return; }
    switch (ch->position)
    {
    case POS_DEAD:
	send_to_char("Lie still; you are DEAD!\n\r",ch);
	return;
    case POS_INCAP:
    case POS_MORTAL:
	send_to_char("You are hurt far too bad for that.\n\r",ch);
	return;
    case POS_STUNNED:
	send_to_char("You are too stunned for that.\n\r",ch);
	return;
    }
    one_argument(argument,arg);
    victim = NULL;
    if (arg[0] == '\0')
    {
	sprintf(buf, "{&[{e*SOC*{&]{B %s{c", social_table[cmd].char_no_arg );
	act_new(buf,ch,NULL,NULL,TO_CHAR,POS_DEAD);
	sprintf(buf, "{&[{e*SOC*{&]{B %s{c", social_table[cmd].others_no_arg );
	for (d = descriptor_list; d != NULL; d = d->next)
	{
	    CHAR_DATA *vch;
	    vch = d->original ? d->original : d->character;
	    if (d->connected == CON_PLAYING &&
		d->character != ch &&
		!IS_SET(vch->comm,COMM_NOOOC) &&
		!IS_SET(vch->comm,COMM_QUIET))
	    {
		act_new(buf,ch,NULL,vch,TO_VICT,POS_DEAD);
	    }
	}
    }
    else if ((victim = get_char_world(ch,arg)) == NULL)
    {
	send_to_char("They aren't here.\n\r",ch);
	return;
    }
    else if ((victim->invis_level || victim->incog_level) && (victim != ch))
    { send_to_char("They aren't here.\n\r",ch);
      return; }
    else if (victim == ch)
    {
	sprintf(buf,"{&[{e*SOC*{&]{B %s{c", social_table[cmd].char_auto);
	act_new(buf,ch,NULL,NULL,TO_CHAR,POS_DEAD);
	sprintf(buf,"{&[{e*SOC*{&]{B %s{c", social_table[cmd].others_auto);
	for (d = descriptor_list; d != NULL; d = d->next)
	{
	    CHAR_DATA *vch;
	    vch = d->original ? d->original : d->character;
	    if (d->connected == CON_PLAYING &&
		d->character != ch &&
		!IS_SET(vch->comm,COMM_NOOOC) &&
		!IS_SET(vch->comm,COMM_QUIET))
	    {
		act_new(buf,ch,NULL,vch,TO_VICT,POS_DEAD);
	    }
	}
    }
    else
    {
	sprintf(buf,"{&[{e*SOC*{&]{B %s{c", social_table[cmd].char_found);
	act_new(buf,ch,NULL,victim,TO_CHAR,POS_DEAD);
	sprintf(buf,"{&[{e*SOC*{&]{B %s{c", social_table[cmd].vict_found);
	act_new(buf,ch,NULL,victim,TO_VICT,POS_DEAD);
	sprintf(buf,"{&[*SOC*]{B %s{c", social_table[cmd].others_found);
	for (counter = 0; buf[counter+1] != '\0'; counter++)
	{
	    if (buf[counter] == '$' && buf[counter + 1] == 'N')
	    {
		strcpy(buf2,buf);
		buf2[counter] = '\0';
		strcat(buf2,victim->name);
		for (count = 0; buf[count] != '\0'; count++)
		{
		    buf[count] = buf[count+counter+2];
		}
		strcat(buf2,buf);
		strcpy(buf,buf2);
	    }
	    else if (buf[counter] == '$' && buf[counter + 1] == 'E')
	    {
		switch (victim->sex)
		{
		default:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"it");
		    for (count = 0; buf[count] != '\0'; count ++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		case 1:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"it");
		    for (count = 0; buf[count] != '\0'; count++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		case 2:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"it");
		    for (count = 0; buf[count] != '\0'; count++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		}
	    }
	    else if (buf[counter] == '$' && buf[counter + 1] == 'M')
	    {
		buf[counter] = '%';
		buf[counter + 1] = 's';
		switch (victim->sex)
		{
		default:
  		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"it");
		    for (count = 0; buf[count] != '\0'; count++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		case 1:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"him");
		    for (count = 0; buf[count] != '\0'; count++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		case 2:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"her");
		    for (count = 0; buf[count] != '\0'; count++);
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		}
	    }
	    else if (buf[counter] == '$' && buf[counter + 1] == 'S')
	    {
		switch (victim->sex)
		{
		default:
		strcpy(buf2,buf);
		buf2[counter] = '\0';
		strcat(buf2,"its");
		for (count = 0;buf[count] != '\0'; count++)
		{
		    buf[count] = buf[count+counter+2];
		}
		strcat(buf2,buf);
		strcpy(buf,buf2);
		break;
		case 1:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"his");
		    for (count = 0; buf[count] != '\0'; count++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		case 2:
		    strcpy(buf2,buf);
		    buf2[counter] = '\0';
		    strcat(buf2,"hers");
		    for (count = 0; buf[count] != '\0'; count++)
		    {
			buf[count] = buf[count+counter+2];
		    }
		    strcat(buf2,buf);
		    strcpy(buf,buf2);
		    break;
		}
	    }
	}
	for (d=descriptor_list; d != NULL; d = d->next)
	{
	    CHAR_DATA *vch;
	    vch = d->original ? d->original : d->character;
	    if (d->connected == CON_PLAYING &&
		d->character != ch &&
		d->character != victim &&
		!IS_SET(vch->comm, COMM_NOOOC) &&
		!IS_SET(vch->comm,COMM_QUIET))
	    {
		act_new(buf,ch,NULL,vch,TO_VICT,POS_DEAD);
	    }
	}
    }
    return;
}
void do_gmote( CHAR_DATA *ch, char *argument )
{
    DESCRIPTOR_DATA *d;
    if ( argument[0] == '\0' )
    {
         send_to_char("Gmote what?\n\r", ch );
         return;
    }
    act_new( "{W[*SOC*] {B$n $t{c\n\r",
		   ch,argument, NULL, TO_CHAR,POS_DEAD );
    for ( d = descriptor_list; d != NULL; d = d->next )
    {
      if ( d->connected == CON_PLAYING )
      {
        act_new( "{W[*SOC*] {B$n $t{c",
		   ch,argument, d->character, TO_VICT,POS_SLEEPING );
      }
    }
}
void do_grab (CHAR_DATA *ch, char *argument)
{
    CHAR_DATA  *victim;
    OBJ_DATA   *obj;
    char 	arg1 [ MAX_INPUT_LENGTH ];
    char 	arg2 [ MAX_INPUT_LENGTH ];
    char 	arg3 [ MAX_INPUT_LENGTH ];
    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    argument = one_argument( argument, arg3 );
    if ( ( arg1[0] == '\0' ) || ( arg2[0] == '\0' ) )
    {
        stc( "Syntax : grab <object> <player>\n\r", ch );
        return;
    }
    if ( !( victim = get_char_world( ch, arg2 ) ) )
    {
        stc( "They are not here!\n\r", ch );
        return;
    }
    if ( !( obj = get_obj_list( ch, arg1, victim->carrying ) ) )
    {
        stc( "They do not have that item.\n\r", ch );
        return;
    }
    if ( victim->level >= ch->level )
    {
        stc( "You Failed.\r\n", ch );
        return;
    }
    if ( obj->wear_loc != WEAR_NONE )
        unequip_char( victim, obj );
    obj_from_char( obj );
    obj_to_char( obj, ch );
    act( "You grab $p from $N.", ch, obj, victim, TO_CHAR );
    if ( arg3[0] == '\0'
    	|| !str_cmp( arg3, "yes" ) || !str_cmp( arg3, "true" ) )
           act( "You no longer own $p.", ch, obj, victim, TO_VICT );
    return;
}
void do_owhere(CHAR_DATA *ch, char *argument )
{
    char buf[MAX_INPUT_LENGTH];
    BUFFER *buffer;
    OBJ_DATA *obj;
    OBJ_DATA *in_obj;
    bool found;
    int number = 0, max_found;
    found = FALSE;
    number = 0;
    max_found = 200;
    buffer = new_buf();

    if (argument[0] == '\0')
    {
	stc("{cFind what?\n\r",ch);
	return;
    }
    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
        if ( !can_see_obj( ch, obj ) || !is_name( argument, obj->name )
        ||   ch->level < obj->level)
            continue;
        found = TRUE;
        number++;
        for ( in_obj = obj; in_obj->in_obj != NULL; in_obj = in_obj->in_obj )
            ;
        if ( in_obj->carried_by != NULL && can_see(ch,in_obj->carried_by)
	&&   in_obj->carried_by->in_room != NULL)
            sprintf( buf, "{c%3d) %s is carried by %s {C[{cRoom %ld{C]{c\n\r",
                number, obj->short_descr,PERS(in_obj->carried_by, ch),
		in_obj->carried_by->in_room->vnum );
        else if (in_obj->in_room != NULL && can_see_room(ch,in_obj->in_room))
            sprintf( buf, "{c%3d) %s is in %s {C[{cRoom %ld{C]{c\n\r",
                number, obj->short_descr,in_obj->in_room->name,
	   	in_obj->in_room->vnum);
	else
            sprintf( buf, "{c%3d) %s is somewhere\n\r",number, obj->short_descr);
        buf[0] = UPPER(buf[0]);
        add_buf(buffer,buf);
        if (number >= max_found)
            break;
    }
    if ( !found )
        stc("{cNothing like that in heaven or earth.\n\r", ch );
    else
        page_to_char(buf_string(buffer),ch);
    free_buf(buffer);
}
void do_mwhere( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    BUFFER *buffer;
    CHAR_DATA *victim;
    bool found;
    int count = 0;
    if ( argument[0] == '\0' )
    {
	DESCRIPTOR_DATA *d;
	/* show characters logged */
	buffer = new_buf();
	for (d = descriptor_list; d != NULL; d = d->next)
	{
	    if (d->character != NULL && d->connected == CON_PLAYING
	    &&  d->character->in_room != NULL && can_see(ch,d->character)
	    &&  can_see_room(ch,d->character->in_room))
	    {
		victim = d->character;
		count++;
		if (d->original != NULL)
		    sprintf(buf,"{c%3d) %s {C({cin the body of %s{C){c is in %s {C[{c%ld{C]{c\n\r",
			count, d->original->name,victim->short_descr,
			victim->in_room->name,victim->in_room->vnum);
		else
		    sprintf(buf,"{c%3d) %s is in %s {C[{c%ld{C]{c\n\r",
			count, victim->name,victim->in_room->name,
			victim->in_room->vnum);
		add_buf(buffer,buf);
	    }
	}
        page_to_char(buf_string(buffer),ch);
	free_buf(buffer);
	return;
    }
    found = FALSE;
    buffer = new_buf();
    for ( victim = char_list; victim != NULL; victim = victim->next )
    {
	if ( victim->in_room != NULL
	&&   is_name( argument, victim->name )
	&& victim->level <= ch->level )
	{
	    found = TRUE;
	    count++;
	    sprintf( buf, "{c%3d) {C[{c%5ld{C]{c %-28s {C[{c%5ld{C]{c %s\n\r", count,
		IS_NPC(victim) ? victim->pIndexData->vnum : 0,
		IS_NPC(victim) ? victim->short_descr : victim->name,
		victim->in_room->vnum,
		victim->in_room->name );
	    add_buf(buffer,buf);
	}
    }
    if ( !found )
	act( "{cYou didn't find any $T.", ch, NULL, argument, TO_CHAR );
    else
    	page_to_char(buf_string(buffer),ch);
    free_buf(buffer);
    return;
}
void do_rollop ( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char arg[MIL];
	char buf[MIL];

	argument = one_argument(argument, arg);

	if ( ( victim = get_char_world(ch,arg) ) == NULL)
	{
		stc("Couldn't find that person anywhere.\n\r",ch);
		return;
	}

	if ( !IS_SET ( victim->op_flags,OP_CHANNELER ) )
	{
		stc("That person isn't a channeler.\n\r",ch);
		return;
	}

	if ( victim->pcdata->op_air    > 0
	||   victim->pcdata->op_fire   > 0
	||   victim->pcdata->op_spirit > 0
	||   victim->pcdata->op_earth  > 0
	||   victim->pcdata->op_water  > 0 )
	{
		stc("This character already has One Power stats.\n\r",ch);
		return;
	}

	victim->pcdata->op_air    = number_range(2,12);
	victim->pcdata->op_fire   = number_range(2,12);
	victim->pcdata->op_spirit = number_range(2,12);
	victim->pcdata->op_earth  = number_range(2,12);
	victim->pcdata->op_water  = number_range(2,12);

	if (victim->sex == SEX_FEMALE && victim->pcdata->op_water < 12)
		victim->pcdata->op_water += 1;
	if (victim->sex == SEX_MALE && victim->pcdata->op_fire < 12)
		victim->pcdata->op_fire += 1;

	stc("Strength in the One Power is as follows:\n\r",ch);
	sprintf(buf, "Air: %d  Fire: %d  Spirit: %d  Earth: %d  Water: %d\n\r",
		victim->pcdata->op_air,
		victim->pcdata->op_fire,
		victim->pcdata->op_spirit,
		victim->pcdata->op_earth,
		victim->pcdata->op_water);
	stc(buf,ch);
	stc(buf,victim);
	return;
}
void do_qpgrant ( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char arg[MIL];
	char arg2[MIL];
	int num = 0;
	argument = one_argument(argument, arg);
	argument = one_argument( argument, arg2 );
	/* Teml test of allowing qpgrant to QC quests */
	/*if ( !is_exact_name(ch->name, MUD_OWNER)
	&& ch->level < MAX_LEVEL )
	{
		stc("You are not authorized to perform this function.\n\r",ch);
		return;
	}*/
	if (!str_cmp(arg,"all"))
	{
		for ( victim = char_list; victim != NULL; victim = victim->next )
		{
			if (IS_NPC(ch))
			continue;
			victim->questpoints += 25;
			ptc(victim,"\n\r{RThe %s{R has granted you {W25{R Quest Points!{x\n\r", is_shadow(victim) ? "{R{rGreat {RL{rord" : "{wCreator");
		}
		return;
	}
	if ( ( victim = get_char_world(ch,arg) ) == NULL)
	{
		stc("Couldn't find that person anywhere.\n\r",ch);
		return;
	}
	num = atoi(arg2);
	if ( arg2[0] != '\0' )
	{
		victim->questpoints += num;
	}
	else
	{
		num = 25;
		victim->questpoints += num;
	}
	ptc( ch, "%s granted %d Quest Points!{x\n\r", victim->name, num );
	ptc(victim,"\n\r{RThe %s{R has granted you %d Quest Points!{x\n\r", is_shadow(victim) ? "{R{rGreat {RL{rord" : "{wCreator", num);

	return;
}
void do_xpgrant ( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	OBJ_DATA *qp_obj, *qp_obj_next;
	char arg[MIL];
	bool found;
	argument = one_argument(argument, arg);
	/* Experiment to grant quest council xpgrant */
	/*if ( !is_exact_name(ch->name, MUD_OWNER)
	&& ch->level < MAX_LEVEL )
	{
		stc("You are not authorized to perform this function.\n\r",ch);
		return;
	}*/
	if ( ( victim = get_char_world(ch,arg) ) == NULL)
	{
		stc("Couldn't find that person anywhere.\n\r",ch);
		return;
	}
	if ( !is_number( argument ) )
	{
		stc("Argument must be numerical.\n\r",ch);
		return;
	}
	if ( atoi( argument ) < 1 || atoi( argument ) > 1000 )
	{
		stc("Argument must be between 1 and 1000.\n\r",ch);
		return;
	}
	else
	{
		gain_exp( victim, atoi( argument ) );
		return;
	}
	found = FALSE;
	for ( qp_obj = victim->carrying; qp_obj != NULL; qp_obj = qp_obj_next )
	{
		qp_obj_next = qp_obj->next_content;
		if ( qp_obj->pIndexData->vnum == 1001 )
		{
			stc("Converting your quest token to experience points.\n\r",victim);
			extract_obj(qp_obj);
			gain_exp( victim, 100 );
			found = TRUE;
		}
		else
			continue;
	}
	if ( !found )
	{
		stc("You do not have any quest tokens to convert to experience.\n\r",victim);
		stc("They dont have any quest tokens to convert to experience.\n\r",ch);
		return;
	}
	return;
}
void do_penalize (CHAR_DATA *ch, char *argument)
{
	char arg1[MIL];
	char arg2[MIL];
	CHAR_DATA *victim;

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);

	if (IS_NPC(ch))
		return;
	if (!IS_SET(ch->war_flags,WAR_MODERATOR))
		return;

	if ( arg1[0] == '\0' || arg2[0] == '\0' )
	{
		stc("Penalize syntax:\n\r",ch);
		stc("{WPenalize person nowalk\n\r",ch);
		stc("{w  This removes the ability to do set Walkin Walkout, RDesc messages.\n\r",ch);
		stc("{w  It will also set the current Walkin, Walkout, RDesc messages to NULL.\n\r",ch);
		stc("{WPenalize person nomove\n\r",ch);
		stc("{w  This option is not active yet.\n\r",ch);
		stc("{WPenalize person nochannel\n\r",ch);
		stc("{w  This powerful setting restricts access to all public channels,\n\r",ch);
		stc("{w  as well as notes.\n\r",ch);
		stc("{WPenalize person noemote\n\r",ch);
		stc("{w  Removes the character's ability to perform emotes.\n\r",ch);
		stc("{WPenalize person noshout\n\r",ch);
		stc("{w  This setting removes the character's ability to send shouts.\n\r",ch);
		stc("{WPenalize person notell\n\r",ch);
		stc("{w  Removes the character's ability to send tells.\n\r",ch);
		stc("{wPenalize OOC_OFF\n\r",ch);
		stc("{w  Turns the global OOC channel off. No one can use OOC.\n\r",ch);
		stc("{wPenalize person unverify\n\r",ch);
		stc("{w  Removes verification from person.\n\r",ch);
		stc("{wPenalize person nochats\n\r",ch);
		stc("{w  Removes ability to use most channels.\n\r",ch);
		return;
	}
	if ( !str_cmp(arg1, "OOC_OFF") && OOC_OFF == 0 )
	{
		stc("Global OOC channel turned OFF.\n\r",ch);
		OOC_OFF = 1;
		return;
	}
	if ( !str_cmp(arg1, "OOC_OFF") && OOC_OFF == 1 )
	{
		stc("Global OOC channel turned ON.\n\r",ch);
		OOC_OFF = 0;
		return;
	}
	if ( ( victim = get_pc_world( ch, arg1 ) ) == NULL )
	{
		stc("Your victim could not be found.\n\r",ch);
		return;
	}
	if (IS_IMMORTAL(victim))
	{
		if ( IS_IMMORTAL(victim) && IS_IMMORTAL(ch) )
		{
			stc("{wYou failed.\n\r", ch );
        		act("{w$n just tried to penalize you!{c",ch,NULL,victim,TO_VICT);
		}
       		return;
	}
    if ( !str_cmp( arg2, "unverify" ) )
    {
		stc("As you wish. Removing verification...\n\r",ch);
		victim->pcdata->allowed = 0;
		mentor_alert( "%s has removed the verification from %s!", ch->name, victim->name );
		return;
	}
	if (!str_cmp(arg2, "nowalk"))
	{
		stc("As you wish. No Walk toggled.\n\r",ch);
		if (!IS_SET(victim->penalty_flags,PENAL_NOWALK))
		{
			stc("You have lost the ability to create Walkin, Walkout, and RoomDesc messages.\n\r",victim);
			SET_BIT(victim->penalty_flags,PENAL_NOWALK);

			free_string( victim->pcdata->wkin );
			victim->pcdata->wkin = str_dup("");

			free_string( victim->pcdata->wout );
			victim->pcdata->wout = str_dup("");

			free_string( victim->pcdata->rdesc );
			victim->pcdata->rdesc = str_dup("");

			stc( "{cWalkIn, WalkOut, and Rdesc have been cleared.\n\r{c", ch);
		}
		else
		{
			stc("You have regained the ability to create Walkin, Walkout, and RoomDesc messages.\n\r",victim);
			REMOVE_BIT(victim->penalty_flags,PENAL_NOWALK);
		}
		return;
	}
	if (!str_cmp(arg2, "nomove"))
	{
		stc("As you wish.\n\r",ch);
		if (!IS_SET(victim->penalty_flags,PENAL_NOMOVE))
		{
		stc("Your feet become rooted to the spot. You can't move!\n\r",victim);
		SET_BIT(victim->penalty_flags,PENAL_NOMOVE);
		}
		else
		{
		stc("Your feet are no longer rooted to the spot. You can move!\n\r",victim);
		REMOVE_BIT(victim->penalty_flags,PENAL_NOMOVE);
		}
		return;
	}
	if (!str_cmp(arg2, "nochannel"))
	{
		stc("As you wish.\n\r",ch);
		if (!IS_SET(victim->penalty_flags,PENAL_NOCHANNELS))
		{
		stc("A big fat zipper closes over your mouth!\n\r",victim);
		SET_BIT(victim->penalty_flags,PENAL_NOCHANNELS);
		}
		else
		{
		stc("The big fat zipper over your mouth opens itself!\n\r",victim);
		REMOVE_BIT(victim->penalty_flags,PENAL_NOCHANNELS);
		}
		return;
	}
	if (!str_cmp(arg2, "noemote"))
	{
		stc("As you wish.\n\r",ch);
		if (!IS_SET(victim->penalty_flags,PENAL_NOEMOTE))
		{
		stc("You feel your emotions slipping away!\n\r",victim);
		SET_BIT(victim->penalty_flags,PENAL_NOEMOTE);
		}
		else
		{
		stc("Your emotions have returned!\n\r",victim);
		REMOVE_BIT(victim->penalty_flags,PENAL_NOEMOTE);
		}
		return;
	}
	if (!str_cmp(arg2, "noshout"))
	{
		stc("As you wish.\n\r",ch);
		if (!IS_SET(victim->penalty_flags,PENAL_NOSHOUT))
		{
		stc("You can no longer shout!\n\r",victim);
		SET_BIT(victim->penalty_flags,PENAL_NOSHOUT);
		}
		else
		{
		stc("You can shout again!\n\r",victim);
		REMOVE_BIT(victim->penalty_flags,PENAL_NOSHOUT);
		}
		return;
	}
	if (!str_cmp(arg2, "notell"))
	{
		stc("As you wish.\n\r",ch);
		if (!IS_SET(victim->penalty_flags,PENAL_NOTELL))
		{
		stc("You can no longer send tells!\n\r",victim);
		SET_BIT(victim->penalty_flags,PENAL_NOTELL);
		}
		else
		{
		stc("You can send tells again!\n\r",victim);
		REMOVE_BIT(victim->penalty_flags,PENAL_NOTELL);
		}
		return;
	}
        if (!str_cmp(arg2, "nochats"))
        {
                stc("As you wish.\n\r",ch);
                if (!IS_SET(victim->penalty_flags,PENAL_NOCHATS))
                {
                stc("You can no longer use chat channels!\n\r",victim);
                SET_BIT(victim->penalty_flags,PENAL_NOCHATS);
                }
                else
                {
                stc("You can use chat channels again!\n\r",victim);
                REMOVE_BIT(victim->penalty_flags,PENAL_NOCHATS);
                }
                return;
        }
	stc("What in particular to do?\n\r",ch);
}
void do_judgement ( CHAR_DATA *ch, char *argument )
{
	char arg1[MSL];
	argument = one_argument( argument, arg1 );
	if ( arg1[0] == '\0' )
	{
		stc("Argument should be greater than three characters.\n\r",ch);
	}
}
bool check_obj args( ( CHAR_DATA *ch, OBJ_INDEX_DATA *pObj ) );

/*
Object checking function, by Kyle Boyd.
This cycles through a given set of vnums and identifies problems
in the objects.  Useful for quickly debugging objects in new areas.
*/

void do_objcheck( CHAR_DATA *ch, char *argument)
{
	OBJ_INDEX_DATA *pObj;
	int tmp1, tmp2, tmp3, tmp4, tmp5, max = 75, mat;
	bool found = FALSE;
	MOB_INDEX_DATA *mob;
	AFFECT_DATA *af;
	char arg[MIL];
	char arg1[MIL];
	char arg2[MIL];
	char arg3[MIL];
	int lvnum, hvnum, i=0, x=0;

	argument = one_argument( argument, arg );
	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( arg1[0] == '\0' )
	{
		stc("Syntax: objcheck M/O <low vnum> <high vnum>\n\r", ch);
		stc("M checks for mobiles, O checks for objects.\n\r",ch);
		stc("Arg must not be null.\n\r",ch);
		return;
	}
	if ( arg2[0] == '\0' )
	{
		stc("Syntax: objcheck M/O <low vnum> <high vnum>\n\r", ch);
		stc("M checks for mobiles, O checks for objects.\n\r",ch);
		stc("Arg2 must not be null.\n\r",ch);
		return;
	}
	if ( arg3[0] == '\0' )
	{
		stc("Syntax: objcheck M/O <low vnum> <high vnum>\n\r", ch);
		stc("M checks for mobiles, O checks for objects.\n\r",ch);
		stc("Arg3 must not be null.\n\r",ch);
		return;
	}
	if ( !is_number( arg2 ) )
	{
		stc("Syntax: objcheck M/O <low vnum> <high vnum>\n\r", ch);
		stc("M checks for mobiles, O checks for objects.\n\r",ch);
		stc("Arg must be a valid numeric.\n\r",ch);
		return;
	}
	if ( !is_number( arg3 ) )
	{
		stc("Syntax: objcheck M/O <low vnum> <high vnum>\n\r", ch);
		stc("M checks for mobiles, O checks for objects.\n\r",ch);
		stc("Arg2 must be a valid numeric.\n\r",ch);
		return;
	}
	lvnum = atoi( arg2 );
	hvnum = atoi( arg3 );
	if( lvnum > hvnum )
	{
		x = lvnum;
		lvnum = hvnum;
		hvnum = x;
		x = 0;
	}
	ptc( ch, "Starting at vnum [%5d].\n\r", i);
	if ( !str_cmp( arg, "O" ) )
	{
		for( x=lvnum; x < hvnum; x++ )
		{
			pObj = get_obj_index( x );
			if( pObj == NULL )
				continue;

			/* big ass mother fucker */
			pObj->level = 1;
			tmp1 = 2;
			tmp2 = 0;
			tmp3 = 0;
			tmp4 = 0;
			tmp5 = 0;
			if (!strcmp(pObj->material,"unknown"))
			{
				pObj->material = str_dup("cloth");
			}
			for ( mat = 0; mat < max-1; mat++ )
			{
				if ( !str_cmp( pObj->material, material_table[mat].name ) )
				{
					found = TRUE;
					break;
				}
			}
			if ( !found )
			{
				stc("ERROR! Unknown material type. Please check the material.\n\r",ch);
					pObj->material = str_dup("cloth");
			}
			for ( af = pObj->affected; af != NULL; af = af->next )
			{
				if ( af->modifier == 0 )
				{
					  af->modifier = material_table[mat].maxAff -1;
					  if ( af->modifier == 0 )
						  af->modifier = 1;
				}
			}
			if( pObj->weight <= 0 )
			{
				i++;
				printf_to_char(ch, "[%5d] Weight = %d\n\r", pObj->vnum, pObj->weight);
				pObj->weight = (pObj->level / 20) + 1;
				ptc( ch, "-Weight set to %d.\n\r", pObj->weight );
			}
			if (!strcmp(pObj->material,"power wrought"))
			{
				ptc(ch, "[%5d] Warning! This item is power wrought!\n\r", pObj->vnum);
			}
			if (!strcmp(pObj->material,"cuendillar"))
			{
				ptc(ch, "[%5d] Warning! This item is cuendillar!\n\r", pObj->vnum);
			}
			if( !str_cmp( pObj->name, "no name" ) || pObj->name[0]=='\0' )
			{
				i++;
			}
			if( !str_cmp( pObj->short_descr, "(no short description)" ) || pObj->short_descr[0]=='\0' )
			{
				i++;
			}
			if( !str_cmp( pObj->description, "(no long description)" ) || pObj->description[0]=='\0' )
			{
				i++;
			}
			if( !(pObj->wear_flags & ITEM_TAKE)
			&& pObj->item_type != ITEM_FURNITURE
			&& pObj->item_type != ITEM_FOUNTAIN
			&& pObj->item_type != ITEM_CONTAINER)
			{
				i++;
			}
			if( pObj->item_type == ITEM_ARMOR && (pObj->wear_flags == 0 || pObj->wear_flags == ITEM_TAKE) )
			{
				i++;
			}
			if( pObj->item_type == ITEM_WEAPON && (pObj->wear_flags == 0 || pObj->wear_flags == ITEM_TAKE) )
			{
				i++;
			}
			if( pObj->item_type == ITEM_ANGREAL_MALE && (pObj->wear_flags != ITEM_WEAR_ANGREAL) )
			{
				i++;
				ptc(ch, "[%5d] {RMale ANGREAL!!{x \n\r",pObj->vnum );
			}
			if( pObj->item_type == ITEM_ANGREAL_FEMALE && (pObj->wear_flags != ITEM_WEAR_ANGREAL) )
			{
				i++;
				ptc(ch, "[%5d] {RFemale ANGREAL!!{x \n\r",pObj->vnum );
			}
			if (pObj->item_type == ITEM_COMM_DEV)
			{
				ptc(ch, "[%5d] is a COMMDEV, 3716 is the only allowed COMMDEV.\n\r", pObj->vnum);
			}
		}

		ptc( ch, "Ending at vnum [%5d].\n\r", x );
		return;

	}
	else if ( !str_cmp( arg, "M" ) )
	{
		for( x=lvnum; x < hvnum; x++ )
		{
			mob = get_mob_index( x );
			if( mob == NULL )
				continue;
			if ( IS_SET( mob-> act,ACT_UNDEAD ) )
			{
				ptc(ch,"Mob [%d] set to Undead -quest mob-.\n\r",mob->vnum);
			}
		}
		return;
	}
}

void do_reward( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	OBJ_DATA *item;

	char arg1[MIL],	arg2[MIL], arg3[MIL], arg4[MIL], arg5[MIL];

        argument = one_argument( argument, arg1 );
        argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	argument = one_argument( argument, arg4 );
	argument = one_argument( argument, arg5 );

	if ( IS_NPC( ch ) )
	{
		stc( "Nope.\n\r", ch );
		return;
	}
	if ( arg2[0] == '\0' )
	{
		stc( "You're missing something.\n\r", ch );
		stc( "Syntax:\n\r", ch );
		stc( "reward <amnt> <gold, xp, qp, item> <character>\n\r", ch );
		return;
	}
	if ( (atoi(arg2)) <= 0)
	{
		stc( "What are you TRYING to crash the game?\n\r", ch );
		stc( "reward <amnt> <gold, xp, qp, item> <character>\n\r", ch );
		return;
	}
	if ( (victim = get_char_world( ch, arg4 ) ) == NULL )
	{
		stc("They are not online.\n\r", ch);
		return;
	}
	if ( !str_cmp( arg2, "gold" ) )
	{
		//////read_mysql_staff( victim );
		victim->money[3][0] += atoi( arg2 );
		victim->pcdata->rewards_gold++;
		ptc( ch, "Rewards_gold: %d\n\r", victim->pcdata->rewards_gold );
		victim->pcdata->total_gold += atoi( arg2 );
		ptc( ch, "Total_gold: %d\n\r", victim->pcdata->total_gold );
		victim->pcdata->last_gold = current_time;
		ptc( victim, "You have been awarded %d gold marks.\n\r", atoi( arg2 ) );
		ptc( ch, "Player received %d gold marks.\n\r", atoi( arg2 ) );
		//update_staff( victim, "rewards" );
		return;
	}
	if ( !str_cmp( arg2, "xp" ) )
	{
		//////read_mysql_staff( victim );
		gain_exp(victim, atoi( arg2 ));
		victim->pcdata->rewards_xp++;
		victim->pcdata->total_xp += atoi( arg2 );
		victim->pcdata->last_xp = current_time;
		ptc( victim, "You have been awarded %d Experience Points.\n\r", atoi( arg2 ) );
		ptc( ch, "Player received %d Experience Points.\n\r", atoi( arg2 ) );
		//update_staff( victim, "rewards" );
		return;
	}
	if ( !str_cmp( arg3, "qp" ) )
	{
		//////read_mysql_staff( victim );
		victim->questpoints += atoi( arg2 );
		victim->pcdata->rewards_qp++;
		victim->pcdata->total_qp += atoi( arg2 );
		victim->pcdata->last_qp = current_time;
		ptc( victim, "You have been awarded %d Quest Points.\n\r", atoi( arg2 ) );
		ptc( ch, "Player received %d Quest Points.\n\r", atoi( arg2 ) );
		//update_staff( victim, "rewards" );
		return;
	}
	if ( !str_cmp( arg3, "item" ) )
	{
		long vnum;
		vnum = atol( arg5 );
		if ( vnum <= 0 )
		{
			stc("Bugger that!\n\r", ch);
			return;
		}
		if ( !get_obj_index( vnum ) )
		{
			stc("Stop! That hurts.\n\r", ch);
			return;
		}
		if ( ( item = create_object( get_obj_index( vnum ), 1 ) ) == NULL )
		{
			stc( "That is not a real vnum.\n\r", ch );
			return;
		}
		//////read_mysql_staff( victim );
		obj_to_char( item, victim );
		victim->pcdata->rewards_items++;
		victim->pcdata->total_items += atoi( arg2 );
		victim->pcdata->last_items = current_time;
		ptc( ch, "last_items: %ld\n\r", victim->pcdata->last_items );
		stc( "You have been awarded an item.\n\r", victim );
		stc( "Player received an item.\n\r", ch );
		//update_staff( victim, "rewards" );
		return;
	}
	if ( !str_cmp( arg3, "points" ) )
	{
		//////read_mysql_staff( victim );
		victim->pcdata->staff_points += atoi( arg2 );
		ptc( victim, "You have been awarded %d points.\n\r", atoi( arg2 ) );
		stc( "Points awarded.\n\r", ch );
		//update_staff( victim, "rewards" );
		return;
	}
	return;
}

void imm_cmd_addict( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( !str_prefix( arg2, "all" ) )
	{
		DESCRIPTOR_DATA *d;
		for ( d = descriptor_list; d != NULL; d = d->next )
		{
			if ( d->connected == CON_PLAYING && d->character && d->character->pcdata )
			{
				d->character->pcdata->addict = TRUE;
				d->character->pcdata->addict_hours = 240;
				stc( "{YYou are now an {yaddict{Y!{x\n\r", d->character );
			}
		}
		return;
	}

	if ( ( victim = get_char_world( ch, arg2 ) ) == NULL )
	{
		stc( "Give who some addiction?\n\r", ch );
		return;
	}
	if ( IS_NPC( victim ) )
	{
		stc( "Not on NPCs.\n\r", ch );
		return;
	}
	if ( !victim->pcdata )
	{
		stc( "They don't appear to have any pcdata right now.\n\r", ch );
		return;
	}
	victim->pcdata->addict = TRUE;
	victim->pcdata->addict_hours = 240;
	stc( "{YYou are now an {yaddict{Y!{x\n\r", victim );
}
void imm_cmd_dismember( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char buf[MSL], arg1[MIL], arg2[MIL];
	int vnum;
	vnum = 0;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( ( victim = get_char_world( ch, arg2 ) ) == NULL )
	{
		stc( "Dismember who?\n\r", ch );
		return;
	}
	if ( IS_NPC( victim ) )
	{
		stc( "Not on NPCs.\n\r", ch );
		return;
	}

	sprintf( buf, "{R%s pulls %s's guts out, then rips off %s head and drops it on the ground," \
		" {Rthen rips %s heart out, then pulls %s arms off, then pulls %s legs off, and finally," \
		" {Rgives the head a kick, crushing it and spilling %s's brains on the floor. %s finally finishes" \
		" {Rthe torture, allowing %s the privilege of death.{x",
		ch->name, victim->name,
		victim->sex == SEX_FEMALE ? "her" : "his",
		victim->sex == SEX_FEMALE ? "her" : "his",
		victim->sex == SEX_FEMALE ? "her" : "his",
		victim->sex == SEX_FEMALE ? "her" : "his",
		victim->name, ch->name, victim->name );

	vnum = OBJ_VNUM_GUTS;
	make_dismemberment( ch, victim, vnum );

	vnum = OBJ_VNUM_SEVERED_HEAD;
	make_dismemberment( ch, victim, vnum );

	vnum = OBJ_VNUM_TORN_HEART;
	make_dismemberment( ch, victim, vnum );

	vnum = OBJ_VNUM_SLICED_ARM;
	make_dismemberment( ch, victim, vnum );

	vnum = OBJ_VNUM_SLICED_LEG;
	make_dismemberment( ch, victim, vnum );

	vnum = OBJ_VNUM_BRAINS;
	make_dismemberment( ch, victim, vnum );

	do_echo( ch, buf );

	raw_kill( ch, victim );
}
void imm_cmd_fightdump( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	FIGHTING_DATA *fights;

	stc("Do not use this command. It is here for historical purposes only.\n\r", ch);
	return;

	if ( ( victim = get_char_world( ch, argument ) ) == NULL)
	{
		stc( "Fight dump who?\n\r",ch);
		return;
	}

	ptc(ch,"CHAR_DATA %x\n\r",victim);

	if(victim->hate)
		ptc(ch,"Hates: %s\n\r",victim->hate->name);

	ptc(ch,"%d fights.\n\r",victim->number_of_fights);

	for(fights = victim->fight_data;fights;fights=fights->next)
	{
		ptc(ch,"fighting %s \n\r", fights->fighter->name);
	}

	if(!victim->advancing_on)
		return;

	ptc(ch,"advancing %d %s \n\r", victim->advancing, victim->advancing_on->name);
}
void imm_cmd_subguild( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char arg1[MIL], arg2[MIL], arg3[MIL], buf[MSL];
	int guild;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
	{
		stc( "Syntax: subguild <victim> <guild name>\n\r", ch );
		return;
	}

	if( !str_cmp("self", arg2))
		victim = ch;
	else
	{
		if ( ( victim = get_pc_world( ch, arg2 ) ) == NULL )
		{
			stc( "Subguild who?\n\r", ch );
			return;
		}
		if ( IS_NPC( victim ) )
		{
			stc( "Not on NPCs.\n\r", ch );
			return;
		}
		if ( ( guild = clan_lookup( arg3 ) ) == 0)
		{
			stc("No such guild exists.\n\r", ch);
			return;
		}
		ptc( ch, "They are now sub-guilded %s.\n\r", clan_table[guild].name);
		ptc( victim, "You are now sub-guilded %s.\n\r", clan_table[guild].name);
		victim->sub_clan = guild;
		victim->sub_rank = 0;

		sprintf(buf, "%s has been sub-guilded %s by %s.", victim->name, clan_table[guild].who_name, ch->name);
		wiznet(buf, ch, NULL, WIZ_FLAGS, 0, 0);
	}
}
void imm_cmd_slotset( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	char arg1[MIL], arg2[MIL], arg3[MIL];
	int slot = 0, vnum = 3700;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
	{
		stc( "Syntax: slotset <victim> <slot>\n\r", ch );
		return;
	}
	if ( ( victim = get_char_world( ch, arg2 ) ) == NULL )
	{
		stc( "Slotset who?\n\r", ch );
		return;
	}
	if ( IS_NPC( victim ) )
	{
		stc( "Not on NPCs.\n\r", ch );
		return;
	}
	if ( !victim->pcdata )
	{
		stc( "They don't appear to have any pcdata right now.\n\r", ch );
		return;
	}

	slot = atoi( arg3 );

	if ( !is_number( arg3 ) )
	{
		stc( "Invalid slot# specified. Not a number.\n\r", ch );
		return;
	}
	if ( slot < 0 || slot > 9 )
	{
		stc( "Invalid slot# specified. Number out of range (0-9).\n\r", ch );
		return;
	}
	vnum = 3700;
	victim->pcdata->beacons[slot] = vnum;
	ptc( ch, "%s remember slot %d set to %d.\n\r", victim->name, slot, vnum );
}
void imm_cmd_process( CHAR_DATA *ch, char *argument )
{
	FILE *fp;
	char proc[124];
	char line[80];
	char buf[MSL];
	BUFFER *output;
	output = new_buf();

	stc("Do not use this command. It is for historical purposes only.\n\r", ch);
	return;

	sprintf( proc, "ps xu > ../data/ps.txt" );
	system( proc );

	fp = fopen( "../data/ps.txt","r" );

	if (!fp)
	{
		stc("An error has occured (couldn't open the file).\n\r", ch);
		return;
	}

	while( fgets( line, 80, fp ) != NULL )
	{
		sprintf(buf, "%s\n\r", line);
		add_buf(output,buf);
	}

	fclose(fp);
	page_to_char( buf_string(output), ch );
	free_buf(output);
}
void imm_cmd_qmobs( CHAR_DATA *ch, char *argument )
{
		CHAR_DATA *qmob;
	    for (qmob = char_list; qmob != NULL; qmob = qmob->next)
	    {
	        if ( IS_SET(qmob->act, ACT_UNDEAD) && IS_NPC(qmob) )
	            ptc( ch, "[%6d] %s{x\n\r", qmob->pIndexData->vnum, qmob->short_descr );
	    }
}
void imm_cmd_webdump( CHAR_DATA *ch, char *argument )
{
	FILE *fp;
	HELP_DATA *pHelp;
	char cmdBuf[MSL], socBuf[MSL], helpBuf[MSL], htxtBuf[MSL], hKey[MSL], helpBuf2[MSL];
	int iSocial;
	int col = 0;
	int  index[MIL*4];
	int  cmd;
	int  i, count = 0;

	stc("Do not use this command.\n\r", ch);
	return;

	fp = fopen( "/home/tsemud/public_html/lists/soc.txt","w" );

	if (!fp)
		return;

	for ( iSocial = 0; social_table[iSocial].name[0] != '\0'; iSocial++ )
	{
		sprintf( socBuf,"%13s",social_table[iSocial].name );
		if ( ++col % 6 == 0 )
			strcat( socBuf, "\n" );
		fprintf( fp, socBuf );
	}

	fclose( fp );
	fp = fopen( "/home/tsemud/public_html/lists/cmd.txt","w" );

	if (!fp)
		return;

	for ( count = 0; cmd_table[count].name[0] != '\0'; count++ )
		index[count]=count;

	qsort( index, count, sizeof(int), CmdCount);

	for ( i = 0; i < count ; i++ )
	{
		cmd = index[i];
		if ( cmd_table[cmd].level < LEVEL_HERO && cmd_table[cmd].show )
		{
			sprintf( cmdBuf, "%13s", cmd_table[cmd].name );
			if ( ++col % 6 == 0 )
				strcat( cmdBuf, "\n" );
			fprintf( fp, cmdBuf );
		}
	}

	fclose(fp);

	for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
	{
		if ( !str_cmp( pHelp->keyword, "PROMPT PROMPTS" ) )
			continue;

		htxtBuf[0] = '\0';
		helpBuf[0] = '\0';
		helpBuf2[0] = '\0';
		hKey[0] = '\0';

		sprintf( htxtBuf, "%s.html", pHelp->keyword );

		smash_forwardslash( htxtBuf );
		smash_space( htxtBuf );
		smash_tics( htxtBuf );
		sprintf( hKey, "/home/tsemud/public_html/lists/" );
		strcat( hKey, htxtBuf );
		fp = fopen( hKey, "w" );

		if (!fp)
			continue;

		sprintf( helpBuf, "%s\n", pHelp->text );
		smash_forwardslash( helpBuf );
		html_colourconv( helpBuf2, helpBuf, ch );
		fprintf( fp, helpBuf2 );
		fclose( fp );
	}

	fp = fopen( "/home/tsemud/public_html/lists/help.txt","w" );

	if (!fp)
		return;

	for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
	{
		fprintf( fp, "<a href=http://www.theshadowsembrace.com/lists/" );
		sprintf( helpBuf, "%s.html", pHelp->keyword );
		smash_forwardslash( helpBuf );
		smash_space( helpBuf );
		smash_tics( helpBuf );
		fprintf( fp, helpBuf );
		fprintf( fp, ">" );
		sprintf( helpBuf, "%s", pHelp->keyword );
		smash_forwardslash( helpBuf );
		smash_space( helpBuf );
		smash_tics( helpBuf );
		fprintf( fp, helpBuf );
		fprintf( fp, "</a>\n" );
	}
	fclose( fp );
}
void imm_cmd_arealookup( CHAR_DATA *ch, char *argument )
{
	AREA_DATA *pArea;
	int i = 0, match = 0;
	bool found = FALSE;
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	for ( pArea = area_first; pArea; pArea = pArea->next )
	{
		if ( strstr( pArea->name, arg2 ) == NULL )
			continue;
		else
		{
			ptc( ch, "Match: %-29s (%-12s) [%-5d-%5d]\n\r",pArea->name,pArea->file_name,pArea->min_vnum,pArea->max_vnum );
			found = TRUE;
			match++;
		}
		i++;
	}
	ptc( ch, "Searched %d records for %s. Found %d matches.\n\r", i, arg2, match  );
}
void imm_cmd_rateus( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d;
	char buf[MSL];
	buf[0] = '\0';

	sprintf(buf, "\n\r  {Y+{y----------------------------{Y+{x\n\r" );
	strcat(buf, "  {y|{w  Don't forget to rate us!  {y|{x\n\r" );
	strcat(buf, "  {Y+{y----------------------------{Y+{x\n\r\n\r" );

	for(d = descriptor_list; d != NULL; d = d->next)
	{
		if( d->character == NULL )
			continue;
		if( d->connected != CON_PLAYING )
			continue;
		stc( buf, d->character );
			vote_message( d->character );
	}
}
void imm_cmd_reports( CHAR_DATA *ch, char *argument )
{
	FILE *fp;
	char line[80];
	char buf[MSL];
	BUFFER *output;
	output = new_buf();

	if ( !read_nstring( ch ) )
	{
		stc( "Only Immortals and Moderators may read reports.\n\r", ch );
		return;
	}


	if ( ( fp = fopen( REPORT_FILE,"rt" ) ) == NULL )
	{
	    log_string("Error reading from warns.txt");
	    return;
	}

	while( fgets( line, 80, fp ) != NULL )
	{
		sprintf(buf, "%s\n\r", line);
    		add_buf(output,buf);
	}

	fclose(fp);
	page_to_char( buf_string(output), ch );
	free_buf(output);
}
void imm_cmd_warns( CHAR_DATA *ch, char *argument )
{
	FILE *fp;
	char line[80];
	char buf[MSL];
	BUFFER *output;
	output = new_buf();

	if ( !read_nstring( ch ) )
	{
		stc( "Only Immortals and Moderators may read warnings.\n\r", ch );
		return;
	}

	if ( ( fp = fopen( WARN_FILE,"rt" ) ) == NULL )
	{
	    log_string("Error reading from warns.txt");
	    return;
	}

	while( fgets( line, 80, fp ) != NULL )
	{
		sprintf(buf, "%s\n\r", line);
    		add_buf(output,buf);
	}

	fclose(fp);
	page_to_char( buf_string(output), ch );
	free_buf(output);
}
void imm_cmd_nstring( CHAR_DATA *ch, char *argument )
{
	FILE *fp;
	char line[80];
	char buf[MSL];
	BUFFER *output;
	output = new_buf();

	if ( !read_nstring( ch ) )
	{
		stc( "Only Immortals and Moderators may read nstrings.\n\r", ch );
		return;
	}

	if ( ( fp = fopen( NSTRING_FILE,"rt" ) ) == NULL )
	{
	    log_string("Error reading from nstring.dat");
	    return;
	}

	while( fgets( line, 80, fp ) != NULL )
	{
		sprintf(buf, "%s\n\r", line);
    		add_buf(output,buf);
	}

	fclose(fp);
	page_to_char( buf_string(output), ch );
	free_buf(output);
}
void imm_cmd_progdump( CHAR_DATA *ch, char *argument )
{
	PROG_CODE *prg;
	BUFFER *buffer;
	int type;
	char buf[MSL];
	buffer = new_buf();
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( arg2[0] == '\0' )
	{
		stc( "Supply type of M O or R.\n\r", ch );
		return;
	}
	if ( !str_cmp( arg2, "M" ) )
		type = PRG_MPROG;
	else if ( !str_cmp( arg2, "O" ) )
		type = PRG_OPROG;
	else if ( !str_cmp( arg2, "R" ) )
		type = PRG_RPROG;
	else
	{
		stc( "Supply type of M O or R.\n\r", ch );
		return;
	}

	switch ( type )
	{
		case PRG_MPROG:
			prg = mprog_list;
		break;
		case PRG_OPROG:
			prg = oprog_list;
		break;
		case PRG_RPROG:
			prg = rprog_list;
		break;
		default:
			return;
	}

	for( ; prg; prg = prg->next )
	{
		sprintf(buf, "%5ld\n\r", prg->vnum );
		add_buf(buffer, buf);
	}
	page_to_char(buf_string(buffer), ch);
	free_buf(buffer);
}
void imm_cmd_mysqlconnect( CHAR_DATA *ch, char *argument )
{
	if ( db_connect )
		stc("Supposedly there is already a connection.\n\r", ch);
	stc("Connecting to MySQL.\n\r", ch);
	connect_db();
}
void imm_cmd_ipcheck( CHAR_DATA *ch, char *argument )
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char mysql_buf[MSL];
	char * ip_host = "0.0.0.0";
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( arg2[0] == '\0' )
	{
		stc("Search for what IP address?\n\r", ch);
		stc("immortal ipcheck <ip address>\n\r", ch);
		return;
	}
	ip_host = str_dup(arg2);
	ptc( ch, "Checking IP address %s...\n\r", arg2 );
	sprintf( mysql_buf, "select host, name, last_name from characters where host = '%s'", arg2 );
	result = query_db(mysql_buf, result_db, FALSE);
	if ( result != NULL )
	{
		while ( (row = mysql_fetch_row(result)) != NULL )
		{
			ptc(ch,"IP matches %s %s.\n\r", row[1], row[2]);
		}
	}
	mysql_free_result(result);
}
void imm_cmd_repop( CHAR_DATA *ch, char *argument )
{
	AREA_DATA *pArea;
	bool found_area = FALSE;
	int vnum;
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( arg2[0] == 0 )
	{
		reset_area( ch->in_room->area );
		stc( "This area has been reset.\n\r", ch );
		return;
	}
	if ( is_number( arg2 ) )
	{
		vnum = atoi( arg2 );
		for ( pArea = area_first; pArea; pArea = pArea->next )
		{
			if ( pArea->vnum == vnum )
			{
				ptc( ch, "%s has been reset.\n\r", pArea->name );
				found_area = TRUE;
				reset_area( pArea );
			}
		}
		if ( !found_area )
			stc( "No such area!\n\r", ch );
		return;
	}
	if ( !strcmp( arg2, "all" ) )
	{
		for ( pArea = area_first; pArea; pArea = pArea->next )
		{
			reset_area( pArea );
		}
		stc( "World has been reset.\n\r", ch );
		return;
	}
	else
		stc("Repop what? <blank|area number|all>", ch );
}
void imm_cmd_repeat( CHAR_DATA *ch, char *argument )
{
	char arg1[MIL], arg2[MIL], arg3[MIL];
	int count = 0;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	//ptc( ch, "Arg1: %s, Arg2: %s, Arg3: %s\n\r", arg1, arg2, arg3 );

	if ( arg1[0] == '\0' || arg2[0] == '\0' || arg3[0] == '\0' )
	{
		stc( "Arguments can't be NULL. Make sure to include '' around the command\r\n", ch );
		stc( "Example: Immortal repeat 3 'at 1000 load mob 1000'\n\r", ch );
		return;
	}
	if ( strlen( arg2 ) < 1 || ( count = atoi( arg2 ) ) < 1 )
	{
		stc( "Syntax:  Immortal Repeat <count> <command>\r\n", ch );
		return;
	}
	if ( count > 20 )
	{
		stc( "That's a little overkill.\r\n", ch );
		return;
	}
	for ( ; count > 0; count-- )
		interpret( ch, arg3 );
}
void imm_cmd_areaexits( CHAR_DATA *ch, char *argument )
{
	ROOM_INDEX_DATA *rid;
	AREA_DATA *area;
	EXIT_DATA *exit;
	bool header_printed = FALSE;
	long avnum, room, door;
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 ); 

	//ptc( ch, "Arg1: %s, Arg2: %s\n\r", arg1, arg2 );

	if ( arg2[0] == '\0' )
	{
		area = ch->in_room->area;
		avnum = ch->in_room->area->vnum;
	}
	else
	{
		avnum = atoi( arg2 );
		if ( avnum == 0 )
		{
			stc( "No such area!\n\r", ch );
			return;
		}
		for ( area = area_first; area != NULL; area = area->next )
		{
			if ( area->vnum == avnum )
				break;
		}
	}
	if ( area == NULL )
	{
		stc( "No such area!\n\r", ch );
		return;
	}
	for ( room = area->min_vnum; room <= area->max_vnum; room++ )
	{
		rid = get_room_index( room );
		if ( rid == NULL )
			continue;
		for ( door = 0; door < MAX_DIR; door++ )
		{
			exit = rid->exit[door];
			if ( exit == NULL )
				continue;
			if ( exit->u1.to_room->area != area )
			{
				if ( !header_printed )
				{
					ptc( ch, "\n\rExits for area: %s\n\r\n\r"
						"[ vnum ] Room in %-25.25s [ vnum ] Room in other area\n\r"
						"-------------------------------------------------------------------------------\n\r",
						area->name, area->file_name );
					header_printed = TRUE;
				}
				ptc( ch, "[%6d] %-25.25s %4.4s to [%6d] %-25.25s in %12.12s\n\r",
					room, color_strip(rid->name), dir_name[door],
					exit->u1.to_room->vnum, color_strip(exit->u1.to_room->name),
					exit->u1.to_room->area->file_name );
			}
		}
	}
}
void imm_cmd_whonamelength( CHAR_DATA *ch, char *argument )
{
    int count,i;
    int who_max = 0;
    int who_max_guild = 0;
    char *who;
    for(i=0;i<MAX_CLAN;i++)
    {
		who = clan_table[i].who_name;
		count = 0;
		while(*who != '\0')
		{
			if(*who=='{')
			{
				who++;
				if(*who=='{')
					count++;
			}
			else
				count++;
			who++;
		}
		if(count > who_max)
		{
			who_max = count;
			who_max_guild = i;
		}
    }
    if(who_max_guild)
		ptc(ch, "The max who name %s (%s) has length %d.\n\r", clan_table[who_max_guild].who_name, clan_table[who_max_guild].name, who_max);
}
void imm_cmd_races( CHAR_DATA *ch, char *argument )
{
	bool rebirths = FALSE;
	int i,num_races,pc_races=0;
	struct remort_race_sort_struct *race_sorter;
	int sort_remort_race_by_alpha(const void *, const void *);
	int sort_remort_race_by_remorts(const void *, const void *);
	char arg1[MIL], arg2[MIL];

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 ); 

	//ptc( ch, "Arg1: %s, Arg2: %s\n\r", arg1, arg2 );

	if(!str_prefix(arg2, "rebirths")) /* blank arg2 defaults to rebirth sorting */
		rebirths = TRUE;
	else if(str_prefix(arg2, "alpha")) /* not alpha */
	{
		stc( "Immortal 'races' <rebirth|alpha>", ch);
		return;
	}
	for ( num_races = 0; race_table[num_races].name != NULL; num_races++ ); /* count races */
	race_sorter = malloc(sizeof(struct remort_race_sort_struct) * num_races);
	for ( i = 0; i < num_races ; i++ )
	{
		if(!race_table[i].pc_race)
			continue;
		race_sorter[pc_races].name = race_table[i].name;
		race_sorter[pc_races].remorts = race_table[i].remort;
		pc_races++;
	}
	if(!rebirths)
		qsort(race_sorter,pc_races,sizeof(struct remort_race_sort_struct),&sort_remort_race_by_alpha);
	else
		qsort(race_sorter,pc_races,sizeof(struct remort_race_sort_struct),&sort_remort_race_by_remorts);
	for(i=0; i < pc_races ; i++)
		ptc( ch , "%-20s%i\n\r",race_sorter[i].name,race_sorter[i].remorts);
	free(race_sorter);
}
void imm_cmd_alts( CHAR_DATA *ch, char *argument )
{
	DESCRIPTOR_DATA *d, *gch;

	stc("\n\r{W<{R---------------------------------------------------------{W>{x\n\r", ch);
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( d->connected != CON_PLAYING || !d->character )
			continue;

		for ( gch = d->next ; gch != NULL; gch = gch->next )
		{
			if ( gch->connected != CON_PLAYING || !gch->character )
				continue;
			if(is_alt_of(gch->character,d->character))
				ptc(ch,"%s is also %s\n",gch->character->name,d->character->name);
		}
	}
	stc("{W<{R---------------------------------------------------------{W>{x\n\r", ch);
}
void imm_cmd_diemess( CHAR_DATA *ch, char *argument )
{
	MOB_INDEX_DATA *pMobIndex;
	AREA_DATA *pArea;
	char buf[MSL];
	BUFFER *buf1;
	bool found;
	int vnum;

	buf1 = new_buf();
	pArea = ch->in_room->area;
	found = FALSE;

	for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
	{
		if ( ( pMobIndex = get_mob_index( vnum ) ) != NULL )
		{
				found = TRUE;
				sprintf( buf, "{c[{c%5ld{C]{c %-17.16s %-24.23s\n\r", 
					pMobIndex->vnum, capitalize( color_strip(pMobIndex->short_descr) ),
					pMobIndex->die_mess );
				add_buf( buf1, buf );
		}
	}

	if ( !found )
	{
		stc( "{cMobile(s) not found in this area.\n\r", ch);
		return;
	}

	page_to_char( buf_string(buf1), ch );
	free_buf(buf1);
}

struct  imm_cmd_type                                                                                                                                                                                     
{                                                                                                                                                                                                    
char * const        cmd;
char * options;
void (* imm_cmd_func)(CHAR_DATA *ch, char *argument);
bool log;
};

const struct imm_cmd_type imm_cmd_table [] =
{
{"repop", "<blank|area number|all>", imm_cmd_repop, TRUE },
{"repeat", "# <command>", imm_cmd_repeat, FALSE },
{"slotset", "<victim> <#> (sets to 3700)", imm_cmd_slotset, FALSE },
{"areaexits", NULL, imm_cmd_areaexits, FALSE },
{"Progdump", "M|O|R", imm_cmd_progdump, FALSE },
{"Nstring", NULL, imm_cmd_nstring, FALSE },
{"AreaLookup", "<string>", imm_cmd_arealookup },
{"WebDump", NULL, imm_cmd_webdump, FALSE },
{"Warns", NULL, imm_cmd_warns, FALSE },
{"Reports", NULL, imm_cmd_reports, FALSE },
{"QMobs", NULL, imm_cmd_qmobs, FALSE },
{"Process", NULL, imm_cmd_process, FALSE },
{"Addict", NULL, imm_cmd_addict, TRUE },
{"Dismember", NULL, imm_cmd_dismember, TRUE },
{"Fightdump", NULL, imm_cmd_fightdump, FALSE },
{"subguild", "<char> <guild>", imm_cmd_subguild, TRUE },
{"whonamelength", NULL, imm_cmd_whonamelength, FALSE },
{"alts", NULL, imm_cmd_alts, FALSE },
{"races", "<rebirth|alpha>", imm_cmd_races, FALSE },
{"rateus", NULL, imm_cmd_rateus, FALSE },
{"mysqlconnect", NULL, imm_cmd_mysqlconnect, FALSE },
{"ipcheck", "<ip address>", imm_cmd_ipcheck, FALSE },
{"diemess", NULL, imm_cmd_diemess, FALSE },
{NULL,NULL,NULL,FALSE},
};

void do_immortal( CHAR_DATA *ch, char *argument )
{
	char arg1[MIL];
	char arg2[MIL];
	char orig[MIL];
	bool found = FALSE;
	int i;

	strcpy( orig, argument );
	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( arg1[0] == '\0' )
	{
		stc( "Command Name     +  Logged  +  Options\n\r", ch );
		stc( "-----------------|----------|---------------\n\r", ch );
		for ( i = 0; imm_cmd_table[i].cmd != NULL; i++ )
		{
			ptc( ch, " %15s | %8s | %s\n\r", 
				imm_cmd_table[i].cmd, imm_cmd_table[i].log == TRUE ? "TRUE" : "FALSE",
				imm_cmd_table[i].options == NULL ? "NONE" : imm_cmd_table[i].options );
		}
		stc( "-----------------|----------|---------------\n\r", ch );
		stc( "                 +          +\n\r", ch );
		return;
	}
	else
	{
		for ( i = 0; imm_cmd_table[i].cmd != NULL; i++ )
		{
			if ( !str_prefix( arg1, imm_cmd_table[i].cmd ) )
			{
				(* imm_cmd_table[i].imm_cmd_func)(ch, orig);
				found = TRUE;
			}
		}
	}
	if ( !found )
	{
		mlog( "BUG", "Unable to locate %s in immortal command table.", orig);
		stc( "What immortal command would you like to use?\n\r", ch );
		do_function( ch, &do_immortal, "" );
		return;
	}

	if ( !str_prefix( arg1, "makeworldlinks" ) )
	{
		draw_world_map( ch );
		return;
	}
	if ( !str_prefix( arg1, "makeworldexits" ) )
	{
		create_map_exits( ch );
		return;
	}
}
