/* event.h
 *
 * This file contains the event data struture, global variables
 * and specially defined values like MAX_EVENT_HASH.
 */
extern  STACK       *   desc_free;       /* the socket free list               */
extern  LLIST        *   desc_list;       /* the linked list of active sockets  */
extern  STACK       *   ch_free;     /* the mobile free list               */
extern  LLIST        *   ch_list;     /* the mobile list of active mobiles  */

/* the size of the event queue */
#define MAX_EVENT_HASH        128

/* the different types of owners */
#define EVENT_UNOWNED           0
#define EVENT_OWNER_NONE        1
#define EVENT_OWNER_SOCKET     	2
#define EVENT_OWNER_CHAR        3
#define EVENT_OWNER_GAME        4

/* the NULL event type */
#define EVENT_NONE              0

/* Mobile events are given a type value here.
 * Each value should be unique and explicit,
 * besides that, there are no restrictions.
 */
#define EVENT_MOBILE_SAVE       1

/* Socket events are given a type value here.
 * Each value should be unique and explicit,
 * besides that, there are no restrictions.
 */
#define EVENT_SOCKET_IDLE       1

/* Game events are given a type value here.
 * Each value should be unique and explicit,
 * besides that, there are no restrictions
 */
#define EVENT_GAME_TICK         1

/* the event prototype */
typedef bool EVENT_FUN ( EVENTS_DATA *event );

/* the event structure */
struct events_data
{
  EVENT_FUN        * fun;              /* the function being called           */
  char             * argument;         /* the text argument given (if any)    */
  sh_int             passes;           /* how long before this event executes */
  sh_int             type;             /* event type EVENT_XXX_YYY            */
  sh_int             ownertype;        /* type of owner (unlinking req)       */
  sh_int             bucket;           /* which bucket is this event in       */

  union
  {                                    /* this is the owner of the event, we  */
    CHAR_DATA       	* ch;             /* use a union to make sure any of the */
    DESCRIPTOR_DATA     * desc;            /* types can be used for an event.     */
  } owner;
};

/* functions which can be accessed outside event-handler.c */
EVENTS_DATA *alloc_event          ( void );
EVENTS_DATA *event_isset_socket   ( DESCRIPTOR_DATA *desc, int type );
EVENTS_DATA *event_isset_mobile   ( CHAR_DATA *ch, int type );
void dequeue_event               ( EVENTS_DATA *event );
void init_event_queue            ( int section );
void init_events_player          ( CHAR_DATA *ch );
void init_events_socket          ( DESCRIPTOR_DATA *desc );
void heartbeat                   ( void );
void add_event_mobile            ( EVENTS_DATA *event, CHAR_DATA *ch, int delay );
void add_event_socket            ( EVENTS_DATA *event, DESCRIPTOR_DATA *desc, int delay );
void add_event_game              ( EVENTS_DATA *event, int delay );
void strip_event_socket          ( DESCRIPTOR_DATA *desc, int type );
void strip_event_mobile          ( CHAR_DATA *ch, int type );

/* all events should be defined here */
bool event_mobile_save           ( EVENTS_DATA *event );
bool event_socket_idle           ( EVENTS_DATA *event );
bool event_game_tick             ( EVENTS_DATA *event );
