/***************************************************************************

                      ___  _      _  _      _
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \__|_  /\_/\  /    \_/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/* forms.h */

/* These are the short names, shortcuts to
 * activate a form for use.
 * do_stance should take an argument, check
 * the argument against the current stance,
 * if any, and if the stance requisite is null
 * or equal to the current stance, do_stance
 * should post a notice about assuming the
 * new stance.
 * Fight should look at the current stance,
 * perform the maneuver associated with it,
 * then place the character into the follow
 * up stance, assuming the weapon type is
 * correct for the stance.
 * Damage should look at the current stance,
 * defend with the stance if the weapon type
 * is appropriate for the stance, and send
 * the correct defense message, then place
 * the character into the follow up stance.
 * Channelers and non channelers alike have
 * access to all of the forms. Non channelers
 * will have better control over the forms,
 * resulting in quicker strikes, better defense,
 * and greater offense.
 * All forms will be automatic, thereby removing
 * the need to type in a stance unless the weapon
 * type is changed, and a stance better suited
 * for that weapon is needed. This will also free
 * the blademaster up to use other combat oriented
 * skills.
 * Each group has a rebirth flag associated with
 * it that determines how many rebirths the
 * character needs to effectively perform stances
 * from within that group. A character with a
 * lesser number of rebirths may still gain and
 * use stances from within a group, but will be
 * penalized.
 */



struct stance_type
{
	char    *realName;
	char    *nameShort[5];
	char    *nameLong[5];
	char    *req;
	sh_int   primeWeapon;
	sh_int   secondWeapon;
	sh_int   tertWeapon;
	sh_int   rbi;
	long     form;
	long     subForm[5];
	int      tertiaryBonus;
	int      secondaryBonus;
	int      primeBonus;
};

struct stance_help_type
{
	char *help;
};
struct stance_specifics_type
{
	char *help;
};

const   struct  stance_type stance_table [] =
{
	{
		"Heron",
		{ "how", "hwr", "hsw", "hra", "sts" },
		{ "Heron On the Wing",
		  "Heron Wading in the Rushes",
		  "Heron Spreads its Wings",
		  "Heron Rides the Air",
		  "Sheathing The Sword"
		},
		NULL,
		WEAPON_SWORD, WEAPON_DAGGER, WEAPON_EXOTIC,
		0,
		StanceHeron,
		{ StanceHeronHOW, StanceHeronHRW, StanceHeronHSW, StanceHeronHRA, StanceHeronSTS },
		1, 2, 3
	},

	{
		"Wind",
		{ "abw", "ctf", "ftf", "lwr", "wom" },
		{ "Apple Blossoms in the Wind",
		  "Courtier Tips his Fan",
		  "Folding The Fan",
		  "Low Wind Rising",
		  "Whirlwind On the Mountain"
		},
		NULL,
		WEAPON_SWORD, WEAPON_STAFF, WEAPON_EXOTIC,
		1,
		StanceWind,
		{ StanceWindABW, StanceWindCTF, StanceWindFTF, StanceWindLWR, StanceWindWOM },
		2, 4, 6
	},

	{
		"Water",
		{ "mow", "mrw", "rub", "wfd", "tws" },
		{ "Moon on the Water",
		  "Moon Rises over the Water",
		  "River Undercuts the Bank",
		  "Water Flows Downhill",
		  "The Watered Silk"
		},
		NULL,
		WEAPON_SWORD, WEAPON_SPEAR, WEAPON_EXOTIC,
		2,
		StanceWater,
		{ StanceWaterMOW, StanceWaterMRW, StanceWaterRUB, StanceWaterWFD, StanceWaterTWS },
		4, 8, 12
	},

	{
		"Sky",
		{ "aom", "lls", "ria", "war", "lis" },
		{ "Arc Of the Moon",
		  "Lightning Lights the Sky",
		  "Ribbon in the Air",
		  "Wind And Rain",
		  "Lightning In the Sky"
		},
		NULL,
		WEAPON_SWORD, WEAPON_FLAIL, WEAPON_EXOTIC,
		3,
		StanceSky,
		{ StanceSkyAOM, StanceSkyLLS, StanceSkyRIA, StanceSkyWAR, StanceSkyLIS },
		6, 12, 18
	},

	{
		"Stone",
		{ "bts", "tgt", "sfc", "sfm", "fos" },
		{ "Bundling The Straw",
		  "The Grapevine Twines",
		  "Stone Falling from the Cliff",
		  "Stones Falling down the Mountain",
		  "Fist of Stone"
		},
		NULL,
		WEAPON_SWORD, WEAPON_MACE, WEAPON_EXOTIC,
		4,
		StanceStone,
		{ StanceStoneBTS, StanceStoneTGT, StanceStoneSFC, StanceStoneSFM, StanceStoneFOS },
		8, 16, 24
	},

	{
		"Storm",
		{ "cag", "tsg", "loh", "tsa", "lss" },
		{ "Clouds Are Gathering",
		  "The Storm Grows",
		  "Lightning On the Horizon",
		  "Thunder Shakes the Air",
		  "Lightning Scores the Sky"
		},
		NULL,
		WEAPON_SWORD, WEAPON_AXE, WEAPON_EXOTIC,
		5,
		StanceStorm,
		{ StanceStormCAG, StanceStormTSG, StanceStormLOH, StanceStormTSA, StanceStormLSS },
		10, 20, 30
	},

	{
		"Lizard",
		{ "tll", "lit", "lta", "tls", "lft" },
		{ "The Lizard Leaps",
		  "Lizard in The Thornbrush",
		  "Lizard Tastes the Air",
		  "The Lizard Scampers",
		  "Lizard Flicks its Tail"
		},
		NULL,
		WEAPON_SWORD, WEAPON_WHIP, WEAPON_EXOTIC,
		6,
		StanceLizard,
		{ StanceLizardTLL, StanceLizardLIT, StanceLizardLTA, StanceLizardTLS, StanceLizardLFT },
		12, 24, 36
	},

	{
		"Boar",
		{ "bib", "hob", "brd", "boh", "brm" },
		{ "Boar In the Brush",
		  "Horns of the Boar",
		  "Boar Rushes Downhill",
		  "Boar On the Hill",
		  "Boar Rushes down the Mountain"
		},
		NULL,
		WEAPON_SWORD, WEAPON_POLEARM, WEAPON_EXOTIC,
		7,
		StanceBoar,
		{ StanceBoarBIB, StanceBoarHOB, StanceBoarBRB, StanceBoarBOH, StanceBoarBRM },
		14, 28, 42
	},

	{
		"Bird",
		{ "hkh", "wgd", "stf", "sra", "sdw" },
		{ "Hummingbird Kisses the Honeyrose",
		  "Wood Grouse Dances",
		  "Swallow Takes Flight",
		  "Swallow Rides the Air",
		  "Swallow Dives in the Wind"
		},
		NULL,
		WEAPON_SWORD, WEAPON_DAGGER, WEAPON_EXOTIC,
		8,
		StanceBird,
		{ StanceBirdHKH, StanceBirdWGD, StanceBirdSTF, StanceBirdSRA, StanceBirdSDW },
		16, 32, 48
	},

	{
		"Cat",
		{ "csp", "ctp", "ctc", "csa", "tcp" },
		{ "Cat Sights its Prey",
		  "Cat Tracks its Prey",
		  "Cat crosses The Courtyard",
		  "Cat Sniffs the Air",
		  "The Cat Pounces"
		},
		NULL,
		WEAPON_SWORD, WEAPON_SPEAR, WEAPON_EXOTIC,
		9,
		StanceCat,
		{ StanceCatCSP, StanceCatCTP, StanceCatCTC, StanceCatCSA, StanceCatTCP },
		18, 36, 54
	},

	{
		"Elemental",
		{ "tfl", "tfg", "utf", "goe", "ror" },
		{ "The Falling Leaf",
		  "Thistledown Floats on a Gust",
		  "Unfolding The Fan",
		  "Grasp Of the Earth",
		  "Roar of the River"
		},
		NULL,
		WEAPON_SWORD, WEAPON_AXE, WEAPON_EXOTIC,
		10,
		StanceElemental,
		{ StanceElementalTFL, StanceElementalTFW, StanceElementalUTF, StanceElementalGOE, StanceElementalROR },
		20, 40, 60
	},

	{
		"Will",
		{ "tow", "sow", "pos", "sis", "fow" },
		{ "Tower Of Will",
		  "Strength Of Will",
		  "Pillar Of Strength",
		  "Standing in the Storm",
		  "Force of Will"
		},
		NULL,
		WEAPON_SWORD, WEAPON_MACE, WEAPON_EXOTIC,
		12,
		StanceWill,
		{ StanceWillTOW, StanceWillSOW, StanceWillPOS, StanceWillSIS, StanceWillFOW },
		22, 44, 66
	},
	{
		"Fire",
		{ "brt", "fls", "pof", "stb", "som" },
		{ "Blaze Rages Through the meadow",
		  "Fire Lights the Sky",
		  "Pillar of Fire",
		  "Stoking The Blaze",
		  "Smoke On the Mountain"
		},
		NULL,
		WEAPON_SWORD, WEAPON_POLEARM, WEAPON_EXOTIC,
		14,
		StanceFire,
		{ StanceFireBRM, StanceFireFLS, StanceFirePOF, StanceFireSTB, StanceFireSOM },
		24, 48, 72
	},
	{
		"Ice",
		{ "sfo", "itr", "tci", "iow", "tfw" },
		{ "Snow Falling on Oak",
		  "Ice on the Rose",
		  "The Cascading Ice",
		  "Ice on The River",
		  "The Frozen Waterfall"
		},
		NULL,
		WEAPON_SWORD, WEAPON_STAFF, WEAPON_EXOTIC,
		18,
		StanceIce,
		{ StanceIceSFO, StanceIceITR, StanceIceTCI, StanceIceIOW, StanceIceTFW },
		26, 52, 78
	},
	{
		"Spirit",
		{ "giw", "vow", "sds", "gim", "ros" },
		{ "Ghost In the Wind",
		  "Voices On the Wind",
		  "Shade Dancing on Shadows",
		  "Ghost in The Mirror",
		  "Reflection Of Self"
		},
		NULL,
		WEAPON_SWORD, WEAPON_AXE, WEAPON_SPEAR,
		21,
		StanceSpirit,
		{ StanceSpiritGIW, StanceSpiritVOW, StanceSpiritSDS, StanceSpiritGIM, StanceSpiritROS },
		28, 56, 84
	},
	{
		"Circle",
		{ "cot", "cof", "cop", "coi", "cos" },
		{ "Circle Of Thorns",
		  "Circle Of Fury",
		  "Circle Of Power",
		  "Circle Of Iron",
		  "Circle Of Steel"
		},
		NULL,
		WEAPON_SWORD, WEAPON_SPEAR, WEAPON_EXOTIC,
		25,
		StanceCircle,
		{ StanceCircleCOT, StanceCircleCOF, StanceCircleCOP, StanceCircleCOI, StanceCircleCOS },
		30, 60, 90
	},
	{
		"Crown",
		{ "tjc", "tcs", "tec", "tcw", "tcu" },
		{ "The Jade Crown",
		  "The Crown of Steel",
		  "The Emerald Crown",
		  "The Crown of War",
		  "The Crown Upraised"
		},
		NULL,
		WEAPON_SWORD, WEAPON_SWORD, WEAPON_SWORD,
		29,
		StanceCrown,
		{ StanceCrownTJC, StanceCrownTCS, StanceCrownTEC, StanceCrownTCW, StanceCrownTCU },
		32, 75, 100
	},
};
const   struct  stance_help_type stance_help_table [] =
{
	{ "Help on Blademaster Forms:" },
	{ "There are several different groups of forms, each with its" },
	{ "own individual forms. For a list of the different forms," },
	{ "type FORMS FULL. To assume a form, the character must type" },
	{ "the appropriate syntax. Typing FORMS with no arguments will" },
	{ "provide a syntax listing. Each individual forms has a short" },
	{ "abbreviation. The character decides if the form is to be" },
	{ "used offensively, or defensively, and type FORMS OFFENSIVE" },
	{ "or FORMS DEFENSIVE followed by the abbreviation. For a list" },
	{ "of abbreviations, type FORMS SHORT. For example, typing" },
	{ "FORMS OFFENSIVE HOW will set the character to attack with" },
	{ "the form Heron On the Wing. Each group of forms has three" },
	{ "weapon types associated with it. The first weapon type is" },
	{ "always sword, while the second may be any type of weapon," },
	{ "and the third is always the exotic weapon type. The first" },
	{ "type provides the greatest offensive and defensive bonuses," },
	{ "the second average bonuses, and the third the least bonus." },
	{ "While ANY character may assume a form, only a character who" },
	{ "has trained in that form type will gain usefull bonuses by" },
	{ "assuming the form. Likewise, each group has an associated" },
	{ "number of rebirths the character must meet to gain the full" },
	{ "effect of the form. In addition, as the character assumes" },
	{ "forms deeper into the group, the rebirth requirement is" },
	{ "greater. Characters may show off their prowess by typing" },
	{ "FORMS DISPLAY." },
};
const   struct  stance_specifics_type stance_specifics_table [] =
{
	{ "Specific Help on Blademaster Forms:" },
	{ "Forms are available to all characters. Non channeling humans will gain" },
	{ "the greatest bonuses to their forms. The sword is the primary weapon" },
	{ "of a Blademaster, and though other weapons may be used with forms," },
	{ "they do not provide as great a bonus as a sword. Forms are gained as" },
	{ "any other skill, though they may not be practiced. Using Forms Display" },
	{ "will allow a character to train forms to a usable level, though combat" },
	{ "use is the most effective way to increase the skills. A potential" },
	{ "Blademaster begins as a Youth, progresses to Hopeful, to Armsman," },
	{ "to Aspirant, to Skilled, to Warrior, Exhalted, to Master, to Lord," },
	{ "to Blademaster. The chosen form will take the place of an attack or" },
	{ "defensive maneuver. For example, a successful strike will display a" },
	{ "message similar to: Your Cat on a Hot Stand BUTCHERS an enemy! Or:" },
	{ "You employ Fire Lights the Sky to parry an ememy's strike! In addition" },
	{ "a successful Blademaster check will grant bonuses to the chance to" },
	{ "hit, damage, dodge, parry, counter, etc. Several factors go into the" },
	{ "Blademaster check. Primary is the skill level of the form. Like other" },
	{ "skills, forms may go over 100% in skill. Anticipation, Precision," },
	{ "Efficiency, Strength, Dexterity, Reflex, and Constitution also play" },
	{ "a primary role. Of lesser effect is the type of weapon used, though" },
	{ "it is still a noticable difference. Any character may attempt to use" },
	{ "any form, though the success or failure will be determined by having" },
	{ "the requisite skills and stats, being a channeler, non-human, or a" },
	{ "pure warrior. Experience, level, etc, do not have any effect whatsoever" },
	{ "on a Blademaster. The most successful Blademaster will be the one who" },
	{ "combines rebirthing with combat training." },
};
extern const struct stance_type stance_table[];
extern const struct stance_help_type stance_help_table[];
extern const struct stance_specifics_type stance_specifics_table[];
