/***************************************************************************

                      ___  _      _  _      _
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \__|_  /\_/\  /    \_/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/* forms.h */

/* These are the short names, shortcuts to
 * activate a form for use.
 * do_stance should take an argument, check
 * the argument against the current stance,
 * if any, and if the stance requisite is null
 * or equal to the current stance, do_stance
 * should post a notice about assuming the
 * new stance.
 * Fight should look at the current stance,
 * perform the maneuver associated with it,
 * then place the character into the follow
 * up stance, assuming the weapon type is
 * correct for the stance.
 * Damage should look at the current stance,
 * defend with the stance if the weapon type
 * is appropriate for the stance, and send
 * the correct defense message, then place
 * the character into the follow up stance.
 * Channelers and non channelers alike have
 * access to all of the forms. Non channelers
 * will have better control over the forms,
 * resulting in quicker strikes, better defense,
 * and greater offense.
 * All forms will be automatic, thereby removing
 * the need to type in a stance unless the weapon
 * type is changed, and a stance better suited
 * for that weapon is needed. This will also free
 * the blademaster up to use other combat oriented
 * skills.
 * Each group has a rebirth flag associated with
 * it that determines how many rebirths the
 * character needs to effectively perform stances
 * from within that group. A character with a
 * lesser number of rebirths may still gain and
 * use stances from within a group, but will be
 * penalized.
 */



struct sstance_type
{
	char    *realName;
	char    *nameShort[5];
	char    *nameLong[5];
	char    *req;
	sh_int   primeWeapon;
	sh_int   secondWeapon;
	sh_int   tertWeapon;
	sh_int   rbi;
	long     form;
	long     subForm[5];
	int      tertiaryBonus;
	int      secondaryBonus;
	int      primeBonus;
};

struct sstance_help_type
{
	char *help;
};
struct sstance_specifics_type
{
	char *help;
};

const   struct  sstance_type sstance_table [] =
{
	{
		"Eagle",
		{ "esw", "esc", "edp", "esp", "tge" },
		{ "Eagle Spreads it Wings",
		  "Eagle Spreads its Claws",
		  "Eagle Dives at Prey",
		  "Eagle Strikes its Prey",
		  "The Guardian Eagle"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		0,
		sStanceEagle,
		{ StanceEagleESW, StanceEagleESC, StanceEagleEDP, StanceEagleESP, StanceEagleTGE },
		1, 2, 3
	},

	{
		"Capar",
		{ "crf", "chg", "cww", "cst", "tcc" },
		{ "Capar Rushes the Flock",
		  "Capar Holds its Ground",
		  "Capar Wades the Water",
		  "Capar Shows its Tusks",
		  "The Capar Charges"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		1,
		sStanceCapar,
		{ StanceCaparCRF, StanceCaparCHG, StanceCaparCWW, StanceCaparCST, StanceCaparTCC },
		2, 4, 6
	},

	{
		"Gara",
		{ "gta", "glt", "gst", "gds", "tgb" },
		{ "Gara Tastes the Air",
		  "Gara Lashes its Tail",
		  "Gara Shows its Fangs",
		  "Gara Digging in the Sands",
		  "The Garas Bite"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		2,
		sStanceGara,
		{ StanceGaraGTA, StanceGaraGLT, StanceGaraGST, StanceGaraGDS, StanceGaraTGB },
		4, 8, 12
	},

	{
		"Hare",
		{ "hsb", "hdh", "hls", "ham", "thw" },
		{ "Hare in the Segade Bush",
		  "Hare Dodges the Hawk",
		  "Hare Leaps the Snake",
		  "Hare Admires the Moon",
		  "The Hare Waits"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		3,
		sStanceHare,
		{ StanceHareHSB, StanceHareHDH, StanceHareHLS, StanceHareHAM, StanceHareTHW },
		6, 12, 18
	},

	{
		"Snake",
		{ "mks", "rcd", "bst", "sss", "ttv" },
		{ "Mountian King Strikes",
		  "Redadder Coils in Defense",
		  "Bloodsnake Shows its Fangs",
		  "Snake Suns on the Sands",
		  "The Twosteps Victory"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		4,
		sStanceSnake,
		{ StanceSnakeMKS, StanceSnakeRCD, StanceSnakeBST, StanceSnakeSSS, StanceSnakeTTV },
		8, 16, 24
	},

	{
		"Ji",
		{ "tdh", "sah", "twh", "dah", "toh" },
		{ "The Dance of Honor",
		  "Strength of Aiel Honor",
		  "The Weight of Honor",
		  "Defending Aiel Honor",
		  "The Obligation of Honor"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		5,
		sStanceJi,
		{ StanceJiTDH, StanceJiSAH, StanceJiTWH, StanceJiDAH, StanceJiTOH },
		10, 20, 30
	},

	{
		"Sand",
		{ "tss", "tbs", "tes", "iow", "sth" },
		{ "The Swirling Sands",
		  "The Burning Sands",
		  "The Endless Sands",
		  "Illusions Of Water",
		  "Sandstorm in The Hold"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		6,
		sStanceSand,
		{ StanceSandTSS, StanceSandTBS, StanceSandTES, StanceSandIOW, StanceSandSTH },
		12, 24, 36
	},

	{
		"Ridgecat",
		{ "rsp", "rbc", "rpc", "rfl", "dth" },
		{ "Ridgecat Stalks its Prey",
		  "Ridgecat Bares its Claws",
		  "Ridgecat Pounces from the Cliff ",
		  "Ridgecat Faces the Lion",
		  "Defending The Hold"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		7,
		sStanceRidgecat,
		{ StanceRidgecatRSP, StanceRidgecatRBC, StanceRidgecatRPC, StanceRidgecatRFL, StanceRidgecatDTH },
		14, 28, 42
	},

	{
		"Lion",
		{ "lsf", "las", "lop", "tlh", "tpw" },
		{ "Lion Stalks the Flock",
		  "Lion Amongst the Sheep",
		  "Lion Overlooks the Pride",
		  "The Lions Hunt",
		  "The Predator Within"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		8,
		sStanceLion,
		{ StanceLionLSF, StanceLionLAS, StanceLionLOP, StanceLionTLH, StanceLionTPW },
		16, 32, 48
	},

	{
		"Dream",
		{ "lad", "wfd", "wtd", "dtw", "doh" },
		{ "Life is A Dream",
		  "Wake From the Dream",
		  "Walking The Dream",
		  "Dream To Wake",
		  "Dreams Of Honor"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		9,
		sStanceDream,
		{ StanceDreamLAD, StanceDreamWFD, StanceDreamWTD, StanceDreamDTW, StanceDreamDOH },
		18, 36, 54
	},

	{
		"Shadow",
		{ "sin", "stn", "tis", "ain", "cts" },
		{ "Spears In the Night",
		  "Stalking The Night",
		  "The Invisible Spear",
		  "Assassin In the Night",
		  "Charging The Shadow"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		10,
		sStanceShadow,
		{ StanceShadowSIN, StanceShadowSTN, StanceShadowTIS, StanceShadowAIN, StanceShadowCTS },
		20, 40, 60
	},

	{
		"Wrath",
		{ "arf", "vow", "dor", "als", "wos" },
		{ "Ancestral Rage of Fury",
		  "Veil Of Wrath",
		  "Dance Of Reckoning",
		  "Avenging Lamans Sin",
		  "Wrath Of the Spears"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		12,
		sStanceWrath,
		{ StanceWrathARF, StanceWrathVOW, StanceWrathDOR, StanceWrathALS, StanceWrathWOS },
		22, 44, 66
	},
	{
		"Mystic",
		{ "stp", "wtp", "sod", "ccc", "gor" },
		{ "Seeking The Prophecy",
		  "Whispers of The Past",
		  "Screams Of Defiance",
		  "Carai an Car'a'Carn",
		  "Guardian Of Rhuidean"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		14,
		sStanceMystic,
		{ StanceMysticSTP, StanceMysticWTP, StanceMysticSOD, StanceMysticCCC, StanceMysticGOR },
		24, 48, 72
	},
	{
		"Ring",
		{ "ros", "rop", "roa", "rol", "rod" },
		{ "Ring Of Spears",
		  "Ring Of Protection",
		  "Ring Of Aggression",
		  "Ring Of Life",
		  "Ring Of Death"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		18,
		sStanceRing,
		{ StanceRingROS, StanceRingROP, StanceRingROA, StanceRingROL, StanceRingROD },
		26, 52, 78
	},
	{
		"Earth",
		{ "rtw", "tjc", "tdm", "tsm", "trl" },
		{ "Rocks in The Wind",
		  "The Jagged Cliffs",
		  "Thunder Down the Mountian",
		  "The Sloping Mountains",
		  "The Ridged Lands"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		21,
		sStanceEarth,
		{ StanceEarthRTW, StanceEarthTJC, StanceEarthTDM, StanceEarthTSM, StanceEarthTRL },
		28, 56, 84
	},
	{
		"Balance",
		{ "bts", "bos", "bow", "bom", "bop" },
		{ "Balance The Spears",
		  "Balance Of Strenght",
		  "Balance Of Will",
		  "Balance Of Mind",
		  "Balance Of Power"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_DAGGER, WEAPON_AXE,
		25,
		sStanceBalance,
		{ StanceBalanceBTS, StanceBalanceBOS, StanceBalanceBOW, StanceBalanceBOM, StanceBalanceBOP },
		30, 60, 90
	},
	{
		"Dance",
		{ "dts", "sor", "dtd", "sow", "dod" },
		{ "Dance The Spears",
		  "Spears Of Rage",
		  "Dance The Dream",
		  "Spears Of Wrath",
		  "Dance Of Death"
		},
		NULL,
		WEAPON_SPEAR, WEAPON_SPEAR, WEAPON_SPEAR,
		29,
		sStanceDance,
		{ StanceDanceDTS, StanceDanceSOR, StanceDanceDTD, StanceDanceSOW, StanceDanceDOD },
		32, 75, 100
	},
};
const   struct  sstance_help_type sstance_help_table [] =
{
	{ "Help on Spear Forms:" },
	{ "There are several different groups of spear forms, each with its" },
	{ "own individual forms. For a list of the different forms," },
	{ "type SPEARFORMS FULL. To assume a form, the character must type" },
	{ "the appropriate syntax. Typing SPEARFORMS with no arguments will" },
	{ "provide a syntax listing. Each individual forms has a short" },
	{ "abbreviation. The character decides if the form is to be used" },
	{ "offensively, or defensively, and type SPEARFORMS OFFENSIVE" },
	{ "or SPEARFORMS DEFENSIVE followed by the abbreviation. For a list" },
	{ "of abbreviations, type SPEARFORMS SHORT. For example, typing" },
	{ "SPEARFORMS OFFENSIVE ESW will set the character to attack with" },
	{ "the form Eagle Spreads its Wings. Each group of forms has three" },
	{ "weapon types associated with it. The first weapon type is" },
	{ "always spear, while the second is dagger, and the third type" },
	{ "is always axe. The first type provides the greatest offensive" },
	{ "and defensive bonuses, the second average bonuses, and the third" },
	{ "the least bonus. While only Aiel may assume any of these" },
	{ "forms, only a character who has trained in that form will gain" },
	{ "usefull bonuses by assuming the form. Likewise, each group has an" },
	{ "associated number of rebirths the character must meet to gain" },
	{ "the full effect of the form. In addition, as the character assumes" },
	{ "forms deeper into the group, the rebirth requirement is greater." },
};
const   struct  sstance_specifics_type sstance_specifics_table [] =
{
	{ "Specific Help on Spear Forms:" },
	{ "Spearforms are available to Aiel only. Non channeling Aiel will gain" },
	{ "the greatest bonuses to their forms. The spear is the primary weapon," },
	{ "and though other weapons may be used with forms,they do not provide" },
	{ "as great a bonus as a spear. Spearforms are gained as any other skill," },
	{ "though they may not be practiced. Using them in combat is the most" },
	{ "effective way to increase the skills. The chosen form will take the" },
	{ "place of an attack or defensive maneuver. For example, a successful" },
	{ "strike will display a message similar to: Your Ridgecat Faces the Lion" },
	{ "BUTCHERS an enemy!" },
};
//extern const struct stance_type stance_table[]; // prool dirty hack
//extern const struct stance_help_type stance_help_table[];
//extern const struct stance_specifics_type stance_specifics_table[];
