/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"

int nAllocPerm;
int nAllocString;

bool	check_social	args( ( CHAR_DATA *ch, char *command,
			    char *argument ) );
/* disable.c */
bool	check_disabled	args( ( const struct cmd_type *command ) );

/*
 * Command logging types.
 */
#define LOG_NORMAL	0
#define LOG_ALWAYS	1
#define LOG_NEVER	2



/*
 * Log-all switch.
 */
bool				fLogAll		= FALSE;
bool fLogAllQuiet = FALSE;

/* signal handling */
char last_command[MAX_STRING_LENGTH];
char last_command2[MAX_INPUT_LENGTH];
int last_descriptor;

const	struct	newb_cmd_type	newb_cmd_table	[] =
{
    { "l",           do_look,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
    { "north",		do_north,	POS_STANDING,    0,  LOG_NEVER, 0, 0, 0 },
    { "east",		do_east,	POS_STANDING,	 0,  LOG_NEVER, 0, 0, 0 },
    { "south",		do_south,	POS_STANDING,	 0,  LOG_NEVER, 0, 0, 0 },
    { "west",		do_west,	POS_STANDING,	 0,  LOG_NEVER, 0, 0, 0 },
    { "up",		    do_up,		POS_STANDING,	 0,  LOG_NEVER, 0, 0, 0 },
    { "down",		do_down,	POS_STANDING,	 0,  LOG_NEVER, 0, 0, 0 },
    { "northeast",  do_northeast,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "southeast",  do_southeast,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "southwest",  do_southwest,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "northwest",  do_northwest,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "ne",         do_northeast,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "se",         do_southeast,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "sw",         do_southwest,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "nw",         do_northwest,POS_STANDING,   0,  LOG_NEVER, 0, 0, 0 },
    { "go",         do_enter,   POS_STANDING,    0,  LOG_NEVER, 0, 0, 0 },
};
/*
 * Command table.
 */
const	struct	cmd_type	cmd_table	[] =
{

	/*
	 * Commands which, for whatever reason, have been commented out so that they don't function
	 */
/* { "darktalk",     do_darktalk, POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 }, */
/* { "darkooctalk", do_darkooc, POS_DEAD,          0, LOG_NORMAL, 1, 0, 1 }, */
/* { "handtalk",     do_handtalk, POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 }, */
/* { "seantalk",     do_seantalk, POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 }, */
/* { "gamble",     do_gamble,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 }, */
/* { "attack", do_kill, POS_FIGHTING, 0, LOG_NORMAL, 0, 0, 1 }, */ /* moved below do_at */
// { "keyring",      do_keyring,     POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
// { "mine",           do_mine,        POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
// { "evaluate",     do_evaluate,POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
// { "mhelp",                do_mysql_help,  POS_DEAD,        0,  LOG_NORMAL, 1, 0, 1 },
// { "alia",           do_alia,        POS_DEAD,        0,  LOG_NORMAL, 0, 0, 1 },
// { "alias",          do_alias,       POS_DEAD,        0,  LOG_NORMAL, 1, 0, 1 },
/* { "title",         do_title,       POS_DEAD,        0,  LOG_NORMAL, 1, 3, 0 }, */
/* { "demand",               do_demand,       POS_STANDING,   0, LOG_NORMAL, 1, 0, 0 }, */
/* { "yell",         do_yell,         POS_STANDING,   1, LOG_NORMAL, 1, 0, 1 },*/
/* { "brandish",     do_brandish,POS_RESTING,         0,  LOG_NORMAL, 1, 0, 1 }, */
// { "cut",          do_cut,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
// { "fletch",               do_fletch,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
// { "infuse",               do_infuse,      POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
// { "hem",          do_hem,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
// { "layout",               do_layout,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
// { "shape",                do_shape,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
// { "smith",      do_smith,   POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
// { "stitch",     do_stitch,  POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
/* { "zap",          do_zap,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 }, */
// { "murde",                do_murde,       POS_FIGHTING,0, LOG_NORMAL, 0, 0, 0 },
// { "murder",         do_murder,      POS_FIGHTING,    5,  LOG_ALWAYS, 1, 1, 0 },
/* { "roulette", do_roulette,POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },*/
/* { "dfhstat",  do_dfhstat,     POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },*/
/* { "shadowpass", do_shadowpass,POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },*/
/* { "return",     do_return,  POS_DEAD,      255,  LOG_ALWAYS, 1, 0 }, */
/* { "switch",        do_switch, POS_DEAD,  255, LOG_ALWAYS, 1, 0 }, */
/* { "steal",                do_steal,       POS_STANDING,  0, LOG_ALWAYS, 1, 0, 0 }, */


	/*
	 * Common commands and commands stacked early to prevent conflict.
	 */

	{ ",",  do_emote,    POS_RESTING,    0, LOG_NORMAL, 0, 1, 0 },
	{ ".",  do_global,   POS_DEAD,   1, LOG_NORMAL, 1, 1, 1 },
	{ ";",  do_gtell,    POS_DEAD,               0, LOG_NORMAL, 0, 0, 1 },
	{ ":",  do_osay,     POS_RESTING,    0, LOG_NORMAL, 1, 0, 1 },
	{ "'",  do_say,      POS_RESTING,    0, LOG_NORMAL, 0, 0, 1 },
	{ "!",  do_esay,         POS_RESTING,    0, LOG_NORMAL, 1, 0, 1 },
	{ "@",  do_emote,       POS_DEAD,       1, LOG_NORMAL, 1, 1, 1 },
        { "/",  do_recall, POS_FIGHTING, 0, LOG_NORMAL, 0, 0, 0 },
        { "*",  do_clanrecall, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },


	{ "l", do_look, POS_RESTING, 0, LOG_NORMAL, 1, 0, 1 },
	{ "north", do_north, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "east", do_east, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "south", do_south, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "west", do_west, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "up", do_up, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "down", do_down, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "northeast", do_northeast,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "southeast", do_southeast,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "southwest", do_southwest,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "northwest", do_northwest,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "ne", do_northeast,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "se", do_southeast,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "sw", do_southwest,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "nw", do_northwest,POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "go", do_enter, POS_STANDING, 0, LOG_NEVER, 0, 0, 0 },
	{ "jog", do_jog, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },

	/*
	 * Commands that start with the letter A
	 */

	{ "accept",     do_accept,  POS_STANDING,   10,  LOG_NORMAL, 1, 1, 0 },
	{ "affects",	do_affects,	POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "age", do_age, POS_DEAD, 0, LOG_NORMAL, 1, 0, 0 },
	{ "adorn",          do_adorn,       POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "answer", do_question, POS_DEAD, 1, LOG_NORMAL, 1, 1, 1 },
	{ "afk",            do_afk,      POS_DEAD,   0, LOG_NORMAL, 1, 0, 0 },
	{ "ambush", do_assassinate,POS_FIGHTING, 0,  LOG_NORMAL, 1, 0, 0 },
	{ "appraise",   do_appraise, POS_RESTING,    0, LOG_NORMAL, 1, 0, 0 },
	{ "areas",		do_areas,	POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "auction", do_auction, POS_SLEEPING, 0, LOG_NORMAL, 1, 1, 0 },
        { "auto", do_toggle, POS_DEAD, 0, LOG_NORMAL, 1, 0, 1 },


	{ "backstab",       do_backstab,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "ban",            do_ban,         POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 0 },
	{ "bandage",        do_bandage,     POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "bash",           do_bash,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "bs",                     do_backstab,POS_FIGHTING,        0,  LOG_NORMAL, 0, 0, 0 },
	{ "berserk",        do_berserk,     POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "battlecry",      do_battlecry,POS_FIGHTING,       0,  LOG_NORMAL, 1, 0, 1 },
	{ "balance",    do_balance, POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "bite",           do_bite,        POS_FIGHTING,    0,  LOG_NORMAL, 0, 0, 1 },
	{ "board",   do_board, POS_DEAD,   0, LOG_NORMAL, 1, 1, 0 },
	{ "brush",		do_brush,	POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "buddy",		do_buddy,	POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "bug",		do_new_bug,		POS_DEAD,        0,  LOG_ALWAYS, 1, 0, 0 },
	{ "butcher",        do_butcher,     POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "buy",		do_buy,		POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 0 },
	{ "battle",         do_battle,      POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "bopen",          do_bopen,       POS_DEAD,    425,  LOG_NORMAL, 1, 0, 0 },
	{ "bclose",         do_bclose,      POS_DEAD,    425,  LOG_NORMAL, 1, 0, 0 },
	{ "bstop",          do_bstop,       POS_DEAD,    425,  LOG_NORMAL, 1, 0, 0 },
	{ "bchat",          do_bchat,       POS_DEAD,    425,  LOG_NORMAL, 1, 0, 0 },
	{ "badd",           do_badd,        POS_DEAD,    425,  LOG_NORMAL, 1, 0, 0 },
	{ "blackjack",      do_blackjack,POS_FIGHTING,   0,  LOG_NORMAL, 1, 0, 0 },
	{ "blue", do_blue, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0},
	{ "bribe",          do_bribe,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "bremove",        do_bremove,     POS_DEAD,    425,  LOG_NORMAL, 1, 0, 0 },
        { "bounty", do_bounty, POS_STANDING,  0, LOG_NORMAL, 1, 0, 0 },
        { "bounties", do_bounties, POS_STANDING,  0, LOG_NORMAL, 1, 0, 0 },
        { "bet", do_bet, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "backup", do_backup, POS_DEAD, 0, LOG_ALWAYS, 1, 0, 0 },
        { "beep",       do_beep,     POS_RESTING,    0, LOG_NORMAL, 1, 3, 0 },
        { "bondmote",   do_bondmote, POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 },
        { "bondtalk",   do_bondtalk, POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 },
        { "bondwhere",  do_bondwhere,POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 },
	{ "brew",           do_brew,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },


	{ "channel",    do_weave,   POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "chat",           do_irc,  POS_DEAD,       0, LOG_NORMAL, 1, 0, 0 },
	{ "chew",       do_chew,    POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "coder",          do_coder,        POS_DEAD,   0, LOG_NORMAL, 1, 0, 0 },
	{ "consider",       do_consider,POS_RESTING, 0,  LOG_NORMAL, 1, 0, 1 },
	{ "continue",   do_continue,POS_DEAD,        0,  LOG_NORMAL, 0, 0, 0 },
	{ "challenge",  do_challenge,POS_STANDING,  10,  LOG_NORMAL, 1, 1, 0 },
	{ "circle",         do_circle,      POS_FIGHTING,    0,  LOG_NORMAL, 0, 0, 0 },
	{ "claw",           do_claw,        POS_FIGHTING,    0,  LOG_NORMAL, 0, 0, 1 },
	{ "climb",      do_climb,   POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "clantalk", do_clantalk, POS_SLEEPING, 0,  LOG_NORMAL, 1, 0, 0 },
	{ "close",          do_close,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "combine",                do_combine,     POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "changes",        do_note,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "commands",       do_commands,POS_DEAD,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "compare",        do_compare,     POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
	{ "colour",         do_colour,      POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "color",          do_colour,      POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "consent",     do_consent, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0},
	{ "count",          do_count,       POS_SLEEPING,0,  LOG_NORMAL, 1, 0, 0 },
	{ "craft",          do_craft,       POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "crecall", do_clanrecall, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "credits",        do_credits,     POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },


	{ "dance",      do_dance,   POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "deaf",           do_deaf,         POS_DEAD,       0, LOG_NORMAL, 1, 0, 0 },
	{ "decline",    do_decline, POS_STANDING,   10,  LOG_NORMAL, 1, 0, 0 },
	{ "description",do_description,     POS_DEAD,0,  LOG_NORMAL, 1, 1, 0 },
	{ "delet",          do_delet,       POS_DEAD,        0,  LOG_ALWAYS, 0, 0, 0 },
	{ "delete",         do_delete,      POS_STANDING,0,  LOG_ALWAYS, 1, 0, 0 },
	{ "deposit",    do_deposit, POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "dirt",           do_dirt,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "disarm",         do_disarm,      POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "draw",       do_draw,    POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "dream",          do_dream,   POS_SLEEPING,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "dislodge",   do_dislodge,POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "dismount", do_dismount, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "disembark", do_disembark, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "dfhunt",     do_dfhunt,  POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "divorce",     do_divorce, POS_DEAD,   300, LOG_NORMAL, 1, 0, 0},
	{ "drink",          do_drink,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "drop",           do_drop,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "drag",           do_drag,        POS_STANDING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "dual",       do_second,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "dye",            do_dye,         POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },


	{ "eat",            do_eat,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "elbow",          do_elbow,       POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "envenom",        do_envenom,     POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "equipment",      do_equipment,   POS_DEAD,0,  LOG_NORMAL, 1, 0, 1 },
	{ "eqstats", do_eqstats, POS_DEAD, 0, LOG_NORMAL, 1, 0, 1 },
        { "enter", do_enter, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
        { "email", do_email, POS_STANDING, 0, LOG_ALWAYS, 1, 1, 0 },
	{ "emote",          do_emote,        POS_RESTING,    0, LOG_NORMAL, 1, 1, 1 },
	{ "engage",         do_engage,      POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "enhance",        do_enhance,     POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "enlist", do_enlist,       POS_DEAD,  1, LOG_NORMAL, 1, 0, 1 },
        { "embark", do_embark, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "esay",           do_esay,         POS_RESTING,    0, LOG_NORMAL, 1, 0, 1 },
	{ "examine",        do_examine,     POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
	{ "events",     do_events,  POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "exhale",     do_exhale,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "exits",		do_exits,	POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 0 },
	{ "explored",	do_explored,	POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 0 },
	{ "exchange",	do_exchange,POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 0 },



	{ "fear",           do_fear,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "forms",      do_forms,   POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "flee",           do_flee,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "fill",           do_fill,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "fire",       do_fire,    POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "finger",	    do_finger,  POS_DEAD,	     0,  LOG_NORMAL, 1, 0, 0 },
	{ "finish",		do_finish,	POS_DEAD,	 0,  LOG_NORMAL, 1, 0, 1 },
        { "follow", do_follow, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "fish", do_tickle, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "feed", do_feed, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "footsweep",      do_sweepkick,POS_FIGHTING,       0,  LOG_NORMAL, 1, 0, 1 },
	{ "forage",         do_forage,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "ftag", do_ftag, POS_SLEEPING, 0, LOG_NORMAL, 1, 0, 0},


	{ "gcom",   do_gcom,         POS_DEAD,  1, LOG_NORMAL, 1, 0, 1 },
	{ "get",		do_get,		POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 0 },
        { "gain", do_gain, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
        { "gquest", do_gquest, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "groups", do_groups, POS_SLEEPING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "gevent",		do_gevent,	POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 0 },
	{ "give",           do_give,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "grasp",      do_grasp,   POS_RESTING, 0,  LOG_NORMAL, 1, 0, 1 },
	{ "group",      do_group,   POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "gltalk",     do_gltalk,      POS_DEAD,       1, LOG_NORMAL,  0, 0, 1 },
	{ "global",         do_global,       POS_DEAD,       1, LOG_NORMAL, 1, 1, 1 },
	{ "gsocial",        do_gsocial,      POS_DEAD,       1, LOG_NORMAL, 1, 1, 1 },
	{ "gtell",          do_gtell,        POS_DEAD,               0, LOG_NORMAL, 1, 0, 1 },


        { "help",           do_help,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 1 },
        { "hide", do_hide, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "home", do_home, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "hit",		do_kill,	POS_FIGHTING,	 0,  LOG_NORMAL, 0, 0, 1 },
	{ "headbutt",       do_headbutt,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "heal",           do_heal,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "hold",           do_wear,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "hook",           do_hook,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "hookkick",       do_hookkick,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "hurricane",  do_hurricane,POS_FIGHTING,   0,  LOG_NORMAL, 1, 0, 0 },


	{ "i",  do_inventory,   POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "inchar",     do_inchar,   POS_STANDING,   1, LOG_NORMAL, 1, 1, 1 },
	{ "ic",         do_inchar,   POS_STANDING,   1, LOG_NORMAL, 1, 1, 1 },
	{ "idea",           do_note,        POS_DEAD,        0,  LOG_NORMAL, 1, 1, 0 },
	{ "info",       do_new_score,   POS_SLEEPING,0,  LOG_NORMAL, 1, 0, 1 },
	{ "inventory",	do_inventory,	POS_DEAD,	 0,  LOG_NORMAL, 1, 0, 0 },


        { "join", do_join, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "journal",        do_journal,     POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "jumpkick",       do_jumpkick,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "junk",       do_junk,    POS_RESTING,    0,  LOG_NORMAL, 1, 0, 0 },


	{ "k",           do_kill,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "keep",           do_keep,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "kill",		do_kill,	POS_FIGHTING,	 0,  LOG_NORMAL, 1, 0, 1 },
	{ "kick",           do_kick,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "knee",           do_knee,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "knock",      do_knock,    POS_RESTING,    0, LOG_NORMAL, 1, 0, 0 },


	{ "last",           do_last,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "leaders",     do_leaders,  POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "leapkick",       do_leapkick,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "legsweep",       do_legsweep,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "letgo",      do_letgo,   POS_RESTING, 0,  LOG_NORMAL, 1, 0, 1 },
	{ "look",		do_look,	POS_RESTING,	 0,  LOG_NORMAL, 1, 0, 1 },
	{ "learn",		do_learn,	POS_STANDING,	 0,  LOG_NORMAL, 1, 0, 0 },
	{ "light",      do_light,   POS_STANDING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "list",           do_list,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "link",		do_link,	POS_STANDING,	 0,  LOG_NORMAL, 1, 0, 0 },
	{ "lock",           do_lock,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },


        { "map", do_map, POS_DEAD, 0, LOG_NORMAL, 1, 0, 1 },
	{ "marry",       do_marry,   POS_DEAD,   300, LOG_NORMAL, 1, 0, 0},
	{ "mentor", do_mentor,       POS_DEAD,  1, LOG_NORMAL, 1, 0, 1 },
	{ "mob",            do_mob,         POS_DEAD,        0,  LOG_NEVER,  0, 0, 1 },
        { "mount", do_mount, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },


	{ "news",           do_note,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "nibble",     do_nibble,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "nmote",      do_nmote,    POS_DEAD,       0, LOG_NORMAL, 1, 1, 1 },
	{ "note",           do_note,         POS_DEAD,       0, LOG_NORMAL, 1, 0, 0 },
	{ "nstring",    do_note,  POS_DEAD,   0, LOG_NORMAL, 1, 0, 0 },


	{ "observe",        do_observe,     POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
        { "oldscore",               do_score,       POS_DEAD,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "ooc",        do_global,      POS_DEAD,   1, LOG_NORMAL, 0, 1, 1 },
	{ "open",           do_open,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "osay",           do_osay,         POS_RESTING,    0, LOG_NORMAL, 1, 0, 1 },
        { "order",          do_order,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "outfit",         do_outfit,      POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },

	{ "practice",   do_practice,POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "password",       do_password,POS_DEAD,    0,  LOG_NEVER,  1, 0, 0 },
        { "passwd",       do_password,POS_DEAD,    0,  LOG_NEVER,  1, 0, 0 },
        { "play", do_play, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "pmote",          do_pmote,        POS_RESTING,    0, LOG_NORMAL, 1, 1, 1 },
	{ "pose",           do_pose,         POS_RESTING,    0, LOG_NORMAL, 1, 1, 0 },
	{ "prompt",         do_prompt,      POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
        { "pathfind",   do_pathfind,POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "pawn",       do_pawn,    POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "peek",           do_peek,        POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
        { "pk",         do_pkill,   POS_DEAD,            10, LOG_NORMAL, 1, 1, 0 },
        { "pkill",          do_pkill,       POS_DEAD,            10, LOG_NORMAL, 1, 1, 0 },
	{ "pick",           do_pick,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "pour",           do_pour,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "put",            do_put,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
        { "proficiency",do_proficiency,POS_SLEEPING, 0,  LOG_NORMAL, 1, 0, 1 },
        { "profile",        do_profile,POS_SLEEPING, 0,  LOG_NORMAL, 1, 0, 0 },
        { "promote",    do_promote,     POS_DEAD,        0,  LOG_NORMAL, 0, 0, 0 },
	{ "propose",     do_propose, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0},
	{ "punch",          do_punch,       POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },


	{ "qa",         do_qa,       POS_DEAD,   0, LOG_NORMAL, 1, 1, 0 },
	{ "quaff",          do_quaff,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "quest",      do_quest,    POS_STANDING,   0, LOG_NORMAL, 1, 1, 0 },
	{ "question", do_question, POS_DEAD, 1, LOG_NORMAL, 1, 1, 1 },
        { "qui", do_qui, POS_DEAD, 0, LOG_NORMAL, 0, 0, 0 },
        { "quit", do_quit, POS_DEAD, 0, LOG_NORMAL, 1, 0, 0 },


        { "red", do_red, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0},
        { "read",           do_read,        POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
	{ "rebirth",    do_rebirth, POS_STANDING,0,  LOG_ALWAYS, 1, 0, 0 },
        { "recall", do_recall, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "recite",         do_recite,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
        { "remember", do_beacon, POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
        { "rent",               do_rent,        POS_DEAD,        0,  LOG_NORMAL, 0, 0, 0 },
        { "roulette", do_roulette,POS_RESTING, 0, LOG_NORMAL, 1, 0, 0 },
	{ "remove",         do_remove,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "repair",         do_repair,      POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "reinforce",      do_reinforce,   POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "rescue",         do_rescue,      POS_FIGHTING,    0,  LOG_NORMAL, 0, 0, 1 },
	{ "rage",           do_rage,        POS_FIGHTING,    0,  LOG_NORMAL, 0, 0, 1 },
	{ "rush",           do_rush,        POS_FIGHTING,    0,  LOG_NORMAL, 0, 0, 1 },
	{ "restring",   do_restring,POS_STANDING,    0,  LOG_NORMAL, 1, 3, 0 },
	{ "release",    do_release, POS_DEAD,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "reply",          do_reply,        POS_DEAD,       0, LOG_NORMAL, 1, 0, 1 },
	{ "replay",         do_replay,   POS_DEAD,   0, LOG_NORMAL, 1, 0, 1 },
	{ "rdesc",      do_rdesc,   POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "repeat",      do_repeat,   POS_DEAD,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "roleplay",   do_note, POS_DEAD,   0, LOG_NORMAL, 1, 1, 0 },
	{ "rplog", do_rplog, POS_DEAD, 0, LOG_NORMAL, 0, 0, 0 },
	{ "report",         do_report,      POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
	{ "rules",          do_rules,       POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "retell",         do_retell,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "rest",           do_rest,        POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "rprecall", do_rprecall, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "rpscore",    do_rpscore, POS_DEAD,    0,  LOG_NORMAL, 1, 0, 1 },
        { "rub",            do_rub,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },


	{ "score",          do_new_score,   POS_DEAD,        0,  LOG_NORMAL, 1, 0, 1 },
        { "sacrifice",      do_sacrifice,POS_RESTING,        0,  LOG_NORMAL, 1, 0, 1 },
        { "save",               do_save,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "say",            do_say,          POS_RESTING,    0, LOG_NORMAL, 1, 0, 1 },
	{ "sayto",          do_sayto,        POS_RESTING,    0, LOG_NORMAL, 1, 0, 1 },
	{ "sell",           do_pawn,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "seal",           do_seal,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "sendto",         do_sendto,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "sheet",       do_new_score,   POS_SLEEPING,0,  LOG_NORMAL, 1, 0, 1 },
        { "shadowwalk", do_new_shadowwalk,POS_STANDING,  0,  LOG_NORMAL, 1, 0, 0 },
        { "sharpen",        do_sharpen, POS_RESTING,         0,  LOG_NORMAL, 1, 0, 0 },
        { "shop",           do_shop,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "sidekick",       do_sidekick,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "study",          do_study,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "sacrifice",      do_sacrifice,POS_RESTING,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "smelt",          do_smelt,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "second",     do_second,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "shave",      do_shave,   POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "sip",        do_sip,     POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "sing",           do_sing,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "sleep",          do_sleep,       POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "smote",      do_smote,    POS_DEAD,       0, LOG_NORMAL, 1, 1, 1 },
        { "sneak",          do_sneak,       POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "scan",           do_scan,        POS_STANDING,0,  LOG_NORMAL, 1, 0, 0 },
	{ "skills",         do_sklist,      POS_DEAD,        0,  LOG_NORMAL, 1, 0, 1 },
	{ "socials",        do_socials,     POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "select",         do_select,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "show",           do_show,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "summary",        do_summary,     POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "story",          do_story,       POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "system",         do_system,      POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "split",          do_split,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
        { "skin",           do_skin,        POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "spar",           do_kill,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
        { "search",         do_search,      POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "sendmail",       do_sendmail,POS_STANDING,        0,  LOG_ALWAYS, 1, 1, 0 },
        { "sit",            do_sit,         POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "smoke",      do_smoke,   POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "spinkick",       do_spinkick,POS_FIGHTING,        0,  LOG_NORMAL, 1, 0, 1 },
        { "spit",       do_spit,    POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "stand",          do_stand,       POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 1 },
        { "step",           do_step,        POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "scroll",         do_scroll,      POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "spousetalk",     do_spouse,  POS_SLEEPING, 0, LOG_NORMAL, 1, 0, 1},
        { "stafftalk",  do_stafftalk,   POS_DEAD,       1, LOG_NORMAL,  0, 0, 1 },
	{ "style",          do_style,       POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
        { "screcall", do_subclanrecall, POS_FIGHTING, 0, LOG_NORMAL, 1, 0, 0 },
        { "schedule",   do_schedule,    POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "surrender",      do_surrender,POS_FIGHTING,   0,  LOG_NORMAL, 1, 0, 1 },
	{ "strangle",       do_strangle,POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "stance",     do_stance,  POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "spearforms",     do_spearforms,POS_FIGHTING,      0,  LOG_NORMAL, 1, 0, 0 },



        { "tag", do_tag, POS_RESTING, 0, LOG_NORMAL, 1, 0, 0},
	{ "take",           do_get,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "tap",            do_sacrifice,POS_RESTING,    0,  LOG_NORMAL, 0, 0, 1 },
	{ "tree",   do_tree,POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "treetalk",   do_treetalk,POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "treesing",       do_treesing,POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "trip",           do_trip,        POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "throw",          do_throw,       POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 1 },
	{ "turn",           do_turn,    POS_RESTING,     0,  LOG_NORMAL, 0, 0, 0 },
        { "toggle", do_toggle, POS_DEAD, 0, LOG_NORMAL, 1, 0, 1 },
        { "tour", do_tour, POS_STANDING, 0, LOG_NORMAL, 0, 0, 1 },
        { "touch", do_touch, POS_STANDING, 0, LOG_NORMAL, 0, 0, 1 },
        { "tourney", do_tourney, POS_DEAD, 0, LOG_NORMAL, 1, 0, 0 },
        { "topen", do_topen, POS_DEAD, 425, LOG_NORMAL, 1, 0, 0 },
        { "tclose", do_tclose, POS_DEAD, 425, LOG_NORMAL, 1, 0, 0 },
        { "tstop", do_tstop, POS_DEAD, 425, LOG_NORMAL, 1, 0, 0 },
        { "tchat", do_tchat, POS_DEAD, 425, LOG_NORMAL, 1, 0, 0 },
        { "tadd", do_tadd, POS_DEAD, 425, LOG_NORMAL, 1, 0, 0 },
        { "tremove", do_tremove, POS_DEAD, 425, LOG_NORMAL, 1, 0, 0 },
        { "teach",          do_teach,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "tell",           do_tell,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "track",          do_track,       POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "train",          do_train,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "time",           do_time,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "transfer",       do_transfer,POS_DEAD,      300,  LOG_ALWAYS, 1, 0, 0 },
        { "translate",       do_translate,   POS_DEAD,       0,   LOG_NORMAL, 1, 0, 0},
	{ "tornado",        do_tornado,     POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "treat",          do_treat,       POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "trivia",     do_trivia,   POS_DEAD,   1, LOG_NORMAL, 1, 1, 1 },
        { "typo",           do_typo,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },


        { "unbounty", do_unbounty,POS_STANDING, 0, LOG_NORMAL, 1, 0, 0 },
        { "unlock",     do_unlock,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "use",            do_use,         POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "unarm",      do_unarm,   POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "unalias",        do_unalias,     POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "unhook",         do_unhook,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "unjoin",     do_unjoin,      POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "unkeep",         do_unkeep,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "unread",         do_board,        POS_DEAD,   0, LOG_NORMAL, 1, 0, 0 },
	{ "unseal",         do_unseal,      POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },


	{ "value",          do_value,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "version",        do_version,     POS_DEAD,            0,  LOG_NORMAL, 1, 0, 0 },
	{ "void",       do_void,    POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
	{ "vault",          do_locker,      POS_STANDING,    0,  LOG_ALWAYS, 1, 0, 0 },
	{ "veil",       do_veil,    POS_RESTING,     0,   LOG_NORMAL, 1, 0, 1 },
        { "visible",        do_visible,     POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 1 },


        { "weather",        do_weather,     POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
        { "when",       do_when,    POS_RESTING, 0,  LOG_NORMAL, 1, 0, 0 },
        { "who",            do_who,         POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "wizlist",        do_wizlist,     POS_DEAD,    0,  LOG_NORMAL, 1, 0, 1 },
        { "worth",          do_worth,       POS_SLEEPING,0,  LOG_NORMAL, 1, 0, 0 },
	{ "whirlwind",  do_whirlwind,POS_FIGHTING,   0,  LOG_NORMAL, 1, 0, 0 },
        { "whisper",        do_tell,        POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "wield",          do_wear,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "wizhelp",        do_wizhelp,     POS_DEAD,            0,  LOG_NORMAL, 1, 0, 0 },
        { "wake",           do_wake,        POS_SLEEPING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "walkin",     do_walkin,  POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "walkout",    do_walkout, POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "where",          do_where,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },
        { "withdraw",   do_withdraw,POS_STANDING,    0,  LOG_NORMAL, 1, 0, 0 },
        { "weave",      do_weave,   POS_FIGHTING,    0,  LOG_NORMAL, 1, 0, 0 },
	{ "weaves",         do_wvlist,      POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
        { "war",                do_war,         POS_RESTING,     10, LOG_NORMAL, 10, 10, 0 },
	{ "wimpy",          do_wimpy,       POS_DEAD,        0,  LOG_NORMAL, 1, 0, 0 },
	{ "wear",           do_wear,        POS_RESTING,     0,  LOG_NORMAL, 1, 0, 1 },
	{ "write",          do_write,       POS_RESTING,     0,  LOG_NORMAL, 1, 0, 0 },



        { "x", do_x, POS_RESTING, 0, LOG_NORMAL, 10, 10, 0 },




	/***************************************************
	 * END of regular mortal commands                  *
	 *                                                 *
	 ***************************************************/




    /*
     * Immortal commands.
     */
    { "faire",	   do_faire, POS_DEAD, 1, LOG_NORMAL, 1, 0, 0 },
    { "penalize",	do_penalize,POS_DEAD,      1,    LOG_ALWAYS, 1, 0, 1 },

	/* Builder Group */
    /*
     * OLC and Builder Commands (editing) Not MobProgs
     */
    { "edit",		do_olc,		POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "asave",      do_asave,	POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "alist",		do_alist,	POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "resets",		do_resets,	POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "redit",		do_redit,	POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "medit",		do_medit,	POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "oedit",		do_oedit,	POS_DEAD, 251,  LOG_NORMAL, 1, 0, 1 },
    { "load",		do_load,	POS_DEAD,      251,  LOG_ALWAYS, 1, 0, 1 },
    { "purge",		do_purge,	POS_DEAD,      251,  LOG_ALWAYS, 1, 0, 1 },
    { "immtalk",	do_immtalk,	POS_DEAD,      251,  LOG_NEVER, 1, 0, 1},
    { "holylight",	do_holylight,POS_DEAD,    251,  LOG_NORMAL, 1, 0, 1 },
    { "goto",       do_new_goto,    POS_DEAD,      251,  LOG_ALWAYS, 1, 0, 1 },

    /* mentor group */
    { "sockets",    do_sockets,	POS_DEAD,      252,  LOG_NORMAL, 1, 0, 1 },
    { "whois",		do_whois,	POS_DEAD,   252,  LOG_NORMAL, 1, 0, 1 },
    { "verify",    do_verify, POS_DEAD, 252, LOG_ALWAYS, 1, 0, 1 },


    /* Interactive group */
    { "transfer",	do_transfer,POS_DEAD,      300,  LOG_ALWAYS, 1, 0, 1 },
    { "teleport",	do_transfer,POS_DEAD,      300,  LOG_ALWAYS, 1, 0, 1 },
    { "gocial",     do_gocial,  POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "echo",		do_recho,	POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "zecho",		do_zecho,	POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "pecho",		do_pecho,	POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "gmote",      do_gmote,   POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "restore",	do_restore,	POS_DEAD,      300,  LOG_ALWAYS, 1, 0, 1 },
    { "taint", 	do_taint,	POS_DEAD, 300, LOG_ALWAYS, 1, 0, 1 },
    { "shadowtaint", 	do_shadowtaint,	POS_DEAD, 300, LOG_ALWAYS, 1, 0, 1 },
    { "peace",      do_peace,   POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "prefi",		do_prefi,	POS_DEAD,      300,  LOG_NORMAL, 0, 0, 1 },
    { "prefix",		do_prefix,	POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "immortal",   do_immortal,POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "poofin",		do_bamfin,	POS_DEAD,      300,  LOG_NORMAL, 1, 0, 1 },
    { "poofout",	do_bamfout,	POS_DEAD,    300,  LOG_NORMAL, 1, 0, 1 },


	/* Advanced Builder Commands */
    { "hedit",		do_hedit,	POS_DEAD, 302,  LOG_NORMAL, 1, 0, 1 },
    { "aedit",		do_aedit,	POS_DEAD, 302,  LOG_NORMAL, 1, 0, 1 },
    { "qedit",		do_qedit,	POS_DEAD, 302,  LOG_NORMAL, 1, 0, 1 },
    { "objcheck",	do_objcheck,POS_DEAD, 302,  LOG_ALWAYS, 1, 0, 1 },

    

	/* MobProgrammer Commands */
    { "mpedit",		do_mpedit,	POS_DEAD, 305,  LOG_NORMAL, 1, 0, 1 },
    { "mpdump",		do_mpdump,	POS_DEAD, 305,  LOG_NEVER,  1, 0, 1 },
    { "mpstat",		do_mpstat,	POS_DEAD, 305,  LOG_NEVER,  1, 0, 1 },
    { "opedit",     do_opedit,  POS_DEAD, 305,  LOG_ALWAYS, 1, 0, 1 },
    { "rpedit",     do_rpedit,  POS_DEAD, 305,  LOG_ALWAYS, 1, 0, 1 },
    { "opdump",		do_opdump,	POS_DEAD, 305,  LOG_NEVER,  1, 0, 1 },
    { "opstat",		do_opstat,	POS_DEAD, 305,  LOG_NEVER,  1, 0, 1 },
    { "rpdump",		do_rpdump,	POS_DEAD, 305,  LOG_NEVER,  1, 0, 1 },
    { "rpstat",		do_rpstat,	POS_DEAD, 305,  LOG_NEVER,  1, 0, 1 },

    /* Guild Leader Group */
    { "guild",		do_guild,	POS_DEAD,      1,  LOG_ALWAYS, 1, 0, 1 },
    { "echo",		do_recho,	POS_DEAD,      400,  LOG_ALWAYS, 1, 0, 1 },
    { "zecho",		do_zecho,	POS_DEAD,      400,  LOG_ALWAYS, 1, 0, 1 },

    { "recruit",	do_recruit,	POS_DEAD,       1,  LOG_ALWAYS, 1, 0, 1 },

	/* Special Treat Group */
    { "spellup",	do_spellup, POS_DEAD,      425,  LOG_ALWAYS, 1, 0, 1 },
    { "bubble",		do_bubble,	POS_DEAD,      425,  LOG_ALWAYS, 1, 0, 1 },
    { "invade",     do_invade,  POS_DEAD,      425,  LOG_ALWAYS, 1, 0, 1 },
    { "qpgrant",	do_qpgrant,	POS_DEAD,      425,  LOG_ALWAYS, 1, 0, 1 },
    { "xpgrant",	do_xpgrant,	POS_DEAD,      425,  LOG_ALWAYS, 1, 0, 1 },
	{ "advance",	do_advance,	POS_DEAD,      425,  LOG_ALWAYS, 1, 0, 1 },
    { "fights",     do_fights,  POS_DEAD,      425,  LOG_NORMAL, 1, 0, 1 },
    { "leaderclear",do_leaderclear,POS_DEAD,   425,  LOG_ALWAYS, 1, 0, 1 },
    { "helpcheck",  do_helpcheck,POS_DEAD,     425,  LOG_NORMAL, 1, 0, 1 },
    { "feast",		do_feast,	POS_DEAD,	 425,  LOG_NORMAL, 1, 0, 1 },

	/* Moderator Group */
    { "goto",       do_new_goto,    POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 1 },
    { "stat",		do_stat,	POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 1 },
    { "freeze",	    do_freeze, POS_DEAD,       450, LOG_ALWAYS, 1, 0, 1 },
    { "sockets",    do_sockets,	POS_DEAD,      450,  LOG_NORMAL, 1, 0, 1 },
    { "rename",     do_rename,  POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 1 },
    { "deny",		do_deny,	POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 1 },
    { "permban",	do_permban,	POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 1 },
    { "disconnect",	do_disconnect,POS_DEAD,    450,  LOG_ALWAYS, 1, 0, 1 },
    { "penalty",	do_penalty,	POS_DEAD,      450,  LOG_ALWAYS, 1, 0, 1 },
    { "immtalk",	do_immtalk,	POS_DEAD,      450,  LOG_NORMAL, 1, 0, 1 },
    { "penalize",	do_penalize,POS_DEAD,     450,    LOG_ALWAYS, 1, 0, 1 },
    { "string",		do_string,	POS_DEAD,   450,  LOG_ALWAYS, 1, 0, 1 },
    { "log",		do_log,		POS_DEAD,   450,  LOG_ALWAYS, 1, 0, 1 },
    { "whois",		do_whois,	POS_DEAD,   450,  LOG_NORMAL, 1, 0, 1 },
    { "iwhere",     do_iwhere,  POS_DEAD,       450,  LOG_NORMAL, 1, 0, 1 },
    { "teleport",	do_transfer,POS_DEAD,   450,  LOG_ALWAYS, 1, 0, 1 },
    { "transfer",	do_transfer,POS_DEAD,   450,  LOG_ALWAYS, 1, 0, 1 },
    { "warn",	    do_warn,    POS_DEAD,   450,  LOG_ALWAYS, 1, 0, 1 },
    { "map",	    do_map,    POS_DEAD,   450,  LOG_NORMAL, 1, 0, 1 },


	/* priv group */
    { "log",		do_log,		POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "set",		do_set,		POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "force",		do_force,	POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "gecho",		do_echo,	POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "grab",		do_grab,	POS_RESTING,   475,  LOG_ALWAYS, 1, 0, 1 },
    { "violate",	do_violate,	POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "sla",		do_sla,		POS_DEAD,      475,  LOG_NORMAL, 0, 0, 1 },
    { "slay",       do_slay,    POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "flag",		do_flag,	POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "wiznet",		do_wiznet,	POS_DEAD,    475,  LOG_NORMAL, 1, 0, 1 },
    { "imotd",      do_imotd,   POS_DEAD,      475,  LOG_NORMAL, 1, 0, 1 },
    { "at",         do_at,      POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "mlevel",     do_mlevel,  POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "olevel",     do_olevel,  POS_DEAD,      475,  LOG_ALWAYS, 1, 0, 1 },
    { "vnum",		do_vnum,	POS_DEAD,     475,  LOG_ALWAYS, 1, 0, 1 },
    { "incognito",	do_incognito,POS_DEAD,     475,  LOG_NORMAL, 1, 0, 1 },
    { "rollop",     do_rollop,  POS_DEAD,      475,  LOG_NORMAL, 1, 0, 1 },
    { "reward",     do_reward, POS_DEAD,       475,  LOG_ALWAYS, 1, 0, 1 },
	/* Implementor group */
    { "newlock",	do_newlock,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
	{ "wizlock",	do_wizlock,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
    { "clone",		do_clone,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
    { "dump",		do_dump,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
    { "protect",	do_protect,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
    { "freev",      do_openvlist,POS_DEAD,     499,  LOG_ALWAYS, 1, 0, 1 },
    { "memory",		do_memory,	POS_DEAD,      499,  LOG_NORMAL, 1, 0, 1 },
    { "wrlist",     do_wrlist,  POS_DEAD,      499,  LOG_NORMAL, 1, 0, 1 },
    { "allow",		do_allow,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
    { "penalty",	do_penalty,	POS_DEAD,      499,  LOG_ALWAYS, 1, 0, 1 },
    { "invis",		do_invis,	POS_DEAD,   499,  LOG_NORMAL, 0, 0, 1 },
    { "wizinvis",	do_invis,	POS_DEAD,   499,  LOG_NORMAL, 1, 0, 1 },
    { "gedit",		do_gedit,	POS_DEAD, 302,  LOG_NORMAL, 1, 0, 1 },


	/* Coder Group */
    { "grant",      do_grant,   POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "gstat",      do_gstat,   POS_DEAD,      500,  LOG_NORMAL, 1, 0, 1 },
    { "revoke",     do_revoke,  POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "snapgear",     do_snapgear,  POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "snoop",		do_snoop,	POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "disable",	do_disable,	POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "copyover",	do_copyover,POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "reboo",		do_reboo,	POS_DEAD,      500,  LOG_NORMAL, 0, 0, 1 },
    { "reboot",		do_reboot,	POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "pload",      do_pload,   POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "punload",	do_punload, POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "shutdow",	do_shutdow,	POS_DEAD,      500,  LOG_NORMAL, 0, 0, 1 },
	{ "shutdown",	do_shutdown,POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "randroom",	do_randroom,POS_DEAD,      500,  LOG_ALWAYS, 1, 0, 1 },
    { "owhere",	   do_owhere, POS_STANDING, 500, LOG_ALWAYS, 1, 0, 1 },
    { "weavehalt",  do_weavehalt,POS_STANDING, 500, LOG_ALWAYS, 1, 0, 1 },
    { "web",        do_web,      POS_STANDING, 500, LOG_ALWAYS, 1, 0, 1 },
    { "shell",        do_shell,      POS_STANDING, 1000, LOG_ALWAYS, 1, 0, 1 },
    { "trainerdump",  do_trainerdump, POS_STANDING, 500, LOG_ALWAYS, 1, 0, 1},
    	/* Stuff that conflicts with immortal commands */
    { "attack",		do_kill,	POS_FIGHTING,	 0,  LOG_NORMAL, 0, 0, 1 },  /* moved below do_at */
    { "",		0,		POS_DEAD,	 0,  LOG_NORMAL, 0, 0, 1 }

};



bool get_portal_here( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *obj, *obj_next;
	for ( obj = ch->in_room->contents; obj != NULL; obj = obj_next )
	{
		obj_next = obj->next_content;
		if ( obj->item_type == ITEM_PORTAL
		&& obj->value[4] == 1
		&& !str_cmp( argument, obj->name ) )
		{
			return TRUE;
		}
	}
	return FALSE;
}

void nf_strings( CHAR_DATA *ch )
{
	int counter;
	char * strings[] = {
		"{wI could not find that command.{x",
		"{wWant to try that again?{x",
		"{wDid you mean to actually do something?{x",
		"{wThat looks like a typo to me.{x",
		"{wI don't think you could spell any worse if you tried.{x",
		"{wBug report entered: User can't spell.{x",
		"{wOne more typo and we're deleting your pfile... Just kidding!{x",
		"{wSecurity alert! Hack attempt! Deleting your pfile... Just kidding!{x",
		"{wTry using only one finger and maybe you will get it right this time.{x",
		"{wTry going to http://www.dictionary.com and learn to spell.{x",
		"{wI dont understand you. We're through. That's it. I can't take it anymore.{x",
		"{wThe MUD has CRASHED!!! haha! Fooled you. Now quit typo'ing.{x",
		"{wTypo opyT!{x",
		"{wAren't you tired of not knowing how to spell?{x",
		"{wI've tried and tried but I just can't find that command.{x",
		"{wError! Error! Could not find that command.{x",
		"{wA darkhound eats your typo before it can cause any damage.{x",
		"{wThe Immortals feed those who typo much to the trollocs.{x",
		"{wBad, bad player!{x",
		"{wBy gorge, u shore do tipe gud!{x",
		"{wNo! I refuse to do that.{x",
		"{wThat would be against the rules.{x",
		"{wI could not find that command.{x",
		"{wQuit typing with your toes!{x",
		"{wQuit typing with your nose!{x",
		"{wYou learn from your mistakes and your skill at typing improves!{x",
		"{wYou must be trying to type in the Old Tongue!{x",
		"{wPEBKAC: Problem exists between keyboard and chair.{x",
		"{wPlease stop typing with your eyes closed!{x",
		"{wAre the keys sticking on your keyboard?{x",
		"{wYour Amish neighbors called. They want their abacus back.{x",
		"{wMaybe you should upgrade to an Etch-a-Sketch.{x",
		"{wIs that a LeapPad you're using?{x",
		"{wPerhaps you should upgrade your keyboard.{x",
		"{wWhat language are you typing in?{x",
		"{wHere, come with me. I'd like you to try this nice white jacket on.{x",
		"{wOkay, now I think we're both confused.{x",
		"{wThat was wrong on so many levels.{x",
		"{wInsanity is completely normal, and you are absolutely normal.{x",
		"{wOf all the things you've lost, do you miss your mind the most?{x",
		"{wWhy in the world would you want me to do that?{x",
		"{wHahaha! You're such a kidder!{x",
		"{wAre you just trying to be mean?{x",
		"{wIs that even English?{x",
		"{wI know, I know, just wait! I'm still trying to find that command!{x",
		"{wWoah! What was that?{x",
		"{wWhat is it you're trying to accomplish here?{x",
		"{wPerhaps you should upgrade your typing fingers.{x",
		"{wIf there were a tax on typos, you'd be broke.{x",
	};
	counter = number_range(1, sizeof(strings) / sizeof(char *) );
	ptc( ch, "%s\n\r", strings[counter-1] );
}
/*
 * The main entry point for executing commands.
 * Can be recursively called from 'at', 'order', 'force'.
 */
void interpret( CHAR_DATA *ch, char *argument )
{
    char command[MAX_INPUT_LENGTH];
    char logline[MAX_INPUT_LENGTH];
    int cmd;
    int trust;
    bool found = FALSE, use_newbie_table = FALSE;
    int string_count = nAllocString ;
    int perm_count = nAllocPerm ;
    char cmd_copy[MAX_INPUT_LENGTH] ;
    char buf[MAX_STRING_LENGTH] ;

    /*
     * Strip leading spaces.
     */
    while ( isspace(*argument) )
	argument++;
    if ( argument[0] == '\0' )
	return;

	/* This is a test to try and keep LD people from issuing commands
	 * and bypassing the handler to log them out. - Dalsor 07-20-02
	 */
//	if ( ch->desc == NULL && !IS_NPC( ch ) )
//		return;
    /*
     * No hiding.
     */
    REMOVE_BIT( ch->affected_by, AFF_HIDE );

    /*
     * Implement freeze command.
     */
    if ( !IS_NPC(ch) && IS_SET(ch->act, PLR_FREEZE) &&strcmp(argument,"quit") && strcmp(argument,"save") )
    {
		stc( "{cYou can't move!\n\r", ch );
		return;
    }

	/* This is to keep AFK people from being stupid and walking around
	 * killing and questing -- morons. - Dalsor 06-16-03
	 */
	if ( ch->desc != NULL
	&& !IS_NPC( ch )
	&& IS_SET( ch->comm, COMM_AFK )
	&& str_prefix( argument, "afk" )
	&& strcmp(argument,"quit")
	&& strcmp(argument,"save")
	&& !IS_IMMORTAL(ch) )
	{
		stc("But you're supposed to be AFK right now!\n\rType AFK if you are ready to return to the keyboard.\n\r", ch);
		return;
	}

    /* For Hold weaves */
    if ( !IS_NPC(ch) && IS_SET(ch->war_flags, WAR_HELD) )
    {
		if (saves_spell(ch->level - (ch->level / 10), ch, DAM_ENERGY))
	    {
			stc( "You manage to break free!\n\r", ch );
			act("$n manages to escape from $s bonds!",ch,NULL,NULL,TO_ROOM);
			REMOVE_BIT(ch->war_flags,WAR_HELD);
	    }
	    else
	    {
			if (!IS_NPC(ch))
			stc( "{cBut you can't move!\n\r", ch );
			WAIT_STATE(ch, PULSE_VIOLENCE * 2);
			return;
		}
    }

        if (!str_cmp(argument, "`"))
        {
                stc("You don't have a command stack to clear.\n\r", ch);
                return;
        }

    /*
     * Grab the command word.
     * Special parsing so ' can be a command,
     *   also no spaces needed after punctuation.
     */
    strcpy( cmd_copy, argument);
    strcpy( logline, argument );

    /*
    if ( ch->pcdata != NULL
    && IS_SET( ch->comm, COMM_RP )
    && ch->pcdata->last_command != NULL
    && ch->pcdata->last_command[0] != '\0'
    && !str_cmp( argument, ch->pcdata->last_command ) )
    {
	    stc("RP Spam!\n\r", ch);
	    return;
    }*/

    if ( ch->pcdata )
    {
	free_string(ch->pcdata->last_command);
	ch->pcdata->last_command = str_dup( argument );
    }

    sprintf ( last_command, "%s in room[%ld]: %s.", ch->name,
              ch->in_room->vnum, argument );
    if ( ch->desc != NULL )
    {
        sprintf ( last_command2, "%s", argument );
        last_descriptor = ch->desc->descriptor;
    }
    else
    {
        last_command2[0] = '\0';
    }

    if ( !isalpha(argument[0]) && !isdigit(argument[0]) )
    {
		command[0] = argument[0];
		command[1] = '\0';
		argument++;
		while ( isspace(*argument) )
		    argument++;
    }
    else
    {
		argument = one_argument( argument, command );
    }

    /*
     * Look for command in command table.
     */
	found = FALSE;
	trust = get_trust( ch );

	/* Parse newbie commands
	 */
	if ( use_newbie_table == TRUE )
	{
		for ( cmd = 0; newb_cmd_table[cmd].name[0] != '\0'; cmd++ )
		{
			if ( command[0] == newb_cmd_table[cmd].name[0]
			&&   !str_prefix( command, newb_cmd_table[cmd].name ))
			{
				if (newb_cmd_table[cmd].level<LEVEL_IMMORTAL)
				{
					if (newb_cmd_table[cmd].level > trust)
						continue;
				}
				else
				if (ch->desc == NULL
				|| !is_granted(ch,newb_cmd_table[cmd].do_fun))
					continue;
				found = TRUE;
	            break;
			}
		}

	    if ( newb_cmd_table[cmd].log == LOG_NEVER )
		strcpy( logline, "" );

		if ( ( !IS_NPC(ch) && IS_SET(ch->act, PLR_LOG) )
		||   fLogAll
		||   newb_cmd_table[cmd].log == LOG_ALWAYS )
		{
		    char    s[2*MAX_INPUT_LENGTH],*ps;
		    int     i;

		    ps=s;
		    sprintf( log_buf, "Log %s: %s", ch->name, logline );
		    for (i=0;log_buf[i];i++) {
			*ps++=log_buf[i];
			if (log_buf[i]=='$')
			    *ps++='$';
			if (log_buf[i]=='{')
			    *ps++='{';
		    }
		    *ps=0;
			wiznet(s,ch,NULL,WIZ_SECURE,0,get_trust(ch));
			log_string( log_buf );
		}
	    if ( ch->desc != NULL && ch->desc->snoop_by != NULL )
	    {
			write_to_buffer( ch->desc->snoop_by, "% ",    2 );
			write_to_buffer( ch->desc->snoop_by, logline, 0 );
			write_to_buffer( ch->desc->snoop_by, "\n\r",  2 );
	    }

	    if ( !found )
	    {
			if ( !check_social( ch, command, argument ) )
			    stc( "I could not find that command.\n\rCheck your spelling and check the help files.\n\r", ch );
			return;
	    }
	    /*
	     * Character not in position for command?
	     */
	    if ( ch->position < newb_cmd_table[cmd].position )
	    {
			switch( ch->position )
			{
				case POS_DEAD:
			    stc( "You are feeling too dead to do that!\n\r", ch );
			    /*act( "A haunting moan fills the air.", ch, NULL, NULL, TO_ROOM );*/
			    break;

				case POS_MORTAL:
				case POS_INCAP:
			    stc( "You are too badly hurt, and can't find the strength for that.\n\r", ch );
			    /*act( "A soft moan escapes from $n's lips.", ch, NULL, NULL, TO_ROOM );*/
			    break;

				case POS_STUNNED:
			    stc( "You are stunned and can't seem to shake the cobwebs.\n\r", ch );
			    /*act( "A soft moan escapes from $n's lips.", ch, NULL, NULL, TO_ROOM );*/
			    break;

				case POS_SLEEPING:
			    stc( "You must wake up before you can do that.\n\r", ch );
			    /*act( "A soft moan escapes from $n's lips.", ch, NULL, NULL, TO_ROOM );*/
			    break;

				case POS_RESTING:
			    stc( "You are resting and can't do that while sitting down.\n\r", ch);
			    /*act( "$n squirms in $s seat.", ch, NULL, NULL, TO_ROOM );*/
			    break;

				case POS_SITTING:
			    stc( "You are sitting down and must stand up before you can do that.\n\r",ch);
			    /*act( "$n squirms in $s seat.", ch, NULL, NULL, TO_ROOM );*/
			    break;

				case POS_FIGHTING:
			    stc( "You are too close to your enemy right now!\n\r", ch);
			    break;
			}
			return;
	    }

	    (*newb_cmd_table[cmd].do_fun) ( ch, argument );


	    if (string_count < nAllocString)
	    {
		    sprintf(buf,"Memcheck : Increase in strings :: %s : %s", ch->name, cmd_copy) ;
		    wiznet(buf, NULL, NULL, WIZ_MEMCHECK,0,0) ;
	    }

	    if (perm_count < nAllocPerm)
	    {
		    sprintf(buf,"Increase in perms :: %s : %s", ch->name, cmd_copy) ;
		    wiznet(buf, NULL, NULL, WIZ_MEMCHECK, 0,0) ;
	    }
	    tail_chain();
	    return;
    }

	/* Parse all other commands
	 */
	else
	{
		for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
		{
			if ( command[0] == cmd_table[cmd].name[0]
			&&   !str_prefix( command, cmd_table[cmd].name ))
			{
				if (cmd_table[cmd].level>20)
				{
					if (cmd_table[cmd].level > trust)
						continue;
				}
				else
				if (ch->desc == NULL
				|| !is_granted(ch,cmd_table[cmd].do_fun))
					continue;
				found = TRUE;
	            break;
			}
		}
	    /*if ( !IS_NPC( ch ) && cmd_table[cmd].hours > 0 )
	    {
			played = ( ch->played + (int) (current_time - ch->logon) ) / 3600;
			if ( played < cmd_table[cmd].hours && ch->pcdata->rebirth <= 0 )
		    {
				stc( "You have not played long enough to use this command!\n\r", ch );
				if (cmd_table[cmd].hours==1)
					stc( "You must have played 1 hour.\n\r",ch );
				else
					ptc( ch, "You must have played %d hours.\n\r", cmd_table[cmd].hours );
				return;
		    }
		}*/
/*
	    if ( is_fighting( ch ) && cmd_table[cmd].unarmed != 1 && !weaponless( ch ) )
	    {
			stc( "You need to remove your weapons to do this!\n\r", ch );
			return;
		}
*/
    }

    /*
     * Log and snoop.
     */
    if ( cmd_table[cmd].log == LOG_NEVER && !IS_SET(ch->act, PLR_LOG ))
	strcpy( logline, "" );

	if ( ( !IS_NPC(ch) && IS_SET(ch->act, PLR_LOG) )
	||   fLogAll
	||   cmd_table[cmd].log == LOG_ALWAYS )
	{
	    char    s[2*MAX_INPUT_LENGTH],*ps;
	    int     i;

	    ps=s;
	    sprintf( log_buf, "Log %s: %s", ch->name, logline );
	    /* Make sure that was is displayed is what is typed */
	    for (i=0;log_buf[i];i++)
	    {
		*ps++=log_buf[i];
		if (log_buf[i]=='$')
		    *ps++='$';
		if (log_buf[i]=='{')
		    *ps++='{';
	    }
	    *ps=0;
		wiznet(s,ch,NULL,WIZ_SECURE,0,get_trust(ch));
		log_string( log_buf );
	}

/*
    if ( ( !IS_NPC(ch) && IS_SET(ch->act, PLR_LOG) )
    ||   fLogAll
    ||   cmd_table[cmd].log == LOG_ALWAYS )
    {
	sprintf( log_buf, "Log %s: %s", ch->name, logline );
	wiznet(log_buf,ch,NULL,WIZ_SECURE,0,get_trust(ch));
	log_string( log_buf );
    }
*/
    if ( ch->desc != NULL && ch->desc->snoop_by != NULL )
    {
		write_to_buffer( ch->desc->snoop_by, "% ",    2 );
		write_to_buffer( ch->desc->snoop_by, logline, 0 );
		write_to_buffer( ch->desc->snoop_by, "\n\r",  2 );
    }

    if ( !found )
    {
		if ( !check_social( ch, command, argument ) )
		{
			if ( !IS_NPC( ch ) )
			{
				int is_sub = 0;
			    if ((is_sub = is_guild_channel(ch, command)))
			    {
					clan_talk( ch, argument, is_sub );
					return;
			    }
			    else if ( (is_sub = is_multi_guild_channel(ch, command )) )
			    {
			    		guild_talk( ch, argument, is_sub);
					return;
			    }
			    else if ( get_portal_here( ch, command ) )
			    {
					do_function( ch, &do_enter, command );
					return;
				}
			}
			nf_strings( ch );
    		stc( "{wCheck your spelling and check the help files.{x\n\r", ch );
			return;
		}
		return;
    }
 	else
 	{
        if (check_disabled( &cmd_table[cmd] ) )
        {
            send_to_char("This command has been temporarily disabled.\n\r",ch);
            return;
        }
	}
    /*
     * Character not in position for command?
     */
    if ( ch->position < cmd_table[cmd].position )
    {
		switch( ch->position )
		{
			case POS_DEAD:
		    stc( "You are feeling too dead to do that!\n\r", ch );
		    break;

			case POS_MORTAL:
			case POS_INCAP:
		    stc( "You are too badly hurt, and can't find the strength for that.\n\r", ch );
		    break;

			case POS_STUNNED:
		    stc( "You are stunned and can't seem to shake the cobwebs.\n\r", ch );
		    break;

			case POS_SLEEPING:
		    stc( "You must wake up before you can do that.\n\r", ch );
		    break;

			case POS_RESTING:
		    stc( "You are resting and can't do that while sitting down.\n\r", ch);
		    break;

			case POS_SITTING:
		    stc( "You are sitting down and must stand up before you can do that.\n\r",ch);
		    break;

			case POS_FIGHTING:
		    stc( "You are too close to your enemy right now!\n\r", ch);
		    break;
		}
		return;
    }

    /* Record the command


    if ( !str_cmp( cmd_table[cmd].name, "emote" )
	|| !str_cmp( cmd_table[cmd].name, "smote" )
	|| !str_cmp( cmd_table[cmd].name, "say" ) )
	    rplog_command( ch, last_command );

    sprintf (last_command, "[%5d] %s in [%5d] %s: %s",
        IS_NPC(ch) ? ch->pIndexData->vnum : 0,
        IS_NPC(ch) ? ch->short_descr : ch->name,
        ch->in_room ? ch->in_room->vnum : 0,
        ch->in_room ? ch->in_room->name : "(not in a room)",
        logline);*/
    /*
     * Dispatch the command.
     */
    (*cmd_table[cmd].do_fun) ( ch, argument );

    /* Record that the command was the last done, but it is finished
    sprintf (last_command, "(Finished) [%5d] %s in [%5d] %s: %s",
        IS_NPC(ch) ? ch->pIndexData->vnum : 0,
        IS_NPC(ch) ? ch->short_descr : ch->name,
        ch->in_room ? ch->in_room->vnum : 0,
        ch->in_room ? ch->in_room->name : "(not in a room)",
        logline);*/

    if (string_count < nAllocString)
    {
	    sprintf(buf,"Memcheck : Increase in strings :: %s : %s", ch->name, cmd_copy) ;
	    wiznet(buf, NULL, NULL, WIZ_MEMCHECK,0,0) ;
    }

    if (perm_count < nAllocPerm)
    {
	    sprintf(buf,"Increase in perms :: %s : %s", ch->name, cmd_copy) ;
	    wiznet(buf, NULL, NULL, WIZ_MEMCHECK, 0,0) ;
    }

    tail_chain( );
    return;
}

/* function to keep argument safe in all commands -- no static strings */
void do_function (CHAR_DATA *ch, DO_FUN *do_fun, char *argument)
{
    char *command_string;

    /* copy the string */
    command_string = str_dup(argument);

    /* dispatch the command */
    (*do_fun) (ch, command_string);

    /* free the string */
    free_string(command_string);
}

bool check_social( CHAR_DATA *ch, char *command, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int cmd;
    bool found;

    found  = FALSE;
    for ( cmd = 0; social_table[cmd].name[0] != '\0'; cmd++ )
    {
	if ( command[0] == social_table[cmd].name[0]
	&&   !str_prefix( command, social_table[cmd].name ) )
	{
	    found = TRUE;
	    break;
	}
    }

    if ( !found )
	return FALSE;

    if ( !IS_NPC(ch) && IS_SET(ch->penalty_flags, PENAL_NOEMOTE) )
    {
	stc( "You are currently not allowing social commands.\n\r", ch );
	return TRUE;
    }

    switch ( ch->position )
    {
    case POS_DEAD:
	    stc( "You are feeling too dead to do that!\n\r", ch );
	    act( "A haunting moan fills the air.", ch, NULL, NULL, TO_ROOM );
		return TRUE;

    case POS_INCAP:
    case POS_MORTAL:
	    stc( "You are too badly hurt, and can't find the strength for that.\n\r", ch );
	    act( "A soft moan escapes from $n's lips.", ch, NULL, NULL, TO_ROOM );
		return TRUE;

    case POS_STUNNED:
	    stc( "You are stunned and can't seem to shake the cobwebs.\n\r", ch );
	    act( "A soft moan escapes from $n's lips.", ch, NULL, NULL, TO_ROOM );
		return TRUE;

    case POS_SLEEPING:
	/*
	 * I just know this is the path to a 12" 'if' statement.  :(
	 * But two players asked for it already!  -- Furey
	 */
	if ( !str_cmp( social_table[cmd].name, "snore" ) )
	    break;
	stc( "You must wake up before you can do that.\n\r", ch );
	act( "A soft moan escapes from $n's lips.", ch, NULL, NULL, TO_ROOM );
	return TRUE;

    }

    one_argument( argument, arg );
    victim = NULL;
    if ( arg[0] == '\0' )
    {
	act( social_table[cmd].others_no_arg, ch, NULL, victim, TO_ROOM    );
	act( social_table[cmd].char_no_arg,   ch, NULL, victim, TO_CHAR    );
    }
    else if ( ( victim = get_char_room( ch,NULL, arg ) ) == NULL )
    {
		stc( "You dont see that person here.\n\r", ch );
    }
    else if ( victim == ch )
    {
	act( social_table[cmd].others_auto,   ch, NULL, victim, TO_ROOM    );
	act( social_table[cmd].char_auto,     ch, NULL, victim, TO_CHAR    );
    }
    else
    {
	act( social_table[cmd].others_found,  ch, NULL, victim, TO_NOTVICT );
	act( social_table[cmd].char_found,    ch, NULL, victim, TO_CHAR    );
	act( social_table[cmd].vict_found,    ch, NULL, victim, TO_VICT    );

	if ( !IS_NPC(ch) && IS_NPC(victim)
	&&   !IS_AFFECTED(victim, AFF_CHARM)
	&&   IS_AWAKE(victim)
	&&   victim->desc == NULL)
	{

	   switch ( number_bits( 4 ) )
	    {
	    case 0:

	    case 1: case 2: case 3: case 4:
	    case 5: case 6: case 7: case 8:
		act( social_table[cmd].others_found,
		    victim, NULL, ch, TO_NOTVICT );
		act( social_table[cmd].char_found,
		    victim, NULL, ch, TO_CHAR    );
		act( social_table[cmd].vict_found,
		    victim, NULL, ch, TO_VICT    );
		break;

	    /*case 9: case 10: case 11: case 12:
		act( "{c$n slaps $N.",  victim, NULL, ch, TO_NOTVICT );
		act( "{cYou slap $N.",  victim, NULL, ch, TO_CHAR    );
		act( "{c$n slaps you.", victim, NULL, ch, TO_VICT    );
		break;*/
	    }
	}
    }

    return TRUE;
}



/*
 * Return true if an argument is completely numeric.
 */
bool is_number ( char *arg )
{

    if ( *arg == '\0' )
        return FALSE;

    if ( *arg == '+' || *arg == '-' )
        arg++;

    for ( ; *arg != '\0'; arg++ )
    {
        if ( !isdigit( *arg ) )
            return FALSE;
    }

    return TRUE;
}



/*
 * Given a string like 14.foo, return 14 and 'foo'
 */
int number_argument( char *argument, char *arg )
{
    char *pdot;
    int number;

    for ( pdot = argument; *pdot != '\0'; pdot++ )
    {
	if ( *pdot == '.' )
	{
	    *pdot = '\0';
	    number = atoi( argument );
	    *pdot = '.';
	    strcpy( arg, pdot+1 );
	    return number;
	}
    }

    strcpy( arg, argument );
    return 1;
}

/*
 * Given a string like 14*foo, return 14 and 'foo'
*/
int mult_argument(char *argument, char *arg)
{
    char *pdot;
    int number;

    for ( pdot = argument; *pdot != '\0'; pdot++ )
    {
        if ( *pdot == '*' )
        {
            *pdot = '\0';
            number = atoi( argument );
            *pdot = '*';
            strcpy( arg, pdot+1 );
            return number;
        }
    }

    strcpy( arg, argument );
    return 1;
}



/*
 * Pick off one argument from a string and return the rest.
 * Understands quotes.
 */
char *one_argument( char *argument, char *arg_first )
{
    char cEnd;

    while ( isspace(*argument) )
	argument++;

    cEnd = ' ';
    if ( *argument == '\'' || *argument == '"' )
	cEnd = *argument++;

    while ( *argument != '\0' )
    {
	if ( *argument == cEnd )
	{
	    argument++;
	    break;
	}
	*arg_first = LOWER(*argument);
	arg_first++;
	argument++;
    }
    *arg_first = '\0';

    while ( isspace(*argument) )
	argument++;

    return argument;
}

/*
 * Pick off one argument from a string and return the rest.
 * Understands quotes.
 * Only seperates on commas, not on spaces
 */
char *one_comma_argument( char *argument, char *arg_first )
{
    char cEnd;

    while ( isspace(*argument) )
	argument++;

    cEnd = ',';
    if ( *argument == '\'' || *argument == '"' )
	cEnd = *argument++;

    while ( *argument != '\0' )
    {
	if ( *argument == cEnd )
	{
	    argument++;
	    break;
	}
	*arg_first = LOWER(*argument);
	arg_first++;
	argument++;
    }
    *arg_first = '\0';

    while ( isspace(*argument) )
	argument++;

    return argument;
}


int CmdCount(const void *v1, const void *v2)
{
    int idx1 = *(int *) v1;
    int idx2 = *(int *) v2;
    int i = 0;

    /* compare the command names */
    for (i = 0; cmd_table[idx1].name[i] != '\0'; i++)
    {
        if (cmd_table[idx1].name[i] == cmd_table[idx2].name[i]) continue;
        if (cmd_table[idx2].name[i] == '\0')                    return  1;
        if (cmd_table[idx1].name[i]  > cmd_table[idx2].name[i]) return  1;
        if (cmd_table[idx1].name[i]  < cmd_table[idx2].name[i]) return -1;
    }
    return 0;
}


void do_commands( CHAR_DATA *ch, char *argument )
{

    int  index[MIL*4];
    int  cmd;
    int  i,count = 0, line = 0;

    for ( count = 0; cmd_table[count].name[0] != '\0'; count++ )
        index[count]=count;

    qsort( index, count, sizeof(int), CmdCount);

    for ( i = 0; i < count ; i++ )
    {
        cmd = index[i];

		if ( cmd_table[cmd].level < LEVEL_HERO
		&&   cmd_table[cmd].show )
        {
			line++;
			printf_to_char( ch, "%-12s ", cmd_table[cmd].name );
			if ( line == 6 )
	        {
			    send_to_char( "\n\r", ch );
			    line = 0;
			}
        }
    }
    send_to_char( "\n\r", ch );
    return;
}
/*
void do_commands( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int cmd;
    int col;

    col = 0;
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
        if ( cmd_table[cmd].level <  LEVEL_HERO
        &&   cmd_table[cmd].level <= get_trust( ch )
	&&   cmd_table[cmd].show)
	{
	    sprintf( buf, "%-12s", cmd_table[cmd].name );
	    stc( buf, ch );
	    if ( ++col % 6 == 0 )
		stc( "\n\r", ch );
	}
    }

    if ( col % 6 != 0 )
	stc( "\n\r", ch );
    return;
}
*/

void do_wizhelp(CHAR_DATA * ch, char *argument)
{
    int  index[MIL*4];
    int  cmd;
    int  i,count = 0, line = 0;

    for ( count = 0; cmd_table[count].name[0] != '\0'; count++ )
        index[count]=count;

    qsort( index, count, sizeof(int), CmdCount);

    for ( i = 0; i < count ; i++ )
    {
        cmd = index[i];

		if ( cmd_table[cmd].level >= LEVEL_HERO
	    && is_granted(ch,cmd_table[cmd].do_fun) )
        {
			line++;
			printf_to_char( ch, "%-12s ", cmd_table[cmd].name );
			if ( line == 6 )
	        {
			    send_to_char( "\n\r", ch );
			    line = 0;
			}
        }
    }
    send_to_char( "\n\r", ch );
    return;
}
/*
void do_wizhelp( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int cmd;
    int col;

    col = 0;
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
        if ( cmd_table[cmd].level >= LEVEL_HERO
        &&   is_granted(ch,cmd_table[cmd].do_fun)
        &&   cmd_table[cmd].show)
	{
	    sprintf( buf, "%-12s", cmd_table[cmd].name );
	    stc( buf, ch );
	    if ( ++col % 6 == 0 )
		stc( "\n\r", ch );
	}
    }

    if ( col % 6 != 0 )
	stc( "\n\r", ch );
    return;
}
*/
