/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/***************************************************************************
*     This file started for Sundered Empires by Thale (Andrew Maslin).     *
*       Thale (Andrew Maslin - amaslin@hotmail.com) *
***************************************************************************/


#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"

/*
 * Local functions.
 */
int   choose_town();
int   get_invader_num(int mob, int race);
int   get_invader_by_vnum(int mob);
bool  check_room_invade(int room);
int   find_invade_room(int town);


/*
 * This function is simply to select a town to be invaded that has
 * at least 1 group that can attack it.
 */
int choose_town()
{
  int x, d;


  for (d = 0; d <= 100; d++)
  {
    x = number_range(1, MAX_TOWN_INVADE) - 1;
    if ((town_invade_table[x].enemy1 != NULL)
     || (town_invade_table[x].enemy2 != NULL)
     || (town_invade_table[x].enemy3 != NULL)
     || (town_invade_table[x].enemy4 != NULL)
     || (town_invade_table[x].enemy5 != NULL))
        return x;
  }

  return -1;
}


/*
 * This function determines the number of mobs in a race's table.
 * Used in determining the maximum different types of mobs for randomly
 * selecting mobs to send on invasions.
 */
int racei_amount(int race)
{
  int x, amount = 0;


  for (x = 0; invader_table[x].group != NULL; x++)
  {
    if (race == racei_lookup(invader_table[x].group))
      amount++;
  }

  return amount;
}



/*
 * This is the main function called by update.c to start the
 * invasion process.
 */
void check_invade()
{
  int town, d, chance;
  int race, min, max, temp;
  int force, amount;


  town = choose_town();
  if (town == -1)
  {
    bug("In check_invade, town chosen = %d and is incorrect.",town);
    return;
  }

  for (d = 0; d <= 100; d++)
  {
    chance = number_range(1,5);
    switch (chance)
    {
      case 1:
        if (town_invade_table[town].enemy1 != NULL)
        {
          race = racei_lookup(town_invade_table[town].enemy1);
          min = race_invade_table[race].min;
          max = race_invade_table[race].max;
          if (min > max)
          {
            temp = min;
            min = max;
            max = temp;
          }
          force = number_range(min,max);
          amount = racei_amount(race);

          if (!invade(town,race,force,amount))
            bug("Invasion failed.",0);

          return;
        }
        else
          break;

      case 2:
        if (town_invade_table[town].enemy2 != NULL)
        {
          race = racei_lookup(town_invade_table[town].enemy2);
          min = race_invade_table[race].min;
          max = race_invade_table[race].max;
          if (min > max)
          {
            temp = min;
            min = max;
            max = temp;
          }
          force = number_range(min,max);
          amount = racei_amount(race);

          if (!invade(town,race,force,amount))
            bug("Invasion failed.",0);

          return;
        }
        else
          break;

      case 3:
        if (town_invade_table[town].enemy3 != NULL)
        {
          race = racei_lookup(town_invade_table[town].enemy3);
          min = race_invade_table[race].min;
          max = race_invade_table[race].max;
          if (min > max)
          {
            temp = min;
            min = max;
            max = temp;
          }
          force = number_range(min,max);
          amount = racei_amount(race);

          if (!invade(town,race,force,amount))
            bug("Invasion failed.",0);

          return;
        }
        else
          break;
      case 4:
        if (town_invade_table[town].enemy4 != NULL)
        {
          race = racei_lookup(town_invade_table[town].enemy4);
          min = race_invade_table[race].min;
          max = race_invade_table[race].max;
          if (min > max)
          {
            temp = min;
            min = max;
            max = temp;
          }
          force = number_range(min,max);
          amount = racei_amount(race);

          if (!invade(town,race,force,amount))
            bug("Invasion failed.",0);

          return;
        }
      case 5:
        if (town_invade_table[town].enemy5 != NULL)
        {
          race = racei_lookup(town_invade_table[town].enemy5);
          min = race_invade_table[race].min;
          max = race_invade_table[race].max;
          if (min > max)
          {
            temp = min;
            min = max;
            max = temp;
          }
          force = number_range(min,max);
          amount = racei_amount(race);

          if (!invade(town,race,force,amount))
            bug("Invasion failed.",0);

          return;
        }
    }
  }
		invasion->running = TRUE;
		free_string( invasion->location );
                invasion->location = str_dup( town_invade_table[town].name );
  return;
}



/*
 * This does the actual invasion once the victim town, the race of
 * the invaders, and the number of invaders involved have been
 * determined.
 */
bool invade(int town, int race, int force, int amount)
{
  DESCRIPTOR_DATA *d;
  int t, x, y, mob_num, temp;
  INVADE_VNUMS invading[amount];
  bool need_fix = FALSE, invaded = FALSE;
  MOB_INDEX_DATA *pMobIndex;
  CHAR_DATA *vch, *vch_next, *victim = NULL;
  ROOM_INDEX_DATA *location;
  char log_buf[MAX_STRING_LENGTH];


  for (y = 0; y < amount; y++)
  {
    invading[y].name = NULL;
    invading[y].vnum = 0;
    invading[y].total = 0;
  }
  for (x = 0; x < force; x++)
  {
    bool entered = FALSE;

    mob_num = number_range(1,amount); /* # mob of this race, such as 4th */
    temp = get_invader_num(mob_num, race); /* table position of this mob */
    for (y = 0; y < amount; y++)
    {
      if (invading[y].vnum != invader_table[temp].vnum)
	continue;
      invading[y].name = str_dup(invader_table[temp].group);
      invading[y].vnum = invader_table[temp].vnum;
      invading[y].total++;
      entered = TRUE;
    }
    if (!entered)
    {
      for (y = 0; y < amount; y++)
      {
	if (invading[y].name == NULL)
	{
	  invading[y].name = str_dup(invader_table[temp].group);
	  invading[y].vnum = invader_table[temp].vnum;
	  invading[y].total++;
	  break;
	}
      }
    }
  }

  for (x = 0; x < amount; x++)
  {
    temp = get_invader_by_vnum(invading[x].vnum);
    if (invading[x].name != NULL
     && invading[x].total > invader_table[temp].max)
      need_fix = TRUE;
  }

  if (need_fix)
  {
    for (x = 0; x < amount; x++)
    {
      temp = get_invader_by_vnum(invading[x].vnum);
      if (invading[x].total > invader_table[temp].max)
      {
	for (t = 0; t < 1000; t++)
	{
	  mob_num = number_range(1,amount);
	  temp = get_invader_num(mob_num, race);
	  for (y = 0; y < amount; y++)
	  {
	    if ((invading[y].vnum == invader_table[temp].vnum)
	     && (invading[y].total < invader_table[temp].max))
	    {
	      invading[y].total++;
	      invading[x].total--;
	      temp = get_invader_by_vnum(invading[x].vnum);
	      if (invading[x].total > invader_table[temp].max)
		x--;
	      break;
	    }
	    else if (invading[y].name == NULL)
	    {
	      invading[y].name = str_dup(invader_table[temp].group);
	      invading[y].vnum = invader_table[temp].vnum;
	      invading[y].total = 1;
	      invading[x].total--;
	      temp = get_invader_by_vnum(invading[x].vnum);
	      if (invading[x].total > invader_table[temp].max)
		x--;
	      break;
	    }
	  }
	}
      }
    }
  }

  for (x = 0; x < amount; x++)
  {
    if (invading[x].name != NULL)
    {
      if ((pMobIndex = get_mob_index(invading[x].vnum)) == NULL)
      {
	bug("Invasion: mob VNUM %d does not exist!",invading[x].vnum);
	invading[x].total = 0;
	continue;
      }

      location = get_room_index(find_invade_room(town));
      if (location != NULL)
{

// create the mobiles
victim = create_mobile(pMobIndex);

if (!IS_SET(victim->act2,ACT2_INVADE))
{
  SET_BIT(victim->act2,ACT2_INVADE);
}

// make sure it's NOT a sentinel
if (IS_SET(victim->act,ACT_SENTINEL))
{
  REMOVE_BIT(victim->act,ACT_SENTINEL);
}
/* make sure it IS not aggro
if (IS_SET(victim->act,ACT_AGGRESSIVE))
{
  SET_BIT(victim->act,ACT_AGGRESSIVE);
}*/

// put the mob in the desired room or area
char_to_room(victim,location);
}
      invading[x].total--;
      if (invading[x].total > 0)
	x--;
      invaded = TRUE;
    }
  }

  if (invaded)
  {
    /* do wiznet and logfile notifications */
    sprintf( log_buf, "Invasion begun of %s by %s with force %d",
	town_invade_table[town].name,
	race_invade_table[race].log_name,
	force);
    sprintf(log_buf,"Invasion begun of %s by the %s.\n\r",
      town_invade_table[town].name,
      race_invade_table[race].log_name);
    wiznet(log_buf,NULL,NULL,WIZ_INVADE,0,0);

    /* give warning to players in the area */
    for (vch = char_list; vch != NULL; vch = vch_next)
    {
      vch_next = vch->next;
      if (vch->in_room == NULL)
	continue;
      if (vch->in_room->area == victim->in_room->area)
	send_to_char("You hear a shout: 'To Arms!  We're under attack!  Defend yourselves!'\n\r",vch);
    }
    for ( d = descriptor_list; d; d = d->next )
    if ( d->connected == CON_PLAYING )
    {
        stc("{CIn the distance you hear: {G'{gDefend yourselves! We are under {Rattack{g!{G'{x\n\r", d->character );
        ptc(d->character, "{CFighting can be heard coming from %s!{G'{x\n\r", capit2(town_invade_table[town].name) );
    }

    return TRUE;
  }

  return FALSE;
}


/*
 * finds the mob's table location in invader_table by looking at the
 * number of mob it is of that race in the table.  For instance, it
 * might look for the 3rd goblin in the table (so that the goblin
 * entries don't have to be contiguous).
 */
int get_invader_num(int mob, int race)
{
  int x, total = 0;

  for (x = 0; invader_table[x].group != NULL; x++)
  {
    if (invader_table[x].group == race_invade_table[race].name)
      total++;
    if (mob == total)
      return x;
  }

  return 0;
}



/* gets the table number for a mob's vnum */
int get_invader_by_vnum(int mvnum)
{
  int x;

  for (x = 0; invader_table[x].group != NULL; x++)
  {
    if (invader_table[x].vnum == mvnum)
      return x;
  }

  return -1;
}



/* returns a TRUE if a mob can invade that room, a FALSE if not */
bool check_room_invade(int room)
{
  int x;

  for (x = 0; room_invade_table[x].town != NULL; x++)
  {
    if (room == room_invade_table[x].vnum)
      return FALSE;
  }

  return TRUE;
}


/*
 * finds a room to invade - cycles through 1000 times seeking room
 * to try and avoid finding no room.
 */
int find_invade_room(int town)
{
  int room, y;
  bool good_room = FALSE;


  for (y = 0; y < 1000; y++)
  {
    room = number_range(town_invade_table[town].min_vnum,
			town_invade_table[town].max_vnum);

    if (!check_room_invade(room))
      continue;

    if (get_room_index(room) != NULL
     && !IS_SET(get_room_index(room)->room_flags,ROOM_SAFE))
    {
      good_room = TRUE;
      break;
    }
  }

  if (good_room)
    return room;

  bug("Last invade room tried, %d, failed",room);
  return -1;
}
