/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"

void 	parse_journal(CHAR_DATA *ch, char *argument, int type);

/* Felix's Journal Code */
void do_journal(CHAR_DATA *ch,char *argument)
{
    parse_journal(ch,argument,PNOTE_JOURNAL);
}


void journal_attach( CHAR_DATA *ch, int type )
{
    PNOTE_DATA *pnote;

    if ( ch->pnote != NULL )
		return;

    pnote = new_pnote();
    pnote->next		= NULL;
    pnote->sender	= str_dup( ch->name );
    pnote->date		= str_dup( "" );
    pnote->to_list	= str_dup( "" );
    pnote->subject	= str_dup( "" );
    pnote->text		= str_dup( "" );
    pnote->type		= type;
    ch->pnote		= pnote;
    return;
}


void parse_journal( CHAR_DATA *ch, char *argument, int type )
{
	char arg[MAX_INPUT_LENGTH];
	char buf[MAX_STRING_LENGTH];
   	EXTRA_DESCR_DATA *ed;
    BUFFER *buffer;
    BUFFER *keybuf;
    BUFFER *descbuf;
    OBJ_DATA *obj;
    int x = 0;


	/* Handle IMMs who are switched into MOBs */
	if (IS_NPC(ch))
	{
		return;
	}

    for ( obj = ch->carrying; obj; obj = obj->next_content )
    {
		if ( obj->item_type == ITEM_JOURNAL )
	    break;
    }

	/* Can't find a journal */
    if ( !obj )
    {
		stc( "You do not own a journal.  Go purchase one.\n\r", ch );
		return;
    }

    argument = one_argument( argument, arg );
    smash_tilde( argument );

    if ( arg[0] == '\0' )
    {
		stc( "{WSyntax:{x journal <add> <keyword>\n\r", ch );
		stc( "{WSyntax:{x journal + <line of text>\n\r", ch );
		stc( "{WSyntax:{x journal - <line of text>\n\r", ch );
		stc( "{WSyntax:{x journal <show>\n\r", ch );
		stc( "{WSyntax:{x journal <clear>\n\r", ch );
		stc( "{WSyntax:{x journal <post>\n\r", ch );
		return;
    }

    if ( !str_prefix( arg, "add" ) )
    {
		journal_attach( ch, type );

        if (ch->pnote->type != type)
        {
            stc("You already have a different note in progress.\n\r",ch);
            return;
        }

		/* Does the journal already have a journal entry (extra description) ?
		 * If so, is the name of the journal entry being added the same as an
		 * existing journal entry ?  If so, stop and tell the player they can't
		 * do that.
		 *
		 * This loop is also used to count the number of extra descriptions the
		 * journal has.  I set a limit of 10 journal entries per journal. Afterall,
		 * the journal only has so much paper.  Feel free to remove the limit or
		 * change it to whatever you want.
		 */
	    if ( obj->extra_descr != NULL )
	    {
			for ( ed = obj->extra_descr; ed != NULL; ed = ed->next )
			{
				if ( (ed->keyword == str_dup( argument )) )
				{
					stc( "This journal already has an entry with that name.\n\r", ch );
					return;
				}

				x += 1;
			}
	    }

		if ( x > 9 )
		{
			stc( "You have run out of blank pages in your journal.\n\r", ch );
			stc( "Time to buy a new one.\n\r", ch );
			return;
		}

		free_string( ch->pnote->to_list );
		free_string( ch->pnote->subject );
		ch->pnote->subject = str_dup( argument );
		stc( "Ok.\n\r", ch );
		return;
    }

    if ( !str_cmp( arg, "+" ) )
    {
		journal_attach( ch,type );

		if (ch->pnote->type != type)
		{
		    stc("You already have a different note in progress.\n\r",ch);
		    return;
		}

		if (strlen(ch->pnote->text)+strlen(argument) >= 4096)
		{
		    stc( "Journal entry too long.\n\r", ch );
		    return;
		}

	 	buffer = new_buf();
		add_buf(buffer, ch->pnote->text);
		add_buf(buffer, argument);
		add_buf(buffer,"\n\r");
		free_string( ch->pnote->text );
		ch->pnote->text = str_dup( buf_string(buffer) );
		free_buf(buffer);
		stc( "Ok.\n\r", ch );
		return;
    }

    if ( !str_cmp( arg, "desc" )
    || !str_cmp( arg, "write") )
    {
    	journal_attach( ch, type );
	    if ( argument[0] == '\0' )
	    	string_append( ch, &ch->pnote->text );
    	return;
    }

    if (!str_cmp(arg,"-"))
    {
 		int len;
		bool found = FALSE;

		journal_attach(ch,type);

		if (ch->pnote->type != type)
        {
            stc("You already have a different note in progress.\n\r",ch);
            return;
        }

		if (ch->pnote->text == NULL || ch->pnote->text[0] == '\0')
		{
		    stc("No lines left to remove.\n\r",ch);
		    return;
		}

		strcpy(buf,ch->pnote->text);

		for (len = strlen(buf); len > 0; len--)
 		{
		    if (buf[len] == '\r')
		    {
				if (!found)
				{
				    if (len > 0)
					len--;
				    found = TRUE;
				}
				else
				{
				    buf[len + 1] = '\0';
				    free_string(ch->pnote->text);
				    ch->pnote->text = str_dup(buf);
				    return;
				}
		    }
		}

		buf[0] = '\0';
		free_string(ch->pnote->text);
		ch->pnote->text = str_dup(buf);
		return;
    }

    if ( !str_prefix( arg, "clear" ) )
    {
		if ( ch->pnote != NULL )
		{
		    free_pnote(ch->pnote);
		    ch->pnote = NULL;
		}

		stc( "Ok.\n\r", ch );
		return;
    }

    if ( !str_prefix( arg, "show" ) )
    {
		if ( ch->pnote == NULL )
		{
		    stc( "You have no journal entry in progress.\n\r", ch );
		    return;
		}

		if (ch->pnote->type != type)
		{
		    stc("You aren't working on that kind of note.\n\r",ch);
		    return;
		}

	    stc( "Journal Entry: ", ch );
		stc( ch->pnote->subject, ch );
	    stc( "\n\r\n\r", ch );
		stc( ch->pnote->text, ch );
		return;
    }

    if ( !str_prefix( arg, "post" ) )
    {
		if ( ch->pnote == NULL )
		{
		    stc( "You have no journal entry in progress.\n\r", ch );
		    return;
		}

        if (ch->pnote->type != type)
        {
            stc("You aren't working on that kind of note.\n\r",ch);
            return;
        }

		/* The following lines of code attach the name of the journal entry
		 * to the extra description as the keyword and also adds all the text
		 * entered by the player to the description of the keyword.
		 */
		ed = new_extra_descr();
		keybuf = new_buf();
		add_buf ( keybuf, ch->pnote->subject );
		ed->keyword = str_dup( buf_string( keybuf ) );
		buffer = new_buf();
		sprintf( buf, "Author: %s\n\r\n\r", ch->name );
   		add_buf( buffer, buf );
   		add_buf( buffer, ch->pnote->text );
		ed->description = str_dup( buf_string( buffer ) );
		ed->next = obj->extra_descr;
		obj->extra_descr = ed;


		/* The following lines of code modify the journals long description.
		 * It adds the name of the journal entry to the long description which
		 * allows players to look at the journal and see what journal entries
		 * have been made to the journal.
		 */
		descbuf = new_buf();
		sprintf( buf, obj->description );
		free_string( obj->description );
		add_buf( descbuf, buf );
		sprintf( buf, "\n\rJournal Entry: {c%s{x\n\r", ch->pnote->subject );
		add_buf( descbuf, buf );
		obj->description = str_dup( buf_string( descbuf ) );
		free_buf( descbuf );
		free_buf( buffer );
		free_buf( keybuf );
		ch->pnote = NULL;
		stc("Your journal entry has been posted.\n\r",ch);
		return;
    }

	stc( "{WSyntax:{x journal <add> <keyword>\n\r", ch );
	stc( "{WSyntax:{x journal + <line of text>\n\r", ch );
	stc( "{WSyntax:{x journal - <line of text>\n\r", ch );
	stc( "{WSyntax:{x journal <show>\n\r", ch );
	stc( "{WSyntax:{x journal <clear>\n\r", ch );
	stc( "{WSyntax:{x journal <post>\n\r", ch );
	return;
}
