/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"

void fread_vault args( ( CHAR_DATA *ch ) );
void fwrite_vault args( ( CHAR_DATA *ch ) );

void fwrite_vobj( CHAR_DATA *ch, OBJ_DATA *obj, FILE *fp, int iNest )
{
    EXTRA_DESCR_DATA *ed;
    AFFECT_DATA *paf;

    fprintf( fp, "#OBJECT\n" );
    fprintf( fp, "Vnum %ld\n",   obj->pIndexData->vnum        );
    if (!obj->pIndexData->new_format)
	fprintf( fp, "Oldstyle\n");
    if (obj->enchanted)
	fprintf( fp,"Enchanted\n");
    fprintf( fp, "Nest %d\n",	iNest	  	     );

    if ( obj->name != obj->pIndexData->name)
    	fprintf( fp, "Name %s~\n",	obj->name		     );
    if ( obj->short_descr != obj->pIndexData->short_descr)
        fprintf( fp, "ShD  %s~\n",	obj->short_descr	     );
    if ( obj->description != obj->pIndexData->description)
        fprintf( fp, "Desc %s~\n",	obj->description	     );
    if ( obj->material != obj->pIndexData->material )
    	fprintf( fp, "Mat %s~\n", obj->material );
    if ( obj->owner != NULL && obj->owner[0] != '\0' )
    	fprintf( fp, "Owner %s~\n", obj->owner );
    if ( obj->extra_flags != obj->pIndexData->extra_flags)
        fprintf( fp, "ExtF %d\n",	obj->extra_flags	     );
    if ( obj->wear_flags != obj->pIndexData->wear_flags)
        fprintf( fp, "WeaF %d\n",	obj->wear_flags		     );
    if ( obj->item_type != obj->pIndexData->item_type)
        fprintf( fp, "Ityp %d\n",	obj->item_type		     );
    if ( obj->weight != obj->pIndexData->weight)
        fprintf( fp, "Wt   %d\n",	obj->weight		     );
    if ( obj->condition != obj->pIndexData->condition)
	fprintf( fp, "Cond %d\n",	obj->condition		     );

    /* variable data */

    fprintf( fp, "Wear %d\n",   obj->wear_loc                );
    if (obj->level != obj->pIndexData->level)
        fprintf( fp, "Lev  %d\n",	obj->level		     );
    if (obj->timer != 0)
        fprintf( fp, "Time %d\n",	obj->timer	     );
    fprintf( fp, "Cost %d\n",	obj->cost		     );
    if (obj->value[0] != obj->pIndexData->value[0]
    ||  obj->value[1] != obj->pIndexData->value[1]
    ||  obj->value[2] != obj->pIndexData->value[2]
    ||  obj->value[3] != obj->pIndexData->value[3]
    ||  obj->value[4] != obj->pIndexData->value[4]
    ||  obj->item_type == ITEM_TREASURECHEST )
    	fprintf( fp, "Val  %d %d %d %d %d\n",
	    obj->value[0], obj->value[1], obj->value[2], obj->value[3],
	    obj->value[4]	     );

    switch ( obj->item_type )
    {
    case ITEM_WEAPON:
	case ITEM_ARMOR:
	case ITEM_TREASURE:
	case ITEM_LIGHT:
	    for ( paf = obj->affected; paf != NULL; paf = paf->next )
	    {
			/*if (paf->type != -1)
			    continue;*/
	        fprintf( fp, "Infu %d %d %d %d %d %d %d\n",
    	        paf->location,
        	    paf->modifier,
            	paf->where,
	            paf->type,
    	        paf->duration,
        	    paf->bitvector,
            	paf->level );
            break;
	    }
	    break;

    case ITEM_POTION:
    case ITEM_SCROLL:
    case ITEM_PILL:
	if ( obj->value[1] > 0 )
	{
	    fprintf( fp, "Spell 1 '%s'\n",
		skill_table[obj->value[1]].name );
	}

	if ( obj->value[2] > 0 )
	{
	    fprintf( fp, "Spell 2 '%s'\n",
		skill_table[obj->value[2]].name );
	}

	if ( obj->value[3] > 0 )
	{
	    fprintf( fp, "Spell 3 '%s'\n",
		skill_table[obj->value[3]].name );
	}

	break;

    case ITEM_STAFF:
    case ITEM_WAND:
		if ( obj->value[3] > 0 )
		{
	    	fprintf( fp, "Spell 3 '%s'\n",
			skill_table[obj->value[3]].name );
		}
	    for ( paf = obj->affected; paf != NULL; paf = paf->next )
	    {
			/*if (paf->type != -1)
			    continue;*/
	        fprintf( fp, "Infu %d %d %d %d %d %d %d\n",
    	        paf->location,
        	    paf->modifier,
            	paf->where,
	            paf->type,
    	        paf->duration,
        	    paf->bitvector,
            	paf->level );
            break;
	    }

	break;
    }

    for ( paf = obj->affected; paf != NULL; paf = paf->next )
    {
		if (paf->type < 0 || paf->type >= MAX_SKILL)
		    continue;
        fprintf( fp, "Affc '%s' %3d %3d %3d %3d %3d %10d\n",
            skill_table[paf->type].name,
            paf->where,
            paf->level,
            paf->duration,
            paf->modifier,
            paf->location,
            paf->bitvector
            );
        break;
    }

    for ( ed = obj->extra_descr; ed != NULL; ed = ed->next )
    {
    	fprintf( fp, "ExDe %s~ %s~\n",
	    ed->keyword, ed->description );
    }

    fprintf( fp, "End\n\n" );


    return;
}

void do_locker( CHAR_DATA *ch, char *argument )
{
  char arg[MIL];
  OBJ_DATA *obj;
  OBJ_DATA *obj_next;

  if ( IS_NPC(ch) )
    return;

  argument = one_argument(argument, arg);
  if ( arg[0] == '\0' )
  {
    stc( "Syntax: vault buy\r\n", ch );
    stc( "Syntax:   This command purchases the vault. It only needs to be done once.\r\n", ch );
    stc( "Syntax:   It does not cost anything to get a vault. You are charged one\r\n", ch );
    stc( "Syntax:   silver mark each time you open the vault.\r\n", ch );
    stc( "Syntax: vault <open | close>\r\n", ch );
    stc( "Syntax:   These commands allow entrance into or out of the vault. Entering\r\n", ch );
    stc( "Syntax:   the vault opens it and allows you to interact with objects that\r\n", ch );
    stc( "Syntax:   are stored within. Closing it saves the vault and exits the vault.\r\n", ch );
    return;
  }

  /* begin buying a vault */
  if ( !str_cmp( arg, "buy" ) )
  {
    if( ch->pcdata->has_vault )
    {
      stc( "You have already purchased access to a vault.\r\n", ch );
      return;
    }
    if( !IS_SET( ch->in_room->room_flags, ROOM_BANK ) )
    {
      stc( "This doesn't look like a proper bank vault.\r\n", ch );
      return;
    }
    if( !ch->pcdata->has_vault )
    {
      ch->pcdata->has_vault = TRUE;
      stc( "You make your mark in the ledgers and now may access your vault.\r\n", ch );
      return;
    }
  }
  /* end buying a vault */

  /* begin opening a vault */
  else if( !str_cmp( arg, "open" ) )
  {
    DESCRIPTOR_DATA *d;
    if( !ch->pcdata->has_vault )
    {
      stc( "You need to sign the vault ledger first.\r\n", ch );
      return;
    }
    if( !IS_SET( ch->in_room->room_flags, ROOM_BANK ) )
    {
      stc( "This doesn't look like a proper bank.\r\n", ch );
      return;
    }
    for ( d = descriptor_list; d != NULL; d = d->next )
    {
      if ( d->connected != CON_PLAYING )
        continue;
      if ( !d->character )
        continue;
      if ( d->character->pcdata->in_vault )
      {
        stc( "The clerk informs you the vault is currently in use.\n\r", ch );
        return;
      }
    }
    if ( check_coins( ch, 100, 0, 0 ) )
    {
      deduct_cost( ch, 100, 0 );
    }
    else if ( check_coins( ch, 100, 1, 1 ) )
    {
      deduct_cost( ch, 100, 1 );
    }
    else
    {
      stc( "The clerk politely informs you that you can't afford the service right now.\r\n", ch );
      return;
    }
    act("$n flips the clerk a silver mark and walks into the vault.", ch, NULL, NULL, TO_ROOM );
    act("You flip the clerk a silver mark and walk into the vault.", ch, NULL, NULL, TO_CHAR );
    stc( "You are now in your personal vault.\r\n", ch );
    ch->pcdata->prior_to_vault = ch->in_room;
    char_from_room( ch );
    char_to_room( ch, get_room_index( VAULT_ROOM ) );
    fread_vault( ch );
    ch->pcdata->in_vault = TRUE;
    do_function( ch, &do_look, "auto" );
    return;
  }
  /* end opening a vault */

  /* begin closing a vault */
  else if( !str_cmp( arg, "close" ) )
  {
    int count = 0;
    if( !ch->pcdata->has_vault )
    {
      stc( "You do not have access to a vault.\r\n", ch );
      return;
    }
    if( !ch->pcdata->in_vault )
    {
      stc( "You are not in a vault.\r\n", ch );
      return;
    }
    for ( obj = object_list; obj != NULL; obj = obj_next )
    {
      obj_next = obj->next;
      if ( !obj->in_room )
        continue;
      if ( obj->in_room->vnum != ch->in_room->vnum )
        continue;
      if( obj->item_type == ITEM_CONTAINER || obj->item_type == ITEM_TREASURECHEST )
      {
        stc( "You may not leave containers or treasure chests in the locker room.\r\n", ch );
        return;
      }
      count++;
      if ( count > 99 )
      {
        stc( "You cannot have more than 99 items in the vault.\n\rPick up some items and try again.\n\r", ch );
        return;
      }
    }
    fwrite_vault( ch );
    char_from_room( ch );
    char_to_room( ch, ch->pcdata->prior_to_vault );
    ch->pcdata->prior_to_vault = NULL;
    ch->pcdata->in_vault = FALSE;
    act("$n walks out of the bank vault.", ch, NULL, NULL, TO_ROOM );
    stc( "You walk out of the bank vault.\r\n", ch );
    do_function( ch, &do_look, "auto" );
  }
  return;
}

void fread_vault( CHAR_DATA *ch )
{
	FILE *fp;
	char name[24];

  sprintf( name, "%s%s", VAULTS_DIR, capitalize( ch->name ) );

	if ( ch == NULL || ch->pcdata == NULL || ch->desc == NULL )
  {
		bug("NULL CH trying to write pc_shop data!", 0 );
		return;
  }
	if ( !ch->pcdata->has_vault )
	{
		stc("But you do not own access to a vault!\n\r",ch);
		return;
	}
  if ( ( fp = fopen( name, "r" ) ) == NULL )
  {
    stc( "No vault data found for you.\n\rIf this is a new vault, you may leave items here and <vault close> to save them.\n\r", ch );
		return;
  }
  else
  {
	  char letter;
	  char *word;

		for( ; ; )
		{
	    letter = fread_letter( fp );
	    word = fread_word( fp );

	    if ( letter != '#' )
	    {
				bugf( "Fread_vault: # not found in vault for %s.", ch->name );
				break;
	    }
      if( !str_cmp(word, "EOF" )  )
        break;
      if ( !str_cmp( word, "OBJECT" ) )
	      fread_obj( ch, fp );
 		}

		fclose( fp );
		stc("You collect the items from your vault.\n\r",ch);
		return;
  }

  return;
}

void fwrite_vault( CHAR_DATA *ch )
{
	FILE *fp;
	char name[24];
  OBJ_DATA *obj;
  OBJ_DATA *obj_next;
  int count = 0, num = 0;

  sprintf( name, "%s%s", VAULTS_DIR, capitalize( ch->name ) );

  if ( ch == NULL || ch->pcdata == NULL || ch->desc == NULL )
  {
    bug("NULL CH trying to write vault data!", 0 );
    return;
  }

  if ( ( fp = fopen( name, "w" ) ) == NULL )
  {
    bugf("Could not open %s for write access!", name );
    return;
  }
  for ( obj = object_list; obj != NULL; obj = obj_next )
  {
    obj_next = obj->next;
    if ( obj->carried_by != NULL )
      continue;
    if ( count > 99 )
    {
      stc( "BUG! Trying to store greater than 99 objects!\n\r", ch );
      break;
    }
    if ( obj->in_room && obj->in_room->vnum == ch->in_room->vnum && !obj->carried_by )
    {
      /*ptc( ch, "FWRITE: Obj %s is in room %d, matching %s's room %d.\n\r", obj->name, obj->in_room->vnum, ch->name, ch->in_room->vnum );*/
      fwrite_vobj( ch, obj, fp, 0 );
      num++;
    }
    else
      continue;
  }
  stc( "Printing EOF\n\r", ch );
  fprintf(fp, "#EOF\n" );

  for ( obj = object_list; obj != NULL; obj = obj_next )
  {
    obj_next = obj->next;
    if ( obj->carried_by != NULL )
      continue;
    if ( !obj->in_room )
      continue;
    if ( obj->in_room->vnum != ch->in_room->vnum )
      continue;
    /*ptc( ch, "EXTRACT: Obj %s is in room %d, matching %s's room %d.\n\r", obj->name, obj->in_room->vnum, ch->name, ch->in_room->vnum );*/
    extract_obj( obj );
  }

  ptc( ch, "Counted %d objects to write.\n\r", num );
  fclose( fp );
  return;
}

