/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "newbie.h"
#include "tables.h"

void set_features( CHAR_DATA *ch )
{
	/* 34 eye colors */
	if(ch->pcdata->eye < 0 || ch->pcdata->eye > 33)
		ch->pcdata->eye = 0;
	/* 6 hair colors */
	if(ch->pcdata->hair1 < 0 || ch->pcdata->hair1 > 5)
		ch->pcdata->hair1 = 0;
	/* 10 hair shades */
	if(ch->pcdata->hair2 < 0 || ch->pcdata->hair2 > 9)
		ch->pcdata->hair2 = 0;
	/* 8 hair styles */
	if(ch->pcdata->hair3 < 0 || ch->pcdata->hair3 > 7)
		ch->pcdata->hair3 = 0;
	/* 9 hair lengths */
	if(ch->pcdata->hair4 < 0 || ch->pcdata->hair4 > 8)
		ch->pcdata->hair4 = 0;

	ch->eye = str_dup(eye_color_table[ch->pcdata->eye].name);
	ch->hair1 = str_dup(hair_color_table[ch->pcdata->hair1].name);
	ch->hair2 = str_dup(hair_shade_table[ch->pcdata->hair2].name);
	ch->hair3 = str_dup(hair_style_table[ch->pcdata->hair3].name);
	ch->hair4 = str_dup(hair_length_table[ch->pcdata->hair4].name);
/*
	ch->eye   = str_dup("blue");
	ch->hair1 = str_dup("brown");
	ch->hair2 = str_dup("dark");
	ch->hair3 = str_dup("straight");
	ch->hair4 = str_dup("short");
*/
}
void clear_newbieroom( CHAR_DATA *ch, int tmproom )
{
    CHAR_DATA *victim, *vnext;
    OBJ_DATA *obj, *obj_next;
    ROOM_INDEX_DATA *clean;

    clean = get_room_index( tmproom );

    if ( clean->vnum == 31019 || clean->vnum == 31014 || clean->vnum == 31015  )
    	return;

	for ( victim = clean->people; victim != NULL; victim = vnext )
	{
	    vnext = victim->next_in_room;
	    if ( IS_NPC(victim) && !IS_SET(victim->act,ACT_NOPURGE)
	    &&   victim != ch )
		extract_char( victim, TRUE );
	}
	for ( obj = clean->contents; obj != NULL; obj = obj_next )
	{
	    obj_next = obj->next_content;
	    if (!IS_OBJ_STAT(obj,ITEM_NOPURGE))
			extract_obj( obj );
	}
	return;
}
bool check_valid_newbie( CHAR_DATA *ch )
{
	int i;
	if ( ch->race <= -1 )
	{
		stc( "{WYou do not appear to have a valid race yet!{x\n\r", ch );
		return FALSE;
	}
	if ( ch->sex <= 0 )
	{
		stc( "{WYou do not appear to have a valid gender yet!{x\n\r", ch );
		return FALSE;
	}
	if ( !IS_SET( ch->hist_flags, HIST_SOLDIER )
	&&   !IS_SET( ch->hist_flags, HIST_MERCHANT )
	&&   !IS_SET( ch->hist_flags, HIST_BEGGAR )
	&&   !IS_SET( ch->hist_flags, HIST_ARTIST )
	&&   !IS_SET( ch->hist_flags, HIST_FARMER )
	&&   !IS_SET( ch->hist_flags, HIST_CHILD )
	&&   !IS_SET( ch->hist_flags, HIST_MUSICIAN )
	&&   !IS_SET( ch->hist_flags, HIST_SAILOR )
	&&   !IS_SET( ch->hist_flags, HIST_NOBLE ) )
	{
		stc( "{WYou do not appear to have a valid background yet!{w\n\r", ch );
		return FALSE;
	}
	if ( (ch->pcdata->eye <= -1
	|| ch->pcdata->hair1 <= -1
	|| ch->pcdata->hair2 <= -1
	|| ch->pcdata->hair3 <= -1
	|| ch->pcdata->hair4 <= -1 )  && ( !ch->race == race_lookup( "trolloc") ))
	{
		stc( "{WYou do not appear to have valid features yet!{x\n\r", ch );
		return FALSE;
	}
	if ( ch->race == race_lookup( "aiel" )
	&& ( ch->pcdata->aiel_clan == NULL || ch->pcdata->aiel_clan[0] == '\0' ) )
	{
		stc( "{WYou appear to be Aiel but do not have a valid clan!{x\n\r", ch );
		return FALSE;
	}
	for ( i = 0; i < MAX_STATS; i++ )
	{
		if ( ch->perm_stat[i] <= 3 )
		{
			stc( "{WOne or more of your ability scores is far too low!{x\n\r", ch );
			return FALSE;
		}
	}
	return TRUE;
}
void init_newbie( CHAR_DATA *ch )
{
	if ( ch->pcdata->rebirth <= 0 )
	{
		ch->race = race_lookup( "andor" );
		ch->sex = 0;
		ch->hit = 50;
		ch->max_hit = 50;
	    ch->stam = 50;
	    ch->max_stam = 50;
		ch->pcdata->eye = -1;
		ch->pcdata->hair1 = -1;
		ch->pcdata->hair2 = -1;
		ch->pcdata->hair3 = -1;
		ch->pcdata->hair4 = -1;
	    ch->pcdata->op_air    = 0;
	    ch->pcdata->op_fire   = 0;
	    ch->pcdata->op_spirit = 0;
	    ch->pcdata->op_earth  = 0;
	    ch->pcdata->op_water  = 0;
	    ch->level  	 = 1;
	    ch->exp	     = exp_per_level(ch,ch->pcdata->points);
	    ch->train	 = 50;
	    /*ch->practice = 25;*/
	    ch->version = 14;

	    ch->money[0][0] = 0;
	    ch->money[1][0] = 0;
	    ch->money[2][0] = 0;
	    ch->money[3][0] = 0;
	    ch->money[0][0] = 0;
	    ch->money[1][1] = 0;
	    ch->money[2][2] = 0;
	    ch->money[3][3] = 0;
	
    SET_BIT(ch->act,PLR_AUTOEXIT);
    SET_BIT(ch->act,PLR_COLOUR);
    SET_BIT(ch->comm,COMM_PDEATH);
    SET_BIT(ch->comm,COMM_PENTER);
	SET_BIT(ch->war_flags,WAR_AUTOFIGHT);
    SET_BIT(ch->auto_flags,AUTO_COUNTER);
    SET_BIT(ch->auto_flags,AUTO_DODGE);
    SET_BIT(ch->auto_flags,AUTO_PARRY);
    SET_BIT(ch->auto_flags,AUTO_AUTOH2H);
    SET_BIT(ch->auto_flags,AUTO_AUTOMAP);

	}
	else
	{
		roll_abilityscores( ch );
		stc("If you wish to reroll your ability scores, type select stat in this room.\n\r",ch);
		stc("If you wish to reroll your OP average scores, type select op in this room.\n\r",ch);
		set_features( ch );
	        if (IS_SET( ch->op_flags, OP_CHANNELER))
			roll_onepower( ch );
//		clear_rebirth();
		compose_title( ch );
		REBIRTH_IN_PROGRESS = 0;
	}

    ch->pcdata->touch_tries = 9;
	ch->pcdata->stat_rolls = 9;
	return;
}
void roll_onepower( CHAR_DATA *ch )
{
    ch->pcdata->op_air    = number_range(3,12);
    ch->pcdata->op_fire   = number_range(3,12);
    ch->pcdata->op_spirit = number_range(3,12);
    ch->pcdata->op_earth  = number_range(3,12);
    ch->pcdata->op_water  = number_range(3,12);

    if (ch->sex == SEX_FEMALE && ch->pcdata->op_water < 12)
     	ch->pcdata->op_water += 2;
    if (ch->sex == SEX_MALE && ch->pcdata->op_fire < 12)
     	ch->pcdata->op_fire += 2;

	ch->pcdata->touch_tries -= 1;

	if ( ch->pcdata->rebirth >= 1 )
	{
		ch->pcdata->op_earth  += ch->pcdata->rebirth / 2;
		ch->pcdata->op_air    += ch->pcdata->rebirth / 2;
		ch->pcdata->op_fire   += ch->pcdata->rebirth / 2;
		ch->pcdata->op_water  += ch->pcdata->rebirth / 2;
		ch->pcdata->op_spirit += ch->pcdata->rebirth / 2;
	    stc("{WStrength in the One Power plus bonuses for being reborn is as follows:{x\n\r",ch);
	}
	else
	    stc("{WStrength in the One Power for a new character is as follows:{x\n\r",ch);

    ptc( ch, "{WAir: %d  Fire: %d  Spirit: %d  Earth: %d  Water: %d{x\n\r",
    ch->pcdata->op_air,
    ch->pcdata->op_fire,
    ch->pcdata->op_spirit,
    ch->pcdata->op_earth,
    ch->pcdata->op_water);
	ptc( ch, "{wRolls remaining: %d.{x\n\r", ch->pcdata->touch_tries );
	return;
}
void roll_abilityscores( CHAR_DATA *ch )
{
	int i;
/*
	for ( i=0; i < MAX_STATS; i++ )
		ch->perm_stat[i] = 0;

	for ( i=0; i < MAX_STATS; i++ )
		ch->perm_stat[i] = number_range( 12, 20 );
	*/
	    for (i = 0; i < MAX_STATS; i ++)
	    {
			ch->perm_stat[i] = number_range( 12, 20 );
			ch->perm_stat[i] += ( 2 * ch->pcdata->rebirth ) / 3;
	    }

	ch->pcdata->stat_rolls -= 1;

	if ( IS_SET( ch->hist_flags, HIST_SOLDIER ) )
		ch->perm_stat[STAT_CON] += ch->perm_stat[STAT_CON] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_MERCHANT ) )
		ch->perm_stat[STAT_INT] += ch->perm_stat[STAT_INT] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_BEGGAR ) )
		ch->perm_stat[STAT_DEX] += ch->perm_stat[STAT_DEX] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_ARTIST ) )
		ch->perm_stat[STAT_CHA] += ch->perm_stat[STAT_CHA] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_FARMER ) )
		ch->perm_stat[STAT_STR] += ch->perm_stat[STAT_STR] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_CHILD ) )
		ch->perm_stat[STAT_INS] += ch->perm_stat[STAT_INS] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_MUSICIAN ) )
		ch->perm_stat[STAT_WIS] += ch->perm_stat[STAT_WIS] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_SAILOR ) )
		ch->perm_stat[STAT_REF] += ch->perm_stat[STAT_REF] / 9;
	else if ( IS_SET( ch->hist_flags, HIST_NOBLE ) )
		ch->perm_stat[STAT_CHA] += ch->perm_stat[STAT_CHA] / 9;
	if ( ch->pcdata->rebirth <= 0 )
		stc( "{wYour ability scores have been rolled as follows:{x\n\r", ch );
	else
	{
		stc( "{wYour ability scores have been modified for being reborn as follows:{x\n\r", ch );
		stc( "{wChoosing to roll your scores again will remove any trains you spent on your stats!{x\n\r", ch );
		stc( "{wChanging your ability to channel will reroll your stats as the bonuses are different.{x\n\r", ch );
	}

	ptc( ch, "{wStr: %d Int: %d Wis: %d Dex: %d Con: %d Ref: %d Ins: %d Cha: %d{x\n\r",
		ch->perm_stat[STAT_STR],
		ch->perm_stat[STAT_INT],
		ch->perm_stat[STAT_WIS],
		ch->perm_stat[STAT_DEX],
		ch->perm_stat[STAT_CON],
		ch->perm_stat[STAT_REF],
		ch->perm_stat[STAT_INS],
		ch->perm_stat[STAT_CHA]);

	ptc( ch, "{wRolls remaining: %d.{x\n\r", ch->pcdata->stat_rolls );
	return;
}
void do_touch( CHAR_DATA *ch, char *argument )
{
	if ( argument[0] == '\0' )
	{
		stc( "{wTouch what?\n\r", ch );
		return;
	}
	if ( ch->pcdata->touch_tries-- <= 0 )
	{
		stc( "{wYou feel unable to touch either. It is time to {WCONTINUE{w.{x\n\r", ch );
		do_function( ch, &do_continue, "" );
		return;
	}
	if ( !str_prefix( argument, "sword" ) )
	{
		if ( ch->in_room->vnum != 31012 )
		{
			stc( "{wThere is no sword to touch.\n\r", ch );
			return;
		}
		stc( "{wYou touch the sword and feel your weapon skill increase.\n\r", ch );
		if ( IS_SET( ch->op_flags, OP_CHANNELER ) )
		{
    		    if (IS_SET( ch->op_flags, OP_GRASPED))
    		    {
    			do_function(ch, &do_release, "");
    			stc("You release your hold on the True Source.\n\r",ch);
		    }
		    REMOVE_BIT( ch->op_flags, OP_CHANNELER );
		    if ( ch->pcdata->rebirth > 0 )
			roll_abilityscores( ch );
		}
		do_function( ch, &do_continue, "" );
		return;
	}
	else if ( !str_prefix( argument, "sphere" ) )
	{
		if ( ch->in_room->vnum != 31012 )
		{
			stc( "{wThere is no sphere to touch.\n\r", ch );
			return;
		}
		if ( ch->pcdata->burnout >= 100 )
		{
			stc( "{wThe sphere seems to elude your touch.\n\r", ch );
			return;
		}

/*
		if (ch->clan > 0 &&
		( ch->clan == clan_lookup("children")
		|| ch->race == race_lookup( "trolloc" )
		|| ch->race == race_lookup( "ogier" )
		|| ( (ch->race == race_lookup( "aiel" ) || ch->race==race_lookup("seanchan") || ch->race==race_lookup("seafolk") ) && ch->sex == SEX_MALE )))
*/

		if (ch->race == race_lookup( "trolloc" )
		|| ch->race == race_lookup( "ogier" ))
		{
			stc( "{wYour hand feels drawn to the sword.\n\r", ch );
			do_function( ch, &do_touch, "sword" );
			return;
		}

		if ( !IS_SET( ch->op_flags, OP_CHANNELER ) )
		{
			SET_BIT( ch->op_flags, OP_CHANNELER );
		    if ( ch->pcdata->rebirth > 0 )
			roll_abilityscores( ch );
		}
		stc( "{wYou touch the sphere and feel your One Power skill increase.\n\r", ch );
		roll_onepower( ch );
		do_function( ch, &do_continue, "" );
		return;
	}
	else
	{
		stc( "{wTouch what?\n\r", ch );
		return;
	}
}
void do_select( CHAR_DATA *ch, char *argument )
{
	char arg[MIL];
	int room;

	argument = one_argument( argument, arg );

	if ( IS_NPC( ch ) )
		return;

	room = ch->in_room->vnum;

	if ( arg[0] == '\0' )
	{
		stc( "What do you wish to select?\n\r", ch );
		return;
	}

	/* Ideas for selections... Might as well use this for something.
	 * select autoskill <skill name> autofire skill per combat round.
	 * select autoweave <weave name> autofire weave per combat round.
	 * select referrer <name of referrer> who referred the char.
	 */

	/* Newbie functions. These are used during the tour process
	 * for newbies to choose their features and such.
	 */
	if ( room == 31004 )
	{
		int race, pcrace;
		char buf[MSL];
		race = race_lookup( arg );
		pcrace = pcrace_lookup( arg );

		if ( pcrace <= 0 )
		{
			stc( "This does not appear to be a valid race.\n\r", ch );
			return;
		}
//		ptc( ch, "Looking at pc race %s.\n\r", pc_race_table[pcrace].name );

		if ( race_table[race].remort > ch->pcdata->rebirth )
		{
			ptc(ch,"That race is too powerful for you. Your need %d remorts.\n\r", race_table[race].remort);
			return;
		}
		if ( pc_race_table[pcrace].lifer && ( ch->pcdata->rebirth > 0 ))
		{
			stc("You can not remort into a life race.\n\r", ch);
			return;
		}
		if ( pc_race_table[pcrace].lifer && !ch->pcdata->confirm_life_race )
		{
			stc( "{RCaution! {wThe race you are choosing is a life race, which means once you{x\n\r", ch );
			stc( "{wchoose it, there is no way out except to delete your character. If you are{x\n\r", ch );
			stc( "{wcertain you wish to be of this race select it again.{x\n\r", ch );
			ch->pcdata->confirm_life_race = TRUE;
			return;
		}
		ch->pcdata->confirm_life_race = FALSE;
		if ( ch->race == race_lookup( "seanchan" )
		|| ch->race == race_lookup( "aiel" )
		|| ch->race == race_lookup( "ogier" )
		|| ch->race == race_lookup( "nym" )
		|| ch->race == race_lookup( "seafolk" ) )
		{
			stc( "{wYou are in a life race. You cannot change your race.{w\n\r", ch );
			char_from_room( ch );
			char_to_room( ch, get_room_index( 31005 ) );
			do_function( ch, &do_look, "auto" );
			return;
		}
		if ( ch->race == race_lookup( "trolloc" ))
		{
			stc( "{wYou are in a life race. You cannot change your race.{w\n\r", ch );
			char_from_room( ch );
			char_to_room( ch, get_room_index( 31013 ) );
			do_function( ch, &do_look, "auto" );
			return;
		}
		if ( race == 0 || !race_table[race].pc_race )
		{
		    stc(  "{wThat does not appear to be valid race. You can type SHOW RACES to{x\n\r",ch);
		    stc(  "{wsee the list of races available to you. Note that note all of the{x\n\r",ch);
		    stc(  "{wraces will be displayed, only the ones you can choose now.{x\n\r",ch);
		    return;
		}

        ch->race = race;
        ptc( ch, "You have chosen %s as your race.\n\r", capit2( race_table[race].name ) );
		if ( IS_SET( ch->op_flags, OP_CHANNELER ) )
		{
			if ( ch->race == race_lookup( "trolloc" )
			|| ch->race == race_lookup( "ogier" )
			|| (ch->race==race_lookup("seanchan")  && ch->sex == SEX_MALE )
			|| (ch->race==race_lookup("seafolk")  && ch->sex == SEX_MALE )
			|| ( ch->race == race_lookup( "aiel" ) && ch->sex == SEX_MALE ) )
			{
			    stc( "{wYou no longer feel the source.\n\r", ch );
				REMOVE_BIT( ch->op_flags, OP_CHANNELER );
			    if ( get_skill( ch, gsn_sword ) < 75 )
				ch->pcdata->learned[gsn_sword] = 75;
			    roll_abilityscores( ch );
			}
		}

		if ( ch->race == race_lookup( "trolloc" ) ) /* bellas trolloc*/
		{
		   stc( "{wYou will be sent to a special room where you can choose your clan.{w\n\r", ch );
					ch->clan = clan_lookup( "shadowspawn" );
					SET_BIT( ch->hist_flags, HIST_SOLDIER );
					char_from_room( ch );
					char_to_room( ch, get_room_index( 31025 ) );
					do_function( ch, &do_look, "auto" );
					return;
		}

		if ( ch->race == race_lookup( "aiel" ) )
		{
			stc( "{wYou will be sent to a special room where you can choose your society.{w\n\r", ch );
			ch->clan = clan_lookup( "Aiel" );
			char_from_room( ch );
			char_to_room( ch, get_room_index( 31024 ) );
			do_function( ch, &do_look, "auto" );
			return;
		}
		if ( ch->race == race_lookup( "tuathaan" ) )
		{
			ch->clan = clan_lookup( "tuathaan" );
		}
		/* Aiel lastname/title is done in room 31024 */
		if ( ch->race == race_lookup("ogier") )
		{
			char *temp;
		        char buf2[2];
			buf2[0] = ch->pcdata->last_name[0];
			buf2[1] = '\0';

			temp = ch->pcdata->last_name;
			ch->clan = clan_lookup( "ogier" );
			if ( ch->sex == 2 )
			{
				sprintf( buf, "{gdaughter of {Y%s{y%s{x", buf2, str_dup(++temp) );
			}
			else
			{
				sprintf( buf, "{gson of {Y%s{y%s{x", buf2, str_dup(++temp) );
			}
			free_string(ch->pcdata->last_name);
			ch->pcdata->last_name = str_dup( buf );
		}
		else if ( ch->race == race_lookup("seanchan"))
				{
					ch->clan = clan_lookup( "seanchan" );
					sprintf( buf, "%s{x", ch->pcdata->last_name );
					free_string(ch->pcdata->last_name);
					ch->pcdata->last_name = str_dup( buf );
		}

		else if ( ch->race == race_lookup("seafolk"))
		{
			sprintf( buf, "%s{x", ch->pcdata->last_name );
			free_string(ch->pcdata->last_name);
			ch->pcdata->last_name = str_dup( buf );
			ch->clan = clan_lookup( "athaanmiere" );
		}

        do_function( ch, &do_continue, "" );
        return;
	}
	else if ( room == 31003 )
	{
		if ( !str_prefix( arg, "male" ) )
		{
			ch->sex = SEX_MALE;
			ch->pcdata->true_sex = SEX_MALE;
			stc( "{wYou have chosen to be male.{x\n\r", ch );
			do_function( ch, &do_continue, "" );
			return;
		}
		else if ( !str_prefix( arg, "female" ) )
		{
			ch->sex = SEX_FEMALE;
			ch->pcdata->true_sex = SEX_FEMALE;
			stc( "{wYou have chosen to be female.{x\n\r", ch );
			do_function( ch, &do_continue, "" );
			return;
		}
		else
		{
			stc( "{wThat doesn't seem to be an appropriate gender. You{x\n\r", ch );
			stc( "{wcan choose to be either male or female. Make your{x\n\r", ch );
			stc( "{wchoice by typing either SELECT MALE or by typing.{x\n\r", ch );
			stc( "{wSELECT FEMALE.{x\n\r", ch );
			return;
		}
	}
	else if ( room == 31024 )
	{
		int x;
		if ( ch->race != race_lookup( "aiel" ) )
		{
			stc( "{wYou have no need to choose an Aiel clan.{x\n\r", ch );
			return;
		}

		if (arg[0] == '\0' )
		{
	    		stc( "{wThat does not appear to be valid Aiel clan. You can type SHOW CLANS{x\n\r",ch);
	    		stc( "{wto see the list of clans available to you.{x\n\r",ch);
			return;
		}
		for ( x = 0; x <= 11; x++ )
		{
			if ( !str_prefix( arg, aiel_clan_table[x].name ) )
			{
				char buf[MSL];
				free_string( ch->pcdata->aiel_clan );
				ch->pcdata->aiel_clan = str_dup(capitalize( aiel_clan_table[x].name ));
				free_string( ch->pcdata->last_name );
				sprintf( buf, "{wof the %s Aiel{x", ch->pcdata->aiel_clan );
				ch->pcdata->last_name = str_dup( buf );
    	    		        ptc( ch, "You have chosen %s as your clan.\n\r", capit2( aiel_clan_table[x].name ) );
				char_from_room( ch );
				char_to_room( ch, get_room_index( 31005 ) );
				do_function( ch, &do_look, "auto" );
	        	return;
	         	}
		}
    	stc( "{wThat does not appear to be valid Aiel clan. You can type SHOW CLANS{x\n\r",ch);
    	stc( "{wto see the list of clans available to you.{x\n\r",ch);
	    return;
	}
	else if ( room == 31006 )
	{
		int x;
	/* 34 eye colors */
		for ( x = 0; x < 34; x++ )
		{
			if ( !str_prefix( arg, eye_color_table[x].name ) )
			{
				ch->pcdata->eye = x;
    	    	ptc( ch, "You have chosen %s as your eye color.\n\r", capit2( eye_color_table[x].name ) );
    		    do_function( ch, &do_continue, "" );
	        	return;
	         }
		}
	    stc( "{wThat does not appear to be valid eye color. You can type SHOW EYES{x\n\r",ch);
	    stc( "{wto see the list of eye colors available to you.{x\n\r",ch);
	    return;
	}
	else if ((room == 3700 || room == 31013) && ch->level <= 5)
	{
		if (!str_prefix(arg, "stats") || !str_prefix(arg, "attributes"))
		{
			if ( ch->pcdata->stat_rolls <= 0 )
			{
				stc( "{wYou have no stat rolls left.{x\n\r", ch );
				if (room == 31013)	stc("{wYou should {WCONTINUE{w with the tour.", ch);
				return;
			}
			roll_abilityscores( ch );
			return;
		}

		if (!str_prefix(arg, "spheres") || !str_prefix(arg, "op"))
		{
			if ( ch->pcdata->touch_tries <= 0 )
			{
				stc( "{wYou have no OP rolls left.{x\n\r", ch );
				if (room == 31013)	stc("{wYou should {WCONTINUE{w with the tour.", ch);
				return;
			}
			roll_onepower( ch );
			return;
		}

	    stc("You can use 'select stats' to reroll your stats or 'select op' to reroll your OP spheres.\n\r", ch);
	    return;
	}
	else if (room == 31007)
	{
		int x;
	/* 6 hair colors */
		for ( x = 0; x < 6; x++ )
		{
			if ( !str_prefix( arg, hair_color_table[x].name ) )
			{
				/*if ( !hair_color_table[x].trolloc && ch->race == race_lookup( "trolloc" ) )
				{
				    stc( "{wThat hair color is not for trollocs. You can type SHOW HAIRCOLOR{x\n\r",ch );
			    	stc( "{wto see the list of hair colors available to you.{x\n\r",ch );
				    return;
				}
				else if ( hair_color_table[x].trolloc && ch->race != race_lookup( "trolloc" ) )
				{
		    		stc( "{wThat hair color is for trollocs only. You can type SHOW HAIRCOLOR{x\n\r",ch );
		    		stc( "{wto see the list of hair colors available to you.{x\n\r",ch );
				    return;
				}*/
				ch->pcdata->hair1 = x;

    	    	ptc( ch, "You have chosen %s as your hair color.\n\r", capit2( hair_color_table[x].name ) );
    		    do_function( ch, &do_continue, "" );
	        	return;
			}
		}
	    stc( "{wThat does not appear to be valid hair color. You can type SHOW HAIRCOLOR{x\n\r",ch);
	    stc( "{wto see the list of hair colors available to you.{x\n\r",ch);
	    return;
	}
	else if ( room == 31008 )
	{
		int x;
	/* 10 hair shades */
		for ( x = 0; x < 10; x++ )
		{
			if ( !str_prefix( arg, hair_shade_table[x].name ) )
			{
				ch->pcdata->hair2 = x;
    	    	ptc( ch, "You have chosen %s as your hair shade.\n\r", capit2( hair_shade_table[x].name ) );
    		    do_function( ch, &do_continue, "" );
	        	return;
			}
		}
	    stc( "{wThat does not appear to be valid hair shade. You can type SHOW HAIRSHADES{x\n\r",ch);
	    stc( "{wto see the list of hair shades available to you.{x\n\r",ch);
	    return;
	}
	else if ( room == 31009 )
	{
		int x;
	/* 8 hair styles */
		for ( x = 0; x < 8; x++ )
		{
			if ( !str_prefix( arg, hair_style_table[x].name ) )
			{
				ch->pcdata->hair3 = x;
    	    	ptc( ch, "You have chosen %s as your hair style.\n\r", capit2( hair_style_table[x].name ) );
    		    do_function( ch, &do_continue, "" );
	        	return;
			}
		}
	    stc( "{wThat does not appear to be valid hair style. You can type SHOW HAIRSTYLES{x\n\r",ch);
	    stc( "{wto see the list of hair styles available to you.{x\n\r",ch);
	    return;
	}
	else if ( room == 31010 )
	{
		int x;
	/* 9 hair lengths */
		for ( x = 0; x < 9; x++ )
		{
			if ( !str_prefix( arg, hair_length_table[x].name ) )
			{
				ch->pcdata->hair4 = x;
    	    	ptc( ch, "You have chosen %s as your hair length.\n\r", capit2( hair_length_table[x].name ) );
    		    do_function( ch, &do_continue, "" );
	        	return;
			}
		}
	    stc( "{wThat does not appear to be valid hair style. You can type SHOW HAIRSTYLES{x\n\r",ch);
	    stc( "{wto see the list of hair styles available to you.{x\n\r",ch);
	    return;
	}
	else if ( room == 31005 )
	{
		int x;
		bool found = FALSE;
		for ( x = 0; x <= 8; x++ )
		{
			if ( !str_prefix( arg, background_table[x].name ) )
			{
		        ptc( ch, "You have chosen %s as your background.\n\r", capit2( background_table[x].name ) );
		        found = TRUE;
		        break;
			}
		}
		if ( !found )
		{
		    stc( "{wThis does not appear to be valid background. You can type SHOW BACKGROUNDS{x\n\r",ch );
		    stc( "{wto see the list of backgrounds available to you.{x\n\r",ch );
		    return;
		}
		else
		{
			int old_hist = ch->hist_flags;
			ch->hist_flags = (~(HIST_SOLDIER|HIST_MERCHANT|HIST_BEGGAR|HIST_ARTIST|HIST_FARMER|HIST_CHILD|HIST_MUSICIAN|HIST_SAILOR|HIST_NOBLE) & ch->hist_flags);
			switch ( x )
			{
				default:
				    stc( "{wThat does not appear to be valid background. You can type SHOW BACKGROUNDS{x\n\r",ch );
			    	stc( "{wto see the list of backgrounds available to you.{x\n\r",ch );
				ch->hist_flags = old_hist;
				    return;
				break;
				case 0:  SET_BIT( ch->hist_flags, HIST_SOLDIER ); break;
				case 1:  SET_BIT( ch->hist_flags, HIST_MERCHANT ); break;
				case 2:  SET_BIT( ch->hist_flags, HIST_BEGGAR ); break;
				case 3:  SET_BIT( ch->hist_flags, HIST_ARTIST ); break;
				case 4:  SET_BIT( ch->hist_flags, HIST_FARMER ); break;
				case 5:  SET_BIT( ch->hist_flags, HIST_CHILD ); break;
				case 6:  SET_BIT( ch->hist_flags, HIST_MUSICIAN ); break;
				case 7:  SET_BIT( ch->hist_flags, HIST_SAILOR ); break;
				case 8:  SET_BIT( ch->hist_flags, HIST_NOBLE ); break;
			}
		}
//       do_function( ch, &do_continue, "" );
/* removed for skipping hair/eye crap -kvash */
                char_from_room( ch );
                char_to_room(ch, get_room_index(31012));
                //stc( "{WType {CLook{W to view your surroundings.{x\n\r", ch);
                do_function( ch, &do_look, "auto" );

        //do_function( ch, &do_continue, "" );
        return;
	}
	else if ( room == 31025 )
	{
			int x;
			if ( ch->race != race_lookup( "trolloc" ) )
			{
				stc( "{wYou have no need to choose a Trolloc clan.{x\n\r", ch );
				return;
			}
			if (arg[0] == '\0' )
			{
		    		stc( "{wThat does not appear to be valid Trolloc clan. You can type SHOW CLANS{x\n\r",ch);
		    		stc( "{wto see the list of CLANS available to you.{x\n\r",ch);
				return;
			}
			for ( x = 0; x <= 11; x++ )
		    {
			if ( !str_prefix( arg, trolloc_face_table[x].name ) )
			{
				ptc( ch, "You have chosen %s {xas your clan.\n\r", capit2( trolloc_face_table[x].trolloc_clan ) );
				free_string(ch->pcdata->aiel_clan);
				ch->pcdata->aiel_clan = str_dup(trolloc_face_table[x].trolloc_clan);
				char_from_room( ch );
				char_to_room( ch, get_room_index( 31006 ) );
				do_function( ch, &do_look, "auto" );
				ch->rank = 0;
				return;
			}
		}
    	stc( "{wThat does not appear to be valid Trolloc clan. You can type SHOW CLANS{x\n\r",ch);
    	stc( "{wto see the list of clans available to you.{x\n\r",ch);
	    return;
	}

}
void do_show( CHAR_DATA *ch, char *argument )
{
    int i = 0;
	if ( !str_prefix( argument, "backgrounds" ) )
	{
	    for ( i = 0; background_table[i].name != NULL; i++ )
	    {
			ptc( ch, "{w[{W%11s{w] %s{x\n\r", capit2( background_table[i].name ), background_table[i].bonus_str );
		}
		return;
	}
	else if ( !str_prefix( argument, "clans" ) )
	{
		int col = 0;
		if ( ch->race == race_lookup( "aiel" ) )
		{
		    for ( i = 0; aiel_clan_table[i].name != NULL; i++ )
		    {
	            if ( aiel_clan_table[i].name == NULL || aiel_clan_table[i].name[0] == '\0' )
	            {
	                ptc( ch, "An error occured processing Aiel clans at position %d.\n\r", i );
	                break;
	            }
				ptc( ch, "{w[{W%13s{w]   {x",
					capit2( aiel_clan_table[i].name ) );
				col++;
				if ( col >= 3 )
				{
					col = 0;
					stc( "\n\r", ch );
				}
			}
			return;
		}
		else	if ( ch->race == race_lookup( "trolloc" ) )
			  {
					for ( i = 0; trolloc_face_table[i].name != NULL; i++ )
			        {
		              if ( trolloc_face_table[i].name == NULL || trolloc_face_table[i].name[0] == '\0' )
		              {
		                ptc( ch, "An error occured processing Trolloc clan at position %d.\n\r", i );
		                break;
		              }
		            	ptc( ch, "{w[{W%13s{w]   {x",
							capit2( trolloc_face_table[i].name ) );
						col++;
						if ( col >= 3 )
						{
							col = 0;
							stc( "\n\r", ch );
						}
				 }
				 return;
		}
		else
		{
			stc( "{WYou have no reason to show clans.{x\n\r", ch );
			return;
		}
	}
	else if ( !str_prefix( argument, "eyes" ) || !str_prefix( argument, "eyecolors" ) )
	{
		int col = 0;
	    for ( i = 0; eye_color_table[i].name != NULL; i++ )
	    {
            if ( eye_color_table[i].name == NULL || eye_color_table[i].name[0] == '\0' )
            {
                ptc( ch, "An error occured processing eye colors at position %d.\n\r", i );
                break;
            }
			ptc( ch, "{w[{W%13s{w]   {x",
				capit2( eye_color_table[i].name ) );
			col++;
			if ( col >= 3 )
			{
				col = 0;
				stc( "\n\r", ch );
			}
		}
		return;
	}
	else if ( !str_prefix( argument, "haircolor" ) )
	{
		int col = 0;
	    for ( i = 0; hair_color_table[i].name != NULL; i++ )
	    {
            if ( hair_color_table[i].name == NULL || hair_color_table[i].name[0] == '\0' )
            {
                ptc( ch, "An error occured processing hair colors at position %d.\n\r", i );
                break;
            }
               		ptc( ch, "{w[{W%13s{w]   {x",
					capit2( hair_color_table[i].name ) );
				col++;
				if ( col >= 3 )
				{
					col = 0;
					stc( "\n\r", ch );
				}
			}
		return;

	}
	else if ( !str_prefix( argument, "hairshades" ) )
	{
		int col = 0;
	    for ( i = 0; hair_shade_table[i].name != NULL; i++ )
	    {
            if ( hair_shade_table[i].name == NULL || hair_shade_table[i].name[0] == '\0' )
            {
                ptc( ch, "An error occured processing hair shades at position %d.\n\r", i );
                break;
            }
			ptc( ch, "{w[{W%13s{w]   {x",
				capit2( hair_shade_table[i].name ) );
			col++;
			if ( col >= 3 )
			{
				col = 0;
				stc( "\n\r", ch );
			}
		}
		return;
	}
	else if ( !str_prefix( argument, "hairstyles" ) )
	{
		int col = 0;
	    for ( i = 0; hair_style_table[i].name != NULL; i++ )
	    {
            if ( hair_style_table[i].name == NULL || hair_style_table[i].name[0] == '\0' )
            {
                ptc( ch, "An error occured processing hair styles at position %d.\n\r", i );
                break;
            }
			ptc( ch, "{w[{W%13s{w]   {x",
				capit2( hair_style_table[i].name ) );
			col++;
			if ( col >= 3 )
			{
				col = 0;
				stc( "\n\r", ch );
			}
		}
		return;
	}
	else if ( !str_prefix( argument, "hairlengths" ) )
	{
		int col = 0;
	    for ( i = 0; hair_length_table[i].name != NULL; i++ )
	    {
            if ( hair_length_table[i].name == NULL || hair_length_table[i].name[0] == '\0' )
            {
                ptc( ch, "An error occured processing hair lengths at position %d.\n\r", i );
                break;
            }
			ptc( ch, "{w[{W%15s{w]   {x",
				capit2( hair_length_table[i].name ) );
			col++;
			if ( col >= 3 )
			{
				col = 0;
				stc( "\n\r", ch );
			}
		}
		return;
	}
	else if ( !str_prefix( argument, "remortreqs" ) )
	{
		int col = 0;
		for ( i = 0; race_table[i].name != NULL; i++ )
		{
	        if ( !str_cmp( race_table[i].name, "unique" ) )
	            continue;
	        if ( !race_table[i].pc_race )
	            continue;
	        if ( race_table[i].remort >= 1000 )
	            continue;
			ptc( ch, " {W%15s{w = {W%4d{x", race_table[i].name, race_table[i].remort );
			col++;
			if ( col >= 2 )
			{
				stc("\n\r", ch);
				col = 0;
			}
		}
		return;
	}
	else if ( !str_prefix( argument, "races" ) )
	{
		stc( "The more times you have been Reborn, the more races available.\n\r",ch);
		stc( "The following races are available for You:\n\r\n\r",ch);
	    for ( i = 0; race_table[i].name != NULL; i++ )
	    {
	        if ( !str_cmp( race_table[i].name, "unique" ) )
	            continue;
	        if ( !race_table[i].pc_race )
	            continue;
	        if ( ch->pcdata->rebirth > 0 )
	        {
			    if ( ch->pcdata->rebirth >= race_table[i].remort && race_table[i].name != NULL )
	                ptc( ch, "{w[{W%12s{w] {w%-69s {x\n\r", capit2( race_table[i].name ), race_table[i].descr );
	            if ( race_table[i].name == NULL || race_table[i].name[0] == '\0' )
	            {
	                ptc( ch, "An error occured processing races at position %d.\n\r", i );
	                break;
	            }
	        }
	        else
	        {
	            if ( race_table[i].remort )
	                continue;
	            if ( race_table[i].name != NULL )
	                ptc( ch, " {w[{W%10s{w], {w%-69s {x\n\r", capit2( race_table[i].name ), race_table[i].descr );
	            if ( race_table[i].name == NULL || race_table[i].name[0] == '\0' )
	            {
	                ptc( ch, "An error occured processing races at position %d.\n\r", i );
	                break;
	            }
	        }
	    }
	    return;
	}
}
void do_tour( CHAR_DATA *ch, char *argument )
{
	int toroom = 0;
	if ( !ch || ch->in_room == NULL )
		return;
	if ( ch->in_room->vnum >= 31000 && ch->in_room->vnum <= 31999 )
	{
		stc( "{wYou're already on the tour.{x\n\r", ch );
		return;
	}
	if ( ch->level >= 20 )
	{
		stc( "{wYou can't go on the tour. You're too big!{x\n\r", ch );
		return;
	}
	toroom = 31000;
	char_from_room( ch );
	char_to_room( ch, get_room_index( toroom ) );
	do_function( ch, &do_look, "auto" );
	ch->pcdata->newbie = 1;
	stc( "{wYou begin the tour. You are now in Newbie World.{x\n\r", ch );
	act( "$n leaves on a tour of the MUD.", ch, NULL, NULL, TO_ROOM );
	if ( ch->pcdata->rebirth >= 1 )
	{
		stc( "{wSince you have been reborn there are areas of the tour that you may.{x\n\r", ch );
		stc( "{wwish to skip. Doing so will not harm your character. You may also{x\n\r", ch );
		stc( "{wtype finish to skip the tour entirely.{x\n\r", ch );
	}
	return;
}
void do_finish( CHAR_DATA *ch, char *argument )
{
	int nroom = 0, toroom = 0;
	if ( !ch || ch->in_room == NULL )
		return;

	nroom = ch->in_room->vnum;

	if ( nroom < 31000 || nroom > 31999 )
	{
		stc( "There's no reason for you to need this command.\n\r", ch );
		return;
	}
	if ( !check_valid_newbie( ch ) )
	{
		stc( "{WBut your character does not seem to be complete yet!{x\n\r", ch );
		return;
	}
	toroom = 3736;
	char_from_room( ch );
	char_to_room( ch, get_room_index( toroom ) );
	do_function( ch, &do_look, "auto" );
	ch->pcdata->newbie = 0;
	ch->pcdata->allowed = 1;
	stc( "{wTurning on exit display, autogold, autoloot, autoassist, autosplit, autosac,{x\n\r", ch );
	stc( "{wcolor, pdeath, penter, autofight, autorecall, counter, dodge, parry, war, autoh2h.{x\n\r", ch );
	stc( "{wUse TOGGLE to customize these settings.{x\n\r", ch );
	/*
    SET_BIT(ch->act,PLR_AUTOEXIT);
    SET_BIT(ch->act,PLR_AUTOGOLD);
    SET_BIT(ch->act,PLR_AUTOLOOT);
    SET_BIT(ch->act,PLR_AUTOASSIST);
    SET_BIT(ch->act,PLR_AUTOSPLIT);
    SET_BIT(ch->act,PLR_AUTOSAC);
    SET_BIT(ch->act,PLR_COLOUR);
    SET_BIT(ch->comm,COMM_PDEATH);
    SET_BIT(ch->comm,COMM_PENTER);
    SET_BIT(ch->comm,COMM_PREBIRTH);
    SET_BIT(ch->war_flags,WAR_AUTOFIGHT);
    SET_BIT(ch->auto_flags,AUTO_AUTORECALL);
    SET_BIT(ch->auto_flags,AUTO_COUNTER);
    SET_BIT(ch->auto_flags,AUTO_DODGE);
    SET_BIT(ch->auto_flags,AUTO_PARRY);
    SET_BIT(ch->auto_flags,AUTO_WAR);
    SET_BIT(ch->auto_flags,AUTO_AUTOH2H);
    */
	stc( "{wYou finish the tour. You are now in the School.{x\n\r", ch );
	if( ch->pcdata)    /* that better be not needed */
	{
		DESCRIPTOR_DATA *d_mentor;
		ch->pcdata->allowed = 1;
		for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
		{
			if ( d_mentor->connected == CON_PLAYING
			&& IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
	        {
           	    act_new("{w({WMentor Alert{w){W::{x $N has finished the tour{x",
	            d_mentor->character,NULL,ch,TO_CHAR,POS_DEAD);
	        }
		}
		SET_BIT(ch->act,PLR_AUTOEXIT);
	    SET_BIT(ch->act,PLR_AUTOGOLD);
	    SET_BIT(ch->act,PLR_AUTOLOOT);
	    SET_BIT(ch->act,PLR_AUTOASSIST);
	    SET_BIT(ch->act,PLR_AUTOSPLIT);
	    SET_BIT(ch->act,PLR_AUTOSAC);
	    SET_BIT(ch->act,PLR_COLOUR);
	    SET_BIT(ch->comm,COMM_PDEATH);
	    SET_BIT(ch->comm,COMM_PENTER);
	    SET_BIT(ch->comm,COMM_PREBIRTH);
	    SET_BIT(ch->war_flags,WAR_AUTOFIGHT);
	    SET_BIT(ch->auto_flags,AUTO_AUTORECALL);
	    SET_BIT(ch->auto_flags,AUTO_COUNTER);
	    SET_BIT(ch->auto_flags,AUTO_DODGE);
	    SET_BIT(ch->auto_flags,AUTO_PARRY);
	    SET_BIT(ch->auto_flags,AUTO_WAR);
	    SET_BIT(ch->auto_flags,AUTO_AUTOH2H);

	}
	set_features( ch );   /* in case they changed a couple, but didn't go to the end */
	compose_title( ch );
	if ( ch->pcdata->rebirth <= 0 )
	{
		do_help( ch, "short_policy" );
	}
	act( "{W$n finishes the tour and appears in the room!{x", ch, NULL, NULL, TO_ROOM );
	return;
}
bool check_valid_next_room( CHAR_DATA *ch, int tmproom )
{
	ROOM_INDEX_DATA *proom;

	proom = get_room_index( tmproom );

	if ( proom == NULL )
	{
		stc( "{wThe Newbie World does not appear to be finished right now.{x\n\r", ch );
		stc( "{wTry back again later. Until then, type {WFINISH{w to move on.{x\n\r", ch );
		return FALSE;
	}
	return TRUE;
}
bool check_to_room( CHAR_DATA *ch, int tmproom )
{
	ROOM_INDEX_DATA *proom;
	CHAR_DATA *people;

	proom = get_room_index( tmproom );

	if ( proom == NULL )
	{
		stc( "{wThe Newbie World does not appear to be finished right now.{x\n\r", ch );
		stc( "{wTry back again later. Until then, type {WFINISH{w to move on.{x\n\r", ch );
		return TRUE;
	}

	for ( people = proom->people; people != NULL; people = people->next_in_room )
	{
		if( IS_NPC( people ) )
			continue;
		if ( !people->desc )
		{
			extract_char( people, TRUE );
			return FALSE;
		}
		else
		{
			ptc( people, "{w%s, there is someone else waiting for you to {WCONTINUE{w.\n\r", people->name );

			stc( "{wPlease try to hurry so that they may learn the game, too.{w.\n\r", people );
			return TRUE;
		}
	}
	return FALSE;
}
void check_mob_extract( CHAR_DATA *ch, int tmproom )
{
	ROOM_INDEX_DATA *proom;
	CHAR_DATA *people, *pnext;

	proom = get_room_index( tmproom );

	if ( proom == NULL )
	{
		stc( "{wThe Newbie World does not appear to be finished right now.{x\n\r", ch );
		stc( "{wTry back again later. Until then, type {WFINISH{w to move on.{x\n\r", ch );
		return;
	}
	if ( proom->vnum == 31014 || proom->vnum == 31015 )
		return;

	for ( people = proom->people; people != NULL; people = pnext )
	{
	    pnext = people->next_in_room;
		/* double wrapped for safety */
	    if ( people == ch )
	    	continue;
	    if ( IS_NPC( people )
	    && !IS_SET( people->act,ACT_NOPURGE )
	    && people != ch )
	    {
			extract_char( people, TRUE );
		}
	}
	return;
}
void do_continue( CHAR_DATA *ch, char *argument )
{
	int nroom = 0, toroom = 0, tmproom = 0;
	OBJ_DATA *obj = NULL, *sack = NULL;
	CHAR_DATA *mob = NULL;
	char descr[MSL];
	bool rollstats = FALSE, setup_punch = FALSE, skip_room = FALSE;
	bool checkrollstats = FALSE;
	if ( !ch || ch->in_room == NULL )
		return;

	nroom = ch->in_room->vnum;
	tmproom = nroom +1;

	if ( nroom < 31000 || nroom > 31999 )
	{
		stc( "There's no reason for you to need this command.\n\r", ch );
		return;
	}
	if ( !check_valid_next_room( ch, tmproom ) )
	{
		WAIT_STATE( ch, 3 );
		return;
	}
	/*
	if ( check_to_room( ch, tmproom ) )
	{
		stc( "{wSomeone else seems to be taking their time.{x\n\r", ch );
		stc( "{wWe'll try to hurry them along. Please wait a moment.{x\n\r", ch );
		WAIT_STATE( ch, 20 );
		return;
	}
	if ( nroom == 31015 )
	{
		if ( check_to_room( ch, 31016 )
		|| check_to_room( ch, 31017 )
		|| check_to_room( ch, 31018 )
		|| check_to_room( ch, 31019 ) )
		{
			stc( "{wSomeone else seems to be taking their time.{x\n\r", ch );
			stc( "{wWe'll try to hurry them along. Please wait a moment.{x\n\r", ch );
			WAIT_STATE( ch, 20 );
			return;
		}
	}
	*/
	clear_newbieroom( ch, tmproom );
	switch( nroom )
	{
		default: break;
		case 31000:
			obj = NULL;
			obj = create_object( get_obj_index( 31000 ), 0 );
			obj->cost = 0;
			obj_to_room( obj, get_room_index( 31001 ) );
			obj = NULL;
			obj = create_object( get_obj_index( 31001 ), 0 );
			obj->cost = 0;
			obj_to_room( obj, get_room_index( 31001 ) );
		break;
		case 31002:
			if ( ch->pcdata->rebirth >= 1 )
			{
				stc( "{wSince you are being reborn just type {WCONTINUE{w.\n\r", ch );
				skip_room = TRUE;
			}
			else
			{
				skip_room = TRUE;
				//stc( "{wTry saying {WREADY{w.\n\r", ch );
				//return;
			}
		break;
		case 31003:
			if ( ch->sex == 0 )
			{
				stc( "{wYou do not appear to have chosen a proper gender.{w\n\r", ch );
				return;
			}
			if ( ch->race == race_lookup( "seanchan" )
			|| ch->race == race_lookup( "aiel" )
			|| ch->race == race_lookup( "ogier" )
			|| ch->race == race_lookup( "nym" )
			|| ch->race == race_lookup( "seafolk" ) )
			{
				stc( "{wYou are in a life race. You cannot change your race.{w\n\r", ch );
				char_from_room( ch );
				char_to_room( ch, get_room_index( 31005 ) );
				do_function( ch, &do_look, "auto" );
				return;
			}
			else if ( ch->race == race_lookup( "trolloc" ) )
			{
							stc( "{wYou are in a life race. You cannot change your race.{w\n\r", ch );
							char_from_room( ch );
							char_to_room( ch, get_room_index( 31013 ) );
							do_function( ch, &do_look, "auto" );
							return;
			}
		break;
		case 31004:
			if ( ch->race == race_lookup( "unique" ) )
			{
				stc( "{wYou do not appear to have chosen a proper race.{w\n\r", ch );
				return;
			}
			if ( ch->race == race_lookup( "aiel" ) )
			{
				stc( "{wYou will be sent to a special room where you can choose your clan.{w\n\r", ch );
				if ( ch->race == race_lookup( "aiel" ) )
				{
					char_from_room( ch );
					char_to_room( ch, get_room_index( 31024 ) );
				}
				do_function( ch, &do_look, "auto" );
				return;
			}
		break;
		case 31005:
			if ( !IS_SET( ch->hist_flags, HIST_SOLDIER )
			&&   !IS_SET( ch->hist_flags, HIST_MERCHANT )
			&&   !IS_SET( ch->hist_flags, HIST_BEGGAR )
			&&   !IS_SET( ch->hist_flags, HIST_ARTIST )
			&&   !IS_SET( ch->hist_flags, HIST_FARMER )
			&&   !IS_SET( ch->hist_flags, HIST_CHILD )
			&&   !IS_SET( ch->hist_flags, HIST_MUSICIAN )
			&&   !IS_SET( ch->hist_flags, HIST_SAILOR )
			&&   !IS_SET( ch->hist_flags, HIST_NOBLE ) )
			{
				stc( "{wYou do not appear to have chosen a proper background.{w\n\r", ch );
				return;
			}
		break;
		case 31006:
			if ( ch->pcdata->eye == -1 )
			{
				stc( "{wYou do not appear to have chosen a proper eye color.{w\n\r", ch );
				return;
			}
		break;
		case 31007:
			if ( ch->pcdata->hair1 == -1 )
			{
				stc( "{wYou do not appear to have chosen a proper hair color.{w\n\r", ch );
				return;
			}
		break;
		case 31008:
			if ( ch->pcdata->hair2 == -1 )
			{
				stc( "{wYou do not appear to have chosen a proper hair sheen.{w\n\r", ch );
				return;
			}
		break;
		case 31009:
			if ( ch->pcdata->hair3 == -1 )
			{
				stc( "{wYou do not appear to have chosen a proper hair style.{w\n\r", ch );
				return;
			}
		break;
		case 31010:
			if ( ch->pcdata->hair4 == -1 )
			{
				stc( "{wYou do not appear to have chosen a proper hair length.{w\n\r", ch );
				return;
			}
			set_features( ch );
			sprintf( descr, "%s %s has %s eyes. %s wears %s %s-%s hair %s and %s.\n\r",
				ch->name, ch->pcdata->last_name, ch->eye,
				(ch->sex == 2) ? "She" : "He",
				(ch->sex == 2) ? "her" : "his",
				ch->hair2, ch->hair1, ch->hair3, ch->hair4 );
		    free_string( ch->description );
		    ch->description = str_dup( descr );
		break;
		case 31012:
			if ( ch->pcdata->rebirth <= 1 )
			{
				int i;
			    for ( i = 0; i < MAX_STATS; i++ )
			    {
				if ( ch->perm_stat[i] <= 3 )
				{
					rollstats = TRUE;
					break;
				}
			    }
			}
			else
			{
				checkrollstats = TRUE;
				ch->pcdata->stat_rolls = 9;
			}
		break;
		case 31013:
			/*
			mob = NULL;
			mob = create_mobile( get_mob_index( 31003 ) );
			char_to_room( mob, get_room_index( tmproom ) );
			*/
		break;
		case 31014:
			/*
			mob = NULL;
			mob = create_mobile( get_mob_index( 31002 ) );
			char_to_room( mob, get_room_index( tmproom ) );
			ch->train	 = 50;
			ch->practice = 25;
		    */
		break;
		case 31015:
			obj = NULL;
			sack = create_object( get_obj_index( 31003 ), 0 );
			sack->cost = 0;
			obj_to_room( sack, get_room_index( 31018 ) );
			obj = create_object( get_obj_index( 31004 ), 0 );
			obj->cost = 0;
			obj_to_obj( obj, sack );
			obj = NULL;
			obj = create_object( get_obj_index( 31005 ), 0 );
			obj->cost = 0;
			obj_to_obj( obj, sack );
			obj = NULL;
			obj = create_object( get_obj_index( 31006 ), 0 );
			obj->cost = 0;
			obj_to_obj( obj, sack );

			obj = NULL;
			sack = create_object( get_obj_index( 31012 ), 0 );
			sack->cost = 0;
			obj_to_room( sack, get_room_index( 31017 ) );
			obj = create_object( get_obj_index( 31009 ), 0 );
			obj->cost = 0;
			obj_to_obj( obj, sack );
			obj = NULL;
			obj = create_object( get_obj_index( 31010 ), 0 );
			obj->cost = 0;
			obj_to_obj( obj, sack );
			obj = NULL;
			obj = create_object( get_obj_index( 31011 ), 0 );
			obj->cost = 0;
			obj_to_obj( obj, sack );
		break;
		case 31016:
		case 31017:
		case 31018:
		case 31019:
			stc( "{wYou cannot continue from this room. Remember the instructions.{w\n\r", ch );
			if ( ch->in_room->vnum == 31016 )
				stc( "{wExplore the rooms east and west then go north.{w\n\r", ch );
			else if ( ch->in_room->vnum == 31017 )
				stc( "{wExplore the room two west then leave east and go north.{w\n\r", ch );
			else if ( ch->in_room->vnum == 31018 )
				stc( "{wExplore the room two east then leave west and go north.{w\n\r", ch );
			else if ( ch->in_room->vnum == 31019 )
				stc( "{wExplore this room. Read the description of the room for hints.{w\n\r", ch );
			else
				;
			return;
		break;
		case 31020:
			mob = NULL;
			mob = create_mobile( get_mob_index( 31000 ) );
			/* this is too much money  --Sil
		    mob->money[3][0] = 10;
		    mob->money[2][0] = 10;
		    	*/
			char_to_room( mob, get_room_index( tmproom ) );
			setup_punch = TRUE;
		break;
		case 31021:
			stc( "{wYou will have to leave {WDOWN{w before you can {WCONTINUE{w.{x\n\r", ch );
			return;
		break;
		case 31022:
			stc( "{wYou have completed the Tour. Just type {WFINISH to enter the world{x.{x\n\r", ch );
			return;
		break;
		case 31024:
			if ( ch->race == race_lookup( "aiel" ) )
			{
				if ( ( ch->pcdata->aiel_clan == NULL || ch->pcdata->aiel_clan[0] == '\0' ) )
				{
					stc( "{WYou appear to be Aiel but do not have a valid clan!{x\n\r", ch );
					return ;
				}
				char_from_room( ch );
				char_to_room( ch, get_room_index( 31005 ) );
			}
			else
			{
				stc( "{wYou're not supposed to be in this room! Sending you back...{x\n\r", ch );
				char_from_room( ch );
				char_to_room( ch, get_room_index( 31000 ) );
			}
			do_function( ch, &do_look, "auto" );
			return;
		break;
	}
	if ( get_room_index( nroom+1 ) == NULL )
	{
		stc( "{wThis is the end of the tour. We'll now send you to the School.{x\n\r", ch );
		char_from_room( ch );
		char_to_room( ch, get_room_index( 3700 ) );
		do_function( ch, &do_look, "auto" );
		return;
	}

	check_mob_extract( ch, nroom );

	if ( skip_room )
		toroom = nroom +2;
	else
		toroom = nroom +1;

	char_from_room( ch );
	char_to_room( ch, get_room_index( toroom ) );
	do_function( ch, &do_look, "auto" );

	if ( rollstats )
		roll_abilityscores( ch );
	if ( checkrollstats )
	{
		stc( "{wSince you are being reborn you may choose to skip rolling your stats{x\n\r", ch );
		stc( "{wand keep the ones you have. Rolling new stats may or may not give you{x\n\r", ch );
		stc( "{wbetter scores but you will have nine chances to try.{x\n\r", ch );
	}
	if ( setup_punch )
	{
		stc( "{wYou have now learned the HAND TO HAND skill!{x\n\r", ch );
		stc( "{wYou have now learned the PUNCH skill!{x\n\r", ch );
		stc( "{wLearning to use hand to hand fighting is very important in TSE{x\n\r", ch );
		stc( "{was most other fighting skills require you to have your hand to{x\n\r", ch );
		stc( "{whand skill at 85% or greater before learning new fighting skills.{x\n\r", ch );
		stc( "{wTo use punch type PUNCH while in melee range with your opponent.{x\n\r", ch );
		if (ch->pcdata->learned[gsn_hand_to_hand] < 50)
			ch->pcdata->learned[gsn_hand_to_hand] = 50;
		if (ch->pcdata->learned[gsn_punch] < 50)
			ch->pcdata->learned[gsn_punch] = 50;
	}
	return;
}
