/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "aww.h"
#include "recycle.h"
#include "tables.h"

/* globals from db.c for load_notes */
#if !defined(macintosh)
extern  int     _filbuf         args( (FILE *) );
#endif
extern FILE *                  fpArea;
extern char                    strArea[MAX_INPUT_LENGTH];

/* local procedures */
void load_thread(char *name, PNOTE_DATA **list, int type, time_t free_time);
void parse_pnote(CHAR_DATA *ch, char *argument, int type);
bool hide_pnote(CHAR_DATA *ch, PNOTE_DATA *pnote);

PNOTE_DATA *pnote_list;
PNOTE_DATA *idea_list;
PNOTE_DATA *penalty_list;
PNOTE_DATA *news_list;
PNOTE_DATA *changes_list;
PNOTE_DATA *roleplay_list;
PNOTE_DATA *restring_list;
PNOTE_DATA *event_list;
int count_spool(CHAR_DATA *ch, PNOTE_DATA *spool)
{
    int count = 0;
    PNOTE_DATA *pnote;

    for (pnote = spool; pnote != NULL; pnote = pnote->next)
	if (!hide_pnote(ch,pnote))
	    count++;

    return count;
}
void do_unread(CHAR_DATA *ch)
{
    char buf[MAX_STRING_LENGTH];
    int count, total = 0;
    bool found = FALSE;

    if (IS_NPC(ch))
	return;
	stc("\n\r",ch);
    sprintf(buf,"  {r){R------------------------------------------------{r({x\n\r");
	stc(buf,ch);
    sprintf(buf,"  {r| {wYou have unread items in the following areas:  {r|{x\n\r");
	stc(buf,ch);
    if ((count = count_spool(ch,event_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wEvents waiting                           {r|{x\n\r",
	    count );
	stc(buf,ch);
    }
    if ((count = count_spool(ch,news_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wNews waiting                             {r|{x\n\r",
	    count );
	stc(buf,ch);
    }
    if ((count = count_spool(ch,changes_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wChanges waiting                          {r|{x\n\r",
	    count );
    stc(buf,ch);
    }
    if ((count = count_spool(ch,pnote_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wNotes waiting                            {r|\n\r",
	    count );
	stc(buf,ch);
    }
    if ((count = count_spool(ch,idea_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wIdeas waiting                            {r|{x\n\r",
	    count );
	stc(buf,ch);
    }
    if ((count = count_spool(ch,roleplay_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wRoleplays waiting                        {r|{x\n\r",
	    count );
	stc(buf,ch);
    }
    if (IS_IMMORTAL(ch) && (count = count_spool(ch,penalty_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wPenalties waiting                        {r|{x\n\r",
	    count );
	stc(buf,ch);
    }
    if (IS_SET(ch->war_flags,WAR_MODERATOR) && (count = count_spool(ch,restring_list)) > 0)
    {
	found = TRUE;
	total += count;
	sprintf(buf,"  {r| {R({w%3d{R) {wRestrings waiting                        {r|{x\n\r",
	    count );
	stc(buf,ch);
    }
    if (!found)
    {
	stc("  {r| {wYou have no unread articles                    {r|{x\n\r",ch);
	}
	else if ( found )
	{
	ptc( ch, "  {r| {wYou have %3d total articles to read.           {r|\n\r{x", total );
	}
    sprintf( buf,"  {r){R------------------------------------------------{r({x\n\r");
	stc(buf,ch);
    sprintf( buf,"  {wType Unread to see this list again.{x\n\r");
	stc(buf,ch);
}

void do_pnote(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_NOTE);
}

void do_idea(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_IDEA);
}

void do_penalty(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_PENALTY);
}

void do_news(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_NEWS);
}

void do_changes(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_CHANGES);
}
void do_roleplay(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_ROLEPLAY);
}
void do_nstring(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_RESTRING);
}
void do_event(CHAR_DATA *ch,char *argument)
{
    parse_pnote(ch,argument,PNOTE_EVENT);
}

void save_pnotes(int type)
{
    FILE *fp;
    char *name;
    PNOTE_DATA *pnote;

    switch (type)
    {
	default:
	    return;
	case PNOTE_NOTE:
	    name = PNOTE_FILE;
	    pnote = pnote_list;
	    break;
	case PNOTE_IDEA:
	    name = IDEA_FILE;
	    pnote = idea_list;
	    break;
	case PNOTE_PENALTY:
	    name = PENALTY_FILE;
	    pnote = penalty_list;
	    break;
	case PNOTE_NEWS:
	    name = NEWS_FILE;
	    pnote = news_list;
	    break;
	case PNOTE_CHANGES:
	    name = CHANGES_FILE;
	    pnote = changes_list;
	    break;
	case PNOTE_ROLEPLAY:
	    name = ROLEPLAY_FILE;
	    pnote = roleplay_list;
	    break;
	case PNOTE_RESTRING:
	    name = RESTRING_FILE;
	    pnote = restring_list;
	    break;
	case PNOTE_EVENT:
	    name = EVENT_FILE;
	    pnote = event_list;
	    break;

    }

    if ( ( fp = fopen( name, "w" ) ) == NULL )
    {
	perror( name );
    }
    else
    {
	for ( ; pnote != NULL; pnote = pnote->next )
	{
	    fprintf( fp, "Sender  %s~\n", pnote->sender);
	    fprintf( fp, "Date    %s~\n", pnote->date);
	    fprintf( fp, "Stamp   %ld\n", pnote->date_stamp);
	    fprintf( fp, "To      %s~\n", pnote->to_list);
	    fprintf( fp, "Subject %s~\n", pnote->subject);
	    fprintf( fp, "Text\n%s~\n",   pnote->text);
	}
	fclose( fp );
   	return;
    }
}
void load_pnotes(void)
{
    load_thread(PNOTE_FILE,&pnote_list, PNOTE_NOTE, 14*24*60*60);
    load_thread(IDEA_FILE,&idea_list, PNOTE_IDEA, 28*24*60*60);
    load_thread(PENALTY_FILE,&penalty_list, PNOTE_PENALTY, 0);
    load_thread(NEWS_FILE,&news_list, PNOTE_NEWS, 0);
    load_thread(CHANGES_FILE,&changes_list,PNOTE_CHANGES, 0);
    load_thread(ROLEPLAY_FILE,&roleplay_list,PNOTE_ROLEPLAY, 0);
    load_thread(RESTRING_FILE,&restring_list,PNOTE_RESTRING, 0);
    load_thread(EVENT_FILE,&event_list,PNOTE_EVENT, 0);
}

void load_thread(char *name, PNOTE_DATA **list, int type, time_t free_time)
{
    FILE *fp;
    PNOTE_DATA *pnotelast;
    char *string;

    if ( ( fp = fopen( name, "r" ) ) == NULL )
	{
	printf("prool debug: can't open name = '%s'\r\n", name);
	return;
	}

    pnotelast = NULL;
    for ( ; ; )
    {
	PNOTE_DATA *pnote;
	char letter;

	do
	{
	    letter = getc( fp );
            if ( feof(fp) )
            {
                fclose( fp );
                return;
            }
        }
        while ( isspace(letter) );
        ungetc( letter, fp );

        pnote           = alloc_perm( sizeof(*pnote) );

        if ( str_cmp( fread_word( fp ), "sender" ) )
            break;

	string = fread_string(fp);
        pnote->sender   = str_dup(string);
	free_string(string);

        if ( str_cmp( fread_word( fp ), "date" ) )
            break;

	string = fread_string(fp);
        pnote->date   = str_dup(string);
	free_string(string);

        if ( str_cmp( fread_word( fp ), "stamp" ) )
            break;

        pnote->date_stamp = fread_number(fp);

        if ( str_cmp( fread_word( fp ), "to" ) )
            break;

	string = fread_string(fp);
        pnote->to_list   = str_dup(string);
	free_string(string);

        if ( str_cmp( fread_word( fp ), "subject" ) )
            break;

	string = fread_string(fp);
        pnote->subject   = str_dup(string);
	free_string(string);

        if ( str_cmp( fread_word( fp ), "text" ) )
            break;

	string = fread_string(fp);
        pnote->text   = str_dup(string);
	free_string(string);

        if (free_time && pnote->date_stamp < current_time - free_time)
        {
	    free_pnote(pnote);
            continue;
        }

	pnote->type = type;

        if (*list == NULL)
            *list           = pnote;
        else
            pnotelast->next     = pnote;

        pnotelast       = pnote;
    }

	fclose(fp);
    strcpy( strArea, PNOTE_FILE );
    fpArea = fp;
    //bug( "Load_pnotes: bad key word.", 0 ); // prool
    printf( "prool debug: Load_pnotes: bad key word.\r\n"); // prool
    //exit( 1 ); // prool
    return;
}

void append_pnote(PNOTE_DATA *pnote)
{
    FILE *fp;
    char *name;
    PNOTE_DATA **list;
    PNOTE_DATA *last;

    switch(pnote->type)
    {
	default:
	    return;
	case PNOTE_NOTE:
	    name = PNOTE_FILE;
	    list = &pnote_list;
	    break;
	case PNOTE_IDEA:
	    name = IDEA_FILE;
	    list = &idea_list;
	    break;
	case PNOTE_PENALTY:
	    name = PENALTY_FILE;
	    list = &penalty_list;
	    break;
	case PNOTE_NEWS:
	     name = NEWS_FILE;
	     list = &news_list;
	     break;
	case PNOTE_CHANGES:
	     name = CHANGES_FILE;
	     list = &changes_list;
	     break;
	case PNOTE_ROLEPLAY:
	     name = ROLEPLAY_FILE;
	     list = &roleplay_list;
	     break;
	case PNOTE_RESTRING:
	     name = RESTRING_FILE;
	     list = &restring_list;
	     break;
	case PNOTE_EVENT:
	     name = EVENT_FILE;
	     list = &event_list;
	     break;

    }

    if (*list == NULL)
	*list = pnote;
    else
    {
	for ( last = *list; last->next != NULL; last = last->next);
	last->next = pnote;
    }

    if ( ( fp = fopen(name, "a" ) ) == NULL )
    {
        perror(name);
    }
    else
    {
        fprintf( fp, "Sender  %s~\n", pnote->sender);
        fprintf( fp, "Date    %s~\n", pnote->date);
        fprintf( fp, "Stamp   %ld\n", pnote->date_stamp);
        fprintf( fp, "To      %s~\n", pnote->to_list);
        fprintf( fp, "Subject %s~\n", pnote->subject);
        fprintf( fp, "Text\n%s~\n", pnote->text);
        fclose( fp );
    }
}

bool is_pnote_to( CHAR_DATA *ch, PNOTE_DATA *pnote )
{
    if ( !str_cmp( ch->name, pnote->sender ) )
	return TRUE;

    if ( is_exact_name( "all", pnote->to_list ) )
	return TRUE;

	/* Penalties */
	if ( pnote->type == PNOTE_PENALTY && IS_SET(ch->war_flags,WAR_MODERATOR) )
	return TRUE;
	if ( pnote->type == PNOTE_PENALTY && is_exact_name( ch->name, "Innowen" ) )
	return TRUE;
	/* end Penalty */

	/* Staff */
    if ( (IS_SET(ch->spec_flags,SPEC_MENTOR) && is_exact_name( "staff", pnote->to_list) )
    || (IS_SET(ch->war_flags,WAR_MENTOR) && is_exact_name( "staff", pnote->to_list)) )
	return TRUE;
    if ( IS_SET(ch->war_flags,WAR_MODERATOR) && is_exact_name( "staff", pnote->to_list ) )
	return TRUE;
    if ( IS_IMMORTAL(ch) && is_exact_name( "staff", pnote->to_list ) )
	return TRUE;
    if ( IS_SET(ch->hist_flags,HIST_GUILDLEADER) && is_exact_name( "staff", pnote->to_list ) )
	return TRUE;

    if ( IS_IMMORTAL(ch) && is_exact_name( "immortal", pnote->to_list ) )
	return TRUE;
    if ( IS_IMMORTAL(ch) && is_exact_name( "imm", pnote->to_list ) )
	return TRUE;

    if ( (IS_SET(ch->spec_flags,SPEC_MENTOR) && is_exact_name( "mentor", pnote->to_list) )
    || (IS_SET(ch->war_flags,WAR_MENTOR) && is_exact_name( "mentor", pnote->to_list)) )
	return TRUE;

    if ( IS_SET(ch->war_flags,WAR_MODERATOR) && is_exact_name( "moderator", pnote->to_list ) )
	return TRUE;
    if ( IS_IMMORTAL(ch) && is_exact_name( "moderator", pnote->to_list ) )
	return TRUE;

    if ( IS_SET(ch->hist_flags,HIST_GUILDLEADER) && is_exact_name( "guildleader", pnote->to_list ) )
	return TRUE;

    if ( IS_IMMORTAL(ch) && is_exact_name( "guildleader", pnote->to_list ) )
	return TRUE;


    if (ch->clan && is_exact_name(clan_table[ch->clan].name,pnote->to_list))
	return TRUE;

	if ( ch->race && is_exact_name(race_table[ch->race].name,pnote->to_list))
	return TRUE;

    if (is_exact_name( ch->name, pnote->to_list ) )
	return TRUE;

    return FALSE;
}



void pnote_attach( CHAR_DATA *ch, int type )
{
    PNOTE_DATA *pnote;

    if ( ch->pnote != NULL )
	return;

    pnote = new_pnote();

    pnote->next		= NULL;
    pnote->sender	= str_dup( ch->name );
    pnote->date		= str_dup( "" );
    pnote->to_list	= str_dup( "" );
    pnote->subject	= str_dup( "" );
    pnote->text		= str_dup( "" );
    pnote->type		= type;
    ch->pnote		= pnote;
    return;
}



void pnote_remove( CHAR_DATA *ch, PNOTE_DATA *pnote, bool delete)
{
    char to_new[MAX_INPUT_LENGTH];
    char to_one[MAX_INPUT_LENGTH];
    PNOTE_DATA *prev;
    PNOTE_DATA **list;
    char *to_list;

    if (!delete)
    {
	/* make a new list */
        to_new[0]	= '\0';
        to_list	= pnote->to_list;
        while ( *to_list != '\0' )
        {
    	    to_list	= one_argument( to_list, to_one );
    	    if ( to_one[0] != '\0' && str_cmp( ch->name, to_one ) )
	    {
	        strcat( to_new, " " );
	        strcat( to_new, to_one );
	    }
        }
        /* Just a simple recipient removal? */
       if ( str_cmp( ch->name, pnote->sender ) && to_new[0] != '\0' )
       {
	   free_string( pnote->to_list );
	   pnote->to_list = str_dup( to_new + 1 );
	   return;
       }
    }
    /* nuke the whole note */

    switch(pnote->type)
    {
	default:
	    return;
	case PNOTE_NOTE:
	    list = &pnote_list;
	    break;
	case PNOTE_IDEA:
	    list = &idea_list;
	    break;
	case PNOTE_PENALTY:
	    list = &penalty_list;
	    break;
	case PNOTE_NEWS:
	    list = &news_list;
	    break;
	case PNOTE_CHANGES:
	    list = &changes_list;
	    break;
	case PNOTE_ROLEPLAY:
	    list = &roleplay_list;
	    break;
	case PNOTE_RESTRING:
            list = &restring_list;
            break;
	case PNOTE_EVENT:
            list = &event_list;
            break;

    }

    /*
     * Remove note from linked list.
     */
    if ( pnote == *list )
    {
	*list = pnote->next;
    }
    else
    {
	for ( prev = *list; prev != NULL; prev = prev->next )
	{
	    if ( prev->next == pnote )
		break;
	}

	if ( prev == NULL )
	{
	    bug( "Note_remove: pnote not found.", 0 );
	    return;
	}

	prev->next = pnote->next;
    }

    save_pnotes(pnote->type);
    free_pnote(pnote);
    return;
}

bool hide_pnote (CHAR_DATA *ch, PNOTE_DATA *pnote)
{
    time_t last_read;

    if (IS_NPC(ch))
	return TRUE;

    switch (pnote->type)
    {
	default:
	    return TRUE;
	case PNOTE_NOTE:
	    last_read = ch->pcdata->last_pnote;
	    break;
	case PNOTE_IDEA:
	    last_read = ch->pcdata->last_idea;
	    break;
	case PNOTE_PENALTY:
	    last_read = ch->pcdata->last_penalty;
	    break;
	case PNOTE_NEWS:
	    last_read = ch->pcdata->last_news;
	    break;
	case PNOTE_CHANGES:
	    last_read = ch->pcdata->last_changes;
	    break;
	case PNOTE_ROLEPLAY:
	    last_read = ch->pcdata->last_roleplay;
	    break;
	case PNOTE_RESTRING:
	    last_read = ch->pcdata->last_restring;
	    break;
	case PNOTE_EVENT:
	    last_read = ch->pcdata->last_event;
	    break;

    }

    if (pnote->date_stamp <= last_read)
	return TRUE;

    if (!str_cmp(ch->name,pnote->sender))
	return TRUE;

    if (!is_pnote_to(ch,pnote))
	return TRUE;

    return FALSE;
}

void update_read(CHAR_DATA *ch, PNOTE_DATA *pnote)
{
    time_t stamp;

    if (IS_NPC(ch))
	return;

    stamp = pnote->date_stamp;

    switch (pnote->type)
    {
        default:
            return;
        case PNOTE_NOTE:
	    ch->pcdata->last_pnote = UMAX(ch->pcdata->last_pnote,stamp);
            break;
        case PNOTE_IDEA:
	    ch->pcdata->last_idea = UMAX(ch->pcdata->last_idea,stamp);
            break;
        case PNOTE_PENALTY:
	    ch->pcdata->last_penalty = UMAX(ch->pcdata->last_penalty,stamp);
            break;
        case PNOTE_NEWS:
	    ch->pcdata->last_news = UMAX(ch->pcdata->last_news,stamp);
            break;
        case PNOTE_CHANGES:
	    ch->pcdata->last_changes = UMAX(ch->pcdata->last_changes,stamp);
            break;
        case PNOTE_ROLEPLAY:
	    ch->pcdata->last_roleplay = UMAX(ch->pcdata->last_roleplay,stamp);
            break;
        case PNOTE_RESTRING:
	    ch->pcdata->last_restring = UMAX(ch->pcdata->last_restring,stamp);
            break;
        case PNOTE_EVENT:
	    ch->pcdata->last_event = UMAX(ch->pcdata->last_event,stamp);
            break;

    }
}

void parse_pnote( CHAR_DATA *ch, char *argument, int type )
{
    BUFFER *buffer, *output;
    DESCRIPTOR_DATA *d;
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    char arg2[MIL];
    PNOTE_DATA *pnote, *pnote2;
    PNOTE_DATA **list;
    char *list_name;
    int vnum = 0;
    int anum;
    bool found = FALSE;
    output = new_buf();

    if ( IS_NPC(ch) )
	return;

    switch(type)
    {
	default:
	    return;
        case PNOTE_NOTE:
            list = &pnote_list;
	    list_name = "notes";
            break;
        case PNOTE_IDEA:
            list = &idea_list;
	    list_name = "ideas";
            break;
        case PNOTE_PENALTY:
            list = &penalty_list;
	    list_name = "penalties";
            break;
        case PNOTE_NEWS:
            list = &news_list;
	    list_name = "news";
            break;
        case PNOTE_CHANGES:
            list = &changes_list;
	    list_name = "changes";
            break;
        case PNOTE_ROLEPLAY:
            list = &roleplay_list;
	    list_name = "roleplays";
            break;
        case PNOTE_RESTRING:
            list = &restring_list;
	    list_name = "restrings";
            break;
        case PNOTE_EVENT:
            list = &event_list;
	    list_name = "events";
            break;

    }

    argument = one_argument( argument, arg );
    smash_tilde( argument );

    if ( arg[0] == '\0' || !str_prefix( arg, "read" ) )
    {
        bool fAll;

        if ( !str_cmp( argument, "all" ) )
        {
            fAll = TRUE;
            anum = 0;
        }

        else if ( argument[0] == '\0' || !str_prefix(argument, "next"))
        /* read next unread note */
        {
            vnum = 0;
            for ( pnote = *list; pnote != NULL; pnote = pnote->next)
            {
                if (!hide_pnote(ch,pnote))
                {
                    sprintf( buf, "[%3d] %s: %s\n\r%s\n\rTo: %s\n\r",
                        vnum,
                        pnote->sender,
                        pnote->subject,
                        pnote->date,
                        pnote->to_list);
                    stc( buf, ch );
                    page_to_char( pnote->text, ch );
                    update_read(ch,pnote);
                    return;
                }
                else if (is_pnote_to(ch,pnote))
                    vnum++;
            }
	    	sprintf(buf,"{wYou have no unread %s.\n\r",list_name);
	    	stc(buf,ch);
            return;
        }

        else if ( is_number( argument ) )
        {
            fAll = FALSE;
            anum = atoi( argument );
        }
        else
        {
            stc( "{wRead which number?\n\r", ch );
            return;
        }

        vnum = 0;
        for ( pnote = *list; pnote != NULL; pnote = pnote->next )
        {
            if ( is_pnote_to( ch, pnote ) && ( vnum++ == anum || fAll ) )
            {
                sprintf( buf, "[%3d] %s: %s\n\r%s\n\rTo: %s\n\r",
                    vnum - 1,
                    pnote->sender,
                    pnote->subject,
                    pnote->date,
                    pnote->to_list
                    );
                stc( buf, ch );
                page_to_char( pnote->text, ch );
				update_read(ch,pnote);
                return;
            }
        }

	sprintf(buf,"{wThere aren't that many %s.\n\r",list_name);
	stc(buf,ch);
        return;
    }

    if ( !str_cmp( arg, "list" ) )
    {
		int total = 0;
		argument = one_argument( argument, arg2 );
		vnum = 0;
		if ( arg2[0] != '\0' && !str_cmp( arg2, "full" ) )
		{
			for ( pnote = *list; pnote != NULL; pnote = pnote->next )
			{
			    if ( is_pnote_to( ch, pnote ) )
			    {
				sprintf( buf, "%3d%s %-8s %-11s{x\n\r       To: %-4s\n\r",
				    vnum, hide_pnote(ch,pnote) ? " " : "N",
				    pnote->sender, pnote->subject, pnote->to_list );
				add_buf(output,buf);
				found = TRUE;
				vnum++;
			    }
			}
			if ( found )
				page_to_char(buf_string(output),ch);
		    free_buf(output);

			if (!vnum)
			{
			    switch(type)
			    {
				case PNOTE_NOTE:
				    stc("{wThere are no notes for you.\n\r",ch);
				    break;
				case PNOTE_IDEA:
				    stc("{wThere are no ideas for you.\n\r",ch);
				    break;
				case PNOTE_PENALTY:
				    stc("{wThere are no penalties for you.\n\r",ch);
				    break;
				case PNOTE_NEWS:
				    stc("{wThere is no news for you.\n\r",ch);
				    break;
				case PNOTE_CHANGES:
				    stc("{wThere are no changes for you.\n\r",ch);
				    break;
				case PNOTE_ROLEPLAY:
				    stc("{wThere are no roleplays for you.\n\r",ch);
				    break;
				case PNOTE_RESTRING:
				    stc("{wThere are no restrings for you.\n\r",ch);
				    break;
				case PNOTE_EVENT:
				    stc("{wThere are no event messages for you.\n\r",ch);
				    break;

			    }
			}
			return;
		}
		else
		{
			vnum = 0;
			total = 0;
			stc("{r+{R------{r+{x\n\r", ch);
			for ( pnote = *list; pnote != NULL; pnote = pnote->next )
			{
				for ( pnote2 = *list; pnote2 != NULL; pnote2 = pnote2->next )
				{
				    if ( is_pnote_to( ch, pnote2 ) )
				    {
						if ( !hide_pnote( ch, pnote ) )
							continue;
						else
						{
					    	total++;
					    	break;
						}
					}
				}
			    if ( is_pnote_to( ch, pnote ) && !hide_pnote( ch,pnote) )
			    {
					sprintf( buf, "{r| {w%3d{R%s {r| {w%s, %-11s{x\n\r{r|      | {wTo: {R%-4s{x\n\r",
					    total, hide_pnote(ch,pnote) ? " " : "N",
					    pnote->sender, pnote->subject, pnote->to_list );
					add_buf(output,buf);
					found = TRUE;
					vnum++;
			    }
			}
			if ( found )
				page_to_char(buf_string(output),ch);
		    free_buf(output);
			stc("{r+{R------{r+{x\n\r", ch);

			if (!vnum)
			{
			    switch(type)
			    {
				case PNOTE_NOTE:
				    stc("{wThere are no notes for you.\n\r",ch);
				    break;
				case PNOTE_IDEA:
				    stc("{wThere are no ideas for you.\n\r",ch);
				    break;
				case PNOTE_PENALTY:
				    stc("{wThere are no penalties for you.\n\r",ch);
				    break;
				case PNOTE_NEWS:
				    stc("{wThere is no news for you.\n\r",ch);
				    break;
				case PNOTE_CHANGES:
				    stc("{wThere are no changes for you.\n\r",ch);
				    break;
				case PNOTE_ROLEPLAY:
				    stc("{wThere are no roleplays for you.\n\r",ch);
				    break;
				case PNOTE_RESTRING:
				    stc("{wThere are no restrings for you.\n\r",ch);
				    break;
				case PNOTE_EVENT:
				    stc("{wThere are no event messages for you.\n\r",ch);
				    break;

			    }
			}
			return;
	    }
	}

    if ( !str_prefix( arg, "remove" ) )
    {
        if ( !is_number( argument ) )
        {
            stc( "{wNote remove which number?\n\r", ch );
            return;
        }

        anum = atoi( argument );
        vnum = 0;
        for ( pnote = *list; pnote != NULL; pnote = pnote->next )
        {
            if ( is_pnote_to( ch, pnote ) && vnum++ == anum )
            {
                pnote_remove( ch, pnote, FALSE );
                stc( "{wNote Removed.\n\r", ch );
                return;
            }
        }

	sprintf(buf,"{wThere aren't that many %s.",list_name);
	stc(buf,ch);
        return;
    }

    /*if ( !str_prefix( arg, "delete" )
    && get_trust(ch) >= MAX_LEVEL - 1 )*/
    if ( !str_prefix( arg, "delete" )
    && IS_SET(ch->war_flags,WAR_MODERATOR) )
    {
        if ( !is_number( argument ) )
        {
            stc( "{wNote delete which number?\n\r", ch );
            return;
        }

        anum = atoi( argument );
        vnum = 0;
        for ( pnote = *list; pnote != NULL; pnote = pnote->next )
        {
            if ( is_pnote_to( ch, pnote ) && vnum++ == anum )
            {
                pnote_remove( ch, pnote,TRUE );
                stc( "{wNote Deleted.\n\r", ch );
                return;
            }
        }

 	sprintf(buf,"{wThere aren't that many %s.",list_name);
	stc(buf,ch);
        return;
    }

    if (!str_prefix(arg,"catchup"))
    {
	switch(type)
	{
	    case PNOTE_NOTE:
		ch->pcdata->last_pnote = current_time;
		break;
	    case PNOTE_IDEA:
		ch->pcdata->last_idea = current_time;
		break;
	    case PNOTE_PENALTY:
		ch->pcdata->last_penalty = current_time;
		break;
	    case PNOTE_NEWS:
		ch->pcdata->last_news = current_time;
		break;
	    case PNOTE_CHANGES:
		ch->pcdata->last_changes = current_time;
		break;
	    case PNOTE_ROLEPLAY:
		ch->pcdata->last_roleplay = current_time;
		break;
	    case PNOTE_RESTRING:
		ch->pcdata->last_restring = current_time;
		break;
	    case PNOTE_EVENT:
		ch->pcdata->last_event = current_time;
		break;

	}
	return;
    }

    /* below this point only certain people can edit notes */
    if ((type == PNOTE_NEWS && !IS_IMMORTAL(ch))
    ||  (type == PNOTE_CHANGES && !IS_IMMORTAL(ch)))
    {
	sprintf(buf,"{wYou aren't high enough level to write %s.",list_name);
	stc(buf,ch);
	return;
    }

    if ( !str_cmp( arg, "+" ) )
    {
	pnote_attach( ch,type );
	if (ch->pnote->type != type)
	{
	    stc(
		"{wYou already have a different note in progress.\n\r",ch);
	    return;
	}

	if (strlen(ch->pnote->text)+strlen(argument) >= 4096)
	{
	    stc( "{wNote too long.\n\r", ch );
	    return;
	}

 	buffer = new_buf();

	add_buf(buffer,ch->pnote->text);
	add_buf(buffer,argument);
	add_buf(buffer,"\n\r");
	free_string( ch->pnote->text );
	ch->pnote->text = str_dup( buf_string(buffer) );
	free_buf(buffer);
	stc( "{wOk.\n\r", ch );
	return;
    }

    if (!str_cmp(arg,"-"))
    {
 	int len;
	bool found = FALSE;

	pnote_attach(ch,type);
        if (ch->pnote->type != type)
        {
            stc(
                "{wYou already have a different note in progress.\n\r",ch);
            return;
        }

	if (ch->pnote->text == NULL || ch->pnote->text[0] == '\0')
	{
	    stc("{wNo lines left to remove.\n\r",ch);
	    return;
	}

	strcpy(buf,ch->pnote->text);

	for (len = strlen(buf); len > 0; len--)
 	{
	    if (buf[len] == '\r')
	    {
		if (!found)  /* back it up */
		{
		    if (len > 0)
			len--;
		    found = TRUE;
		}
		else /* found the second one */
		{
		    buf[len + 1] = '\0';
		    free_string(ch->pnote->text);
		    ch->pnote->text = str_dup(buf);
		    return;
		}
	    }
	}
	buf[0] = '\0';
	free_string(ch->pnote->text);
	ch->pnote->text = str_dup(buf);
	return;
    }
    if ( !str_cmp( arg, "desc" )
    || !str_cmp( arg, "write") )
    {
    	pnote_attach( ch, type );
	    if ( argument[0] == '\0' )
	    {
    		string_append( ch, &ch->pnote->text );
    	}
    	return;
    }

    if ( !str_prefix( arg, "subject" ) )
    {
    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
    	stc("{cYou cannot use channels.{c\n\r",ch);
    	return;
 	}
	pnote_attach( ch,type );
        if (ch->pnote->type != type)
        {
            stc(
                "{wYou already have a different note in progress.\n\r",ch);
            return;
        }

	free_string( ch->pnote->subject );
	ch->pnote->subject = str_dup( argument );
	stc( "{wOk.\n\r", ch );
	return;
    }

    if ( !str_prefix( arg, "to" ) )
    {
    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
    	stc("{cYou cannot use channels.{c\n\r",ch);
    	return;
 	}

	if ( type == PNOTE_IDEA
	&& !IS_IMMORTAL( ch )
	&& strstr( argument, "all" ) )
	{
		stc("Ideas may only be sent to Immortal or Staff.\n\r", ch);
		return;
	}
	if ( type == PNOTE_NOTE
	&& strstr( argument, "all" ) )
	{
		if ( IS_IMMORTAL( ch ) )
		;
		else if ( IS_SET( ch->war_flags, WAR_MODERATOR ) )
		;
		else
		{
		stc("Only Staff may send notes to all.\n\rPlease address your note to the appropriate parties.\n\r", ch);
		return;
		}
	}

	pnote_attach( ch,type );
        if (ch->pnote->type != type)
        {
            stc(
                "{wYou already have a different note in progress.\n\r",ch);
            return;
        }
	free_string( ch->pnote->to_list );
	ch->pnote->to_list = str_dup( argument );
	stc( "{wOk.\n\r", ch );
	return;
    }

    if ( !str_prefix( arg, "clear" ) )
    {
	if ( ch->pnote != NULL )
	{
	    free_pnote(ch->pnote);
	    ch->pnote = NULL;
	}

	stc( "{wOk.\n\r", ch );
	return;
    }

    if ( !str_prefix( arg, "show" ) )
    {
    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
    	stc("{cYou cannot use channels.{c\n\r",ch);
    	return;
 	}
	if ( ch->pnote == NULL )
	{
	    stc( "{wYou have no note in progress.\n\r", ch );
	    return;
	}

	if (ch->pnote->type != type)
	{
	    stc("{wYou aren't working on that kind of note.\n\r",ch);
	    return;
	}

	sprintf( buf, "%s: %s\n\rTo: %s\n\r",
	    ch->pnote->sender,
	    ch->pnote->subject,
	    ch->pnote->to_list
	    );
	stc( buf, ch );
	stc( ch->pnote->text, ch );
	return;
    }

    if ( !str_prefix( arg, "post" ) || !str_prefix(arg, "send"))
    {
	char *strtime;

	if ( ch->pnote == NULL )
	{
	    stc( "{wYou have no note in progress.\n\r", ch );
	    return;
	}

    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
    {
    	stc("{cYou cannot use channels.{c\n\r",ch);
    	return;
 	}
        if (ch->pnote->type != type)
        {
            stc("{wYou aren't working on that kind of note.\n\r",ch);
            return;
        }

	if (!str_cmp(ch->pnote->to_list,""))
	{
	    stc(
		"{wYou need to provide a recipient (name, all, a guild, mentor, moderator, or immortal).\n\r",
		ch);
	    return;
	}

	if (!str_cmp(ch->pnote->subject,""))
	{
	    stc("{wYou need to provide a subject.\n\r",ch);
	    return;
	}

	ch->pnote->next			= NULL;
	strtime				= ctime( &current_time );
	strtime[strlen(strtime)-1]	= '\0';
	ch->pnote->date			= str_dup( strtime );
	ch->pnote->date_stamp		= current_time;

	append_pnote(ch->pnote);

	stc( "{wDone.{x\n\r", ch );

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( d->connected == CON_PLAYING
		&&   d->character != ch
		&&   is_pnote_to( d->character, ch->pnote ) )
		{
		sprintf( buf, "{w***{R %s has sent %s a new %s.{w ***{x\n\r", ch->name,
		!str_prefix( ch->pnote->to_list, "all" ) ? "everyone" :
		(   !str_prefix( ch->pnote->to_list, "imm" )
		|| !str_prefix( ch->pnote->to_list, "immortal" ) ? "Immortals" :
		"you" ), list_name );
		stc( buf, d->character );
		}
	}

	ch->pnote = NULL;

	return;
    }

    stc( "{wYou can't do that.\n\r", ch );
    return;
}

