/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "magic.h"
#include "recycle.h"

int check_burnout (CHAR_DATA *ch, int pwrLvl);
void check_stat_improve args( ( CHAR_DATA *ch, int stat, int power ) );
int sphere_strength (CHAR_DATA *ch, int stat);
bool check_blind             args( ( CHAR_DATA *ch ) );


int calc_weave_damage2( CHAR_DATA *ch, int sn, int pwr, int bpwr )
{
	/* weave stats */
	int base, mod, talent, talent_bonus, tertiary, total, foundation;
	int req_earth, req_air, req_fire, req_water, req_spirit, reqs, sphtot;
	/* character stats */
	int opavg, over, pwrmax = 0;
	float pwratio;

	/*bpwr = pwr;  Need a value, The # in 'weave # weave_name target' */

	ptc( ch, "Base PWR : %d\n\r", bpwr );
	ptc( ch, "PWR: %d\n\r", pwr );

	talent_bonus = 1;
	pwratio = 1;
	sphtot = 0;
	foundation = skill_table[sn].base_dmg;
	req_earth = skill_table[sn].sphere_earth;
	req_air = skill_table[sn].sphere_air;
	req_fire = skill_table[sn].sphere_fire;
	req_water = skill_table[sn].sphere_water;
	req_spirit = skill_table[sn].sphere_spirit;


	if (req_earth > 0)
		sphtot += 1;
	if (req_air > 0)
		sphtot += 1;
	if (req_fire > 0)
		sphtot += 1;
	if (req_water > 0)
		sphtot += 1;
	if (req_spirit > 0)
		sphtot += 1;

	reqs = req_earth + req_air + req_fire + req_water + req_spirit;
	reqs = reqs / sphtot;	/* Average requirement */

	ptc( ch, "REQS: %d\n\r", reqs );

/* Formula for weave damage: MOD = [((DF+pwr+opavg)*reqs*Pwratio)/6] */
/* pwratio = (1 + (bpwr - reqs/ pwrmax) */



	ptc( ch, "Pwrmax: %d\n\r", pwrmax );

	if ( IS_NPC( ch ) )
		opavg = ( ch->level / 5 );
	else
		opavg = ch->pcdata->op_air+ch->pcdata->op_earth+ch->pcdata->op_fire+ch->pcdata->op_spirit+ch->pcdata->op_water;
		opavg /= 5;

	ptc( ch, "OPAVG: %d\n\r", opavg );

	pwrmax = ch->pcdata->op_air+ch->pcdata->op_earth+ch->pcdata->op_fire+ch->pcdata->op_spirit+ch->pcdata->op_water;
	pwrmax += int_app[get_curr_stat(ch,STAT_INT)].op_focus;
	pwrmax /= 5;
	pwrmax += 15; /* The number before one blacks out */

	if (IS_NPC (ch) )
		pwratio = 1.5;				/* For NPCs */
	else
		pwratio = bpwr - reqs;			/* For Players */
		pwratio /= pwrmax;
		pwratio += 1;

	ptc( ch, "PWRatio: (%f)\n\r", pwratio );

	base = foundation;
	base += pwr;
	base += opavg;

	ptc( ch, "mod Base: (%d) = (DMG(%d) + pwr (%d) + opavg(%d))\n\r", base,
foundation, pwr, opavg );

	over = reqs * pwratio;

	ptc( ch, "Overpower factor: (%d) = reqs*pwratio\n\r", over);


	mod = (base * over);
	mod /= 5;

	ptc( ch, "MOD : (%d) = (base*over)/5\n\r", mod );

	talent = skill_table[sn].talent;

	switch ( talent )
	{
		case TALENT_NONE:
		break;
		case TALENT_WARDING:
			if ( IS_SET( ch->talent_flags, W_TALENT_WARDING ) )
				talent_bonus = 2;
		break;
		case TALENT_ELEMENTALISM:
			if ( IS_SET( ch->talent_flags, W_TALENT_ELEMENTALISM ) )
				talent_bonus = 2;
		break;
		case TALENT_HEALING:
			if ( IS_SET( ch->talent_flags, W_TALENT_HEALING ) )
				talent_bonus = 2;
		break;
		case TALENT_CLOUD_DANCING:
			if ( IS_SET( ch->talent_flags, W_TALENT_CLOUD_DANCING ) )
				talent_bonus = 2;
		break;
		case TALENT_EARTH_SINGING:
			if ( IS_SET( ch->talent_flags, W_TALENT_EARTH_SINGING ) )
				talent_bonus = 2;
		break;
	}

	tertiary = ( mod * talent_bonus );

	ptc( ch, "TERT: %d (mod * talent_bonus (%d))\n\r", tertiary, talent_bonus
);

	total = tertiary;

	ptc( ch, "TOTAL CWD: %d.\n\r", total );

	return total;
}

int calc_weave_damage( CHAR_DATA *ch, int sn, int pwr )
{
	/* weave stats */
	int base, mod, talent, talent_bonus, tertiary, total, foundation;
	int req_earth, req_air, req_fire, req_water, req_spirit, reqs, sphtot;
	/* character stats */
	int over, rbi, pwrmax, canPwr;
	float pwratio;

	base = 1;
	mod = 1;
	talent = 1;
	talent_bonus = 1;
	tertiary = 1;
	foundation = 1;
	over = 1;
	rbi = 1;
	pwrmax = 1;
	canPwr = 1;

	req_earth = skill_table[sn].sphere_earth;
	req_air = skill_table[sn].sphere_air;
	req_fire = skill_table[sn].sphere_fire;
	req_water = skill_table[sn].sphere_water;
	req_spirit = skill_table[sn].sphere_spirit;

	reqs = req_earth;
	reqs += req_air;
	reqs += req_fire;
	reqs += req_water;
	reqs += req_spirit;

	ptc( ch, "PWR: %d\n\r", pwr );

/* Changed reqs, to gives a weighted reqs */

	sphtot = 0;
	if (req_earth > 0)
		sphtot += 1;
	if (req_air > 0)
		sphtot += 1;
	if (req_fire > 0)
		sphtot += 1;
	if (req_water > 0)
		sphtot += 1;
	if (req_spirit > 0)
		sphtot += 1;
	//ptc( ch, "Sphtot: %d\n\r", sphtot );
	reqs = reqs / sphtot;		/* Sum of spheres / total number of spheres */

	ptc( ch, "REQS: %d\n\r", reqs );


/* Formula for weave damage: MOD = (DF+pwr+Remorts)*(reqs/5)*Pwratio */
/* pwratio = (1 + (pwr-reqs/ the number before blacking out) Need Pwrmax
from Mana cost code */
/* Also check out the excel file I sent you, on sheet 2 there is a damage
mod simulator */

	canPwr = ch->pcdata->op_air+ch->pcdata->op_earth+ch->pcdata->op_fire+ch->pcdata->op_spirit+ch->pcdata->op_water;
	canPwr += int_app[get_curr_stat(ch,STAT_INT)].op_focus;
	canPwr /= 5;

	pwrmax = canPwr + 15;  /* Insert here */
	ptc( ch, "Pwrmax: %d\n\r", pwrmax );
	talent_bonus = 1;
	foundation = skill_table[sn].base_dmg;

	if ( IS_NPC( ch ) )
		rbi = 1 + ( ch->level / 2 );
	else
		rbi = ch->pcdata->rebirth;

	ptc( ch, "RBI: %d\n\r", rbi );


	if (IS_NPC (ch) )
		pwratio = 1.5;				/* For NPCs */
	else
	{
		//pwratio = 1 + ( ( pwr - reqs ) / pwrmax ); 	/* For Players */
		pwratio = pwr - reqs;
		pwratio /= pwrmax;
		pwratio += 1;
	}
	ptc( ch, "PWRatio: (%f)\n\r", pwratio );


	base = foundation;
	base += pwr;
	base += rbi;

	ptc( ch, "mod Base: (%d) = (DMG(%d) + pwr (%d) + rbi(%d))\n\r", base, foundation, pwr, rbi );


	over = ( reqs / 5 ) * pwratio;

	if ( over <= 0 )
		over = 1;

	ptc( ch, "Overpower factor: (%d) = (reqs/5)*pwratio\n\r", over);


	mod = (base * over);


	ptc( ch, "MOD : (%d) = (base*over)\n\r", mod );

	talent = skill_table[sn].talent;

	switch ( talent )
	{
		case TALENT_NONE:
		break;
		case TALENT_WARDING:
			if ( IS_SET( ch->talent_flags, W_TALENT_WARDING ) )
				talent_bonus = 2;
		break;
		case TALENT_ELEMENTALISM:
			if ( IS_SET( ch->talent_flags, W_TALENT_ELEMENTALISM ) )
				talent_bonus = 2;
		break;
		case TALENT_HEALING:
			if ( IS_SET( ch->talent_flags, W_TALENT_HEALING ) )
				talent_bonus = 2;
		break;
		case TALENT_CLOUD_DANCING:
			if ( IS_SET( ch->talent_flags, W_TALENT_CLOUD_DANCING ) )
				talent_bonus = 2;
		break;
		case TALENT_EARTH_SINGING:
			if ( IS_SET( ch->talent_flags, W_TALENT_EARTH_SINGING ) )
				talent_bonus = 2;
		break;
	}

	tertiary = ( mod * talent_bonus );

	ptc( ch, "TERT: %d (mod * talent_bonus (%d))\n\r", tertiary, talent_bonus );

	total = tertiary;

	ptc( ch, "TOTAL CWD: %d.\n\r", total );

	return total;
}




/*
int calc_weave_damage( CHAR_DATA *ch, int sn, int pwr )
{
	int base, mod, talent, talent_bonus, tertiary, total, foundation;
    int req_earth, req_air, req_fire, req_water, req_spirit, reqs;
	int rbi, over, overmod;

	req_earth = skill_table[sn].sphere_earth;
	req_air = skill_table[sn].sphere_air;
	req_fire = skill_table[sn].sphere_fire;
	req_water = skill_table[sn].sphere_water;
	req_spirit = skill_table[sn].sphere_spirit;

	reqs = req_earth;
	reqs += req_air;
	reqs += req_fire;
	reqs += req_water;
	reqs += req_spirit;
	ptc( ch, "REQS: %d\n\r", reqs );

	talent_bonus = 1;
	foundation = skill_table[sn].base_dmg;

	base = skill_table[sn].remort_only + 1;
	base += foundation;

	ptc( ch, "BASE: %d (RBI (%d) + DMG (%d))\n\r", base, skill_table[sn].remort_only, foundation );

	if ( pwr > reqs )
	{
		over = (pwr / reqs);
		ptc( ch, "OVER %d (pwr (%d) / reqs (%d))\n\r", over, pwr, reqs );
	}
	if ( over > 0 )
	{
		mod = (over * over);
		ptc( ch, "OVERPOWER %d (mod *= over)\n\r", mod );
	}

	if ( IS_NPC( ch ) )
		rbi = 1 + ( ch->level / 2 );
	else
		rbi = ch->pcdata->rebirth;
	ptc( ch, "RBI: %d\n\r", rbi );

	mod += ( base + ( pwr + rbi ) );
	ptc( ch, "MOD: %d base + (pwr + rbi)\n\r", mod );

	mod += reqs;
	ptc( ch, "MOD %d (mod += reqs)\n\r", mod );

	talent = skill_table[sn].talent;

	switch ( talent )
	{
		case TALENT_NONE:
		break;
		case TALENT_WARDING:
			if ( IS_SET( ch->talent_flags, W_TALENT_WARDING ) )
				talent_bonus = 2;
		break;
		case TALENT_ELEMENTALISM:
			if ( IS_SET( ch->talent_flags, W_TALENT_ELEMENTALISM ) )
				talent_bonus = 2;
		break;
		case TALENT_HEALING:
			if ( IS_SET( ch->talent_flags, W_TALENT_HEALING ) )
				talent_bonus = 2;
		break;
		case TALENT_CLOUD_DANCING:
			if ( IS_SET( ch->talent_flags, W_TALENT_CLOUD_DANCING ) )
				talent_bonus = 2;
		break;
		case TALENT_EARTH_SINGING:
			if ( IS_SET( ch->talent_flags, W_TALENT_EARTH_SINGING ) )
				talent_bonus = 2;
		break;
	}

	tertiary = ( mod * talent_bonus );
	ptc( ch, "TERT: %d (mod * talent_bonus (%d))\n\r", tertiary, talent_bonus );

	total = tertiary;

	ptc( ch, "TOTAL CWD: %d.\n\r", total );

	return total;
}
*/
int calc_weave_duration( CHAR_DATA *ch, int sn, int pwr )
{
	return 0;
}

bool is_offensive( CHAR_DATA *ch, int sn )
{
	if ( skill_table[sn].offensive == FALSE )
	{
		return FALSE;
	}
	return TRUE;
}

void do_weave( CHAR_DATA *ch, char *argument )
{
    char pwr[MIL];
    char weave[MIL];
    char targ[MIL];
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    WEAVE_DATA *new_weave;
    void *vo;
    int mana, sn, target;
	long bpwr;
/*    int wis_chk, int_chk, con_chk;*/
    int manaCost = 0, trueMana = 0;
    int wvStr = 0;
    int chAvg = 0;
    int outcome = 0;
    int req_earth, req_air, req_fire, req_water, req_spirit, reqs;
	ovrPwr = 0;

    /* set defaults for the globals */
    truePwr = 0;
    trueOverPwr = 0;
    pwrBonus = 0;
    trueOverPwr = 0;
    wvDmg = 0;
    /* and the locals */
    req_earth = 0;
    req_air = 0;
    req_fire = 0;
    req_water = 0;
    req_spirit = 0;
    reqs = 0;

    if ( IS_NPC(ch) && ch->desc == NULL)
	return;

	argument    = one_argument( argument, pwr );
	argument    = one_argument( argument, weave );
    argument    = one_argument( argument, targ );
/*    target_name = str_dup(targ);  */

    if (!IS_SET(ch->op_flags,OP_GRASPED))
    {
		ptc( ch, "But you are not holding %s.\n\r", ch->sex == SEX_MALE ? "Saidin" : "Saidar" );
		return;
	}

	if ( !str_cmp( pwr, "stop" ) || !str_cmp( pwr, "halt" ) )
	{
            if ( ch->weaves == NULL )
                stc("Nothing to stop.\n\r", ch);
	    else
		stop_spell( ch );
	    return;
	}
	if ( !str_cmp( pwr, "list") )
	{
	    int i = 0;
            if ( ch->weaves == NULL )
	    {
		    stc("No weaves queued.\n\r",ch);
		    return;
	    }
	    for(new_weave = ch->weaves; new_weave; new_weave = new_weave->next)
	    {
		    ptc(ch,"(%3d)%-20s ", new_weave->power[0],capit2(skill_table[new_weave->spell_vars[5]].name));
		    i++;
		    if(i == 3)
		    {
			stc("\n\r",ch);
			i = 0;
		    }
	    }
	    if(i!=0)
	        stc("\n\r",ch);
	    return;
	}


	if ( IS_SET( ch->in_room->room_flags,ROOM_STEDDING ) )
	{
		stc("You cannot seem to sense the True Source for some reason.\n\r", ch);
		return;
	}

	if ( pwr[0] == '\0' )
	{
		stc( "Weave what flow of the One Power, at what strength, at whom?\n\r",ch );
		stc( "Syntax: Weave (#Strength) (flow name) (target)\n\r",ch);
		stc( "Example: Weave 3 'Cure Light' Avryel\n\r",ch);
		stc( "Remember to include '' around weaves with two or more parts to the name.\n\r",ch);
		stc( "Weave halt: Removes the next to be cast weave from the queue.\n\r",ch);
		stc( "Weave list: Lists the pending weaves in the queue.\n\r",ch);
		return;
	}

	if ( is_number(pwr) && atoi(pwr) > 0 )
	{
		bpwr = atoi(pwr);
		truePwr = atoi(pwr);
		trueOverPwr = ( atoi(pwr) - true_op_average(ch) );
		ovrPwr = atoi(pwr) - (check_angreal_bonus(ch) / 10 );
		if ( ovrPwr < 0 )
			ovrPwr = 0;
	    if ( ovrPwr > 0 )
	    {
	    	pwrBonus = check_burnout(ch,ovrPwr);
		}
	}
	else
	{
		stc("Either pwr was not a number, or it was less than 0.\n\r",ch);
		stc( "Syntax: Weave (#Strength) (flow name) (target)\n\r",ch);
		stc( "Example: Weave 3 'Cure Light' Avryel\n\r",ch);
		stc( "Remember to include '' around weaves with two or more parts to the name.\n\r",ch);
		stc( "Weave halt: Removes the next to be cast weave from the queue.\n\r",ch);
		stc( "Weave list: Lists the pending weaves in the queue.\n\r",ch);
		return;
	}
    if ( ( sn = find_spell( ch, weave ) ) <= 0 )
    {
	    stc( "{cYou don't know any weaves of that name.\n\r", ch );
		return;
    }
    if ( ( sn = find_spell( ch, weave ) ) == 0 )
    {
	    stc( "Weave what flow?\n\r", ch );
		return;
    }
    if( skill_table[sn].spell_fun == spell_null)
    {
	    stc( "That's not a weave!.\n\r", ch );
	    return;
    }
    if ( ch->pcdata->learned[sn] <= 0 )
    {
	    stc( "{cYou don't know that weave.\n\r", ch );
		return;
    }
    if ( ch->position < skill_table[sn].minimum_position )
    {
		stc( "{cYou can't concentrate enough.\n\r", ch );
		return;
    }

	mana = skill_table[sn].min_mana;
	req_earth = skill_table[sn].sphere_earth;
	req_air = skill_table[sn].sphere_air;
	req_fire = skill_table[sn].sphere_fire;
	req_water = skill_table[sn].sphere_water;
	req_spirit = skill_table[sn].sphere_spirit;

	reqs = req_earth;
	reqs += req_air;
	reqs += req_fire;
	reqs += req_water;
	reqs += req_spirit;

	if(ch->pcdata) /* if npcs are gonna use this, make it better! */
	{

	    if ( ch->pcdata->op_earth < req_earth )
	    {
		stc("You do not possess the required strength in Earth for this weave.\n\r", ch);
		return;
	    }
	    if ( ch->pcdata->op_air < req_air )
	    {
		stc("You do not possess the required strength in Air for this weave.\n\r", ch);
		return;
	    }
	    if ( ch->pcdata->op_fire < req_fire )
	    {
		stc("You do not possess the required strength in Fire for this weave.\n\r", ch);
		return;
	    }
	    if ( ch->pcdata->op_water < req_water )
	    {
		stc("You do not possess the required strength in Water for this weave.\n\r", ch);
		return;
	    }
	    if ( ch->pcdata->op_spirit < req_spirit )
	    {
		stc("You do not possess the required strength in Spirit for this weave.\n\r", ch);
		return;
	    }
	}
	/* temp removed -- Dalsor */
	/*
	if ( truePwr < reqs )
	{
		ptc( ch, "You must channel at least %d power to form this weave.\n\r", reqs );
		return;
	}
	*/

	trueMana = mana;
	manaCost = mana;
	wvStr = truePwr;
	chAvg = true_op_average(ch);
	if ( chAvg <= 10 && !IS_SET( ch->op_flags, OP_CHANNELER ) )
		chAvg = 10;

	outcome = ( manaCost*(wvStr - true_angreal_bonus( ch ) ) / chAvg );

	if ( outcome <= manaCost )
		outcome = manaCost;
    mana = outcome;

    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );

    wvDmg = sphere_strength( ch, skill_table[sn].sphere );

	/*ptc( ch, "Total Check is %d.\n\r", calc_weave_damage2( ch, sn, truePwr, bpwr ) );*/

    victim	= NULL;
    obj		= NULL;
    vo		= NULL;
    target	= TARGET_NONE;

    switch ( skill_table[sn].target )
    {
    default:
	bug( "Do_weave: bad target for sn %d.", sn );
	return;

    case TAR_IGNORE:
	break;

    case TAR_CHAR_OFFENSIVE:
	if ( targ[0] == '\0' )
	{
	    if ( !is_fighting(ch) )
	    {
	        stc( "{cWeave the flow on whom?\n\r", ch );
			return;
	    }
	    else
		    ( victim = ch->advancing_on  );
	}
	else
	{
	    if ( ( victim = get_char_room( ch,NULL, targ ) ) == NULL )
	    {
		stc( "{cThey aren't here.\n\r", ch );
		return;
	    }
	}

	if (!IS_NPC(ch))
	{
		if (!IS_NPC(victim)
		&& (ch->in_room) && (victim->in_room)
		&& !IS_SET(ch->in_room->room_flags,ROOM_PK)
		&& !IS_SET(victim->in_room->room_flags,ROOM_PK)
		&& !IS_SET(ch->war_flags,WAR_BATTLE)
		&& !IS_SET(ch->war_flags,WAR_TOURNEY)
		&& !IS_SET(victim->war_flags,WAR_BATTLE)
		&& !IS_SET(victim->war_flags,WAR_TOURNEY)
		&& !IS_IMMORTAL(ch))
		{
		    stc("{cGo to the Arena if you want to PK.\n\r",ch);
		    return;
		}
	}
	if ( !IS_NPC(ch) )
	{
        if (is_safe(ch,victim) && victim != ch)
	    {
			stc( "{cNot on that target.\n\r",ch);
			return;
	    }
		check_killer(ch,victim);
	}

	/* quest mobs */
	if ( IS_SET( victim->act, ACT_UNDEAD ) )
	{
		if ( ( victim->questor != NULL && victim->questor != ch )
		&& ( victim->pIndexData->vnum != ch->questmob ) )
		{
			stc("That is not your quest mob.\n\r",ch);
			return;
		}
	}
	/* end quest mobs */

    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
	{
	    stc( "{cYou can't do that on your own follower.\n\r",ch );
	    return;
	}

	vo = (void *) victim;
	target = TARGET_CHAR;
	if ( IS_NPC( victim ) && !is_fighting(victim) )
		start_extra_fight(ch, victim, RANGE_MISSLE);

	break;

    case TAR_CHAR_DEFENSIVE:
	if ( targ[0] == '\0' )
	{
	    victim = ch;
	}
	else
	{
	    if ( ( victim = get_char_room( ch,NULL, targ ) ) == NULL )
	    {
			stc( "{cThey aren't here.\n\r", ch );
			return;
	    }
	}

	vo = (void *) victim;
	target = TARGET_CHAR;
	break;

    case TAR_CHAR_SELF:
	if ( weave[0] != '\0' && targ[0] != '\0' && !is_name( targ, ch->name ) )
	{
		stc( "{cYou cannot weave this flow on another.\n\r", ch );
	    return;
	}

	vo = (void *) ch;
	target = TARGET_CHAR;
	break;

    case TAR_OBJ_INV:
	if ( weave[0] == '\0' )
	{
            stc( "{cWhat should the One Power be woven upon?\n\r",ch );
	    return;
	}

	if ( ( obj = get_obj_carry( ch, targ, ch ) ) == NULL )
	{
	    stc( "{cYou are not carrying that.\n\r", ch );
	    return;
	}

	vo = (void *) obj;
	target = TARGET_OBJ;
	break;

    case TAR_OBJ_CHAR_OFF:
	if (weave[0] == '\0')
	{
	    if (!is_fighting(ch) || (victim = ch->advancing_on) == NULL)
	    {
	        stc( "{cWeave the flow on whom or what?\n\r",ch);
			return;
	    }

	    target = TARGET_CHAR;
	}
	else if ((victim = get_char_room(ch,NULL,targ)) != NULL)
	{
	    target = TARGET_CHAR;
	}

	if (target == TARGET_CHAR) /* check the sanity of the attack */
	{
	    if(is_safe_spell(ch,victim,FALSE) && victim != ch)
	    {
			stc( "{cNot on that target.\n\r",ch);
			return;
	    }
        if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim )
        {
	        stc( "{cYou can't do that on your own follower.\n\r",ch );
	        return;
        }

	    if (!IS_NPC(ch))
		check_killer(ch,victim);

	    vo = (void *) victim;

		/* quest mobs */
		if ( IS_SET( victim->act, ACT_UNDEAD ) )
		{
			if ( ( victim->questor != NULL && victim->questor != ch )
			&& ( victim->pIndexData->vnum != ch->questmob ) )
			{
				stc("That is not your quest mob.\n\r",ch);
				return;
			}
		}
		/* end quest mobs */
		if ( IS_NPC( victim ) && !is_fighting(victim) )
		{
			start_extra_fight(ch, victim, RANGE_MISSLE);
		}
 	}
	else if ((obj = get_obj_here(ch,NULL,targ)) != NULL)
	{
	    vo = (void *) obj;
	    target = TARGET_OBJ;
	}
	else
	{
	    stc( "{cYou don't see that here.\n\r",ch);
	    return;
	}
	break;

    case TAR_OBJ_CHAR_DEF:
        if (weave[0] == '\0')
        {
            vo = (void *) ch;
            target = TARGET_CHAR;
        }
        else if ((victim = get_char_room(ch,NULL,targ)) != NULL)
        {
            vo = (void *) victim;
            target = TARGET_CHAR;
		}
		else if ((obj = get_obj_carry(ch,targ,ch)) != NULL)
		{
		    vo = (void *) obj;
		    target = TARGET_OBJ;
		}
		else
		{
		    stc( "{cYou don't see that here.\n\r",ch);
		    return;
		}
		break;
    }

    if ( !IS_NPC(ch) && ch->stam < mana )
    {
		stc( "{cYou don't have enough mana.\n\r", ch );
		return;
    }

    /* stack implementation */
    /* create spell_cast_data type that contains these values */
    /*
    ch->spell_vars[0] = sn;
    ch->spell_vars[1] = wvDmg;
    ch->spell_vars[2] = target;
    ch->spell_vars[3] = skill_table[sn].beats;
    ch->spell_vars[4] = mana;
    ch->spell_vars[5] = sn;
    ch->vo = vo;
    ch->action[0] = ACTION_CASTING;
    */
    if(ch->weaves)
    {
        for(new_weave = ch->weaves;new_weave->next;new_weave=new_weave->next);
        new_weave->next = new_weave_data();
        new_weave = new_weave->next;
    }
    else
    {
	ch->weaves = new_weave_data();
	new_weave = ch->weaves;
    }

    new_weave->spell_vars[0] = sn;
    new_weave->spell_vars[1] = wvDmg;
    new_weave->spell_vars[2] = target;
    new_weave->spell_vars[3] = skill_table[sn].beats;
    new_weave->spell_vars[4] = mana;
    new_weave->spell_vars[5] = sn;
    new_weave->spell_vars[6] = skill_table[sn].target;
    new_weave->vo = vo;
    new_weave->action[0] = ACTION_CASTING;
    new_weave->target_name = str_dup(targ);
    new_weave->power[0] = truePwr;
    new_weave->power[1] = trueOverPwr;
    new_weave->power[2] = pwrBonus;
    new_weave->power[3] = trueOverPwr;
    new_weave->power[4] = angBonus;
    new_weave->power[5] = wvDmg;



    stc ("You channel the One Power and begin to form your weave.\n\r", ch);
    return;
}

bool has_object ( CHAR_DATA *ch, OBJ_DATA *obj)
{
	OBJ_DATA *obj1;

	for ( obj1 = ch->carrying; obj != NULL; obj = obj->next_content )
	{
		if ( obj1 == obj )
			return TRUE;
	}
	return FALSE;
}
/*
void release_spell( int sn, int level, CHAR_DATA *ch, void *vo, int target)
*/
void release_spell( CHAR_DATA *ch )
{
	OBJ_DATA *obj;
	CHAR_DATA *victim = NULL;
	WEAVE_DATA *trash;
	void *vo    = ch->weaves->vo;
	int  sn     = ch->weaves->spell_vars[0];
	int  level  = ch->weaves->spell_vars[1];
	int  target = ch->weaves->spell_vars[2];
	int  mana   = ch->weaves->spell_vars[4];
	int  ttarg  = ch->weaves->spell_vars[6];
	truePwr     = ch->weaves->power[0];
	trueOverPwr = ch->weaves->power[1];
	pwrBonus    = ch->weaves->power[2];
	trueOverPwr = ch->weaves->power[3];
    	angBonus    = ch->weaves->power[4];
	wvDmg 	    = ch->weaves->power[5];

	switch (target)
	{
		default:
			break;

		case TARGET_CHAR:
			victim = (CHAR_DATA *) vo;

            if (victim == NULL || victim->in_room != ch->in_room)
            {
                send_to_char ("They are no longer here.\n\r", ch);
                stop_spell( ch );
                return;
            }
	    if( !((ttarg == TAR_CHAR_DEFENSIVE) ))   /* add other types that can be cast on safe victims */
	    {
              if (is_safe (ch, victim) && ch != victim)
              {
                send_to_char ("You can't weave something like that on them, they are safe.\n\r", ch);
				stop_spell( ch );
                return;
              }
	    }
            break;


        case TARGET_OBJ:
            obj = ( OBJ_DATA *) vo;
			if ( obj == NULL || (obj->in_room != ch->in_room && !has_object(ch, obj)))
			{
				stc ("That item is no longer here.\n\r", ch);
				stop_spell( ch);
				return;
			}
            break;
	}

    if ( number_percent( ) > get_skill(ch,sn) + (int_app[get_curr_stat(ch,STAT_INT)].op_focus * 2) )
    {
		int wis_chk	= wis_app[get_curr_stat(ch,STAT_WIS)].op_will;
		int int_chk	= int_app[get_curr_stat(ch,STAT_INT)].op_focus;
		int con_chk	= con_app[get_curr_stat(ch,STAT_CON)].op_stam;
	    stc( "You nearly lost your grasp on the Source.\n\r", ch );
	    if ( ch->stam < (wis_chk + int_chk + con_chk) )
	    {
	    	stc("{wThe flows become wild, and you're not sure how much longer you can hold onto the Source.\n\r", ch );
		ch->stam -= mana / 2;
		stop_spell(ch);
	        return;
		}
    }
    	if(ch->stam < mana)
    	{
	    stc("You don't have enough mana for that.\n\r", ch);
	    stop_spell(ch);
	    return;
        }

	ch->stam -= mana;
	target_name = ch->weaves->target_name; /* freed in free_weave_data */
	(*skill_table[sn].spell_fun) (sn, level, ch, vo, target);
	check_improve (ch, sn, TRUE, 3);

	trash = ch->weaves;
	ch->weaves = trash->next;
	free_weave_data( trash );
	/* ch->action[0] = ACTION_NONE; */

	if ((skill_table[sn].target == TAR_CHAR_OFFENSIVE
	|| (skill_table[sn].target == TAR_OBJ_CHAR_OFF
	&& target == TARGET_CHAR))
	&& victim != ch
	&& victim->master != ch)
	{
        CHAR_DATA *vch;
        CHAR_DATA *vch_next;

        for (vch = ch->in_room->people; vch; vch = vch_next)
        {
		    vch_next = vch->next_in_room;
		    if ( victim == vch && !is_fighting(victim) )
		    {
				if (IS_NPC(victim))
				{
					if (IS_SET(victim->act,ACT_TRAIN)
					||  IS_SET(victim->act,ACT_PRACTICE)
		    	    ||  (victim->pIndexData->pShop != NULL)
					||  IS_SET(victim->act,ACT_IS_HEALER)
					||  IS_SET(victim->act,ACT_IS_CHANGER))
					{
						stc("The Creator would not approve.\n\r",ch);
						break;
					}
				}
				check_killer(victim,ch);
				if ( IS_NPC( victim ) && !is_fighting(victim) )
				{
					start_extra_fight(ch, victim, RANGE_MISSLE);
				}
				break;
		    }
        }
    }
    return;
}

void stop_spell( CHAR_DATA *ch )
{
	WEAVE_DATA *trash;

        if ( ch->weaves == NULL )
                return;

	if (ch->weaves->action[0] != ACTION_CASTING)
		return;

	stc("Your concentration is interrupted.\n\r", ch);

	trash = ch->weaves;
	ch->weaves = ch->weaves->next;
	free_weave_data( trash );
/*
	ch->spell_vars[0] = 0;
	ch->spell_vars[1] = 0;
	ch->spell_vars[2] = 0;
	ch->spell_vars[5] = 0;
	ch->vo            = NULL;
	ch->action[0] = ACTION_NONE;
	*/
	return;
}

void update_casting( CHAR_DATA *ch )
{
	int x, y;

	y = --ch->weaves->spell_vars[3];
	if ( y <= 0 )
		/*
		release_spell( ch->spell_vars[0], ch->spell_vars[1], ch, ch->vo, ch->spell_vars[2] );
		*/
		release_spell( ch );
	else
	{
		x = ch->weaves->spell_vars[3];
		ptc( ch, "{g%s{G%s",
			capit2(skill_table[ch->weaves->spell_vars[5]].name),
			x > 6 ? "{R" : x > 3 ? "{Y" : "{G" );
		while ( x > 0 )
		{
			ptc( ch, ">" );
			x--;
		}
		stc("{x\n\r", ch);
	}

	return;
}

void do_release (CHAR_DATA *ch,  char *argument)
{
	CHAR_DATA *vch;
	CHAR_DATA *vch_next;
	char buf[MAX_INPUT_LENGTH];
    if ( IS_NPC(ch) )
    {
		send_to_char_bw("Why worry about it? Only players can channel.\n\r",ch);
    	return;
    }
    if (!IS_SET( ch->op_flags, OP_GRASPED))
    {
	    if (ch->sex == SEX_FEMALE)
	    {
			stc("{wBut you aren't holding {WSa{wid{War.{c\n\r",ch);
	    	return;
		}
	    else
	    {
			stc("{wBut you aren't holding {WSa{wid{Win.{c\n\r",ch);
	    	return;
		}
	}
	else
	{
		REMOVE_BIT(ch->op_flags,OP_GRASPED);
		stc("{wYou release the {WTrue Source.{c\n\r",ch);
	}
	while (ch->weaves && ch->weaves->action[0] == ACTION_CASTING)
		stop_spell( ch );
	vch = ch->in_room->people;
	for( ; vch != NULL; vch = vch_next)
	{
	  	vch_next = vch->next_in_room;
	    if (vch == ch)
	    	continue;
	    if ( vch->desc == NULL )
			return;
	    if ( vch->position < POS_SLEEPING )
			return;
	    if ( vch->position == POS_SLEEPING )
			return;
	    if ( !check_blind( vch ) )
			return;
	    if ( IS_SET( vch->op_flags, OP_CHANNELER) )
	    {
		    if (vch->sex == SEX_MALE && ch->sex == SEX_MALE /*&& IS_SET( vch->op_flags, OP_CHANNELER )*/ )
		    {
			    sprintf(buf, "{wThe resonance of {WSa{wid{Win from %s fades out.{x\n\r", ch->name);
			    stc(buf, vch);
		    }
		    else if (vch->sex == SEX_FEMALE && ch->sex == SEX_FEMALE /*&& IS_SET( vch->op_flags, OP_CHANNELER )*/)
		    {
			    sprintf(buf, "{wThe glow of {WSa{wid{War surrounding %s winks out.{x\n\r", ch->name);
			    stc(buf, vch);
		    }
		}
    }
    return;

}

void do_grasp (CHAR_DATA *ch,  char *argument)
{
	CHAR_DATA *vch;
	CHAR_DATA *vch_next;

/*	char arg[MIL];*/
	char buf[MAX_INPUT_LENGTH];

	int x = 0;

    if ( IS_NPC(ch) )
    {
		send_to_char_bw("Only players can channel\n\r",ch);
    	return;
    }
    if (!IS_SET( ch->op_flags, OP_CHANNELER))
    {
		stc("{wBut you can't channel the {WOne Power.{c\n\r",ch);
    	return;
	}
	if ( IS_SET( ch->in_room->room_flags,ROOM_STEDDING ) )
	{
		stc("{wYou cannot seem to sense the True Source for some reason.\n\r", ch);
		return;
	}
	if (IS_SET(ch->op_flags,OP_GRASPED))
	{
		stc("{wYou are already holding the {WTrue Source.{c\n\r",ch);
		return;
	}
/*
	if ( !is_number( arg ) )
	{
		stc("{wYou must actually grasp an amount of the {WOne Power.{c\n\r",ch);
		return;
	}
	x = atoi( arg );
	if ( x <= 0 )
	{
		stc("{wA great sucking vortex opens up and swallows you whole.{c\n\r",ch);
		return;
	}
*/
	SET_BIT(ch->op_flags,OP_GRASPED);
	ch->pcdata->grasped = x;

	if (ch->sex == SEX_FEMALE)
	{
		stc("{wYou open yourself to Saidar.{c\n\r",ch);
	}
	else
	{
		stc("{wYou can feel the {rD{Rar{rk O{Rne{r's{w taint on the {WTrue Source.{c\n\r",ch);
		stc("{wYou wrestle {WSa{wid{Win{w under your control.{c\n\r",ch);
	}
	vch = ch->in_room->people;
	for( ; vch != NULL; vch = vch_next)
	{
	  	vch_next = vch->next_in_room;
	    if (vch == ch)
	    	continue;
	    if ( vch->desc == NULL )
			return;
	    if ( vch->position < POS_SLEEPING )
			return;
	    if ( vch->position == POS_SLEEPING )
			return;
	    if ( !check_blind( vch ) )
			return;
	    if ( IS_SET( vch->op_flags, OP_CHANNELER) )
	    {
		    if (vch->sex == SEX_MALE && ch->sex == SEX_MALE)
		    {
			    sprintf(buf, "{wYou feel an echo of {WSa{wid{Win{w resonate from %s.{x\n\r", ch->name);
			    stc(buf, vch);
		    	return;
		    }
		    else if (vch->sex == SEX_FEMALE && ch->sex == SEX_FEMALE)
		    {
			    sprintf(buf, "{wThe glow of {WSa{wid{War{w surrounds %s.{x\n\r", ch->name);
			    stc(buf, vch);
		    	return;
		    }
		}
    }
}
