/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

/* This code is based of the Homes code written by Dalsor
 * It uses certain aspects of OLC, and OLC or the proper componants,
 * must be installed in order for this code to function correctly.
 * The function stc() is a re-defined send_to_char. Either #define
 * it, or change the references to stc(). The function ptc() is a
 * re-defined printf_to_char. This function is available from Erwin's
 * web site, Kyndig's website, and a host of other areas. It also
 * should be #defined or changed.
 *
 * Add pc_shops.o to the Makefile
 *
 * Additions to external files:
 *
 * struct pc_data
 * add: int shop_vnum;
 * add: int shopmob_vnum;
 *
 * Add a new ACT_SHOPMOB flag
 * add: SHOPS_DIR "../player/shops"
 *
 * SAVE.C (or the equivelant)
 * void fwrite_char
 * add: fprintf( fp, "ShpV %d\n",   ch->pcdata->shop_vnum );
 * add: fprintf( fp, "ShpM %d\n",   ch->pcdata->shopmob_vnum );
 *
 * void fread_char
 * at the head of the function
 * add: ch->pcdata->shop_vnum = 0;
 * add: ch->pcdata->shopmob_vnum = 0;
 *
 * in the case 'S' section
 * KEY( "ShpM",	ch->pcdata->shopmob_vnum, fread_number( fp ) );
 * KEY( "ShpV",	ch->pcdata->shop_vnum, fread_number( fp ) );
 *
 * Create a blank area for the holding of the shop rooms and shop
 * mobiles. Create a mobile for each shop room. The mobile vnums
 * should correspond to the room vnums. Simply creating a blank
 * mobile index for each shop room should suffice.
 */

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "recycle.h"
#include "interp.h"
#include "tables.h"

/* Global calls */
RESET_DATA *new_reset_data args ( ( void ) );
SHOP_DATA	*new_shop		args ( ( void ) );
void add_reset args ( ( ROOM_INDEX_DATA *room, RESET_DATA *shop_reset, int index ) );
void free_reset_data args ( ( RESET_DATA *pReset ) );
/* Local calls */
void fwrite_shops args( ( CHAR_DATA *ch, CHAR_DATA *shop_mob ) );
void fread_shops  args( ( CHAR_DATA *ch ) );
void shop_buy      args( ( CHAR_DATA *ch, char *argument ) );
void shop_open     args( ( CHAR_DATA *ch ) );
void shop_close    args( ( CHAR_DATA *ch, char *argument ) );
void shop_describe args( ( CHAR_DATA *ch ) );
void shop_provide  args( ( CHAR_DATA *ch, char *argument, int amount ) );
void shop_collect  args( ( CHAR_DATA *ch ) );
void shop_update   args( ( CHAR_DATA *mob ) );

int shop_cost = 50000;
ROOM_INDEX_DATA *shop;
AREA_DATA *shop_area;
RESET_DATA *shop_reset;

void do_shop( CHAR_DATA *ch, char *argument )
{
	char arg1[MAX_INPUT_LENGTH];
	char arg2[MAX_INPUT_LENGTH];
	char arg3[MAX_INPUT_LENGTH];

	shop = ch->in_room;
	shop_area = ch->in_room->area;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( IS_NPC(ch)
	|| ch == NULL
	|| ch->desc == NULL )
		return;

	if ( arg1[0] == '\0'
	|| !str_prefix ( arg1, "help" )
	|| !str_prefix ( arg1, "list" ) )
	{
		stc("\n\r{r-----------------------------------------------------------{x\n\r",ch);
		stc("{wShop Buy{W:      {r<{Rshop buy (confirm){r>{x\n\r",ch);
		stc("               {wPurchase a shop and place it in the current room.{x\n\r",ch);
		ptc(ch,"               {wCost is %d gold.{x\n\r",shop_cost);
		stc("{wShop Open{W:     {r<{Rshop open{r>{x\n\r",ch);
		stc("               {wIf the MUD crashes, this command will reopen your shop.{x\n\r",ch);
		stc("{wShop Close{W:    {r<{Rshop close (confirm){r>{x\n\r",ch);
		stc("               {wSell the current shop for half of the purchase{x\n\r",ch);
		stc("               {wcost. Will release the shop mobile and all items{x\n\r",ch);
		stc("               {win the mobile's inventory.{x\n\r",ch);
		stc("{wShop Describe{W: {r<{Rshop describe{r>{x\n\r",ch);
		stc("               {wWhile standing in the shop, describe the shop using{x\n\r",ch);
		stc("               {wthe OLC editor. Careful thought should go into the{x\n\r",ch);
		stc("               {wdescription.{x\n\r",ch);
		stc("{wShop Provide{W:  {r<{Rshop provide item_name amount{r>{x\n\r",ch);
		stc("               {wProvide the shop mobile with item_name. The item must{x\n\r",ch);
		stc("               {whave a positive net worth and be held in the inventory.{x\n\r",ch);
		stc("               {wThe amount must be a positive numeric value greater than{x\n\r",ch);
		stc("               {w0 and less than 32000.{x\n\r",ch);
		stc("{wShop Collect{W:  {r<{Rshop collect{r>{x\n\r",ch);
		stc("               {wCollect monies from the shop mobile. The mobile will always{x\n\r",ch);
		stc("               {wretain at least 100 silver coins for making change and to{x\n\r",ch);
		stc("               {wcover basic business needs.{x\n\r",ch);
		stc("{wShop Save{W: {r<{Rshop save{r>{x\n\r",ch);
		stc("               {wForces the data to be written to file. Testing only.{x\n\r",ch);
		stc("{r-----------------------------------------------------------{x\n\r",ch);
		return;
	}

	shop = ch->in_room;
	shop_area = ch->in_room->area;

	if ( shop == NULL || shop_area == NULL )
		return;

	if ( !str_prefix( arg1, "save" ) )
	{
		/*fwrite_shops(ch, arg2);*/
	}
	else if ( !str_prefix( arg1, "buy" ) )
	{
		shop_buy(ch, arg2);
	}
	else if ( !str_prefix( arg1, "open" ) )
	{
		shop_open(ch);
	}
	else if ( !str_prefix( arg1, "close" )
	|| !str_prefix( arg1, "sell" ) )
	{
		shop_close(ch, arg2);
	}
	else if ( !str_prefix( arg1, "describe" ) )
	{
		shop_describe(ch);
	}
	else if ( !str_prefix( arg1, "provide" ) )
	{
		shop_provide(ch, arg2, atoi(arg3));
	}
	else if ( !str_prefix( arg1, "collect" ) )
	{
		shop_collect(ch);
	}
	else if ( !str_prefix( arg1, "test" ) )
	{
		ptc( ch, "ShopV: %d ShopM: %d\n\r", ch->pcdata->shop_vnum, ch->pcdata->shopmob_vnum );
		return;
	}
	else if ( !str_prefix( arg1, "reset" ) )
	{
		ch->in_room->owner = NULL;
		ch->pcdata->shop_vnum = 0;
		ch->pcdata->shopmob_vnum = 0;
		return;
	}
	else
		do_function( ch, &do_shop, "" );
	return;
}

CHAR_DATA * make_shop_mob( CHAR_DATA *ch )
{
	CHAR_DATA *shop_mob;
	int i;

	shop_mob = create_mobile( get_mob_index( shop->vnum ) );
    shop_mob->group	            = 0;
	/* sex */
   	shop_mob->sex		        = ch->sex;
   	shop_mob->pIndexData->sex		        = ch->sex;
	/* race */
    shop_mob->race		        = ch->race;
    shop_mob->pIndexData->race		        = ch->race;
	/* armor class */
    for (i = 0; i < 4; i++)
   		shop_mob->armor[i]		= 1;
    for (i = 0; i < 4; i++)
   		shop_mob->pIndexData->ac[i]		= 1;
	/* damage dice */
    shop_mob->damage[DICE_NUMBER]=51;
   	shop_mob->damage[DICE_TYPE]  = 51;
    shop_mob->damage[DICE_BONUS] = 51;
    shop_mob->pIndexData->damage[DICE_NUMBER]=51;
   	shop_mob->pIndexData->damage[DICE_TYPE]  = 51;
    shop_mob->pIndexData->damage[DICE_BONUS] = 51;
    shop_mob->damroll	        = 100;
	/* level */
   	shop_mob->level	            = 51;
   	shop_mob->pIndexData->level	            = 51;
	/* hitroll */
   	shop_mob->hit		        = 500;
   	shop_mob->pIndexData->hitroll	        = 500;
	/* hit points */
    shop_mob->pIndexData->hit[DICE_NUMBER] = 1;
    shop_mob->pIndexData->hit[DICE_TYPE]   = 499;
    shop_mob->pIndexData->hit[DICE_BONUS]  = 1;
    shop_mob->max_hit	        = 500;
	/* mana points */
   	shop_mob->mana		        = 500;
    shop_mob->pIndexData->mana[DICE_NUMBER] = 1;
    shop_mob->pIndexData->mana[DICE_TYPE]   = 499;
    shop_mob->pIndexData->mana[DICE_BONUS]  = 1;
    shop_mob->max_mana	        = 500;
	/* wealth - THIS NEEDS TO BE READ FROM PFILE OR OTHER EXTERNAL FILE!!! */
   	shop_mob->money[3][0]	    = 1;
    shop_mob->money[2][0]	    = 1;
   	shop_mob->pIndexData->wealth	    	    = 1;
   	shop_mob->imm_flags  	    = 0;
    shop_mob->res_flags  	    = 0;
   	shop_mob->vuln_flags   	    = 0;
   	shop_mob->pIndexData->imm_flags  	    = 0;
    shop_mob->pIndexData->res_flags  	    = 0;
   	shop_mob->pIndexData->vuln_flags   	    = 0;
   	shop_mob->act               += ACT_SENTINEL;
   	shop_mob->act2              += ACT2_SHOPMOB;
   	shop_mob->pIndexData->act               += ACT_SENTINEL;
   	shop_mob->pIndexData->act2              += ACT2_SHOPMOB;
    shop_mob->position	        = POS_STANDING;
   	shop_mob->start_pos	        = POS_STANDING;
    shop_mob->pIndexData->default_pos        = POS_STANDING;
   	shop_mob->pIndexData->start_pos	        = POS_STANDING;
   	shop_mob->saving_throw	    = 0;
    shop_mob->alignment	        = 0;
    shop_mob->pIndexData->alignment	        = 0;
   	shop_mob->form		        = race_table[shop_mob->race].form;
   	shop_mob->pIndexData->form		        = race_table[shop_mob->race].form;
    shop_mob->parts	            = race_table[shop_mob->race].parts;
    shop_mob->pIndexData->parts	            = race_table[shop_mob->race].parts;
   	shop_mob->size		        = SIZE_MEDIUM;
   	shop_mob->pIndexData->size		        = SIZE_MEDIUM;
	shop_mob->dam_type	        = attack_lookup("pound");
	shop_mob->pIndexData->dam_type	        = attack_lookup("pound");

	free_string( shop_mob->name );
    shop_mob->name              = str_dup( "shop mobile shopkeeper" );
	free_string( shop_mob->name );
    shop_mob->pIndexData->player_name              = str_dup( "shop mobile shopkeeper" );

    free_string( shop_mob->short_descr );
    shop_mob->short_descr       = str_dup( "The shopkeeper" );
    free_string( shop_mob->short_descr );
    shop_mob->pIndexData->short_descr       = str_dup( "The shopkeeper" );

    free_string( shop_mob->long_descr );
    shop_mob->long_descr        = str_dup( "The shopkeeper stands patiently nearby.\n\r" );
    free_string( shop_mob->long_descr );
    shop_mob->pIndexData->long_descr        = str_dup( "The shopkeeper stands patiently nearby.\n\r" );

   	if ( !shop_mob->pIndexData->pShop )
	{

		shop_mob->pIndexData->pShop		= new_shop();
		if ( !shop_first )
        	shop_first	= shop_mob->pIndexData->pShop;
		if ( shop_last )
			shop_last->next	= shop_mob->pIndexData->pShop;
		shop_last		= shop_mob->pIndexData->pShop;
		shop_mob->pIndexData->pShop->keeper	= shop_mob->pIndexData->vnum;
		shop_mob->pIndexData->pShop->open_hour = 0;
		shop_mob->pIndexData->pShop->close_hour = 24;
		shop_mob->pIndexData->pShop->profit_buy  = 100;
		shop_mob->pIndexData->pShop->profit_sell = 100;
		/*shop_mob->pIndexData->pShop->buy_type = 0;*/
	}

	shop_reset               = new_reset_data();
	shop_reset->command      = 'M';
	shop_reset->arg1         = shop_mob->pIndexData->vnum;
	shop_reset->arg2         = 1;
	shop_reset->arg3         = shop->vnum;
	shop_reset->arg4         = 1;
	add_reset( shop, shop_reset, 0 );

    ch->pcdata->shopmob_vnum = shop_mob->pIndexData->vnum;
    ch->pcdata->shop_vnum = shop->vnum;

	char_to_room( shop_mob, shop );
    return shop_mob;
}
void shop_open ( CHAR_DATA *ch )
{
	/* reopen a shop in case of MUD crash.
	 * if the MUD never crashes, this is not
	 * necessary, but the option is here just
	 * in case.
   * This is just a buffer command for now.
   * Later it should perform preemptive error
   * checking before read.
	 */
	if ( !IS_SET( shop->room_flags,ROOM_HOME ) )
	{
		stc("But this is not a shop!\n\r",ch);
		return;
	}
/*
  if ( ch->pcdata->shop_vnum == 0 || ch->pcdata->shop_vnum == 0 )
	{
		stc("But you do not own a shop!\n\r",ch);
		return;
	}
*/
  if ( !is_room_owner ( ch, shop ) )
	{
		stc("But you do not own this shop!\n\r",ch);
		return;
	}
  fread_shops( ch );
  return;
}
void shop_buy ( CHAR_DATA *ch, char *argument )
{
	/* create shop from blank room.
	 * create blank mobile like faire mobs and place into resets
	 * Each mobile should be unique to the room, ie. 100 rooms
	 * equals 100 different mobs.
	 * assign ownership of room and mob to pfile
	 * flag room as safe
	 * flag area as changed
	 */
	char buf[MSL];
	buf[0] = '\0';

	if ( !IS_SET ( shop->room_flags, ROOM_HOME ) )
	{
		stc("This isn't a shop.\n\r",ch);
		return;
	}
	/* Shops are free during testing
	if ( ch->gold < 50000 )
	{
		stc("You can't afford it.\n\r",ch);
		return;
	}*/
	if ( ch->pcdata->shop_vnum != 0 )
	{
		stc("You already have a shop.\n\r",ch);
		return;
	}
	if ( str_cmp( argument, "confirm" ) )
	{
		stc("Confirm expenditure of 50,000 gold?\n\r",ch);
		stc("Type Shop Buy Confirm\n\r",ch);
		return;
	}
  ch->pcdata->shop_vnum = ch->in_room->vnum;
  ch->pcdata->shopmob_vnum = ch->in_room->vnum;

    free_string ( shop->owner );
    shop->owner = str_dup ( ch->name );
    /* Shops are free during testing */
    /*ch->gold -= 50000;*/
    free_string ( shop->name );
    sprintf ( buf, "%s's Shop", ch->name );
    shop->name = str_dup ( buf );

	/* flag the room safe it its not */
    if ( !IS_SET(shop->room_flags,ROOM_SAFE) )
    	SET_BIT( shop->room_flags,ROOM_SAFE );

    make_shop_mob( ch );
    stc("Congratulations on purchasing your new shop!\n\r",ch);
    stc("You are now 50,000 gold coins lighter and the owner\n\r",ch);
    stc("of the room in which you now stand! Be sure to describe\n\r",ch);
    stc("your shop and provide merchandise for it.\n\r",ch);
	return;
}
void shop_close ( CHAR_DATA *ch, char *argument )
{
	/* check for comfirmation
	 * remove mobile and items
	 * credit the char for half of item value
	 * strip resets
	 * remove room and mobile vnums from pfile
	 * flag area as changed
	 */
	char shop_file[MIL];
	RESET_DATA *current_reset;
	RESET_DATA *next_reset;
  CHAR_DATA *shop_mob;
	sprintf( shop_file, "%s%s", SHOPS_DIR, capitalize( ch->name ) );

	if ( !is_room_owner ( ch, shop ) )
	{
		stc("But you do not own this shop!\n\r",ch);
		return;
	}
	if ( argument[0] == '\0' )
	{
		stc("This command allows you to sell your shop.\n\r",ch);
		stc("You will no longer own your shop once it is sold,\n\r",ch);
		stc("and you will be compensated half the cost of the shop.\n\r",ch);
		stc("You must be standing in the shop which you own.\n\r",ch);
		stc("\n\rSyntax: Shop (close) (confirm)\n\r",ch);
		return;
	}
	else if ( !strcmp ( argument, "confirm" ) )
	{
	    free_string ( shop->owner );
	    shop->owner = str_dup ( "" );
	    /* Shops are free during testing */
	    /*ch->gold += 25000;*/

	    ch->pcdata->shopmob_vnum = 0;
	    ch->pcdata->shop_vnum = 0;
	    free_string ( shop->name );
	    shop->name = str_dup ( "An Empty Building" );
	    free_string ( shop->description );
	    shop->description = str_dup ( "" );

	    /* remove safe flags if set */
	    if ( IS_SET(shop->room_flags,ROOM_SAFE) )
	    	REMOVE_BIT( shop->room_flags,ROOM_SAFE );

	    if ( !ch->in_room->reset_first )
	    {
			return;
		}
		for( current_reset = ch->in_room->reset_first; current_reset;
		    current_reset = next_reset)
		{
		   next_reset = current_reset->next;
		   free_reset_data(current_reset);
		}

		ch->in_room->reset_first = NULL;

    for ( shop_mob = ch->in_room->people; shop_mob != NULL; shop_mob = shop_mob->next_in_room )
    {
      if ( IS_NPC( shop_mob )
      && shop_mob->pIndexData
      && shop_mob->pIndexData->vnum == ch->pcdata->shopmob_vnum )
      {
        extract_char( shop_mob, TRUE );
        break;
      }
    }
    
    ch->pcdata->shop_vnum = 0;
    ch->pcdata->shopmob_vnum = 0;

		stc("Your shop has been sold and you are now 25,000 gold coins richer!\n\r",ch);
        SET_BIT(shop_area->area_flags, AREA_CHANGED);

	/* unlink the data file from the shops directory */
	unlink( shop_file );

        return;
	}
	else
	{
		stc("This command allows you to sell your shop.\n\r",ch);
		stc("You will no longer own your shop once it is sold,\n\r",ch);
		stc("and you will be compensated half the cost of the shop.\n\r",ch);
		stc("You must be standing in the shop which you own.\n\r",ch);
		stc("\n\rSyntax: Shop (close) (confirm)\n\r",ch);
		return;
	}
	return;
}
void shop_describe ( CHAR_DATA *ch )
{
	/* show help on using the editor
	 * place character into editor
	 * save room desc and flag as changed
	 */
	if ( !is_room_owner ( ch, shop ) )
	{
		stc("But you do not own this shop!\n\r",ch);
		return;
	}
	else
	{
		stc("This command allows you to describe your shop.\n\r",ch);
		stc("If you currently own this shop, you will be placed into.\n\r",ch);
		stc("the OLC editor. Be warned that while in the OLC editor,\n\r",ch);
		stc("you are only allowed to type the description. If you are\n\r",ch);
		stc("unsure or hesitant about this, please note the Staff,\n\r",ch);
		stc("or better, discuss the how-to's with a Builder.\n\r",ch);
		stc("Syntax: Shop (describe)\n\r",ch);
    	string_append( ch, &shop->description );
        SET_BIT(shop_area->area_flags, AREA_CHANGED);
		return;
	}
	return;
}
void shop_provide ( CHAR_DATA *ch, char *argument, int amount )
{
	/* check for valid item
	 * check for valid mobile
	 * fix item cost according to amount
	 * add item to resets
	 * flag area as changed
	 * inform next copyover item will be available
	 */
	CHAR_DATA *shop_mob;
	OBJ_DATA *item;

	if ( argument[0] == '\0' || amount <= 0 || amount > 32000 )
	{
		stc("{wShop Provide{W:  {r<{Rshop provide item_name amount{r>{x\n\r",ch);
		stc("               {wProvide the shop mobile with item_name. The item must{x\n\r",ch);
		stc("               {whave a positive net worth and be held in the inventory.{x\n\r",ch);
		stc("               {wThe amount must be a positive numeric value greater than{x\n\r",ch);
		stc("               {w0 and less than 32000.{x\n\r",ch);
		return;
	}
    if ( ( item = get_obj_carry( ch, argument, ch ) ) == NULL )
    {
		stc( "Check your pockets: You do not have that item.\n\r", ch );
		return;
    }
    /* do some error checking against bad merchandise */
    if ( item->item_type == ITEM_LIGHT && item->value[0] <= 0 )
    {
		stc( "That light will be useless soon and wouldn't make good merchandise.\n\r", ch );
		return;
    }
    if ( item->item_type == ITEM_TRASH )
    {
		stc( "Why would anyone want to buy trash?\n\r", ch );
		return;
    }
    if ( item->item_type == ITEM_ROOM_KEY )
    {
		stc( "Selling room keys probably isn't good business.\n\r", ch );
		return;
    }
    if ( item->item_type == ITEM_TREE_PC )
    {
		stc( "You'd have better luck giving that to an Ogier.\n\r", ch );
		return;
    }
    if ( item->item_type == ITEM_TREASURECHEST )
    {
		stc( "Try selling the items inside the chest instead.\n\r", ch );
		return;
    }
    if ( item->item_type == ITEM_CONTAINER )
    {
		stc( "Sorry, you can't sell containers this way.\n\r", ch );
		return;
    }
    if ( !CAN_WEAR(item, ITEM_TAKE) )
    {
		stc( "How do you propose to sell something that can't be picked up?\n\r", ch );
		return;
    }
    /* ok, that's enough checking */
    item->cost = amount;

	if ( !IS_SET( shop->room_flags,ROOM_HOME ) )
	{
		stc("But this is not a shop!\n\r",ch);
		return;
	}
	if ( ch->pcdata->shop_vnum == 0 || ch->pcdata->shop_vnum == 0 )
	{
		stc("But you do not own a shop!\n\r",ch);
		return;
	}
	if ( !is_room_owner ( ch, shop ) )
	{
		stc("But you do not own this shop!\n\r",ch);
		return;
	}
	for (shop_mob = ch->in_room->people; shop_mob != NULL; shop_mob = shop_mob->next_in_room )
	{
  	if ( shop_mob == NULL )
	  {
  		stc( "No shopkeeper found here!\n\r", ch );
	  	return;
  	}
		if ( IS_NPC(shop_mob) && shop_mob->pIndexData->vnum == ch->pcdata->shopmob_vnum )
			break;
	}
	ptc( ch, "You confer with %s for a bit.\n\r", shop_mob->short_descr );
	act( "$n confers with $N for a bit.", ch,NULL,shop_mob->short_descr,TO_ROOM );
	obj_from_char( item);
	obj_to_char( item, shop_mob );
	fwrite_shops( ch, shop_mob );
	return;
}
void shop_collect ( CHAR_DATA *ch )
{
	/* check for valid mobile
	 * check for mobile wealth over 100
	 * deduct mob and add wealth over 100 to character
	 */
	if ( !is_room_owner ( ch, shop ) )
	{
		stc("But you do not own this shop!\n\r",ch);
		return;
	}
	return;
}

void fread_shops( CHAR_DATA *ch )
{
	FILE *fp;
	char name[24];
  CHAR_DATA * shop_mob;
  bool shopMob = FALSE;

  sprintf( name, "%s%s", SHOPS_DIR, capitalize( ch->name ) );

	if ( ch == NULL || ch->pcdata == NULL || ch->desc == NULL )
  {
		bug("NULL CH trying to write pc_shop data!", 0 );
		return;
  }

	if ( ch->pcdata->shop_vnum == 0 || ch->pcdata->shopmob_vnum == 0 )
	{
		stc("But you do not own a shop!\n\r",ch);
		return;
	}

  for ( shop_mob = ch->in_room->people; shop_mob != NULL; shop_mob = shop_mob->next_in_room )
  {
    if ( IS_NPC( shop_mob )
    && shop_mob->pIndexData
    && shop_mob->pIndexData->vnum == ch->pcdata->shopmob_vnum )
    {
      shopMob = TRUE;
      break;
    }
  }
  if ( !shopMob )
  {
    stc( "No shop mob found here.\n\r", ch );
    return;
  }
  else
  {
    stc( "Found a shop mob here. Reading shop data and supplying the mobile.\n\r", ch );
  }
  if ( ( fp = fopen( name, "r" ) ) == NULL )
  {
    stc( "No shop data found for you.\n\r", ch );
		bugf("Could not open %s for read access!", name );
		return;
  }
  else
  {
	  char letter;
	  char *word;
    shop = ch->in_room;

		for( ; ; )
		{
      /*word = feof(fp) ? "End" : fread_word(fp);*/

	    letter = fread_letter( fp );
	    word = fread_word( fp );

	    if ( letter != '#' )
	    {
				bugf( "Fread_shops: # not found in shop for %s.", ch->name );
				break;
	    }
      if( !str_cmp(word, "EOF" )  )
        break;
      if ( !str_cmp( word, "OBJECT" ) )
	      fread_obj( shop_mob, fp );
 		}

		fclose( fp );
		return;
  }

  return;
}
void fwrite_shops( CHAR_DATA *ch, CHAR_DATA *shop_mob )
{
	FILE *fp;
	char name[24];

    sprintf( name, "%s%s", SHOPS_DIR, capitalize( ch->name ) );

	if ( ch == NULL || ch->pcdata == NULL || ch->desc == NULL )
    {
		bug("NULL CH trying to write pc_shop data!", 0 );
		return;
    }
	if ( shop_mob == NULL )
    {
		bug("NULL SHOP_MOB trying to write pc_shop data!", 0 );
		return;
    }

    if ( ( fp = fopen( name, "w" ) ) == NULL )
    {
		bugf("Could not open %s for write access!", name );
		return;
    }
    else
    {
/*      
	    fprintf( fp, "Char    %s~\n", ch->name);
	    fprintf( fp, "MVnum   %d\n", shop_mob->pIndexData->vnum);
*/
		if ( shop_mob->carrying != NULL )
		    fwrite_obj( shop_mob, shop_mob->carrying, fp, 0 );
    fprintf(fp, "EOF" );
		fclose( fp );
	   	return;
    }
	return;
}
