/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
        intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

        ROM 2.4 is copyright 1993-1998 Russ Taylor
        ROM has been brought to you by the ROM consortium
            Russ Taylor (rtaylor@hypercube.org)
            Gabrielle Taylor (gtaylor@hypercube.org)
            Brian Moore (zump@rom.org)
        By using this code, you have agreed to follow the terms of the
        ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "magic.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "mysql5_interface.h"

int sort_skill_by_weave_then_alpha(const void *askill, const void *bskill )
{
        struct sort_skill_struct aaskill;
        struct sort_skill_struct bbskill;
        aaskill = *(struct sort_skill_struct *) askill;
        bbskill = *(struct sort_skill_struct *) bskill;
        if (aaskill.weave && !bbskill.weave)
                return 1;
        else if (!aaskill.weave && bbskill.weave)
                return -1;
        else
        return strcmp(aaskill.name,bbskill.name);
}

void do_practice( CHAR_DATA *ch, char *argument )
{
        char buf[MSL];
        char buf2[MSL];
	char arg1[MIL], arg2[MIL];
        char cbuf[MSL];
        int sn, cost, stat, amt;
        CHAR_DATA *viewer = NULL;

        if ( IS_NPC(ch) )
                return;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	//ptc( ch, "Argument: %s  Arg1: %s  Arg2: %s\n\r", argument, arg1, arg2 );

	if ( arg2[0] != '\0' )
	{
		if ( !is_number( arg2 ) )
		{
			stc("If specifying a practice amount, the argument must be a valid numeric.\n\r", ch);
			stc("If the skill or weave contains two or more words, enclose them with single quotes.\n\r", ch);
			stc("Example: practice void 7\n\r", ch);
			stc("Example: practice 'hand to hand' 5\n\r", ch);
			return;
		}
		amt = atoi ( arg2 );
		if ( amt <= 0 || amt >= 99 )
		{
			stc("Valid ranges are 1 - 99.\n\r", ch);
			return;
		}
	}
	else
	{
		amt = 1;
		//strcpy ( argument, arg );
	}

	if ( amt > 1 )
		ptc( ch, "Multiplying your cost and number of practices by %d.\n\r", amt );

        if ( arg1[0] == '\0' || ((( viewer = get_pc_world( ch, arg1 ) ) != NULL) && !(viewer->invis_level>0 && !IS_IMMORTAL(ch))) )
        {
                int col;
                int skills;
                struct sort_skill_struct sort_skill[MAX_SKILL];
                int i;
                BUFFER *output;
                output = new_buf();

                col    = 0;
                skills = 0;
                i      = 0;
                for ( sn = 0; sn < MAX_SKILL; sn++ )
                {
                        if ( skill_table[sn].name == NULL )
                                break;
                        if ( ch->pcdata->learned[sn] < 1 )
                                continue;
                        if ( !str_cmp( skill_table[sn].name, "reserved" ) )
                                continue;
                        if (!can_gain_sk(ch, sn))
                                continue;

                        sort_skill[i].name = skill_table[sn].name;
                        sort_skill[i].learned = ch->pcdata->learned[sn];
                        sort_skill[i].weave = ( skill_table[sn].spell_fun != spell_null);
                        i++;
                }

                qsort(sort_skill,i,sizeof(struct sort_skill_struct),sort_skill_by_weave_then_alpha);
                for ( sn = 0; sn < i; sn++ )
                {
                        if (is_qskill(sn))
                                sprintf(cbuf,"{C");
                        else
                                sprintf(cbuf,"{w");

                        if (sort_skill[sn].learned < 75)
                                sprintf( buf2, "{x");/*x*/
                        else if (sort_skill[sn].learned < 85)
                                sprintf( buf2, "{y");
                        else if (sort_skill[sn].learned < 100)
                                sprintf( buf2, "{g");/*g*/
                        else
                                sprintf( buf2, "{w");

                        sprintf( buf, "%s%-18s %s%3d%%  ", cbuf, sort_skill[sn].name , buf2, sort_skill[sn].learned );
                        add_buf(output,buf);

                        if ( ++col % 3 == 0 )
                                add_buf(output,"{c\n\r{x");
                }

                if ( col % 3 != 0 )
                        add_buf(output,"{c\n\r{x");

                sprintf( buf, "You have %d training sessions left.\n\r", ch->train);
                add_buf(output,buf);

                if(viewer)
                {
                        page_to_char( buf_string(output), viewer );
                        ptc(ch,"You show your practice sheet to %s.\n\r",viewer->name);
                }
                else
                        page_to_char( buf_string(output), ch );

                free_buf(output);
	}
	else
	{
		CHAR_DATA *mob;
		int adept;

		if ( !IS_AWAKE(ch) )
		{
			stc( "In your dreams, or what?\n\r", ch );
			return;
		}

		for ( mob = ch->in_room->people; mob != NULL; mob = mob->next_in_room )
		{
			if ( IS_NPC(mob) && IS_SET(mob->act, ACT_PRACTICE) )
				break;
		}

		if ( mob == NULL )
		{
			stc( "You can't do that here.\n\r", ch );
			return;
		}

		if ( ch->train <= 0 )
		{
			stc( "You have no training sessions left.\n\r", ch );
			return;
		}

		if ( ( sn = find_spell( ch, arg1 ) ) < 0 )
		{
			stc( "You can't practice that.\n\r", ch );
			return;
		}

		if ( ch->pcdata->learned[sn] < 1 )
		{
			stc( "You need to gain that first.\n\r", ch );
			return;
		}

		if ( !str_cmp( skill_table[sn].name, "reserved" ) )
		{
			stc("No point in that.\n\r",ch);
			return;
		}

		if ( skill_table[sn].rating[ch->class] == 0 )
		{
			stc( "You wouldn't know what to do with that.\n\r", ch );
			return;
		}


		/* newbies can practice up to 100% before they've been reborn
		 * Each rebirth decrements the max practice by 1%
		 * Gifted otherwise always allows practicing up to 85%
		 */

		if (IS_NPC(ch))
		{
			adept = 100;
		}


		if ( !IS_NPC( ch ) )
		{
			int base = 100;

			base -= ch->pcdata->rebirth;
			//ptc( ch, "BASE = %d after subtracting %d remorts.\n\r", base, ch->pcdata->rebirth );

			/*
			if ( base < 25 ) 
				base = 25;
			*/

			adept = base;
			//ptc( ch, "ADEPT = %d after subtracting %d modified BASE.\n\r", adept, base );

			if ( adept < 75 )
				adept = 75;
			if ( ch->pcdata->learned[gsn_gifted] > 0 && adept < 85 )
				adept = 85;
			//ptc( ch, "ADEPT = %d after final modifications.\n\r", adept );
		}


		if ( ch->pcdata->learned[sn] >= adept )
		{
			ptc( ch, "You are already learned at %s.\n\r", skill_table[sn].name );
		}
		else
		{
			cost = skill_table[sn].rating[ch->class];
			//ptc( ch, "COST before modifications = %d\n\r", cost );

			stat = get_curr_stat(ch,STAT_INT) / 40;
			//ptc( ch, "STAT before modifications = %d\n\r", stat );

			stat += get_curr_stat(ch,STAT_CHA) / 60;
			//ptc( ch, "STAT after initial divisions = %d\n\r", stat );

                        stat += get_curr_stat(ch,STAT_WIS) / 60;
                        //ptc( ch, "STAT after initial divisions = %d\n\r", stat );

                        stat += get_curr_stat(ch,STAT_INS) / 60;
                        //ptc( ch, "STAT after initial divisions = %d\n\r", stat );

			cost -= stat;

			if (ch->pcdata->rebirth <= 25)
			{
				cost = 1;
				stat = 10;
			}
			if (cost < 1) cost = 1;
			if (stat < 1) stat = 1;

			cost *= amt;
			stat *= amt;

			//ptc( ch, "Cost = %d  Stat = %d\n\r", cost, stat );

			if ( ch->train < cost )
			{
				ptc( ch, "You do not have enough training points. You need %d to increase by %d.\n\r", cost, stat);
				return;
			}
			ch->pcdata->learned[sn] += stat;
			ch->train -= cost;
			ptc(ch, "It cost you %d training points to increase %s %d percent.\n\r", cost, skill_table[sn].name, stat );

			if (ch->pcdata->rebirth <= 0)
			{
				stc("Because you are new, you are gaining a bonus to your training.\n\r",ch);
				stc("After you have remorted once you will no longer gain this bonus.\n\r",ch);
			}

			if ( ch->pcdata->learned[sn] < adept )
				act( "You practice $T.",ch, NULL, skill_table[sn].name, TO_CHAR );
			else
			{
				ch->pcdata->learned[sn] = adept;
				act( "You are now learned at $T.",ch, NULL, skill_table[sn].name, TO_CHAR );
			}
		}
	}
	return;
}

