/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "tables.h"

/* Quest Skill Functions */

/* Anticipate    - Gives a person a chance to anticipate an opponents
 *                 attack for a better dodge or parry.
 * Precision     - Grants a better chance to hit.
 * Efficient     - Decreases the amount of mana and move used when
 *                 weaving or travelling.
 * Gifted        - Allows raising of a stat greater than 100.
 * Windfall      - Grants a bonus to experience and selling items.
 * Evaluate      - Allows extended information on shop items.
 * Intimidate    - Automatic check to see if char can intimidate an
 *                 opponent and lower its armor class.
 * Disguise      - Difficult skills which allows passing of guards.
 * Bloodlust     - Damage roll and damage increase, but to hit and
 *                 armor class decrease.
 * Bribe         - Bribe a shopkeeper into giving information about
 *                 a player who is currently online.
 * Angreal Study - More power in this skill grants better bonuses
 *                 when weaving.
 * Engraving     - Ability to engrave items with insignias.
 * Balancing     - Like sharpening, but only works on forged weapons.
 * True Sense    - Grants the ability to double attack weaves so that
 *                 the channeler can weave two fireballs at once.
 * Feint         - Tricks the opponent for a better next attack
 * Lunge         - Forward for less chance to hit, but better damage
 * Draw          - Step backwards and strike, more damage, less defense
 * Stagger       - Use the weapon to try and stagger the opponent backwards
 * Shieldbash    - Bash the opponent with a powerful shield attack, spiked
                   shields hit harder
 * Swordbreak    - Try and break the opponents weapon, chance it backfires
 * Shieldbreak   - Try to break the opponents shield
 * Chop          - Attack with a strong downwards strike
 * Sweep         - Try and sweep the opponents legs out from under them,
                   better than trip or footsweep
 * Dismember     - Try and cut off the opponents body parts
 * Ravage        - Ravage a corpse with the weapon
 * Focus         - Focus whirlwind, tornado, hurricane on one opponent
 */

/* Global calls */
CHAR_DATA *find_keeper args(( CHAR_DATA *ch ));
int	get_cost	args( (CHAR_DATA *keeper, OBJ_DATA *obj, bool fBuy ) );

/* The booleans
 *
 */
bool check_anticipate( CHAR_DATA *ch ) /* done */
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_anticipate) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_anticipate );
	chance += ch->perm_stat[STAT_INS];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_anticipate, TRUE, 2 );
		return TRUE;
	}
}
bool check_focus( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_focus) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_focus );
	chance += ch->perm_stat[STAT_REF];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_focus, TRUE, 2 );
		return TRUE;
	}
}
bool check_precision( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_precision) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_precision );
	chance += ch->perm_stat[STAT_DEX];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_precision, TRUE, 2 );
		return TRUE;
	}
}
bool check_efficient( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_efficient) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_efficient );
	chance += ch->perm_stat[STAT_REF];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_efficient, TRUE, 2 );
		return TRUE;
	}
}
bool check_gifted( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_gifted) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_gifted );
	chance += ch->perm_stat[STAT_CHA];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
		return TRUE;
}
bool check_windfall( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_windfall) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_windfall ) * 2; /* Only matters up to 100, when they get to keep the skill past remort */
	chance += ch->level / 4;			/* Can go up to 1000, but typically about 400 at the level I want it to stay */
	chance /= 2;
	chance = URANGE(10, chance, 90);		/* Makes it always atleast 10% chance and 90% at most. */

	if (number_percent() >= chance)
	{
		check_improve( ch, gsn_windfall, FALSE, 2);
		return FALSE;
	}
	else
	{
		check_improve( ch, gsn_windfall, TRUE, 4);
		return TRUE;
	}
}

void show_evaluation( CHAR_DATA *ch, OBJ_DATA *obj )
{
    char buf[MSL];
    AFFECT_DATA *paf;

    sprintf( buf, "{cThis is %s{c\n\r",obj->short_descr );
    stc( buf, ch );

    sprintf( buf, "{cIt weighs %d stones and might cost %d silver coins.{c\n\r",
	obj->weight, obj->cost );
    stc( buf, ch );

    sprintf( buf, "{cIt belongs to the %s Guild.{c\n\r",
	obj->guild );
    stc( buf, ch );

    if (IS_SET(obj->extra_flags,ITEM_REMORT))
    {
		stc("It is a remort-only item.\n\r",ch);
	}

	ptc( ch, "Wear flags: %s\n\r", wear_bit_name(obj->wear_flags) );
    /* now give out vital statistics as per identify */

    switch ( obj->item_type )
    {
    	case ITEM_SCROLL:
    	case ITEM_POTION:
    	case ITEM_PILL:
	    ptc( ch, "{cIt is a valuable treasure!\n\r" );

	    if ( obj->value[1] >= 0 && obj->value[1] < MAX_SKILL )
	    {
	    	stc( "{c '", ch );
	    	stc( skill_table[obj->value[1]].name, ch );
	    	stc( "{c'", ch );
	    }

	    if ( obj->value[2] >= 0 && obj->value[2] < MAX_SKILL )
	    {
	    	stc( "{c '", ch );
	    	stc( skill_table[obj->value[2]].name, ch );
	    	stc( "{c'", ch );
	    }

	    if ( obj->value[3] >= 0 && obj->value[3] < MAX_SKILL )
	    {
	    	stc( "{c '", ch );
	    	stc( skill_table[obj->value[3]].name, ch );
	    	stc( "{c'", ch );
	    }

	    if (obj->value[4] >= 0 && obj->value[4] < MAX_SKILL)
	    {
		stc("{c '",ch);
		stc(skill_table[obj->value[4]].name,ch);
		stc("{c'",ch);
	    }

	    stc( "{c.{c\n\r", ch );
	break;

    case ITEM_QUIVER:
		ptc(ch,"{WIt holds {R%d %d{Wd{R%d {Warrows.{c{c\n\r",
		obj->value[0], obj->value[1], obj->value[2]);
		break;
	case ITEM_ARROW:
		ptc(ch,"{WThis arrow will do {R%d{Wd{R%d {Wdamage for an average of {R%d{W.{c{c\n\r",
		obj->value[1], obj->value[2], ( obj->value[1] + obj->value[2] ) / 2 );
		break;

    	case ITEM_WAND:
    	case ITEM_STAFF:
	    ptc( ch, "{cThis is an item of the One Power.\n\rHas %d(%d) charges ",
	    	obj->value[1], obj->value[2] );

	    if ( obj->value[3] >= 0 && obj->value[3] < MAX_SKILL )
	    {
	    	stc( "{c '", ch );
	    	stc( skill_table[obj->value[3]].name, ch );
	    	stc( "{c'", ch );
	    }

	    stc( "{c.{c\n\r", ch );
	break;

	case ITEM_DRINK_CON:
	    ptc(ch,"{cIt holds %s-colored %s.{c\n\r",
		liq_table[obj->value[2]].liq_color,
		liq_table[obj->value[2]].liq_name);
	    break;


    	case ITEM_WEAPON:
 	    stc("{cThis weapon is a ",ch);
	    switch (obj->value[0])
	    {
	    	case(WEAPON_EXOTIC):
		    stc("{cexotic{c\n\r",ch);
		    break;
	    	case(WEAPON_SWORD):
		    stc("{csword{c\n\r",ch);
		    break;
	    	case(WEAPON_DAGGER):
		    stc("{cdagger{c\n\r",ch);
		    break;
	    	case(WEAPON_SPEAR):
		    stc("{cspear{c\n\r",ch);
		    break;
	    	case(WEAPON_STAFF):
		    stc("{cstaff{c\n\r",ch);
		    break;
	    	case(WEAPON_MACE):
		    stc("{cmace/club{c\n\r",ch);
		    break;
		   	case(WEAPON_AXE):
		    stc("{caxe{c\n\r",ch);
		    break;
	    	case(WEAPON_FLAIL):
		    stc("{cflail{c\n\r",ch);
		    break;
	    	case(WEAPON_WHIP):
		    stc("{cwhip{c\n\r",ch);
		    break;
	    	case(WEAPON_POLEARM):
		    stc("{cpolearm{c\n\r",ch);
		    break;
	    	default:
		    stc("{cunknown{c\n\r",ch);
		    break;
 	    }
        ptc( ch, "It looks as if it will do %s %s damage.\n\r",
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 3 ? "almost non-existant" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 5 ? "extremely poor" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 8 ? "very poor" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 12 ? "poor" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 15 ? "extremely low" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 18 ? "very low" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 21 ? "low" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 24 ? "fairly low" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 38 ? "moderately low" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 41 ? "very light" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 44 ? "light" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 47 ? "fairly light" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 50 ? "fair" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 53 ? "decent" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 56 ? "moderate" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 59 ? "medium" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 62 ? "good" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 65 ? "very good" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 68 ? "extremely good" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 71 ? "strong" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 74 ? "very strong" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 77 ? "extremely strong" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 81 ? "excellent" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 84 ? "high" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 87 ? "fairly high" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 90 ? "very high" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 93 ? "extremely high" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 96 ? "heavy" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 99 ? "fairly heavy" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 102 ? "very heavy" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 105 ? "extremely heavy" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 110 ? "great" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 115 ? "very great" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 120 ? "extreme" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 125 ? "massive" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 130 ? "extremely massive" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 135 ? "critical" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 150 ? "awesome" :
        	( 1+ obj->value[2] * obj->value[1] / 2 ) <= 176 ? "god-like" :
        	"This weapon is Illegal. Report it to the Immortals.",
			(obj->value[3] > 0 && obj->value[3] < MAX_DAMAGE_MESSAGE) ?
		    attack_table[obj->value[3]].noun : "{cdo an unknown something{c");

	break;

    	case ITEM_ARMOR:
	    ptc( ch,"{cArmor class is {w%d {cpierce, {w%d {cbash, {w%d {cslash, and {w%d {cvs. magic{c\n\r",
	        obj->value[0], obj->value[1], obj->value[2], obj->value[3] );
	break;

        case ITEM_CONTAINER:
            ptc(ch,"{cCapacity{W: {w%d#  {cMaximum weight{W: {w%d#  {cflags{W: {w%s{c\n\r",
                obj->value[0], obj->value[3], cont_bit_name(obj->value[1]));
            if (obj->value[4] != 100)
            {
                ptc(ch,"{cWeight multiplier{W: {w%d%%{c\n\r",
			    obj->value[4]);
            }
        break;
    }


    if ( obj->extra_descr != NULL || obj->pIndexData->extra_descr != NULL )
    {
	EXTRA_DESCR_DATA *ed;

	stc( "{cExtra description keywords: '", ch );

	for ( ed = obj->extra_descr; ed != NULL; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
	    	stc( "{c ", ch );
	}

	for ( ed = obj->pIndexData->extra_descr; ed != NULL; ed = ed->next )
	{
	    stc( ed->keyword, ch );
	    if ( ed->next != NULL )
		stc( "{c ", ch );
	}

	stc( "{c'{c\n\r", ch );
    }

    for ( paf = obj->affected; paf != NULL; paf = paf->next )
    {
	ptc( ch, "{cAffects {w%s {cby {w%d{c",
	    affect_loc_name( paf->location ), paf->modifier );
	if ( paf->duration > -1)
	    ptc(ch,"{c, {w%d {chours.{c\n\r",paf->duration);
	else
	    ptc(ch,"{c.\n\r");
	if (paf->bitvector)
	{
	    switch(paf->where)
	    {
		case TO_AFFECTS:
		    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
			affect_bit_name(paf->bitvector));
		    break;
		case TO_AFFECTS2:
		    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
			affect_bit_name(paf->bitvector));
		    break;
        case TO_WEAPON:
            ptc(ch,"{cAdds {w%s {cweapon flags.{c{c\n",
            weapon_bit_name(paf->bitvector));
		    break;
		case TO_OBJECT:
		    ptc(ch,"{cAdds {w%s {cobject flag.{c{c\n",
			extra_bit_name(paf->bitvector));
		    break;
		case TO_IMMUNE:
		    ptc(ch,"{cAdds immunity to {w%s{c.{c{c\n",
			imm_bit_name(paf->bitvector));
		    break;
		case TO_RESIST:
		    ptc(ch,"{cAdds resistance to {w%s{c.{c\n\r",
			imm_bit_name(paf->bitvector));
		    break;
		case TO_VULN:
		    ptc(ch,"{cAdds vulnerability to {w%s{c.{c\n\r",
			imm_bit_name(paf->bitvector));
		    break;
		default:
		    ptc(ch,"{cUnknown bit {w%d{c: {w%d{c\n\r",
			paf->where,paf->bitvector);
		    break;
	    }
	}
    }

	if ( !strcmp(obj->material, "unknown")
	||   !strcmp(obj->material, "(Null)"))
	;
	else
		ptc(ch,"It is made from %s\n\r", obj->material);

	if ( obj->condition < 1 )
		ptc( ch, "%s is on the verge of breakage.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 10 )
		ptc( ch, "%s looks ready to break any time.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 20 )
		ptc( ch, "%s is in very poor condition.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 30 )
		ptc( ch, "%s is in pretty bad shape.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 40 )
		ptc( ch, "%s has seen a lot of wear and tear.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 50 )
		ptc( ch, "%s is in poor condition.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 60 )
		ptc( ch, "%s has seen better days.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 70 )
		ptc( ch, "%s is nicked and worn.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 80 )
		ptc( ch, "%s is in fair condition.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 90 )
		ptc( ch, "%s is in good condition.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition <= 100 )
		ptc( ch, "%s is in excellent condition.\n\r", capitalize(obj->short_descr) );
	else if ( obj->condition > 100 )
		ptc( ch, "%s is in mint condition.\n\r", capitalize(obj->short_descr) );

    if (!obj->enchanted)
    for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
    {
	ptc(ch, "{cAffects {w%s {cby {w%d{c\n\r",
	    affect_loc_name( paf->location ), paf->modifier );
        if (paf->bitvector)
        {
            switch(paf->where)
            {
                case TO_AFFECTS:
                    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
                        affect_bit_name(paf->bitvector));
                    break;
                case TO_AFFECTS2:
                    ptc(ch,"{cAdds {w%s {caffect.{c{c\n",
                        affect2_bit_name(paf->bitvector));
                    break;
                case TO_OBJECT:
                    ptc(ch,"{cAdds {w%s{c object flag.{c\n",
                        extra_bit_name(paf->bitvector));
                    break;
                case TO_IMMUNE:
                    ptc(ch,"{cAdds immunity to {w%s{c.{c\n",
                        imm_bit_name(paf->bitvector));
                    break;
                case TO_RESIST:
                    ptc(ch,"{cAdds resistance to {w%s{c.{c\n\r",
                        imm_bit_name(paf->bitvector));
                    break;
                case TO_VULN:
                    ptc(ch,"{cAdds vulnerability to {w%s{c.{c\n\r",
                        imm_bit_name(paf->bitvector));
                    break;
                default:
                    ptc(ch,"{cUnknown bit {w%d{c: {w%d{c\n\r",
                        paf->where,paf->bitvector);
                    break;
            }
        }
    }
    return;
}
void do_evaluate( CHAR_DATA *ch, char *argument )
{
	int chance = 0;
	CHAR_DATA *keeper;
	OBJ_DATA *obj;
	int cost;
	bool found;
	char arg[MIL];


	if ( IS_NPC( ch ) )                                                                                                                                                                                                                  
		return; 

	/* eval a mob */
	if ( !emptyStr( argument ) )
	{
		if ( ( keeper = get_char_room( ch, NULL, argument ) ) != NULL )
			advanced_consider( ch, keeper );
		else
			stc("Evaluate whom?\n\r", ch);
		return;
	}

	one_argument(argument,arg);


	if ( get_skill(ch,gsn_evaluate) == 0 )
	{
		stc("You are not skilled at evaluation.\n\r",ch);
		return;
	}

	else
	{
		if ( ( keeper = find_keeper( ch ) ) == NULL )
		{
			stc("No keeper here.\n\r",ch);
			return;
		}

		found = FALSE;
		chance = get_skill( ch, gsn_evaluate ); /* yes this could change during a long eval session. So what. */
		chance += ch->perm_stat[STAT_INT];
		chance /= 2;
		for ( obj = keeper->carrying; obj; obj = obj->next_content )
		{
		    /*if ( obj->wear_loc == WEAR_NONE
		    && can_see_obj( ch, obj )
		    && ( cost = get_cost( keeper, obj, TRUE ) ) > 0 )
		    {*/
		    if ( obj->wear_loc == WEAR_NONE
		    &&   can_see_obj( ch, obj )
		    &&   ( cost = get_cost( keeper, obj, TRUE ) ) > 0
		    &&   ( arg[0] == '\0'
		    ||  is_name(arg,obj->name) ))
		    {
				stc("\n\r{w-{r=={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r={w--{r=={w-{x\n\r",ch);
				if ( !found )
				    found = TRUE;

				if ( number_range(2,101) >= chance )
				{
					ptc(ch,"Could not gather any information on %s.\n\r",obj->short_descr);
				}
				else if ( IS_OBJ_STAT(obj,ITEM_INVENTORY) )
				{
					show_evaluation( ch, obj );  /* FIXME This should be paged */
					check_improve( ch, gsn_evaluate, TRUE, 9 );
				}
		    }
		}
	}
	WAIT_STATE( ch, PULSE_VIOLENCE );
	return;
}
void do_bribe( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *keeper;
	CHAR_DATA *victim;
	OBJ_DATA *obj;
	OBJ_DATA *obj2;
	int cost = 0;
	long worth = 0;
	int i = 0;

	if ( IS_NPC( ch ) )
	return;

	if ( get_skill(ch,gsn_bribery) == 0 )
	{
		stc("You are not skilled at bribery.\n\r",ch);
		return;
	}

	if ( ( keeper = find_keeper( ch ) ) == NULL )
	{
		stc("No shopkeeper here to bribe.\n\r",ch);
	    return;
	}
	if ( argument[0] == '\0' )
	{
		act("$N lifts an eyebrow and asks, 'Who do you want information on?'",ch,NULL,keeper,TO_CHAR);
	    return;
	}
	if ( ( victim = get_char_world( ch, argument ) ) == NULL )
	{
		act("$N shrugs and says, 'I dont know anybody by that name.'",ch,NULL,keeper,TO_CHAR);
	    return;
	}
	if ( IS_NPC( victim ) )
	{
		act("$N says shortly, 'Ask me about a player.'",ch,NULL,keeper,TO_CHAR);
	    return;
	}
	cost = number_range(5*victim->level+victim->level,15*victim->level+victim->level);
	if ( !check_coins( ch, cost*100,0,0 ) )
	{
		act("$N says curtly, 'You can't afford my services.'",ch,NULL,keeper,TO_CHAR);
	    return;
	}
	if ( keeper->level + ch->level < victim->level || victim->level > ch->level )
	{
		act("$N says nervously, 'We shouldn't talk about that.'",ch,NULL,keeper,TO_CHAR);
	    return;
	}

	deduct_cost(ch,cost*100,0);

	ptc(ch,"%s is a %s %s{x\n\r",
		victim->name,
		pc_race_table[victim->race].who_name,
		victim->clan == clan_lookup("blackajah") ? "Sedai" :
		victim->clan == clan_lookup("darkwarder") ? "Warder" :
		victim->clan == clan_lookup("warder") ? "Warder" :
		victim->clan == clan_lookup("aessedai") ? "Sedai" :
		victim->clan == clan_lookup("darkfriend") ? "Noble" :
		victim->clan == clan_lookup("dreadlord") ? "Noble" :
		victim->clan == clan_lookup("forsaken") ? "Noble" :
		victim->clan == clan_lookup("chafaile") ? "Noble" :
		victim->clan == clan_lookup("youngling") ? "Noble" :
		clan_table[victim->clan].who_name);
		ptc(ch,"%s has been reborn %d times{x\n\r",
		victim->name,
		victim->pcdata->rebirth);

	if ( ( obj = get_eq_char( victim, WEAR_WIELD ) ) != NULL )
		ptc(ch,"%s uses %s{x\n\r",
		victim->sex == SEX_MALE ? "He" :
		victim->sex == SEX_FEMALE ? "She" : "It",
		obj->short_descr);
	if ( ( obj2 = get_eq_char( victim, WEAR_SECONDARY ) ) != NULL )
		ptc(ch,"%s also uses %s{x\n\r",
		victim->sex == SEX_MALE ? "He" :
		victim->sex == SEX_FEMALE ? "She" : "It",
		obj2->short_descr);

	for (i=0;i<4;i++)
	{
		worth += victim->money[i][0] + victim->money[i][1];
		/* FIXME Make this care about total value, not number of coins */
	}
	ptc(ch,"%s is a %s %s{x\n\r",
	victim->sex == SEX_MALE ? "He" : "She",
	worth <= 10         ? "very poor" :
	worth <= 50         ? "poor" :
	worth <= 100        ? "struggling" :
	worth <= 1000       ? "moderately wealthy" :
	worth <= 10000      ? "semi wealthy" :
	worth <= 100000     ? "well-to-do" :
	worth <= 1000000    ? "very wealthy" :
	worth <= 10000000   ? "rich" :
	worth <= 50000000   ? "very rich" : "filthy rich",
/*	(victim->silver_penny + 100 * victim->silver_mark) <= 10         ? "very poor" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 50         ? "poor" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 100        ? "struggling" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 1000       ? "moderately wealthy" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 10000      ? "semi wealthy" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 100000     ? "well-to-do" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 1000000    ? "very wealthy" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 10000000   ? "rich" :
	(victim->silver_penny + 100 * victim->silver_mark) <= 50000000   ? "very rich" : "filthy rich",*/
	victim->sex == SEX_MALE ? "man" : "woman");
	if ( IS_SET(victim->op_flags,OP_CHANNELER) )
		ptc(ch,"%s says in a low voice, 'I've heard it rumored %s can channel the One Power'\n\r{x",
		keeper->short_descr,
		victim->name);
	check_improve( ch, gsn_bribery, TRUE, 9 );
	tail_chain( );
}
bool check_intimidate( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_intimidate) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_intimidate );
	chance += ch->perm_stat[STAT_STR];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_intimidate, TRUE, 2 );
		return TRUE;
	}
}
bool check_angrealstudy( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_angrealstudy) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_angrealstudy );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
		return TRUE;
}
bool check_truesense( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	if ( get_skill(ch,gsn_truesense) == 0 )
	return FALSE;

	chance = get_skill( ch, gsn_truesense );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_truesense, TRUE, 2 );
		return TRUE;
	}
}

/* The voids
 *
 */
void quest_engrave( CHAR_DATA *ch, char *argument )
{
    char buf[MIL];
    char note_buf[MSL];
	OBJ_DATA *obj;
    note_buf[0] = '\0';

    smash_tilde( argument );

	if ( IS_NPC( ch ) )
		return;
	else
	{
	    if ( ( obj = ( get_obj_carry( ch, argument, ch ) ) ) == NULL )
	    {
			stc( "Null object.\n\r", ch );
	    	return;
		}
		if ( obj->item_type == ITEM_LIGHT
		|| obj->item_type == ITEM_TRASH
		|| obj->item_type == ITEM_MONEY
		|| obj->item_type == ITEM_CORPSE_NPC
		|| obj->item_type == ITEM_CORPSE_PC
		|| obj->item_type == ITEM_FOUNTAIN
		|| obj->item_type == ITEM_GEM
		|| obj->item_type == ITEM_ANGREAL_MALE
		|| obj->item_type == ITEM_ANGREAL_FEMALE )
	    {
			stc( "Object not appropriate.\n\r", ch );
	    	return;
		}
		else
		{
			if ( strlen( argument ) > 64 )
			{
				stc( "Line too long.\n\r", ch );
				return;
			}
		}
		sprintf( buf, "with %s's insignia", capit2( ch->name ) );
		free_string( obj->short_descr );
		strcpy( obj->short_descr, str_dup( buf ) );
		WAIT_STATE( ch, skill_table[gsn_engraving].beats );
	}
	return;
}
void quest_balance( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *upgrade;

	if ( IS_NPC( ch ) )
		return;
	else
	{
		if ( ( upgrade = get_obj_carry( ch, argument, ch ) ) == NULL )
		{
			stc( "You must supply a valid weapon to balance.\n\r", ch );
			return;
		}
		if ( upgrade->item_type != ITEM_WEAPON )
		{
			stc( "Only weapons can be balanced.\n\r", ch );
			return;
		}
		if ( upgrade->weight <= 2 )
		{
			stc( "This item cannot be balanced! It is too light.\n\r", ch );
			return;
		}
		act( "$n tinkers with $T and sets to work balancing it.",
			ch,NULL,upgrade->short_descr,TO_ROOM );
		act( "You tinker with your $T, setting to work balancing it.",
			ch,NULL,upgrade->short_descr,TO_CHAR );
		upgrade->weight -= 2;
		if ( IS_SET( upgrade->value[4], WEAPON_TWO_HANDS ) )
			upgrade->value[4] -= WEAPON_TWO_HANDS;
		if ( !IS_SET( upgrade->extra_flags, ITEM_BALANCED ) )
			SET_BIT( upgrade->extra_flags, ITEM_BALANCED );
		WAIT_STATE( ch, skill_table[gsn_balancing].beats );
		return;
	}
	return;
}
void quest_disguise( CHAR_DATA *ch, char *argument )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return;

	chance = get_skill( ch, gsn_disguise );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return;
	else
		return;
}
void quest_bloodlust( CHAR_DATA *ch, char *argument )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return;

	chance = get_skill( ch, gsn_bloodlust );
	chance += ch->perm_stat[STAT_CON];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return;
	else
		return;
}
/* aids wind forms */
bool check_feint ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_feint );
	chance += ch->perm_stat[STAT_REF];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_feint, TRUE, 2 );
		return TRUE;
	}
}
/* aids animal forms */
bool check_lunge ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_lunge );
	chance += ch->perm_stat[STAT_STR];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_lunge, TRUE, 2 );
		return TRUE;
	}
}
/* aids stone forms */
bool check_stagger ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_stagger );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_stagger, TRUE, 2 );
		return TRUE;
	}
}
bool check_shieldbash ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_shieldbash );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_shieldbash, TRUE, 2 );
		return TRUE;
	}
}
bool check_swordbreak ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_swordbreak );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_swordbreak, TRUE, 2 );
		return TRUE;
	}
}
/* aids sky forms */
bool check_chop ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_chop );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_chop, TRUE, 2 );
		return TRUE;
	}
}
/* aids wind forms */
bool check_draw ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_drawback );
	chance += ch->perm_stat[STAT_DEX];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_drawback, TRUE, 2 );
		return TRUE;
	}
}
/* aids water forms */
bool check_sweep ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_sweep );
	chance += ch->perm_stat[STAT_DEX];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_sweep, TRUE, 2 );
		return TRUE;
	}
}
bool check_dismember ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_dismember );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_dismember, TRUE, 2 );
		return TRUE;
	}
}
bool check_ravage ( CHAR_DATA *ch )
{
	int chance = 0;

	if ( IS_NPC( ch ) )
	return FALSE;

	chance = get_skill( ch, gsn_ravage );
	chance += ch->perm_stat[STAT_INT];

	chance /= 2;

	if ( number_range(2,101) >= chance )
		return FALSE;
	else
	{
		check_improve( ch, gsn_ravage, TRUE, 2 );
		return TRUE;
	}
}
