/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

    Automated Quest code written by Vassago of MOONGATE, moongate.ams.com
    4000. Copyright (c) 1996 Ryan Addams, All Rights Reserved. Use of this
    code is allowed provided you add a credit line to the effect of:
    "Quest Code (c) 1996 Ryan Addams" to your logon screen with the rest
    of the standard diku/rom credits. If you use this or a modified version
    of this code, let me know via email: moongate@moongate.ams.com. Further
    updates will be posted to the rom mailing list. If you'd like to get
    the latest version of quest.c, please send a request to the above add-
    ress. Quest Code v2.03. Please do not remove this notice from this file.

 ***************************************************************************/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "tables.h"
#include "magic.h"
#include "quest.h"
#include "olc.h"

/* Object vnums for Quest Rewards */

#define QUEST_RESTRING_TOKEN 912
#define QUEST_CASINO_TOKEN 913
#define QUEST_TRAVEL_TICKET 914

#define qobj_vnum 1008
#define QUEST_OBJQUEST1 955
#define QUEST_OBJQUEST2 956
#define QUEST_OBJQUEST3 957
#define QUEST_OBJQUEST4 958
#define QUEST_OBJQUEST5 959

/* Local functions */

void generate_quest     args(( CHAR_DATA *ch, CHAR_DATA *questman ));
void quest_update       args(( void ));
bool quest_level_diff   args(( int clevel, int mlevel));
bool chance             args(( int num ));
ROOM_INDEX_DATA *find_location args(( CHAR_DATA *ch, char *arg ));
CHAR_DATA *get_questman args((CHAR_DATA *ch));
void gevent_echo args(( char *argument));
void gevent_create args(( CHAR_DATA *ch, int gnum, bool begin, int option2, int option3 ));
void do_questspellup	( CHAR_DATA *ch );
bool questman_here args(( CHAR_DATA *ch ));
int qtype;

bool isQobj( OBJ_DATA *obj )
{
	if ( obj->pIndexData->vnum == 1008 )
	return TRUE;

	return FALSE;
}
bool chance(int num)
{
	if (number_range(1,100) <= num)
		return TRUE;
	else
		return FALSE;
}

/* For Story QUESTS */

void reset_qsteps(CHAR_DATA *ch)
{
	int i;

	for (i= 0; i < MAX_QSTEPS; i++)
		ch->qstep[i] = 0;

	return;
}

bool check_qstep_completed (CHAR_DATA *ch, int step)
{
	if (step == -1)			return TRUE;
	if (ch->qstep[step] == 1)	return TRUE;
	return FALSE;
}


bool check_on_qstep (CHAR_DATA *ch, int step)
{
	QUEST_DATA *quest;
	OBJECTIVE_DATA *objective;

	if (ch->curr_quest == 0)				return FALSE;
	if ((quest = get_quest_data(ch->curr_quest)) == NULL)	return FALSE;
	if ((objective = get_qstep_data(quest, step)) == NULL)	return FALSE;
	if (ch->qstep[step] == 1)				return FALSE;

	if (objective->requirement[1] == 0)			return TRUE;
	if (objective->requirement[2] == 0)			return TRUE;
	if (objective->requirement[3] == 0)			return TRUE;
	if (ch->qstep[objective->requirement[1]] == 1)		return TRUE;
	if (ch->qstep[objective->requirement[2]] == 1)		return TRUE;
	if (ch->qstep[objective->requirement[3]] == 1)		return TRUE;

	return FALSE;
}

void story_quest_info (CHAR_DATA *ch)
{
	QUEST_DATA *quest;
	OBJECTIVE_DATA *objective;
	OBJECTIVE_DATA *qcomplete;
	int x;

	if (ch->curr_quest == 0)
		return;

	if ((quest = get_quest_data(ch->curr_quest)) == NULL)
		return;

	qcomplete = get_qstep_data(quest, quest->qcomplete);

	if (qcomplete == NULL)
	{
		stc("Buggy quest.. go type quest complete at the questor.\n\r", ch);
		ptc(ch, "Tell an imm you were on quest #%d.\n\r", quest->vnum);
		ch->questmob = -1;
		return;
	}

	for (x = 0; x <= quest->qcomplete; x++)
	{
		if ((objective = get_qstep_data(quest, x)) == NULL)	continue;
		if (ch->qstep[x] == -1)					continue;	/* disabled */
		if (check_qstep_completed(ch, x))
			ptc(ch, " {D[ {YDONE {D] {w%s{x\n\r", objective->objective);
		else if (check_on_qstep(ch, x))
			ptc(ch, "          {W%s{x\n\r", objective->objective);
	}

	return;
}

void complete_qstep(CHAR_DATA *ch, int step)
{
	QUEST_DATA *quest;
	OBJECTIVE_DATA *objective;
	OBJECTIVE_DATA *qcomplete;

	if (ch->curr_quest == 0)
	{
		bugf("%s is trying to complete a qstep, but is not on a quest.\n\r", ch->name);
		return;
	}

	if ((quest = get_quest_data(ch->curr_quest)) == NULL)
	{
		bugf("%s is trying to complete a qstep, but is on a NULL quest.\n\r", ch->name);
		return;
	}

	qcomplete = get_qstep_data(quest, quest->qcomplete);

	if (qcomplete == NULL)
	{
		stc("Buggy quest.. go type quest complete at the questor.\n\r", ch);
		ptc(ch, "Tell an imm you were on quest #%d.\n\r", quest->vnum);
		ch->questmob = -1;
		return;
	}

	ch->qstep[step] = 1;

	if (step == qcomplete->requirement[1])
		ch->questmob = -1;	/* Change this later */
	if (step == qcomplete->requirement[2])
		ch->questmob = -1;	/* Change this later */
	if (step == qcomplete->requirement[3])
		ch->questmob = -1;	/* Change this later */


	if (step > 0 && step != qcomplete->vnum)
		stc("{WYou have made progress on your quest!{x\n\r\n\r", ch);

	/* Check for rewards */

	/* Check for new objectives that are opened up */

	for (objective = quest->objectives; objective != NULL; objective = objective->next)
	{
		if (objective->requirement[1] == step)
			ptc(ch, " {w-> {Y%s{x\n\r", objective->objective);
		if (objective->requirement[2] == step)
			ptc(ch, " {w-> {Y%s{x\n\r", objective->objective);
		if (objective->requirement[3] == step)
			ptc(ch, " {w-> {Y%s{x\n\r", objective->objective);

	}

	return;
}

bool check_quest_guild (QUEST_DATA *quest, int clan)
{
	char *argument;
	char arg[MSL];

	argument = str_dup(quest->guild);

	if (!str_cmp(argument, "all"))		return TRUE;
	if (clan_table[clan].name == NULL)	return FALSE;

	while ((argument = one_argument(argument, arg)) != NULL)
	{
		if (arg[0] == '\0')				break;
		if (!str_cmp(clan_table[clan].name, arg))	return TRUE;
		if (!str_suffix(clan_table[clan].name, arg)
		&& !str_prefix("not", arg))			return FALSE;
	}

	return TRUE;
}

int quest_stat_cost (int stat)
{
	/* amt	cost	total	*/
	/* 1	50 	50	*/
	/* 2	100 	150	*/
	/* 3	150 	300	*/
	/* 4	200 	500	*/
	/* 5	250 	750	*/
	/* 6	300 	1050	*/
	/* 7	350 	1400	*/
	/* 8	400 	1800	*/
	/* 9	450 	2250	*/
	/* 10	500 	2750	*/

	if (stat <= 0)	return 50;
	return (stat + 1) * 50;
}


void quest_cmd_set( CHAR_DATA *ch, char *argument )
{
	if (!is_number(argument))
	{
		stc("Syntax: Quest Set #\n\r", ch);
		return;
	}
	ch->curr_quest = atoi(argument);
	reset_qsteps(ch);
	ptc(ch, "Your current quest: %d.  Qsteps reset.\n\r", ch->curr_quest);
}
void quest_cmd_bond( CHAR_DATA *ch, char *argument )
{
	int maxlength=20;
	CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}
	if (argument[0] == '\0')
	{
		stc("What would you like your colored bond mate's name to look like?\n\r",ch);
		return;
	}
	if ( ch->questpoints < 500 )
	{
		stc("You need 500 questpoints to do that.\n\r", ch);
		return;
	}
	if (!( IS_SET(ch->hist_flags,HIST_BONDED) || IS_SET(ch->hist_flags,HIST_BONDER)))
	{
		stc("You must be part of a bond for this.\n\r",ch);
		return;
	}
	maxlength = strlen(ch->pcdata->bonded_to) + 6;
	if (strstr(color_strip(argument),ch->pcdata->bonded_to)==NULL)
	{
		stc("You have to add your bond's name in the line.\n\r", ch);
		return;
	}
	if (((strlen_color(argument) > maxlength)) ||( strlen(argument) > 2 * maxlength))
	{
		stc("Line too long.\n\r", ch);
		return;
	}
	act("A clerk adds an entry to the book of records.", ch, NULL, NULL, TO_ROOM);
	refresh_bonded_list (ch, ch->name, argument);
	ch->questpoints -= 500;
	stc("You spend 500 quest points.\n\r",ch);
	compose_title(ch);
}
void quest_cmd_colorname( CHAR_DATA *ch, char *argument )
{
        CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}
	if (argument[0] == '\0')
	{
		stc("What would you like your colored name to look like?\n\r",ch);
		return;
	}
	if ( ch->questpoints < 5000 )
	{
		stc("You need 5000 questpoints to do that.\n\r", ch);
		return;
	}
	if (str_cmp(color_strip(argument), ch->name))
	{
		stc("That does not match your name.\n\r", ch);
		return;
	}
	if (str_suffix("{x", argument) && str_suffix("{X", argument))
	{
		stc("Please make sure to use {{x after your color name.\n\r", ch);
		return;
	}
	if (strlen(argument) > 30)
	{
		stc("Line too long.\n\r", ch);
		return;
	}
	act("A clerk adds an entry to the book of records.", ch, NULL, NULL, TO_ROOM);
	free_string(ch->colorname);
	ch->colorname = str_dup(argument);
	ch->questpoints -= 5000;
	stc("You spend 5000 quest points.\n\r",ch);
}
void quest_cmd_lastname( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}
	if (argument[0] == '\0')
	{
		stc("What would you like your new last name be?\n\r",ch);
		return;
	}
	if ( ch->questpoints < 500 )
	{
		stc("You need 500 questpoints to do that.\n\r", ch);
		return;
	}
	if (str_suffix("{x", argument) && str_suffix("{X", argument))
	{
		stc("Please close with {{x after your last name.\n\r", ch);
		return;
	}
	if (strlen(argument) > 30)
	{
		stc("Line too long.\n\r", ch);
		return;
	}
	act("A clerk adds an entry to the book of records.", ch, NULL, NULL, TO_ROOM);
	free_string(ch->pcdata->last_name);
	ch->pcdata->last_name = str_dup(argument);
	ch->questpoints -= 500;
	stc("You spend 500 quest points.\n\r",ch);
}
void quest_cmd_spouse( CHAR_DATA *ch, char *argument )
{
	int maxlength=20;
	CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	if (argument[0] == '\0')
	{
		stc("What would you like your spouse to read as?\n\r",ch);
		return;
	}
	if ( ch->questpoints < 500 )
	{
		stc("You need 500 questpoints to do that.\n\r", ch);
		return;
	}
	if (IS_MARRIED(ch))
	{
		maxlength = strlen(ch->pcdata->spouse) + 6;
		if (strstr(color_strip(argument),ch->pcdata->spouse)==NULL)
		{
			stc("You have to add your spouses name in the line.\n\r", ch);
			return;
		}
	}
	if (((strlen_color(argument) > maxlength)) ||( strlen(argument) > 2 * maxlength))
	{
		stc("Line too long.\n\r", ch);
		return;
	}
	act("A clerk adds an entry to the book of records.", ch, NULL, NULL, TO_ROOM);
	refresh_married_list (ch, ch->name, argument);
	ch->questpoints -= 500;
	stc("You spend 500 quest points.\n\r",ch);
	if (!IS_MARRIED(ch))
	{
		if(ch->pcdata->spouse)
			free_string(ch->pcdata->spouse);
		if (str_cmp(argument, "NULL"))
			ch->pcdata->spouse = str_dup("cheat");
		else
			ch->pcdata->spouse = NULL;
	}
	compose_title(ch);
}
void quest_cmd_reset( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	if ( !IS_IMMORTAL(ch) )
		return;
	else if ( ( victim = get_char_world( ch, argument ) ) == NULL )
		return;
	else
		victim->nextquest = 1;
	stc( "It has been done.\n\r", ch );
	stc( "Your quest timer has been reset.\n\r", victim );
}
void quest_cmd_relocate( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	stc( "Where do you want to relocate to?\n\r\n\r", ch );
	stc( "List of possible relocation areas:\n\r", ch );
	stc( "Caemlyn      Illian\n\r", ch );
	stc( "Ebou Dar     Fal Dara\n\r", ch );
	stc( "Chachin      Cairhien\n\r", ch );
	stc( "Tar Valon    Tear\n\r", ch );
	stc( "Falme        Lugard\n\r", ch );
	stc( "Maradon      Shol Arbela\n\r", ch );
	stc( "Tanchico     Amadicia\n\r", ch );
	stc( "Far Madding\n\r", ch );
	stc( "Note, this doesn't work yet. It will work when all areas are finished.\n\r", ch );
}
void quest_cmd_increase( CHAR_DATA *ch, char *argument )
{
	int sn;
        CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	if ( argument[0] == '\0' )
	{
		stc("Syntax: Quest Increase <skill>\n\r",ch);
		return;
	}
	sn = skill_lookup(argument);
	if ( sn == -1 )
	{
		stc("You dont know that skill.\n\r",ch);
		return;
	}
	if ( ch->questpoints <= 49 )
	{
		stc("You do not have enough quest points.\n\r",ch);
		return;
	}
	if ( ch->pcdata->learned[sn] <= 0 )
	{
		stc("You do not know that skill.\n\r",ch);
		return;
	}
	if ( ch->pcdata->learned[sn] >= 85 )
	{
		stc("You cannot increase that skill further with quest points.\n\r",ch);
		return;
	}
	ch->pcdata->learned[sn] += 1;
	ch->questpoints -= 50;
	ptc( ch, "Your %s increases!\n\r", skill_table[sn].name );
}
void quest_cmd_info( CHAR_DATA *ch, char *argument )
{
	ptc( ch, "Questinfo: %s\n\r", ch->pcdata->questinfo_data );
	ptc( ch, "Questgiver: %s\n\r", ch->questgiver == NULL ? "NULL" : ch->questgiver->short_descr );
	ptc( ch, "Countdown: %d\n\r", ch->countdown );
	ptc( ch, "Questmob: %d\n\r", ch->questmob );
	ptc( ch, "Questobj: %d\n\r", ch->questobj );
	ptc( ch, "Qtype: %d\n\r", ch->pcdata->qtype );

/*
                ch->pcdata->summary[SUM_QUESTS_DONE] += 1;
                ch->pcdata->summary[SUM_QUESTS_DONE_EVER] += 1;
                ch->pcdata->summary[SUM_QPS_EARNED] += pointreward;
                ch->pcdata->summary[SUM_QPS_EARNED_EVER] += pointreward;

                ch->questgiver = NULL;
                ch->countdown = 0;
                ch->questmob = 0;
                ch->qmob = NULL;
                ch->qcomp = FALSE;
                ch->questobj = 0;
                ch->pcdata->qtype = 0;
                ch->nextquest = 10;
                ch->money[0][0] += reward;
                ch->questpoints += pointreward;
                ch->pcdata->questsrun++;
*/
	if (IS_SET(ch->act, WAR_QUESTOR))
	{
		stc( "\n\r{YYour current quest info{w:{x\n\r", ch );
		if (ch->curr_quest != 0)
			story_quest_info(ch);
		else
		{
			stc( ch->pcdata->questinfo_data, ch );
			stc( "\n\r", ch );
		}
		ptc(ch, "Time left for current quest: %d\n\r",ch->countdown);
	}
	else
	{
		stc("You aren't currently on a quest.\n\r",ch);
		if (ch->nextquest > 1)
			ptc(ch, "There are %d minutes remaining until you can go on another quest.\n\r",ch->nextquest);
		else if (ch->nextquest == 1)
			ptc(ch, "There is less than a minute remaining until you can go on another quest.\n\r");
		else
			stc("You may request a quest at this time.\n\r",ch);
	}
	ptc(ch, "You have %d quest points.\n\r",ch->questpoints);
}
void quest_cmd_quit( CHAR_DATA *ch, char *argument )
{
        CHAR_DATA *questman;

	if (!IS_SET(ch->act, WAR_QUESTOR))
		stc("You aren't currently on a quest.\n\r",ch);
	else
	{
		REMOVE_BIT(ch->act, WAR_QUESTOR);

                ch->countdown = 0;
                ch->qmob = NULL;
                ch->qcomp = FALSE;
                ch->pcdata->qtype = 0;
		ch->questgiver = NULL;
		ch->countdown = 0;
		ch->questmob = 0;
		ch->questobj = 0;
		ch->nextquest = 3;
		free_string( ch->pcdata->questinfo_data );
		ch->pcdata->questinfo_data = str_dup("No current quest.");

		if ( ( questman = get_questman(ch) ) != NULL )
		{
			act("$n admits to $N that $e cannot complete $S quest",ch,NULL,questman,TO_ROOM);
			act("$N says flatly, 'Very well, I understand if fame and glory aren't among your desires.'",ch,NULL,questman,TO_CHAR);
		}
		do_function( ch, &do_save, "" );
	}
}
void quest_cmd_convert( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *qp_obj, *qp_obj_next;
	bool found = FALSE;
	CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	for ( qp_obj = ch->carrying; qp_obj != NULL; qp_obj = qp_obj_next )
	{
		qp_obj_next = qp_obj->next_content;
		if ( qp_obj->pIndexData->vnum == 1001 )
		{
			stc("Converting your quest tokens to quest points.\n\r",ch);
			extract_obj(qp_obj);
			ch->questpoints += 1;
			found = TRUE;
		}
		else
			continue;
	}
	if ( !found )
		stc("You do not have any quest tokens to convert to points.\n\r",ch);
}
void quest_cmd_list( CHAR_DATA *ch, char *argument )
{
	int i;
	stc( "+--------------+-----------------------------------------------------+\n\r", ch);
	stc( "|      {wKeyword{x | {wDescription of Command{x                              |\n\r", ch);
	stc( "+--------------+-----------------------------------------------------+\n\r", ch);
	for ( i = 0; quest_commands_table[i].cmd != NULL; i++ )
	{
		if ( quest_commands_table[i].immcmd && !IS_IMMORTAL(ch) )
			continue;

		ptc( ch, "| {c%12s   %-50s  |\n\r", quest_commands_table[i].cmd, quest_commands_table[i].description );
	}
	stc( "+--------------+-----------------------------------------------------+\n\r", ch);
}
void quest_cmd_gevent( CHAR_DATA *ch, char *argument )
{
	int qpcost = 0, gnum = -1, option2, option3;
	char buf[MSL];
	CHAR_DATA *questman;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	if (argument[0] == '\0')
	{
		stc("To purchase a global event, type 'QUEST BUY <keyword>'.\n\r",ch);
		stc("To see a list of events, type 'QUEST BUY LIST'.\n\r", ch);
		return;
	}

	if ( !str_prefix(argument,"dragon") )
	{
		qpcost = 100; gnum = 1;
	}
	else if ( !str_prefix(argument,"lynch") )
	{
		qpcost = 100; gnum = 15;
	}
	else if ( !str_prefix(argument,"shadows") )
	{
		qpcost = 100; gnum = 0;
	}
	else if ( !str_prefix(argument,"raffle") )
	{
		qpcost = 1000; gnum = 2;
	}
	else if ( !str_prefix(argument,"xps") )
	{
		qpcost = 100; gnum = 3;
	}
        else if ( !str_prefix(argument,"qps") )
	{
		qpcost = 100; gnum = 4;
	}
	else if ( !str_prefix(argument,"heals") )
	{
		qpcost = 100; gnum = 6;
	}
	else if ( !str_prefix(argument,"happiness") )
	{
		qpcost = 100; gnum = 9;
	}
	else
	{
		stc("What type of global event would you like to purchase?\n\r", ch);
		return;
	}
	if ( ch->questpoints <= qpcost )
	{
		stc("You can't afford to buy that event.\n\r",ch);
		return;
	}
	if ( gevent->running )
	{
		ptc( ch, "Global Event: %s is already running.\n\r", gevent->name );
		return;
	}
	if ( gnum == -1 )
	{
		ptc( ch, "Global Event: An internal error occured.\n\r", gevent->name );
		return;
	}
	option2 = 3;
	if ( gnum == 2 )
		option3 = 15;
	else
		option3 = 60;
	buf[0] = '\0';
	ch->questpoints -= qpcost;
	sprintf( buf, "{R[{WG{wlobal {WE{wvent{R]{w: %s{w has been started by %s.{x\n\r", gevent_table[gnum].cname, ch->name );
	gevent_echo( buf );
	buf[0] = '\0';
	sprintf( buf, "{G%s{w\n\r", gevent_table[gnum].longd );
	gevent_echo( buf );
	gevent_create( ch, gnum, TRUE, option2, option3 );
}
void quest_cmd_skill( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *questman;
	int i, sn;
	bool found = FALSE;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}
	if ( argument == NULL || argument[0] =='\0' )
	{
		stc("+--------------+-----------+\n\r", ch);
		stc("|   {wSkill Name{x | {wCost{x      |\n\r",ch);
		stc("+--------------+-----------+\n\r", ch);
		for ( i = 0; quest_skills_table[i].skill_name != NULL; i++ )
		{
			ptc( ch, "| {c%12s   %4d      |\n\r", quest_skills_table[i].skill_name, quest_skills_table[i].qp_cost );
		}
		stc("+--------------+-----------+\n\r", ch);
		return;
	}
	for ( i = 0; quest_skills_table[i].skill_name != NULL; i++ )
	{
		if ( !str_prefix( argument, quest_skills_table[i].skill_name ) )
		{
			found = TRUE;
			ptc( ch, "So you want to buy the %s quest skill for %d quest points?\n\r",
				quest_skills_table[i].skill_name, quest_skills_table[i].qp_cost );
			if ( ch->questpoints < quest_skills_table[i].qp_cost )
			{
				ptc( ch, "It's too bad you only have %d quest points.\n\r", ch->questpoints );
				return;
			}
			else
			{
				sn = skill_lookup( quest_skills_table[i].skill_name );
				if ( sn <= 0 )
					ptc( ch, "An error ocurred. Error code %d.\n\r", sn );
				else if ( ch->pcdata->learned[sn] > 0 )
					ptc( ch, "But you already know the %s skill!\n\r", quest_skills_table[i].skill_name );
				else
				{
					ptc( ch, "You have learned the %s skill!\n\r", quest_skills_table[i].skill_name );
					ch->questpoints -= quest_skills_table[i].qp_cost;
					ch->pcdata->learned[sn] = 25;
				}
				return;
			}
		}
	}
	if ( !found )
		ptc( ch, "Sorry, %s, but I don't know of any skills named %s.\n\r", ch->name, argument );
}
bool quest_gevent_running( CHAR_DATA *ch )
{
        if ( gevent->running )
        {
                ptc( ch, "Global Event: %s is already running.\n\r", gevent->name );
                return TRUE;
        }
	else
		return FALSE;
}
void assign_quest_reward( CHAR_DATA *ch, CHAR_DATA *questman, int reward_num, int cost )
{
	char buf[MIL];
	switch( reward_num )
	{
		default: ptc( ch, "Error code %d.\n\r", reward_num ); return;
		case 0: ch->train += quest_rewards_table[reward_num].quantity; break;
                case 1: 
			ch->max_hit += quest_rewards_table[reward_num].quantity;
			ch->pcdata->perm_hit += quest_rewards_table[reward_num].quantity;
			ch->hit += quest_rewards_table[reward_num].quantity;
		break;
                case 2:
			ch->max_stam += quest_rewards_table[reward_num].quantity;
			ch->pcdata->perm_mana += quest_rewards_table[reward_num].quantity;
			ch->stam += quest_rewards_table[reward_num].quantity;
		break;
                case 3: ch->quest_stat[STAT_STR] += 1; break;
                case 4:	ch->quest_stat[STAT_WIS] += 1; break;
                case 5: ch->quest_stat[STAT_INS] += 1; break;
                case 6: ch->quest_stat[STAT_REF] += 1; break;
                case 7: ch->quest_stat[STAT_DEX] += 1; break;
                case 8: ch->quest_stat[STAT_CHA] += 1; break;
                case 9: ch->quest_stat[STAT_INT] += 1; break;
                case 10: ch->quest_stat[STAT_CON] += 1; break;
                case 11:
		{
			OBJ_DATA *obj;
			obj = create_object(get_obj_index(QUEST_RESTRING_TOKEN),1);
			obj_to_char( obj, ch );
			ptc(ch,"%s hands you %s in exchange for your quest points.\n\r", questman->short_descr, obj->short_descr);
			break;
		}
                case 12: ch->pcdata->op_fire += 1; break;
                case 13: ch->pcdata->op_air += 1; break;
		case 14: ch->pcdata->op_water += 1; break;
                case 15: ch->pcdata->op_earth += 1; break;
                case 16: ch->pcdata->op_spirit += 1; break;
                case 17: do_questspellup(ch); break;
                case 18: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "shadows" ); break;
                case 19: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "raffle" ); break;
                case 20: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "xps" ); break;
                case 21: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "qps" ); break;
                case 22: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "heals" ); break;
                case 23: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "happiness" ); break;
                case 24: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "lynch" ); break;
                case 25: if ( quest_gevent_running(ch) ) return; else quest_cmd_gevent( ch, "dragon" ); break;
	}
	ch->questpoints -= cost;
	sprintf( buf, "{g%s says, {G'{gYour purchase has cost you %d quest points. You have %d remaining.{G'{x", questman->short_descr, cost, ch->questpoints );
	act( buf, questman, NULL, ch, TO_VICT );
}
void quest_cmd_buy( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *questman;
	int i;
	char buf[MIL];
	bool found = FALSE;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}
	if ( argument == NULL || argument[0] == '\0' )
	{
		stc( "+-------------+------+-----+-----------------------+\n\r", ch);
		stc( "|      {wReward{x | {wCost{x | {wAmt{x | {wDescription{x           |\n\r", ch);
		stc( "+-------------+------+-----+-----------------------+\n\r", ch);
		for ( i = 0; quest_rewards_table[i].keyword != NULL; i++ )
		{
			ptc( ch, "| {c%12s  %4d  %4d  %-22s | (%d)\n\r", 
				quest_rewards_table[i].keyword, 
				quest_rewards_table[i].is_stat ? quest_stat_cost(ch->quest_stat[quest_rewards_table[i].stat]) : quest_rewards_table[i].cost, 
				quest_rewards_table[i].quantity, 
				quest_rewards_table[i].description, quest_rewards_table[i].reward_num );
		}
		stc( "+--------------------------------------------------+\n\r", ch);
		return;
	}
	for ( i = 0; quest_rewards_table[i].keyword != NULL; i++ )
	{
		if ( !str_prefix( argument, quest_rewards_table[i].keyword ) )
		{
			sprintf( buf, "{g%s asks, {G'{gSo you want to buy %s %s%s?{G'{x", 
				questman->short_descr,
				quest_rewards_table[i].quantity > 1 ? "some" : "a",
				quest_rewards_table[i].description,
				quest_rewards_table[i].quantity > 1 ? "s" : "" );
			act( buf, questman, NULL, ch, TO_VICT );
			if ( ch->questpoints < quest_rewards_table[i].cost )
			{
				sprintf( buf, "{g%s says, {G'{gIt's too bad that you can't afford the %d quest points.{G'{x", questman->short_descr, quest_rewards_table[i].cost );
				act( buf, questman, NULL, ch, TO_VICT );
				return;
			}
			found = TRUE;
			assign_quest_reward( ch, questman, quest_rewards_table[i].reward_num, quest_rewards_table[i].cost );
			break;
		}
	}
	if ( !found )
	{
		sprintf( buf, "{g%s says, {G'{gSorry, %s, but I don't offer a reward called %s.{G'", questman->short_descr, ch->name, argument );
		act( buf, questman, NULL, ch, TO_VICT );
		return;
	}
}
void quest_cmd_staydeath( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *questman;
	OBJ_DATA *obj;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	if ((obj = get_obj_carry(ch, argument, ch)) == NULL)
	{
		stc("To flag an aura staydeath, type 'QUEST <object> STAYDEATH'.\n\r", ch);
		return;
	}
	if (!CAN_WEAR(obj, ITEM_WEAR_FLOAT))
	{
		stc("Only float items such as auras may be flagged staydeath.\n\r", ch);
		return;
	}
	if (IS_SET(obj->extra_flags, ITEM_STAYDEATH))
	{
		stc("This item is already flagged to stay over deaths!\n\r", ch);
		return;
	}
	if (ch->questpoints < 500)
	{
		ptc(ch, "%s says, 'Sorry, %s, but you don't have enough quest points for that.\n\r'",questman->short_descr,ch->name);
		return;
	}
	SET_BIT(obj->extra_flags, ITEM_STAYDEATH);
	ch->questpoints -= 500;
	act( "$N tinkers with $p and hands it back to $n.", ch, obj, questman, TO_ROOM );
	act( "$N tinkers with $p and hands it back to you.", ch, obj, questman, TO_CHAR );
}
void quest_cmd_capture( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *qvictim;
	if ( ( qvictim = get_char_room( ch, NULL, argument ) ) == NULL )
	{
		stc("Capture who?\n\r",ch);
		return;
	}
	if ( ch->questmob == 0 )
	{
		stc("You're not on that kind of quest!\n\r",ch);
		return;
	}
	stc("Quest capture will allow you to capture a prisoner for return to Caemlyn.\n\r",ch);
}
void quest_cmd_request( CHAR_DATA *ch, char *argument )
{
	char qinfo[MSL];
	CHAR_DATA *questman;
	qinfo[0] = '\0';

	if ( argument[0] == '\0' || !is_number(argument) )
	{
		stc("What type of quest would you like?\n\r",ch);
		stc("+-----------------------------------+\n\r",ch);
		stc("| 1) Recover a Treasure             |\n\r",ch);
		stc("| 2) Kill a Villan                  |\n\r",ch);
		stc("| 3) Make me regret this...         |\n\r",ch);
		stc("| 4) Surprise me!                   |\n\r",ch);
		stc("| 5) Story Quest (in progress)      |\n\r",ch);
		stc("+-----------------------------------+\n\r",ch);
		return;
	}

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here that can do that for you.\n\r", ch);
		return;
	}

	wiznet("{c$N has requested a quest.",ch,NULL,WIZ_QUESTS,0,0);

	qtype = atoi(argument);
	if ( qtype <1 || qtype > 5 )
	{
		stc("Valid choices are 1 through 5.\n\r", ch);
		return;
	}
	if ( qtype == 4 )
		qtype = number_range(1,3);

	if ( ch->level < 20 )
	{
		act("You are too small to be asking me for quests!",ch,0,questman,TO_CHAR);
		act("$N laughs merrily at $n!",ch,0,questman,TO_ROOM);
		return;
	}

	sprintf( qinfo, "%s", qtype == 1 ? "treasure recovery" : qtype == 2 ? "justice bringer" : qtype == 3 ? "challenging" : "mystery" );

	act( "$n asks $N for a $t quest.", ch, qinfo, questman, TO_ROOM);
	act ("You ask $N for a $t quest.",ch, qinfo, questman, TO_CHAR);

	if (IS_SET(ch->act, WAR_QUESTOR))
	{
		act("$N says, 'But you're already on a quest!'\n\r",ch,NULL,questman,TO_CHAR);
		return;
	}
	if (ch->nextquest > 0)
	{
		ptc(ch, "%s says, 'You're very brave, %s, but let someone else have a chance.'\n\r",questman->short_descr,ch->name);
		ptc(ch, "%s says, 'Come back later.'\n\r",questman->short_descr);
		return;
	}

	ptc(ch, "%s says, 'Thank you, brave %s!'\n\r",questman->short_descr,ch->name);
	ch->questmob = 0;
	ch->questobj = 0;
	ch->qmob = NULL;
	ch->pcdata->qtype = qtype;

	generate_quest(ch, questman);

	if (ch->questmob > 0 || ch->questobj > 0 || ch->qmob != NULL)
	{
		ch->countdown = number_range(10,30);
		SET_BIT(ch->act, WAR_QUESTOR);
		ptc(ch, "%s says, 'You have %d minutes to complete this quest.'\n\r",questman->short_descr,ch->countdown);
		ptc(ch, "%s %s, 'May the %s bless you!'\n\r",questman->short_descr, is_shadow(ch) ? "whispers" : "says", is_shadow(ch) ? "Great Lord" : "Creator");
	}
}
void quest_cmd_complete( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *questman;
	bool qiscomp = FALSE;

	if ( ( questman = get_questman(ch) ) == NULL )
	{
		stc("There is no one here who can do that for you.\n\r", ch);
		return;
	}

	act( "$n informs $N that $e has completed $S quest.", ch, NULL, questman, TO_ROOM);
	act ("You inform $N that you have completed $S quest.",ch, NULL, questman, TO_CHAR);

	if (ch->questgiver != questman)
	{
		ptc(ch, "%s says, 'I didn't send you on a quest!\n\r",questman->short_descr);
		return;
	}


	if ( ch->pcdata->qtype == 3 && ch->qmob != NULL && ch->qcomp == TRUE ) /* finished a quest 3 mob quest */
		qiscomp = TRUE;
	else if ( ch->pcdata->qtype == 2 && ch->questmob == -1 && ch->countdown > 0 ) /* finished a q2 mob quest */
		qiscomp = TRUE;
	else if ( ch->pcdata->qtype == 1 && ch->questobj > 0 && ch->countdown > 0 ) /* finished a find object quest */
	{
		OBJ_DATA *obj, *obj_next;
		bool obj_found = FALSE;
		for (obj = ch->carrying; obj != NULL; obj= obj_next)
		{
			obj_next = obj->next_content;
			if (obj != NULL && obj->pIndexData->vnum == ch->questobj)
			{
				obj_found = TRUE;
				qiscomp = TRUE;
				break;
                        }
		}
	}
	else
	{
		stc("An error occured processing your qtype.\n\r", ch );
		stc("It's probable that you did not complete your quest.\n\r", ch);
		stc("If you are certain that you completed your quest, contact an Immortal.\n\r", ch);
		return;
	}

	if ( !qiscomp )
	{
		ptc(ch, "%s says, 'You haven't completed the quest yet!\n\r",questman->short_descr);
		return;
	}

	if (IS_SET(ch->act, WAR_QUESTOR))
	{
		int reward, pointreward, pracreward;
		reward = number_range(150,500);

		if ( ch->pcdata->qtype == 1 )
			pointreward = number_range(45,69);
		else if ( ch->pcdata->qtype == 2 )
			pointreward = number_range(20,49);
		else if ( ch->pcdata->qtype == 3 )
			pointreward = number_range(45,69);
		else
		{
			stc("There was an error determining the type of quest you were on. Report this to an Immortal for compensation.", ch);
			return;
		}
		wiznet("{c$N has completed a quest.",ch,NULL,WIZ_QUESTS,0,0);

		if ( gevent->running && (gevent->num == 4 || gevent->num == 14 || gevent->num == 10) )
		{
			ptc( ch, "You are awarded a bonus to your quest points from the %s event!\n\r", gevent->cname );
			pointreward = (pointreward * 1.5);
		}

		if (ch->questobj > 0)
		{
			OBJ_DATA *obj, *obj_next;
			bool obj_found = FALSE;
			for (obj = ch->carrying; obj != NULL; obj= obj_next)
			{
				obj_next = obj->next_content;
				if (obj != NULL && obj->pIndexData->vnum == ch->questobj)
				{
					obj_found = TRUE;
					act("You hand $p to $N.",ch, obj, questman, TO_CHAR);
					act("$n hands $p to $N.",ch, obj, questman, TO_ROOM);
					extract_obj(obj);
					break;
				}
			}
		}

		ptc(ch, "%s says, 'Congratulations on completing your quest!\n\r",questman->short_descr);
		ptc(ch,"%s says, 'As a reward, I am giving you %d quest points, and %d copper pennies.\n\r",questman->short_descr,pointreward,reward);

		if (chance(15) && ch->max_hit < 30000)
		{
			ch->max_hit += 5;
			ch->pcdata->perm_hit += 5;
			ch->hit += 5;
			ptc(ch,"%s says, 'I grant you the %s's Blessing for your deeds!\n\r",
				questman->short_descr, is_shadow(ch) ? "Great Lord" : "Creator" );
			stc("You gain 5 hit points!\n\r",ch);
		}
		else if (chance(15))
		{
			gain_exp (ch, 35);
			ptc(ch,"%s says, 'I grant you the %s's Luck for your deeds!\n\r",
			questman->short_descr, is_shadow(ch) ? "Great Lord" : "Creator" );
			stc("You gain 35 experience points!\n\r",ch);
		}
		else if (chance(15) && ch->questpoints < 30000)
		{
			ptc(ch,"%s says, 'I grant you the %s's Boon for your deeds!\n\r",
				questman->short_descr, is_shadow(ch) ? "Great Lord" : "Creator" );
			stc("You gain 35 extra quest points!\n\r",ch);
			pointreward += 35;
		}
		else if (chance(15) && ch->questpoints < 30000)
		{
			ptc(ch,"%s says, 'I grant you the %s's Boon for your deeds!\n\r",
				questman->short_descr, is_shadow(ch) ? "Great Lord" : "Creator" );
			stc("You gain 40 extra quest points!\n\r",ch);
			pointreward += 40;
		}
		else if (chance(15) && ch->questpoints < 30000)
		{
			ptc(ch,"%s says, 'I grant you the %s's Boon for your deeds!\n\r",
				questman->short_descr, is_shadow(ch) ? "Great Lord" : "Creator" );
			stc("You gain 45 extra quest points!\n\r",ch);
			pointreward += 45;
		}
		else if (chance(15) && ch->max_stam < 30000)
		{
			ptc(ch,"%s says, 'The %s %s you!'\n\r",
				questman->short_descr, is_shadow(ch) ? "Shadow" : "Creator", is_shadow(ch) ? "embraces" : "shines upon" );
			ptc(ch, "You gain 10 extra stamina points!\n\r");
			ch->max_stam += 10;
			ch->pcdata->perm_mana += 10;
			ch->stam += 10;
		}
		else if (chance(30) && ch->train < 30000)
		{
			ptc(ch,"%s says, 'I grant you the %s's Blessing!'\n\r",
				questman->short_descr, is_shadow(ch) ? "Great Lord" : "Creator" );
			pracreward = number_range(6,9);
			ptc(ch, "You gain %d trains!\n\r",pracreward);
			ch->train += pracreward;
		}
		REMOVE_BIT(ch->act, WAR_QUESTOR);
		ch->pcdata->summary[SUM_QUESTS_DONE] += 1;
		ch->pcdata->summary[SUM_QUESTS_DONE_EVER] += 1;
		ch->pcdata->summary[SUM_QPS_EARNED] += pointreward;
		ch->pcdata->summary[SUM_QPS_EARNED_EVER] += pointreward;

		ch->questgiver = NULL;
		ch->countdown = 0;
		ch->questmob = 0;
		ch->qmob = NULL;
		ch->qcomp = FALSE;
		ch->questobj = 0;
		ch->pcdata->qtype = 0;
		ch->nextquest = 10;
		ch->money[0][0] += reward;
		ch->questpoints += pointreward;
		ch->pcdata->questsrun++;
		free_string( ch->pcdata->questinfo_data );
		ch->pcdata->questinfo_data = str_dup( "No current quest." );
		do_function( ch, &do_save, "" );
		return;
	}
}

CHAR_DATA *get_questman( CHAR_DATA *ch )
{
        CHAR_DATA *questman;
        for ( questman = ch->in_room->people; questman != NULL; questman = questman->next_in_room )
        {
                if ( questman->spec_fun == NULL )
                        continue;
                if (questman->spec_fun == spec_lookup( "spec_questmaster" ))
                        return questman;
        }
        return NULL;
}

bool questman_here( CHAR_DATA *ch )
{
	CHAR_DATA *questman;
	for ( questman = ch->in_room->people; questman != NULL; questman = questman->next_in_room )
	{
		if ( questman->spec_fun == NULL )
			continue;
		if (questman->spec_fun == spec_lookup( "spec_questmaster" ))
			return TRUE;
	}
	return FALSE;
}

void do_quest(CHAR_DATA *ch, char *argument)
{
	char arg1[MIL];
	bool found = FALSE;

	argument = one_argument(argument, arg1);

	if ( IS_NPC( ch ) || ch->desc == NULL )
		return;

	if (arg1[0] == '\0')
	{
		quest_cmd_list(ch, NULL);
		return;
	}
	else
	{
		int i;
		for ( i = 0; quest_commands_table[i].cmd != NULL; i++ )
		{
			if ( quest_commands_table[i].immcmd && !IS_IMMORTAL(ch) )
				continue;
			if ( !str_prefix( arg1, quest_commands_table[i].cmd ) )
			{
				//ptc( ch, "Firing command %s with argument %s (orig %s).\n\r", quest_commands_table[i].cmd, argument, orig );
				(* quest_commands_table[i].quest_cmd_func)(ch, argument);
				found = TRUE;
				break;
			}
		}
	}
	if ( !found )
	{
		stc( "What quest command would you like to use?\n\r", ch );
		do_function( ch, &do_quest, "" );
		return;
	}
}

int quest_count ( CHAR_DATA *ch )
{
	int cnt = 0;
	QUEST_DATA *quest;

	for (quest = quest_first; quest != NULL; quest = quest->next)
	{
		if (quest->active == FALSE)	continue;
		if (!check_quest_guild(quest, ch->clan))	continue;
		cnt++;
	}

	return cnt;
}

void generate_quest(CHAR_DATA *ch, CHAR_DATA *questman)
{
	CHAR_DATA *victim;
	DESCRIPTOR_DATA *d;
	ROOM_INDEX_DATA *room;
	bool found = FALSE;
	char qbuf[MSL];
	int randroom = -1;

	int qmob_rooms[] =
	{
		4211, 4367, 4457, 5778, 5773,  /* 5 */
		9109, 9111, 9117, 9135, 9196,  /* 10 */
		2103, 2114, 2123, 1551, 1578,  /* 15 */
		1519, 1669, 5235, 5221, 7762,  /* 20 */
		7764, 7765, 19512,19511,695,   /* 25 */
		661,  669,  1839, 685,  691,   /* 30 */
		3454, 3555, 3556, 3557, 3558,  /* 35 */
		6296, 6252, 6297, 6306, 10376, /* 40 */
		2275, 8079, 8084, 8046, 8156,  /* 45 */
		8054, 8182, 3905, 3912, 3913,  /* 50 */
		3914, 3915, 3945, 3946, 3957,  /* 55 */
		3962, 3961, 3959, 3935, 3936,  /* 60 */
		3930, 3931, 3941, 3967, 3985,  /* 65 */
		3987, 3991, 4010, 4017, 4027,  /* 70 */
		4025, 22111,22106,22106,22114, /* 75 */
		5905, 5940, 5939, 5938, 17005, /* 80 */
		17156,17171,17056,17062,15531, /* 85 */
		7051, 304,  23549,7790, 5623,  /* 90 */
		7017, 7039, 7038, 7037, 7036,  /* 95 */
		7210, 7021, 7102, 7107, 7086,  /* 100 */
		18305,18306,18307,18311,18312, /* 105 */
		18313,18314,18315,18316,18317, /* 110 */
		18318,18319,18320,1124, 1233,  /* 115 */
		23534,7719, 1641, 5807, 5808,  /* 120 */
		5814, 5810, 5811, 5820, 5822,  /* 125 */
		5827, 5831, 5834, 5825, 5826,  /* 130 */
		5821, 5845, 5847, 5846, 5841,  /* 135 */
		5842, 5844, 5851, 5859, 5863,  /* 140 */
		5871, 5873, 5874, 5875, 5876,  /* 145 */
		5877, 5890, 5892, 5894, 5895,  /* 150 */
		5897, 5896, 5879, 5880, 5898,  /* 155 */
		5891, 5882, 5884, 10301,10305  /* 160 */
	};

	qbuf[0] = '\0';
	randroom = number_range( 1, 160 );
	ch->questgiver = questman;

	if (qtype == 5)
	{
		QUEST_DATA *quest;
		int roll = number_range(1, quest_count(ch));
		int cnt = 0;

		for (quest = quest_first; quest != NULL; quest = quest->next)
		{
			if (quest->active == FALSE)			continue;
			if (!check_quest_guild(quest, ch->clan))	continue;
			if (++cnt == roll)				break;
		}

		if (quest == NULL)
		{
			do_say(questman, "Sorry, something didn't quite work out right..");
			return;			
		}

		sprintf(qbuf, "$n says, '%s'", quest->questsay);
		act(qbuf, questman, NULL, ch, TO_VICT);
		ch->curr_quest = quest->vnum;
		ch->questmob = 1;	/* For now */
		reset_qsteps(ch);
		complete_qstep(ch, 0);
		return;
	}

	/* make me regret this */
	if ( qtype == 3 )
	{
		CHAR_DATA *bubble;
		int rmob = number_range( 1, 44 );
		int i;
		ROOM_INDEX_DATA *room;
		char nBuf[MSL];
		char sBuf[MSL];
		char lBuf[MSL];
		nBuf[0] = '\0';
		sBuf[0] = '\0';
		lBuf[0] = '\0';

		rmob -= 1;

		room = get_room_index( qmob_rooms[randroom-1] );

		bubble = create_mobile( get_mob_index( MOB_VNUM_BUBBLE ) );

		sprintf( nBuf, qmob_table[rmob].name );
		sprintf( sBuf, qmob_table[rmob].shortd );
		sprintf( lBuf, qmob_table[rmob].longd );

		strcat( lBuf, "\n\r" );

		free_string(bubble->name);
		    bubble->name 	= str_dup( nBuf );
		free_string(bubble->colorname);
		    bubble->colorname 	= str_dup( sBuf );
		free_string(bubble->short_descr);
		    bubble->short_descr	= str_dup( sBuf );
		free_string(bubble->long_descr);
		    bubble->long_descr	= str_dup( lBuf );
		free_string(bubble->description);
		    bubble->description	= str_dup("");
		bubble->sex		= qmob_table[rmob].sex;
		bubble->race		= race_lookup(qmob_table[rmob].race);
		bubble->level	= ch->level + 20;
		bubble->trust	= 0;
		bubble->hit		= ch->level * 350 + ch->level;
		bubble->max_hit	= ch->level * 350 + ch->level;
		bubble->stam	= ch->max_stam * 2;
		bubble->max_stam	= ch->stam;
		bubble->money[3][0]		= 0;
		bubble->money[2][0]	= 0;
		bubble->act += ACT_UNDEAD;
		bubble->off_flags = OFF_FAST|OFF_DODGE;
		bubble->imm_flags	= ch->imm_flags;
		bubble->res_flags	= ch->res_flags;
		bubble->vuln_flags	= ch->vuln_flags;
		bubble->position	= POS_STANDING;
		bubble->saving_throw	= ch->saving_throw;
		bubble->alignment	= -1000;
		bubble->hitroll	= ch->level * 8;
		bubble->damroll	= ch->level * 8;
		bubble->form		= ch->form;
		bubble->parts	= ch->parts;
		bubble->size		= ch->size;
		bubble->dam_type	= attack_lookup( qmob_table[rmob].damtype );
		bubble->start_pos	= POS_STANDING;

		for (i = 0; i < 4; i++)
			bubble->armor[i]	= ch->armor[i];

		for (i = 0; i < MAX_STATS; i++)
		{
			bubble->perm_stat[i]	= ch->perm_stat[i] * 5;
			bubble->mod_stat[i]	= ch->mod_stat[i] * 5;
		}

		bubble->damage[0]	= 1;
		bubble->damage[1]	= bubble->level;
		bubble->damage[2]	= bubble->level;

		bubble->questor = ch;
		bubble->timer = 500;
		ch->pcdata->qtype = 3;
		ch->qmob = bubble;
		ch->qcomp = FALSE;

		char_from_room( bubble );
		char_to_room( bubble, room );
		stc( "The clerk says:\n\r", ch );
		ptc( ch, "'%s escaped from prison and murdered several guards in doing so!'\n\r",bubble->short_descr );
		ptc( ch, "'Find them in %s and kill them!'\n\r", bubble->in_room->name );
		ptc( ch, "'You can find that place in %s.'\n\r",bubble->in_room->area->name );

		sprintf( qbuf, "Kill %s in %s.", bubble->short_descr, bubble->in_room->name );
		free_string( ch->pcdata->questinfo_data );
		ch->pcdata->questinfo_data = str_dup( qbuf );

		return;
	}/* end q3 */

	for (victim = char_list; victim != NULL; victim = victim->next)
	{
		if (quest_level_diff(ch->level, victim->level) == TRUE
		&& IS_SET(victim->act, ACT_UNDEAD) && IS_NPC(victim) 
		&& victim->pIndexData->vnum != MOB_VNUM_BUBBLE )
		break;
	}

	if ( victim == NULL  )
	{
		ptc(ch, "%s says, 'I'm sorry, but I don't have any quests for you at this time.\n\r",questman->short_descr);
		ptc(ch, "%s says, 'Try again later.\n\r",questman->short_descr);
		ch->nextquest = 2;
		return;
	}
	if ( ( room = victim->in_room ) == NULL )
	{
		ptc(ch, "%s says, 'I'm sorry, but I don't have any quests for you at this time.\n\r",questman->short_descr);
		ptc(ch, "%s says, 'Try again later.\n\r",questman->short_descr);
		ch->nextquest = 2;
		return;
	}
	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if ( d->connected != CON_PLAYING )
			continue;
		if ( d->character == ch )
			continue;
		if ( d->character->questmob == 0 )
			continue;
		if ( d->character->questmob == victim->pIndexData->vnum )
		{
			ptc(ch, "%s says, 'I'm sorry, but someone already has the quest for you.\n\r",questman->short_descr);
			ptc(ch, "%s says, 'Try again later.\n\r",questman->short_descr);
			ch->nextquest = 2;
			found = TRUE;
			break;
		}
	}
	if ( found )
		return;

	/*  check qtype to see what quest type the character wants */
	if (qtype == 1)
	{
		OBJ_DATA *qobj;

		qobj = create_object( get_obj_index( qobj_vnum ), 20 );
		qobj->timer = 30;

		/* 50 50 chance the char is sent to one of the q3 random rooms instead to help cut down on quest botting -- Dalsor 01/27/2013 */
		if ( chance( 60 ) )
		{
			room = get_room_index( qmob_rooms[randroom-1] );
		}
		if ( !qobj )
		{
			ptc(ch,"An error processing vnum %d occured. Aborting.\n\r",qobj_vnum);
			return;
		}

		switch(number_range(0,9))
		{
			case 0:
			free_string( qobj->name );
			qobj->name = str_dup("laurel crown");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("the Laurel Crown");
			free_string( qobj->description );
			qobj->description = str_dup("The Laurel Crown.");
			break;

			case 1:
			free_string( qobj->name );
			qobj->name = str_dup("creator palm");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("the Palm of the Creator");
			free_string( qobj->description );
			qobj->description = str_dup("The Palm of the Creator.");
			break;

			case 2:
			free_string( qobj->name );
			qobj->name = str_dup("wheel spoke");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("a Spoke of the Wheel");
			free_string( qobj->description );
			qobj->description = str_dup("A Spoke of the Wheel.");
			break;

			case 3:
			free_string( qobj->name );
			qobj->name = str_dup("heaven fire");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("a Fire of Heaven");
			free_string( qobj->description );
			qobj->description = str_dup("A Fire of Heaven.");
			break;

			case 4:
			free_string( qobj->name );
			qobj->name = str_dup("dragon prophecies");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("the Prophecies of the Dragon");
			free_string( qobj->description );
			qobj->description = str_dup("The Prophecies of the Dragon.");
			break;

			case 5:
			free_string( qobj->name );
			qobj->name = str_dup("dragon sceptre");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("the Dragon Sceptre");
			free_string( qobj->description );
			qobj->description = str_dup("The Dragon Sceptre.");
			break;

			case 6:
			free_string( qobj->name );
			qobj->name = str_dup("crown swords");
			free_string( qobj->short_descr );
			qobj->short_descr = str_dup("the Crown of Swords");
			free_string( qobj->description );
			qobj->description = str_dup("The Crown of Swords.");
			break;

                        case 7:
                        free_string( qobj->name );
                        qobj->name = str_dup("memory light");
                        free_string( qobj->short_descr );
                        qobj->short_descr = str_dup("a Memory of Light");
                        free_string( qobj->description );
                        qobj->description = str_dup("A Memory of Light.");
                        break;

                        case 8:
                        free_string( qobj->name );
                        qobj->name = str_dup("heart winter");
                        free_string( qobj->short_descr );
                        qobj->short_descr = str_dup("a Heart of Winter");
                        free_string( qobj->description );
                        qobj->description = str_dup("A Heart of Winter.");
                        break;

                        case 9:
                        free_string( qobj->name );
                        qobj->name = str_dup("knife of dreams");
                        free_string( qobj->short_descr );
                        qobj->short_descr = str_dup("a Knife of Dreams");
                        free_string( qobj->description );
                        qobj->description = str_dup("A Knife of Dreams.");
                        break;

		}

		qobj->owner = str_dup( ch->name );
		obj_to_room(qobj, room);
		ch->questobj = qobj->pIndexData->vnum;
		ch->pcdata->qtype = 1;

		ptc(ch, "%s says, 'Vile pilferers have stolen %s!\n\r",questman->short_descr,qobj->short_descr);
		ptc(ch, "%s says, 'The sniffers have pinpointed its location.\n\r",questman->short_descr);

		ptc(ch, "%s says, 'Look in the general area of %s for %s!\n\r",questman->short_descr,room->area->name, room->name);
		sprintf( qbuf, "Find %s in %s.", qobj->short_descr, room->name );
		free_string( ch->pcdata->questinfo_data );
		ch->pcdata->questinfo_data = str_dup( qbuf );
		return;
    }

    /* Quest to kill a mob */

    else if ( qtype == 2 )
    {
		if ( victim->pIndexData->vnum == 14801
		|| victim->pIndexData->vnum == 14802
		|| victim->pIndexData->vnum == 14803
		|| victim->pIndexData->vnum == 14804
		|| victim->pIndexData->vnum == 14805
		|| victim->pIndexData->vnum == 14806
		|| victim->pIndexData->vnum == 14807
		|| victim->pIndexData->vnum == 14808
		|| victim->pIndexData->vnum == 14809
		|| victim->pIndexData->vnum == 14810
		|| victim->pIndexData->vnum == 14811
		|| victim->pIndexData->vnum == 14812
		|| victim->pIndexData->vnum == 14813 )
		{
			ptc(ch, "%s says, '%s, threatens trade with the Borderlands.'\n\r",capit2(questman->short_descr),victim->short_descr);
			ptc(ch, "%s says, 'Restore safety to the trade route by eliminating this threat!'\n\r",	questman->short_descr);
		}
		if ( victim->pIndexData->vnum == 18300
		|| victim->pIndexData->vnum == 18301
		|| victim->pIndexData->vnum == 18302
		|| victim->pIndexData->vnum == 18303
		|| victim->pIndexData->vnum == 18304
		|| victim->pIndexData->vnum == 18305 )
		{
			ptc(ch, "%s says, '%s, has staged attacks on innocent citizens!\n\r",capit2(questman->short_descr),capit2(victim->short_descr));
			ptc(ch, "%s says, 'Restore safety to our citizens by removing this threat!!\n\r", questman->short_descr);
		}
		if ( victim->pIndexData->vnum == 20352
		|| victim->pIndexData->vnum == 20353
		|| victim->pIndexData->vnum == 20354
		|| victim->pIndexData->vnum == 20355
		|| victim->pIndexData->vnum == 20356
		|| victim->pIndexData->vnum == 20357
		|| victim->pIndexData->vnum == 20358
		|| victim->pIndexData->vnum == 20359 )
		{
			ptc(ch, "%s says, '%s threatens the safety of Tar Valon!\n\r", capit2(questman->short_descr),capit2(victim->short_descr));
			ptc(ch, "%s says, 'Restore safety to Tar Valon by removing this threat!!\n\r", questman->short_descr);
		}
		else
		{
			switch(number_range(0,1))
			{
				case 0:
					ptc(ch, "%s says, 'An enemy of Caemlyn, %s, is making vile threats against the Queen.\n\r",questman->short_descr,victim->short_descr);
					ptc(ch, "%s says, 'This threat must be eliminated!\n\r", questman->short_descr);
				break;
				case 1:
					ptc(ch, "%s says, 'Caemlyn's most heinous criminal, %s, has escaped from the dungeon!\n\r", questman->short_descr,victim->short_descr);
					ptc(ch, "%s says, 'Since the escape, %s has murdered %d civillians!\n\r", questman->short_descr,victim->short_descr, number_range(2,20));
					ptc(ch,"%s says, 'The penalty for this crime is death, and you are to deliver the sentence!\n\r", questman->short_descr);
				break;
			}
		}

		if (room->name != NULL)
		{
			ptc(ch, "%s says, 'Seek %s out somewhere in the vicinity of %s!\n\r", questman->short_descr,victim->short_descr,room->name);
			ptc(ch, "%s says, 'That location is in the general area of %s.\n\r", questman->short_descr,room->area->name);
		}
		ch->questmob = victim->pIndexData->vnum;
		victim->questor = ch;
		sprintf( qbuf, "Kill %s in %s.\n\r", victim->short_descr, victim->in_room->name );
		free_string( ch->pcdata->questinfo_data );
		ch->pcdata->questinfo_data = str_dup( qbuf );
		ch->pcdata->qtype = 2;
	}
	else
	{
		stc("An error processing qtype occured. Aborting.\n\r", ch);
		return;
	}
	return;
}

bool quest_level_diff(int clevel, int mlevel)
{
	int levtmp;
	levtmp = clevel - mlevel ;

	if ( levtmp >= -5 && levtmp <= 5)
		return TRUE;
	else
		return FALSE;
}

/* Called from update_handler() by pulse_area */

void quest_update(void)
{
	DESCRIPTOR_DATA *d;

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
		if (d->character != NULL && d->connected == CON_PLAYING)
		{
			if (d->character->nextquest > 0)
			{
				d->character->nextquest -= 1;
				if (d->character->nextquest == 0)
				{
					if ( d->character->pcdata->newbie != 1 )
					{
						stc("You may now quest again.\n\r",d->character);
						continue;
					}
				}
			}
			else if (IS_SET(d->character->act,WAR_QUESTOR))
			{
				if (--d->character->countdown <= 0)
				{
					d->character->nextquest = 10;
					ptc(d->character, "You have run out of time for your quest!\n\rYou may quest again in %d minutes.\n\r",	d->character->nextquest);
					REMOVE_BIT(d->character->act, WAR_QUESTOR);
					d->character->questgiver = NULL;
					d->character->countdown = 0;
					d->character->questmob = 0;
				}
				if (d->character->countdown > 0 && d->character->countdown < 6)
				{
					stc("Hurry! You're almost out of time for your quest!\n\r",d->character);
				}
			}
		}
	}
	return;
}


