
/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "recycle.h"


EMAIL_DATA *email_data_free;

EMAIL_DATA *new_email_data (void)
{
	static EMAIL_DATA email_zero;
	EMAIL_DATA *email;

	if (email_data_free == NULL)
		email = alloc_perm (sizeof(*email));
	else
	{
		email = email_data_free;
		email_data_free = email_data_free->next;
	}
	email->to = &str_empty[0];
	email->subject = &str_empty[0];
	email->body = &str_empty[0];
	*email = email_zero;
	VALIDATE (email);
	return email;
}

void free_email_data (EMAIL_DATA * email)
{
	if (!IS_VALID (email))
		return;

	free_string (email->to);
	free_string (email->subject);
	free_string (email->body);

	INVALIDATE (email);

	email->next = email_data_free;
	email_data_free = email;
}

/* stuff for recyling pnotes */
PNOTE_DATA *pnote_free;

PNOTE_DATA *new_pnote(void)
{
    PNOTE_DATA *note;

    if (pnote_free == NULL)
	note = alloc_perm(sizeof(*note));
    else
    {
	note = pnote_free;
	pnote_free = pnote_free->next;
    }
    VALIDATE(note);
    return note;
}

void free_pnote(PNOTE_DATA *note)
{
	if (!IS_VALID(note))
		return;

	free_string( note->text    );
	free_string( note->subject );
	free_string( note->to_list );
	free_string( note->date    );
	free_string( note->sender  );
	free_string( note->poster_host );
	INVALIDATE(note);

	note->next = pnote_free;
	pnote_free   = note;
}

/* Stuff for recycling dynamic auction objects */
AUCTION_DATA *auction_free;

AUCTION_DATA *new_auction(void)
{
    AUCTION_DATA *auction;

    if (auction_free == NULL)
	auction = alloc_perm(sizeof(*auction));
    else
    {
	auction = auction_free;
	auction_free = auction_free->next;
    }
	VALIDATE(auction);
    return auction;
}

void free_auction(AUCTION_DATA *auction)
{
	if (!IS_VALID(auction))
	return;

    auction->current_bid = 0;
	auction->gold_held	 = 0;
	auction->high_bidder = NULL;
	auction->item		 = NULL;
	auction->min_bid	 = 0;
	auction->owner		 = NULL;
	auction->status		 = 0;
	INVALIDATE(auction);

    auction->next  = auction_free;
    auction_free   = auction;
}

/* stuff for recycling timed events */
EVENT_DATA *ev_free;

EVENT_DATA *new_event(void)
{
    static EVENT_DATA ev_zero;
    EVENT_DATA *ev;

    if ( ev_free == NULL )
       ev = alloc_perm(sizeof(*ev));
    else
    {
	ev = ev_free;
	ev_free = ev_free->next;
    }

    *ev = ev_zero;
    VALIDATE(ev);
    ev->to   = NULL;
    return ev;
}

void free_event(EVENT_DATA *ev)
{
    if (!IS_VALID(ev))
	return;

    INVALIDATE(ev);

    ev->next = ev_free;
    ev_free = ev;
}

WEAVE_DATA *weave_data_free;

WEAVE_DATA *new_weave_data(void)
{
    WEAVE_DATA *weave;
    if(weave_data_free)
    {
	weave = weave_data_free;
	weave_data_free = weave_data_free->next;
    }
    else   /* for speed, make this allocate 10 at a time and put remaining on free list */
	weave = alloc_perm(sizeof(*weave));
	weave->target_name = NULL; /*  Make sure that every instance of new_weave_data() sets this */
    weave->next = NULL;
    return weave;
}

void free_weave_data(WEAVE_DATA *weave)
{
	free_string(weave->target_name);
	weave->target_name = NULL;
	weave->next = weave_data_free;
	weave_data_free = weave;
}
/* stuff for recycling ban structures */
BAN_DATA *ban_free;

BAN_DATA *new_ban(void)
{
    static BAN_DATA ban_zero;
    BAN_DATA *ban;

    if (ban_free == NULL)
	ban = alloc_perm(sizeof(*ban));
    else
    {
	ban = ban_free;
	ban_free = ban_free->next;
    }

    *ban = ban_zero;
    VALIDATE(ban);
    ban->name = &str_empty[0];
    return ban;
}

void free_ban(BAN_DATA *ban)
{
    if (!IS_VALID(ban))
	return;

    free_string(ban->name);
    INVALIDATE(ban);

    ban->next = ban_free;
    ban_free = ban;
}

WIZ_DATA *wiz_free;

WIZ_DATA *new_wiz( void )
{
    static WIZ_DATA wiz_zero;
    WIZ_DATA *wiz;

    if (wiz_free == NULL)
	wiz = alloc_perm(sizeof(*wiz));
    else
    {
	wiz = wiz_free;
	wiz_free = wiz_free->next;
    }

    *wiz = wiz_zero;
    VALIDATE(wiz);
    wiz->name = &str_empty[0];
    wiz->email = &str_empty[0];
    wiz->job = &str_empty[0];
    return wiz;
}

void free_wiz( WIZ_DATA *wiz )
{
    if (!IS_VALID(wiz))
	return;

    free_string(wiz->name);
    free_string(wiz->email);
    free_string(wiz->job);
    INVALIDATE(wiz);

    wiz->next = wiz_free;
    wiz_free = wiz;
}

/* stuff for recycling descriptors */
DESCRIPTOR_DATA *descriptor_free;

DESCRIPTOR_DATA *new_descriptor2(void)
{
    static DESCRIPTOR_DATA d_zero;
    DESCRIPTOR_DATA *d;

    if (descriptor_free == NULL)
	d = alloc_perm(sizeof(*d));
    else
    {
	d = descriptor_free;
	descriptor_free = descriptor_free->next;
    }

    *d = d_zero;
    VALIDATE(d);

    d->connected	= CON_GET_NAME;
    d->showstr_head	= NULL;
    d->showstr_point = NULL;
    d->outsize	= 2000;
    d->outbuf	= alloc_mem( d->outsize );

//	if (d->pProtocol == NULL)
//	   d->pProtocol     = ProtocolCreate();

    return d;
}




DESCRIPTOR_DATA *new_descriptor ( void )
{
   static DESCRIPTOR_DATA d_zero;
   DESCRIPTOR_DATA * d;
   if (descriptor_free == NULL)
      d = (DESCRIPTOR_DATA *) alloc_perm (sizeof (*d));
   else
   {
      d = descriptor_free;
      descriptor_free = descriptor_free->next;
   }
   *d = d_zero;
   d->connected = CON_GET_NAME;
   d->showstr_head = NULL;
   d->showstr_point = NULL;
   d->outsize = 2000;
   d->outbuf = (char *) alloc_mem (d->outsize);
   d->pProtocol     = ProtocolCreate();
   return d;
}


void free_descriptor (DESCRIPTOR_DATA * d)
{
   free_string (d->host);
   free_mem (d->outbuf, d->outsize);

//   if (d->pProtocol)
//   ProtocolDestroy( d->pProtocol );
/* <--- Add this line */

   d->next = descriptor_free;
   descriptor_free = d;
}

void free_descriptor2(DESCRIPTOR_DATA *d)
{
    if (!IS_VALID(d))
	return;

   #ifndef THREADED_DNS
	free_string( d->host );
   #endif
    free_string( d->showstr_head);
    free_mem( d->outbuf, d->outsize );
    INVALIDATE(d);
    d->next = descriptor_free;
    descriptor_free = d;
    d->character = NULL; /* added to hopefully solve the buffer overflow and then drop link problem */
}

/* stuff for recycling gen_data */
GEN_DATA *gen_data_free;

GEN_DATA *new_gen_data(void)
{
    static GEN_DATA gen_zero;
    GEN_DATA *gen;

    if (gen_data_free == NULL)
	gen = alloc_perm(sizeof(*gen));
    else
    {
	gen = gen_data_free;
	gen_data_free = gen_data_free->next;
    }
    *gen = gen_zero;
    VALIDATE(gen);
    return gen;
}

void free_gen_data(GEN_DATA *gen)
{
    if (!IS_VALID(gen))
	return;

    INVALIDATE(gen);

    gen->next = gen_data_free;
    gen_data_free = gen;
}

/* stuff for recycling extended descs */
EXTRA_DESCR_DATA *extra_descr_free;

EXTRA_DESCR_DATA *new_extra_descr(void)
{
    EXTRA_DESCR_DATA *ed;

    if (extra_descr_free == NULL)
	ed = alloc_perm(sizeof(*ed));
    else
    {
	ed = extra_descr_free;
	extra_descr_free = extra_descr_free->next;
    }

    ed->keyword = &str_empty[0];
    ed->description = &str_empty[0];
    VALIDATE(ed);
    return ed;
}

void free_extra_descr(EXTRA_DESCR_DATA *ed)
{
    if (!IS_VALID(ed))
	return;

    free_string(ed->keyword);
    free_string(ed->description);
    INVALIDATE(ed);

    ed->next = extra_descr_free;
    extra_descr_free = ed;
}


/* stuff for recycling affects */
AFFECT_DATA *affect_free;

AFFECT_DATA *new_affect(void)
{
    static AFFECT_DATA af_zero;
    AFFECT_DATA *af;

    if (affect_free == NULL)
	af = alloc_perm(sizeof(*af));
    else
    {
	af = affect_free;
	affect_free = affect_free->next;
    }

    *af = af_zero;


    VALIDATE(af);
    return af;
}

void free_affect(AFFECT_DATA *af)
{
    if (!IS_VALID(af))
	return;

    INVALIDATE(af);
    af->next = affect_free;
    affect_free = af;
}

/* stuff for recycling objects */
OBJ_DATA *obj_free;

OBJ_DATA *new_obj(void)
{
    static OBJ_DATA obj_zero;
    OBJ_DATA *obj;

    if (obj_free == NULL)
	obj = alloc_perm(sizeof(*obj));
    else
    {
	obj = obj_free;
	obj_free = obj_free->next;
    }
    *obj = obj_zero;
    VALIDATE(obj);

    return obj;
}

void free_obj(OBJ_DATA *obj)
{
    AFFECT_DATA *paf, *paf_next;
    EXTRA_DESCR_DATA *ed, *ed_next;
    //KEY_DATA	* key, * key_next;

    if (!IS_VALID(obj))
	return;

    for (paf = obj->affected; paf != NULL; paf = paf_next)
    {
	paf_next = paf->next;
	free_affect(paf);
    }
    obj->affected = NULL;

    for (ed = obj->extra_descr; ed != NULL; ed = ed_next )
    {
	ed_next = ed->next;
	free_extra_descr(ed);
     }
     obj->extra_descr = NULL;

    free_string( obj->name        );
    free_string( obj->description );
    free_string( obj->short_descr );
    free_string( obj->owner     );
    INVALIDATE(obj);

    obj->next   = obj_free;
    obj_free    = obj;

/*    if (obj->keys)
    {
	// loop thru the keys and free the allocated memory
 	for ( key = obj->keys; key != NULL; key = key_next)
	{
	    key_next = key->next;
    	    free_key(key);
    	}
    }
*/
}


/* stuff for recyling characters */
CHAR_DATA *char_free;

CHAR_DATA *new_char (void)
{
    static CHAR_DATA ch_zero;
    CHAR_DATA *ch;
    int i;

    if (char_free == NULL)
	ch = alloc_perm(sizeof(*ch));
    else
    {
	ch = char_free;
	char_free = char_free->next;
    }

    *ch				= ch_zero;
    VALIDATE(ch);
    ch->name                    = &str_empty[0];
    ch->short_descr             = &str_empty[0];
    ch->long_descr              = &str_empty[0];
    ch->description             = &str_empty[0];
    ch->prompt                  = &str_empty[0];
    ch->prefix			= &str_empty[0];
    ch->colorname		= &str_empty[0];
	ch->eye = &str_empty[0];
	ch->hair1 = &str_empty[0];
	ch->hair2 = &str_empty[0];
	ch->hair3 = &str_empty[0];
	ch->hair4 = &str_empty[0];
	ch->misc1 = &str_empty[0];
	ch->misc2 = &str_empty[0];
    ch->logon                   = current_time;
    ch->lines                   = PAGELEN;
    for (i = 0; i < 4; i++)
        ch->armor[i]            = 100;
    ch->position                = POS_STANDING;
    ch->hit                     = 50;
    ch->max_hit                 = 50;
    ch->stam                    = 50;
    ch->max_stam                = 50;
    ch->mod_hit			= 0;
    ch->mod_stam		= 0;
    ch->rank 			= 0;
    ch->sub_rank 		= 0;
    for (i = 0; i < MAX_STATS; i ++)
    {
        ch->perm_stat[i] = 13;
        ch->mod_stat[i] = 0;
    }

	reset_qsteps(ch);

    ch->weaves			= NULL;
    return ch;
}

void free_char (CHAR_DATA *ch)
{
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    AFFECT_DATA *paf;
    AFFECT_DATA *paf_next;

    if (!IS_VALID(ch))
	return;

    if (IS_NPC(ch))
	mobile_count--;

    for (obj = ch->carrying; obj != NULL; obj = obj_next)
    {
	obj_next = obj->next_content;
	extract_obj(obj);
    }

    for (paf = ch->affected; paf != NULL; paf = paf_next)
    {
	paf_next = paf->next;
	affect_remove(ch,paf);
    }

/*    free_string(ch->name); */ /* this keeps buggins, so why not? */
    free_string(ch->short_descr);
    free_string(ch->long_descr);
    free_string(ch->description);
    free_string(ch->prompt);
    free_string(ch->prefix);
    free_string(ch->eye);
    free_string(ch->hair1); /* added 1-3, better not crash */
    free_string(ch->hair2);
    free_string(ch->hair3);
    free_string(ch->hair4);
	free_string(ch->misc1);
	free_string(ch->misc2);
	ch->misc2 = &str_empty[0];
    free_pnote  (ch->pnote);
    free_pcdata(ch->pcdata);

    ch->next = char_free;
    char_free  = ch;

    INVALIDATE(ch);
    return;
}

PC_DATA *pcdata_free;

PC_DATA *new_pcdata(void)
{
    static PC_DATA pcdata_zero;
    PC_DATA *pcdata;

    if (pcdata_free == NULL)
		pcdata = alloc_perm(sizeof(*pcdata));
    else
    {
		pcdata = pcdata_free;
		pcdata_free = pcdata_free->next;
    }

    *pcdata = pcdata_zero;
/*
    for (alias = 0; alias < MAX_ALIAS; alias++)
    {
		pcdata->alias[alias] = NULL;
		pcdata->alias_sub[alias] = NULL;
    }
    for (ignore = 0; ignore < MAX_IGNORE; ignore++)
    {
		pcdata->ignore[ignore] = NULL;
    }
*/


	pcdata->spouse = &str_empty[0];
	pcdata->battle_cry = &str_empty[0];
    pcdata->proposal = &str_empty[0];
    pcdata->buffer = new_buf();
    pcdata->wkin = &str_empty[0];
    pcdata->wout = &str_empty[0];
    pcdata->rdesc = &str_empty[0];
    pcdata->bonded_to = &str_empty[0];
    pcdata->bonded_by = &str_empty[0];
    pcdata->aiel_clan = &str_empty[0];
    pcdata->last_name = &str_empty[0];
    pcdata->email = &str_empty[0];
    pcdata->title = &str_empty[0];
    pcdata->email = str_dup ( "" );
    pcdata->last_command = str_dup( "" );

    pcdata->mod_warnings    = 0;
    pcdata->mod_freezings   = 0;
    pcdata->mod_disconnects = 0;
    pcdata->verifications   = 0;
    pcdata->rewards_gold    = 0;
    pcdata->rewards_qp      = 0;
    pcdata->rewards_xp      = 0;
    pcdata->rewards_items   = 0;


    VALIDATE(pcdata);
    return pcdata;
}


void free_pcdata(PC_DATA *pcdata)
{
	int /*alias,*/ ignore;
	GRANT_DATA *gran, *gran_next;
	BUDDY_DATA *buddy;

	if (!IS_VALID(pcdata))
        return;

    for (gran = pcdata->granted; gran != NULL; gran = gran_next)
    {
        gran_next = gran->next;
        free_string(gran->name);
        free_mem(gran,sizeof(gran));
    }

	free_string(pcdata->battle_cry);
    free_string(pcdata->pwd);
    free_string(pcdata->proposal);
    free_string(pcdata->rdesc);
    free_string(pcdata->wkin);
    free_string(pcdata->wout);
    free_string(pcdata->bonded_to);
    free_string(pcdata->bonded_by);
    free_string(pcdata->bamfin);
    free_string(pcdata->bamfout);
    free_string(pcdata->title);
    free_string(pcdata->aiel_clan);
    free_string(pcdata->email);
    free_string(pcdata->spouse);
    free_string(pcdata->last_name);
    free_string(pcdata->last_command);
    free_buf(pcdata->buffer);
    /*free_string ( pcdata->email );*/

/*
    for (alias = 0; alias < MAX_ALIAS; alias++)
    {
		free_string(pcdata->alias[alias]);
		free_string(pcdata->alias_sub[alias]);
    }
*/
  for (ignore = 0; ignore < MAX_IGNORE; ignore++)
    {
		free_string(pcdata->ignore[ignore]);
    }

	while(pcdata->buddies != NULL)
	{
		buddy = pcdata->buddies;
		pcdata->buddies = buddy->next;
		free_buddy(buddy);
	}

    INVALIDATE(pcdata);
    pcdata->next = pcdata_free;
    pcdata_free = pcdata;
    return;
}

/* stuff for setting ids */
long	last_pc_id;
long	last_mob_id;

long get_pc_id(void)
{
    int val;

    val = (current_time <= last_pc_id) ? last_pc_id + 1 : current_time;
    last_pc_id = val;
    return val;
}

long get_mob_id(void)
{
    last_mob_id++;
    return last_mob_id;
}

MEM_DATA *mem_data_free;

/* procedures and constants needed for buffering */

BUFFER *buf_free;

MEM_DATA *new_mem_data(void)
{
    MEM_DATA *memory;

    if (mem_data_free == NULL)
	memory = alloc_mem(sizeof(*memory));
    else
    {
	memory = mem_data_free;
	mem_data_free = mem_data_free->next;
    }

    memory->next = NULL;
    memory->id = 0;
    memory->reaction = 0;
    memory->when = 0;
    VALIDATE(memory);

    return memory;
}

void free_mem_data(MEM_DATA *memory)
{
    if (!IS_VALID(memory))
	return;

    memory->next = mem_data_free;
    mem_data_free = memory;
    INVALIDATE(memory);
}



/* buffer sizes */
const int buf_size[MAX_BUF_LIST] =
{
    16,32,64,128,256,1024,2048,4096,8192,16384
};

/* local procedure for finding the next acceptable size */
/* -1 indicates out-of-boundary error */
int get_size (int val)
{
    int i;

    for (i = 0; i < MAX_BUF_LIST; i++)
	if (buf_size[i] >= val)
	{
	    return buf_size[i];
	}

    return -1;
}

BUFFER *new_buf()
{
    BUFFER *buffer;

    if (buf_free == NULL)
	buffer = alloc_perm(sizeof(*buffer));
    else
    {
	buffer = buf_free;
	buf_free = buf_free->next;
    }

    buffer->next	= NULL;
    buffer->state	= BUFFER_SAFE;
    buffer->size	= get_size(BASE_BUF);

    buffer->string	= alloc_mem(buffer->size);
    buffer->string[0]	= '\0';
    VALIDATE(buffer);

    return buffer;
}

BUFFER *new_buf_size(int size)
{
    BUFFER *buffer;

    if (buf_free == NULL)
        buffer = alloc_perm(sizeof(*buffer));
    else
    {
        buffer = buf_free;
        buf_free = buf_free->next;
    }

    buffer->next        = NULL;
    buffer->state       = BUFFER_SAFE;
    buffer->size        = get_size(size);
    if (buffer->size == -1)
    {
        bug("new_buf: buffer size %d too large.",size);
        exit(1);
    }
    buffer->string      = alloc_mem(buffer->size);
    buffer->string[0]   = '\0';
    VALIDATE(buffer);

    return buffer;
}


void free_buf(BUFFER *buffer)
{
    if (!IS_VALID(buffer))
	return;

    free_mem(buffer->string,buffer->size);
    buffer->string = NULL;
    buffer->size   = 0;
    buffer->state  = BUFFER_FREED;
    INVALIDATE(buffer);

    buffer->next  = buf_free;
    buf_free      = buffer;
}


bool add_buf(BUFFER *buffer, char *string)
{
    int len;
    char *oldstr;
    int oldsize;

    oldstr = buffer->string;
    oldsize = buffer->size;

    if (buffer->state == BUFFER_OVERFLOW) /* don't waste time on bad strings! */
	return FALSE;

    len = strlen(buffer->string) + strlen(string) + 1;

    while (len >= buffer->size) /* increase the buffer size */
    {
	buffer->size 	= get_size(buffer->size + 1);
	{
	    if (buffer->size == -1) /* overflow */
	    {
		buffer->size = oldsize;
		buffer->state = BUFFER_OVERFLOW;
		bug("buffer overflow past size %d",buffer->size);
		return FALSE;
	    }
  	}
    }

    if (buffer->size != oldsize)
    {
	buffer->string	= alloc_mem(buffer->size);

	strcpy(buffer->string,oldstr);
	free_mem(oldstr,oldsize);
    }

    strcat(buffer->string,string);
    return TRUE;
}


void clear_buf(BUFFER *buffer)
{
    buffer->string[0] = '\0';
    buffer->state     = BUFFER_SAFE;
}


char *buf_string(BUFFER *buffer)
{
    return buffer->string;
}

/* stuff for recycling mobprograms */
PROG_LIST *mprog_free;
PROG_LIST *oprog_free;
PROG_LIST *rprog_free;

PROG_LIST *new_mprog(void)
{
   static PROG_LIST mp_zero;
   PROG_LIST *mp;

   if (mprog_free == NULL)
       mp = alloc_perm(sizeof(*mp));
   else
   {
       mp = mprog_free;
       mprog_free=mprog_free->next;
   }

   *mp = mp_zero;
   mp->vnum             = 0;
   mp->trig_type        = 0;
   mp->code             = str_dup("");
   VALIDATE(mp);
   return mp;
}

void free_mprog(PROG_LIST *mp)
{
   if (!IS_VALID(mp))
      return;

   INVALIDATE(mp);
   mp->next = mprog_free;
   mprog_free = mp;
}
PROG_LIST *new_oprog(void)
{
   static PROG_LIST op_zero;
   PROG_LIST *op;

   if (oprog_free == NULL)
       op = alloc_perm(sizeof(*op));
   else
   {
       op = oprog_free;
       oprog_free=oprog_free->next;
   }

   *op = op_zero;
   op->vnum             = 0;
   op->trig_type        = 0;
   op->code             = str_dup("");
   VALIDATE(op);
   return op;
}

void free_oprog(PROG_LIST *op)
{
   if (!IS_VALID(op))
      return;

   INVALIDATE(op);
   op->next = oprog_free;
   oprog_free = op;
}

PROG_LIST *new_rprog(void)
{
   static PROG_LIST rp_zero;
   PROG_LIST *rp;

   if (rprog_free == NULL)
       rp = alloc_perm(sizeof(*rp));
   else
   {
       rp = rprog_free;
       rprog_free=rprog_free->next;
   }

   *rp = rp_zero;
   rp->vnum             = 0;
   rp->trig_type        = 0;
   rp->code             = str_dup("");
   VALIDATE(rp);
   return rp;
}

void free_rprog(PROG_LIST *rp)
{
   if (!IS_VALID(rp))
      return;

   INVALIDATE(rp);
   rp->next = rprog_free;
   rprog_free = rp;
}

HELP_AREA * had_free;

HELP_AREA * new_had ( void )
{
	HELP_AREA * had;
static	HELP_AREA   zHad;

	if ( had_free )
	{
		had		= had_free;
		had_free	= had_free->next;
	}
	else
		had		= alloc_perm( sizeof( *had ) );

	*had = zHad;

	return had;
}

HELP_DATA * help_free;

HELP_DATA * new_help ( void )
{
	HELP_DATA * help;

	if ( help_free )
	{
		help		= help_free;
		help_free	= help_free->next;
	}
	else
		help		= alloc_perm( sizeof( *help ) );

	return help;
}

void free_help(HELP_DATA *help)
{
	free_string(help->keyword);
	free_string(help->text);
	help->next = help_free;
	help_free = help;
}

FIGHTING_DATA * fight_free;

FIGHTING_DATA * new_fight ( void )
{
	FIGHTING_DATA * fight;

	if ( fight_free )
	{
		fight		= fight_free;
		fight_free	= fight_free->next;
	}
	else
		fight		= alloc_perm( sizeof( *fight ) );

	return fight;
}

void free_fight(FIGHTING_DATA *fight)
{
	fight->fighter = NULL;

	fight->next = fight_free;
	fight_free = fight;
}
