/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"

#define RP_DIR "../rp/"

void do_rpgroup( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    argument = one_argument( argument, arg );
    argument = one_argument( argument, arg2 );

    if ( arg[0] == '\0' )
    {
		CHAR_DATA *gch;
		CHAR_DATA *leader;

		leader = (ch->leader != NULL) ? ch->leader : ch;
		ptc( ch, "{c%s's group:{c\n\r", PERS(leader, ch) );

		for ( gch = char_list; gch != NULL; gch = gch->next )
		{
		    if ( is_same_group( gch, ch ) )
		    {

			int hp_perc = 0, op_perc = 0;
			if (gch->max_hit > 0)
				hp_perc = gch->hit * 100 / get_maxhit(gch);
			if (gch->max_stam > 0)
				op_perc = gch->stam * 100 / get_maxstam(gch);

			ptc( ch,"{b[{w%-16s {wHP: {W%s%3d%{w OP: {W%s%3d%{w MV: {W%s%3d%{x\n\r",
			    capit_colorsafe( PERS(gch, ch) ),
			    hp_perc >= 65 ? "{g" : hp_perc >= 25 ? "{y" : "{r",
			    hp_perc,
			    op_perc >= 65 ? "{g" : op_perc >= 25 ? "{y" : "{r",
			    op_perc );
		    }
		}
		return;
    }

    if ( !str_cmp( arg, "finish" ) )
    {
		CHAR_DATA *grp;
		for ( grp = ch->in_room->people; grp != NULL; grp = grp->next_in_room )
		{
			if ( !IS_NPC( grp ) && grp->leader == ch )
			{
				ptc( grp, "%s calls the Rp session to an end.\n\r", ch->name );
				grp->leader = grp;
				grp->master = grp;
			}
		}
		ch->leader = ch;
		ch->master = ch;
		return;
	}

	if ( ( victim = get_char_world( ch, arg ) ) == NULL )
	{
		stc("Invite who to rp? The person has to be online.", ch);
		return;
	}
	if ( IS_NPC( victim ) )
	{
		stc("You can't Rp with mobs you flaming woolhead!\n\r", ch);
		return;
	}

    if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
    {
		stc( "{cThey aren't here.{c\n\r", ch );
		return;
    }

    if ( ch->master != ch || ch->leader != ch )
    {
		stc( "{cDisband your current group or rp group before starting a new one.{c\n\r", ch );
		return;
    }

    /*if (IS_AFFECTED(victim,AFF_CHARM))
    {
        stc("{cYou can't remove charmed mobs from your group.{c\n\r",ch);
        return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM))
    {
    	act_new("{cYou like your master too much to leave $m!",
	    ch,NULL,victim,TO_VICT,POS_SLEEPING);
    	return;
    }

    if ( !str_cmp( arg2, "remove" ) )
    {
		if ( !is_same_group( victim, ch ) )
		{
			stc( "You're not in the same rp session.\n\r", ch );
			return;
		}
		if ( victim->leader != ch )
		{
			stc( "You're not the leader of their rp session.\n\r", ch );
			return;
		}
		ptc( victim, "%s removes you from the group, leaving you leaderless.\n\r", ch->name );
		ptc( ch, "You remove %s from the group, leaving them leaderless.\n\r", victim->name );
		victim->leader = victim;
		victim->master = victim;
		return;
	}

    if ( !str_cmp( arg2, "leader" ) )
    {
		CHAR_DATA *grp;
		if ( !is_same_group( victim, ch ) )
		{
			stc( "You're not in the same group.\n\r", ch );
			return;
		}
		if ( victim->leader != ch )
		{
			stc( "You're not their leader.\n\r", ch );
			return;
		}
		for ( grp = ch->in_room->people; grp != NULL; grp = grp->next_in_room )
		{
			if ( !IS_NPC( grp ) && grp->leader == ch )
			{
				ptc( grp, "%s passes leadership of the group to %s, and you follow.\n\r", ch->name, victim->name );
				grp->leader = victim;
				grp->master = victim;
			}
		}
		ch->leader = victim;
		ch->master = victim;
		return;
	}*/

    if ( is_same_group( victim, ch ) && ch != victim )
    {
	    if ( victim->master != ch && ch != victim )
	    {
		act_new("{c$N isn't following you.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
		return;
	    }
		victim->leader = victim;
		victim->master = victim;
		act_new("{c$n removes $N from $s group.",ch,NULL,victim,TO_NOTVICT,POS_RESTING);
		act_new("{c$n removes you from $s group.",ch,NULL,victim,TO_VICT,POS_SLEEPING);
		act_new("{cYou remove $N from your group.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
		return;
    }
	if ( ch->pcdata->accept_follower_name == victim )
	{
	    CHAR_DATA *gch;
	    if ( victim->master != victim || ( victim->leader != victim && victim->leader != ch ) )
	    {
			stc( "{cThey are following someone else. Read help rpgroup to see the conditions of Rpgrouping.{c\n\r", ch );
			return;
	    }
	    if(is_alt_of(ch,victim))
	    {
		stc( "{cYou can not initiate an Rp group with your alts. Really now.\n\r", ch );
		return;
	    }
	    for ( gch = char_list; gch != NULL; gch = gch->next )
	    {
		    if ( is_same_group( gch, ch) && (is_alt_of(gch,victim)))
		    {
			stc( "{cThey can not Rp with their alts...duh?\n\r", ch );
			return;
		    }
	    }
	    victim->leader = ch;
	    victim->master = ch;
	    act_new("{c$N joins $n's group.",ch,NULL,victim,TO_NOTVICT,POS_RESTING);
	    act_new("{cYou join $n's group.",ch,NULL,victim,TO_VICT,POS_SLEEPING);
	    act_new("{c$N joins your group.",ch,NULL,victim,TO_CHAR,POS_SLEEPING);
	    ch->pcdata->accept_follower_name = NULL;
	    ch->pcdata->accept_follower = FALSE;
	    return;
	}
    else
    {
	    stc("They haven't been asked to Rp with you.\n\r",ch);
	    return;
    }
}

/*void do_rpinvite( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	stc( "{cFollow whom?{c\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch, NULL, arg ) ) == NULL )
    {
	stc( "{cThey aren't here.{c\n\r", ch );
	return;
    }
    if ( !victim->desc )
    {
    stc("That person is battling the Net Beast at the moment.\n\r",ch);
    return;
    }
    if ( IS_NPC( victim ) )
    {
    stc("You cannot follow mobiles.\n\r",ch);
    return;
    }
    if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master != NULL )
    {
	act("{cBut you'd rather follow $N!{c", ch, NULL, ch->master, TO_CHAR );
	return;
    }

    if ( victim == ch )
    {

	    if(ch->leader != NULL && ch->leader != ch )
	    {
		ptc(ch->leader,"%s no longer follows you.\n\r",ch->name);
	    }
	    ch->leader = ch;
	    ch->master = ch;
	    return;
    }

    if (!IS_NPC(victim) && IS_SET(victim->act,PLR_NOFOLLOW) && !IS_IMMORTAL(ch))
    {
		act("{c$N doesn't seem to want any followers.{c\n\r",ch,NULL,victim, TO_CHAR);
        return;
    }

    if ( victim->pcdata->accept_follower_name != NULL )
    {
		act("{c$N already has a pending querry to follow $M.{c\n\r",ch,NULL,victim,TO_CHAR);
        return;
    }
    ptc(ch,"You ask %s if you may join their group.\n\r", victim->name);
    ptc(victim,"%s wishes to join your group. Will you allow them?\n\r If so, type Group <char name> ",ch->name);
    victim->pcdata->accept_follower_name = ch;
    WAIT_STATE( ch, PULSE_VIOLENCE * 2 );
    return;
  }*/

void do_rplog ( CHAR_DATA *ch, char *argument )
{
	BUFFER *pbuf;
	FILE *fp;
	struct dirent *Dir;
	DIR *Directory;
    int fd;
    struct stat st;
	char fName[80];
	char buf[MSL], arg1[MIL], arg2[MIL];
	char *strtime;
	int nMatch, rpsize;
	nMatch = 0;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );


	strtime                    = ctime( &current_time );
    strtime[strlen(strtime)-1] = '\0';

	if ( !str_cmp( arg1, "help" ) )
	{
		stc( "Syntax Examples:\n\r", ch );
		stc( "RPLog Help\n\r", ch );
		stc( "RPLog Start|End\n\r", ch );
		if ( IS_IMMORTAL( ch ) )
		{
			stc( "RPLog List\n\r", ch );
			stc( "RPLog Review <name>\n\r", ch );
		}
		return;
	}
	if ( !str_cmp( arg1, "end" ) )
	{
		if ( !IS_SET( ch->comm, COMM_RP ) )
		{
			stc( "You are not in RP mode.\n\r", ch );
			return;
		}
		REMOVE_BIT( ch->comm, COMM_QUIET);
		stc( "Turning RPLog OFF! Quiet mode has been removed.\n\r", ch );
		stc( "An Immortal will soon review your log and determine if the points you receive are worth it.\n\r", ch );
		stc( "If you have attempted to spam the logs you will be frozen for at least 30 days. If the logs\n\r", ch );
		stc( "are not deserving of a reward the points will be dedicated from you.\n\r", ch );
		REMOVE_BIT( ch->comm, COMM_RP );
		return;
	}
	if ( !str_cmp( arg1, "start" ) )
	{
		if ( IS_SET( ch->comm, COMM_RP ) )
		{
			stc( "You are already in RP mode.\n\r", ch );
			return;
		}
		SET_BIT( ch->comm, COMM_QUIET);
		stc( "Turning RPLog ON! You have automatically been set into quiet mode.\n\r", ch );
		stc( "During this time all of your says, emotes, and smotes will be logged. You will earn points for\n\r", ch );
		stc( "the time your spend engaged in live roleplay.\n\r\n\r", ch );
		stc( "DO NOT try to abuse this system. The logs will be reviewed and if it is found you have tried to\n\r", ch );
		stc( "abuse the system you will be frozen for at least 30 days and your IP will be locked from the game.\n\r", ch );
		stc( "so that you cannot even play an alternate character.\n\r", ch );
		SET_BIT( ch->comm, COMM_RP );
		sprintf( fName, "%s%s", RP_DIR, ch->name);
		fp = fopen( fName, "w" );
		fclose (fp);
		return;
	}
	if ( IS_IMMORTAL( ch ) && !str_cmp( arg1, "list" ) )
	{
		pbuf = new_buf();
		fName[0] = '\0';
		Directory = opendir( RP_DIR );
		Dir = readdir( Directory );
		stc("{R+---------------------------------------------+{x\n\r",ch);
		stc("{R|{r         RPLOGS PENDING REVIEW               {R|{x\n\r",ch);
		stc("{R+---------------------------------------------+{x\n\r",ch);
		stc("{R Name            File Size{x\n\r",ch);
		while( Dir != NULL )
		{
			rpsize = 0;
			if ( Dir->d_name[0] >= 'A' && Dir->d_name[0] <= 'Z' )
			{
				nMatch++;
				sprintf( fName, "%s%s", RP_DIR, Dir->d_name );
			    if ( ( fp = fopen( fName, "r" ) ) != NULL )
			    {
					fd = fileno( fp );
					if ( fstat( fd, &st ) )
					{
						bugf( " Error while trying to fstat %s", Dir->d_name );
					}
					rpsize = st.st_size;
					sprintf( buf, " {w%-15s %-9d", Dir->d_name, rpsize );
					add_buf( pbuf, buf );
					fclose( fp );
			    }
				else
				{
					sprintf( buf, " Could not open file %s for read access.", Dir->d_name );
					add_buf( pbuf, buf );
				}
				fName[0] = '\0';
			}
			Dir = readdir( Directory );
		}
		closedir( Directory );
		page_to_char(buf_string(pbuf), ch);
		free_buf(pbuf);
		sprintf( buf, "\n\r{RPending Roleplay Logs Found{r: {Y%d{x\n\r", nMatch );
		stc( buf, ch );
		return;
	}
	if ( IS_IMMORTAL( ch ) && !str_cmp( arg1, "review" ) )
	{
		char *rptext;
		if ( arg2[0] == '\0' )
		{
			stc( "Syntax: rplog review <name>\n\r", ch );
			stc( "Use rplog list to see pending reviews.\n\r", ch );
			return;
		}
		rptext = str_dup("");
		pbuf = new_buf();
		fName[0] = '\0';
		stc("{R+---------------------------------------------+{x\n\r",ch);
		stc("{R|{r            REVIEWING RPLOG                  {R|{x\n\r",ch);
		stc("{R+---------------------------------------------+{x\n\r",ch);
		stc("{R Name            File Size{x\n\r",ch);
		rpsize = 0;
		sprintf( fName, "%s%s", RP_DIR, capit2(arg2) );
	    if ( ( fp = fopen( fName, "r" ) ) != NULL )
	    {
			char letter;
			char *word;
			fd = fileno( fp );
			if ( fstat( fd, &st ) )
			{
				bugf( "RPLOG Error while trying to fstat %s.", capit2(arg2) );
			}
			rpsize = st.st_size;
			sprintf( buf, " {w%-15s %-9d", capit2(arg2), rpsize );
			add_buf( pbuf, buf );
		    for ( ; ; )
		    {
				letter = fread_letter( fp );
				word = fread_word( fp );
				if ( letter != '#' )
				{
					bugf( "Fread_RP: # not found in rp for %s.", fName );
					break;
				}
				if( !str_cmp(word, "EOF" )  )
					break;

		        if ( !str_cmp( word, "TEXT" ) )
					rptext = fread_string( fp );
				if ( rptext != NULL && rptext[0] != '\0' )
				{
					add_buf( pbuf, "\n\r" );
					add_buf( pbuf, rptext );
					free_string( rptext );
				}
				else
				{
					bugf( "RPLOG Error while trying to add rptext to buffer.", capit2(arg2) );
				}
		    }
			fclose( fp );
		}
		else
		{
			sprintf( buf, " Could not open file %s for read access.\n\r Does %s exist in RP_DIR?", fName, capit2(arg2) );
			add_buf( pbuf, buf );
		}
		add_buf( pbuf, "\n\r" );
		fName[0] = '\0';
		page_to_char(buf_string(pbuf), ch);
		free_buf(pbuf);
		return;
	}
	do_function( ch, &do_rplog, "help" );
	return;
}
void rplog_command( CHAR_DATA *ch, char * command )
{
	FILE *fp;
	char fName[80];
	sprintf( fName, "%s%s", RP_DIR, capit2(ch->name) );

	/* no hackers */
	smash_tilde( command );

	if ( ( fp = fopen( fName, "a" ) ) != NULL )
	{
		fprintf( fp, "#TEXT %s~\n", command );
		fclose( fp );
	}

	return;
}

int gen_rp_reward(CHAR_DATA *ch)
{
	int base = 1500;
	int bonus, total;

	bonus = number_range(1, 250);

	total = base + bonus;

	return total;
}

