/***************************************************************************

                      ___  _      _  _      _
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \__|_  /\_/\  /    \_/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
        intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

        ROM 2.4 is copyright 1993-1998 Russ Taylor
        ROM has been brought to you by the ROM consortium
            Russ Taylor (rtaylor@hypercube.org)
            Gabrielle Taylor (gtaylor@hypercube.org)
            Brian Moore (zump@rom.org)
        By using this code, you have agreed to follow the terms of the
        ROM license, in the file Rom24/doc/rom.license


***************************************************************************/
#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "ship.h"
#include "tables.h"



/* Simple boolean check to see if the char is in a valid casino room. If
not, can
 * assume they are somewhere else on the ship. Casino rooms don't receive
messages
 * as the ship travels.
 */
bool in_casino_room( CHAR_DATA *ch, ROOM_INDEX_DATA *room )
{
	int ship;
        for(ship = 1; ship<= MAX_SHIPS; ship++)
		if ( (ch->in_room->vnum == ShipTable[ship].highVnum));
			return 1;
	return 0;
}



int docked_at(int ship)
{
	int i;
	for(i=0;i<ShipTable[ship].num_docks;i++)
		if (ShipTable[ship].docks[i].loc == ShipActive[ship].location)
		{
			ShipActive[ship].next_dock=i;
			return ShipTable[ship].docks[i].vnum;
		}
	return 0;
}

int on_which_ship(CHAR_DATA *ch)
{
	int ship;
	for (ship = 1;ship <=MAX_SHIPS; ship++)
	{
		if ((ch->in_room->vnum >=  ShipTable[ship].lowVnum) && (ch->in_room->vnum <= ShipTable[ship].highVnum))
			return ship;
	}
	return 0;
}
/* The the ship is at a valid dock, can leave the ship for the dock, else
they jump
 * overboard and learn to swim.
 */
/* really should implement that sometime
 * leap from barge into Haddon Mirk */
/*
	if ( ch->in_room->vnum == 7819 )
	{
		act("$n leaps from the barge and swims for shore!",ch,NULL,NULL,TO_ROOM);
	    char_from_room(ch);
	    char_to_room(ch,get_room_index( 10277 ) );
	    act("You leap from the barge and swim towards the shore!",ch,NULL,NULL,TO_CHAR);
	    act("$n leaps into the river!",ch,NULL,NULL,TO_ROOM);
	    if (ch->pet != NULL)
	    {
			char_from_room( ch->pet );
			char_to_room( ch->pet, ch->in_room );
		}
		do_function( ch, &do_look, "auto" );
	}
	*/
/* FIXME */

void do_disembark(CHAR_DATA *ch, char *argument )
{
        int dock;
	int ship;
	ship = on_which_ship(ch);
        dock = docked_at(ship);

        if (ship == 0)   /* ship 0 is the null ship, means not on a
ship*/
                stc("{wYou aren't on a ship.{x\n\r",ch);
	else if((ShipTable[ship].motion_type == 1) && (ShipActive[ship].location == 11))
	{
		act("$n leaps from the barge and swims for shore!",ch,NULL,NULL,TO_ROOM);
	    char_from_room(ch);
	    char_to_room(ch,get_room_index( 10277 ) );
	    act("You leap from the barge and swim towards the shore!",ch,NULL,NULL,TO_CHAR);
	    act("$n leaps into the river!",ch,NULL,NULL,TO_ROOM);
	    if (ch->pet != NULL)
	    {
			char_from_room( ch->pet );
			char_to_room( ch->pet, ch->in_room );
		}
		do_function( ch, &do_look, "auto" );
	}
        else if (dock == 0)
                stc("{wYou are not docked.{x\n\r",ch);
        else if (ch->in_room->vnum != ShipTable[ship].lowVnum)
                stc("{wYou are not on the main deck.{x\n\r",ch);   /*
modify so that they know you have to be in a certain room to get off */
        else
        {
    		CHAR_DATA *fch;
    		CHAR_DATA *fch_next;
    		ROOM_INDEX_DATA *in_room;
    		ROOM_INDEX_DATA *to_room;
    		char buf[MSL];
		to_room = get_room_index(dock);
		in_room = ch-> in_room;

    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
/*        if (ch->pcdata != NULL && ch->pcdata->wout[0] != '\0') */ /* walkins broken thanks to Anthony's %s scheme */
/*            act( "{c$t heading $T.{c",ch,ch->pcdata->wout,"towards the dock",TO_ROOM);
        else
            act( "{c$n leaves $T.{c", ch, NULL, dir_name[door], TO_ROOM );*/
    {
	if (MOUNTED(ch))
	{
	    if( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		sprintf(buf, "$n leaves $T, riding on %s.", MOUNTED(ch)->short_descr);
        else
           sprintf(buf, "$n soars $T, on %s.", MOUNTED(ch)->short_descr);
           act( buf, ch, NULL, "towards the dock", TO_ROOM );
    }
	else
	{
	    act( "$n leaves $T.", ch, NULL, "towards the dock", TO_ROOM );
    }
    }
    }
                char_from_room(ch);
                char_to_room(ch,to_room);
    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
/*    if (ch->pcdata != NULL && ch->pcdata->wkin[0] != '\0')
           act( "{c$t from the $T.{c",ch,ch->pcdata->wkin,ShipTable[ship].name,TO_ROOM);
    else */ /* since the new walkins with a %s are compatible */
           /*act( "{c$n has arrived.{c", ch, NULL, NULL, TO_ROOM );*/
    {
	if(!MOUNTED(ch))
	{
	    act( "$n has arrived from the $T.", ch, NULL,ShipTable[ship].name , TO_ROOM );
	}
	else
	{
	    if ( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		act( "$n has arrived, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
	    else
		act( "$n soars in, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
        }
    }

    if (MOUNTED(ch))
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), to_room );
    }
    }

    do_function(ch, &do_look, "auto" );

    if (in_room == to_room) /* no circular follows */
	return;

    for ( fch = in_room->people; fch != NULL; fch = fch_next )
    {
	fch_next = fch->next_in_room;

	if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM)
	&&   fch->position < POS_STANDING)
	    do_function(fch, &do_stand, "");

	if ( fch->master == ch && fch->position == POS_STANDING
	&&   can_see_room(fch,to_room))
	{

	    if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
	    &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
	    {
		act( "{cYou can't bring $N into the city.{c",
		    ch,NULL,fch,TO_CHAR);
		act( "{cYou aren't allowed in the city.{c",
		    fch,NULL,NULL,TO_CHAR);
		continue;
	    }

	    act( "{cYou follow $N.{c", fch, NULL, ch, TO_CHAR );
	    do_function(fch,&do_disembark,"");
	}
    }


    /*
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC( ch ) && HAS_TRIGGER_MOB( ch, TRIG_ENTRY ) )
	p_percent_trigger( ch, NULL, NULL, NULL, NULL, NULL, TRIG_ENTRY );

    if ( !IS_NPC( ch ) )
    {
   	p_greet_trigger( ch, PRG_MPROG );
	p_greet_trigger( ch, PRG_OPROG );
	p_greet_trigger( ch, PRG_RPROG );
    }

   /*             if (ch->pet != NULL)
                {
                        char_from_room(ch->pet);
                        char_to_room(ch->pet, dock_index);
                }
                if (ch->mount != NULL)
                {
                        char_from_room(ch->mount);
                        char_to_room(ch->mount, dock_index);
                }
    		do_function(ch, &do_look, "auto" );
	*/
        }
}
int pick_ship(CHAR_DATA *ch,char *argument)
{
	int i;
	char arg1[MIL];
	argument = one_argument( argument, arg1 );

        for(i = 1; i<= MAX_SHIPS; i++)
        {
                if (ch->in_room->vnum == docked_at(i))
			if ((arg1==NULL)||(!str_prefix(arg1,ShipTable[i].name)))
                        	return i;
        }
                return 0;
}


/* The the char is at a valid dock, can embark on the ship by expending a
number of
 * coins equal to the cost for the boat ride, found in the header.
 */
void do_embark( CHAR_DATA *ch,char *argument)
{
        int ship;
        ship = pick_ship(ch,argument);
        if (ship == 0)
	{
		if (argument[0]=='\0')
			stc("{wThere is no ship docked here.{x\n\r",ch);
		else
                	stc("{wThat ship is not docked here.{x\n\r",ch);
	}
	else if (!check_coins(ch,25,0,0))
		ptc(ch,"{xYou don't have %d copper.{x\n\r",25);
        else
        {
    		CHAR_DATA *fch;
    		CHAR_DATA *fch_next;
    		ROOM_INDEX_DATA *in_room;
    		ROOM_INDEX_DATA *to_room;
    		char buf[MSL];
		in_room = ch-> in_room;
		to_room = get_room_index(ShipTable[ship].lowVnum);
		deduct_cost(ch,25,0);
		ptc(ch,"{wYou pay %d copper.{x\n\r",25);

    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
        if (ch->pcdata != NULL && ch->pcdata->wout[0] != '\0')
            act( "{c$t heading towards the $T.{c",ch,ch->pcdata->wout,ShipTable[ship].name,TO_ROOM);
        else
            /*act( "{c$n leaves $T.{c", ch, NULL, dir_name[door], TO_ROOM );*/
    {
	if (MOUNTED(ch))
	{
	    if( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		sprintf(buf, "$n leaves towards the $T, riding on %s.", MOUNTED(ch)->short_descr);
        else
           sprintf(buf, "$n soars towards the $T, on %s.", MOUNTED(ch)->short_descr);
           act( buf, ch, NULL, ShipTable[ship].name, TO_ROOM );
    }
	else
	{
	    act( "$n leaves towards the $T.", ch, NULL, ShipTable[ship].name, TO_ROOM );
    }
    }
    }
                char_from_room(ch);
                char_to_room(ch,to_room);
    if ( !IS_AFFECTED(ch, AFF_SNEAK)
    &&   ch->invis_level < LEVEL_HERO)
    {
/*    if (ch->pcdata != NULL && ch->pcdata->wkin[0] != '\0')
           act( "{c$t from the $T.{c",ch,ch->pcdata->wkin,"dock",TO_ROOM);
    else  */
           /*act( "{c$n has arrived.{c", ch, NULL, NULL, TO_ROOM );*/
    {
	if(!MOUNTED(ch))
	{
	    act( "$n has arrived from the $T.", ch, NULL, "docks", TO_ROOM );
	}
	else
	{
	    if ( !IS_AFFECTED(MOUNTED(ch), AFF_FLYING) )
		act( "$n has arrived, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
	    else
		act( "$n soars in, riding on $N.", ch, NULL, MOUNTED(ch), TO_ROOM );
        }
    }

    if (MOUNTED(ch))
    {
        char_from_room( MOUNTED(ch) );
        char_to_room( MOUNTED(ch), to_room );
    }
    }

    do_function(ch, &do_look, "auto" );

    if (in_room == to_room) /* no circular follows */
	return;

    for ( fch = in_room->people; fch != NULL; fch = fch_next )
    {
	fch_next = fch->next_in_room;

	if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM)
	&&   fch->position < POS_STANDING)
	    do_function(fch, &do_stand, "");

	if ( fch->master == ch && fch->position == POS_STANDING
	&&   can_see_room(fch,to_room))
	{

	    if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
	    &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
	    {
		act( "{cYou can't bring $N into the city.{c",
		    ch,NULL,fch,TO_CHAR);
		act( "{cYou aren't allowed in the city.{c",
		    fch,NULL,NULL,TO_CHAR);
		continue;
	    }

	    act( "{cYou follow $N.{c", fch, NULL, ch, TO_CHAR );
	    do_function(fch,&do_embark,argument);
	}
    }


    /*
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC( ch ) && HAS_TRIGGER_MOB( ch, TRIG_ENTRY ) )
	p_percent_trigger( ch, NULL, NULL, NULL, NULL, NULL, TRIG_ENTRY );

    if ( !IS_NPC( ch ) )
    {
   	p_greet_trigger( ch, PRG_MPROG );
	p_greet_trigger( ch, PRG_OPROG );
	p_greet_trigger( ch, PRG_RPROG );
    }
        }
}
bool caravan_schedule(CHAR_DATA *ch, char *arg1);
void do_schedule(CHAR_DATA *ch, char *argument)
{
	int ship,i,j;
	char arg1[MIL];
	bool found = FALSE;
	argument = one_argument( argument, arg1 );

	found = caravan_schedule(ch,arg1);

	if(on_which_ship(ch) != 0)
	{
		stc("But you are already on a ship, isn't that enough?\n\r",ch);
		return;
	}
        for(ship = 1; ship<= MAX_SHIPS; ship++)
        {
	    if (!((arg1==NULL)||(!str_prefix(arg1,ShipTable[ship].name))||!str_prefix(arg1,"list")))
		continue;
	    	    i = ShipActive[ship].next_dock;
               if (ch->in_room->vnum == docked_at(ship))
		{
			ptc(ch,"You may embark upon the {g%s {xnow.\n\r",ShipTable[ship].name);
			ptc(ch,"The {g%s {xwill then head for {y%s.{x\n\r\n\r",ShipTable[ship].name,ShipTable[ship].docks[i].next_docks);
			j = 1;
	    	        i = (i + 1) % ShipTable[ship].num_docks;
		}else
		{
			j = 0;
		}
	    for(; j<ShipTable[ship].num_docks; j++)
	    {
		    
                if (ch->in_room->vnum == ShipTable[ship].docks[i].vnum)
			{
				ptc(ch,"You can embark upon the {g%s {xhere in about {y%d {xminutes or so.\n\r",ShipTable[ship].name,((ShipTable[ship].hops - ShipActive[ship].location + ShipTable[ship].docks[i].loc ) % ShipTable[ship].hops) + j );
				ptc(ch,"The {g%s {xwill then head for {y%s.{x\n\r\n\r",ShipTable[ship].name,ShipTable[ship].docks[i].next_docks);
				found = TRUE;
			}
		i = (i + 1/* + ShipTable[ship].num_docks*/ ) % ShipTable[ship].num_docks;
	    }
	}
	if(!found)
		stc("Sorry, but nothing is scheduled to stop here.\n\r",ch);
}


/* Ship advance moves the ship and char from one virtual room to another.
It also
 * sends a nifty message to any chars aboard the ship concerning the
voyage. If the
 * ship enter or leaves a dock, it will inform any chars waiting for or on
the ship.
 */
void ship_advance( void )
{

int ship;
int dock;
DESCRIPTOR_DATA *d;
char buf[MSL];
#ifdef DOCKOBJ
OBJ_DATA *obj;
#endif

for (ship = 1; ship <=MAX_SHIPS; ship++)
{
	dock = docked_at(ship);
        if (dock != 0)
        {
	   if (ShipActive[ship].wait_at_dock<=1)
	   {
                for ( d = descriptor_list; d != NULL; d = d->next )
                {
                        if ( d->connected == CON_PLAYING
                        && d->character->in_room != NULL
                        && !is_fighting(d->character) )
                        {

                                if  (d->character->in_room->vnum == dock)
                                {
                                        ptc(d->character,"{wThe %s leaves the docks.\n\r{x", ShipTable[ship].name);
                                }
                        }
                }
		ShipActive[ship].next_dock++;
		if (ShipActive[ship].next_dock == ShipTable[ship].num_docks)
			ShipActive[ship].next_dock = 0;
#ifdef DOCKOBJ
		
		obj_from_room(ShipActive[ship].dock_obj);
		
#endif
	   }
	   else
	   {
		ShipActive[ship].location--;
	   }
	   ShipActive[ship].wait_at_dock--;
	}

        if (ShipActive[ship].location >= ShipTable[ship].hops)
                ShipActive[ship].location = 0;
        else
                ShipActive[ship].location++;





/* ptc is printf to char   */
        /* Debugable  */

        if ((ShipActive[ship].location < 0) || (ShipActive[ship].location > ShipTable[ship].hops) )
        {
                bugf("Ship Error: %d", ship ); /* writes error message to
logs */
                ShipActive[ship].location = 1;
        }
        /* end Debugable */
}
for (ship = 1; ship <=MAX_SHIPS; ship++)
{
	dock = docked_at(ship);
        if ((dock != 0) && (ShipActive[ship].wait_at_dock<=0))
        {
		ShipActive[ship].wait_at_dock = SHIP_WAIT;
                for ( d = descriptor_list; d != NULL; d = d->next )
                {
                        if ( d->connected == CON_PLAYING
                        && d->character->in_room != NULL
                        && !is_fighting(d->character))
                        {
                                if  (d->character->in_room->vnum == dock)
                                {
                                        ptc(d->character,"{wThe %s docks here. It will be headed for %s soon.\n\r{x", ShipTable[ship].name,ShipTable[ship].docks[ShipActive[ship].next_dock].next_docks);
                                }
                        }
		}
#ifdef DOCKOBJ

	obj = ShipActive[ship].dock_obj;
	if (obj->short_descr)
		free_string( obj->short_descr );
       	sprintf(buf,"The %s is docked here.",ShipTable[ship].name);
        obj->short_descr = str_dup(buf);
	if (obj->description)
		free_string( obj->description);
       	sprintf(buf,"The %s will be headed for %s soon.{x",ShipTable[ship].name,ShipTable[ship].docks[ShipActive[ship].next_dock].next_docks);
        obj->description = str_dup(buf);
	obj_to_room(ShipActive[ship].dock_obj,get_room_index(dock));

#endif
        }
}
/* for all char on ship */
/* note that dir is parsed before location */

        for ( d = descriptor_list; d != NULL; d = d->next )
        {
                if ( d->connected == CON_PLAYING
                && d->character->in_room != NULL
                && !is_fighting(d->character) )
                {
			int ship;
			ship = on_which_ship(d->character);
                        if  ((ship != 0) && ((ShipActive[ship].wait_at_dock==SHIP_WAIT)||(docked_at(ship)==0)))
                        {
                                stc("{w",d->character);
           			sprintf(buf,ShipMotionTable[ShipTable[ship].motion_type][ShipActive[ship].location],ShipTable[ship].name);
                                ptc(d->character,buf);
                                stc("\n\r{x",d->character);
                        }
                }
        }
}
#ifdef DOCKOBJ
void ship_init(void)
{
	int ship,dock;
	for (ship=1;ship<=MAX_SHIPS;ship++)
	{
		ShipActive[ship].dock_obj = create_object( get_obj_index( DOCK_OBJ_VNUM ), 1 );
		dock = docked_at(ship);
        	if (dock != 0)
			obj_to_room(ShipActive[ship].dock_obj,get_room_index(dock));
		/* no sense setting the desc since it'll be gone before any actions can occur */
	}
}
#endif
