/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aww.h"
#include "recycle.h"
#include "interp.h"
#include "magic.h"

DECLARE_DO_FUN(do_yell		);
DECLARE_DO_FUN(do_sleep		);
DECLARE_DO_FUN(do_sit		);
DECLARE_DO_FUN(do_bash_door	);
DECLARE_DO_FUN(do_throw_spear	);

DECLARE_SPELL_FUN(	spell_poison	);
DECLARE_SPELL_FUN(	spell_blindness	);

OBJ_DATA *  get_random_material args((CHAR_DATA *mob, int type, bool nullallowed));
bool    is_obj_char_clan args( ( CHAR_DATA *ch, 
OBJ_DATA *obj) );
void	one_hit		args( ( CHAR_DATA *ch, CHAR_DATA *victim, int dt ,bool secondary) );
/*void	set_fighting	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );*/
bool	remove_obj	args( (CHAR_DATA *ch, int iWear, bool fReplace ) );
bool	check_dodge	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
bool	check_parry	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
bool    check_shield_block     args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );
OBJ_DATA *make_special args(( OBJ_DATA *fObj ));

void do_throw( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *victim;
    char arg[MAX_INPUT_LENGTH];
    int chance;

    argument = one_argument(argument,arg);

    if ( IS_NPC(ch) )
    {
	stc("{cYou couldn't throw a fit if you tried.\n\r", ch );
	return;
    }

    if ( get_skill( ch, gsn_throw ) <= 0 )
    {
	stc("{cYou couldn't throw a fit if you tried.\n\r", ch );
	return;
    }

    if (IS_AFFECTED(ch,AFF_FLYING))
	{
	 stc("{cBut your feet aren't touching the ground!\n\r",ch);
	 return;
	}

    if ( !is_fighting(ch) )
    {
	stc( "{cYou aren't fighting anyone.\n\r", ch );
	return;
    }
	victim = ch->advancing_on;

    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
	act("{cBut $N is your friend!",ch,NULL,victim,TO_CHAR);
	return;
    }

    if (is_safe(ch,victim))
      return;

    WAIT_STATE( ch, skill_table[gsn_throw].beats );

    chance = get_skill(ch,gsn_throw);

    if (ch->size < victim->size)
	chance += (ch->size - victim->size) * 10;
    else
	chance += (ch->size - victim->size) * 25;


    /* stats */
    chance += get_curr_stat(ch,STAT_STR);
    chance -= get_curr_stat(victim,STAT_DEX) * 4/3;

    if (IS_AFFECTED(victim,AFF_FLYING) ) chance += 10;

    /* speed */
    if (IS_SET(ch->off_flags,OFF_FAST))
	chance += 10;
    if (IS_SET(victim->off_flags,OFF_FAST))
	chance -= 20;

	/* race */
	if (ch->race == race_lookup( "trolloc"))
	chance += 10;
    /* level */
    chance += (ch->level - victim->level) * 2;

    if ( IS_NPC(ch) || number_percent() < chance )
    {
      act("{cYou throw $N to the ground with stunning force.",ch,NULL,victim,TO_CHAR);
      act("{c$n throws you to the ground with stunning force.",ch,NULL,victim,TO_VICT);
      act("{c$n throws $N to the ground with stunning force.",ch,NULL,victim,TO_NOTVICT);
      WAIT_STATE(victim,2 * PULSE_VIOLENCE);

      damage( ch, victim,ch->level + get_curr_stat(ch,STAT_STR),
	     gsn_throw,DAM_BASH, TRUE );
      check_improve(ch,gsn_throw,TRUE,1);
    }
    else
    {
	act( "{cYou fail to grab your opponent.", ch, NULL, NULL, TO_CHAR);
	act( "{c$N tries to throw you, but fails.", victim, NULL, ch,TO_CHAR);
	act( "{c$n tries to grab $N's arm.", ch, NULL, victim, TO_NOTVICT);
	check_improve(ch,gsn_throw,FALSE,1);
    }

    return;
}

void do_strangle(CHAR_DATA *ch, char *argument)
{
    CHAR_DATA *victim;
    AFFECT_DATA af;

    if ( IS_NPC(ch) )
    {
	stc("{cYou lack the knowledge.\n\r",ch);
	return;
    }

    if ( get_skill( ch, gsn_strangle ) <= 0 )
    {
	stc("{cYou couldn't strangle a chicken even if you tried.\n\r", ch );
	return;
    }

    if ( IS_AFFECTED( ch, AFF_CHARM ) )
    {
	stc( "{cYou don't want to grab your masters' neck!\n\r",ch);
	return;
    }

    if ( (victim = get_char_room(ch,NULL,argument)) == NULL)
    {
	stc("{cYou do not see that person here.\n\r",ch);
	return;
    }

    if (victim == ch)
    {
	stc("{cEven you are not that stupid.\n\r",ch);
	return;
    }

	if ( is_fighting(ch) )
	{
		stc("Not while you're fighting.\n\r",ch);
		return;
	}
	if (!IS_NPC(victim))
	{
		stc("Only on NPCs.\n\r",ch);
		return;
	}
	else
	{
		if (IS_SET ( victim->act, ACT_AGGRESSIVE ))
		{
			start_first_fight(ch, victim, RANGE_MELEE);
			return;
		}
	}

    if (is_affected(victim,gsn_strangle))
    	return;

    if (is_safe(ch,victim))
	{
		stc("{cYou can't do that here.\n\r",ch);
      	return;
	}
	if ( is_fighting(victim))
	{
		stc("Not while they're fighting.\n\r",ch);
		return;
	}

    WAIT_STATE( ch, PULSE_VIOLENCE );

    if ( number_percent() < 0.6 * get_skill ( ch,gsn_strangle ) )
    {
		act("{cYou grab hold of $N's neck and put $M to sleep.",
		    ch,NULL,victim,TO_CHAR);
		act("{c$n grabs hold of your neck and puts you to sleep.",
		    ch,NULL,victim,TO_VICT);
		act("{c$n grabs hold of $N's neck and puts $M to sleep.",
		    ch,NULL,victim,TO_NOTVICT);
		check_improve(ch,gsn_strangle,TRUE,10);

		af.type = gsn_strangle;
	    af.where = TO_AFFECTS;
		af.level = ch->level;
		af.duration = number_range(1,3);
		af.location = APPLY_NONE;
		af.modifier = 0;
		af.bitvector = AFF_SLEEP;
		affect_join ( victim,&af );

		if (IS_AWAKE(victim))
		{
			victim->position = POS_SLEEPING;
	    }
	}
    else
    {
		act("You try to strangle $N without much luck.",ch,NULL,victim,TO_CHAR);
		act("$n tries to strangle $N without much luck.",ch,NULL,victim,TO_ROOM);
		check_improve(ch,gsn_strangle,FALSE,10);
	}
}

void do_blackjack(CHAR_DATA *ch, char *argument)
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    AFFECT_DATA af;
    int chance;

    if ( IS_NPC(ch) )
    {
    	stc("{cYou lack the knowledge.\n\r",ch);
    	return;
    }

    if ( get_skill( ch, gsn_blackjack ) <= 0 )
    {
	stc("{cYou couldn't throw a fit if you tried.\n\r", ch );
	return;
    }

    if ( (victim = get_char_room(ch,NULL,argument)) == NULL)
    {
    	stc("{cYou do not see that person here.\n\r",ch);
    	return;
    }

    if (victim == ch)
    {
    	stc("{cBlackjack your self?! Really now...\n\r",ch);
    	return;
    }

    if ( IS_AFFECTED( ch, AFF_CHARM ) )
    {
    	stc( "{cBut you don't want to hit your beloved masters.\n\r",ch);
    	return;
    }

    if (IS_AFFECTED(victim,AFF_SLEEP))
    {
        act("{c$E is already asleep.",ch,NULL,victim,TO_CHAR);
        return;
    }

    if ( ( obj = get_eq_char( ch, WEAR_WIELD ) ) == NULL)
    {
	stc( "You need to wield a weapon to blackjack.\n\r", ch );
	return;
    }

    if (attack_table[obj->value[3]].damage != DAM_BASH)
    {
	stc("You need to wield a bashing weapon to blackjack.\n\r",ch);
	return;
    }

    if (is_safe(ch,victim))
        return;

	if (!IS_NPC(victim))
	{
		stc("Only on NPCs.\n\r",ch);
		return;
	}

    WAIT_STATE(ch,skill_table[gsn_blackjack].beats);

    chance = 0.5 * get_skill(ch,gsn_blackjack);
    chance += URANGE( 0, (get_curr_stat(ch,STAT_DEX)-20)*2, 10);
    chance += can_see(victim, ch) ? 0 : 5;

	if (IS_NPC(victim) && victim->pIndexData->pShop != NULL)
		chance -= 99;

	if ( chance < get_skill(ch,gsn_blackjack))
	{
		act("$n just tried to blackjack you!",ch,NULL,NULL,TO_VICT);
		act("$n just tried to blackjack $N!",ch,NULL,victim,TO_ROOM);
		act("Uh oh... Maybe that wasn't such a good idea.",ch,NULL,NULL,TO_CHAR);
		multi_hit( victim, ch, TYPE_UNDEFINED );
		return;
	}

    if (IS_NPC(ch) || number_percent() < chance)
    {
	act("{cYou hit $N's over the head!",   ch,NULL,victim,TO_CHAR);
	act("{cYou feel a sudden pain erupts through your skull!",  ch,NULL,victim,TO_VICT);
	act("{c$n whacks $N at the back of $S head with a heavy looking sack!  *OUCH*",
	    ch,NULL,victim,TO_NOTVICT);
	check_improve(ch,gsn_blackjack,TRUE,1);

	af.type = gsn_blackjack;
    af.where = TO_AFFECTS;
	af.level = ch->level;
	af.duration = ch->level / 15 + 1;
	af.location = APPLY_NONE;
	af.modifier = 0;
	af.bitvector = AFF_SLEEP;
	affect_join ( victim,&af );

	if (IS_AWAKE(victim))
	{
	  victim->position = POS_SLEEPING;
    }
    else
    {
	damage(ch,victim,ch->level / 2,gsn_blackjack,DAM_NONE, TRUE);
	check_improve(ch,gsn_blackjack,FALSE,1);
    }
	}
}

void do_treat( CHAR_DATA *ch, char *argument)
{
    int heal;
    CHAR_DATA *victim;

	if (IS_NPC(ch))
    {
		stc("{cHuh?.\n\r",ch);
		return;
    }

    if ( get_skill( ch, gsn_treat ) <= 0 )
    {
	stc("{cYou know nothing about treating wounds.\n\r", ch );
	return;
    }

    if ( argument[0] == '\0' )
	    victim = ch;
    else if ( (victim = get_char_room(ch,NULL,argument)) == NULL)
    	{
		stc("{cYou do not see that person here.\n\r",ch);
		return;
    	}
	if (IS_NPC(victim))
	{
		stc("You cannot do that on a mob.\n\r", ch);
		return;
	}

    if (is_affected(victim, gsn_treat))
    {
		stc("{cYou see some bandages already applied.\n\r",ch);
		return;
    }
    if (number_percent() < get_skill(ch,gsn_treat) )
    {
		AFFECT_DATA af;
		WAIT_STATE(ch, PULSE_VIOLENCE);
		if ( victim == ch )
			stc( "You bandage your wounds.\n\r", ch );
		else
		{
			act("{cYou bandage $N's wounds.",ch,NULL,victim,TO_CHAR);
			act("{c$n bandages $N's wounds.",ch,NULL,victim,TO_ROOM);
		}
		check_improve(ch,gsn_treat,TRUE,2);

	    heal = dice(4, 8) + ch->level / 2;
	    victim->hit = UMIN( victim->hit + heal, get_maxhit(victim) );
	    update_pos( victim );
	    stc( "{cYou feel better!\n\r", victim );

		af.where	= TO_AFFECTS;
		af.type		= gsn_treat;
		af.level	= ch->level;
		af.duration	= URANGE(1, ch->level / 20, 10);
		af.modifier	= URANGE(10, ch->level/8, 25);
		af.bitvector 	= AFF_REGENERATION;
		af.location	= APPLY_CON;
		affect_to_char(victim,&af);
    }
    else
    {
		WAIT_STATE(ch, PULSE_VIOLENCE);
		stc("You fail the treatment.\n\r", ch);
		check_improve(ch,gsn_treat,FALSE,5);
    }
}

void do_void( CHAR_DATA *ch, char *argument)
{
    int chance = 0;

    /*if ((chance = get_skill(ch,gsn_void)) == 0
    ||  (!IS_NPC(ch)
    &&   ch->level < skill_table[gsn_void].skill_level[ch->class]))*/
    if ( IS_NPC( ch ) )
    	return;

    chance = get_skill( ch, gsn_void );

    if ( chance <= 0 )
    {
		stc("{cYou focus, but the Void eludes you.\n\r",ch);
		return;
    }

    if (IS_AFFECTED(ch,AFF_VOID) || is_affected(ch,gsn_void)
    ||  is_affected(ch,skill_lookup("void")))
    {
	stc("{cYou concentrate even harder on maintaining the Void.\n\r",ch);
	return;
    }

    if (ch->stam < 50)
    {
	stc("{cYou don't have enough energy to attain the void.\n\r",ch);
	return;
    }

    /* modifiers */

    /* fighting */
    if (is_fighting(ch))
	chance -= 10;

	chance += (ch->mod_stat[STAT_WIS] + ch->mod_stat[STAT_INS]) / 10;

    if (number_percent() < chance)
    {
		AFFECT_DATA af;

		WAIT_STATE(ch,PULSE_VIOLENCE/2);

		stc("{cYou focus on the {rFlame{c inside the {wVoid.{c\n\r",ch);
		act("{c$n's eyes focus sharply.",ch,NULL,NULL,TO_ROOM);
		check_improve(ch,gsn_void,TRUE,10);

		af.where	= TO_AFFECTS;
		af.type		= gsn_void;
		af.level	= ch->level;
		af.duration	= number_fuzzy(ch->level / 10);
		af.modifier	= UMAX(1,ch->level/8);
		af.bitvector 	= AFF_VOID;

		af.location	= APPLY_HITROLL;
		affect_to_char(ch,&af);
    }
    else
    {
		WAIT_STATE(ch,PULSE_VIOLENCE);
		stc("{cYou focus, but the {rFlame{c inside the {wVoid{c remains just out of reach.\n\r",ch);
		check_improve(ch,gsn_void,FALSE,9);
    }
}

/*
 * Disarm a creature.
 * Caller must check for successful attack.
 */
void disarm( CHAR_DATA *ch, CHAR_DATA *victim )
{
    OBJ_DATA *obj;

    if ( ( obj = get_eq_char( victim, WEAR_WIELD ) ) == NULL )
	return;

    if ( IS_OBJ_STAT(obj,ITEM_NOREMOVE))
    {
	act("{c$S weapon won't budge!",ch,NULL,victim,TO_CHAR);
	act("{c$n tries to disarm you, but your weapon won't budge!",
	    ch,obj,victim,TO_VICT);
	act("{c$n tries to disarm $N, but fails.",ch,NULL,victim,TO_NOTVICT);
	return;
    }

    if ( !IS_NPC(victim) &&
	victim->level > skill_table[gsn_grip].skill_level[victim->class])
    {
    int skill = get_skill(victim,gsn_grip);
    skill += (get_curr_stat(victim,STAT_STR) - get_curr_stat(ch,STAT_STR)) * 5;
    skill += (get_curr_stat(victim,STAT_REF) - get_curr_stat(ch,STAT_REF)) * 5;
    skill /= 2;
	if ( number_percent() < skill )
	{
	 act("$N grips tightly, and you fail!",ch,obj,victim,TO_CHAR);
	 act("$n tries to disarm you, but you grip your weapon tightly!",
	    ch,obj,victim,TO_VICT);
	 act("$n tries to disarm $N, but fails.",ch,obj,victim,TO_NOTVICT);
	 check_improve(victim,gsn_grip,TRUE,1);
	 return;
	}
	else 	 check_improve(victim,gsn_grip,FALSE,1);
    }

    act( "{c$n {r!!! {RDISARMS{r !!!{c you and sends your weapon flying!",
	 ch, NULL, victim, TO_VICT    );
    act( "{cYou {r!!! {RDISARMS{r !!!{c $N!",  ch, NULL, victim, TO_CHAR    );
    act( "{c$n {r!!! {RDISARMS{r !!!{c $N!",  ch, NULL, victim, TO_NOTVICT );

    obj_from_char( obj );
    if ( IS_OBJ_STAT(obj,ITEM_NODROP) || IS_OBJ_STAT(obj,ITEM_INVENTORY) )
	obj_to_char( obj, victim );
    else
    {
	obj_to_room( obj, victim->in_room );
	if (IS_NPC(victim) && victim->wait == 0 && can_see_obj(victim,obj))
	    get_obj(victim,obj,NULL);
    }

    return;
}

void do_berserk( CHAR_DATA *ch, char *argument)
{
    int chance = 0, race_chk;

	if ( IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_BERSERK) )
	return;

    if ( !IS_NPC(ch) )
    {
		if ( get_skill( ch, gsn_berserk ) <= 0 )
	    {
		stc("Huh?\n\r", ch );
		return;
	    }
	    else
	    	chance = get_skill( ch, gsn_berserk );
	}

    if (IS_AFFECTED(ch,AFF_BERSERK) || is_affected(ch,gsn_berserk)
    ||  is_affected(ch,skill_lookup("frenzy")))
    {
	stc("{cYou get a little madder.\n\r",ch);
	return;
    }

    if (IS_AFFECTED(ch,AFF_CALM))
    {
	stc("{cYou're feeling to mellow to berserk.\n\r",ch);
	return;
    }

    if (ch->stam < 50)
    {
	stc("{cYou can't get up enough energy.\n\r",ch);
	return;
    }

    /* modifiers */

    /* fighting */
    if (is_fighting(ch))
	chance += 10;

    if (number_percent() < chance)
    {
	AFFECT_DATA af;

	WAIT_STATE(ch,PULSE_VIOLENCE);
	/*ch->mana -= 50;*/

	race_chk = ch->level;
	if ((ch->race == race_lookup( "trolloc")
	|| ch->race == race_lookup( "ogier")
	|| !IS_SET(ch->op_flags,OP_CHANNELER)))
		race_chk += 2.5;

	stc("{cYour pulse races as you are consumed by {Rrage{c!\n\r",ch);
	act("{c$n gets a wild look in $s eyes.",ch,NULL,NULL,TO_ROOM);
	check_improve(ch,gsn_berserk,TRUE,22);

	af.where	= TO_AFFECTS;
	af.type		= gsn_berserk;
	af.level	= ch->level;
	af.duration	= number_fuzzy(race_chk / 10);
	af.modifier	= UMAX(1,ch->level/8);
	af.bitvector 	= AFF_BERSERK;

	/*af.location	= APPLY_HITROLL;
	affect_to_char(ch,&af);*/

	af.location	= APPLY_DAMROLL;
	affect_to_char(ch,&af);
    }

    else
    {
	WAIT_STATE(ch,3 * PULSE_VIOLENCE);
	ch->stam -= 25;

	stc("{cYour pulse speeds up, but nothing happens.\n\r",ch);
	check_improve(ch,gsn_berserk,FALSE,12);
    }
}

void do_bash( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance = 0;

    one_argument(argument,arg);

    /*if ( (chance = get_skill(ch,gsn_bash)) == 0
    ||	 (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_BASH))
    ||	 (!IS_NPC(ch)
    &&	  ch->level < skill_table[gsn_bash].skill_level[ch->class]))
    {
	stc("{cBashing? What's that?\n\r",ch);
	return;
    }*/

	if ( IS_NPC( ch ) && !IS_SET(ch->off_flags,OFF_BASH ) )
		return;
	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_bash ) <= 0 )
	    {
			stc("You dont know how to bash.\n\r",ch);
			return;
	    }
	    else
	    	chance = get_skill( ch, gsn_bash );
	}
	if (!is_fighting(ch))
	{
		stc("You're not fighting anyone.\n\r",ch);
		return;
	}
    if (arg[0] == '\0')
    {
	victim = ch->advancing_on;
	if (victim == NULL) /*should be impossible - Anthony*/
	{
	    stc("{cBut you aren't fighting anyone!\n\r",ch);
	    return;
	}
    }

    else if ((victim = get_char_room(ch,NULL,arg)) == NULL)
    {
	stc("{cThey aren't here.\n\r",ch);
	return;
    }

    if (victim->position < POS_FIGHTING)
    {
	act("{cYou'll have to let $M get back up first.",ch,NULL,victim,TO_CHAR);
	return;
    }

    if (victim == ch)
    {
	stc("{cYou try to bash your brains out, but fail.\n\r",ch);
	return;
    }

    if (is_safe(ch,victim))
	return;

    if ( IS_NPC(victim) &&
	is_fighting(victim) &&
	!is_same_group(ch,victim->advancing_on))
    {
        stc("{cKill stealing is not permitted.\n\r",ch);
        return;
    }
    if ( ch->range < RANGE_MELEE )
    {
	stc( "{cYou cannot hit them, they are not in melee.\n\r", ch );
	return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
	act("{cBut $N is your friend!",ch,NULL,victim,TO_CHAR);
	return;
    }

    /* modifiers */

    /* size  and weight */
    chance += ch->carry_weight / 250;
    chance -= victim->carry_weight / 200;

    if (ch->size < victim->size)
	chance += (ch->size - victim->size) * 15;
    else
	chance += (ch->size - victim->size) * 10;

	/* race */
	if (ch->race == race_lookup( "trolloc"))
		chance += 10;

    /* stats */
    chance += get_curr_stat(ch,STAT_STR);
    chance += get_curr_stat(ch,STAT_REF);
    chance /= 2;
    chance -= (get_curr_stat(victim,STAT_DEX) * 4)/3;
    chance -= GET_AC(victim,AC_BASH) /25;
    /* speed */
    if (IS_SET(ch->off_flags,OFF_FAST) || IS_AFFECTED(ch,AFF_HASTE))
        chance += 10;
    if (IS_SET(victim->off_flags,OFF_FAST) || IS_AFFECTED(victim,AFF_HASTE))
        chance -= 30;

    /* level */
    chance += (ch->level - victim->level);

    if (!IS_NPC(victim)
	&& chance < get_skill(victim,gsn_dodge) )
    {	/*
        act("{5$n tries to bash you, but you dodge it.{x",ch,NULL,victim,TO_VICT);
        act("{5$N dodges your bash, you fall flat on your face.{x",ch,NULL,victim,TO_CHAR);
        WAIT_STATE(ch,skill_table[gsn_bash].beats);
        return;*/
	chance -= 3 * (get_skill(victim,gsn_dodge) - chance);
    }

    /* now the attack */
    if (number_percent() < chance )
    {

	    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
	    {
			act("{c$n sends you sprawling with a powerful bash!",
				ch,NULL,victim,TO_VICT);
			act("{cYou slam into $N, and send $M flying!",ch,NULL,victim,TO_CHAR);
			act("{c$n sends $N sprawling with a powerful bash.",
				ch,NULL,victim,TO_NOTVICT);
			check_improve(ch,gsn_bash,TRUE,1);

			DAZE_STATE(victim, 3 * PULSE_VIOLENCE);
			WAIT_STATE(ch,skill_table[gsn_bash].beats);
			victim->position = POS_RESTING;
			damage(ch,victim,number_range(2,2 + 2 * ch->size + chance/20),gsn_bash,
			    DAM_BASH,FALSE);
		}
		else
		{
			act("{c$n sends you sprawling with a powerful bash!",
				ch,NULL,victim,TO_VICT);
			act("{cYou slam into $N, and send $M flying!",ch,NULL,victim,TO_CHAR);
			act("{c$n sends $N sprawling with a powerful bash.",
				ch,NULL,victim,TO_NOTVICT);
			check_improve(ch,gsn_bash,TRUE,1);

			WAIT_STATE(ch,skill_table[gsn_bash].beats);
			damage(ch,victim,number_range(2,ch->size + chance/20),gsn_bash,
			    DAM_BASH,FALSE);
		}

    }
    else
    {
	damage(ch,victim,0,gsn_bash,DAM_BASH,FALSE);
	act("{cYou fall flat on your face!",
	    ch,NULL,victim,TO_CHAR);
	act("{c$n falls flat on $s face.",
	    ch,NULL,victim,TO_NOTVICT);
	act("{cYou evade $n's bash, causing $m to fall flat on $s face.",
	    ch,NULL,victim,TO_VICT);
	check_improve(ch,gsn_bash,FALSE,1);
	ch->position = POS_RESTING;
	WAIT_STATE(ch,skill_table[gsn_bash].beats * 3/2);
    }
	check_killer(ch,victim);
}
void do_rub( CHAR_DATA * ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	int chance = 0;

	one_argument(argument,arg);

	if ( arg[0] == '\0' )
	{
		stc("Rub what?\n\r",ch);
		return;
	}
	if ( !str_cmp( arg, "eyes" ) )
	{
		AFFECT_DATA *paf, *paf_next;
		chance = 1;

		if (IS_AFFECTED(ch,AFF_BLIND))
		{
			/* modifiers for treat and bandage */
			if ( get_skill( ch, gsn_treat ) >= 1 )
				chance += get_skill( ch, gsn_treat ) / 4;
			if ( get_skill( ch, gsn_bandage ) >= 1 )
				chance += get_skill( ch, gsn_bandage ) / 4;
			if ( chance >= number_percent() )
			{
				act("$n rubs $s eyes, clearing $s vision.",ch,NULL,NULL,TO_ROOM);
				stc("You rub your eyes, clearing your vision.\n\r", ch);


				for (paf = ch->affected; paf != NULL; paf = paf_next)
				{
					paf_next = paf->next;

					if (paf->type == skill_lookup("plague")
					|| paf->type == skill_lookup("blindness")
					|| paf->type == skill_lookup("dirt"))
						affect_remove(ch, paf);
				}
				/*
				if ( IS_AFFECTED( ch, gsn_blindness ) || is_affected( ch, gsn_blindness ) )
					affect_strip( ch, gsn_blindness );
				if ( IS_AFFECTED( ch, gsn_dirt ) || is_affected( ch, gsn_dirt ) )
					affect_strip( ch, gsn_dirt );
				*/
				WAIT_STATE( ch, 4 );
			}
                        else
                        {
                                act("$n rubs $s eyes, trying to clear $s vision.",ch,NULL,NULL,TO_ROOM);
                                stc("You rub your eyes, trying to clear your vision, but it doesn't help.\n\r", ch);
				WAIT_STATE( ch, 4 );
                        }
		}
	}
	return;
}
void do_dirt( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance = 0;

    one_argument(argument,arg);

    /*if ( (chance = get_skill(ch,gsn_dirt)) == 0
    ||   (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_KICK_DIRT))
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_table[gsn_dirt].skill_level[ch->class]))
    {
	stc("{cYou get your feet dirty.\n\r",ch);
	return;
    }*/
	if ( IS_NPC( ch ) && !IS_SET(ch->off_flags,OFF_KICK_DIRT ) )
		return;
	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_dirt ) <= 0 )
	    {
			stc("You dont know how to kick dirt.\n\r",ch);
			return;
	    }
	    else
	    	chance = get_skill( ch, gsn_dirt );
	}

	if (!is_fighting(ch))
	{
		stc("You're not fighting anyone.\n\r",ch);
		return;
	}
    if ( ch->range < RANGE_MELEE )
    {
	stc( "{cYou cannot hit them , they are not in melee.\n\r", ch );
	return;
    }

    if (arg[0] == '\0')
    {
	victim = ch->advancing_on;
	if (victim == NULL)
	{
	    stc("{cBut you aren't in combat!\n\r",ch);
	    return;
	}
    }

    else if ((victim = get_char_room(ch,NULL,arg)) == NULL)
    {
	stc("{cThey aren't here.\n\r",ch);
	return;
    }

    if (IS_AFFECTED(victim,AFF_BLIND))
    {
	act("{c$E's already been blinded.",ch,NULL,victim,TO_CHAR);
	return;
    }

    if (victim == ch)
    {
	stc("{cVery funny.\n\r",ch);
	return;
    }

    if (is_safe(ch,victim))
	return;

    if (IS_NPC(victim) && !check_fighting(ch, victim))
    {
        stc("{cKill stealing is not permitted.\n\r",ch);
        return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
	act("{cBut $N is such a good friend!",ch,NULL,victim,TO_CHAR);
	return;
    }

    /* modifiers */

    /* dexterity */
    chance += get_curr_stat(ch,STAT_DEX);
    chance -= 2 * get_curr_stat(victim,STAT_DEX);

    /* speed  */
    if (IS_SET(ch->off_flags,OFF_FAST) || IS_AFFECTED(ch,AFF_HASTE))
	chance += 10;
    if (IS_SET(victim->off_flags,OFF_FAST) || IS_AFFECTED(victim,AFF_HASTE))
	chance -= 25;

    /* level */
    chance += (ch->level - victim->level) * 2;

    /* sloppy hack to prevent false zeroes */
    if (chance % 5 == 0)
	chance += 1;

    /* terrain */

    switch(ch->in_room->sector_type)
    {
	case(SECT_INSIDE):				chance -= 20;	break;
	case(SECT_CITY):				chance -= 10;	break;
	case(SECT_FIELD):				chance +=  5;	break;
	case(SECT_FOREST):				chance +=  2;   break;
	case(SECT_HILLS):				chance +=  1;   break;
	case(SECT_MOUNTAIN):			chance -= 10;	break;
	case(SECT_WATER_SWIM):			chance  =  0;	break;
	case(SECT_WATER_NOSWIM):		chance  =  0;	break;
	case(SECT_AIR):					chance  =  0;  	break;
    case(SECT_TUNDRA):              chance +=  5;   break;
    case(SECT_BLIGHT):              chance +=  5;   break;
    case(SECT_WASTE):               chance +=  5;   break;
    case(SECT_COASTAL):             chance +=  5;   break;
    case(SECT_ROAD):                chance -= 10;   break;
    case(SECT_MARSH):               chance +=  1;   break;
	case(SECT_DESERT):				chance += 10;   break;
    }

    if (chance == 0)
    {
	stc("{cThere isn't any dirt to kick.\n\r",ch);
	return;
    }

    /* now the attack */
    if (number_percent() < chance)
    {
	AFFECT_DATA af;
	act("{c$n is blinded by the dirt in $s eyes!",victim,NULL,NULL,TO_ROOM);
	act("{c$n kicks dirt in your eyes!",ch,NULL,victim,TO_VICT);
        damage(ch,victim,number_range(2,5),gsn_dirt,DAM_NONE,FALSE);
	stc("{cYou can't see a thing!\n\r",victim);
	check_improve(ch,gsn_dirt,TRUE,2);
	WAIT_STATE(ch,skill_table[gsn_dirt].beats);

	af.where	= TO_AFFECTS;
	af.type 	= gsn_dirt;
	af.level 	= ch->level;
	af.duration	= 0;
	af.location	= APPLY_HITROLL;
	af.modifier	= -4;
	af.bitvector 	= AFF_BLIND;

	affect_to_char(victim,&af);
    }
    else
    {
	damage(ch,victim,0,gsn_dirt,DAM_NONE,TRUE);
	check_improve(ch,gsn_dirt,FALSE,2);
	WAIT_STATE(ch,skill_table[gsn_dirt].beats);
    }
	check_killer(ch,victim);
}

void do_trip( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int chance = 0;

    one_argument(argument,arg);

    /*if ( (chance = get_skill(ch,gsn_trip)) == 0
    ||   (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_TRIP))
    ||   (!IS_NPC(ch)
	  && ch->level < skill_table[gsn_trip].skill_level[ch->class]))
    {
	stc("{cTripping?  What's that?\n\r",ch);
	return;
    }*/
	if ( IS_NPC( ch ) && !IS_SET(ch->off_flags,OFF_TRIP ) )
		return;
	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_trip ) <= 0 )
	    {
			stc("You dont know how to trip.\n\r",ch);
			return;
	    }
	    else
	    	chance = get_skill( ch, gsn_trip );
	}

	if (!is_fighting(ch))
	{
		stc("You're not fighting anyone.\n\r",ch);
		return;
	}

    if (arg[0] == '\0')
    {
	victim = ch->advancing_on;
	if (victim == NULL)
	{
		bugf("Yikes, trip to NULL victim after fight check :(");
	    stc("{cBut you aren't fighting anyone!\n\r",ch);
	    return;
 	}
    }

    else if ((victim = get_char_room(ch,NULL,arg)) == NULL)
    {
	stc("{cThey aren't here.\n\r",ch);
	return;
    }
    if ( ch->range < RANGE_MELEE )
    {
	stc( "{cYou cannot hit them , they are not in melee.\n\r", ch );
	return;
    }

    if (!check_fighting(ch,victim))
		return;

    if (IS_AFFECTED(victim,AFF_FLYING))
    {
	act("{c$S feet aren't on the ground.",ch,NULL,victim,TO_CHAR);
	return;
    }

    if (victim->position < POS_FIGHTING)
    {
	act("{c$N is already down.",ch,NULL,victim,TO_CHAR);
	return;
    }

    if (victim == ch)
    {
	stc("{cYou fall flat on your face!\n\r",ch);
	WAIT_STATE(ch,2 * skill_table[gsn_trip].beats);
	act("{c$n trips over $s own feet!",ch,NULL,NULL,TO_ROOM);
	return;
    }

    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
	act("{c$N is your beloved master.",ch,NULL,victim,TO_CHAR);
	return;
    }

    /* modifiers */

    /* size */
    if (ch->size < victim->size)
        chance += (ch->size - victim->size) * 10;  /* bigger = harder to trip */

    /* dex */
    chance += get_curr_stat(ch,STAT_DEX);
    chance -= get_curr_stat(victim,STAT_DEX) * 3 / 2;

    /* speed */
    if (IS_SET(ch->off_flags,OFF_FAST) || IS_AFFECTED(ch,AFF_HASTE))
	chance += 10;
    if (IS_SET(victim->off_flags,OFF_FAST) || IS_AFFECTED(victim,AFF_HASTE))
	chance -= 20;

    /* level */
    chance += (ch->level - victim->level) * 2;


    /* now the attack */
    if (number_percent() < chance)
    {
	act("{c$n trips you and you go down!",ch,NULL,victim,TO_VICT);
	act("{cYou trip $N and $N goes down!",ch,NULL,victim,TO_CHAR);
	act("{c$n trips $N, sending $M to the ground.",ch,NULL,victim,TO_NOTVICT);
	check_improve(ch,gsn_trip,TRUE,1);

	DAZE_STATE(victim,2 * PULSE_VIOLENCE);
        WAIT_STATE(ch,skill_table[gsn_trip].beats);
	victim->position = POS_RESTING;
	damage(ch,victim,number_range(2, 2 +  2 * victim->size),gsn_trip,
	    DAM_BASH,TRUE);
    }
    else
    {
	damage(ch,victim,0,gsn_trip,DAM_BASH,TRUE);
	WAIT_STATE(ch,skill_table[gsn_trip].beats*2/3);
	check_improve(ch,gsn_trip,FALSE,1);
    }
	check_killer(ch,victim);
}

void do_backstab( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    OBJ_DATA *obj2;
    int chance = 0;
    int check = 0;

    one_argument( argument, arg );

	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_backstab ) <= 0 )
	    {
			stc("You dont know how to backstab.\n\r",ch);
			return;
	    }
	}
    if (arg[0] == '\0')
    {
        stc("{cBackstab whom?\n\r",ch);
        return;
    }

    if ( ( victim = get_char_room( ch,NULL,arg ) ) == NULL )
    {
        stc("{cThey aren't here.\n\r",ch);
        return;
    }

    if (is_fighting(ch))
    {
	stc("{cYou're facing the wrong end.\n\r",ch);
	return;
    }

    if ( ch->in_room->area->file_name != NULL
    && !str_cmp( ch->in_room->area->file_name, "school.are" ) )
    {
		stc("It would be most cowardly to kill in such a way in this place.\n\r",ch);
		return;
    }

    if ( is_fighting(victim) )
    {
	stc( "They're busy with another fight and you can't kill steal.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	stc( "{cHow can you sneak up on yourself?\n\r", ch );
	return;
    }

    check = get_skill(ch,gsn_backstab);

	if (!IS_NPC(ch))
	{
		if (!IS_NPC(victim)
		&& !IS_SET(ch->in_room->room_flags,ROOM_PK)
		&& !IS_SET(victim->in_room->room_flags,ROOM_PK)
		&& !IS_IMMORTAL(ch))
		{
	    stc("{cGo to the Arena if you want to PK.\n\r",ch);
	    return;
		}
	}

	if ( IS_SET( ch->war_flags, WAR_BATTLE )
	|| IS_SET( ch->war_flags, WAR_TOURNEY )
	|| IS_SET( victim->war_flags, WAR_BATTLE )
	|| IS_SET( victim->war_flags, WAR_TOURNEY ) )
	{
		stc( "That would be most dishonorable.\n\r", ch );
		return;
	}

    if ( ( obj = get_eq_char( ch, WEAR_WIELD ) ) == NULL)
    {
	stc( "{cYou need to wield a weapon to backstab.\n\r", ch );
	return;
    }

    if ( victim->hit < victim->max_hit)
    {
	act( "{c$N is hurt and suspicious ... you can't sneak up.",
	    ch, NULL, victim, TO_CHAR );
	return;
    }

	if (IS_SET ( victim->act, ACT_AGGRESSIVE ))
	{
		start_first_fight(ch, victim, RANGE_MELEE);
		return;
	}

	if (!start_first_fight(ch, victim, RANGE_MELEE))
		return;

	ptc( ch, "You sneak up on %s, preparing to backstab them.\n\r",
	IS_NPC(victim) ? victim->short_descr : victim->name );

	WAIT_STATE( ch, PULSE_VIOLENCE / 2 );
	chance = number_percent();
	if ( chance < get_skill(ch,gsn_backstab)
	|| ( get_skill(ch,gsn_backstab) >= 2 && !IS_AWAKE(victim) ) )
	{
		check_improve(ch,gsn_backstab,TRUE,5);
		one_hit( ch, victim, gsn_backstab, TRUE );
	    if ( ( obj2 = get_eq_char( ch, WEAR_SECONDARY ) ) != NULL)
	    {
		    if ( number_percent( ) < get_skill(ch,gsn_backstab)
		    || ( get_skill(ch,gsn_backstab) >= 2 && !IS_AWAKE(victim) ) )
		    {
				check_improve(ch,gsn_backstab,TRUE,3);
				one_hit( ch, victim, gsn_backstab, TRUE );
			}
	    }
	}
	else
    {
		check_improve(ch,gsn_backstab,FALSE,1);
		damage( ch, victim, 0, gsn_backstab,DAM_NONE,TRUE);
    }
    return;
}

void do_rescue( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    CHAR_DATA *fch;

    one_argument( argument, arg );

	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_rescue ) <= 0 )
	    {
			stc("You dont know how to rescue.\n\r",ch);
			return;
	    }
	}

    if ( arg[0] == '\0' )
    {
	stc( "{cRescue whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch,NULL, arg ) ) == NULL )
    {
	stc( "{cThey aren't here.\n\r", ch );
	return;
    }

    if ( victim == ch )
    {
	stc( "{cWhat about fleeing instead?\n\r", ch );
	return;
    }

    if ( !IS_NPC(ch) && IS_NPC(victim) )
    {
	stc( "{cDoesn't need your help!\n\r", ch );
	return;
    }

    if ( check_fighting(ch,victim))
    {
	stc( "{cToo late.\n\r", ch );
	return;
    }

    if ( ( fch = victim->advancing_on ) == NULL )
    {
	stc( "{cThat person is not fighting right now.\n\r", ch );
	return;
    }

    if ( IS_NPC(fch) && !is_same_group(ch,victim))
    {
        stc("{cKill stealing is not permitted.\n\r",ch);
        return;
    }

    WAIT_STATE( ch, skill_table[gsn_rescue].beats );
    if ( number_percent( ) >
    (get_skill(ch,gsn_rescue) + ch->mod_stat[STAT_REF] + ch->mod_stat[STAT_INS]) / 3 )
    {
		if ( get_skill(ch,gsn_advanced_rescue ) > 0 )
		{
			if ( number_percent() > ( get_skill(ch,gsn_rescue) + get_skill(ch,gsn_rescue) ) )
			{
			stc( "{cYou fail the rescue.\n\r", ch );
			check_improve(ch,gsn_advanced_rescue,FALSE,1);
			return;
			}
		}
		else
		{
			stc( "{cYou fail the rescue.\n\r", ch );
			check_improve(ch,gsn_rescue,FALSE,1);
			return;
		}
    }

    act( "{cYou rescue $N!",  ch, NULL, victim, TO_CHAR    );
    act( "{c$n rescues you!", ch, NULL, victim, TO_VICT    );
    act( "{c$n rescues $N!",  ch, NULL, victim, TO_NOTVICT );
    check_improve(ch,gsn_rescue,TRUE,1);

    /*stop_fighting( fch, FALSE );*/
    /*stop_fighting( victim, fch );*/
    start_extra_fight(ch, fch, RANGE_MELEE);

    check_killer( ch, fch );
    /*set_fighting( ch, fch );
    set_fighting( fch, ch );*/
    fch->advancing_on = ch;
    return;
}

void do_disarm( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    int chance,hth,ch_weapon,vict_weapon,ch_vict_weapon;

    hth = 0;

    if ((chance = get_skill(ch,gsn_disarm)) == 0)
    {
	stc( "{cYou don't know how to disarm opponents.\n\r", ch );
	return;
    }

    if ( get_eq_char( ch, WEAR_WIELD ) == NULL
    &&   ((hth = get_skill(ch,gsn_hand_to_hand)) == 0
    ||    (IS_NPC(ch) && !IS_SET(ch->off_flags,OFF_DISARM))))
    {
	stc( "{cYou must wield a weapon to disarm.\n\r", ch );
	return;
    }

    if (!is_fighting(ch) ||  ( victim = ch->advancing_on ) == NULL )
    {
	stc( "{cYou aren't fighting anyone.\n\r", ch );
	return;
    }
    if ( ch->range < RANGE_MELEE )
    {
	stc( "{cYou cannot hit them , they are not in melee.\n\r", ch );
	return;
    }

    if ( ( obj = get_eq_char( victim, WEAR_WIELD ) ) == NULL )
    {
	stc( "{cYour opponent is not wielding a weapon.\n\r", ch );
	return;
    }

    /* find weapon skills */
    ch_weapon = get_weapon_skill(ch,get_weapon_sn(ch));
    vict_weapon = get_weapon_skill(victim,get_weapon_sn(victim));
    ch_vict_weapon = get_weapon_skill(ch,get_weapon_sn(victim));

    /* modifiers */

    /* skill */
    if ( get_eq_char(ch,WEAR_WIELD) == NULL)
	chance = chance * hth/150;
    else
	chance = chance * ch_weapon/100;

    chance += (ch_vict_weapon/2 - vict_weapon) / 2;

    /* dex vs. strength */
    chance += get_curr_stat(ch,STAT_DEX);
    chance += get_curr_stat(ch,STAT_REF);
    chance -= 2 * get_curr_stat(victim,STAT_STR);
    chance -= 2 * get_curr_stat(victim,STAT_REF);

    /* level */
    chance += (ch->level - victim->level) * 2;

    /* and now the attack */
    if (number_percent() < chance)
    {
    	WAIT_STATE( ch, skill_table[gsn_disarm].beats );
	disarm( ch, victim );
	check_improve(ch,gsn_disarm,TRUE,1);
    }
    else
    {
	WAIT_STATE(ch,skill_table[gsn_disarm].beats);
	act("{cYou fail to disarm $N.",ch,NULL,victim,TO_CHAR);
	act("{c$n tries to disarm you, but fails.",ch,NULL,victim,TO_VICT);
	act("{c$n tries to disarm $N, but fails.",ch,NULL,victim,TO_NOTVICT);
	check_improve(ch,gsn_disarm,FALSE,1);
    }
    check_killer(ch,victim);/*QUESTION: Anthony - why are all these in functions were the folks are already fighting??*/
    return;
}


/* Function: do_teach, by Sedron
 * Purpose: Teaching Skills or Weaves to other characters.
 * Called from: Skills.c
 * Extra: Add to interp.c, do_fun
 */

DECLARE_SPELL_FUN( spell_null );

void do_teach_old (CHAR_DATA *ch, char *argument)
{
	char tSkill[MAX_STRING_LENGTH];
	char tWho[MAX_STRING_LENGTH];
	CHAR_DATA *student;
	int sn=0;
	int chance;
    int skReq = 0;
    bool canGain = FALSE, skillReq = FALSE;
    int req_earth, req_air, req_fire, req_water, req_spirit;

    req_earth = 0;
    req_air = 0;
    req_fire = 0;
    req_water = 0;
    req_spirit = 0;

	argument = one_argument (argument, tSkill);
	argument = one_argument (argument, tWho);

	if ( IS_NPC( ch ) )
		return;

	if (tSkill[0] == '\0' || tWho[0] == '\0'){
		stc ("{cTeach what to whom?\n\r", ch);
		stc ("{cUsage TEACH skill Character.\n\r", ch);
		return;
	}

	if ( get_skill( ch, gsn_teach ) <= 0 ) {
		stc ("{cYou dont know how to teach!\n\r", ch);
		return;
	}
	if ((student = get_char_room(ch,NULL,tWho)) == NULL) {
		stc ("{cThey aren't in the room.\n\r", ch);
		return;
	}
	if (IS_NPC(student)){
		stc ("{cYou can't teach npc's.\n\r", ch);
		return;
	}
	if ((sn = skill_lookup (tSkill)) < 0){
		stc ("{cWhich skill?\n\r", ch);
		return;
	}
	if (ch->pcdata->learned[sn] < 60) {
		stc ("{cYou don't know enough about it to teach.\n\r", ch);
		return;
	}
	if (IS_SET(student->spec_flags, SPEC_NOTEACH ) ) {
		act ("{c$N is not listening to you.", ch, NULL, student, TO_CHAR);
		return;
	}
	if (student->pcdata->learned[sn]) {
		act ("{cThat skill is nothing new to $N.", ch, NULL, student, TO_CHAR);
		return;
	}
	if (skill_table[sn].rating[student->class] <= 0) {
		act("{cThat skill is beyond $N's ability.",ch,NULL,student,TO_CHAR);
		return;
	}


	if ( skill_table[sn].req_skill == NULL
	|| skill_table[sn].req_skill == 0 )
	{
		skillReq = FALSE;
		canGain = TRUE;
	}
	if ( skill_table[sn].req_skill != NULL && !canGain )
	{
		skillReq = TRUE;
		if ( skill_table[sn].req_skill > 0 || skill_table[sn].req_skill != NULL )
		{
			skReq = skill_lookup(skill_table[sn].req_skill);
			canGain = TRUE;
		}
		if ( !str_cmp( skill_table[sn].req_skill, "reserved" ) )
			canGain = TRUE;
	}
	if ( skillReq )
	{
		if ( get_skill( student, skill_lookup( skill_table[sn].req_skill ) ) < 85 )
		{
			ptc(student,"The %s skill is required at 85% before you can learn %s.\n\r",
				skill_table[skReq].name, skill_table[sn].name );
			return;
		}
	}

	if ( skill_table[sn].spell_fun != spell_null && !IS_SET( student->op_flags, OP_CHANNELER ) )
	{
		stc("You cannot gain this weave because you are not a channeler.\n\r", student );
		stc("They cannot gain this weave because they are not a channeler.\n\r", ch );
		return;
	}

	if ( skill_table[sn].spell_fun != spell_null )
	{
		if ( skill_table[sn].req_weave == NULL )
		{
			skillReq = FALSE;
			canGain = TRUE;
		}
	}

	if (student->train < skill_table[sn].rating[student->class])
	{
		act("{cBy the confusion on $N's face, you understand that $E must train more for that skill.",
			ch,NULL,student,TO_CHAR);
		return;
	}
	if ( student->pcdata->rebirth < skill_table[sn].remort_only )
	{
		stc("Your student has not attained the proper status for this skill.\n\r",ch);
		return;
	}
	if (skill_table[sn].spell_fun != spell_null
	&& student->pcdata->true_sex != ch->pcdata->true_sex && !IS_IMMORTAL(ch))
	{
		stc("That would be like a bird trying to teach a fish to fly.\n\r",ch);
		return;
	}

	req_earth = skill_table[sn].sphere_earth;
	req_air = skill_table[sn].sphere_air;
	req_fire = skill_table[sn].sphere_fire;
	req_water = skill_table[sn].sphere_water;
	req_spirit = skill_table[sn].sphere_spirit;

	if ( student->pcdata->op_earth < req_earth )
	{
		stc("Your student does not have the strength necessary for this weave.\n\r", ch);
		stc("You do not possess the required strength in Earth for this weave.\n\r", student);
		return;
	}
	if ( student->pcdata->op_air < req_air )
	{
		stc("Your student does not have the strength necessary for this weave.\n\r", ch);
		stc("You do not possess the required strength in Air for this weave.\n\r", student);
		return;
	}
	if ( student->pcdata->op_fire < req_fire )
	{
		stc("Your student does not have the strength necessary for this weave.\n\r", ch);
		stc("You do not possess the required strength in Fire for this weave.\n\r", student);
		return;
	}
	if ( student->pcdata->op_water < req_water )
	{
		stc("Your student does not have the strength necessary for this weave.\n\r", ch);
		stc("You do not possess the required strength in Water for this weave.\n\r", student);
		return;
	}
	if ( student->pcdata->op_spirit < req_spirit )
	{
		stc("Your student does not have the strength necessary for this weave.\n\r", ch);
		stc("You do not possess the required strength in Spirit for this weave.\n\r", student);
		return;
	}

	chance = (get_skill (ch, gsn_teach) >0 ) ? get_skill (ch, gsn_teach) /4 : 5;
	chance += get_curr_stat (student, STAT_WIS)/2;
	chance += get_curr_stat (student, STAT_INT)/2;
	chance += get_curr_stat (ch, STAT_CHA)/2;
	chance /= 2;
	chance += (100 - ch->pcdata->learned[sn])/2;

	WAIT_STATE (ch, 12);
	WAIT_STATE (student, 12);

	if (number_percent() < chance){
		student->pcdata->learned[sn] = 1;
		act("{c$n trains you in the art of $t.", ch,skill_table[sn].name,student,TO_VICT);
		student->train -= skill_table[sn].rating[ch->class];
		act("{cYou teach $N the art of $t.", ch, skill_table[sn].name, student, TO_CHAR);
		if (get_skill (ch, gsn_teach) > 0)
			check_improve (ch, gsn_teach, TRUE, 5);
		return;
	}
	else {
		act ("{cAfter pouring your soul out, $N doesn't seem to catch on to $t.",
			ch, skill_table[sn].name, student, TO_CHAR);
		act ("{cFor all of $n's ranting and raving you make no sense out of $s lesson.",
			ch, NULL, student, TO_VICT);
		if (get_skill (ch, gsn_teach) > 0)
			check_improve (ch, gsn_teach, FALSE, 5);
		return;
	}
}

/* for wielding a secondary weapon - Dal - original by Erwin */
void do_second (CHAR_DATA *ch, char *argument)
{
    OBJ_DATA *obj;
	int sn,skill,chance;
    sn = get_weapon_sn(ch);
    skill = get_weapon_skill(ch,sn);

    if ((chance = get_skill(ch,gsn_dual)) == 0)
    {
	stc("{cYou are not skilled at dual weaponry.\n\r",ch);
	return;
    }

    if (argument[0] == '\0')
    {
        stc( "{cWield what weapon in your off-hand?\n\r",ch);
        return;
    }
    obj = get_obj_carry (ch, argument, ch);
    if (obj == NULL)
    {
        stc( "{cYou have no such thing in your backpack.\n\r",ch);
        return;
    }
    if (obj->item_type != ITEM_WEAPON)
    {
        stc( "{cYou can only wield weapons.\n\r",ch);
        return;
    }
    if (get_eq_char (ch,WEAR_HOLD) != NULL)
    {
        stc( "{cYou cannot use a secondary weapon while holding an item.\n\r",ch);
        return;
    }
    if (get_eq_char (ch,WEAR_SHIELD) != NULL)
    {
        stc( "{cYou cannot use a shield while holding an item.\n\r",ch);
        return;
    }
    /*if (ch->level < obj->level)
    {
        sprintf( buf, "{cYou must be level %d to use this object.\n\r", obj->level);
    stc(buf, ch);
    act( "{c$n tries to use $p, but is too inexperienced.", ch, obj, NULL, TO_ROOM);
    return;
    }*/

    if (IS_SET(obj->extra_flags,ITEM_REMORT)
    && ch->pcdata->rebirth < 1)
    {
		act("You are not powerful enough to get that.",ch,0,0,TO_CHAR);
		return;
	}

    if (!is_obj_char_clan(ch,obj) && is_obj_clan(obj))
    {
		stc("You can't wear items that dont belong to your Guild.\n\r",ch);
		return;
	}

    if (get_eq_char (ch, WEAR_WIELD) == NULL)
    {
      stc( "{cYou need to wield a primary weapon before wielding a secondary weapon.\n\r",ch);
      return;
    }

    if ( get_obj_weight(obj) > ( get_curr_stat(ch,STAT_STR) + 25 ) )
    {
        stc( "{cYou aren't strong enough to wield this as a secondary weapon.\n\r",ch);
        return;
    }

    if (!remove_obj(ch, WEAR_SECONDARY, TRUE))
      return;

    act( "{c$n wields $p in $s off-hand.",ch,obj,NULL,TO_ROOM);
    act( "{cYou wield $p in your off-hand.",ch,obj,NULL,TO_CHAR);

    if (skill >= 100)
        act( "{c$p feels like a part of you!",ch,obj,NULL,TO_CHAR);
    else if (skill > 85)
        act( "{cYou feel quite confident with $p.",ch,obj,NULL,TO_CHAR);
    else if (skill > 70)
        act( "{cYou are skilled with $p.",ch,obj,NULL,TO_CHAR);
    else if (skill > 50)
        act( "{cYour skill with $p is adequate.",ch,obj,NULL,TO_CHAR);
    else if (skill > 25)
        act( "{c$p feels a little clumsy in your hands.",ch,obj,NULL,TO_CHAR);
    else if (skill > 1)
        act( "{cYou fumble and almost drop $p.",ch,obj,NULL,TO_CHAR);
    else
        act( "{cYou don't even know which end is up on $p.",ch,obj,NULL,TO_CHAR);

    equip_char (ch, obj, WEAR_SECONDARY);
    return;
}


/* for poisoning weapons and food/drink */
void do_envenom(CHAR_DATA *ch, char *argument)
{
    OBJ_DATA *obj;
    AFFECT_DATA af;
    int percent,skill;

    /* find out what */
    if (argument[0] == '\0')
    {
	stc( "{cEnvenom what item?\n\r",ch);
	return;
    }

    obj =  get_obj_list(ch,argument,ch->carrying);

    if (obj== NULL)
    {
	stc( "{cYou don't have that item.\n\r",ch);
	return;
    }

    if ((skill = get_skill(ch,gsn_envenom)) < 1)
    {
	stc( "You'd poison yourself!\n\r",ch);
	return;
    }

    if (obj->item_type == ITEM_FOOD || obj->item_type == ITEM_DRINK_CON)
    {
	if (IS_OBJ_STAT(obj,ITEM_BURN_PROOF))
	{
	    act( "{cYou fail to poison $p.",ch,obj,NULL,TO_CHAR);
	    return;
	}

	if (number_percent() < skill)  /* success! */
	{
	    act( "{c$n treats $p with deadly poison.",ch,obj,NULL,TO_ROOM);
	    act( "{cYou treat $p with deadly poison.",ch,obj,NULL,TO_CHAR);
	    if (!obj->value[3])
	    {
		obj->value[3] = 1;
		check_improve(ch,gsn_envenom,TRUE,4);
	    }
	    WAIT_STATE(ch,skill_table[gsn_envenom].beats);
	    return;
	}

	act( "{cYou fail to poison $p.",ch,obj,NULL,TO_CHAR);
	if (!obj->value[3])
	    check_improve(ch,gsn_envenom,FALSE,4);
	WAIT_STATE(ch,skill_table[gsn_envenom].beats);
	return;
     }

    if (obj->item_type == ITEM_WEAPON)
    {
        if (IS_WEAPON_STAT(obj,WEAPON_FLAMING)
        ||  IS_WEAPON_STAT(obj,WEAPON_FROST)
        ||  IS_WEAPON_STAT(obj,WEAPON_VAMPIRIC)
        ||  IS_WEAPON_STAT(obj,WEAPON_SHARP)
        ||  IS_WEAPON_STAT(obj,WEAPON_VORPAL)
        ||  IS_WEAPON_STAT(obj,WEAPON_SHOCKING)
        ||  IS_OBJ_STAT(obj,ITEM_BURN_PROOF))
        {
            act( "{cYou can't seem to envenom $p.",ch,obj,NULL,TO_CHAR);
            return;
        }
	if(obj->affected)  /* fwrite_obj  can only seem to handle one affect */
        {
            act( "{cYou can't seem to envenom $p.",ch,obj,NULL,TO_CHAR);
            return;
        }

	if (obj->value[3] < 0
	||  attack_table[obj->value[3]].damage == DAM_BASH)
	{
	    stc( "{cYou can only envenom edged weapons.\n\r",ch);
	    return;
	}

        if (IS_WEAPON_STAT(obj,WEAPON_POISON))
        {
            act( "{c$p is already envenomed.",ch,obj,NULL,TO_CHAR);
            return;
        }

	percent = number_percent();
	if (percent < skill)
	{

            af.where     = TO_WEAPON;
            af.type      = gsn_poison;
            af.level     = ch->level * percent / 100;
            af.duration  = ch->level/2 * percent / 100;
            af.location  = 0;
            af.modifier  = 0;
            af.bitvector = WEAPON_POISON;
            affect_to_obj(obj,&af);

            act( "{c$n coats $p with deadly venom.",ch,obj,NULL,TO_ROOM);
	    act( "{cYou coat $p with venom.",ch,obj,NULL,TO_CHAR);
	    check_improve(ch,gsn_envenom,TRUE,3);
	    WAIT_STATE(ch,skill_table[gsn_envenom].beats);
            return;
        }
	else
	{
	    act( "{cYou fail to envenom $p.",ch,obj,NULL,TO_CHAR);
	    check_improve(ch,gsn_envenom,FALSE,3);
	    WAIT_STATE(ch,skill_table[gsn_envenom].beats);
	    return;
	}
    }

    act( "{cYou can't poison $p.",ch,obj,NULL,TO_CHAR);
    return;
}
void do_butcher(CHAR_DATA *ch, char *argument)
{

    /* Butcher skill, created by Argawal */
    /* Original Idea taken fom Carrion Fields Mud */
    /* If you have an interest in this skill, feel free */
    /* to use it in your mud if you so desire. */
    /* All I ask is that Argawal is credited with creating */
    /* this skill, as I wrote it from scratch. */

    char buf[MAX_STRING_LENGTH];
    char arg[MAX_STRING_LENGTH];
    OBJ_DATA *steak;
    OBJ_DATA *obj;

    one_argument(argument, arg);

    if(get_skill(ch,gsn_butcher)==0)
    {
       stc( "{cButchering is beyond your skills.\n\r",ch);
       return;
    }
	if (IS_SET(ch->in_room->room_flags,ROOM_SAFE))
	{
       stc( "{cNot in this room.\n\r",ch);
       return;
    }
    if(arg[0]=='\0')
    {
       stc( "{cButcher what?\n\r",ch);
       return;
    }

    obj = get_obj_list( ch, arg, ch->in_room->contents );
    if ( obj == NULL )
    {
        stc( "{cIt's not here.\n\r", ch );
        return;
    }

    if( (obj->item_type != ITEM_CORPSE_NPC)
        && (obj->item_type!=ITEM_CORPSE_PC) )
    {
        stc( "{cYou can only butcher corpses.\n\r", ch );
        return;
    }

    /* create and rename the steak */
    buf[0]='\0';
    strcat(buf,"A steak of ");
    strcat(buf,str_dup(obj->short_descr));
    strcat(buf," is here.");

    steak = create_object( get_obj_index( OBJ_VNUM_STEAK ), 0 );

    steak->description=str_dup(buf);
    steak->value[0] = ch->level / 2;
    steak->value[1] = ch->level;

    buf[0]='\0';
    strcat(buf,"A steak of ");
    strcat(buf,str_dup(obj->short_descr));

    steak->short_descr=str_dup(buf);

    if(number_percent( ) < (get_skill(ch,gsn_butcher) + ch->mod_stat[STAT_REF]) / 3)
    {
    steak = create_object( get_obj_index( OBJ_VNUM_STEAK ), 0 );
    steak->value[0] = ch->level / 2;
    steak->value[1] = ch->level / 2;
    steak->timer = number_range(ch->level, 1000);
    steak->weight = 20;
    obj_to_char( steak, ch );
    act( "{c$n butchers a corpse and creates a steak.", ch, steak, NULL, TO_ROOM );
    act( "{cYou butcher a corpse and create a steak.", ch, steak, NULL, TO_CHAR );
    check_improve(ch,gsn_butcher,TRUE,3);
    }
    else
    {
    act( "{c$n fails to butcher a corpse, and destroys it.", ch, steak, NULL, TO_ROOM );
    act( "{cYou fail to butcher a corpse, and destroy it.", ch, steak, NULL, TO_CHAR );
    check_improve(ch,gsn_butcher,FALSE,1);
    }
    if ( obj->item_type == ITEM_CORPSE_PC )
    {
        OBJ_DATA *t_obj, *next_obj;
        for (t_obj = obj->contains; t_obj != NULL; t_obj = next_obj)
        {
            next_obj = t_obj->next_content;
            obj_from_obj(t_obj);
            if (obj->in_obj) /* in another object */
                obj_to_obj(t_obj,obj->in_obj);
            else if (obj->carried_by) /* carried */
                if (obj->wear_loc == WEAR_FLOAT)
                    if (obj->carried_by->in_room == NULL)
                        extract_obj(t_obj);
                    else
                        obj_to_room(t_obj,obj->carried_by->in_room);
                else
                    obj_to_char(t_obj,obj->carried_by);
           else if (obj->in_room == NULL) /* destroy it */
                extract_obj(t_obj);
            else /* to a room */
                obj_to_room(t_obj,obj->in_room);
        }
    }

    if ( obj->item_type == ITEM_CORPSE_NPC )
    {
        OBJ_DATA *t_obj, *next_obj;
        for (t_obj = obj->contains; t_obj != NULL; t_obj = next_obj)
        {
            next_obj = t_obj->next_content;
            obj_from_obj(t_obj);
            if (obj->in_obj) /* in another object */
                obj_to_obj(t_obj,obj->in_obj);
            else if (obj->carried_by) /* carried */
                if (obj->wear_loc == WEAR_FLOAT)
                    if (obj->carried_by->in_room == NULL)
                        extract_obj(t_obj);
                    else
                        obj_to_room(t_obj,obj->carried_by->in_room);
                else
                    obj_to_char(t_obj,obj->carried_by);
            else if (obj->in_room == NULL) /* destroy it */
                extract_obj(t_obj);
            else /* to a room */
                obj_to_room(t_obj,obj->in_room);
         }
     }

    /* Now remove the corpse */
    extract_obj(obj);
    return;
}

void do_skin(CHAR_DATA *ch, char *argument)
{

    /* Butcher skill, created by Argawal */
    /* Original Idea taken fom Carrion Fields Mud */
    /* If you have an interest in this skill, feel free */
    /* to use it in your mud if you so desire. */
    /* All I ask is that Argawal is credited with creating */
    /* this skill, as I wrote it from scratch. */
    /* Enhanced by Dal of As the Wheel Weaves */
    /* Completely rewritten to be used with crafting system by Rokann of The Shadows Embrace */
    char arg[MAX_STRING_LENGTH];
    OBJ_DATA *obj;

    one_argument(argument, arg);

	if ( IS_NPC( ch ) )
	return;

    if(get_skill(ch,gsn_skin)==0)
    {
       stc( "{cSkinning is beyond your skills.\n\r",ch);
       return;
    }
	if (IS_SET(ch->in_room->room_flags,ROOM_SAFE))
	{
       stc( "{cNot in this room.\n\r",ch);
       return;
    }
    if(arg[0]=='\0')
    {
       stc( "{cSkin what?\n\r",ch);
       return;
    }

    obj = get_obj_list( ch, arg, ch->in_room->contents );
    if ( obj == NULL )
    {
        stc( "{cIt's not here.\n\r", ch );
        return;
    }

    skin(ch, obj, FALSE);
    return;
}


/*void do_stitch(CHAR_DATA *ch, char *argument)
{
    char buf[MSL];
    char arg[MSL];
    char arg2[MSL];
    char arg3[MSL];
    char short_buf[MSL];
    OBJ_DATA *obj = NULL;
    OBJ_DATA *obj2 = NULL;
    AFFECT_DATA *af = new_affect();
	bool foundSkin, foundCloth, skin, cloth;
	int chance, special, specChance, lvl, ac;
	char *objBuf;
	chance = 0;
	special = 1;
	ac = 0;

    argument = one_argument(argument, arg);
    argument = one_argument(argument, arg2);
    argument = one_argument(argument, arg3);

    short_buf[0] = '\0';

    if ( IS_NPC( ch ) )
    return;

	lvl = number_range( 1, ch->level );

    if ( get_skill (ch,gsn_stitch) == 0 )
    {
       stc( "{cStitching is beyond your skills.\n\r",ch);
       return;
    }

	chance = ch->pcdata->learned[gsn_stitch];
	ptc( ch, "Your chance to stitch is %d.\n\r", chance );
	foundSkin = FALSE;
	foundCloth = FALSE;
	skin = FALSE;
	cloth = FALSE;

    if ( arg[0] == '\0'
    || arg2[0] == '\0'
    || arg3[0] == '\0' )
    {
        stc( "{cWhat do you want to stitch?\n\r", ch );
        stc( "{c<cloak boots belt gloves pants shirt>\n\r", ch );
        stc( "Syntax:\n\r", ch );
        stc( "stitch skin into <type>\n\r", ch );
        stc( "stitch cloth into <type>\n\r", ch );
        return;
    }
    if ( strcmp( arg2, "into" ) )
    {
        stc( "Missing 'into'. What do you want to stitch?\n\r", ch );
        stc( "<cloak boots belt gloves pants shirt>\n\r", ch );
        stc( "Syntax:\n\r", ch );
        stc( "stitch skin into <type>\n\r", ch );
        stc( "stitch cloth into <type>\n\r", ch );
        return;
    }

	if ( !str_cmp( arg, "skin" ) )
	{
	    skin = TRUE;
		for ( obj = ch->carrying; obj != NULL; obj = obj->next )
		{
			if (obj->pIndexData->vnum == OBJ_VNUM_SKIN)
			{
		    foundSkin = TRUE;
		    ptc(ch,"The OBJ is %s\n\r",obj->short_descr);
		    break;
			}
		}
	}
	else if ( !str_cmp( arg, "cloth" ) )
	{
	    cloth = TRUE;
		for ( obj2 = ch->carrying; obj2 != NULL; obj2 = obj2->next )
		{
			if (obj2->item_type == ITEM_CLOTH_BOLT)
			{
		    foundCloth = TRUE;
		    break;
			}
		}
	}
	else
    {
        stc( "{cWhat do you want to create?\n\r", ch );
        stc( "{c<cloak boots belt gloves pants shirt>\n\r", ch );
        stc( "Syntax:\n\r", ch );
        stc( "stitch skin into <type>\n\r", ch );
        stc( "stitch cloth into <type>\n\r", ch );
        return;
    }

    if ( skin && !foundSkin )
    {
        stc( "Could not find a valid skin material in your inventory.\n\r", ch );
        return;
    }
    if ( cloth && !foundCloth )
    {
        stc( "Could not find a valid cloth material in your inventory.\n\r", ch );
        return;
    }

	if ( obj )
	{
		if ( obj->timer > 0 )
			obj->timer = 0;
	}
	if ( obj2 )
	{
		if ( obj2->timer > 0 )
			obj2->timer = 0;
		obj = obj2;
	}

    ac = ch->pcdata->learned[gsn_stitch] / 2;
    ac = number_range( ac * 8 / 10, ac * 9 / 10 );

	if ( (get_skill( ch, gsn_tailor ) == 0) || (IS_SET(ch->op_flags,OP_CHANNELER)) )
	{
		special = 0;
	}
	objBuf = str_dup( obj->short_descr );

	if ( skin )
	{
	strcpy( short_buf, obj->short_descr );
    if ( number_percent( ) < (ch->pcdata->learned[gsn_skin] + ch->mod_stat[STAT_REF]) / 3 )
    {
	    obj->value[0]              = ac;
	    obj->value[1]              = ac;
	    obj->value[2]              = ac;
	    obj->value[3]              = ac;
	    obj->level                 = lvl;
	    obj->cost                  = lvl*10;
	    obj->condition             = 100;
	    obj->guild                 = str_dup("Loner");
	    obj->item_type = ITEM_ARMOR;
	    if ( special > 0 )  changed from > 1
	    {
		    af->next       =   obj->affected;  */   /* hack to prevent double aff
			obj->affected =   af;
		}
		if ( !str_cmp(arg3,"cloak") )
		{
			sprintf(buf,"A cloak created from %s",obj->short_descr);
			obj->wear_flags       = ITEM_WEAR_ABOUT;
			obj->wear_flags       += ITEM_TAKE;
			free_string(obj->name);
		    obj->name             = str_dup("cloak");
			free_string(obj->description);
		    obj->description      = str_dup(buf);
			free_string(obj->short_descr);
		    obj->short_descr      = str_dup(buf);
		    chance                -= 50;
		}
		else if ( !str_cmp(arg3,"gloves") )
		{
			sprintf(buf,"A pair of gloves created from %s",obj->short_descr);
			obj->wear_flags       = ITEM_WEAR_HANDS;
			obj->wear_flags       += ITEM_TAKE;
			free_string(obj->name);
		    obj->name             = str_dup("gloves");
			free_string(obj->description);
		    obj->description      = str_dup(buf);
			free_string(obj->short_descr);
		    obj->short_descr      = str_dup(buf);
		    chance                -= 20;
		}
		else if ( !str_cmp(arg3,"boots") )
		{
			sprintf(buf,"A pair of boots created from %s",obj->short_descr);
			obj->wear_flags       = ITEM_WEAR_FEET;
			obj->wear_flags       += ITEM_TAKE;
			free_string(obj->name);
		    obj->name             = str_dup("boots");
			free_string(obj->description);
		    obj->description      = str_dup(buf);
			free_string(obj->short_descr);
		    obj->short_descr      = str_dup(buf);
		    chance                -= 20;
		}
		else if ( !str_cmp(arg3,"belt") )
		{
			sprintf(buf,"A belt created from %s",obj->short_descr);
			obj->wear_flags       = ITEM_WEAR_WAIST;
			obj->wear_flags       += ITEM_TAKE;
			free_string(obj->name);
		    obj->name             = str_dup("belt");
			free_string(obj->description);
		    obj->description      = str_dup(buf);
			free_string(obj->short_descr);
		    obj->short_descr      = str_dup(buf);
		    chance                -= 10;
		}
		else if ( !str_cmp(arg3,"shirt") )
		{
			sprintf(buf,"A shirt created from %s",obj->short_descr);
			obj->wear_flags       = ITEM_WEAR_BODY;
			obj->wear_flags       += ITEM_TAKE;
			free_string(obj->name);
		    obj->name             = str_dup("shirt");
			free_string(obj->description);
		    obj->description      = str_dup(buf);
			free_string(obj->short_descr);
		    obj->short_descr      = str_dup(buf);
		    chance                -= 30;
		}
		else if ( !str_cmp(arg3,"pants") )
		{
			sprintf(buf,"A pair of pants created from %s",obj->short_descr);
			obj->wear_flags       = ITEM_WEAR_LEGS;
			obj->wear_flags       += ITEM_TAKE;
			free_string(obj->name);
		    obj->name             = str_dup("pants");
			free_string(obj->description);
		    obj->description      = str_dup(buf);
			free_string(obj->short_descr);
		    obj->short_descr      = str_dup(buf);
		    chance                -= 30;
		}
		else
	    {
	        stc( "{cWhat do you want to create?\n\r", ch );
	        stc( "{c<cloak boots belt gloves pants shirt>\n\r", ch );
	        return;
	    }
		if ( special > 0 && !IS_SET(ch->op_flags,OP_CHANNELER))
		{
			special += ch->pcdata->learned[gsn_tailor];
			special /= 6;  changed to 6 from 4
			if (special > 15)   added to keep affects in limits
				special = 15;  can be removed if won't unbalance game
			ac += ( ch->pcdata->learned[gsn_tailor] / 10 );
		    check_improve(ch,gsn_tailor,TRUE,3);
			 special handling for make_special
			if ( special > 1 )
			{
			specChance = number_range(1,12);
			switch( specChance )
			{
				case 1:
		    		af->location   =   APPLY_STR;
				    af->modifier   =   special;
					break;
				case 2:
		    		af->location   =   APPLY_DEX;
				    af->modifier   =   special;
					break;
				case 3:
		    		af->location   =   APPLY_CON;
				    af->modifier   =   special;
					break;
				case 4:
		    		af->location   =   APPLY_WIS;
				    af->modifier   =   special;
					break;
				case 5:
		    		af->location   =   APPLY_INT;
				    af->modifier   =   special;
					break;
				case 6:
		    		af->location   =   APPLY_CHA;
				    af->modifier   =   special;
					break;
				case 7:
		    		af->location   =   APPLY_INS;
				    af->modifier   =   special;
					break;
				case 8:
		    		af->location   =   APPLY_REF;
				    af->modifier   =   special;
					break;
				case 9:
		    		af->location   =   APPLY_HITROLL;
				    af->modifier   =   special;
					break;
				case 10:
		    		af->location   =   APPLY_DAMROLL;
				    af->modifier   =   special;
					break;
				case 11:
		    		af->location   =   APPLY_HIT;
				    af->modifier   =   ( special * 3 );
					break;
				case 12:
		    		af->location   =   APPLY_MANA;
				    af->modifier   =   ( special * 3 );
					break;
				case 13: case 14:
		    		af->location   =   APPLY_AC;
				    af->modifier   =   -( special );
					break;
			}
			af->where	   =   TO_OBJECT;
		    af->type       =   -1;
		   	af->duration   =   -1;
		   	af->bitvector  =   0;
			af->level      =   lvl;
			}
		}
		WAIT_STATE( ch, PULSE_EVENT * 15 );
		wait_printf(ch,5,"You begin to stitch %s.\n\r",objBuf);
		wait_act(5,"$n begins to stitch $t.",ch,objBuf,0,TO_ROOM);

		wait_act(8,"You lean over $t, taking great care on your work.",ch,objBuf,0,TO_CHAR);
		wait_act(8,"$n leans over $t, taking great care on $s work.",ch,objBuf,0,TO_ROOM);

		wait_act(11,"You pull the stitches on $t tight, finishing the work.",ch,objBuf,0,TO_CHAR);
		wait_act(11,"$n pulls the stitches on $t tight, finishing the work.",ch,objBuf,0,TO_ROOM);

	    wait_act(15,"{c$n admires the finished work on $t.", ch, objBuf, NULL, TO_ROOM );
	    wait_act(15,"{cYou admire the finished work on $t.", ch, objBuf, NULL, TO_CHAR );
	    obj_to_room( obj, ch->in_room );
	    check_improve(ch,gsn_stitch,TRUE,9);
    }
    else
    {
        act( "{c$n attempts to work a skin, but destroys it.", ch, obj, NULL, TO_ROOM );
        act( "{cYou attempt to work a skin, but destroy it.", ch, obj, NULL, TO_CHAR );
        WAIT_STATE( ch, PULSE_EVENT * 3 );
        check_improve(ch,gsn_stitch,FALSE,1);
        extract_obj(obj);
    }
	}

	if ( cloth )
	{
	strcpy( short_buf, obj2->short_descr );
    if ( number_percent( ) < (ch->pcdata->learned[gsn_skin] + ch->mod_stat[STAT_REF]) / 3 )
    {
	    obj2->value[0]              = ac;
	    obj2->value[1]              = ac;
	    obj2->value[2]              = ac;
	    obj2->value[3]              = ac;
	    obj->level                  = lvl;
	    obj->cost                   = lvl*10;
	    obj2->condition             = 100;
	    obj2->guild                 = str_dup("Loner");
	    obj2->item_type = ITEM_ARMOR;
	    if ( special > 0 )
	    {
		    af->next       =   obj2->affected;  */ /* hack to prevent double aff */
/*			obj2->affected =   af;
		}
		if ( !str_cmp(arg3,"cloak") )
		{
			sprintf(buf,"A cloak created from %s",short_buf);
			obj2->wear_flags       = ITEM_WEAR_ABOUT;
			obj2->wear_flags       += ITEM_TAKE;
			free_string(obj2->name);
		    obj2->name             = str_dup("cloak");
			free_string(obj2->description);
		    obj2->description      = str_dup(buf);
			free_string(obj2->short_descr);
		    obj2->short_descr      = str_dup(buf);
		    chance                -= 50;
		}
		else if ( !str_cmp(arg3,"gloves") )
		{
			sprintf(buf,"A pair of gloves created from %s",short_buf);
			obj2->wear_flags       = ITEM_WEAR_HANDS;
			obj2->wear_flags       += ITEM_TAKE;
			free_string(obj2->name);
		    obj2->name             = str_dup("gloves");
			free_string(obj2->description);
		    obj2->description      = str_dup(buf);
			free_string(obj2->short_descr);
		    obj2->short_descr      = str_dup(buf);
		    chance                -= 20;
		}
		else if ( !str_cmp(arg3,"boots") )
		{
			sprintf(buf,"A pair of boots created from %s",short_buf);
			obj2->wear_flags       = ITEM_WEAR_FEET;
			obj2->wear_flags       += ITEM_TAKE;
			free_string(obj2->name);
		    obj2->name             = str_dup("boots");
			free_string(obj2->description);
		    obj2->description      = str_dup(buf);
			free_string(obj2->short_descr);
		    obj2->short_descr      = str_dup(buf);
		    chance                -= 20;
		}
		else if ( !str_cmp(arg3,"belt") )
		{
			sprintf(buf,"A belt created from %s",short_buf);
			obj2->wear_flags       = ITEM_WEAR_WAIST;
			obj2->wear_flags       += ITEM_TAKE;
			free_string(obj2->name);
		    obj2->name             = str_dup("belt");
			free_string(obj2->description);
		    obj2->description      = str_dup(buf);
			free_string(obj2->short_descr);
		    obj2->short_descr      = str_dup(buf);
		    chance                -= 10;
		}
		else if ( !str_cmp(arg3,"shirt") )
		{
			sprintf(buf,"A shirt created from %s",short_buf);
			obj2->wear_flags       = ITEM_WEAR_BODY;
			obj2->wear_flags       += ITEM_TAKE;
			free_string(obj2->name);
		    obj2->name             = str_dup("shirt");
			free_string(obj2->description);
		    obj2->description      = str_dup(buf);
			free_string(obj2->short_descr);
		    obj2->short_descr      = str_dup(buf);
		    chance                -= 30;
		}
		else if ( !str_cmp(arg3,"pants") )
		{
			sprintf(buf,"A pair of pants created from %s",short_buf);
			obj2->wear_flags       = ITEM_WEAR_LEGS;
			obj2->wear_flags       += ITEM_TAKE;
			free_string(obj2->name);
		    obj2->name             = str_dup("pants");
			free_string(obj2->description);
		    obj2->description      = str_dup(buf);
			free_string(obj2->short_descr);
		    obj2->short_descr      = str_dup(buf);
		    chance                -= 30;
		}
		else
	    {
	        stc( "{cWhat do you want to create?\n\r", ch );
	        stc( "{c<cloak boots belt gloves pants shirt>\n\r", ch );
	        return;
	    }
		if ( special > 0 && !IS_SET(ch->op_flags,OP_CHANNELER))
		{
			special += ch->pcdata->learned[gsn_tailor];
			special /= 6;
			if (special > 15)
				special = 15;
			ac += ( ch->pcdata->learned[gsn_tailor] / 10 );
		    check_improve(ch,gsn_tailor,TRUE,9);*/
			/* special handling for make_special */
/*			if ( special > 1 )
			{
			specChance = number_range(1,12);
			switch( specChance )
			{
				case 1:
		    		af->location   =   APPLY_STR;
				    af->modifier   =   special;
					break;
				case 2:
		    		af->location   =   APPLY_DEX;
				    af->modifier   =   special;
					break;
				case 3:
		    		af->location   =   APPLY_CON;
				    af->modifier   =   special;
					break;
				case 4:
		    		af->location   =   APPLY_WIS;
				    af->modifier   =   special;
					break;
				case 5:
		    		af->location   =   APPLY_INT;
				    af->modifier   =   special;
					break;
				case 6:
		    		af->location   =   APPLY_CHA;
				    af->modifier   =   special;
					break;
				case 7:
		    		af->location   =   APPLY_INS;
				    af->modifier   =   special;
					break;
				case 8:
		    		af->location   =   APPLY_REF;
				    af->modifier   =   special;
					break;
				case 9:
		    		af->location   =   APPLY_HITROLL;
				    af->modifier   =   special;
					break;
				case 10:
		    		af->location   =   APPLY_DAMROLL;
				    af->modifier   =   special;
					break;
				case 11:
		    		af->location   =   APPLY_HIT;
				    af->modifier   =   ( special * 3 );
					break;
				case 12:
		    		af->location   =   APPLY_MANA;
				    af->modifier   =   ( special * 3 );
					break;
				case 13:
				case 14:
		    		af->location   =   APPLY_AC;
				    af->modifier   =   -( special );
					break;
			}
			af->where	   =   TO_OBJECT;
		    af->type       =   -1;
		   	af->duration   =   -1;
		   	af->bitvector  =   0;
			af->level      =   lvl;
			}
		}
		WAIT_STATE( ch, PULSE_EVENT * 15 );
		wait_printf(ch,5,"You begin to stitch %s.\n\r",short_buf);
		wait_act(5,"$n begins to stitch $t.",ch,short_buf,NULL,TO_ROOM);

		wait_act(8,"You lean over $t, taking great care on your work.",ch,short_buf,NULL,TO_CHAR);
		wait_act(8,"$n leans over $t, taking great care on $s work.",ch,short_buf,NULL,TO_ROOM);

		wait_act(11,"You pull the stitches on $t tight, finishing the work.",ch,short_buf,NULL,TO_CHAR);
		wait_act(11,"$n pulls the stitches on $t tight, finishing the work.",ch,short_buf,NULL,TO_ROOM);

	    wait_act(15,"{c$n admires the finished work on $t.", ch, short_buf, NULL, TO_ROOM );
	    wait_act(15,"{cYou admire the finished work on $t.", ch, short_buf, NULL, TO_CHAR );
	   */ /*obj2_to_room( obj2, ch->in_room );*/
	   /* check_improve(ch,gsn_stitch,TRUE,9);
    }
    else
    {
        act( "{c$n attempts to work a skin, but destroys it.", ch, obj2, NULL, TO_ROOM );
        act( "{cYou attempt to work a skin, but destroy it.", ch, obj2, NULL, TO_CHAR );
        WAIT_STATE( ch, PULSE_EVENT * 3 );
        check_improve(ch,gsn_stitch,FALSE,1);
        extract_obj(obj2);
    }
	}

	return;
}*/


void do_claw( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	int dam;

	if (IS_NPC(ch))	return;

	if (ch->race != race_lookup("trolloc") && !WOLF(ch))
	{
		stc("You dont have claws, silly.\n\r",ch);
		return;
	}

    if (!is_fighting(ch) )
    {
	stc( "{cYou aren't fighting anyone.\n\r", ch );
	return;
    }
    victim = ch->advancing_on;

    if ( ch->range < RANGE_MELEE )
    {
	stc( "{cYou cannot hit them , they are not in melee.\n\r", ch );
	return;
    }

	act("$n rakes $N with $s claws.",ch,0,victim,TO_ROOM);
	act("You rake $N with your claws.",ch,0,victim,TO_CHAR);
	dam = number_range(ch->level / 2, ch->level);
	dam += get_curr_stat(ch, STAT_STR);
	dam += GET_DAMROLL(ch);

	damage( ch, victim, dam, gsn_claw, DAM_SLASH,TRUE);
	WAIT_STATE( ch, number_range( 2,6 ) );
	return;
}

void do_bite( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	int dam;

	if (IS_NPC(ch))	return;

	if (ch->race != race_lookup("trolloc") && !WOLF(ch))
	{
		stc("You gnash your teeth.\n\r",ch);
		return;
	}

	if (!is_fighting(ch) ||  ( victim = ch->advancing_on ) == NULL )
	{
		stc( "{cYou aren't fighting anyone.\n\r", ch );
		return;
	}

	if ( ch->range < RANGE_MELEE )
	{
		stc( "{cYou cannot hit them , they are not in melee.\n\r", ch );
		return;
	}

	act("$n tears at the first exposed body part with $m teeth.",ch,0,victim,TO_ROOM);
	act("You tear at $N's first exposed body part with your teeth.",ch,0,victim,TO_CHAR);
	dam = number_range(ch->level / 2, ch->level);
	dam += get_curr_stat(ch, STAT_STR);
	dam += GET_DAMROLL(ch) * 1.5;

	damage( ch, victim, dam, gsn_bite, DAM_SLASH,TRUE);
	WAIT_STATE( ch, 8 );
	return;
}

/*
 * Engage Combat code
 *
 *  Written by John Patrick (j.s.patrick@ieee.org)
 *   for Ansalon (ansalon.mudservices.com 8679)
 *   Engage skill, allows changing of targets in battle.
 *   Increasing ability with Level.
 */

void do_engage(CHAR_DATA *ch, char *argument)
{
  char arg[MAX_INPUT_LENGTH];
  CHAR_DATA *victim;
/*  FIGHTING_DATA *target; */
  int chance;

  /* Must be fighting.  */
  if (!is_fighting(ch))
  {
      stc("You're not fighting anyone.\n\r",ch);
      return;
  }

  one_argument( argument, arg );

  /* Check for argument.  */
  if (arg[0] == '\0')
  {
      stc("Engage who?\n\r",ch);
      return;
  }

  /* Check for victim.  */
  if ((victim = get_char_room(ch,NULL,arg)) == NULL)
  {
      stc("Shadowbox some other time.\n\r",ch);
      return;
  }

  if (victim == ch)
  {
      stc("Attacking yourself in combat isn't a smart thing.\n\r",ch);
      return;
  }

  if (check_fighting(victim, ch))
  {
	  /*
	  	target = ch->fight_data;
	  	if (target != NULL)
	  	{
	  		if (target->fighter != victim)
	  		{
	  			while ((target->next != NULL) && (target->fighter != victim))
	  				target = target->next;


	  			if (target->next == NULL)
				{
	  				stc("{WAn {RError{W occured, please report (using 'bug') having a faulty target when trying to switch victims.{x\n\r", ch);
					return;
				}
	  		}
	  	}
	  	else
	  	{
	  		bugf("engage (switch victim) - NULL target" );
	  		return;
	  	}

	  	if (target->fighter == victim)
	  	{
		*/
	  if(victim == ch->advancing_on)
	  {
      		stc("You're already pummelling that target as hard as you can!\n\r",ch);
	        return;
	  }

	  		ch->advancing_on = victim;
		  act( "$n has turned $s attacks toward you!",ch,NULL,victim,TO_VICT);
		  act("You turn your attacks toward $N.",ch,NULL,victim,TO_CHAR);
		  act("$n has turned $s attacks toward $N!",ch,NULL,victim,TO_NOTVICT);
	/*  	}
	  	else
	  		bugf("engage (switch victim) - wrong target" );
*/
	  	return;

  }
	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_engage ) <= 0 )
	    {
			stc("Engage? But you're not even dating!\n\r",ch);
			return;
	    }
	}

  /* This lets higher-level characters engage someone that isn't already fighting them.
     Quite powerful.  Raise level as needed.  */
  if (is_fighting(victim))
  {
      stc("They're already in a fight!\n\r",ch);
      return;
  }

  /* Get chance of success, and allow max 95%.  */
  chance = get_skill(ch,gsn_engage);
  chance = UMIN(chance,95);

  if ((number_percent() < chance))
  {
      /* It worked!  */
	  start_extra_fight(ch, victim, RANGE_MELEE);

      check_improve(ch,gsn_engage,TRUE,3);
      act( "$n has turned $s attacks toward you!",ch,NULL,victim,TO_VICT);
      act("You turn your attacks toward $N.",ch,NULL,victim,TO_CHAR);
      act("$n has turned $s attacks toward $N!",ch,NULL,victim,TO_NOTVICT);
  }
  else
  {
      stc("You couldn't get your attack in.\n\r",ch);
      check_improve(ch,gsn_engage,FALSE,3);
  }
}


void do_veil(CHAR_DATA *ch, char *argument)
{
    AFFECT_DATA af;

	if (IS_NPC(ch))
		return;

	if (ch->race != race_lookup("aiel"))
	{
		stc("You try and imitate an Aiel.\n\r",ch);
		return;
	}
	if (IS_AFFECTED2(ch, AFF_VEILED))
	{
		stc("You are already veiled.\n\r",ch);
		return;
	}

	act("$n dons a black veil.",ch,0,0,TO_ROOM);
	stc("You veil in readiness for battle.\n\r",ch);
	WAIT_STATE( ch, skill_table[gsn_veil].beats * 2 );

    af.where	 = TO_AFFECTS2;
    af.type      = gsn_veil;
    af.level	 = ch->level;
	af.duration	= number_fuzzy(ch->level / 10);
	af.modifier	= UMAX(1,ch->level/15);
    af.bitvector = AFF_VEILED;

/*    affect_to_char( ch, &af );   this is too soon, and give the 16394 bugs */

	af.location	= APPLY_HITROLL;
	affect_to_char(ch,&af);

	af.location	= APPLY_DAMROLL;
	affect_to_char(ch,&af);
}

void do_circle( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    OBJ_DATA *obj;


	act("You spin around and around in a dizzying circle!",ch,NULL,NULL,TO_CHAR);
	act("$n spins around and around in a dizzying circle!",ch,NULL,NULL,TO_ROOM);
	return;

    one_argument( argument, arg );

    if (arg[0] == '\0')
    {
        stc("Circle whom?\n\r",ch);
        return;
    }


    else if ((victim = get_char_room(ch,NULL,arg)) == NULL)
    {
        stc("They aren't here.\n\r",ch);
        return;
    }

    if (IS_NPC(victim) && is_fighting(victim) &&
        !is_same_group(ch,victim->advancing_on))

    {
        stc("Kill stealing is not permitted.\n\r",ch);
        return;
    }

    if ( ( obj = get_eq_char( ch, WEAR_WIELD ) ) == NULL)
    {
    	if (!IS_NPC(ch))
    	{
        stc( "You need to wield a weapon to circle.\n\r", ch );
        return;
    	}
	}

    if ( !check_fighting(ch, victim) )
    {
        stc( "You must be fighting in order to circle.\n\r", ch );
        return;
    }

    check_killer( ch, victim );
    WAIT_STATE( ch, skill_table[gsn_circle].beats );
    if ( number_percent( ) < get_skill(ch,gsn_circle)
    || ( get_skill(ch,gsn_circle) >= 2 && !IS_AWAKE(victim) ) )
    {
        check_improve(ch,gsn_circle,TRUE,1);
        one_hit( ch, victim, gsn_circle, TRUE );
    }
    else
    {
        check_improve(ch,gsn_circle,FALSE,1);
        damage( ch, victim, 0, gsn_circle,DAM_NONE,TRUE);
    }

    return;
}

void do_demand(CHAR_DATA *ch,char *argument)
{
    CHAR_DATA  *victim;
    OBJ_DATA *obj;
    char i_name[MAX_INPUT_LENGTH];
    char m_name[MAX_INPUT_LENGTH];
    char buf1[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char *target_name;
    int chance, vlevel;

    target_name = one_argument(argument,i_name);
    one_argument(target_name,m_name);

	if ( IS_NPC(ch) )
		return;

    chance = get_skill( ch, gsn_demand );
    if (chance == 0 )
    {
        stc("You are hardly intimidating enough to demand off others.\n\r",ch);
        return;
    }

    if ((victim = get_char_room(ch,NULL,m_name)) == NULL )
    {
        stc("They aren't here.\n\r",ch);
        return;
    }
    if (!IS_NPC(victim))
    {
        stc("Why not just threaten them in person?\n\r",ch);
        return;
    }
    vlevel = victim->level;
    chance /= 2;
    chance += ch->mod_stat[STAT_CHA] / 3;
    chance += (3*ch->level);
    chance -= 2*vlevel;
    chance /= 2;

    if (IS_GOOD(victim))
        chance -= 4*vlevel;
    else if (IS_EVIL(victim))
        chance -= 2*vlevel;
    else
        chance -= 3*vlevel;

    vlevel += 8;

    if ((obj = get_obj_list(victim, i_name, victim->carrying)) == NULL)
    {
        stc("They do not have that object.\n\r",ch);
        return;
    }
    if (!can_see_obj(ch,obj) )
    {
        stc("They dont have that object.\n\r",ch);
        return;
    }

    if (vlevel > ch->level || number_percent() > chance)
    {
        check_improve(ch,gsn_demand,FALSE,2);
        sprintf(buf1,"I don't think I'd give my belongings to one as weak as you!");
        sprintf(buf2,"Help! I'm being attacked by %s!",victim->short_descr);
        do_say(victim,buf1);
        do_yell(ch,buf2);
        multi_hit(victim,ch,TYPE_UNDEFINED);
        return;
    }

    if (!can_see(victim,ch))
    {
        act("$N tells you, 'I can't give to those I can't see.'",ch,0,victim,TO_CHAR);
        return;
    }

    if (!can_see_obj(victim,obj))
    {
        act("$N tells you, 'I can't see such an object.'",ch,0,victim,TO_CHAR);
        return;
    }

    if (obj->level > ch->level + 8)
    {
        do_say(victim, "That item is far too precious to hand over to scum like you!");
        sprintf(buf1,"Help! I'm being attacked by %s!",victim->short_descr);
        do_yell(ch,buf1);
        multi_hit(victim,ch,TYPE_UNDEFINED);
        return;
    }

    if (ch->stam < obj->level)
    {
        act("$N tells you, 'Hah! You couldn't even get away if I chased you!'.",ch,0,victim,TO_CHAR);
        sprintf(buf1,"Help! I'm being attacked by %s!",victim->short_descr);
        do_yell(ch,buf1);
        multi_hit(victim,ch,TYPE_UNDEFINED);
        return;
    }

    if (ch->hit < (ch->max_hit*3/7))
    {
        do_say(victim,"Hah! You look weak enough that even I could kill you!");
        sprintf(buf1,"Help! I'm being attacked by %s!",victim->short_descr);
        do_yell(ch,buf1);
        multi_hit(victim,ch,TYPE_UNDEFINED);
        return;
    }

    if (ch->stam < 30)
    {
        stc("You don't have the mana.\n\r",ch);
        return;
    }

    if ((obj->wear_loc != WEAR_NONE) && IS_SET(obj->extra_flags, ITEM_NOREMOVE) )
    {
        act("$N tells you, 'I'm unable to release $p'.",ch,obj,victim,TO_CHAR);
        act("$N cowers back from you in fright.",ch,0,victim,TO_CHAR);
        act("$N cowers back from $n in fright.",ch,0,victim,TO_NOTVICT);
        return;
    }
    if (IS_SET(obj->extra_flags,ITEM_NODROP) )
    {
        act("$N tells you, 'I'm unable to release $p'.",ch,obj,victim,TO_CHAR);
        act("$N cowers back from you in fright.",ch,0,victim,TO_CHAR);
        act("$N cowers back from $n in fright.",ch,0,victim,TO_NOTVICT);
        return;
    }

    if ( ch->carry_weight + get_obj_weight(obj)  > can_carry_w(ch) )
    {
        act("$N tells you, 'You can't carry the weight $n.'",ch,0,victim,TO_CHAR);
        return;
    }
    if ( ch->carry_number + 1 > can_carry_n(ch) )
    {
        act("$N tells you, 'You can't carry that many items $n.'",ch,0,victim,TO_CHAR);
        return;
    }

    act("$N caves in to the bully tactics of $n.",ch,0,victim,TO_NOTVICT);
    act("$N shivers in fright and caves in to your bully tactics.",ch,0,victim,TO_CHAR);

    if (obj->wear_loc != WEAR_NONE)
    {
        act("$n stops using $p.",victim,obj,NULL,TO_ROOM);
        act("You stop using $p.",victim,obj,NULL,TO_CHAR);
    }
    act("$N gives $p to $n.",ch,obj,victim,TO_NOTVICT);
    act("$N gives you $p.",ch,obj,victim,TO_CHAR);
    check_improve(ch,gsn_demand,TRUE,2);
    WAIT_STATE(ch,24);
    obj_from_char(obj);
    obj_to_char(obj,ch);
    ch->stam -= 30;
    ch->hit -= 30;
    return;
}

void do_drag( CHAR_DATA * ch, char * argument )
{

   char buf[MAX_INPUT_LENGTH];
   OBJ_DATA  *obj;
   ROOM_INDEX_DATA *rvnum;
   CHAR_DATA *vch;
   int door;

   argument = one_argument (argument, buf);


   obj = get_obj_list( ch, buf, ch->in_room->contents );

   if ( !obj )
   {
	  stc ( "I do not see anything like that here.\n\r", ch);
	  return;
   }

   if ( (!IS_SET( obj->wear_flags, ITEM_TAKE )))
   {
 	  act( "You try to drag $p, but without succes.\n\r", ch,obj, NULL, TO_CHAR);
 	  act( "$n tries to move $p, but it doesn't budge.\n\r",ch,obj, NULL, TO_ROOM);
	  return;
   }

   if ( obj->weight >  (2 * can_carry_w (ch)) )
   {
      act( "You try, but $p is too heavy.\n\r", ch, obj, NULL, TO_CHAR);
      act( "$n tries to move $p, but $s fails.\n\r", ch, obj, NULL, TO_CHAR);
	  return;
   }
	return;
   if ( argument[0] == '\0' )
   {
      stc ( "Where do you want to drag it ?\n\r", ch);
	  return;
   }

   rvnum = ch->in_room;


   if ( !str_cmp( argument, "n" ) || !str_cmp( argument, "north" ) ) door = 0;
   else if ( !str_cmp( argument, "e" ) || !str_cmp( argument, "east"  ) ) door = 1;
   else if ( !str_cmp( argument, "s" ) || !str_cmp( argument, "south" ) ) door = 2;
   else if ( !str_cmp( argument, "w" ) || !str_cmp( argument, "west"  ) ) door = 3;
   else if ( !str_cmp( argument, "u" ) || !str_cmp( argument, "up"    ) ) door = 4;
   else if ( !str_cmp( argument, "d" ) || !str_cmp( argument, "down"  ) ) door = 5;
   else if ( !str_cmp( argument, "ne" ) || !str_cmp( argument, "northeast" ) ) door = 6;
   else if ( !str_cmp( argument, "se" ) || !str_cmp( argument, "southeast" ) ) door = 7;
   else if ( !str_cmp( argument, "sw" ) || !str_cmp( argument, "southwest" ) ) door = 8;
   else if ( !str_cmp( argument, "nw" ) || !str_cmp( argument, "northwest" ) ) door = 9;

   if (door == -1)
   {
      stc("You don't see exit in that direction.\n\r", ch);
	  return;
   }

   move_char( ch, door, TRUE);

   if (ch->in_room == rvnum )
      return;

   obj_from_room (obj);
   obj_to_room (obj, ch->in_room);

   act ( "You dragged $p with you.\n\r", ch, obj, NULL, TO_CHAR );
   act ( "$n dragged $p with $s.\n\r", ch, obj, NULL, TO_ROOM );

   if ( !(vch = rvnum->people) )
      return;

   act ( "$N dragged $p out.\n\r", vch, obj, ch, TO_CHAR );
   act ( "$N dragged $p out.\n\r", vch, obj, ch, TO_ROOM );

   return;
}

void do_brew ( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *vial, *herb;
	char arg1[MIL], arg2[MIL];
	bool fail = FALSE;
	char buf[MSL];

	if (IS_NPC(ch))
		return;

	if ( get_skill( ch, gsn_brew ) <= 0 )
	{
		stc("You dont know how to brew.\n\r",ch);
		return;
	}

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( arg1[0] == '\0' || (herb = get_obj_carry(ch, arg1, ch)) == NULL)
	{
		stc("Brew what herb?\n\r", ch );
		stc("Syntax: Brew <herb> <vial>\n\r",ch);
		return;
	}

	if (herb->item_type != ITEM_HERB)
	{
		stc("That's not an herb!\n\r", ch);
		return;
	}

	if ( arg2[0] == '\0' || (vial = get_obj_carry(ch, arg2, ch)) == NULL)
	{
		stc("Brew this herb with which vial?\n\r", ch );
		stc("Syntax: Brew <herb> <vial>\n\r",ch);
		return;
	}

	if (vial->pIndexData->vnum != 3279)
	{
		stc("You can't use this to brew in.\n\rSee Mother Grubb east of Caemlyn for empty vials.\n\r", ch);
		return;
	}

	if (vial->condition >= 100 && vial->item_type != ITEM_TREASURE)
	{
		stc("This vial is already full!\n\r", ch);
		return;
	}

	if (number_percent() > 3 * get_skill(ch, gsn_brew) / 4)
		fail = TRUE;

	/* Empty vial */
	if (vial->item_type == ITEM_TREASURE)
	{
		vial->item_type = ITEM_POTION;

		if (fail)
			sprintf(buf, "failure");
		else	sprintf(buf, "%s", herb->short_descr);

		free_string(vial->material);
		vial->material = str_dup(buf);

		free_string(vial->name);
		vial->name = str_dup("vial herbal mixture potion");

		free_string(vial->short_descr);
		vial->short_descr = str_dup("a vial filled with an herbal mixture");

		free_string(vial->description);
		vial->description = str_dup("a vial filled with an herbal mixture is lying here on the ground.");
		vial->condition = 50;

		if (!fail)
		vial->value[0] = get_skill(ch, gsn_brew) / 3 + calc_remort_tier(ch) * 5 + number_range(1, ch->level / 10);
	}
	else
	{
		if (fail && !is_name("failure", vial->material))
			sprintf(buf, "%s, failure", vial->material);
		else if (!is_name(herb->short_descr, vial->material))
			sprintf(buf, "%s, %s", vial->material, herb->short_descr);

		free_string(vial->material);
		vial->material = str_dup(buf);
		vial->condition += 25;

		if (!fail)
		vial->value[0] += get_skill(ch, gsn_brew) / 3 + calc_remort_tier(ch) * 5 + number_range(1, ch->level / 10);
	}

	act("You grind and smash $p with your mortar and pestle.", ch, herb, NULL, TO_CHAR);
	act("$n grinds and smashes $p with $s mortar and pestle.", ch, herb, NULL, TO_ROOM);

	wait_act(4, "You seep the herbs into some warm water.", ch, NULL, NULL, TO_CHAR);
	wait_act(4, "$n seeps the herbs into some warm water.", ch, NULL, NULL, TO_ROOM);

	wait_act(8, "You pour the herbal mixture into your vial.", ch, NULL, NULL, TO_CHAR);
	if (fail)
	wait_act(8, "You feel like something went wrong.", ch, NULL, NULL, TO_CHAR);
	wait_act(8, "$n pours the herbal mixture into $s vial.", ch, NULL, NULL, TO_ROOM);

	check_improve(ch, gsn_brew, !fail, 5);
	extract_obj(herb);
	return;
}

OBJ_DATA *get_sharpen_stone (CHAR_DATA *ch)
{
	OBJ_DATA *stone;

	for (stone = ch->carrying; stone != NULL; stone = stone->next_content)
	{
		if (stone->pIndexData->vnum == 18)	break;
		if (stone->next_content == NULL)	return NULL;
	}

	return stone;
}

bool check_sharpness (OBJ_DATA *obj)
{
	if (obj->value[2] >= 150)	return TRUE;

	return FALSE;
}

void do_sharpen(CHAR_DATA *ch, char *argument)
{
	OBJ_DATA *wield, *stone;
	int wCond, chance, /*sharp,*/ skill;
	bool killit = FALSE;
	char *objBuf;

	if (IS_NPC(ch))
	{
		return;
	}

	skill = get_skill( ch, gsn_sharpen );

	if ( skill <= 0 )
	{
		stc("You couldn't sharpen a bread knife.\n\r", ch );
		return;
	}

	wield  = get_obj_carry( ch, argument, ch ) ;

	if (argument[0] == '\0' )
	{
		stc( "What are you trying to sharpen?\n\r", ch );
		return;
	}
	if ( is_fighting(ch) )
	{
		stc( "While you are fighting? Yeah right!\n\r", ch );
		return;
	}
	if ( wield == NULL )
	{
		stc( "You do not have that weapon.\n\r", ch );
		return;
	}
	if ( wield->item_type != ITEM_WEAPON )
	{
		stc( "That item is not a weapon.\n\r", ch );
		return;
	}

	if ((stone = get_sharpen_stone(ch)) == NULL)
	{
		stc("You're going to need a whetstone to sharpen your weapon.\n\r", ch);
		return;
	}

	if (check_sharpness(wield))
	{
		stc("It's sharp enough already.\n\r", ch );
		return;
	}

	wCond = wield->condition;
	chance = get_skill(ch,gsn_sharpen);
	chance += get_curr_stat(ch, STAT_REF);
	chance /= 3;
	objBuf = str_dup( wield->short_descr );
	stone->condition -= 20;

	if (stone->condition <= 0)
		extract_obj(stone);

	act("$n begins to sharpen $t.",ch,objBuf,0,TO_ROOM);
	act("You begin to sharpen $t.",ch,objBuf,0,TO_CHAR);

	WAIT_STATE( ch, PULSE_EVENT * 20 );

	wait_printf(ch,5,"You continue to sharpen %s.\n\r",objBuf);
	wait_act(5,"$n continues to sharpen $t.",ch,objBuf,0,TO_ROOM);

	wait_act(8,"You concentrate intently on honing $t.",ch,objBuf,0,TO_CHAR);
	wait_act(8,"$n concentrates intently on honing $t.",ch,objBuf,0,TO_ROOM);

	wait_act(11,"You focus on the balance of $t.",ch,objBuf,0,TO_CHAR);
	wait_act(11,"$n focuses on the balance of $t.",ch,objBuf,0,TO_ROOM);

	if(skill<5)
	    skill = 5;
	if (number_range(1, skill) ==  skill)
	{
	        wait_act(15,"$n attempts to sharpen $t, but it does not appear any sharper.",ch,objBuf,0,TO_ROOM);
                wait_act(15,"You attempt to sharpen $t, but you don't notice a difference.",ch,objBuf,0,TO_CHAR);
        	check_improve(ch,gsn_sharpen,FALSE,5);
		return;
	}
	if (wCond < 15)
	{
		wait_act(15,"$n ruins $t by attempting to sharpen it.",ch,objBuf,0,TO_ROOM);
		wait_act(15,"You ruin $t by attempting to sharpen it.",ch,objBuf,0,TO_CHAR);
		killit = TRUE;
	}
	else if (wCond < 75)
	{
		if (wCond < 3 * ( 100 - skill )  )
		{
			wait_act(15,"$n nicks $t, ruining it.",ch,objBuf,0,TO_ROOM);
			wait_act(15,"You nick $t, ruining it.",ch,objBuf,0,TO_CHAR);
			killit = TRUE;
		}
	}

	if (wCond < 85)
	{
		wield->value[2] += 1;
		wield->condition -= number_range(3, 5);
	}
	else
	{
		wield->value[2] += 2;
		wield->condition -= number_range(1, 5);
	}

	if ( killit )
	{
		check_improve(ch,gsn_sharpen,FALSE,5);
		extract_obj(wield);
		return;
	}

		wait_act(15,"$n holds $t aloft, admiring the sharpness.",ch,objBuf,0,TO_ROOM);
		wait_act(15,"You hold $t aloft, admiring the sharpness.",ch,objBuf,0,TO_CHAR);

	if (wield)
		SET_BIT(wield->value[4], WEAPON_SHARP);

	check_improve(ch,gsn_sharpen,TRUE,8);
	return;
}

void do_search( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *vch;
    OBJ_DATA *obj;
    int chance, num_found, rank, base, oppose;
    bool found;
    char arg[MIL];

	argument = one_argument( argument, arg );

	/* init the variables */
	if ( IS_NPC( ch ) )
		chance = 75;
	else
		chance = get_skill(ch,gsn_search);
	num_found = 0;
	found = FALSE;

    /*if ( ch->level < skill_table[gsn_search].skill_level[ch->class] )
    {
	stc("{cMight as well search for the needle in the haystack.\n\r", ch );
	return;
    }*/
    if ( IS_NPC( ch ) )
    {
		/* npcs dont get checked for skill */

		if ( arg[0] == '\0' )
		{
			act( "$n searches around the area for a moment.", ch, NULL, NULL, TO_ROOM );
		    for ( vch = ch->in_room->people; vch != NULL; vch = vch->next )
		    {
				/* hunt up some hiders */
				if (vch->in_room != ch->in_room)
					continue;
				if (vch == ch)
					continue;
				/* do this partly to keep people from finding wizi imms, and
				 * to allow mobs to be unsearchable */
				if (vch->level >= LEVEL_HERO && ch->level < LEVEL_HERO)
					continue;
				/* formulate the chance */
				if (ch->level >= vch->level)
					chance += 1;
				else
					chance -= 2;
				if ( get_curr_stat(ch,STAT_INT) >= get_curr_stat(vch,STAT_INT) )
					chance += 1;
				else
					chance -= 2;
				if ( get_curr_stat(ch,STAT_DEX) >= get_curr_stat(vch,STAT_DEX) )
					chance += 1;
				else
					chance -= 2;
				if ( get_curr_stat(ch,STAT_WIS) >= get_curr_stat(vch,STAT_WIS) )
					chance += 1;
				else
					chance -= 2;
				if ( get_curr_stat(ch,STAT_INS) >= get_curr_stat(vch,STAT_INS) )
					chance += 1;
				else
					chance -= 2;
				if ( get_curr_stat(ch,STAT_REF) >= get_curr_stat(vch,STAT_REF) )
					chance += 1;
				else
					chance -= 2;
				if (IS_SET( ch->spec_flags, SPEC_WOLFBROTHER))
				{
					chance += 5;
				}
				/* Always a chance to fail */
				if (chance > 99) chance = 99;
				/* ok, now show the found people */
				if (number_percent() > chance)
					continue;
			    if (is_affected(vch, gsn_invis)
			    || is_affected(vch, gsn_sneak)
				|| IS_AFFECTED(vch, AFF_SNEAK)
				|| IS_AFFECTED(vch, AFF_INVISIBLE)
				|| IS_AFFECTED(vch, AFF_HIDE)
			    || is_affected(vch, gsn_hide))
				{
				    printf_to_char(ch,"You see %s.\n\r",vch->name);
				    printf_to_char(vch,"%s has spotted you!\n\r",ch->name);
				    affect_strip ( vch, gsn_invis			);
				    affect_strip ( vch, gsn_sneak			);
				    REMOVE_BIT   ( vch->affected_by, AFF_HIDE		);
				    REMOVE_BIT   ( vch->affected_by, AFF_INVISIBLE	);
				    REMOVE_BIT   ( vch->affected_by, AFF_SNEAK		);
				    found = TRUE;
				    num_found += 1;
				    check_improve(ch,gsn_search,TRUE,9);
				}
			}
			for ( obj = ch->in_room->contents; obj; obj = obj->next_content )
			{
				if (obj->item_type == ITEM_PORTAL)
				{
			    printf_to_char(ch,"You see %s.\n\r",obj->short_descr);
			    found = TRUE;
			    num_found += 1;
				}
			}
			if (!found)
			{
				stc("You didn't find anyone here.\n\r",ch);
				printf_to_char(ch,"Chance is %d.\n\r",chance);
			}
			/* make em wait longer for each person found */
			WAIT_STATE(ch,PULSE_VIOLENCE * num_found);
			return;
		}

	}
	if ( !IS_NPC( ch ) )
	{
		if ( get_skill( ch, gsn_search ) <= 0 )
	    {
			stc("You dont know enough about searching.\n\r",ch);
			return;
	    }
	}

	if ( arg[0] == '\0' )
	{
	    for ( vch = ch->in_room->people; vch != NULL; vch = vch->next )
	    {
			/* hunt up some hiders */
			if (vch->in_room != ch->in_room)
				continue;
			if (vch == ch)
				continue;
			/* do this partly to keep people from finding wizi imms, and
			 * to allow mobs to be unsearchable */
			if (vch->level >= LEVEL_HERO && ch->level < LEVEL_HERO)
				continue;
			/* formulate the chance */
			if (ch->level >= vch->level)
				chance += 1;
			else
				chance -= 2;
			if ( get_curr_stat(ch,STAT_INT) >= get_curr_stat(vch,STAT_INT) )
				chance += 1;
			else
				chance -= 2;
			if ( get_curr_stat(ch,STAT_DEX) >= get_curr_stat(vch,STAT_DEX) )
				chance += 1;
			else
				chance -= 2;
			if ( get_curr_stat(ch,STAT_WIS) >= get_curr_stat(vch,STAT_WIS) )
				chance += 1;
			else
				chance -= 2;
			if ( get_curr_stat(ch,STAT_INS) >= get_curr_stat(vch,STAT_INS) )
				chance += 1;
			else
				chance -= 2;
			if ( get_curr_stat(ch,STAT_REF) >= get_curr_stat(vch,STAT_REF) )
				chance += 1;
			else
				chance -= 2;
			if (IS_SET( ch->spec_flags, SPEC_WOLFBROTHER))
			{
				chance += 5;
			}
			/* Always a chance to fail */
			if (chance > 99) chance = 99;
			/* ok, now show the found people */
			if (number_percent() > chance)
				continue;
		    if (is_affected(vch, gsn_invis)
		    || is_affected(vch, gsn_sneak)
			|| IS_AFFECTED(vch, AFF_SNEAK)
			|| IS_AFFECTED(vch, AFF_INVISIBLE)
			|| IS_AFFECTED(vch, AFF_HIDE)
		    || is_affected(vch, gsn_hide))
			{
			    printf_to_char(ch,"You see %s.\n\r",vch->name);
			    printf_to_char(vch,"%s has spotted you!\n\r",ch->name);
			    affect_strip ( vch, gsn_invis			);
			    affect_strip ( vch, gsn_sneak			);
			    REMOVE_BIT   ( vch->affected_by, AFF_HIDE		);
			    REMOVE_BIT   ( vch->affected_by, AFF_INVISIBLE	);
			    REMOVE_BIT   ( vch->affected_by, AFF_SNEAK		);
			    found = TRUE;
			    num_found += 1;
			    check_improve(ch,gsn_search,TRUE,9);
			}
		}
		for ( obj = ch->in_room->contents; obj; obj = obj->next_content )
		{
			if (obj->item_type == ITEM_PORTAL)
			{
			ptc(ch,"You see %s.\n\r",obj->short_descr);
			found = TRUE;
			num_found += 1;
			check_improve(ch,gsn_search,TRUE,9);
			}
		}
		if (!found)
		{
			stc("You didn't find anyone here.\n\r",ch);
/*			printf_to_char(ch,"Chance is %d.\n\r",chance);   This gives away if there is someone in the room or not */
		}
		/* make em wait longer for each person found */
		WAIT_STATE(ch,PULSE_VIOLENCE * num_found);
		return;
	}
	/* search obj for traps */
	if ( ( obj = get_obj_carry( ch, arg, ch ) ) == NULL )
	{
		stc( "Search what? You don't see that here.\n\r", ch );
		return;
	}
	if ( obj->item_type != ITEM_TREASURECHEST )
	{
		stc( "That's not a treasure chest.\n\r", ch );
		return;
	}

	rank = ch->pcdata->learned[gsn_unarm_traps];
	rank += get_curr_stat( ch, STAT_INT );
	base = number_range( 1, 100 );
	oppose = obj->value[4];

	base += oppose;

	if ( number_range( 1, rank ) < number_range( 1, base ) )
	{
		stc( "It doesn't appear to be trapped.\n\r", ch );
		return;
	}
	else if ( obj->value[3] == TRAP_NONE )
	{
		stc( "It doesn't appear to be trapped.\n\r", ch );
		return;
	}
	else
	{
		ptc( ch, "You see a %s.\n\r", obj->value[3] == TRAP_BOOMER ? "small black cube inside the lock" :
			obj->value[3] == TRAP_GASSER ? "small green cannister hidden within the lock" :
			obj->value[3] == TRAP_NEEDLE ? "small needle dripping with a green liquid inside the lock" :
			obj->value[3] == TRAP_BLADE ? "razor sharp blade hidden within the lock" :
			obj->value[3] == TRAP_MANA ? "small shimmering sphere tucked inside the lock" :
			obj->value[3] == TRAP_FIRE ? "reddish-gold casement placed behind the lock" :
			obj->value[3] == TRAP_ACID ? "bluish-green vial rigged to burst inside the lock" :
			"odd shape within the lock");
			check_improve(ch,gsn_search,TRUE,9);
	}
    return;
}

/*void do_speardance(CHAR_DATA *ch, char *argument)
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    char arg[MIL];
    int num_att = 0,i,rbi,type = 0,bonus = 0,dam;

    argument = one_argument(argument,arg);

	if (ch->race != race_lookup("aiel"))
	{
		stc("You're not an Aielman to dance the spears.\n\r",ch);
		return;
	}
    obj = get_eq_char( ch, WEAR_WIELD );
    if ( obj == NULL )
    {
		stc("You should wield a spear.\n\r",ch);
		return;
	}
	if ( obj->value[0] != WEAPON_SPEAR )
    {
		stc("You should wield a spear.\n\r",ch);
		return;
	}
	if ( arg[0] == '\0' )
	{
		stc("Below are the available speardances and their requirments.\n\r",ch);
		stc("Dance Name          Dance Short     Dance Requirement\n\r",ch);
		stc("Dance the Spears    DTS             Remort 0\n\r",ch);
		stc("The Flashing Spears TFS             Remort 3\n\r",ch);
		stc("Spears of Rage      SOR             Remort 6\n\r",ch);
		stc("Dance of Waking     DOW             Remort 9\n\r",ch);
		stc("Dance the Dream     DTD             Remort 12\n\r",ch);
		stc("Spears of Wrath     SOW             Remort 15\n\r",ch);
		stc("Dance of Death      DOD             Remort 20\n\r",ch);
		stc("To use the different speardances, type speardance <short>\n\r",ch);
		stc("The greater the remort requirement, the greater the bonus\n\r",ch);
		stc("to damage will be.\n\r",ch);
		return;
	}
    rbi = ch->pcdata->rebirth;

	if ( rbi < 10 )
	{
		rbi = 10;
		rbi += ch->pcdata->rebirth;
	}
	rbi = number_range( 1, rbi );
	if ( !str_prefix( arg, "DTS" ) && ch->pcdata->rebirth >= 0 )
	{
		type = 1.5;
		bonus = rbi * type;
	}
	else if ( !str_prefix( arg, "TFS" ) && ch->pcdata->rebirth >= 3 )
	{
		type = 2;
		bonus = rbi * type;
	}
	else if ( !str_prefix( arg, "SOR" ) && ch->pcdata->rebirth >= 6 )
	{
		type = 2.5;
		bonus = rbi * type;
	}
	else if ( !str_prefix( arg, "DOW" ) && ch->pcdata->rebirth >= 9 )
	{
		type = 3;
		bonus = rbi * type;
	}
	else if ( !str_prefix( arg, "DTD" ) && ch->pcdata->rebirth >= 12 )
	{
		type = 3.5;
		bonus = rbi * type;
	}
	else if ( !str_prefix( arg, "SOW" ) && ch->pcdata->rebirth >= 15 )
	{
		type = 4;
		bonus = rbi * type;
	}
	else if ( !str_prefix( arg, "DOD" ) && ch->pcdata->rebirth >= 20 )
	{
		type = 4.5;
		bonus = rbi * type;
	}
    if ( ( victim = ch->fighting ) == NULL )
    {
		stc( "{cYou aren't fighting anyone.\n\r", ch );
		return;
    }
    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
		act("{cBut $N is your friend!",ch,NULL,victim,TO_CHAR);
		return;
    }
    if (is_safe(ch,victim))
        return;

	act("$n spins $p around in a dizzying circle.",ch,obj,victim,TO_ROOM);
	act("You spin $p around in a dizzying circle.",ch,obj,victim,TO_CHAR);

	num_att = number_range(1,rbi);
	dam = ( bonus + ( ( obj->value[2] * obj->value[1] ) / 2 ) );
    if ( saves_spell( victim->level / 2, victim, DAM_HARM))
		num_att /= 2;
	dam += num_att;
	dam *= type;

	damage( ch, victim, dam, gsn_heron, DAM_PIERCE, TRUE );
	act("$n strikes at $N with $p!",ch,obj,victim,TO_ROOM);
	act("You strike at $N with $p!",ch,obj,victim,TO_CHAR);
    WAIT_STATE( ch, PULSE_VIOLENCE / 2 );


	for (i = 0; i < num_att; i++)
	{
		dam = number_range( (dam*8/10), (dam*10/9) );
		damage( ch, victim, dam, gsn_heron, DAM_PIERCE, TRUE );
		act("$n strikes at $N with $p!",ch,obj,victim,TO_ROOM);
		act("You strike at $N with $p!",ch,obj,victim,TO_CHAR);
	    WAIT_STATE( ch, PULSE_VIOLENCE / 2 );
	}

	check_improve( ch, gsn_speardance, TRUE, 5 );

    return;
}*/

/*void do_whirlwind (CHAR_DATA *ch, char *argument)*/	/*moved to skill_dal where hurricane and tornado are*/

/* ogier treesinging */
void do_treesing(CHAR_DATA *ch, char *argument)
{
    OBJ_DATA *staff;
	int chance;
	bool found;
	int cost = get_maxstam(ch) / 10;

	chance = 0;

    if ( (ch->race != race_lookup("ogier")) && (ch->race != race_lookup("nym")))
    {
		stc("What do you think you are, an Ogier?\n\r",ch);
		WAIT_STATE(ch, PULSE_VIOLENCE * 3);
		return;
	}

	found = FALSE;
	/*for ( obj = ch->in_room->contents; obj; obj = obj->next_content )
	{
		if (obj->item_type == ITEM_TREE_PC)
		{
	    printf_to_char(ch,"You search among the trees until you find %s.\n\r",obj->short_descr);
	    found = TRUE;
	    break;
		}
	}

	if (!found)
	{
		stc("You search among the trees, but fail to find a likely one.\n\r",ch);
		WAIT_STATE(ch, PULSE_VIOLENCE * 3);
		return;
	}*/

    switch(ch->in_room->sector_type)
    {
		case(SECT_INSIDE):				chance = 0;	break;
		case(SECT_CITY):				chance = 0;	break;
		case(SECT_FIELD):				chance = 0;	break;
		case(SECT_FOREST):				chance = 1; break;
		case(SECT_HILLS):				chance = 0; break;
		case(SECT_MOUNTAIN):			chance = 0;	break;
		case(SECT_WATER_SWIM):			chance = 0;	break;
		case(SECT_WATER_NOSWIM):		chance = 0;	break;
		case(SECT_AIR):					chance = 0; break;
	    case(SECT_TUNDRA):              chance = 0; break;
	    case(SECT_BLIGHT):              chance = 0; break;
	    case(SECT_COASTAL):             chance = 0; break;
	    case(SECT_ROAD):                chance = 0; break;
	    case(SECT_MARSH):               chance = 0; break;
		case(SECT_DESERT):				chance = 0; break;
    }

	if (cost > ch->stam)
	{
		stc("You are too tired.\n\r",ch);
		return;
	}

    if (chance < 1)
	{
		stc("You search for a likely tree, but fail to find one.\n\r",ch);
		WAIT_STATE(ch, PULSE_VIOLENCE * 3);
		return;
	}

    if (number_percent() > get_skill(ch,gsn_treesing))
    {
	check_improve(ch,gsn_treesing,FALSE,3);
	stc("You sing to the trees, but they fail to hear you.\n\r",ch);
	WAIT_STATE(ch, PULSE_VIOLENCE * 3);
	return;
    }

    act( "$n sings to the trees.",ch,NULL,NULL,TO_ROOM);
	ch->stam -= cost;

    if ( number_range( 1,5 ) == 1 )
    {
    staff = create_object(get_obj_index(OBJ_VNUM_TREESTAFF),ch->level);
    /* level should be set back to 0, maybe */
	if ( !CAN_WEAR(staff, ITEM_TAKE) ) /* set take if not set */
		staff->wear_flags = ITEM_TAKE;
	if ( !CAN_WEAR(staff, ITEM_WIELD) ) /* set take if not set */
		staff->wear_flags += ITEM_WIELD;
    staff->short_descr    = str_dup("A staff of Ogier sung-wood");
    staff->description    = str_dup("Here lies a staff of Ogier sung-wood.");
    staff->name           = str_dup("ogier sung-wood staff");
	staff->value[1]       = 1;
	staff->value[2]       = ch->level / 4 + get_curr_stat(ch,STAT_CHA) / 4;
	staff->value[2]       += 7;
	if ( staff->value[2] > 151 )
		staff->value[2] = 151;
	staff->weight         = 24;
	staff->guild          = str_dup("Ogier");
	staff->condition      = 100;
	staff->cost           = 0;

	if ( number_percent() > number_percent() )
		make_special(staff);
    }
    else
    {
	staff = get_random_material(NULL, CRAFT_WOODCRAFT, TRUE);
/*
	staff = create_object( get_obj_index( 909 ), 1 );
	if ( !CAN_WEAR(staff, ITEM_TAKE) )
		staff->wear_flags = ITEM_TAKE;
	if ( !CAN_WEAR(staff, ITEM_WIELD) )
		staff->wear_flags += ITEM_WIELD;
	staff->name = str_dup( "wood block" );
	staff->short_descr = str_dup( "a block of wood" );
	staff->description = str_dup( "Here lies a block of wood." );
	staff->item_type = ITEM_SHAPE_BLOCK;
	staff->material = str_dup( "softwood" );
	staff->weight = 5;
	staff->cost = 5;
	staff->guild = str_dup("Loner");
	staff->value[0] = 0;
*/
    }

	if (staff == NULL)
	{
		check_improve(ch,gsn_treesing,FALSE,3);
		stc("You sing to the trees, but they fail to hear you.\n\r",ch);
		WAIT_STATE(ch, PULSE_VIOLENCE * 3);
		return;
	}

    obj_to_room( staff,ch->in_room);

    /*extract_obj( obj );*/
    act( "You sing $p!", ch, staff, NULL, TO_CHAR );
    check_improve(ch,gsn_treesing,TRUE,3);
	WAIT_STATE(ch, PULSE_VIOLENCE * 3);
    return;
}
/*
void do_shadowpass( CHAR_DATA *ch, char *argument )
{
    AFFECT_DATA af;

    if ( IS_AFFECTED(ch, AFF_PASS_DOOR) )
	return;
	if ( ch->race != race_lookup("trolloc") )
	{
		act("You make funny little shadow figures on the walls with your hands... heh.",ch,0,0,TO_CHAR);
		act("$n makes funny little shadow figures on the walls with $s hands... heh.",ch,0,0,TO_ROOM);
		return;
	}
	if ( !IS_SET(ch->hist_flags,HIST_MYRDRAAL) )
	{
		act("You make funny little shadow figures on the walls with your hands... heh.",ch,0,0,TO_CHAR);
		act("$n makes funny little shadow figures on the walls with $s hands... heh.",ch,0,0,TO_ROOM);
		return;
	}
	if ( ch->pcdata->rebirth < 1 )
	{
		act("You make funny little shadow figures on the walls with your hands... heh.",ch,0,0,TO_CHAR);
		act("$n makes funny little shadow figures on the walls with $s hands... heh.",ch,0,0,TO_ROOM);
		return;
	}
	if (ch->mana < 250)
	{
		stc("You cannot focus enough on the Shadow deeply enough.\n\r",ch);
		return;
	}
	if (ch->mana < 250)
	{
		stc("You are far too exhausted to Shadow Pass.\n\r",ch);
		return;
	}
	if (ch->hit < 250)
	{
		stc("You are too weak to Shadow Pass.\n\r",ch);
		return;
	}
	if ( IS_SET( ch->war_flags, WAR_BATTLE ) )
	{
		stc("You're currently in a battle.\n\r",ch);
		return;
	}

	af.where     = TO_AFFECTS;
    af.type      = gsn_shadow_pass;
    af.duration  = ch->level / 20;
    af.location  = APPLY_NONE;
    af.modifier  = 0;
    af.bitvector = AFF_PASS_DOOR;
    affect_to_char( ch, &af );
    ch->mana -= 200;
    act( "$n turns sideways, becoming a thin line.", ch, NULL, NULL, TO_ROOM );
    send_to_char( "You turn sideways, becoming a thin line of shadow.\n\r", ch );
    return;
}
*/
void do_infuse( CHAR_DATA *ch, char *argument )
{
	char item[MIL], affect[MIL], mod[MIL];
	OBJ_DATA *obj;
	AFFECT_DATA *aff;

	argument = one_argument( argument, item );
	argument = one_argument( argument, affect );
	argument = one_argument( argument, mod );

	if (get_skill(ch, gsn_infuse) < 1)
	{
	stc("You dont know anything about this ancient art.\n\r", ch);
	return;
	}
	if (get_skill(ch, skill_lookup(affect)) < 100)
	{
	stc("You must be a Master in the affect to add.\n\r", ch);
	return;
	}
    if (item[0] == '\0' || affect[0] == '\0')
    {
    stc( "Infuse <item> <affect> [modifier]\n\r", ch );
    return;
    }
    if ( ( obj = get_obj_carry( ch, item, ch )) == NULL )
    {
    stc( "You don't have that item.\n\r", ch );
    return;
    }
    if (obj->item_type != ITEM_WEAPON)
    {
    stc( "Only on weapons.\n\r", ch );
    return;
    }

    if (ch->stam < 10000)
    {
    stc( "You cannot concentrate enough.\n\r", ch );
    return;
    }

	ch->stam -= 10000;
    /*aff = malloc( sizeof( *aff ) );*/
	aff = new_affect();
    aff->where = TO_OBJECT;
    aff->level = ch->level;
    aff->duration = -1;
    aff->bitvector = 0;
    aff->type = skill_lookup(affect);
    aff->location = APPLY_NONE;
    if (mod [0] != '\0')
	    aff->modifier = atoi (mod);
    else
	    aff->modifier = ch-> level;
    affect_to_obj( obj, aff );

    return;
}

void do_charge (CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj;
    CHAR_DATA *victim;
	int csize;
	int vsize;
	int diff;

    if ( !is_fighting(ch) || ( victim = ch->advancing_on ) == NULL )
    {
		stc( "{cYou aren't fighting anyone.\n\r", ch );
		return;
    }
    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
		act("{cBut $N is your friend!",ch,NULL,victim,TO_CHAR);
		return;
    }
    obj = get_eq_char( ch, WEAR_WIELD );
    if ( obj == NULL )
    {
		act("$n charges toward $N with a gutteral roar!\n\r",ch,NULL,victim,TO_NOTVICT);
		act("You charge toward $N with a gutteral roar!\n\r",ch,NULL,victim,TO_CHAR);
		act("$n charges toward you with a gutteral roar!\n\r",ch,NULL,victim,TO_VICT);
	}
	else
	{
		act("$n roars and charges toward $N, $p held high!\n\r",ch,obj,victim,TO_NOTVICT);
		act("You roar and charge toward $N, $p held high!\n\r",ch,obj,victim,TO_CHAR);
		act("$n roars and charges toward you, $p held high!\n\r",ch,obj,victim,TO_VICT);
	}
	csize = ch->size;
		csize += get_curr_stat(ch,STAT_CON);
		csize += get_curr_stat(ch,STAT_STR);
		csize += get_curr_stat(ch,STAT_DEX);
	vsize = victim->size;
		vsize += get_curr_stat(victim,STAT_CON);
		vsize += get_curr_stat(victim,STAT_STR);
		vsize += get_curr_stat(victim,STAT_DEX);
	diff = csize - vsize;
	/*diff = diff * 99 / 2 * 3;*/
	//diff = diff * diff * diff;
	if ( diff > number_percent() ) {
		victim->position = POS_SITTING;
	}
	damage(ch,victim,number_range(1,diff) + ch->level/3,gsn_bash,DAM_BASH,TRUE);
	return;
}

void do_mobforms( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *victim;
	int i;

    if (!is_fighting(ch) ||  ( victim = ch->advancing_on ) == NULL )
    {
		stc( "{cYou aren't fighting anyone.\n\r", ch );
		return;
    }
    if (IS_AFFECTED(ch,AFF_CHARM) && ch->master == victim)
    {
		act("{cBut $N is your friend!",ch,NULL,victim,TO_CHAR);
		return;
    }

	i = number_range( 1, 10 );
	switch(i)
	{
	case 1:
		act("$n assumes the form Heron Wading in the Rushes.",ch,NULL,victim,TO_ROOM);
		act("$n strikes in a smooth, graceful blow to $N.",ch,NULL,victim,TO_ROOM);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 2:
		act("$n assumes the form Heron Spreads it's Wings.",ch,NULL,victim,TO_ROOM);
		act("$n attacks, moving in an almost elegant arc, and ending the blow perfectly balanced to defend.",ch,NULL,victim,TO_ROOM);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 3:
		act("$n assumes the form Hummingbird Kisses the Honeyrose.",ch,NULL,victim,TO_ROOM);
		act("$n's attack flows with distinct precision, striking with a sharp, stabbing thrust at $N.",ch,NULL,victim,TO_ROOM);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 4:
		act("$n assumes the form Wood Grouse Dances.",ch,NULL,victim,TO_ROOM);
		act("$n dances to one side, feinting with quick thrusts.",ch,NULL,victim,TO_ROOM);
		act("$n assumes the form Wood Grouse Dances.",ch,NULL,victim,TO_VICT);
		act("$n dances to one side, feinting with quick thrusts.",ch,NULL,victim,TO_VICT);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 5:
		act("$n shifts into the form Swallow Takes Flight.",ch,NULL,victim,TO_ROOM);
		act("$n launches a flurry of attacks against $N.",ch,NULL,victim,TO_ROOM);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 6:
		act("$n shifts into the form Swallow Rides the Air.",ch,NULL,victim,TO_ROOM);
		act("$n jabs, thrusts, then spins into a forward assault against $N.",ch,NULL,victim,TO_ROOM);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 7:
		act("$n launches into Cat on a Hot Stand, crouching low, then springing forward.",ch,NULL,victim,TO_ROOM);
		act("$n launches into Cat on a Hot Stand, crouching low, then springing forward.",ch,NULL,victim,TO_VICT);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 8:
		act("$n charges forward in the form, Boar Rushes Downhill.",ch,NULL,victim,TO_ROOM);
		act("$n charges forward in the form, Boar Rushes Downhill.",ch,NULL,victim,TO_VICT);
		damage(ch,victim,number_range(ch->level/5, ch->level/2),gsn_heron,DAM_HARM,TRUE);
		return;
	case 9:
	case 10:
		return;
	break;
	}
	return;
}
