/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <time.h>
#include "aww.h"
#include "magic.h"
#include "interp.h"

/* skill tables */
/*
* The skill and spell table.
* Slot numbers must never be changed as they appear in #OBJECTS sections.
*/
#define SLOT(n)	n

const	struct	skill_type	skill_table	[MAX_SKILL]	=
{

/*
* Weaves.
*
*/
   	{
		"reserved",
    		{ -1, -1, -1, -1 }, /* level gained at - make useless */
    		{ 99, 99, 99, 99 }, /* train cost - cost should double */
		0, /* weave pointer to spell_fun */
		TAR_IGNORE, /* legal targets */
		POS_STANDING, /* minimum position */
		NULL, /* skill gsn pointer */
		SLOT( 0), /* for #OBJECT loading */
		0, /* min mana used when skill is at 100% */
		0, /* wait_state to inflict on user */
		"", /* damage message */
		"", /* wear off message for characters */
		"", /* wear off message for objects */
		STAT_NONE, /* weave one power primary sphere */
		0, /* minimum remort tier to gain */
		0, /* pointer to guild table */
		0, /* pointer to pre-required skill to learn */
		NULL, /* pointer to pre-required weave to learn */
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE, /*Is is an offensive weave?*/
		TALENT_NONE, /* affecting talent */
		0,
		"Nothing. Used as a placeholder."
	},


/* All Weaves */

/* Offensive Weaves -- in order of power.
*/
    {
		"burn",
		{1,1,1,1},
		{1,1,1,1},
		weave_burn,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 5),
		15,
		1,
		"burn","!Burn!","",
		STAT_FIRE,
		0,
		0,
		0,
		NULL,
		0, 0, 1, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use */
		TRUE,
		TALENT_ELEMENTALISM, /* affecting talent */
		25,
		"Fire weave. Minor offensive"
	},{
		"flameblade",
		{5,5,5,5},
		{1,1,1,1},
		weave_flameblade,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(26),
		15,
		1,
		"flameblade","!FlameBlade!","",
		STAT_FIRE,
		0,
		0,
		0,
		weave_burn,
		5, 0, 7, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use */
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		25,
		"Fire weave. Minor offensive."
	},{
		"airclub",
		{5,5,5,5},
		{1,1,1,1},
    	weave_airclub,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
		NULL,
		SLOT(26),
		15,
		1,
    	"airclub","!AirClub!","",
		STAT_AIR,
		0,
		0,
		0,
		0,
		0, 1, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		25,
		"Air weave. Minor offensive."
	},{
		"shatter",
		{5,5,5,5},
		{1,1,1,1},
    	weave_shatter,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
		NULL,
		SLOT(26),
		15,
		1,
    	"shatter","!Shatter!","",
		STAT_EARTH,
		0,
		0,
		0,
		0,
		1, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_EARTH_SINGING, /* affecting talent */
		25,
		"Earth weave. Minor offensive."
	},{
		"lash",
		{12,12,12,12},
		{1,1,1,1},
		weave_lash,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(62),
		15,
		1,
    	"lash", "!Lash!", "",
		STAT_AIR,
		0,
		0,
		0,
		weave_airclub,
		0, 8, 0, 8, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		30,
		"Air weave. Minor offensive."
	},{
		"flog",
		{18,18,18,18},
		{1,1,1,1},
		weave_flog,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(64),
		17,
		1,
    	"flog","!Flog!","",
		STAT_WATER,
		0,
		0,
		0,
		weave_lash,
		0, 0, 0, 1, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		25,
		"Water weave. Minor offensive."
	},{
		"lightning",
		{20,20,20,20},
		{1,1,1,1},
		weave_lightning,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(30),
		15,
		1,
		"lightning","!Lightning!","",
		STAT_FIRE,
		0,
		0,
		0,
		weave_glow,
        0, 8, 8, 8, 0, /* earth, air, fire, water, spirit - min sphere str
to use */
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		50,
		"Fire weave. Moderate offensive."
	},{
		"flay",
		{24,24,24,24},
		{1,1,1,1},
		weave_flay,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(63),
		20,
		1,
    	"flay","!Flay!","",
		STAT_AIR,
		0,
		0,
		0,
		weave_flog,
		0, 5, 5, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		75,
		"Air weave. Moderate offensive."
	},{
		"shield",
		{10,10,10,10},
		{2,2,2,2},
		weave_shield,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(67),
		12,
		3,
    	"","{cYour shield shimmers then fades away.","",
		STAT_AIR,
		0,
		0,
		0,
		weave_defense,
		7, 9, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Minor defensive."
	},{
		"shielding",
		{30,30,30,30},
		{4,4,4,4},
		weave_shielding,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(59),
		15,
		3,
    	"","!Shielding!","",
		STAT_ALL,
		6,
		0,
		0,
		weave_seeing,
		0, 12, 0, 0, 13, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
		TALENT_WARDING, /* affecting talent */
		0,
		"A shield of spirit stopping opponent from sensing source."
	},{
		"blinding",
		{35,35,35,35},
		{1,1,1,1},
		weave_blinding,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_blindness,
		SLOT( 4),
		5,
		2,
		"","{cYou can see again.","",
		STAT_EARTH,
		0,
		0,
		0,
		weave_cure_blinding,
		8, 7, 7, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Earth weave. Moderate offensive causing blindness."
	},{
		"calm",
		{45,45,45,45},
		{1,1,1,1},
		weave_calm,
		TAR_IGNORE,
		POS_FIGHTING,
		NULL,
		SLOT(509),
		30,
		1,
		"","{cYou have lost your peace of mind.","",
		STAT_SPIRIT,
		0,
		0,
		0,
		0,
		0, 0, 5, 5, 10, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Ceases fights and desire to fight."
	},{
		"air blast",
		{52,52,52,52},
		{4,4,4,4},
		weave_air_blast,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 2),
		35,
		1,
		"blast","!Air Blast!","",
		STAT_AIR,
		0,
		0,
		0,
		weave_flay,
		0, 10, 0, 0, 8, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		75,
		"Air weave. Moderate offensive."
	},{
		"earthquake",
		{175,175,175,175},
		{15,15,15,15},
		weave_earthquake,
		TAR_IGNORE,
		POS_FIGHTING,
		NULL,
		SLOT(23),
		50,
		1,
		"earthquake","!Earthquake!","",
		STAT_EARTH,
		10,
		0,
		0,
		weave_shatter,
		16, 11, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_EARTH_SINGING, /* affecting talent */
		90,
		"Earth weave. Moderate offensive. Area attack."
	},{
		"freeze finger",
		{75,75,75,75},
		{20,20,20,20},
    	weave_freeze_finger,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	25,
    	2,
    	"Freeze finger","!Freeze finger!","",
		STAT_WATER,
		1,
		0,
		0,
		0,
		0, 8, 0, 11, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		100,
		"Water weave. Major offensive."
	},{
		"spirit fist",
		{125,125,125,125},
		{10,10,10,10},
    	weave_spirit_fist,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	50,
    	2,
    	"Spirit fist","!Spirit Fist!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_glow,
		0, 0, 0, 0, 5, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		70,
		"Spirit weave. Moderate offensive."
	},{
		"firestorm",
		{75,75,75,75},
		{3,3,3,3},
		weave_firestorm,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(26),
		15,
		2,
    	"firestorm","!Firestorm!","",
		STAT_FIRE,
		0,
		0,
		0,
		weave_fire_bolt,
		0, 7, 7, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		100,
		"Fire weave. Major offensive."
	},{
		"sever",
		{90,90,90,90},
		{1,1,1,1},
		weave_sever,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(507),
		20,
		2,
    	"","!Sever!","",
		STAT_ALL,
		10,
		0,
		0,
		weave_shielding,
		0, 0, 17, 17, 18, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Removes ability to sense the source."
	},{
		"sicken",
		{120,120,120,120},
		{8,8,8,8},
		weave_sicken,
		TAR_CHAR_OFFENSIVE,
		POS_STANDING,
		NULL,
		SLOT( 1),
		5,
		2,
    	"","{cYou sweat out the poison.","",
		STAT_WATER,
		0,
		0,
		0,
		weave_weaken,
		0, 0, 0, 8, 7, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		90,
		"Water weave. Major offensive. Saps strength."
	},{
		"weaken",
		{145,145,145,145},
		{25,25,25,25},
    	weave_weaken,
    	TAR_CHAR_OFFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Weaken","!Weaken!","",
		STAT_WATER,
		1,
		0,
		0,
		weave_cure_disease,
		0, 0, 11, 10, 10, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		90,
		"Water weave. Major offensive."
	},{
		"spirit hammer",
		{150,150,150,150},
		{12,12,12,12},
    	weave_spirit_hammer,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	75,
    	2,
    	"Spirit Hammer","!Spirit Hammer!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_spirit_fist,
		0, 6, 0, 0, 5, /* earth, air, fire, water, spirit - min sphere str to use
*/
		TRUE,
	TALENT_HEALING, /* affecting talent */
		100,
		"Spirit weave. Major offensive."
	},{
		"hold",
    	{175,175,175,175},
    	{25,25,25,25},
    	weave_hold,
    	TAR_CHAR_OFFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Hold","!You break free of your hold!","",
		STAT_AIR,
		0,
		0,
		0,
		weave_air_blast,
		8, 9, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Offensive. Holds opponent."
	},{
		"spirit shatter",
		{175,175,175,175},
		{15,15,15,15},
    	weave_spirit_shatter,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	100,
    	2,
    	"Spirit Shatter","!Spirit Shatter!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_spirit_hammer,
		0, 0, 0, 7, 6, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		150,
		"Spirit weave. Major offensive."
	},{
		"mirrormirror",
		{240,240,240,240},
		{50,50,50,50},
    	weave_mirrormirror,
    	TAR_CHAR_OFFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	1000,
    	3,
    	"Mirror Mirror","!Mirror Mirror!","",
		STAT_ALL,
		60,
		0,
		0,
		weave_mind_shatter,
		30, 35, 35, 30, 35, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_NONE, /* affecting talent */
		0,
		"All spheres weave. Major offensive. Creates bubble of evil."
	},{
		"thunder blast",
		{185,185,185,185},
		{13,13,13,13},
    	weave_thunder_blast,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	50,
    	1,
    	"Thunder Blast","!Thunder Blast!","",
		STAT_AIR,
		0,
		0,
		0,
		weave_flay,
		0, 8, 0, 7, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		90,
		"Air weave. Moderate offensive."
	},{
		"thunder strike",
	    {205,205,205,205},
	    {17,17,17,17},
    	weave_thunder_blast,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	65,
    	1,
    	"Thunder Strike","!Thunder Strike!","",
		STAT_AIR,
		0,
		0,
		0,
		weave_thunder_blast,
		0, 8, 0, 9, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		120,
		"Air weave. Moderate offensive."
	},{
		"thunder lance",
		{225,225,225,225},
		{21,21,21,21},
    	weave_thunder_blast,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	90,
    	1,
    	"Thunder Lance","!Thunder Lance!","",
		STAT_AIR,
		0,
		0,
		0,
		weave_thunder_strike,
		0, 10, 0, 10, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		150,
		"Air weave. Moderate offensive."
	},{
		"geyser",
		{115,115,115,115},
		{10,10,10,10},
    	weave_geyser,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	30,
    	1,
    	"Geyser","!Geyser!","",
		STAT_WATER,
		0,
		0,
		0,
		0,
		8, 0, 7, 7, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		150,
		"Water weave. Moderate offensive."
	},{
		"drown",
		{145,145,145,145},
		{14,14,14,14},
    	weave_drown,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	50,
    	2,
    	"Drown","!Drown!","",
		STAT_WATER,
		0,
		0,
		0,
		weave_geyser,
		0, 10, 0, 10, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		200,
		"Water weave. Major offensive."
	},{
		"fire bolt",
		{130,130,130,130},
		{12,12,12,12},
    	weave_fire_bolt,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	NULL,
    	SLOT( 0),
    	50,
    	2,
    	"Fire Bolt","!Fire Bolt!","",
		STAT_FIRE,
		0,
		0,
		0,
		weave_flameblade,
		10, 0, 10, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		200,
		"Fire weave. Major offensive."
	},{
		"ice storm",
		{250,250,250,250},
		{20,20,20,20},
     	weave_ice_storm,
     	TAR_IGNORE,
     	POS_FIGHTING,
     	NULL,
     	SLOT( 0),
     	180,
     	3,
     	"Ice Storm","!ICE Storm!","",
  	 	STAT_WATER,
  	 	35,
  	 	0,
		0,
  	 	weave_freeze_finger,
  	 	0, 25, 0, 25, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		100,
		"Water weave. Major offensive. Area attack."
	},{
		"fork lightning",
		{250,250,250,250},
		{20,20,20,20},
     	weave_fork_lightning,
     	TAR_IGNORE,
     	POS_FIGHTING,
     	NULL,
     	SLOT( 0),
     	110,
     	3,
     	"Fork Lightning","!Fork Lightning!","",
  	 	STAT_FIRE,
  	 	35,
  	 	0,
		0,
  	 	weave_thunder_strike,
  	 	0, 20, 25, 20, 0, /* earth, air, fire, water, spirit - min sphere str
to use */
        	TRUE,
	TALENT_CLOUD_DANCING, /* affecting talent */
		100,
		"Fire weave. Major offensive. Area attack."
	},{
		"hellfire",
		{ 250, 250, 250, 250 },
		{ 50, 50, 50, 50},
		weave_hellfire,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		100,
		2,
		"Hellfire",
		"!Hellfire!",
		"Hellfire",
		STAT_FIRE,
		1,
		0,
		NULL,
		weave_fire_bolt,
		0, 0, 13, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		500,
		"Fire weave. Massive offensive."
    },{
		"purgatory",
		{ 250, 250, 250, 250 },
		{ 50, 50, 50, 50},
		weave_purgatory,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		150,
		2,
		"Purgatory",
		"Purgatory",
		"Purgatory",
		STAT_EARTH,
		1,
		0,
		NULL,
		weave_sicken,
		12, 12, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_EARTH_SINGING, /* affecting talent */
		500,
		"Earth weave. Massive offensive"
    },{
		"crushing blow",
		{ 250, 250, 250, 250 },
		{ 20, 20, 20, 20},
		weave_crushing_blow,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		90,
		2,
		"Crushing Blow",
		"Crushing Blow",
		"Crushing Blow",
		STAT_AIR,
		1,
		0,
		NULL,
		weave_flay,
		10, 13, 10, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_ELEMENTALISM, /* affecting talent */
		500,
		"Air weave. Massive offensive."
    },{
		"mind shatter",
		{ 250, 250, 250, 250 },
		{ 50, 50, 50, 50},
		weave_mind_shatter,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		100,
		2,
		"Mind Shatter",
		"Mind Shatter",
		"Mind Shatter",
		STAT_SPIRIT,
		1,
		0,
		NULL,
		weave_spirit_shatter,
		0, 8, 0, 0, 12, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	TRUE,
	TALENT_HEALING, /* affecting talent */
		500,
		"Spirit weave. Massive offensive."
    },{
		"reign of terror",
		{ 100, 100, 100, 100 },
		{ 100, 100, 100, 100},
		weave_reign_of_terror,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		250,
		3,
		"Reign of Terror",
		"Reign of Terror",
		"Reign of Terror",
		STAT_EARTH,
		20,
		0,
		NULL,
		NULL,
		17, 0, 17, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_NONE, /* affecting talent */
		600,
		"Earth weave. Incredible offensive."
    },{
		"reign of justice",
		{ 125, 125, 125, 125 },
		{ 125, 125, 125, 125},
		weave_reign_of_justice,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		300,
		3,
		"Reign of Justice",
		"Reign of Justice",
		"Reign of Justice",
		STAT_AIR,
		35,
		0,
		NULL,
		weave_reign_of_terror,
		0, 19, 22, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_NONE, /* affecting talent */
		750,
		"Air weave. Incredible offensive."
    },{
		"reign of blood",
		{ 150, 150, 150, 150  },
		{ 150, 150, 150, 150 },
		weave_reign_of_blood,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		350,
		3,
		"Reign of Blood",
		"Reign of Blood",
		"Reign of Blood",
		STAT_WATER,
		60,
		0,
		NULL,
		weave_reign_of_justice,
		0, 35, 35, 40, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_NONE, /* affecting talent */
		900,
		"Water weave. Incredible offensive."
    },{
		"reign of power",
		{ 200, 200, 200, 200  },
		{ 200, 200, 200, 200 },
		weave_reign_of_blood,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		400,
		3,
		"Reign of Power",
		"Reign of Power",
		"Reign of Power",
		STAT_WATER,
		90,
		0,
		NULL,
		weave_reign_of_blood,
		48, 50, 48, 52, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	TRUE,
	TALENT_NONE, /* affecting talent */
		1250,
		"Water weave. Incredible offensive."
    },{
		"reign of death",
		{ 220, 220, 220, 220  },
		{ 220, 220, 220, 220 },
		weave_reign_of_blood,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		500,
		3,
		"Reign of Death",
		"Reign of Death",
		"Reign of Death",
		STAT_WATER,
		120,
		0,
		NULL,
		weave_reign_of_power,
		63, 63, 60, 65, 58, /* earth, air, fire, water, spirit - min sphere str to use */
		TRUE,
        TALENT_NONE, /* affecting talent */
		1500,
		"Water weave. Tremendous offensive."
    },

/* Miscellaneous weaves
*/
    {
		"glow",
		{1,1,1,1},
		{1,1,1,1},
		weave_glow,
		TAR_IGNORE,
		POS_STANDING,
		NULL,
		SLOT(57),
		7,
		1,
    	"","!Glow!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		NULL,
		0, 10, 10, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use */
		FALSE,
		TALENT_ELEMENTALISM, /* affecting talent */
		0,
		"Fire weave. Create light."
	},{
		"darken",
		{1,1,1,1},
		{1,1,1,1},
		weave_darken,
		TAR_IGNORE,
		POS_STANDING,
		NULL,
		SLOT(57),
		7,
		1,
    	"","!Darken!","",
		STAT_EARTH,
		0,
		0,
		0,
		NULL,
		0, 10, 10, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use */
		FALSE,
		TALENT_ELEMENTALISM, /* affecting talent */
		0,
		"Earth weave. Dispels light from affected objects."
	},{
		"see light",
		{1,1,1,1},
		{1,1,1,1},
    	weave_see_light,
    	TAR_CHAR_SELF,
    	POS_STANDING,
    	NULL,
    	SLOT(513),
    	5,
    	1,
    	"","{cThe gold in your vision disappears.","",
		STAT_AIR,
		0,
		0,
		0,
		NULL,
		0, 6, 0, 0, 6, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Air weave. Enhanced senses."
	},{
		"see shadow",
		{1,1,1,1},
		{1,1,1,1},
		weave_see_shadow,
		TAR_CHAR_SELF,
		POS_STANDING,
		NULL,
		SLOT(18),
		5,
		1,
		"","{cThe red in your vision disappears.","",
		STAT_EARTH,
		0,
		0,
		0,
		NULL,
        	6, 0, 0, 0, 6, /* earth, air, fire, water, spirit - min sphere str
to use */
        	FALSE,
	TALENT_NONE, /* affecting talent */
		0,
		"Earth weave. Enhanced senses."
	},{
		"smell poison",
		{10,10,10,10},
		{1,1,1,1},
		weave_smell_poison,
		TAR_OBJ_INV,
		POS_STANDING,
		NULL,
		SLOT(21),
		5,
		1,
    	"","!Smell Poison!","",
		STAT_EARTH,
		0,
		0,
		0,
		0,
		8, 0, 0, 0, 7, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Earth weave. Detects poisons."
	},{
		"see hidden",
		{12,12,12,12},
		{1,1,1,1},
		weave_see_hidden,
		TAR_CHAR_SELF,
		POS_STANDING,
		NULL,
		SLOT(44),
		5,
		1,
		"","{cYou feel less aware of your surroundings.","",
		STAT_AIR,
		0,
		0,
		0,
		0,
		0, 6, 0, 0, 6, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_NONE, /* affecting talent */
		0,
		"Air weave. Enhanced senses."
	},{
		"see invis",
		{12,12,12,12},
		{1,1,1,1},
		weave_see_invis,
		TAR_CHAR_SELF,
		POS_STANDING,
		NULL,
		SLOT(19),
		5,
		1,
		"","{cYou no longer see invisible objects.","",
		STAT_EARTH,
		0,
		0,
		0,
		weave_see_hidden,
		6, 0, 0, 0, 6, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_NONE, /* affecting talent */
		0,
		"Earth weave. Enhanced senses."
	},{
		"seeing",
		{12,12,12,12},
		{2,2,2,2},
		weave_seeing,
		TAR_CHAR_SELF,
		POS_STANDING,
		NULL,SLOT(20),
		5,
		1,
    	"","{cThe Seeing fades.","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_see_invis,
		0, 0, 7, 0, 8, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_NONE, /* affecting talent */
		0,
		"Spirit weave. Enhanced senses."
	},{
		"bond",
		{60,60,60,60},
		{9,9,9,9},
		weave_bond,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT( 1),
		100,
		5,
    	"","{cYour bond is shattered!","",
		STAT_ALL,
		1,
		0,
		0,
		0,
		10, 10, 10, 10, 11, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"All spheres weave. Causes physical and mental link."
	},{
		"infuse",
		{40,40,40,40},
		{16,16,16,16},
		weave_infuse,
		TAR_OBJ_INV,
		POS_STANDING,
		NULL,
		SLOT(67),
		3000,
		10,
    	"!Infuse!","!Infuse!","",
		STAT_EARTH,
		18,
		0,
		0,
		0,
		18, 0, 15, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Earth weave. Allows creation of power wrought weapons."
	},{
		"weavesight",
		{ 100,100,100,100},
		{10,10,10,10},
        weave_weavesight,
        TAR_CHAR_DEFENSIVE,
        POS_STANDING,
        NULL,
        SLOT(0),
        50,
        3,
        "", "","",
        STAT_SPIRIT,
        2,
        0,
	0,
        0,
       	0, 10, 11, 0, 10, /* earth, air, fire, water, spirit - min sphere
str to use */
        FALSE,
	TALENT_ELEMENTALISM, /* affecting talent */
		0,
		"Spirit weave. Senses weaves on target."
	},{
		"delve",
		{ 100,100,100,100},
		{10,10,10,10},
        weave_delve,
        TAR_CHAR_DEFENSIVE,
        POS_STANDING,
        NULL,
        SLOT(0),
        50,
        3,
        "", "","",
        STAT_SPIRIT,
        3,
        0,
		0,
        0,
        10, 0, 0, 10, 11, /* earth, air, fire, water, spirit - min sphere
str to use */
	FALSE,
        TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Senses affects on target."
	},

/* Healing weave -- in order of healing strength
*/
    {
		"cure light",
		{5,5,5,5},
		{3,3,3,3},
		weave_cure_light,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(16),
		20,
		1,
		"","!Cure Light!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		NULL,
		0, 0, 0, 0, 1, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
        TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Minor wound healing."
	},{
		"cure serious",
		{25,25,25,25},
		{3,3,3,3},
		weave_cure_serious,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(61),
		30,
		2,
		"","!Cure Serious!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_cure_light,
		0, 0, 0, 6, 6, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Moderate healing."
	},{
		"cure critical",
		{35,35,35,35},
		{3,3,3,3},
		weave_cure_critical,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(15),
		40,
		3,
		"","!Cure Critical!","",
		STAT_SPIRIT,
		6,
		0,
		0,
		weave_cure_serious,
		0, 0, 11, 11, 11, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
        TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Major healing."
	},{
		"heal",
		{80,80,80,80},
		{5,5,5,5},
		weave_heal,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(28),
		100,
		3,
		"","!Heal!","",
		STAT_SPIRIT,
		10,
		0,
		0,
		weave_cure_critical,
		0, 0, 12, 12, 13, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Major healing."
	},{
		"repair flesh",
		{130,130,130,130},
		{9,9,9,9},
		weave_repair_flesh,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(28),
		100,
		4,
		"repair flesh","!repair flesh!","",
		STAT_ALL,
		15,
		0,
		0,
		weave_heal,
		10, 10, 10, 14, 15, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"All spheres weave. Masive healing."
	},{
		"mass heal",
		{200,200,200,200},
		{15,15,15,15},
		weave_mass_heal,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(28),
		250,
		3,
		"Mass Heal","!Mass Heal!","",
		STAT_ALL,
		20,
		0,
		0,
		weave_repair_flesh,
		10, 11, 12, 10, 12, /* earth, air, fire, water, spirit - min sphere str to
use */
       		FALSE,
	 TALENT_HEALING, /* affecting talent */
		0,
		"All spheres weave. Moderate healing. Entire group."
	},{
		"knit flesh",
		{50,50,50,50},
		{3,3,3,3},
		weave_knit_flesh,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(15),
		75,
		4,
		"","!Knit Flesh!","",
		STAT_SPIRIT,
		30,
		0,
		0,
		weave_heal,
		0, 0, 21, 21, 24, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Massive healing."
	},{
		"refresh",
		{ 100,100,100,100},
		{12,12,12,12},
        weave_refresh,
        TAR_CHAR_DEFENSIVE,
        POS_STANDING,
        NULL,
        SLOT(0),
        50,
        3,
        "", "","",
        STAT_SPIRIT,
        3,
        0,
		0,
        0,
      	0, 10, 0, 11, 11, /* earth, air, fire, water, spirit - min sphere str
to use */
        FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Restores movement points."
	},{
		"cure blinding",
		{1,1,1,1},
		{3,3,3,3},
		weave_cure_blinding,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT(14),
		5,
		1,
    	"","!Cure Blinding!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_cure_light,
		0, 5, 6, 0, 8, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Heals blinding."
	},{
		"cure poison",
		{5,5,5,5},
		{3,3,3,3},
		weave_cure_poison,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(43),
		20,
		1,
		"","!Cure Poison!","",
		STAT_WATER,
		0,
		0,
		0,
		weave_smell_poison,
		0, 6, 0, 6, 6, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Water weave. Cures minor poisons."
	},{
		"cure disease",
		{20,20,20,20},
		{3,3,3,3},
		weave_cure_disease,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(501),
		20,
		2,
		"","!Cure Disease!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_cure_light,
		6, 0, 6, 0, 8, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Cures moderate diseases."
	},{
		"remove curse",
		{30,30,30,30},
		{2,2,2,2},
		weave_remove_curse,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(35),
		5,
		3,
		"","!Remove Curse!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_heal,
		0, 8, 0, 0, 10, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_HEALING, /* affecting talent */
		0,
		"Spirit weave. Removes Harmful weaves from objects."
	},{
		"spinning the wheel",
		{ 225, 225, 225, 225  },
		{ 225, 225, 225, 225  },
		weave_spinning_the_wheel,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		10000,
		10,
		"Spinning the Wheel",
		"",
		"",
		STAT_ALL,
		90,
		0,
		NULL,
		weave_spinning_the_wheel,
		50, 50, 50, 50, 50, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
	        TALENT_HEALING, /* affecting talent */
		0,
		"All spheres weave. Reconnects target to the source."
    },

/* Defensive Weaves -- in order of defensive strength
*/
    {
		"defense",
		{8,8,8,8},
		{1,1,1,1},
		weave_defense,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT( 1),
		5,
		3,
    	"","You feel less defended.","",
		STAT_AIR,
		0,
		0,
		0,
		0,
		7, 8, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
	        TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Minor defensive."
	},{
		"vigor",
		{18,18,18,18},
		{1,1,1,1},
		weave_vigor,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT( 3),
		5,
		3,
    	"","{cYou feel less envigored.","$p looks less vigorous.",
		STAT_WATER,
		0,
		0,
		0,
		0,
		6, 0, 0, 9, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Water weave. Minor enhancement."
	},{
		"invert",
		{40,40,40,40},
		{2,2,2,2},
		weave_invert,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		&gsn_invis,
		SLOT(29),
		5,
		1,
    	"","{cThe inverted light fades.","{c$p fades into view.",
		STAT_AIR,
		0,
		0,
		0,
		weave_shield,
		0, 9, 0, 9, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Causes target to be hidden from sight."
	},{
		"fire shield",
		{120,120,120,120},
		{5,5,5,5},
		weave_fire_shield,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(67),
		50,
		3,
    	"","{cThe flames surrounding you fade away.","",
		STAT_FIRE,
		0,
		0,
		0,
		weave_firestorm,
		0, 9, 9, 0, 5, /* earth, air, fire, water, spirit - min sphere str to use
*/
        	FALSE,
	TALENT_WARDING, /* affecting talent */
		0,
		"Fire weave. Creates a shield of fire to burn attackers."
	},{
		"barrier",
		{60,60,60,60},
		{15,15,15,15},
     	weave_barrier,
     	TAR_CHAR_DEFENSIVE,
     	POS_FIGHTING,
     	NULL,
     	SLOT( 0),
     	100,
     	3,
     	"Barrier","!Your barrier has been shattered!!","",
  	 	STAT_AIR,
  	 	10,
  	 	0,
		0,
  	 	weave_invert,
  	 	14, 14, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
		TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Major defensive. Blocks damage."
	},{
		"advanced shield",
		{20,20,20,20},
		{8,8,8,8},
		weave_advanced_shield,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(67),
		12,
		3,
    	"","{cYour advanced shield fizzles then fades away.","",
		STAT_AIR,
		5,
		0,
		0,
		weave_invert,
		10, 11, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
		TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Moderate defensive."
	},{
		"greater shield",
		{40,40,40,40},
		{16,16,16,16},
		weave_greater_shield,
		TAR_CHAR_DEFENSIVE,
		POS_STANDING,
		NULL,
		SLOT(67),
		12,
		3,
    	"","{cYour greater shield flares one last time, then fades away.","",
		STAT_AIR,
		12,
		0,
		0,
		weave_advanced_shield,
		13, 15, 0, 0, 11, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
		TALENT_WARDING, /* affecting talent */
		0,
		"Air weave. Massive defensive."
	},{
		"tower of defense",
		{ 25, 25, 25, 25  },
		{ 25, 25, 25, 25  },
		weave_tower_of_defense,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		500,
		3,
		"Tower of Defense",
		"A slight aura of purple and green disappears from around you.",
		"Tower of Defense",
		STAT_WATER,
		20,
		0,
		NULL,
		weave_greater_shield,
		15, 15, 0, 17, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
		TALENT_WARDING, /* affecting talent */
		0,
		"Water weave. Massive defensive."
    },{
		"tower of mind",
		{ 50, 50, 50, 50  },
		{ 50, 50, 50, 50  },
		weave_tower_of_mind,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		500,
		4,
		"Tower of Mind",
		"A slight aura of purple and red disappears from around you.",
		"Tower of Mind",
		STAT_WATER,
		35,
		0,
		NULL,
		weave_greater_shield,
		0, 20, 25, 25, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
		TALENT_WARDING, /* affecting talent */
		0,
		"Water weave. Massive defensive."
    },{
		"tower of war",
		{ 75, 75, 75, 75  },
		{ 75, 75, 75, 75  },
		weave_tower_of_war,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		500,
		4,
		"Tower of War",
		"A slight aura of purple and black disappears from around you.",
		"Tower of War",
		STAT_WATER,
		60,
		0,
		NULL,
		weave_greater_shield,
		35, 33, 0, 35, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_WARDING, /* affecting talent */
		0,
		"Water weave. Massive defensive."
    },{
		"tower of justice",
		{ 150, 150, 150, 150  },
		{ 150, 150, 150, 150  },
		weave_tower_of_justice,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		500,
		5,
		"Tower of Justice",
		"A slight aura of purple and gray disappears from around you.",
		"Tower of Justice",
		STAT_WATER,
		90,
		0,
		NULL,
		weave_greater_shield,
		48, 48, 0, 52, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_WARDING, /* affecting talent */
		0,
		"Water weave. Incredible defensive."
    },{
		"tower of power",
		{ 225, 225, 225, 225  },
		{ 225, 225, 225, 225  },
		weave_tower_of_power,
		TAR_CHAR_DEFENSIVE,
		POS_FIGHTING,
		NULL,
		SLOT( 0),
		500,
		5,
		"Tower of Power",
		"A slight aura of purple and white disappears from around you.",
		"Tower of Power",
		STAT_WATER,
		120,
		0,
		NULL,
		weave_greater_shield,
		65, 0, 68, 68, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
        TALENT_WARDING, /* affecting talent */
		0,
		"Water weave. Tremendous defensive."
    },

/* Enhancement Weaves -- in order of addititive value
*/
    {
		"bigger",
		{45,45,45,45},
		{12,12,12,12},
    	weave_bigger,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Bigger","Your muscles shrink.","",
		STAT_EARTH,
		3,
		0,
		0,
		weave_endurance,
		11, 0, 0, 11, 10, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
        TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Earth weave. Strength enhancement."
    },{
		"large",
		{45,45,45,45},
		{18,18,18,18},
    	weave_large,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Large","You shrink back down to size.","",
		STAT_EARTH,
		28,
		0,
		0,
		weave_endurance,
		22, 0, 0, 22, 20, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Earth weave. Strength enhancement."
	},{
		"quicken",
		{45,45,45,45},
		{12,12,12,12},
     	weave_quicken,
     	TAR_CHAR_DEFENSIVE,
     	POS_STANDING,
     	NULL,
     	SLOT( 0),
     	100,
     	3,
     	"Quicken","You no longer feel as quick.","",
		STAT_AIR,
		4,
		0,
		0,
		weave_large,
		0, 11, 5, 11, 10, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Air weave. Dexterity enhancement."
	},{
		"speed",
		{45,45,45,45},
		{18,18,18,18},
     	weave_speed,
     	TAR_CHAR_DEFENSIVE,
     	POS_STANDING,
     	NULL,
     	SLOT( 0),
     	100,
     	3,
     	"Speed","You are no longer as fast and coordinated.","",
		STAT_AIR,
		30,
		0,
		0,
		weave_large,
		0, 10, 22, 22, 20, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Air weave. Dexterity enhancement."
	},{
		"fortitude",
		{45,45,45,45},
		{12,12,12,12},
    	weave_fortitude,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Fortitude","You no longer feel as sturdy.","",
		STAT_EARTH,
		5,
		0,
		0,
		weave_vigor,
		12, 12, 0, 12, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Earth weave. Constitution enhancement."
	},{
		"endurance",
		{45,45,45,45},
		{18,18,18,18},
    	weave_endurance,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Endurance","Your body returns to its normal state.","",
		STAT_EARTH,
		32,
		0,
		0,
		weave_vigor,
		24, 24, 0, 24, 0, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Earth weave. Constitution enhancement."
	},{
		"sixth sense",
		{45,45,45,45},
		{12,12,12,12},
    	weave_sixth_sense,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Sixth Sense","You feel like you've lost some sense.","",
		STAT_AIR,
		6,
		0,
		0,
		weave_speed,
		0, 12, 0, 11, 11, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
        TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Air weave. Instinct enhancement."
	},{
		"hunters instinct",
		{45,45,45,45},
		{18,18,18,18},
    	weave_hunters_instinct,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Hunters Instinct","You feel less aware.","",
		STAT_AIR,
		34,
		0,
		0,
		weave_speed,
		0, 24, 0, 22, 22, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Air weave. Instinct enhancement."
	},{
		"innervation",
		{45,45,45,45},
		{12,12,12,12},
    	weave_innervation,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Innervation","Your nerves no longer tingle.","",
		STAT_FIRE,
		7,
		0,
		0,
		weave_hunters_instinct,
		0, 12, 13, 0, 12, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Fire weave. Reflex enhancement."
	},{
		"panther reflex",
		{45,45,45,45},
		{18,18,18,18},
    	weave_panther_reflex,
    	TAR_CHAR_DEFENSIVE,
    	POS_STANDING,
    	NULL,
    	SLOT( 0),
    	100,
    	3,
    	"Panther Reflex","Your senses dull..","",
		STAT_FIRE,
		36,
		0,
		0,
		weave_hunters_instinct,
		0, 24, 26, 0, 24, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_EARTH_SINGING, /* affecting talent */
		0,
		"Fire weave. Reflex enhancement."
	},

/* Travel Weaves
*/
    {
		"gateway",
		{150,150,150,150},
		{5,5,5,5},
		weave_gateway,
		TAR_IGNORE,
		POS_FIGHTING,
		NULL,
		SLOT(83),
		80,
		3,
		"","!Gateway!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_portal,
		10, 10, 10, 10, 10, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
	        TALENT_NONE, /* affecting talent */
		0,
		"All spheres weave. Travelling."
	},{
		"portal",
		{100,100,100,100},
		{2,2,2,2},
    	weave_portal,
    	TAR_IGNORE,
    	POS_STANDING,
    	NULL,
    	SLOT(519),
    	100,
    	4,
    	"","!Portal!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		0,
		10, 10, 10, 10, 10, /* earth, air, fire, water, spirit - min sphere str to
use */
        	FALSE,
	TALENT_NONE, /* affecting talent */
		0,
		"All spheres weave. Travelling."
	},{
		"teleport",
		{-1,-1,-1,-1},
		{0,0,0,0},
		weave_teleport,
		TAR_CHAR_SELF,
		POS_FIGHTING,
		NULL,
		SLOT( 2),
		35,
		5,
		"","!Teleport!","",
		STAT_SPIRIT,
		0,
		0,
		0,
		weave_gateway,
		10, 10, 10, 10, 10, /* earth, air, fire, water, spirit - min sphere str to
use */
		FALSE,
        TALENT_NONE, /* affecting talent */
		0,
		"Spirit weave. Summoning."
	},

/* END WEAVES */

/*
* other combat/weapon
*/

    {
    	"shadow pass",
    	{ -1, -1, -1, -1 },
    	{ 0, 0, 0, 0},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_shadow_pass,
    	SLOT( 0),
    	0,
    	6,
    	"Shadow Pass",
    	"You turn, phasing back into reality", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Myrddraal skill."
    },

    {
    	"tail",
    	{ -1, -1, -1, -1 },
    	{ 0, 0, 0, 0},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_tail,
    	SLOT( 0),
    	0,
    	6,
    	"tail slap", "!tail slap!",	"",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Physcial attack."
    },

    {
    	"claw",
    	{ -1, -1, -1, -1 },
    	{ 0, 0, 0, 0},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_claw,
    	SLOT( 0),
    	0,
    	6,
    	"claw", "!claw!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Physical attack."
    },

    {
    	"veil",
    	{ -1, -1, -1, -1 },
    	{ 0, 0, 0, 0},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_veil,
    	SLOT( 0),
    	0,
    	6,
    	"veil", "Your veil slips.",	"",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Aiel enhancement skill."
    },

    {
    	"bite",
    	{ -1, -1, -1, -1 },
    	{ 0, 0, 0, 0},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_bite,
    	SLOT( 0),
    	0,
    	6,
    	"bite", "!bite!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Physcial attack."
    },

	{
    	"bow",
    	{ 40, 40, 40, 40 },
    	{ 8, 8, 8, 8 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_bow,
    	SLOT( 0),
    	0,
    	0,
    	"arrow", "!Bow!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Ranged weapon skill."
    },

    {
		"axe",
		{ 1, 1, 1, 1 },
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_axe,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Axe!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
    	"dagger",
    	{ 1,  1,  1,  1 },
    	{ 2, 2, 2, 2},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_dagger,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Dagger!",	"",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"flail",
		{ 1,  1, 1,  1 },
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_flail,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Flail!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"mace",
		{ 1, 1, 1, 1 },
		{ 5, 5, 5, 5},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_mace,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Mace!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"polearm",
		{ 1, 1, 1,  1 },
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_polearm,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Polearm!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"shield block",
		{ 1, 1, 1, 1 },
		{ 6, 6, 6, 6},
		spell_null,
		TAR_IGNORE,
		POS_FIGHTING,
		&gsn_shield_block,
		SLOT(0),
		0,
		0,
		"", "!Shield!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Use shield to block attacks."
    },

    {
		"spear",
		{ 1, 1, 1, 1 },
		{ 4, 4, 4, 4},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_spear,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Spear!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"staff",
		{ 1, 1, 1, 1 },
		{ 4, 4, 4, 4 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_staff,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Staff!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"sword",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_sword,
    	SLOT( 0),
    	0,
    	0,
    	"", "!sword!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"whip",
		{ 1, 1, 1, 1},
		{ 5, 5, 5, 5},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_whip,
    	SLOT( 0),
    	0,
    	0,
    	"", "!Whip!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"net",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_net,
    	SLOT( 0),
    	0,
    	0,
    	"", "!net!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"hammer",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_hammer,
    	SLOT( 0),
    	0,
    	0,
    	"", "!hammer!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"sling",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_sling,
    	SLOT( 0),
    	0,
    	0,
    	"", "!sling!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"swordbreaker",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_swordbreaker,
    	SLOT( 0),
    	0,
    	0,
    	"", "!swordbreaker!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"sap",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_sap,
    	SLOT( 0),
    	0,
    	0,
    	"", "!sap!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"shortsword",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_shortsword,
    	SLOT( 0),
    	0,
    	0,
    	"", "!shortsword!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"lance",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_lance,
    	SLOT( 0),
    	0,
    	0,
    	"", "!lance!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
		"ashandarei",
		{ 1, 1, 1, 1},
		{ 6, 6, 6, 6},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_ashandarei,
    	SLOT( 0),
    	0,
    	0,
    	"", "!ashandarei!", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Weapon skill."
    },

    {
    	"rush",
    	{ 1, 1, 1, 1 },
    	{ 10, 10, 10, 10 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_rush,
    	SLOT( 0),
    	0,
    	6,
    	"",
    	"", "",
		STAT_NONE,
		0,
		0,
		NULL,
		0,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Charge into combat."
    },

    {
    	"backstab",
    	{ 99, 99, 99, 99 },
    	{ 8, 8, 8, 8 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_backstab,
    	SLOT( 0),
    	0,
    	24,
    	"backstab", "!Backstab!", "",
		STAT_NONE,
		0,
		0,
		"sneak",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Causes enhanced damage with intial attack."
    },

    {
    	"ambush",
    	{ 99, 99, 99, 99 },
    	{ 26, 26, 26, 26 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_assassinate,
    	SLOT( 0),
    	7,
    	24,
    	"ambush", "!ambush!", "",
		STAT_NONE,
		0,
		0,
		"backstab",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Causes enhanced damage on initial attack plus give bonus weapon attacks."
    },

    {
		"bash",
		{ 55, 55, 55, 55 },
		{ 8, 8, 8, 8},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_bash,
    	SLOT( 0),
    	0,
    	24,
    	"bash", "!Bash!", "",
		STAT_NONE,
		0,
		0,
		"shield block",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Use body to bash into opponent."
    },

    {
		"berserk",
		{ 63, 63, 63, 63 },
		{ 8, 8, 8, 8 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_berserk,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"{cYou feel your pulse slow down.",
    	"",
		STAT_NONE,
		0,
		0,
		"void",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"State of mind granting enhanced physical prowess."
    },

    {
    	"counter",
    	{ 55, 55, 55, 55 },
    	{ 8, 8, 8, 8 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_counter,
    	SLOT( 0),
    	0,
    	0,
    	"counterattack",
    	"!Counter!",
    	"",
		STAT_NONE,
		0,
		0,
		"dodge",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Use weapon to parry an attack and counter with an opposing attack."
    },

    {
		"dirt kicking",
		{ 33, 33, 33, 33 },
		{ 6, 6, 6, 6 },
		spell_null,
		TAR_IGNORE,
		POS_FIGHTING,
		&gsn_dirt,
		SLOT( 0),
		0,
		24,

		"kicked dirt",
		"You rub the dirt out of your eyes.",
		"",
		STAT_NONE,
		0,
		0,
		"kick",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Kick dirt into opponent's eyes to causes blindness."
    },

    {
    	"disarm",
    	{ 73, 53, 25, 15 },
    	{ 8, 7, 6, 4},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_disarm,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"!Disarm!",
    	"",
		STAT_NONE,
		0,
		0,
		"parry",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Causes opponent to drop primary weapon."
    },

    {
    	"dodge",
    	{ 20, 20,  20, 20 },
    	{ 7, 7, 7, 7 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_dodge,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!Dodge!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Dodge some physical attacks."
    },

    {
    	"dual weaponry",
    	{ 10, 10,  10, 10 },
    	{ 7, 7, 7, 7 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_dual,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!Dual!",
    	"",
		STAT_NONE,
		0,
		0,
		"second attack",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Use two weapons at once with proficiency."
    },

    {
    	"enhanced damage",
    	{ 30, 30, 30, 30 },
    	{ 10, 10, 10, 10 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_enhanced_damage,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!Enhanced Damage!",
    	"",
		STAT_NONE,
		0,
		0,
		"second attack",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Cause additional damage with a successful weapon hit."
    },

    {
		"envenom",
		{ 100, 100, 100, 100 },
		{ 12, 12, 12, 12 },
		spell_null,
		TAR_IGNORE,
		POS_RESTING,
		&gsn_envenom,
		SLOT(0),
		0,
		36,
		"",
		"!Envenom!",
		"",
		STAT_NONE,
		2,
		0,
		"brew",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Lace weapons with a minor poison."
    },
	/* Unarmed combat skills - Dalsor */
    {
		"hand to hand",
		{ 5, 5, 5, 5 },
		{ 5, 5, 5, 5 },
		spell_null,
		TAR_IGNORE,
		POS_FIGHTING,
		&gsn_hand_to_hand,
		SLOT( 0),
		0,
		0,
		"",
		"!Hand to Hand!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Basic unarmed combat skill. Required for all unarmed combat"
    },
    {
    	"punch",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_punch,
    	SLOT( 0),
    	0,
    	6,
    	"punch",
    	"!punch!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Using a balled fist to punch for damage."
    },
    {
    	"sidekick",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_sidekick,
    	SLOT( 0),
    	0,
    	6,
    	"sidekick",
    	"!sidekick!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Leaning to the side and thrusting the foot forward for a midsection kick."
    },
    {
    	"jumpkick",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_jumpkick,
    	SLOT( 0),
    	0,
    	6,
    	"jumpkick",
    	"!jumpkick!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Jumping up and forward to kick into an opponent's chest or abdomen area."
    },
    {
    	"leapkick",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_leapkick,
    	SLOT( 0),
    	0,
    	6,
    	"leapkick",
    	"!leapkick!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Leaping upward to kick into an opponent's chest or face area."
    },
    {
    	"headbutt",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_headbutt,
    	SLOT( 0),
    	0,
    	6,
    	"headbutt",
    	"!headbutt!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Using the forehead to bash into an opponent's nose or mouth area."
    },
    {
    	"elbow",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_elbow,
    	SLOT( 0),
    	0,
    	6,
    	"elbow",
    	"!elbow!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Using the elbow to bash into an opponent's nose, mouth, or temple area."
    },
    {
    	"knee",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_knee,
    	SLOT( 0),
    	0,
    	6,
    	"knee",
    	"!knee!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Using the knee to bash into an opponent's groin or abdoment area."
    },
    {
    	"legsweep",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_legsweep,
    	SLOT( 0),
    	0,
    	6,
    	"legsweep",
    	"!legsweep!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Using the feet to kick an opponent's legs from under them."
    },
    {
    	"hookkick",
    	{ 2, 2, 2, 2 },
    	{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_hookkick,
    	SLOT( 0),
    	0,
    	6,
    	"hookkick",
    	"!hookkick!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"A kick aimed toward an opponent's temple area."
    },
    {
    	"kick",
    	{ 6, 6, 6, 6 },
    	{ 6, 6, 6, 6},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_kick,
    	SLOT( 0),
    	0,
    	6,
    	"kick",
    	"!Kick!",
    	"",
		STAT_NONE,
		0,
		0,
		"hand to hand",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Basic straight on kick."
    },

    {
    	"spinkick",
    	{ 75, 75, 75, 75 },
    	{ 5, 5, 5, 5 },
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_spinkick,
    	SLOT( 0),
    	0,
    	6,
    	"spinning kick",
    	"!Spinning Kick!",
    	"",
		STAT_NONE,
		0,
		0,
		"kick",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spinning kick designed to use force of spin to create extra damage."
    },
    {
    	"footsweep",
    	{ 75, 75, 75, 75 },
    	{ 6, 6, 6, 6},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_sweepkick,
    	SLOT( 0),
    	0,
    	6,
    	"sweeping kick",
    	"!Sweeping Kick!",
    	"",
		STAT_NONE,
		0,
		0,
		"kick",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spinning sweep to knock opponent's legs from under them."
    },
    {
    	"parry",
    	{ 32, 32, 32, 32 },
    	{ 4, 4, 4, 4 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_parry,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!Parry!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Using a weapon to block a physical attack."
    },
    {
    	"rescue",
    	{ 27, 27, 27, 27 },
    	{ 9, 9, 9, 9 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_rescue,
    	SLOT( 0),
    	0,
    	12,
    	"",
    	"!Rescue!",
    	"",
		STAT_NONE,
		0,
		0,
		"tank",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Place yourself in front of a friend and assume their position in a fight."
    },

	{
		"tank",
		{ 1, 1, 1, 1 },
		{ 29, 29, 29, 29 },
		spell_null,
		TAR_IGNORE,
		POS_FIGHTING,
		&gsn_tank,
		SLOT( 0),
		0,
		12,
		"", 
		"!Tank!","",
		STAT_NONE,
		25,
		0,
		NULL,
		NULL, 
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"When leading the group, the tank assumes all damage, unless from an area attack skill or weave."
	},


    {
		"engage",
		{ 13, 13, 13, 13 },
		{ 7, 7, 7, 7 },
	    spell_null,
	    TAR_IGNORE,
	    POS_FIGHTING,
	    &gsn_engage,
	    SLOT( 0),
	    0,
	    24,
	    "",
	    "!Engage!",
	    "",
		STAT_NONE,
		0,
		0,
		"parry",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Engage multiple enemies in physical combat."
    },
    {
		"trip",
		{ 35, 30, 1, 15 },
		{ 8, 7, 2, 4},
		spell_null,
		TAR_IGNORE,
		POS_FIGHTING,
		&gsn_trip,
		SLOT( 0),
		0,
		24,
		"trip",
		"!Trip!",
		"",
		STAT_NONE,
		0,
		0,
		"kick",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Trip an opponent and force them to the ground."
    },
    {
    	"second attack",
    	{ 30, 30, 30, 30 },
    	{ 13, 13, 13, 13 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_second_attack,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!Second Attack!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gain an additional attack with the primary weapon."
    },
    {
    	"third attack",
    	{ 70, 70, 70, 70 },
    	{ 14, 14, 14, 14 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_third_attack,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!Third Attack!",
    	"",
		STAT_NONE,
		0,
		0,
		"second attack",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gain an additional attack with the primary weapon."
    },
    {
		"void",
		{ 5, 5, 5, 5 },
		{ 5, 5, 5, 5 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_void,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"{wYou lose the {rVoid.{c",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Focus on the Flame inside the Void for additional concentration."
    },
    {
		"scan",
		{ 2, 2, 2, 2 },
		{ 2, 2, 2, 2 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_scan,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"See into adjacent rooms."
    },


{
	"far sight", { 2, 2, 2, 2 }, { 12, 12, 12, 12 },
	spell_null,                                                                                                                                                                                                                          
	TAR_IGNORE,                                                                                                                                                                                                                          
	POS_FIGHTING,                                                                                                                                                                                                                        
	&gsn_far_sight,
	SLOT( 0),                                                                                                                                                                                                                            
	0,                                                                                                                                                                                                                                   
	24,                                                                                                                                                                                                                                  
	"",                                                                                                                                                                                                                                  
	"",                                                                                                                                                                                                                                  
	"",
	STAT_NONE,
	0,
	0,
	NULL, 
	NULL, 
	0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use */  
	FALSE, 
	TALENT_NONE, /* affecting talent */
	0,
	"See farther on the game map."
},

    {
		"hurricane",
		{ 109, 109, 109, 109 },
		{ 15, 15, 15, 15},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_hurricane,
    	SLOT( 0),
    	0,
    	12,
    	"hurricane",
    	"!Hurricane!",	"",
		STAT_NONE,
		3,
		0,
		"tornado",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Attack more than one enemy with a spinning attack."
    },
    {
		"tornado",
		{ 109, 109, 109, 109 },
		{ 15, 15, 15, 15},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_tornado,
    	SLOT( 0),
    	0,
    	16,
    	"tornado",
    	"!Tornado!",	"",
		STAT_NONE,
		2,
		0,
		"whirlwind",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Attack more than one enemy with a spinning attack."
    },
    {
		"whirlwind",
		{ 109, 109, 109, 109 },
		{ 15, 15, 15, 15},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_whirlwind,
    	SLOT( 0),
    	0,
    	20,
    	"whirlwind",                     "!Whirlwind!",	"",
		STAT_NONE,
		1,
		0,
		"enhanced damage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Attack more then one enemy with a spinning attack."
    },
    {
		"treesing",
		{ 15, 15, 15, 15 },
		{ 17, 17, 17, 17},
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_treesing,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"!Treesing!",
    	"",
		STAT_NONE,
		1,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Ogier skill. Sing to trees to create objects."
    },
    {
		"adorn",
		{ 1, 1, 1, 1 },
		{ 25, 25, 25, 25},
    		spell_null,
    		TAR_IGNORE,
    		POS_STANDING,
    		&gsn_adorn,
    		SLOT( 0),
    		0,
    		24,
    		"",
    		"!Layout!",
    		"",
		STAT_NONE,
		1,
		0,
		"jewelcraft",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Add some personality to your crafted items with emblems, engravings and other adornments."
    },
    {
	    	"sew",
		{ 1, 1, 1, 1 },
		{ 100, 100, 100, 100 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_sew,
		SLOT( 0),
		0,
		24,
		"",
		"!Sew!",
		"",
		STAT_NONE,
		1,
		0,
		"forage",
		NULL,
		0, 0, 0, 0, 0,
		FALSE,
		TALENT_NONE,
		0,
		"Craft clothing and other useful tools from cloth and fur."
    },
    {
	    	"hem",
		{ 1, 1, 1, 1 },
		{ 19, 19, 19, 19 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_hem,
		SLOT( 0),
		0,
		24,
		"",
		"!Hem!",
		"",
		STAT_NONE,
		1,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0,
		FALSE,
		TALENT_NONE,
		0,
		"Part of the tailoring skill set."
    },
    {
	    	"jewelcraft",
		{ 1, 1, 1, 1 },
		{ 100, 100, 100, 100 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_jewelcraft,
		SLOT( 0),
		0,
		24,
		"",
		"!Jewelcraft!",
		"",
		STAT_NONE,
		1,
		0,
		"forage",
		NULL,
		0, 0, 0, 0, 0,
		FALSE,
		TALENT_NONE,
		0,
		"Craft jewelry and other useful tools using jewels."
    },
    {
        "grip",
    	{ 10, 10, 10, 10 },
    	{ 10, 10, 10, 10 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_grip,
    	SLOT( 0),
    	0,
    	24,
    	"Grip",
    	"!Grip!",
    	"",
		STAT_NONE,
		1,
		0,
		"enhanced damage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Prevents disarming."
    },
    {
    	"demand",
    	{ -1, -1, -1, -1 },
    	{ 0, 0, 0, 0},
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_demand,
    	SLOT( 0),
    	0,
    	24,
    	"Demand",
    	"!Demand!",
    	"",
		STAT_NONE,
		1,
		0,
/*		"rage",  removed to get it off tree list */
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Bully shop owners into giving monies."
    },
    {
    	"drag",
    	{ 39, 39, 39, 39 },
    	{ 16, 16, 16, 16 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_drag,
    	SLOT( 0),
    	0,
    	24,
    	"Drag",
    	"!Drag!",
    	"",
		STAT_NONE,
		1,
		0,
		"engage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Drag objects or enemies around."
    },

    {
    	"brew",
    	{ 89, 89, 89, 89 },
    	{ 21, 21, 21, 21 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_brew,
    	SLOT( 0),
    	0,
    	24,
    	"Brew",
    	"!Brew!",
    	"",
		STAT_NONE,
		1,
		0,
		"herbal knowledge",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Create tonics, remedies and poisons from herbs."
    },

    {
    	"sharpen",
    	{ 69, 69, 69, 69 },
    	{ 20, 20, 20, 20},
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_sharpen,
    	SLOT( 0),
    	0,
    	24,
    	"Sharpen",
    	"!Sharpen!",
    	"",
		STAT_NONE,
		1,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Sharpen weapons to increase damage."
    },
    {
		"battlecry",
		{ 77, 77, 77, 77 },
		{ 13, 13, 13, 13},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_battlecry,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"You feel your added courage slip away.",
    	"",
		STAT_NONE,
		1,
		0,
		"rage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Instill allies with courage through a rallying cry."
    },
    {
		"rage",
		{ 77, 77, 77, 77 },
		{ 13, 13, 13, 13},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_rage,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"You feel your rage slip away.",
    	"",
		STAT_NONE,
		1,
		0,
		"berserk",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Use anger to your advantage."
    },
    {
    	"advanced rescue",
    	{ 101, 101, 101, 101 },
    	{ 13, 13, 13, 13},
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_advanced_rescue,
    	SLOT( 0),
    	0,
    	12,
    	"",
    	"!Rescue!",
    	"",
		STAT_NONE,
		1,
		0,
		"rescue",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Increases chance to resuce friends from combat."
    },
    {
    	"woodcraft",
	{ 1, 1, 1, 1 },
	{ 100, 100, 100, 100 },
    	spell_null,
    	TAR_IGNORE,
    	POS_RESTING,
    	&gsn_woodcraft,
    	SLOT( 0),
    	0,
    	24,
    	"Woodcraft",
    	"!Woodcraft!",
    	"",
		STAT_NONE,
		3,
		0,
		"forage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
TALENT_NONE, /* affecting talent */
		0,
		"Craft weapons and other useful tools from wood."
    },
    {
    	"forge",
	{ 1, 1, 1, 1 },
	{ 100, 100, 100, 100 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_forge,
    	SLOT( 0),
    	0,
    	24,
    	"Forge",
    	"!Forge!",
    	"",
		STAT_NONE,
		3,
		0,
		"forage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Craft weapons, armor and other useful tools from metal ores."
    },
    {
        "smelting",
        { 69, 69, 69, 69 },
        { 20, 20, 20, 20},
        spell_null,
        TAR_IGNORE,
        POS_STANDING,
        &gsn_smelting,
        SLOT( 0),
        0,
        24,
        "Smelting",
        "!Smelting!",
        "",
                STAT_NONE,
                5,
	        0,
	        "forge",
	        NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
	        "Create metal ores from items."
    },


/*
* guild only skills
*/

    {
		"speardance",
		{ 1, 1, 1, 1 },
		{ 4, 4, 4, 4 },
    	spell_null,
    	TAR_IGNORE,
    	POS_FIGHTING,
    	&gsn_speardance,
    	SLOT( 0),
    	0,
    	24,
    	"",
    	"!Speardance!",
    	""
		,STAT_NONE,
		1,
		&GSKILL_SPEARDANCE,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Aiel weapon attack."
    },

/*
* non-combat skills
*/

    {
		"tracking",
		{  1,  1,  1,  1 },
		{ 10, 10, 10, 10},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_tracking,
		SLOT( 0),
		0,
		12,
		"tracking",
		"!Tracking!",
		"",
	    STAT_NONE,
	    0,
	    0,
	    "sneak",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Track persons from distant rooms."
	},

    {
		"riding",
		{  1,  1,  1,  1 },
		{ 10, 10, 10, 10},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_riding,
		SLOT( 0),
		0,
		12,
		"",
		"!Riding!",
		"",
	    STAT_NONE,
	    0,
	    0,
	    NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Basic horsemanship."
	},

    {
		"fast healing",
		{ 25, 25, 25, 25 },
		{ 8, 8, 8, 8 },
		spell_null,
		TAR_IGNORE,
		POS_SLEEPING,
		&gsn_fast_healing,
		SLOT( 0),
		0,
		0,
		"",
		"!Fast Healing!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Heal more quickly."
    },

    {
		"climb",
		{ 1, 1, 1, 1 },
		{ 5, 5, 5, 5 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_climb,
		SLOT( 0),
		0,
		0,
		"",
		"!Climb!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Climbing hills and tress."
    },
    {
		"search",
		{ 1, 1, 1, 1 },
		{ 9, 9, 9, 9 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_search,
		SLOT( 0),
		0,
		0,
		"",
		"!Search!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Searching for traps, hidden doors and persons."
    },
    {
		"unarm traps",
		{ 3, 3, 3, 3 },
		{ 9, 9, 9, 9 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_unarm_traps,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Remove traps from objects."
    },
    {
		"haggle",
		{ 7, 7, 7, 7 },
		{ 5, 5, 5, 5 },
		spell_null,
		TAR_IGNORE,
		POS_RESTING,
		&gsn_haggle,
		SLOT( 0),
		0,
		0,
		"",
		"!Haggle!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Haggle with shopkeepers for better prices."
    },
    {
		"hide",
		{ 10, 10, 10, 10 },
		{ 8, 8, 8, 8 },
		spell_null,
		TAR_IGNORE,
		POS_RESTING,
		&gsn_hide,
		SLOT( 0),
		0,
		12,
		"",
		"!Hide!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Basic hiding skill."
    },
    {
		"meditation",
		{ 25, 25, 25, 25 },
		{ 6, 6, 6, 6 },
		spell_null,
		TAR_IGNORE,
		POS_SLEEPING,
		&gsn_meditation,
		SLOT( 0),
		0,
		0,
		"",
		"Meditation",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Increase mana regeneration."
    },
    {
		"peek",
		{ 52, 52, 52, 52 },
		{ 15, 15, 15, 15 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_peek,
		SLOT( 0),
		0,
		0,
		"",
		"!Peek!",
		"",
		STAT_NONE,
		0,
		0,
		"sneak",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"See into target's inventory."
    },
    {
		"pick lock",
		{ 5, 5, 5, 5 },
		{ 7, 7, 7, 7 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_pick_lock,
		SLOT( 0),
		0,
		12,
		"",
		"!Pick!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Pick locks on doors and objects."
    },
    {
    	"butcher",
    	{ 75, 65, 55, 30 },
    	{ 10, 8, 6, 5 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_butcher,
    	SLOT( 0),
    	0,
    	36,
    	"",
    	"!Butcher!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Create food from animal corpses."
    },
    {
    	"skin",
    	{ 75, 65, 55, 30 },
    	{ 10, 8, 6, 5 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_skin,
    	SLOT( 0),
    	0,
    	36,
    	"",
    	"!Skin!",
    	"",
		STAT_NONE,
		0,
		0,
		"butcher",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Skin animals to create hides."
    },
    {
	    "throw",
	    { 108, 108, 108, 108 },
	    { 15, 15, 15, 15 },
	    spell_null,
	    TAR_IGNORE,
	    POS_STANDING,
	    &gsn_throw,
	    SLOT( 0),
	    0,
	    36,
	    "",
	    "!Throw!",
	    "",
		STAT_NONE,
		0,
		0,
		"bash",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Throw an opponent over your shoulder for damage."
    },
    {
	    "strangle",
	    { 150, 150, 150, 150 },
	    { 18, 18, 18, 18 },
	    spell_null,
	    TAR_IGNORE,
	    POS_STANDING,
	    &gsn_strangle,
	    SLOT( 0),
	    0,
	    36,
	    "",
	    "!Strangle!",
	    "",
		STAT_NONE,
		0,
		0,
		"backstab",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Strangle an opponent to put them to sleep."
    },
    {
    	"blackjack",
    	{ 150, 150, 115, 101 },
    	{ 18, 18, 18, 18 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_blackjack,
    	SLOT( 0),
    	0,
    	36,
    	"",
    	"!Blackjack!",
    	"",
		STAT_NONE,
		0,
		0,
		"backstab",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Bash an opponent to knock them unconcious."
    },
    {
    	"treat",
    	{ 11, 11, 11, 11 },
    	{ 11, 11, 11, 11 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_treat,
    	SLOT( 0),
    	0,
    	36,
    	"",
    	"{cYour treatment wears off.",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Treat wounds so they heal faster."
    },
    {
    	"bandage",
    	{ 22, 22, 22, 22 },
    	{ 22, 22, 22, 22 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_bandage,
    	SLOT( 0),
    	0,
    	36,
    	"",
    	"{cYour bandages fall apart and become useless.",
    	"",
		STAT_NONE,
		3,
		0,
		"treat",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Bandage wounds so they heal more quickly."
    },
    {
		"sneak",
		{ 62, 31,  4,  28 },
		{ 10, 10, 10, 10 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_sneak,
		SLOT( 0),
		0,
		12,
		"",
		"{cYou no longer feel stealthy.",
		"",
		STAT_NONE,
		0,
		0,
		"hide",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Sneak through rooms without being seen."
    },

    {
		"steal",
		{ -1, -1,  -1, -1 },
		{ 0, 0, 0, 0 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_steal,
		SLOT( 0),
		0,
		24,
		"",
		"!Steal!",
		"",
		STAT_NONE,
		0,
		0,
		"sneak",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Steal money and gems from a victim."
    },
    {
		"scrolls",
		{  1, 1, 1, 1 },
		{ 3, 3, 3, 3 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_scrolls,
		SLOT( 0),
		0,
		24,
		"",
		"!Scrolls!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Reading an understanding mystical scrolls."
    },

    {
		"staves",
		{  500, 500, 500, 500 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_staves,
		SLOT( 0),
		0,
		12,
		"",
		"!Staves!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Defunct skill."
    },
    {
		"teach",
		{ 1, 1,  1,  1},
		{ 4, 4, 4, 4 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_teach,
    	SLOT( 0),
    	0,
    	0,
    	"",
    	"!teach!",
    	"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Teach skills and weaves to others."
    },

    {
		"fletching",
		{ 1 , 1, 1, 1 },
		{ 5, 5, 5, 5},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_fletching,
		SLOT( 0),
		0,
		12,
		"",
		"!Fletching!",
		"",
		STAT_NONE,
		0,
		0,
		"archery",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Create bows and arrows from wood."
    },

    {
		"recall",
		{  1,  1,  1,  1 },
		{ 5, 5, 5, 5 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_recall,
		SLOT( 0),
		0,
		12,
		"",
		"!Recall!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Knowledge of how to move through Tel'aran'rhiod."
    },
    {
		"swim",
		{ 1, 1, 1, 1 },
		{ 5, 5, 5, 5 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_swim,
		SLOT( 0),
		0,
		0,
		"",
		"!Swim!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Basic swimming skill."
    },
    {
		"camping",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_camping,
		SLOT( 0),
		0,
		0,
		"",
		"!Camping!",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Create campsites to induce greater regeneration."
    },

/*
* musical affects
*/
    {
		"merriment",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_music_merriment,
		SLOT( 0),
		0,
		0,
		"",
		"You no longer feel so merry.",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gleeman skill."
    },
    {
		"serenade",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_music_serenade,
		SLOT( 0),
		0,
		0,
		"",
		"You no longer feel so charismatic.",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gleeman skill."
    },
    {
		"inspiration",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_music_inspiration,
		SLOT( 0),
		0,
		0,
		"",
		"You no longer feel inspired.",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gleeman skill."
    },
    {
		"courage",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_music_courage,
		SLOT( 0),
		0,
		0,
		"",
		"You no longer feel encouraged.",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gleeman skill."
    },
    {
		"confidence",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_music_confidence,
		SLOT( 0),
		0,
		0,
		"",
		"You no longer feel so confident.",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gleeman skill."
    },

    {
		"instrument",
		{ 10, 10, 10, 10 },
		{ 10, 10, 10, 10 },
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_instrument,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Gleeman skill."
    },

/*
* quest only skills
*/
    {
		"focus",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_focus,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"anticipate",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_anticipate,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"precision",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_precision,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"efficient",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_efficient,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"gifted",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_gifted,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"windfall",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_windfall,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"evaluate",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_evaluate,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"intimidate",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_intimidate,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"disguise",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_disguise,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"bloodlust",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_bloodlust,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"bribery",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_bribery,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"angrealstudy",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_angrealstudy,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"engraving",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_engraving,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"balancing",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_balancing,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"truesense",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_truesense,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"feint",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_feint,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"lunge",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_lunge,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"sweep",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_sweep,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"stagger",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_stagger,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"shieldbash",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_shieldbash,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"swordbreak",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_swordbreak,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"shieldbreak",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_shieldbreak,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"chop",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_chop,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"drawback",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_drawback,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"dismember",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_dismember,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },
    {
		"ravage",
		{ -1, -1, -1, -1 },
		{ 0, 0, 0, 0},
		spell_null,
		TAR_IGNORE,
		POS_STANDING,
		&gsn_ravage,
		SLOT( 0),
		0,
		0,
		"",
		"",
		"",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Quest skill."
    },

/*
* blademaster forms
*/
	{
    	"heron",
    	{ 80, 80, 80, 80 },
    	{ 12, 12, 12, 12},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_heron,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		0,
		0,
		"sword",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Primary blademaster form."
	},

	{
    	"bird",
    	{ 80, 80, 80, 80 },
    	{ 13, 13, 13, 13},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_bird,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		8,
		0,
		"boar",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"wind",
    	{ 80, 80, 80, 80 },
    	{ 14, 14, 14, 14},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_wind,
    	SLOT( 0),
    	1,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		0,
		0,
		"heron",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"water",
    	{ 80, 80, 80, 80 },
    	{ 15, 15, 15, 15},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_water,
    	SLOT( 0),
    	1,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		0,
		0,
		"wind",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"stone",
    	{ 80, 80, 80, 80 },
    	{ 16, 16, 16, 16},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_stone,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		4,
		0,
		"sky",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"animal",
    	{ 80, 80, 80, 80 },
    	{ 17, 17, 17, 17},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_animal,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		0,
		0,
		"stone",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"sky",
    	{ 80, 80, 80, 80 },
    	{ 18, 18, 18, 18},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_sky,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		3,
		0,
		"water",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"cat",
    	{ 80, 80, 80, 80 },
    	{ 20, 20, 20, 20},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_cat,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		9,
		0,
		"bird",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"storm",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_storm,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		5,
		0,
		"stone",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"lizard",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_lizard,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		6,
		0,
		"storm",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

	{
    	"boar",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_boar,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		7,
		0,
		"lizard",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},
	{
    	"elemental",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_elemental,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		10,
		0,
		"cat",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},
	{
    	"will",
    	{ 80, 80, 80, 80 },
    	{ 35, 35, 35, 35},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_will,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		12,
		0,
		"elemental",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},
	{
    	"fire",
    	{ 80, 80, 80, 80 },
    	{ 45, 45, 45, 45},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_fire,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		14,
		0,
		"will",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},
	{
    	"ice",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_ice,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		18,
		0,
		"fire",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},
	{
    	"spirit",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_spirit,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		21,
		0,
		"ice",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},
    {
    	"circle",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25 },
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_circle,
    	SLOT( 0),
    	0,
    	24,
    	"Circle",
    	"!Circle!",
    	"",
		STAT_NONE,
		25,
		0,
		"spirit",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
    },
	{
    	"crown",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_crown,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		29,
		0,
		"circle",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Blademaster form."
	},

/* spearforms */

	{
    	"eagle",
    	{ 80, 80, 80, 80 },
    	{ 12, 12, 12, 12},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_eagle,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		0,
		0,
		"spear",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"capar",
    	{ 80, 80, 80, 80 },
    	{ 13, 13, 13, 13},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_capar,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		1,
		0,
		"eagle",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"gara",
    	{ 80, 80, 80, 80 },
    	{ 14, 14, 14, 14},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_gara,
    	SLOT( 0),
    	1,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		2,
		0,
		"capar",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"hare",
    	{ 80, 80, 80, 80 },
    	{ 15, 15, 15, 15},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_hare,
    	SLOT( 0),
    	1,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		3,
		0,
		"gara",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"snake",
    	{ 80, 80, 80, 80 },
    	{ 16, 16, 16, 16},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_snake,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		4,
		0,
		"hare",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"ji",
    	{ 80, 80, 80, 80 },
    	{ 17, 17, 17, 17},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_ji,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		5,
		0,
		"snake",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"sand",
    	{ 80, 80, 80, 80 },
    	{ 18, 18, 18, 18},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_sand,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		6,
		0,
		"ji",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"ridgecat",
    	{ 80, 80, 80, 80 },
    	{ 20, 20, 20, 20},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_ridgecat,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		7,
		0,
		"sand",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"lion",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_lion,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		8,
		0,
		"ridgecat",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"dream",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_dream,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		9,
		0,
		"lion",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},

	{
    	"shadow",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_shadow,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		10,
		0,
		"dream",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},
	{
    	"wrath",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_wrath,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		12,
		0,
		"shadow",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},
	{
    	"mystic",
    	{ 80, 80, 80, 80 },
    	{ 35, 35, 35, 35},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_mystic,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		14,
		0,
		"wrath",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},
	{
    	"ring",
    	{ 80, 80, 80, 80 },
    	{ 45, 45, 45, 45},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_ring,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		18,
		0,
		"mystic",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},
	{
    	"earth",
    	{ 80, 80, 80, 80 },
    	{ 25, 25, 25, 25},
    	spell_null,
    	TAR_CHAR_OFFENSIVE,
    	POS_FIGHTING,
    	&gsn_earth,
    	SLOT( 0),
    	0,
    	6,
    	"maneuver", "!maneuver!", "",
		STAT_NONE,
		21,
		0,
		"ring",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},
	{
    		"balance",
    		{ 80, 80, 80, 80 },
    		{ 25, 25, 25, 25},
    		spell_null,
    		TAR_CHAR_OFFENSIVE,
    		POS_FIGHTING,
    		&gsn_balance,
    		SLOT( 0),
    		0,
    		6,
    		"maneuver", "!maneuver!", "",
		STAT_NONE,
		25,
		0,
		"earth",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
	},
    {
    		"dance",
    		{ 80, 80, 80, 80 },
    		{ 25, 25, 25, 25 },
		spell_null,
    		TAR_IGNORE,
    		POS_STANDING,
    		&gsn_dance,
    		SLOT( 0),
    		0,
    		24,
		"maneuver",
    		"!maneuver!",
    		"maneuver",
    		STAT_NONE,
		29,
		0,
		"balance",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Spearmaster form."
    },

    {
		"archery",
		{150,150,150,150},
		{10,10,10,10},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_archery,
		SLOT( 7),
		50,
		12,
		"pierce",
		"pierce",
		"!pierce!",
		STAT_NONE,
		0,
		0,
		"void",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Increases proficiency with ranged weapons."
    },




/* end spearforms */

   {
		"fear",
		{-1,-1,-1,-1},
		{10,10,10,10},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_fear,
		0,
		50,
		12,
		"",
		"",
		"Your fear subsides.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Paralyzes your enemy with fear (fade-only)."
    },


   {
		"plague",
		{-1,-1,-1,-1},
		{-1,-1,-1,-1},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_plague,
		0,
		50,
		12,
		"",
		"",
		"You have recovered from the plague.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Plague Affect"
    },


   {
		"poison",
		{-1,-1,-1,-1},
		{-1,-1,-1,-1},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_poison,
		0,
		50,
		12,
		"",
		"",
		"You recover from the poison.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Poison Affect"
    },

   {
		"headache",
		{-1,-1,-1,-1},
		{-1,-1,-1,-1},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_headache,
		0,
		50,
		12,
		"",
		"",
		"Your head stops throbbing.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Headache Affect"
    },

   {
		"stomach ache",
		{-1,-1,-1,-1},
		{-1,-1,-1,-1},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_stomachache,
		0,
		50,
		12,
		"",
		"",
		"Your stomach stops churning.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Stomach Ache Affect"
    },

   {
		"fever",
		{-1,-1,-1,-1},
		{-1,-1,-1,-1},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_fever,
		0,
		50,
		12,
		"",
		"",
		"Your fever subsides.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Fever Affect"
    },

   {
		"restful",
		{-1,-1,-1,-1},
		{-1,-1,-1,-1},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		NULL,
		0,
		50,
		12,
		"",
		"",
		"You no longer feel so restful.",
		STAT_NONE,
		0,
		0,
		NULL,
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Restful Affect"
    },


   {
		"herbal knowledge",
		{15,15,15,15},
		{5,10,10,10},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_herbalknowledge,
		0,
		50,
		12,
		"",
		"",
		"!Herb Knowledge!",
		STAT_NONE,
		0,
		0,
		"forage",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Knoledge about herbs."
    },

   {
		"forage",
		{15,15,15,15},
		{10,10,10,10},
		spell_null,
		TAR_CHAR_OFFENSIVE,
		POS_FIGHTING,
		&gsn_forage,
		0,
		50,
		12,
		"",
		"",
		"!Forage!",
		STAT_NONE,
		0,
		0,
		"search",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Find useful items while you explore."
    },

    {
    	"repair",
    	{ 69, 69, 69, 69 },
    	{ 20, 20, 20, 20},
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_repair,
    	SLOT( 0),
    	0,
    	24,
    	"Repair",
    	"!Repair!",
    	"",
		STAT_NONE,
		1,
		0,
		"sharpen",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Repair weapons and armor to restore their condition."
    },

    {
    	"reinforce",
    	{ 69, 69, 69, 69 },
    	{ 20, 20, 20, 20},
    	spell_null,
    	TAR_IGNORE,
    	POS_STANDING,
    	&gsn_reinforce,
    	SLOT( 0),
    	0,
    	24,
    	"Reinforce",
    	"!Reinforce!",
    	"",
		STAT_NONE,
		1,
		0,
		"repair",
		NULL,
		0, 0, 0, 0, 0, /* earth, air, fire, water, spirit - min sphere str to use
*/
		FALSE,
		TALENT_NONE, /* affecting talent */
		0,
		"Reinforce your armor and increase the defense it gives."
    },
    {
		"summon darkhound",
		{-1, -1, -1, -1},
		{999, 999, 999, 999},
		weave_summon_darkhound,
		TAR_IGNORE,
		POS_STANDING,
		NULL,
		SLOT(25),
		250,
		10,
		"!Summon Darkhound!", "You power over the darkhound has ended","",
		STAT_SPIRIT,
		4,
		0,
		0,
		0,
		12, 0, 13, 0, 11,
		FALSE,
		TALENT_NONE,
		0,
		"Spirit weave. Summons evil companion."
	},

};

/*
	{
		"summon darkhound",
		{ -1,-1,-1,-1 },
		{999,999,999,999},
        weave_summon_darkhound,
        TAR_IGNORE,
        POS_STANDING,
        NULL,
        SLOT(0),
        250,
        10,
        "!Summon Darkhound!", "You power over the darkhound has ended","",
        STAT_SPIRIT,
        4,
        0,
		0,
        weave_gateway,
        12, 0, 13, 0, 11,
        TALENT_NONE,
		0,
		"Spirit weave. Summons evil companion."
    },{
		"spirit of wind",
		{ -1,-1,-1,-1 },
		{999,999,999,999},
        weave_spirit_of_wind,
        TAR_IGNORE,
        POS_STANDING,
        NULL,
        SLOT(0),
        250,
        10,
        "!Spirit of Wind!", "The wind spirit leave you.","",
        STAT_SPIRIT,
        2,
        0,
		0,
        0,
		0, 12, 0, 13, 13,
        TALENT_NONE,
		0,
		"Spirit weave. Summon Aiel companion."
	},*/

