#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "aww.h"
#include "tables.h"
#include "olc.h"
#include "recycle.h"
#include "lookup.h"
#include "interp.h"
#include "mysql5_interface.h"


void appraise_object args((CHAR_DATA *ch, OBJ_DATA *obj, bool plrmarket));

/* SQL driven Player Markets by Rokann */


/* Tells which floor a shop id resides on. */
int market_shop_floor (int shop_id)
{
	return (shop_id - 1) / 10 + 1;
}

/* Quick way to create a new room */
ROOM_INDEX_DATA *create_room(AREA_DATA *area, int vnum)
{
	int iHash;

	ROOM_INDEX_DATA *pRoom;
	pRoom				= new_room_index();
	pRoom->area			= area;
	pRoom->vnum			= vnum;

	if ( vnum > top_vnum_room )
	top_vnum_room = vnum;

	iHash			= vnum % MAX_KEY_HASH;
	pRoom->next			= room_index_hash[iHash];
	room_index_hash[iHash]	= pRoom;

	return pRoom;
}

/* Used for links */
int reverse_dir(int dir)
{
	switch (dir)
	{
		default:		return -1;		break;
		case DIR_NORTH:		return DIR_SOUTH;	break;
		case DIR_EAST:		return DIR_WEST;	break;
		case DIR_SOUTH:		return DIR_NORTH;	break;
		case DIR_WEST:		return DIR_EAST;	break;
		case DIR_NORTHEAST:	return DIR_SOUTHWEST;	break;
		case DIR_SOUTHEAST:	return DIR_NORTHWEST;	break;
		case DIR_SOUTHWEST:	return DIR_NORTHEAST;	break;
		case DIR_NORTHWEST:	return DIR_SOUTHEAST;	break;
		case DIR_UP:		return DIR_DOWN;	break;
		case DIR_DOWN:		return DIR_UP;		break;
	}

	return -1;
}

/* Used to clear out all old links in the market */
void clear_links (ROOM_INDEX_DATA *room)
{
	int dir;

	logf("Clearing links for room %d.", room->vnum);

	for (dir = 0; dir <= MAX_DIR; dir++)
	{
		if (room->exit[dir])	free_exit(room->exit[dir]);
		room->exit[dir] = NULL;
	}

	return;
}

/* Used to quickly link two rooms together */
void link_rooms (ROOM_INDEX_DATA *orig_room, ROOM_INDEX_DATA *to_room, int dir)
{
	if (orig_room == NULL)
	{
		bug("Link rooms: Original room is NULL!", 0);
		return;
	}

	if (to_room == NULL)
	{
		bug("Link rooms: To room is NULL!", 0);
		return;
	}

	if (orig_room->exit[dir] == NULL)
		orig_room->exit[dir] = new_exit();

	if (to_room->exit[reverse_dir(dir)] == NULL)
		to_room->exit[reverse_dir(dir)] = new_exit();

	orig_room->exit[dir]->u1.to_room = to_room;
	orig_room->exit[dir]->orig_door = dir;

	to_room->exit[reverse_dir(dir)]->u1.to_room = orig_room;
	to_room->exit[reverse_dir(dir)]->orig_door = reverse_dir(dir);

	return;
}

/* Builds or updates one floor for the markets */
void build_market_floor (int floor)
{
	bool is_even;
	int start = 34715 + (floor - 1) * 15;
	int x, slot;
	AREA_DATA *area = get_vnum_area( 34700 );
	ROOM_INDEX_DATA *room = NULL;
	CHAR_DATA *mob;
	char buf[MSL];

	is_even = floor % 2 == 0;

	for (x = 0; x < 15; x++)
	{
		if (get_room_index(x + start) == NULL)
		room = create_room(area, start + x);
		else room = get_room_index(x + start);

		clear_links(room);
	}
	
	for (x = 0; x < 15; x++)
	{
		if ((room = get_room_index(x + start)) == NULL)
			continue;

		if (x <= 4)
		{
			/* Hallways */
			free_string(room->name);
			sprintf(buf, "A Walk Between the Shops, Floor %d", floor);
			room->name = str_dup(buf);

			free_string(room->description);
			room->description = str_dup("Shops spread out along either side of the walk, to the right and left.  Many shoppers fill the walk as the venture from one shop to the next, looking for just the right thing.  Signs stand out in front of each shop, denoting what is offered inside, and their cries can be heard over the clatter of the buzzing shoppers, trying to entice buyers to view their goods.");
			format_string(room->description);

			if (x == 0)
			{
				if (floor == 1)
					link_rooms(room, get_room_index(34706), DIR_DOWN);
				else
				{
					link_rooms(room, get_room_index(34719 + (floor - 2) * 15), DIR_DOWN);
				}
			}

			if (x < 4)
			{
				slot = (x + 1) + (5 * (floor - 1));

				/* Don't link if there's no shop to link off of the hall.
				0 1 2 3 4
				(x + 1) + (5 * floor) = the hall #
				if (hall# > count_shops() / 2)	don't link	*/

				if ((count_shops() + 1) / 2 > slot)
				link_rooms(room, get_room_index(x+start+1), is_even ? DIR_WEST : DIR_EAST);
			}
		}
		else
		{
			/* Shops */
			free_string(room->name);
			room->name = str_dup("An Empty Shop");

			free_string(room->description);
			room->description = str_dup("This room is empty.");
			format_string(room->description);

			if ((slot = get_slot_shop(room->vnum)) > 0)
			{
				free_string(room->name);
				room->name = str_dup(show_shop(slot, "name"));

				free_string(room->description);
				room->description = str_dup(show_shop(slot, "desc"));				

				mob = create_mobile(get_mob_index(34709));
				char_to_room(mob, room);

				free_string(mob->name);
				mob->name = str_dup(color_strip(show_shop(slot, "keeper_name")));

				free_string(mob->short_descr);
				mob->short_descr = str_dup(show_shop(slot, "keeper_name"));

				free_string(mob->colorname);
				mob->colorname = str_dup(show_shop(slot, "keeper_name"));

				free_string(mob->description);
				mob->description = str_dup(show_shop(slot, "keeper_long"));

				if (x % 2 == 0)
					link_rooms(room, get_room_index(start + (x-6) / 2), DIR_NORTH);
				else	link_rooms(room, get_room_index(start + (x-5) / 2), DIR_SOUTH);
			} 
		}
	}

	return;
}

/* Sweeps through the market shops and updates the positions and names of all shopkeepers. */
void reset_shopkeepers ()
{
	int vnum;
	ROOM_INDEX_DATA *room;
	CHAR_DATA *mob, *mob_next;
	int i, slot;

	for (i = 1; i <= (1 + ((count_shops() - 1) / 10)); i++)
	{
		logf("Rebuilding market level %d.", i);
		build_market_floor(i);
	}

	bug("Refreshing market mobiles.", 0);

	for (vnum = 34715; vnum < 34999; vnum++)
	{
		if ((room = get_room_index(vnum)) == NULL)	return;

		for ( mob = room->people; mob != NULL; mob = mob_next )
		{
		    mob_next = mob->next_in_room;
		    if ( IS_NPC(mob) && mob->pIndexData->vnum == 34709)
			extract_char( mob, TRUE );
		}

		slot = get_slot_shop(room->vnum);

		if (slot > 0)
		{
			mob = create_mobile(get_mob_index(34709));
			char_to_room(mob, room);

			free_string(mob->name);
			mob->name = str_dup(color_strip(show_shop(slot, "keeper_name")));

			free_string(mob->short_descr);
			mob->short_descr = str_dup(show_shop(slot, "keeper_name"));

			free_string(mob->long_descr);
			mob->long_descr = str_dup(show_shop(slot, "keeper_long"));
		}
	}

	return;
}

/* Gets the shop ID for a character */
int get_shop_id (CHAR_DATA *ch)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];

	sprintf(sql_buf, "select id from market_shops where owner = '%s'", ch->name);
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return -1;
        row = mysql_fetch_row(result);

	mysql_free_result(result);

	if (row == NULL || row[0] == NULL)
		return -1;

	return atoi(row[0]);
}

/* Finds the shop # that corresponds to the room vnum */
int get_slot_shop (int vnum)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	int floor, slot, start = 34715, x, count = 0;

	x = 0;

	if (get_vnum_area(vnum) == NULL || (str_cmp(get_vnum_area(vnum)->name, "Player Markets")))	return -1;
	if (vnum < 34715)	return -1;

	floor = 1 + (vnum - start) / 15;
	slot = ((vnum - start) % 15) - 4;

	if (slot <= 0)		return -1;

	x = ((floor - 1) * 10) + slot;

       	result = query_db("select id from market_shops order by id asc", result_db, FALSE);
	if (result == NULL)	return -1;

	slot = -1;

	while ((row = mysql_fetch_row(result)) != NULL)
	{
		count++;

		if (count == x)
		{
			slot = atoi(row[0]);
			break;
		}
	}

	mysql_free_result(result);
	return slot;
}

/* Shows what a shop has for sale */
void shop_list (CHAR_DATA *ch, int shop)
{
	bool found = FALSE;
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];

	sprintf(sql_buf, "select id, price, wear_flags, short from market_obj where seller='%s'", show_shop(shop, "owner"));
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return;

	while ((row = mysql_fetch_row(result)) != NULL)
	{
		if ( !found )
		{
			found = TRUE;
			stc( "\n\r", ch );
			stc( "{C[{w    ID  Gold      Where Worn{C ]{c Item{x\n\r", ch );
		}

		ptc(ch, "{C[{w %5s %5s %15s {C] {x%s\n\r", row[0], row[1], wear_bit_name(atoi(row[2])), row[3]);
	}

	if (!found)
		stc("There are no items for sale here.\n\r", ch);

	mysql_free_result(result);
	return;
}

/* Grabs various information from the shop table */
char *show_shop (int shop, char *type)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	char *show = str_dup("Error.");

	if (!str_cmp(type, "name"))
		sprintf(sql_buf, "select name from market_shops where id = %d", shop);
	else if (!str_cmp(type, "desc"))
		sprintf(sql_buf, "select description from market_shops where id = %d", shop);
	else if (!str_cmp(type, "owner"))
		sprintf(sql_buf, "select owner from market_shops where id = %d", shop);
	else if (!str_cmp(type, "keeper_name"))
		sprintf(sql_buf, "select keeper_name from market_shops where id = %d", shop);
	else if (!str_cmp(type, "keeper_long"))
		sprintf(sql_buf, "select keeper_long from market_shops where id = %d", shop);
	else if (!str_cmp(type, "greeting"))
		sprintf(sql_buf, "select keeper_greeting from market_shops where id = %d", shop);
	else return "ERROR!!! Invalid Type.";

       	result = query_db(sql_buf, result_db, FALSE);

	if (result != NULL)
	{
	        row = mysql_fetch_row(result);
		free_string(show);
		show = str_dup(row[0]);
		mysql_free_result(result);
	}

	return show;
}

/* Grabs the cost of an item in the market */
int get_market_obj_cost (int id)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	int cost;

	sprintf(sql_buf, "select price from market_obj where id = %d", id);
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return 0;
        row = mysql_fetch_row(result);

	if (row == NULL)	return 0;

	cost = atoi(row[0]);
	mysql_free_result(result);
	return cost;
}

/* Loads an object from the sql tables. */
OBJ_DATA *load_market_obj (int id)
{
	OBJ_DATA *obj;
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	char *string;
	char buf[MSL];
	AFFECT_DATA *paf;

	sprintf(sql_buf, "select * from market_obj where id = %d", id);
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return NULL;
        row = mysql_fetch_row(result);
	mysql_free_result(result);

	if (row == NULL || get_obj_index(atoi(row[1])) == NULL)
		return NULL;

	obj = create_object(get_obj_index(atoi(row[1])), 0);

	free_string(obj->name);
	obj->name = str_dup(row[2]);

	free_string(obj->short_descr);
	obj->short_descr = str_dup(row[3]);

	free_string(obj->description);
	obj->description = str_dup(row[4]);
	
	free_string(obj->material);
	obj->material = str_dup(row[5]);

	obj->wear_flags = atoi(row[6]);
	obj->item_type = atoi(row[7]);
	obj->cost = atoi(row[8]);

	string = str_dup(row[9]);
	string = one_argument(string, buf);
	obj->value[0] = atoi(buf);
	string = one_argument(string, buf);
	obj->value[1] = atoi(buf);
	string = one_argument(string, buf);
	obj->value[2] = atoi(buf);
	string = one_argument(string, buf);
	obj->value[3] = atoi(buf);
	string = one_argument(string, buf);
	obj->value[4] = atoi(buf);
	free_string(string);

	if (str_cmp(row[10], "-1"))
	{
		string = str_dup(row[10]);
		paf = new_affect();

		string = one_argument(string, buf);
		paf->location = atoi(buf);
		string = one_argument(string, buf);
		paf->modifier = atoi(buf);
		string = one_argument(string, buf);
		paf->where = atoi(buf);
		string = one_argument(string, buf);
		paf->type = atoi(buf);
		string = one_argument(string, buf);
		paf->duration = atoi(buf);
		string = one_argument(string, buf);
		paf->bitvector = atoi(buf);
		string = one_argument(string, buf);
		paf->level = atoi(buf);

		affect_to_obj(obj, paf);
		free_string(string);
	}

	if (str_cmp(row[11], "-1"))
	{
		string = str_dup(row[11]);
		paf = new_affect();

		string = one_argument(string, buf);
		paf->location = atoi(buf);
		string = one_argument(string, buf);
		paf->modifier = atoi(buf);
		string = one_argument(string, buf);
		paf->where = atoi(buf);
		string = one_argument(string, buf);
		paf->type = atoi(buf);
		string = one_argument(string, buf);
		paf->duration = atoi(buf);
		string = one_argument(string, buf);
		paf->bitvector = atoi(buf);
		string = one_argument(string, buf);
		paf->level = atoi(buf);

		affect_to_obj(obj, paf);
		free_string(string);
	}

	if (str_cmp(row[12], "-1"))
	{
		string = str_dup(row[12]);
		paf = new_affect();

		string = one_argument(string, buf);
		paf->location = atoi(buf);
		string = one_argument(string, buf);
		paf->modifier = atoi(buf);
		string = one_argument(string, buf);
		paf->where = atoi(buf);
		string = one_argument(string, buf);
		paf->type = atoi(buf);
		string = one_argument(string, buf);
		paf->duration = atoi(buf);
		string = one_argument(string, buf);
		paf->bitvector = atoi(buf);
		string = one_argument(string, buf);
		paf->level = atoi(buf);

		affect_to_obj(obj, paf);
		free_string(string);
	}

	if (str_cmp(row[13], "-1"))
	{
		string = str_dup(row[13]);
		paf = new_affect();

		string = one_argument(string, buf);
		paf->location = atoi(buf);
		string = one_argument(string, buf);
		paf->modifier = atoi(buf);
		string = one_argument(string, buf);
		paf->where = atoi(buf);
		string = one_argument(string, buf);
		paf->type = atoi(buf);
		string = one_argument(string, buf);
		paf->duration = atoi(buf);
		string = one_argument(string, buf);
		paf->bitvector = atoi(buf);
		string = one_argument(string, buf);
		paf->level = atoi(buf);

		affect_to_obj(obj, paf);
		free_string(string);
	}

	return obj;
}

/* Grabs the name of the seller for an item */
char *market_obj_seller (int id)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	char *seller = str_dup("Unknown");

	sprintf(sql_buf, "select seller from market_obj where id = %d", id);
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return "Unknown";
	mysql_free_result(result);
        row = mysql_fetch_row(result);

	if (row == NULL || row[0] == NULL)
		return "Unknown";

	free_string(seller);
	seller = str_dup(row[0]);
	return seller;
}

/* Adds an item to the market */
void add_market_obj (CHAR_DATA *ch, OBJ_DATA *obj, int price)
{
	char sql_buf[MSL];
	char obj_values[MSL];
	char obj_affect1[MSL], obj_affect2[MSL];
	char obj_affect3[MSL], obj_affect4[MSL];
	AFFECT_DATA *paf;

	obj_affect1[0] = '\0';
	obj_affect2[0] = '\0';
	obj_affect3[0] = '\0';
	obj_affect4[0] = '\0';

	sprintf(obj_values, "%d %d %d %d %d", obj->value[0], obj->value[1], obj->value[2], obj->value[3], obj->value[4]);

	for (paf = obj->affected; paf != NULL; paf = paf->next)
	{
		if (obj_affect1[0] == '\0')
		sprintf(obj_affect1, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
		else if (obj_affect2[0] == '\0')
		sprintf(obj_affect2, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
		else if (obj_affect3[0] == '\0')
		sprintf(obj_affect3, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
		else if (obj_affect4[0] == '\0')
		sprintf(obj_affect4, "%d %d %d %d %d %d %d",
		paf->location, paf->modifier, paf->where, paf->type,
		paf->duration, paf->bitvector, paf->level);
	}

	if (obj_affect1[0] == '\0')	sprintf(obj_affect1, "%d", -1);
	if (obj_affect2[0] == '\0')	sprintf(obj_affect2, "%d", -1);
	if (obj_affect3[0] == '\0')	sprintf(obj_affect3, "%d", -1);
	if (obj_affect4[0] == '\0')	sprintf(obj_affect4, "%d", -1);

	sprintf(sql_buf, "insert into market_obj values ('', %ld, \"%s\", \"%s\", \"%s\", \"%s\", %d, %d, %d, \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", %d)",
	obj->pIndexData->vnum, obj->name, obj->short_descr, obj->description, obj->material, obj->wear_flags, obj->item_type, 
	obj->cost, obj_values, obj_affect1, obj_affect2, obj_affect3, obj_affect4, ch->name, price);
	mysql_real_query(result_db, sql_buf, strlen(sql_buf));
	return;
}

/* Removes an item from the market. If sold is specified, it also adds an entry to market_gold */
void delete_market_obj (int id, CHAR_DATA *buyer, bool sold)
{
	char sql_buf[MSL];
	int cost;
	OBJ_DATA *obj = load_market_obj(id);
	char msg[MSL];

	if (obj == NULL)
	{
		stc("delete_market_obj: OBJ is NULL! Please alert an IMM!\n\r", buyer);
		return;
	}

	cost = get_market_obj_cost(id);

	if (buyer != NULL && sold)
	{
		sprintf(msg, "{W%s {wbought %s {wfor {W%d gold{w.{x", buyer->name, obj->short_descr, cost);
		sprintf(sql_buf, "insert into market_gold values ('', \"%s\", %d, \"%s\")", market_obj_seller(id), cost, msg);
		mysql_real_query(result_db, sql_buf, strlen(sql_buf));
	}

	sprintf(sql_buf, "delete from market_obj where id = %d", id);
	mysql_real_query(result_db, sql_buf, strlen(sql_buf));
	extract_obj(obj);
	return;
}

/* Counts the total # of shops in the market place */
int count_shops ()
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	int count;

       	result = query_db("select count(id) from market_shops", result_db, FALSE);
	if (result == NULL)	return 0;
        row = mysql_fetch_row(result);

	if (row == NULL || row[0] == NULL)
		return 0;

	count = atoi(row[0]);
	mysql_free_result(result);
	return count;
}

/* Counts the total # of items in a shop in the market place */
int count_items (int id)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	int count;

	sprintf(sql_buf, "select count(id) from market_obj where seller = '%s'", show_shop(id, "owner"));
       	result = query_db(sql_buf, result_db, FALSE);

	if (result == NULL)	return 0;
        row = mysql_fetch_row(result);

	if (row == NULL || row[0] == NULL)
		return 0;

	count = atoi(row[0]);
	mysql_free_result(result);
	return count;
}

/* Finds what shop the item belongs to */
int get_obj_shop (int id)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	int shop;

	sprintf(sql_buf, "select id from market_shops where owner = '%s'", market_obj_seller(id));
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return -1;
        row = mysql_fetch_row(result);

	if (row == NULL || row[0] == NULL)
		return -1;

	shop = atoi(row[0]);
	mysql_free_result(result);
	return shop;
}

/* Creates a new shop */
void create_shop (CHAR_DATA *ch)
{
	char sql_buf[MSL];

	sprintf(sql_buf, "insert into market_shops values ('', \"%s\", \"A Newly Opened Shop\", \"This shop looks as if it has just opened its doors for business. There is a whole mess of random merchandise sitting in boxes on the ground and on makeshift tables that line the walls. There is also a simple desk set up for transactions, but other than that the place is depressingly bare.\", \"a shopkeeper\", \"A shopkeeper is here, helping customers find what they are looking for.\n\r\", \"Hello $N, welcome to $t! Please let me know if I can help you with anything!\")", ch->name);
	mysql_real_query(result_db, sql_buf, strlen(sql_buf));
	reset_shopkeepers();
	return;
}

void shop_collect (CHAR_DATA *ch)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char sql_buf[MSL];
	int total = 0;
	int shop = get_shop_id(ch);

	if (shop <= 0)
	{
		stc("You need to actually own a shop before you can start collecting coins from it.\n\r", ch);
		return;
	}

	sprintf(sql_buf, "select id, gold, msg from market_gold where owner='%s'", show_shop(shop, "owner"));
       	result = query_db(sql_buf, result_db, FALSE);
	if (result == NULL)	return;

	while ((row = mysql_fetch_row(result)) != NULL)
	{
		total += atoi(row[1]);
		ptc(ch, "%s\n\r", row[2]);
		sprintf(sql_buf, "delete from market_gold where id = %d", atoi(row[0]));
		mysql_real_query(result_db, sql_buf, strlen(sql_buf));
	}

	if (total == 0)
		stc("{gLillyan says, {G'{gSorry, it doesn't look like your shop has any coins to collect!{G'{x\n\r", ch);
	else
	{
		ptc(ch, "{gLillyan says, {G'{gYour shop has earned %d gold! As you know, we keep 20%% for administration, taxes, and wages.{G'{x\n\r", total);
		stc("{wLillyan does some quick math on a ledger.{x\n\r", ch);

		total *= 10;		/* Convert to Silver Marks */
		total -= total * .2;	/* Deduct 15% */

		stc("{gLillyan continues, {G'{gSo that leaves you with", ch);

		if (total / 10 > 0)
		ptc(ch, " %d gold marks", total / 10);

		if (total / 10 > 0 && total % 10 > 0)
		stc(" and", ch);

		if (total % 10 > 0)
		ptc(ch, " %d silver marks!{G'{x\n\r", total % 10);
		else stc("!{G'{x\n\r", ch);

		ch->money[2][0] += total % 10;
		ch->money[3][0] += total / 10;
	}

	mysql_free_result(result);
	return;
}

/* Primary function for all player shop stuff */
void do_shop (CHAR_DATA *ch, char *argument)
{
	OBJ_DATA *obj;
	char arg1[MSL];
	char sql_buf[MSL];
	int cost = 0;

	argument = one_argument(argument, arg1);

	if (arg1[0] == '\0')
	{
		stc("-------------------- Shopping Commands --------------------\n\r", ch);
		stc("Shop Appraise <ID>          : Shows the stats and affects on an item.\n\r", ch);
		stc("Shop Buy <ID>               : Purchases an item in a shop.\n\r", ch);

		if (get_shop_id(ch) <= 0)
		{
			stc("If you want to purchase a shop, type {Wshop register{x.\n\r", ch);
			stc("You will need to be at the shop registrar and have 5,000 gold.\n\r", ch);
			return;
		}
		else
		{
			stc("\n\r-------------- Inventory Management Commands --------------\n\r", ch);
			stc("Shop Add <Item>             : Adds an item into your shop.\n\r", ch);
			stc("Shop Remove <ID>            : Removes an item from your shop.\n\r", ch);
			stc("Shop Collect                : Collects the coins earned by your shop.\n\r", ch);

			stc("\n\r--------------- Shop Customization Commands ---------------\n\r", ch);
			stc("Shop Name <New Name>        : Sets the name of your shop.\n\r", ch);
			stc("Shop Description <New Desc> : Sets the description of your shop.\n\r", ch);
			stc("Shop KeeperName <New Name>  : Sets the name of your shopkeeper.\n\r", ch);
			stc("Shop KeeperLong <New Name>  : Sets the long name of your shopkeeper.\n\r", ch);
			stc("Shop Greeting <New Name>    : Sets the greeting for your shop.\n\r", ch);
			return;
		}
	}

	if (!str_cmp(arg1, "register"))
	{
		if (ch->in_room->vnum != 34713)
		{
			stc("If you wish to register a new shop, please see Lillyan in the markets.\n\r", ch);
			return;
		}

		if (get_shop_id(ch) > 0)
		{
			ptc(ch, "{gLillyan says to you, {G'{gI'm sorry %s, but you've already registered a shop!{G'{x\n\r", ch->name);
			return;
		}

		if (!check_coins(ch, 5000000, 0, 0))
		{
			stc("{gLillyan says to you, {G'{gI'm sorry, it costs 5,000 gold marks to open a new shop here.{G'{x\n\r", ch);
			return;
		}

		deduct_cost(ch, 5000000, 0);
		create_shop(ch);
		stc("{gLillyan says to you, {G'{gAlright, you're all set up! See the {Gshop{g command to get started!{G'{x\n\r", ch);
		return;
	}

	if (!str_cmp(arg1, "appraise"))
	{
		if (!is_number(argument) || (obj = load_market_obj(atoi(argument))) == NULL)
		{
			stc("That's not a valid item. Make sure you're using the ID number of the item.\n\r", ch);
			return;
		}
	
		act("$n begins to carefully examine $p.",ch,obj,NULL,TO_ROOM);
		appraise_object(ch, obj, TRUE);
		extract_obj(obj);
		return;
	}

	else if (!str_cmp(arg1, "buy"))
	{
		if (!is_number(argument) || (obj = load_market_obj(atoi(argument))) == NULL)
		{
			stc("That's not a valid item. Make sure you're using the ID number of the item.\n\r", ch);
			return;
		}

		if (get_obj_shop(atoi(argument)) != get_slot_shop(ch->in_room->vnum))
		{
			stc("That's not a valid item. Make sure you're using the ID number of the item.\n\r", ch);
			return;
		}

		if (!str_cmp(ch->name, market_obj_seller(atoi(argument))))
		{
			stc("You cannot purchase your own items!\n\rUse shop remove if you wish to remove this item from your shop.\n\r", ch);
			return;
		}

		cost = get_market_obj_cost(atoi(argument));

		if (!check_coins(ch, cost * 1000, 0, 0))
		{
			ptc(ch, "You can't afford to buy %s for %d gold.\n\r", obj->short_descr, cost);
			return;
		}

		obj_to_char(obj, ch);
		deduct_cost(ch, cost * 1000, 0);
		delete_market_obj(atoi(argument), ch, TRUE);
		ptc(ch, "You have purchased %s for %d gold.\n\r", obj->short_descr, cost);
		return;
	}

	if (get_shop_id(ch) < 0)
	{
		stc("-------------------- Shopping Commands --------------------\n\r", ch);
		stc("Shop Appraise <ID>          : Shows the stats and affects on an item.\n\r", ch);
		stc("Shop Buy <ID>               : Purchases an item in a shop.\n\r", ch);

		stc("If you want to purchase a shop, type {Wshop register{x.\n\r", ch);
		stc("You will need to be at the shop registrar and have 5,000 gold.\n\r", ch);
		return;
	}

	if (!str_cmp(arg1, "add"))
	{
		if (ch->in_room->vnum != 34713)
		{
			stc("Please see Lillyan in the markets if you wish to add an item to your shop.\n\r", ch);
			return;
		}

		argument = one_argument(argument, arg1);

		if ((obj = get_obj_carry(ch, arg1, ch)) == NULL)
		{
			stc("You're not carrying that item.\n\r", ch);
			return;
		}

		if (obj->contains)
		{
			stc("{gLillyan says to you, {G'{gYou'll need to empty that out before you can sell it.{G'{x\n\r", ch);
			return;
		}

		if (count_items(get_shop_id(ch)) >= 25)
		{
			stc("{gLillyan says to you, {G'{gI'm sorry, but your shop is already packed full of merchandise! Take something out first if you want to sell that.{G'{x\n\r", ch);
			return;
		}

		cost = atoi(argument);

		if (!is_number(argument) || (cost = atoi(argument)) < UMAX(1, obj->cost / 500))
		{
			ptc(ch, "{gLillyan says to you, {G'{gI'm sorry, but you have to sell this item for atleast %d gold.{G'{x\n\r", UMAX(1, obj->cost / 500));
			return;
		}

		add_market_obj(ch, obj, cost);
		extract_obj(obj);
		stc("{gLillyan says to you, {G'{gOkay! I'll send this up to your shop immediately!{G'{x\n\r", ch);
		return;
	}
	else if (!str_cmp(arg1, "collect"))
	{
		if (ch->in_room->vnum != 34713)
		{
			stc("Please see Lillyan in the markets if you wish to collect your earnings.\n\r", ch);
			return;
		}

		shop_collect(ch);
		return;
	}
	else if (!str_cmp(arg1, "remove"))
	{
		if (!is_number(argument) || (obj = load_market_obj(atoi(argument))) == NULL)
		{
			stc("That's not a valid item. Make sure you're using the ID number of the item.\n\r", ch);
			return;
		}

		if (str_cmp(ch->name, market_obj_seller(atoi(argument))))
		{
			stc("That's not your item to remove!\n\r", ch);
			return;
		}

		obj_to_char(obj, ch);
		delete_market_obj(atoi(argument), ch, FALSE);
		stc("{gLillyan says to you, {G'{gAlright, let me send someone to grab that for you. Give me just a second!{G'{x\n\r", ch);
		ptc(ch, "{wLillyan motions to a man standing by the door and he disappears through the doors for a moment.\n\rHe returns with a large bag in hand, pulls %s from it, and returns it back to you.\n\r", obj->short_descr);
		return;
	}
	else if (!str_cmp(arg1, "name"))
		sprintf(sql_buf, "update market_shops set name=\"%s\" where owner = '%s'", sql_escape_string(argument), ch->name);
	else if (!str_cmp(arg1, "description"))
	{
		format_string(argument);
		sprintf(sql_buf, "update market_shops set description=\"%s\" where owner = '%s'", sql_escape_string(argument), ch->name);
	}
	else if (!str_cmp(arg1, "keepername"))
		sprintf(sql_buf, "update market_shops set keeper_name=\"%s\" where owner = '%s'", sql_escape_string(argument), ch->name);
	else if (!str_cmp(arg1, "keeperlong"))
		sprintf(sql_buf, "update market_shops set keeper_long=\"%s\n\r\" where owner = '%s'", sql_escape_string(argument), ch->name);
	else if (!str_cmp(arg1, "greeting"))
		sprintf(sql_buf, "update market_shops set keeper_greeting=\"%s\" where owner = '%s'", sql_escape_string(argument), ch->name);
	else
	{
		stc("-------------------- Shopping Commands --------------------\n\r", ch);
		stc("Shop Appraise <ID>          : Shows the stats and affects on an item.\n\r", ch);
		stc("Shop Buy <ID>               : Purchases an item in a shop.\n\r", ch);

		stc("\n\r-------------- Inventory Management Commands --------------\n\r", ch);
		stc("Shop Add <Item>             : Adds an item into your shop.\n\r", ch);
		stc("Shop Remove <ID>            : Removes an item from your shop.\n\r", ch);
		stc("Shop Collect                : Collects the coins earned by your shop.\n\r", ch);

		stc("\n\r--------------- Shop Customization Commands ---------------\n\r", ch);
		stc("Shop Name <New Name>        : Sets the name of your shop.\n\r", ch);
		stc("Shop Description <New Desc> : Sets the description of your shop.\n\r", ch);
		stc("Shop KeeperName <New Name>  : Sets the name of your shopkeeper.\n\r", ch);
		stc("Shop KeeperLong <New Name>  : Sets the long name of your shopkeeper.\n\r", ch);
		stc("Shop Greeting <New Name>    : Sets the greeting for your shop.\n\r", ch);
		return;
	}

	if (ch->in_room->vnum != 34711)
	{
		stc("Please see Carienne in the markets if you wish to customize your shop.\n\r", ch);
		return;
	}

	stc("{gCarienne says to you, {G'{gAlrighty! I'll make a note of this and have the maintenance team get on it as soon as possible!{G'{x\n\r", ch);
	mysql_real_query(result_db, sql_buf, strlen(sql_buf));
	reset_shopkeepers();
	return;
}
