#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "aww.h"
#include "recycle.h"
#include "interp.h"
#include "tailoring.h"


bool check_vowell args((const char *str));

/*#define MAX_CLOTHING 31
#define MAX_COLORS 11
#define MAX_PATTERNS 11*/


// local function for capitalizing first non-color-code character of a string,
//	since original MERC version smashes case and tries to capitalize {
char *capit( const char *str )
{
	static char strcap[MSL];
	int i;

	for ( i = 0; str[i] != '\0'; i++ )
	{
		strcap[i] = str[i];
	}

	strcap[i] = '\0';
	if ( strcap[0] != '{' )
		strcap[0] = UPPER(strcap[0]);
	else
		strcap[2] = UPPER(strcap[2]);

	return strcap;
}

// local function to determine cloth amounts
char *amnt_check( int amnt )
{
	char *newstr;

	switch ( amnt )
	{
	default:  /* no values */
		newstr = "";
		bug( "Old cloth amount is %d after cutting", amnt );
		break;

	case 1:
		newstr = "eighth";
		break;
	case 2:
		newstr = "quarter";
		break;
	case 3:
		newstr = "three-eighths";
		break;
	case 4:
		newstr = "half";
		break;
	case 5:
		newstr = "five-eighths";
		break;
	case 6:
		newstr = "three-quarters";
		break;
	case 7:
		newstr = "seven-eighths";
		break;
	case 8:
		newstr = "full";
		break;
	}

	return newstr;
}



void do_cut( CHAR_DATA *ch, char *argument )
{

	/*******************************************************
	* - Will need to use a v[x] to determine cloth length  *
	* - Need to create new obj with remaining length       *
	* - Need to change descs to reflect cloth length       *
	* - Syntax: cut <half|quarter|eighth> from <cloth>     *
    *******************************************************/
	OBJ_DATA *cloth;
	OBJ_DATA *new_cloth;
	char arg1[MIL], arg2[MIL], arg3[MIL], cloth_short[MSL], cloth_long[MSL], cloth_name[MSL], ocloth_long[MSL], ocloth_short[MSL];
	char const *color, *colorn, *ocloth_amnt;
	int amnt = 0, i;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	cloth_short[0] = '\0';
	cloth_long[0] = '\0';
	cloth_name[0] = '\0';
	ocloth_long[0] = '\0';
	ocloth_short[0] = '\0';
	color = "";
	colorn = "";
	ocloth_amnt = "";

	if ( IS_NPC( ch ) )
	{
		stc( "I don't think so.\n\r", ch );
		return;
	}

	if ( str_cmp( arg2, "from" ) )
	{
		stc( "You must provide the correct, exact syntax.\n\r", ch );
		stc( "Syntax: cut <half|quarter|eighth> from cloth>\n\r", ch );
		stc( "Example: cut half from silk\n\r", ch );
		return;
	}

	if ( arg1[0] == '\0' || arg3[0] == '\0' )
	{
		stc( "None of the arguments can be empty, and syntax must be exact.\n\r", ch );
		stc( "Syntax: cut <half|quarter|eighth> from cloth>\n\r", ch );
		stc( "Example: cut half from silk\n\r", ch );
		return;
	}

	if ( ( cloth = get_obj_carry( ch, arg3, ch ) ) == NULL )
	{
		stc( "That's not in your inventory.\n\r", ch );
		return;
	}

	if ( cloth->item_type != ITEM_CLOTH_BOLT )
	{
		ptc( ch, "%s is not a bolt of cloth.\n\r", capit(cloth->short_descr) );
		return;
	}

	if ( cloth->value[3] != 0 )
	{
		ptc( ch,  "You cannot cut %s.\n\r", cloth->short_descr );
		return;
	}

	else
	{
		for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
		{
			if ( cloth->value[1] == cloth_colors_table[i].number )
			{
				color = cloth_colors_table[cloth_colors_table[i].number].aname;
				colorn = cloth_colors_table[cloth_colors_table[i].number].bname;
				break;
			}
		}

		if ( !str_cmp( arg1, "half" ) )
		{
			if ( cloth->value[0] <= 4 )
			{
				stc( "That piece of cloth is too small to cut to a half length.\n\r", ch );
				return;
			}
			act( "You cut a piece of $t $T cloth in half.", ch, color, cloth->material, TO_CHAR );
			act( "$n cuts a piece of $t $T cloth in half.", ch, color, cloth->material, TO_ROOM );
			amnt = 4;
		}

		if ( !str_cmp( arg1, "quarter" ) )
		{
			if ( cloth->value[0] <= 2 )
			{
				stc( "That piece of cloth is too small to cut to a quarter length.\n\r", ch );
				return;
			}
			act( "You cut off a quarter of a piece of $t $T.", ch, color, cloth->material, TO_CHAR );
			act( "$n cuts off a quarter of a piece of $t $T", ch, color, cloth->material, TO_ROOM );
			amnt = 2;
		}

		if ( !str_cmp( arg1, "eighth" ) )
		{
			if ( cloth->value[0] <= 1 )
			{
				stc( "That piece of cloth is too small to cut to an eighth length.\n\r", ch );
				return;
			}
			act( "You cut off an eighth of a piece of $t $T.", ch, color, cloth->material, TO_CHAR );
			act( "$n cuts off an eighth of a piece of $t $T.", ch, color, cloth->material, TO_ROOM );
			amnt = 1;
		}

		if ( str_cmp( arg1, "half" ) && str_cmp( arg1, "quarter" ) && str_cmp( arg1, "eighth" ) )
		{
			stc( "Syntax: cut <half|quarter|eighth> from <cloth>\n\r", ch );
			return;
		}

		cloth->value[0] -= amnt;
		ocloth_amnt = amnt_check( cloth->value[0] );

		new_cloth = create_object( get_obj_index( OBJ_VNUM_GEM ), 1 );
		sprintf( cloth_name, "%s %s cloth", colorn, cloth->material );
		sprintf( cloth_short, "%s %s of a length of %s %s{X", check_vowell(arg1) ? "an" : "a", arg1, color, cloth->material );
		sprintf( cloth_long, "Here lies %s %s of a length of %s %s{X.", check_vowell(arg1) ? "an" : "a", arg1, color, cloth->material );
		sprintf( ocloth_short, "%s %s of a length of %s %s{X", check_vowell(arg1) ? "an" : "a", ocloth_amnt, color, cloth->material );
		sprintf( ocloth_long, "Here lies %s %s of a length of %s %s{X.", check_vowell(arg1) ? "an" : "a", ocloth_amnt, color, cloth->material );
		new_cloth->item_type = ITEM_CLOTH_BOLT;
		new_cloth->value[0] = amnt;
		new_cloth->value[1] = cloth->value[1];
		new_cloth->value[3] = 0;
		new_cloth->wear_flags = ITEM_TAKE|ITEM_HOLD;
		new_cloth->material = cloth->material;
		new_cloth->name = str_dup( cloth_name );
		new_cloth->short_descr = str_dup( cloth_short );
		new_cloth->description = str_dup( cloth_long );
		new_cloth->weight = cloth->weight * amnt;
		new_cloth->cost = cloth->cost * amnt;
		free_string( cloth->short_descr );
		cloth->short_descr = str_dup( ocloth_short );
		free_string( cloth->description );
		cloth->description = str_dup( ocloth_long );
		free_string( cloth->name );
		cloth->name = str_dup( cloth_name );
		obj_to_char(new_cloth, ch);
	}

	return;
}

void do_layout( CHAR_DATA *ch, char *argument )
{
	/**************************************************************************
	* - Need external table to list items that can be made and cloth needed   *
	* - Need to change descs to reflect what has been laid out		  *
	* - Need to change v[x] to reflect stage of cloth			  *
	* - Failure results only in having to layout the fabric again		  *
	* - Syntax: layout <cloth> for <clothing type>				  *
	**************************************************************************/
	OBJ_DATA *cloth;
	char arg1[MIL], arg2[MIL], arg3[MIL], cloth_short[MSL], cloth_long[MSL], cloth_name[MSL];
	char const *color, *colorn;
	char buf1[MSL], buf2[MSL];
	int i, type = 0, skill, difficulty;
	bool found = FALSE;

	cloth_short[0] = '\0';
	cloth_long[0] = '\0';
	cloth_name[0] = '\0';
	color = "";
	colorn = "";
	buf1[0] = '\0';
	buf2[0] = '\0';

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );

	if ( IS_NPC( ch ) )
	{
		stc( "You can not do that.\n\r", ch );
		return;
	}

	if ( (skill = get_skill(ch, gsn_layout)) == 0 )
	{
		stc( "You don't know how to lay out cloth.\n\r", ch );
		return;
	}

	if ( !str_cmp( arg1, "types" ) )
	{
		int col;
		col = 0;
		stc( "Item Type    Length    Item Type    Length\n\r", ch );
		stc( "=========   ========   =========   ========\n\r", ch );
		for ( i = 0; stitch_items_table[i].name != NULL; i++ )
		{
			ptc( ch, "%-12s%+8s   ", stitch_items_table[i].name, amnt_check(stitch_items_table[i].length) );
			if ( ++col % 2 == 0 )
				stc( "\n\r", ch );
		}

		if ( col % 2 != 0 )
			stc( "\n\r", ch );
		return;
	}

	if ( str_cmp( arg2, "for" ) )
	{
		stc( "You must provide the correct, exact syntax.\n\r", ch );
		stc( "Syntax: layout <cloth> for <clothing type>\n\r", ch );
		stc( "Example: layout silk for cloak\n\r", ch );
		return;
	}

	if ( arg1 == '\0' || arg3 == '\0' )
	{
		stc( "Layout what for what?.\n\r", ch );
		stc( "Syntax: layout <cloth> for <clothing type>\n\r", ch );
		stc( "Example: layout silk for cloak\n\r", ch );
		stc( "See \"layout types\" for available choices.\n\r", ch );
		return;
	}

	if ( ( cloth = get_obj_carry( ch, arg1, ch ) ) == NULL )
	{
		stc( "That's not in your inventory.\n\r", ch );
		return;
	}

	if ( cloth->item_type != ITEM_CLOTH_BOLT )
	{
		ptc( ch, "%s is not a bolt of cloth.\n\r", capit(cloth->short_descr) );
		return;
	}

	if ( cloth->value[3] != 0 )
	{
		ptc( ch, "You can't layout %s.\n\r", cloth->short_descr );
		return;
	}

	else
	{
		for ( i = 0; stitch_items_table[i].name != NULL; i++ )
		{
			if ( !str_cmp( arg3, stitch_items_table[i].name ) )
			{
				found = TRUE;
				type = i;
				break;
			}
		}
		if ( !found )
		{
			stc( "That is not a valid clothing type. Please try again.\n\r", ch );
			stc( "See \"layout types\" for a list of available choices.\n\r", ch );
			return;
		}

		if ( cloth->value[0] != stitch_items_table[type].length )
		{
			ptc( ch, "%s is not the proper length for %s %s.\n\r", capit(cloth->short_descr), check_vowell(arg3) ? "an" : "a", arg3 );
			stc( "See \"layout types\" for a list of sizes.\n\r", ch );
			return;
		}
		else
		{
			if ( skill < stitch_items_table[type].difficulty )
			{
				ptc( ch, "You do not know enough about tailoring to lay out %s.\n\r", cloth->short_descr );
				return;
			}
			difficulty = number_range(1,(stitch_items_table[type].difficulty/2));
			difficulty += stitch_items_table[type].difficulty;
			difficulty -= skill/5;

			if ( difficulty <= 0 )
				difficulty = 1;

			WAIT_STATE( ch, stitch_items_table[type].beats );
			sprintf( buf1, "You begin laying out %s in the shape of %s %s.", cloth->short_descr, check_vowell(arg3) ? "an" : "a", arg3 );
			sprintf( buf2, "%s begins laying out %s in the shape of %s %s.", ch->name, cloth->short_descr, check_vowell(arg3) ? "an" : "a", arg3 );
			act( buf1, ch, NULL, NULL, TO_CHAR );
			act( buf2, ch, NULL, NULL, TO_ROOM );


			if ( difficulty > skill )
			{
				act("You fail to lay out the material properly.", ch, NULL, NULL, TO_CHAR );
				act ("$n fails to lay out the material properly.", ch, NULL, NULL, TO_ROOM );
				return;
			}

			check_improve( ch, gsn_layout, TRUE, 9 );

			for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
			{
				if ( cloth->value[1] == cloth_colors_table[i].number )
				{
					color = cloth_colors_table[cloth_colors_table[i].number].aname;
					colorn = cloth_colors_table[cloth_colors_table[i].number].bname;
					break;
				}
			}
			sprintf(buf1, "You lay out the fabric and cut out the shape of %s %s.", check_vowell(arg3) ? "an" : "a", arg3 );
			sprintf(buf2, "%s lays out the fabric and cuts out the shape of %s %s.", ch->name, check_vowell(arg3) ? "an" : "a", arg3 );
			act( buf1, ch, NULL, NULL, TO_CHAR );
			act( buf2, ch, NULL, NULL, TO_ROOM );
			sprintf( cloth_short, "a piece of %s %s{X cut into the pattern of %s %s", color, cloth->material, check_vowell(arg3) ? "an" : "a", arg3 );
			sprintf( cloth_long, "Here lies a piece of %s %s{X cut into the pattern of %s %s.", color, cloth->material, check_vowell(arg3) ? "an" : "a", arg3 );
			sprintf( cloth_name, "%s %s cloth %s", colorn, cloth->material, arg3 );
			free_string( cloth->short_descr );
			cloth->short_descr = str_dup( cloth_short );
			free_string( cloth->description );
			cloth->description = str_dup( cloth_long );
			free_string( cloth->name );
			cloth->name = str_dup( cloth_name );
			cloth->value[3] = 1;
			cloth->value[4] = type;
		}
	}
	return;
}
/*
void do_sew( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *cloth;
	char arg[MIL], cloth_short[MSL], cloth_long[MSL];//, type[MSL];
	char const *color, *type;
	char buf1[MSL], buf2[MSL];
	int i, skill, itype, difficulty;
	//bool found = FALSE;

	cloth_short[0] = '\0';
	cloth_long[0] = '\0';
	color = "";
	type = "";
	buf1[0] = '\0';
	buf2[0] = '\0';

	argument = one_argument( argument, arg );

	if ( IS_NPC(ch) )
	{
		stc( "You cannot do that.\n\r", ch );
		return;
	}

	if ( (skill = get_skill(ch,gsn_sew)) == 0 )
	{
		stc( "You don't know how to sew.\n\r", ch );
		return;
	}

	if ( arg[0] == '\0' )
	{
		stc( "Sew what?\n\r", ch );
		return;
	}

	if ( ( cloth = get_obj_carry( ch, arg, ch ) ) == NULL )
	{
		stc( "That's not in your inventory.\n\r", ch );
		return;
	}

	if ( cloth->item_type != ITEM_CLOTH_BOLT )
	{
		ptc( ch, "%s is not a bolt of cloth!\n\r", capit(cloth->short_descr) );
		return;
	}

	if ( cloth->value[3] != 1 )
	{
		ptc( ch, "You can't sew %s.\n\r", cloth->short_descr );
		return;
	}
	else
	{
		for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
		{
			if ( cloth->value[1] == cloth_colors_table[i].number )
			{
				color = cloth_colors_table[cloth_colors_table[i].number].aname;
				break;
			}
		}

		type = stitch_items_table[cloth->value[4]].name;
		itype = cloth->value[4];

		if ( skill < stitch_items_table[itype].difficulty )
		{
			ptc( ch, "You do not know enough about tailoring to sew %s.\n\r", cloth->short_descr );
			return;
		}

		difficulty = number_range(1,(stitch_items_table[itype].difficulty/2));
		difficulty += stitch_items_table[itype].difficulty;
		difficulty -= skill/5;

		if ( difficulty <= 0 )
			difficulty = 1;

		sprintf( buf1, "You begin sewing %s.", cloth->short_descr );
		sprintf( buf2, "%s begins sewing %s.", ch->name, cloth->short_descr );
		act( buf1, ch, NULL, NULL, TO_CHAR );
		act( buf2, ch, NULL, NULL, TO_ROOM );

		WAIT_STATE( ch, stitch_items_table[itype].beats );

		if ( difficulty > skill )
		{
			act( "You fail to sew the material.", ch, NULL, NULL, TO_CHAR );
			act( "$n fails to sew the material.", ch, NULL, NULL, TO_ROOM );
			return;
		}

		check_improve( ch, gsn_sew, TRUE, 9 );

		sprintf( buf1, "You finish sewing %s.", cloth->short_descr );
		sprintf( buf2, "%s finishes sewing %s", ch->name, cloth->short_descr );
		act( buf1, ch, NULL, NULL, TO_CHAR );
		act( buf2, ch, NULL, NULL, TO_ROOM );
		sprintf( cloth_short, "a piece of %s %s{X sewn into %s %s", color, cloth->material, check_vowell(type) ? "an" : "a", type );
		sprintf( cloth_long, "Here lies a piece of %s %s{X sewn into %s %s", color, cloth->material, check_vowell(type) ? "an" : "a", type );
		free_string( cloth->short_descr );
		cloth->short_descr = str_dup( cloth_short );
		free_string( cloth->description );
		cloth->description = str_dup( cloth_long );
		cloth->value[3] = 2;
	}
	return;
}
*/
void do_hem( CHAR_DATA *ch, char *argument )
{
	/**************************************************************
	* - Extract cloth and create clothing			      *
	* - Edit descs to include color, clothing type, and fabric    *
	* - Failure results in needing to attempt hemming again       *
	* - Syntax: hem <cloth>					      *
	**************************************************************/
	OBJ_DATA *cloth, *obj;
	char arg[MIL], cloth_short[MSL], cloth_long[MSL], cloth_name[MSL], *adj;
	char const *color, *colorn, *type;
	char buf1[MSL], buf2[MSL];
	char *cloth_extra;
	int i, itype, skill, difficulty;

	cloth_short[0] = '\0';
	cloth_long[0] = '\0';
	cloth_name[0] = '\0';
	cloth_extra = "";
	adj = "";
	color = "";
	colorn = "";
	type = "";
	buf1[0] = '\0';
	buf2[0] = '\0';

	argument = one_argument( argument, arg );

	if ( IS_NPC(ch) )
	{
		stc( "You cannot do that.\n\r", ch );
		return;
	}

	if ( (skill = get_skill(ch,gsn_hem)) == 0 )
	{
		stc( "You don't know how to hem.\n\r", ch );
		return;
	}

	if ( arg == '\0' )
	{
		stc( "Hem what?\n\r", ch );
		return;
	}

	if ( ( cloth = get_obj_carry( ch, arg, ch ) ) == NULL )
	{
		stc( "That's not in your inventory.\n\r", ch );
		return;
	}

	if ( cloth->item_type != ITEM_CLOTH_BOLT )
	{
		ptc( ch, "%s is not a bolt of cloth!\n\r", capit(cloth->short_descr) );
		return;
	}

	if ( cloth->value[3] != 2 )
	{
		ptc( ch, "You can't hem %s.\n\r", cloth->short_descr );
		return;
	}

	else
	{
		for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
		{
			if ( cloth->value[1] == cloth_colors_table[i].number )
			{
				color = cloth_colors_table[cloth_colors_table[i].number].aname;
				colorn = cloth_colors_table[cloth_colors_table[i].number].bname;
				break;
			}
		}

		type = stitch_items_table[cloth->value[4]].name;
		itype = cloth->value[4];

		if ( skill < stitch_items_table[itype].difficulty )
		{
			ptc( ch, "You do not know enough about tailoring to hem %s.\n\r", cloth->short_descr );
			return;
		}

		difficulty = number_range(1,(stitch_items_table[itype].difficulty/2));
		difficulty += stitch_items_table[itype].difficulty;
		difficulty -= skill/5;

		if ( difficulty <= 0 )
			difficulty = 1;

		if ( skill <= 10 )
			adj = "poorly";
		else if ( skill <= 25 )
			adj = "simply";
		else if ( skill <= 50 )
			adj = "properly";
		else if ( skill <= 75 )
			adj = "well";
		else if ( skill == 90 )
			adj = "finely";
		else if ( skill >= 100 )
			adj = "expertly";
		else
			adj = "plainly";

		act( "You begin to hem $p.", ch, cloth, NULL, TO_CHAR );
		act( "$n begins to hem $p.", ch, cloth, NULL, TO_ROOM );

		WAIT_STATE( ch, stitch_items_table[itype].beats );

		if ( difficulty > skill )
		{
			act( "You fail to hem the material.", ch, NULL, NULL, TO_CHAR );
			act( "$n fails to hem the material.", ch, NULL, NULL, TO_ROOM );
			return;
		}

		check_improve( ch, gsn_hem, TRUE, 9 );

		sprintf( buf1, "You finish hemming %s.", cloth->short_descr );
		sprintf( buf2, "%s finishes hemming %s.", ch->name, cloth->short_descr );

		act( buf1, ch, NULL, NULL, TO_CHAR );
		act( buf2, ch, NULL, NULL, TO_ROOM );

		sprintf( cloth_short, "%s %s, %s tailored from %s %s{X", check_vowell(type) ? "an" : "a", type, adj, color, cloth->material );
		sprintf( cloth_long, "Here lie %s %s, %s tailored from %s %s{X", check_vowell(type) ? "an" : "a", type, adj, color, cloth->material );
		sprintf( cloth_name, "%s %s %s", colorn, cloth->material, type );
		obj = create_object( get_obj_index( OBJ_VNUM_GEM), 1 );
		obj->level = 1;
		obj->item_type = ITEM_ARMOR;
		obj->value[0] = stitch_items_table[cloth->value[4]].base_pierce;
		obj->value[1] = stitch_items_table[cloth->value[4]].base_bash;
		obj->value[2] = stitch_items_table[cloth->value[4]].base_slash;
		obj->value[3] = stitch_items_table[cloth->value[4]].base_exotic;
		obj->wear_flags = stitch_items_table[cloth->value[4]].wear_flags;
		obj->material = cloth->material;
		free_string( obj->name );
		obj->name = str_dup( cloth_name );
		free_string( obj->short_descr );
		obj->short_descr = str_dup( cloth_short );
		free_string( obj->description );
		obj->description = str_dup( cloth_long );
		obj->weight = stitch_items_table[cloth->value[4]].weight;
		obj->cost = cloth->cost * 10;
		obj->condition = 100;
		obj->is_tailored = TRUE;

		extract_obj( cloth );
		obj_to_char( obj, ch );
	}
	return;
}

void do_stitch( CHAR_DATA *ch, char *argument )
{
	/***********************************************************************
	* - Create table of things that can be added (Rose, Flame of Tv, etc)  *
	* - Include choice of colors as argument						       *
	* - Possibility of stitching lettering							       *
	* - Change descs to reflect stitching								   *
	* - Syntax: stitch <item> with <color> <type> for <stat>		       *
	* -	Note: lettering removed due to insane complications with		   *
	* -		syntax and one_argument's smashing of case					   *
	***********************************************************************/
	OBJ_DATA *obj;
	char arg1[MIL], arg2[MIL], arg3[MIL], arg4[MIL], arg5[MIL], arg6[MIL];
	char new_short[MSL], new_long[MSL], new_name[MSL], *color, *colorn, *pattern;
	char old_type[MSL], old_color[MSL], *old_color2, old_material[MSL], *old_name;
	int i, itype = 0, skill, difficulty, bonus, stat = 0;
	//bool lFound = FALSE;
	bool cFound = FALSE;
	bool pFound = FALSE;
	AFFECT_DATA *af;

	new_short[0] = '\0';
	new_long[0] = '\0';
	new_name[0] = '\0';
	old_type[0] = '\0';
	old_color[0] = '\0';
	old_material[0] = '\0';
	old_color2 = "";
	color = "";
	colorn = "";
	pattern = "";
	old_name = "";

	argument = one_argument( argument, arg1 ); // item
	argument = one_argument( argument, arg2 ); // with
	argument = one_argument( argument, arg3 ); // color
	argument = one_argument( argument, arg4 ); // pattern
	argument = one_argument( argument, arg5 ); // for
	argument = one_argument( argument, arg6 ); // stat

	if ( IS_NPC(ch) )
	{
		stc( "You cannot do that.\n\r", ch );
		return;
	}

	if ( (skill = get_skill(ch,gsn_stitch)) == 0 )
	{
		stc( "You don't know how to stitch.\n\r", ch );
		return;
	}

	if ( !str_cmp( arg1, "colors" ) )
	{
		int col;
		col = 0;

		for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
		{
			ptc( ch, "%-16s", cloth_colors_table[i].aname );
			if ( ++col % 4 == 0 )
				stc( "\n\r", ch );
		}

		if ( col % 4 != 0 )
		{
			stc( "\n\r", ch );
		}
		return;
	}

	if ( !str_cmp( arg1, "patterns" ) )
	{
		int col;
		col = 0;

		for ( i = 0; stitch_patterns_table[i].aname != NULL; i++ )
		{
			ptc( ch, "%-16s", stitch_patterns_table[i].aname );
			if ( ++col % 4 == 0 )
				stc( "\n\r", ch );
		}

		if ( col % 4 != 0 )
		{
			stc( "\n\r", ch );
		}
		return;
	}

	if ( !str_cmp( arg1, "stats" ) )
	{
		ptc( ch, "%-8s%-8s%-8s%-8s%-8s", "str", "dex", "con", "ref", "int\n\r" );
		ptc( ch, "%-8s%-8s%-8s%-8s%-8s", "wis", "ins", "cha", "dr", "hr\n\r" );
		return;
	}

	if ( arg1[0] == '\0' )
	{
		stc( "Stitch what?\n\r", ch );
		return;
	}

	if ( arg3[0] == '\0' || arg4[0] == '\0' )
	{
		stc( "Stitch that with what?\n\r", ch );
		stc( "Syntax: stitch <item> with <color> <type> for <stat>\n\r", ch );
		stc( "Example: stitch cloak with red \"dragon's fang\" for CON\n\r", ch );
		stc( "See \"stitch colors\" and \"stitch patterns\"\n\r", ch );
		stc( "     for available choices.\n\r", ch );
		return;
	}

	if ( str_cmp( arg2, "with" ) || str_cmp( arg5, "for" ) )
	{
		stc( "You must provide the correct, exact syntax.\n\r", ch );
		stc( "Syntax: stitch <item> with <color> <type> for <stat>\n\r", ch );
		stc( "Example: stitch cloak with red \"dragon's fang\" for CON\n\r", ch );
		return;
	}

	if ( ( obj = get_obj_carry( ch, arg1, ch ) ) == NULL )
	{
		stc( "You don't have that item.\n\r", ch );
		return;
	}

	if ( obj->item_type != ITEM_ARMOR )
	{
		ptc( ch, "%s is not suitable for stitching.\n\r", capit(obj->short_descr) );
		return;
	}

	if ( obj->is_stitched )
	{
		stc( "This item has already been stitched.\n\r", ch );
		return;
	}

	if ( !obj->is_tailored )
	{
		stc( "You may only stitch tailored items.\n\r", ch );
		return;
	}


	if ( arg6[0] == '\0' )
	{
		stc( "Stitch the pattern for what?\n\r", ch );
		stc( "Syntax: stitch <item> with <color> <type> for <stat>\n\r", ch );
		stc( "See \"stitch stats\" for available choices.\n\r", ch );
		return;
	}

	else
	{
		bool found = FALSE;
		switch ( arg6[0] )
		{
		default: // not a valid stat
			break;
		case 'c':
			if ( !str_cmp( arg6, "con" ) )
			{
				stat = APPLY_CON;
				found = TRUE;
			}
			if ( !str_cmp( arg6, "cha" ) )
			{
				stat = APPLY_CHA;
				found = TRUE;
			}
			break;
		case 'd':
			if ( !str_cmp( arg6, "dex" ) )
			{
				stat = APPLY_DEX;
				found = TRUE;
			}
			if ( !str_cmp( arg6, "dr" ) || !str_cmp( arg6, "dam" ) )
			{
				stat = APPLY_DAMROLL;
				found = TRUE;
			}
			break;
		case 'h':
			if ( !str_cmp( arg6, "hr" ) || !str_cmp( arg6, "hit" ) )
			{
				stat = APPLY_HITROLL;
				found = TRUE;
			}
			break;
		case 'i':
			if ( !str_cmp( arg6, "ins" ) )
			{
				stat = APPLY_INS;
				found = TRUE;
			}
			if ( !str_cmp( arg6, "int" ) )
			{
				stat = APPLY_INT;
				found = TRUE;
			}
			break;
		case 'r':
			if ( !str_cmp( arg6, "ref" ) )
			{
				stat = APPLY_REF;
				found = TRUE;
			}
			break;
		case 's':
			if ( !str_cmp( arg6, "str" ) )
			{
				stat = APPLY_STR;
				found = TRUE;
			}
			break;
		case 'w':
			if ( !str_cmp( arg6, "wis" ) )
			{
				stat = APPLY_WIS;
				found = TRUE;
			}
			break;
		}

		if ( !found )
		{
			ptc( ch, "%s is not a valid stat. See \"stitch stats\" for available choices.\n\r", arg6 );
			return;
		}

		for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
		{
			if ( !str_cmp( arg3, cloth_colors_table[i].cname ) )
			{
				color = cloth_colors_table[cloth_colors_table[i].number].aname;
				colorn = cloth_colors_table[cloth_colors_table[i].number].bname;
				cFound = TRUE;
				break;
			}
		}

		for ( i = 0; stitch_patterns_table[i].aname != NULL; i++ )
		{
			if ( !str_cmp( arg4, stitch_patterns_table[i].aname ) )
			{
				pattern = stitch_patterns_table[i].bname;
				itype = i;
				pFound = TRUE;
				break;
			}
		}

		if (!cFound)
		{
			ptc( ch, "%s is not a valid color choice. See \"stitch colors\" for available choices.\n\r", arg3 );
			return;
		}

		if (!pFound)
		{
			ptc( ch, "%s is not a valid pattern choice. See \"stitch patterns\" for available choices.\n\r", arg4 );
			return;
		}
	}

	if ( skill < stitch_patterns_table[itype].difficulty )
	{
		ptc( ch, "You do not know enough about stitching to make %s %s %s.", check_vowell(color) ? "an" : "a", color, pattern );
		return;
	}

	difficulty = number_range(1,(stitch_patterns_table[itype].difficulty/2));
	difficulty += stitch_patterns_table[itype].difficulty;
	difficulty -= skill/5;

	if ( difficulty < 0 )
		difficulty = 1;

	act( "You begin to stitch $p.", ch, obj, NULL, TO_CHAR );
	act( "$n begins to stitch $p.", ch, obj, NULL, TO_ROOM );

	WAIT_STATE( ch, stitch_patterns_table[itype].beats );

	if ( difficulty > skill )
	{
		act( "You fail to stitch the pattern.", ch, NULL, NULL, TO_CHAR );
		act( "$n fails to stitch the pattern.", ch, NULL, NULL, TO_ROOM );
		return;
	}

	check_improve( ch, gsn_stitch, TRUE, 9 );

	old_name = obj->name;
	old_name = one_argument( old_name, old_color );
	old_name = one_argument( old_name, old_material );
	old_name = one_argument( old_name, old_type );

	for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
	{
		if ( !str_cmp( old_color, cloth_colors_table[i].bname ) )
		{
			old_color2 = cloth_colors_table[cloth_colors_table[i].number].aname;
			break;
		}
	}


/*	if (lFound)
	{
		act( "You stitch lettering into $p.", ch, obj, NULL, TO_CHAR );
		act( "$n stitches lettering into $p.", ch, obj, NULL, TO_ROOM );
		sprintf( new_short, "%s %s %s %s stitched with '%s'", word1, old_color, old_material, old_type, pattern );
		sprintf( new_long, "Here lie %s %s %s %s stitched with %s", word1, old_color, old_material, old_type, pattern );
		sprintf( new_name, "%s %s %s lettered", old_color, old_material, old_type );
	}
	else
	{*/
	act( "You stitch the pattern into $p.", ch, obj, NULL, TO_CHAR );
	act( "$n stitches a pattern into $p.", ch, obj, NULL, TO_ROOM );
	sprintf( new_short, "%s %s %s %s{X stitched with %s %s %s{X", check_vowell(old_color2) ? "an" : "a", old_color2, old_material, old_type, check_vowell(color) ? "an" : "a", color, pattern );
	sprintf( new_long, "Here lie %s %s %s %s{X stitched with %s %s %s{X", check_vowell(old_color2) ? "an" : "a", old_color2, old_material, old_type, check_vowell(color) ? "an" : "a", color, pattern );
	sprintf( new_name, "%s %s %s %s %s", old_color, old_material, old_type, colorn, pattern );
//	}
	// affect time!
	bonus = skill / 10;
	if ( bonus > 10 )
		bonus = 10;

	af				= new_affect();
	af->location	= stat;
	af->modifier	= bonus;
	af->where		= TO_OBJECT;
	af->type		= -1;
	af->duration	= -1;
	af->bitvector	= 0;
	af->level		= obj->level;
	af->next		= obj->affected;
	obj->affected	= af;

	free_string( obj->short_descr );
	obj->short_descr = str_dup( new_short );
	free_string( obj->description );
	obj->description = str_dup( new_long );
	free_string( obj->name );
	obj->name = str_dup( new_name );
	obj->is_stitched = TRUE;

	return;
}

