/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

 ***************************************************************************/


/* This code generates a treasure chest, populates it with
 * coins, gemstones, and possibly a random item. The gems
 * and coins come from this file, the items from random.c
 * Chests require lockpicks to open, which is a ITEM_LOCKPICK
 * kinda thing with a v0 value ranging from 1-100.
 *
 */

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aww.h"
#include "recycle.h"
#include "interp.h"
#include "tables.h"
#include "combine.h"
#include "olc.h"

#define OBJ_VNUM_TREASURE_CHEST 1002
OBJ_DATA *	random_herb	args(( void ));

OBJ_DATA *	get_random_manual args((CHAR_DATA *ch, bool));
OBJ_DATA *	get_random_material args((CHAR_DATA *mob, int type, bool nullallowed));
OBJ_DATA *	get_random_dye args((CHAR_DATA *mob, bool nullallowed));
OBJ_DATA *random_adornment args((void));
bool check_vowell args((const char *str));


char *check_dye args((OBJ_DATA *obj, char *string, int dye_vnum));

OBJ_DATA *	make_treasure_item	args( ( int level ) );
OBJ_DATA * make_gem( int level )
{
	OBJ_DATA *gem;

	char gem_short[MSL];
	char gem_long[MSL];
	char gem_name[MSL];
	char *gem_quality_table[] = {"flawed","chipped","shiny","sparkling","flawless"};
	char *gem_size_table[]    =	{"tiny","small","medium","large"};
	char *gem_name_table[]    =	{"cat's eye gem-stone","round bloodstone","clear-white topaz",
					 "angel's skin coral","blue spinel","clear quartz",
					 "fire-colored agate","yellow-gold crysoberyl","black moonstone",
					 "bright-green peridot","amber gemstone","yellow citrine gemstone",
					 "yellow garnet","smokey-colored quartz","milky-white pearl",
					 "reddish-pink tourmaline","rock-crystal","fire opal",
					 "blue sapphire","diamond","ruby-red alexandrite",
					 "yellowish-orange jade stone","gray-white jade stone",
					 "vibrant purple amethyst","rose-colored quartz","bluish-colored quartz",
					 "milky-white topaz","angel's skin coral","greenish-blue aquamarine",
					 "pale-blue turquoise","pink sapphire","green emerald","rock-crystal",
					 "yellowed ivory","bright green beryl stone","blue turquoise",
					 "light red morganite stone","black pearl","black opal","cat's-eye beryl gemstone",
					 "slab of pure white ivory","silver nugget","gold nugget",
					 "black onyx stone","golden colored sapphire","white pearl",
					 "red ruby","light pink conch pearl","reddish-pink alexandrite",
					 "demantoid garnet stone","rose-colored quartz","green beryl stone",
					 "lapis lazuli stone","opaque colored jasper","deep blue tigereye stone",
					 "reddish-brown sardonyx","sea-green aquamarine gem","milk-and-honey colored cat's-eye chrysoberyl",
					 "star-diopside","cat's-eye diopside","reddish-cast sunstone",
					 "silver bar","gold bar","jet stone","snowflake obsidian","imperial topaz",
					 "star ruby","turquoise stone","demantoid garnet stone"};

	/* If you add more figurines, make sure to add it into is_figurine() in treasure2.c for the junk filters to work */
	char *gem_name_table2[]={"small bloodstone, sculpted into a tiny female figurine",
							 "small carving of petrified wood, in the form of a woodsman",
							 "rose crafted from seafolk porcelain",
							 "small ruby lion, reaching forward with one paw",
							 "rectangular piece of ivory, etched with a glowing sun",
							 "emerald replica of an avendesora leaf",
							 "small pony, crafted from seafolk porcelain",
							 "silverpike, fashioned from crystal edged with silver flecks",
							 "miniature seafolk raker, crafted from polished petrified wood",
							 "white quartz seagull",
							 "oyster shell, made from mother-of-pearl",
							 "silver medallion etched with a swooping falcon",
							 "gold medallion, etched with a running wolf",
							 "miniature jade wagon, etched in intricate detail",
							 "small handful of brilliant red fire drops",
							 "small moonstone obelisk",
							 "miniature copper horn, encrusted with several tiny pearls",
							 "perfect replica of a golden crane",
							 "perfect onyx hawk"};


	int i, value = 1, quality=0, size=0;
	int gem_name_int=0,gem_name2_int=0;

	gem_short[0]    = '\0';
	gem_long[0]     = '\0';
	gem_name[0]     = '\0';


	if ( level < 1 ) level = 1;

	gem = create_object( get_obj_index(OBJ_VNUM_GEM), 0 );

	i = number_range(1,level);
	i = i * 9 / 11;
	i = number_range(1,i);

	if ( i > 220 )
	{
		i = number_range(1,level);
		i = i * 9 / 11;
		i = number_range(1,i);
	}

	quality       = number_range(1,5);
	size          = number_range(1,4);
	gem_name_int  = number_range(1,69);
	gem_name2_int =	number_range(1,19);

	sprintf(gem_name, "%s %s %s", gem_size_table[size - 1], gem_quality_table[quality - 1], i < 200 ? gem_name_table[gem_name_int - 1] : gem_name_table2[gem_name2_int - 1]);
	sprintf(gem_short, "a %s", gem_name);
	sprintf(gem_long, "Here lies a %s.", gem_name);

	free_string( gem->short_descr );
	free_string( gem->description );
	free_string( gem->name );

	gem->short_descr = str_dup(gem_short);
	gem->description = str_dup(gem_long);
	gem->name = str_dup(gem_name);

	value = number_range( i, i + 50 + (quality) );
	if ( value < 1 ) value = 1;

	gem->wear_flags     = ITEM_TAKE;
	gem->guild          = str_dup("Loner");
	gem->condition      = 100;
	gem->weight         = 1;
	gem->cost           = value;

	return gem;
}

OBJ_DATA * make_treasure_chest ( int level )
{
	OBJ_DATA *box;

	char buf_name[MSL];
	char buf_short[MSL];
	char buf_long[MSL];
	char *box_desc_store[]={"iron-bound","wood","brass-bound"};
	char *box_keywd_store[]={"rusted","sturdy","heavy"};
	char *box_keywd_storeWB[]={"rotting","sturdy","heavy"};
	char *box_type_store[]={"chest","trunk","coffer","coinbox","strongbox","safebox"};
	int desc_num  = 0;
	int roll = number_range(1, 3);
	int roll2 = number_range(1, 6);

	buf_name[0]  = '\0';
	buf_short[0] = '\0';
	buf_long[0]  = '\0';

	/* Begin Creating Treasure Box
	   Treasure Box:  chest, trunk, coffer, coinbox, strongbox, safebox
	   Use pointer arrays to fill values for variables
	   box_type
	   box_desc
	   box_keywd */

	desc_num = number_range(1,3);
	/*End Creating Treasure Box */

	sprintf( buf_name, "treasure_box %s %s %s",
	desc_num == 2 ? box_keywd_storeWB[roll - 1] : box_keywd_store[roll - 1],
	box_desc_store[desc_num - 1], box_type_store[roll2 - 1] );
	sprintf( buf_short,"a %s %s %s",
	desc_num == 2 ? box_keywd_storeWB[roll - 1] : box_keywd_store[roll - 1],
	box_desc_store[desc_num - 1], box_type_store[roll2 - 1] );
	sprintf( buf_long, "Here lies a %s %s %s.",
	desc_num == 2 ? box_keywd_storeWB[roll - 1] : box_keywd_store[roll - 1],
	box_desc_store[desc_num - 1], box_type_store[roll2 - 1] );

    box = create_object(get_obj_index(OBJ_VNUM_TREASURE_CHEST),0);

	if (desc_num==2)/*if its a wood box to this*/
	{
		free_string( box->material );
		box->material = str_dup("wood");
	}
	else/*if its a metal box do this*/
	{
		free_string( box->material );
		box->material = str_dup("metal");
	}

	free_string(box->short_descr);
	box->short_descr = str_dup(buf_short);
	free_string(box->description);
	box->description = str_dup(buf_long);
	free_string(box->name);
	box->name = str_dup(buf_name);

	if ( box->extra_flags )
		box->extra_flags = 0;

	box->wear_flags     = ITEM_TAKE;
	box->item_type      = ITEM_TREASURECHEST;
	box->weight     = 30;
	box->timer          = 50;
	box->value[0]       = number_range(50, 200); /* silver coins */
	box->value[1]       = CONT_CLOSEABLE; /* container-type flags */
	box->value[1]       += CONT_CLOSED; /* container-type flags */
	box->value[1]       += CONT_LOCKED; /* container-type flags */
	box->value[2]       = box->value[0]; /* box lock difficulty */
	switch ( number_range(1,10) )
	{
		case 1:
		box->value[3]   = TRAP_BOOMER; box->trapped = TRUE; break;
		case 2:
		box->value[3]   = TRAP_GASSER; box->trapped = TRUE; break;
		case 3:
		box->value[3]   = TRAP_BLADE; box->trapped = TRUE; break;
		case 4:
		box->value[3]   = TRAP_NEEDLE; box->trapped = TRUE; break;
		case 5:
		box->value[3]   = TRAP_MANA; box->trapped = TRUE; break;
		case 6:
		box->value[3]   = TRAP_FIRE; box->trapped = TRUE; break;
		case 7:
		box->value[3]   = TRAP_ACID; box->trapped = TRUE; break;
		default:
		box->value[3]   = TRAP_NONE; box->trapped = FALSE; break;
	}

	box->value[4]       = box->value[0];
	box->guild          = str_dup("Loner");
	box->condition      = 100;
	box->cost           = box->value[0] / 4;
	box->level          = level;

	return box;
}

OBJ_DATA *make_combination_manual ( void )
{
	OBJ_DATA *manual;
	char buf[MSL];
	int roll = number_range(1, 17); // increase as more are available
	EXTRA_DESCR_DATA *ed;
	BUFFER *buffer = new_buf();

	manual = create_object(get_obj_index(36999), 1);
	ed = new_extra_descr();

	if (get_obj_index(combine_table[roll].comVnum) == NULL)
		add_buf(buffer, "This manual is broken. Please alert an IMM (Doesn't create a valid item).");
	else
	{
		free_string(manual->name);
		sprintf(buf, "combination manual %s", get_obj_index(combine_table[roll].comVnum)->name);
		manual->name = str_dup(buf);

		free_string(manual->short_descr);
		sprintf(buf, "a {wcombination manual {xfor crafting %s %s{x",
		check_vowell(get_obj_index(combine_table[roll].comVnum)->short_descr) ? "an" : "a",
		get_obj_index(combine_table[roll].comVnum)->short_descr);
		manual->short_descr = str_dup(buf);

		sprintf(buf, "The manual reads: To make %s %s,", check_vowell(get_obj_index(combine_table[roll].comVnum)->short_descr) ? "an" : "a",
		get_obj_index(combine_table[roll].comVnum)->short_descr);
		add_buf(buffer, buf);
	}

	if (get_obj_index(combine_table[roll].vnum1) == NULL)
		add_buf(buffer, "This manual is broken. Please alert an IMM (Vnum1 isn't a valid item).");
	else
	{
		sprintf(buf, " simply combine %s %s", check_vowell(get_obj_index(combine_table[roll].vnum1)->short_descr) ? "an" : "a",
		get_obj_index(combine_table[roll].vnum1)->short_descr);
		add_buf(buffer, buf);
	}

	if (get_obj_index(combine_table[roll].vnum2) == NULL)
		add_buf(buffer, "This manual is broken. Please alert an IMM (Vnum2 isn't a valid item).");
	else
	{
		sprintf(buf, " with %s %s.\n\r", check_vowell(get_obj_index(combine_table[roll].vnum2)->short_descr) ? "an" : "a",
		get_obj_index(combine_table[roll].vnum2)->short_descr);
		add_buf(buffer, buf);
	}


	ed->description = str_dup(buf_string(buffer));

	sprintf(buf, "%s", "manual");
	ed->keyword = str_dup(buf);
	ed->next            = manual->extra_descr;
	manual->extra_descr    = ed;

	return manual;
}

void open_treasurechest( OBJ_DATA *box )
{
	int coins = 0, sp = 0, sm = 0, gm = 0, level = 0;

	/* put some coins in the box */
	level = box->level;

	coins = number_range( 10, 30 );

	if ( number_percent() / 3 > number_percent() )
	{
		sp = number_range( 3, 5 );
	}
	if ( number_percent() / 5 > number_percent() )
	{
		sm = number_range( 2, 3 );
	}
	if ( number_percent() / 7 > number_percent() )
	{
		gm = number_range( 1, 2 );
	}
	obj_to_obj( create_money( coins, sp, sm, gm ), box );

	/* Check to see if they get crafting manuals/materials/dyes */
	if (number_range(1, 50) == 1)	obj_to_obj(get_random_material(NULL, 0, FALSE), box);
	if (number_range(1, 50) == 1)	obj_to_obj(get_random_material(NULL, 0, FALSE), box);
	if (number_range(1, 50) == 1)	obj_to_obj(get_random_manual(NULL, FALSE), box);
	if (number_range(1, 50) == 1)	obj_to_obj(get_random_dye(NULL, FALSE), box);
	if (number_range(1, 75) == 1)	obj_to_obj(make_combination_manual(), box);
	if (number_range(1, 50) == 1)	obj_to_obj(random_adornment(), box);

	/* see if the box has any gems - now they always have gems
	 * plus we add a chance or any case to have an item. this
	 * will work much better when we have all the new items in
	 * from the iTyper project
	 */

	switch ( number_range(1,6) )
	{
		case 1:
			obj_to_obj(make_gem( level ), box);
			if ( number_percent() / 2 > number_percent() )
				obj_to_obj( make_treasure_item(level), box );
		break;
		case 2:
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			if ( number_percent() / 2 > number_percent() )
				obj_to_obj( make_treasure_item(level), box );
		break;
		case 3:
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			if ( number_percent() / 2 > number_percent() )
				obj_to_obj( make_treasure_item(level), box );
		break;
		case 4:
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			if ( number_percent() / 2 > number_percent() )
				obj_to_obj( make_treasure_item(level), box );
		break;
		case 5:
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			if ( number_percent() / 2 > number_percent() )
				obj_to_obj( make_treasure_item(level), box );
		break;
		case 6:
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			obj_to_obj(make_gem( level ), box);
			if ( number_percent() / 2 > number_percent() )
				obj_to_obj( make_treasure_item(level), box );
		break;
	}
	return;
}


void do_snapgear (CHAR_DATA *ch, char *argument)
{
	OBJ_DATA *obj;
	char arg[MSL], arg2[MSL];
	CHAR_DATA *victim;
	OBJ_INDEX_DATA *pobj;
	int i, r, g, b;
	char buf[MSL], buf2[MSL*4];

	argument = one_argument(argument, arg);
	argument = one_argument(argument, arg2);

	if (!str_cmp(arg, "gear"))
	{
		obj = make_treasure_item(1000);
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", obj->short_descr);
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	if (!str_cmp(arg, "color"))
	{
		buf2[0] = '\0';
		r = 0;
		g = 0;
		b = 0;

		for (;;)
		{
			b++;
			if (b > 5) { b = 0; g++; }
			if (g > 5) { g = 0; r++; }
			if (r > 5) break;

			sprintf(buf, "{:[F%d%d%d][%d%d%d]", r, g, b, r, g, b);
			strcat(buf2, buf);
		}

		stc(buf2, ch);
	}
	else if (!str_cmp(arg, "setman"))
	{
		for (i=36500; i < 36999; i += 2)
		{
			if ((pobj = get_obj_index(i)) == NULL)		continue;
			if (pobj->item_type != ITEM_CRAFTING_MANUAL)	continue;
			

			free_string(pobj->name);
			sprintf(buf, "%s manual", get_obj_index(i+1)->name);
			pobj->name = str_dup(buf);

			free_string(pobj->short_descr);
			sprintf(buf, "a manual for %s %s %s%s",
			flag_string(craftskill_flags, pobj->value[0]),
			pobj->value[3] != 0 ? "a" : check_vowell(get_obj_index(i+1)->short_descr) ? "an" : "a",
			pobj->value[3] != 0 ? flag_string(craftpadj_flags, pobj->value[3]) : "",
			get_obj_index(i+1)->short_descr);
			pobj->short_descr = str_dup(buf);

			free_string(pobj->description);
			sprintf(buf, "A manual for %s %s %s%s is lying here.",
			flag_string( craftskill_flags, pobj->value[0]),
			pobj->value[3] != 0 ? "a" : check_vowell(get_obj_index(i+1)->short_descr) ? "an" : "a",
			pobj->value[3] != 0 ? flag_string(craftpadj_flags, pobj->value[3]) : "",
			get_obj_index(i+1)->short_descr);
			pobj->description = str_dup(buf);

			pobj->value[1] = i+1;
		}

		stc("You snap your fingers and all of the manuals are adjusted!\n\r", ch);
	}
	else if (!str_cmp(arg, "material"))
	{
		if (!str_cmp(argument, "wood"))
			obj = get_random_material(ch, CRAFT_WOODCRAFT, FALSE);
		else if (!str_cmp(argument, "metal"))
			obj = get_random_material(ch, CRAFT_FORGE, FALSE);
		else if (!str_cmp(argument, "jewel"))
			obj = get_random_material(ch, CRAFT_JEWELCRAFT, FALSE);
		else if (!str_cmp(argument, "cloth"))
			obj = get_random_material(ch, CRAFT_SEW, FALSE);
		else
			obj = get_random_material(ch, 0, FALSE);
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", check_dye(obj, NULL, obj->dye_vnum));
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "manual"))
	{
		obj = get_random_manual(ch, FALSE);
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", check_dye(obj, NULL, obj->dye_vnum));
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "cmanual"))
	{
		obj = make_combination_manual();
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", check_dye(obj, NULL, obj->dye_vnum));
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "herb"))
	{
		obj = random_herb();
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", obj->short_descr);
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "chest"))
	{
		obj = make_treasure_chest(100);
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", obj->short_descr);
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
		REMOVE_BIT(obj->value[1], CONT_LOCKED);
		REMOVE_BIT(obj->value[1], CONT_CLOSED);
		open_treasurechest(obj);
	}
	else if (!str_cmp(arg, "jewel"))
	{
		obj = make_gem(1000);
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", obj->short_descr);
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "adornment"))
	{
		obj = random_adornment();
		obj_to_char(obj, ch);

		ptc(ch, "You snap your fingers and %s appears in your hands!\n\r", check_dye(obj, NULL, obj->dye_vnum));
		act("$n snaps $s fingers and $p appears in $s hands!", ch, obj, NULL, TO_ROOM);
	}
	else if (!str_cmp(arg, "reset"))
	{
		if (arg2[0] == '\0')
		{
			stc("Reset whom?\n\r", ch);
			return;
		}

		if ((victim = get_char_world(ch, arg2)) == NULL)
		{
			stc("Character not found.\n\r", ch);
			return;
		}

		reset_char(victim);
		act("$n snaps $s fingers and resets your character!\n\r", ch, NULL, victim, TO_VICT);
		ptc(ch, "You snap your fingers and reset %s's character!\n\r", victim->name);
	}


	return;
}

