/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"

/* [footmen, archers, cavalry, channelers][number of this unit]
 * --------------------------------------------------------------------------------
 * one unit is comprised of 100 soldiers excluding channelers.
 * one unit of channelers is one channeler
 * units can be compared to companies of today's armies
 * units can be combined to form batallions
 * battalions can be combined to form brigades
 * brigades can be combined to form divisions
 * divisions can be combined to form corps
 * for every 100 troops a captain should be hired at 1/10 the total unit cost
 * a captain gives a unit a 1/10 bonus to attack and defense
 * no more than 1 captain per 100 tropps
 * for every 1000 troops a commander should be hired at 1/10 the combined units' cost
 * a commander gives a batallion a 1/10 bonus to attack and defense
 * for every 10,000 troops a general should be hired at 1/10 the combined units' cost
 * a general gives all units a 1/10 bonus to attack and defense
 * every channeler gives all troops a 1/10 bonus to attack and defense
 * channelers are the most expensive unit
 * footmen = 100 gold / unit
 * archer = 200 gold / unit
 * cavalry = 500 gold / unit
 * channelers = 1000 gold / unit
 * --------------------------------------------------------------------------------
 */

/*
 * New Skills:
 * -----------
 * Combat Tactics
 * Siege Warfare
 *

 * ch->in_castle

 */
#define msg ptc

void war_deploy        args( ( CHAR_DATA *ch, char *argument ) );
void war_charge        args( ( CHAR_DATA *ch, char *argument ) );
void war_talk          args( ( CHAR_DATA *ch, char *argument ) );
void war_outfit        args( ( CHAR_DATA *ch, char *argument ) );
void war_surrender     args( ( CHAR_DATA *ch, char *argument ) );
void war_retreat       args( ( CHAR_DATA *ch, char *argument ) );
void war_rename        args( ( CHAR_DATA *ch, char *argument ) );
void war_sell          args( ( CHAR_DATA *ch, char *argument ) );
void war_combine       args( ( CHAR_DATA *ch, char *argument ) );
void war_formation     args( ( CHAR_DATA *ch, char *argument ) );
void war_info          args( ( CHAR_DATA *ch, char *argument ) );
void war_build         args( ( CHAR_DATA *ch, char *argument ) );
void war_wall          args( ( CHAR_DATA *ch, char *argument ) );
void war_garrison      args( ( CHAR_DATA *ch, char *argument ) );
void war_declare       args( ( CHAR_DATA *ch, char *argument ) );
void war_claim         args( ( CHAR_DATA *ch, char *argument ) );
void war_purchase      args( ( CHAR_DATA *ch, char *argument ) );
void war_help          args( ( CHAR_DATA *ch, char *argument ) );
void do_war            args( ( CHAR_DATA *ch, char *argument ) );
void war_null          args( ( CHAR_DATA *ch, char *argument ) );
void war_titles        args( ( CHAR_DATA *ch, char *argument ) );

struct warType
{
	char * keyword;
	void ( * command )( CHAR_DATA *ch, char *argument );
	char * syntax;
};
const struct warType warTable [] =
{
	{ "outfit",    war_outfit,    "War outfit 'unit name' with 'item name'"    },
	{ "surrender", war_surrender, "War surrender to 'enemy name'" },
	{ "retreat",   war_retreat,   "War retreat 'unit name' 'quickly|guarded'"   },
	{ "rename",    war_retreat,   "War rename 'unit name' 'new name'"   },
	{ "sell",      war_sell,      "War sell 'unit name'"      },
	{ "combine",   war_combine,   "War combine 'unit name' with 'unit name' into 'unit name'"   },
	{ "formation", war_formation, "War formation 'unit name' <optional> 'position'" },
	{ "info",      war_info,      "War info 'enemy name'"      },
	{ "build",     war_build,     "War build 'camp|fort|hold|keep|castle|stronghold"      },
	{ "wall",      war_wall,      "War wall 'shields|pikes"      },
	{ "garrison",  war_garrison,  "War garrison 'unit name'"      },
	{ "declare",   war_declare,   "War declare on 'enemy name'"   },
	{ "purchase",  war_purchase,  "War purchase 'unit type' 'unit name'"  },
	{ "claim",     war_claim,     "War claim (attempt to lay ownership claim the area)" },
	{ "help",      war_help,      "War help..."      },
	{ "?",         war_null,      "War ? <function> (display specific funcion's help)"      },
	{ "deploy",    war_deploy,    "War deploy 'unit name' against 'enemy name' 'enemy unit'" },
	{ "charge",    war_charge,    "War charge 'unit name' against 'enemy name' 'enemy unit'" },
	{ "titles",    war_titles,    "Displays titles and required honor points"      },
	{ "talk",      war_talk,      NULL      },
	{ NULL,        0,             ""             },
};

void war_help( CHAR_DATA *ch, char *argument )
{
	sh_int i;
	for ( i = 0; warTable[i].keyword != NULL; i++ )
	{
		if ( warTable[i].syntax == NULL )
			continue;
		msg( ch, "%-11s %s\n\r", warTable[i].keyword, warTable[i].syntax );
	}
}

void do_war( CHAR_DATA *ch, char *argument )
{
	char arg[MIL];
	bool found = FALSE;
	sh_int i;

	argument = one_argument( argument, arg );

	for ( i = 0; warTable[i].keyword != NULL; i++ )
	{
		if ( !str_cmp( arg, warTable[i].keyword ) )
		{
			( *warTable[i].command )( ch, argument );
			found = TRUE;
			return;
		}
	}
	if ( !found )
	{
		msg( ch, "War: Could not find that command. Try War Help.\n\r" );
		return;
	}
    return;
}

void war_claim( CHAR_DATA *ch, char *argument )
{
        return;
}
void war_titles( CHAR_DATA *ch, char *argument )
{
	show_war_titles( ch );
	return;
}
void war_null( CHAR_DATA *ch, char *argument )
{
	return;
}
void war_garrison( CHAR_DATA *ch, char *argument )
{
	return;
}
void war_build( CHAR_DATA *ch, char *argument )
{
	return;
}
void war_wall( CHAR_DATA *ch, char *argument )
{
	return;
}

void war_purchase( CHAR_DATA *ch, char *argument )
{
	char arg[MIL], unitname[MIL];

	argument = one_argument( argument, arg );
	argument = one_argument_with_caps( argument, unitname );

	//msg( ch, "Arg: %s :: Unitname: %s (%s)\n\r", arg, unitname, unit_name );
	/* purchase a unit type and give it a name */
	/* example: war purchase footmen 'Red Hand' */
	/* example: war purchase cavalry 'Thunder Hooves' */
	/* the name the unit is given is how the unit is called by other commands */
	/* example: war deploy 'Red Hand' against Dalsor 'Blade Runners' */
	/* example: war outfit 'Thunder Hooves' lance */
	if ( emptyStr( arg ) || emptyStr( unitname ) )
	{
		msg( ch, "Purchase what type of unit and name the company what?\n\r" );
		msg( ch, "Example: war purchase cavalry 'Red Hand'\n\r" );
		return;
	}
	if ( has_color_codes( unitname ) )
	{
		msg( ch, "Unit names cannot contain color codes.\n\r" );
		return;
	}
	if ( !war_purchase_unit( ch, arg, unitname ) )
		return;

	return;
}

void war_info( CHAR_DATA *ch, char *argument )
{
	CHAR_DATA *enemy;
	int i = 0;
	char * war_title = "General";

	//msg( ch, "Argument: %s\n\r", argument );
	/* show enemy units and their names */
	/* example: war info Dalsor */
	/* Dalsor has an army comprised of:
	 * 100 cavalry 'Night Lances'
	 * 1000 footmen 'Steel Teeth'
	 * 2 channelers 'Storm Guard'
	 */
	if ( emptyStr( argument ) )
	{
		msg( ch, "Get info about who?\n\r" );
		return;
	}
	if ( ( enemy = get_exactchar_world( ch, argument ) ) == NULL )
	{
		msg( ch, "No luck getting info about non-existent people.\n\r" );
		return;
	}
	if ( IS_NPC( enemy ) || !enemy->desc )
	{
		msg( ch, "You can't get info about NPCs or link-dead people.\n\r" );
		return;
	}
	msg( ch, "%s %s has an army comprised of the following units:\n\r", war_title, enemy->name );
	msg( ch, "\n\rInfantry Units\n\r-----------------------------------------\n\r" );
	for ( i = 0; i < MAX_UNITS; i++ )
	{
		msg( ch, "Division %d :: %-5d footmen %-15s    with strength of %d\n\r",
			i+1,
			enemy->pcdata->war_footmen_num[i],
			emptyStr( enemy->pcdata->war_footmen[i] ) ? "None" : enemy->pcdata->war_footmen[i],
			new_get_unit_strength( enemy, enemy->pcdata->war_footmen[i], 1 ) );
	}
	for ( i = 0; i < MAX_UNITS; i++ )
	{
		msg( ch, "Division %d :: %-5d archers %-15s    with strength of %d\n\r",
			i+1,
			enemy->pcdata->war_archers_num[i],
			emptyStr( enemy->pcdata->war_archers[i] ) ? "None" : enemy->pcdata->war_archers[i],
			new_get_unit_strength( enemy, enemy->pcdata->war_archers[i], 2 ) );
	}
	for ( i = 0; i < MAX_UNITS; i++ )
	{
		msg( ch, "Division %d :: %-5d pikemen %-15s    with strength of %d\n\r",
			i+1,
			enemy->pcdata->war_pikemen_num[i],
			emptyStr( enemy->pcdata->war_pikemen[i] ) ? "None" : enemy->pcdata->war_pikemen[i],
			new_get_unit_strength( enemy, enemy->pcdata->war_pikemen[i], 3 ) );
	}
	msg( ch, "\n\rHeavy Units\n\r-----------------------------------------\n\r" );
	for ( i = 0; i < MAX_UNITS; i++ )
	{
		msg( ch, "Division %d :: %-5d cavalry %-15s    with strength of %d\n\r",
			i+1,
			enemy->pcdata->war_cavalry_num[i],
			emptyStr( enemy->pcdata->war_cavalry[i] ) ? "None" : enemy->pcdata->war_cavalry[i],
			new_get_unit_strength( enemy, enemy->pcdata->war_cavalry[i], 4 ) );
	}
	for ( i = 0; i < MAX_UNITS; i++ )
	{
		msg( ch, "Division %d :: %-5d channelers %-15s with strength of %d\n\r",
			i+1,
			enemy->pcdata->war_channelers_num[i],
			emptyStr( enemy->pcdata->war_channelers[i] ) ? "None" : enemy->pcdata->war_channelers[i],
			new_get_unit_strength( enemy, enemy->pcdata->war_channelers[i], 5 ) );
	}
	msg( ch, "\n\rThe total strength of the army is %d.\n\r", count_army( enemy, TRUE ) );
	return;
}

void war_deploy( CHAR_DATA *ch, char *argument )
{
	/* deploy a unit against an enemy unit */
	/* example: war deploy 'Red Hand' against Dalsor 'Blade Runners' */
	/* once a unit is deployed they attack every round until dead, victorious, or they retreat */
	char arg1[MIL], arg2[MIL], arg3[MIL], arg4[MIL], buf[MSL];
	char *unit, *enemy_unit;
	CHAR_DATA *enemy;

	argument = one_argument_with_caps( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	argument = one_argument_with_caps( argument, arg4 );

	//msg( ch, "Arg1: %s :: Arg2: %s :: Arg3: %s :: Arg4: %s\n\r", arg1, arg2, arg3, arg4 );
	if ( emptyStr( ch->warring_with ) )
	{
		msg( ch, "You're not at war.\n\r" );
		return;
	}
	if ( emptyStr( arg1 ) )
	{
		msg( ch, "Deploy *which unit* against who's unit?\n\r" );
		return;
	}
	if ( ( unit = get_war_unit( ch, NULL, arg1 ) ) == NULL )
	{
		msg( ch, "Can't find any units by that name. Check your army info.\n\r" );
		return;
	}
	if ( emptyStr( arg2 ) )
	{
		msg( ch, "Deploy which unit *against* who's unit?\n\r" );
		return;
	}
	if ( strcmp( arg2, "against" ) )
	{
		msg( ch, "Deploy which of your units *against* who's unit?\n\r" );
		return;
	}
	if ( emptyStr( arg3 ) )
	{
		msg( ch, "Deploy which unit against *who's* unit?\n\r" );
		return;
	}
	if ( ( enemy = get_exactchar_world( ch, arg3 ) ) == NULL )
	{
		msg( ch, "No luck deploying units against non-existent people.\n\r" );
		return;
	}
/*
	if ( ch == enemy )
	{
		msg( ch, "Do you really want your soldiers to revolt?\n\r" );
		return;
	}
*/
	if ( strcmp( enemy->name, ch->warring_with ) )
	{
		msg( ch, "You can't deploy against someone you're not at war with.\n\r" );
		return;
	}
	if ( IS_NPC( enemy ) || !enemy->desc )
	{
		msg( ch, "You can't deploy against NPCs or link-dead people.\n\r" );
		return;
	}
	if ( emptyStr( arg4 ) )
	{
		msg( ch, "Deploy which unit against who's *unit*?\n\r" );
		return;
	}
	if ( ( enemy_unit = get_war_unit( NULL, enemy, arg4 ) ) == NULL )
	{
		msg( ch, "Can't find any enemy units by that name. Check your enemy's army info.\n\r" );
		return;
	}
	sprintf( buf, "{W%s deploys %s against %s's %s!{x", ch->name, unit, enemy->name, enemy_unit );
	war_talk( ch, buf );
}

void war_charge( CHAR_DATA *ch, char *argument )
{
	/* Charge a unit against an enemy unit */
	/* example: war charge 'Red Hand' against Dalsor 'Blade Runners' */
	/* once a unit charges they are deployed and received an initial attack bonus, but defense penalty */
	/* as with deploy, they attack every round until dead, victorious, or they retreat */
	char arg1[MIL], arg2[MIL], arg3[MIL], arg4[MIL], buf[MSL];
	char *unit, *enemy_unit;
	CHAR_DATA *enemy;

	argument = one_argument_with_caps( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	argument = one_argument_with_caps( argument, arg4 );

	//msg( ch, "Arg1: %s :: Arg2: %s :: Arg3: %s :: Arg4: %s\n\r", arg1, arg2, arg3, arg4 );
	if ( emptyStr( ch->warring_with ) )
	{
		msg( ch, "You're not at war.\n\r" );
		return;
	}
	if ( emptyStr( arg1 ) )
	{
		msg( ch, "Charge *which unit* against who's unit?\n\r" );
		return;
	}
	if ( ( unit = get_war_unit( ch, NULL, arg1 ) ) == NULL )
	{
		msg( ch, "Can't find any units by that name. Check your army info.\n\r" );
		return;
	}
	if ( emptyStr( arg2 ) )
	{
		msg( ch, "Charge which unit *against* who's unit?\n\r" );
		return;
	}
	if ( strcmp( arg2, "against" ) )
	{
		msg( ch, "Charge which of your units *against* who's unit?\n\r" );
		return;
	}
	if ( emptyStr( arg3 ) )
	{
		msg( ch, "Charge which unit against *who's* unit?\n\r" );
		return;
	}
	if ( ( enemy = get_exactchar_world( ch, arg3 ) ) == NULL )
	{
		msg( ch, "No luck charging units against non-existent people.\n\r" );
		return;
	}
/*
	if ( ch == enemy )
	{
		msg( ch, "Do you really want your soldiers to revolt?\n\r" );
		return;
	}
*/
	if ( strcmp( enemy->name, ch->warring_with ) )
	{
		msg( ch, "You can't charge against someone you're not at war with.\n\r" );
		return;
	}
	if ( IS_NPC( enemy ) || !enemy->desc )
	{
		msg( ch, "You can't charge against NPCs or link-dead people.\n\r" );
		return;
	}
	if ( emptyStr( arg4 ) )
	{
		msg( ch, "Charge which unit against who's *unit*?\n\r" );
		return;
	}
	if ( ( enemy_unit = get_war_unit( NULL, enemy, arg4 ) ) == NULL )
	{
		msg( ch, "Can't find any enemy units by that name. Check your enemy's army info.\n\r" );
		return;
	}
	sprintf( buf, "{W%s charges %s against %s's %s!{x", ch->name, unit, enemy->name, enemy_unit );
	war_talk( ch, buf );
}

void war_talk( CHAR_DATA *ch, char *argument )
{
	/* called by external functions. players do not use */
	DESCRIPTOR_DATA *d;
	CHAR_DATA *original;

	for (d = descriptor_list; d != NULL; d = d->next)
	{
		original = d->original ? d->original : d->character;
		if ( d && d->character && d->connected == CON_PLAYING )
			msg( original, "{W<{RWAR{W>{r %s{x\n\r", argument );
	}
}

void war_outfit( CHAR_DATA *ch, char *argument )
{
	/* outfit a unit with armor and weapons */
	/* outfit <unit> with <item type> */
	/* the cost is determined by the armament type and number of troops */
	/* outfitting gives a unit a bonus to attack or defense */
}

void war_surrender( CHAR_DATA *ch, char *argument )
{
	/* surrender all units to an enemy. the enemy gains half of those units.
	 * the other half are considered deserters or whatever.
	 */
}

void war_retreat( CHAR_DATA *ch, char *argument )
{
	/* a unit attempts to retreat to safety */
	/* can retreat safely and enemy gets severe penalty to last attack */
	/* can retreat quickly and units flee but enemy gets minor penalty to last attack */
}

void war_sell( CHAR_DATA *ch, char *argument )
{
	/* sell a unit */
}

void war_combine( CHAR_DATA *ch, char *argument )
{
	/* combine units to form batallions, brigades, etc
	 * example: warm combine 'Night Lances' with 'Steel Teeth' into 'Fangs'
	 * the new units 'Fangs' now has a bonus to attack and defense equal to the number
	 * of total units plus half of the strongest units attack and defense
	 */
}

void war_formation( CHAR_DATA *ch, char *argument )
{
	/* set units into formations
	 * once in battle, units cannot change formation unless the enemy retreats
	 */
}

void war_declare( CHAR_DATA *ch, char *argument )
{
	char arg1[MIL], arg2[MIL], buf[MIL];
	CHAR_DATA *enemy;

	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );

	if ( emptyStr( arg1 ) )
	{
		msg( ch, "Declare war on who?\n\r" );
		return;
	}
	if ( strcmp( arg1, "on" ) )
	{
		msg( ch, "Declare war *on* who?\n\r" );
		return;
	}
	if ( emptyStr( arg2 ) )
	{
		msg( ch, "You have to specify someone to declare war on.\n\r" );
		return;
	}
	if ( ( enemy = get_exactchar_world( ch, arg2 ) ) == NULL )
	{
		msg( ch, "No luck declaring war on non-existent people.\n\r" );
		return;
	}
	if ( IS_NPC( enemy ) || !enemy->desc )
	{
		msg( ch, "You can't declare war on NPCs or link-dead people.\n\r" );
		return;
	}
	if ( !count_army( enemy, FALSE ) )
	{
		msg( ch, "%s doesn't have an army and therefore cannot do battle with you.\n\r", enemy->name );
		return;
	}
	if ( !emptyStr( ch->warring_with ) )
		free_string( ch->warring_with );

	ch->warring_with = str_dup(enemy->name);
	sprintf( buf, "{W%s declares {RWAR{W on %s!{x", ch->name, enemy->name );
	war_talk( ch, buf );
	return;
}
