/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.

    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.

    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "magic.h"
#include "recycle.h"

/*
 * Local functions.
 */
void	say_spell	args( ( CHAR_DATA *ch, int sn ) );
void    can_see_channel args( ( CHAR_DATA *ch, CHAR_DATA *list ) );
int 	calc_weave_damage( CHAR_DATA *ch, int sn, int ang_bonus );
int		calc_weave_damage2( CHAR_DATA *ch, int sn, int ang_bonus );
/* imported functions */
bool    remove_obj      args( ( CHAR_DATA *ch, int iWear, bool fReplace ) );
void 	wear_obj	args( ( CHAR_DATA *ch, OBJ_DATA *obj, bool fReplace ) );
/*bool    can_gate_to args( ( CHAR_DATA *ch, char *target_name ) );*/
void	raw_kill	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) );


/*
 * Lookup a skill by name.
 */

int weave_damage( CHAR_DATA *ch, CHAR_DATA *victim, int input, int sn )
{
	int dmg = 0, op = 0, pwrBonus = 0, total = 0;

	dmg = ch->pcdata->learned[sn];
	ptc( ch, "dmg ch->pcdata->learned[sn] %d\n\r", dmg );
	total += dmg;
	ptc( ch, "total now %d\n\r", total );

	op = true_op_average( ch );
	ptc( ch, "op true_op_average( ch ) %d\n\r", op );
	total += op;
	ptc( ch, "total now %d\n\r", total );

	pwrBonus = true_angreal_bonus( ch );
	ptc( ch, "pwrBonus true_angreal_bonus( ch ) %d\n\r", pwrBonus );
	total += pwrBonus;
	ptc( ch, "total now %d\n\r", total );

	total += int_app[get_curr_stat(ch,STAT_INT)].op_focus;
	ptc( ch, "focus int_app[get_curr_stat(ch,STAT_INT)].op_focus %d\n\r", total );

	ptc( ch, "\n\r" );
	total += skill_table[sn].min_mana;
	ptc( ch, "skill perc skill_table[sn].min_mana %d\n\r", total );

	ptc( ch, "Last Total %d\n\r", total );
	total *= input;
	ptc( ch, "Times Input for Final total: %d\n\r", total );

	return total;
}

int skill_lookup( const char *name )
{
    int sn,found = -1;
    for ( sn = 0; sn < MAX_SKILL; sn++ )
    {
		if ( skill_table[sn].name == NULL )
		    break;
		if ( LOWER(name[0]) == LOWER(skill_table[sn].name[0])
		&&   !str_prefix( name, skill_table[sn].name ) )
		{
		    if ( found == -1)
			found = sn; /* else return first in line */
		    if(! str_cmp(name,skill_table[sn].name) ) /* exact match, return it */
		    	return sn;
		}
    }
    return found;
}

int find_spell( CHAR_DATA *ch, const char *name )
{
    /* finds a spell the character can cast if possible */
    int sn, found = -1;

    if (IS_NPC(ch))
	return skill_lookup(name);

    for ( sn = 0; sn < MAX_SKILL; sn++ )
    {
		if (skill_table[sn].name == NULL)
		    break;

		if ( LOWER(name[0]) == LOWER(skill_table[sn].name[0] )
				&& !str_prefix(name,skill_table[sn].name ) )
		{
			if ( found == -1)
				found = sn; /* else return first in line */
			if(! str_cmp(name,skill_table[sn].name) ) /* exact match, return it */
				return sn;
			/*
		    	if ( ch->pcdata->learned[sn] > 0)
				return sn;
				*/

		}
    }
    return found;
}

/*
 * Lookup a skill by slot number.
 * Used for object loading.
 */
int slot_lookup( int slot )
{
    extern bool fBootDb;
    int sn;

    if ( slot <= 0 )
	return -1;

    for ( sn = 0; sn < MAX_SKILL; sn++ )
    {
	if ( slot == skill_table[sn].slot )
	    return sn;
    }

    if ( fBootDb )
    {
	bug( "Slot_lookup: bad slot %d.", slot );
	abort( );
    }

    return -1;
}

void check_healing ( CHAR_DATA *ch, CHAR_DATA *victim )
{
	int percent;
	if (victim->max_hit > 0)
	{
		percent = victim->hit * 100 / get_maxhit(victim);
		ptc(ch,"Your patient is up to %d percent of %s health.\n\r",percent,
		victim->sex == SEX_FEMALE ? "her" : "his");
	}
}

int op_average (CHAR_DATA *ch)
{
	int air = 0;
	int earth = 0;
	int fire = 0;
	int water = 0;
	int spirit = 0;
	int total = 0;
	if (IS_NPC(ch))
		return 1;
	if (!IS_SET(ch->op_flags,OP_CHANNELER))
		return get_curr_stat( ch, STAT_INT ) / 10;
	air = ch->pcdata->op_air;
	earth = ch->pcdata->op_earth;
	fire = ch->pcdata->op_fire;
	water = ch->pcdata->op_water;
	spirit = ch->pcdata->op_spirit;
	total = air+earth+fire+water+spirit;
	/*total /= 5;*/
	return total;
}
int true_op_average (CHAR_DATA *ch)
{
	int air = 0;
	int earth = 0;
	int fire = 0;
	int water = 0;
	int spirit = 0;
	int total = 0;
	if (IS_NPC(ch))
		return 1;
	if (!IS_SET(ch->op_flags,OP_CHANNELER))
		return get_curr_stat( ch, STAT_INT ) / 10;
	air = ch->pcdata->op_air;
	earth = ch->pcdata->op_earth;
	fire = ch->pcdata->op_fire;
	water = ch->pcdata->op_water;
	spirit = ch->pcdata->op_spirit;
	total = air+earth+fire+water+spirit;
	total /= 5;
	return total;
}
int sphere_strength( CHAR_DATA *ch, int stat )
{
	if ( IS_NPC( ch ) )
		return 1;
	if ( stat == STAT_SPIRIT )
		return ch->pcdata->op_spirit;
	else if ( stat == STAT_EARTH )
		return ch->pcdata->op_earth;
	else if ( stat == STAT_AIR )
		return ch->pcdata->op_air;
	else if ( stat == STAT_FIRE )
		return ch->pcdata->op_fire;
	else if ( stat == STAT_WATER )
		return ch->pcdata->op_water;
	else return 1;
}
void check_stat_improve ( CHAR_DATA *ch, int stat, int power )
{
	int chance;
	int chance2;

	if (IS_NPC(ch))
	return;
	if ( !IS_SET( ch->op_flags,OP_CHANNELER ) )
	return;
	if ( stat == STAT_SPIRIT )
	{
		if ( ch->pcdata->op_spirit >= 12 )
		return;
		chance = (power / 2 * number_range(2,3));
		chance += ch->pcdata->op_spirit;
		chance = UMIN(25,chance);
		if ( chance > (number_percent() + true_op_average(ch)) )
		{
			if ( chance > number_percent() )
			{
			stc( "* You feel your power in Spirit increase *\n\r",ch );
			ch->pcdata->op_spirit += 1;
			}
		}
		return;
	}
	else if ( stat == STAT_AIR )
	{
		if ( ch->pcdata->op_air >= 12 )
		return;
		chance = (power / 2 * number_range(2,3));
		chance += ch->pcdata->op_air;
		chance = UMIN(25,chance);
		if ( chance > (number_percent() + true_op_average(ch)) )
		{
			if ( chance > number_percent() )
			{
			stc( "* You feel your power in Air increase *\n\r",ch );
			ch->pcdata->op_air += 1;
			}
		}
		return;
	}
	else if ( stat == STAT_EARTH )
	{
		if ( ch->pcdata->op_earth >= 12 )
		return;
		chance = (power / 2 * number_range(2,3));
		chance += ch->pcdata->op_earth;
		chance = UMIN(25,chance);
		if ( chance > (number_percent() + true_op_average(ch)) )
		{
			if ( chance > number_percent() )
			{
			stc( "* You feel your power in Earth increase *\n\r",ch );
			ch->pcdata->op_earth += 1;
			}
		}
		return;
	}
	else if ( stat == STAT_FIRE )
	{
		if ( ch->pcdata->op_fire >= 12 )
		return;
		chance = (power / 2 * number_range(2,3));
		chance += ch->pcdata->op_fire;
		chance2 = number_percent() + true_op_average(ch);
		chance = UMIN(25,chance);
		if ( chance > chance2 )
		{
			if ( chance > number_percent() )
			{
			stc( "* You feel your power in Fire increase *\n\r",ch );
			ch->pcdata->op_fire += 1;
			}
		}
		return;
	}
	else if ( stat == STAT_WATER )
	{
		if ( ch->pcdata->op_water >= 12 )
		return;
		chance = (power / 2 * number_range(2,3));
		chance += ch->pcdata->op_water;
		chance = UMIN(25,chance);
		if ( chance > (number_percent() + true_op_average(ch)) )
		{
			if ( chance > number_percent() )
			{
			stc( "* You feel your power in Water increase *\n\r",ch );
			ch->pcdata->op_water += 1;
			}
		}
		return;
	}
	else if ( stat == STAT_ALL )
	{
		chance = power;
		if ( chance > (number_percent() + op_average(ch)) )
		{
			if ( chance > number_percent() )
			check_stat_improve(ch,STAT_SPIRIT,power);
			check_stat_improve(ch,STAT_AIR,power);
			check_stat_improve(ch,STAT_EARTH,power);
			check_stat_improve(ch,STAT_FIRE,power);
			check_stat_improve(ch,STAT_WATER,power);
			return;
		}
	}

}

int check_angreal_bonus (CHAR_DATA *ch)
{
	OBJ_DATA *ang;
	int ang_bonus = 0;
	char buf[MSL];

	ang = get_eq_char( ch, WEAR_ANGREAL );
	if ( ang == NULL )
		return 0;

	if ( ang->value[1] > 20 )
		ang->value[1] = 20;

	if (ch->sex == SEX_MALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_MALE)
	    {
			sprintf(buf,"You draw power through %s\n\r",ang->short_descr);
			stc(buf,ch);
			ang_bonus = ang->value[1] * 10;
		}
	}
	if (ch->sex == SEX_FEMALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_FEMALE)
	    {
			sprintf(buf,"You draw power through %s\n\r",ang->short_descr);
			stc(buf,ch);
			ang_bonus = ang->value[1] * 10;
		}
	}
    ang_bonus += ch->pcdata->op_spirit;
    ang_bonus += ch->pcdata->op_fire;
    ang_bonus += ch->pcdata->op_water;
    ang_bonus += ch->pcdata->op_air;
    ang_bonus += ch->pcdata->op_earth;
    if ( check_angrealstudy( ch ) )
    {
	    ang_bonus += get_skill( ch, gsn_angrealstudy );
	}

	/* What? */
	/*
	if ( ang->value[1] >= 999 && true_op_average( ch ) < 100 )
	{
		stc( "The One Power sears your body and mind!\n\r", ch );
		act("$n glows with a blinding light before exploding in a violent fury of flesh and bone!", ch, NULL, NULL, TO_ROOM );
		raw_kill( ch, ch );
		return 0;
	}
	*/

    return ang_bonus;
}

int true_angreal_bonus (CHAR_DATA *ch)
{
	OBJ_DATA *ang;
	int ang_bonus = 0;
	bool found = FALSE;

	ang = get_eq_char( ch, WEAR_ANGREAL );
	if ( ang == NULL )
		return 0;
	if (ch->sex == SEX_MALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_MALE)
	    {
			ang_bonus = ang->value[1];
			found = TRUE;
		}
	}
	if (ch->sex == SEX_FEMALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_FEMALE)
	    {
			ang_bonus = ang->value[1];
			found = TRUE;
		}
	}
    return ang_bonus;
}

int check_angreal (CHAR_DATA *ch)
{
	OBJ_DATA *ang;
	int ang_bonus = 0;
	bool found = FALSE;

	ang = get_eq_char( ch, WEAR_ANGREAL );
	if ( ang == NULL )
		return 0;

	if (ch->sex == SEX_MALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_MALE)
	    {
		    check_improve( ch, gsn_angrealstudy, TRUE, 6 );
			ang_bonus = ang->value[1] * 10;
			found = TRUE;
		}
	}
	if (ch->sex == SEX_FEMALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_FEMALE)
	    {
		    check_improve( ch, gsn_angrealstudy, TRUE, 6 );
			ang_bonus = ang->value[1] * 10;
			found = TRUE;
		}
	}
    ang_bonus += ch->pcdata->op_spirit;
    ang_bonus += ch->pcdata->op_fire;
    ang_bonus += ch->pcdata->op_water;
    ang_bonus += ch->pcdata->op_air;
    ang_bonus += ch->pcdata->op_earth;
    if ( check_angrealstudy( ch ) )
    {
	    ang_bonus += get_skill( ch, gsn_angrealstudy );
	}
    return ang_bonus;
}

bool true_check_angreal (CHAR_DATA *ch)
{
	OBJ_DATA *ang;
	bool found = FALSE;

	ang = get_eq_char( ch, WEAR_ANGREAL );
	if ( ang == NULL )
		return FALSE;
	if (ch->sex == SEX_MALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_MALE)
	    {
			found = TRUE;
			return TRUE;
		}
	}
	if (ch->sex == SEX_FEMALE)
	{
	    if (ang->item_type == ITEM_ANGREAL_FEMALE)
	    {
			found = TRUE;
			return TRUE;
		}
	}
    return FALSE;
}
int check_burnout (CHAR_DATA *ch, int pwrLvl)
{
	/* pwr # is input as pwrLvl */
	//int *canPwr = 0;
	int diff = 0;
	int chance = 0;
	int canPwr = 0;
	canPwr = ch->pcdata->op_air+ch->pcdata->op_earth+ch->pcdata->op_fire+ch->pcdata->op_spirit+ch->pcdata->op_water;
	canPwr += int_app[get_curr_stat(ch,STAT_INT)].op_focus;
	canPwr /= 5;
	diff = pwrLvl - canPwr;

	if (diff < 1)
		stc("You bend the One Power to its task.\n\r",ch);
	else if (diff < 3) {
		chance = 1;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("You twist and shape the One Power.\n\r",ch);
	}
	else if (diff < 5) {
		chance = 2;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("You mold the One Power, shaping the weaves.\n\r",ch);
	}
	else if (diff < 7) {
		chance = 3;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("You soak in the One Power, letting it flow into you.\n\r",ch);
	}
	else if (diff < 9) {
		chance = 4;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("You drink in the One Power, letting it fill you entirely.\n\r",ch);
	}
	else if (diff < 10) {
		chance = 5;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("You wrestle with the True Source, forging it into your weave.\n\r",ch);
	}
	else if (diff < 11) {
		chance = 6;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("The One Power blazes like a torch in your grasp.\n\r",ch);
	}
	else if (diff < 12) {
		chance = 7;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("The One Power blazes like a bonfire, threatening to consume you.\n\r",ch);
	}
	else if (diff < 13) {
		chance = 8;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("You bring raging torrents of the One Power to bear.\n\r",ch);
	}
	else if (diff < 14) {
		chance = 9;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("Waves of the One Power buffet you like a boat in a maelstrom.\n\r",ch);
	}
	else if (diff < 15) {
		chance = 10;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("The One Power writhes like a live snake in your grasp, threatening to strike.\n\r",ch);
	}
	else if (diff < 16) {
		chance = 11;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		stc("The weight of the True Source nearly buckles your knees, driving you down in relentless waves of power.\n\r",ch);
	}
	else if (diff >= 16) {
		chance = 15;
		if ( chance >= number_percent() )
			ch->pcdata->burnout += 1;
		if (ch->sex == SEX_MALE)
			ch->pcdata->insanity += 1;
		ch->hit = 1;
		ch->stam = 1;
		DAZE_STATE(ch, 3 * PULSE_VIOLENCE);
		ch->position = POS_STUNNED;
		update_pos(ch);
		stc("{RToo much to control, the One Power escapes your hold!{x\n\r",ch);
		stc("{RA deepening blackness follows the light.{x\n\r",ch);
		REMOVE_BIT(ch->op_flags,OP_GRASPED);
		while (ch->weaves && ch->weaves->action[0] == ACTION_CASTING)
		    stop_spell( ch );
	}
	if (ch->pcdata->burnout >= 100) {
		ch->hit = 1;
		ch->stam = 1;
		DAZE_STATE(ch, 3 * PULSE_VIOLENCE);
		ch->position = POS_STUNNED;
		update_pos(ch);
		stc("{RThe One Power struggles free from your grasp, bearing down on you in a maelstrom of power.{x\n\r",ch);
		stc("{RMore than you can bear, you lose control over the source, and it sears your mind with fury.{x\n\r",ch);
		REMOVE_BIT(ch->op_flags,OP_GRASPED);
		REMOVE_BIT(ch->op_flags,OP_CHANNELER);
	}
	diff += chance;
	return diff;
}

/*
 * Compute a saving throw.
 * Negative apply's make saving throw better.
 */
bool saves_spell( int level, CHAR_DATA *victim, int dam_type )
{
    int save;

    save = 50 + ( victim->level - level) * 5 - victim->saving_throw * 2;
    if (IS_AFFECTED(victim,AFF_BERSERK))
	save += victim->level/2;

    switch(check_immune(victim,dam_type))
    {
	case IS_IMMUNE:		return TRUE;
	case IS_RESISTANT:	save += 2;	break;
	case IS_VULNERABLE:	save -= 2;	break;
    }

    if (!IS_NPC(victim) && class_table[victim->class].fMana)
	save = 9 * save / 10;
    save = URANGE( 5, save, 95 );
    return number_percent( ) < save;
}

/* RT save for dispels */
bool saves_dispel( int dis_level, int spell_level, int duration)
{
    int save;

    if (duration == -1)
      spell_level += 5;
      /* very hard to dispel permanent effects */

    save = 50 + (spell_level - dis_level) * 5;
    save = URANGE( 5, save, 95 );
    return number_percent( ) < save;
}

/* co-routine for dispel magic and cancellation */
bool check_dispel( int dis_level, CHAR_DATA *victim, int sn)
{
    AFFECT_DATA *af;

    if (is_affected(victim, sn))
    {
        for ( af = victim->affected; af != NULL; af = af->next )
        {
            if ( af->type == sn )
            {
                if (!saves_dispel(dis_level,af->level,af->duration))
                {
                    affect_strip(victim,sn);
        	    if ( skill_table[sn].msg_off )
        	    {
            		stc( skill_table[sn].msg_off, victim );
            		stc( "{c\n\r", victim );
        	    }
		    return TRUE;
		}
		else
		    af->level--;
            }
        }
    }
    return FALSE;
}

/* for finding mana costs -- temporary version */
int mana_cost (CHAR_DATA *ch, int min_mana, int level)
{
    if (ch->level + 2 == level)
	return 1000;
    return UMAX(min_mana,(100/(2 + ch->level - level)));
}

void obj_cast_spell( int sn, int level, CHAR_DATA *ch, CHAR_DATA *victim, OBJ_DATA *obj )
{
    void *vo;
    int target = TARGET_NONE;

    if ( sn <= 0 )
	return;

    if ( sn >= MAX_SKILL || skill_table[sn].spell_fun == 0 )
    {
	bug( "Obj_cast_spell: bad sn %d.", sn );
	return;
    }

    switch ( skill_table[sn].target )
    {
    default:
	bug( "Obj_cast_spell: bad target for sn %d.", sn );
	return;

    case TAR_IGNORE:
	vo = NULL;
	break;

    case TAR_CHAR_OFFENSIVE:
	if ( victim == NULL && !is_fighting(ch))
	{
	    stc( "{cYou can't do that.\n\r", ch );
	    return;
	}
	else
	          victim = ch->advancing_on;

/*	if ( victim == NULL )
	{
	    stc( "{cYou can't do that.\n\r", ch );
	    return;
	}*/

	if (is_safe(ch,victim) && ch != victim)
	{
	    stc( "{cSomething isn't right...\n\r",ch);
	    return;
	}
	vo = (void *) victim;
	target = TARGET_CHAR;
	if ( IS_NPC( victim ) && !is_fighting(victim) )
	{
		start_extra_fight(ch, victim, RANGE_MISSLE);
	}
	break;

    case TAR_CHAR_DEFENSIVE:
    case TAR_CHAR_SELF:
	if ( victim == NULL )
	    victim = ch;
	vo = (void *) victim;
	target = TARGET_CHAR;
	break;

    case TAR_OBJ_INV:
	if ( obj == NULL )
	{
	    stc( "{cYou can't do that.\n\r", ch );
	    return;
	}
	vo = (void *) obj;
	target = TARGET_OBJ;
	break;

    case TAR_OBJ_CHAR_OFF:
        if ( victim == NULL && obj == NULL)
        {
		    if (is_fighting(ch))
				victim = ch->advancing_on;
		    else
		    {
			stc( "{cYou can't do that.\n\r",ch);
			return;
		    }
		}

	    if (victim != NULL)
	    {
		if (is_safe_spell(ch,victim,FALSE) && ch != victim)
		{
                    stc( "{cSomething isn't right...\n\r",ch);
		    return;
		}

		vo = (void *) victim;
		target = TARGET_CHAR;
	    }
	    else
	    {
	    	vo = (void *) obj;
	    	target = TARGET_OBJ;
	    }
        break;


    case TAR_OBJ_CHAR_DEF:
	if (victim == NULL && obj == NULL)
	{
	    vo = (void *) ch;
	    target = TARGET_CHAR;
	}
	else if (victim != NULL)
	{
	    vo = (void *) victim;
	    target = TARGET_CHAR;
	}
	else
	{
	    vo = (void *) obj;
	    target = TARGET_OBJ;
	}

	break;
    }

    target_name = str_dup("");
    (*skill_table[sn].spell_fun) ( sn, level, ch, vo,target);

    if ( (skill_table[sn].target == TAR_CHAR_OFFENSIVE
    ||   (skill_table[sn].target == TAR_OBJ_CHAR_OFF && target == TARGET_CHAR))
    &&   victim != ch
    &&   victim->master != ch )
    {
	CHAR_DATA *vch;
	CHAR_DATA *vch_next;

	for ( vch = ch->in_room->people; vch; vch = vch_next )
	{
	    vch_next = vch->next_in_room;
	    if ( victim == vch && !is_fighting(victim) )
	    {
		check_killer(victim,ch);
		multi_hit( victim, ch, TYPE_UNDEFINED );
		break;
	    }
	}
    }

    return;
}

/*****************************************************************
 * Begin the weaves. These are the actual functions, divided into
 * Healing, Offensive, Defensive, Misc weave.
 *****************************************************************/

void spell_null( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    stc( "{cThat's not a weave!\n\r", ch );
    return;
}

/*
 * One Power functions.
 *
 * Healing Weaves
 * --------------
 */

void weave_mass_heal( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *vict2;
    int healnum;
    /* Spirit/Fire/Earth/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water;
    op_bonus /= 4;
	}
	op_bonus += ang_bonus;
    healnum = ((ch->level*2) + (int_app[get_curr_stat(ch,STAT_INT)].op_focus *11/9));
    healnum += op_bonus;
    healnum += pwrBonus;
    if (!IS_NPC(ch))
    {
	    if (IS_SET(ch->ajah_flags, AJAH_YELLOW) || ch->clan == clan_lookup("kin"))
    	healnum += ch->level;
	}
	for (vict2 = ch->in_room->people; vict2 != NULL; vict2 = vict2->next_in_room)
	{
		if (vict2 == ch)
			continue;
		if (IS_NPC(vict2))
			continue;
		if (!is_same_group(ch,vict2))
			continue;
	    vict2->hit = UMIN( vict2->hit + healnum, get_maxhit(vict2));
	    update_pos( vict2 );
		act( "{cYou step from person to person, Healing the wounds of each.",ch,0,vict2,TO_CHAR );
	    act("{c$n takes your head between $s hands. Fingers of ice and fire shoot up your spine.",ch,NULL,vict2,TO_VICT);
	    stc( "{cA warm feeling fills your body.\n\r", vict2 );
	    act("{c$n takes $N's head between $s hands. $N shivers violently.",ch,NULL,vict2,TO_ROOM);
		check_healing(ch,vict2);
	}
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_cure_blinding(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( !is_affected( victim, gsn_blindness ) )
    {
        if (victim == ch)
          stc( "{cYou aren't blind.\n\r",ch);
        else
          act("{c$N doesn't appear to be blinded.",ch,NULL,victim,TO_CHAR);
        return;
    }

    if (check_dispel(level+op_bonus,victim,gsn_blindness))
    {
        stc( "{cYour vision returns!\n\r", victim );
        act("{c$n is no longer blinded.",victim,NULL,NULL,TO_ROOM);
    }
    else
        stc( "{cWeave failed.\n\r",ch);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
}

void weave_cure_critical( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int heal;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    heal = dice(1, ch->level) + level - 6 + op_bonus;
    victim->hit = UMIN( victim->hit + heal, get_maxhit(victim) );
    update_pos( victim );
    stc( "{cYou feel better!\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
	check_healing(ch,victim);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

/* RT added to cure plague */
void weave_cure_disease( int sn, int level, CHAR_DATA *ch,void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( !is_affected( victim, gsn_plague ) )
    {
        if (victim == ch)
          stc( "{cYou aren't ill.\n\r",ch);
        else
          act("{c$N doesn't appear to be diseased.",ch,NULL,victim,TO_CHAR);
        return;
    }

    if (check_dispel(level+op_bonus,victim,gsn_plague))
    {
	stc( "{cYour sores vanish.\n\r",victim);
	act("{c$n looks relieved as $s sores vanish.",victim,NULL,NULL,TO_ROOM);
    }
    else
		stc( "{cWeave failed.\n\r",ch);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
}

void weave_cure_light( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int heal;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    heal = dice(1, ch->level) + level / 3 + op_bonus;
    victim->hit = UMIN( victim->hit + heal, get_maxhit(victim) );
    update_pos( victim );
    stc( "{cYou feel better!\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
	check_healing(ch,victim);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_cure_poison( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
	CHAR_DATA *victim = (CHAR_DATA *) vo;
	/* Spirit */
	int op_bonus = 0;
	int ang_bonus;

	if (!IS_NPC(ch))
		ang_bonus = check_angreal(ch);
	else ang_bonus = 0;

	if (!IS_NPC(ch))
	{
		op_bonus = ch->pcdata->op_spirit;
	}

	op_bonus += ang_bonus;
	op_bonus += pwrBonus;

	if ( !is_affected( victim, gsn_poison ) )
	{
		if (victim == ch)
			stc( "{cYou aren't poisoned.\n\r",ch);
		else
			act("{c$N doesn't appear to be poisoned.",ch,NULL,victim,TO_CHAR);
		return;
	}

	if (check_dispel(level+op_bonus,victim,gsn_poison))
	{
		stc( "{cA warm feeling runs through your body.\n\r",victim);
		act("{c$n looks much better.",victim,NULL,NULL,TO_ROOM);
	}
	else
		stc( "{cThe weave appears to have failed.\n\r",ch);

	check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
}

void weave_cure_serious( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int heal;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    heal = dice(1, ch->level) + level / 2 + op_bonus ;
    victim->hit = UMIN( victim->hit + heal, get_maxhit(victim) );
    update_pos( victim );
    stc( "{cYou feel better!\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
	check_healing(ch,victim);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_heal( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int healnum;
    /* Spirit/Fire/Earth/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water;
    op_bonus /= 4;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    healnum = ch->level + int_app[get_curr_stat(ch,STAT_INT)].op_focus *11/9;
    if (IS_SET(ch->ajah_flags,AJAH_YELLOW) || ch->clan == clan_lookup("kin"))
    	healnum += ch->level/2;
    healnum += op_bonus;
    victim->hit = UMIN( victim->hit + healnum, get_maxhit(victim));
    update_pos( victim );
    if ( victim != ch )
    {
		act( "{cYou weave flows of the One Power, Healing $N's wounds.",ch,0,victim,TO_CHAR );
	    act("{c$n takes your head between $s hands. Fingers of ice and fire shoot up your spine.",ch,NULL,victim,TO_VICT);
	    stc( "{cA warm feeling fills your body.\n\r", victim );
	    act("{c$n takes $N's head between $s hands. $N shivers violently.",ch,NULL,victim,TO_ROOM);
	}
	else {
	    stc( "{cAs you draw upon the True Source, a warm feeling fills your body.\n\r", victim );
	}
	check_healing(ch,victim);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_remove_curse( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    bool found = FALSE;
    /* Spirit/Fire/Earth/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water;
    op_bonus /= 4;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    stc("You weave flows of spirit, earth, air, fire, and water.\n\r",ch);
    /* do object cases first */
    if (target == TARGET_OBJ)
    {
	obj = (OBJ_DATA *) vo;

	if (IS_OBJ_STAT(obj,ITEM_NODROP) || IS_OBJ_STAT(obj,ITEM_NOREMOVE))
	{
	    if (!IS_OBJ_STAT(obj,ITEM_NOUNCURSE)
	    &&  !saves_dispel(level + 2 + op_bonus,obj->level,0))
	    {
		REMOVE_BIT(obj->extra_flags,ITEM_NODROP);
		REMOVE_BIT(obj->extra_flags,ITEM_NOREMOVE);
		act("{c$p glows blue.",ch,obj,NULL,TO_ALL);
		return;
	    }

	    act("{cThe curse on $p is beyond your power.",ch,obj,NULL,TO_CHAR);
	    return;
	}
	act("{cThere doesn't seem to be a curse on $p.",ch,obj,NULL,TO_CHAR);
	return;
    }

    /* characters */
    victim = (CHAR_DATA *) vo;

    if (check_dispel(level+op_bonus,victim,gsn_curse))
    {
	stc( "{cYou feel better.\n\r",victim);
	act("{c$n looks more relaxed.",victim,NULL,NULL,TO_ROOM);
    }

   for (obj = victim->carrying; (obj != NULL && !found); obj = obj->next_content)
   {
        if ((IS_OBJ_STAT(obj,ITEM_NODROP) || IS_OBJ_STAT(obj,ITEM_NOREMOVE))
		&&  !IS_OBJ_STAT(obj,ITEM_NOUNCURSE))
        {   /* attempt to remove curse */
            if (!saves_dispel(level+op_bonus,obj->level,0))
            {
                found = TRUE;
                REMOVE_BIT(obj->extra_flags,ITEM_NODROP);
                REMOVE_BIT(obj->extra_flags,ITEM_NOREMOVE);
                act("{cYour $p glows blue.",victim,obj,NULL,TO_CHAR);
                act("{c$n's $p glows blue.",victim,obj,NULL,TO_ROOM);
            }
         }
    }
}

void weave_repair_flesh( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int healnum;
    /* Spirit/Fire/Earth/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water;
    op_bonus /= 4;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if (victim == ch)
    {
		stc("You cannot weave this flow upon yourself.\n\r",ch);
		return;
	}
    healnum = ch->level*2 + int_app[get_curr_stat(ch,STAT_INT)].op_focus *11/9;
    healnum += op_bonus;
    if (!IS_NPC(ch))
    {
    if (IS_SET(ch->ajah_flags, AJAH_YELLOW) || ch->clan == clan_lookup("kin"))
    	healnum += ch->level / 2;
	}
    victim->hit = UMIN( victim->hit + healnum, get_maxhit(victim));
    update_pos( victim );
    if ( victim != ch )
    {
		act( "{cYou weave flows of the One Power, Healing $N's wounds.",ch,0,victim,TO_CHAR );
	    act("{c$n takes your head between $s hands. Fingers of ice and fire shoot up your spine.",ch,NULL,victim,TO_VICT);
	    stc( "{cA warm feeling fills your body.\n\r", victim );
	    act("{c$n takes $N's head between $S hands. $N shivers violently.",ch,NULL,victim,TO_ROOM);
	}
	else
	    stc( "{cAs you draw upon the True Source, a warm feeling fills your body.\n\r", victim );
	check_healing(ch,victim);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

/*
 * One Power functions.
 *
 * Offensive Weaves
 * --------------
 */

void weave_blinding( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Air/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit + ch->pcdata->op_air + ch->pcdata->op_water;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;

    if ( IS_AFFECTED(victim, AFF_BLIND) || saves_spell(level,victim,DAM_OTHER))
	return;

    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.location  = APPLY_HITROLL;
    af.modifier  = -4;
    af.duration  = 1+level + op_bonus;
    af.bitvector = AFF_BLIND;
    affect_to_char( victim, &af );
    stc( "{cYou are blinded!\n\r", victim );
    act("{c$n appears to be blinded.",victim,NULL,NULL,TO_ROOM);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_burn(int sn,int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;

    static const sh_int dam_each[] =
    {
         0,
         0,  0,  0,  0, 14,     17, 20, 23, 26, 29,
        29, 29, 30, 30, 31,     31, 32, 32, 33, 33,
        34, 34, 35, 35, 36,     36, 37, 37, 38, 38,
        39, 39, 40, 40, 41,     41, 42, 42, 43, 43,
        44, 44, 45, 45, 46,     46, 47, 47, 48, 48
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
        ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_fire + ch->pcdata->op_air;
    op_bonus /= 2;
        }
        op_bonus += ang_bonus;
        op_bonus += pwrBonus;

    level       = UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level       = UMAX(0, level);
    dam         = number_range( dam_each[level] / 2, dam_each[level] * 2 );
    if ( saves_spell( level, victim,DAM_FIRE) )
        dam /= 2;
        dam += op_bonus;
    damage( ch, victim, dam, sn, DAM_FIRE,TRUE);
    act("{cFire leaps from $n's hands!\n\r",ch,NULL,victim,TO_ROOM );
    act( "{cFire leaps from $n's hands to strike at you!\n\r", ch,NULL,victim,TO_VICT );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
/*
void weave_burn(int sn,int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;

    int dam;
    int ang_bonus;

    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    ang_bonus += truePwr;

    dam = calc_weave_damage2( ch, sn, ang_bonus );

    if ( saves_spell( level, victim,DAM_FIRE) )
		dam /= 2;

    damage( ch, victim, dam, sn, DAM_FIRE,TRUE);

    act("{cFire leaps from $n's hands!\n\r",ch,NULL,victim,TO_ROOM );
    act( "{cFire leaps from $n's hands to strike at you!\n\r", ch,NULL,victim,TO_VICT );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
*/
void weave_sever( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    bool found = FALSE;

    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
    op_bonus /= 2;
	}
	op_bonus -= ang_bonus;
	op_bonus += pwrBonus;
    level -= 2;
    level -= op_bonus;

    if ((!IS_NPC(ch) && IS_NPC(victim) &&
	!(IS_AFFECTED(ch, AFF_CHARM) && ch->master == victim) ) ||
    (IS_NPC(ch) && !IS_NPC(victim) ) )
    {
    stc( "{cYou failed, try shielding.\n\r",ch);
	return;
    }

    /* unlike dispel magic, the victim gets NO save */

    /* begin running through the spells */

    if (check_dispel(level,victim,skill_lookup("defense")))
    {
        act("{cA vague outline around $n shimmers before vanishing.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
	}
    if (check_dispel(level,victim,skill_lookup("vigor")))
    {
        act("{c$n no longer looks so vigorous.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
	}
    if (check_dispel(level,victim,skill_lookup("blinding")))
    {
        found = TRUE;
        act("{c$n is no longer blinded.",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("sicken")))
    {
        found = TRUE;
        act("{c$n is no longer sick.",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("calm")))
    {
		found = TRUE;
		act("{c$n no longer looks so peaceful...",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("glow")))
    {
        act("{c$n no longer glows.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
	}
    if (check_dispel(level,victim,skill_lookup("see shadow")))
    {
        act("{cA vague outline around $n shimmers before vanishing.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
	}
    if (check_dispel(level,victim,skill_lookup("see light")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("see hidden")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("see invis")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("seeing")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("invert")))
    {
        act("{c$n fades into existance.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
    }
    if (check_dispel(level,victim,skill_lookup("hold")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("weaken")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("large")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("shield")))
    {
        act("{cThe shield protecting $n vanishes.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
    }

    if (found)
        stc( "{cOk.\n\r",ch);
    else
        stc( "{cWeave failed.\n\r",ch);
}

void weave_lash( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Air/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air+ch->pcdata->op_water;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    stc( "{cYou weave the flows of lash!\n\r", ch );
    stc( "{cInvisible lashes strike your body!\n\r", victim );
    act( "{c$N swats at an unseen foe.\n\r", ch,NULL,victim,TO_NOTVICT );
    damage( ch, (CHAR_DATA *) vo, dice(1, 8) + level / 3 + op_bonus, sn,DAM_HARM,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_flay(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Air/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air+ch->pcdata->op_water;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    stc( "{cYou weave the flows of flay!\n\r", ch );
    stc( "{cAn unseen force flays your body!\n\r", victim );
    act("{c$n writhes in agony!\n\r",victim,NULL,NULL,TO_NOTVICT );
    damage( ch, (CHAR_DATA *) vo, dice(3, 8) + level + op_bonus, sn,DAM_HARM,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_flog(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Air/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air+ch->pcdata->op_water;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    stc( "{cYou weave the flows of flog!\n\r", ch );
    stc( "{cAn unseen force flogs your body!\n\r", victim );
    act("{c$n writhes in pain!\n\r",victim,NULL,NULL,TO_NOTVICT );
    damage( ch, (CHAR_DATA *) vo, dice(2, 8) + level / 2 + op_bonus, sn,DAM_HARM,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
void weave_compulsion( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    return;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if (is_safe(ch,victim)) return;

    if ( victim == ch )
    {
	stc( "{cYou like yourself even better!\n\r", ch );
	return;
    }

    if ( IS_AFFECTED(victim, AFF_CHARM)
    ||   IS_AFFECTED(ch, AFF_CHARM)
    ||   level + op_bonus < victim->level
    ||   IS_SET(victim->imm_flags,IMM_CHARM)
    ||   saves_spell( level + op_bonus, victim,DAM_CHARM) )
	return;

    if (IS_SET(victim->in_room->room_flags,ROOM_LAW))
    {
	ptc(ch,"The %s does not allow compulsion here.\n\r", is_shadow(ch) ? "Great Lord" : "Creator" );
	return;
    }

    if (IS_NPC(victim))
    	return;

    if ( victim->master )
	stop_follower( victim );
    add_follower( victim, ch );
    victim->leader = ch;
    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level	 = level;
    af.duration  = number_fuzzy( level / 4 + op_bonus);
    af.location  = 0;
    af.modifier  = 0;
    af.bitvector = AFF_CHARM;
    affect_to_char( victim, &af );
    act("{cIsn't $n just so nice?", ch, NULL, victim, TO_VICT );
    if ( ch != victim )
	act("{c$N looks at you with adoring eyes.",ch,NULL,victim,TO_CHAR);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_shielding( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    bool found = FALSE;
    /* Spirit/Fire/Earth/Water */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water;
    op_bonus /= 4;
	}
	op_bonus -= ang_bonus;
	op_bonus -= pwrBonus;
	level -= op_bonus;
    if (saves_spell(level, victim, DAM_OTHER))
    {
	stc( "{cYou feel a brief tingling sensation.\n\r",victim);
	stc( "{cYou failed.\n\r", ch);
	return;
    }
	/* make a check for channeling */
	if (!IS_SET( ch->op_flags, OP_CHANNELER))
	{
		stc("Your victim is not a channeler to weave this flow on.\n\r",ch);
		return;
	}
	if (!IS_SET( ch->op_flags, OP_CHANNELER) && ch->sex == victim->sex)
	{
		act( "{w$n's shield attempts to close you off from the True Source.",ch,0,victim,TO_VICT);
		act( "{wYou weave a shield of the One Power over $N.", ch,0,victim,TO_CHAR);
	}

    /* begin running through the spells */

    if (check_dispel(level,victim,skill_lookup("defense")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("vigor")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("blinding")))
    {
        found = TRUE;
        act("{c$n is no longer blinded.",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("sicken")))
    {
        found = TRUE;
        act("{c$n is no longer sick.",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("calm")))
    {
	found = TRUE;
	act("{c$n no longer looks so peaceful...",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("compulsion")))
    {
        found = TRUE;
        act("{c$n regains $s free will.",victim,NULL,NULL,TO_ROOM);
    }
    if (check_dispel(level,victim,skill_lookup("glow")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("see shadow")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("see light")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("see hidden")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("see invis")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("seeing")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("invert")))
    {
        act("{c$n fades into existance.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
    }
    if (check_dispel(level,victim,skill_lookup("hold")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("weaken")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("large")))
        found = TRUE;
    if (check_dispel(level,victim,skill_lookup("shield")))
    {
        act("{cThe shield protecting $n vanishes.",victim,NULL,NULL,TO_ROOM);
        found = TRUE;
    }

    if (found)
        stc( "{cOk.\n\r",ch);
    else
        stc( "{cWeave failed.\n\r",ch);
	return;
}

void weave_earthquake( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *vict;
    /* Earth */
    int i = 0;

    int ang_bonus;

    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    ang_bonus += truePwr;

    stc("You weave flows of earth and fire.\n\r",ch);
    act("{cThe earth trembles and shivers, throwing debris everywhere.", ch, NULL, NULL, TO_ROOM );

    for (vict = ch->in_room->people; vict != NULL && i<3; vict = vict->next_in_room)
	{

		if (vict == ch)
			continue;
		if (is_same_group (ch, vict))
			continue;
		if ( is_safe( ch, vict ) )
			continue;
		if ( is_fighting(vict) &&	!is_same_group(ch,vict->advancing_on))
			continue;
		if ( IS_AFFECTED(ch, AFF_CHARM) && ch->master == vict )
			continue;
		if ( !is_safe_spell(ch,vict,TRUE))
		{
			if (check_fighting(ch, vict) || start_extra_fight(ch, vict, RANGE_MELEE))
			{
				i++;
				if((i == 1) || ( i== 2 && (number_percent() > 50)) || (i == 3 && (number_percent() > 66) ) )
				{
					damage( ch,vict,level + dice(2, 8), sn, DAM_BASH,TRUE);
					/*
					dam = calc_weave_damage( ch, sn, ang_bonus );
					if ( IS_AFFECTED( vict,AFF_FLYING ) )
						damage( ch, vict, dam, sn, DAM_BASH, TRUE );
					else
						damage( ch, vict, dam, sn, DAM_BASH, TRUE );
					*/
				}
			}
		}

    }


    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_firestorm( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Fire/Air */
    static const sh_int dam_each[] =
    {
	  0,
	  0,   0,   0,   0,   0,	  0,   0,   0,   0,   0,
	  0,   0,   0,   0,  30,	 35,  40,  45,  50,  55,
	 60,  65,  70,  75,  80,	 82,  84,  86,  88,  90,
	 92,  94,  96,  98, 100,	102, 104, 106, 108, 110,
	112, 114, 116, 118, 120,	122, 124, 126, 128, 130
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_air+ch->pcdata->op_fire;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam     += op_bonus;

    stc("You weave flows of air and fire.\n\r",ch);

    if ( saves_spell( level, victim, DAM_FIRE) )
		dam /= 2;

    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
	    stc( "{cA storm of {w-{R+{w- {rFIRE{x {w-{R+{w-{c crashes down to strike at you!\n\r",victim );
	    act("{cA storm of {w-{R+{w- {rFIRE{x {w-{R+{w-{c crashes down on $N!\n\r",ch,NULL,victim,TO_NOTVICT );
	}
	else
	{
		dam /= 2;
	    stc( "{cYou duck under your shield as a storm of {w-{R+{w- {rFIRE{x {w-{R+{w-{c crashes down to strike at you!\n\r",victim );
	    act("{cA storm of {w-{R+{w- {rFIRE{x {w-{R+{w-{c crashes down on $N as $E ducks under a shield!\n\r",ch,NULL,victim,TO_NOTVICT );
	}

    act( "{cYour {w-{R+{w- {rFIRE{x {w-{R+{w-{c crashes down to strike at $N!\n\r",ch,NULL,victim,TO_CHAR );

    damage( ch, victim, dam, sn, DAM_FIRE ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_lightning(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Fire/Air */
    static const sh_int dam_each[] =
    {
	 0,
	 0,  0,  0,  0,  0,	 0,  0,  0, 25, 28,
	31, 34, 37, 40, 40,	41, 42, 42, 43, 44,
	44, 45, 46, 46, 47,	48, 48, 49, 50, 50,
	51, 52, 52, 53, 54,	54, 55, 56, 56, 57,
	58, 58, 59, 60, 60,	61, 62, 62, 63, 64
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_air;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam += op_bonus;

    if ( saves_spell( level, victim,DAM_LIGHTNING) )
	dam /= 2;

    stc("You weave flows of earth, air, fire, and water.\n\r",ch);
    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
	    stc( "{cLightning crashes down from the sky to strike at you!\n\r", victim );
	    act("{cLightning forks down from the sky to strike at $N!\n\r",ch,NULL,victim,TO_NOTVICT);
	}
	else
	{
		dam /= 2;
	    stc( "{cLightning crashes down from the sky to strike at you as you duck under your shield!\n\r", victim );
	    act("{cLightning forks down from the sky to strike at $N as $E ducks under a shield!\n\r",ch,NULL,victim,TO_NOTVICT);
	}


    damage( ch, victim, dam, sn, DAM_LIGHTNING ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_flameblade( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Fire/Air */
    static const sh_int dam_each[] =
    {
	 0,
	 0,  0,  0,  0,  0,	 0,  0,  0, 25, 28,
	31, 34, 37, 40, 40,	41, 42, 42, 43, 44,
	44, 45, 46, 46, 47,	48, 48, 49, 50, 50,
	51, 52, 52, 53, 54,	54, 55, 56, 56, 57,
	58, 58, 59, 60, 60,	61, 62, 62, 63, 64
    };
    int dam;
    int ang_bonus;
    int op_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_air;
    op_bonus /= 3;
	}
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
	op_bonus += pwrBonus;
    dam +=  op_bonus;
    dam +=  ang_bonus;

    if ( saves_spell( level, victim,DAM_FIRE) )
	dam /= 2;

	stc("You weave flows of air and fire.\n\r",ch);
    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
	    act( "{c$n slashes you with a {R*{rBlazing Blade of Fire{R*{c !\n\r", ch,NULL,victim,TO_VICT );
	    act("{c$n slashes at $N with a {R*{rBlazing Blade of Fire{R*{c !\n\r",ch,NULL,victim,TO_NOTVICT);
	}
	else
	{
		dam /= 2;
	    act( "{c$n slashes you with a {R*{rBlazing Blade of Fire{R*{c as you duck behind your shield!\n\r", ch,NULL,victim,TO_VICT );
	    act("{c$n slashes at $N with a {R*{rBlazing Blade of Fire{R*{c as $E ducks behind a shield!\n\r",ch,NULL,victim,TO_NOTVICT);
	}

    damage( ch, victim, dam, sn, DAM_FIRE ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_airclub( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Air */
    static const sh_int dam_each[] =
    {
	 0,
	 0,  0,  0,  0,  0,	 0,  0,  0, 25, 28,
	31, 34, 37, 40, 40,	41, 42, 42, 43, 44,
	44, 45, 46, 46, 47,	48, 48, 49, 50, 50,
	51, 52, 52, 53, 54,	54, 55, 56, 56, 57,
	58, 58, 59, 60, 60,	61, 62, 62, 63, 64
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam += op_bonus;
    stc("You weave flows of air.\n\r",ch);
    stc( "{c\n\rYou are bashed by a *<> Club of Force <>* !\n\r", victim );
    act("{c\n\r$N is bashed by an invisible force !\n\r",ch,NULL,victim,TO_NOTVICT);
    if ( saves_spell( level, victim,DAM_BASH) )
	dam /= 2;
    damage( ch, victim, dam, sn, DAM_BASH ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_shatter( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Earth */
    static const sh_int dam_each[] =
    {
	 0,
	 0,  0,  0,  0,  0,	 0,  0,  0, 25, 28,
	31, 34, 37, 40, 40,	41, 42, 42, 43, 44,
	44, 45, 46, 46, 47,	48, 48, 49, 50, 50,
	51, 52, 52, 53, 54,	54, 55, 56, 56, 57,
	58, 58, 59, 60, 60,	61, 62, 62, 63, 64
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water;
    op_bonus /= 4;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam += op_bonus;

    if ( saves_spell( level, victim,DAM_BASH) )
	dam /= 2;

    stc("You weave flows of earth.\n\r",ch);
    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
	    stc( "{cThe ground {R-{r={wEXPLODES{r={R-{x before you in a shower of dirt and stone!\n\r", victim );
	    act("{cThe ground {R-{r={wEXPLODES{r={R-{x before $N in a shower of dirt and stone!\n\r",ch,NULL,victim,TO_NOTVICT);
	}
	else
	{
		dam /= 2;
	    stc( "{cThe ground {R-{r={wEXPLODES{r={R-{x before you in a shower of dirt and stone as you duck behind your shield!\n\r", victim );
	    act("{cThe ground {R-{r={wEXPLODES{r={R-{x before $N in a shower of dirt and stone as $E ducks behind a shield!\n\r",ch,NULL,victim,TO_NOTVICT);
	}

    damage( ch, victim, dam, sn, DAM_BASH ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_air_blast( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Air */
    static const sh_int dam_each[] =
    {
	 0,
	 0,  0,  0,  0,  0,	 0,  0,  0, 28, 31,
	34, 37, 40, 42, 44,	46, 48, 50, 52, 54,
	44, 56, 56, 56, 57,	58, 58, 59, 60, 60,
	61, 62, 62, 63, 64,	64, 65, 66, 66, 67,
	68, 68, 69, 70, 70,	71, 72, 72, 73, 74
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam +=  op_bonus;
    stc("You weave flows of air and fire.\n\r",ch);
    if ( weather_info.mmhg <=  980
	|| weather_info.mmhg <= 1000
	|| weather_info.mmhg <= 1020 )
    	dam += ch->level / 5;
    stc( "{c\n\rA blast of hor air scours your flesh!\n\r", victim );
    act("{c\n\r$N is blasted by super-heated air!\n\r",ch,NULL,victim,TO_NOTVICT);
    if ( saves_spell( level, victim,DAM_BASH) )
	dam /= 2;
    damage( ch, victim, dam, sn, DAM_BASH ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

/*void	set_fighting	args( ( CHAR_DATA *ch, CHAR_DATA *victim ) ); */

void weave_mirrormirror( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    CHAR_DATA *bubble;
    int i;
	if ( !IS_NPC(victim) )
	{
		stc("Not on players.\n\r",ch);
		return;
	}

	bubble = create_mobile(get_mob_index(MOB_VNUM_BUBBLE));
	free_string( bubble->name );
	    bubble->name 	= str_dup(victim->name);
	free_string( bubble->short_descr );
	    bubble->short_descr	= str_dup(victim->name);
	free_string( bubble->long_descr );
	    bubble->long_descr	= str_dup(victim->name);
	free_string( bubble->description );
	    bubble->description	= str_dup("");
    bubble->group	= victim->group;
    bubble->sex		= victim->sex;
    bubble->race		= victim->race;
    bubble->level	= victim->level;
    bubble->trust	= 0;
    bubble->max_hit	= victim->max_hit;
    bubble->max_stam	= victim->max_stam;
    bubble->hit		= bubble->max_hit;
    bubble->stam	= bubble->max_stam;
    /*bubble->gold		= 0;
    bubble->silver	= 0;*/
    bubble->imm_flags	= victim->imm_flags;
    bubble->res_flags	= victim->res_flags;
    bubble->vuln_flags	= victim->vuln_flags;
    bubble->position	= POS_STANDING;
    bubble->saving_throw	= victim->saving_throw;
    bubble->alignment	= -1000;
    bubble->hitroll	= victim->hitroll;
    bubble->damroll	= victim->damroll;
    bubble->form		= victim->form;
    bubble->parts	= victim->parts;
    bubble->size		= victim->size;
    bubble->dam_type	= victim->dam_type;
    bubble->start_pos	= victim->start_pos;
	ch->stam = 1;
    for (i = 0; i < 4; i++)
    	bubble->armor[i]	= victim->armor[i];

    for (i = 0; i < MAX_STATS; i++)
    {
	bubble->perm_stat[i]	= victim->perm_stat[i];
	bubble->mod_stat[i]	= victim->mod_stat[i];
    }

    for (i = 0; i < 3; i++)
	bubble->damage[i]	= victim->damage[i];
	char_to_room(bubble,victim->in_room);
	act("The likeness of $n appears from out of a sudden mist.",victim,NULL,bubble,TO_NOTVICT);
	act("Your likeness appears from out of a sudden mist.",victim,NULL,bubble,TO_VICT);
    start_hating(bubble, victim);
    start_extra_fight(bubble, victim, RANGE_MELEE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
	return;
}

void weave_freeze_finger( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit/Water/Air */
    static const sh_int dam_each[] =
    {
	 0,
	 0,  0,  0,  0,  0,	 0,  0,  0, 29, 32,
	35, 38, 41, 43, 45,	47, 49, 51, 53, 55,
	55, 57, 57, 58, 58,	59, 59, 59, 62, 62,
	62, 63, 63, 64, 65,	65, 66, 66, 67, 68,
	69, 70, 71, 72, 73,	74, 75, 76, 77, 78
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air+ch->pcdata->op_water;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam += op_bonus;

    if ( saves_spell( level, victim,DAM_BASH) )
	dam /= 2;

    stc("You reach forth with a finger tipped with frost.\n\r",ch);
    stc( "{c\n\rFreezing cold numbs you to your bones !\n\r", victim );
    act("{c\n\r$N shivers and convulses as $s body is wracked by cold!\n\r",ch,NULL,victim,TO_NOTVICT);

    damage( ch, victim, dam, sn, DAM_BASH ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_weaken( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( is_affected( victim, sn ) || saves_spell( level, victim,DAM_OTHER) )
	return;

    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.duration  = (4 + level / 12 + op_bonus);
    af.location  = APPLY_STR;
    af.modifier  = -1 * (2 + level / 12);
    af.bitvector = AFF_WEAKEN;
    affect_to_char( victim, &af );
    send_to_char( "You feel your strength slip away.\n\r", victim );
    act("$n looks tired and weak.",victim,NULL,NULL,TO_NOTVICT);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_hold( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    act("You weave a shield of holding over $N!",ch,NULL,victim,TO_CHAR);
    act("$n glares balefully at $N!",ch,NULL,victim,TO_NOTVICT);
    act("$n glares balefully at you!",ch,NULL,victim,TO_VICT);

    if ( IS_SET(victim->war_flags, WAR_HELD) )
    {
	send_to_char( "But they are already held in a shield!\n\r", ch );
	return;
    }

    if ( is_safe(ch, victim) ) return;

	if (saves_spell(level+op_bonus, victim,DAM_ENERGY))
    {
	send_to_char( "You manage to break free!\n\r", victim );
	act("$n manages to escape!",victim,NULL,NULL,TO_ROOM);
	return;
    }
	SET_BIT(victim->war_flags,WAR_HELD);
    send_to_char( "You are held by some invisible forse!\n\r", victim );
    act("$n is immobilized!",victim,NULL,NULL,TO_ROOM);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_sicken( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    act("You weave the flows of sicken over $N!",ch,NULL,victim,TO_CHAR);
    act("$n crooks a finger at $N and grins malevolently!",ch,NULL,victim,TO_NOTVICT);
    act("$n crooks a finger at you and grins malevolently!",ch,NULL,victim,TO_VICT);

    if ( is_safe(ch, victim) ) return;

	if (saves_spell(level+op_bonus, victim,DAM_ENERGY))
    {
	stc( "You struggle against the poison and manage to overcome it!\n\r", victim );
	act("$n shivers and sweats, but appears otherwise un-phased!",victim,NULL,NULL,TO_ROOM);
	return;
    }
    af.type      = gsn_poison;
    af.duration  = 12 + (level/10);
    af.location  = APPLY_STR;
    af.modifier  = -3;
    af.bitvector = AFF_POISON;
    affect_join( victim, &af );
    stc( "You feel very sick.\n\r", victim );
    act( "$n looks very sick.", victim, NULL, NULL, TO_ROOM );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_spirit_fist( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit */
    static const sh_int dam_each[] =
    {
	 0,
	 2,  4,  8, 16, 32,	40, 48, 56, 64, 68,
	72, 76, 80, 84, 88,	92, 96, 100,104,108,
	110,112,114,116,118,120,122,124,126,128,
	130,132,133,134,136,138,140,142,144,146,
	148,150,152,155,158,161,164,167,170,175
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    dam +=  op_bonus;
    stc("You weave flows of pure spirit.\n\r",ch);
    stc( "\n\rAn invisible fist clenches around your heart, squeezing tightly!\n\r", victim );
    act("\n\r$N clutches $S chest, moaning in agony!\n\r",ch,NULL,victim,TO_ROOM );
    if ( saves_spell( level, victim, DAM_HARM) )
	dam /= 2;
    damage( ch, victim, dam, sn, DAM_HARM ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_spirit_hammer( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit */
    static const sh_int dam_each[] =
    {
	 0,
	 4,  8, 16, 32, 40,	48, 56, 64, 68, 72,
	76, 80, 84, 88, 92,	96, 100,104,108,112,
	116,120,124,128,132,136,140,144,148,152,
	156,160,164,168,172,174,176,178,180,182,
	184,186,188,190,192,194,196,198,199,200,
	202,204,206,208,210,212,214,216,218,220,
	222,223,224,225,226,227,228,229,230,231,
	232,233,234,235,236,237,238,239,240,250
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level] / 1.5, dam_each[level] * 2 );
    dam +=  op_bonus;
    stc("You weave flows of pure spirit.\n\r",ch);
    stc( "\n\rYou cry out in agony as something hammers against your very soul!\n\r", victim );
    act("\n\r$N cries out in agony, eyes rolling up into the back of $S head!\n\r",ch,NULL,victim,TO_ROOM );
    if ( saves_spell( level, victim, DAM_HARM) )
	dam /= 2;
    damage( ch, victim, dam, sn, DAM_HARM ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_spirit_shatter( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    /* Spirit */
    static const sh_int dam_each[] =
    {
	 0,
	 4,  8, 16, 32, 40,	48, 56, 64, 68, 72,
	76, 80, 84, 88, 92,	96, 100,104,108,112,
	116,120,124,128,132,136,140,144,148,152,
	156,160,164,168,172,174,176,178,180,182,
	184,186,188,190,192,194,196,198,199,200,
	202,204,206,208,210,212,214,216,218,220,
	222,223,224,225,226,227,228,229,230,231,
	232,233,234,235,236,237,238,239,240,250,
	251,252,253,254,255,256,257,258,259,260,
	261,262,263,264,265,266,267,268,269,270,
	271,272,273,274,275,276,277,278,279,280,
	281,282,283,284,285,286,287,288,289,290,
	291,292,293,294,295,296,297,298,299,300,
	301,302,303,304,305,306,307,308,309,310,
	311,312,313,314,315,316,317,318,319,320,
	321,322,323,324,325,326,327,328,329,330,
	331,332,333,334,335,336,337,338,339,340,
	341,342,343,344,345,346,347,348,349,350,
	351,352,353,354,355,356,357,358,359,360,
	361,362,363,364,365,366,367,368,369,370,
	371,372,373,374,375,376,377,378,379,380,
	381,382,383,384,385,386,387,388,389,390,
	391,392,393,394,395,396,397,398,399,400
    };
    int dam;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    level	= UMIN(level, sizeof(dam_each)/sizeof(dam_each[0]) - 1);
    level	= UMAX(0, level);
    dam		= number_range( dam_each[level], dam_each[level] * 2 );
    dam +=  op_bonus;
    stc("You weave flows of pure spirit.\n\r",ch);
    stc( "\n\rSome force wracks your soul, threatening to toss you over the edge of oblivion!\n\r", victim );
    act("\n\r$N grows pale, seeming to struggle within $Mself!\n\r",ch,NULL,victim,TO_ROOM );
    if ( saves_spell( level, victim, DAM_HARM) )
	dam /= 2;
    damage( ch, victim, dam, sn, DAM_HARM ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
void weave_hellfire( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int op_bonus = 0;
    int ang_bonus;
    int dam = 1;
    if (!IS_NPC(ch))
        ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_fire;
        dam = get_skill(ch,sn);
        }
        op_bonus += ang_bonus;
        op_bonus += pwrBonus;
        dam += ( op_bonus + (number_range(op_bonus*10/9,op_bonus*13/9) ) );

    if ( saves_spell( level+(level / 10), victim, DAM_HARM) )
        dam /= 2;

    stc("You weave flows of the purest {rf{Rir{re.{x\n\r",ch);
    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
            stc( "\n\rFire rains down upon you, searing and scorching your flesh to the bone!\n\r", victim );
            act("\n\rFire rains down upon $N searing and scorching $M to the bone!\n\r",ch,NULL,victim,TO_ROOM );
        }
        else
        {
                dam /= 2;
            stc( "\n\rSearing fire rains down upon you as you duck under your shield!\n\r", victim );
            act("\n\rSearing fire rains down upon $N as $E ducks under a shield!\n\r",ch,NULL,victim,TO_ROOM );
        }

    damage( ch, victim, dam, sn, DAM_HARM ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_newhellfire( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int ang_bonus;
    int dam;

    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    ang_bonus += truePwr;

	/*dam += ( op_bonus + (number_range(op_bonus*10/9,op_bonus*13/9) ) );*/
	dam = calc_weave_damage2( ch, sn, ang_bonus );

    if ( saves_spell( level+(level / 10), victim, DAM_HARM) )
	dam /= 2;

    stc("You weave flows of the purest {rf{Rir{re.{x\n\r",ch);
    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
	    stc( "\n\rFire rains down upon you, searing and scorching your flesh to the bone!\n\r", victim );
	    act("\n\rFire rains down upon $N searing and scorching $M to the bone!\n\r",ch,NULL,victim,TO_ROOM );
	}
	else
	{
		dam /= 2;
	    stc( "\n\rSearing fire rains down upon you as you duck under your shield!\n\r", victim );
	    act("\n\rSearing fire rains down upon $N as $E ducks under a shield!\n\r",ch,NULL,victim,TO_ROOM );
	}

    damage( ch, victim, dam, sn, DAM_HARM ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
void weave_purgatory( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int dam = 1;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_earth;
	dam = get_skill(ch,sn);
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
	dam += ( op_bonus * 2 );

    act("You weave the flows of sicken over $N!",ch,NULL,victim,TO_CHAR);
    act("$n crooks a finger at $N and grins malevolently!",ch,NULL,victim,TO_NOTVICT);
    act("$n crooks a finger at you and grins malevolently!",ch,NULL,victim,TO_VICT);

    if ( is_safe(ch, victim) ) return;

	if ( saves_spell( level + ( op_bonus * 2 ), victim, DAM_ENERGY ) )
    {
	stc( "You struggle against the swarm and manage to overcome it!\n\r", victim );
	act("$n shivers and sweats, but appears otherwise un-phased!",victim,NULL,NULL,TO_ROOM);
	return;
    }
/*
    af.type      = gsn_poison;
    af.duration  = 12 + (level/7);
    af.location  = APPLY_STR;
    af.modifier  = -20;
    af.bitvector = AFF_POISON;
    affect_join( victim, &af );

    af2.type      = gsn_poison;
    af2.duration  = 12 + (level/7);
    af2.location  = APPLY_CHA;
    af2.modifier  = -50;
    af2.bitvector = AFF_POISON;
    affect_join( victim, &af2 );
*/
    if ( get_eq_char( victim,WEAR_SHIELD ) == NULL )
    {
	    stc( "A cloud of insects swarm down upon you, biting and stinging.\n\r", victim );
	    act( "A cloud of insects swarm down upon $n, biting and stinging.", victim, NULL, NULL, TO_ROOM );
	}
	else
	{
		dam /= 2;
	    stc( "A cloud of insects swarm down upon you as you duck behind your shield!\n\r", victim );
	    act( "A cloud of insects swarm down upon $n as $E ducks behind a shield!", victim, NULL, NULL, TO_ROOM );
	}
    damage( ch, victim, dam, sn, DAM_ENERGY ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
void weave_crushing_blow( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int dam = 1;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_air;
	dam = get_skill(ch,sn);
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
	dam += ( op_bonus + (number_range(op_bonus*10/9,op_bonus*12/9) ) );

    stc("You weave a crushing blow of air.\n\r",ch);
    stc( "\n\rA great force slams into you, sending you flying through the air!\n\r", victim );
    act("\n\rA great force slams into $N, sending $M flying through the air!!\n\r",ch,NULL,victim,TO_ROOM );

    if ( !saves_spell( level, victim, DAM_ENERGY) )
		WAIT_STATE( victim, skill_table[sn].beats );
    if ( saves_spell( level, victim, DAM_ENERGY) )
    	dam /= 3;

    damage( ch, victim, dam, sn, DAM_ENERGY ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}
void weave_mind_shatter( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    int dam = 1;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	dam = get_skill(ch,sn);
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
	dam += ( op_bonus * 2 );

    stc( "\n\rA great force wracks your mind, leaving you reeling!\n\r", victim );
    act("\n\rA great force slams into $N, leaving $M reeling!\n\r",ch,NULL,victim,TO_ROOM );
    if ( !saves_spell( level, victim, DAM_ENERGY) )
		WAIT_STATE( victim, skill_table[sn].beats * 2 );
    if ( saves_spell( level, victim, DAM_ENERGY) )
    	dam /= 3;
    damage( ch, victim, dam, sn, DAM_ENERGY ,TRUE);
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
}
/*
 * One Power functions.
 *
 * Defensive Weaves
 * --------------
 */

void weave_defense( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
	int ang_bonus = 0;
    if ( is_affected( victim, sn ) )
    {
	if (victim == ch)
          stc( "{cYou are already defended by the One Power.\n\r",ch);
	else
          act("{c$N is already defended by the One Power.",ch,NULL,victim,TO_CHAR);
	return;
    }
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
	ang_bonus += pwrBonus;

    af.where	 = TO_AFFECTS;
    af.type      = sn;
    af.level	 = level;
    af.duration  = 24 + level / 2 + ang_bonus;
    af.modifier  = -20;
    af.location  = APPLY_AC;
    af.bitvector = 0;
    affect_to_char( victim, &af );
    stc( "{cYou feel the One Power defending you.\n\r", victim );
    if ( ch != victim )
        act("{c$N is defended by your weave.",ch,NULL,victim,TO_CHAR);
    return;
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
}

void weave_vigor( int sn, int level, CHAR_DATA *ch, void *vo, int target)
{
    CHAR_DATA *victim;
    AFFECT_DATA af;

    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    op_bonus += ch->pcdata->op_spirit;
    op_bonus += ang_bonus;
	op_bonus += pwrBonus;

    /* character target */
    victim = (CHAR_DATA *) vo;

    if ( victim->position == POS_FIGHTING || is_affected( victim, sn ) )
    {
	if (victim == ch)
          stc( "{cYou are already envigored.\n\r",ch);
	else
          act("{c$N is already envigored.",ch,NULL,victim,TO_CHAR);
	return;
    }

    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level	 = level;
    af.duration  = 6+level + op_bonus;
    af.location  = APPLY_HITROLL;
    af.modifier  = level / 8 + op_bonus;
    af.bitvector = 0;
    affect_to_char( victim, &af );

    af.location  = APPLY_SAVING_SPELL;
    af.modifier  = 0 - level / 8;
    affect_to_char( victim, &af );
    stc( "{cYou feel envigorated!\n\r", victim );
    if ( ch != victim )
        act("{cYou weave a vigor upon $N.",ch,NULL,victim,TO_CHAR);
    return;
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
}

/* RT calm spell stops all fighting in the room */
void weave_calm( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *vch;
    int mlevel = 0;
    int count = 0;
    int high_level = 0;
    int chance;
    AFFECT_DATA af;
    /* Spirit */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;

    /* get sum of all mobile levels in the room */
    for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
    {
	if (vch->position == POS_FIGHTING)
	{
	    count++;
	    if (IS_NPC(vch))
	      mlevel += vch->level;
	    else
	      mlevel += vch->level/2;
	    high_level = UMAX(high_level,vch->level);
	}
    }

    /* compute chance of stopping combat */
    chance = 4 * level - high_level + 2 * count;
	chance += op_bonus;
    if (IS_IMMORTAL(ch)) /* always works */
      mlevel = 0;

    if (number_range(0, chance) >= mlevel)  /* hard to stop large fights */
    {
	for (vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room)
   	{
	    if (IS_NPC(vch) && (IS_SET(vch->imm_flags,IMM_MAGIC) ||
				IS_SET(vch->act,ACT_UNDEAD)))
	      return;

	    if (IS_AFFECTED(vch,AFF_CALM) || IS_AFFECTED(vch,AFF_BERSERK)
	    ||  is_affected(vch,skill_lookup("frenzy")))
	      return;

	    stc( "{cA wave of calm passes over you.\n\r",vch);

	    if (vch->advancing_on || vch->position == POS_FIGHTING)
		    vch->range = RANGE_MISSLE;


	    af.where = TO_AFFECTS;
	    af.type = sn;
  	    af.level = level;
	    af.duration = level/4 + op_bonus;
	    af.location = APPLY_HITROLL;
	    if (!IS_NPC(vch))
	      af.modifier = -5;
	    else
	      af.modifier = -2;
	    af.bitvector = AFF_CALM;
	    affect_to_char(vch,&af);

	    af.location = APPLY_DAMROLL;
	    affect_to_char(vch,&af);
	}
    }
}

void weave_shield( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire/Earth/Water/Air */
    int op_bonus2 = 0;
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_earth+ch->pcdata->op_water+ch->pcdata->op_air;
    op_bonus /= 5;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( is_affected( victim, sn ) )
    {
	if (victim == ch)
	  stc( "{cYou are already shielded from harm.\n\r",ch);
	else
	  act("{c$N is already protected by a shield.",ch,NULL,victim,TO_CHAR);
	return;
    }
	op_bonus2 = 20;
	op_bonus += op_bonus2;
    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.duration  = 8 + level + op_bonus;
    af.location  = APPLY_AC;
    af.modifier  = -op_bonus;
    af.bitvector = 0;
    affect_to_char( victim, &af );
    stc("You weave flows of spirit.\n\r",ch);
    act("{c$n is enveloped by a brief flash of light.", victim, NULL, NULL, TO_ROOM );
    stc( "{cYou are enveloped in a brief flash of light.\n\r", victim );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_fire_shield( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire/Air */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire+ch->pcdata->op_air;
    op_bonus /= 3;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( is_affected( victim, sn ) )
    {
	if (victim == ch)
          stc( "{cYou are already surrounded by a fire shield.\n\r",ch);
	else
          act("{c$N is already surrounded by a fire shield.",ch,NULL,victim,TO_CHAR);
	return;
    }
    af.where	 = TO_AFFECTS;
    af.type      = sn;
    af.level	 = ch->level/2;
    af.duration  = ch->level/2 + op_bonus;
    af.modifier  = -20;
    af.location  = APPLY_AC;
    af.bitvector = OP_FSHIELD;
    affect_to_char( victim, &af );
    if (victim == ch)
    {
    	stc( "{cYou are surrounded by a {WS{whi{rel{Rd of{r F{wir{We.{x\n\r", victim );
	    act( "$n is surrounded by a {WS{whi{rel{Rd of{r F{wir{We!{x",ch,0,0,TO_ROOM);
	}
	else
	{
    	    act( "$N is surrounded by a {WS{whi{rel{Rd of{r F{wir{We!{x",ch,0,victim,TO_NOTVICT);
    	    act( "$N is surrounded by a {WS{whi{rel{Rd of{r F{wir{We!{x",ch,0,victim,TO_CHAR);
    	    act( "You are surrounded by a {WS{whi{rel{Rd of{r F{wir{We!{x",ch,0,victim,TO_VICT);
	}
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

/*
 * One Power functions.
 *
 * Misc Weaves
 * --------------
 */

void weave_glow(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    OBJ_DATA *light;

    if (target_name[0] != '\0') /*do a glow on some object*/
    {
	light = get_obj_carry(ch,target_name,ch);

	if (light == NULL)
	{
	    stc( "{cYou don't see that here.\n\r",ch);
	    return;
	}

	if (IS_OBJ_STAT(light,ITEM_GLOW))
	{
	    act("{c$p is already glowing.",ch,light,NULL,TO_CHAR);
	    return;
	}

	SET_BIT(light->extra_flags,ITEM_GLOW);
        act("{cA glowing ball of the {WOne Power{c settles over $p brightly.",ch,light,NULL,TO_ALL);
	return;
    }

    light = create_object( get_obj_index( OBJ_VNUM_LIGHT_BALL ), 0 );
    light->value[2] = (ch->pcdata->op_air * ch->pcdata->op_fire) / 2;
    obj_to_char( light, ch );
    act("{c$n weaves the {WOne Power{c and $p appears.",   ch, light, NULL, TO_ROOM );
    act("{cYou weave flows of the {WOne Power{c and $p appears.", ch, light, NULL, TO_CHAR );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_darken(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    OBJ_DATA *light;

	/* remove a glow from an object -- Dalsor 04/20/2004 */
    if ( target_name[0] != '\0' )
    {
		light = get_obj_carry(ch,target_name,ch);

		if (light == NULL)
		{
		    stc( "{cYou don't see that here.\n\r",ch);
		    return;
		}

		if (IS_OBJ_STAT(light,ITEM_GLOW))
		{
		    act("{c$n weaves the {WOne Power{c and $p darkens.",   ch, light, NULL, TO_ROOM );
		    act("{cYou weave flows of the {WOne Power{c and $p darkens.", ch, light, NULL, TO_CHAR );
		    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
			REMOVE_BIT(light->extra_flags,ITEM_GLOW);
		    return;
		}

    }
    return;
}

void weave_see_shadow( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( IS_AFFECTED(victim, AFF_DETECT_EVIL) )
    {
	if (victim == ch)
	  stc( "{cYou can already sense evil.\n\r",ch);
	else
	  act("{c$N can already detect evil.",ch,NULL,victim,TO_CHAR);
	return;
    }
    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level	 = level;
    af.duration  = level + op_bonus;
    af.modifier  = 0;
    af.location  = APPLY_NONE;
    af.bitvector = AFF_DETECT_EVIL;
    affect_to_char( victim, &af );
    stc( "{cYour eyes tingle.\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_see_light( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( IS_AFFECTED(victim, AFF_DETECT_GOOD) )
    {
        if (victim == ch)
          stc( "{cYou can already sense good.\n\r",ch);
        else
          act("{c$N can already detect good.",ch,NULL,victim,TO_CHAR);
        return;
    }
    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.duration  = level+op_bonus;
    af.modifier  = 0;
    af.location  = APPLY_NONE;
    af.bitvector = AFF_DETECT_GOOD;
    affect_to_char( victim, &af );
    stc( "{cYour eyes tingle.\n\r", victim );
    if ( ch != victim )
        stc( "{cOk.\n\r", ch );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_see_hidden(int sn,int level,CHAR_DATA *ch,void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( IS_AFFECTED(victim, AFF_DETECT_HIDDEN) )
    {
        if (victim == ch)
          stc( "{cYou are already as alert as you can be. \n\r",ch);
        else
          act("{c$N can already sense hidden lifeforms.",ch,NULL,victim,TO_CHAR);
        return;
    }
    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.duration  = level+op_bonus;
    af.location  = APPLY_NONE;
    af.modifier  = 0;
    af.bitvector = AFF_DETECT_HIDDEN;
    affect_to_char( victim, &af );
    stc( "{cYour awareness improves.\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_see_invis( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( IS_AFFECTED(victim, AFF_DETECT_INVIS) )
    {
        if (victim == ch)
          stc( "{cYou can already see invisible.\n\r",ch);
        else
          act("{c$N can already see invisible things.",ch,NULL,victim,TO_CHAR);
        return;
    }

    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.duration  = level+op_bonus;
    af.modifier  = 0;
    af.location  = APPLY_NONE;
    af.bitvector = AFF_DETECT_INVIS;
    affect_to_char( victim, &af );
    stc( "{cYour eyes tingle.\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_seeing( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    /* Spirit/Fire */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_fire;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    if ( IS_AFFECTED(victim, AFF_DETECT_MAGIC) )
    {
        if (victim == ch)
          stc( "{cYou can already sense weaves.\n\r",ch);
        else
          act("{c$N can already see weaves.",ch,NULL,victim,TO_CHAR);
        return;
    }

    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level	 = level;
    af.duration  = level+op_bonus;
    af.modifier  = 0;
    af.location  = APPLY_NONE;
    af.bitvector = AFF_DETECT_MAGIC;
    affect_to_char( victim, &af );
    stc( "{cYour eyes tingle.\n\r", victim );
    if ( ch != victim )
	stc( "{cOk.\n\r", ch );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_smell_poison( int sn, int level, CHAR_DATA *ch, void *vo,int target)
{
    OBJ_DATA *obj = (OBJ_DATA *) vo;

    if ( obj->item_type == ITEM_DRINK_CON || obj->item_type == ITEM_FOOD )
    {
	if ( obj->value[3] != 0 )
	    stc( "{cYou smell poisonous fumes.\n\r", ch );
	else
	    stc( "{cIt looks delicious.\n\r", ch );
    }
    else
    {
	stc( "{cIt doesn't look poisoned.\n\r", ch );
    }

    return;
}

void weave_gateway( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
	CHAR_DATA *fch, *fch_next;
	DESCRIPTOR_DATA	 *d_mentor;
	ROOM_INDEX_DATA *pRoomIndex = NULL;
    ROOM_INDEX_DATA *old_room;
//	bool can_gate = FALSE;
	bool gate_pet;
	int a;

	if ( IS_NPC(ch) )
		return;

	if ( IS_SET( ch->war_flags, WAR_BATTLE ) )
	{
		stc("You're currently in a battle.\n\r",ch);
		return;
	}
	if ( IS_SET( ch->war_flags, WAR_TOURNEY ) )
	{
		stc("You're currently in a tourney.\n\r",ch);
		return;
	}
    if (IS_SET (ch->pcdata->tag_flags, TAG_PLAYING | TAG_WAITING))
    {
      send_to_char ("Wait till the tag game is over.\n\r", ch);
      return;
    }

	if ( truePwr < 10 )
	{
		stc("The One Power escapes your control and the gateway collapses, slicing you in two!\n\r",ch);
	    raw_kill( ch, ch );
		return;
	}
	if ( ch->pcdata && ch->pcdata->allowed == 0 )
    {
		stc( "You are not allowed to leave the school until a Staff member\n\r",ch);
		stc( "has granted you access to the game. You should be approached\n\r",ch);
		stc( "soon by a Mentor. If you are not approached within an hour or\n\r",ch);
		stc( "two, you may ask over OOC, or by sending a Note to Staff.\n\r",ch);
        for ( d_mentor = descriptor_list; d_mentor != NULL; d_mentor = d_mentor->next )
        {
	        if ( d_mentor->connected == CON_PLAYING
            && IS_SET( d_mentor->character->spec_flags,SPEC_MENTOR ) )
	        {
            act("{w({GMentor Alert{w){W::{x {G$N needs verification!{x",
            d_mentor->character,NULL,ch,TO_CHAR);
	        }
	    }
		return;
	}
	if ( true_op_average(ch) < 10 )
	{
		stc("The One Power escapes your control and the gateway collapses, slicing you in two!\n\r",ch);
	    raw_kill( ch, ch );
		return;
	}
    if ( ch->pcdata->in_vault )
    {
      do_function( ch, &do_locker, "close" );
    }

	if ( target_name[0] == '\0' )
	{
	stc("To must specify a memorized room as the location.\n\r",ch);
	return;
	}

	if ( !is_number(target_name) )
	{
		stc("You must specify a memorized location.\n\r",ch);
		return;
	}

	a = atoi(target_name);

	if( a < 0 || a >= MAX_BEACONS )
	{
		stc("Beacon number out of range.\n\r", ch );
		return;
	}
/*
	for( a = 0; a < MAX_BEACONS; a++ )
	{
		pRoomIndex = get_room_index( ch->pcdata->beacons[a] );
		if ( a == atoi(target_name) )
		{
		can_gate = TRUE;
		break;
		}
	}
	*/
	pRoomIndex = get_room_index( ch->pcdata->beacons[a] );
	/*
	if (!can_gate)
	{
		stc("You can't gate there.\n\r",ch);
		return;
	}
	*/

	if ( is_fighting(ch) )
	{
		stc( "You struggle, but you're too close to your enemy fully form the weave.\n\r", ch );
		return;
	}
    /*if ( !can_gate_to( ch, pRoomIndex ) );
    {
        stc( "{cYou failed.\n\r", ch );
        return;
    }*/
    old_room = ch->in_room;
    stc("You weave flows of earth, air, and fire.\n\r",ch);
    if (ch->pet != NULL && ch->in_room == ch->pet->in_room)
		gate_pet = TRUE;
    else
		gate_pet = FALSE;
	stc("{cYou tear a hole in the Pattern forming a gateway.\n\r",ch);
    act("{cA slash of light widens into a hole, through which another landscape is visible.",ch,NULL,NULL,TO_ROOM);
    act("{c$n steps through a gateway.",ch,NULL,NULL,TO_ROOM);
    stc( "{cYou step through a gateway.\n\r",ch);
    char_from_room(ch);
    /*char_to_room(ch,victim->in_room);*/
    char_to_room(ch,pRoomIndex);
    do_function(ch, &do_look, "auto" );
    act("{cA slash of light widens into a hole, through which another landscape is visible.",ch,NULL,NULL,TO_ROOM);
    act("{c$n steps through a gateway.",ch,NULL,NULL,TO_ROOM);
	/*if (old_room == victim->in_room)*/
	/*if (old_room == victim->in_room)
	    return;*/

   	for ( fch = old_room->people; fch != NULL; fch = fch_next )
   	{
        fch_next = fch->next_in_room;

        if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM)
        &&   fch->position < POS_STANDING)
       	do_function(fch, &do_stand, "");

        if ( fch->master == ch && fch->position == POS_STANDING)
        {

            if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
            &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
            {
                act("{cYou can't take $N there.{c",ch,NULL,fch,TO_CHAR);
                continue;
          	}
           	act( "{cYou follow $N.{c", fch, NULL, ch, TO_CHAR );
			char_from_room(fch);
			/*char_to_room(fch,victim->in_room);*/
			char_to_room(fch,pRoomIndex);
        }
   	}

    if( MOUNTED(ch) )
    {
        act("$n steps through a gateway.",MOUNTED(ch),NULL,NULL,TO_ROOM);
        send_to_char("You step through a gateway.\n\r",MOUNTED(ch));
        char_from_room(MOUNTED(ch));
        /*char_to_room(MOUNTED(ch),victim->in_room);*/
        char_to_room(MOUNTED(ch),pRoomIndex);
        act("$n has arrived through a gateway.",MOUNTED(ch),NULL,NULL,TO_ROOM);
        do_look(MOUNTED(ch),"auto");
    }

    if (gate_pet)
    {
        act("{c$n steps through a gateway.",ch->pet,NULL,NULL,TO_ROOM);
        stc( "{cYou step through a gateway.\n\r",ch->pet);
		char_from_room(ch->pet);
		char_to_room(ch->pet,pRoomIndex);
        act("{c$n has arrived through a gateway.",ch->pet,NULL,NULL,TO_ROOM);
		do_function(ch->pet, &do_look, "auto");
    }
}

void weave_invert( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    AFFECT_DATA af;
    /* Spirit/Air */
    int op_bonus = 0;
    int ang_bonus;
    if (!IS_NPC(ch))
    	ang_bonus = check_angreal(ch);
    else ang_bonus = 0;
    if (!IS_NPC(ch))
    {
    op_bonus = ch->pcdata->op_spirit+ch->pcdata->op_air;
    op_bonus /= 2;
	}
	op_bonus += ang_bonus;
	op_bonus += pwrBonus;
    /* object invisibility */
    if (target == TARGET_OBJ)
    {
	obj = (OBJ_DATA *) vo;

	if (IS_OBJ_STAT(obj,ITEM_INVIS))
	{
	    act("{c$p is already invisible.",ch,obj,NULL,TO_CHAR);
	    return;
	}

	af.where	= TO_OBJECT;
	af.type		= sn;
	af.level	= level;
	af.duration	= level + 12 + op_bonus;
	af.location	= APPLY_NONE;
	af.modifier	= 0;
	af.bitvector	= ITEM_INVIS;
	affect_to_obj(obj,&af);

    stc("You weave flows of spirit, earth, and air.\n\r",ch);
	act("{c$p fades out of sight.",ch,obj,NULL,TO_ALL);
	return;
    }

    /* character invisibility */
    victim = (CHAR_DATA *) vo;

    if ( IS_AFFECTED(victim, AFF_INVISIBLE) )
	return;

	if ( IS_NPC( victim ) )
	{
		stc("Inverting mobiles is not an acceptable use of this weave.\n\r",ch);
		stc("The Immortals have been alerted to your actions.\n\r",ch);
		wiznet("Wiznet alert> $N is trying to invert mobiles!",ch,NULL,0,0,0);
		return;
	}

    act("{c$n fades out of existence.", victim, NULL, NULL, TO_ROOM );

    af.where     = TO_AFFECTS;
    af.type      = sn;
    af.level     = level;
    af.duration  = level + 12 + op_bonus;
    af.location  = APPLY_NONE;
    af.modifier  = 0;
    af.bitvector = AFF_INVISIBLE;
    affect_to_char( victim, &af );
    stc( "{cYou fade out of existence.\n\r", victim );
    check_stat_improve( ch, skill_table[sn].sphere, pwrBonus );
    return;
}

void weave_teleport( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    ROOM_INDEX_DATA *pRoomIndex;

    if ( victim->in_room == NULL
    ||   IS_SET(victim->in_room->room_flags, ROOM_NO_RECALL)
    || ( victim != ch && IS_SET(victim->imm_flags,IMM_SUMMON))
    || ( !IS_NPC(ch) && is_fighting(victim))
    || ( victim != ch
    && ( saves_spell( level - 5, victim,DAM_OTHER))))
    {
        stc( "{cYour gateway fails.\n\r", ch );
	return;
    }
	return;
    pRoomIndex = get_random_room(victim);
    stc("You weave flows of earth, air, spirit, and fire.\n\r",ch);

    if (victim != ch)
        stc( "{cYou have been drawn through a gateway!\n\r",victim);

    act("{c$n is drawn into a gateway!", victim, NULL, NULL, TO_ROOM );
    char_from_room( victim );
    char_to_room( victim, pRoomIndex );
    act("{c$n is drawn into a gateway.", victim, NULL, NULL, TO_ROOM );
    do_function(victim, &do_look, "auto" );
    return;
}

void weave_bond( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{

	char buf[MSL];
    CHAR_DATA *victim = (CHAR_DATA *) vo;

    if ( is_safe ( ch,victim ) ) return;

	if (IS_NPC(victim))
		return;

    if ( victim == ch )
    {
		stc( "{cExactly how do you propose to bond yourself?\n\r", ch );
		return;
    }
    if ( IS_AFFECTED(victim, AFF_CHARM) )
    {
		stc( "{cSomething is interferring with your weave.\n\r", ch );
		return;
	}

    if ( IS_SET(victim->hist_flags, HIST_BONDED) )
    {
		stc( "{cYour weave meets resistance from an existing knot of weaves.\n\r", ch );
		return;
	}
    if ( IS_SET(ch->hist_flags, HIST_BONDED) )
    {
		stc( "{cYou already have a bondmate.\n\r", ch );
		return;
	}
    if ( IS_SET(ch->hist_flags, HIST_BONDER) )
    {
		stc( "{cYou already have a bondmate.\n\r", ch );
		return;
	}
    if ( IS_SET(victim->hist_flags, HIST_BONDER) )
    {
		stc( "{cYour weave meets resistance from an existing knot of weaves.\n\r", ch );
		return;
	}
    if ( IS_SET(victim->imm_flags,IMM_CHARM) )
    {
		stc( "{cYour weave meets an invisible wall and shatters.\n\r", ch );
		return;
	}
	else
	{
    stc("You weave flows of Spirit, twining them with flows of Earth, Fire, Air, and Water.\n\r",ch);
	act("$n places $m palms against your temples.",ch,0,victim,TO_VICT);
	act("$n places $m palms against $N's temples.",ch,0,victim,TO_NOTVICT);
	act("$N's eyes grow wide for a moment, then $E grits $S teeth in pain.",ch,0,victim,TO_NOTVICT);
	stc("A gentle caress flows across your mind. Then a thousand needles pierce your skull!\n\r",victim);
	SET_BIT(victim->hist_flags,HIST_BONDED);
	SET_BIT(ch->hist_flags,HIST_BONDER);
	ch->pcdata->bonded_to = str_dup(victim->name);
	victim->pcdata->bonded_to = str_dup(ch->name);
	victim->pcdata->bonded_by = str_dup(ch->name);
    	sprintf(buf, "{B[{w%s{B]", victim->name);
    	refresh_bonded_list(ch, ch->name, buf);
    	sprintf(buf, "{B[{w%s{B]", ch->name);
    	refresh_bonded_list(victim, victim->name, buf);
	compose_title(ch);
	compose_title(victim);
    act( "{cYou can feel $N's presence in your mind.", victim,0,ch,TO_VICT);
    act( "{cYou can feel $N's presence in your mind.", ch,0,victim,TO_CHAR);
	}
	return;
}
/*    if ( ( victim = get_char_world( ch, target_name ) ) == NULL
    ||   victim == ch
    ||   victim->in_room == NULL
    ||   IS_SET(victim->in_room->room_flags, ROOM_SAFE)
    ||   IS_SET(victim->in_room->room_flags, ROOM_PRIVATE)
    ||   IS_SET(victim->in_room->room_flags, ROOM_SOLITARY)
    ||   IS_SET(victim->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_SET(victim->act,ACT_UNDEAD)
    ||   IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_SET(victim->in_room->room_flags, ROOM_NOGATE)
    ||   (victim->in_room->clan > 2 && ch->clan != victim->in_room->clan))*/
