/***************************************************************************
                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/
    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.
    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.
    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.
    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.
    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.
	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license
 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "aww.h"
#include "interp.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "db.h"
#include "olc.h"
#include "mysql5_interface.h"

void change_wizlist args( (CHAR_DATA *ch, bool add, int level, char *argument, char *job, char *email));
WIZ_DATA *wiz_list;

void save_wizlist( void )
{
    WIZ_DATA *pwiz;
    FILE *fp;
    bool found = FALSE;
    if ( ( fp = fopen( WIZ_FILE, "w" ) ) == NULL )
    {
        perror( WIZ_FILE );
	return;
    }

    for (pwiz = wiz_list; pwiz != NULL; pwiz = pwiz->next)
    {
	found = TRUE;
        fprintf(fp,"%s %d %s~ %s~\n", pwiz->name, pwiz->level, pwiz->job, pwiz->email);
    }
    fclose(fp);
    if ( !found )
	unlink(WIZ_FILE);
}

void load_wizlist( void )
{
    FILE *fp;
    WIZ_DATA *wiz_last;
    if ( ( fp = fopen( WIZ_FILE, "r" ) ) == NULL )
    {
        perror( WIZ_FILE );
        return;
    }
    wiz_last = NULL;
    for ( ; ; )
    {
        WIZ_DATA *pwiz;
        if ( feof(fp) )
        {
            fclose( fp );
            return;
        }
        pwiz            = new_wiz();
        pwiz->name      = str_dup(fread_word(fp));
        pwiz->level     = fread_number(fp);
        pwiz->job       = str_dup(fread_string(fp));
        pwiz->email     = str_dup(fread_string(fp));
	fread_to_eol(fp);
        if (wiz_list == NULL)
	    wiz_list = pwiz;
	else
	    wiz_last->next = pwiz;
	wiz_last = pwiz;
    }
    return;
}

void do_wizlist(CHAR_DATA *ch, char *argument)
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char arg3[MAX_INPUT_LENGTH];
    char arg4[MAX_INPUT_LENGTH];
    char arg5[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    BUFFER *buffer;
    int level;
    WIZ_DATA *pwiz;
    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    argument = one_argument( argument, arg3 );
    argument = one_argument( argument, arg4 );
    argument = one_argument( argument, arg5 );
    if ((arg1[0] != '\0') && (ch->level == MAX_LEVEL))
    {
	if ( !str_prefix( arg1, "add" ) )
	{
	    if ( !is_number( arg2 ) || ( arg3[0] == '\0' ) || (arg4[0] == '\0') || (arg5[0] == '\0'))
	    {
                send_to_char( "Syntax: wizlist add <level> <name> <job> <email>\n\r", ch );
		return;
	    }
	    level = atoi(arg2);
	    change_wizlist(ch, TRUE, level, arg3,arg4,arg5);
	    return;
	}
	if ( !str_prefix( arg1, "delete" ) )
	{
	    if ( arg2[0] == '\0' )
	    {
		send_to_char( "Syntax: wizlist delete <name>\n\r", ch );
		return;
	    }
	    change_wizlist(ch, FALSE, 0, arg2, arg4, arg5);
	    return;
	}
	send_to_char( "Syntax:\n\r", ch );
	send_to_char( "       wizlist delete <name>\n\r", ch );
	send_to_char( "       wizlist add <level> <name> <job> <email>\n\r", ch );
	return;
    }
    if (wiz_list == NULL)
    {
	send_to_char("No immortals listed at this time.\n\r",ch);
	return;
    }
    buffer = new_buf();
    sprintf(buf, "              {RAs The Wheel Weaves{x              \n\r");
    add_buf(buffer, buf);
    sprintf(buf, "{C------------------------------------------------------------------------------{x\n\r");
    add_buf(buffer, buf);
    sprintf(buf, "{R%-3s {M%-15s {G%-20s {Y%-30s{x", "Lev", "Name", "Job/Position", "Email/Icq" );
    add_buf(buffer, buf);
    sprintf(buf, "{C------------------------------------------------------------------------------{x\n\r");
    add_buf(buffer, buf);
    for (level = 250; level > 243; level--)
    {
    	for (pwiz = wiz_list;pwiz != NULL;pwiz = pwiz->next)
    	{
    	    if (pwiz->level == level)
    	    {
                sprintf(buf, "{R%3d {M%-15s {G%-20s {Y%-30s{x\n\r",
                    pwiz->level, pwiz->name, pwiz->job, pwiz->email);
    		add_buf(buffer, buf);
    	    }
    	}
    }
    sprintf(buf, "{C------------------------------------------------------------------------------{x\n\r");
    add_buf(buffer,  buf);
    page_to_char( buf_string(buffer), ch );
    free_buf(buffer);
    return;
}

void update_wizlist(CHAR_DATA *ch, int level, char *job, char *email)
{
    WIZ_DATA *prev;
    WIZ_DATA *curr;
    if (IS_NPC(ch))
	return;
    prev = NULL;
    for ( curr = wiz_list; curr != NULL; prev = curr, curr = curr->next )
    {
        if ( !str_cmp( ch->name, curr->name ) )
        {
            if ( prev == NULL )
                wiz_list   = wiz_list->next;
            else
                prev->next = curr->next;
            free_wiz(curr);
	    save_wizlist();
        }
    }
    if (level <= HERO)
	return;
    curr                = new_wiz();
    curr->name          = str_dup(ch->name);
    curr->level         = level;
    curr->job           = job;
    curr->email         = email;
    curr->next          = wiz_list;
    wiz_list            = curr;
    save_wizlist();
    return;
}

void change_wizlist(CHAR_DATA *ch, bool add, int level, char *argument, char *job, char *email)
{
    char arg[MAX_INPUT_LENGTH];
    WIZ_DATA *prev;
    WIZ_DATA *curr;
    one_argument( argument, arg );
    if (arg[0] == '\0')
    {
	send_to_char( "Syntax:\n\r", ch );
	if ( !add )
	    send_to_char( "    wizlist delete <name>\n\r", ch );
	else
	    send_to_char( "    wizlist add <level> <name> <job> <email>\n\r", ch );
	return;
    }
    if ( add )
    {
	if ( ( level <= HERO ) || ( level > MAX_LEVEL ) )
	{
	    send_to_char( "Syntax:\n\r", ch );
	    send_to_char( "    wizlist add <level> <name> <job> <email>\n\r", ch );
	    return;
	}
    }
    if ( !add )
    {
	prev = NULL;
	for ( curr = wiz_list; curr != NULL; prev = curr, curr = curr->next )
	{
	    if ( !str_cmp( arg, curr->name ) )
	    {
		if ( prev == NULL )
		    wiz_list   = wiz_list->next;
		else
		    prev->next = curr->next;
		free_wiz(curr);
		save_wizlist();
	    }
	}
    }
    else
    {
        curr            = new_wiz();
        curr->name      = str_dup( arg );
        curr->level     = level;
        curr->job       = str_dup( job );
        curr->email     = str_dup( email );
        curr->next      = wiz_list;
        wiz_list        = curr;
	save_wizlist();
    }
    return;
}
