/***************************************************************************

                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/

    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.

    File: olc_act.c

    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.

    This code was freely distributed with the The Isles 1.1 source code,
    and has been used here for OLC - OLC would not be what it is without
    all the previous coders who released their source code.

 ***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "aww.h"
#include "tables.h"
#include "olc.h"
#include "recycle.h"
#include "lookup.h"
//#include "tailoring.h"

bool is_human args(( CHAR_DATA *ch ));
void unlink_reset args(( ROOM_INDEX_DATA *pRoom, RESET_DATA *pReset ));
void unlink_obj_index args(( OBJ_INDEX_DATA *pObj ));
void unlink_mob_index args(( MOB_INDEX_DATA *pMob ));
void unlink_room_index args(( ROOM_INDEX_DATA *pRoom ));

char * prog_type_to_name( int type );

#define ALT_FLAGVALUE_SET( _blargh, _table, _arg )		\
	{							\
		int blah = flag_value( _table, _arg );		\
		_blargh = (blah == NO_FLAG) ? 0 : blah;		\
	}

#define ALT_FLAGVALUE_TOGGLE( _blargh, _table, _arg )		\
	{							\
		int blah = flag_value( _table, _arg );		\
		_blargh ^= (blah == NO_FLAG) ? 0 : blah;	\
	}

/* Return TRUE if area changed, FALSE if not. */
#define REDIT( fun )		bool fun( CHAR_DATA *ch, char *argument )
#define OEDIT( fun )		bool fun( CHAR_DATA *ch, char *argument )
#define MEDIT( fun )		bool fun( CHAR_DATA *ch, char *argument )
#define AEDIT( fun )		bool fun( CHAR_DATA *ch, char *argument )
#define GEDIT( fun )            bool fun( CHAR_DATA *ch, char *argument )



struct olc_help_type
{
    char *command;
    const void *structure;
    char *desc;
};

void clear_obj_values( OBJ_INDEX_DATA *pObj )
{
	if ( !pObj )
	return;

	pObj->level = 0;
	pObj->cost = 0;
	pObj->weight = 0;
	pObj->value[0] = 0;
	pObj->value[1] = 0;
	pObj->value[2] = 0;
	pObj->value[3] = 0;
	pObj->value[4] = 0;
	return;
}
bool is_wood( OBJ_INDEX_DATA *pObj )
{
	if ( !pObj )
	return FALSE;
	if ( !str_cmp( pObj->material, "pine" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "leatherleaf" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "rosewood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "satinwood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "hemlock" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "poplar" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "fir" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "beech" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "spruce" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "cypress" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "juniper" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "walnut" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "cedar" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "maple" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "mahogany" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "teak" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "ebony" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "redwood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "willow" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "oak" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "yew" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "soft wood" ) )
	return TRUE;
	if ( !str_cmp( pObj->material, "hard wood" ) )
	return TRUE;

	return FALSE;
}

bool show_version( CHAR_DATA *ch, char *argument )
{
    stc( VERSION, ch );
    stc( "{c\n\r", ch );
    stc( AUTHOR, ch );
    stc( "{c\n\r", ch );
    stc( DATE, ch );
    stc( "{c\n\r", ch );
    stc( CREDITS, ch );
    stc( "{c\n\r", ch );

    return FALSE;
}

/*
 * This table contains help commands and a brief description of each.
 * ------------------------------------------------------------------
 */
const struct olc_help_type help_table[] =
{
    {	"area",		area_flags,	 "{w Area attributes.{x"		 },
    {	"room",		room_flags,	 "{w Room attributes.{x"		 },
    {	"sector",	sector_flags,	 "{w Sector types, terrain.{x"	 },
    {	"exit",		exit_flags,	 "{w Exit types.{x"			 },
    {	"type",		type_flags,	 "{w Types of objects.{x"		 },
    {	"extra",	extra_flags,	 "{w Object attributes.{x"		 },
    {	"wear",		wear_flags,	 "{w Where to wear object.{x"	 },
    {	"spec",		spec_table,	 "{w Available special programs.{x" 	 },
    {	"sex",		sex_flags,	 "{w Sexes.{x"			 },
    {	"act",		act_flags,	 "{w Mobile attributes.{x"		 },
    {	"affect",	affect_flags,	 "{w Mobile affects.{x"		 },
    {	"wear-loc",	wear_loc_flags,	 "{w Where mobile wears object.{x"	 },
    {	"spells",	skill_table,	 "{w Names of current spells.{x" 	 },
    {	"container",	container_flags, "{w Container status.{x"		 },
    {   "guild",        guild_flags,     "Guild flags."			 },

/* ROM specific bits: */

    {	"armor",	ac_type,	 "{w AC for different attacks.{x"	 },
    {   "apply",	apply_flags,	 "{w Apply flags.{x"			 },
    {	"form",		form_flags,	 "{w Mobile body form.{x"	         },
    {	"part",		part_flags,	 "{w Mobile body parts.{x"		 },
    {	"imm",		imm_flags,	 "{w Mobile immunity.{x"		 },
    {	"res",		res_flags,	 "{w Mobile resistance.{x"	         },
    {	"vuln",		vuln_flags,	 "{w Mobile vulnerability.{x"	 },
    {	"off",		off_flags,	 "{w Mobile offensive behaviour.{x"	 },
    {	"size",		size_flags,	 "{w Mobile size.{x"			 },
    {   "position",     position_flags,  "{w Mobile positions.{x"             },
    {   "wclass",       weapon_class,    "{w Weapon class.{x"                 },
    {   "wtype",        weapon_type2,    "{w Special weapon type.{x"          },
    {	"portal",	portal_flags,	 "{w Portal types.{x"		 },
    {	"furniture",	furniture_flags, "{w Furniture types.{x"		 },
    {   "liquid",	liq_table,	 "{w Liquid types.{x"		 },
    {	"apptype",	apply_types,	 "{w Apply types.{x"			 },
    {	"weapon",	attack_table,	 "{w Weapon types.{x"		 },
    {	"mprog",	mprog_flags,	 "{w MobProgram flags.{x"		 },
    {	"oprog",	oprog_flags,	 "ObjProgram flags."		 },
    {	"rprog",	rprog_flags,	 "RoomProgram flags."		 },
    {	NULL,		NULL,		 NULL				 }
};



/*****************************************************************************
 Name:		show_flag_cmds
 Purpose:	Displays settable flags and stats.
 Called by:	show_help(olc_act.c).
 ****************************************************************************/
void show_flag_cmds( CHAR_DATA *ch, const struct flag_type *flag_table )
{
    char buf  [ MAX_STRING_LENGTH ];
    char buf1 [ MAX_STRING_LENGTH ];
    int  flag;
    int  col;

    buf1[0] = '\0';
    col = 0;
    for (flag = 0; flag_table[flag].name != NULL; flag++)
    {
	if ( flag_table[flag].settable )
	{
	    sprintf( buf, "{c{B%-19.18s{c", flag_table[flag].name );
	    strcat( buf1, buf );
	    if ( ++col % 4 == 0 )
		strcat( buf1, "\n\r" );
	}
    }

    if ( col % 4 != 0 )
	strcat( buf1, "\n\r" );

    stc( buf1, ch );
    return;
}


/*****************************************************************************
 Name:		show_skill_cmds
 Purpose:	Displays all skill functions.
 		Does remove those damn immortal commands from the list.
 		Could be improved by:
 		(1) Adding a check for a particular class.
 		(2) Adding a check for a level range.
 Called by:	show_help(olc_act.c).
 ****************************************************************************/
void show_skill_cmds( CHAR_DATA *ch, int tar )
{
    char buf  [ MAX_STRING_LENGTH ];
    char buf1 [ MAX_STRING_LENGTH*2 ];
    int  sn;
    int  col;

    buf1[0] = '\0';
    col = 0;
    for (sn = 0; sn < MAX_SKILL; sn++)
    {
	if ( !skill_table[sn].name )
	    break;

	if ( !str_cmp( skill_table[sn].name, "reserved" )
	  || skill_table[sn].spell_fun == spell_null )
	    continue;

	if ( tar == -1 || skill_table[sn].target == tar )
	{
	    sprintf( buf, "{c{B%-19.18s{c", skill_table[sn].name );
	    strcat( buf1, buf );
	    if ( ++col % 4 == 0 )
		strcat( buf1, "\n\r" );
	}
    }

    if ( col % 4 != 0 )
	strcat( buf1, "\n\r" );

    stc( buf1, ch );
    return;
}



/*****************************************************************************
 Name:		show_spec_cmds
 Purpose:	Displays settable special functions.
 Called by:	show_help(olc_act.c).
 ****************************************************************************/
void show_spec_cmds( CHAR_DATA *ch )
{
    char buf  [ MAX_STRING_LENGTH ];
    char buf1 [ MAX_STRING_LENGTH ];
    int  spec;
    int  col;

    buf1[0] = '\0';
    col = 0;
    stc( "{cPreceed special functions with 'spec_'\n\r\n\r", ch );
    for (spec = 0; spec_table[spec].function != NULL; spec++)
    {
	sprintf( buf, "{c{B%-19.18s{c", &spec_table[spec].name[5] );
	strcat( buf1, buf );
	if ( ++col % 4 == 0 )
	    strcat( buf1, "\n\r" );
    }

    if ( col % 4 != 0 )
	strcat( buf1, "\n\r" );

    stc( buf1, ch );
    return;
}



/*****************************************************************************
 Name:		show_help
 Purpose:	Displays help for many tables used in OLC.
 Called by:	olc interpreters.
 ****************************************************************************/
bool show_help( CHAR_DATA *ch, char *argument )
{
    char buf[MSL];
    char arg[MIL];
    char spell[MIL];
    int cnt;

    argument = one_argument( argument, arg );
    one_argument( argument, spell );

    /*
     * Display syntax.
     */
    if ( arg[0] == '\0' )
    {
	stc( "{cSyntax:  ? {C[{ccommand{C]{c\n\r\n\r", ch );
	stc( "{c[{ccommand{C]  [{cdescription{C]{c\n\r", ch );
	for (cnt = 0; help_table[cnt].command != NULL; cnt++)
	{
	    sprintf( buf, "{c{B%-10.10s -%s{c\n\r",
	        capitalize( help_table[cnt].command ),
		help_table[cnt].desc );
	    stc( buf, ch );
	}
	return FALSE;
    }

    /*
     * Find the command, show changeable data.
     * ---------------------------------------
     */
    for (cnt = 0; help_table[cnt].command != NULL; cnt++)
    {
        if (  arg[0] == help_table[cnt].command[0]
          && !str_prefix( arg, help_table[cnt].command ) )
	{
	    if ( help_table[cnt].structure == spec_table )
	    {
		show_spec_cmds( ch );
		return FALSE;
	    }
	    else
	    if ( help_table[cnt].structure == liq_table )
	    {
	        show_liqlist( ch );
	        return FALSE;
	    }
	    else
	    if ( help_table[cnt].structure == attack_table )
	    {
	        show_damlist( ch );
	        return FALSE;
	    }
	    else
	    if ( help_table[cnt].structure == skill_table )
	    {

		if ( spell[0] == '\0' )
		{
		    stc( "{cSyntax:  ? spells "
		        "{C[{cignore/attack/defend/self/object/all{C]{c\n\r", ch );
		    return FALSE;
		}

		if ( !str_prefix( spell, "all" ) )
		    show_skill_cmds( ch, -1 );
		else if ( !str_prefix( spell, "ignore" ) )
		    show_skill_cmds( ch, TAR_IGNORE );
		else if ( !str_prefix( spell, "attack" ) )
		    show_skill_cmds( ch, TAR_CHAR_OFFENSIVE );
		else if ( !str_prefix( spell, "defend" ) )
		    show_skill_cmds( ch, TAR_CHAR_DEFENSIVE );
		else if ( !str_prefix( spell, "self" ) )
		    show_skill_cmds( ch, TAR_CHAR_SELF );
		else if ( !str_prefix( spell, "object" ) )
		    show_skill_cmds( ch, TAR_OBJ_INV );
		else
		    stc( "{cSyntax:  ? spell "
		        "{C[{cignore/attack/defend/self/object/all{C]{c\n\r", ch );

		return FALSE;
	    }
	    else
	    {
		show_flag_cmds( ch, help_table[cnt].structure );
		return FALSE;
	    }
	}
    }

    show_help( ch, "" );
    return FALSE;
}

REDIT( redit_rlist )
{
    ROOM_INDEX_DATA	*pRoomIndex;
    AREA_DATA		*pArea;
    char		buf  [ MAX_STRING_LENGTH   ];
    BUFFER		*buf1;
    char		arg  [ MAX_INPUT_LENGTH    ];
    bool found,fAll;
    int vnum;
    int  col = 0;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc( "{cSyntax:  rlist <all/name>\n\r", ch );
	return FALSE;
    }

    pArea = ch->in_room->area;
    buf1=new_buf();
/*    buf1[0] = '\0'; */
    fAll    = !str_cmp( arg, "all" );
    found   = FALSE;

    for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
    {
	if ( ( pRoomIndex = get_room_index( vnum ) ) )
	{
	    if ( fAll || is_name( arg, pRoomIndex->name ) )
	    {
		found = TRUE;
		sprintf( buf, "{c[{c%5d{C]{c %-17.16s",
		    vnum, capitalize( pRoomIndex->name ) );
		add_buf( buf1, buf );
		if ( ++col % 3 == 0 )
		    add_buf( buf1, "\n\r" );
	    }
	}
    }

    if ( !found )
    {
	stc( "{cRoom(s) not found in this area.\n\r", ch);
	return FALSE;
    }

    if ( col % 3 != 0 )
	add_buf( buf1, "\n\r" );

    page_to_char( buf_string(buf1), ch );
    free_buf(buf1);
    return FALSE;
}
REDIT( redit_mplist )
{
    int count = 1;
    PROG_CODE *mprg;
    char buf[MAX_STRING_LENGTH];
    BUFFER *buffer;
    MOB_INDEX_DATA *mob;
	int hash;
	PROG_LIST *mpl;
	bool found;
    buffer = new_buf();
	send_to_char("{MNumber{x  {CMPVnum{x  {GMobVnum{x\n\r", ch);
	send_to_char("{Y------------------------{x\n\r", ch);

	for (mprg = mprog_list; mprg !=NULL; mprg = mprg->next)
	if ( mprg != NULL )
	{

	if ( mprg->vnum >= ch->in_room->area->min_vnum && mprg->vnum <= ch->in_room->area->max_vnum )
	{
		found = FALSE;
		for ( hash = 0; hash < MAX_KEY_HASH; hash++ )
	         for ( mob = mob_index_hash[hash]; mob; mob = mob->next )
				 for ( mpl = mob->mprogs; mpl; mpl = mpl->next )
					 if ( mpl->vnum == mprg->vnum)
					 {
						found = TRUE;
						sprintf(buf, "[%3d]   %5ld   %5ld\n\r", count, mprg->vnum, mob->vnum );
						add_buf(buffer, buf);
						count++;
					}
	if (!found)
	{
	sprintf(buf, "[%3d]   %5ld     {Rnone{x\n\r", count, mprg->vnum );
	add_buf(buffer, buf);
	count++;
	}
	}
	}
	if ( count == 1 )
    {
    	add_buf( buffer, "No MOBProgs exist in this area.\n\r" );
    }

    page_to_char(buf_string(buffer), ch);
    free_buf(buffer);

    return FALSE;
}
REDIT( redit_mlist )
{
    MOB_INDEX_DATA	*pMobIndex;
    AREA_DATA		*pArea;
    char		buf  [ MAX_STRING_LENGTH   ];
    BUFFER		*buf1;
    char		arg  [ MAX_INPUT_LENGTH    ];
    bool fAll, found;
    int vnum;
    int  col = 0;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc( "{cSyntax:  mlist <all/name>\n\r", ch );
	return FALSE;
    }

    buf1=new_buf();
    pArea = ch->in_room->area;
    fAll    = !str_cmp( arg, "all" );
    found   = FALSE;

    for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
    {
	if ( ( pMobIndex = get_mob_index( vnum ) ) != NULL )
	{
	    if ( fAll || is_name( arg, pMobIndex->player_name ) )
	    {
		found = TRUE;
		sprintf( buf, "{c[{c%5ld{C]{c %-17.16s",
		    pMobIndex->vnum, capitalize( pMobIndex->short_descr ) );
		add_buf( buf1, buf );
		if ( ++col % 3 == 0 )
		    add_buf( buf1, "\n\r" );
	    }
	}
    }

    if ( !found )
    {
	stc( "{cMobile(s) not found in this area.\n\r", ch);
	return FALSE;
    }

    if ( col % 3 != 0 )
	add_buf( buf1, "\n\r" );

    page_to_char( buf_string(buf1), ch );
    free_buf(buf1);
    return FALSE;
}



REDIT( redit_olist )
{
    OBJ_INDEX_DATA	*pObjIndex;
    AREA_DATA		*pArea;
    char		buf  [ MAX_STRING_LENGTH   ];
    BUFFER		*buf1;
    char		arg  [ MAX_INPUT_LENGTH    ];
    bool fAll, found;
    int vnum;
    int  col = 0;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	stc( "{cSyntax:  olist <all/name/item_type>\n\r", ch );
	return FALSE;
    }

    pArea = ch->in_room->area;
    buf1=new_buf();
    fAll    = !str_cmp( arg, "all" );
    found   = FALSE;

    for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
    {
	if ( ( pObjIndex = get_obj_index( vnum ) ) )
	{
	    if ( fAll || is_name( arg, pObjIndex->name )
	    || flag_value( type_flags, arg ) == pObjIndex->item_type )
	    {
		found = TRUE;
		sprintf( buf, "{c[{c%5ld{C]{c %-17.16s",
		    pObjIndex->vnum, capitalize( pObjIndex->short_descr ) );
		add_buf( buf1, buf );
		if ( ++col % 3 == 0 )
		    add_buf( buf1, "\n\r" );
	    }
	}
    }

    if ( !found )
    {
	stc( "{cObject(s) not found in this area.\n\r", ch);
	return FALSE;
    }

    if ( col % 3 != 0 )
	add_buf( buf1, "\n\r" );

    page_to_char( buf_string(buf1), ch );
    free_buf(buf1);
    return FALSE;
}



REDIT( redit_mshow )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  mshow <vnum>\n\r", ch );
	return FALSE;
    }

    if ( !is_number( argument ) )
    {
       stc( "{cREdit: Enter a VNUM to edit.\n\r", ch);
       return FALSE;
    }

    if ( is_number( argument ) )
    {
	value = atoi( argument );
	if ( !( pMob = get_mob_index( value ) ))
	{
	    stc( "{cREdit:  That mobile does not exist.\n\r", ch );
	    return FALSE;
	}

	ch->desc->pEdit = (void *)pMob;
    }

    medit_show( ch, argument );
    ch->desc->pEdit = (void *)ch->in_room;
    return FALSE;
}



REDIT( redit_oshow )
{
    OBJ_INDEX_DATA *pObj;
    int value;

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  oshow <vnum>\n\r", ch );
	return FALSE;
    }

    if ( !is_number( argument ) )
    {
       stc( "{cREdit: Number required.\n\r", ch);
       return FALSE;
    }

    if ( is_number( argument ) )
    {
	value = atoi( argument );
	if ( !( pObj = get_obj_index( value ) ))
	{
	    stc( "{cREdit:  That object does not exist.\n\r", ch );
	    return FALSE;
	}

	ch->desc->pEdit = (void *)pObj;
    }

    oedit_show( ch, argument );
    ch->desc->pEdit = (void *)ch->in_room;
    return FALSE;
}



/*****************************************************************************
 Name:		check_range( lower vnum, upper vnum )
 Purpose:	Ensures the range spans only one area.
 Called by:	aedit_vnum(olc_act.c).
 ****************************************************************************/
bool check_range( int lower, int upper )
{
    AREA_DATA *pArea;
    int cnt = 0;

    for ( pArea = area_first; pArea; pArea = pArea->next )
    {
        if ( ( lower <= pArea->min_vnum && pArea->min_vnum <= upper )
		||   ( lower <= pArea->max_vnum && pArea->max_vnum <= upper ) )
	    ++cnt;

	if ( cnt > 1 )
	    return FALSE;
    }
    return TRUE;
}



AREA_DATA *get_vnum_area( int vnum )
{
    AREA_DATA *pArea;

    for ( pArea = area_first; pArea; pArea = pArea->next )
    {
        if ( vnum >= pArea->min_vnum
          && vnum <= pArea->max_vnum )
            return pArea;
    }

    return 0;
}



/*
 * Area Editor Functions.
 */
AEDIT( aedit_show )
{
    AREA_DATA *pArea;

    EDIT_AREA(ch, pArea);

    ptc( ch, "{cName:     {C[{c%5d{C]{c %s\n\r", pArea->vnum, pArea->name );

#if 0  /* ROM OLC */
    ptc( ch, "{cRecall:   {C[{c%5d{C]{c %s\n\r", pArea->recall,
	get_room_index( pArea->recall )
	? get_room_index( pArea->recall )->name : "none" );
#endif /* ROM */

    ptc( ch, "{cFile:     %s\n\r", pArea->file_name );
    ptc( ch, "{cVnums:    {C[{c%d-%d{C]{c\n\r", pArea->min_vnum, pArea->max_vnum );
    ptc( ch, "{cAge:      {C[{c%d{C]{c\n\r",	pArea->age );
    ptc( ch, "{cPlayers:  {C[{c%d{C]{c\n\r", pArea->nplayer );
    ptc( ch, "{cSecurity: {C[{c%d{C]{c\n\r", pArea->security );
    ptc( ch, "{cBuilders: {C[{c%s{C]{c\n\r", pArea->builders );
    ptc( ch, "{cCredits : {C[{c%s{C]{c\n\r", pArea->credits );
    ptc( ch, "{cOwner     {C[{c%s{C]{c\n\r", pArea->owner );
    ptc( ch, "{cFlags:    {C[{c%s{C]{c\n\r", flag_string( area_flags, pArea->area_flags ) );
    return FALSE;
}



AEDIT( aedit_reset )
{
    AREA_DATA *pArea;

    EDIT_AREA(ch, pArea);

    reset_area( pArea );
    stc( "{cArea reset.\n\r", ch );

    return FALSE;
}



AEDIT( aedit_create )
{
    AREA_DATA *pArea;

    pArea               =   new_area();
    area_last->next     =   pArea;
    area_last		=   pArea;	/* Thanks, Walker. */
    ch->desc->pEdit     =   (void *)pArea;

    SET_BIT( pArea->area_flags, AREA_ADDED );
    stc( "{cArea Created.\n\r", ch );
    return FALSE;
}



AEDIT( aedit_name )
{
    AREA_DATA *pArea;

    EDIT_AREA(ch, pArea);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:   name {C[{c$name{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pArea->name );
    pArea->name = str_dup( argument );

    stc( "{cName set.\n\r", ch );
    return TRUE;
}

AEDIT( aedit_credits )
{
    AREA_DATA *pArea;

    EDIT_AREA(ch, pArea);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:   credits {C[{c$credits{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pArea->credits );
    pArea->credits = str_dup( argument );

    stc( "{cCredits set.\n\r", ch );
    return TRUE;
}


AEDIT( aedit_file )
{
    AREA_DATA *pArea;
    char file[MSL];
    int i, length;

    EDIT_AREA(ch, pArea);

    one_argument( argument, file );	/* Forces Lowercase */

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  filename {C[{c$file{C]{c\n\r", ch );
	return FALSE;
    }

    /*
     * Simple Syntax Check.
     */
    length = strlen( argument );
    if ( length > 15 )
    {
	stc( "{cNo more than eight characters allowed.\n\r", ch );
	return FALSE;
    }

    /*
     * Allow only letters and numbers.
     */
    for ( i = 0; i < length; i++ )
    {
	if ( !isalnum( file[i] ) )
	{
	    stc( "{cOnly letters and numbers are valid.\n\r", ch );
	    return FALSE;
	}
    }

    free_string( pArea->file_name );
    strcat( file, ".are" );
    pArea->file_name = str_dup( file );

    stc( "{cFilename set.\n\r", ch );
    return TRUE;
}



AEDIT( aedit_age )
{
    AREA_DATA *pArea;
    char age[MSL];

    EDIT_AREA(ch, pArea);

    one_argument( argument, age );

    if ( !is_number( age ) || age[0] == '\0' )
    {
	stc( "{cSyntax:  age {C[{c#xage{C]{c\n\r", ch );
	return FALSE;
    }

    pArea->age = atoi( age );

    stc( "{cAge set.\n\r", ch );
    return TRUE;
}


#if 0 /* ROM OLC */
AEDIT( aedit_recall )
{
    AREA_DATA *pArea;
    char room[MSL];
    int  value;

    EDIT_AREA(ch, pArea);

    one_argument( argument, room );

    if ( !is_number( argument ) || argument[0] == '\0' )
    {
	stc( "{cSyntax:  recall {C[{c#xrvnum{C]{c\n\r", ch );
	return FALSE;
    }

    value = atoi( room );

    if ( !get_room_index( value ) )
    {
	stc( "{cAEdit:  Room vnum does not exist.\n\r", ch );
	return FALSE;
    }

    pArea->recall = value;

    stc( "{cRecall set.\n\r", ch );
    return TRUE;
}
#endif /* ROM OLC */


AEDIT( aedit_security )
{
    AREA_DATA *pArea;
    char sec[MSL];
    int  value;

    EDIT_AREA(ch, pArea);

    one_argument( argument, sec );

    if ( !is_number( sec ) || sec[0] == '\0' )
    {
	stc( "{cSyntax:  security {C[{c#xlevel{C]{c\n\r", ch );
	return FALSE;
    }

    value = atoi( sec );

    if ( value > ch->pcdata->security || value < 0 )
    {
	if ( ch->pcdata->security != 0 )
	{
	    ptc( ch, "{cSecurity is 0-%d.\n\r", ch->pcdata->security );
	}
	else
	    stc( "{cSecurity is 0 only.\n\r", ch );
	return FALSE;
    }

    pArea->security = value;

    stc( "{cSecurity set.\n\r", ch );
    return TRUE;
}



AEDIT( aedit_builder )
{
    AREA_DATA *pArea;
    char name[MSL];
    char buf[MSL];

    EDIT_AREA(ch, pArea);

    one_argument( argument, name );

    if ( name[0] == '\0' )
    {
	stc( "{cSyntax:  builder {C[{c$name{C]{c  -toggles builder\n\r", ch );
	stc( "{cSyntax:  builder All      -allows everyone\n\r", ch );
	return FALSE;
    }

    name[0] = UPPER( name[0] );

    if ( strstr( pArea->builders, name ) != '\0' )
    {
	pArea->builders = string_replace( pArea->builders, name, "\0" );
	pArea->builders = string_unpad( pArea->builders );

	if ( pArea->builders[0] == '\0' )
	{
	    free_string( pArea->builders );
	    pArea->builders = str_dup( "None" );
	}
	stc( "{cBuilder removed.\n\r", ch );
	return TRUE;
    }
    else
    {
	buf[0] = '\0';
	if ( strstr( pArea->builders, "None" ) != '\0' )
	{
	    pArea->builders = string_replace( pArea->builders, "None", "\0" );
	    pArea->builders = string_unpad( pArea->builders );
	}

	if (pArea->builders[0] != '\0' )
	{
	    strcat( buf, pArea->builders );
	    strcat( buf, " " );
	}
	strcat( buf, name );
	free_string( pArea->builders );
	pArea->builders = string_proper( str_dup( buf ) );

	stc( "{cBuilder added.\n\r", ch );
	stc( pArea->builders,ch);
	return TRUE;
    }

    return FALSE;
}



AEDIT( aedit_vnum )
{
    AREA_DATA *pArea;
    char lower[MSL];
    char upper[MSL];
    int  ilower;
    int  iupper;

    EDIT_AREA(ch, pArea);

    argument = one_argument( argument, lower );
    one_argument( argument, upper );

    if ( !is_number( lower ) || lower[0] == '\0'
    || !is_number( upper ) || upper[0] == '\0' )
    {
	stc( "{cSyntax:  vnum {C[{c#xlower{C] [{c#xupper{C]{c\n\r", ch );
	return FALSE;
    }

    if ( ( ilower = atoi( lower ) ) > ( iupper = atoi( upper ) ) )
    {
	stc( "{cAEdit:  Upper must be larger then lower.\n\r", ch );
	return FALSE;
    }

    if ( !check_range( atoi( lower ), atoi( upper ) ) )
    {
	stc( "{cAEdit:  Range must include only this area.\n\r", ch );
	return FALSE;
    }

    if ( get_vnum_area( ilower )
    && get_vnum_area( ilower ) != pArea )
    {
	stc( "{cAEdit:  Lower vnum already assigned.\n\r", ch );
	return FALSE;
    }

    pArea->min_vnum = ilower;
    stc( "{cLower vnum set.\n\r", ch );

    if ( get_vnum_area( iupper )
    && get_vnum_area( iupper ) != pArea )
    {
	stc( "{cAEdit:  Upper vnum already assigned.\n\r", ch );
	return TRUE;	/* The lower value has been set. */
    }

    pArea->max_vnum = iupper;
    stc( "{cUpper vnum set.\n\r", ch );

    return TRUE;
}



AEDIT( aedit_lvnum )
{
    AREA_DATA *pArea;
    char lower[MSL];
    int  ilower;
    int  iupper;

    EDIT_AREA(ch, pArea);

    one_argument( argument, lower );

    if ( !is_number( lower ) || lower[0] == '\0' )
    {
	stc( "{cSyntax:  min_vnum {C[{c#xlower{C]{c\n\r", ch );
	return FALSE;
    }

    if ( ( ilower = atoi( lower ) ) > ( iupper = pArea->max_vnum ) )
    {
	stc( "{cAEdit:  Value must be less than the max_vnum.\n\r", ch );
	return FALSE;
    }

    if ( !check_range( ilower, iupper ) )
    {
	stc( "{cAEdit:  Range must include only this area.\n\r", ch );
	return FALSE;
    }

    if ( get_vnum_area( ilower )
    && get_vnum_area( ilower ) != pArea )
    {
	stc( "{cAEdit:  Lower vnum already assigned.\n\r", ch );
	return FALSE;
    }

    pArea->min_vnum = ilower;
    stc( "{cLower vnum set.\n\r", ch );
    return TRUE;
}



AEDIT( aedit_uvnum )
{
    AREA_DATA *pArea;
    char upper[MSL];
    int  ilower;
    int  iupper;

    EDIT_AREA(ch, pArea);

    one_argument( argument, upper );

    if ( !is_number( upper ) || upper[0] == '\0' )
    {
	stc( "{cSyntax:  max_vnum {C[{c#xupper{C]{c\n\r", ch );
	return FALSE;
    }

    if ( ( ilower = pArea->min_vnum ) > ( iupper = atoi( upper ) ) )
    {
	stc( "{cAEdit:  Upper must be larger then lower.\n\r", ch );
	return FALSE;
    }

    if ( !check_range( ilower, iupper ) )
    {
	stc( "{cAEdit:  Range must include only this area.\n\r", ch );
	return FALSE;
    }

    if ( get_vnum_area( iupper )
    && get_vnum_area( iupper ) != pArea )
    {
	stc( "{cAEdit:  Upper vnum already assigned.\n\r", ch );
	return FALSE;
    }

    pArea->max_vnum = iupper;
    stc( "{cUpper vnum set.\n\r", ch );

    return TRUE;
}



/*
 * Room Editor Functions.
 */
REDIT( redit_show )
{
    ROOM_INDEX_DATA	*pRoom;
    char		buf  [MSL];
    char		buf1 [2*MAX_STRING_LENGTH];
    OBJ_DATA		*obj;
    CHAR_DATA		*rch;
	 PROG_LIST		*list;
    int			door;
    bool		fcnt;

    EDIT_ROOM(ch, pRoom);

    buf1[0] = '\0';

    sprintf( buf, "{cDescription:\n\r%s", pRoom->description );
    strcat( buf1, buf );

    sprintf( buf, "{cName:       {C[{c%s{C]{c\n\rArea:       {C[{c%5d{C]{c %s\n\r",
	    pRoom->name, pRoom->area->vnum, pRoom->area->name );
    strcat( buf1, buf );

    sprintf( buf, "{cVnum:       {C[{c%5ld{C]{c\n\rSector:     {C[{c%s{C]{c\n\r",
	    pRoom->vnum, flag_string( sector_flags, pRoom->sector_type ) );
    strcat( buf1, buf );

    sprintf( buf, "{cRoom flags: {C[{c%s{C]{c\n\r",
	    flag_string( room_flags, pRoom->room_flags ) );
    strcat( buf1, buf );

    if ( pRoom->heal_rate != 100 || pRoom->mana_rate != 100 )
    {
	sprintf( buf, "{cHealth rec: {C[{c%d{C]{c\n\rMana rec  : {C[{c%d{C]{c\n\r",
		pRoom->heal_rate , pRoom->mana_rate );
	strcat( buf1, buf );
    }

	if ( IS_SET( pRoom->room_flags, ROOM_CLIMB ) )
	{
		sprintf( buf, "Room Grade: [Steepness %d]\n\r", pRoom->grade );
		strcat( buf1, buf );
	}

    if ( pRoom->clan > 0 )
    {
	sprintf( buf, "{cClan      : {C[{c%d{C]{c %s\n\r",
		pRoom->clan,
		clan_table[pRoom->clan].name );
	strcat( buf1, buf );
    }

    if ( !IS_NULLSTR(pRoom->owner) )
    {
	sprintf( buf, "{cOwner     : {C[{c%s{C]{c\n\r", pRoom->owner );
	strcat( buf1, buf );
    }
/*
    if ( !IS_NULLSTR(pRoom->url) )
	{
	sprintf( buf, "{cUrl     : {C[{c%s{C]{c\n\r", pRoom->url );
	strcat( buf1, buf );
	}
*/

    if ( pRoom->extra_descr )
    {
	EXTRA_DESCR_DATA *ed;

	strcat( buf1, "Desc Kwds:  {C[{c" );
	for ( ed = pRoom->extra_descr; ed; ed = ed->next )
	{
	    strcat( buf1, ed->keyword );
	    if ( ed->next )
		strcat( buf1, " " );
	}
	strcat( buf1, "]\n\r" );
    }

    strcat( buf1, "Characters: {C[{c" );
    fcnt = FALSE;
    for ( rch = pRoom->people; rch; rch = rch->next_in_room )
    {
	one_argument( rch->name, buf );
	strcat( buf1, buf );
	strcat( buf1, " " );
	fcnt = TRUE;
    }

    if ( fcnt )
    {
	int end;

	end = strlen(buf1) - 1;
	buf1[end] = ']';
	strcat( buf1, "\n\r" );
    }
    else
	strcat( buf1, "none{C]{c\n\r" );

    strcat( buf1, "Objects:    {C[{c" );
    fcnt = FALSE;
    for ( obj = pRoom->contents; obj; obj = obj->next_content )
    {
	one_argument( obj->name, buf );
	strcat( buf1, buf );
	strcat( buf1, " " );
	fcnt = TRUE;
    }

    if ( fcnt )
    {
	int end;

	end = strlen(buf1) - 1;
	buf1[end] = ']';
	strcat( buf1, "\n\r" );
    }
    else
	strcat( buf1, "none{C]{c\n\r" );

    for ( door = 0; door < MAX_DIR; door++ )
    {
	EXIT_DATA *pexit;

	if ( ( pexit = pRoom->exit[door] ) )
	{
	    char word[MIL];
	    char reset_state[MSL];
	    char *state;
	    int i, length;

	    sprintf( buf, "{c-%-5s to {C[{c%5ld{C]{c Key: {C[{c%5ld{C]{c ",
		capitalize(dir_name[door]),
		pexit->u1.to_room ? pexit->u1.to_room->vnum : 0,      /* ROM OLC */
		pexit->key );
	    strcat( buf1, buf );

	    /*
	     * Format up the exit info.
	     * Capitalize all flags that are not part of the reset info.
	     */
	    strcpy( reset_state, flag_string( exit_flags, pexit->rs_flags ) );
	    state = flag_string( exit_flags, pexit->exit_info );
	    strcat( buf1, " {cExit flags: {C[{c" );
	    for (; ;)
	    {
		state = one_argument( state, word );

		if ( word[0] == '\0' )
		{
		    int end;

		    end = strlen(buf1) - 1;
		    buf1[end] = ']';
		    strcat( buf1, "\n\r" );
		    break;
		}

		if ( str_infix( word, reset_state ) )
		{
		    length = strlen(word);
		    for (i = 0; i < length; i++)
			word[i] = UPPER(word[i]);
		}
		strcat( buf1, word );
		strcat( buf1, " " );
	    }

	    if ( pexit->keyword && pexit->keyword[0] != '\0' )
	    {
		sprintf( buf, "{cKwds: {C[{c%s{C]{c\n\r", pexit->keyword );
		strcat( buf1, buf );
	    }
	    if ( pexit->description && pexit->description[0] != '\0' )
	    {
		sprintf( buf, "{c%s", pexit->description );
		strcat( buf1, buf );
	    }
	}
    }

    stc( buf1, ch );
    if ( pRoom->rprogs )
    {
	int cnt;

	sprintf(buf, "\n\rROOMPrograms for [%5ld]:\n\r", pRoom->vnum);
	send_to_char( buf, ch );

	for (cnt=0, list=pRoom->rprogs; list; list=list->next)
	{
		if (cnt ==0)
		{
			send_to_char ( " Number Vnum Trigger Phrase\n\r", ch );
			send_to_char ( " ------ ---- ------- ------\n\r", ch );
		}

		sprintf(buf, "[%5d] %4ld %7s %s\n\r", cnt,
			list->vnum,prog_type_to_name(list->trig_type),
			list->trig_phrase);
		send_to_char( buf, ch );
		cnt++;
	}
    }
    return FALSE;
}


REDIT( redit_clear )
{
	ROOM_INDEX_DATA *pToRoom;
    ROOM_INDEX_DATA *pRoom;
    EXTRA_DESCR_DATA *ed;
	sh_int rev;
	int door;

    EDIT_ROOM(ch, pRoom);

	/*
	 * Kill the room name and desc
	 */
	free_string( pRoom->description );
	pRoom->description = str_dup( "" );

	free_string( pRoom->name );
	pRoom->name = str_dup( "" );

	for ( ed = pRoom->extra_descr; ed; ed = ed->next )
	{
		if ( ed != NULL )
			free_extra_descr( ed );
	}

	/*
	 * Kill the doors and reverse doors
	 */
	for ( door = 0; door < MAX_DIR; door++ )
	{
		if ( pRoom->exit[door] )
		{
			rev = rev_dir[door];
			pToRoom = pRoom->exit[door]->u1.to_room;

			/*
			 * Remove any linked exits.
			 */
			if ( pToRoom->exit[rev] )
			{
			    free_exit( pToRoom->exit[rev] );
			    pToRoom->exit[rev] = NULL;
			}

			/*
			 * Remove this exit.
			 */
			free_exit( pRoom->exit[door] );
			pRoom->exit[door] = NULL;

			ptc( ch, "{wExit %s unlinked.\n\r", dir_name[door] );
		}
	}
	return FALSE;
}

/* Local function. */
bool change_exit( CHAR_DATA *ch, char *argument, int door )
{
    ROOM_INDEX_DATA *pRoom;
    char command[MIL];
    char arg[MIL];
    int  value;

    EDIT_ROOM(ch, pRoom);

    /*
     * Set the exit flags, needs full argument.
     * ----------------------------------------
     */
    if ( ( value = flag_value( exit_flags, argument ) ) != NO_FLAG )
    {
	ROOM_INDEX_DATA *pToRoom;
	sh_int rev;                                    /* ROM OLC */

	if ( !pRoom->exit[door] )
	{
		stc("{cExit does not exist.\n\r",ch);
		return FALSE;
	}

	/*
	 * This room.
	 */
	TOGGLE_BIT(pRoom->exit[door]->rs_flags,  value);
	/* Don't toggle exit_info because it can be changed by players. */
	pRoom->exit[door]->exit_info = pRoom->exit[door]->rs_flags;

	/*
	 * Connected room.
	 */
	pToRoom = pRoom->exit[door]->u1.to_room;     /* ROM OLC */
	rev = rev_dir[door];

	if (pToRoom->exit[rev] != NULL)
	{
		pToRoom->exit[rev]->rs_flags = pRoom->exit[door]->rs_flags;
		pToRoom->exit[rev]->exit_info = pRoom->exit[door]->exit_info;
	}

	stc( "{cExit flag toggled.\n\r", ch );
	return TRUE;
    }

    /*
     * Now parse the arguments.
     */
    argument = one_argument( argument, command );
    one_argument( argument, arg );

    if ( command[0] == '\0' && argument[0] == '\0' )	/* Move command. */
    {
	move_char( ch, door, TRUE );                    /* ROM OLC */
	return FALSE;
    }

    if ( command[0] == '?' )
    {
	do_help( ch, "EXIT" );
	return FALSE;
    }

    if ( !str_cmp( command, "delete" ) )
    {
	ROOM_INDEX_DATA *pToRoom;
	sh_int rev;                                     /* ROM OLC */

	if ( !pRoom->exit[door] )
	{
	    stc( "{cREdit:  Cannot delete a null exit.\n\r", ch );
	    return FALSE;
	}

	/*
	 * Remove ToRoom Exit.
	 */
	rev = rev_dir[door];
	pToRoom = pRoom->exit[door]->u1.to_room;       /* ROM OLC */

	if ( pToRoom->exit[rev] )
	{
	    free_exit( pToRoom->exit[rev] );
	    pToRoom->exit[rev] = NULL;
	}

	/*
	 * Remove this exit.
	 */
	free_exit( pRoom->exit[door] );
	pRoom->exit[door] = NULL;

	stc( "{cExit unlinked.\n\r", ch );
	return TRUE;
    }

    if ( !str_cmp( command, "link" ) )
    {
	EXIT_DATA *pExit;
	ROOM_INDEX_DATA *toRoom;

	if ( arg[0] == '\0' || !is_number( arg ) )
	{
	    stc( "{cSyntax:  {C[{cdirection{C]{c link {C[{cvnum{C]{c\n\r", ch );
	    return FALSE;
	}

	value = atoi( arg );

	if ( ! (toRoom = get_room_index( value )) )
	{
	    stc( "{cREdit:  Cannot link to non-existant room.\n\r", ch );
	    return FALSE;
	}

	if ( !IS_BUILDER( ch, toRoom->area ) )
	{
	    stc( "{cREdit:  Cannot link to an area you cannot build in.\n\r", ch );
	    return FALSE;
	}

	if ( toRoom->exit[rev_dir[door]] )
	{
	    stc( "REdit:  Remote sides exit already exists.\n\r", ch );
	    return FALSE;
	}

	if ( !pRoom->exit[door] )
		pRoom->exit[door] = new_exit();

	pRoom->exit[door]->u1.to_room = toRoom;
	pRoom->exit[door]->orig_door = door;

	door                    = rev_dir[door];
	pExit                   = new_exit();
	pExit->u1.to_room       = pRoom;
	pExit->orig_door	= door;
	toRoom->exit[door]       = pExit;

	stc( "{cTwo-way link established.\n\r", ch );
	return TRUE;
    }

    if ( !str_cmp( command, "dig" ) )
    {
	char buf[MSL];

	if ( arg[0] == '\0' || !is_number( arg ) )
	{
	    stc( "{cSyntax: {C[{cdirection{C]{c dig <vnum>\n\r", ch );
	    return FALSE;
	}

	redit_create( ch, arg );
	sprintf( buf, "link %s", arg );
	change_exit( ch, buf, door);
	return TRUE;
    }

    if ( !str_cmp( command, "room" ) )
    {
	ROOM_INDEX_DATA *toRoom;

	if ( arg[0] == '\0' || !is_number( arg ) )
	{
	    stc( "{cSyntax:  {C[{cdirection{C]{c room {C[{cvnum{C]{c\n\r", ch );
	    return FALSE;
	}

	value = atoi( arg );

	if ( !(toRoom = get_room_index( value )) )
	{
	    stc( "{cREdit:  Cannot link to non-existant room.\n\r", ch );
	    return FALSE;
	}

	if ( !pRoom->exit[door] )
		pRoom->exit[door] = new_exit();

	pRoom->exit[door]->u1.to_room = toRoom;    /* ROM OLC */
	pRoom->exit[door]->orig_door = door;

	stc( "{cOne-way link established.\n\r", ch );
	return TRUE;
    }

    if ( !str_cmp( command, "key" ) )
    {
	OBJ_INDEX_DATA *key;

	if ( arg[0] == '\0' || !is_number( arg ) )
	{
	    stc( "{cSyntax:  {C[{cdirection{C]{c key {C[{cvnum{C]{c\n\r", ch );
	    return FALSE;
	}

	if ( !pRoom->exit[door] )
	{
		stc("{cExit does not exist.\n\r",ch);
		return FALSE;
	}

	value = atoi( arg );

	if ( !(key = get_obj_index( value )) )
	{
	    stc( "{cREdit:  Key doesnt exist.\n\r", ch );
	    return FALSE;
	}

	if ( key->item_type != ITEM_KEY )
	{
	    stc( "{cREdit:  Object is not a key.\n\r", ch );
	    return FALSE;
	}

	pRoom->exit[door]->key = value;

	stc( "{cExit key set.\n\r", ch );
	return TRUE;
    }

    if ( !str_cmp( command, "name" ) )
    {
	if ( arg[0] == '\0' )
	{
	    stc( "{cSyntax:  {C[{cdirection{C]{c name {C[{cstring{C]{c\n\r", ch );
	    stc( "{c         {C[{cdirection{C]{c name none\n\r", ch );
	    return FALSE;
	}

	if ( !pRoom->exit[door] )
	   {
	   	stc("{cExit does not exist.\n\r",ch);
	   	return FALSE;
	   }

	free_string( pRoom->exit[door]->keyword );

	if (str_cmp(arg,"none"))
		pRoom->exit[door]->keyword = str_dup( arg );
	else
		pRoom->exit[door]->keyword = str_dup( "" );

	stc( "{cExit name set.\n\r", ch );
	return TRUE;
    }

    if ( !str_prefix( command, "description" ) )
    {
	if ( arg[0] == '\0' )
	{
	   if ( !pRoom->exit[door] )
	   {
	   	stc("{cExist does not exist.\n\r",ch);
	   	return FALSE;
	   }

	    string_append( ch, &pRoom->exit[door]->description );
	    return TRUE;
	}

	stc( "{cSyntax:  {C[{cdirection{C]{c desc\n\r", ch );
	return FALSE;
    }

    return FALSE;
}



REDIT( redit_north )
{
    if ( change_exit( ch, argument, DIR_NORTH ) )
	return TRUE;

    return FALSE;
}



REDIT( redit_south )
{
    if ( change_exit( ch, argument, DIR_SOUTH ) )
	return TRUE;

    return FALSE;
}



REDIT( redit_east )
{
    if ( change_exit( ch, argument, DIR_EAST ) )
	return TRUE;

    return FALSE;
}
/*
REDIT( redit_out )
{
    if ( change_exit( ch, argument, DIR_OUT ) )
	return TRUE;

    return FALSE;
}
*/

REDIT( redit_west )
{
    if ( change_exit( ch, argument, DIR_WEST ) )
	return TRUE;

    return FALSE;
}



REDIT( redit_up )
{
    if ( change_exit( ch, argument, DIR_UP ) )
	return TRUE;

    return FALSE;
}



REDIT( redit_down )
{
    if ( change_exit( ch, argument, DIR_DOWN ) )
	return TRUE;

    return FALSE;
}

REDIT( redit_northeast )
{
    if ( change_exit( ch, argument, DIR_NORTHEAST ) )
        return TRUE;

        return FALSE;
}


REDIT( redit_southeast )
{
    if ( change_exit( ch, argument, DIR_SOUTHEAST ) )
        return TRUE;

        return FALSE;
}


REDIT( redit_southwest )
{
    if ( change_exit( ch, argument, DIR_SOUTHWEST ) )
        return TRUE;

        return FALSE;
}


REDIT( redit_northwest )
{
    if ( change_exit( ch, argument, DIR_NORTHWEST ) )
        return TRUE;

        return FALSE;
}


REDIT( redit_ed )
{
    ROOM_INDEX_DATA *pRoom;
    EXTRA_DESCR_DATA *ed;
    char command[MIL];
    char keyword[MIL];

    EDIT_ROOM(ch, pRoom);

    argument = one_argument( argument, command );
    one_argument( argument, keyword );

    if ( command[0] == '\0' || keyword[0] == '\0' )
    {
	stc( "{cSyntax:  ed add {C[{ckeyword{C]{c\n\r", ch );
	stc( "{c         ed edit {C[{ckeyword{C]{c\n\r", ch );
	stc( "{c         ed delete {C[{ckeyword{C]{c\n\r", ch );
	stc( "{c         ed format {C[{ckeyword{C]{c\n\r", ch );
	return FALSE;
    }

    if ( !str_cmp( command, "add" ) )
    {
	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed add {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	ed			=   new_extra_descr();
	ed->keyword		=   str_dup( keyword );
	ed->description		=   str_dup( "" );
	ed->next		=   pRoom->extra_descr;
	pRoom->extra_descr	=   ed;

	string_append( ch, &ed->description );

	return TRUE;
    }


    if ( !str_cmp( command, "edit" ) )
    {
	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed edit {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	for ( ed = pRoom->extra_descr; ed; ed = ed->next )
	{
	    if ( is_name( keyword, ed->keyword ) )
		break;
	}

	if ( !ed )
	{
	    stc( "{cREdit:  Extra description keyword not found.\n\r", ch );
	    return FALSE;
	}

	string_append( ch, &ed->description );

	return TRUE;
    }


    if ( !str_cmp( command, "delete" ) )
    {
	EXTRA_DESCR_DATA *ped = NULL;

	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed delete {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	for ( ed = pRoom->extra_descr; ed; ed = ed->next )
	{
	    if ( is_name( keyword, ed->keyword ) )
		break;
	    ped = ed;
	}

	if ( !ed )
	{
	    stc( "{cREdit:  Extra description keyword not found.\n\r", ch );
	    return FALSE;
	}

	if ( !ped )
	    pRoom->extra_descr = ed->next;
	else
	    ped->next = ed->next;

	free_extra_descr( ed );

	stc( "{cExtra description deleted.\n\r", ch );
	return TRUE;
    }


    if ( !str_cmp( command, "format" ) )
    {
	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed format {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	for ( ed = pRoom->extra_descr; ed; ed = ed->next )
	{
	    if ( is_name( keyword, ed->keyword ) )
		break;
	}

	if ( !ed )
	{
	    stc( "{cREdit:  Extra description keyword not found.\n\r", ch );
	    return FALSE;
	}

	ed->description = format_string( ed->description );

	stc( "{cExtra description formatted.\n\r", ch );
	return TRUE;
    }

    redit_ed( ch, "" );
    return FALSE;
}



REDIT( redit_create )
{
    AREA_DATA *pArea;
    ROOM_INDEX_DATA *pRoom;
    int value;
    int iHash;

    EDIT_ROOM(ch, pRoom);

    value = atoi( argument );

    if ( argument[0] == '\0' || value <= 0 )
    {
	stc( "{cSyntax:  create {C[{cvnum > 0{C]{c\n\r", ch );
	return FALSE;
    }

    pArea = get_vnum_area( value );
    if ( !pArea )
    {
	stc( "{cREdit:  That vnum is not assigned an area.\n\r", ch );
	return FALSE;
    }

    if ( !IS_BUILDER( ch, pArea ) )
    {
	stc( "{cREdit:  Vnum in an area you cannot build in.\n\r", ch );
	return FALSE;
    }

    if ( get_room_index( value ) )
    {
	stc( "{cREdit:  Room vnum already exists.\n\r", ch );
	return FALSE;
    }

    pRoom			= new_room_index();
    pRoom->area			= pArea;
    pRoom->vnum			= value;

    if ( value > top_vnum_room )
        top_vnum_room = value;

    iHash			= value % MAX_KEY_HASH;
    pRoom->next			= room_index_hash[iHash];
    room_index_hash[iHash]	= pRoom;
    ch->desc->pEdit		= (void *)pRoom;

    stc( "{cRoom created.\n\r", ch );
    return TRUE;
}



REDIT( redit_name )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  name {C[{cname{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pRoom->name );
    pRoom->name = str_dup( argument );

    stc( "{cName set.\n\r", ch );
    return TRUE;
}



REDIT( redit_desc )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    if ( argument[0] == '\0' )
    {
	string_append( ch, &pRoom->description );
	return TRUE;
    }

    stc( "{cSyntax:  desc\n\r", ch );
    return FALSE;
}

REDIT( redit_grade )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    if (is_number(argument))
	{
	    if ( atoi(argument) > 10 || atoi(argument) < 1 )
	    {
	    stc("{cGrade cannot be greater than 10 or less than 0.\n\r",ch);
		return FALSE;
		}
        pRoom->grade = atoi ( argument );
        stc ( "{cGrade set.\n\r", ch);
        return TRUE;
    }

    stc ( "{cSyntax : grade <#xnumber>\n\r", ch);
    return FALSE;
}

REDIT( redit_heal )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    if (is_number(argument))
	{
	    if ( atoi(argument) > 250 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
	    {
	    stc("{cHeal cannot be greater than 250.\n\r",ch);
		return FALSE;
		}
        pRoom->heal_rate = atoi ( argument );
        stc ( "{cHeal rate set.\n\r", ch);
        return TRUE;
    }

    stc ( "{cSyntax : heal <#xnumber>\n\r", ch);
    return FALSE;
}

REDIT( redit_mana )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    if (is_number(argument))
    {
	    if ( atoi(argument) > 250 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP) )
	    {
	    stc("{cHeal cannot be greater than 250.\n\r",ch);
		return FALSE;
		}
        pRoom->mana_rate = atoi ( argument );
        stc ( "{cMana rate set.\n\r", ch);
        return TRUE;
	}
    stc ( "{cSyntax : mana <#xnumber>\n\r", ch);
    return FALSE;
}

REDIT( redit_clan )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    pRoom->clan = clan_lookup(argument);

    stc ( "{cClan set.\n\r", ch);
    return TRUE;
}

REDIT( redit_format )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    pRoom->description = format_string( pRoom->description );

    stc( "{cString formatted.\n\r", ch );
    return TRUE;
}



REDIT( redit_mreset )
{
    ROOM_INDEX_DATA	*pRoom;
    MOB_INDEX_DATA	*pMobIndex;
    CHAR_DATA		*newmob;
    char		arg [ MAX_INPUT_LENGTH ];
    char		arg2 [ MAX_INPUT_LENGTH ];

    RESET_DATA		*pReset;
    char		output [ MAX_STRING_LENGTH ];

    EDIT_ROOM(ch, pRoom);

    argument = one_argument( argument, arg );
    argument = one_argument( argument, arg2 );

    if ( arg[0] == '\0' || !is_number( arg ) )
    {
	stc ( "{cSyntax:  mreset <vnum> <max #x> <mix #x>\n\r", ch );
	return FALSE;
    }

    if ( !( pMobIndex = get_mob_index( atoi( arg ) ) ) )
    {
	stc( "{cREdit: No mobile has that vnum.\n\r", ch );
	return FALSE;
    }

    if ( pMobIndex->area != pRoom->area )
    {
	stc( "{cREdit: No such mobile in this area.\n\r", ch );
	return FALSE;
    }

    /*
     * Create the mobile reset.
     */
    pReset              = new_reset_data();
    pReset->command	= 'M';
    pReset->arg1	= pMobIndex->vnum;
    pReset->arg2	= is_number( arg2 ) ? atoi( arg2 ) : MAX_MOB;
    pReset->arg3	= pRoom->vnum;
    pReset->arg4	= is_number( argument ) ? atoi (argument) : 1;
    add_reset( pRoom, pReset, 0/* Last slot*/ );

    /*
     * Create the mobile.
     */
    newmob = create_mobile( pMobIndex );
    char_to_room( newmob, pRoom );

    sprintf( output, "{c%s (%ld) has been loaded and added to resets.\n\r"
	"There will be a maximum of %d loaded to this room.\n\r",
	capitalize( pMobIndex->short_descr ),
	pMobIndex->vnum,
	pReset->arg2 );
    stc( output, ch );
    act( "{c$n has created $N!", ch, NULL, newmob, TO_ROOM );
    return TRUE;
}



struct wear_type
{
    int	wear_loc;
    int	wear_bit;
};


/* WEAR FLAGS FOR RESETS */
const struct wear_type wear_table[] =
{
    {	WEAR_NONE,	ITEM_TAKE		},
    {	WEAR_LIGHT,	ITEM_LIGHT		},
    {	WEAR_FINGER_L,	ITEM_WEAR_FINGER	},
    {	WEAR_FINGER_R,	ITEM_WEAR_FINGER	},
    {	WEAR_NECK_1,	ITEM_WEAR_NECK		},
    {	WEAR_NECK_2,	ITEM_WEAR_NECK		},
    {	WEAR_BODY,	ITEM_WEAR_BODY		},
    {	WEAR_BACK,	ITEM_WEAR_BACK		},
    {	WEAR_HEAD,	ITEM_WEAR_HEAD		},
    {	WEAR_LEGS,	ITEM_WEAR_LEGS		},
    {	WEAR_FEET,	ITEM_WEAR_FEET		},
    {	WEAR_HANDS,	ITEM_WEAR_HANDS		},
    {	WEAR_ARMS,	ITEM_WEAR_ARMS		},
    {	WEAR_SHIELD,	ITEM_WEAR_SHIELD	},
    {	WEAR_ABOUT,	ITEM_WEAR_ABOUT		},
    {	WEAR_WAIST,	ITEM_WEAR_WAIST		},
    {	WEAR_WRIST_L,	ITEM_WEAR_WRIST		},
    {	WEAR_WRIST_R,	ITEM_WEAR_WRIST		},
    {	WEAR_WIELD,	ITEM_WIELD		},
    {	WEAR_HOLD,	ITEM_HOLD		},
    {	WEAR_ANGREAL,	ITEM_WEAR_ANGREAL		},
    {	WEAR_FACE,	ITEM_WEAR_FACE		},
    {	NO_FLAG,	NO_FLAG			}
};



/*****************************************************************************
 Name:		wear_loc
 Purpose:	Returns the location of the bit that matches the count.
 		1 = first match, 2 = second match etc.
 Called by:	oedit_reset(olc_act.c).
 ****************************************************************************/
int wear_loc(int bits, int count)
{
    int flag;

    for (flag = 0; wear_table[flag].wear_bit != NO_FLAG; flag++)
    {
        if ( IS_SET(bits, wear_table[flag].wear_bit) && --count < 1)
            return wear_table[flag].wear_loc;
    }

    return NO_FLAG;
}



/*****************************************************************************
 Name:		wear_bit
 Purpose:	Converts a wear_loc into a bit.
 Called by:	redit_oreset(olc_act.c).
 ****************************************************************************/
int wear_bit(int loc)
{
    int flag;

    for (flag = 0; wear_table[flag].wear_loc != NO_FLAG; flag++)
    {
        if ( loc == wear_table[flag].wear_loc )
            return wear_table[flag].wear_bit;
    }

    return 0;
}



REDIT( redit_oreset )
{
    ROOM_INDEX_DATA	*pRoom;
    OBJ_INDEX_DATA	*pObjIndex;
    OBJ_DATA		*newobj;
    OBJ_DATA		*to_obj;
    CHAR_DATA		*to_mob;
    char		arg1 [ MAX_INPUT_LENGTH ];
    char		arg2 [ MAX_INPUT_LENGTH ];
    int			olevel = 0;

    RESET_DATA		*pReset;

    EDIT_ROOM(ch, pRoom);

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || !is_number( arg1 ) )
    {
	stc ( "{cSyntax:  oreset <vnum> <args>\n\r", ch );
	stc ( "{c        -no_args               = into room\n\r", ch );
	stc ( "{c        -<obj_name>            = into obj\n\r", ch );
	stc ( "{c        -<mob_name> <wear_loc> = into mob\n\r", ch );
	return FALSE;
    }

    if ( !( pObjIndex = get_obj_index( atoi( arg1 ) ) ) )
    {
	stc( "{cREdit: No object has that vnum.\n\r", ch );
	return FALSE;
    }

    if ( pObjIndex->area != pRoom->area )
    {
	stc( "{cREdit: No such object in this area.\n\r", ch );
	return FALSE;
    }

    /*
     * Load into room.
     */
    if ( arg2[0] == '\0' )
    {
	pReset		= new_reset_data();
	pReset->command	= 'O';
	pReset->arg1	= pObjIndex->vnum;
	pReset->arg2	= 0;
	pReset->arg3	= pRoom->vnum;
	pReset->arg4	= 0;
	add_reset( pRoom, pReset, 0/* Last slot*/ );

	newobj = create_object( pObjIndex, number_fuzzy( olevel ) );
	obj_to_room( newobj, pRoom );

	ptc( ch, "{c%s (%d) has been loaded and added to resets.\n\r",
	    capitalize( pObjIndex->short_descr ),
	    pObjIndex->vnum );
    }
    else
    /*
     * Load into object's inventory.
     */
    if ( argument[0] == '\0'
    && ( ( to_obj = get_obj_list( ch, arg2, pRoom->contents ) ) != NULL ) )
    {
	pReset		= new_reset_data();
	pReset->command	= 'P';
	pReset->arg1	= pObjIndex->vnum;
	pReset->arg2	= 0;
	pReset->arg3	= to_obj->pIndexData->vnum;
	pReset->arg4	= 1;
	add_reset( pRoom, pReset, 0/* Last slot*/ );

	newobj = create_object( pObjIndex, number_fuzzy( olevel ) );
	newobj->cost = 0;
	obj_to_obj( newobj, to_obj );

	ptc( ch, "{c%s (%d) has been loaded into "
	    "{c%s (%d) and added to resets.\n\r",
	    capitalize( newobj->short_descr ),
	    newobj->pIndexData->vnum,
	    to_obj->short_descr,
	    to_obj->pIndexData->vnum );
    }
    else
    /*
     * Load into mobile's inventory.
     */
    if ( ( to_mob = get_char_room( ch,NULL, arg2 ) ) != NULL )
    {
	int	wear_loc;

	/*
	 * Make sure the location on mobile is valid.
	 */
	if ( (wear_loc = flag_value( wear_loc_flags, argument )) == NO_FLAG )
	{
	    stc( "{cREdit: Invalid wear_loc.  '? wear-loc'\n\r", ch );
	    return FALSE;
	}

	/*
	 * Disallow loading a sword(WEAR_WIELD) into WEAR_HEAD.
	 */
	if ( !IS_SET( pObjIndex->wear_flags, wear_bit(wear_loc) ) )
	{
	    ptc( ch,
	        "{c%s (%d) has wear flags: {C[{c%s{C]{c\n\r",
	        capitalize( pObjIndex->short_descr ),
	        pObjIndex->vnum,
		flag_string( wear_flags, pObjIndex->wear_flags ) );
	    return FALSE;
	}

	/*
	 * Can't load into same position.
	 */
	if ( get_eq_char( to_mob, wear_loc ) )
	{
	    stc( "{cREdit:  Object already equipped.\n\r", ch );
	    return FALSE;
	}

	pReset		= new_reset_data();
	pReset->arg1	= pObjIndex->vnum;
	pReset->arg2	= wear_loc;
	if ( pReset->arg2 == WEAR_NONE )
	    pReset->command = 'G';
	else
	    pReset->command = 'E';
	pReset->arg3	= wear_loc;

	add_reset( pRoom, pReset, 0/* Last slot*/ );

	olevel  = URANGE( 0, to_mob->level - 2, LEVEL_HERO );
        newobj = create_object( pObjIndex, number_fuzzy( olevel ) );

	if ( to_mob->pIndexData->pShop )	/* Shop-keeper? */
	{
	    switch ( pObjIndex->item_type )
	    {
	    default:		olevel = 0;				break;
	    case ITEM_PILL:	olevel = number_range(  0, 10 );	break;
	    case ITEM_POTION:	olevel = number_range(  0, 10 );	break;
	    case ITEM_SCROLL:	olevel = number_range(  5, 15 );	break;
	    case ITEM_WAND:	olevel = number_range( 10, 20 );	break;
	    case ITEM_STAFF:	olevel = number_range( 15, 25 );	break;
	    case ITEM_ARMOR:	olevel = number_range(  5, 15 );	break;
	    case ITEM_WEAPON:	if ( pReset->command == 'G' )
	    			    olevel = number_range( 5, 15 );
				else
				    olevel = number_fuzzy( olevel );
		break;
	    }

	    newobj = create_object( pObjIndex, olevel );
	    if ( pReset->arg2 == WEAR_NONE )
		SET_BIT( newobj->extra_flags, ITEM_INVENTORY );
	}
	else
	    newobj = create_object( pObjIndex, number_fuzzy( olevel ) );

	obj_to_char( newobj, to_mob );
	if ( pReset->command == 'E' )
	    equip_char( to_mob, newobj, pReset->arg3 );

	ptc( ch, "{c%s (%d) has been loaded "
	    "{c%s of %s (%d) and added to resets.\n\r",
	    capitalize( pObjIndex->short_descr ),
	    pObjIndex->vnum,
	    flag_string( wear_loc_strings, pReset->arg3 ),
	    to_mob->short_descr,
	    to_mob->pIndexData->vnum );
    }
    else	/* Display Syntax */
    {
	stc( "{cREdit:  That mobile isnt here.\n\r", ch );
	return FALSE;
    }

    act( "{c$n has created $p!", ch, newobj, NULL, TO_ROOM );
    return TRUE;
}



/*
 * Object Editor Functions.
 * VALUES
 */
void show_obj_values( CHAR_DATA *ch, OBJ_INDEX_DATA *obj )
{
    switch( obj->item_type )
    {
	default:	/* No values. */
	    break;

    case ITEM_FIREOIL:
    	ptc( ch, "[v0] Timer           [%d]\n\r", obj->value[0] );
	    ptc( ch, "[v2] Damage          [%d]\n\r", obj->value[2] );
    break;

    case ITEM_COMM_DEV:
    	ptc( ch, "[v0] Charges         [%d]\n\r", obj->value[0] );
    break;

	case ITEM_LIGHT:
        if ( obj->value[2] == -1 || obj->value[2] == 999 ) /* ROM OLC */
			ptc( ch, "{C[{Cv2] Light:  Infinite[-1]\n\r" );
        else
			ptc( ch, "{C[{Cv2] Light:  {C[{c%d{C]{c\n\r", obj->value[2] );
	    break;

	case ITEM_JOURNAL:
		stc("No values need setting.\n\r",ch);
		break;

	case ITEM_CLOTH_BOLT:
		ptc( ch, "{C[{cv0{C]{c Cloth Length:	     %d\n\r", obj->value[0] );
		ptc( ch, "{C[{cv1{C]{c Color:		     %s\n\r", cloth_colors_table[obj->value[1]].bname );
		ptc( ch, "{C[{cv2{C]{c Not Used		       \n\r" );
		ptc( ch, "{C[{cv3{C]{c Step:		     %d\n\r", obj->value[3] );
		ptc( ch, "{C[{cv4{C]{c Layout Type:	     %d\n\r", obj->value[4] );
		break;
	case ITEM_ANGREAL_MALE:
	case ITEM_ANGREAL_FEMALE:
		ptc( ch, "{C[{Cv1] Power:  {C[{c%d{C]{c\n\r", obj->value[1] );
	    break;

	case ITEM_TREE_PC:
		ptc( ch, "{C[{Cv1] Climb?:  {C[{c%d{C]{c\n\r", obj->value[0] );
	    break;

	case ITEM_PIPE_TABAC:
	case ITEM_CHEW_TABAC:
	    ptc( ch, "{C[{cv0{C]{c Tabac Total: {C[{c%d{C]{c\n\r", obj->value[0] );
	    ptc( ch, "{C[{cv1{C]{c Tabac Left:  {C[{c%d{C]{c\n\r", obj->value[1] );
	    ptc( ch, "{C[{cv2{C]{c Tabac Lit:   {C[{c%d{C]{c\n\r", obj->value[2] );
	    ptc( ch, "{C[{cv1{C]{c Tabac Burn:  {C[{c%d{C]{c\n\r", obj->value[3] );
	    break;

	case ITEM_QUIVER:
		ptc( ch, "{C[{cv0{C]{c Number of Arrows:     %d\n\r", obj->value[0] );
		ptc( ch, "{C[{cv1{C]{c Number of Dice:       %d\n\r", obj->value[1] );
		ptc( ch, "{C[{cv2{C]{c Type of Dice:         %d\n\r", obj->value[2] );
		ptc( ch, "{C[{cv3{C]{c Spell:                %s\n\r",
		obj->value[3] != -1 ? skill_table[obj->value[3]].name : "none" );
		break;

	case ITEM_ARROW:
		ptc( ch, "{C[{cv1{C]{c Number of Dice:       %d\n\r", obj->value[1] );
		ptc( ch, "{C[{cv2{C]{c Type of Dice:         %d\n\r", obj->value[2] );
		ptc( ch, "{C[{cv3{C]{c Spell:                %s\n\r",
		obj->value[3] != -1 ? skill_table[obj->value[3]].name : "none" );
		break;

	case ITEM_WAND:
	case ITEM_STAFF:
        ptc( ch,
		"{C[{cv0{C]{c Level:          {C[{c%d{C]{c\n\r"
		"{C[{cv1{C]{c Charges Total:  {C[{c%d{C]{c\n\r"
		"{C[{cv2{C]{c Charges Left:   {C[{c%d{C]{c\n\r"
		"{C[{cv3{C]{c Spell:          %s\n\r",
		obj->value[0],
		obj->value[1],
		obj->value[2],
		obj->value[3] != -1 ? skill_table[obj->value[3]].name : "none" );
	    break;

	case ITEM_WINDOW:
	    ptc( ch,
	        "{C[{cv0{C]{c Views:        {C[{c%d{C]{c\n\r",
	        obj->value[0] );
	    break;

	case ITEM_PORTAL:
	    ptc( ch,
	        "{C[{cv0{C]{c Charges:        {C[{c%d{C]{c\n\r"
	        "{C[{cv1{C]{c Exit Flags:     %s\n\r"
	        "{C[{cv2{C]{c Portal Flags:   %s\n\r"
	        "{C[{cv3{C]{c Goes to (vnum): {C[{c%d{C]{c\n\r"
		"{C[{cv4{C]{c Visible?        {C[{c%s{C]{c\n\r",
	        obj->value[0],
	        flag_string( exit_flags, obj->value[1]),
	        flag_string( portal_flags , obj->value[2]),
	        obj->value[3], obj->value[4] == 1 ? "Yes" : "No" );
	    break;
/*
	case ITEM_WINDOW:
		ptc( ch,
	        "{C[{cv0{C]{c Room View:        {C[{c%d{C]{c\n\r",
	        obj->value[0] );
	    break;
*/
	case ITEM_TRIGGER:
		ptc( ch,
			"{C[{cv0{C]{c Turn/Push/Pull?  {C[{c%s{C]{c\n\r"
			"{C[{cv1{C]{c Trigger Action?  {C[{c%s{C]{c\n\r"
			"{C[{cv2{C]{c Trigger Result?  {C[{c%s{C]{c\n\r",
			flag_string( trigger_flags, obj->value[0] ),
			flag_string( trigger_action_flags, obj->value[1] ),
			dir_name[obj->value[2]] );
		break;

	case ITEM_FURNITURE:
	    ptc( ch,
	        "{C[{cv0{C]{c Max people:      {C[{c%d{C]{c\n\r"
	        "{C[{cv1{C]{c Max weight:      {C[{c%d{C]{c\n\r"
	        "{C[{cv2{C]{c Furniture Flags: %s\n\r"
	        "{C[{cv3{C]{c Heal bonus:      {C[{c%d{C]{c\n\r"
	        "{C[{cv4{C]{c Mana bonus:      {C[{c%d{C]{c\n\r",
	        obj->value[0],
	        obj->value[1],
	        flag_string( furniture_flags, obj->value[2]),
	        obj->value[3],
	        obj->value[4] );
	    break;

	case ITEM_SCROLL:
	case ITEM_POTION:
	case ITEM_PILL:
        ptc( ch,
		"{C[{cv0{C]{c Level:  {C[{c%d{C]{c\n\r"
		"{C[{cv1{C]{c Spell:  %s\n\r"
		"{C[{cv2{C]{c Spell:  %s\n\r"
		"{C[{cv3{C]{c Spell:  %s\n\r"
		"{C[{cv4{C]{c Spell:  %s\n\r",
		obj->value[0],
		obj->value[1] != -1 ? skill_table[obj->value[1]].name
		                    : "none",
		obj->value[2] != -1 ? skill_table[obj->value[2]].name
                                    : "none",
		obj->value[3] != -1 ? skill_table[obj->value[3]].name
		                    : "none",
		obj->value[4] != -1 ? skill_table[obj->value[4]].name
		                    : "none" );
	    break;

/* ARMOR for ROM */

        case ITEM_ARMOR:
	    ptc( ch,
		"{C[{cv0{C]{c Ac pierce       {C[{c%d{C]{c\n\r"
		"{C[{cv1{C]{c Ac bash         {C[{c%d{C]{c\n\r"
		"{C[{cv2{C]{c Ac slash        {C[{c%d{C]{c\n\r"
		"{C[{cv3{C]{c Ac exotic       {C[{c%d{C]{c\n\r",
		obj->value[0],
		obj->value[1],
		obj->value[2],
		obj->value[3] );
	    break;

/* WEAPON changed in ROM: */
/* I had to split the output here, I have no idea why, but it helped -- Hugin */
/* It somehow fixed a bug in showing scroll/pill/potions too ?! */
	case ITEM_WEAPON:
        ptc(ch, "{c[{cv0] Weapon class:   %s\n\r", flag_string( weapon_class, obj->value[0] ) );
	    ptc(ch, "{c[{cv1] Number of dice: {C[{c%d{C]{c\n\r", obj->value[1] );
	    ptc(ch, "{c[{cv2] Type of dice:   {C[{c%d{C]{c\n\r", obj->value[2] );
	    ptc(ch, "{c[{cv3] Type:           %s\n\r", attack_table[obj->value[3]].name );
 	    ptc(ch, "{c[{cv4] Special type:   %s\n\r", flag_string( weapon_type2,  obj->value[4] ) );
	    break;

	case ITEM_CONTAINER:
	    ptc(ch,
		"{C[{cv0{C]{c Weight:     [%d kg]\n\r"
		"{C[{cv1{C]{c Flags:      {C[{c%s{C]{c\n\r"
		"{C[{cv2{C]{c Key:     %s {C[{c%d{C]{c\n\r"
		"{C[{cv3{C]{c Capacity    {C[{c%d{C]{c\n\r"
		"{C[{cv4{C]{c Weight Mult {C[{c%d{C]{c\n\r",
		obj->value[0],
		flag_string( container_flags, obj->value[1] ),
                get_obj_index(obj->value[2])
                    ? get_obj_index(obj->value[2])->short_descr
                    : "none",
                obj->value[2],
                obj->value[3],
                obj->value[4] );
	    break;

	case ITEM_TREASURECHEST:
	    ptc(ch,
		"{C[{cv0{C]{c Silver:       {C[{c%d{C]{c\n\r"
		"{C[{cv1{C]{c Flags:        {C[{c%s{C]{c\n\r"
		"{C[{cv2{C]{c Lock Diff: %s {C[{c%d{C]{c\n\r"
		"{C[{cv3{C]{c Trap Type     {C[{c%d{C]{c\n\r"
		"{C[{cv4{C]{c Trap Dmg      {C[{c%d{C]{c\n\r",
		obj->value[0],
		flag_string( container_flags, obj->value[1] ),
		obj->value[2],
		obj->value[3],
		obj->value[4] );
	    break;

	case ITEM_KEYRING:
		ptc( ch, "No vars to set.\n\r" );
	    break;

	case ITEM_LOCKPICK:
	    ptc(ch,
		"{C[{cv0{C]{c Strength:     {C[{c%d{C] {c1-100\n\r",
		obj->value[0]);
	    break;

	case ITEM_RAZOR:
	    ptc(ch,
		"{C[{cv0{C]{c Sharpness:    {C[{c%d{C] {c1-100\n\r",
		obj->value[0]);
	    break;

	case ITEM_BRUSH:
	    ptc(ch,
		"{C[{cv0{C]{c Bristles:    {C[{c%d{C] {c1-100\n\r",
		obj->value[0]);
	    break;

	case ITEM_DRINK_CON:
	    ptc(ch,
	        "{C[{cv0{C]{c Liquid Total: {C[{c%d{C]{c\n\r"
	        "{C[{cv1{C]{c Liquid Left:  {C[{c%d{C]{c\n\r"
	        "{C[{cv2{C]{c Liquid:       %s\n\r"
	        "{C[{cv3{C]{c Poisoned:     %s\n\r",
	        obj->value[0],
	        obj->value[1],
	        liq_table[obj->value[2]].liq_name,
	        obj->value[3] != 0 ? "Yes" : "No" );
	    break;

	case ITEM_FOUNTAIN:
	    ptc(ch,
	        "{C[{cv0{C]{c Liquid Total: {C[{c%d{C]{c\n\r"
	        "{C[{cv1{C]{c Liquid Left:  {C[{c%d{C]{c\n\r"
	        "{C[{cv2{C]{c Liquid:	    %s\n\r",
	        obj->value[0],
	        obj->value[1],
	        liq_table[obj->value[2]].liq_name );
	    break;

	case ITEM_FOOD:
	    ptc(ch,
		"{C[{cv0{C]{c Food hours: {C[{c%d{C]{c\n\r"
		"{C[{cv1{C]{c Full hours: {C[{c%d{C]{c\n\r"
		"{C[{cv3{C]{c Poisoned:   %s\n\r",
		obj->value[0],
		obj->value[1],
		obj->value[3] != 0 ? "Yes" : "No" );
	    break;

	case ITEM_MONEY:
        ptc(ch, "{c[{cv0] Gold Mark:    {C[{c%d{C]{c\n\r", obj->value[0] );
	    break;
    }

    return;
}



bool set_obj_values( CHAR_DATA *ch, OBJ_INDEX_DATA *pObj, int value_num, char *argument)
{
	char buf[MSL];
	int i;
	bool found = FALSE;
    switch( pObj->item_type )
    {
    default:
    break;

	    case ITEM_FIREOIL:
    	switch ( value_num )
	    {
	    default:
	    do_help( ch, "ITEM_FIREOIL" );
    	return FALSE;
	    case 0:
    		stc( "TIMER SET.\n\r\n\r", ch);
		    pObj->value[0] = atoi( argument );
	    break;
    	case 2:
	        if ( atoi(argument) > pObj->level && ch->level < MAX_LEVEL )
	        {
	    	    stc("{cOil damage cannot be greater than the oil level.\n\r",ch);
        		return FALSE;
			}
            stc( "DAMAGE SET.\n\r\n\r", ch );
            pObj->value[2] = atoi( argument );
            break;
    	}
		break;

	    case ITEM_COMM_DEV:
    	switch ( value_num )
	    {
	    default:
		    do_help( ch, "ITEM_COMM_DEV" );
    		return FALSE;
	    case 0:
	    	if ( atoi( argument ) > 50 )
	    	{
	    	stc("Devices may have a maximum of 50 charges.\n\r",ch);
	    	return FALSE;
	    	}
    		stc( "CHARGES SET.\n\r\n\r", ch);
		    pObj->value[0] = atoi( argument );
		    if ( pObj->value[0] > 50 )
		    	pObj->value[0] = 50;
	    	break;
    	}
		break;

		case ITEM_JOURNAL:
		stc("No values need setting.\n\r",ch);
		break;

        case ITEM_LIGHT:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_LIGHT" );
	            return FALSE;
	        case 2:
	            stc( "{cHOURS OF LIGHT SET.\n\r\n\r", ch );
	            pObj->value[2] = atoi( argument );
	            break;
	    }
        break;
	case ITEM_CLOTH_BOLT:
		switch ( value_num )
		{
			default:
				do_help( ch, "ITEM_CLOTH_BOLT" );
				return FALSE;
			case 0:
				stc( "{cCloth length set.\n\r\n\r", ch );
				pObj->value[0] = atoi( argument );
				if ( pObj->value[0] < 1 || pObj->value[0] > 8)
				{
					stc( "{cCloth can not be smaller then an eigth of a full length (1) nor larger then a full length (8).\n\r", ch );
					pObj->value[0] = 8;
					return FALSE;
				}
				break;
			case 1:
			 	for ( i = 0; cloth_colors_table[i].aname != NULL; i++ )
				{
				        if ( !str_cmp( argument, cloth_colors_table[i].cname ) )
					{
						stc( "{cCloth color set.\n\r", ch );
						pObj->value[1] = cloth_colors_table[i].number;
						found = TRUE;
						break;
					}
				}
				if (!found)
				{
				        stc( "{cInvalid cloth color. Set to default of black.\n\r", ch );
				        pObj->value[1] = 0;
					return FALSE;
			    	}
				break;
			case 2:
			case 3:
			case 4:
				stc( "Don't touch that!\n\r\n\r", ch );
				pObj->value[2] = 0;
				pObj->value[3] = 0;
				pObj->value[4] = 0;
				break;
		}
		break;
	case ITEM_ANGREAL_MALE:
        case ITEM_ANGREAL_FEMALE:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_ANGREAL" );
	            return FALSE;
	        case 1:
	            stc( "{cPOWER MULTIPLIER SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            if ( pObj->value[1] > 5 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
	            {
	    	    stc("{cAngreal strength cannot be greater than 5.\n\r",ch);
        		return FALSE;
				}
	            pObj->value[1] = 5;
	            break;
	    }
            break;

        case ITEM_TREE_PC:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_TREE_PC" );
	            return FALSE;
	        case 1:
	            stc( "{cCLIMB SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            if (pObj->value[0] > 1)
	                pObj->value[0] = 1;
	            if (pObj->value[0] < 0)
	                pObj->value[0] = 0;
	            break;
	    }
            break;

        case ITEM_PIPE_TABAC:
        case ITEM_CHEW_TABAC:
	    switch ( value_num )
	    {
	        default:
	        stc("You only need to set v0 and v1 for this item type.\n\r",ch);
		    do_help( ch, "ITEM_PIPE" );
	            return FALSE;
	        case 0:
	            stc( "{cMAXIMUM AMOUT OF TABAC HOURS SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	        case 1:
	            stc( "{cCURRENT AMOUNT OF TABAC HOURS SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	    }
            break;

        case ITEM_WAND:
        case ITEM_STAFF:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_STAFF_WAND" );
	            return FALSE;
	        case 0:
				if ( pObj->level > atoi(argument) )
				{
					stc("Spell level cannot be greater than obj level.\n\r",ch);
					return FALSE;
				}
	            stc( "{cSPELL LEVEL SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	        case 1:
	            stc( "{cTOTAL NUMBER OF CHARGES SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 2:
	            stc( "{cCURRENT NUMBER OF CHARGES SET.\n\r\n\r", ch );
	            pObj->value[2] = atoi( argument );
	            break;
	        case 3:
	            stc( "{cSPELL TYPE SET.\n\r", ch );
	            pObj->value[3] = skill_lookup( argument );
	            break;
	    }
            break;

	case ITEM_QUIVER:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_QUIVER" );
	            return FALSE;
	        case 0:
	            stc( "{cNUMBER OF ARROWS SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	        case 1:
	            stc( "{cNUMBER OF DICE SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 2:
	            stc( "{cTYPE OF DICE SET.\n\r\n\r", ch );
	            pObj->value[2] = atoi( argument );
	            break;
	    }
            break;

		case ITEM_ARROW:
        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP) )
        {
        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
       	return FALSE;
		}
		if ( !is_wood( pObj ) )
        {
        stc("{cInvalid material type for an arrow.\n{cMaterial must be wood.\n",ch);
       	return FALSE;
		}
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_ARROW" );
	            return FALSE;

	        case 1:
	            stc( "{cNUMBER OF DICE SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 2:
	            stc( "{cTYPE OF DICE SET.\n\r\n\r", ch );
	            pObj->value[2] = atoi( argument );
	            break;
	    }
        break;

        case ITEM_SCROLL:
        case ITEM_POTION:
        case ITEM_PILL:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_SCROLL_POTION_PILL" );
	            return FALSE;
	        case 0:
			if ( atoi(argument) > 150
			&& !is_exact_name(ch->name,MUD_OWNER)
			&& !is_exact_name(ch->name,MUD_COIMP))
			{
				stc("{cSpell level cannot be greater than 150.\n\r",ch);
				return FALSE;
			}
			if ( pObj->level > atoi(argument) )
			{
				stc("Spell level cannot be greater than obj level.\n\r",ch);
				return FALSE;
			}
            stc( "{cSPELL LEVEL SET.\n\r\n\r", ch);
            pObj->value[0] = atoi ( argument );
			break;
	        case 1:
	            stc( "{cSPELL TYPE 1 SET.\n\r\n\r", ch );
	            pObj->value[1] = skill_lookup( argument );
	            break;
	        case 2:
	            stc( "{cSPELL TYPE 2 SET.\n\r\n\r", ch );
	            pObj->value[2] = skill_lookup( argument );
	            break;
	        case 3:
				stc("Cannot have more than two weaves.\n\r",ch);
				break;
				/*
	            stc( "{cSPELL TYPE 3 SET.\n\r\n\r", ch );
	            pObj->value[3] = skill_lookup( argument );
	            break;
	            */
	        case 4:
				stc("Cannot have more than two weaves.\n\r",ch);
				break;
				/*
	            stc( "{cSPELL TYPE 4 SET.\n\r\n\r", ch );
	            pObj->value[4] = skill_lookup( argument );
	            break;
	            */
 	    }
	    break;

/* ARMOR for ROM: */

        case ITEM_ARMOR:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_ARMOR" );
		    return FALSE;
	        case 0:
	        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP) )
	        {
	        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
        	return FALSE;
			}
		    stc( "{cAC PIERCE SET.\n\r\n\r", ch );
		    pObj->value[0] = atoi( argument );
		    break;
	        case 1:
	        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP))
	        {
	        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
        	return FALSE;
			}
		    stc( "{cAC BASH SET.\n\r\n\r", ch );
		    pObj->value[1] = atoi( argument );
		    break;
	        case 2:
	        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP))
	        {
	        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
        	return FALSE;
			}
		    stc( "{cAC SLASH SET.\n\r\n\r", ch );
		    pObj->value[2] = atoi( argument );
		    break;
	        case 3:
	        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP))
	        {
	        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
        	return FALSE;
			}
		    stc( "{cAC EXOTIC SET.\n\r\n\r", ch );
		    pObj->value[3] = atoi( argument );
		    break;
	    }
	    break;

/* WEAPONS changed in ROM */

        case ITEM_WEAPON:
	    switch ( value_num )
	    {
	        default:
	        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
            return FALSE;
	        case 0:
	        if ( pObj->value[0] == WEAPON_ASHANDAREI && !is_exact_name( ch->name, MUD_OWNER ) )
	        {
				stc( "This type of weapon cannot be made.\n\r", ch );
				break;
			}
		    stc( "{cWEAPON CLASS SET.\n\r\n\r", ch );
		    ALT_FLAGVALUE_SET( pObj->value[0], weapon_class, argument );
		    sprintf(buf, "%d", pObj->level);
		    oedit_level(ch, buf);
		    break;
	        case 1:
		        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP))
		        {
		        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
	        	return FALSE;
				}
	            stc( "{cNUMBER OF DICE SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 2:
		        if ( !is_exact_name(ch->name, MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP))
		        {
		        stc("{cManual editting for this item type is not supported.\n{cSet the level to produce default values.\n",ch);
	        	return FALSE;
				}
	            stc( "{cTYPE OF DICE SET.\n\r\n\r", ch );
	            pObj->value[2] = atoi( argument );
	            break;
	        case 3:
	            stc( "{cWEAPON TYPE SET.\n\r\n\r", ch );
	            pObj->value[3] = attack_lookup( argument );
	            break;
	        case 4:
                stc( "{cSPECIAL WEAPON TYPE TOGGLED.\n\r\n\r", ch );
		    ALT_FLAGVALUE_TOGGLE( pObj->value[4], weapon_type2, argument );
		    break;
	    }
            break;

	case ITEM_WINDOW:
	    switch ( value_num )
	    {
	        default:
	            do_help(ch, "ITEM_PORTAL" );
	            return FALSE;

	    	case 0:
	    	    stc( "{cVIEW SET.\n\r\n\r", ch);
	    	    pObj->value[0] = atoi ( argument );
	    	    break;
		}
		break;

	case ITEM_PORTAL:
	    switch ( value_num )
	    {
	        default:
	            do_help(ch, "ITEM_PORTAL" );
	            return FALSE;

	    	case 0:
	    	    stc( "{cCHARGES SET.\n\r\n\r", ch);
	    	    pObj->value[0] = atoi ( argument );
	    	    break;
	    	case 1:
	    	    stc( "{cEXIT FLAGS SET.\n\r\n\r", ch);
		    ALT_FLAGVALUE_SET( pObj->value[1], exit_flags, argument );
	    	    break;
	    	case 2:
	    	    stc( "{cPORTAL FLAGS SET.\n\r\n\r", ch);
		    ALT_FLAGVALUE_SET( pObj->value[2], portal_flags, argument );
	    	    break;
	    	case 3:
	    	    stc( "{cEXIT VNUM SET.\n\r\n\r", ch);
	    	    pObj->value[3] = atoi ( argument );
	    	    break;
		case 4:
			if ( !str_prefix( argument, "Visible" ) || !str_cmp( argument, "Yes" ) )
			{
				stc("Visibility set.\n\r", ch);
				pObj->value[4] = 1;
			}
			else
			{
				stc("Visibility removed.\n\r", ch);
				pObj->value[4] = 0;
			}
			break;
	   }
	   break;

	case ITEM_TRIGGER:
		switch ( value_num )
		{
			default:
				for ( i = 0; i <= 9; i++ )
				{
					ptc( ch, "%d = %s\n\r", i, dir_name[i] );
				}
				do_help( ch, "ITEM_TRIGGER" );
				return FALSE;
			case 0:
				ALT_FLAGVALUE_TOGGLE(pObj->value[0], trigger_flags, argument);
			break;
			case 1:
				ALT_FLAGVALUE_TOGGLE(pObj->value[1], trigger_action_flags, argument);
			break;
			case 2:
	    	    stc( "{cDIRECTION SET.\n\r\n\r", ch);
				pObj->value[2] = atoi( argument );
			break;
		}
		break;

	case ITEM_FURNITURE:
	    switch ( value_num )
	    {
	        default:
	            do_help( ch, "ITEM_FURNITURE" );
	            return FALSE;

	        case 0:
	            stc( "{cNUMBER OF PEOPLE SET.\n\r\n\r", ch);
	            pObj->value[0] = atoi ( argument );
	            break;
	        case 1:
	            stc( "{cMAX WEIGHT SET.\n\r\n\r", ch);
	            pObj->value[1] = atoi ( argument );
	            break;
	        case 2:
	            stc( "{cFURNITURE FLAGS TOGGLED.\n\r\n\r", ch);
		    ALT_FLAGVALUE_TOGGLE( pObj->value[2], furniture_flags, argument );
	            break;
	        case 3:
			    if ( atoi(argument) > 200 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
			    {
			    stc("{cHeal cannot be greater than 200.\n\r",ch);
				return FALSE;
				}
	            stc( "{cHEAL BONUS SET.\n\r\n\r", ch);
	            pObj->value[3] = atoi ( argument );
	            break;
	        case 4:
			    if ( atoi(argument) > 200 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
			    {
			    stc("{cMana cannot be greater than 200.\n\r",ch);
				return FALSE;
				}
	            stc( "{cMANA BONUS SET.\n\r\n\r", ch);
	            pObj->value[4] = atoi ( argument );
	            break;
	    }
	    break;

        case ITEM_CONTAINER:
	    switch ( value_num )
	    {
		int value;

		default:
		    do_help( ch, "ITEM_CONTAINER" );
	            return FALSE;
		case 0:
	            stc( "{cWEIGHT CAPACITY SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
		case 1:
	            if ( ( value = flag_value( container_flags, argument ) ) != NO_FLAG )
			TOGGLE_BIT(pObj->value[1], value);
		    else
		    {
			do_help ( ch, "ITEM_CONTAINER" );
			return FALSE;
		    }
	            stc( "{cCONTAINER TYPE SET.\n\r\n\r", ch );
	            break;
		case 2:
		    if ( atoi(argument) != 0 )
		    {
			if ( !get_obj_index( atoi( argument ) ) )
			{
			    stc( "{cTHERE IS NO SUCH ITEM.\n\r\n\r", ch );
			    return FALSE;
			}

			if ( get_obj_index( atoi( argument ) )->item_type != ITEM_KEY )
			{
			    stc( "{cTHAT ITEM IS NOT A KEY.\n\r\n\r", ch );
			    return FALSE;
			}
		    }
		    stc( "{cCONTAINER KEY SET.\n\r\n\r", ch );
		    pObj->value[2] = atoi( argument );
		    break;
		case 3:
		    stc( "{cCONTAINER MAX WEIGHT SET.\n\r", ch);
		    pObj->value[3] = atoi( argument );
		    break;
		case 4:
		    stc( "{cWEIGHT MULTIPLIER SET.\n\r\n\r", ch );
		    pObj->value[4] = atoi ( argument );
		    break;
	    }
	    break;

	case ITEM_DRINK_CON:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_DRINK" );
/* OLC		    do_help( ch, "liquids" );    */
	            return FALSE;
	        case 0:
	            stc( "{cMAXIMUM AMOUT OF LIQUID HOURS SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	        case 1:
	            stc( "{cCURRENT AMOUNT OF LIQUID HOURS SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 2:
	            stc( "{cLIQUID TYPE SET.\n\r\n\r", ch );
	            pObj->value[2] = ( liq_lookup(argument) != -1 ?
	            		       liq_lookup(argument) : 0 );
	            break;
	        case 3:
	            stc( "{cPOISON VALUE TOGGLED.\n\r\n\r", ch );
	            pObj->value[3] = ( pObj->value[3] == 0 ) ? 1 : 0;
	            break;
	    }
            break;

	case ITEM_FOUNTAIN:
	    switch (value_num)
	    {
	    	default:
		    do_help( ch, "ITEM_FOUNTAIN" );
/* OLC		    do_help( ch, "liquids" );    */
	            return FALSE;
	        case 0:
	            stc( "{cMAXIMUM AMOUT OF LIQUID HOURS SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	        case 1:
	            stc( "{cCURRENT AMOUNT OF LIQUID HOURS SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 2:
	            stc( "{cLIQUID TYPE SET.\n\r\n\r", ch );
	            pObj->value[2] = ( liq_lookup( argument ) != -1 ?
	            		       liq_lookup( argument ) : 0 );
	            break;
            }
	break;

	case ITEM_FOOD:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_FOOD" );
	            return FALSE;
	        case 0:
	            stc( "{cHOURS OF FOOD SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	        case 1:
	            stc( "{cHOURS OF FULL SET.\n\r\n\r", ch );
	            pObj->value[1] = atoi( argument );
	            break;
	        case 3:
	            stc( "{cPOISON VALUE TOGGLED.\n\r\n\r", ch );
	            pObj->value[3] = ( pObj->value[3] == 0 ) ? 1 : 0;
	            break;
	    }
            break;

	case ITEM_LOCKPICK:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_LOCKPICK" );
	            return FALSE;
	        case 0:
	        	if ( atoi(argument) > 100 && !is_exact_name(ch->name,MUD_OWNER)&& !is_exact_name(ch->name,MUD_COIMP) )
	        	{
					stc("Valid vlaues are between 1 and 100.\n\r",ch);
					return FALSE;
				}
	            stc( "{cLOCKPICK STRENGTH SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	    }
        break;

	case ITEM_RAZOR:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_RAZOR" );
	            return FALSE;
	        case 0:
	        	if ( atoi(argument) > 100 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
	        	{
					stc("Valid values are between 1 and 100.\n\r",ch);
					return FALSE;
				}
	            stc( "{cRAZOR SHARPNESS SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	    }
        break;

	case ITEM_BRUSH:
	    switch ( value_num )
	    {
	        default:
		    do_help( ch, "ITEM_BRUSH" );
	            return FALSE;
	        case 0:
	        	if ( atoi(argument) > 100 && !is_exact_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
	        	{
					stc("Valid values are between 1 and 100.\n\r",ch);
					return FALSE;
				}
	            stc( "{cBRUSH BRISTLES SET.\n\r\n\r", ch );
	            pObj->value[0] = atoi( argument );
	            break;
	    }
        break;

	case ITEM_MONEY:
	    switch ( value_num )
	    {
        default:
		    do_help( ch, "ITEM_MONEY" );
            return FALSE;
        case 0:
            stc( "{cGOLD AMOUNT SET.\n\r\n\r", ch );
            pObj->value[0] = atoi( argument );
            break;
		case 1:
		    stc( "{cSILVER AMOUNT SET.\n\r\n\r", ch );
		    pObj->value[1] = atoi( argument );
		    break;
	    }
        break;
    }

    show_obj_values( ch, pObj );

    return TRUE;
}


OEDIT( oedit_delete )
{
	OBJ_DATA *obj, *obj_next;
	OBJ_INDEX_DATA *pObj;
	RESET_DATA *pReset, *wReset;
	ROOM_INDEX_DATA *pRoom;
	char arg[MIL];
	char buf[MSL];
	int index, rcount, ocount, i, iHash;

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  oedit delete [vnum]\n\r", ch );
		return FALSE;
	}

	one_argument( argument, arg );

	if( is_number( arg ) )
	{
		index = atoi( arg );
		pObj = get_obj_index( index );
	}
	else
	{
		send_to_char( "That is not a number.\n\r", ch );
		return FALSE;
	}

	if( !pObj )
	{
		send_to_char( "No such object.\n\r", ch );
		return FALSE;
	}

	SET_BIT( pObj->area->area_flags, AREA_CHANGED );

	if( top_vnum_obj == index )
		for( i = 1; i < index; i++ )
			if( get_obj_index( i ) )
				top_vnum_obj = i;


	top_obj_index--;

	/* remove objects */
	ocount = 0;
	for( obj = object_list; obj; obj = obj_next )
	{
		obj_next = obj->next;

		if( obj->pIndexData == pObj )
		{
			extract_obj( obj );
			ocount++;
		}
	}

	/* crush resets */
	rcount = 0;
	for( iHash = 0; iHash < MAX_KEY_HASH; iHash++ )
	{
		for( pRoom = room_index_hash[iHash]; pRoom; pRoom = pRoom->next )
		{
			for( pReset = pRoom->reset_first; pReset; pReset = wReset )
			{
				wReset = pReset->next;
				switch( pReset->command )
				{
				case 'O':
				case 'E':
				case 'P':
				case 'G':
					if( ( pReset->arg1 == index ) ||
						( ( pReset->command == 'P' ) && (
					pReset->arg3 == index ) ) )
					{
						unlink_reset( pRoom, pReset );
						free_reset_data( pReset );

						rcount++;
						SET_BIT( pRoom->area->area_flags,
						AREA_CHANGED );

					}
				}
			}
		}
	}

	unlink_obj_index( pObj );

	pObj->area = NULL;
	pObj->vnum = 0;

	free_obj_index( pObj );

	sprintf( buf, "Removed object vnum %d and"
		" %d resets.\n\r", index,rcount );

	send_to_char( buf, ch );

	sprintf( buf, "%d occurences of the object"
		" were extracted from the mud.\n\r", ocount );

	send_to_char( buf, ch );
	edit_done(ch);
	return TRUE;
}

OEDIT( oedit_show )
{
    OBJ_INDEX_DATA *pObj;
    AFFECT_DATA *paf;
	PROG_LIST			*list;
    int cnt;

    EDIT_OBJ(ch, pObj);

    ptc(ch, "{cName:        {C[{c%s{C]{c\n\rArea:        {C[{c%5d{C]{c %s\n\r",
	pObj->name,
	!pObj->area ? -1        : pObj->area->vnum,
	!pObj->area ? "No Area" : pObj->area->name );

    ptc(ch, "{cVnum:        {C[{c%5d{C]{c\n\rType:        {C[{c%s{C]{c\n\r",
	pObj->vnum,
	flag_string( type_flags, pObj->item_type ) );

    ptc(ch, "{cLevel:       {C[{c%5d{C]{c\n\r", pObj->level );

    ptc(ch, "{cWear flags:  {C[{c%s{C]{c\n\r",
	flag_string( wear_flags, pObj->wear_flags ) );

    ptc(ch, "{cExtra flags: {C[{c%s{C]{c\n\r",
	flag_string( extra_flags, pObj->extra_flags ) );

    ptc(ch, "{cGuild:       {C[{c%s{C]{c\n\r",
	pObj->guild );

    ptc(ch, "{cMaterial:    {C[{c%s{C]{c\n\r",
	pObj->material );

    ptc(ch, "{cCondition:   {C[{c%5d{C]{c\n\r",               /* ROM */
	pObj->condition );

    ptc(ch, "{cWeight:      {C[{c%5d{C]{c\n\rCost:        {C[{c%5d{C]{c\n\r",
	pObj->weight, pObj->cost );

    if ( pObj->extra_descr )
    {
	EXTRA_DESCR_DATA *ed;

	stc( "{cEx desc kwd: ", ch );

	for ( ed = pObj->extra_descr; ed; ed = ed->next )
	{
	    stc( "{C[{c", ch );
	    stc( ed->keyword, ch );
	    stc( "{C]{c", ch );
	}

	stc( "{c\n\r", ch );
    }

    ptc(ch, "{cShort desc:{C  %s{c\n\r{cLong desc:{C\n\r     %s{c\n\r",
	pObj->short_descr, pObj->description );

    for ( cnt = 0, paf = pObj->affected; paf; paf = paf->next )
    {
	if ( cnt == 0 )
	{
	    stc( "{cNumber Modifier Affects\n\r", ch );
	    stc( "{w------ -------- -------{c\n\r", ch );
	}
	ptc(ch, "{C[{c%4d{C]{c %-8d %s\n\r", cnt,
	    paf->modifier,
	    flag_string( apply_flags, paf->location ) );
	cnt++;
    }

    show_obj_values( ch, pObj );

    if ( pObj->oprogs )
    {
		int cnt;
		ptc(ch, "\n\rOBJPrograms for [%5d]:\n\r", pObj->vnum);
		for (cnt=0, list=pObj->oprogs; list; list=list->next)
		{
			if (cnt ==0)
			{
				stc( " Number Vnum Trigger Phrase\n\r", ch );
				stc( " ------ ---- ------- ------\n\r", ch );
			}
			ptc(ch, "[%5d] %4d %7s %s\n\r",
			    cnt,
				list->vnum,prog_type_to_name(list->trig_type),
				list->trig_phrase);
			cnt++;
		}
    }
    return FALSE;
}


/*
 * Need to issue warning if flag isn't valid. -- does so now -- Hugin.
 * Mod to prevent builders from loading up addaffects on objs
 * Mod to prevent more than one addaffect from being added
 */
OEDIT( oedit_addaffect )
{
    int value, mat = 75;
    OBJ_INDEX_DATA *pObj;
    AFFECT_DATA *pAf;
    char loc[MSL];
    char mod[MSL];
    bool found = FALSE;

    EDIT_OBJ(ch, pObj);

    argument = one_argument( argument, loc );
    one_argument( argument, mod );

    if ( loc[0] == '\0' || mod[0] == '\0' || !is_number( mod ))
    {
		stc( "{cSyntax:  addaffect {C[{clocation{C]{c {C[{c#xmod{C]{c\n\r", ch );
		return FALSE;
    }
    if (atoi(mod) < 0 && ch->level < MAX_LEVEL -1)
    {
		stc("Only positive numbers are allowed.\n\r",ch);
		return FALSE;
	}

	/* check for a valid argument */
    if ( ( value = flag_value( apply_flags, loc ) ) == NO_FLAG ) /* Hugin */
    {
        stc( "{cValid affects are:\n\r", ch );
		show_help( ch, "apply" );
		return FALSE;
    }
    /* check for an existing affect */
    if (pObj->affected !=NULL && ch->level < MAX_LEVEL)
    {
		stc("{cObject already posseses affect flag.\n\r",ch);
		return FALSE;
	}
	/* check for level vs strength of affect */
	/* catch anything that might have slipped through */
/*	for ( mat = 0; mat < max-1; mat++ ) */
	for ( mat = 0; material_table[mat].name != NULL; mat++ )
	{
		if ( !str_cmp( pObj->material, material_table[mat].name ) )
		{
			found = TRUE;
			break;
		}
	}
	if ( !found )
    {
		stc("Could not find a valid material. Set the material and retry.\n\r",ch);
		return FALSE;
	}

    if ( atoi(mod) > material_table[mat].maxAff )
    {
		ptc( ch,"{cValue is too high for this object type.\n\rValue must not be greater than %d.\n\r",
			material_table[mat].maxAff );
		return FALSE;
	}
	/* add the affect */
    pAf             =   new_affect();
    pAf->location   =   value;
    pAf->modifier   =   atoi( mod );
    pAf->where	    =   TO_OBJECT;
    pAf->type       =   -1;
    pAf->duration   =   -1;
    pAf->bitvector  =   0;
    pAf->level      =	pObj->level;
    pAf->next       =   pObj->affected;
    pObj->affected  =   pAf;
    pObj->cost      =   atoi(mod) * pObj->cost;

    stc( "{cAffect added.\n\r", ch);
    return TRUE;
}

OEDIT( oedit_addapply )
{
    int value,bv,typ;
    OBJ_INDEX_DATA *pObj;
    AFFECT_DATA *pAf;
    char loc[MSL];
    char mod[MSL];
    char type[MSL];
    char bvector[MSL];

    EDIT_OBJ(ch, pObj);

    argument = one_argument( argument, type );
    argument = one_argument( argument, loc );
    argument = one_argument( argument, mod );
    one_argument( argument, bvector );

    if ( type[0] == '\0' || ( typ = flag_value( apply_types, type ) ) == NO_FLAG )
    {
  	stc( "{cInvalid apply type. Valid apply types are:\n\r", ch);
   	show_help( ch, "apptype" );
   	return FALSE;
    }

    if ( loc[0] == '\0' || ( value = flag_value( apply_flags, loc ) ) == NO_FLAG )
    {
    stc( "{cValid applys are:\n\r", ch );
	show_help( ch, "apply" );
	return FALSE;
    }

    if ( bvector[0] == '\0' || ( bv = flag_value( bitvector_type[typ].table, bvector ) ) == NO_FLAG )
    {
  	stc( "{cInvalid bitvector type.\n\r", ch );
	stc( "{cValid bitvector types are:\n\r", ch );
	show_help( ch, bitvector_type[typ].help );
    	return FALSE;
    }

    if ( mod[0] == '\0' || !is_number( mod ) )
    {
	stc( "{cSyntax:  addapply {C[{ctype{C]{c {C[{clocation{C]{c {C[{c$xmod{C]{c {C[{cbitvector{C]{c\n\r", ch );
	return FALSE;
    }

    pAf             =   new_affect();
    pAf->location   =   value;
    pAf->modifier   =   atoi( mod );
    pAf->where	    =   apply_types[typ].bit;
    pAf->type	    =	-1;
    pAf->duration   =   -1;
    pAf->bitvector  =   bv;
    pAf->level      =	pObj->level;
    pAf->next       =   pObj->affected;
    pObj->affected  =   pAf;

    stc( "{cApply added.\n\r", ch);
    return TRUE;
}

/*
 * My thanks to Hans Hvidsten Birkeland and Noam Krendel(Walker)
 * for really teaching me how to manipulate pointers.
 */
OEDIT( oedit_delaffect )
{
    OBJ_INDEX_DATA *pObj;
    AFFECT_DATA *pAf;
    AFFECT_DATA *pAf_next;
    char affect[MSL];
    int  value;
    int  cnt = 0;

    EDIT_OBJ(ch, pObj);

    one_argument( argument, affect );

    if ( !is_number( affect ) || affect[0] == '\0' )
    {
	stc( "{cSyntax:  delaffect {C[{c#xaffect{C]{c\n\r", ch );
	return FALSE;
    }

    value = atoi( affect );

    if ( value < 0 )
    {
	stc( "{cOnly non-negative affect-numbers allowed.\n\r", ch );
	return FALSE;
    }

    if ( !( pAf = pObj->affected ) )
    {
	stc( "{cOEdit:  Non-existant affect.\n\r", ch );
	return FALSE;
    }

    if( value == 0 )	/* First case: Remove first affect */
    {
	pAf = pObj->affected;
	pObj->affected = pAf->next;
	free_affect( pAf );
    }
    else		/* Affect to remove is not the first */
    {
	while ( ( pAf_next = pAf->next ) && ( ++cnt < value ) )
	     pAf = pAf_next;

	if( pAf_next )		/* See if it's the next affect */
	{
	    pAf->next = pAf_next->next;
	    free_affect( pAf_next );
	}
	else                                 /* Doesn't exist */
	{
	     stc( "{cNo such affect.\n\r", ch );
	     return FALSE;
	}
    }

    stc( "{cAffect removed.\n\r", ch);
    return TRUE;
}



OEDIT( oedit_name )
{
    OBJ_INDEX_DATA *pObj;

    EDIT_OBJ(ch, pObj);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  name {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pObj->name );
    pObj->name = str_dup( argument );

    stc( "{cName set.\n\r", ch);
    return TRUE;
}



OEDIT( oedit_short )
{
    OBJ_INDEX_DATA *pObj;

    EDIT_OBJ(ch, pObj);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  short {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pObj->short_descr );
    pObj->short_descr = str_dup( argument );

    stc( "{cShort description set.\n\r", ch);
    return TRUE;
}



OEDIT( oedit_long )
{
    OBJ_INDEX_DATA *pObj;

    EDIT_OBJ(ch, pObj);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  long {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pObj->description );
    pObj->description = str_dup( argument );
    pObj->description[0] = UPPER( pObj->description[0] );

    stc( "{cLong description set.\n\r", ch);
    return TRUE;
}



bool set_value( CHAR_DATA *ch, OBJ_INDEX_DATA *pObj, char *argument, int value )
{
    if ( argument[0] == '\0' )
    {
	set_obj_values( ch, pObj, -1, "" );     /* '\0' changed to "" -- Hugin */
	return FALSE;
    }

    if ( set_obj_values( ch, pObj, value, argument ) )
	return TRUE;

    return FALSE;
}



/*****************************************************************************
 Name:		oedit_values
 Purpose:	Finds the object and sets its value.
 Called by:	The four valueX functions below. (now five -- Hugin )
 ****************************************************************************/
bool oedit_values( CHAR_DATA *ch, char *argument, int value )
{
    OBJ_INDEX_DATA *pObj;

    EDIT_OBJ(ch, pObj);

    if ( set_value( ch, pObj, argument, value ) )
        return TRUE;

    return FALSE;
}


OEDIT( oedit_value0 )
{
    if ( oedit_values( ch, argument, 0 ) )
        return TRUE;

    return FALSE;
}



OEDIT( oedit_value1 )
{
    if ( oedit_values( ch, argument, 1 ) )
        return TRUE;

    return FALSE;
}



OEDIT( oedit_value2 )
{
    if ( oedit_values( ch, argument, 2 ) )
        return TRUE;

    return FALSE;
}



OEDIT( oedit_value3 )
{
    if ( oedit_values( ch, argument, 3 ) )
        return TRUE;

    return FALSE;
}



OEDIT( oedit_value4 )
{
    if ( oedit_values( ch, argument, 4 ) )
        return TRUE;

    return FALSE;
}



OEDIT( oedit_weight )
{
    OBJ_INDEX_DATA *pObj;

    EDIT_OBJ(ch, pObj);

    if ( pObj->weight > 0 && !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP))
    {
	stc( "{cWeight for this object has already been set. Set the material,\n\rthen set the level to change the weight.{x\n\r", ch );
	return FALSE;
    }
    if ( argument[0] == '\0' || !is_number( argument ) )
    {
	stc( "{cSyntax:  weight {C[{cnumber{C]{c\n\r", ch );
	return FALSE;
    }

    pObj->weight = atoi( argument );

    stc( "{cWeight set.\n\r", ch);
    return TRUE;
}

OEDIT( oedit_cost )
{
    OBJ_INDEX_DATA *pObj;

    EDIT_OBJ(ch, pObj);

    if ( pObj->cost > 0 && !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP) )
    {
	stc( "{cCost for this object has already been set. Set the material,\n\rthen set the level to change the cost.{x\n\r", ch );
	return FALSE;
    }
    if ( argument[0] == '\0' || !is_number( argument ) )
    {
	stc( "{cSyntax:  cost {C[{cnumber{C]{c\n\r", ch );
	return FALSE;
    }

    pObj->cost = atoi( argument );

    stc( "{cCost set.\n\r", ch);
    return TRUE;
}



OEDIT( oedit_create )
{
    OBJ_INDEX_DATA *pObj;
    AREA_DATA *pArea;
    int  value;
    int  iHash;

    value = atoi( argument );
    if ( argument[0] == '\0' || value == 0 )
    {
	stc( "{cSyntax:  oedit create {C[{cvnum{C]{c\n\r", ch );
	return FALSE;
    }

    pArea = get_vnum_area( value );
    if ( !pArea )
    {
	stc( "{cOEdit:  That vnum is not assigned an area.\n\r", ch );
	return FALSE;
    }

    if ( !IS_BUILDER( ch, pArea ) )
    {
	stc( "{cOEdit:  Vnum in an area you cannot build in.\n\r", ch );
	return FALSE;
    }

    if ( get_obj_index( value ) )
    {
	stc( "{cOEdit:  Object vnum already exists.\n\r", ch );
	return FALSE;
    }

    pObj			= new_obj_index();
    pObj->vnum			= value;
    pObj->area			= pArea;
    pObj->material      = str_dup("cloth");

    if ( value > top_vnum_obj )
	top_vnum_obj = value;

    iHash			= value % MAX_KEY_HASH;
    pObj->next			= obj_index_hash[iHash];
    obj_index_hash[iHash]	= pObj;
    ch->desc->pEdit		= (void *)pObj;

    stc( "{cObject Created.\n\r", ch );
    return TRUE;
}



OEDIT( oedit_ed )
{
    OBJ_INDEX_DATA *pObj;
    EXTRA_DESCR_DATA *ed;
    char command[MIL];
    char keyword[MIL];

    EDIT_OBJ(ch, pObj);

    argument = one_argument( argument, command );
    one_argument( argument, keyword );

    if ( command[0] == '\0' )
    {
	stc( "{cSyntax:  ed add {C[{ckeyword{C]{c\n\r", ch );
	stc( "{c         ed delete {C[{ckeyword{C]{c\n\r", ch );
	stc( "{c         ed edit {C[{ckeyword{C]{c\n\r", ch );
	stc( "{c         ed format {C[{ckeyword{C]{c\n\r", ch );
	return FALSE;
    }

    if ( !str_cmp( command, "add" ) )
    {
	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed add {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	ed                  =   new_extra_descr();
	ed->keyword         =   str_dup( keyword );
	ed->next            =   pObj->extra_descr;
	pObj->extra_descr   =   ed;

	string_append( ch, &ed->description );

	return TRUE;
    }

    if ( !str_cmp( command, "edit" ) )
    {
	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed edit {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	for ( ed = pObj->extra_descr; ed; ed = ed->next )
	{
	    if ( is_name( keyword, ed->keyword ) )
		break;
	}

	if ( !ed )
	{
	    stc( "{cOEdit:  Extra description keyword not found.\n\r", ch );
	    return FALSE;
	}

	string_append( ch, &ed->description );

	return TRUE;
    }

    if ( !str_cmp( command, "delete" ) )
    {
	EXTRA_DESCR_DATA *ped = NULL;

	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed delete {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	for ( ed = pObj->extra_descr; ed; ed = ed->next )
	{
	    if ( is_name( keyword, ed->keyword ) )
		break;
	    ped = ed;
	}

	if ( !ed )
	{
	    stc( "{cOEdit:  Extra description keyword not found.\n\r", ch );
	    return FALSE;
	}

	if ( !ped )
	    pObj->extra_descr = ed->next;
	else
	    ped->next = ed->next;

	free_extra_descr( ed );

	stc( "{cExtra description deleted.\n\r", ch );
	return TRUE;
    }


    if ( !str_cmp( command, "format" ) )
    {
	EXTRA_DESCR_DATA *ped = NULL;

	if ( keyword[0] == '\0' )
	{
	    stc( "{cSyntax:  ed format {C[{ckeyword{C]{c\n\r", ch );
	    return FALSE;
	}

	for ( ed = pObj->extra_descr; ed; ed = ed->next )
	{
	    if ( is_name( keyword, ed->keyword ) )
		break;
	    ped = ed;
	}

	if ( !ed )
	{
	    stc( "{cOEdit:  Extra description keyword not found.\n\r", ch );
	    return FALSE;
	}

	ed->description = format_string( ed->description );

	stc( "{cExtra description formatted.\n\r", ch );
	return TRUE;
    }

    oedit_ed( ch, "" );
    return FALSE;
}

/* ROM object functions : */

OEDIT( oedit_extra )      /* Moved out of oedit() due to naming conflicts -- Hugin */
{
    OBJ_INDEX_DATA *pObj;
    int value;
    if ( argument[0] != '\0' )
    {
		EDIT_OBJ(ch, pObj);

		if ( ( value = flag_value( extra_flags, argument ) ) != NO_FLAG )
		{
		    TOGGLE_BIT(pObj->extra_flags, value);

		    stc( "{cExtra flag toggled.\n\r", ch);
			ptc( ch, "Forcing level to 0. Please reset level.\n\r" );
			clear_obj_values( pObj );
		    return TRUE;
		}
    }

    stc( "{cSyntax:  extra {C[{cflag{C]{c\n\r"
		  "Type '? extra' for a list of flags.\n\r", ch );
    return FALSE;
}


OEDIT( oedit_wear )      /* Moved out of oedit() due to naming conflicts -- Hugin */
{
    OBJ_INDEX_DATA *pObj;
    AFFECT_DATA *pAf;
    int value;

    if ( argument[0] != '\0' )
    {
		EDIT_OBJ(ch, pObj);

		if ( ( value = flag_value( wear_flags, argument ) ) != NO_FLAG )
		{
		    TOGGLE_BIT(pObj->wear_flags, value);
		    if ( pObj->affected != NULL )
		    {
				pAf = pObj->affected;
				pObj->affected = pAf->next;
				free_affect( pAf );
				stc("Affect data removed.\n\r",ch);
			}
			ptc( ch, "Forcing level to 0. Please reset level.\n\r" );
			clear_obj_values( pObj );
		    stc( "Wear flag toggled.\n\r", ch);
			if ( pObj->item_type == ITEM_LAMPPOST && pObj->wear_flags == ITEM_TAKE )
			{
				stc("Lamppost objects cannot be set wear take.\n\r", ch);
				REMOVE_BIT( pObj->wear_flags, ITEM_TAKE );
			}
		    return TRUE;
		}
    }

    stc( "{cSyntax:  wear {C[{cflag{C]{c\n\r"
		  "Type '? wear' for a list of flags.\n\r", ch );
    return FALSE;
}


OEDIT( oedit_type )      /* Moved out of oedit() due to naming conflicts -- Hugin */
{
    OBJ_INDEX_DATA *pObj;
    int value;

    if ( argument[0] != '\0' )
    {
	EDIT_OBJ(ch, pObj);

	if ( ( value = flag_value( type_flags, argument ) ) != NO_FLAG )
	{
		if ( value == 3 || value == 4 || value == 13 || value == 20 || value == 22
		|| value == 23 || value == 24 || value == 27 || value == 30 || value == 31
		|| value == 32 || value == 34 || value == 35 || value == 39
		|| value == 40 || value == 45 || value == 46
		|| value == 51 || value == 52 || value == 58 || value == 59 || value == 60
		|| value == 61 || value == 62 || value == 63 || value == 64 || value == 65
		|| value == 66 || value == 67 || value == 68 || value == 69 || value == 70
		|| value == 71 )
		{
			if ( !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP))
			{
				stc("You cannot make this item type.\n\r",ch);
				return FALSE;
			}
		}
	    pObj->item_type = value;

	    stc( "{cType set.\n\r", ch);

	    /*
	     * Clear the values.
	     */
		clear_obj_values( pObj );

	    return TRUE;
	}
    }

    stc( "{cSyntax:  type {C[{cflag{C]{c\n\r"
		  "Type '? type' for a list of flags.\n\r", ch );
    return FALSE;
}

OEDIT( oedit_guild )
{
    OBJ_INDEX_DATA *pObj;
    int guild;

    EDIT_OBJ(ch, pObj);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  guild {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    if ((guild = clan_lookup(argument)) == 0)
    {
	stc( "{cNo such guild exists.\n\r", ch );
	return FALSE;
    }

    pObj->guild = str_dup(argument);

    stc( "{cGuild set.\n\r", ch);
    return TRUE;
}

OEDIT( oedit_material )
{
    OBJ_INDEX_DATA *pObj;
    int mat = 0, /*max = 79,*/ lb = 0;

    EDIT_OBJ(ch, pObj);

    if ( argument[0] == '\0' ) {
		stc( "{cSyntax:  Material {C[{cstring{C]{c\n\r", ch );
		stc( "Valid materials are as follows.\n\r\n\r",ch);
		for ( mat = 0; material_table[mat].name != NULL; mat++ )
		{
			lb++;
			ptc( ch, "{C*{c %14s {C*{x    ", material_table[mat].name );
			if ( lb >= 3 )
			{
				stc("\n\r",ch);
				lb = 0;
			}
		}
	    return FALSE;
	}

	if ( (!str_cmp( argument, "black steel")) && ((!is_exact_name(ch->name,MUD_COIMP)) && (!is_exact_name(ch->name,MUD_OWNER))))
	{
		stc( "Sorry only senior staff can create items of that material.\n\r", ch);
		return FALSE;
	}
	if ( (!str_cmp( argument, "black steel")) && ((!is_exact_name(ch->name,MUD_COIMP)) && (!is_exact_name(ch->name,MUD_OWNER))))
	{
		stc( "Sorry only senior staff can create items of that material.\n\r", ch);
		return FALSE;
	}

	for ( mat = 0; material_table[mat].name != NULL; mat++ )
	{
		if ( !str_cmp( argument, material_table[mat].name ) )
		{
			ptc( ch, "Material set to %s.\n\r", material_table[mat].name );
			ptc( ch, "Density is %d.\n\r", material_table[mat].density );
			ptc( ch, "Value is %d.\n\r", material_table[mat].value );
			ptc( ch, "MaxAffect is %d.\n\r", material_table[mat].maxAff );
			pObj->condition += material_table[mat].density;
			free_string( pObj->material );
			pObj->material = str_dup(material_table[mat].name);
			if ( pObj->condition > 100 )
				pObj->condition = 100;
			ptc( ch, "Forcing level to 0. Please reset level.\n\r" );
			clear_obj_values( pObj );
			return TRUE;
		}
	}
	if ( pObj->condition > 100 )
		pObj->condition = 100;
    stc( "{cCould not find a material with that name.\n\rMake sure to type the name exactly as it appears in the material table.\n\r", ch);
    return FALSE;
}

bool check_obj ( CHAR_DATA *ch, OBJ_INDEX_DATA *pObj )
{
	if ( pObj->item_type == ITEM_WEAPON )
	{
		if ( pObj->value[3] == 0 )
		{
		stc( "{RERROR!{c Weapon value3 is not valid.\n\r? WEAPON for a valid value3 and retry.\n\r", ch );
		return FALSE;
		}
		if ( !CAN_WEAR( pObj, ITEM_TAKE ) )
		{
		stc( "{RERROR!{c Weapon cannot be taken.\n\r? WEAR for valid wear flags and retry.\n\r", ch );
		return FALSE;
		}
		if ( !CAN_WEAR( pObj, ITEM_WIELD ) )
		{
		stc( "{RERROR!{c Weapon cannot be wielded.\n\r? WEAR for valid wear flags and retry.\n\r", ch );
		return FALSE;
		}
	}
	return TRUE;
}

OEDIT( oedit_level )
/* Dal */
{
    OBJ_INDEX_DATA *pObj;
	int tmp1, tmp2, tmp3, tmp4, tmp5, /*max = 75*/ mat;
	AFFECT_DATA *pAf;
    bool found = FALSE;

    EDIT_OBJ(ch, pObj);

    if ( argument[0] == '\0' || !is_number( argument ) || (atoi(argument) < 0 ))
    {
		stc( "{cSyntax:  level {C[{cnumber{C]{c\n\r", ch );
		return FALSE;
    }

    if ( pObj->affected != NULL )
    {
		pAf = pObj->affected;
		pObj->affected = pAf->next;
		free_affect( pAf );
		stc("Affect data removed.\n\r",ch);
	}

	pObj->cost = 0;
	pObj->level = 1;
	tmp1 = 2;
	tmp2 = 0;
	tmp3 = 0;
	tmp4 = 0;
	tmp5 = 0;
	if (!strcmp(pObj->material,"unknown"))
	{
		stc("{RWARNING{r!{x Invalid material type.\n\r",ch);
		return FALSE;
	}
	for ( mat = 0; material_table[mat].name != NULL; mat++ )
	{
		if ( !str_cmp( pObj->material, material_table[mat].name ) )
		{
			found = TRUE;
			if ( material_table[mat].weapon == 0 && pObj->item_type == ITEM_WEAPON )
			{
				stc("{RERROR{r!{x Material type not valid for weapon.\n\r",ch);
				stc("Item values NOT set. Set a valid material.\n\r",ch);
				return FALSE;
			}
			if ( pObj->item_type == ITEM_ARROW && !is_wood( pObj ) )
	        {
	        stc("{cInvalid material type for an arrow.\n{cMaterial must be wood.\n",ch);
	       	return FALSE;
			}
			if ( pObj->item_type == ITEM_WEAPON )
			{
				tmp2 = material_table[mat].value * 10;
				tmp3 = material_table[mat].density;
				tmp4 = material_table[mat].density * 2;
				tmp5 = material_table[mat].weapon * 5;
			}
			if ( pObj->item_type == ITEM_ARMOR )
			{
				tmp2 = material_table[mat].value * 12;
				if (IS_SET(pObj->extra_flags,ITEM_REMORT))
	   				tmp2 *= 2;
			}
			break;
		}
	}
	if ( !found )
	{
		stc("ERROR! Unknown material type. Please check the material.\n\r",ch);
		return FALSE;
	}
	stc("Object level forced to 1, per new OLC rules.\n\r",ch);
	switch ( pObj->item_type )
	{
		default:
			pObj->weight = material_table[mat].density;
   			pObj->cost = material_table[mat].value * material_table[mat].density;
			if (pObj->weight <= 0) {
				pObj->weight = 1;
			}
		break;
		case ITEM_KEYRING:
			pObj->value[0] = 0;
			pObj->value[1] = 0;
			pObj->value[2] = 0;
			pObj->value[3] = 0;
			pObj->value[4] = 0;
   			pObj->cost = 1000;
			pObj->weight = material_table[mat].density;
		break;
		case (ITEM_ANGREAL_MALE): case (ITEM_ANGREAL_FEMALE):
			pObj->value[1] = material_table[mat].density;
   			pObj->cost = material_table[mat].value * material_table[mat].density;
			pObj->weight = material_table[mat].density;
		break;
        case ITEM_SCROLL:
        case ITEM_POTION:
        case ITEM_PILL:
			pObj->cost = (pObj->value[0] * pObj->value[0]) / 2;
			if ( pObj->cost < 20 )
				pObj->cost = 20;
			pObj->weight = material_table[mat].density;
		break;
		case (ITEM_JOURNAL):
			pObj->cost = material_table[mat].value * material_table[mat].density;
			pObj->weight = 10 + material_table[mat].density;
		break;
		case (ITEM_LIGHT):
			pObj->cost = material_table[mat].value * material_table[mat].density;
			pObj->weight = material_table[mat].density;
			pObj->value[2] = material_table[mat].value * 10;
		break;
		case (ITEM_ARROW):
			pObj->cost = material_table[mat].value * material_table[mat].density / 10;
			pObj->weight = material_table[mat].density / 10;
			pObj->value[1] = 2;
			pObj->value[2] = material_table[mat].value * 10;
		break;
		case (ITEM_FOOD):
			pObj->cost = material_table[mat].value * material_table[mat].density;
			pObj->weight = material_table[mat].density;
			pObj->value[0] = tmp1 / 2;
			pObj->value[1] = tmp1 / 2;
		break;
		case (ITEM_FIREOIL):
			pObj->cost = material_table[mat].value * material_table[mat].density;
			pObj->value[0] = 5;
			pObj->value[2] = tmp1;
			pObj->weight = material_table[mat].density;
		break;
		case (ITEM_QUIVER):
   			pObj->value[0] = tmp1 / 2;
   			pObj->value[1] = 1;
   			pObj->value[2] = tmp1;
   			pObj->cost = material_table[mat].value * material_table[mat].density;
			pObj->weight = material_table[mat].density;
   		break;

     	case (ITEM_WEAPON):
        if (pObj->value[0] == (WEAPON_SWORD))
        {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
   			pObj->value[2] = (tmp1 / 4) + 7 + tmp5;
   			if (pObj->value[2] == 0)
   			{ pObj->value[2] = 1; }
   			pObj->cost = 15 + tmp2;
			pObj->weight = 4 + material_table[mat].density;
    		break;
		}
       	else if (pObj->value[0] == (WEAPON_EXOTIC))
       	{
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
   			pObj->value[2] = (tmp1 / 5) + 5 + tmp5;
   			if (pObj->value[2] == 0)
   			{ pObj->value[2] = 1; }
   			pObj->cost = 10 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
       	else if (pObj->value[0] == (WEAPON_DAGGER))
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 6) + 4 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 2 + tmp2;
			pObj->weight = 1 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == (WEAPON_SPEAR)
	    || pObj->value[0] == (WEAPON_STAFF))
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.5) + 7 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 1 + tmp2;
			pObj->weight = 4 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == (WEAPON_MACE))
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.5) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 12 + tmp2;
			pObj->weight = 12 + material_table[mat].density;
		    break;
		}
	   	else if (pObj->value[0] == (WEAPON_AXE))
	   	{
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 3.5) + 8 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 10 + tmp2;
			pObj->weight = 7 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == (WEAPON_FLAIL))
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 5) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 5 + tmp2;
			pObj->weight = 6 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == (WEAPON_WHIP))
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 7) + 3 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 1 + tmp2;
			pObj->weight = 2 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == (WEAPON_POLEARM))
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4) + 7 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 5 + tmp2;
			pObj->weight = 9 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_SHORTBOW)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.2) + 7 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 30 + tmp2;
			pObj->weight = 2 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_LONGBOW)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.2) + 8 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 100 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_RECURVEDBOW)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.2) + 9 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 100 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_COMPOSITEBOW)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.2) + 10 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 100 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_CROSSBOW)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.2) + 11 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 50 + tmp2;
			pObj->weight = 9 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_NET)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 7) + 3 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 20 + tmp2;
			pObj->weight = 10 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_HAMMER)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.5) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 12 + tmp2;
			pObj->weight = 8 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_SLING)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 6) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 2 + tmp2;
			pObj->weight = 2 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_SWORDBREAKER)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.2) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 25 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_SAP)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 5.5) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 2 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_SHORTSWORD)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.5) + 7 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 10 + tmp2;
			pObj->weight = 3 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_LANCE)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4.5) + 7 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 10 + tmp2;
			pObj->weight = 10 + material_table[mat].density;
		    break;
		}
	    else if (pObj->value[0] == WEAPON_ASHANDAREI)
	    {
        	if ( !check_obj( ch, pObj ) )
	        	return FALSE;
			if (!IS_SET(pObj->extra_flags,ITEM_REMORT))
   				pObj->value[1] = 1;
   			else
   				pObj->value[1] = 2;
			pObj->value[2] = (tmp1 / 4) + 6 + tmp5;
			if (pObj->value[2] == 0)
			{ pObj->value[2] = 1; }
   			pObj->cost = 80 + tmp2;
			pObj->weight = 15 + material_table[mat].density;
		    break;
		}
	    else
	    {
		    stc("{RWARNING!{c UNKNOWN WEAPON TYPE.\n\rCheck value0 and retry.{x\n\r",ch);
		    break;
		}
	   	case (ITEM_ARMOR):
		   	if (CAN_WEAR(pObj, ITEM_WEAR_SHIELD))
		   	{
				pObj->value[0] = material_table[mat].pierce + 18;
				pObj->value[1] = material_table[mat].bash + 18;
				pObj->value[2] = material_table[mat].slash + 18;
				pObj->value[3] = material_table[mat].weaves + 18;
	   			pObj->cost = material_table[mat].value * 9;
	   			pObj->weight = 18 + material_table[mat].density;
				break;
			}
		   	else if (CAN_WEAR(pObj, ITEM_WEAR_BODY))
		   	{
				pObj->value[0] = material_table[mat].pierce + 20;
				pObj->value[1] = material_table[mat].bash + 20;
				pObj->value[2] = material_table[mat].slash + 20;
				pObj->value[3] = material_table[mat].weaves + 20;
	   			pObj->cost = material_table[mat].value * 10;
	   			pObj->weight = 20 + material_table[mat].density;
				break;
			}
			else if (CAN_WEAR(pObj, ITEM_WEAR_WAIST)
			|| CAN_WEAR(pObj, ITEM_HOLD)
			|| CAN_WEAR(pObj, ITEM_WEAR_BACK)
			|| CAN_WEAR(pObj, ITEM_WEAR_HEAD)
			|| CAN_WEAR(pObj, ITEM_WEAR_FOREARM)
			|| CAN_WEAR(pObj, ITEM_WEAR_THIGH)
			|| CAN_WEAR(pObj, ITEM_WEAR_CALF)
			|| CAN_WEAR(pObj, ITEM_WEAR_WRIST)
			|| CAN_WEAR(pObj, ITEM_WEAR_ABOUT)
			|| CAN_WEAR(pObj, ITEM_WEAR_NECK)
			|| CAN_WEAR(pObj, ITEM_WEAR_ARMS)
			|| CAN_WEAR(pObj, ITEM_WEAR_HANDS)
			|| CAN_WEAR(pObj, ITEM_WEAR_FEET)
			|| CAN_WEAR(pObj, ITEM_WEAR_LEGS) )
		   	{
				pObj->value[0] = material_table[mat].pierce + 8;
				pObj->value[1] = material_table[mat].bash + 8;
				pObj->value[2] = material_table[mat].slash + 8;
				pObj->value[3] = material_table[mat].weaves + 8;
	   			pObj->cost = material_table[mat].value * 4;
	   			pObj->weight = 8 + material_table[mat].density;
				break;
			}
			else if (CAN_WEAR(pObj, ITEM_WEAR_SHOULDERS))
		   	{
				pObj->value[0] = material_table[mat].pierce + 12;
				pObj->value[1] = material_table[mat].bash + 12;
				pObj->value[2] = material_table[mat].slash + 12;
				pObj->value[3] = material_table[mat].weaves + 12;
	   			pObj->cost = material_table[mat].value * 6;
	   			pObj->weight = 12 + material_table[mat].density;
				break;
			}
			else if (CAN_WEAR(pObj, ITEM_WEAR_NOSE)
			|| CAN_WEAR(pObj, ITEM_WEAR_EYES))
		   	{
				pObj->value[0] = material_table[mat].pierce + 2;
				pObj->value[1] = material_table[mat].bash + 2;
				pObj->value[2] = material_table[mat].slash + 2;
				pObj->value[3] = material_table[mat].weaves + 2;
	   			pObj->cost = material_table[mat].value;
	   			pObj->weight = 2 + material_table[mat].density;
				break;
			}
	        else if (CAN_WEAR(pObj, ITEM_WEAR_FLOAT))
		   	{
				pObj->value[0] = material_table[mat].pierce + 1;
				pObj->value[1] = material_table[mat].bash + 1;
				pObj->value[2] = material_table[mat].slash + 1;
				pObj->value[3] = material_table[mat].weaves + 1;
	   			pObj->cost = material_table[mat].value;
	   			pObj->weight = 1 + material_table[mat].density;
				break;
			}
			else if (CAN_WEAR(pObj, ITEM_WEAR_EAR)
			|| CAN_WEAR(pObj,ITEM_WEAR_FINGER))
		   	{
				pObj->value[0] = material_table[mat].pierce + 1;
				pObj->value[1] = material_table[mat].bash + 1;
				pObj->value[2] = material_table[mat].slash + 1;
				pObj->value[3] = material_table[mat].weaves + 1;
	   			pObj->cost = material_table[mat].value;
	   			pObj->weight = 2 + material_table[mat].density;
				break;
			}
			else if (CAN_WEAR(pObj, ITEM_WEAR_ANKLE))
		   	{
				pObj->value[0] = material_table[mat].pierce + 2;
				pObj->value[1] = material_table[mat].bash + 2;
				pObj->value[2] = material_table[mat].slash + 2;
				pObj->value[3] = material_table[mat].weaves + 2;
	   			pObj->cost = material_table[mat].value * 2;
	   			pObj->weight = 2 + material_table[mat].density;
				break;
			}
			else if (CAN_WEAR(pObj, ITEM_WEAR_FACE))
		   	{
				pObj->value[0] = material_table[mat].pierce + 3;
				pObj->value[1] = material_table[mat].bash + 3;
				pObj->value[2] = material_table[mat].slash + 3;
				pObj->value[3] = material_table[mat].weaves + 3;
	   			pObj->cost = material_table[mat].value * 2;
	   			pObj->weight = 2 + material_table[mat].density;
				break;
			}
		    else
		    {
			    stc("{RWARNING!{c UNKNOWN ARMOR TYPE.\n\rCheck the wear location and try again.\n\r",ch);
			    break;
			}
		}
	    stc( "{cLevel set.\n\r", ch);
	    stc( "{cDefault values for this level automatically generated.\n\r", ch);
	    return FALSE;
}



OEDIT( oedit_condition )
{
    OBJ_INDEX_DATA *pObj;
    int value;

    if ( argument[0] != '\0'
    && ( value = atoi (argument ) ) >= 0
    && ( value <= 100 ) )
    {
	EDIT_OBJ( ch, pObj );

	pObj->condition = value;
	stc( "{cCondition set.\n\r", ch );

	return TRUE;
    }

    stc( "{cSyntax:  condition {C[{cnumber{C]{c\n\r"
		  "{cWhere number can range from 0 (ruined) to 100 (perfect).\n\r",
		  ch );
    return FALSE;
}

/*
 * Mobile Editor Functions.
 */
MEDIT( medit_delete )
{
	CHAR_DATA *wch, *wnext;
	MOB_INDEX_DATA *pMob;
	RESET_DATA *pReset, *wReset;
	ROOM_INDEX_DATA *pRoom;
	char arg[MIL];
	int index, mcount, rcount, iHash, i;
	bool foundmob = FALSE;
	bool foundobj = FALSE;

	if( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  medit delete [vnum]\n\r", ch );
		return FALSE;
	}

	one_argument( argument, arg );

	if( is_number( arg ) )
	{
		index = atoi( arg );
		pMob = get_mob_index( index );
	}
	else
	{
		send_to_char( "That is not a number.\n\r", ch );
		return FALSE;
	}

	if( !pMob )
	{
		send_to_char( "No such mobile.\n\r", ch );
		return FALSE;
	}

	SET_BIT( pMob->area->area_flags, AREA_CHANGED );

	if( top_vnum_mob == index )
		for( i = 1; i < index; i++ )
			if( get_mob_index( i ) )
				top_vnum_mob = i;

	top_mob_index--;

	/* Now crush all resets and take out mobs while were at it */
	rcount = 0;
	mcount = 0;

	for( iHash = 0; iHash < MAX_KEY_HASH; iHash++ )
	{
		for( pRoom = room_index_hash[iHash]; pRoom; pRoom = pRoom->next )
		{

			for( wch = pRoom->people; wch; wch = wnext )
			{
				wnext = wch->next_in_room;
				if( wch->pIndexData == pMob )
				{
					extract_char( wch, TRUE );
					mcount++;
				}
			}

			for( pReset = pRoom->reset_first; pReset; pReset = wReset )
			{
				wReset = pReset->next;
				switch( pReset->command )
				{
				case 'M':
					if( pReset->arg1 == index )
					{
						foundmob = TRUE;

						unlink_reset( pRoom, pReset );
						free_reset_data( pReset );

						rcount++;
						SET_BIT( pRoom->area->area_flags,
						AREA_CHANGED );

					}
					else
						foundmob = FALSE;

					break;
				case 'E':
				case 'G':
					if( foundmob )
					{
						foundobj = TRUE;

						unlink_reset( pRoom, pReset );
						free_reset_data( pReset );

						rcount++;
						SET_BIT( pRoom->area->area_flags,
						AREA_CHANGED );

					}
					else
						foundobj = FALSE;

					break;
				case '0':
					foundobj = FALSE;
					break;
				case 'P':
					if( foundobj && foundmob )
					{
						unlink_reset( pRoom, pReset );
						free_reset_data( pReset );

						rcount++;
						SET_BIT( pRoom->area->area_flags,
						AREA_CHANGED );
					}
				}
			}
		}
	}

	unlink_mob_index( pMob );

	pMob->area = NULL;
	pMob->vnum = 0;

	free_mob_index( pMob );

	printf_to_char( ch, "Removed mobile vnum %d and"
		" %d resets.\n\r", index, rcount );
	printf_to_char( ch, "%d mobiles were extracted"
		" from the mud.\n\r",mcount );
	edit_done(ch);
	return TRUE;
}

MEDIT( medit_show )
{
    MOB_INDEX_DATA *pMob;
    PROG_LIST *list;

    EDIT_MOB(ch, pMob);

    ptc(ch, "{cName:        {C[{c%s{C]{c\n\rArea:        {C[{c%5d{C]{c %s\n\r",
	pMob->player_name,
	!pMob->area ? -1        : pMob->area->vnum,
	!pMob->area ? "{cNo Area" : pMob->area->name );

    ptc(ch, "{cAct:         {C[{c%s{C]{c\n\r",
	flag_string( act_flags, pMob->act ) );

    ptc(ch, "{cVnum:        {C[{c%5d{C]{c Sex:   {C[{c%s{C]{c   Race: {C[{c%s{C]{c\n\r",
	pMob->vnum,
	pMob->sex == SEX_MALE    ? "male   " :
	pMob->sex == SEX_FEMALE  ? "female " :
	pMob->sex == 3           ? "random " : "neutral",
	race_table[pMob->race].name );

    ptc(ch,
		  "{cLevel:       {C[{c%2d{C]{c    Align: {C[{c%4d{C]{c      Hitroll: {C[{c%2d{C]{c DamType:    {C[{c%s{C]{c\n\r",
	pMob->level,	pMob->alignment,
	pMob->hitroll,	attack_table[pMob->dam_type].name );

    if ( pMob->group )
    {
	ptc(ch, "{cGroup:       {C[{c%5d{C]{c\n\r", pMob->group );
    }

    ptc(ch, "{cHitdice:     {C[{c%2dd%-3d+%4d{C]{c ",
	     pMob->hit[DICE_NUMBER],
	     pMob->hit[DICE_TYPE],
	     pMob->hit[DICE_BONUS] );

    ptc(ch, "{cDamdice: {C[{c%2dd%-3d+%4d{C]{c ",
	     pMob->damage[DICE_NUMBER],
	     pMob->damage[DICE_TYPE],
	     pMob->damage[DICE_BONUS] );

    ptc(ch, "{cManadice:   {C[{c%2dd%-3d+%4d{C]{c\n\r",
	     pMob->mana[DICE_NUMBER],
	     pMob->mana[DICE_TYPE],
	     pMob->mana[DICE_BONUS] );

/* ROM values end */

    ptc(ch, "{cAff:         {C[{c%s{C]{c\n\r",
	flag_string( affect_flags, pMob->affected_by ) );

/* ROM values: */

    ptc(ch, "{cArmor:       {C[{cpierce: %d  bash: %d  slash: %d  magic: %d{C]{c\n\r",
	pMob->ac[AC_PIERCE], pMob->ac[AC_BASH],
	pMob->ac[AC_SLASH], pMob->ac[AC_EXOTIC] );

    ptc(ch, "{cForm:        {C[{c%s{C]{c\n\r",
	flag_string( form_flags, pMob->form ) );

    ptc(ch, "{cParts:       {C[{c%s{C]{c\n\r",
	flag_string( part_flags, pMob->parts ) );

    ptc(ch, "{cImm:         {C[{c%s{C]{c\n\r",
	flag_string( imm_flags, pMob->imm_flags ) );

    ptc(ch, "{cRes:         {C[{c%s{C]{c\n\r",
	flag_string( res_flags, pMob->res_flags ) );

    ptc(ch, "{cVuln:        {C[{c%s{C]{c\n\r",
	flag_string( vuln_flags, pMob->vuln_flags ) );

    ptc(ch, "{cOff:         {C[{c%s{C]{c\n\r",
	flag_string( off_flags,  pMob->off_flags ) );

    ptc(ch, "{cSize:        {C[{c%s{C]{c\n\r",
	flag_string( size_flags, pMob->size ) );

    ptc(ch, "{cMaterial:    {C[{c%s{C]{c\n\r",
        pMob->material );

    ptc(ch, "{cStartpos.    {C[{c%s{C]{c\n\r",
	flag_string( position_flags, pMob->start_pos ) );

    ptc(ch, "{cDefaultpos   {C[{c%s{C]{c\n\r",
	flag_string( position_flags, pMob->default_pos ) );

    ptc(ch, "{cWealth:      {C[{c%5ld{C] silver{c\n\r",
	pMob->wealth );

	ptc(ch, "{cDieMess:{C %s{c\n\r", pMob->die_mess );
	ptc(ch, "{cKillMess:{C %s{c\n\r", pMob->kill_mess );

/* ROM values end */

    if ( pMob->spec_fun )
    {
	ptc(ch, "{cSpec:    {C[{c%s{C]{c\n\r",  spec_name( pMob->spec_fun ) );
	}

    ptc(ch, "{cShort:{C %s{c\n\rLong:{C\n\r%s{c",
	pMob->short_descr,
	pMob->long_descr );

    ptc(ch, "{cDescription:{C\n\r%s{c", pMob->description );

    if ( pMob->pShop )
    {
	SHOP_DATA *pShop;
	int iTrade;

	pShop = pMob->pShop;

	ptc(ch,
	  "{cShop data for {C[{c%5d{C]:{c\n\r"
	  "  {cMarkup for purchaser:{C %d%%{c\n\r"
	  "  {cMarkdown for seller:{C  %d%%{c\n\r",
	    pShop->keeper, pShop->profit_buy, pShop->profit_sell );
	ptc(ch, "{c  {cHours:{C %d{c to{C %d.{c\n\r",
	    pShop->open_hour, pShop->close_hour );

	for ( iTrade = 0; iTrade < MAX_TRADE; iTrade++ )
	{
	    if ( pShop->buy_type[iTrade] != 0 )
	    {
		if ( iTrade == 0 )
		{
		    stc( "{c {c Number Trades Type\n\r", ch );
		    stc( "{c {C {w------ -----------{c\n\r", ch );
		}
		ptc(ch, "{c  {C[{c%4d{C]{c %s\n\r", iTrade,
		    flag_string( type_flags, pShop->buy_type[iTrade] ) );
		}
	}
    }

    if ( pMob->mprogs )
    {
	int cnt;

	ptc(ch, "\n\rMOBPrograms for {C[{c%5d{C]{c:\n\r", pMob->vnum);

	for (cnt=0, list=pMob->mprogs; list; list=list->next)
	{
		if (cnt ==0)
		{
			stc ( "{c Number Vnum Trigger Phrase\n\r", ch );
			stc ( "{w ------ ---- ------- ------{c\n\r", ch );
		}

		ptc(ch, "{C[{c%5d{C]{c %4d %7s %s\n\r", cnt,
			list->vnum,prog_type_to_name(list->trig_type),
			list->trig_phrase);
		cnt++;
	}
    }

    return FALSE;
}



MEDIT( medit_create )
{
    MOB_INDEX_DATA *pMob;
    AREA_DATA *pArea;
    int  value;
    int  iHash;

    value = atoi( argument );
    if ( argument[0] == '\0' || value == 0 )
    {
	stc( "{cSyntax:  medit create {C[{cvnum{C]{c\n\r", ch );
	return FALSE;
    }

    pArea = get_vnum_area( value );

    if ( !pArea )
    {
	stc( "{cMEdit:  That vnum is not assigned an area.\n\r", ch );
	return FALSE;
    }

    if ( !IS_BUILDER( ch, pArea ) )
    {
	stc( "{cMEdit:  Vnum in an area you cannot build in.\n\r", ch );
	return FALSE;
    }

    if ( get_mob_index( value ) )
    {
	stc( "{cMEdit:  Mobile vnum already exists.\n\r", ch );
	return FALSE;
    }

    pMob			= new_mob_index();
    pMob->vnum			= value;
    pMob->area			= pArea;

    if ( value > top_vnum_mob )
	top_vnum_mob = value;

    pMob->act			= ACT_IS_NPC;
    iHash			= value % MAX_KEY_HASH;
    pMob->next			= mob_index_hash[iHash];
    mob_index_hash[iHash]	= pMob;
    ch->desc->pEdit		= (void *)pMob;

    stc( "{cMobile Created.\n\r", ch );
    return TRUE;
}



MEDIT( medit_spec )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  spec {C[{cspecial function{C]{c\n\r", ch );
	return FALSE;
    }


    if ( !str_cmp( argument, "none" ) )
    {
        pMob->spec_fun = NULL;

        stc( "{cSpec removed.\n\r", ch);
        return TRUE;
    }

    else if ( spec_lookup( argument ) )
    {
	pMob->spec_fun = spec_lookup( argument );
	stc( "{cSpec set.\n\r", ch);
	return TRUE;
    }

    stc( "{cMEdit: No such special function.\n\r", ch );
    return FALSE;
}

MEDIT( medit_damtype )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  damtype {C[{cdamage message{C]{c\n\r", ch );
	stc( "{cIn order to see a list of messages, type {C'{c? weapon{C'{c.\n\r", ch );
	return FALSE;
    }

    pMob->dam_type = attack_lookup(argument);
    stc( "{cDamage type set.\n\r", ch);
    return TRUE;
}


MEDIT( medit_align )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( !is_exact_name( ch->name, MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
    {
    stc( "Alignment is not used.\n\rAlignment not set.", ch );
    return FALSE;
    }
    if ( argument[0] == '\0' || !is_number( argument ) )
    {
	stc( "{cSyntax:  alignment {C[{cnumber{C]{c\n\r", ch );
	stc("{cValid range is -1000 to 1000.\n\r",ch);
	stc("{cConsider 1000 as never having even considered an evil act.\n\r",ch);
	stc("{cConsider -1000 as the vilest abomination ever created.\n\r",ch);
	return FALSE;
    }

    pMob->alignment = atoi( argument );

    stc( "{cAlignment set.\n\r", ch);
    return TRUE;
}



MEDIT( medit_level )
{
    MOB_INDEX_DATA *pMob;
	int autoac, autotmp1, autotmp2, autotmp3;
    int pierce, bash, slash, exotic;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' || !is_number( argument ) )
    {
	stc( "{cSyntax:  level {C[{cnumber{C]{c\n\r", ch );
	return FALSE;
    }

    pMob->level = atoi( argument );
    if ( is_name(ch->name,MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
    {
		if ( atoi( argument ) > 999 )
		{
		stc( "Valid levels are 1 through 999. \n\r", ch);
		return FALSE;
		}
	}
	else if ( atoi( argument ) > 999 )
	{
		stc( "Valid levels are 1 through 999. \n\r", ch);
		return FALSE;
	}
	if ( pMob->dam_type == 0 )
	{
		stc( "{RERROR!{c Mobile has no damtype.\n\r{c? WEAPON to see possible damtypes and retry.{x\n\r", ch);
		return FALSE;
	}
	if ( is_name( pMob->player_name, "no name" ) )
	{
		stc( "{RERROR!{c Mobile has no name!\n\r{cSet the mobile name and retry.{x\n\r", ch);
		return FALSE;
	}
	if ( is_name( pMob->short_descr, "(no short description)" ) )
	{
		stc( "{RERROR!{c Mobile has no short description!\n\r{cSet the mobile short description and retry.{x\n\r", ch);
		return FALSE;
	}
	if ( is_name( pMob->short_descr, "(no long description)" ) )
	{
		stc( "{RERROR!{c Mobile has no long description!\n\r{cSet the mobile long description and retry.{x\n\r", ch);
		return FALSE;
	}
	if ( pMob->race == race_lookup("unique") )
	{
		stc( "{RERROR!{c Mobile race is not valid!\n\r{cSet the mobile race and retry.{x\n\r", ch);
		return FALSE;
	}
	if ( pMob->wealth > atoi(argument) )
	{
		stc( "{RWARNING!{c Mobile wealth is too high!\n\r{cOLC will Set the mobile wealth.{x\n\r", ch);
		pMob->wealth = atoi(argument);
	}

	autotmp1 = 100;
	autotmp2 = 2*pMob->level;

    if (IS_SET(pMob->act,ACT_WARRIOR))
    	autotmp2 = number_range(autotmp2*1.5,autotmp2*2.5);
	autoac = autotmp1 - autotmp2;

	/* figure out the armor class */
    if (pMob->level > 10 && pMob->level <= 50)
        autoac *= 1.15;
    else if (pMob->level > 50 && pMob->level <= 75)
        autoac *= 1.25;
    else if (pMob->level > 75 && pMob->level <= 100)
        autoac *= 1.5;
    else if (pMob->level > 100 && pMob->level <= 125)
        autoac *= 1.75;
    else if (pMob->level > 125 && pMob->level <= 150)
        autoac *= 2;
    else if (pMob->level > 150 && pMob->level <= 175)
        autoac *= 2.25;
    else if (pMob->level > 175 && pMob->level <= 199)
        autoac *= 2.5;
    else if (pMob->level > 199 && pMob->level <= 225)
        autoac *= 2.75;
    else if (pMob->level > 225 && pMob->level <= 250)
        autoac *= 3;
    else if (pMob->level > 250 && pMob->level <= 300)
        autoac *= 4;
    else if (pMob->level > 300 && pMob->level <= 350)
        autoac *= 4.5;
    else if (pMob->level > 350 && pMob->level <= 400)
        autoac *= 5;
    else if (pMob->level > 400 && pMob->level <= 450)
        autoac *= 5.5;
    else if (pMob->level > 450 && pMob->level <= 500)
        autoac *= 6;
    else if (pMob->level > 500 && pMob->level <= 550)
        autoac *= 6.5;
    else if (pMob->level > 550 && pMob->level <= 600)
        autoac *= 7;
    else if (pMob->level > 600 && pMob->level <= 650)
        autoac *= 7.5;
    else if (pMob->level > 650 && pMob->level <= 700)
        autoac *= 8;
    else if (pMob->level > 700 && pMob->level <= 750)
        autoac *= 8,5;
    else if (pMob->level > 750 && pMob->level <= 800)
        autoac *= 9;
    else if (pMob->level > 800 && pMob->level <= 850)
        autoac *= 9.5;
    else if (pMob->level > 850 && pMob->level <= 900)
        autoac *= 10;
    else if (pMob->level > 900 && pMob->level <= 1000)
        autoac *= 12;

    /*if (autoac > 999)
        autoac = 900;*/

    pierce = number_range(autoac*.75,autoac*1.25);
    bash   = number_range(autoac*.75,autoac*1.25);
    slash  = number_range(autoac*.75,autoac*1.25);
    exotic = number_range(autoac*.75,autoac*1.25);
	pMob->ac[AC_PIERCE] = pierce;
	pMob->ac[AC_BASH]   = bash;
	pMob->ac[AC_SLASH]  = slash;
	pMob->ac[AC_EXOTIC] = exotic;

	/* figure out the hit dice */
    autotmp3 = pMob->level;
    autotmp2 = 2*pMob->level;
    pMob->hit[DICE_NUMBER] = autotmp3;
    pMob->hit[DICE_TYPE]   = autotmp3 / 1.25;

    if (IS_SET(pMob->act,ACT_WARRIOR))
	    pMob->hit[DICE_BONUS]  = autotmp3 * 25;
    else
	    pMob->hit[DICE_BONUS]  = autotmp3 * 15;

    /*if (autotmp3 > 240)
    {
    pMob->hit[DICE_NUMBER] = 30000;
    pMob->hit[DICE_TYPE]   = 1;
    pMob->hit[DICE_BONUS]  = 0;
	}*/

	/* figure out the mana dice */
    pMob->mana[DICE_NUMBER] = 1;
    pMob->mana[DICE_TYPE]   = autotmp3;
    pMob->mana[DICE_BONUS]  = 1;

	/* figure out the damage dice */
   	pMob->damage[DICE_NUMBER] = 1;
    pMob->damage[DICE_TYPE]   = autotmp3;
    if (IS_SET(pMob->act,ACT_WARRIOR))
    	pMob->damage[DICE_BONUS]  = pMob->level / 1.25;
    else
    	pMob->damage[DICE_BONUS]  = pMob->level / 2.5;

	/* figure out the hitroll */
    autotmp3 = pMob->level;
    if (pMob->level > 10 && pMob->level <= 50)
        autotmp3 *= 1.25;
    else if (pMob->level > 50 && pMob->level <= 75)
        autotmp3 *= 1.5;
    else if (pMob->level > 75 && pMob->level <= 100)
        autotmp3 *= 2;
    else if (pMob->level > 100 && pMob->level <= 125)
        autotmp3 *= 2.25;
    else if (pMob->level > 125 && pMob->level <= 150)
        autotmp3 *= 2.75;
    else if (pMob->level > 150 && pMob->level <= 175)
        autotmp3 *= 3;
    else if (pMob->level > 175 && pMob->level <= 199)
        autotmp3 *= 3.5;
    else if (pMob->level > 199 && pMob->level <= 225)
        autotmp3 *= 4;
    else if (pMob->level > 225 && pMob->level <= 250)
        autotmp3 *= 4.35;
    else if (pMob->level > 250 && pMob->level <= 300)
        autotmp3 *= 5;
    else if (pMob->level > 300 && pMob->level <= 350)
        autotmp3 *= 5.25;
    else if (pMob->level > 350 && pMob->level <= 400)
        autotmp3 *= 5.75;
    else if (pMob->level > 400 && pMob->level <= 450)
        autotmp3 *= 6.15;
    else if (pMob->level > 450 && pMob->level <= 500)
        autotmp3 *= 6.75;
    else if (pMob->level > 500 && pMob->level <= 550)
        autotmp3 *= 7.5;
    else if (pMob->level > 550 && pMob->level <= 600)
        autotmp3 *= 8;
    else if (pMob->level > 600 && pMob->level <= 650)
        autotmp3 *= 8.5;
    else if (pMob->level > 650 && pMob->level <= 700)
        autotmp3 *= 9;
    else if (pMob->level > 700 && pMob->level <= 750)
        autotmp3 *= 10;
    else if (pMob->level > 750 && pMob->level <= 800)
        autotmp3 *= 11;
    else if (pMob->level > 800 && pMob->level <= 850)
        autotmp3 *= 12;
    else if (pMob->level > 850 && pMob->level <= 900)
        autotmp3 *= 13;
    else if (pMob->level > 900 && pMob->level <= 950)
        autotmp3 *= 14;
    else if (pMob->level > 950 && pMob->level <= 1000)
        autotmp3 *= 15;
    if (IS_SET(pMob->act,ACT_WARRIOR))
    	pMob->hitroll = autotmp3 / 1.25;
    else
    	pMob->hitroll = autotmp3 / 2.5;

	if (pMob->race == race_lookup("andor")
	|| pMob->race == race_lookup("amador")
	|| pMob->race == race_lookup("illian")
	|| pMob->race == race_lookup("shienar")
	|| pMob->race == race_lookup("domani")
	|| pMob->race == race_lookup("aiel")
	|| pMob->race == race_lookup("ogier")
	|| pMob->race == race_lookup("tairen")
	|| pMob->race == race_lookup("cairhien")
	|| pMob->race == race_lookup("seanchan")
	|| pMob->race == race_lookup("altara")
	|| pMob->race == race_lookup("trolloc")
	|| pMob->race == race_lookup("arafel")
	|| pMob->race == race_lookup("tarabon")
	|| pMob->race == race_lookup("tarvalon")
	|| pMob->race == race_lookup("kandor")
	|| pMob->race == race_lookup("mayene")
	|| pMob->race == race_lookup("tworiver")
	|| pMob->race == race_lookup("malkier"))
    {
	    pMob->wealth = number_range( 10,20 ); /* Wealth */
	    if (IS_SET(pMob->act,ACT_WARRIOR))
	    	pMob->wealth *= 2;
	    pMob->wealth = number_range( pMob->wealth * 9 / 11, pMob->wealth * 11 / 9 ); /* Wealth */
    }
    else
    pMob->wealth = 0;


    stc( "{cLevel set.\n\r", ch);
    stc("{cAverage MOB values for this level generated.\n\r",ch);
    return TRUE;
}



MEDIT( medit_desc )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	string_append( ch, &pMob->description );
	return TRUE;
    }

    stc( "{cSyntax:  desc    - line edit\n\r", ch );
    return FALSE;
}




MEDIT( medit_long )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  long {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pMob->long_descr );
    strcat( argument, "\n\r" );
    pMob->long_descr = str_dup( argument );
    pMob->long_descr[0] = UPPER( pMob->long_descr[0]  );

    stc( "{cLong description set.\n\r", ch);
    return TRUE;
}



MEDIT( medit_short )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  short {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pMob->short_descr );
    pMob->short_descr = str_dup( argument );

    stc( "{cShort description set.\n\r", ch);
    return TRUE;
}

MEDIT( medit_diemess )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  diemess {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pMob->die_mess );
    pMob->die_mess = str_dup( argument );

    stc( "{cDeath Message set.\n\r", ch);
    return TRUE;
}

MEDIT( medit_killmess )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  killmess {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pMob->kill_mess );
    pMob->kill_mess = str_dup( argument );

    stc( "{cKill Message set.\n\r", ch);
    return TRUE;
}

MEDIT( medit_name )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  name {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pMob->player_name );
    pMob->player_name = str_dup( argument );

    stc( "{cName set.\n\r", ch);
    return TRUE;
}

MEDIT( medit_shop )
{
    MOB_INDEX_DATA *pMob;
    char command[MIL];
    char arg1[MIL];

    argument = one_argument( argument, command );
    argument = one_argument( argument, arg1 );

    EDIT_MOB(ch, pMob);

    if ( command[0] == '\0' )
    {
	stc( "{cSyntax:  shop hours [#xopening] [#xclosing]\n\r", ch );
	stc( "{c         shop profit [#xbuying%] [#xselling%]\n\r", ch );
	stc( "{c         shop type [#x0-4] [item type]\n\r", ch );
	stc( "{c         shop assign\n\r", ch );
	stc( "{c         shop remove\n\r", ch );
	return FALSE;
    }


    if ( !str_cmp( command, "hours" ) )
    {
	if ( arg1[0] == '\0' || !is_number( arg1 )
	|| argument[0] == '\0' || !is_number( argument ) )
	{
	    stc( "{cSyntax:  shop hours [#xopening] [#xclosing]\n\r", ch );
	    return FALSE;
	}

	if ( !pMob->pShop )
	{
	    stc( "{cMEdit:  You must create a shop first (shop assign).\n\r", ch );
	    return FALSE;
	}

	pMob->pShop->open_hour = atoi( arg1 );
	pMob->pShop->close_hour = atoi( argument );

	stc( "{cShop hours set.\n\r", ch);
	return TRUE;
    }


    if ( !str_cmp( command, "profit" ) )
    {
	if ( arg1[0] == '\0' || !is_number( arg1 )
	|| argument[0] == '\0' || !is_number( argument ) )
	{
	    stc( "{cSyntax:  shop profit [#xbuying%] [#xselling%]\n\r", ch );
	    return FALSE;
	}

	if ( !pMob->pShop )
	{
	    stc( "{cMEdit:  You must create a shop first (shop assign).\n\r", ch );
	    return FALSE;
	}

	pMob->pShop->profit_buy     = 100;
	pMob->pShop->profit_sell    = 100;

	stc( "{cShop profit set.\n\r", ch);
	return TRUE;
    }


    if ( !str_cmp( command, "type" ) )
    {
	int value;

	if ( arg1[0] == '\0' || !is_number( arg1 )
	|| argument[0] == '\0' )
	{
	    stc( "{cSyntax:  shop type [#x0-4] [item type]\n\r", ch );
	    return FALSE;
	}

	if ( atoi( arg1 ) >= MAX_TRADE )
	{
	    ptc(ch, "{cMEdit:  May sell %d items max.\n\r", MAX_TRADE );
	    return FALSE;
	}

	if ( !pMob->pShop )
	{
	    stc( "{cMEdit:  You must create a shop first (shop assign).\n\r", ch );
	    return FALSE;
	}

	if ( ( value = flag_value( type_flags, argument ) ) == NO_FLAG )
	{
	    stc( "{cMEdit:  That type of item is not known.\n\r", ch );
	    return FALSE;
	}

	pMob->pShop->buy_type[atoi( arg1 )] = value;

	stc( "{cShop type set.\n\r", ch);
	return TRUE;
    }

    /* shop assign && shop delete by Phoenix */

    if ( !str_prefix(command, "assign") )
    {
    	if ( pMob->pShop )
    	{
        	stc("{cMob already has a shop assigned to it.\n\r", ch);
        	return FALSE;
	}

	pMob->pShop		= new_shop();
	if ( !shop_first )
        	shop_first	= pMob->pShop;
	if ( shop_last )
		shop_last->next	= pMob->pShop;
	shop_last		= pMob->pShop;

	pMob->pShop->keeper	= pMob->vnum;

	stc("{cNew shop assigned to mobile.\n\r", ch);
	return TRUE;
    }

    if ( !str_prefix(command, "remove") )
    {
	SHOP_DATA *pShop;

	pShop		= pMob->pShop;
	pMob->pShop	= NULL;

	if ( pShop == shop_first )
	{
		if ( !pShop->next )
		{
			shop_first = NULL;
			shop_last = NULL;
		}
		else
			shop_first = pShop->next;
	}
	else
	{
		SHOP_DATA *ipShop;

		for ( ipShop = shop_first; ipShop; ipShop = ipShop->next )
		{
			if ( ipShop->next == pShop )
			{
				if ( !pShop->next )
				{
					shop_last = ipShop;
					shop_last->next = NULL;
				}
				else
					ipShop->next = pShop->next;
			}
		}
	}

	free_shop(pShop);

	stc("{cMobile is no longer a shopkeeper.\n\r", ch);
	return TRUE;
    }

    medit_shop( ch, "" );
    return FALSE;
}


/* ROM medit functions: */


MEDIT( medit_sex )          /* Moved out of medit() due to naming conflicts -- Hugin */
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( sex_flags, argument ) ) != NO_FLAG )
	{
	    pMob->sex = value;

	    stc( "{cSex set.\n\r", ch);
	    return TRUE;
	}
    }

    stc( "{cSyntax: sex [sex]\n\r"
		  "Type '? sex' for a list of flags.\n\r", ch );
    return FALSE;
}


MEDIT( medit_act )          /* Moved out of medit() due to naming conflicts -- Hugin */
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( act_flags, argument ) ) != NO_FLAG )
	{
	    pMob->act ^= value;
	    SET_BIT( pMob->act, ACT_IS_NPC );

	    stc( "{cAct flag toggled.\n\r", ch);
	    return TRUE;
	}
    }

    stc( "{cSyntax: act {C[{cflag{C]{c\n\r"
		  "Type '? act' for a list of flags.\n\r", ch );
    return FALSE;
}


MEDIT( medit_affect )      /* Moved out of medit() due to naming conflicts -- Hugin */
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP))
    {
    stc("{RERROR!{c Manual editting of affect values is not supported.\n\r{cSet the mobile race to assign affects.\n\r", ch);
    return FALSE;
    }

    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( affect_flags, argument ) ) != NO_FLAG )
	{
	    pMob->affected_by ^= value;

	    stc( "{cAffect flag toggled.\n\r", ch);
	    return TRUE;
	}
    }

    stc( "{cSyntax: affect {C[{cflag{C]{c\n\r"
		  "Type '? affect' for a list of flags.\n\r", ch );
    return FALSE;
}



MEDIT( medit_ac )
{
    MOB_INDEX_DATA *pMob;
    char arg[MIL];
    int pierce, bash, slash, exotic;

    if ( !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP))
    {
    stc("{RERROR!{c Manual editting of AC values is not supported.\n\r{cSet the mobile level and act to assign AC.\n\r",ch);
    return FALSE;
    }
    do   /* So that I can use break and send the syntax in one place */
    {
	if ( argument[0] == '\0' )  break;

	EDIT_MOB(ch, pMob);
	argument = one_argument( argument, arg );

	if ( !is_number( arg ) )  break;
	pierce = atoi( arg );
	argument = one_argument( argument, arg );

	if ( arg[0] != '\0' )
	{
	    if ( !is_number( arg ) )  break;
	    bash = atoi( arg );
	    argument = one_argument( argument, arg );
	}
	else
	    bash = pMob->ac[AC_BASH];

	if ( arg[0] != '\0' )
	{
	    if ( !is_number( arg ) )  break;
	    slash = atoi( arg );
	    argument = one_argument( argument, arg );
	}
	else
	    slash = pMob->ac[AC_SLASH];

	if ( arg[0] != '\0' )
	{
	    if ( !is_number( arg ) )  break;
	    exotic = atoi( arg );
	}
	else
	    exotic = pMob->ac[AC_EXOTIC];

	pMob->ac[AC_PIERCE] = pierce;
	pMob->ac[AC_BASH]   = bash;
	pMob->ac[AC_SLASH]  = slash;
	pMob->ac[AC_EXOTIC] = exotic;

	stc( "{cAc set.\n\r", ch );
	return TRUE;
    } while ( FALSE );    /* Just do it once.. */

    stc( "{cSyntax:  ac [ac-pierce [ac-bash [ac-slash [ac-exotic]]]]\n\r"
		  "{chelp MOB_AC  gives a list of reasonable ac-values.\n\r", ch );
    return FALSE;
}

MEDIT( medit_form )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( !is_exact_name( ch->name, MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP) )
    {
    stc("{RERROR!{c Manual editting of form values is not supported.\n\r{cSet the mobile race to assign the form.\n\r",ch);
    return FALSE;
    }
    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( form_flags, argument ) ) != NO_FLAG )
	{
	    pMob->form ^= value;
	    stc( "{cForm toggled.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: form {C[{cflags{C]{c\n\r"
		  "Type '? form' for a list of flags.\n\r", ch );
    return FALSE;
}

MEDIT( medit_part )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( !is_exact_name( ch->name, MUD_OWNER) && !is_exact_name(ch->name,MUD_COIMP))
    {
    stc("{RERROR!{c Manual editting of part values is not supported.\n\r{cSet the mobile race to assign parts.\n\r",ch);
    return FALSE;
    }
    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( part_flags, argument ) ) != NO_FLAG )
	{
	    pMob->parts ^= value;
	    stc( "{cParts toggled.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: part {C[{cflags{C]{c\n\r"
		  "Type '? part' for a list of flags.\n\r", ch );
    return FALSE;
}

MEDIT( medit_imm )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( !is_exact_name( ch->name, MUD_OWNER )
    && !is_exact_name(ch->name,MUD_COIMP)
    //&& !is_exact_name(ch->name,MUD_SIS)
    && !is_exact_name(ch->name,MUD_HB) )
    {
    stc("{RERROR!{c Manual editting of immunities is not supported.\n\r{cSet the mobile race to assign immunities.\n\r",ch);
    return FALSE;
    }
    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( imm_flags, argument ) ) != NO_FLAG )
	{
	    pMob->imm_flags ^= value;
	    stc( "{cImmunity toggled.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: imm {C[{cflags{C]{c\n\r"
		  "Type '? imm' for a list of flags.\n\r", ch );
    return FALSE;
}

MEDIT( medit_res )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( !is_exact_name( ch->name, MUD_OWNER )
    && !is_exact_name(ch->name,MUD_COIMP)
    //&& !is_exact_name(ch->name,MUD_SIS)
    && !is_exact_name(ch->name,MUD_HB) )
    {
    stc("{RERROR!{c Manual editting of immunities is not supported.\n\r{cSet the mobile race to assign immunities.\n\r",ch);
    return FALSE;
    }

    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( res_flags, argument ) ) != NO_FLAG )
	{
	    pMob->res_flags ^= value;
	    stc( "{cResistance toggled.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: res {C[{cflags{C]{c\n\r"
		  "Type '? res' for a list of flags.\n\r", ch );
    return FALSE;
}

MEDIT( medit_vuln )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( !is_exact_name( ch->name, MUD_OWNER )
    && !is_exact_name(ch->name,MUD_COIMP)
    //&& !is_exact_name(ch->name,MUD_SIS)
    && !is_exact_name(ch->name,MUD_HB) )
    {
    stc("{RERROR!{c Manual editting of immunities is not supported.\n\r{cSet the mobile race to assign immunities.\n\r",ch);
    return FALSE;
    }
    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( vuln_flags, argument ) ) != NO_FLAG )
	{
	    pMob->vuln_flags ^= value;
	    stc( "{cVulnerability toggled.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: vuln {C[{cflags{C]{c\n\r"
		  "Type '? vuln' for a list of flags.\n\r", ch );
    return FALSE;
}

MEDIT( medit_material )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  material {C[{cstring{C]{c\n\r", ch );
	return FALSE;
    }

    free_string( pMob->material );
    pMob->material = str_dup( argument );

    stc( "{cMaterial set.\n\r", ch);
    return TRUE;
}

MEDIT( medit_off )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( off_flags, argument ) ) != NO_FLAG )
	{
	    pMob->off_flags ^= value;
	    stc( "{cOffensive behaviour toggled.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: off {C[{cflags{C]{c\n\r"
		  "Type '? off' for a list of flags.\n\r", ch );
    return FALSE;
}

MEDIT( medit_size )
{
    MOB_INDEX_DATA *pMob;
    int value;

    if ( argument[0] != '\0' )
    {
	EDIT_MOB( ch, pMob );

	if ( ( value = flag_value( size_flags, argument ) ) != NO_FLAG )
	{
	    pMob->size = value;
	    stc( "{cSize set.\n\r", ch );
	    return TRUE;
	}
    }

    stc( "{cSyntax: size [size]\n\r"
		  "Type '? size' for a list of sizes.\n\r", ch );
    return FALSE;
}

MEDIT( medit_hitdice )
{
    static char syntax[] = "{cSyntax:  hitdice <number> d <type> + <bonus>\n\r";
    char *num, *type, *bonus, *cp;
    MOB_INDEX_DATA *pMob;

    if ( !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP))
    {
    stc("{RERROR!{c Manual editting of hitdice is not supported.\n\r{cSet the mobile level to assign manadice.\n\r",ch);
    return FALSE;
    }

    EDIT_MOB( ch, pMob );

    if ( argument[0] == '\0' )
    {
	stc( syntax, ch );
	return FALSE;
    }

    num = cp = argument;

    while ( isdigit( *cp ) ) ++cp;
    while ( *cp != '\0' && !isdigit( *cp ) )  *(cp++) = '\0';

    type = cp;

    while ( isdigit( *cp ) ) ++cp;
    while ( *cp != '\0' && !isdigit( *cp ) ) *(cp++) = '\0';

    bonus = cp;

    while ( isdigit( *cp ) ) ++cp;
    if ( *cp != '\0' ) *cp = '\0';

    if ( ( !is_number( num   ) || atoi( num   ) < 1 )
    ||   ( !is_number( type  ) || atoi( type  ) < 1 )
    ||   ( !is_number( bonus ) || atoi( bonus ) < 0 ) )
    {
	stc( syntax, ch );
	return FALSE;
    }

    pMob->hit[DICE_NUMBER] = atoi( num   );
    pMob->hit[DICE_TYPE]   = atoi( type  );
    pMob->hit[DICE_BONUS]  = atoi( bonus );

    stc( "{cHitdice set.\n\r", ch );
    return TRUE;
}

MEDIT( medit_manadice )
{
    static char syntax[] = "{cSyntax:  manadice <number> d <type> + <bonus>\n\r";
    char *num, *type, *bonus, *cp;
    MOB_INDEX_DATA *pMob;

    if ( !is_exact_name( ch->name, MUD_OWNER ) && !is_exact_name(ch->name,MUD_COIMP))
    {
    stc("{RERROR!{c Manual editting of manadice is not supported.\n\r{cSet the mobile level to assign manadice.\n\r",ch);
    return FALSE;
    }

    EDIT_MOB( ch, pMob );

    if ( argument[0] == '\0' )
    {
	stc( syntax, ch );
	return FALSE;
    }

    num = cp = argument;

    while ( isdigit( *cp ) ) ++cp;
    while ( *cp != '\0' && !isdigit( *cp ) )  *(cp++) = '\0';

    type = cp;

    while ( isdigit( *cp ) ) ++cp;
    while ( *cp != '\0' && !isdigit( *cp ) ) *(cp++) = '\0';

    bonus = cp;

    while ( isdigit( *cp ) ) ++cp;
    if ( *cp != '\0' ) *cp = '\0';

    if ( !( is_number( num ) && is_number( type ) && is_number( bonus ) ) )
    {
	stc( syntax, ch );
	return FALSE;
    }

    if ( ( !is_number( num   ) || atoi( num   ) < 1 )
    ||   ( !is_number( type  ) || atoi( type  ) < 1 )
    ||   ( !is_number( bonus ) || atoi( bonus ) < 0 ) )
    {
	stc( syntax, ch );
	return FALSE;
    }

    pMob->mana[DICE_NUMBER] = atoi( num   );
    pMob->mana[DICE_TYPE]   = atoi( type  );
    pMob->mana[DICE_BONUS]  = atoi( bonus );

    stc( "{cManadice set.\n\r", ch );
    return TRUE;
}

MEDIT( medit_damdice )
{
    static char syntax[] = "{cSyntax:  damdice <number> d <type> + <bonus>\n\r";
    char *num, *type, *bonus, *cp;
    MOB_INDEX_DATA *pMob;

    if ( !is_exact_name( ch->name, MUD_OWNER )&& !is_exact_name(ch->name,MUD_COIMP) )
    {
    stc("{RERROR!{c Manual editting of damdice is not supported.\n\r{cSet the mobile level to assign damdice.\n\r",ch);
    return FALSE;
    }

    EDIT_MOB( ch, pMob );

    if ( argument[0] == '\0' )
    {
	stc( syntax, ch );
	return FALSE;
    }

    num = cp = argument;

    while ( isdigit( *cp ) ) ++cp;
    while ( *cp != '\0' && !isdigit( *cp ) )  *(cp++) = '\0';

    type = cp;

    while ( isdigit( *cp ) ) ++cp;
    while ( *cp != '\0' && !isdigit( *cp ) ) *(cp++) = '\0';

    bonus = cp;

    while ( isdigit( *cp ) ) ++cp;
    if ( *cp != '\0' ) *cp = '\0';

    if ( !( is_number( num ) && is_number( type ) && is_number( bonus ) ) )
    {
	stc( syntax, ch );
	return FALSE;
    }

    if ( ( !is_number( num   ) || atoi( num   ) < 1 )
    ||   ( !is_number( type  ) || atoi( type  ) < 1 )
    ||   ( !is_number( bonus ) || atoi( bonus ) < 0 ) )
    {
	stc( syntax, ch );
	return FALSE;
    }

    pMob->damage[DICE_NUMBER] = atoi( num   );
    pMob->damage[DICE_TYPE]   = atoi( type  );
    pMob->damage[DICE_BONUS]  = atoi( bonus );

    stc( "{cDamdice set.\n\r", ch );
    return TRUE;
}


MEDIT( medit_race )
{
    MOB_INDEX_DATA *pMob;
    int race;

    if ( argument[0] != '\0'
    && ( race = race_lookup( argument ) ) != 0 )
    {
	EDIT_MOB( ch, pMob );

	pMob->race = race;
	pMob->act	  |= race_table[race].act;
	pMob->affected_by |= race_table[race].aff;
	pMob->off_flags   |= race_table[race].off;
	pMob->imm_flags   |= race_table[race].imm;
	pMob->res_flags   |= race_table[race].res;
	pMob->vuln_flags  |= race_table[race].vuln;
	pMob->form        |= race_table[race].form;
	pMob->parts       |= race_table[race].parts;

	stc( "{cRace set.\n\r", ch );
	return TRUE;
    }

    if ( argument[0] == '?' )
    {

	stc( "{cAvailable races are:", ch );

	for ( race = 0; race_table[race].name != NULL; race++ )
	{
	    if ( ( race % 3 ) == 0 )
		stc( "{c\n\r", ch );
	    ptc(ch, "{c %-15s", race_table[race].name );
	}

	stc( "{c\n\r", ch );
	return FALSE;
    }

    stc( "{cSyntax:  race [race]\n\r"
		  "Type 'race ?' for a list of races.\n\r", ch );
    return FALSE;
}


MEDIT( medit_position )
{
    MOB_INDEX_DATA *pMob;
    char arg[MIL];
    int value;

    argument = one_argument( argument, arg );

    switch ( arg[0] )
    {
    default:
	break;

    case 'S':
    case 's':
	if ( str_prefix( arg, "start" ) )
	    break;

	if ( ( value = flag_value( position_flags, argument ) ) == NO_FLAG )
	    break;

	EDIT_MOB( ch, pMob );

	pMob->start_pos = value;
	stc( "{cStart position set.\n\r", ch );
	return TRUE;

    case 'D':
    case 'd':
	if ( str_prefix( arg, "default" ) )
	    break;

	if ( ( value = flag_value( position_flags, argument ) ) == NO_FLAG )
	    break;

	EDIT_MOB( ch, pMob );

	pMob->default_pos = value;
	stc( "{cDefault position set.\n\r", ch );
	return TRUE;
    }

    stc( "{cSyntax:  position [start/default] [position]\n\r"
		  "Type '? position' for a list of positions.\n\r", ch );
    return FALSE;
}


MEDIT( medit_gold )
{
    MOB_INDEX_DATA *pMob;

    EDIT_MOB(ch, pMob);

    if ( !is_exact_name( ch->name, MUD_OWNER )&& !is_exact_name(ch->name,MUD_COIMP) )
    {
    stc("{RERROR!{c Manual editting of wealth is not supported.\n\r{cSet the mobile wealth to assign parts.\n\r",ch);
    return FALSE;
    }
    if ( argument[0] == '\0' || !is_number( argument ) )
    {
	stc( "{cSyntax:  wealth {C[{cnumber{C]{c\n\r", ch );
	return FALSE;
    }

    pMob->wealth = atoi( argument );

    stc( "{cWealth set.\n\r", ch);
    return TRUE;
}

MEDIT( medit_hitroll )
{
    MOB_INDEX_DATA *pMob;

    if ( !is_exact_name( ch->name, MUD_OWNER )&& !is_exact_name(ch->name,MUD_COIMP) )
    {
    stc("{RERROR!{c Manual editting of hitroll is not supported.\n\r{cSet the mobile level and act to assign hitroll.\n\r",ch);
    return FALSE;
    }

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' || !is_number( argument ) )
    {
	stc( "{cSyntax:  hitroll {C[{cnumber{C]{c\n\r", ch );
	return FALSE;
    }

    pMob->hitroll = atoi( argument );

    stc( "{cHitroll set.\n\r", ch);
    return TRUE;
}

void show_liqlist(CHAR_DATA *ch)
{
    int liq;
    BUFFER *buffer;
    char buf[MSL];

    buffer = new_buf();

    for ( liq = 0; liq_table[liq].liq_name != NULL; liq++)
    {
	if ( (liq % 21) == 0 )
	    add_buf(buffer,"{CName                 Color          Proof Full Thirst Food Ssize\n\r");

	sprintf(buf, "{c%-20s %-14s %5d %4d %6d %4d %5d{c\n\r",
		liq_table[liq].liq_name,liq_table[liq].liq_color,
		liq_table[liq].liq_affect[0],liq_table[liq].liq_affect[1],
		liq_table[liq].liq_affect[2],liq_table[liq].liq_affect[3],
		liq_table[liq].liq_affect[4] );
	add_buf(buffer,buf);
    }

    page_to_char(buf_string(buffer),ch);
    free_buf(buffer);

return;
}

void show_damlist(CHAR_DATA *ch)
{
    int att;
    BUFFER *buffer;
    char buf[MSL];

    buffer = new_buf();

    for ( att = 0; attack_table[att].name != NULL; att++)
    {
	if ( (att % 21) == 0 )
	    add_buf(buffer,"{CName                 Noun\n\r");

	sprintf(buf, "{c%-20s %-20s{c\n\r",
		attack_table[att].name,attack_table[att].noun );
	add_buf(buffer,buf);
    }

    page_to_char(buf_string(buffer),ch);
    free_buf(buffer);

return;
}

MEDIT( medit_group )
{
    MOB_INDEX_DATA *pMob;
    MOB_INDEX_DATA *pMTemp;
    char arg[MSL];
    char buf[MSL];
    int temp;
    BUFFER *buffer;
    bool found = FALSE;

    EDIT_MOB(ch, pMob);

    if ( argument[0] == '\0' )
    {
    	stc( "{cSyntax: group {C[{cnumber{C]{c\n\r", ch);
    	stc( "{c        group show {C[{cnumber{C]{c\n\r", ch);
    	return FALSE;
    }

    if (is_number(argument))
    {
	pMob->group = atoi(argument);
    	stc( "{cGroup set.\n\r", ch );
	return TRUE;
    }

    argument = one_argument( argument, arg );

    if ( !strcmp( arg, "show" ) && is_number( argument ) )
    {
	if (atoi(argument) == 0)
	{
		stc( "{cAre you crazy?\n\r", ch);
		return FALSE;
	}

	buffer = new_buf ();

    	for (temp = 0; temp < 65536; temp++)
    	{
    		pMTemp = get_mob_index(temp);
    		if ( pMTemp && ( pMTemp->group == atoi(argument) ) )
    		{
			found = TRUE;
    			sprintf( buf, "{C[{c%5ld{C]{c %s\n\r", pMTemp->vnum, pMTemp->player_name );
			add_buf( buffer, buf );
    		}
    	}

	if (found)
		page_to_char( buf_string(buffer), ch );
	else
		stc( "{cNo mobs in that group.\n\r", ch );

	free_buf( buffer );
        return FALSE;
    }

    return FALSE;
}

REDIT( redit_delete )
{
	ROOM_INDEX_DATA *pRoom, *pRoom2;
	RESET_DATA *pReset;
	EXIT_DATA *ex;
	OBJ_DATA *Obj, *obj_next;
	CHAR_DATA *wch, *wnext;
	EXTRA_DESCR_DATA *pExtra;
	char arg[MIL];
	char buf[MSL];
	int index, i, iHash, rcount, ecount, mcount, ocount, edcount;

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  redit delete [vnum]\n\r", ch );
		return FALSE;
	}

	one_argument( argument, arg );

	if( is_number( arg ) )
	{
		index = atoi( arg );
		pRoom = get_room_index( index );
	}
	else
		{
		send_to_char( "That is not a number.\n\r", ch );
		return FALSE;
	}

	if( !pRoom )
	{
		send_to_char( "No such room.\n\r", ch );
		return FALSE;
	}

	/* Move the player out of the room. */
	if( ch->in_room->vnum == index )
	{
		send_to_char( "Moving you out of the room"
			" you are deleting.\n\r", ch);
		if( is_fighting(ch) )
			stop_all_fighting( ch );

		char_from_room( ch );
		char_to_room( ch, get_room_index( 3001 ) );
		ch->was_in_room = ch->in_room;
	}

	SET_BIT( pRoom->area->area_flags, AREA_CHANGED );

	/* Count resets. They are freed by free_room_index. */
	rcount = 0;

	for( pReset = pRoom->reset_first; pReset; pReset = pReset->next )
	{
		rcount++;
	}

	/* Now contents */
	ocount = 0;
	for( Obj = pRoom->contents; Obj; Obj = obj_next )
	{
		obj_next = Obj->next_content;

		extract_obj( Obj );
		ocount++;
	}

	/* Now PCs and Mobs */
	mcount = 0;
	for( wch = pRoom->people; wch; wch = wnext )
	{
		wnext = wch->next_in_room;
		if( IS_NPC( wch ) )
		{
			extract_char( wch, TRUE );
			mcount++;
		}
		else
			{
			send_to_char( "This room is being deleted. Moving"
				" you somewhere safe.\n\r", ch );
			if( is_fighting(wch) )
				stop_all_fighting( wch );

			char_from_room( wch );

			/* Oval Plaza */
			char_to_room( wch, get_room_index( 3001 ) );
			wch->was_in_room = wch->in_room;
		}
	}

	/* unlink all exits to the room. */
	ecount = 0;
	for( iHash = 0; iHash < MAX_KEY_HASH; iHash++ )
	{
		for( pRoom2 = room_index_hash[iHash]; pRoom2; pRoom2 = pRoom2->next )
		{
			for( i = 0; i <= MAX_DIR; i++ )
			{
				if( !( ex = pRoom2->exit[i] ) )
					continue;

				if( pRoom2 == pRoom )
				{
					/* these are freed by free_room_index */
					ecount++;
					continue;
				}

				if( ex->u1.to_room == pRoom )
				{
					free_exit( pRoom2->exit[i] );
					pRoom2->exit[i] = NULL;
					SET_BIT( pRoom2->area->area_flags, AREA_CHANGED );
					ecount++;
				}
			}
		}
	}

	/* count extra descs. they are freed by free_room_index */
	edcount = 0;
	for ( pExtra = pRoom->extra_descr; pExtra; pExtra = pExtra->next )
	{
		edcount++;
	}

	if( top_vnum_room == index )
		for( i = 1; i < index; i++ )
			if( get_room_index( i ) )
				top_vnum_room = i;

	top_room--;

	unlink_room_index( pRoom );

	pRoom->area = NULL;
	pRoom->vnum = 0;

	free_room_index( pRoom );

	/* Na na na na! Hey Hey Hey, Good Bye! */

	sprintf( buf, "Removed room vnum %d, %d resets, %d extra "
		"descriptions and %d exits.\n\r", index, rcount, edcount, ecount );
	send_to_char( buf, ch );
	sprintf( buf, "%d objects and %d mobiles were extracted "
		"from the room.\n\r", ocount, mcount );
	send_to_char( buf, ch );
	edit_done(ch);

	return TRUE;
}

REDIT( redit_url )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);
	return FALSE;
}


REDIT( redit_owner )
{
    ROOM_INDEX_DATA *pRoom;

    EDIT_ROOM(ch, pRoom);

    if ( argument[0] == '\0' )
    {
	stc( "{cSyntax:  owner {C[{cowner{C]{c\n\r", ch );
	stc( "{c         owner none\n\r", ch );
	return FALSE;
    }

    free_string( pRoom->owner );
    if (!str_cmp(argument, "none"))
    	pRoom->owner = str_dup("");
    else
	pRoom->owner = str_dup( argument );

    stc( "{cOwner set.\n\r", ch );
    return TRUE;
}

MEDIT ( medit_addmprog )
{
  int value;
  MOB_INDEX_DATA *pMob;
  PROG_LIST *list;
  PROG_CODE *code;
  char trigger[MSL];
  char phrase[MSL];
  char num[MSL];

  EDIT_MOB(ch, pMob);
  argument=one_argument(argument, num);
  argument=one_argument(argument, trigger);
  argument=one_argument(argument, phrase);

  if (!is_number(num) || trigger[0] =='\0' || phrase[0] =='\0' )
  {
        stc("{cSyntax:   addmprog {C[{cvnum{C]{c {C[{ctrigger{C] [{cphrase{C]{c\n\r",ch);
        return FALSE;
  }

  if ( (value = flag_value (mprog_flags, trigger) ) == NO_FLAG)
  {
        stc("{cValid flags are:\n\r",ch);
        show_help( ch, "mprog");
        return FALSE;
  }

  if ( ( code = get_prog_index (atoi(num), PRG_MPROG ) ) == NULL)
  {
        stc("{cNo such MOBProgram.\n\r",ch);
        return FALSE;
  }

  list                  = new_mprog();
  list->vnum            = atoi(num);
  list->trig_type       = value;
  list->trig_phrase     = str_dup(phrase);
  list->code            = code->code;
  SET_BIT(pMob->mprog_flags,value);
  list->next            = pMob->mprogs;
  pMob->mprogs          = list;

  stc( "{cMprog Added.\n\r",ch);
  return TRUE;
}

MEDIT ( medit_delmprog )
{
    MOB_INDEX_DATA *pMob;
    PROG_LIST *list;
    PROG_LIST *list_next;
    char mprog[MSL];
    int value;
    int cnt = 0;

    EDIT_MOB(ch, pMob);

    one_argument( argument, mprog );
    if (!is_number( mprog ) || mprog[0] == '\0' )
    {
       stc("{cSyntax:  delmprog {C[{c#mprog{C]{c\n\r",ch);
       return FALSE;
    }

    value = atoi ( mprog );

    if ( value < 0 )
    {
        stc("{cOnly non-negative mprog-numbers allowed.\n\r",ch);
        return FALSE;
    }

    if ( !(list= pMob->mprogs) )
    {
        stc("{cMEdit:  Non existant mprog.\n\r",ch);
        return FALSE;
    }

    if ( value == 0 )
    {
	REMOVE_BIT(pMob->mprog_flags, pMob->mprogs->trig_type);
        list = pMob->mprogs;
        pMob->mprogs = list->next;
        free_mprog( list );
    }
    else
    {
        while ( (list_next = list->next) && (++cnt < value ) )
                list = list_next;

        if ( list_next )
        {
		REMOVE_BIT(pMob->mprog_flags, list_next->trig_type);
                list->next = list_next->next;
                free_mprog(list_next);
        }
        else
        {
                stc("{cNo such mprog.\n\r",ch);
                return FALSE;
        }
    }

    stc("{cMprog removed.\n\r", ch);
    return TRUE;
}

REDIT( redit_room )
{
	ROOM_INDEX_DATA *room;
	int value;

	EDIT_ROOM(ch, room);

	if ( (value = flag_value( room_flags, argument )) == NO_FLAG )
	{
		stc( "{cSyntax: room {C[{cflags{C]{c\n\r", ch );
		return FALSE;
	}

        TOGGLE_BIT(room->room_flags, value);
	stc( "{cRoom flags toggled.\n\r", ch );
        return TRUE;
}

REDIT( redit_sector )
{
	ROOM_INDEX_DATA *room;
	int value;

	EDIT_ROOM(ch, room);

	if ( (value = flag_value( sector_flags, argument )) == NO_FLAG )
	{
		stc( "{cSyntax: sector {C[{ctype{C]{c\n\r", ch );
		stc( "{cType ? sector for a list of valid types.\n\r", ch );
		return FALSE;
	}

	room->sector_type = value;
	stc( "{cSector type set.\n\r", ch );

	return TRUE;
}

OEDIT ( oedit_addoprog )
{
  int value;
  OBJ_INDEX_DATA *pObj;
  PROG_LIST *list;
  PROG_CODE *code;
  char trigger[MAX_STRING_LENGTH];
  char phrase[MAX_STRING_LENGTH];
  char num[MAX_STRING_LENGTH];

  EDIT_OBJ(ch, pObj);
  argument=one_argument(argument, num);
  argument=one_argument(argument, trigger);
  argument=one_argument(argument, phrase);

  if (!is_number(num) || trigger[0] =='\0' || phrase[0] =='\0' )
  {
        send_to_char("Syntax:   addoprog [vnum] [trigger] [phrase]\n\r",ch);
        return FALSE;
  }

  if ( (value = flag_value (oprog_flags, trigger) ) == NO_FLAG)
  {
        send_to_char("Valid flags are:\n\r",ch);
        show_help( ch, "oprog");
        return FALSE;
  }

  if ( ( code =get_prog_index (atoi(num), PRG_OPROG ) ) == NULL)
  {
        send_to_char("No such OBJProgram.\n\r",ch);
        return FALSE;
  }

  list                  = new_oprog();
  list->vnum            = atoi(num);
  list->trig_type       = value;
  list->trig_phrase     = str_dup(phrase);
  list->code            = code->code;
  SET_BIT(pObj->oprog_flags,value);
  list->next            = pObj->oprogs;
  pObj->oprogs          = list;

  send_to_char( "Oprog Added.\n\r",ch);
  return TRUE;
}

OEDIT ( oedit_deloprog )
{
    OBJ_INDEX_DATA *pObj;
    PROG_LIST *list;
    PROG_LIST *list_next;
    char oprog[MAX_STRING_LENGTH];
    int value;
    int cnt = 0;

    EDIT_OBJ(ch, pObj);

    one_argument( argument, oprog );
    if (!is_number( oprog ) || oprog[0] == '\0' )
    {
       send_to_char("Syntax:  deloprog [#oprog]\n\r",ch);
       return FALSE;
    }

    value = atoi ( oprog );

    if ( value < 0 )
    {
        send_to_char("Only non-negative oprog-numbers allowed.\n\r",ch);
        return FALSE;
    }

    if ( !(list= pObj->oprogs) )
    {
        send_to_char("OEdit:  Non existant oprog.\n\r",ch);
        return FALSE;
    }

    if ( value == 0 )
    {
	REMOVE_BIT(pObj->oprog_flags, pObj->oprogs->trig_type);
        list = pObj->oprogs;
        pObj->oprogs = list->next;
        free_oprog( list );
    }
    else
    {
        while ( (list_next = list->next) && (++cnt < value ) )
                list = list_next;

        if ( list_next )
        {
		REMOVE_BIT(pObj->oprog_flags, list_next->trig_type);
                list->next = list_next->next;
                free_oprog(list_next);
        }
        else
        {
                send_to_char("No such oprog.\n\r",ch);
                return FALSE;
        }
    }

    send_to_char("Oprog removed.\n\r", ch);
    return TRUE;
}

REDIT ( redit_addrprog )
{
  int value;
  ROOM_INDEX_DATA *pRoom;
  PROG_LIST *list;
  PROG_CODE *code;
  char trigger[MAX_STRING_LENGTH];
  char phrase[MAX_STRING_LENGTH];
  char num[MAX_STRING_LENGTH];

  EDIT_ROOM(ch, pRoom);
  argument=one_argument(argument, num);
  argument=one_argument(argument, trigger);
  argument=one_argument(argument, phrase);

  if (!is_number(num) || trigger[0] =='\0' || phrase[0] =='\0' )
  {
        send_to_char("Syntax:   addrprog [vnum] [trigger] [phrase]\n\r",ch);
        return FALSE;
  }

  if ( (value = flag_value (rprog_flags, trigger) ) == NO_FLAG)
  {
        send_to_char("Valid flags are:\n\r",ch);
        show_help( ch, "rprog");
        return FALSE;
  }

  if ( ( code =get_prog_index (atoi(num), PRG_RPROG ) ) == NULL)
  {
        send_to_char("No such ROOMProgram.\n\r",ch);
        return FALSE;
  }

  list                  = new_rprog();
  list->vnum            = atoi(num);
  list->trig_type       = value;
  list->trig_phrase     = str_dup(phrase);
  list->code            = code->code;
  SET_BIT(pRoom->rprog_flags,value);
  list->next            = pRoom->rprogs;
  pRoom->rprogs          = list;

  send_to_char( "Rprog Added.\n\r",ch);
  return TRUE;
}

REDIT ( redit_delrprog )
{
    ROOM_INDEX_DATA *pRoom;
    PROG_LIST *list;
    PROG_LIST *list_next;
    char rprog[MAX_STRING_LENGTH];
    int value;
    int cnt = 0;

    EDIT_ROOM(ch, pRoom);

    one_argument( argument, rprog );
    if (!is_number( rprog ) || rprog[0] == '\0' )
    {
       send_to_char("Syntax:  delrprog [#rprog]\n\r",ch);
       return FALSE;
    }

    value = atoi ( rprog );

    if ( value < 0 )
    {
        send_to_char("Only non-negative rprog-numbers allowed.\n\r",ch);
        return FALSE;
    }

    if ( !(list= pRoom->rprogs) )
    {
        send_to_char("REdit:  Non existant rprog.\n\r",ch);
        return FALSE;
    }

    if ( value == 0 )
    {
	REMOVE_BIT(pRoom->rprog_flags, pRoom->rprogs->trig_type);
        list = pRoom->rprogs;
        pRoom->rprogs = list->next;
        free_rprog( list );
    }
    else
    {
        while ( (list_next = list->next) && (++cnt < value ) )
                list = list_next;

        if ( list_next )
        {
		REMOVE_BIT(pRoom->rprog_flags, list_next->trig_type);
                list->next = list_next->next;
                free_rprog(list_next);
        }
        else
        {
                send_to_char("No such rprog.\n\r",ch);
                return FALSE;
        }
    }

    send_to_char("Rprog removed.\n\r", ch);
    return TRUE;
}

/** Guild Editor Functions.
 */

/** Function: gedit_flags
  * Descr   : Sets the various flags associated with guilds.
  * Returns : True/False if changed.
  * Syntax  : flags flag_id...
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_flags )
{
  CLAN_DATA *pClan;
  long value = 0;

  if ( argument[0] != '\0' )
  {
    EDIT_GUILD( ch, pClan );

    if ( ( value = flag_value( guild_flags, argument ) ) != NO_FLAG )
    {
      pClan->flags ^= value;
      send_to_char( "Guild flag(s) toggled.\n\r", ch);
      return TRUE;
    }
  }

  send_to_char( "Syntax: flag [flag ID]\n\r"
                "Type '? guild' for a list of valid flags.\n\r", ch );
  return FALSE;
}

GEDIT( gedit_save )
{
	save_guilds( ch, "" );
	send_to_char( "Guilds saved.\n\r", ch);
	return TRUE;
}

/** Function: gedit_rank
  * Descr   : Sets the guild rank name
  * Returns : True/False if changed.
  * Syntax  : rank rank_id rank_name
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_rank )
{
  CLAN_DATA *pClan;
  char arg1[4];


  EDIT_GUILD(ch, pClan);

  argument = one_argument(argument, arg1);

  if (is_number(arg1) && atoi(arg1) <= MAX_RANK)
  {
    int value;

    value = atoi(arg1) -1;

    if (argument[0] != '\0')
    {
      free_string(pClan->rank[value].rankname);
      pClan->rank[value].rankname = str_dup( argument );
      send_to_char("Rank name changed.\n\r", ch);
      return TRUE;
    }
    else if ( !str_cmp( argument, "clear" ) )
    {
	free_string(pClan->rank[value].rankname);
	pClan->rank[value].rankname = str_dup("");
	stc("Cleared.\n\r", ch);
	return TRUE;
    }

  }

  send_to_char("Syntax: rank rank# newname\n\r", ch);
  return FALSE;
}

GEDIT( gedit_related )
{
	CLAN_DATA *pClan;
	sh_int clan;
	char arg1[4];

	EDIT_GUILD(ch, pClan);

	argument = one_argument(argument, arg1);

	if (is_number(arg1) && atoi(arg1) <= MAX_GUILD_RELATIONS)
	{
		int value;
		/* FIXME add a way to remove a guild relation */

		value = atoi(arg1) -1;

	    if (argument[0] != '\0')
	    {
			clan = clan_lookup( argument );
			/* 1 is Loner so we need better than that. -- Dalsor */
			if ( clan <= 1 )
			{
				stc( "That doesn't appear to be a valid guild.\n\r", ch );
				return FALSE;
			}
			free_string(pClan->related[value]);
			pClan->related[value] = str_dup( clan_table[clan].name );
			ptc( ch, "Relation %d name changed to %s.\n\r", value+1, clan_table[clan].name );
			return TRUE;
		}
		else
		{
			stc( "Syntax: relation relation# guildname\n\r", ch);
			return FALSE;
		}
	}
	stc( "Syntax: relation relation# guildname\n\r", ch);
	return FALSE;
}

GEDIT( gedit_chats )
{
        CLAN_DATA *pClan;
        sh_int clan;
        char arg1[4];

        EDIT_GUILD(ch, pClan);

        argument = one_argument(argument, arg1);

        if (is_number(arg1) && atoi(arg1) <= MAX_GUILD_RELATIONS)
        {
                int value;
                /* FIXME add a way to remove a guild relation */

                value = atoi(arg1) -1;

            if (argument[0] != '\0')
            {
                        clan = clan_lookup( argument );
                        /* 1 is Loner so we need better than that. -- Dalsor */
                        if ( clan <= 1 )
                        {
                                stc( "That doesn't appear to be a valid guild.\n\r", ch );
                                return FALSE;
                        }
                        free_string(pClan->chats[value]);
                        pClan->chats[value] = str_dup( clan_table[clan].name );
                        ptc( ch, "Chat %d name changed to %s.\n\r", value+1, clan_table[clan].name );
                        return TRUE;
                }
                else
                {
                        stc( "Syntax: relation relation# guildname\n\r", ch);
                        return FALSE;
                }
        }
        stc( "Syntax: chats chat# guildname\n\r", ch);
        return FALSE;
}
GEDIT( gedit_lists )
{
        CLAN_DATA *pClan;
        sh_int clan;
        char arg1[4];

        EDIT_GUILD(ch, pClan);

        argument = one_argument(argument, arg1);

        if (is_number(arg1) && atoi(arg1) <= MAX_GUILD_RELATIONS)
        {
                int value;
                /* FIXME add a way to remove a guild relation */

                value = atoi(arg1) -1;

            if (argument[0] != '\0')
            {
                        clan = clan_lookup( argument );
                        /* 1 is Loner so we need better than that. -- Dalsor */
                        if ( clan <= 1 )
                        {
                                stc( "That doesn't appear to be a valid guild.\n\r", ch );
                                return FALSE;
                        }
                        free_string(pClan->lists[value]);
                        pClan->lists[value] = str_dup( clan_table[clan].name );
                        ptc( ch, "GList %d name changed to %s.\n\r", value+1, clan_table[clan].name );
                        return TRUE;
                }
                else
                {
                        stc( "Syntax: glists relation# guildname\n\r", ch);
                        return FALSE;
                }
        }
        stc( "Syntax: chats chat# guildname\n\r", ch);
        return FALSE;
}

/** Function: gedit_skill
  * Descr   : Sets the skills associated with the various ranks.
  * Returns : True/False if changed & notice if skill dosent exist.
  * Syntax  : skill rank_# skill_name
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_skill )
{
  CLAN_DATA *pClan;
  char arg1[4];


  EDIT_GUILD(ch, pClan);

  argument = one_argument(argument, arg1);

  if (is_number(arg1) && atoi(arg1) <= MAX_RANK)
  {
    int value;

    value = atoi(arg1) -1;

    if (argument[0] != '\0')
    {
      free_string(pClan->rank[value].skillname);
      pClan->rank[value].skillname = str_dup( argument );
      if (skill_lookup(argument) == -1)
        send_to_char("Notice: That skill does not exist.\n\r", ch);
      send_to_char("Skill changed.\n\r", ch);
      return TRUE;
    }

  }

  send_to_char("Syntax: skill rank# newskill\n\r", ch);
  return FALSE;
}


/** Function: gedit_create
  * Descr   : Creates a new, empty guild if room is available in the array.
  * Returns : True/False if created
  * Syntax  : (n/a)
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_create )
{
  int i = 0;

  for (i=1; i < MAX_CLAN; i++) /* just loop through and find an open slot */
  {
    if (clan_lookup(clan_table[i].name) == 0)
    break;
  }

  if (i <= MAX_CLAN) /* open slot */
  {
    CLAN_DATA *pClan;
    int x;

    clan_table[i].name = str_dup("New Guild");
    clan_table[i].who_name = str_dup("New Guild");

    for (x = 0; x < MAX_RANK; x++)
    {
      clan_table[i].rank[x].rankname = str_dup("Empty");
      clan_table[i].rank[x].skillname = str_dup("");
    }

    pClan = &clan_table[i]; /* return new clan data */
    ch->desc->pEdit = pClan;
    send_to_char("Guild created.\n\r", ch);
    return TRUE;
  }

  send_to_char("No room to create a new guild. Increase MAX_CLAN\n\r", ch);
  return FALSE;
}


/** Function: gedit_list
  * Descr   : List's all of the current guilds, including those not yet
  *         : saved to the data file, and those marked for deletion.
  * Returns : list of guilds.
  * Syntax  : (N/A)
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_list )
{
  char buf[MIL];
  BUFFER *buffer;
  int i;

  buffer = new_buf();

  sprintf(buf, "Num  Guild Name  Flags\n\r-----------------------------------------------\n\r");
  add_buf(buffer, buf);

  for (i=1; i <= MAX_CLAN; i++)
  {
    if (clan_table[i].name != NULL && clan_table[i].name[0] != '\0')
    {
      sprintf(buf,"[%2d]  %s\t Flags:\t[%s]\n\r",
                  i,
                  clan_table[i].name,
                  flag_string( guild_flags, clan_table[i].flags ) );
      add_buf(buffer, buf);
    }
  }

  page_to_char( buf_string(buffer), ch );
  free_buf(buffer);
  return FALSE;
}


/** Function: gedit_show
  * Descr   : Displays currently selected guild data to the screen.
  * Returns : (N/A)
  * Syntax  : (N/A)
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_show )
{
	CLAN_DATA *pClan;

	char buf[MIL];
	BUFFER *buffer;
	int i, x;

	EDIT_GUILD(ch, pClan);

	buffer = new_buf();

	sprintf(buf, "{CName:{x %s %s %s %s {CWho Name:{x %-10s  {CFlags:{x [%s]\n\r",
		pClan->name, pClan->tag,
		IS_SET(pClan->flags, GUILD_CHANGED) ? "{c[{B*{c]{x" : "",
		IS_SET(pClan->flags, GUILD_DELETED) ? "{R[{rX{R]{x" : "",
		pClan->who_name, flag_string( guild_flags, pClan->flags ) );
	add_buf(buffer, buf);

	sprintf(buf, "{CRooms:{x Hall [%ld] Morgue [%ld] Temple [%ld]\n\r",
		pClan->room[0], pClan->room[1], pClan->room[2]);
	add_buf(buffer, buf);

	sprintf( buf, "{CChannels:{x %s%s%s %s%s%s{x\n\r",
		pClan->lefty, pClan->gcom, pClan->righty,
		pClan->lefty, pClan->gcomooc, pClan->righty );
	add_buf( buffer, buf );

	sprintf(buf,"{CML Rights:{x [Guild: %s] [Deguild: %s] [Promote: %s] [Demote: %s]\n\r",
		(pClan->ml[0]==1) ? "True" : "False",
		(pClan->ml[1]==1) ? "True" : "False",
		(pClan->ml[2]==1) ? "True" : "False",
		(pClan->ml[3]==1) ? "True" : "False");
	add_buf(buffer, buf);

	for (i=0; i < MAX_RANK; i++)
	{
		sprintf(buf,"{C[{c%2d{C]{x %-25s %s\n\r",
			i+1,
			(pClan->rank[i].rankname  != '\0') ? pClan->rank[i].rankname : "None",
			(pClan->rank[i].skillname != '\0') ? pClan->rank[i].skillname : "");
		add_buf(buffer, buf);
	}

	add_buf(buffer,"--------------------------------------------------------------\n\r");

	sprintf( buf, "Relations:\n\r" );
	add_buf(buffer, buf);

	x = 1;
	for ( i = 0; i < MAX_GUILD_RELATIONS; i++ )
	{
		sprintf( buf, "[%2d] %-13s",i+1, pClan->related[i] );
		add_buf(buffer, buf);
		x++;
		if ( x == 5 )
		{
			add_buf(buffer, "\n\r");
			x = 1;
		}
	}
	sprintf( buf, "\n\rChats:\n\r" );
	add_buf(buffer, buf);

	x = 1;
	for ( i = 0; i < MAX_GUILD_RELATIONS; i++ )
	{
		sprintf( buf, "[%2d] %-13s",i+1, pClan->chats[i] );
		add_buf(buffer, buf);
		x++;
		if ( x == 5 )
		{
			add_buf(buffer, "\n\r");
			x = 1;
		}
	}

	sprintf( buf, "\n\rGLists:\n\r" );
	add_buf(buffer, buf);

	x = 1;
	for ( i = 0; i < MAX_GUILD_RELATIONS; i++ )
	{
		sprintf( buf, "[%2d] %-13s",i+1, pClan->lists[i] );
		add_buf(buffer, buf);
		x++;
		if ( x == 5 )
		{
			add_buf(buffer, "\n\r");
			x = 1;
		}
	}

	x = 1;
	page_to_char( buf_string(buffer), ch );
	free_buf(buffer);
	return FALSE;
}


/** Function: gedit_name
  * Descr   : Changes the name of the currently selected guild.
  * Returns : True/False if modified
  * Syntax  : name new_name
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_name )
{
	CLAN_DATA *pClan;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  name [name]\n\r", ch );
		return FALSE;
	}

	if (clan_lookup(argument) != 0)
	{
		send_to_char("That guild allready exists.\n\r", ch);
		return FALSE;
	}

	if (pClan->name[0] != '\0')
		free_string( pClan->name );

	pClan->name = str_dup( argument );

	send_to_char( "Name set.\n\r", ch );
	return TRUE;
}
/** Function: gedit_righty/lefty
  * Descr   : Changes the brackets that surround the guild channel command..
  * Returns : True/False if modified.
  * Syntax  : gedit lefty colors + brackets + colors
  * Written : v1.0 08/31
  * Author  : Dalsor
  */
GEDIT( gedit_lefty )
{
        CLAN_DATA *pClan;

        EDIT_GUILD(ch, pClan);

        if ( argument[0] == '\0' )
        {
                send_to_char( "Syntax:  lefty [string]\n\r", ch );
                return FALSE;
        }
        if (pClan->lefty[0] != '\0')
                free_string( pClan->lefty );

        pClan->lefty = str_dup( argument );
        send_to_char( "Lefty set. Don't forget righty!\n\r", ch );
        return TRUE;
}
GEDIT( gedit_righty )
{
        CLAN_DATA *pClan;

        EDIT_GUILD(ch, pClan);

        if ( argument[0] == '\0' )
        {
                send_to_char( "Syntax:  righty [string]\n\r", ch );
                return FALSE;
        }
        if (pClan->righty[0] != '\0')
                free_string( pClan->righty );

        pClan->righty = str_dup( argument );
        send_to_char( "Righty set. Don't forget lefty!\n\r", ch );
        return TRUE;
}


/** Function: gedit_whoname
  * Descr   : Changes the who_name of the currently selected guild.
  * Returns : True/False if modified.
  * Syntax  : whoname new_who_name
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_whoname )
{
	CLAN_DATA *pClan;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  whoname [name]\n\r", ch );
		return FALSE;
	}

	if (pClan->who_name[0] != '\0')
		free_string( pClan->who_name );

	pClan->who_name = str_dup( argument );

	send_to_char( "Who Name set.\n\r", ch );
	return TRUE;
}

/** Function: gedit_gcom
  * Descr   : Changes the guild channel command of the currently selected guild.
  * Returns : True/False if modified.
  * Syntax  : gcom channel_name
  * Written : v1.0 08/03
  * Author  : Dalsor
  */
GEDIT( gedit_gcom )
{
	CLAN_DATA *pClan;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  GCom [command name no color]\n\r", ch );
		return FALSE;
	}

	if (pClan->gcom[0] != '\0')
		free_string( pClan->gcom );

	pClan->gcom = str_dup( argument );

	send_to_char( "Guild Channel Command set.\n\r", ch );
	return TRUE;
}

/** Function: gedit_gcomooc
  * Descr   : Changes the guild ooc channel command of the currently selected guild.
  * Returns : True/False if modified.
  * Syntax  : gcomooc channel_name
  * Written : v1.0 08/03
  * Author  : Dalsor
  */
GEDIT( gedit_gcomooc )
{
	CLAN_DATA *pClan;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  GComOOC [command name no color]\n\r", ch );
		return FALSE;
	}

	if (pClan->gcomooc[0] != '\0')
		free_string( pClan->gcomooc );

	pClan->gcomooc = str_dup( argument );

	send_to_char( "Guild OOC Channel Command set.\n\r", ch );
	return TRUE;
}

GEDIT( gedit_tag )
{
	CLAN_DATA *pClan;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  tag [tag string]\n\r", ch );
		return FALSE;
	}

	if (pClan->tag && pClan->tag != NULL && pClan->tag[0] != '\0')
		free_string( pClan->tag );

	pClan->tag = str_dup( argument );

	send_to_char( "Tag set.\n\r", ch );
	return TRUE;
}

/** Function: gedit_rooms
  * Descr   : Changes the vnum of the room(s) selected.
  * Returns : True/False if modified.
  * Syntax  : rooms [hall new#|morgue new#|temple new#]
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_rooms )
{
	CLAN_DATA *pClan;
	char arg1[10], arg2[10], arg3[10], arg4[10], arg5[10], arg6[10];
	bool set = FALSE;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  rooms [hall #|morgue #|temple #]\n\r", ch );
		return FALSE;
	}

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
	argument = one_argument(argument, arg3);
	argument = one_argument(argument, arg4);
	argument = one_argument(argument, arg5);
	argument = one_argument(argument, arg6);

  /* kinda of convoluted, I know, but this seemed the easiest way
     to allow for any combination of commands. ie:
       hall 1 morgue 2 temple 3
       morgue 2 hall 3 temple 1
         or even
       hall 1
       morgue 2 hall 1
  */
	if (!str_cmp(arg1,"hall") && is_number(arg2) )
	{ pClan->room[0] = atoi(arg2); set = TRUE; }
	else
	if (!str_cmp(arg1,"morgue") && is_number(arg2) )
	{ pClan->room[1] = atoi(arg2); set = TRUE; }
	else
	if (!str_cmp(arg1,"temple") && is_number(arg2) )
	{ pClan->room[2] = atoi(arg2); set = TRUE; }

	if (!str_cmp(arg3,"hall") && is_number(arg4) )
	{ pClan->room[0] = atoi(arg4); set = TRUE; }
	else
	if (!str_cmp(arg3,"morgue") && is_number(arg4) )
	{ pClan->room[1] = atoi(arg4); set = TRUE; }
	else
	if (!str_cmp(arg3,"temple") && is_number(arg4) )
	{ pClan->room[2] = atoi(arg4); set = TRUE; }

	if (!str_cmp(arg5,"hall") && is_number(arg6) )
	{ pClan->room[0] = atoi(arg6); set = TRUE; }
	else
	if (!str_cmp(arg5,"morgue") && is_number(arg6) )
	{ pClan->room[1] = atoi(arg6); set = TRUE; }
	else
	if (!str_cmp(arg5,"temple") && is_number(arg6) )
	{ pClan->room[2] = atoi(arg6); set = TRUE; }

	if (set)
	{
		send_to_char("Room(s) set.\n\r", ch);
		return TRUE;
	}

	return FALSE;
}


/** Function: gedit_ml
  * Descr   : Changes the mortal leader settings.
  * Returns : True/False if modified
  * Syntax  : ml [true/false|true/false|true/false|true/false]
  * Written : v1.0 3/98
  * Author  : Gary McNickle <gary@dharvest.com>
  */
GEDIT( gedit_ml )
{
	CLAN_DATA *pClan;
	char arg1[6], arg2[6], arg3[6], arg4[6];
	bool set = FALSE;

	EDIT_GUILD(ch, pClan);

	if ( argument[0] == '\0' )
	{
		send_to_char( "Syntax:  ml TRUE|FALSE TRUE|FALSE TRUE|FALSE TRUE|FALSE\n\r", ch );
		return FALSE;
	}

	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
	argument = one_argument(argument, arg3);
	argument = one_argument(argument, arg4);

	if (!str_prefix(arg1,"true") )
	{ pClan->ml[0] = TRUE; set = TRUE; }
	else
	if (!str_prefix(arg1, "false") )
    { pClan->ml[0] = FALSE; set = TRUE; }

	if (!str_prefix(arg2,"true") )
	{ pClan->ml[1] = TRUE; set = TRUE; }
	else
	if (!str_prefix(arg2, "false") )
	{ pClan->ml[1] = FALSE; set = TRUE; }

	if (!str_prefix(arg3,"true") )
	{ pClan->ml[2] = TRUE; set = TRUE; }
	else
	if (!str_prefix(arg3, "false") )
	{ pClan->ml[2] = FALSE; set = TRUE; }

	if (!str_prefix(arg4,"true") )
	{ pClan->ml[3] = TRUE; set = TRUE; }
	else
	if (!str_prefix(arg4, "false") )
	{ pClan->ml[3] = FALSE; set = TRUE; }

	if (set)
	{
		send_to_char("Mortal Leader traits set.\n\r", ch);
		return TRUE;
	}

	return FALSE;
}
