/***************************************************************************
                      _____  __      __  __      __
                     /  _  \/  \    /  \/  \    /  \
                    /  /_\  \   \/\/   /\   \/\/   /
                   /    |    \        /  \        /
                   \____|__  /\__/\  /    \__/\  /
                           \/      \/          \/
    As the Wheel Weaves based on ROM 2.4. Original code by Dalsor.
    See changes.log for a list of changes from the original ROM code.
    Credits for code created by other authors have been left
 	intact at the head of each function.
    Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,
    Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.
    Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael
    Chastain, Michael Quan, and Mitchell Tse.
    In order to use any part of this Merc Diku Mud, you must comply with
    both the original Diku license in 'license.doc' as well the Merc
    license in 'license.txt'.  In particular, you may not remove either of
    these copyright notices.
    Much time and thought has gone into this software and you are
    benefitting.  We hope that you share your changes too.  What goes
    around, comes around.
	ROM 2.4 is copyright 1993-1998 Russ Taylor
	ROM has been brought to you by the ROM consortium
	    Russ Taylor (rtaylor@hypercube.org)
	    Gabrielle Taylor (gtaylor@hypercube.org)
	    Brian Moore (zump@rom.org)
	By using this code, you have agreed to follow the terms of the
	ROM license, in the file Rom24/doc/rom.license
 ***************************************************************************/
#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include "aww.h"
#include "interp.h"
#include "tables.h"
#include "faire.h"
#include "recycle.h"


const   struct  faire_angreal3_type faire_angreal3_table [] =
{
	{ "a rising sun" },
	{ "a setting sun" },
	{ "a rising moon" },
	{ "a crescent moon" },
	{ "a quarter moon" },
	{ "a half moon" },
	{ "a full moon" },
	{ "a lightning bolt" },
	{ "a storm cloud" },
	{ "a blooming rose" },
	{ "a cascading waterfall" },
	{ "a thorny vine" },
	{ "a creeping ivy" },
	{ "a towering oak" },
	{ "a weeping willow" },
	{ "a smith's hammer" },
	{ "a horseman's mace" },
	{ "a knight's shield" },
	{ "a downy feather" },
	{ "an eye" },
};
const   struct  faire_angreal2_type faire_angreal2_table [] =
{
	{ "a roaring lion" },
	{ "a roaring lioness" },
	{ "a sleeping lion" },
	{ "a sleeping lioness" },
	{ "a heavy maned lion" },
	{ "a snarling lioness" },
	{ "a howling wolf" },
	{ "a snarling wolf" },
	{ "a running wolf" },
	{ "a feeding wolf" },
	{ "a howling wolf hound" },
	{ "a snarling wolf hound" },
	{ "a running wolf hound" },
	{ "a snarling panther" },
	{ "a sleeping panther" },
	{ "a running panther" },
	{ "a snarling bear" },
	{ "a growling bear" },
	{ "a charging bear" },
	{ "a sleeping bear" },
};
const   struct  faire_angreal1_type faire_angreal1_table [] =
{
	{ "a rearing stallion" }, /*1*/
	{ "a rearing warhorse" }, /*2*/
	{ "a short fat man" }, /*3*/
	{ "a praying woman" }, /*4*/
	{ "a kneeling man" }, /*5*/
	{ "a kneeling woman" }, /*6*/
	{ "a praying child" }, /*7*/
	{ "a kneeling child" }, /*8*/
	{ "a kneeling girl child" }, /*9*/
	{ "a praying girl child" }, /*10*/
	{ "a kneeling boy child" }, /*11*/
	{ "a praying boy child" }, /*12*/
	{ "a buzzing bumblebee" }, /*13*/
	{ "an angry hornet" }, /*14*/
	{ "an angry wasp" }, /*15*/
	{ "a sleeping cat" }, /*16*/
	{ "a curled up kitten" }, /*17*/
	{ "a playing kitten" }, /*18*/
	{ "a fat cat" }, /*19*/
	{ "a sleeping kitten" }, /*20*/
};
const   struct  faire_drink_type faire_drink_table [] =
{
    { "water"}, /*1*/
    { "beer"},
    { "red wine"},
    { "ale"},
    { "dark ale"}, /*5*/
    { "whisky"},
    { "pale ale"},
    { "lager"},
    { "XXX"},
    { "grog"}, /*10*/
    { "warm cider"},
    { "milk"},
    { "tea"},
    { "kaf"},
    { "blood"}, /*15*/
    { "salt water"},
    { "Illian wine"},
    { "Tairen wine"},
    { "Caemlyn wine"},
    { "white wine"}, /*20*/
    { "champagne"},
    { "mead"},
    { "rose wine"},
    { "Tar Valon wine"},
    { "spirits"}, /*25*/
    { "berry juice"},
    { "orange juice"},
    { "oosquai"},
    { "brandy"},
    { "homebrew"}, /*30*/
    { "boilermaker"},
    { "icewine"},
    { "watered wine"},
    { "sherry"},
    { "honey mead"}, /*35*/
    { "rum"},
    { "cordial"}, /*37*/
};
const   struct  faire_bow_wood_type faire_bow_wood_table [] =
{
	{ "pine"      },/*1*/
	{ "leatherleaf" },
	{ "rosewood"  },
	{ "satinwood"   },
	{ "hemlock"   },
	{ "poplar"    },
	{ "fir"       },
	{ "beech"     },
	{ "spruce"    },
	{ "cypress"   },
	{ "juniper"   },
	{ "walnut"    },
	{ "cedar"     },
	{ "maple"     },
	{ "mahogany"  },
	{ "teak"      },
	{ "ebony"      },
	{ "redwood"     },
	{ "willow"      },
	{ "yew"         },/*20*/
};
const   struct  faire_pipewood_type faire_pipewood_table [] =
{
	{ "corncob"   },
	{ "pine"      },
	{ "fir"       },
	{ "cedar"     },
	{ "cypress"   },
	{ "juniper"   },
	{ "spruce"    },
	{ "hemlock"   },
	{ "maple"     },
	{ "mahogany"  },
	{ "poplar"    },
	{ "walnut"    },
	{ "rosewood"  },
	{ "beech"     },
	{ "teak"      },
	{ "ebony"      },
	{ "redwood"     },
	{ "satinwood"   },
	{ "leatherleaf" },
	{ "willow"      },
	{ "yew"         },
};
const   struct  faire_heraldry_type faire_heraldry_table [] =
{
	{ "a lion"             },
	{ "a hawk"             },
	{ "a trefoil leaf"     },
	{ "an eagle"           },
	{ "three frogs"        },
	{ "a gryphon"          },
	{ "a serpent"          },
	{ "a winged steed"     },
	{ "three lillies"      },
	{ "two crossed arrows" },
	{ "a sword and axe"    },
	{ "a sword and hammer" },
	{ "an axe and hammer"  },
	{ "a mailled fist"     },
	{ "a crane"            },
};
const   struct  faire_shield_armor_type faire_shield_armor_table [] =
{
	{ "buckler shield", 3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "round shield",   4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "oval shield",    4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "heater shield",  4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "round bukler",   4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "oval buckler",   4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "kite shield",    4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "great shield",   4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "body shield",    5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_HANDS },
};
const   struct  faire_hand_armor_type faire_hand_armor_table [] =
{
	{ "pair of leather gloves",        3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of mail mittens",          4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of ringmail gloves",       4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of chainmail gloves",      4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of ringmail gauntlets",    4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of chainmail gauntlets",   4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of scalemail gauntlets",   4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of platemail gauntlets",   4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_HANDS },
	{ "pair of articulated gauntlets", 5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_HANDS },
};
const   struct  faire_back_armor_type faire_back_armor_table [] =
{
	{ "set of leather back armor",        3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of brigandine back armor",     4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of ringmail back armor",       4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of chainmail back armor",      4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of scalemail back armor",      4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of half plate back armor",     4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of plate and chain back armor",4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "set of full plate back armor",     4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_BACK },
	{ "fully articulated backplate",      5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_BACK },
};
const   struct  faire_arm_armor_type faire_arm_armor_table [] =
{
	{ "set of leather sleeves",     3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of ringmail sleeves",    4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of chainmail sleeves",   4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of scalemail sleeves",   4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of ringmail couters",    4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of chainmail couters",   4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of scalemail couters",   4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of platemail couters",   4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_ARMS },
	{ "set of articulated couters", 5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_ARMS },
};
const   struct  faire_head_armor_type faire_head_armor_table [] =
{
	{ "leather cap",    3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "steel cap",      4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "pot helmet",     4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "basinet helmet", 4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "sallet helmet",  4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "burgonet helm",  4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "armet helmet",   4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "visored helm",   4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_HEAD },
	{ "great helm",     5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_HEAD },
};
const   struct  faire_body_armor_type faire_body_armor_table [] =
{
	{ "leather coat",             3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "ringmail coat",            4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "leather cuirass",          4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "ringmail cuirass",         4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "chainmail cuirass",        4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "scalemail coat",           4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "scalemail cuirass",        4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "breastplate",              4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_BODY },
	{ "articulated breastplate",  5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_BODY },
};
const   struct  faire_waist_armor_type faire_waist_armor_table [] =
{
	{ "belt",           3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "leather belt",   4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "ring belt",      4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "chain belt",     4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "scale belt",     4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "plated belt",    4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "leather girdle", 4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "chain girdle",   4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_WAIST },
	{ "plated girdle",  5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_WAIST },
};
const   struct  faire_foot_armor_type faire_foot_armor_table [] =
{
	{ "shoes",          3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "ankle boots",    4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "calf boots",     4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "thigh boots",    4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "knee boots",     4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "marching boots", 4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "riding boots",   4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "armored boots",  4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_FEET },
	{ "plated boots",   5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_FEET },
};
const   struct  faire_calf_armor_type faire_calf_armor_table [] =
{
	{ "calf binding",       3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf shank",         4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf harness",       4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf guard",         4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf plate",         4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf chause",        4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf garter",        4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "calf cuisse",        4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_CALF },
	{ "articulated greave", 5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_CALF },
};
const   struct  faire_thigh_armor_type faire_thigh_armor_table [] =
{
	{ "thigh binding",      3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "thigh shank",        4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "thigh harness",      4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "thigh guard",        4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "thigh plate",        4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "thigh chause",       4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "greave",             4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "thigh cuisse",       4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_THIGH },
	{ "articulated greave", 5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_THIGH },
};
const   struct  faire_ankle_armor_type faire_ankle_armor_table [] =
{
	{ "ankle binding",      3, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "ankle shank",        4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "ankle harness",      4, 3, 3, 3, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "ankle guard",        4, 4, 3, 3, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "scaled anklet",      4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "ankle chause",       4, 4, 4, 3, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "ankle bard",         4, 4, 5, 4, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "ankle cuisse",       4, 4, 5, 5, ITEM_TAKE|ITEM_WEAR_ANKLE },
	{ "plated anklet",      5, 5, 5, 5, ITEM_TAKE|ITEM_WEAR_ANKLE }
};
const   struct  faire_polearm_type faire_polearm_table [] =
{
	{ "poleaxe",  100, 1, 10, "cleave", WEAPON_SHARP },
	{ "spetum",   110, 1, 11, "cleave", WEAPON_SHARP },
	{ "guisarme", 120, 1, 12, "cleave", WEAPON_SHARP },
	{ "glaive",   130, 1, 13, "cleave", WEAPON_SHARP },
	{ "bardiche", 140, 1, 14, "cleave", WEAPON_SHARP },
	{ "pike",     150, 1, 15, "pierce", WEAPON_SHARP },
	{ "voulge",   160, 1, 16, "cleave", WEAPON_SHARP },
	{ "halberd",  170, 1, 17, "cleave", WEAPON_SHARP },
};
const   struct  faire_whip_type faire_whip_table [] =
{
	{ "quirt",            90,  1, 9,  "lash",  WEAPON_SHARP },
	{ "lash",             100, 1, 10, "flay",  WEAPON_SHARP },
	{ "flog",             110, 1, 11, "flog",  WEAPON_SHARP },
	{ "whip",             120, 1, 12, "whip",  WEAPON_SHARP },
	{ "cat-o'nine tails", 130, 1, 13, "sting", WEAPON_SHARP },
};
const   struct  faire_flail_type faire_flail_table [] =
{
	{ "scourge",          90,  1, 9,  "whap",   WEAPON_SHARP },
	{ "sap",              100, 1, 10, "whap",   WEAPON_SHARP },
	{ "blackjack",        110, 1, 11, "thwack", WEAPON_SHARP },
	{ "bludgeon",         120, 1, 12, "thwack", WEAPON_SHARP },
	{ "horseman's flail", 130, 1, 13, "crunch", WEAPON_SHARP },
	{ "footman's flail",  140, 1, 14, "crunch", WEAPON_SHARP },
	{ "morning-star",     150, 1, 15, "maul",   WEAPON_SHARP },
};
const   struct  faire_axe_type faire_axe_table [] =
{
	{ "hatchet",    90,  1, 9,  "chop",   WEAPON_SHARP },
	{ "hand-axe",   100, 1, 10, "chop",   WEAPON_SHARP },
	{ "wood-axe",   110, 1, 11, "chop",   WEAPON_SHARP },
	{ "francisca",  120, 1, 12, "cleave", WEAPON_SHARP },
	{ "great-axe",  130, 1, 13, "cleave", WEAPON_SHARP },
	{ "battle-axe", 140, 1, 14, "sunder", WEAPON_SHARP },
	{ "war-axe",    150, 1, 15, "maul",   WEAPON_SHARP },
};
const   struct  faire_dagger_type faire_dagger_table [] =
{
	{ "knife",         50,  1, 5,  "slice",  WEAPON_SHARP },
	{ "stiletto",      60,  1, 6,  "stab",   WEAPON_SHARP },
	{ "dagger",        70,  1, 7,  "pierce", WEAPON_SHARP },
	{ "dirk",          80,  1, 8,  "slice",  WEAPON_SHARP },
	{ "poinard",       90,  1, 9,  "slice",  WEAPON_SHARP },
	{ "long knife",    100, 1, 10, "slash",  WEAPON_SHARP },
	{ "hunting knife", 110, 1, 11, "gash",   WEAPON_SHARP },
};
const   struct  faire_mace_type faire_mace_table [] =
{
	{ "horseman's mace",  80,  1, 8,  "beating", WEAPON_SHARP },
	{ "footman's mace",   90,  1, 9,  "beating", WEAPON_SHARP },
	{ "horseman's hammer",100, 1, 10, "pound",   WEAPON_SHARP },
	{ "footman's hammer", 110, 1, 11, "pound",   WEAPON_SHARP },
	{ "war hammer",       120, 1, 12, "crunch",  WEAPON_SHARP },
	{ "battle hammer",    130, 1, 13, "crush",   WEAPON_SHARP },
	{ "great hammer",     150, 1, 15, "crush",   WEAPON_SHARP },
};
const   struct  faire_staff_type faire_staff_table [] =
{
	{ "pole",          80,  1, 8,  "thwack",  WEAPON_SHARP },
	{ "rod",           90,  1, 9,  "thwock",  WEAPON_SHARP },
	{ "cane",          100, 1, 10, "beating", WEAPON_SHARP },
	{ "bo",            110, 1, 11, "pound",   WEAPON_SHARP },
	{ "stave",         120, 1, 12, "crunch",  WEAPON_SHARP },
	{ "staff",         130, 1, 13, "blow",    WEAPON_SHARP },
	{ "quarter-staff", 140, 1, 15, "whack",   WEAPON_SHARP },
};
const   struct  faire_spear_type faire_spear_table [] =
{
	{ "angon",   80,  1, 8,  "pierce", WEAPON_SHARP },
	{ "assegai", 90,  1, 9,  "lunge",  WEAPON_SHARP },
	{ "spear",   100, 1, 10, "thrust", WEAPON_SHARP },
	{ "pilum",   110, 1, 11, "pierce", WEAPON_SHARP },
	{ "javelin", 120, 1, 13, "pierce", WEAPON_SHARP },
	{ "lance",   150, 1, 15, "thrust", WEAPON_SHARP },
};
const   struct  faire_sword_type faire_sword_table [] =
{
	{ "longsword",     100, 1, 10, "slice",  WEAPON_SHARP },
	{ "shortword",     80,  1, 8,  "jab",    WEAPON_SHARP },
	{ "ringsword",     120, 1, 12, "slash",  WEAPON_SHARP },
	{ "broadsword",    110, 1, 11, "slash",  WEAPON_SHARP },
	{ "gladius",       90,  1, 9,  "pierce", WEAPON_SHARP },
	{ "cutlass",       90,  1, 9,  "slice",  WEAPON_SHARP },
	{ "sabre",         100, 1, 10, "slash",  WEAPON_SHARP },
	{ "rapier",        90,  1, 9,  "lunge",  WEAPON_SHARP },
	{ "cinqueda",      100, 1, 10, "slash",  WEAPON_SHARP },
	{ "spatha",        100, 1, 10, "slice",  WEAPON_SHARP },
	{ "falchion",      110, 1, 11, "cleave", WEAPON_SHARP },
	{ "scimitar",      110, 1, 11, "slash",  WEAPON_SHARP },
	{ "nimcha",        120, 1, 12, "chop",   WEAPON_SHARP },
	{ "hanger",        90,  1, 9,  "slice",  WEAPON_SHARP },
	{ "kastane",       90,  1, 9 , "slice",  WEAPON_SHARP },
	{ "yatagan",       100, 1, 10, "slash",  WEAPON_SHARP },
	{ "backsword",     100, 1, 10, "slash",  WEAPON_SHARP },
	{ "flamberge",     110, 1, 11, "slash",  WEAPON_SHARP },
	{ "bastard-sword", 120, 1, 12, "slash",  WEAPON_SHARP },
	{ "greatsword",    130, 1, 13, "chop",   WEAPON_SHARP },
	{ "war-sword",     140, 1, 14, "cleave", WEAPON_SHARP },
	{ "claymore",      150, 1, 15, "fury",   WEAPON_SHARP },
};
const   struct  faire_scents_type faire_scents_table [] =
{
	{ "{Bbl{bueber{Bry{x" }, /* 1 */
	{ "{Rstr{rawbe{Rrry{x" }, /* 2 */
	{ "{rra{Rspber{rry{x" }, /* 3 */
	{ "{mgr{Ma{mpe{x" }, /* 4 */
	{ "{wva{Wnil{wla{x" }, /* 5 */
	{ "{rc{Ri{wn{Wna{wm{Ro{rn{x" }, /* 6 */
	{ "{Mlav{men{Mder{x" }, /* 7 */
	{ "{rr{Ros{re{x" }, /* 8 */
	{ "{gp{Gin{ge{x" }, /* 9 */
	{ "{Rch{rer{Rry{x" }, /* 10 */
	{ "{Ga{gp{wp{gl{Ge{x" }, /* 11 */
	{ "{yhon{Yey{ws{Yuc{ykle{x" }, /* 12 */
	{ "{wgar{Wde{wnia{x" }, /* 13 */
	{ "{Wg{winge{Wr{x" }, /* 14 */
	{ "{geuc{Galyp{gtus{x" }, /* 15 */
	{ "{ysan{wdelw{yood{x" }, /* 16 */
	{ "{Yh{yone{Yy{x" }, /* 17 */
	{ "{wta{Yll{wow{x" }, /* 18 */
	{ "{bo{Bc{we{Wan {bb{Br{we{Weze{x" }, /* 19 */
	{ "{yp{Ye{wa{Wr{x" }, /* 20 */
	{ "{Yp{ye{Wa{wch{x" }, /* 21 */
};
const   struct  faire_guild_type faire_guild_table [] =
{
	{ "{WW{wa{Wrd{we{Wr{x" },          /* 1 */
	{ "{wT{Wo{ww{We{wr {WG{Wu{wa{Wr{wd{x" },     /* 2 */
	{ "{yAe{ws {WSe{wd{yai{x" },       /* 3 */
	{ "{WY{wo{gungli{wn{Wg{x" },       /* 4 */
	{ "{yBl{Yig{Wht G{Yua{yrd{x" },/* 5 */
	{ "{rB{Rand of the {rR{Red {rH{Rand{x" }, /* 6 */
	{ "{rTh{Rie{yf T{Rak{rer{x" },     /* 7 */
	{ "{aG{bl{ce{de{em{fa{gn{x" },         /* 8 */
	{ "{aG{bl{ce{de{ewom{fa{gn{x" },       /* 9 */
	{ "{wC{Whildren of the {wL{Wight{x" }, /* 10 */
	{ "{RQu{re{We{wn's G{Wu{ra{Rrd{x" },   /* 11 */
	{ "{MC{Ro{Yv{We{Gn{Ba{Cn{Rt{x" }, /* 12 */
	{ "{rA{Rs{Yha{R'{rm{Ra{Yn{x" },        /* 13 */
	{ "{bI{Bl{Cli{Ba{bn {bCo{Bmp{Ca{Bni{bon{x" },/* 14 */
	{ "{YKn{yitti{Yng {YC{yircl{Ye{x" }, /* 15 */
	{ "{YKi{Yndre{Yd{x" },         /* 16 */
	{ "{BDar{bkfri{Bend{x" },      /* 17 */
	{ "{BDre{badl{Bord{x" },       /* 18 */
	{ "{BL{Ce{Ygi{Co{Bn{x" },    /* 19 */
	{ "{YWi{ys{we{Y O{yn{we{x" },        /* 20 */
	{ "{bS{Bei{Ca {bD{Boo{Cn{x" },       /* 21 */
	{ "{WD{wu{Ca{cdhe {WM{wa{Ch{cdi{W'{cin{x" }, /* 22 */
	{ "{MSh{mae{W'{wen {MM{W'{mta{wal{x" },  /* 23 */
	{ "{WRa{yhi{Yen {WSo{yre{Yi{x" },    /* 24 */
	{ "{WSo{wvi{cn {WN{wa{ci{x" },       /* 25 */
	{ "{bF{Ba{wr {bDa{Bre{wis {bM{Ba{wi{x" },  /* 26 */
	{ "{cF{Ca{br {cAl{Cda{bzar {cD{Ci{bn{x" }, /* 27 */
	{ "{YH{ya{wm{Wa {YN{W'{yd{wo{Wre{x" },     /* 28 */
	{ "{bC{Bo{br {bDa{Br{bei{x" },       /* 29 */
	{ "{rAe{Rth{ran {rD{Ro{rr{x" },      /* 30 */
	{ "{mS{Mh{ma{W'{mm{Ma{md {mC{Mond{me{x" },   /* 31 */
	{ "{rTa{Rin {rSh{Rari{x" },      /* 32 */
	{ "{yC{Yla{yn C{Yhie{yf{x" },    /* 33 */
	{ "{gW{Ga{Yns{Gh{go{x" },     /* 34 */
	{ "{rTr{Roll{roc{x" },         /* 35 */
	{ "{rBl{Rood{x" },           /* 36 */
	{ "{rD{Rea{wt{Whw{wa{Rtc{rh{x" },      /* 37 */
	{ "{bH{Bi{wg{Wh {bL{Bo{wr{Wd{x" },       /* 38 */
	{ "{bH{Bi{wg{Wh {bL{Ba{wd{Wy{x" },       /* 39 */
	{ "{rSo{W'{Rjhin{x" },         /* 40 */
	{ "{wM{yo{Cr{ya{wt{x" },     /* 41 */
	{ "{MC{Ch{ca {WFa{ci{Cl{Me{x" },       /* 42 */
	{ "{DS{we{Wek{we{Dr{x" },       /* 43 */
	{ "{BS{Cu{Yl'd{Ca{Bm{x" },       /* 44 */
	{ "{rH{Da{Rilen{re{x" },       /* 45 */
	{ "{YCa{yrg{wo{Wm{wa{yst{Yer{x" },       /* 46 */
	{ "{cWi{Cn{Wd{wfi{Wn{Cd{cer{x" },       /* 47 */
	{ "{gSa{Gil{Wm{wis{Wt{Gre{gss{x" },       /* 48 */
	{ "{wN{Wo{wvice{x" },			/* 49 */
};
const   struct  faire_paintings_type faire_paintings_table [] =
{
	{ "a portrait of Ishara, the first Queen of Andor",
	  "ishara painting" },/* 1 */
	{ "an oil painting of the Great Wheel, pulsing as the True Source turns it",
	  "great wheel painting" },/* 2 */
	{ "a sketch of the Web of Destiny, depicted like a vast spider's web",
	  "destiny web sketch" },/* 3 */
	{ "an image of two gigantic statues, a man and a woman, each holding huge crystal globes aloft",
	  "crystal globe statue image" },/* 4 */
	{ "a brittle sketch on yellowed parchment showing ancient Paaran Disen through the artist's eyes",
	  "brittle paaran disen sketch" },/* 5 */
	{ "a beautiful rendition of the college of Collam Daan from the Age of Legends",
	  "collam daan rendition" },/*6*/
	{ "a tantalizing image of sho-wings and jo-cars speeding through a city from the Age of Legends",
	  "sho-wing jo-car image" },/*7*/
	{ "a painting of the Nym, Ogier and the Dah'shain Aiel seed singing a thriving field",
	  "nym ogier aiel painting" },/*8*/
	{ "a pefectly rendered painting of the Avendoraldera, the tree of life, in it's prime",
	  "avendoraldera painting" },/*9*/
	{ "a portait of Lews Therin Telamon wearing the ring of the Tamyrlin, summoning the Nine Rods of Dominion",
	  "lews therin telamon tamyrlin dominion portrait" },/*10*/
	{ "a painting of the tri-arched ter'angreal in the heart of the White Tower",
	  "tri-arched angreal painting" },/*11*/
	{ "a depiction of the destruction of the floating Sharom and the opening of the Bore",
	  "destruction sharom depiction" },/*12*/
	{ "an image of a thin line of liquid fire streaming from a man's hand, and the destruction of a city",
	  "liquid fire city destruction image" },/*13*/
	{ "an image of the crystal sword, Callandor, hanging in the Heart of the Stone",
	  "crystal callandor sword image" },/*14*/
	{ "a painting of the seven cuendillar seals surrounding the Bore",
	  "cuendillar seals painting" },/*15*/
	{ "a painting of Shayul Ghul, rising above the fog shrouded valleys of Thakan'dar",
	  "shayul ghul painting" },/*16*/
	{ "an image of the signing of the Fateful Concord, with Latra Posae at the head",
	  "fateful concord latra posae image" },/*17*/
	{ "a portrait of the thirteen Forsaken, shrouded in dark shadow",
	  "thirteen forsaken portait" },/*18*/
	{ "a depiction of the sealing of the Bore by Lews Therin and the Hundred Companions",
	  "bore sealing hundred companions depiction" },/*19*/
	{ "a portrait of Elan Morin Tedronai, or Ishamael, the Betrayer of Hope",
	  "ishamael portrait" },/*20*/
	{ "a portrait of Ishar Morrad Chuain, called Aginor",
	  "aginor portrait" },/*21*/
	{ "a portrait of Balthamel, born as Eval Ramman",
	  "balthamel portrait" },/*22*/
	{ "a portrait of Tel Janin Aellinsar, known as Sammael",
	  "sammael portrait" },/*23*/
	{ "a portrait of Ared Mosinel, called Rahvin",
	  "rahvin portrait" },/*24*/
	{ "a portrait of Duram Laddel Cham, or Be'lal the Netweaver",
	  "belal netweaver portrait" },/*25*/
	{ "a portrait of Barid Bel Medar, the man called Demandred",
	  "demandred portrait" },/*26*/
	{ "a portrait of Joar Addam Nessosin, known as Asmodean",
	  "asmodean portrait" },/*27*/
	{ "a portrait of Mierin Eronaile, or Lanfear, Daughter of the Night",
	  "lanfear portrait" },/*28*/
	{ "a portrait of Kamarile Maradim Nindar, or Graendal",
	  "graendal portrait" },/*29*/
	{ "a portrait of Nemene Damendar Boann, called Semirhage",
	  "semirhage portrait" },/*30*/
	{ "a portrait of Mesaana, born into the world as Saine Tarasind",
	  "mesaana portrait" },/*31*/
	{ "a portrait of Lillen Moiral, called Moghedien the Spider",
	  "moghedien portrait" },/*32*/
	{ "a painting of the signing of the Compact of the Ten Nations, headed by Queen Mabriam of Aramaelle",
	  "ten nations compact mabriam painting" },/*33*/
	{ "an artist's rendition of Manetheren, the Mountain Home",
	  "manetheren mountain home rendition" },/*34*/
	{ "a portrait of King Aemon of Manetheren, and Queen Eldrene at his side",
	  "aemon eldrene portrait" },/*35*/
	{ "an image of the shattering of Manetheren and the destruction of the shadow horde by balefire",
	  "manetheren shattering image" },/*36*/
	{ "a portrait of Artur Paendrag Tanreall, the Hawkwing at his coronation in Shandalle",
	  "artur hawkwing coronation portrait" },/*37*/
	{ "a portrait of Guaire Amalasan, the Second Dragon",
	  "quaire amalasan portrait" },/*38*/
	{ "a depiction of the Battle of Endersole between Artur Hawkwing and Guaire Amalasan",
	  "endersole battle depiction" },/*39*/
	{ "a painting of the siege of Tar Valon by the armies of the High King, Artur Hawkwing",
	  "tar valon siege painting" },/*40*/
	{ "a painting of a blood covered banner, the golden crane of Malkier",
	  "blood banner malkier painting" },/*41*/
	{ "a painting of an Aielman carrying a sapling called Avendesora to the Cairhienin",
	  "avendesora painting" },/*42*/
	{ "a painting of Laman Damodred, axe in hand and Avendesora split in twain",
	  "laman damodred avendesora painting" },/*43*/
	{ "a depiction of Laman Damodred's death at the Battle of the Shining Walls",
	  "shining walls battle laman damodred depiction" },/*44*/
	{ "a portrait of Rand al'Thor, the Dragon Reborn",
	  "rand dragon reborn portrait" },/*45*/
	{ "a portrait of Perrin ay'Barra, Lord Goldeneyes",
	  "perrin goldeneyes portrait" },/*46*/
	{ "a portrait of Matrim Cauthon, the Gambler",
	  "matrim cauthon gambler portrait" },/*47*/
	{ "a painting of an Aes Sedai, staff in hand, and her Gaidin wearing fancloth cloak",
	  "aes sedai gaidin painting" },/*48*/
	{ "a painting of an army of Aes Sedai, pitched against an army of Aes Sedai",
	  "aes sedai army painting"  },/*49*/
	{ "a depiction of the taking of Callandor by Rand al'Thor from the Stone of Tear",
	  "rand callandor stone tear depiction" },/*50*/
};
const   struct  faire_material_type faire_material_table [] =
{
	{ "a {ybr{Yon{yze{x",   20  },   /* 1 */
	{ "a {Yb{yras{Ys{x",    15  },    /* 2 */
	{ "a {wsi{Wlv{wer{x",   35  },   /* 3 */
	{ "a {Ygold{x",     45 },     /* 4 */
	{ "a {Wpla{wti{Wnum{x", 50 }, /* 5 */
};
const   struct  faire_metals_type faire_metals_table [] =
{
	{ "bronze",   10  },   /* 1 */
	{ "iron",    20  },    /* 2 */
	{ "steel",   30  },   /* 3 */
};
const   struct  faire_gem_type faire_gem_table [] =
{
	{ "{wp{Wearl{ws{x",     	150  },   /* 1 */
	{ "{gem{Geral{gds{x",   	300  },   /* 2 */
	{ "{rru{Rbi{res{x",   		500  },   /* 3 */
	{ "{Wdiamonds{x", 		5000 },   /* 4 */
	{ "{rf{Rir{re{w-{rd{Rrop{rs{x", 250  },   /* 5 */
};
const   struct  faire_mobs_type faire_mobs_table [] =
{
	{ "old man",            15900, SEX_MALE,   "andor",    "Andor" }, /* 1 */
	{ "old woman",          15901, SEX_FEMALE, "andor",    "Andor"}, /* 2 */
	{ "young man",          15902, SEX_MALE,   "domani",   "Arad Doman"},
	{ "old man",            15903, SEX_MALE,   "tuathaan", "Tinker" },
	{ "old woman",          15904, SEX_FEMALE, "tuathaan", "Tinker" },
	{ "young man",          15905, SEX_MALE,   "tuathaan", "Tinker" },
	{ "wise women",         15906, SEX_FEMALE, "domani",   "Bandar Eban" },
	{ "greedy man",         15907, SEX_MALE,   "domani",   "Arad Doman" },
	{ "loud man",           15908, SEX_MALE,   "cairhien", "Cairhien" },
	{ "wrinkled man",       15909, SEX_MALE,   "tarabon",  "Tanchico" },
	{ "wizened man",        15910, SEX_MALE,   "cairhien", "Cairhien" },
	{ "stone-faced man",    15911, SEX_MALE,   "shienar",  "Shienar" },
	{ "wrinkled woman",     15912, SEX_FEMALE, "shienar",  "Fal Dara" },
	{ "plump man",          15913, SEX_MALE,   "cairhien", "Cairhien" },
	{ "plump woman",        15914, SEX_FEMALE, "arafel",   "Shol Arbela" },
	{ "fat man",            15915, SEX_MALE,   "cairhien", "Cairhien" },
	{ "fat woman",          15916, SEX_FEMALE, "cairhien", "Cairhien Foregate" },
	{ "overweight man",     15917, SEX_MALE,   "altara",   "Altara" },
	{ "overweight woman",   15918, SEX_FEMALE, "tairen",   "Tear" },
	{ "skinny man",         15919, SEX_MALE,   "cairhien", "Cairehien" },
	{ "skinny woman",       15920, SEX_FEMALE, "cairhien", "Cairhien Foregate" },
	{ "slender man",        15921, SEX_MALE,   "amador",   "Amadicia" },
	{ "slender woman",      15922, SEX_FEMALE, "cairhien", "Cairhien Foregate" },
	{ "frail old man",      15923, SEX_MALE,   "cairhien", "Cairehien" },
	{ "frail old woman",    15924, SEX_FEMALE, "illian",   "Illian" },
	{ "plump man",          15925, SEX_MALE,   "andor",    "Andor" },
	{ "plump woman",        15926, SEX_FEMALE, "andor",    "Baerlon" },
	{ "fat man",            15927, SEX_MALE,   "andor",    "Whitebridge" },
	{ "fat woman",          15928, SEX_FEMALE, "andor",    "Four Kings" },
	{ "overweight man",     15929, SEX_MALE,   "andor",    "Market Sheran" },
	{ "overweight woman",   15930, SEX_FEMALE, "andor",    "Aringill" },
	{ "skinny man",         15931, SEX_MALE,   "andor",    "Carysford" },
	{ "skinny woman",       15932, SEX_FEMALE, "andor",    "Caemlyn" },
	{ "slender man",        15933, SEX_MALE,   "andor",    "Baerlon" },
	{ "slender woman",      15934, SEX_FEMALE, "altara"    "Ebou Dar" },
	{ "frail old man",      15935, SEX_MALE,   "andor",    "Four Kings" },
	{ "frail old woman",    15936, SEX_FEMALE, "andor",    "Andor" },
	{ "kind young man",     15937, SEX_MALE,   "cairhien", "Cairhien Foregate" },
	{ "kind young woman",   15938, SEX_FEMALE, "cairhien", "Cairhien" },
	{ "kindly old man",     15939, SEX_MALE,   "cairhien", "Cairhien Foregate" },
	{ "kindly old woman",   15940, SEX_FEMALE, "cairhien", "Cairhien" },
	{ "kind young man",     15941, SEX_MALE,   "andor",    "Whitebridge" },
	{ "kind young woman",   15942, SEX_FEMALE, "andor",    "Andor" },
	{ "kindly old man",     15943, SEX_MALE,   "andor",    "Caemlyn" },
	{ "kindly old woman",   15944, SEX_FEMALE, "andor",    "Whitebridge" },
	{ "brazen young man",   15945, SEX_MALE,   "shienar",  "Fal Moran" },
	{ "bold young man",     15946, SEX_MALE,   "shienar",  "Shienar" },
	{ "pretty young woman", 15947, SEX_FEMALE, "tarabon",  "Tarabon" },
	{ "pretty young lady",  15948, SEX_FEMALE, "tarabon",  "Tanchico" },
	{ "pretty young woman", 15949, SEX_FEMALE, "arafel",   "Arafel" },
	{ "pretty young lady",  15950, SEX_FEMALE, "arafel",   "Shol Arbela" },
	{ "pretty young woman", 15951, SEX_FEMALE, "kandor",   "Kandor" },
	{ "pretty young lady",  15952, SEX_FEMALE, "kandor",   "Chachin" },
	{ "pretty young woman", 15953, SEX_FEMALE, "mayene",   "Mayene" },
	{ "pretty young lady",  15954, SEX_FEMALE, "mayene",   "Mayene" },
	{ "pretty young woman", 15955, SEX_FEMALE, "gheldean", "Jehannah" },
	{ "pretty young lady",  15956, SEX_FEMALE, "gheldean", "Gheldean" },
	{ "smiling man",        15957, SEX_MALE,   "murandy",  "Murandy" },
	{ "grinning woman",     15958, SEX_FEMALE, "murandy",  "Lugard" },
	{ "grinning man",       15959, SEX_MALE,   "murandy",  "Tamille" },
	{ "frowning man",       15960, SEX_MALE,   "murandy",  "Murandy" },
	{ "frowning woman",     15961, SEX_FEMALE, "murandy",  "Lugard" },
	{ "smiling woman",      15962, SEX_FEMALE, "murandy",  "Tamille" },
	{ "cold-eyed man",      15963, SEX_MALE,   "tarvalon", "Tar Valon" },
	{ "cold-eyed woman",    15964, SEX_FEMALE, "tarvalon", "Darien" },
	{ "green-eyed man",     15965, SEX_MALE,   "tarvalon", "Tar Valon" },
	{ "blue-eyed lady",     15966, SEX_FEMALE, "tarvalon", "Jualdhe" },
	{ "rougish gentleman",  15967, SEX_MALE,   "saldaea",  "Saldaea" },
	{ "gentleman",          15968, SEX_MALE,   "saldaea",  "Maradon" },
	{ "retired solder",     15969, SEX_MALE,   "kandor",   "Kandor" },
	{ "old soldier",        15970, SEX_MALE,   "altara",   "Salidar" },
	{ "matronly woman",     15971, SEX_FEMALE, "murandy",  "Far Madding" },
	{ "matronly woman",     15972, SEX_FEMALE, "murandy",  "Lugard" },
	{ "matronly woman",     15973, SEX_FEMALE, "andor",    "Caemlyn" },
	{ "matronly woman",     15974, SEX_FEMALE, "cairhien", "Cairhien" },
	{ "matronly woman",     15975, SEX_FEMALE, "tarvalon", "Tar Valon" },
	{ "gentleman",          15976, SEX_MALE,   "andor",    "Whitebridge" },
	{ "gentleman",          15977, SEX_MALE,   "illian",   "Illian" },
	{ "merchant",           15978, SEX_MALE,   "andor",    "Andor" },
	{ "merchant",           15979, SEX_MALE,   "tairen",   "Tear" },
	{ "merchant",           15980, SEX_MALE,   "tairen",   "Godan" },
	{ "merchant",           15981, SEX_MALE,   "tarvalon", "Osenrein" },
	{ "merchantess",        15982, SEX_FEMALE, "tarvalon", "Daghain" },
	{ "merchantess",        15983, SEX_FEMALE, "tarvalon", "Luagde" },
	{ "merchantess",        15984, SEX_FEMALE, "tarabon",  "Elmora" },
	{ "merchantess",        15985, SEX_FEMALE, "murandy",  "Far Madding" },
	{ "monger",             15986, SEX_MALE,   "andor",    "Andor" },
	{ "monger",             15987, SEX_MALE,   "cairhien", "Cairhien" },
	{ "trader",             15988, SEX_MALE,   "andor",    "Whitebridge" },
	{ "trader",             15989, SEX_MALE,   "cairhien", "Cairhien Foregate" },
	{ "tradesman",          15990, SEX_MALE,   "andor",    "Baerlon" },
	{ "tradesman",          15991, SEX_MALE,   "andor",    "Four Kings" },
	{ "old dealer",         15992, SEX_MALE,   "andor",    "Whitebridge" },
	{ "old dealer",         15993, SEX_MALE,   "kandor",   "Kandor" },
	{ "crafty trader",      15994, SEX_MALE,   "mandor",   "Chachin" },
	{ "crafty trader",      15995, SEX_MALE,   "arafel",   "Shol Arbela" },
	{ "crafty merchant",    15996, SEX_MALE,   "andor",    "Caemlyn" },
	{ "crafty merchant",    15997, SEX_MALE,   "cairhien", "Cairhien" },
	{ "fox-eyed merchant",  15998, SEX_MALE,   "murandy",  "Lugard" },
	{ "fox-eyed merchant",  15999, SEX_MALE,   "domani",   "Arad Doman" },
};

const   struct  faire_location_type faire_location_table [] =
{
	{   "caemlyn",     "Caemlyn",     3029,  "a reddish-gold striped tent"  }, /* 0 */
	{   "aringill",    "Aringill",    605,   "a bright-gray striped tent"   }, /* 1 */
	{   "tarvalon",    "Tar Valon",   7007,  "a silvery-gray striped tent"  }, /* 2 */
	{   "cairhien",    "Cairhien",    5304,  "a bright green tent"        }, /* 3 */
	{   "illian",      "Illian",      4301,  "a bright blue tent"      }, /* 4 */
	{   "tear",        "Tear",        12047, "a sparkling gold tent"     }, /* 5 */
	{   "eboudar",     "Ebou Dar",    8037,  "a green and blue tent"     }, /* 6 */
	{   "lugard",      "Lugard",      1508,  "a blue and gray tent"     }, /* 7 */
	{   "whitebridge", "Whitebridge", 5617,  "a solid white tent"     }, /* 8 */
	{   "sholarbela",  "Shol Arbela", 26771, "a silver slashed tent"     }, /* 9 */
	{   "farmadding",  "Far Madding", 13817, "a gold striped tent"     }, /* 10 */
	{   "chachin",     "Chachin",     6739,  "a black slashed tent"     }, /* 11 */
	{   "jehannah",    "Jehannah",    27972, "a green and white tent"     }, /* 12 */
	{   "tanchico",    "Tanchico",    20042, "a green and gold white tent"     }, /* 13 */
	{   "maradon",     "Maradon",     27013, "a gleaming white tent"     }, /* 14 */
	{   "amador",      "Amador",      28160, "a pearly white tent"     }, /* 15 */
	{   NULL,        0   }
};

/* faires checklist */
/* random merchants, check
 * fortune teller (1,2)
 * raffle tickets and unique item (1)
 * tattoo artists (1,2)
 * jeweller
 * balancer, check
 * sharpener (1)
 * repairer, check
 * upgrader, check
 * poker game (1,2)
 * blackjack game (1,2)
 * hi-lo game (1,2)
 */
#define FairePortalVnum    15900
#define FaireSignVnum      15901
#define FaireRoomFirst     15900
#define FaireRoomLast      15999
#define FaireObjectVnum    15902
#define FaireMobVnum       15901
#define MaxFaireLocations  16
#define MaxFaireMobs       100
#define MaxFaireMaterials  5
#define MaxFaireGuilds     49
#define MaxFaireSwords     22
#define MaxFaireSpears     6
#define MaxFaireStaves     7
#define MaxFairePaintings  50
void init_faire_portal    args(( int location, int fLookup ));
void init_faire_rooms     args(( CHAR_DATA *ch, int location, int fLookup ));
void init_faire_mobs      args(( int location, int fLookup ));
void init_faire_objects   args(( CHAR_DATA *mob, int FaireLocation, int fLookup ));
int  rand_offensive_weave args(( void ));
int  rand_defensive_weave args(( void ));
int  rand_misc_weave      args(( void ));
bool chance               args(( int num ));
int FaireLocation = 0;
int FaireBankRoom = 0;
int FaireRepairRoom = 0;
int FaireUpgradeRoom = 0;
int FaireBalanceRoom = 0;
int FaireTattooRoom = 0;
int FaireStringRoom = 0;
bool MaleAngrealExists = FALSE;
bool FemaleAngrealExists = FALSE;
OBJ_DATA *faire_portal;
OBJ_DATA *faire_wayback;
OBJ_DATA *faire_sign;

void do_faire( CHAR_DATA *ch, char *argument )
{
	char arg1[MIL];
	char arg2[MIL];
	char arg3[MIL];
	char arg4[MIL];
    DESCRIPTOR_DATA *d;
	int fLookup = 0, ugCost = 0, repCost = 0, balCost = 0;
	bool found = FALSE;
	OBJ_DATA *upgrade;
	argument = one_argument( argument, arg1 );
	argument = one_argument( argument, arg2 );
	argument = one_argument( argument, arg3 );
	if ( arg1[0] == '\0' )
	{
		if ( ch->level < MAX_LEVEL )
		{
			stc("Syntax:  faire upgrade <item name> <modifier>\n\r",ch );
			stc("         The cost of an upgrade is 1000 gold for angreal.\n\r",ch );
			stc("         The cost of an upgrade is 100 gold for weapons and armor.\n\r",ch );
			stc("         You can only upgrade Weapons, Armors, and Angreal.\n\r",ch );
			stc("         Weapon upgrades increase by one *point*, NOT one rank.\n\r", ch );
			stc("Syntax:  faire repair <item name>\n\r", ch );
			stc("         The cost of a repair is 10 gold for one *point*.\n\r", ch );
			stc("Syntax:  faire balance <item name>\n\r", ch );
			stc("         The cost of a balance is 100 gold for two stones.\n\r", ch );
			stc("         Balancing allows greater upgrades of Weapons and Armor.\n\r",ch );
			stc("Syntax:  faire string <item name> <short|long|name> <string>\n\r", ch );
			stc("         String enable the changing of an item's short, long,.\n\r",ch );
			stc("         and name descriptions. One ticket is required per field.\n\r",ch );
			return;
		}
		stc("Syntax: faire create  <area file name>\n\r",ch);
		stc("Syntax: faire kill    <area file name>\n\r",ch);
		stc("Syntax: faire list    (list possible areas)\n\r",ch);
		stc("Syntax: faire upgrade <item name> <modifier>\n\r",ch );
		stc("Syntax: faire repair  <item name>\n\r", ch );
		stc("Syntax: faire balance <item name>\n\r", ch );
		stc("Syntax: faire string  <item name> <short|long|name> <string>\n\r", ch );
		return;
	}

	if ( !str_cmp( arg1, "string" ) || !str_cmp( arg1, "restring" ) )
	{
		OBJ_DATA *ticket, *ticket_next;
		bool found = FALSE;
		FILE *fp;

		strcpy( arg4, argument );

	    if (IS_SET(ch->penalty_flags,PENAL_NOCHANNELS))
	    {
	    	stc("{cYou cannot string while NoChanneled.{c\n\r",ch);
	    	return;
	 	}
		if ( ch->in_room->vnum != FaireStringRoom )
		{
			stc( "You must be in the stringing shoppe to do this.\n\r", ch );
			return;
		}
		if ( ( upgrade = get_obj_carry( ch, arg2, ch ) ) == NULL )
		{
			stc( "You must supply a valid item to string.\n\r", ch );
			return;
		}
		if ( str_cmp( arg3, "name" ) && str_cmp( arg3, "short" ) && str_cmp( arg3, "long" ) )
		{
			stc( "You can only string the name, short, or long.\n\r", ch );
			return;
		}
		for ( ticket = ch->carrying; ticket != NULL; ticket = ticket_next )
		{
		    ticket_next = ticket->next_content;
			if ( ticket->pIndexData->vnum == 912 )
			{
				found = TRUE;
				break;
			}
	    }
	    if ( !found )
	    {
			stc("You do not have any quest tickets to string with.\n\r",ch);
	    	return;
		}
	      if ((arg4[0] == '\0'))
	      {
		      stc("If you are going to string something, you need to give it a new string.\n\n",ch);
		      stc("Syntax: faire string  <item name> <short|long|name> <string>\n\r", ch );
		      return;
	      }
		sprintf( log_buf, "NSTRING:: [%s] (%s) %s", ch->name, upgrade->name, upgrade->short_descr );
		wiznet(log_buf,ch,NULL,WIZ_STRING,0,0);

		if ( !str_cmp( arg3, "name" ) )
		{
			if (strlen_color(arg4) > 64)
			{
				stc("Line too long.\n\r",ch);
				return;
			}
			act("A Tinker takes $n's $p and tinkers with it.",ch,upgrade,NULL,TO_ROOM);
			free_string( upgrade->name );
			upgrade->name = str_dup( arg4 );
			extract_obj( ticket );
		}
		else if ( !str_cmp( arg3, "short" ) )
		{
			if (strlen_color(arg4) > 64)
			{
				stc("Line too long.\n\r",ch);
				return;
			}
			act("A Tinker takes $n's $p and tinkers with it.",ch,upgrade,NULL,TO_ROOM);
			free_string( upgrade->short_descr );
			upgrade->short_descr = str_dup( arg4 );
			extract_obj( ticket );
		}
		else if ( !str_cmp( arg3, "long" ) )
		{
			if (strlen_color(arg4) > 64)
			{
				stc("Line too long.\n\r",ch);
				return;
			}
			act("A Tinker takes $n's $p and tinkers with it.",ch,upgrade,NULL,TO_ROOM);
			free_string( upgrade->description );
			upgrade->description = str_dup( arg4 );
			extract_obj( ticket );
		}
	    stc("{w\n\rA ghostly voice fills your head.\n\r",ch);
	    stc("{W  {B+============================================================{B+{x\n\r",ch);
	    stc("{B  | {WRemember, if the Immortals find your new string offensive, {B|{x\n\r", ch);
	    stc("{B  | {Wthey will not be happy, and you will find the item removed {B|{x\n\r",ch);
	    stc("{B  | {Wand the quest tokens you spent gone.                       {B|{x\n\r",ch);
	    stc("{B  | {WThis is your ONE AND ONLY Warning.                         {B|{x\n\r", ch);
	    stc("{W  {B+============================================================{B+{x\n\r",ch);

		if ( ( fp = fopen( NSTRING_FILE,"a" ) ) == NULL )
		{
		    log_string("Error writing to nstring.dat");
		    return;
		}
		fprintf( fp, "NSTR %s: (%s) %s\n", ch->name, arg3, arg4 );
		fclose( fp );
		sprintf( log_buf, "NSTRING:: [%s] (%s) %s", ch->name, arg3, arg4 );
		wiznet(log_buf,ch,NULL,WIZ_STRING,0,0);
	    stc("\n\r{wYou have posted a note to the Immortals containing information on this restring.{c\n\r", ch);
		return;
	}

	if ( !str_cmp( arg1, "balance" ) )
	{
		if ( ch->in_room->vnum != FaireBalanceRoom )
		{
			stc( "You must be in the balancing shoppe to do this.\n\r", ch );
			return;
		}
		if ( ( upgrade = get_obj_carry( ch, arg2, ch ) ) == NULL )
		{
			stc( "You must supply a valid weapon to balance.\n\r", ch );
			return;
		}
		if ( upgrade->item_type != ITEM_WEAPON )
		{
			stc( "Only weapons can be balanced.\n\r", ch );
			return;
		}
		if ( upgrade->weight <= 2 )
		{
			stc( "This item cannot be balanced! It is too light.\n\r", ch );
			return;
		}
		balCost = 1000;
		if ( !check_coins( ch, balCost*100,0,0 ) )
		{
		    stc( "You can't afford to balance that item.\n\r", ch );
		    ptc( ch, "It will cost you %d silver marks to do that.\n\r", balCost );
		    return;
		}
		act( "The tinker takes $n's $T and sets to work balancing it.",
			ch,NULL,upgrade->short_descr,TO_ROOM );
		act( "The tinker takes your $T and sets to work balancing it.",
			ch,NULL,upgrade->short_descr,TO_CHAR );
		upgrade->weight -= 2;
		if ( IS_SET( upgrade->value[4], WEAPON_TWO_HANDS ) )
			upgrade->value[4] -= WEAPON_TWO_HANDS;
		deduct_cost( ch, balCost*100,0 );
		if ( !IS_SET( upgrade->extra_flags, ITEM_BALANCED ) )
			SET_BIT( upgrade->extra_flags, ITEM_BALANCED );
		return;
	}
	if ( !str_cmp( arg1, "repair" ) )
	{
		if ( ch->in_room->vnum != FaireRepairRoom )
		{
			stc( "You must be in the repair shoppe to do this.\n\r", ch );
			return;
		}
		if ( ( upgrade = get_obj_carry( ch, arg2, ch ) ) == NULL )
		{
			stc( "You must supply a valid item to repair.\n\r", ch );
			return;
		}
		if ( upgrade->condition >= 100 )
		{
			stc( "This item isn't in need of repair!\n\r", ch );
			return;
		}
		repCost = 100;
		if ( !check_coins( ch, repCost*100,0,0 ) )
		{
		    stc( "You can't afford to repair that item.\n\r", ch );
		    ptc( ch, "It will cost you %d silver marks to do that.\n\r", repCost );
		    return;
		}
		act( "The repairman takes $n's $T and sets to work repairing it.",
			ch,NULL,upgrade->short_descr,TO_ROOM );
		act( "The repairman takes your $T and sets to work repairing it.",
			ch,NULL,upgrade->short_descr,TO_CHAR );
		upgrade->condition += 1;
		deduct_cost( ch, repCost *100,0);
		return;
	}
	if ( !str_cmp( arg1, "tattoo" ) )
	{
		if ( ch->in_room->vnum != FaireTattooRoom )
		{
			stc( "You must be at the tattoo artist's cart to do this.\n\r", ch );
			return;
		}
		if ( strlen( arg2 ) == 0 )
		{
			stc( " Valid body location undefined.\n\r", ch );
			stc( " Syntax: faire tattoo <location> <type>.\n\r", ch );
			stc( " Valid locations are:\n\r", ch );
			stc( "{W*{x left wrist     {W*{x left rear      {W*{x left finger\n\r", ch );
			stc( "{W*{x right wrist    {W*{x right rear     {W*{x right finger\n\r", ch );
			stc( "{W*{x left ankle     {W*{x head           {W*{x left calf\n\r", ch );
			stc( "{W*{x right ankle    {W*{x face           {W*{x right calf\n\r", ch );
			stc( "{W*{x right bicep    {W*{x stomach        {W*{x left forearm\n\r", ch );
			stc( "{W*{x left bicep     {W*{x chest          {W*{x right forearm\n\r", ch );
			stc( "{W*{x left shoulder  {W*{x right shoulder {W*{x neck\n\r", ch );
			stc( " Place '' around locations with two words .\n\r", ch );
			stc( " Faire tattoo 'left ankle'\n\r", ch );
			return;
		}
		if ( arg3[0] == '\0' )
		{
			stc( " Valid tattoo type undefined.\n\r", ch );
			stc( " Syntax: faire tattoo <location> <type>.\n\r", ch );
			stc( " Valid types are:\n\r", ch );
			stc( "{W*{xa frog                   {W*{xa laughing skull{x \n\r", ch );
			stc( "{W*{xa rainbow ribbon         {W*{xa wheel with seven spokes{x \n\r", ch );
			stc( "{W*{xstars moons and sun      {W*{xa dragon's fang{x \n\r", ch );
			stc( "{W*{xa grinning skull         {W*{xa swooping heron{x \n\r", ch );
			stc( "{W*{xtwisting vines           {W*{xcrossed spears\n\r", ch );
			stc( "{W*{xflame of Tar Valon       {W*{xa loved one's name\n\r", ch );
			stc( "{W*{xHorn of Valere           \n\r", ch );
			stc( "{W*{xserpent biting its tail  \n\r", ch );
			stc( "{W*{xfacing ravens            \n\r", ch );
			stc( "{W*{xa crystal throne           \n\r", ch );
			stc( "{W*{xa red hand               \n\r", ch );
			stc( "{W*{xa golden sunburst          \n\r", ch );
			stc( "{W*{xa red eagle              \n\r", ch );
			stc( "{W*{xan Avendesora leaf       \n\r", ch );
			return;
		}
		else if ( !str_cmp( arg2, "" ) )
		{
		}
		else if ( !str_cmp( arg2, "" ) )
		{
		}
		else if ( !str_cmp( arg2, "" ) )
		{
		}
		else if ( !str_cmp( arg2, "" ) )
		{
		}
		else if ( !str_cmp( arg2, "" ) )
		{
		}
		else if ( !str_cmp( arg2, "" ) )
		{
		}
		return;
	}
	if ( !str_cmp( arg1, "upgrade" ) )
	{
		int upgrade_mod = 0;
		if ( ch->in_room->vnum != FaireUpgradeRoom )
		{
			stc( "You must be in the upgrade room to do this.\n\r", ch );
			return;
		}
		if ( ( upgrade = get_obj_carry( ch, arg2, ch ) ) == NULL )
		{
			stc( "You must supply a valid item to upgrade.\n\r", ch );
			return;
		}
		ugCost = 1000;  /* 100 gold */
		upgrade_mod = atoi(arg3);
		if ( upgrade_mod > 10 )
		{
			stc( "You can only upgrade 10 points at a time.\n\r", ch );
			return;
		}
		if (upgrade_mod < 1)
		{
			stc( "You should upgrade at least 1 point.\n\r", ch);
			return;
		}
		ugCost *= upgrade_mod;
		if ( upgrade->item_type == ITEM_ANGREAL_MALE || upgrade->item_type == ITEM_ANGREAL_FEMALE )
		{
			ugCost *= 10;
		}
		if ( !check_coins( ch, ugCost*100,0,0 ) )
		{
		    stc( "You can't afford to upgrade that item.\n\r", ch );
		    ptc( ch, "It will cost you %d silver marks to do that.\n\r", ugCost );
		    return;
		}
		if ( arg3[0] == '\0' )
		{
			stc("Syntax:  faire upgrade <item name> <modifier>\n\r",ch );
			stc("         The cost of an upgrade is 1000 gold for angreal.\n\r",ch );
			stc("         The cost of an upgrade is 100 gold for weapons and armor.\n\r",ch );
			stc("         You can only upgrade Weapons, Armors, and Angreal.\n\r",ch );
			stc("         Weapon upgrades increase by one *point*, NOT one rank.\n\r", ch );
			/*
			stc("Syntax:  faire upgrade <item name> <modifier>\n\r",ch );
			stc("         The cost of an upgrade is 100 times the item's cost.\n\r",ch );
			stc("         You can only upgrade Weapons, Armors, and Angreal.\n\r",ch );
			stc("         Weapon upgrades increase by one *point*, NOT one rank.\n\r", ch );
			stc("Syntax:  faire repair <item name>\n\r", ch );
			stc("         The cost of a repair is 10 times the item's damage.\n\r", ch );
			stc("Syntax:  faire balance <item name>\n\r", ch );
			stc("         The cost of a balance is 1000 times the item's cost.\n\r", ch );
			stc("         Balancing allows greater upgrades of Weapons and Armor.\n\r",ch );
			*/
			return;
		}
		if ( upgrade->item_type == ITEM_WEAPON )
		{
    			if ( (upgrade_mod + upgrade->value[2] ) > 175 ) /* non-remort */
			{
				stc( "That item can't be upgraded that far.\n\r", ch );
				return;
			}
			if ( (upgrade->value[1] > 1 ) && (upgrade_mod + upgrade->value[2]) > 76 ) /* remort */
			{
				stc( "That item can't be upgraded that far.\n\r", ch );
				return;
			}
			deduct_cost( ch, ugCost*100,0 );
			upgrade->value[2] += upgrade_mod;
			act( "The tinker takes $n's $T and sets to work upgrading it.",
				ch,NULL,upgrade->short_descr,TO_ROOM );
			act( "The tinker takes your $T and sets to work upgrading it.",
				ch,NULL,upgrade->short_descr,TO_CHAR );
			return;
		}
		if ( upgrade->item_type == ITEM_ARMOR )
		{
			if ( (upgrade_mod + upgrade->value[0]) > 100 )
			{
				stc( "This item has been upgraded as far as it can be.\n\r", ch );
				return;
			}
			deduct_cost( ch, ugCost *100,0);
			upgrade->value[0] += upgrade_mod;
			upgrade->value[1] += upgrade_mod;
			upgrade->value[2] += upgrade_mod;
			upgrade->value[3] += upgrade_mod;
			act( "The tinker takes $n's $T and sets to work improving it.",
				ch,NULL,upgrade->short_descr,TO_ROOM );
			act( "The tinker takes your $T and sets to work improving it.",
				ch,NULL,upgrade->short_descr,TO_CHAR );
			return;
		}
		if ( upgrade->item_type == ITEM_ANGREAL_MALE || upgrade->item_type == ITEM_ANGREAL_FEMALE )
		{
			if ( upgrade->value[1] >= 20 )
			{
				stc( "This item has been upgraded as far as it can be.\n\r", ch );
				return;
			}
			if ( upgrade_mod > 1 )
			{
				stc( "Due to abuse of the system you can only upgrade an angreal one point at a time.\n\r", ch );
				return;
			}
			deduct_cost( ch, ugCost*100,0 );
			upgrade->value[1] += 1;	/* since it can only be upgraded by 1 */
			act( "The tinker takes $n's $T and sets to work strengthening it.",
				ch,NULL,upgrade->short_descr,TO_ROOM );
			act( "The tinker takes your $T and sets to work strengthening it.",
				ch,NULL,upgrade->short_descr,TO_CHAR );
			return;
		}
		else
		{
			stc( "This item cannot be upgraded.\n\r", ch );
			return;
		}
	}
	if ( !str_cmp( arg1, "list" ) )
	{
		if ( ch->level < MAX_LEVEL )
		{
			stc( "You can't do that.\n\r", ch );
			return;
		}
	    for ( fLookup = 0; fLookup < MaxFaireLocations; fLookup++ )
    	{
	    	ptc( ch, "Possible Faire in %s.\n\r", faire_location_table[fLookup].fName );
	    	ptc( ch, "             vnum %d\n\r", faire_location_table[fLookup].vnum );
	    	ptc( ch, "             tent %s\n\r", str_dup( faire_location_table[fLookup].portalName ) );
		}
		return;
	}
	if ( arg2[0] == '\0' )
	{
		stc("Syntax: faire create  <area file name> <Immortal Only>\n\r",ch);
		stc("Syntax: faire kill    <area file name> <Immortal Only>\n\r",ch);
		stc("Syntax: faire list    (list possible areas)\n\r",ch);
		stc("Syntax: faire upgrade <item name> <modifier>\n\r",ch );
		stc("Syntax: faire repair  <item name>\n\r", ch );
		stc("Syntax: faire balance <item name>\n\r", ch );
		return;
	}
	if ( !str_cmp( arg1, "kill" ) )
	{
		if ( !IS_IMMORTAL(ch) )
		{
			stc( "You can't do that.\n\r", ch );
			return;
		}
		faire->open = FALSE;
		faire->location = NULL;
		FaireBankRoom = 1;
		FaireRepairRoom = 1;
		FaireUpgradeRoom = 1;
		FaireBalanceRoom = 1;
		FaireTattooRoom = 1;
		FaireStringRoom = 1;
		return;
	}
	if ( !str_cmp( arg1, "create" ) )
	{
		if ( !IS_IMMORTAL(ch) )
		{
			stc( "You can't do that.\n\r", ch );
			return;
		}
	    for ( fLookup = 0; fLookup < MaxFaireLocations; fLookup++ )
    	{
    		if ( !str_cmp( faire_location_table[fLookup].fName, arg2 ) )
	    	{
	    		if ( fLookup > MaxFaireLocations )
	    		{
	    			stc("fLookup went haywire.\n\r",ch);
	    			return;
	    		}
		    	ptc( ch, "Placing a Faire in %s.\n\r", faire_location_table[fLookup].fName );
    			FaireLocation = faire_location_table[fLookup].vnum;
				faire->open = TRUE;
				faire->location = str_dup( faire_location_table[fLookup].fName );
    			found = TRUE;
	    		init_faire_portal( FaireLocation, fLookup );
	    		init_faire_mobs( FaireLocation, fLookup );
	    		init_faire_rooms( ch, FaireLocation, fLookup );
			    for ( d = descriptor_list; d; d = d->next )
			    {
					if ( d->connected == CON_PLAYING )
					{
						stc( "\n\r              {R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{x\n\r",
						d->character );
				    	ptc( d->character,
				    	"{wA crier rushes by, bringing news of a Faire in %s!\n\r",
				    	capit2( faire_location_table[fLookup].pName ) );
						stc( "              {R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{r={R-{x\n\r",
						d->character );
					}
			    }
    			return;
	    	}
		}
   		if ( fLookup > MaxFaireLocations )
   		{
   			stc("fLookup went haywire.\n\r",ch);
   			return;
   		}
    	if ( !found )
    	{
    		stc("Arg2 did not compare to a valid Faire location.\n\r",ch);
    		return;
    	}
	}
}

void clear_old_mobs (ROOM_INDEX_DATA *room)
{
	CHAR_DATA *mob, *mob_next;

	for (mob = room->people; mob != NULL; mob = mob_next)
	{
		mob_next = mob->next_in_room;

		if (!IS_NPC(mob))	continue;
		if (mob->pIndexData->vnum == FaireMobVnum)
			extract_char(mob, TRUE);
	}

	return;
}

void init_faire_mobs( int location, int fLookup )
{
	ROOM_INDEX_DATA *fRoom;
	CHAR_DATA *mob;
	char mBuf[MSL];
	int rMob;
	int vnum = 0, i = 0;
	mBuf[0] = '\0';

	FemaleAngrealExists = FALSE;
	MaleAngrealExists = FALSE;

    for ( vnum = FaireRoomFirst; vnum <= FaireRoomLast; vnum++ )
    {
		if ( ( fRoom = get_room_index( vnum ) ) )
		{
			if ( fRoom == NULL )
			{
				bugf("FAIRE:: Vnum %s non-existant in init_faire_mobs.", vnum);
				return;
			}
			if ( fRoom->vnum == FaireBankRoom
			|| fRoom->vnum == FaireUpgradeRoom
			|| fRoom->vnum == FaireRepairRoom
			|| fRoom->vnum == FaireBalanceRoom )
				continue;
		}

		clear_old_mobs(fRoom);

		rMob = number_range(1,100);
		mob = create_mobile( get_mob_index( FaireMobVnum ) );
	    mob->group	            = 0;
    	mob->sex		        = faire_mobs_table[rMob-1].mSex;
	    mob->race		        = race_lookup( faire_mobs_table[rMob-1].mRace );
	    for (i = 0; i < 4; i++)
    		mob->armor[i]		= 1;
	    mob->damage[DICE_NUMBER]=51;
    	mob->damage[DICE_TYPE]  = 51;
	    mob->damage[DICE_BONUS] = 51;
    	mob->level	            = 51;
    	mob->hit		        = 500;
	    mob->max_hit	        = 500;
    	mob->stam		        = 500;
	    mob->max_stam	        = 500;
    	mob->money[3][0]	    = 1;
	    mob->money[2][0]	    = 1;
    	mob->imm_flags  	    = 0;
	    mob->res_flags  	    = 0;
    	mob->vuln_flags   	    = 0;
    	mob->act                += ACT_SENTINEL;
	    mob->position	        = POS_STANDING;
    	mob->start_pos	        = POS_STANDING;
    	mob->saving_throw	    = 0;
	    mob->alignment	        = -1000;
    	mob->hitroll	        = 100;
	    mob->damroll	        = 100;
    	mob->form		        = race_table[mob->race].form;
	    mob->parts	            = race_table[mob->race].parts;
    	mob->size		        = SIZE_MEDIUM;
	    mob->dam_type	        = attack_lookup("pound");
	    mob->name               = str_dup( faire_mobs_table[rMob-1].mName );
		mBuf[0] = '\0';
	    strcat( mBuf, "a " );
	    strcat( mBuf, faire_mobs_table[rMob-1].mName );
	    free_string( mob->short_descr );
	    mob->short_descr        = str_dup( mBuf );
	    free_string( mob->colorname );
	    mob->colorname = str_dup( mBuf );
		mBuf[0] = '\0';
	    strcat( mBuf, "There is a " );
	    strcat( mBuf, faire_mobs_table[rMob-1].mName );
	    strcat( mBuf, " here." );
	    strcat( mBuf, "\n\r" );
	    free_string( mob->long_descr );
	    mob->long_descr        = str_dup( mBuf );
		mBuf[0] = '\0';
		char_to_room( mob, fRoom );
		init_faire_objects( mob, FaireLocation, fLookup );
	}
	return;
}

OBJ_DATA *faire_treasure_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,20);
//	int guild = number_range(1, MaxFaireGuilds);
	int guild = number_range(2,MAX_CLAN-2);

	int i = 0, baseCost = 100;
	char oBuf[MSL];
	char staticBuf[MSL];
	int stones = number_range(1,64);
	char *stone_array[] = {
	 "cat's eye gem-stone", "round bloodstone",       "clear-white topaz",
	 "angel's skin coral",  " blue spinel",           " clear quartz",
	 "fire-colored agate",  "yellow-gold crysoberyl", "black moonstone",
	 "bright-green peridot","amber gemstone",         "yellow citrine gemstone",
	 "yellow garnet",       "smokey-colored quartz",  "milky-white pearl",
	 "reddish-pink tourmaline","rock-crystal","fire opal",
	 "blue sapphire", "diamond", "ruby-red alexandrite",
	 "yellowish-orange jade stone","gray-white jade stone", "vibrant purple amethyst",
	 "rose-colored quartz", "bluish-colored quartz", "milky-white topaz",
	 "angel's skin coral", "greenish-blue aquamarine", "pale-blue turquoise",
	 "pink sapphire","green emerald","rock-crystal",
	 "yellowed ivory","bright green beryl stone","blue turquoise",
	 "light red morganite stone","black pearl","black opal",
	 "cat's-eye beryl gemstone", "black onyx stone","golden colored sapphire",
	 "white pearl", "red ruby","light pink conch pearl",
	 "reddish-pink alexandrite", "demantoid garnet stone","rose-colored quartz",
	 "green beryl stone", "lapis lazuli stone","opaque colored jasper",
	 "deep blue tigereye stone", "reddish-brown sardonyx","sea-green aquamarine gem",
	 "milk-and-honey colored cat's-eye chrysoberyl", "star-diopside","cat's-eye diopside",
	 "reddish-cast sunstone", "jet stone","snowflake obsidian",
	 "imperial topaz", "star ruby","turquoise stone",
	 "demantoid garnet stone"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies a " );
	switch( base )
	{
		case 1: /* ring */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " ring" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_FINGER;
				fObj->weight = 5;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
		case 2: /* medallion */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " medallion" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_NECK;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
		case 3: /* earring */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
				strcat( oBuf, clan_table[guild].who_name );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, " earring" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_EAR;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
		case 4: /* nose ring */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " nose ring" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_NOSE;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
		case 5: /* band */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
				strcat( oBuf, clan_table[guild].who_name );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, " band" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_FINGER;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
		case 6: /* chain */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " chain" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_NECK;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
		case 7: /* tiara */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " tiara" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				strcat( oBuf, " encrusted with " );
				strcat( oBuf, faire_gem_table[i-1].oName );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_HEAD;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod + faire_gem_table[i-1].costMod;
			}
			break;
		case 8: /* diadem */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " diadem" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				strcat( oBuf, " encrusted with " );
				strcat( oBuf, faire_gem_table[i-1].oName );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_HEAD;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod + faire_gem_table[i-1].costMod;
			}
			break;
		case 9: /* wrist-band */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " wrist-band" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				strcat( oBuf, " encrusted with " );
				strcat( oBuf, faire_gem_table[i-1].oName );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_WRIST;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod + faire_gem_table[i-1].costMod;
			}
			break;
		case 10: /* ring */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " head-band" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				strcat( oBuf, " encrusted with " );
				strcat( oBuf, faire_gem_table[i-1].oName );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_EYES;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod + faire_gem_table[i-1].costMod;
			}
			break;
		case 11: /* painting */
			i = number_range(1,50);
			{
				free_string( fObj->material );
				fObj->material = str_dup( "cloth" );
				free_string( fObj->name );
				fObj->name = str_dup( faire_paintings_table[i-1].pName );
				strcat( oBuf, faire_paintings_table[i-1].pScene );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE;
				fObj->weight = 10;
				fObj->cost = baseCost * (10 + strlen(faire_paintings_table[i-1].pScene));
			}
			break;
		case 12: /* flowers */
			{
				strcat( oBuf, "a long stemmed red rose" );
				free_string( fObj->name );
				fObj->name = str_dup("red rose");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10) / 2;
			}
			break;
		case 13: /* flowers */
			{
				strcat( oBuf, "a long stemmed white rose" );
				free_string( fObj->name );
				fObj->name = str_dup("white rose");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10) / 2;
			}
			break;
		case 14: /* flowers */
			{
				strcat( oBuf, "a long stemmed pink rose" );
				free_string( fObj->name );
				fObj->name = str_dup("pink rose");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10) / 2;
			}
			break;
		case 15: /* flowers */
			{
				strcat( oBuf, "a long stemmed black rose" );
				free_string( fObj->name );
				fObj->name = str_dup("black rose");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10) / 2;
			}
			break;
		case 16: /* flowers */
			{
				strcat( oBuf, "a dozen long stemmed red roses" );
				free_string( fObj->name );
				fObj->name = str_dup("dozen red roses");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
		case 17: /* flowers */
			{
				strcat( oBuf, "a dozen long stemmed white roses" );
				free_string( fObj->name );
				fObj->name = str_dup("dozen white roses");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
		case 18: /* flowers */
			{
				strcat( oBuf, "a dozen long stemmed pink roses" );
				free_string( fObj->name );
				fObj->name = str_dup("dozen pink roses");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
		case 19: /* flowers */
			{
				strcat( oBuf, "a dozen long stemmed black roses" );
				free_string( fObj->name );
				fObj->name = str_dup("dozen black roses");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
/*
		case 20:
			{
				strcat( oBuf, "a fold open card painted with a bright red heart" );
				free_string( fObj->name );
				fObj->name = str_dup("red heart card");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
		case 21:
			{
				strcat( oBuf, "a fold open card painted with the words, I Love You" );
				free_string( fObj->name );
				fObj->name = str_dup("I Love You card");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
		case 22: 
			{
				strcat( oBuf, "a fold open card painted with the words, You Are My Heart" );
				free_string( fObj->name );
				fObj->name = str_dup("You Are My Heart card");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
		case 23: 
			{
				strcat( oBuf, "a fold open card painted with the words, For the Shade Of My Heart" );
				free_string( fObj->name );
				fObj->name = str_dup("Shade of my heart card");
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
				fObj->weight = 1;
				fObj->cost = baseCost * number_range(1,10);
			}
			break;
*/
		case 20: /* ring */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcpy( oBuf, fObj->material );
				strcat( oBuf, " ring set with a " );
				strcat( oBuf, stone_array[stones-1] );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_FINGER;
				fObj->weight = 5;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
			}
			break;
	}
	return( fObj );
}

OBJ_DATA *faire_light_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,5);
//	int guild = number_range(1,MaxFaireGuilds);
	int guild = number_range(2,MAX_CLAN-2);
	int i = 0, baseCost = 50, scents = 0;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* lamp */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				strcat( oBuf, fObj->material );
				strcat( oBuf, " lamp emblazoned with the seal of the " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " guild" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
				fObj->value[2] = 1500;
			}
			break;
		case 2: /* lantern */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				/*strcat( oBuf, "a " );*/
				strcat( oBuf, fObj->material );
				strcat( oBuf, " lantern engraved with the seal of the " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " guild" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
				fObj->value[2] = 1500;
			}
			break;
		case 3: /* lanthorn */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				/*strcat( oBuf, "a " );*/
				strcat( oBuf, fObj->material );
				strcat( oBuf, " lanthorn scribed with the seal of the " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " guild" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
				fObj->value[2] = 1500;
			}
			break;
		case 4: /* flambeau */
			i = number_range(1,5);
			{
				free_string( fObj->material );
				fObj->material = str_dup( faire_material_table[i-1].oName );
				/*strcat( oBuf, "a " );*/
				strcat( oBuf, fObj->material );
				strcat( oBuf, " flambeau etched with the seal of the " );
//				strcat( oBuf, faire_guild_table[guild-1].gName );
				strcat( oBuf, clan_table[guild].who_name );
				strcat( oBuf, " guild" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
				fObj->value[2] = 1500;
			}
			break;
		case 5: /* candle */
			i = number_range(1,5);
			scents = number_range(1,21);
			{
				free_string( fObj->material );
				fObj->material = str_dup( "wax" );
				/*strcat( oBuf, "a " );*/
				strcat( oBuf, faire_scents_table[scents-1].smell );
				strcat( oBuf, " scented candle" );
				free_string( fObj->name );
				fObj->name = str_dup( oBuf );
				free_string( fObj->short_descr );
				fObj->short_descr = str_dup( oBuf );
				strcat( staticBuf, oBuf );
				strcat( staticBuf, "." );
				free_string( fObj->description );
				fObj->description = str_dup( staticBuf );
				fObj->wear_flags += ITEM_TAKE;
				fObj->weight = 10;
				fObj->cost = baseCost * faire_material_table[i-1].costMod;
				fObj->value[2] = 1500;
			}
			break;
	}
	return( fObj );
}
OBJ_DATA *faire_scroll_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,15);
	int baseCost = 95;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* scroll */
			strcat( oBuf, "a tightly sealed scroll" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "paper" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 2: /* tome */
			strcat( oBuf, "a leather bound tome" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "leather" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 300;
			fObj->value[0] = 300;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 3: /* manuscript */
			strcat( oBuf, "a manuscript scribed with odd sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "parchment" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 4: /* volume */
			strcat( oBuf, "a heavy black-worked volume" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "wood and leather" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 400;
			fObj->value[0] = 400;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 5: /* writ */
			strcat( oBuf, "a writ scored with the spinning Wheel" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			free_string( fObj->material );
			fObj->material = str_dup( "parchment" );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 6: /* scroll */
			strcat( oBuf, "a scroll sealed with a leather tie" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "paper" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 7: /* tome */
			strcat( oBuf, "a tome scored with twisting sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "thick leather" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 300;
			fObj->value[0] = 300;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 8: /* manuscript */
			strcat( oBuf, "a manuscript scribed with writhing sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "aged vellum" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 9: /* volume */
			strcat( oBuf, "a heavy volume engraved with twisting sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "crinkled leather" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 400;
			fObj->value[0] = 400;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 10: /* writ */
			strcat( oBuf, "a writ scored with a warped Wheel" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "yellowed parchment" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 11: /* scroll */
			strcat( oBuf, "a scroll sealed with red wax" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "papyrus" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = rand_offensive_weave();
			fObj->value[2] = rand_offensive_weave();
			fObj->value[3] = rand_offensive_weave();
			fObj->value[4] = rand_offensive_weave();
			break;
		case 12: /* tome */
			strcat( oBuf, "a tome scored with writhing sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "leather and wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 300;
			fObj->value[0] = 300;
			fObj->value[1] = rand_offensive_weave();
			fObj->value[2] = rand_offensive_weave();
			fObj->value[3] = rand_offensive_weave();
			fObj->value[4] = rand_offensive_weave();
			break;
		case 13: /* manuscript */
			strcat( oBuf, "a manuscript scribed with blazing sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "hardened leather" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = rand_offensive_weave();
			fObj->value[2] = rand_offensive_weave();
			fObj->value[3] = rand_offensive_weave();
			fObj->value[4] = rand_offensive_weave();
			break;
		case 14: /* volume */
			strcat( oBuf, "a heavy volume engraved with blazing sigils" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "blackened leather" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 400;
			fObj->value[0] = 400;
			fObj->value[1] = rand_offensive_weave();
			fObj->value[2] = rand_offensive_weave();
			fObj->value[3] = rand_offensive_weave();
			fObj->value[4] = rand_offensive_weave();
			break;
		case 15: /* writ */
			strcat( oBuf, "a writ scored with a burning Wheel" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "paper" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = rand_offensive_weave();
			fObj->value[2] = rand_offensive_weave();
			fObj->value[3] = rand_offensive_weave();
			fObj->value[4] = rand_offensive_weave();
			break;
	}
	return( fObj );
}
OBJ_DATA *faire_wand_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,12);
	int baseCost = 90;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* pine */
			strcat( oBuf, "a slender pine wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "pine wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = 50;
			fObj->value[2] = 50;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 2: /* oak */
			strcat( oBuf, "a slender oak wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "oak wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = 100;
			fObj->value[2] = 100;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 3: /* ironwood */
			strcat( oBuf, "a slender ironwood wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "ironwood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 450;
			fObj->value[0] = 450;
			fObj->value[1] = 150;
			fObj->value[2] = 150;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 4: /* blackwood */
			strcat( oBuf, "a slender blackwood wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "blackwood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = 200;
			fObj->value[2] = 200;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 5: /* pine */
			strcat( oBuf, "a sturdy pine wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "pine wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = 50;
			fObj->value[2] = 50;
			fObj->value[3] = rand_misc_weave();
			break;
		case 6: /* oak */
			strcat( oBuf, "a sturdy oak wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "oak wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = 100;
			fObj->value[2] = 100;
			fObj->value[3] = rand_misc_weave();
			break;
		case 7: /* ironwood */
			strcat( oBuf, "a sturdy ironwood wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "ironwood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 450;
			fObj->value[0] = 450;
			fObj->value[1] = 150;
			fObj->value[2] = 150;
			fObj->value[3] = rand_misc_weave();
			break;
		case 8: /* blackwood */
			strcat( oBuf, "a heavy blackwood wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "blackwood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = 200;
			fObj->value[2] = 200;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 9: /* pine */
			strcat( oBuf, "a heavy pine wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "pine wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = 50;
			fObj->value[2] = 50;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 10: /* oak */
			strcat( oBuf, "a heavy oak wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "oak wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = 100;
			fObj->value[2] = 100;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 11: /* ironwood */
			strcat( oBuf, "a heavy ironwood wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "iron wood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 450;
			fObj->value[0] = 450;
			fObj->value[1] = 150;
			fObj->value[2] = 150;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 12: /* blackwood */
			strcat( oBuf, "a heavy blackwood wand" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->material );
			fObj->material = str_dup( "blackwood" );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = 200;
			fObj->value[2] = 200;
			fObj->value[3] = rand_offensive_weave();
			break;
	}
	return( fObj );
}
OBJ_DATA *faire_staff_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,12);
	int baseCost = 100;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* fir */
			strcat( oBuf, "a tall fir staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = 50;
			fObj->value[2] = 50;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 2: /* white oak */
			strcat( oBuf, "a tall white oak staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = 100;
			fObj->value[2] = 100;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 3: /* spruce */
			strcat( oBuf, "a tall spruce staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 450;
			fObj->value[0] = 450;
			fObj->value[1] = 150;
			fObj->value[2] = 150;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 4: /* walnut */
			strcat( oBuf, "a tall walnut staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = 200;
			fObj->value[2] = 200;
			fObj->value[3] = rand_defensive_weave();
			break;
		case 5: /* fir */
			strcat( oBuf, "a stout fir staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = 50;
			fObj->value[2] = 50;
			fObj->value[3] = rand_misc_weave();
			break;
		case 6: /* white oak */
			strcat( oBuf, "a stout white oak staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = 100;
			fObj->value[2] = 100;
			fObj->value[3] = rand_misc_weave();
			break;
		case 7: /* spruce */
			strcat( oBuf, "a stout spruce staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 450;
			fObj->value[0] = 450;
			fObj->value[1] = 150;
			fObj->value[2] = 150;
			fObj->value[3] = rand_misc_weave();
			break;
		case 8: /* walnut */
			strcat( oBuf, "a stout walnut staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = 200;
			fObj->value[2] = 200;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 9: /* fir */
			strcat( oBuf, "a heavy fir staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = 50;
			fObj->value[2] = 50;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 10: /* white oak */
			strcat( oBuf, "a heavy white oak staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = 100;
			fObj->value[2] = 100;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 11: /* spruce */
			strcat( oBuf, "a heavy spruce staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 450;
			fObj->value[0] = 450;
			fObj->value[1] = 150;
			fObj->value[2] = 150;
			fObj->value[3] = rand_offensive_weave();
			break;
		case 12: /* walnut */
			strcat( oBuf, "a heavy walnut staff" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = 200;
			fObj->value[2] = 200;
			fObj->value[3] = rand_offensive_weave();
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "hard wood" );
	return( fObj );
}
OBJ_DATA *faire_weapon_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,9);
//	int guild = number_range(1,MaxFaireGuilds);
	int guild = number_range(2,MAX_CLAN-2);
	int i = 0, x = 0, baseCost = 250;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* sword */
			i = number_range(1,22);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_SWORD;
			fObj->value[1] = faire_sword_table[i-1].v1;
			fObj->value[2] = faire_sword_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_sword_table[i-1].v3 );
			fObj->value[4] = faire_sword_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_sword_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			strcat( oBuf, clan_table[guild].who_name );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 2: /* spear */
			i = number_range(1,6);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_SPEAR;
			fObj->value[1] = faire_spear_table[i-1].v1;
			fObj->value[2] = faire_spear_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_spear_table[i-1].v3 );
			fObj->value[4] = faire_spear_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_spear_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			strcat( oBuf, clan_table[guild].who_name );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 3: /* staff */
			i = number_range(1,7);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_STAFF;
			fObj->value[1] = faire_staff_table[i-1].v1;
			fObj->value[2] = faire_staff_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_staff_table[i-1].v3 );
			fObj->value[4] = faire_staff_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_staff_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 4: /* mace */
			i = number_range(1,7);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_MACE;
			fObj->value[1] = faire_mace_table[i-1].v1;
			fObj->value[2] = faire_mace_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_mace_table[i-1].v3 );
			fObj->value[4] = faire_mace_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_mace_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 5: /* dagger */
			i = number_range(1,7);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_DAGGER;
			fObj->value[1] = faire_dagger_table[i-1].v1;
			fObj->value[2] = faire_dagger_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_dagger_table[i-1].v3 );
			fObj->value[4] = faire_dagger_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_dagger_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 6: /* axe */
			i = number_range(1,7);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_AXE;
			fObj->value[1] = faire_axe_table[i-1].v1;
			fObj->value[2] = faire_axe_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_axe_table[i-1].v3 );
			fObj->value[4] = faire_axe_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_axe_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 7: /* flail */
			i = number_range(1,7);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_FLAIL;
			fObj->value[1] = faire_flail_table[i-1].v1;
			fObj->value[2] = faire_flail_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_flail_table[i-1].v3 );
			fObj->value[4] = faire_flail_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_flail_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 8: /* whip */
			i = number_range(1,5);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_WHIP;
			fObj->value[1] = faire_whip_table[i-1].v1;
			fObj->value[2] = faire_whip_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_whip_table[i-1].v3 );
			fObj->value[4] = faire_whip_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_whip_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
		case 9: /* polearm */
			i = number_range(1,8);
			x = number_range(1,3);
			fObj->value[0] = WEAPON_POLEARM;
			fObj->value[1] = faire_polearm_table[i-1].v1;
			fObj->value[2] = faire_polearm_table[i-1].v2;
			fObj->value[3] = attack_lookup( faire_polearm_table[i-1].v3 );
			fObj->value[4] = faire_polearm_table[i-1].v4;
			fObj->cost = faire_metals_table[x-1].costMod * baseCost;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_metals_table[x-1].oName );
			strcat( oBuf, " " );
			strcat( oBuf, faire_polearm_table[i-1].oName );
			strcat( oBuf, " stamped with the seal of the " );
			strcat( oBuf, clan_table[guild].who_name );
//			strcat( oBuf, faire_guild_table[guild-1].gName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			free_string( fObj->material );
			fObj->material = str_dup( faire_metals_table[x-1].oName );
			break;
	}


	fObj->cost = 250 * number_range(4, 20);
	if(fObj->value[3] < 1)
		bugf("Bad faire dam(v3) %s: (type) %d (faire_?_table)%d",fObj->name, fObj->value[0], i);
	fObj->wear_flags += ITEM_TAKE|ITEM_WIELD;
	return( fObj );
}
OBJ_DATA *faire_armor_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,11), heraldry = number_range(1,15), x = number_range(1,9);
	char oBuf[MSL];
	char staticBuf[MSL];
	char *faire_heraldry_array[] = {
	"a lion", "a hawk", "a trefoil leaf",
	"an eagle", "three frogs", "a gryphon",
	"a serpent", "a winged steed", "three lillies",
	"two crossed arrows", "a sword and axe",
	"a sword and hammer", "an axe and hammer", "a mailed fist", "a crane"
	};
	int decors = number_range(1,26);
	char *decor_type[] = {
	" carved with running wolves",       " carved with howling wolves" ,
	" carved with moons and stars" ,     " carved with twining vines" ,
	" decorated with eagle feathers" ,   " decorated with hawk feathers" ,
	" decorated with falcon feathers" ,  " decorated with heron feathers" ,
	" etched with twisting flames" ,     " etched with scenes of battle" ,
	" etched with twining vines" ,       " with boar skin padding" ,
	" with deer skin padding" ,          " with doe skin padding" ,
	" with buck skin padding" ,          " with wolf skin padding" ,
	" with bear skin padding" ,          " fashioned with a panther",
	" fashioned with a cougar",          " fashioned with a buck",
	" fashioned with an eagle",          " fashioned with a hawk",
	" fashioned with a crow",            " fashioned with a raven",
	" scored with vines and leaves",     " scored with lightning bolts"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	fObj->cost = 1250;
	switch( base )
	{
		case 1:
			strcat( oBuf, faire_thigh_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_thigh_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_thigh_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_thigh_armor_table[x-1].pierce;
			fObj->value[1] = faire_thigh_armor_table[x-1].bash;
			fObj->value[2] = faire_thigh_armor_table[x-1].slash;
			fObj->value[3] = faire_thigh_armor_table[x-1].exotic;
			break;
		case 2:
			strcat( oBuf, faire_calf_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_calf_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_calf_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_calf_armor_table[x-1].pierce;
			fObj->value[1] = faire_calf_armor_table[x-1].bash;
			fObj->value[2] = faire_calf_armor_table[x-1].slash;
			fObj->value[3] = faire_calf_armor_table[x-1].exotic;
			break;
		case 3:
			strcat( oBuf, faire_foot_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->name );
			fObj->name = str_dup( faire_foot_armor_table[x-1].oName );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_foot_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_foot_armor_table[x-1].pierce;
			fObj->value[1] = faire_foot_armor_table[x-1].bash;
			fObj->value[2] = faire_foot_armor_table[x-1].slash;
			fObj->value[3] = faire_foot_armor_table[x-1].exotic;
			break;
		case 4:
			strcat( oBuf, faire_waist_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->name );
			fObj->name = str_dup( faire_waist_armor_table[x-1].oName );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_waist_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_waist_armor_table[x-1].pierce;
			fObj->value[1] = faire_waist_armor_table[x-1].bash;
			fObj->value[2] = faire_waist_armor_table[x-1].slash;
			fObj->value[3] = faire_waist_armor_table[x-1].exotic;
			break;
		case 5:
			strcat( oBuf, faire_body_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_body_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_body_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_body_armor_table[x-1].pierce;
			fObj->value[1] = faire_body_armor_table[x-1].bash;
			fObj->value[2] = faire_body_armor_table[x-1].slash;
			fObj->value[3] = faire_body_armor_table[x-1].exotic;
			break;
		case 6:
			strcat( oBuf, faire_head_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_head_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_head_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_head_armor_table[x-1].pierce;
			fObj->value[1] = faire_head_armor_table[x-1].bash;
			fObj->value[2] = faire_head_armor_table[x-1].slash;
			fObj->value[3] = faire_head_armor_table[x-1].exotic;
			break;
		case 7:
			strcat( oBuf, faire_hand_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_hand_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_hand_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_hand_armor_table[x-1].pierce;
			fObj->value[1] = faire_hand_armor_table[x-1].bash;
			fObj->value[2] = faire_hand_armor_table[x-1].slash;
			fObj->value[3] = faire_hand_armor_table[x-1].exotic;
			break;
		case 8:
			strcat( oBuf, faire_arm_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_arm_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_arm_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_arm_armor_table[x-1].pierce;
			fObj->value[1] = faire_arm_armor_table[x-1].bash;
			fObj->value[2] = faire_arm_armor_table[x-1].slash;
			fObj->value[3] = faire_arm_armor_table[x-1].exotic;
			break;
		case 9:
			strcat( oBuf, faire_back_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_back_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_back_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_back_armor_table[x-1].pierce;
			fObj->value[1] = faire_back_armor_table[x-1].bash;
			fObj->value[2] = faire_back_armor_table[x-1].slash;
			fObj->value[3] = faire_back_armor_table[x-1].exotic;
			break;
		case 10:
			fObj->wear_flags += faire_shield_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_shield_armor_table[x-1].pierce;
			fObj->value[1] = faire_shield_armor_table[x-1].bash;
			fObj->value[2] = faire_shield_armor_table[x-1].slash;
			fObj->value[3] = faire_shield_armor_table[x-1].exotic;
			oBuf[0] = '\0';
			strcat( oBuf, "a " );
			strcat( oBuf, faire_shield_armor_table[x-1].oName );
			strcat( oBuf, " emblazoned with " );
			strcat( oBuf, faire_heraldry_array[heraldry-1] );
			free_string( fObj->name );
			fObj->name = str_dup( faire_shield_armor_table[x-1].oName );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			break;
		case 11:
			strcat( oBuf, faire_ankle_armor_table[x-1].oName );
			free_string( fObj->name );
			fObj->name = str_dup( faire_ankle_armor_table[x-1].oName );
			strcat( oBuf, decor_type[decors-1] );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += faire_ankle_armor_table[x-1].wearFlags;
			fObj->value[0] = faire_ankle_armor_table[x-1].pierce;
			fObj->value[1] = faire_ankle_armor_table[x-1].bash;
			fObj->value[2] = faire_ankle_armor_table[x-1].slash;
			fObj->value[3] = faire_ankle_armor_table[x-1].exotic;
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "good material" );
	fObj->cost *= number_range(1, 3);
	return( fObj );
}
OBJ_DATA *faire_potion_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,8);
	int baseCost = 35;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* elixir */
			strcat( oBuf, "a smokey-glass elixir" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 2: /* vial */
			strcat( oBuf, "a clear-glass vial" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 300;
			fObj->value[0] = 300;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 3: /* tinted-glass tonic */
			strcat( oBuf, "a tinted glass tonic" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 4: /* beaker */
			strcat( oBuf, "a darkened glass beaker" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 400;
			fObj->value[0] = 400;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 5: /* cruet */
			strcat( oBuf, "a clear glass cruet" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 500;
			fObj->value[0] = 500;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 6: /* decanter */
			strcat( oBuf, "an etched glass decanter" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 250;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 7: /* skeel */
			strcat( oBuf, "a stiff leather skeel" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 300;
			fObj->value[0] = 300;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 8: /* tumbler */
			strcat( oBuf, "a fired clay tumbler" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 350;
			fObj->value[0] = 350;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "good quality material" );
	return( fObj );
}
OBJ_DATA *faire_container_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,15);
	int baseCost = 50;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
	case 1: /*chest*/
		fObj->value[0] = 250;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 250;
		fObj->value[4] = 100;
		strcat( oBuf, "an oak wood chest" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE;
		fObj->weight = 2;
		fObj->cost = baseCost * 60;
		break;
	case 2:	/*backpack*/
		fObj->value[0] = 300;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 300;
		fObj->value[4] = 100;
		strcat( oBuf, "a supple leather backpack" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_BACK;
		fObj->weight = 2;
		fObj->cost = baseCost * 70;
		break;
	case 3:	/*sack*/
		fObj->value[0] = 350;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 350;
		fObj->value[4] = 100;
		strcat( oBuf, "a large canvas sack" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE;
		fObj->weight = 2;
		fObj->cost = baseCost * 80;
		break;
	case 4:	/*satchel*/
		fObj->value[0] = 400;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 400;
		fObj->value[4] = 100;
		strcat( oBuf, "a softened leather satchel" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_SHOULDERS;
		fObj->weight = 2;
		fObj->cost = baseCost * 90;
		break;
	case 5:	/*shoulder pack*/
		fObj->value[0] = 500;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 500;
		fObj->value[4] = 100;
		strcat( oBuf, "a supple doeskin shoulder pack" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_SHOULDERS;
		fObj->weight = 2;
		fObj->cost = baseCost * 100;
		break;
	case 6:	/*belt pouch*/
		fObj->value[0] = 550;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 550;
		fObj->value[4] = 100;
		strcat( oBuf, "a wide leather belt pouch" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_WAIST;
		fObj->weight = 2;
		fObj->cost = baseCost * 110;
		break;
	case 7:	/*bag*/
		fObj->value[0] = 600;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 600;
		fObj->value[4] = 100;
		strcat( oBuf, "a spacious canvas bag" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE;
		fObj->weight = 2;
		fObj->cost = baseCost * 120;
		break;
	case 8:	/*sheath*/
		fObj->value[0] = 650;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 650;
		fObj->value[4] = 100;
		strcat( oBuf, "a sturdy leather sheath" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_WAIST;
		fObj->weight = 2;
		fObj->cost = baseCost * 130;
		break;
	case 9:	/*scabbard*/
		fObj->value[0] = 700;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 700;
		fObj->value[4] = 100;
		strcat( oBuf, "a sturdy leather scabbard" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_WAIST;
		fObj->weight = 2;
		fObj->cost = baseCost * 140;
		break;
	case 10:/*scrip*/
		fObj->value[0] = 750;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 750;
		fObj->value[4] = 100;
		strcat( oBuf, "a large canvas scrip" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE;
		fObj->weight = 2;
		fObj->cost = baseCost * 150;
		break;
	case 11:/*knap sack*/
		fObj->value[0] = 800;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 800;
		fObj->value[4] = 100;
		strcat( oBuf, "a hardy leather knap sack" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_BACK;
		fObj->weight = 2;
		fObj->cost = baseCost * 160;
		break;
	case 12:/*haver sack*/
		fObj->value[0] = 850;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 850;
		fObj->value[4] = 100;
		strcat( oBuf, "a supple doeskin haver sack" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_BACK;
		fObj->weight = 2;
		fObj->cost = baseCost * 170;
		break;
	case 13:/*saddle bag*/
		fObj->value[0] = 900;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 900;
		fObj->value[4] = 100;
		strcat( oBuf, "a sturdy leather saddle bag" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_BACK;
		fObj->weight = 2;
		fObj->cost = baseCost * 180;
		break;
	case 14:/*valise*/
		fObj->value[0] = 950;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 950;
		fObj->value[4] = 100;
		strcat( oBuf, "an oiled leather valise" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_SHOULDERS;
		fObj->weight = 2;
		fObj->cost = baseCost * 190;
		break;
	case 15:/*cannister*/
		fObj->value[0] = 1000;
		fObj->value[1] += CONT_CLOSEABLE;
		fObj->value[3] = 1000;
		fObj->value[4] = 100;
		strcat( oBuf, "a vast leather cannister" );
		free_string( fObj->name );
		fObj->name = str_dup( oBuf );
		free_string( fObj->short_descr );
		fObj->short_descr = str_dup( oBuf );
		strcat( staticBuf, oBuf );
		strcat( staticBuf, "." );
		free_string( fObj->description );
		fObj->description = str_dup( staticBuf );
		fObj->wear_flags += ITEM_TAKE;
		fObj->weight = 2;
		fObj->cost = baseCost * 250;
		break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "first rate material" );
	return( fObj );
}
OBJ_DATA *faire_drink_obj( OBJ_DATA *fObj )
{
	int container = number_range(1,5);
	int base = number_range(1,37);
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( container )
	{
		case 1: /*glass*/
			fObj->value[0] = 25;
			fObj->value[1] = 25;
			fObj->value[2] = liq_lookup(faire_drink_table[base-1].dName);
			strcat( oBuf, "a glass of " );
			strcat( oBuf, faire_drink_table[base-1].dName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			break;
		case 2: /*flask*/
			fObj->value[0] = 30;
			fObj->value[1] = 30;
			fObj->value[2] = liq_lookup(faire_drink_table[base-1].dName);
			strcat( oBuf, "a flask of " );
			strcat( oBuf, faire_drink_table[base-1].dName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			break;
		case 3: /*canteen*/
			fObj->value[0] = 35;
			fObj->value[1] = 35;
			fObj->value[2] = liq_lookup(faire_drink_table[base-1].dName);
			strcat( oBuf, "a canteen of " );
			strcat( oBuf, faire_drink_table[base-1].dName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			break;
		case 4: /*flagon*/
			fObj->value[0] = 40;
			fObj->value[1] = 40;
			fObj->value[2] = liq_lookup(faire_drink_table[base-1].dName);
			strcat( oBuf, "a flagon of " );
			strcat( oBuf, faire_drink_table[base-1].dName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			break;
		case 5: /*bottle*/
			fObj->value[0] = 50;
			fObj->value[1] = 50;
			fObj->value[2] = liq_lookup(faire_drink_table[base-1].dName);
			strcat( oBuf, "a bottle of " );
			strcat( oBuf, faire_drink_table[base-1].dName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			break;
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "liquid" );
	return( fObj );
}
OBJ_DATA *faire_food_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,22);
	int baseCost = 1;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1:/*pork chops*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of pork chops" );
			free_string( fObj->name );
			fObj->name = str_dup("pork shop platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 10;
			break;
		case 2:/*thyme seasoned bread*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a loaf of thyme seasoned bread" );
			free_string( fObj->name );
			fObj->name = str_dup("thyme seasoned bread loaf");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 10;
			break;
		case 3:/*plum pudding*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a bowl of plum pudding" );
			free_string( fObj->name );
			fObj->name = str_dup("bowl plum pudding");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 9;
			break;
		case 4:/*roast lamb*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of roast lamb" );
			free_string( fObj->name );
			fObj->name = str_dup("roast lamb platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 9;
			break;
		case 5:/*roast beef*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a plate of roast beef" );
			free_string( fObj->name );
			fObj->name = str_dup("roast beef plate");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 9;
			break;
		case 6:/*stewed beets*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of stewed beets" );
			free_string( fObj->name );
			fObj->name = str_dup("stewed beets platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 8;
			break;
		case 7:/*brocoli soup*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a bowl of brocoli soup" );
			free_string( fObj->name );
			fObj->name = str_dup("bowl brocoli soup");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 9;
			break;
		case 8:/*roasted potatos*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a plate of roasted potatos" );
			free_string( fObj->name );
			fObj->name = str_dup("plate roasted potatos");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 4;
			break;
		case 9:/*roast suckling*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of roast suckling" );
			free_string( fObj->name );
			fObj->name = str_dup("roast suckling platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 5;
			break;
		case 10:/*broiled redfish*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of broiled redfish" );
			free_string( fObj->name );
			fObj->name = str_dup("broiled redfish platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 2;
			break;
		case 11:/*turkey leg*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a large turkey leg" );
			free_string( fObj->name );
			fObj->name = str_dup("large turkey leg");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 10;
			break;
		case 12:/*roast duck*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			free_string( fObj->name );
			fObj->name = str_dup("roast duck platter");
			strcat( oBuf, "a platter of roast duck" );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 5;
			break;
		case 13:/*stewed duck*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of stewed duck" );
			free_string( fObj->name );
			fObj->name = str_dup("stewed duck platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 3;
			break;
		case 14:/*fried squirrel*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "several slabs of fried squirrel" );
			free_string( fObj->name );
			fObj->name = str_dup("fried squirrel slab");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 1;
			break;
		case 15:/*roast rabbit*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a plate of roast rabbit" );
			free_string( fObj->name );
			fObj->name = str_dup("roast rabbit plate");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 7;
			break;
		case 16:/*boiled yams*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of boiled yams" );
			free_string( fObj->name );
			fObj->name = str_dup("boiled yam platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 8;
			break;
		case 17:/*braized carrots*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a plate of braized carrots" );
			free_string( fObj->name );
			fObj->name = str_dup("braized carrots plate");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 2;
			break;
		case 18:/*roast venison*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of roast venison" );
			free_string( fObj->name );
			fObj->name = str_dup("roast venison platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 5;
			break;
		case 19:/*broiled veal*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a platter of broiled veal" );
			free_string( fObj->name );
			fObj->name = str_dup("broiled veal platter");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 4;
			break;
		case 20:/*hard biscuits*/
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a plate of hard biscuits" );
			free_string( fObj->name );
			fObj->name = str_dup("hard biscuits plate");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 1;
			break;
		case 21:/*chocolates*/
			fObj->value[0] = 5;
			fObj->value[1] = 5;
			strcat( oBuf, "a heart shaped chocolate candy" );
			free_string( fObj->name );
			fObj->name = str_dup("heart chocolate candy");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 2;
			break;
		case 22:/*tarts*/
			fObj->value[0] = 5;
			fObj->value[1] = 5;
			strcat( oBuf, "a tart, heart shaped candy" );
			free_string( fObj->name );
			fObj->name = str_dup("heart tart candy");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * 2;
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "food" );
	return( fObj );
}
OBJ_DATA *faire_pill_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,8);
	int baseCost = 100;
	char oBuf[MSL];
	char staticBuf[MSL];
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1: /* dark brown */
			strcat( oBuf, "a dark brown pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 50;
			fObj->value[0] = 250;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 2: /* deep red pill */
			strcat( oBuf, "a deep red pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 100;
			fObj->value[0] = 300;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 3: /* deep blue pill */
			strcat( oBuf, "a deep blue pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 150;
			fObj->value[0] = 350;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 4: /* pale white */
			strcat( oBuf, "a pale white pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 200;
			fObj->value[0] = 400;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 5: /* gleaming silver pill */
			strcat( oBuf, "a gleaming silver pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 250;
			fObj->value[0] = 500;
			fObj->value[1] = rand_defensive_weave();
			fObj->value[2] = rand_defensive_weave();
			fObj->value[3] = rand_defensive_weave();
			fObj->value[4] = rand_defensive_weave();
			break;
		case 6: /* milky white pill */
			strcat( oBuf, "a milky white pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 50;
			fObj->value[0] = 250;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 7: /* blood red pill */
			strcat( oBuf, "a blood red pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 100;
			fObj->value[0] = 300;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
		case 8: /* shadow black pill */
			strcat( oBuf, "a shadow black pill" );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->wear_flags += ITEM_TAKE;
			fObj->weight = 2;
			fObj->cost = baseCost * 150;
			fObj->value[0] = 350;
			fObj->value[1] = rand_misc_weave();
			fObj->value[2] = rand_misc_weave();
			fObj->value[3] = rand_misc_weave();
			fObj->value[4] = rand_misc_weave();
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "an odd material" );
	return( fObj );
}
OBJ_DATA *faire_quiver_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,23);
	switch( base )
	{
		case 1:/*balsam*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 10;
			free_string( fObj->name );
			fObj->name = str_dup("balsam arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with balsam arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with balsam arrows.");
			fObj->cost = 500;
			break;
		case 2:/*pine arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 12;
			free_string( fObj->name );
			fObj->name = str_dup("pine arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with pine arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with pine arrows.");
			fObj->cost = 600;
			break;
		case 3:/*fir arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 14;
			free_string( fObj->name );
			fObj->name = str_dup("fir arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with fir arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with fir arrows.");
			fObj->cost = 700;
			break;
		case 4:/*cedar arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 16;
			free_string( fObj->name );
			fObj->name = str_dup("cedar arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with cedar arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with cedar arrows.");
			fObj->cost = 800;
			break;
		case 5:/*cypress arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 18;
			free_string( fObj->name );
			fObj->name = str_dup("cypress arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with cypress arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with cypress arrows.");
			fObj->cost = 900;
			break;
		case 6:/*juniper arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 20;
			free_string( fObj->name );
			fObj->name = str_dup("juniper arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with juniper arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with juniper arrows.");
			fObj->cost = 1000;
			break;
		case 7:/*spruce arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 22;
			free_string( fObj->name );
			fObj->name = str_dup("spruce arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with spruce arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with spruce arrows.");
			fObj->cost = 1100;
			break;
		case 8:/*hemlock arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 24;
			free_string( fObj->name );
			fObj->name = str_dup("hemlock arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with hemlock arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with hemlock arrows.");
			fObj->cost = 1200;
			break;
		case 9:/*maple arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 26;
			free_string( fObj->name );
			fObj->name = str_dup("maple arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with maple arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with maple arrows.");
			fObj->cost = 1300;
			break;
		case 10:/*mahogany arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 28;
			free_string( fObj->name );
			fObj->name = str_dup("mahogany arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with mahogany arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with mahogany arrows.");
			fObj->cost = 1400;
			break;
		case 11:/*poplar*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 30;
			free_string( fObj->name );
			fObj->name = str_dup("poplar arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with poplar arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with poplar arrows.");
			fObj->cost = 1500;
			break;
		case 12:/*walnut*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 32;
			free_string( fObj->name );
			fObj->name = str_dup("walnut arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with walnut arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with walnut arrows.");
			fObj->cost = 1600;
			break;
		case 13:/*rosewood*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 34;
			free_string( fObj->name );
			fObj->name = str_dup("rosewood arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with rosewood arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with rosewood arrows.");
			fObj->cost = 1700;
			break;
		case 14:/*beech*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 36;
			free_string( fObj->name );
			fObj->name = str_dup("beech arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with beech arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with beech arrows.");
			fObj->cost = 1800;
			break;
		case 15:/*teak*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 38;
			free_string( fObj->name );
			fObj->name = str_dup("teak arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with teak arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with teak arrows.");
			fObj->cost = 1900;
			break;
		case 16:/*ebony*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 40;
			free_string( fObj->name );
			fObj->name = str_dup("ebony arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with ebony arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with ebony arrows.");
			fObj->cost = 2000;
			break;
		case 17:/*redwood*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 42;
			free_string( fObj->name );
			fObj->name = str_dup("redwood arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with redwood arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with redwood arrows.");
			fObj->cost = 2100;
			break;
		case 18:/*satinwood*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 44;
			free_string( fObj->name );
			fObj->name = str_dup("satinwood arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with satinwood arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with satinwood arrows.");
			fObj->cost = 2200;
			break;
		case 19:/*greenheart*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 46;
			free_string( fObj->name );
			fObj->name = str_dup("greenheart arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with greenheart arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with greenheart arrows.");
			fObj->cost = 2300;
			break;
		case 20:/*leatherleaf arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 48;
			free_string( fObj->name );
			fObj->name = str_dup("leatherleaf arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with leatherleaf arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with leatherleaf arrows.");
			fObj->cost = 2500;
			break;
		case 21:/*willow arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 50;
			free_string( fObj->name );
			fObj->name = str_dup("willow arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with willow arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with willow arrows.");
			fObj->cost = 2600;
			break;
		case 22:/*elm arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 52;
			free_string( fObj->name );
			fObj->name = str_dup("elm arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with elm arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with elm arrows.");
			fObj->cost = 2800;
			break;
		case 23:/*yew arrows*/
			fObj->value[0] = 25;
			fObj->value[1] = 1;
			fObj->value[2] = 55;
			free_string( fObj->name );
			fObj->name = str_dup("yew arrow quiver");
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup("a quiver filled with yew arrows");
			free_string( fObj->description );
			fObj->description = str_dup("Here lies a quiver filled with yew arrows.");
			fObj->cost = 3000;
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "good solid wood" );
	fObj->wear_flags += ITEM_TAKE|ITEM_WEAR_SHOULDERS;
	return( fObj );
}
OBJ_DATA *faire_pipetabac_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,10), i = number_range(1,21), baseCost = 100;
	char oBuf[MSL];
	char staticBuf[MSL];
	int stems = number_range(1,15);
	char *stem_type[] = {
	"long thick stem",      "long thin stem",      "slender curving stem",
	"slightly curved stem", "arrow straight stem", "long narrow stem",
	"deeply curved stem",   "long straight stem",  "smoothly enamled stem",
	"richly carved stem",   "deeply curved stem",  "darkly stained stem",
	"lightly painted stem", "stem that twists upon itself twice",
	"stem that twists upon itself thrice"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1:
			fObj->value[0] = 10;
			fObj->value[1] = 10;
			strcat( oBuf, "a " );
			free_string( fObj->name );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 100;
			break;
		case 2:
			fObj->value[0] = 15;
			fObj->value[1] = 15;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 150;
			break;
		case 3:
			fObj->value[0] = 20;
			fObj->value[1] = 20;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 200;
			break;
		case 4:
			fObj->value[0] = 25;
			fObj->value[1] = 25;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 250;
			break;
		case 5:
			fObj->value[0] = 30;
			fObj->value[1] = 30;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 300;
			break;
		case 6:
			fObj->value[0] = 35;
			fObj->value[1] = 35;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 350;
			break;
		case 7:
			fObj->value[0] = 40;
			fObj->value[1] = 40;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 400;
			break;
		case 8:
			fObj->value[0] = 45;
			fObj->value[1] = 45;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 450;
			break;
		case 9:
			fObj->value[0] = 50;
			fObj->value[1] = 50;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 500;
			break;
		case 10:
			fObj->value[0] = 60;
			fObj->value[1] = 60;
			strcat( oBuf, "a " );
			strcat( oBuf, faire_pipewood_table[i-1].pName );
			strcat( oBuf, " pipe with a " );
			strcat( oBuf, stem_type[stems-1] );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost + 600;
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( faire_pipewood_table[i-1].pName );
	return( fObj );
}
OBJ_DATA *faire_angreal_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,2), i = number_range(1,20), baseCost = 1150, pwr = dice(2,10);
	int x = number_range(1,15);
	char oBuf[MSL];
	char staticBuf[MSL];
	char *angreal_array[] = {
	"a statuary", "a relief",    "a carving",   "a miniature", "a cameo",
	"a carving",  "a sculpture", "a figurine",  "a statuette", "a copy",
	"a likeness", "a bust",      "a depiction", "an icon",     "a figurine"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1:
		case 2:
			fObj->value[1] = pwr;
			fObj->wear_flags += ITEM_TAKE;
			fObj->wear_flags += ITEM_WEAR_ANGREAL;
			strcat( oBuf, angreal_array[x-1] );
			strcat( oBuf, " of " );
			strcat( oBuf, faire_angreal1_table[i-1].aName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * (pwr * 10);
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "an extremely hard material" );
	return( fObj );
}
OBJ_DATA *raffle_angreal_obj( OBJ_DATA *fObj )
{
	int base = number_range(1,2), i = number_range(1,20), baseCost = 1150, pwr = number_range(12,15);
	int x = number_range(1,15);
	char oBuf[MSL];
	char staticBuf[MSL];
	char *angreal_array[] = {
	"a statuary", "a relief",    "a carving",   "a miniature", "a cameo",
	"a carving",  "a sculpture", "a figurine",  "a statuette", "a copy",
	"a likeness", "a bust",      "a depiction", "an icon",     "a figurine"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	switch( base )
	{
		case 1:
		case 2:
			fObj->value[1] = pwr;
			fObj->wear_flags += ITEM_TAKE;
			fObj->wear_flags += ITEM_WEAR_ANGREAL;
			strcat( oBuf, angreal_array[x-1] );
			strcat( oBuf, " of " );
			strcat( oBuf, faire_angreal1_table[i-1].aName );
			free_string( fObj->name );
			fObj->name = str_dup( oBuf );
			free_string( fObj->short_descr );
			fObj->short_descr = str_dup( oBuf );
			strcat( staticBuf, oBuf );
			strcat( staticBuf, "." );
			free_string( fObj->description );
			fObj->description = str_dup( staticBuf );
			fObj->cost = baseCost * (pwr * 10);
			break;
	}
	free_string( fObj->material );
	fObj->material = str_dup( "an extremely hard material" );
	return( fObj );
}
OBJ_DATA *faire_notepaper_obj( OBJ_DATA *fObj )
{
	int baseCost = 120;
	char oBuf[MSL];
	char staticBuf[MSL];
	int pattern_colors = number_range(1,20);
	char *pattern_color_array[] = {
	" green tinted", " blue hued",   " yellow tinted",  " red tinged",
	" scarlet hued", " navy tinted", " crimson tinted", " hunter green",
	" forest green", " light green", " light blue",     " aqua green",
	" dark blue",    " sea green",   " cobalt blue",    " emerald green",
	" flame red",    " bright red",  " dark red",       " deep red"
	};
	int borders = number_range(1,12);
	char *border_array[] = {
	" gentle spirals",       " sweeping feathers", " interlocking rings",
	" tairen maze",          " creeping vines",    " braided ropes",
	" intricate scrollwork", " connectings rings", " twining flowers",
	" parallel bars",        " swooping gulls",    " falling leaves"
	};
	int paper_type = number_range(1,9);
	char *paper_type_array[] = {
	" paper", " parchment", " vellum", " scroll", " stationary", " papyrus",
	" foolscap", " style",  " slate"
	};
	int piece_type = number_range(1,7);
	char *piece_type_array[] = {
	" slip", " sheet", " piece", " scrap", " leaf", " leaflet", " broadsheet"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	strcat( oBuf, "a" );
	strcat( oBuf, piece_type_array[piece_type-1] );
	strcat( oBuf, " of" );
	strcat( oBuf, paper_type_array[paper_type-1] );
	strcat( oBuf, " bordered with" );
	strcat( oBuf, pattern_color_array[pattern_colors-1] );
	strcat( oBuf, border_array[borders-1] );
	strcat( staticBuf, oBuf );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->weight = 2;
	fObj->condition = 100;
	fObj->cost = number_range(1,5) * 10 + baseCost;
	free_string( fObj->material );
	fObj->material = str_dup( "writing material" );
	return( fObj );
}
OBJ_DATA *faire_lockpick_obj( OBJ_DATA *fObj )
{
	int baseCost = 700;
	char oBuf[MSL];
	char staticBuf[MSL];
	int tips = number_range(1,64);
	char *tip_array[] = {
	 "cat's eye gem-stone", "round bloodstone",       "clear-white topaz",
	 "angel's skin coral",  " blue spinel",           " clear quartz",
	 "fire-colored agate",  "yellow-gold crysoberyl", "black moonstone",
	 "bright-green peridot","amber gemstone",         "yellow citrine gemstone",
	 "yellow garnet",       "smokey-colored quartz",  "milky-white pearl",
	 "reddish-pink tourmaline","rock-crystal","fire opal",
	 "blue sapphire", "diamond", "ruby-red alexandrite",
	 "yellowish-orange jade stone","gray-white jade stone", "vibrant purple amethyst",
	 "rose-colored quartz", "bluish-colored quartz", "milky-white topaz",
	 "angel's skin coral", "greenish-blue aquamarine", "pale-blue turquoise",
	 "pink sapphire","green emerald","rock-crystal",
	 "yellowed ivory","bright green beryl stone","blue turquoise",
	 "light red morganite stone","black pearl","black opal",
	 "cat's-eye beryl gemstone", "black onyx stone","golden colored sapphire",
	 "white pearl", "red ruby","light pink conch pearl",
	 "reddish-pink alexandrite", "demantoid garnet stone","rose-colored quartz",
	 "green beryl stone", "lapis lazuli stone","opaque colored jasper",
	 "deep blue tigereye stone", "reddish-brown sardonyx","sea-green aquamarine gem",
	 "milk-and-honey colored cat's-eye chrysoberyl", "star-diopside","cat's-eye diopside",
	 "reddish-cast sunstone", "jet stone","snowflake obsidian",
	 "imperial topaz", "star ruby","turquoise stone",
	 "demantoid garnet stone"
	};
	int metals = number_range(1,7);
	char *metal_array[] = {
	"a bronze", "a brass", "an iron", "a steel", "a silver", "a bone", "a platinum"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	strcat( oBuf, metal_array[metals-1] );
	strcat( oBuf, " lockpick tipped with a " );
	strcat( oBuf, tip_array[tips-1] );
	strcat( staticBuf, oBuf );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->weight = 2;
	fObj->condition = 100;
	fObj->cost = number_range(5,10) * 10 + baseCost;
	fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
	free_string( fObj->material );
	fObj->material = str_dup( "top quality materials" );
	return( fObj );
}
OBJ_DATA *faire_mirror_obj( OBJ_DATA *fObj )
{
	int baseCost = 500;
	char oBuf[MSL];
	char staticBuf[MSL];
	int frame = number_range(1,20);
	char *frame_array[] = {
	" pine", " fir", " cedar", " cypress", " juniper",
	" spruce", " hemlock", " maple", " mahogany", " poplar",
	" walnut", " rosewood", " beech", " teak", " ebony",
	" redwood", " satinwood", " leatherleaf", " willow", " yew"
	};
	int metals = number_range(1,5);
	char *metal_array[] = {
	" bronze", " brass", " gold", " silver", " platinum"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( oBuf, "a" );
	strcat( oBuf, metal_array[metals-1] );
	strcat( oBuf, " gilt mirror with a" );
	strcat( oBuf, frame_array[frame-1] );
	strcat( oBuf, " frame" );
	strcat( staticBuf, "Here lies " );
	strcat( staticBuf, oBuf );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
 	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->weight = 10;
	fObj->condition = 100;
	fObj->cost = number_range(5,10) * 10 + baseCost;
	free_string( fObj->material );
	fObj->material = str_dup( "solid wood" );
	return( fObj );
}
OBJ_DATA *faire_razor_obj( OBJ_DATA *fObj )
{
	int baseCost = 500;
	char oBuf[MSL];
	char staticBuf[MSL];
//	int guild = number_range(1,MaxFaireGuilds);
	int guild = number_range(2,MAX_CLAN-2);
	int metals = number_range(1,5);
	char *metal_array[] = {
	" bronze", " brass", " gold", " silver", " platinum"
	};
	int emblems = number_range(1,4);
	char *emblem_type[] = {
	" emblazoned", " engraved", " scribed", " etched"
	};
	int crests = number_range(1,7);
	char *crest_type[] = {
	" seal", " crest", " emblem", " sigil", " standard", " insignia", " stamp"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( oBuf, "a" );
	strcat( oBuf, metal_array[metals-1] );
	strcat( oBuf, " razor" );
	strcat( oBuf, emblem_type[emblems-1] );
	strcat( oBuf, " with the" );
	strcat( oBuf, crest_type[crests-1] );
	strcat( oBuf, " of the " );
	strcat( oBuf, clan_table[guild].who_name );
//	strcat( oBuf, faire_guild_table[guild-1].gName );
	strcat( oBuf, " guild" );
	strcat( staticBuf, "Here lies " );
	strcat( staticBuf, oBuf );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->weight = 5;
	fObj->condition = 100;
	fObj->cost = number_range(5,10) * 10 + baseCost;
	fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
	free_string( fObj->material );
	fObj->material = str_dup( metal_array[metals-1] );
	return( fObj );
}
OBJ_DATA *faire_brush_obj( OBJ_DATA *fObj )
{
	int baseCost = 500;
	char oBuf[MSL];
	char staticBuf[MSL];
//	int guild = number_range(1,MaxFaireGuilds);
	int guild = number_range(2,MAX_CLAN-2);
	int metals = number_range(1,5);
	char *metal_array[] = {
	" bronze", " brass", " gold", " silver", " platinum"
	};
	int emblems = number_range(1,4);
	char *emblem_type[] = {
	" emblazoned", " engraved", " scribed", " etched"
	};
	int crests = number_range(1,7);
	char *crest_type[] = {
	" seal", " crest", " emblem", " sigil", " standard", " insignia", " stamp"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( oBuf, "a" );
	strcat( oBuf, metal_array[metals-1] );
	strcat( oBuf, " brush" );
	strcat( oBuf, emblem_type[emblems-1] );
	strcat( oBuf, " with the" );
	strcat( oBuf, crest_type[crests-1] );
	strcat( oBuf, " of the " );
	strcat( oBuf, clan_table[guild].who_name );
//	strcat( oBuf, faire_guild_table[guild-1].gName );
	strcat( oBuf, " guild" );
	strcat( staticBuf, "Here lies " );
	strcat( staticBuf, oBuf );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->weight = 5;
	fObj->condition = 100;
	fObj->cost = number_range(5,10) * 10 + baseCost;
	fObj->wear_flags += ITEM_TAKE|ITEM_HOLD;
	free_string( fObj->material );
	fObj->material = str_dup( metal_array[metals-1] );
	return( fObj );
}

OBJ_DATA *faire_fireoil_obj( OBJ_DATA *fObj )
{
	int baseCost = 800;
	char oBuf[MSL];
	char staticBuf[MSL];
	int sticks = number_range(1,5);
	char *stick_type[] = {
	" round tube", " flask", " slender rod", " long stick", " cylinder"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( oBuf, "a" );
	strcat( oBuf, stick_type[sticks-1] );
	strcat( oBuf, " with a short fuse portruding from the end" );
	strcat( staticBuf, "Here lies " );
	strcat( staticBuf, oBuf );
	strcat( staticBuf, "." );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->weight = 5;
	fObj->condition = 100;
	fObj->cost = number_range(7,10) * (10 + baseCost);
	fObj->wear_flags += ITEM_TAKE;
	fObj->value[0] = 5;
	fObj->value[2] = 500;
	free_string( fObj->material );
	fObj->material = str_dup( "some black powdery substance" );
	return( fObj );
}
OBJ_DATA *make_special( OBJ_DATA *fObj )
{
	int chance = number_range(1,10);
    AFFECT_DATA *af;

	if (number_percent() <= 75)
	{
		af = new_affect();

		if (number_percent() < 50)
    		af->location   =   APPLY_HIT;
		else
    		af->location   =   APPLY_MANA;

		af->modifier   =   number_range(2,12) * 25;
		af->where      =   TO_OBJECT;
		af->type       =   -1;
	   	af->duration   =   -1;
	   	af->bitvector  =   0;
		af->level      =   250;
		af->next       =   fObj->affected;
		fObj->affected =   af;
	}


	af = new_affect();
	switch( chance )
	{
		case 1:
    		af->location   =   APPLY_STR;
		    af->modifier   =   number_range(10,25);
			break;
		case 2:
    		af->location   =   APPLY_DEX;
		    af->modifier   =   number_range(10,25);
			break;
		case 3:
    		af->location   =   APPLY_CON;
		    af->modifier   =   number_range(10,25);
			break;
		case 4:
    		af->location   =   APPLY_WIS;
		    af->modifier   =   number_range(10,25);
			break;
		case 5:
    		af->location   =   APPLY_INT;
		    af->modifier   =   number_range(10,25);
			break;
		case 6:
    		af->location   =   APPLY_CHA;
		    af->modifier   =   number_range(10,25);
			break;
		case 7:
    		af->location   =   APPLY_INS;
		    af->modifier   =   number_range(10,25);
			break;
		case 8:
    		af->location   =   APPLY_REF;
		    af->modifier   =   number_range(10,25);
			break;
		case 9:
    		af->location   =   APPLY_HITROLL;
		    af->modifier   =   number_range(10,25);
			break;
		case 10:
    		af->location   =   APPLY_DAMROLL;
		    af->modifier   =   number_range(10,25);
			break;
	}


	af->where	   =   TO_OBJECT;
        af->type       =   -1;
   	af->duration   =   -1;
   	af->bitvector  =   0;
	af->level      =   250;
        af->next       =   fObj->affected;
	fObj->affected =   af;

	fObj->cost *= number_range(15, 20);
	fObj->cost /= 10;
	return( fObj );
}
OBJ_DATA *make_special_raffle( OBJ_DATA *fObj )
{
	int chance = number_range(1,12);
    AFFECT_DATA *af = new_affect();
	switch( chance )
	{
		case 1:
    		af->location   =   APPLY_STR;
		    af->modifier   =   number_range(17,25);
			break;
		case 2:
    		af->location   =   APPLY_DEX;
		    af->modifier   =   number_range(17,25);
			break;
		case 3:
    		af->location   =   APPLY_CON;
		    af->modifier   =   number_range(17,25);
			break;
		case 4:
    		af->location   =   APPLY_WIS;
		    af->modifier   =   number_range(17,25);
			break;
		case 5:
    		af->location   =   APPLY_INT;
		    af->modifier   =   number_range(17,25);
			break;
		case 6:
    		af->location   =   APPLY_CHA;
		    af->modifier   =   number_range(17,25);
			break;
		case 7:
    		af->location   =   APPLY_INS;
		    af->modifier   =   number_range(17,25);
			break;
		case 8:
    		af->location   =   APPLY_REF;
		    af->modifier   =   number_range(17,25);
			break;
		case 9:
    		af->location   =   APPLY_HITROLL;
		    af->modifier   =   number_range(17,25);
			break;
		case 10:
    		af->location   =   APPLY_DAMROLL;
		    af->modifier   =   number_range(17,25);
			break;
		case 11:
    		af->location   =   APPLY_HIT;
		    af->modifier   =   ( number_range(1,10) * 10 );
			break;
		case 12:
    		af->location   =   APPLY_MANA;
		    af->modifier   =   ( number_range(1,10) * 10 );
			break;
	}
	af->where	   =   TO_OBJECT;
    af->type       =   -1;
   	af->duration   =   -1;
   	af->bitvector  =   0;
	af->level      =   250;
    af->next       =   fObj->affected;
	fObj->affected =   af;
	fObj->cost += ( af->modifier * 20 );
	return( fObj );
}
OBJ_DATA *faire_bow_obj( OBJ_DATA *fObj )
{
	int woods = number_range(1,20);
	int bows = number_range(1,5);
	int costMod = 0;
	char oBuf[MSL];
	char staticBuf[MSL];
	char * bow_types[] = {
	" shortbow", " longbow", " recurved bow", " composite bow", " crossbow"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	free_string( fObj->material );
	fObj->material = str_dup( faire_bow_wood_table[woods-1].pName );
	strcat( oBuf, "a " );
	strcat( oBuf, fObj->material );
	strcat( oBuf, bow_types[bows-1] );

	if ( bows == 1 ) fObj->value[0] = WEAPON_SHORTBOW;
	if ( bows == 2 ) fObj->value[0] = WEAPON_LONGBOW;
	if ( bows == 3 ) fObj->value[0] = WEAPON_RECURVEDBOW;
	if ( bows == 4 ) fObj->value[0] = WEAPON_COMPOSITEBOW;
	if ( bows == 5 ) fObj->value[0] = WEAPON_CROSSBOW;

	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	if ( chance(10) ) {
		strcat( oBuf, " carved with running wolves" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " carved with howling wolves" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " carved with moons and stars" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " carved with twining vines" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " decorated with eagle feathers" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " decorated with hawk feathers" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " decorated with falcon feathers" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " decorated with heron feathers" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with a leather wrapped grip" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with a braided leather grip" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with a darkened leather grip" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with a snakeskin grip" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with a doeskin grip" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with a wolf hide grip" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with bone tips at the ends" );
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with iron tips at the ends" );
		costMod = 2;
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with steel tips at the ends" );
		costMod = 5;
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with silver tips at the ends" );
		costMod = 10;
	}
	else if ( chance(10) ) {
		strcat( oBuf, " with gold tips at the ends" );
		costMod = 25;
	}
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->cost = 100 * ( (bows * woods) * costMod );
	strcat( staticBuf, oBuf );
	fObj->wear_flags += ITEM_TAKE|ITEM_WIELD;
	return( fObj );
}
OBJ_DATA *faire_journal_obj( OBJ_DATA *fObj )
{
	int mat = number_range(1,10);
	char oBuf[MSL];
	char staticBuf[MSL];
	char * mat_table[] = {
		"soft leather bound","cured leather bound",
		"stiff leather bound","boiled leather bound",
		"supple leather bound","thin wood backed",
		"scored leather bound","etched wood backed",
		"treated leather bound","treated wood backed"
	};
	oBuf[0] = '\0';
	staticBuf[0] = '\0';
	strcat( staticBuf, "Here lies " );
	free_string( fObj->material );
	fObj->material = str_dup( mat_table[mat-1] );
	strcat( oBuf, "a " );
	strcat( oBuf, fObj->material );
	strcat( oBuf, " journal" );
	free_string( fObj->name );
	fObj->name = str_dup( oBuf );
	free_string( fObj->short_descr );
	fObj->short_descr = str_dup( oBuf );
	free_string( fObj->description );
	fObj->description = str_dup( staticBuf );
	fObj->cost = number_range(1, 10) * 10 + 800;
	strcat( staticBuf, oBuf );
	fObj->wear_flags += ITEM_TAKE;
	return( fObj );
}
OBJ_DATA *faire_rand_itype( OBJ_DATA *fObj, bool Raffle )
{
	if ( Raffle )
	{
		int chance = number_range(1,20);
		switch( chance )
		{
			default: fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
			case 1: case 2: case 3: fObj->item_type = ITEM_LIGHT; faire_light_obj( fObj ); break;
			case 4: case 5: case 6: case 7: fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;
			case 8: case 9: case 10: case 11: case 12: fObj->item_type = ITEM_ARMOR; faire_armor_obj( fObj ); break;
			case 13: case 14: case 15:  fObj->item_type = ITEM_CONTAINER; faire_container_obj( fObj ); break;
			case 16:  fObj->item_type = ITEM_ANGREAL_MALE; raffle_angreal_obj( fObj ); break;
			case 17:  fObj->item_type = ITEM_ANGREAL_FEMALE; raffle_angreal_obj( fObj ); break;
		}
		if ( fObj->cost < 1 ) fObj->cost = 1000;
		if ( fObj->weight < 1 ) fObj->weight = 10;
		if ( fObj->level < 1 ) fObj->level = 1;
		if ( fObj->condition < 100 ) fObj->condition = 100;
		if ( !CAN_WEAR( fObj, ITEM_TAKE ) )
			fObj->wear_flags += ITEM_TAKE;
		free_string( fObj->guild );
		fObj->guild = str_dup("Loner");
		if ( fObj->item_type == ITEM_ARMOR
		|| fObj->item_type == ITEM_WEAPON
		|| fObj->item_type == ITEM_TREASURE
		|| fObj->item_type == ITEM_STAFF
		|| fObj->item_type == ITEM_WAND
		|| fObj->item_type == ITEM_LIGHT )
			make_special_raffle( fObj );
	}
	else
	{
		int chance = number_range(1,28);
		switch( chance )
		{
			default: fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
			case 1:  fObj->item_type = ITEM_LIGHT; faire_light_obj( fObj ); break;
			case 2:  fObj->item_type = ITEM_SCROLL; faire_scroll_obj( fObj ); break;
//			case 3:  fObj->item_type = ITEM_WAND; faire_wand_obj( fObj ); break;
//			case 4:  fObj->item_type = ITEM_STAFF; faire_staff_obj( fObj ); break;
			case 3:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
			case 4:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
			case 5:  fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;
			case 6:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
			case 7:  fObj->item_type = ITEM_ARMOR; faire_armor_obj( fObj ); break;
			case 8:  fObj->item_type = ITEM_POTION; faire_potion_obj( fObj ); break;
			case 9:  fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;
			case 10: fObj->item_type = ITEM_CONTAINER; faire_container_obj( fObj ); break;
			case 11: fObj->item_type = ITEM_DRINK_CON; faire_drink_obj( fObj ); break;
			case 12:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
			case 13:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
//			case 12: fObj->item_type = ITEM_FOOD; faire_food_obj( fObj ); break;
//			case 13: fObj->item_type = ITEM_PILL; faire_pill_obj( fObj ); break;
			case 14: fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
//			case 15: fObj->item_type = ITEM_QUIVER; faire_quiver_obj( fObj ); break;
			case 15: fObj->item_type = ITEM_ARMOR; faire_armor_obj( fObj ); break;
			case 16: fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;

//			case 17: fObj->item_type = ITEM_PIPE_TABAC; faire_pipetabac_obj( fObj ); break;
//			case 18: fObj->item_type = ITEM_PIPE_TABAC; faire_pipetabac_obj( fObj ); break;
			case 17: fObj->item_type = ITEM_ARMOR; faire_armor_obj( fObj ); break;
//			case 18: fObj->item_type = ITEM_WEAPON; faire_bow_obj( fObj ); break;
			case 18: fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;
			case 19:
				if ( number_percent() / 2 > number_percent() && !MaleAngrealExists )
				{
					fObj->item_type = ITEM_ANGREAL_MALE; faire_angreal_obj( fObj );
					MaleAngrealExists = TRUE;
				}
				else
				{
					fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj );
					
				}
				break;
			case 20:
				if ( number_percent() / 2 > number_percent() && !FemaleAngrealExists )
				{
					fObj->item_type = ITEM_ANGREAL_FEMALE; faire_angreal_obj( fObj );
					FemaleAngrealExists = TRUE;
				}
				else
				{
					fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj );
				}
				break;
			case 21:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
//			case 21: fObj->item_type = ITEM_NOTEPAPER; faire_notepaper_obj( fObj ); break;
			case 22: fObj->item_type = ITEM_LOCKPICK; faire_lockpick_obj( fObj ); break;
//			case 23: fObj->item_type = ITEM_MIRROR; faire_mirror_obj( fObj ); break;
//			case 24: fObj->item_type = ITEM_RAZOR; faire_razor_obj( fObj ); break;
			case 23: fObj->item_type = ITEM_ARMOR; faire_armor_obj( fObj ); break;
			case 24: fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;
/*			case 25: fObj->item_type = ITEM_FIREOIL; faire_fireoil_obj( fObj ); break; */
			case 25: fObj->item_type = ITEM_ARMOR; faire_armor_obj( fObj ); break;
			case 26: fObj->item_type = ITEM_WEAPON; faire_weapon_obj( fObj ); break;
			case 27: fObj->item_type = ITEM_JOURNAL; faire_journal_obj( fObj ); break;
			case 28:  fObj->item_type = ITEM_TREASURE; faire_treasure_obj( fObj ); break;
//			case 28: fObj->item_type = ITEM_BRUSH; faire_brush_obj( fObj ); break;

		}
		if ( fObj->cost < 1 ) fObj->cost = 1000;
		if ( fObj->weight < 1 ) fObj->weight = 10;
		if ( fObj->level < 1 ) fObj->level = 1;
		if ( fObj->condition < 100 ) fObj->condition = 100;
		if ( !CAN_WEAR( fObj, ITEM_TAKE ) )
			fObj->wear_flags += ITEM_TAKE;
		free_string( fObj->guild );
		fObj->guild = str_dup("Loner");
		if ( fObj->item_type == ITEM_ARMOR
		|| fObj->item_type == ITEM_WEAPON
		|| fObj->item_type == ITEM_TREASURE
		|| fObj->item_type == ITEM_STAFF
		|| fObj->item_type == ITEM_WAND
		|| fObj->item_type == ITEM_LIGHT )
			make_special( fObj );
	}
	return fObj;
}
void init_faire_objects( CHAR_DATA *mob, int FaireLocation, int fLookup )
{
	OBJ_DATA *fObj;
	int numObj = number_range(4,7);

	for (; numObj > 0; numObj--)
	{
		fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
		faire_rand_itype( fObj, FALSE );
		obj_to_char( fObj, mob );
	}

	return;

	switch( numObj )
	{
		case 2:
		case 3:
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			break;
		case 4:
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			break;
		case 5:
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			break;
		case 6:
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			break;
		case 7:
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			fObj = create_object( get_obj_index( FaireObjectVnum ), 1 );
			faire_rand_itype( fObj, FALSE );
				obj_to_char( fObj, mob );
			break;
	}
	return;
}
void init_faire_rooms( CHAR_DATA *ch, int location, int fLookup )
{
	ROOM_INDEX_DATA *fRoom;
	int vnum, ugLoc = 0, repLoc = 0, balLoc = 0, bank = 0, tatLoc = 0, strLoc = 0;
	char nameBuf[MSL];
	char roomBuf[MSL*4];
	int tent_dark_color, tent_bright_color;
	int tent_striped_color1, tent_striped_color2, tent_striped_color3;
	int tent_top_type;
	int tent_location, sign_location1, sign_location2;
	/* Cierin */
	int mAnimals = number_range(1, 3);
	/* end Cierin */

	char *tent_dark_color_array[] = {
	" dark", " deep", " shadowy", " gloomy", " swarthy"
	};
	char *tent_bright_color_array[] = {
	" bright", " shiny", " flambouyant", " scintillating", " brilliant"
	};
	char *tent_striped_color_array1[] = {
	" blue and green", " red and black", " gray and blue", " blue and gray", " black and silver"
	};
	char *tent_striped_color_array2[] = {
	" green and blue", " black and red", " blue and gray", " green and gray", " black and gold"
	};
	char *tent_striped_color_array3[] = {
	" red and green", " purple and gray", " blue and white", " black and white", " green and silver"
	};
	char *tent_top_array[] = {
	" with a sloped roof", " with a slanted top", " with a v-shaped top", " with a domed top",
	" with a conical top", " with an a-shaped frame", " with an open-air top", " with a slitted roof"
	};
	char *tent_location_array[] = {
	" near the road.", " beside the road.", " just off the road.", " close to the road.", " along the road."
	};
	char *sign_location_array1[] = {
	" hanging just above", " planted in the ground before", " staked into the ground in front of",
	" hanging from the top of", " standing just before"
	};
	char *sign_location_array2[] = {
	" the opening to", " the flap of", " the entrance to", " the entryway to", " the way into"
	};

	/* Cierin */
	char *mAnimals_array[] = {
	"monkey", "bear", "lion"
	};
	/* end Cierin */

	roomBuf[0] = '\0';
	nameBuf[0] = '\0';
	bank = number_range( FaireRoomFirst, FaireRoomLast );
	ugLoc = number_range( FaireRoomFirst, FaireRoomLast );
	repLoc = number_range( FaireRoomFirst, FaireRoomLast );
	balLoc = number_range( FaireRoomFirst, FaireRoomLast );
	tatLoc = number_range( FaireRoomFirst, FaireRoomLast );
	strLoc = number_range( FaireRoomFirst, FaireRoomLast );
	FaireBankRoom = bank;
	FaireRepairRoom = repLoc;
	FaireUpgradeRoom = ugLoc;
	FaireBalanceRoom = balLoc;
	FaireTattooRoom = tatLoc;
	FaireStringRoom = strLoc;
	if (ch != NULL)
	ptc( ch, "BankRoom: %d\n\rRepairRoom %d\n\rUpgradeRoom %d\n\rBalanceRoom %d\n\rTattooRoom %d\n\rStringRoom %d",
		FaireBankRoom,FaireRepairRoom,FaireUpgradeRoom,FaireBalanceRoom,FaireTattooRoom,FaireStringRoom);

    for ( vnum = FaireRoomFirst; vnum <= FaireRoomLast; vnum++ )
    {
		if ( ( fRoom = get_room_index( vnum ) ) )
		{
			if ( fRoom == NULL )
			{
				bugf("FAIRE:: Vnum %s non-existant in init_faire_rooms.", vnum);
				return;
			}
			sprintf( nameBuf, "The Faire in %s", capit2( faire_location_table[fLookup].pName ) );
			free_string( fRoom->name );
			fRoom->name[0] = '\0';
			free_string( fRoom->description );
			fRoom->description[0] = '\0';
			strcat( roomBuf, "There is a" );
			/* Set the first tent color.
			 */
			if ( chance(50) ) {
				tent_dark_color = number_range(1,5);
				strcat( roomBuf, tent_dark_color_array[tent_dark_color-1] );
			}
			else {
				tent_bright_color = number_range(1,5);
				strcat( roomBuf, tent_bright_color_array[tent_bright_color-1] );
			}
			/* Set the second tent color
			 */
			if ( chance(33) ) {
				tent_striped_color1 = number_range(1,5);
				strcat( roomBuf, tent_striped_color_array1[tent_striped_color1-1] );
			}
			else if ( chance(33) ) {
				tent_striped_color2 = number_range(1,5);
				strcat( roomBuf, tent_striped_color_array2[tent_striped_color2-1] );
			}
			else {
				tent_striped_color3 = number_range(1,5);
				strcat( roomBuf, tent_striped_color_array3[tent_striped_color3-1] );
			}
			strcat( roomBuf, " tent" );
			/* Set the tent roof type
			 */
			tent_top_type = number_range(1,8);
			strcat( roomBuf, tent_top_array[tent_top_type-1] );
			/* Set the tent location
			 */
			tent_location = number_range(1,5);
			strcat( roomBuf, tent_location_array[tent_location-1] );
			strcat( roomBuf, " A sign is" );
			/* Set the sign location
			 */
			sign_location1 = number_range(1,5);
			strcat( roomBuf, sign_location_array1[sign_location1-1] );
			/* Plant the sign at the location
			 */
			sign_location2 = number_range(1,5);
			strcat( roomBuf, sign_location_array2[sign_location2-1] );
			strcat( roomBuf, " the tent." );

			/* Cierin */
			if ( chance( 20 ) )
			{
				strcat( roomBuf, "Nearby, a large cage holds several " );
				strcat( roomBuf, mAnimals_array[mAnimals-1] );
				strcat( roomBuf, " ." );
			}
			else if ( chance( 20 ) )
			{
				strcat( roomBuf, "Close to here, an oddly shaped cage holds a few " );
				strcat( roomBuf, mAnimals_array[mAnimals-1] );
				strcat( roomBuf, " ." );
			}
			/* end Cierin */

			if ( vnum == bank )
			{
				strcat( roomBuf, " There has also been a temporary reserve set up between the tents for financial purposes." );
				fRoom->room_flags += ROOM_BANK;
			}
			if ( vnum == ugLoc )
			{
				strcat( roomBuf, " A tinker has also set up shop here for item upgrades." );
			}
			if ( vnum == repLoc )
			{
				strcat( roomBuf, " A tinker has also set up shop here for item repairs." );
			}
			if ( vnum == balLoc )
			{
				strcat( roomBuf, " A tinker has also set up shop here for weapon balancing." );
			}
			if ( vnum == tatLoc )
			{
				strcat( roomBuf, " An aged old artist has erected a small cart and chair for tattooing." );
			}
			if ( vnum == strLoc )
			{
				strcat( roomBuf, " An wizened woman has erected a small shoppe here for the changing on items into other items." );
			}
			free_string( fRoom->name );
			fRoom->name = str_dup( nameBuf );
			free_string( fRoom->description );
			fRoom->description = format_string( str_dup(roomBuf) );
			roomBuf[0] = '\0';
			nameBuf[0] = '\0';
		}
    }
    return;
}
void init_faire_portal( int location, int fLookup )
{
	OBJ_DATA *portal = faire_portal;
	OBJ_DATA *wayback = faire_wayback;
	OBJ_DATA *sign = faire_sign;

	if (portal == NULL)	portal = create_object( get_obj_index( OBJ_VNUM_PORTAL ), 0 );
	else			obj_from_room(portal);

	faire_portal = portal;
	portal->item_type =   ITEM_PORTAL;
	portal->weight =      1;
	portal->cost =        0;
	portal->timer =       32000;
	free_string( portal->name );
	portal->name =        str_dup( "tent" );
	free_string( portal->short_descr );
	portal->short_descr = str_dup( faire_location_table[fLookup].portalName );
	free_string( portal->description );
	portal->description = str_dup( faire_location_table[fLookup].portalName );
	portal->value[0] =    0;
	portal->value[1] =    EX_ISDOOR|EX_NOCLOSE;
	portal->value[2] =    GATE_NORMAL_EXIT;
	portal->value[3] =    FaireRoomFirst;
	portal->value[4] =    1;
	obj_to_room( portal, get_room_index( faire_location_table[fLookup].vnum ) );

	if (wayback == NULL)	wayback = create_object( get_obj_index( OBJ_VNUM_PORTAL ), 0 );
	else			obj_from_room(wayback);

	faire_wayback = wayback;
	wayback->item_type =   ITEM_PORTAL;
	wayback->weight =      1;
	wayback->cost =        0;
	wayback->timer =       32000;
	free_string( wayback->name );
	wayback->name =        str_dup( faire_location_table[fLookup].portalName );
	free_string( wayback->short_descr );
	wayback->short_descr = str_dup( faire_location_table[fLookup].portalName );
	free_string( wayback->description );
	wayback->description = str_dup( faire_location_table[fLookup].portalName );
	wayback->value[0] =    0;
	wayback->value[1] =    EX_ISDOOR|EX_NOCLOSE;
	wayback->value[2] =    GATE_NORMAL_EXIT;
	wayback->value[3] =    faire_location_table[fLookup].vnum;
	obj_to_room( wayback, get_room_index( FaireRoomFirst ) );

	if (sign == NULL)	sign = create_object( get_obj_index( FaireSignVnum ), 0 );
	else			obj_from_room(sign);

	faire_sign = sign;
	sign->item_type =     ITEM_FURNITURE;
	sign->weight =        1;
	sign->cost =          0;
	sign->timer =         32000;
	free_string( sign->name );
	sign->name =          str_dup("faire sign");
	free_string( sign->short_descr );
	sign->short_descr =   str_dup("a sign pointing to a Faire");
	free_string( sign->description );
	sign->description =   str_dup("A sign points towards a nearby tent, leading into the Faire.");
	obj_to_room( sign, get_room_index( faire_location_table[fLookup].vnum ) );
	return;
}
int rand_defensive_weave( )
{
    if      (chance(10)) return skill_lookup("defense"); /* check */
    else if (chance(10)) return skill_lookup("vigor"); /* check */
    else if (chance(10)) return skill_lookup("shield");
    else if (chance(10)) return skill_lookup("advanced shield");
    else if (chance(10)) return skill_lookup("greater shield");
    else if (chance(10)) return skill_lookup("fire shield");
    else if (chance(10)) return skill_lookup("large");
    else if (chance(10)) return skill_lookup("glow");
    else if (chance(10)) return skill_lookup("speed");
    else if (chance(10)) return skill_lookup("endurance");
    else if (chance(10)) return skill_lookup("hunters instinct");
    else if (chance(10)) return skill_lookup("panther reflex");
    else return skill_lookup("invert");
}
int rand_misc_weave( )
{
	if (chance(10)) return skill_lookup("cure blinding");
	else if (chance(10)) return skill_lookup("cure critical");
	else if (chance(10)) return skill_lookup("cure disease");
	else if (chance(10)) return skill_lookup("cure light");
	else if (chance(10)) return skill_lookup("cure poison");
	else if (chance(10)) return skill_lookup("cure serious");
	else if (chance(10)) return skill_lookup("see shadow");
	else if (chance(10)) return skill_lookup("see light");
	else if (chance(10)) return skill_lookup("see hidden");
	else if (chance(10)) return skill_lookup("see invis");
	else if (chance(10)) return skill_lookup("seeing");
	else if (chance(10)) return skill_lookup("smell poison");
	else if (chance(10)) return skill_lookup("heal");
	else if (chance(10)) return skill_lookup("remove curse");
	else if (chance(10)) return skill_lookup("repair flesh");
	else if (chance(10)) return skill_lookup("mass heal");
	else return skill_lookup("repair flesh");
}
int rand_offensive_weave( )
{
    if      (chance(10)) return skill_lookup("blinding");
    else if (chance(10)) return skill_lookup("burn");
    else if (chance(10)) return skill_lookup("calm");
    else if (chance(10)) return skill_lookup("sever");
    else if (chance(10)) return skill_lookup("flay");
    else if (chance(10)) return skill_lookup("lash");
    else if (chance(10)) return skill_lookup("flog");
    else if (chance(10)) return skill_lookup("shielding");
    else if (chance(10)) return skill_lookup("earthquake");
    else if (chance(10)) return skill_lookup("firestorm");
    else if (chance(10)) return skill_lookup("lightning");
    else if (chance(10)) return skill_lookup("flameblade");
    else if (chance(10)) return skill_lookup("airclub");
    else if (chance(10)) return skill_lookup("shatter");
    else if (chance(10)) return skill_lookup("air blast");
    else if (chance(10)) return skill_lookup("freeze finger");
    else if (chance(10)) return skill_lookup("weaken");
    else if (chance(10)) return skill_lookup("hold");
    else if (chance(10)) return skill_lookup("sicken");
	else if (chance(10)) return skill_lookup("spirit fist");
	else if (chance(10)) return skill_lookup("spirit hammer");
	else if (chance(10)) return skill_lookup("spirit shatter");
	else if (chance(10)) return skill_lookup("thunder blast");
	else if (chance(10)) return skill_lookup("thunder strike");
	else if (chance(10)) return skill_lookup("thunder lance");
    else if (chance(10)) return skill_lookup("geyser");
    else if (chance(10)) return skill_lookup("fire bolt");
    else if (chance(10)) return skill_lookup("ice storm");
    else if (chance(10)) return skill_lookup("fork lightning");
    else if (chance(10)) return skill_lookup("drown");
    else return skill_lookup("fire bolt");
}
